//	<Name>	Bon_3SuluVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "Bon_3SuluVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Bon_3Sulu.h"
#include "Bon_3SuluVictory.h"
#include "MissionText.h"
#include "KlingonTeam.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
              
tBon_3SuluVictoryState::tBon_3SuluVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBon_3SuluVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{

}

void tBon_3SuluVictoryState::mBeginState()
{
	fPlayerDead = false;
	fPlayerDisengaged = false;
	fExcelsiorDestroyed = false;
	fExcelsiorDisengaged = false;
	fAllKlingonsDestroyed = false;
	fKlingonsInPlay = 3;
}

void tBon_3SuluVictoryState::mEndState()
{
}

eVictoryLevels tBon_3SuluVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) ) );
	tKlingonTeam* klingonTeam = dynamic_cast<tKlingonTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kKlingonTeam ) ) );
	int32 fireshipsThatReachedPlanet = klingonTeam->mGetTotalFireshipsReachedPlanet();
	
	if( fPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if( fPlayerDisengaged )
	{
		return kDefeat;
	}
	else if( fireshipsThatReachedPlanet == 0 )
	{
		return kAstoundingVictory;
	}
	else if( fireshipsThatReachedPlanet == 1 )
	{
		return kVictory;
	}
	else
	{
		return kDraw;
	}

}
int32 tBon_3SuluVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tBon_3SuluVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
	tKlingonTeam* klingonTeam = dynamic_cast<tKlingonTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kKlingonTeam ) ) );
	int32 wave = klingonTeam->mGetWaveCount();

	if( ship->mGetTeam() == kKlingonTeam )
	{
		fKlingonsInPlay = mShipsInPlay( klingonTeam );
	}
	if( fKlingonsInPlay == 0 )
	{
		fAllKlingonsDestroyed = true;
		if( wave > 2)
		{
			mSetTimedInterval( kTimerSecondsSet, kGameOverID, 3.0f );
		}
	}
	
}
void tBon_3SuluVictoryState::mOnDeath( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tTeamInfo* klingonTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kKlingonTeam ) );
	tTeamInfo* excelsiorTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kExcelsiorTeam ) );

	if( ship->mGetOriginalTeam() == playerTeam->mGetTeam() )
	{
		fPlayerDead = true;
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );		
	}
	else if( ship->mGetOriginalTeam() == excelsiorTeam->mGetTeam() )
	{
		fExcelsiorDestroyed = true;
	}
	mCheckAllTeams( ship );
}
void tBon_3SuluVictoryState::mOnCapture( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	if( ship->mGetOriginalTeam() == playerTeam->mGetTeam() )
	{
		fPlayerDead = true;
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );		
	}

}

void tBon_3SuluVictoryState::mOnDisengaged( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tTeamInfo* excelsiorTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kExcelsiorTeam ) );

	if( ship->mGetOriginalTeam() == playerTeam->mGetTeam() )
	{
		mSetTimedInterval( kTimerSecondsSet, kPlayerDisengagedID, 4.0f );
	}
	else if( ship->mGetOriginalTeam() == excelsiorTeam->mGetTeam() )
	{
		fExcelsiorDisengaged = true;
	}
}
void tBon_3SuluVictoryState::mOnTimedInterval( int32 timerID )
{
	tKlingonTeam* klingonTeam = dynamic_cast<tKlingonTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kKlingonTeam ) ) );
	int32 fireshipsThatReachedPlanet = klingonTeam->mGetTotalFireshipsReachedPlanet();
	
	if( timerID == kPlayerDisengagedID )
	{
		fPlayerDisengaged = true;
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
	}
	else if( timerID == kGameOverID )
	{
		if( fireshipsThatReachedPlanet > 1 )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
		}
		else
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
		}
	}
}
void tBon_3SuluVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{
}
