/********************************************************************************************************\

	File Name:	CommonBaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonBaseVictoryState.h"
#include "CommonDefs.h"
#include "CommonShip1.h"
#include "CommonTeam.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"


//
//	Serialization events
//
tCommonBaseVictoryState::tCommonBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tCommonBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tCommonBaseVictoryState::mBeginState()
{}
void tCommonBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tCommonBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tCommonBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tCommonTeam*	team = static_cast< tCommonTeam* >( fMissionInfo->mGetTeamHandle( ID ) );
	Assert( team );
	if ( team )
	{
		if ( gWC.mShipsDestroyed( ID ) == team->mNumShips() )
		{
			return kDefeat;
		}
		else
		{
			return kVictory;
		}
	}
	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!

	void
	tCommonBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tCommonBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	switch ( ship->mGetCustomID() )
	{
	//
	//	A ship was lost, so increment the number of ships destroyed on that team and check if the
	//	game is over.
	//
	case kShip1:
	case kShip2:
	case kShip3:
		{
			if ( fTeam->mGetTeam() == team->mGetTeam() )
			{
				gWC.mIncrementShipsDestroyed( teamID );

				//
				//	If all of the ships are gone on a team, then the game has ended for that team.
				//
				if ( gWC.mShipsDestroyed( teamID ) == ( static_cast< tCommonTeam* >( team )->mNumShips() ) )
				{
					fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
					gScriptInterface.mSetGameStatus( teamID, kPlayerLoses );
					gWC.mSetTeamWon( teamID, false );
					gWC.mCheckForGameEnd();
				}
			}
		}
		break;
	}
}
void tCommonBaseVictoryState::mOnCapture( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	switch ( ship->mGetCustomID() )
	{
	//
	//	A ship was lost, so increment the number of ships destroyed on that team and check if the
	//	game is over.
	//
	case kShip1:
	case kShip2:
	case kShip3:
		{
			if ( fTeam->mGetTeam() == team->mGetTeam() )
			{
				gWC.mIncrementShipsDestroyed( teamID );

				//
				//	If all of the ships are gone on a team, then the game has ended for that team.
				//
				if ( gWC.mShipsDestroyed( teamID ) == ( static_cast< tCommonTeam* >( team )->mNumShips() ) )
				{
					fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
					gScriptInterface.mSetGameStatus( teamID, kPlayerLoses );
					gWC.mSetTeamWon( teamID, false );
					gWC.mCheckForGameEnd();
				}
			}
		}
		break;
	}
}
void tCommonBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonBaseVictoryState::mOnFlashingButtonPressed()
{}
void tCommonBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tCommonBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() )
	{
		gWC.mSetTeamWon( team->mGetTeam(), false );
		gWC.mCheckForGameEnd();
	}
}
void tCommonBaseVictoryState::mOnHostChanged()
{}
void tCommonBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tCommonBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
