#include "Dem_Press1.h"

#include "MissionText.h"
#include "MissionMaps.h"

#include "VictoryCondition.h"

#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "PlanetTeam.h"
#include "ISCNPCTeam.h"
#include "FreighterTeam.h"
#include "PromisesVictory.h"


DEFINE_SCRIPT_CLASS( tDem_Press1 );

tDem_Press1::tDem_Press1()
{
}

tVictoryCondition* tDem_Press1::mSetupVictoryCondition()
{
	fVictoryCondition = new tPromisesVictory;
	gEventMgr.mRegisterActor( fVictoryCondition );							
	return fVictoryCondition;
}

//Register all you states here. Then mGotoState on the first state to run
void tDem_Press1::mSetupState()
{
}

//Register all you comm states here. 
void tDem_Press1::mSetupCommState()
{
}

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tDem_Press1::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tDem_Press1::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kMultiplayerMission;//kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fSectorType = kAllSectors;//kEnemyBorder | kUnexplored | kWarzone | kOccupiedColonial | kEnemyColonial;
	Specs->fPlayerRace = kSpecAnyRace;//kSpecISC;
	Specs->fEnemyRace = kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecMirak | kSpecHydran;
	Specs->fNumShips = kMissSpecNoShip; //kMissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 0;
	Specs->fMaxDate = 40;
	Specs->fWait = 6;
	Specs->fMenuStates = kNoMenuButtons;

	// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", kPlayerTeam, 1.0f, kFalse, kPlayableTeam, 1.0f,
												tTeamSpec::tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
												kSpecAnyRace, kAnyRace, typeid( tPlayerTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", kEnemyTeam, 1.0f, kFalse, kNPCTeam, 1.0f,
												tTeamSpec::tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
								 kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecMirak | kSpecHydran,
																		kAnyRace, typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", kPlanetTeam, 1.0f, kFalse, kNPCTeam, 1.0f,
												tTeamSpec::tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
												kSpecAnyRace, kAnyRace, typeid( tPlanetTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", kISCNPCTeam, 1.0f, kFalse, kNPCTeam, 1.0f,
												tTeamSpec::tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
												kSpecKling, kAnyRace, typeid( tISCNPCTeam ) );

/*	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5", kFreighterTeam, 1.0f, kFalse, kNPCTeam, 1.0f,
												tTeamSpec::tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
												kSpecRomulan, kAnyRace, typeid( tFreighterTeam ) );

*/
	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
//	Specs->mSetupTeam( team5 );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tDem_Press1::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tDem_Press1::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tDem_Press1::mCreateExternalShips( int32 Mode )
{

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tDem_Press1::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tDem_Press1::mStartMission( void )
{
	tScript::mStartMission();

	// setup team relations and target goals
	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		fPlayersTeam,
		kEnemyTeam,
		kPlanetTeam,
		kISCNPCTeam,
//		kFreighterTeam
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr	= fPlayersTeam,
		//Enemy = kEnemyTeam,
		//PLNT  = kPlanetTeam,
		//NPC   = kISCNPCTeam,
		//Frtr	= kFreighterTeam,


		//			Plyr	Enemy	PLNT	NPC		Frtr
		/*Plyr*/	"--",	"WD",	"AI",	"AI",//	"WD",
		/*Enemy*/	"WI",	"--",	"AI",	"WI",//	"AI",	
		/*PLNT*/	"AI",	"AI",	"--",	"AI",//	"AI",	
		/*NPC*/		"AI",	"WI",	"AI",	"--",//	"WI",
//		/*Frtr*/	"WI",	"AI",	"AI",	"WI",	"--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tDem_Press1::mNewInternalShip( tShipInfo* Ship )
{
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tDem_Press1::mGetDebriefingList( eVictoryLevels VictoryLevel )
{
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tDem_Press1::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}


