/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllyShip.h"
#include "AllyTeam.h"
#include "BaseVictoryState.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "Organia.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"
#include "Dem_Press2.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return kDraw;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( playerRace )
	{
	case kKlingon:
	case kGorn:
	case kLyran:
	case kMirak:
	default:
		{
			if ( fChildActor->mReachedOrgania() )
			{
				if ( fChildActor->mISCShipsDestroyed() < 4 )
				{
					return kVictory;
				}
				else
				{
					return kAstoundingVictory;
				}
			}
			else
			{
				if ( fChildActor->mISCShipsDestroyed() < 4 )
				{
					return kDevastatingDefeat;
				}
				else
				{
					return kDefeat;
				}
			}
		}
		break;
	}
	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam::kISCShip1:
	case tEnemyTeam::kISCShip2:
	case tEnemyTeam::kISCShip3:
	case tEnemyTeam::kISCShip4:
		{
			fChildActor->mIncreaseISCShipsDestroyed();
			fMissionInfo->mDisplayMessage( kPlayerTeam, kCustom1_msg );
			
			if ( fChildActor->mISCShipsDestroyed() == 4 )
			{
				fMissionInfo->mDisplayMessage( kPlayerTeam, kEvent7_1_msg );
			}

			switch ( playerRace )
			{
			case kKlingon:
			case kGorn:
			case kMirak:
			case kLyran:
				{
					mAddToBonusPrestige( 20 );
					fMissionInfo->mDisplayMessage( kPlayerTeam, kEvent7_2_msg );
				}
				break;
			}
		}
		break;

	case tEnemyTeam::kISCDefensePlatform1:
	case tEnemyTeam::kISCDefensePlatform2:
		{
			tTeamInfo*	planetTeam = fMissionInfo->mGetTeamHandle( kNPC1Team );
			Assert( planetTeam );
			if ( !planetTeam )
				return;
			tShipInfo*	planet = planetTeam->mFindShipCustomID( tNPC1Team::kOrgania );
			fChildActor->mIncreaseISCDefensePlatformsDestroyed();
			fMissionInfo->mDisplayMessage( kPlayerTeam, kCustom3_msg );

			if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
			{
				fMissionInfo->mDisplayMessage( kPlayerTeam, kEvent12_1_msg );
				if ( planet )
					playerTeam->mGotoCommState( planet->mGetShipID(), tOrgania::kCanReachCommState );
			}
		}
		break;

	case tAllyTeam::kAllyShip1:
		{
			fMissionInfo->mSetTeamRelations( kEnemyTeam, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0 );
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemaining() )
					gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam::kISCShip1:
	case tEnemyTeam::kISCShip2:
	case tEnemyTeam::kISCShip3:
	case tEnemyTeam::kISCShip4:
		{
			fChildActor->mIncreaseISCShipsDestroyed();
			fMissionInfo->mDisplayMessage( kPlayerTeam, kCustom2_msg );
			
			if ( fChildActor->mISCShipsDestroyed() == 4 )
			{
				fMissionInfo->mDisplayMessage( kPlayerTeam, kEvent9_1_msg );
			}

			switch ( playerRace )
			{
			case kKlingon:
			case kGorn:
			case kMirak:
			case kLyran:
				{
					fMissionInfo->mDisplayMessage( kPlayerTeam, kEvent9_2_msg );
				}
				break;
			}
		}
		break;

	case tEnemyTeam::kISCDefensePlatform1:
	case tEnemyTeam::kISCDefensePlatform2:
		{
			tTeamInfo*	planetTeam = fMissionInfo->mGetTeamHandle( kNPC1Team );
			Assert( planetTeam );
			if ( !planetTeam )
				return;
			tShipInfo*	planet = planetTeam->mFindShipCustomID( tNPC1Team::kOrgania );
			fChildActor->mIncreaseISCDefensePlatformsDestroyed();
			fMissionInfo->mDisplayMessage( kPlayerTeam, kCustom3_msg );

			if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
			{				
				fMissionInfo->mDisplayMessage( kPlayerTeam, kEvent12_1_msg );
				if ( planet )
					playerTeam->mGotoCommState( planet->mGetShipID(), tOrgania::kCanReachCommState );
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemaining() )
					gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerCaptured );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( kPlayerTeam, kEvent14_1_msg );
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDisengaged );
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	//
	//	Look to end the mission if the hail button is pressed on Organia and the defense platforms are
	//	both destroyed.
	//
	if ( buttonID == k_HAIL_Organia_btn )
	{
		if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
		{
			fChildActor->mSetReachedOrgania( true );
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
		}
	}
}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;

	switch ( damagedShip->mGetCustomID() )
	{
	case tAllyTeam::kAllyShip1:
		{
			//
			//	Check for the player attacking the ally.  Must also check if a dust cloud did not
			//	hit the ally because a dust cloud apparently does not get passed in as a tShipInfo*.
			//
			if ( hostileShip )
			{
				if ( hostileShip->mGetTeam() == kPlayerTeam )
				{
					fChildActor->mSetAttackedAlly( true );
				}
			}
		}
		break;

	}
}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( kPlayerTeam, kRandChat1_msg );		break;
		case 1:
			fMissionInfo->mDisplayMessage( kPlayerTeam, kRandChat2_msg );		break;
		case 2:
			fMissionInfo->mDisplayMessage( kPlayerTeam, kRandChat3_msg );		break;
		case 3:
			fMissionInfo->mDisplayMessage( kPlayerTeam, kRandChat4_msg );		break;
		case 4:
			fMissionInfo->mDisplayMessage( kPlayerTeam, kRandChat5_msg );		break;
		case 5:
			fMissionInfo->mDisplayMessage( kPlayerTeam, kRandChat6_msg );		break;
		}
		fChildActor->mSetTimedInterval( kTimerSecondsSet, timerID,
										( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif

	switch ( timerID )
	{
	case tAllyShip::kFirstOfferID:
		{
			fChildActor->mSetFirstOfferMade( true );
		}
		break;

	case tAllyShip::kFirstOfferDecisionID:
		{
			fChildActor->mSetFirstOfferISC( true );
		}
		break;

	case tAllyShip::kSecondOfferID:
		{
			fChildActor->mSetSecondOfferMade( true );
		}
		break;

	case tAllyShip::kSecondOfferDecisionID:
		{
			fChildActor->mSetSecondOfferISC( true );
		}
		break;
	}
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

