/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "Dem_Press2.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	fMissionInfo->mAddToDebriefingList( ( eTeamID )kPlayerTeam, victoryLevel + kAstoundingVictory_msg );

/*

	HELPER NOTE:	The following is the analysis of the mission.  You are able to place additional text
					dependant upon the victory outcome in the MissionText.cpp file.  Simply find the
					corresponding values.

*/
	//
	//	Add the appropriate messages to the debriefing list according to the player's performance.
	//
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( playerRace )
	{
	case kFederation:
	case kRomulan:
	case kHydran:
		{
			switch ( victoryLevel )
			{
			case kAstoundingVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictoryFederationRomulanHydran_msg );	break;
			case kVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebVictoryFederationRomulanHydran_msg );			break;
			case kDraw:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );										break;
			case kDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeatFederationRomulanHydran_msg );				break;
			case kDevastatingDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeatFederationRomulanHydran_msg );	break;
			}
		}
		break;

	case kKlingon:
	case kGorn:
	case kLyran:
	case kMirak:
		{
			switch ( victoryLevel )
			{
			case kAstoundingVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictoryKlingonGornLyranMirak_msg );	break;
			case kVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebVictoryKlingonGornLyranMirak_msg );				break;
			case kDraw:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );										break;
			case kDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeatKlingonGornLyranMirak_msg );				break;
			case kDevastatingDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeatKlingonGornLyranMirak_msg );	break;
			}
		}
		break;
	}
}
/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	You may notice that the mCreateShipsForTeam(...) in this file has a different signature
					and body then the other team files.  This is because the 'tPlayerTeam' is of a
					'tPlayableTeam' class and it would be unknown how many ships the player has control
					of until the game has already begun.  However, if you want to force the player to play
					with a certain amount of ships (perhaps for a special mission), then the body of this
					function would look much more like the other team files.  To get an idea of this, refer
					to the helper notes on those team files.

*/
void tPlayerTeam::mCreateShipsForTeam()
{
	mCreateShip( typeid( tPlayerShip ), "Z-CC", kStartPosition_G, kPlayerShip1 );
}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{}
