/********************************************************************************************************\

	File Name:	DefendTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Dem_Press4.h"
#include "DefendTeam.h"
#include "DefendTeamInitialState.h"
//#include "DefendTeamShip.h"
#include "ShipScriptDescription.h"
#include "DefendVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

tClassAllocator<tDefendTeam> gDefendTeamAllocator;

tDefendTeam::tDefendTeam()
{
	gWC.mAddDefendTeam();
}
void tDefendTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tDefendVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDraw_msg );
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDefeat_msg );
	} 
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDevastatingDefeat_msg );
	}
}

void tDefendTeam::mSetupState()
{
	mRegisterState( new tDefendTeamInitialState() );

	mGotoState( typeid( tDefendTeamInitialState ) );
}
void tDefendTeam::mSetupCommState()
{
}
void tDefendTeam::mCreateShipsForTeam()
{
	tTeamInfo* attackTeam = fMissionInfo->mGetTeamHandle( kAttackTeam1 );

	if( !( mIsTeamHumanControlled() ) )
	{
		int32 attackBPV = attackTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
		int32 numberOfShip = fMissionInfo->mRandomInt32( 1, 3 );
		int32 defendBPV; 
		if( diffLevel == 0 )
		{
			defendBPV = attackBPV * 0.8;
		}
		else if( diffLevel == 1 )
		{
			defendBPV = attackBPV * 1.1;
		}
		else if( diffLevel == 2 )
		{
			defendBPV = attackBPV * 1.4;
		}

		mCreateShip( typeid( tShipInfo ),
							( 3 * defendBPV / 4 ),
							  kClassFrigate, 
							  kClassDreadnought, 
							  20,
							  kStartPosition_G | kGroupAround );

		if( numberOfShip > 1 )
		{
			mCreateShip( typeid( tShipInfo ),
								( defendBPV / 4 ),
								  kClassFrigate, 
								  kClassDreadnought, 
								  20,
								  kStartPosition_G | kGroupAround );
		}

		if( numberOfShip == 3 )
		{
			mCreateShip( typeid( tShipInfo ),
								( defendBPV / 2 ),
								  kClassFrigate, 
								  kClassDreadnought, 
								  20,
								  kStartPosition_G | kGroupAround );
		}
	}
	else
	{
		mCreateShip( typeid( tShipInfo ), "Z-BCH", kStartPosition_G );
	}
}
void tDefendTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	if( mIsTeamHumanControlled() )
	{
		mCreateShip( typeid( tShipInfo ), 
					 shipScriptDescription.fClassName.c_str(), 
					 kStartPosition_G + shipScriptDescription.fTeamID,
					 -1, 0, 0, -1, -1, //Default data,so we can set the custom name
					 shipScriptDescription.fCustomName.c_str() );
	}

}