/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "sfcAssert.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	fOrigNumAttackTeams = 0;
	fOrigNumDefendTeams = 0;
	fNumAttackTeams = 0;
	fNumDefendTeams = 0;
	fNumTeams = 6;	//Hard coded max number of teams
	fOrigNumbOfFreighters = 0;	//Number of Freighters at start of game
	fCurrNumbOfFreighters = 0;	//Number of Freighters currently in the game
	fEscpNumbOfFreighters = 0;	//Number of Freighters that have disengaged
	fDestNumbOfFreighters = 0;	//Number of Freighters destroyed

}
int32 tWorldConditions::mGetFreightersEscaped()
{
	return fEscpNumbOfFreighters;
}
int32 tWorldConditions::mGetFreightersOriginal()
{
	return fOrigNumbOfFreighters;
}
int32 tWorldConditions::mGetFreightersDestroyed()
{
	return fDestNumbOfFreighters;
}
void tWorldConditions::mAddFreighter()
{
	++fOrigNumbOfFreighters;
	++fCurrNumbOfFreighters;
}
void tWorldConditions::mRemoveFreighter()
{
	--fCurrNumbOfFreighters;
}
void tWorldConditions::mFreighterEscaped()
{
	--fCurrNumbOfFreighters;
	++fEscpNumbOfFreighters;
}
void tWorldConditions::mFreighterDestroyed()
{
	--fCurrNumbOfFreighters;
	++fDestNumbOfFreighters;
}
void tWorldConditions::mAddAttackTeam()
{
	++fNumAttackTeams;
	++fOrigNumAttackTeams;
}
void tWorldConditions::mRemoveAttackTeam()
{
	--fNumAttackTeams;
}
int32 tWorldConditions::mGetAttackTeams()
{
	return fNumAttackTeams;
}
void tWorldConditions::mAddDefendTeam()
{
	++fNumDefendTeams;
	++fOrigNumDefendTeams;
}
void tWorldConditions::mRemoveDefendTeam()
{
	--fNumDefendTeams;
}
int32 tWorldConditions::mGetDefendTeams()
{
	return fNumDefendTeams;
}
void tWorldConditions::mCheckForGameEnd( void )
{

	for ( int i = 0; i < fNumTeams; i++ )
	{
		tMissionInfo*	thisMission = gScriptInterface.mMission();

		tTeamInfo* currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );

		if ( !currentTeam )
			continue;
		
		
		if( ( fNumAttackTeams == 0 ) || ( fNumDefendTeams == 0 ) )
		{
			int32 destroyedFreighters = gWC.mGetFreightersDestroyed();
			int32 attackTeamsRemaining = gWC.mGetAttackTeams();
			int32 escapedFreighters = gWC.mGetFreightersEscaped();
			int32 originalNumberFreighters = gWC.mGetFreightersOriginal();

			if( attackTeamsRemaining > 0 )
			{
				destroyedFreighters = originalNumberFreighters - escapedFreighters;
			}
			
			if( destroyedFreighters < 2 )
			{
				if( ( i == 0 ) || ( i == 2 ) | ( i == 3 ) )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
				}
				else
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
				}
			}
			else
			{
				if( ( i == 0 ) || ( i == 2 ) | ( i == 3 ) )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
				}
				else
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
				}
			}
		}
	}
}
