/********************************************************************************************************\

	Mission:	Demo Press 5
	Script:		"Dem_Press5"
	Descript:	Rescue a legendary diplomat.

	Created by:		Daniel J. Suleski		06/23/00
	Last Updated:	<NAME>					mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "Dem_Press5.h"
#include "BaseVictory.h"
#include "EnemyTeam.h"
#include "Enemy2Team.h"
#include "Enemy3Team.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "NPC1Team.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "VictoryCondition.h"


DEFINE_SCRIPT_CLASS( tDem_Press5 );

tDem_Press5::tDem_Press5()
{}

//Register all you states here. Then mGotoState on the first state to run
void tDem_Press5::mSetupState()
{}

//Register all you comm states here. 
void tDem_Press5::mSetupCommState()
{}

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tDem_Press5::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tDem_Press5::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fSectorType = kAlliedBorder | kUnexplored | kWarzone;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fEnemyRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecNoShip;
	Specs->fMinBPV = 900;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 0;
	Specs->fMaxDate = 99;
	Specs->fWait = kUniqueMission;
	Specs->fMenuStates = kNoMenuButtons;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team",
													kPlayerTeam,
													1.0f,
													kFalse,
													kPlayableTeam,
													1.0f,
													tTeamSpec::tStrength(	0, 0,
																			kClassFrigate,
																			kClassFrigate,
																			0, 300 ),
													kSpecISC,
													kAnyRace,
													typeid( tPlayerTeam ) );

	const tTeamSpec& playerNPCTeam = mCreateTeamSpec(	"Player NPC Team",
														kPlayerNPCTeam,
														1.0f,
														kFalse,
														kNPCTeam,
														1.0f,
														tTeamSpec::tStrength(	0, 0,
																				kClassFrigate,
																				kClassFrigate,
																				0, 300 ),
														kSpecISC,
														kAnyRace,
														typeid( tPlayerNPCTeam ) );

	const tTeamSpec& enemyTeam = mCreateTeamSpec(	"Enemy Team",
													kEnemyTeam,
													1.0f,
													kFalse,
													kNPCTeam,
													1.0f,
													tTeamSpec::tStrength(	0, 0,
																			kClassFrigate,
																			kClassFrigate,
																			0, 300 ),
													kSpecGorn,
													kAnyRace,
													typeid( tEnemyTeam ) );

	const tTeamSpec& enemy2Team = mCreateTeamSpec(	"Enemy 2 Team",
													kEnemy2Team,
													1.0f,
													kFalse,
													kNPCTeam,
													1.0f,
													tTeamSpec::tStrength(	0, 0,
																			kClassFrigate,
																			kClassFrigate,
																			0, 300 ),
													kSpecHydran,
													kAnyRace,
													typeid( tEnemy2Team ) );

	const tTeamSpec& enemy3Team = mCreateTeamSpec(	"Enemy 3 Team",
													kEnemy3Team,
													1.0f,
													kFalse,
													kNPCTeam,
													1.0f,
													tTeamSpec::tStrength(	0, 0,
																			kClassFrigate,
																			kClassFrigate,
																			0, 300 ),
													kSpecKling,
													kAnyRace,
													typeid( tEnemy3Team ) );

	const tTeamSpec& npc1Team = mCreateTeamSpec(	"NPC 1 Team",
													kNPC1Team,
													1.0f,
													kFalse,
													kNPCTeam,
													1.0f,
													tTeamSpec::tStrength(	0, 0,
																			kClassFrigate,
																			kClassFrigate,
																			0, 300 ),
													kSpecISC,
													kAnyRace,
													typeid( tNPC1Team ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( playerNPCTeam );
	Specs->mSetupTeam( enemyTeam );
	Specs->mSetupTeam( enemy2Team );
	Specs->mSetupTeam( enemy3Team );
	Specs->mSetupTeam( npc1Team );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tDem_Press5::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tDem_Press5::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tDem_Press5::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tDem_Press5::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tDem_Press5::mStartMission( void )
{
	tScript::mStartMission();

/*

	HELPER NOTE:	The teamList array that you see below should reflect those constants that have been or you
					have declared in the matching header file (e.g. Dem_Press5.h).

*/
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kEnemy2Team,
		kEnemy3Team,
		kNPC1Team,
	};

/*

	HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
					meaning has a meaning???

	Relationship Lookup
	- = kTeamSelfAware	//	use this parameter with itself
	A = kTeamAllied		//	cannot fire upon an ally
	F = kTeamFriendly
	N = kTeamNeutral
	H = kTeamHostile	//	will not fire / capture / cripple until provoked
	W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
	P = kTeamPassive

	Relationship Lookup
	- = kGoalIgnore		//	use this parameter with itself
	I = kGoalIgnore
	C = kGoalCripple
	X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
	D = kGoalDestroy

*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = PlayersTeam,
		//PNPC = PlayerNPCTeam, -- freighter
		//Nme  = EnemyTeam,
		//Nme2 = Enemy2Team,
		//Nme3 = Enemy3Team,
		//NPC1 = NPC1 -- planet

		//	     Plyr  PNPC  Nme   Nme2  Nme3  NPC1
		/*Plyr*/ "--", "NI", "WD", "WD", "WD", "WD",
		/*PNPC*/ "NI", "--", "HD", "HD", "HD", "HD",
		/*Nme */ "WD", "HC", "--", "AI", "AI", "AI",
		/*Nme2*/ "WD", "HC", "AI", "--", "AI", "AI",
		/*Nme3*/ "WD", "HC", "AI", "AI", "--", "AI",
		/*NPC1*/ "WD", "WD", "AI", "AI", "AI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tDem_Press5::mNewInternalShip( tShipInfo* Ship )
{}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tDem_Press5::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tDem_Press5::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

