/********************************************************************************************************\

	File Name:	Enemy3ShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyShip.h"
#include "Enemy2Ship.h"
#include "Enemy3ShipBaseState.h"
#include "MissionText.h"
#include "Dem_Press5.h"
#include "TeamInfo.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tEnemy3ShipBaseState::mBeginState()
{}
void tEnemy3ShipBaseState::mEndState()
{}

void tEnemy3ShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( ( !ship ) || ( !targetShip ) )
		return;
	if ( fChildActor->mGetCustomID() == targetShip->mGetCustomID() )
	{
		//
		//	Set the surrendered flags in the local class for this hydran ship.
		//
		switch ( buttonID )
		{
		case k_YOUSURRENDER_EnemyForReal_btn:
			{
				float	hullDamage = targetShip->mGetDamagePercent();

				if ( hullDamage >= 10.0f )
				{
					fChildActor->mSetIsSurrendered( true );
					fChildActor->mDisengage( kByMapEdge, kAbsolutePriority );
					fMissionInfo->mSetTeamRelations( kEnemy3Team, kPlayerTeam, kTeamNeutral, kGoalIgnore, 1.0f );
					fMissionInfo->mSetTeamRelations( kPlayerTeam, kEnemy3Team, kTeamNeutral, kGoalIgnore, 1.0f );
				}
			}
			break;
		}
	}
}
void tEnemy3ShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;
	float	hullDamage = damagedShip->mGetDamagePercent();
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	if ( hostileShip && ( fChildActor->mGetCustomID() == damagedShip->mGetCustomID() ) )
	{
		//
		//	Set the flag for the player attacking a surrendered ship.
		//
		if (	( fChildActor->mIsSurrendered() ) && ( !fChildActor->mAttackedSurrenderedVessel() ) &&
				( hostileShip->mGetTeam() == kPlayerTeam )	)
		{
			fMissionInfo->mDisplayMessage( kPlayerTeam, kCustom2_msg );
			fMissionInfo->mSetTeamRelations( kEnemy3Team, kPlayerTeam, kTeamAtWar, kGoalDestroy, 1.0f );
			fMissionInfo->mSetTeamRelations( kPlayerTeam, kEnemy3Team, kTeamAtWar, kGoalDestroy, 1.0f );
			fChildActor->mSetAttackedSurrenderedVessel( true );

			//
			//	Set the flags for attacking the surrendered ship in the other enemy classes.
			//
			tTeamInfo*		enemyTeam = fMissionInfo->mGetTeamHandle( kEnemyTeam );
			tTeamInfo*		enemy2Team = fMissionInfo->mGetTeamHandle( kEnemy2Team );
			Assert( enemyTeam && enemy2Team );
			if ( ( !enemyTeam ) || ( !enemy2Team ) )
				return;
			tShipIterator	enemyShipIterator = enemyTeam->mGetFirstShip();
			tShipIterator	enemy2ShipIterator = enemy2Team->mGetFirstShip();
			if ( ( *enemyShipIterator == NULL ) || ( *enemy2ShipIterator == NULL ) )
				return;
			tEnemyShip*		enemyShip = static_cast< tEnemyShip* >( *enemyShipIterator );
			tEnemy2Ship*	enemy2Ship = static_cast< tEnemy2Ship* >( *enemy2ShipIterator );
			if ( enemyShip )
			{
				playerTeam->mGotoCommState( enemyShip->mGetShipID(), tEnemyShip::kCommState1 );
				enemyShip->mSetAttackedSurrenderedVessel( true );
			}
			if ( enemy2Ship )
			{
				playerTeam->mGotoCommState( enemy2Ship->mGetShipID(), tEnemy2Ship::kCommState1 );
				enemy2Ship->mSetAttackedSurrenderedVessel( true );
			}
		}

		//
		//	Change comm state according to damage taken.
		//
		if ( fChildActor->mAttackedSurrenderedVessel() )
		{
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tEnemy3Ship::kCommState1 );
		}
		else
		{
			if ( hullDamage < 10.0f )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tEnemy3Ship::kCommState2 );
			}
			else
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tEnemy3Ship::kCommState3 );
			}
		}
	}

	fChildActor->mSetShipCallbackEvents( kNotifyDamage );
}
void tEnemy3ShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemy3ShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemy3ShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemy3ShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemy3ShipBaseState::mOnFlashingButtonPressed()
{}
void tEnemy3ShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemy3ShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemy3ShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tEnemy3ShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tEnemy3ShipBaseState::mOnHostChanged()
{}
void tEnemy3ShipBaseState::mOnTimedInterval( int32 timerID )
{}
void tEnemy3ShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemy3ShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


