/********************************************************************************************************\

	File Name:	FreighterBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "FreighterBaseState.h"
#include "Dem_Press5.h"
#include "MissionText.h"
#include "ScriptInterface.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tFreighterBaseState::mBeginState()
{}
void tFreighterBaseState::mEndState()
{}

void tFreighterBaseState::mOnDeath( tShipInfo* ship )
{}
void tFreighterBaseState::mOnCapture( tShipInfo* ship )
{}
void tFreighterBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tFreighterBaseState::mAtDistance( tShipInfo* ship )
{}
void tFreighterBaseState::mAtDestination( tShipInfo* ship )
{
}
void tFreighterBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( targetShip->mGetCustomID() == fChildActor->mGetCustomID() )
	{
		switch ( buttonID )
		{
		case k_DROPSHIELDS_Freighter_btn:
			{
				fChildActor->mSetPercentageFromOrig( kAllShieldsHardPoint, 0.0f );
			}
			break;
		}
	}
}
void tFreighterBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;
	float	hullDamage = damagedShip->mGetDamagePercent();
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	if ( fChildActor->mDiplomatRescued() )
	{
		//
		//	The ally ship was attacked by the player, so the ally ship should no longer respond to the
		//	comm states from the player.
		//
		if (	( hostileShip ) && ( ( fChildActor->mAttackedAlly() ) || 
				( hostileShip->mGetTeam() == kPlayerTeam ) )	)
		{
			fChildActor->mSetAttackedAlly( true );
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState1 );
			fChildActor->mSetShipCallbackEvents( kNotifyDamage );;
		}
		//
		//	Change comm states according to the damage taken by the allied ship.
		//
		else if ( damagedShip->mGetCustomID() == fChildActor->mGetCustomID() )
		{
			if ( fChildActor->mShipsDestroyed() == 3 )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState2 );
			}
			else
			{
				if ( hullDamage == 0.0f )
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState3 );
				}
				else if ( hullDamage < 30.0f )
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState4 );
				}
				else if ( hullDamage < 60.0f )
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState5 );
				}
				else
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState6 );
				}
			}
			fChildActor->mSetShipCallbackEvents( kNotifyDamage );
		}
	}
	else
	{

		//
		//	The ally ship was attacked by the player, so the ally ship should no longer respond to the
		//	comm states from the player.
		//
		if (	( hostileShip ) && ( ( fChildActor->mAttackedAlly() ) || 
				( hostileShip->mGetTeam() == kPlayerTeam ) )	)
		{
			fChildActor->mSetAttackedAlly( true );
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState7 );
			fChildActor->mSetShipCallbackEvents( kNotifyDamage );;
		}
		//
		//	Change comm states according to the damage taken by the allied ship.
		//
		else if ( damagedShip->mGetCustomID() == fChildActor->mGetCustomID() )
		{
			if ( fChildActor->mShipsDestroyed() == 3 )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState8 );
			}
			else
			{
				if ( hullDamage == 0.0f )
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState9 );
				}
				else if ( hullDamage < 30.0f )
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState10 );
				}
				else if ( hullDamage < 60.0f )
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState11 );
				}
				else
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState12 );
				}
			}
			fChildActor->mSetShipCallbackEvents( kNotifyDamage );
		}
	}
}
void tFreighterBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tFreighterBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tFreighterBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tFreighterBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tFreighterBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tFreighterBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tFreighterBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tFreighterBaseState::mOnFlashingButtonPressed()
{}
void tFreighterBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if ( ( !shipTo ) || ( !shipFrom ) )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	if ( shipFrom->mGetCustomID() == fChildActor->mGetCustomID() )
	{

		//
		//	If the player is removing something from a ship.
		//
		if ( shipTo->mGetTeam() == kPlayerTeam )
		{
			switch ( item )
			{
			case kTransDiplomat:
				{
					eAlertLevels	alertLevel = fChildActor->mGetAlertStatus();
					
					fChildActor->mSetAlertStatus( kAlertRed );
					fChildActor->mSetPercentageFromOrig( kAllShieldsHardPoint, 1.0f, kTrue );
					fChildActor->mSetAlertStatus( alertLevel );

					float	hullDamage = fChildActor->mGetDamagePercent();

					//
					//	It is known that the diplmoat has been rescued.
					//
					if ( fChildActor->mShipsDestroyed() == 3 )
					{
						playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState2 );
					}
					else
					{
						if ( hullDamage == 0.0f )
						{
							playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState3 );
						}
						else if ( hullDamage < 30.0f )
						{
							playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState4 );
						}
						else if ( hullDamage < 60.0f )
						{
							playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState5 );
						}
						else
						{
							playerTeam->mGotoCommState( fChildActor->mGetShipID(), tFreighter::kCommState6 );
						}
					}
				}
				break;
			}
		}
	}
}
void tFreighterBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tFreighterBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tFreighterBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tFreighterBaseState::mOnHostChanged()
{}
void tFreighterBaseState::mOnTimedInterval( int32 timerID )
{}
void tFreighterBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tFreighterBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


