/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "His_NextGen.h"
#include "EnemyTeam.h"
#include "AllyTeam.h"


void tBaseVictoryState::mClearScreen()
{
	fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kClearScreen_msg );
	fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kClearScreen_msg, kNonCommScriptMessage );
}

tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval(kTimerSecondsSet, kBegin_timer, 2.0f );
}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{

	case kBegin_timer:
		{
			static bool beenhere12 = false;

			if( !beenhere12 )
			{
				fMissionInfo->mPlayVoiceSample( kVoice_Taunt1_Kli );
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCentury_msg, kNonCommScriptMessage );
				mSetTimedInterval(kTimerSecondsSet, kConduit_timer, 20.0f );
				beenhere12 = true;
			}
		}
		break;


	case kConduit_timer:
		{
			static bool beenhere1 = false;

			if( !beenhere1 )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kConduit_msg );
				mSetTimedInterval( kTimerSecondsSet, kBorg_timer, 3.0f );	

				beenhere1 = true;
			}
		}
		break;

	case kBorg_timer:
		{
			static bool beenhere2 = false;

			if( !beenhere2 )
			{
				tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
				ASSERT( enemyTeam );
				if( !enemyTeam ) return;

				fMissionInfo->mPlaySound( kSndWarpIn );
				enemyTeam->mCreateCube();

				mClearScreen();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBorgCome_msg );

				mSetTimedInterval( kTimerSecondsSet, kBorg2_timer, 3.0f );

				beenhere2 = true;
			}
		}
		break;

	case kBorg2_timer:
		{
			static bool beenhere3 = false;

			if( !beenhere3 )
			{
				tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
				ASSERT( playerTeam );
				if( !playerTeam ) return;

				tShipIterator i = playerTeam->mGetFirstShip();
				ASSERT( *i );
				if( !*i ) return;
				
				if( i->mGetAlertStatus() !=  kAlertRed )
				{
					i->mSetAlertStatus( kAlertRed );
				}
				
				mSetTimedInterval( kTimerSecondsSet, kBorgHail_timer, 3.0f );

				beenhere3 = true;
			}
		}
		break;

	case kBorgHail_timer:
		{
			static bool beenhere4 = false;

			if( !beenhere4 )
			{
				mClearScreen();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBorgHail_msg );
				mSetTimedInterval( kTimerSecondsSet, kBorgHail2_timer, 2.0f );
				beenhere4 = true;
			}
		}
		break;

	case kBorgHail2_timer:
		{
			static bool beenhere5 = false;

			if( !beenhere5 )
			{
				fMissionInfo->mPlaySound( kSndCompBeepComm );
				fMissionInfo->mPlayVoiceSample( kVoice_BorgA, kTrue );
				mSetTimedInterval( kTimerSecondsSet, kBorgHail3_timer, 4.2f );
				beenhere5 = true;
			}
		}
		break;

	case kBorgHail3_timer:
		{
			static bool beenhere6 = false;

			if( !beenhere6 )
			{
				fMissionInfo->mPlayVoiceSample( kVoice_BorgB, kFalse );
				mSetTimedInterval( kTimerSecondsSet, kNoMatch_timer, 4.0f );
				beenhere6 = true;
			}
		}
		break;

	case kNoMatch_timer:
		{
			static bool beenhere7 = false;

			if( !beenhere7 )
			{
				mClearScreen();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNoMatch_msg );
				mSetTimedInterval( kTimerTurnsSet, kAlly_timer, 2.0f );
				beenhere7 = true;
			}
		}
		break;
		
	case kAlly_timer:
		{
			static bool beenhere8 = false;

			if( !beenhere8 )
			{
				tAllyTeam* allyTeam = dynamic_cast<tAllyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) ) );
				ASSERT( allyTeam );
				if( !allyTeam ) return;

				fMissionInfo->mPlaySound( kSndWarpIn );
				allyTeam->mCreateReinforcements();

				mClearScreen();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllies_msg );

				beenhere8 = true;
			}
		}
		break;
	}
}



eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	return kDraw;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	Assert( enemyTeam );
	if ( !enemyTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
			}
		}
		break;

	case tEnemyTeam::kEnemyShip1:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	Assert( enemyTeam );
	if ( !enemyTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
			}
		}
		break;

	case tEnemyTeam::kEnemyShip1:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	Assert( enemyTeam );
	if ( !enemyTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
			}
		}
		break;

	case tEnemyTeam::kEnemyShip1:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

void tBaseVictoryState::mEndState()
{}