#include "His_HostileSkirmish.h"

#include "His_HostileSkirmishText.h"
#include "His_HostileSkirmishMaps.h"


// LNF add #include for states
#include "His_HostileSkirmishStateStarting.h"

// LNF add #include for victory condition
#include "His_HostileSkirmishVictory.h"
#include "PlayableTeam.h"
/*
int32 EnemyCaptured = 0;
bool PlayerDestroyed = false;
bool PlayerDisengaged = false;
bool PlayerDead = false;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;

*/


DEFINE_SCRIPT_CLASS( tHis_HostileSkirmish );

tHis_HostileSkirmish::tHis_HostileSkirmish()
{
	fDiffLevel = 0;
	StartingEnemies = false;
}

tVictoryCondition* tHis_HostileSkirmish::mSetupVictoryCondition()
{
	fVictoryCondition = new tHis_HostileSkirmishVictory;
	gEventMgr.mRegisterActor( fVictoryCondition );							
	return fVictoryCondition;	
}

void tHis_HostileSkirmish::mSetupState()
{
	mRegisterState( new tStateStarting( this ) );
//	mRegisterState( new tStateSabotage( this,  ) );
//	mRegisterState( new tStateRepairing( this, fMissionInfo, fDiffLevel )  );
//	mRegisterState( new tStateRepaired( this, fMissionInfo ) );
	
	mGotoState( typeid( tStateStarting ) );
}

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tHis_HostileSkirmish::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tHis_HostileSkirmish::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fMenuStates = kEnableSupplyDock;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tHis_HostileSkirmish::mInitializeStart( tDynaverseScriptInfo& info )
{
	// check the first AI teams ship count - if its 0 then we need to fill in the default values before proceeding
	if (info.fMemberShipCounts[kTeam2] == 0)
	{
		mSetDefaultMenuItems( info );
	}

	// must setup the races and relations of the reserved teams in the script
	fMissionInfo->mSetSkirmishTeams( info );

	// pick a map
	return 1;
}


//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tHis_HostileSkirmish::mCreateMissionShips( int32 Mode )
{
	tTeamInfo* Team;
	int32 position = kStartPosition_H;

	// create friendly AI ships first
	for( int32 i = 0; i < fMissionInfo->fTotalAlliedCPUTeams; i++ )
	{
		Team = fMissionInfo->mGetTeamHandle( kCPU_AlliedTeams, i );
		Team->mCreateCPUFleet( typeid( tShipInfo ), 20, position | kGroupAround );
		position++;
	}

	position = kStartPosition_U;

	// create enemy AI ships last
	for( i = 0; i < fMissionInfo->fTotalEnemyCPUTeams; i++ )
	{
		Team = fMissionInfo->mGetTeamHandle( kCPU_EnemyTeams, i );
		Team->mCreateCPUFleet( typeid( tShipInfo ), 20, position | kGroupAround );
		position++;
	}

	if( fMissionInfo->fTotalEnemyCPUTeams > 0 )
	{
		StartingEnemies = true;
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tHis_HostileSkirmish::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tHis_HostileSkirmish::mStartMission( void )
{
	tTeamInfo* PlayerTeam = fMissionInfo->mGetTeamHandle( fPlayersTeam );

	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tHis_HostileSkirmish::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tHis_HostileSkirmish::mGetDebriefingList( eVictoryLevels VictoryLevel )
{
	fMissionInfo->mAddToDebriefingList( kYourScore_msg );
	fMissionInfo->mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tHis_HostileSkirmish::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

//---------------------------------------------------------------------------------------------
//
void tHis_HostileSkirmish::mSetDefaultMenuItems( tDynaverseScriptInfo& info )
{
	// default player settings:
	info.fMemberShipCounts[kTeam1]	= 1;				// must be 1-3
	info.fMemberBPVLimits[kTeam1]	= 400;
	info.fMemberSmallHulls[kTeam1]	= kClassFrigate;
	info.fMemberLargeHulls[kTeam1]	= kClassHeavyCruiser;

	// default AI settings:
	//	ships:		- 1
	//	small hull:	- kClassFrigate
	//	large hull:	- kClassHeavyCruiser
	//	BPV:		- 400
	//
	//	team 2:		- Klingon (or Romulan if player is Klingon)
	//	teams 3-6:	- disabled

	for (int32 i = kTeam2; i <= kTeam6; i++)
	{
		info.fMembersInGame[i]		= kPlayerTypeNoOne;
		info.fMemberTeams[i]		= eTeamID(i);
		info.fMemberRaces[i]		= kNoRaceType;
		info.fMemberShipCounts[i]	= 1;				// must be 1-3
		info.fMemberBPVLimits[i]	= 400;
		info.fMemberSmallHulls[i]	= kClassFrigate;
		info.fMemberLargeHulls[i]	= kClassHeavyCruiser;
	}
	info.fMembersInGame[kTeam2]		= kPlayerTypeAI;
	info.fMemberRaces[kTeam2]		= (info.fPlayerRace != kKlingon) ? kKlingon : kRomulan;

//	info.fMembersInGame[kTeam2]		= kPlayerTypeAI;
//	info.fMemberRaces[kTeam2]		= info.fPlayerRace;
//	info.fMemberLargeHulls[kTeam2]	= kClassFrigate;
//	info.fMemberTeams[kTeam2]		= kTeam1;
//
//	info.fMembersInGame[kTeam3]		= kPlayerTypeAI;
//	info.fMemberRaces[kTeam3]		= info.fPlayerRace;
//	info.fMemberLargeHulls[kTeam3]	= kClassFrigate;
//	info.fMemberTeams[kTeam3]		= kTeam1;
}
