#include "His_theDuel.h"

#include "MissionText.h"
#include "MissionMaps.h"

#include "VictoryCondition.h"
#include "PlayableTeam.h"

eRaceName PlayerRace;
bool EnemyDestroyed = false;
bool EnemyCaptured = false;
bool PlayerDestroyed = false;
bool Monster = false;

DEFINE_SCRIPT_CLASS( tHis_theDuel );

tHis_theDuel::tHis_theDuel()
{
}

tVictoryCondition* tHis_theDuel::mSetupVictoryCondition()
{
	fVictoryCondition = new tVictoryCondition( fPlayersTeam, kEnemyTeam, kCaptureBonus, 150, 100, 50, 25, 0 );
	gEventMgr.mRegisterActor( fVictoryCondition );							
	return fVictoryCondition;	
}

//Register all you states here. Then mGotoState on the first state to run
void tHis_theDuel::mSetupState()
{

}

//Register all you comm states here. 
void tHis_theDuel::mSetupCommState()
{
}

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tHis_theDuel::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tHis_theDuel::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecOneShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fMenuStates = kNoMenuButtons;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tHis_theDuel::mInitializeStart( tDynaverseScriptInfo& info )
{
	// must set up the races of the teams in the script
	fMissionInfo->mSetupTeam( fPlayersTeam, info.fPlayerRace, typeid( tPlayableTeam ) );
	fMissionInfo->mSetupTeam( kEnemyTeam, info.fEnemyRace );

	// setup team relations and target goals
	fMissionInfo->mSetTeamRelations( kEnemyTeam, fPlayersTeam, kTeamAtWar, kGoalDestroy, 1.0 );

	// determine players Race
	PlayerRace = info.fPlayerRace;

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tHis_theDuel::mCreateMissionShips( int32 Mode )
{
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( kEnemyTeam );
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( fPlayersTeam );

	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
//	PlayerRace = info.fPlayerRace;
/*
	playerTeam->mCreateShip( typeid( tShipInfo ), "R-SPA" , kStartPosition_G );
	enemyTeam->mCreateShip( typeid( tShipInfo ), "F-CA", kStartPosition_Z );
*/
	switch (diffLevel)
	{ 
	   case 0:
		   if( PlayerRace == kFederation)
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "F-CC+", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "K-D6", kStartPosition_Z );
			}
            else if( PlayerRace == kKlingon )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ) ,"K-D7L", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "F-CA", kStartPosition_Z );
			}
			else if( PlayerRace == kRomulan )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "R-K7R", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "G-CL", kStartPosition_Z );
			}
			else if( PlayerRace == kGorn )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ),"G-BC", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "R-WE", kStartPosition_Z );
			}
			else if( PlayerRace == kLyran )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "L-CC+", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "H-TR", kStartPosition_Z );
			}
			else if( PlayerRace == kHydran )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "H-LM", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "L-CL", kStartPosition_Z );
			}
			else
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), kMaxBPV, kClassLightCruiser, 0, kStartPosition_Z );
			}
			break;
		case 1:
			if( PlayerRace == kFederation )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "F-CA", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "K-D7", kStartPosition_Z );
			}
			else if( PlayerRace == kKlingon )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "K-D7", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "F-CA", kStartPosition_Z );
			}
			else if( PlayerRace == kRomulan )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "R-SPA", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "G-CA", kStartPosition_Z );
			}
			else if( PlayerRace == kGorn )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "G-CA", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "R-KR", kStartPosition_Z );
			}
			else if( PlayerRace == kLyran )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "L-CA", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "H-RN", kStartPosition_Z );
			}
			else if( PlayerRace == kHydran )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "H-RN", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "L-CA", kStartPosition_Z );
			}
			else
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_Z );
			}
			break;

		case 2:
			if( PlayerRace == kFederation )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "F-CA", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "K-D7L", kStartPosition_Z );
			}
			else if( PlayerRace == kKlingon )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "K-D6", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "F-CC+", kStartPosition_Z );
			}
			else if( PlayerRace == kRomulan )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "R-WE", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "G-BC", kStartPosition_Z );
			}
			else if( PlayerRace == kGorn )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "G-CL", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "R-K7R", kStartPosition_Z );
			}
			else if( PlayerRace == kLyran )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "L-CL", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "H-LM", kStartPosition_Z );
			}
			else if( PlayerRace == kHydran )
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), "H-TR", kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), "L-CC+", kStartPosition_Z );
			}
			else
			{
				playerTeam->mCreateShip( typeid( tShipInfo ), kMaxBPV, kClassLightCruiser, 0, kStartPosition_G );
				enemyTeam->mCreateShip( typeid( tShipInfo ), kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_Z );
			}
			break;
	}
}  	  
//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tHis_theDuel::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tHis_theDuel::mStartMission( void )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( fPlayersTeam );
	playerTeam->mGetFirstShip()->mSetShipPlayerControlled();
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tHis_theDuel::mNewMissionShip( tShipInfo* Ship )
{
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tHis_theDuel::mGetDebriefingList( eVictoryLevels VictoryLevel )
{
	fMissionInfo->mAddToDebriefingList( VictoryLevel + kAstoundingVictory_msg );
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tHis_theDuel::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}


