/********************************************************************************************************\

	File Name:	BaseVictory.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ISC_10Criminals.h"


tClassAllocator<tBaseVictory> gBaseVictoryAllocator;

/*

	HELPER NOTE:	You must remember to initialize all of your victory attributes here.  You should
					do this for both constructors until informed otherwise.  For example:

	tTeam1BaseVictory::tTeam1BaseVictory( ) : fBSed( false ), field2( value ), field3 ( value ), etc...
	{}

*/
tBaseVictory::tBaseVictory() :
	 fFreighterDestroyed( false ),
	 fEnemyShipsDestroyed( 0 ),
	 fSignalled( false ),
	 fDeserted( false ),
	 fFreighterScanned( false )
{}
tBaseVictory::tBaseVictory( tTeamInfo* team ) : tVictoryCondition( team ),
	 fFreighterDestroyed( false ),
	 fEnemyShipsDestroyed( 0 ),
	 fSignalled( false ),
	 fDeserted( false ),
	 fFreighterScanned( false )
{}

/*

	HELPER NOTE:	Really, the only items in this function that you could change are the values passed into
					the tBaseVictoryState constructor.  These values are the capture bonus and prestige
					points awarded according to the player's victory status -- starting at
					kAstoundingVictory and working its way down to kDevastatingDefeat.

*/
void tBaseVictory::mSetupState()
{
	tBaseVictoryState* baseVictoryState = new tBaseVictoryState( 300 , 225 , 75 , 0 , -75 );
	mRegisterState( baseVictoryState );
	mGotoState( typeid( tBaseVictoryState ) );

	//mSetTimedInterval( kTimerSecondsSet, kRandChatID, 20.0f );
}

/*

	HELPER NOTE:	The following is simply an example of setting and modifying the current victory condition
					objectives.

	bool
	tTeam1BaseVictory::mBSed( void )
	{
		return fBSed;
	}
	void
	tTeam1BaseVictory::mSetBSed( bool x )
	{
		fBSed = x;
	}

*/

//
//	Accessors.
//
bool tBaseVictory::mGetFreighterDestroyed()
{
	return fFreighterDestroyed;
}
int32 tBaseVictory::mGetEnemyShipsDestroyed()
{
	return fEnemyShipsDestroyed;
}
bool tBaseVictory::mGetSignalled()
{
	return fSignalled;
}
bool tBaseVictory::mGetDeserted()
{
	return fDeserted;
}
bool tBaseVictory::mFreighterScanned()
{
	return fFreighterScanned;
}
//
//	Modifiers.
//
void tBaseVictory::mSetFreighterDestroyed(bool x)
{
	
	fFreighterDestroyed = x;
	if(x)
	{

		CreateOrganian();
		fMissionInfo->mDisplayMessage(	(eTeamID) kPlayerTeam, 	kFreighterDestroyed_msg);
		// ** Organian arrives
		fMissionInfo->mDisplayMessage(	(eTeamID) kPlayerTeam, kOrganianSurrender1_msg);
		mDisableWeapons();
		mSetTimedInterval( kTimerSecondsSet, kEndGameID, 20.0f );
	}
}
void tBaseVictory::mIncEnemyShipsDestroyed()
{
	fEnemyShipsDestroyed++;
	if( fEnemyShipsDestroyed == 4)
	{
// ** Organian arrives

		// Create an Organian Monster here
		CreateOrganian();
		fMissionInfo->mDisplayMessage(	(eTeamID) kPlayerTeam, kOrganianSurrender1_msg	);
		mDisableWeapons();
		mSetTimedInterval( kTimerSecondsSet, kEndGameID, 10.0f );
		
	}

}
void tBaseVictory::mDisableWeapons()
{
	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,
		//Nme  = kEnemyTeam,
		//Org  = kOrganianTeam

		//	     Plyr  PNPC  Nme  
		/*Plyr*/ "--", "AI", "FI",
		/*PNPC*/ "AI", "--", "FI",
		/*Nme */ "FI", "FI", "--",
		


	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tBaseVictory::mSetSignalled( bool x)
{
	fSignalled = x;
	if(x)
	{
		CreateOrganian();
		fMissionInfo->mDisplayMessage(	(eTeamID) kPlayerTeam, 	kFreighterReachesPlanet_msg);
		// ** Organian arrives
		// Create an Organian Monster here
		fMissionInfo->mDisplayMessage(	(eTeamID) kPlayerTeam, kOrganianSurrender2_msg);
		mDisableWeapons();
		mSetTimedInterval( kTimerSecondsSet, kEndGameID, 10.0f );
	}

}
void tBaseVictory::mSetDeserted( bool x)
{
	fDeserted = x;
	if(x)
	{
		fMissionInfo->mDisplayMessage(	(eTeamID) kPlayerTeam, 	kLeftSystem_msg);
	}

}
void tBaseVictory::mSetFreighterScanned(bool x)
{
	fFreighterScanned = x;
}

void tBaseVictory::CreateOrganian()
{
	int32 OrganiaX, OrganiaY;
	int32 PlayerX, PlayerY;
	int32 NewX, NewY;

	tTeamInfo * util = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerNPCTeam);
	Assert(util);
	tShipIterator organia = util->mGetFirstShip();
	(*organia)->mGetGridPosition(&OrganiaX, &OrganiaY);
	util = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(util);

	tShipIterator player = util->mGetFirstShip();
	(*player)->mGetGridPosition(&PlayerX, &PlayerY);
	NewX = (PlayerX > OrganiaX) ? PlayerX - 1 : PlayerX + 1;
	NewY = (PlayerY > OrganiaY) ? PlayerY - 1 : PlayerY + 1;

//	fMissionInfo->mDebugMessage("Creating Organian");
	fMissionInfo->mCreateMapObject(*"^", NewX, NewY);
}