/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ISC_10Criminals.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if(fChildActor->mGetFreighterDestroyed() || (fChildActor->mGetEnemyShipsDestroyed() >= 4 ))
		return kAstoundingVictory;
	
	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	case tEnemyTeam::kFreighter:
		{
			fChildActor->mSetFreighterDestroyed(true);
			break;
		}
	case tEnemyTeam::kEnemyShip1:
	case tEnemyTeam::kEnemyShip2:
	case tEnemyTeam::kEnemyShip3:
	case tEnemyTeam::kEnemyShip4:
	case tEnemyTeam::kEnemyShip5:
	case tEnemyTeam::kEnemyShip6:
	case tEnemyTeam::kEnemyShip7:
	case tEnemyTeam::kEnemyShip8:
		fChildActor->mIncEnemyShipsDestroyed();
		break;

	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
	Assert( team );


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		if ( team )
		{
			if ( !team->mShipsRemainingInPlay() )
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
		}
		break;
	case tEnemyTeam::kEnemyShip1:
	case tEnemyTeam::kEnemyShip2:
	case tEnemyTeam::kEnemyShip3:
	case tEnemyTeam::kEnemyShip4:
	case tEnemyTeam::kEnemyShip5:
	case tEnemyTeam::kEnemyShip6:
	case tEnemyTeam::kEnemyShip7:
	case tEnemyTeam::kEnemyShip8:
		fChildActor->mIncEnemyShipsDestroyed();
		break;

	case tEnemyTeam::kFreighter:
		fChildActor->mSetFreighterDestroyed(true);
		break;

	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		fChildActor->mSetDeserted(true);
		gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if(ship->mGetCustomID() == tEnemyTeam::kFreighter)
	{
		fChildActor->mSetSignalled(true);
	}

}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn)
	{
		fScanning = true;
		mSetTimedInterval( kTimerTurnsSet, fChildActor->kScanID, 0.75f );
	}
	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, fChildActor->kScanID, 0.0f );
	}
}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	int32 kMessage;
	tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID) kEnemyTeam);
	Assert(enemy);
	tTeamInfo * player = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(player);
	tShipIterator ship = player->mGetFirstShip();
	tShipIterator target = enemy->mGetFirstShip();
	bool validtarget = false;
	static kRandChatFlag = 0;


	switch(timerID)
	{
	case fChildActor->kRandChatID:
		kMessage = fMissionInfo->mRandomInt32(kChatter1_msg, kChatter6_msg);
		fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kMessage);
		fChildActor->mSetTimedInterval(kTimerSecondsSet, fChildActor->kRandChatID, (float) 20.0 + fMissionInfo->mRandomInt32(1, 20));
		break;
	case fChildActor->kEndGameID:
		gScriptInterface.mSetGameStatus((eTeamID) kPlayerTeam, kPlayerWins);
		break;
	case fChildActor->kScanID:
		if( !fScanning )
			return;
		
		
		for(target; *target!= NULL; target++)
		{
			if((*ship)->mGetTacticalDistance((*target)) <= 15.0)
			{
				validtarget = true;
				if((*target)->mGetCustomID() == tEnemyTeam::kFreighter)
				{
					kMessage = kScanningFreighter_msg;
					fChildActor->mSetFreighterScanned(true);
					fScanning = false;
					break;
				}
			}
			if(validtarget)
				kMessage = kNothingAbnormal_msg;
			else
				kMessage = kOutOfScanRange_msg;
		}

		fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kMessage);
		if( ! fChildActor->mFreighterScanned())
			mSetTimedInterval( kTimerTurnsSet, fChildActor->kScanID, 0.2f );
		break;
	}
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

