/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonDefs.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyShip.h"
#include "Freighter.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "ISC_10Criminals.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;


void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, race );
		sl = gST.stripOutRestricted( sl );
        sl = gST.stripOutSPECIAL( sl );
		ShipList ship_sl = gST.selectWarShips(sl);

		// Strip out cargo roles from ships: don't want confusion!
		{
			std::list< eEnhancedRoles >  RolesList;
			RolesList.push_back(kCargo);
			fShiplistCount = ship_sl.Subset(RolesList, false);
			gST.mDebug_fprintf("Stripped out CARGO role ships completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(ship_sl);
		}


        // DNs and BBs
        ShipList DN_sl = ship_sl;
		{
			std::list< eEnhancedClassTypes >  DNClassTypeList;
			DNClassTypeList.push_back(kEClassLargeMonster);
			for (int32 x = kEClassDreadnought; x <= kEClassBattleship; x++) {
				DNClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
			}
			fShiplistCount = DN_sl.Subset(DNClassTypeList, true);
			gST.mDebug_fprintf("subset for DN/BB filter completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(DN_sl);
		}

		// CLs to BCHs
		ShipList Cruiser_sl = ship_sl;
		{
			std::list< eEnhancedClassTypes >  CruiserClassTypeList;
			CruiserClassTypeList.push_back(kEClassMediumMonster);
			for (int32 x = kEClassLightCruiser; x <= kEClassHeavyBattlecruiser; x++) {
				CruiserClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
			}
			fShiplistCount = Cruiser_sl.Subset(CruiserClassTypeList, true);
			gST.mDebug_fprintf("subset for Cruiser filter completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(Cruiser_sl);
		}

		// FFs, DDs, DWs.
		ShipList Frigate_sl = ship_sl;
		{
			std::list< eEnhancedClassTypes >  FrigateClassTypeList;
			FrigateClassTypeList.push_back(kEClassSmallMonster);
			for (int32 x = kEClassFrigate; x <= kEClassWarDestroyer; x++) {
				FrigateClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
			}
			fShiplistCount = Frigate_sl.Subset(FrigateClassTypeList, true);
			gST.mDebug_fprintf("subset for Frigate filter completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(Frigate_sl);
		}

		// Freighter.
		ShipList Freighter_sl = sl;
		{
			std::list< eEnhancedClassTypes >  FreighterClassTypeList;
			for (int32 x = kEClassPriorityTransport; x <= kEClassLargeFreighter; x++) {
				FreighterClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
			}
			fShiplistCount = Freighter_sl.Subset(FreighterClassTypeList, true);
			gST.mDebug_fprintf("subset for Freighter filter completed. ships: %d\n", fShiplistCount);

			std::list< eEnhancedRoles >  RolesList;
			RolesList.push_back(kCargo);
			fShiplistCount = Freighter_sl.Subset(RolesList, true);
			gST.mDebug_fprintf("subset for cargo freighters completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(Freighter_sl);


		}

		if (
			(DN_sl.size() > 0) &&
			(Cruiser_sl.size() > 0) &&
			(Frigate_sl.size() > 0) &&
			(Freighter_sl.size() > 0)
		) {

			int32 i;
			int32 index = 1;

			// Create the enemy flagship
			{
                Ship s1;
                DN_sl.SelectRandomShip(s1);
                gST.mTrackOrions( s1 );
                gST.mDebug_fprintf("DN/BB %s selected. BPV: %d  YFA: %d  YLA %d\n",
                    s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

				mCreateShip( typeid( tEnemyShip ),
					s1.shipCode.c_str(),
					kStartPosition_Z,
					kNoMetaShipID,
					kEnemyShip1,
					0, 0, -1, -1,
					NULL,
					kDefaultShipOptions );
			}

			// Cruisers
			for(i = kEnemyShip2; i <= kEnemyShip4; i++)
			{
				{
					Ship s1;
					Cruiser_sl.SelectRandomShip(s1);
					gST.mTrackOrions( s1 );
					gST.mDebug_fprintf("Cruiser %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

					mCreateShip( typeid( tEnemyShip ),
						s1.shipCode.c_str(),
						(kStartPosition_Z - index),
						kNoMetaShipID,
						i,
						0, 0, -1, -1,
						NULL,
						kDefaultShipOptions );
				}

				index++;
			}

			// Frigates
			for(i = kEnemyShip5; i <= kEnemyShip8; i++)
			{
				{
					Ship s1;
					Frigate_sl.SelectRandomShip(s1);
					gST.mTrackOrions( s1 );
					gST.mDebug_fprintf("Frigate %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

					mCreateShip( typeid( tEnemyShip ),
						s1.shipCode.c_str(),
						(kStartPosition_Z - index),
						kNoMetaShipID,
						i,
						0, 0, -1, -1,
						NULL,
						kDefaultShipOptions );
				}

				index++;
			}

			// Freighter
			{
				Ship s1;
				Freighter_sl.SelectRandomShip(s1);
				gST.mTrackOrions( s1 );
				gST.mDebug_fprintf("Freighter %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

				mCreateShip( typeid( tFreighter ),
					s1.shipCode.c_str(),
					(kStartPosition_Z - index),
					kNoMetaShipID,
					kFreighter,
					0, 0, -1, -1,
					NULL,
					kDefaultShipOptions );
			}

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	int32 i;
	int32 randomclass;
	int32 index = 1;

	// Create the enemy flagship
	mCreateShip(	typeid( tEnemyShip ),
				kMaxBPV,
				kClassDreadnought,
				kClassDreadnought,
				0,
				kStartPosition_Z,
				kEnemyShip1,
				0, 0, -1, -1, NULL,
				kDefaultShipOptions );


	for(i = kEnemyShip2; i <= kEnemyShip4; i++)
	{
		randomclass = fMissionInfo->mRandomInt32(kClassLightCruiser, kClassHeavyBattlecruiser);
		
		mCreateShip(	typeid( tEnemyShip ),
					kMaxBPV,
					(eClassTypes) randomclass,
					(eClassTypes) randomclass,
					0,
					(kStartPosition_Z - index),
					i,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		index++;
	}

	for(i = kEnemyShip5; i <= kEnemyShip8; i++)
	{
		randomclass = fMissionInfo->mRandomInt32(kClassFrigate, kClassDestroyer);
		
		mCreateShip(	typeid( tEnemyShip ),
						kMaxBPV,
						(eClassTypes) randomclass,
						(eClassTypes) randomclass,
						0,
						(kStartPosition_Z - index),
						i,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );

		index++;
	}

	mCreateShip(	typeid( tFreighter ),
				kMaxBPV,
				kClassFreighter,
				kClassFreighter,
				0,
				kStartPosition_Z - index,
				kFreighter,
				0, 0, -1, -1, NULL,
				kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}



void tEnemyTeam::mTeamSetupComplete()
{
	int32 X, Y;

	tTeamInfo* EnemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kEnemyTeam ) );
	tShipInfo * freighter = EnemyTeam->mFindShipCustomID(tEnemyTeam::kFreighter);
	Assert(freighter);
	
	tShipInfo * Organia = *(fMissionInfo->mGetTeamHandle((eTeamID) kPlayerNPCTeam)->mGetFirstShip());
	if(Organia != NULL)
	{
		Organia->mGetGridPosition(&X, &Y);
		freighter->mSetFinalDestination(X, Y);
		freighter->mTrackTacticalDistance(SIGNAL_RADIUS, X, Y);
	}
	freighter->mSetShipThrust(15);
	freighter->mSetShipMaxSpeed(15);
	freighter->mSetShipAIControlled(kTrue);


	if (EnemyTeam) {
		tShipIterator tsi = EnemyTeam->mGetFirstShip();
		for (;;)
		{
			Ship aShip;
			tShipInfo *si = *tsi;
			if (!si)
				break;


			if ( 
				(si->mGetCustomID() != tEnemyTeam::kFreighter) &&
				(
					// Admiral level: smaller ships to attack directly..
					(si->mGetCustomID() <= tEnemyTeam::kEnemyShip4) ||
					(fMissionInfo->mGetDifficultyLevel() < 2)
				)
			) {
				std::string shipCode = si->mGetClassName();
				gST.mDebug_fprintf("Ship %s: defend the freighter!\n", shipCode.c_str());

				si->mSetShipThrust(15);
				si->mSetShipMaxSpeed(15);
				si->mSetFinalDestination(X, Y);
				si->mDefendEntity(freighter, 30.0, 5.0, kVeryHighPriority);  // for some reason, this screws up AI. Gotta give it same speed and destination.
				si->mSetShipAIControlled(kTrue);
			}

			tsi++;
		}
	}

	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}
