/********************************************************************************************************\

	Mission:	War Criminals
	Script:		"ISC_10Criminals"
	Descript:	?

	Created by:		Dutch Blomenkamp	?
	Last Updated:	DJS					09/06/00
					Added tactical icon code and kPrimaryTeam
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "ISC_10Criminals.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tISC_10Criminals );

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tISC_10Criminals::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	gST.mDebug_fprintf("Creating Team 1 Spec\n");
	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	gST.mDebug_fprintf("Creating Team 2 Spec\n");
	const tTeamSpec& team2 = mCreateTeamSpec(	"Organian Team",
												static_cast< eTeamID >( kPlayerNPCTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tPlayerNPCTeam ) );

	gST.mDebug_fprintf("Creating Team 3 Spec\n");
	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak,
												kAnyEnemyOf | kPlayerTeam,
												typeid( tEnemyTeam ) );

	gST.mDebug_fprintf("Creating Team 4 Spec\n");
	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
												static_cast< eTeamID >( kDraft1Team ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tDraftTeam ) );
   
	gST.mDebug_fprintf("Creating Team 5 Spec\n");
	const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
												static_cast< eTeamID >( kDraft2Team ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tDraftTeam ) );

	gST.mDebug_fprintf("Creating Team 6 Spec\n");
	const tTeamSpec& draft3Team = mCreateTeamSpec(  "Draft 3 Team",
												static_cast< eTeamID >( kDraft3Team ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tDraftTeam ) );

	
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
	Specs->mSetupTeam( draft3Team );
	
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tISC_10Criminals::mInitializeStart( tDynaverseScriptInfo& info )
{
	gST.mDebug_fprintf("tISC_10Criminals::mInitializeStart\n");
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tISC_10Criminals::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tISC_10Criminals::mInitializeFinished( void )
{
	gST.mDebug_fprintf("tISC_10Criminals::mInitializeFinished\n");
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tISC_10Criminals::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tISC_10Criminals::mStartMission()
{
	gST.mDebug_fprintf("--- Starting: tISC_10Criminals::mStartMission\n");

	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,
		//Nme  = kEnemyTeam,

		//	     Plyr  PNPC  Nme 
		/*Plyr*/ "--", "FI", "WD",
		/*PNPC*/ "FI", "--", "NI",
		/*Nme */ "WD", "NI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
	gST.mDebug_fprintf("--- Finished: tISC_10Criminals::mStartMission\n");
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_10Criminals::mASCIIOverrides( tMapOverrides* Map )
{
	gST.mDebug_fprintf("--- Starting: tISC_10Criminals::mASCIIOverrides\n");

	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['K'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['M'] = kObjectPlanet;

	Map->fTacticalMapIcon['Q'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['R'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['S'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;

	gST.mDebug_fprintf("--- Finished: tISC_10Criminals::mASCIIOverrides\n");
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_10Criminals::mDefineBriefings( void )
{
	gST.mDebug_fprintf("--- Starting: tISC_10Criminals::mDefineBriefings\n");

	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;

	gST.mDebug_fprintf("--- Finished: tISC_10Criminals::mDefineBriefings\n");
}


void tISC_10Criminals::mDefineTeamShipStrengths( void )
{
	gST.mDebug_fprintf("--- Starting: tISC_10Criminals::mDefineTeamShipStrengths\n");

	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	gST.mDebug_fprintf("--- Finished: tISC_10Criminals::mDefineTeamShipStrengths\n");
}