//	<Name>	FedTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ISC_1Peace.h"
#include "FedTeam.h"
#include "FedTeamShip.h"
#include "PeaceVictory.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tFedTeam> gFedTeamAllocator;

tFedTeam::tFedTeam() : tTeamInfo()
{
}

void tFedTeam::mSetupState()
{

}


void tFedTeam::mSetupCommState()
{

}


void tFedTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tFedTeam::mCreateShipsForTeam\n");
   	gST.mCompleteNarrowedList();

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.6;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 0.8;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.0;
	}

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.selectWarShips(gST.narrowed_sl);
		sl = gST.mFilterShipListToSingleRace( sl, mGetRace() );
		sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

		fShiplistCount = sl.Subset( sl.kSubsetBPV, int(0.825 * nPCBPV), int(1.175 * nPCBPV), true);
        gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
	        int(0.825 * nPCBPV), int(1.175 * nPCBPV), fShiplistCount);

		Ship aShip;
		if ((sl.size() > 0) && (sl.SelectRandomShip(aShip))) {
			gST.mDebug_fprintf("aShip %s selected. BPV: %d  YFA: %d  YLA %d\n",
				aShip.shipCode.c_str(), aShip.bpv, aShip.yearFirstAvail, aShip.yearLastAvail);
			gST.mTrackOrions( aShip );

            mCreateShip(
                typeid( tFedTeamShip ),
                aShip.shipCode.c_str(),
                kStartPosition_K,
                kNoMetaShipID,
                -1, 0, 0, -1, -1, NULL,
                kDefaultShipOptions );
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tFedTeam::mCreateShipsForTeam\n");
}


void tFedTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tFedTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.6;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 0.8;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.0;
	}

	mCreateShip( typeid( tFedTeamShip ), nPCBPV, kClassFrigate, kClassHeavyBattlecruiser, 0, kStartPosition_K,
				-1, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tFedTeam::mOldCreateShipsForTeam\n");
}


void tFedTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tFedTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
	int32 metaid = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team: %d\n", mGetTeam());
		metaid = kNoMetaShipID;
	}
#endif

   // don't actually draft it. We're just collecting era data!
	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

	gST.mDebug_fprintf("--- Finished: tFedTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}

