#include "ISC_1Peace.h"

#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "PlayerTeam.h"
#include "OrganianTeam.h"
#include "FedTeam.h"
#include "KlingonTeam.h"
#include "RomulanTeam.h"
#include "EnemyTeam.h"
#include "PeaceVictory.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tISC_1Peace );

tISC_1Peace::tISC_1Peace()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tISC_1Peace::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;
	

	// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID>( kPlayerTeam ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tPlayerTeam ), kNoTeamTag, Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID>( kEnemyTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecLyran,
														kAnyRace,
														typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", static_cast< eTeamID>( kFedTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecFed,
															kAnyRace,
															typeid( tFedTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", static_cast< eTeamID>( kKlingonTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecKling,
															kAnyRace,
															typeid( tKlingonTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5", static_cast< eTeamID>( kRomulanTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecRomulan,
															kAnyRace,
															typeid( tRomulanTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6", static_cast< eTeamID>( kOrganianTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecAnyRace,
															kAnyRace,
															typeid( tOrganianTeam ) );


	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tISC_1Peace::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tISC_1Peace::mCreateExternalShips( int32 Mode )
{

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tISC_1Peace::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tISC_1Peace::mStartMission( void )
{
	//Must call mStartMission();
	tScript::mStartMission();
	
	// setup team relations and target goals
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kFedTeam,
		kKlingonTeam,
		kRomulanTeam,
		kOrganianTeam
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr	= fPlayersTeam,
		//Enemy = kEnemyTeam, Lyrans
		//Fed   = kFedTeam,
		//Klin  = kKlingonTeam,
		//Rom   = kRomulanTeam,
		//Orgn	= kOrganian

		//			Plyr	Enemy	Fed		Klin	Rom		Organian  
		/*Plyr*/	"--",	"WD",	"HI",	"WD",	"HI",	"AI",
		/*Enemy*/	"WD",	"--",	"FI",	"FI",	"FI",	"AI",
		/*Fed*/ 	"HI",	"FI",	"--",	"FI",	"FI",	"AI",
		/*Klin*/	"HI",	"FI",	"FI",	"--",	"FI",	"AI",
		/*Rom*/		"HI",	"FI",	"FI",	"FI",	"--",	"AI",
		/*Orgn*/	"AI",	"AI",	"AI",	"AI",	"AI",	"--",	
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tISC_1Peace::mNewInternalShip( tShipInfo* Ship )
{
}

void tISC_1Peace::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;

	Map->fTacticalMapIcon['^'] = kObjectOrganian;

	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['I'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['K'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['J'] = kObjectEnemyShip;
}

void tISC_1Peace::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}

void tISC_1Peace::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}

