//	<Name>	PeaceVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "PeaceVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "ISC_1Peace.h"
#include "PeaceVictory.h"
#include "MissionText.h"
#include "FedTeamShip.h"
#include "KlingonTeamShip.h"
#include "RomulanTeamShip.h"
                               
tPeaceVictoryState::tPeaceVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tPeaceVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fNPCsDestroyed = 0;
	fNumberofTruces = 0;
	fRomTruce = false;
	fFedsInPlay = 1;
	fRomsInPlay = 1;
	fKlingonsInPlay = 1;
	fLyransInPlay = 1;
	fPlayerShipsInPlay = 1;
}

void tPeaceVictoryState::mBeginState()
{

}

void tPeaceVictoryState::mEndState()
{
}

eVictoryLevels tPeaceVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	bool fedTruce     = fChildActor->mGetFedTruce();
	bool klingonTruce = fChildActor->mGetKlingonTruce();
	bool romulanTruce = fChildActor->mGetRomulanTruce();

	fNPCsDestroyed  = 4 - ( fFedsInPlay + fRomsInPlay + fKlingonsInPlay + fLyransInPlay );


    if(	( fedTruce ) &&  ( ( fNPCsDestroyed > 1 ) || ( fNumberofTruces > 1 ) ) )
	{
		return kAstoundingVictory;
	}
	else if( ( fNPCsDestroyed > 0 ) && ( fedTruce ) )
	{
		return kVictory;
	}
	else if( ( fNPCsDestroyed > 0 ) && ( !fedTruce ) )
	{
		return kDraw;
	}
	else if( ( fNPCsDestroyed == 0 ) && fedTruce )
	{
		return kDefeat;
	}
	else 
	{
		return kDevastatingDefeat;
	}

}
int32 tPeaceVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tPeaceVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	tTeamInfo* fedTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kFedTeam ) );
	tTeamInfo* klingonTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kKlingonTeam ) );
	tTeamInfo* romulanTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kRomulanTeam ) );
	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kEnemyTeam ) );
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );


	if( ship->mGetOriginalTeam() == kFedTeam )
	{
		fFedsInPlay = mShipsInPlay( fedTeam );
	}
	else if( ship->mGetOriginalTeam() == kKlingonTeam )
	{
		fKlingonsInPlay = mShipsInPlay( klingonTeam );
	}
	else if( ship->mGetOriginalTeam() == kRomulanTeam )
	{
		fRomsInPlay = mShipsInPlay( romulanTeam );
	}
	else if( ship->mGetOriginalTeam() == kEnemyTeam )
	{
		fLyransInPlay = mShipsInPlay( lyranTeam );
	}
	else if( ship->mGetOriginalTeam() == kPlayerTeam )
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );
	}
}
void tPeaceVictoryState::mOnDeath( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

	mCheckAllTeams( ship );

	//see how many ships are still in play
	fNPCsDestroyed  = 4 - ( fFedsInPlay + fRomsInPlay + fKlingonsInPlay + fLyransInPlay );

	if ( ( ship->mGetOriginalTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		if ( fPlayerShipsInPlay < 1 )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerDestroyed );
		}
	}

	// Check to see if game is complete
	if ( ( fNumberofTruces + fNPCsDestroyed ) > 3 )
	{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
	}

}

void tPeaceVictoryState::mOnCapture( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

	mCheckAllTeams( ship );

	if( ship->mGetTeam() == playerTeam->mGetTeam() )
	{
		mAddToPrestige( 30 );
	}

	//see how many ships are still in play
	fNPCsDestroyed  = 4 - ( fFedsInPlay + fRomsInPlay + fKlingonsInPlay + fLyransInPlay );

	if ( ( ship->mGetTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		if ( fPlayerShipsInPlay == 0 )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerDestroyed );
		}
	}
	// Check to see if game is complete
	if ( ( fNumberofTruces + fNPCsDestroyed ) > 3 )
	{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
	}
}

void tPeaceVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();
	int32 fThisTeamShipsInPlay;	
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

	if( playerTeam )
	{
		if ( playerTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( playerTeam );

			if( fThisTeamShipsInPlay < 1 )
			{
				if ( fNPCsDestroyed == 0 )
				{
					gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
				}
				else
				{
					gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
				}
			}
		}
	}
}
void tPeaceVictoryState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{
}
void tPeaceVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	//Check for a new Klingon Truce
	if( buttonID == kPowerTwiceKlingon_btn )
	{
		fChildActor->mSetKlingonTruce( true );
		fNumberofTruces++;
	}
	else if( buttonID == kPeaceAfterTruceKlingon_btn) // Check for Mad Klingon
	{
		fChildActor->mSetKlingonTruce( false );
		fNumberofTruces--;
	}
	else if( buttonID == kHailRomReady_btn ) //Check for Romulan Truce
	{
		fChildActor->mSetRomulanTruce( true );
		fNumberofTruces++;
	}
	else if( buttonID == kReassureFed_btn )
	{
		fChildActor->mSetFedTruce( true );
		fNumberofTruces++;
	}

		// check to see if all teams are at truce or destroyed
	if ( ( fNumberofTruces + fNPCsDestroyed ) > 3 )
	{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
	}
	// If there is a truce with both the Klinks and Feds the Roms are Ready 
	if ( (!fRomTruce ) && ( fChildActor->mGetFedTruce() ) && ( ( fNPCsDestroyed > 1 ) || ( fChildActor->mGetKlingonTruce() ) ) )
	{
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
		tTeamInfo* romTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kRomulanTeam ) );
		tShipIterator ship = romTeam->mGetFirstShip();
		tShipInfo* romShip = *ship;

		fRomTruce = true;
		if( romShip )
		{
			playerTeam->mGotoCommState( romShip->mGetShipID(), tRomulanTeamShip::kRomReadyCommState);
		}
	}
}