//	<Name>	PlayerTeam.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "ISC_1Peace.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamInitialState.h"
#include "PeaceVictory.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tPeaceVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels VictoryLevel )
{
	if( VictoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kAstoundingVictory_msg );
	}
	else if( VictoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kVictory_msg );
	}
	else if( VictoryLevel == kDraw )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kDraw_msg );
	}
	else if( VictoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kDefeat_msg );
	}
	else if( VictoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kDevastatingDefeat_msg );
	}
	else if( VictoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kLeftEarly_msg );
	}
}
void tPlayerTeam::mCreateShipsForTeam()
{
}
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamInitialState( ) );
	mGotoState( typeid( tPlayerTeamInitialState ) );
}
void tPlayerTeam::mSetupCommState()
{
}
void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}


}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

	// Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
	int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
	if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
		gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
		MetaID = kNoMetaShipID;
	}
#endif

	mCreateShip( typeid( tPlayerTeamShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 -1, 0, 0, -1, -1, //Default data,so we can set the custom name
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanBeCarrier | kStartPositionCanBeOffset ) );

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;
	fMissionScheduler.fNextMissionTitle = "ISC_2Promises";

	switch ( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
		fMissionScheduler.fMedal = kMedalSpecialOne;
		break;
	}
}