//	<Name>	PlayerTeamInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "ISC_1Peace.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "PlayerTeamInitialState.h"
#include "FedTeamShip.h"
#include "KlingonTeamShip.h"
#include "RomulanTeamShip.h"
#include "EnemyTeamShip.h"
#include "ShipList_API.h"


tPlayerTeamInitialState::tPlayerTeamInitialState( )
{
	fDeepScanON = false;
}

void tPlayerTeamInitialState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 10.0f );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}

void tPlayerTeamInitialState::mEndState()
{
}
void tPlayerTeamInitialState::mOnTimedInterval( int32 timerID )	
{
	tTeamInfo* fedTeam		= fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kFedTeam ) );
	tTeamInfo* klingonTeam	= fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kKlingonTeam ) );
	tTeamInfo* romulanTeam	= fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kRomulanTeam ) );
	tTeamInfo* enemyTeam	= fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kEnemyTeam ) );
	tTeamInfo* playerTeam	= fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
	tShipIterator ship = fedTeam->mGetFirstShip();
	tShipInfo* fedShip = *ship;
	ship = klingonTeam->mGetFirstShip();
	tShipInfo* klingonShip = *ship;
	ship = romulanTeam->mGetFirstShip();
	tShipInfo* romulanShip = *ship;
	ship = enemyTeam->mGetFirstShip();
	tShipInfo* enemyShip = *ship;

	switch( timerID )
	{
		case kStartingID:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kStarting_msg, kNonCommScriptMessage );
			mSetTimedInterval( kTimerSecondsSet, kOrganian1ID, 6.0f );	
		break;
		case kOrganian1ID:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kOrganian1_msg );
			mSetTimedInterval( kTimerSecondsSet, kOrganian2ID, 18.0f );
		break;
		case kOrganian2ID:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kOrganian2_msg );
			mSetTimedInterval( kTimerSecondsSet, kUltimatumID, 10.0f );
		break;
		case kUltimatumID:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kISCUltimatum1_msg );
			mSetTimedInterval( kTimerSecondsSet, kNonCompliance, 15.0f );	
			fMissionInfo->mSetTeamRelations( ( klingonTeam->mGetTeam() ), ( playerTeam->mGetTeam() ), kTeamAtWar, kGoalDestroy, 1.0, kTrue );	
		break;
		case kNonCompliance:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kNonCompliance_msg );
			if( fedShip )
			{
				playerTeam->mGotoCommState( fedShip->mGetShipID(), tFedTeamShip::kHailCommState );
			}
			if( klingonShip )
			{
				playerTeam->mGotoCommState( klingonShip->mGetShipID(), tKlingonTeamShip::kHailCommState );
				klingonShip->mClearCommands();
			}
			if( romulanShip )
			{
				playerTeam->mGotoCommState( romulanShip->mGetShipID(), tRomulanTeamShip::kHailCommState );
			}
			if( enemyShip )
			{
				playerTeam->mGotoCommState( enemyShip->mGetShipID(), tEnemyTeamShip::kDiePigCommState );
			}
		break;
		case kDeepScanONID:
			if( fDeepScanON )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kScanning_msg, kScienceScriptMessage );
			}
		break;
		case kUpgradeFightersID:
#ifdef FIGHTERS
			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
			if (playerTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
			if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
			if (playerTeam->mIsTeamHumanControlled()) return;               // only for AI teams
			if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC. 

			gST.mDebug_fprintf("--- Entering tPlayerTeamInitialState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

			int32 BPVinc = gST.mUpgradeFightersForTeam(playerTeam);

			gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc);
			gST.mDebug_fprintf("--- Finished: tPlayerTeamInitialState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
		break;
	}

}
void tPlayerTeamInitialState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fDeepScanON = true;
	}
	else
	{
		fDeepScanON = false;
	}
	ship->mSetShipCallbackEvents(  kNotifyDeepScan );
	mSetTimedInterval( kTimerTurnsSet, kDeepScanONID, 0.75f );
}