//	<Name>	EnemyTeam.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "ISC_2Promises.h"
#include "EnemyTeam.h"
#include "EnemyTeamCruiser.h"
#include "EnemyTeamInitialState.h"
#include "FreighterTeamShip.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

tEnemyTeam::tEnemyTeam() : tTeamInfo()
{
}

void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamInitialState( ) );
	mGotoState( typeid( tEnemyTeamInitialState ) );
}

void tEnemyTeam::mSetupCommState()
{
}


void tEnemyTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
   	gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 nPCBPV;
		int32 freighterBPV = 0;
		std::list< eEnhancedClassTypes >  freighterClassList;
		std::list< eEnhancedRoles >  freighterRolesList;

		if( diffLevel == 0 )
		{
			nPCBPV = playerBPV * 0.4;
			freighterClassList.push_back(kEClassSmallFreighter);
			freighterClassList.push_back(kEClassPriorityTransport);

			freighterRolesList.push_back(kCargo);
		}
		else if( diffLevel == 1 )
		{
			nPCBPV = playerBPV * 0.6;
			freighterClassList.push_back(kEClassSmallFreighter);
			freighterClassList.push_back(kEClassLargeFreighter);
			freighterClassList.push_back(kEClassPriorityTransport);

			freighterRolesList.push_back(kCargo);
			freighterRolesList.push_back(kQship);
			freighterBPV = 65;
		}
		else if( diffLevel == 2 )
		{
			nPCBPV = playerBPV * 0.8;
			freighterClassList.push_back(kEClassLargeFreighter);
			freighterClassList.push_back(kEClassLargeAuxiliary);

			freighterRolesList.push_back(kQship);
			freighterRolesList.push_back(kLineShip);
			freighterRolesList.push_back(kWarShip);
			freighterRolesList.push_back(kLeader);
			freighterRolesList.push_back(kDroneBombardmentShip);
			freighterRolesList.push_back(kEscort);
			freighterRolesList.push_back(kCarrier);
			freighterRolesList.push_back(kPFTender);
			freighterRolesList.push_back(kFastCruiser);
			freighterRolesList.push_back(kMauler);
			freighterRolesList.push_back(kPolice);
			freighterBPV = kMaxBPV;
		}

		ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
		sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

		ShipList freighter_sl = sl;
		sl = gST.selectWarShips(sl);

		fShiplistCount = freighter_sl.Subset(freighterClassList);
		gST.mDebug_fprintf("freighter class subset: %d\n", fShiplistCount);
		fShiplistCount = freighter_sl.Subset(freighterRolesList);
        gST.mDebug_fprintf("freighter roles subset: %d\n", fShiplistCount);

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassHeavyBattlecruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);


		fShiplistCount = sl.Subset( sl.kSubsetBPV, int(0.825 * nPCBPV), int(1.175 * nPCBPV), true);
        gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
	        int(0.825 * nPCBPV), int(1.175 * nPCBPV), fShiplistCount);

		Ship s1, s2, s3;
		if (sl.size() > 0) {
			sl.SelectRandomShip(s1);
			gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
			gST.mTrackOrions( s1 );

			sl.SelectRandomShip(s2);
			gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
			gST.mTrackOrions( s2 );

            mCreateShip(
                typeid( tEnemyTeamCruiser ),
                s1.shipCode.c_str(),
                kStartPosition_S,
                kNoMetaShipID,
                kEnemyOneID, 
				0, 0, -1, -1, NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tEnemyTeamCruiser ),
                s2.shipCode.c_str(),
                kStartPosition_S,
                kNoMetaShipID,
                kEnemyTwoID, 
				0, 0, -1, -1, NULL,
                kDefaultShipOptions );

			if (freighter_sl.size() > 0) {
				freighter_sl.SelectRandomShip(s3);
				gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
				gST.mTrackOrions( s3 );

				mCreateShip(
					typeid( tFreighterTeamShip ),
					s3.shipCode.c_str(),
					kStartPosition_T,
					kNoMetaShipID,
					kFreighterID, 
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

			} else {
				gST.mDebug_fprintf("oops, no freighters in freighter_sl list: using old style freighter creation\n");
				mCreateShip(	typeid( tFreighterTeamShip ), freighterBPV, kClassFreighter, kClassFreighter,
					0, kStartPosition_T, kFreighterID, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			}

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	int32 playerBPV = playerTeam->mGetCombatBPV();

	int32 freighterBPV; 
	int32 enemyBPV; 
	if( diffLevel == 0 )
	{
		enemyBPV = playerBPV * 0.4;
		freighterBPV = 0;
	}
	else if( diffLevel == 1 )
	{
		enemyBPV = playerBPV * 0.6;
		freighterBPV = 65;
	}
	else if( diffLevel == 2 )
	{
		enemyBPV = playerBPV * 0.8;
		freighterBPV = kMaxBPV;
	}
	
	mCreateShip(	typeid( tEnemyTeamCruiser ), enemyBPV, kClassFrigate, kClassHeavyBattlecruiser,
					30, kStartPosition_S, kEnemyOneID, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	mCreateShip(	typeid( tEnemyTeamCruiser ), enemyBPV, kClassFrigate, kClassHeavyBattlecruiser,
					30, kStartPosition_S, kEnemyTwoID, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	mCreateShip(	typeid( tFreighterTeamShip ), freighterBPV, kClassFreighter, kClassFreighter,
					0, kStartPosition_T, kFreighterID, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}


void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}