#include "ISC_2Promises.h"

#include "MissionText.h"
#include "MissionMaps.h"

#include "VictoryCondition.h"

#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "PlanetTeam.h"
#include "ISCNPCTeam.h"
#include "FreighterTeam.h"
#include "PromisesVictory.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tISC_2Promises );

tISC_2Promises::tISC_2Promises()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tISC_2Promises::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

/*	
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fSectorType = kAllSectors;//kEnemyBorder | kUnexplored | kWarzone | kOccupiedColonial | kEnemyColonial;
	Specs->fPlayerRace = kSpecAnyRace;//kSpecISC;
	Specs->fEnemyRace = kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecMirak | kSpecHydran;
	Specs->fNumShips = kMissSpecThreeShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 0;
	Specs->fMaxDate = 40;
	Specs->fWait = 6;
	Specs->fMenuStates = kNoMenuButtons;
	Specs->fBandwidthPart = kAllBand;
*/
	// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID>( kPlayerTeam ),
															kMaxChance,
															kTrue,
															kPrimaryTeam,
															kMaxChance,
															kSpecAnyRace,
															kAnyRace,
															typeid( tPlayerTeam ), kNoTeamTag, Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID>( kEnemyTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecMirak | kSpecHydran,
															kAnyRace,
															typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", static_cast< eTeamID>( kPlanetTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecAnyRace,
															kAnyRace,
															typeid( tPlanetTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", static_cast< eTeamID>( kDraftTeam1 ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecAnyRace,
															kAnyRace,
															typeid( tDraftTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5", static_cast< eTeamID>( kDraftTeam2 ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecAnyRace,
															kAnyRace,
															typeid( tDraftTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6", static_cast< eTeamID>( kDraftTeam3 ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecAnyRace,
															kAnyRace,
															typeid( tDraftTeam ) );



	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );

}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tISC_2Promises::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tISC_2Promises::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tISC_2Promises::mCreateExternalShips( int32 Mode )
{

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tISC_2Promises::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tISC_2Promises::mStartMission( void )
{
	tScript::mStartMission();

	// setup team relations and target goals
	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kPlanetTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr	= fPlayersTeam,
		//Enemy = kEnemyTeam,
		//PLNT  = kPlanetTeam,
		//NPC   = kISCNPCTeam,
		//Frtr	= kFreighterTeam,

		//			Plyr	Enemy	PLNT 
		/*Plyr*/	"--",	"WD",	"AI",
		/*Enemy*/	"WI",	"--",	"AI",
		/*PLNT*/	"AI",	"AI",	"--",


	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tISC_2Promises::mNewInternalShip( tShipInfo* Ship )
{
}

void tISC_2Promises::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;

	Map->fTacticalMapIcon['S'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;

}
void tISC_2Promises::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kEnemyTeam, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kPlanetTeam, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlanetTeam, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlanetTeam, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDraftTeam1, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDraftTeam1, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDraftTeam1, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDraftTeam2, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDraftTeam2, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDraftTeam2, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDraftTeam3, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDraftTeam3, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDraftTeam3, kClassFrigate, kClassHeavyBattlecruiser, kMinBPV, kMaxBPV );
}
void tISC_2Promises::mDefineBriefings( void )
{
		fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}

