//	<Name>	PromisesVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "PromisesVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "ISC_2Promises.h"
#include "PromisesVictory.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "FreighterTeamShipReachedPlanetState.h"

              
tPromisesVictoryState::tPromisesVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tPromisesVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fEnemyCruisersDestoyed = 0;
	fPlayerShipsInPlay = 1;
	fEnemyFreighterDestroyed = false;
	fPrimaryObjectiveMet = true;		// if the enemy reaches the planet becomes false
	fFreighterScanned = false;
	fEnemyOneDestroyed = false;
	fEnemyTwoDestroyed = false;
	fEnemyOneCaptured = false;
	fEnemyTwoCaptured = false;
	fEnemyFreighterCaptured = false;
}

void tPromisesVictoryState::mBeginState()
{

}
eVictoryLevels tPromisesVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>(  fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) ) );
	Assert( playerTeam );

	fFreighterScanned = playerTeam->mGetEnemyFreighterScanned();

	if( ( fPlayerShipsInPlay < 1 ) && 
		( ! fEnemyFreighterCaptured || ! fEnemyFreighterDestroyed ) )
	{
		return kDevastatingDefeat;
	}
	if( fPrimaryObjectiveMet && fFreighterScanned )
	{
		return kAstoundingVictory;
	}
	else if( fPrimaryObjectiveMet ) 
	{
		return kVictory;
	}
	else if( ( fEnemyOneDestroyed || fEnemyOneCaptured ) && ( fEnemyOneDestroyed || fEnemyOneCaptured ) && ( fEnemyTwoDestroyed || fEnemyTwoCaptured ) )
	{
		return kDefeat;
	}
	else
	{
		return kDevastatingDefeat;
	}

}
int32 tPromisesVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tPromisesVictoryState::mOnDeath( tShipInfo* ship )
{

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
	tShipIterator pship = playerTeam->mGetFirstShip();
	tShipInfo* playerShip = *pship;

	bool enemyOneOut = false;
	bool enemyTwoOut = false;
	bool enemyFreighterOut = false;


	if( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyOneID ) ) )
	{
		fEnemyOneDestroyed = true;
	}
	else if( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyTwoID ) ) )
	{
		fEnemyTwoDestroyed = true;
	}
	else if( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kFreighterID ) ) )
	{
		fEnemyFreighterDestroyed = true;
	}
	else if ( ( ship->mGetOriginalTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );

		if( fPlayerShipsInPlay < 1 )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerDestroyed );
		}
	}

	if ( fEnemyOneDestroyed || fEnemyOneCaptured )
	{
		enemyOneOut = true;
	}
	if ( fEnemyTwoDestroyed || fEnemyTwoCaptured )
	{
		enemyTwoOut = true;
	}
	if ( fEnemyFreighterCaptured || fEnemyFreighterDestroyed  )
	{
		enemyFreighterOut = true;
	}
	if ( fPrimaryObjectiveMet && enemyOneOut && enemyTwoOut && enemyFreighterOut )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
	}
	else if ( enemyOneOut && enemyTwoOut && enemyFreighterOut )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerLoses );
	}
}

void tPromisesVictoryState::mOnCapture( tShipInfo* ship )
{
	bool enemyOneOut = false;
	bool enemyTwoOut = false;
	bool enemyFreighterOut = false;


	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

	if( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyOneID ) ) )
	{
		fEnemyOneCaptured = true;
	}
	else if( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyTwoID ) ) )
	{
		fEnemyTwoCaptured = true;
	}
	else if( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kFreighterID ) ) )
	{
		fEnemyFreighterCaptured = true;
	}
	else if ( ( ship->mGetOriginalTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );

		if( fPlayerShipsInPlay < 1 )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerDestroyed );
		}
	}

	if ( fEnemyOneDestroyed || fEnemyOneCaptured )
	{
		enemyOneOut = true;
	}
	if ( fEnemyTwoDestroyed || fEnemyTwoCaptured )
	{
		enemyTwoOut = true;
	}
	if ( fEnemyFreighterCaptured || fEnemyFreighterDestroyed  )
	{
		enemyFreighterOut = true;
	}
	if ( fPrimaryObjectiveMet && enemyOneOut && enemyTwoOut && enemyFreighterOut )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
	}
	else if ( enemyOneOut && enemyTwoOut && enemyFreighterOut )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerLoses );
	}
}

void tPromisesVictoryState::mOnDisengaged( tShipInfo* ship )
{
	bool enemyOneOut = false;
	bool enemyTwoOut = false;
	bool enemyFreighterOut = false;

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

	if ( ( ship->mGetTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );

		if( fPlayerShipsInPlay < 1 )
		{
			if ( fEnemyOneDestroyed || fEnemyOneCaptured )
			{
				enemyOneOut = true;
			}
			if ( fEnemyTwoDestroyed || fEnemyTwoCaptured )
			{
				enemyTwoOut = true;
			}
			if ( fEnemyFreighterCaptured || fEnemyFreighterDestroyed  )
			{
				enemyFreighterOut = true;
			}
			if ( enemyOneOut && enemyTwoOut && enemyFreighterOut )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
			}
			else
			{
				fPrimaryObjectiveMet = false;
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerLoses );
			}
		}
	}
	else if( ( ship->mGetTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kFreighterID ) ) )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerLoses );
	}
}
void tPromisesVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

	if( (*actor == typeid( tFreighterTeamShip ) ) &&  // Check to see if Primary Objective is Complete
		(*state == typeid(tFreighterTeamShipReachedPlanetState) ) && 
		(eventType == kStartingNewState ) )
	{
		fPrimaryObjectiveMet = false;
	}
	
}