//	<Name>	BaseVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "FreighterTeam.h"
#include "ISC_3IronCage.h"
#include "MissionText.h"
#include "NonISCTeam.h"
#include "NPC1Team.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"
#include "BaseVictoryState.h"

#include <sstream>


tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if ( fChildActor->mPrisonersRescued() )
	{
		return kDevastatingDefeat;
	}
	else if ( !fChildActor->mPrisonersReachPlanet() )
	{
		return kDefeat;
	}
	else if ( fChildActor->mEnemiesDestroyed() < 3 )
	{
		return kVictory;
	}
	else
	{
		return kAstoundingVictory;
	}
	return kDraw;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tNonISCTeam::kNonISCShip1:
	case tNonISCTeam::kNonISCShip2:
	case tNonISCTeam::kNonISCShip3:
		{
			mAddToBonusPrestige( 20 );
			fChildActor->mIncreaseEnemiesDestroyed();
			
			if ( fChildActor->mEnemiesDestroyed() < 3 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
			}
			else if ( fChildActor->mEnemiesDestroyed() == 3 && fChildActor->mPrisonersRetrieved() )
			{
				if ( fChildActor->mPrisonersReachPlanet() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				}
			}
			else if ( fChildActor->mEnemiesDestroyed() == 3 && !fChildActor->mPrisonersRetrieved() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_2_msg );
			}
		}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerNPCTeam::kISCCL:
		{}
		break;

	case tPlayerNPCTeam::kDefensePlatform1:
	case tPlayerNPCTeam::kDefensePlatform2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2B_1_msg );
		}
		break;

	case tFreighterTeam::kPrisonFreighter:
		{
			if ( !fChildActor->mPrisonersReachPlanet() )
			{
				fChildActor->mSetPrisonShipDestroyed( true );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2A_1_msg );
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tNonISCTeam::kNonISCShip1:
	case tNonISCTeam::kNonISCShip2:
	case tNonISCTeam::kNonISCShip3:
		{
			mAddToBonusPrestige( 20 );
			fChildActor->mIncreaseEnemiesDestroyed();
			
			if ( fChildActor->mEnemiesDestroyed() < 3 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
			}
			else if ( fChildActor->mEnemiesDestroyed() == 3 && fChildActor->mPrisonersRetrieved() )
			{
				if ( fChildActor->mPrisonersReachPlanet() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				}
			}
			else if ( fChildActor->mEnemiesDestroyed() == 3 && !fChildActor->mPrisonersRetrieved() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_2_msg );
			}
		}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerNPCTeam::kISCCL:
		{}
		break;

	case tPlayerNPCTeam::kDefensePlatform1:
	case tPlayerNPCTeam::kDefensePlatform2:
		{}
		break;

	case tFreighterTeam::kPrisonFreighter:
		{
			fChildActor->mSetPrisonersTaken( !fChildActor->mPrisonersTaken() );
			if ( !fChildActor->mPrisonersRetrieved() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_1_msg );
				fChildActor->mSetPrisonersRetrieved( true );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );
				fChildActor->mSetPrisonersRetrieved( false );
			}
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{
			fChildActor->mSetSuppliesTaken( !fChildActor->mSuppliesTaken() );
			if ( fChildActor->mSuppliesTaken() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom5_msg );
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tNonISCTeam::kNonISCShip1:
	case tNonISCTeam::kNonISCShip2:
	case tNonISCTeam::kNonISCShip3:
		{
			mAddToBonusPrestige( 20 );
			fChildActor->mIncreaseEnemiesDestroyed();
			
			if ( fChildActor->mEnemiesDestroyed() < 3 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );
			}
			else if ( fChildActor->mEnemiesDestroyed() == 3 && fChildActor->mPrisonersRetrieved() )
			{
				if ( fChildActor->mPrisonersReachPlanet() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				}
			}
			else if ( fChildActor->mEnemiesDestroyed() == 3 && !fChildActor->mPrisonersRetrieved() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_2_msg );
			}
		}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerNPCTeam::kISCCL:
		{}
		break;

	case tPlayerNPCTeam::kDefensePlatform1:
	case tPlayerNPCTeam::kDefensePlatform2:
		{}
		break;

	case tFreighterTeam::kPrisonFreighter:
		{
			fChildActor->mSetPrisonersRescued( true );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent9_1_msg );
			if ( fChildActor->mPrisonersReachPlanet() )
			{
				//
				//	Should never go in here since mission ends when the prisoners reach the planet; therefore,
				//	the player would never have a chance to disengage.
				//
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tNonISCTeam::kNonISCShip1:
	case tNonISCTeam::kNonISCShip2:
	case tNonISCTeam::kNonISCShip3:
		{}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerNPCTeam::kISCCL:
		{}
		break;

	case tPlayerNPCTeam::kDefensePlatform1:
	case tPlayerNPCTeam::kDefensePlatform2:
		{}
		break;

	case tFreighterTeam::kPrisonFreighter:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent7_1_msg );
			fChildActor->mSetPrisonersReachPlanet( true );
			if ( fChildActor->mEnemiesDestroyed() == 3 )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{}
		break;
	}
}



void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );		break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );		break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );		break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );		break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );		break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );		break;
		}
		mSetTimedInterval(	kTimerSecondsSet, timerID, ( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif
}