/********************************************************************************************************\

	File Name:	FreighterTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "FreighterTeam.h"
#include "PlayerNPCFreighter.h"
#include "ShipList_API.h"


tClassAllocator<tFreighterTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tFreighterTeam::mSetupState()
{}

void tFreighterTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/


void tFreighterTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tFreighterTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        // ense platforms
        std::list< eEnhancedClassTypes >  ClassTypeList;
        ClassTypeList.push_back(kEClassSmallFreighter);
		ClassTypeList.push_back(kEClassCivilianFreighter);
		ClassTypeList.push_back(kEClassPriorityTransport);
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for small freighter classes filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kCargo);
        fShiplistCount = sl.Subset(RolesList, true);
        gST.mDebug_fprintf("subset for cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);

        Ship s1, s2;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

			mCreateShip(
                typeid( tPlayerNPCFreighter ),
                s1.shipCode.c_str(),
                kStartPosition_I,
                kNoMetaShipID,
                kPrisonFreighter,
                0, 0, -1, -1, NULL,
                static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );

			mCreateShip(
                typeid( tPlayerNPCFreighter ),
                s2.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kPrisonFreighter,
                0, 0, -1, -1, NULL,
                static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tFreighterTeam::mCreateShipsForTeam\n");
}


void tFreighterTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tFreighterTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tPlayerNPCFreighter ), "I-FSPh", kStartPosition_I, kNoMetaShipID, kPrisonFreighter,
					0, 0, -1, -1, NULL, static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );
	mCreateShip(	typeid( tPlayerNPCFreighter ), "I-FSPh", kStartPosition_J, kNoMetaShipID, kSupplyFreighter,
					0, 0, -1, -1, NULL, static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );

	gST.mDebug_fprintf("---- finished: tFreighterTeam::mOldCreateShipsForTeam\n");
}


void tFreighterTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tFreighterTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tFreighterTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tFreighterTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}