/********************************************************************************************************\

	Mission:	"Nor Iron Bars A Cage (ISC POV)"
	Script:		"ISC_3IronCage"
	Descript:	The player attempts to deliver a prison ship to a planet.

	Created by:		Daniel J. Suleski		3/15/00
	Last Updated:	DJS						3/16/00
					First draft completion
					DJS						3/30/00
					Second draft revisions
					DJS						04/11/00
					Fix multiple-player ship random chatter
					Got rid of references to time.h
					DJS						05/02/00
					Starting conversion from old API to new API
					DJS						05/20/00
					Asserts added and code cleaned
					DJS						05/26/00
					tShipIterator::mGetFirstShipInPlay() replaced with original form
					DJS						08/23/00
					Map overrides.
					DJS						09/25/00
					mCreateShip() "default-removal" modifications, kCampaignMission, tTeamSpec, etc
					DJS						09/26/00
					Meta script modifications
					DJS						10/04/00
					mAssignMapTerrainTypes() implementation

\********************************************************************************************************/
#include "ISC_3IronCage.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "BaseVictory.h"
#include "FreighterTeam.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "NonISCTeam.h"
#include "NPC1Team.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tISC_3IronCage );

tISC_3IronCage::tISC_3IronCage()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tISC_3IronCage::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team",
													static_cast< eTeamID >( kPlayerTeam ),
													kMaxChance,
													kTrue,
													kPrimaryTeam,
													kMinChance,
													kSpecISC,
													kAnyRace,
													typeid( tPlayerTeam ),
													kNoTeamTag,
													Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& nonISCTeam = mCreateTeamSpec(	"Non ISC Team",
													static_cast< eTeamID >( kNonISCTeam ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecPlayableRace & ~kSpecISC,
													kAnyRace,
													typeid( tNonISCTeam ) );

	const tTeamSpec& npc1Team = mCreateTeamSpec(	"NPC 1 Team",
													static_cast< eTeamID >( kNPC1Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecPlayableRace,
													kAnyRace,
													typeid( tNPC1Team ) );

	const tTeamSpec& playerNPCTeam = mCreateTeamSpec(	"Player NPC Team",
														static_cast< eTeamID >( kPlayerNPCTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tPlayerNPCTeam ) );

	const tTeamSpec& freighterTeam = mCreateTeamSpec(	"Freighter Team",
														static_cast< eTeamID >( kFreighterTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tFreighterTeam ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( nonISCTeam );
	Specs->mSetupTeam( npc1Team );
	Specs->mSetupTeam( playerNPCTeam );
	Specs->mSetupTeam( freighterTeam );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tISC_3IronCage::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tISC_3IronCage::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tISC_3IronCage::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tISC_3IronCage::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tISC_3IronCage::mStartMission( void )
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kNonISCTeam,
		kNPC1Team,
		kPlayerNPCTeam,
		kFreighterTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = fPlayersTeam,
		//NISC = Non-ISCTeam,
		//NPC1 = NPC1,
		//PNPC = PlayerNPC Team,
		//Frei = Freighter Team

		//	     Plyr  NISC  NPC1  PNPC  Frei
		/*Plyr*/ "--", "WD", "FI", "AI", "AI",
		/*NISC*/ "HD", "--", "HI", "WD", "WX",
		/*NPC1*/ "FI", "FI", "--", "FI", "FI",
		/*PNPC*/ "AI", "WD", "FI", "--", "AI",
		/*Frei*/ "AI", "WD", "FI", "AI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tISC_3IronCage::mNewInternalShip( tShipInfo* Ship )
{
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_3IronCage::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['K'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['L'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['M'] = kObjectPlanet;

	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_3IronCage::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tISC_3IronCage::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}