//	<Name>	NonISCShipBaseState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "FreighterTeam.h"
#include "ISC_3IronCage.h"
#include "NonISCShipBaseState.h"
#include "PlayerTeam.h"
#include "PlayerNPCLightCruiser.h"
#include "PlayerNPCTeam.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"

void tNonISCShipBaseState::mBeginState()
{}

void tNonISCShipBaseState::mEndState()
{}

void tNonISCShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( fChildActor->mGetCustomID() == damagedShip->mGetCustomID() )
	{
		float hullIntegrity = fChildActor->mGetDamagePercent();

		tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNonISCTeam ) );
		Assert( enemyTeam );
		if ( !enemyTeam )
			return;
		eRaceName	enemyRace = enemyTeam->mGetRace();

		tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( playerTeam );
		if ( !playerTeam )
			return;

		switch ( enemyRace )
		{
		case kFederation:
		case kHydran:
			{
				if ( hullIntegrity < 50.0f )
				{
					playerTeam->mGotoCommState( damagedShip->mGetShipID(), tNonISCShip::kCommState2 );
				}
				else
				{
					playerTeam->mGotoCommState( damagedShip->mGetShipID(), tNonISCShip::kCommState3 );
				}
			}
			break;
		}
	}
}

void tNonISCShipBaseState::mOnDeath( tShipInfo* ship )
{
	if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
	{
		tTeamInfo*		playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		tTeamInfo*		playerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
		tTeamInfo*		freighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kFreighterTeam ) );
		tTeamInfo*		nonISCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNonISCTeam ) );
		Assert( playerTeam && playerNPCTeam && freighterTeam && nonISCTeam );
		if ( ( !playerTeam ) || ( !playerNPCTeam ) || ( !freighterTeam ) || ( !nonISCTeam ) )
			return;

		tShipIterator	playerNPCCL = playerNPCTeam->mGetFirstShip();
		tShipIterator	prisonShip = freighterTeam->mGetFirstShip();
		tShipIterator	supplyShip = freighterTeam->mGetFirstShip();
		tShipCustomIDEval	b( tPlayerNPCTeam::kISCCL ), c( tFreighterTeam::kPrisonFreighter ), d( tFreighterTeam::kSupplyFreighter );
		tShipInPlayEval		x, y, z;
		playerNPCCL.mAddEvaluator( &b );	playerNPCCL.mAddEvaluator( &x );
		prisonShip.mAddEvaluator( &c );		prisonShip.mAddEvaluator( &y );
		supplyShip.mAddEvaluator( &d );		supplyShip.mAddEvaluator( &z );( tFreighterTeam::kSupplyFreighter );
		tShipIterator	targetShip = nonISCTeam->mGetFirstShip();

		tShipInPlayEval	inPlayEval;
		targetShip.mAddEvaluator( &inPlayEval );

		if ( *playerNPCCL != NULL )
		{
			//
			//	If there is another Non-ISC ship to destroy, then target it with the player CL regardless of
			//	whether or not it is in an intercept position.
			//
			if ( *targetShip != NULL )
			{
				float hullDamage = ( *playerNPCCL )->mGetDamagePercent();
				if ( hullDamage == 0.0f )
				{
					playerTeam->mGotoCommState( ship->mGetShipID(), tPlayerNPCLightCruiser::kCommState6 );
				}
				else if ( hullDamage < 30.0f )
				{
					playerTeam->mGotoCommState( ship->mGetShipID(), tPlayerNPCLightCruiser::kCommState7 );
				}
				else if ( hullDamage < 60.0f )
				{
					playerTeam->mGotoCommState( ship->mGetShipID(), tPlayerNPCLightCruiser::kCommState8 );
				}
				else
				{
					playerTeam->mGotoCommState( ship->mGetShipID(), tPlayerNPCLightCruiser::kCommState9 );
				}
				( *playerNPCCL )->mSetShipTarget( *targetShip, kMediumPriority );
			}

			//
			//	If there is no longer a Non-ISC ship to destroy, then send the player CL to orbit around the
			//	supply ship again by sending it to its base state.
			//
			else
			{
				( *playerNPCCL )->mDefendEntity( *prisonShip, 50.0, 25.0, kAbsolutePriority );
			}
		}
	}
}