//	<Name>	NonISCTeam.cpp
//
//
//	<Info>	DESCRIPTION
//


#include "tclient.h"
#include "ISC_3IronCage.h"
#include "NonISCShip.h"
#include "NonISCTeam.h"
#include "NonISCTeamBaseState.h"
#include "NonISCTeamKillState.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tNonISCTeam> gNonISCTeamAllocator;

void tNonISCTeam::mSetupState()
{
	mRegisterState( new tNonISCTeamBaseState() );
	mRegisterState( new tNonISCTeamKillState() );

	mGotoState( typeid( tNonISCTeamBaseState ) );
}

void tNonISCTeam::mSetupCommState()
{}


void tNonISCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tNonISCTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( playerTeam );
		if ( !playerTeam )
			return;

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 nPCBPV; 
		switch ( diffLevel )
		{
		case 0:
			nPCBPV = playerBPV * 0.75;	break;
		case 1:
			nPCBPV = playerBPV * 1.0;	break;
		case 2:
			nPCBPV = playerBPV * 1.25;	break;
		}

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShips( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassHeavyBattlecruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);


		fShiplistCount = sl.Subset( sl.kSubsetBPV, int(0.825 * nPCBPV), int(1.175 * nPCBPV), true);
        gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
            int(0.825 * nPCBPV), int(1.175 * nPCBPV), fShiplistCount);

        Ship s1, s2, s3;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

            mCreateShip(
                typeid( tNonISCShip ),
                s1.shipCode.c_str(),
                kStartPosition_Z,
                kNoMetaShipID,
                kNonISCShip1,
                0, 0, -1, -1, NULL,
                kCanAddToFleetUponCapture );

            mCreateShip(
                typeid( tNonISCShip ),
                s2.shipCode.c_str(),
                kStartPosition_Y,
                kNoMetaShipID,
                kNonISCShip2,
                0, 0, -1, -1, NULL,
                kCanAddToFleetUponCapture );

            mCreateShip(
                typeid( tNonISCShip ),
                s3.shipCode.c_str(),
                kStartPosition_X,
                kNoMetaShipID,
                kNonISCShip3,
                0, 0, -1, -1, NULL,
                kCanAddToFleetUponCapture );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tNonISCTeam::mCreateShipsForTeam\n");

}


void tNonISCTeam::mOldCreateShipsForTeam()
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	switch ( diffLevel )
	{
	case 0:
		nPCBPV = playerBPV * 0.75;	break;
	case 1:
		nPCBPV = playerBPV * 1.0;	break;
	case 2:
		nPCBPV = playerBPV * 1.25;	break;
	}

	mCreateShip(	typeid( tNonISCShip ),
					nPCBPV,
					kClassDestroyer,
					0,
					kStartPosition_Z,
					kNonISCShip1,
					0, 0, -1, -1,
					NULL,
					kCanAddToFleetUponCapture );
	mCreateShip(	typeid( tNonISCShip ),
					nPCBPV,
					kClassDestroyer,
					0,
					kStartPosition_Y,
					kNonISCShip2,
					0, 0, -1, -1,
					NULL,
					kCanAddToFleetUponCapture );
	mCreateShip(	typeid( tNonISCShip ),
					nPCBPV,
					kClassDestroyer,
					0,
					kStartPosition_X,
					kNonISCShip3,
					0, 0, -1, -1,
					NULL,
					kCanAddToFleetUponCapture );
}


void tNonISCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tNonISCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tNonISCTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tNonISCTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}