/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/


#include "tclient.h"
#include "ShipScriptDescription.h"
#include "PlayerNPCFreighter.h"
#include "PlayerNPCLightCruiser.h"
#include "PlayerNPCPlatform.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );

	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}


void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        ShipList DEF_sl = sl;

        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShips(sl);


        // Defense platforms
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);


        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        for (int32 i = kEClassLightCruiser; i <= kEClassMediumCruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        Ship s1, s2, s3;
        if ((sl.size() > 0) && (DEF_sl.size() > 0)) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );


            DEF_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            DEF_sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

			mCreateShip(
                typeid( tPlayerNPCLightCruiser ),
                s1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kISCCL,
                0, 0, -1, -1, NULL,
                kCanAddToFleetUponCapture );

			mCreateShip(
                typeid( tPlayerNPCPlatform ),
                s2.shipCode.c_str(),
                kStartPosition_K,
                kNoMetaShipID,
                kDefensePlatform1,
                0, 0, -1, -1, NULL,
                kCanAddToFleetUponCapture );

			mCreateShip(
                typeid( tPlayerNPCPlatform ),
                s3.shipCode.c_str(),
                kStartPosition_L,
                kNoMetaShipID,
                kDefensePlatform1,
                0, 0, -1, -1, NULL,
                kCanAddToFleetUponCapture );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tPlayerNPCTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tPlayerNPCLightCruiser ), "I-CA", kStartPosition_H, kNoMetaShipID, kISCCL,
					0, 0, -1, -1, NULL, kCanAddToFleetUponCapture );

	mCreateShip(	typeid( tPlayerNPCPlatform ), "F-DEF" , kStartPosition_K, kNoMetaShipID, kDefensePlatform1,
					0, 0, -1, -1, NULL, kCanAddToFleetUponCapture );
	mCreateShip(	typeid( tPlayerNPCPlatform ), "F-DEF" , kStartPosition_L, kNoMetaShipID, kDefensePlatform2,
					0, 0, -1, -1, NULL, kCanAddToFleetUponCapture );

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}


void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}