//	<Name>	EnemyTeamInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "ISC_4Resolve.h"
#include "EnemyTeamShip.h"
//#include "EnemyTeamPlayerKillerShip.h"
#include "EnemyTeamInitialState.h"
//#include "EnemyTeamArrivalState.h"
#include "ShipScriptDescription.h"
#include "MissionInfo.h"
#include "TeamInfo.h"
#include "PlayerTeamShipInitialState.h"
#include "MissionText.h"
#include "ShipList_API.h"

void tEnemyTeamInitialState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kEnemyArrivalID, 10.0f );
}

void tEnemyTeamInitialState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEnemyArrivalID )
	{
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 enemyBPV;
		int32 enemySmallBPV;

		if( diffLevel == 0 )
		{
			enemyBPV = playerBPV * 0.2;
		}
		else if( diffLevel == 1 )
		{
			enemyBPV = playerBPV * 0.3;
		}
		else if( diffLevel == 2 )
		{
			enemyBPV = playerBPV * 0.6;
		}
		enemySmallBPV = ( enemyBPV * 0.75 ); 


		gST.mDebug_fprintf("--- Entering: tEnemyTeamInitialState::mOnTimedInterval -- kEnemyArrivalID\n");
		gST.mCompleteNarrowedList();

		if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
			int32 fShiplistCount;

			ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, fChildActor->mGetRace() );
			sl = gST.stripOutSPECIAL( sl );
			sl = gST.stripOutRestricted( sl );
			sl = gST.selectWarShips(sl);

			std::list< eEnhancedClassTypes >  shipClassTypeList;
			shipClassTypeList.push_back(kEClassSmallMonster);
			shipClassTypeList.push_back(kEClassMediumMonster);
			shipClassTypeList.push_back(kEClassLargeMonster);
			for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
				shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
			fShiplistCount = sl.Subset(shipClassTypeList, true);
			gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

			ShipList small_sl = sl;
			ShipList large_sl = sl;

			fShiplistCount = small_sl.Subset( small_sl.kSubsetBPV, int(enemySmallBPV * 0.825), int(enemySmallBPV * 1.175), true);
			gST.mDebug_fprintf("small_sl subset for BPV range (%d - %d) completed. ships: %d\n",
				int(enemySmallBPV * 0.825), int(enemySmallBPV * 1.175), fShiplistCount);

			fShiplistCount = large_sl.Subset( large_sl.kSubsetBPV, int(enemyBPV * 0.825), int(enemyBPV * 1.175), true);
			gST.mDebug_fprintf("large_sl subset for BPV range (%d - %d) completed. ships: %d\n",
				int(enemyBPV * 0.825), int(enemyBPV * 1.175), fShiplistCount);

			Ship s1, s2;
			if ((small_sl.size() > 0) && (large_sl.size() > 0)) {
				small_sl.SelectRandomShip(s1);
			    gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				    s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				large_sl.SelectRandomShip(s2);
			    gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				    s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
				gST.mTrackOrions( s2 );

				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), s1.shipCode.c_str(), kStartPosition_W, tTeamInfo::kNoMetaShipID, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), s2.shipCode.c_str(), kStartPosition_X, tTeamInfo::kNoMetaShipID, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );

			} else {
				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemySmallBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_W, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemyBPV, kClassFrigate, kClassDreadnought, 10, kStartPosition_X, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			}
		} else {
			fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemySmallBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_W, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemyBPV, kClassFrigate, kClassDreadnought, 10, kStartPosition_X, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
		}

		gST.mDebug_fprintf("--- Finished: tEnemyTeamInitialState::mOnTimedInterval -- kEnemyArrivalID\n");

		fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),kFirstTwoEnemyShipsArrive_msg, kScienceScriptMessage );
		mSetTimedInterval( kTimerTurnsSet, kLastEnemyArrivalID, 5.0f );
	}

	if ( timerID == kLastEnemyArrivalID )
	{
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 enemyBPV;
		int32 enemySmallBPV;

		if( diffLevel == 0 )
		{
			enemyBPV = playerBPV * 0.2;
		}
		else if( diffLevel == 1 )
		{
			enemyBPV = playerBPV * 0.3;
		}
		else if( diffLevel == 2 )
		{
			enemyBPV = playerBPV * 0.8;
		}
		enemySmallBPV = ( enemyBPV * 0.75 ); 


		gST.mDebug_fprintf("--- Entering: tEnemyTeamInitialState::mOnTimedInterval -- kLastEnemyArrivalID\n");
		gST.mCompleteNarrowedList();

		if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
			int32 fShiplistCount;

			ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, fChildActor->mGetRace() );
			sl = gST.stripOutSPECIAL( sl );
			sl = gST.stripOutRestricted( sl );
			sl = gST.selectWarShips(sl);

			std::list< eEnhancedClassTypes >  shipClassTypeList;
			shipClassTypeList.push_back(kEClassSmallMonster);
			shipClassTypeList.push_back(kEClassMediumMonster);
			shipClassTypeList.push_back(kEClassLargeMonster);
			for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
				shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
			}
			fShiplistCount = sl.Subset(shipClassTypeList, true);
			gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

			ShipList small_sl = sl;
			ShipList large_sl = sl;

			fShiplistCount = small_sl.Subset( small_sl.kSubsetBPV, int(enemySmallBPV * 0.825), int(enemySmallBPV * 1.175), true);
			gST.mDebug_fprintf("small_sl subset for BPV range (%d - %d) completed. ships: %d\n",
				int(enemySmallBPV * 0.825), int(enemySmallBPV * 1.175), fShiplistCount);

			fShiplistCount = large_sl.Subset( large_sl.kSubsetBPV, int(enemyBPV * 0.825), int(enemyBPV * 1.175), true);
			gST.mDebug_fprintf("large_sl subset for BPV range (%d - %d) completed. ships: %d\n",
				int(enemyBPV * 0.825), int(enemyBPV * 1.175), fShiplistCount);

			Ship s1, s2;
			if ((small_sl.size() > 0) && (large_sl.size() > 0)) {
				small_sl.SelectRandomShip(s1);
			    gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				    s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				large_sl.SelectRandomShip(s2);
			    gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				    s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
				gST.mTrackOrions( s2 );

				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), s1.shipCode.c_str(), kStartPosition_W, tTeamInfo::kNoMetaShipID, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), s2.shipCode.c_str(), kStartPosition_X, tTeamInfo::kNoMetaShipID, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );

			} else {
				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemySmallBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_W, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
				fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemyBPV, kClassFrigate, kClassDreadnought, 30, kStartPosition_X, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			}
		} else {
			fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemySmallBPV, kClassFrigate, kClassHeavyCruiser, 0, kStartPosition_W, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			fChildActor->mCreateShip( typeid( tEnemyTeamShip ), enemyBPV, kClassFrigate, kClassDreadnought, 30, kStartPosition_X, -1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
		}

		gST.mDebug_fprintf("--- Finished: tEnemyTeamInitialState::mOnTimedInterval -- kLastEnemyArrivalID\n");

		fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kSecondTwoEnemyShipsArrive_msg, kScienceScriptMessage );
	}
}
