

#include "ISC_4Resolve.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "PlanetTeam.h"
#include "FreighterTeam.h"
#include "DryDockTeam.h"
#include "ExplosiveFreighterTeam.h"
#include "ResolveVictory.h"
#include "ShipList_API.h"

DEFINE_SCRIPT_CLASS( tISC_4Resolve );

tISC_4Resolve::tISC_4Resolve()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tISC_4Resolve::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID>( kPlayerTeam ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tPlayerTeam ), kNoTeamTag, Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID>( kEnemyTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecMirak | kSpecHydran,
														kAnyRace,
														typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", static_cast< eTeamID>( kPlanetTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecAnyRace,
														kAnyRace,
														typeid( tPlanetTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", static_cast< eTeamID>( kFreighterTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tFreighterTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5", static_cast< eTeamID>( kDryDockTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tDryDockTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6", static_cast< eTeamID>( kExplosiveFreighterTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tExplosiveFreighterTeam ) );


	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tISC_4Resolve::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tISC_4Resolve::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tISC_4Resolve::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tISC_4Resolve::mStartMission( void )
{
	tScript::mStartMission();

	// setup team relations and target goals
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kFreighterTeam,
		kPlanetTeam,
		kEnemyTeam,
		kDryDockTeam,
		kExplosiveFreighterTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		// Plyr = fPlayersTeam,
		// Frtr = kFreighterTeam,
		// PLNT = kPlanetTeam,
		// Enemy = kEnemyTeam,
		// DryD = kMainDryDockTeam,
		// ExpF = kExplosiveFreighterTeam

		//			Plyr	Frtr	PLNT	Enemy	Dryd	ExpF
		/*Plyr*/	"--",	"AI",	"AI",	"WD",	"AI",	"AI",
		/*Frtr*/	"AI",	"--",	"AI",	"AI",	"AI",	"AI",	
		/*PLNT*/	"AI",	"AI",	"--",	"WD",	"AI",	"AI",	
		/*Enemy*/	"WD",	"WI",	"WI",	"--",	"WC",	"AI",	
		/*DryD*/	"AI",	"AI",	"AI",	"WD",	"--",	"AI",
		/*ExpF*/	"AI",	"AI",	"AI",	"AI",	"AI",	"AI",

	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}
//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tISC_4Resolve::mNewInternalShip( tShipInfo* Ship )
{}
void tISC_4Resolve::mCreateExternalShips( int32 Mode )
{}

void tISC_4Resolve::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;

	Map->fTacticalMapIcon['^'] = kObjectOrganian;

	Map->fTacticalMapIcon['L'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['M'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['K'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['T'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['U'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['V'] = kObjectAlliedShip;
}
void tISC_4Resolve::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tISC_4Resolve::mDefineBriefings( void )
{
		fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}

