//	<Name>	PlayerTeamShipInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "ISC_4Resolve.h"
#include "ShipInfo.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamShipInitialState.h"
#include "Planet.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"
#include "FreighterTeam.h"
#include "ExplosiveFreighterTeam.h"

tPlayerTeamShipInitialState::tPlayerTeamShipInitialState( )
{
	fDeepScanON = false;
	fFreighterOneScanned = false;
	fFreighterTwoScanned = false;
	fExplosiveFreighterScanned = false;
}

void tPlayerTeamShipInitialState::mBeginState()
{
}

void tPlayerTeamShipInitialState::mEndState()
{
}
void tPlayerTeamShipInitialState::mOnTimedInterval( int32 timerID )
{
	int32 rangeToFreighterOneThisInterval = 100;
	int32 rangeToFreighterTwoThisInterval = 100;
	int32 rangeToExplosiveFreighterThisInterval = 100;

	tTeamInfo* freighterTeam  = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kFreighterTeam ) );
	tTeamInfo* explosiveFreighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kExplosiveFreighterTeam ) );


	tShipIterator pship = freighterTeam->mGetFirstShip();
	tShipCustomIDEval x( tFreighterTeam::kFreighterOneID );
	pship.mAddEvaluator( &x );
	tShipInfo* freighter1 = *pship;	
	
	tShipIterator pship2 = freighterTeam->mGetFirstShip();
	tShipCustomIDEval y( tFreighterTeam::kFreighterTwoID );
	pship2.mAddEvaluator( &y );
	tShipInfo* freighter2 = *pship2;	

	tShipIterator pship3 = explosiveFreighterTeam->mGetFirstShip();
	tShipCustomIDEval z( tExplosiveFreighterTeam::kExplosiveFreighterID );
	pship3.mAddEvaluator( &z );
	tShipInfo* explosiveFreighter = *pship3;	

	eRaceName  playerRace;

 	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) ) );
	Assert( playerTeam );
	if( playerTeam == NULL )
	{
		return;
	}
	playerRace = playerTeam->mGetRace();

	fFreighterOneScanned = playerTeam->mGetFreighterOneScanned();
	fFreighterTwoScanned = playerTeam->mGetFreighterTwoScanned();
	fExplosiveFreighterScanned = playerTeam->mGetfExplosiveFreighterScanned();

	if( timerID == 	kCheckDistanceID )
	{
		if( freighter1 )
		{
			rangeToFreighterOneThisInterval = fChildActor->mGetTacticalDistance( freighter1 );
		}
		if( freighter2 )
		{
			rangeToFreighterTwoThisInterval = fChildActor->mGetTacticalDistance( freighter2 );
		}
		if( explosiveFreighter )
		{
			rangeToExplosiveFreighterThisInterval = fChildActor->mGetTacticalDistance( explosiveFreighter );
		}

		if( ( freighter1 ) && !fFreighterOneScanned && ( rangeToFreighterOneThisInterval < 10 ) && ( fDeepScanON ) && ( ( fChildActor->mGetShipTargetID() ) == freighter1->mGetShipID() ) )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kScannedRegFreighter_msg, kScienceScriptMessage );
			playerTeam->mSetFreighterOneScanned( true );
		}
		else if( ( freighter2 ) && !fFreighterTwoScanned && ( rangeToFreighterTwoThisInterval < 10 ) && ( fDeepScanON ) && ( ( fChildActor->mGetShipTargetID() ) == freighter2->mGetShipID() ) )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kScannedRegFreighter_msg, kScienceScriptMessage );
			playerTeam->mSetFreighterTwoScanned( true );
		}
		else if( ( explosiveFreighter ) && !fExplosiveFreighterScanned && ( rangeToExplosiveFreighterThisInterval < 10 ) && ( fDeepScanON ) && ( ( fChildActor->mGetShipTargetID() ) == explosiveFreighter->mGetShipID() ) )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kScannedExpFreighter_msg, kScienceScriptMessage );
			playerTeam->mSetfExplosiveFreighterScanned( true );
			fMissionInfo->mSetTeamRelations( playerTeam->mGetTeam(), static_cast< eTeamID>( kExplosiveFreighterTeam ), kTeamAtWar, kGoalDestroy, 1.0, kFalse );
		}
	}
	else if( timerID == kDeepScanONID )
	{
		if( fDeepScanON )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kScanning_msg, kScienceScriptMessage );
			mSetTimedInterval( kTimerTurnsSet, kCheckDistanceID, 0.5f );
		}
	}


	mSetTimedInterval( kTimerTurnsSet, kCheckDistanceID, 1.0f );



}
void tPlayerTeamShipInitialState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fDeepScanON = true;
	}
	else
	{
		fDeepScanON = false;
	}
	ship->mSetShipCallbackEvents(  kNotifyDeepScan );
	mSetTimedInterval( kTimerTurnsSet, kDeepScanONID, 0.75f );
}

