/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "EnemyTeam1.h"
#include "EnemyTeam2.h" 
#include "ISC_5Accuse.h"
#include "MissionText.h"
#include "NPCTeam1.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"


tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if ( ( fChildActor->mISCDefensePlatformsDestroyed() == 2 ) || fChildActor->mDeserted() )
	{
		return kDevastatingDefeat;
	}
	else
	{
		return kAstoundingVictory;
	}
	return kDraw;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam1::kCruiser1:
	case tEnemyTeam1::kCruiser2:
	case tEnemyTeam1::kCruiser3:
		{
			mAddToBonusPrestige( 25 );
			fChildActor->mIncreaseCruisersDestroyed();

			if ( fChildActor->mCruisersDestroyed() == 6 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );
			}
		}
		break;

	case tEnemyTeam2::kCruiser4:
	case tEnemyTeam2::kCruiser5:
	case tEnemyTeam2::kCruiser6:
		{
			mAddToBonusPrestige( 25 );
			fChildActor->mIncreaseCruisersDestroyed();

			if ( fChildActor->mCruisersDestroyed() == 6 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );
			}
		}
		break;
	
	case tPlayerNPCTeam::kDefensePlatform1:
	case tPlayerNPCTeam::kDefensePlatform2:
		{
			fChildActor->mIncreaseISCDefensePlatformsDestroyed();

			if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_2_msg );
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
				{
					fChildActor->mSetDeserted( true );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
				}
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam1::kCruiser1:
	case tEnemyTeam1::kCruiser2:
	case tEnemyTeam1::kCruiser3:
		{
			mAddToBonusPrestige( 25 );
			fChildActor->mIncreaseCruisersDestroyed();

			if ( fChildActor->mCruisersDestroyed() == 6 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
			}
		}
		break;

	case tEnemyTeam2::kCruiser4:
	case tEnemyTeam2::kCruiser5:
	case tEnemyTeam2::kCruiser6:
		{
			mAddToBonusPrestige( 25 );
			fChildActor->mIncreaseCruisersDestroyed();

			if ( fChildActor->mCruisersDestroyed() == 6 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
			}
		}
		break;

	case tPlayerNPCTeam::kDefensePlatform1:
	case tPlayerNPCTeam::kDefensePlatform2:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
				{
					fChildActor->mSetDeserted( true );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
				}
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam1::kCruiser1:
	case tEnemyTeam1::kCruiser2:
	case tEnemyTeam1::kCruiser3:
		{
			mAddToBonusPrestige( 25 );
			fChildActor->mIncreaseCruisersDestroyed();

			if ( fChildActor->mCruisersDestroyed() == 6 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tEnemyTeam2::kCruiser4:
	case tEnemyTeam2::kCruiser5:
	case tEnemyTeam2::kCruiser6:
		{
			mAddToBonusPrestige( 25 );
			fChildActor->mIncreaseCruisersDestroyed();

			if ( fChildActor->mCruisersDestroyed() == 6 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );			
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
				{
					fChildActor->mSetDeserted( true );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
				}
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam1::kCruiser1:
	case tEnemyTeam1::kCruiser2:
	case tEnemyTeam1::kCruiser3:
		{
			if ( !fChildActor->mApproachWarning() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
			}
			fChildActor->mSetApproachWarning( true );
		}
		break;

	case tEnemyTeam2::kCruiser4:
	case tEnemyTeam2::kCruiser5:
	case tEnemyTeam2::kCruiser6:
		{
			if ( !fChildActor->mApproachWarning() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
			}
			fChildActor->mSetApproachWarning( true );
		}
		break;
	}
}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}
