/********************************************************************************************************\

	File Name:	EnemyTeam1.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"
#include "Cruiser1.h"
#include "Cruiser2.h"
#include "Cruiser3.h"
#include "EnemyTeam1.h"
#include "EnemyTeam1BaseState.h"
#include "ISC_5Accuse.h"


tClassAllocator<tEnemyTeam1> gEnemyTeam1Allocator;

void tEnemyTeam1::mSetupState()
{
	mRegisterState( new tEnemyTeam1BaseState() );

	mGotoState( typeid( tEnemyTeam1BaseState ) );
}

void tEnemyTeam1::mSetupCommState()
{}


void tEnemyTeam1::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam1::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( playerTeam );
		if ( !playerTeam )
			return;

		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 start_class = kEClassDestroyer;
		int32 end_class = kEClassWarCruiser;
		switch ( diffLevel )
		{
		case 0:
			start_class--;
			end_class--;
			break;
		case 2:
			start_class++;
			end_class++;
			break;
		case 1:
		default:
			break;
		}

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
		sl = gST.stripOutSPECIAL( sl );
		sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips(sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassSmallMonster);
		shipClassTypeList.push_back(kEClassMediumMonster);

		for (int32 i = start_class; i <= end_class; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);


        Ship s1, s2, s3;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

			mCreateShip(	typeid( tCruiser1 ), s1.shipCode.c_str(), kStartPosition_Z, kNoMetaShipID,
							kCruiser1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			mCreateShip(	typeid( tCruiser2 ), s2.shipCode.c_str(), kStartPosition_Y, kNoMetaShipID,
							kCruiser2, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			mCreateShip(	typeid( tCruiser3 ), s3.shipCode.c_str(), kStartPosition_X, kNoMetaShipID,
							kCruiser3, 0, 0, -1, -1, NULL, kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tEnemyTeam1::mCreateShipsForTeam\n");
}


void tEnemyTeam1::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam1::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	switch ( diffLevel )
	{
	case 0:
		nPCBPV = playerBPV * 0.75;	break;
	case 1:
		nPCBPV = playerBPV * 1.0;	break;
	case 2:
		nPCBPV = playerBPV * 1.25;	break;
	}

	mCreateShip(	typeid( tCruiser1 ),
					nPCBPV,
					kClassDestroyer,
					kClassLightCruiser,
					0,
					kStartPosition_Z,
					kCruiser1,
					0, 0, -1, -1,
					NULL, kDefaultShipOptions );
	mCreateShip(	typeid( tCruiser2 ),
					nPCBPV,
					kClassDestroyer,
					kClassLightCruiser,
					0,
					kStartPosition_Y,
					kCruiser2,
					0, 0, -1, -1,
					NULL, kDefaultShipOptions );
	mCreateShip(	typeid( tCruiser3 ),
					nPCBPV,
					kClassDestroyer,
					kClassLightCruiser,
					0,
					kStartPosition_X,
					kCruiser3,
					0, 0, -1, -1,
					NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam1::mOldCreateShipsForTeam\n");
}


void tEnemyTeam1::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam1::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam1::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tEnemyTeam1::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}