/********************************************************************************************************\

	File Name:	EnemyTeam2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Cruiser4.h"
#include "Cruiser5.h"
#include "Cruiser6.h"
#include "EnemyTeam2.h"
#include "EnemyTeam2BaseState.h"
#include "ISC_5Accuse.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam2> gEnemyTeam2Allocator;

void tEnemyTeam2::mSetupState()
{
	mRegisterState( new tEnemyTeam2BaseState() );

	mGotoState( typeid( tEnemyTeam2BaseState ) );
}

void tEnemyTeam2::mSetupCommState()
{}


void tEnemyTeam2::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam2::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( playerTeam );
		if ( !playerTeam )
			return;

		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 start_class = kEClassDestroyer;
		int32 end_class = kEClassWarCruiser;
		switch ( diffLevel )
		{
		case 0:
			start_class--;
			end_class--;
			break;
		case 2:
			start_class++;
			end_class++;
			break;
		case 1:
		default:
			break;
		}

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
		sl = gST.stripOutSPECIAL( sl );
		sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips(sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassSmallMonster);
		shipClassTypeList.push_back(kEClassMediumMonster);

		for (int32 i = start_class; i <= end_class; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);



        Ship s1, s2, s3;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

			mCreateShip(	typeid( tCruiser4 ), s1.shipCode.c_str(), kStartPosition_W, kNoMetaShipID,
							kCruiser4, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			mCreateShip(	typeid( tCruiser5 ), s2.shipCode.c_str(), kStartPosition_V, kNoMetaShipID,
							kCruiser5, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			mCreateShip(	typeid( tCruiser6 ), s3.shipCode.c_str(), kStartPosition_U, kNoMetaShipID,
							kCruiser6, 0, 0, -1, -1, NULL, kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tEnemyTeam2::mCreateShipsForTeam\n");
}


void tEnemyTeam2::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entered: tEnemyTeam2::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV;
	switch ( diffLevel )
	{
	case 0:
		nPCBPV = playerBPV * 0.75;	break;
	case 1:
		nPCBPV = playerBPV * 1.0;	break;
	case 2:
		nPCBPV = playerBPV * 1.25;	break;
	}

	mCreateShip(	typeid( tCruiser4 ),
					nPCBPV,
					kClassDestroyer,
					kClassLightCruiser,
					0,
					kStartPosition_W,
					kCruiser4,
					0, 0, -1, -1,
					NULL,
					kDefaultShipOptions );
	mCreateShip(	typeid( tCruiser5 ),
					nPCBPV,
					kClassDestroyer,
					kClassLightCruiser,
					0,
					kStartPosition_V,
					kCruiser5,
					0, 0, -1, -1,
					NULL,
					kDefaultShipOptions );
	mCreateShip(	typeid( tCruiser6 ),
					nPCBPV,
					kClassDestroyer,
					kClassLightCruiser,
					0,
					kStartPosition_U,
					kCruiser6,
					0, 0, -1, -1,
					NULL,
					kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam2::mOldCreateShipsForTeam\n");
}

void tEnemyTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}
void tEnemyTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam2::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tEnemyTeam2::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}