/********************************************************************************************************\

	Mission:	J'Accuse
	Script:		"ISC_5Accuse"
	Descript:	The player allies with another alien race to deliver a protest to the Organians about
				the ISC.  The ISC don't like this and will try to stop them.

	Created by:		Daniel J. Suleski		03/18/00
	Last Updated:	DJS						03/31/00
					Second draft completed
					DJS						04/11/00
					Fix multiple-player ship random chatter
					Got rid of references to time.h
					DJS						05/15/00
					Initial conversion from old API to new multiplayer API
					DJS						05/20/00
					Asserts added and code cleaned
					DJS						05/26/00
					tShipIterator::mGetFirstShipInPlay() replaced with original form
					DJS						08/23/00
					Map overrides.
					DJS						09/25/00
					mCreateShip() "default-removal" modifications, kCampaignMission, tTeamSpec, etc
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "ISC_5Accuse.h"
#include "BaseVictory.h"
#include "EnemyTeam1.h"
#include "EnemyTeam2.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "NPCTeam1.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "SFCAssert.h"
#include "VictoryCondition.h"
#include "DraftTeam.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tISC_5Accuse );

tISC_5Accuse::tISC_5Accuse()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tISC_5Accuse::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team",
													static_cast< eTeamID >( kPlayerTeam ),
													kMaxChance,
													kTrue,
													kPrimaryTeam,
													kMinChance,
													kSpecISC,
													kAnyRace,
													typeid( tPlayerTeam ),
													kNoTeamTag,
													Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& enemy1Team = mCreateTeamSpec(	"Enemy 1 Team",
													static_cast< eTeamID >( kEnemyTeam1 ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak,
													kAnyEnemyOf | kPlayerTeam,
													typeid( tEnemyTeam1 ) );

	const tTeamSpec& enemy2Team = mCreateTeamSpec(	"Enemy 2 Team",
													static_cast< eTeamID >( kEnemyTeam2 ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak,
													kAnyEnemyOf | kDifferentThan | kEnemyTeam1,
													typeid( tEnemyTeam2 ) );

	const tTeamSpec& npcTeam = mCreateTeamSpec(	"NPC Team",
												static_cast< eTeamID >( kNPCTeam1 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak,
												kAnyRace,
												typeid( tNPCTeam1 ) );

	const tTeamSpec& playerNPCTeam = mCreateTeamSpec(	"Player NPC Team",
														static_cast< eTeamID >( kPlayerNPCTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tPlayerNPCTeam ) );

	const tTeamSpec& draftTeam = mCreateTeamSpec(	"Draft Team",
												static_cast< eTeamID >( kDraftTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tDraftTeam ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( enemy1Team );
	Specs->mSetupTeam( enemy2Team );
	Specs->mSetupTeam( npcTeam );
	Specs->mSetupTeam( playerNPCTeam );
	Specs->mSetupTeam( draftTeam );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tISC_5Accuse::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tISC_5Accuse::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tISC_5Accuse::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tISC_5Accuse::mInitializeFinished( void )
{
	tTeamInfo*	enemyTeam1 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam1 ) );
	tTeamInfo*	enemyTeam2 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam2 ) );
	Assert( enemyTeam1 && enemyTeam2 );
	if ( enemyTeam1 && enemyTeam2 )
	{
		eRaceName	enemyRace1 = enemyTeam1->mGetRace();
		eRaceName	enemyRace2 = enemyTeam2->mGetRace();

		//
		//	Since there are no tokens set for multiple enemy races, 7 different briefing messages must
		//	be created and 1 of which is returned depending upon the race of the first enemy.
		//
		switch ( enemyRace1 )
		{
		case kFederation:
		case kKlingon:
			{
				return kBriefingFederationKlingon_msg;
			}
			break;

		case kRomulan:
		case kGorn:
			{
				return kBriefingRomulanGorn_msg;
			}
			break;

		case kLyran:
			{
				switch ( enemyRace2 )
				{
				case kHydran:
					{
						return kBriefingLyranHydran_msg;
					}
					break;

				case kMirak:
					{
						return kBriefingLyranMirak_msg;
					}
					break;
				}
			}
			break;
		}
	}

	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tISC_5Accuse::mStartMission( void )
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam1,
		kEnemyTeam2,
		kNPCTeam1,
		kPlayerNPCTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = PlayersTeam,
		//Nme1 = EnemyTeam1,
		//Nme2 = EnemyTeam2,
		//NPC1 = NPC1
		//PNPC = PlayerNPC

		//	     Plyr  Nme1  Nme2  NPC1  PNPC
		/*Plyr*/ "--", "HD", "HD", "AI", "AI",
		/*Nme1*/ "HD", "--", "FI", "AI", "WD",
		/*Nme2*/ "HD", "FI", "--", "AI", "WD",
		/*NPC1*/ "AI", "AI", "AI", "--", "AI",
		/*PNPC*/ "AI", "HD", "HD", "AI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tISC_5Accuse::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_5Accuse::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['J'] = kObjectPlanet;

	Map->fTacticalMapIcon['M'] = kObjectPlayerBase;

	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_5Accuse::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}


/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tISC_5Accuse::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}