/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllianceShipGenerated.h"
#include "BaseVictoryState.h"
#include "EnemyTeam.h"
#include "ISC_6Immunity.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ShipList_API.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if ( fChildActor->mBaseDestroyed() || !fChildActor->mBaseCaptured() )
	{
		if ( fChildActor->mShipsDestroyed() < 3 )
		{
			return kDevastatingDefeat;
		}
		else
		{
			return kDefeat;
		}
	}
	else if ( fChildActor->mBaseCaptured() )
	{
		if ( fChildActor->mShipsDestroyed() < 3 )
		{
			return kVictory;
		}
		else
		{
			return kAstoundingVictory;
		}
	}
	return kDraw;
}


void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam::kAlliance1:
	case tEnemyTeam::kAlliance2:
	case tEnemyTeam::kAlliance3:
	case tEnemyTeam::kAllianceGen1:
	case tEnemyTeam::kAllianceGen2:
	case tEnemyTeam::kAllianceGen3:
		{

			mAddToBonusPrestige( 25 );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_1_msg );
			fChildActor->mIncreaseShipsDefeated();
			fChildActor->mIncreaseShipsDestroyed();

			if ( fChildActor->mShipsDefeated() == 6 && !fChildActor->mBaseCaptured() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_2_msg );
			}
			else if (	( fChildActor->mShipsDefeated() == 6 ) &&
						( fChildActor->mShipsDestroyed() < 6 ) &&
						( fChildActor->mBaseCaptured() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_3_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else if ( fChildActor->mShipsDestroyed() == 6 && fChildActor->mBaseCaptured() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_4_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tNPC1Team::kStarbase:
		{
			fChildActor->mSetBaseDestroyed( true );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam::kAlliance1:
	case tEnemyTeam::kAlliance2:
	case tEnemyTeam::kAlliance3:
	case tEnemyTeam::kAllianceGen1:
	case tEnemyTeam::kAllianceGen2:
	case tEnemyTeam::kAllianceGen3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
			fChildActor->mIncreaseShipsDefeated();

			if ( fChildActor->mShipsDefeated() == 6 && !fChildActor->mBaseCaptured() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_2_msg );
			}
			else if ( fChildActor->mShipsDefeated() == 6 && fChildActor->mBaseCaptured() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_3_msg );
			}
		}
		break;

	case tNPC1Team::kStarbase:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
			fChildActor->mSetBaseCaptured( true );
			gST.mDebug_fprintf("Base captured.\n");

			if ( fChildActor->mShipsDefeated() == 6 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam::kAlliance1:
	case tEnemyTeam::kAlliance2:
	case tEnemyTeam::kAlliance3:
	case tEnemyTeam::kAllianceGen1:
	case tEnemyTeam::kAllianceGen2:
	case tEnemyTeam::kAllianceGen3:
		{}
		break;

	case tNPC1Team::kStarbase:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent8_1_msg );
			if ( fChildActor->mBaseCaptured() )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );		break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );		break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );		break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );		break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );		break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );		break;
		}
		fChildActor->mSetTimedInterval( kTimerSecondsSet, timerID,
										( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}

#endif

	switch ( timerID )
	{
	case tBaseVictory::kReinforcementsTimerID:
		mCreateAllianceReinforcements();
		break;
	}
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


void tBaseVictoryState::mCreateAllianceReinforcements()
{
    gST.mDebug_fprintf("--- Entering: tBaseVictoryState::mCreateAllianceReinforcements\n");
    gST.mCompleteNarrowedList();

	fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_1_msg );
	tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	Assert( enemyTeam );
	if ( !enemyTeam )
		return;

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
		ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
		sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips(sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassSmallMonster);
		shipClassTypeList.push_back(kEClassMediumMonster);
		for (int32 i = kEClassFrigate; i <= kEClassNewHeavyCruiser; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		if (sl.size() > 0) {

			for (int32 ii = 0; ii <= 2; ii++) {
				eRace race_picked = kNeutralRace;
				switch ( rand() % 7 )
				{
				case 0: //	Federation
					race_picked = kFederation; break;
				case 1:	//	Klingon
					race_picked = kKlingon; break;
				case 2:	//	Romulan
					race_picked = kRomulan; break;
				case 3:	//	Lyran
					race_picked = kLyran; break;
				case 4:	//	Gorn
					race_picked = kGorn; break;
				case 5:	//	Hydran
					race_picked = kHydran; break;
				case 6:	//	Mirak
					race_picked = kMirak; break;
				}
				ShipList race_sl = gST.mFilterShipListToSingleRace( sl, race_picked );

				Ship s1;
				race_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("pass %d: ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
					ii, s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				enemyTeam->mCreateShip(
					typeid( tAllianceShipGenerated ), 
					s1.shipCode.c_str(), 
					(kStartPosition_V - ii), 
					tTeamInfo::kNoMetaShipID,
					(tEnemyTeam::kAllianceGen1 + ii), 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
			} // end for loop

		} else {
			mOldCreateAllianceReinforcements();
		}
	} else {
		mOldCreateAllianceReinforcements();
	}
	gST.mDebug_fprintf("--- Finished: tBaseVictoryState::mCreateAllianceReinforcements\n");
}


void tBaseVictoryState::mOldCreateAllianceReinforcements()
{
	gST.mDebug_fprintf("--- Entering: tOldBaseVictoryState::mCreateAllianceReinforcements\n");

	fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_1_msg );
	tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	Assert( enemyTeam );
	if ( !enemyTeam )
		return;

	//
	//	First alliance ship
	//
	switch ( fMissionInfo->mRandomInt32( 0 , 6 ) )
	{
	case 0: //	Federation
		enemyTeam->mCreateShip(	typeid( tAllianceShipGenerated ), "F-CB" , kStartPosition_V,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen1, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		enemyTeam->mCreateShip(	typeid( tAllianceShipGenerated ), "K-FQL" , kStartPosition_V,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen1, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		enemyTeam->mCreateShip(	typeid( tAllianceShipGenerated ), "R-KRC" , kStartPosition_V,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen1, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "L-CL" , kStartPosition_V,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen1, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "G-CM" , kStartPosition_V,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen1, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "H-DWF" , kStartPosition_V,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen1, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "Z-DD" , kStartPosition_V,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen1, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	}

	//
	//	Second alliance ship
	//
	switch ( fMissionInfo->mRandomInt32( 0 , 6 ) )
	{
	case 0: //	Federation
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "F-CB" , kStartPosition_U,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen2, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "K-FQL" , kStartPosition_U,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen2, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "R-KRC" , kStartPosition_U,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen2, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "L-CL" , kStartPosition_U,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen2, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "G-CM" , kStartPosition_U,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen2, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "H-DWF" , kStartPosition_U,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen2, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "Z-DD" , kStartPosition_U,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen2, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	}

	//
	//	Thrid alliance ship
	//
	switch ( fMissionInfo->mRandomInt32( 0 , 6 ) )
	{
	case 0: //	Federation
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "F-CB" , kStartPosition_T,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen3, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "K-FQL" , kStartPosition_T,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen3, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "R-KRC" , kStartPosition_T,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen3, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "L-CL" , kStartPosition_T,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen3, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "G-CM" , kStartPosition_T,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen3, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "H-DWF" , kStartPosition_T,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen3, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		enemyTeam->mCreateShip( typeid( tAllianceShipGenerated ), "Z-DD" , kStartPosition_T,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kAllianceGen3, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );	break;
	}

	gST.mDebug_fprintf("--- Finished: tOldBaseVictoryState::mCreateAllianceReinforcements\n");
}	
	
