/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AllianceShip.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "ISC_6Immunity.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}


void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
		ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
		sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips(sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassSmallMonster);
		shipClassTypeList.push_back(kEClassMediumMonster);
		for (int32 i = kEClassFrigate; i <= kEClassNewHeavyCruiser; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		if (sl.size() > 0) {

			for (int32 ii = 0; ii <= 2; ii++) {
				eRace race_picked = kNeutralRace;
				switch ( rand() % 7 )
				{
				case 0: //	Federation
					race_picked = kFederation; break;
				case 1:	//	Klingon
					race_picked = kKlingon; break;
				case 2:	//	Romulan
					race_picked = kRomulan; break;
				case 3:	//	Lyran
					race_picked = kLyran; break;
				case 4:	//	Gorn
					race_picked = kGorn; break;
				case 5:	//	Hydran
					race_picked = kHydran; break;
				case 6:	//	Mirak
					race_picked = kMirak; break;
				}
				ShipList race_sl = gST.mFilterShipListToSingleRace( sl, race_picked );

				Ship s1;
				race_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("pass %d: ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
					ii, s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				mCreateShip(
					typeid( tAllianceShip ), 
					s1.shipCode.c_str(), 
					(kStartPosition_Y - ii), 
					kNoMetaShipID,
					(kAlliance1 + ii), 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
			} // end for loop

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	//
	//	First alliance ship
	//
	switch ( rand() % 7 )
	{
	case 0: //	Federation
		mCreateShip(	typeid( tAllianceShip ), "F-CB" , kStartPosition_Y,
						kNoMetaShipID, kAlliance1, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		mCreateShip(	typeid( tAllianceShip ), "K-FQL" , kStartPosition_Y,
						kNoMetaShipID, kAlliance1, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		mCreateShip(	typeid( tAllianceShip ), "R-KRC" , kStartPosition_Y,
						kNoMetaShipID, kAlliance1, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		mCreateShip(	typeid( tAllianceShip ), "L-CL" , kStartPosition_Y,
						kNoMetaShipID, kAlliance1, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		mCreateShip(	typeid( tAllianceShip ), "G-CM" , kStartPosition_Y,
						kNoMetaShipID, kAlliance1, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		mCreateShip(	typeid( tAllianceShip ), "H-DWF" , kStartPosition_Y,
						kNoMetaShipID, kAlliance1, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		mCreateShip(	typeid( tAllianceShip ), "Z-DD" , kStartPosition_Y,
						kNoMetaShipID, kAlliance1, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	}

	//
	//	Second alliance ship
	//
	switch ( rand() % 7 )
	{
	case 0: //	Federation
		mCreateShip(	typeid( tAllianceShip ), "F-CB" , kStartPosition_X,
						kNoMetaShipID, kAlliance2, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		mCreateShip(	typeid( tAllianceShip ), "K-FQL" , kStartPosition_X,
						kNoMetaShipID, kAlliance2, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		mCreateShip(	typeid( tAllianceShip ), "R-KRC" , kStartPosition_X,
						kNoMetaShipID, kAlliance2, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		mCreateShip(	typeid( tAllianceShip ), "L-CL" , kStartPosition_X,
						kNoMetaShipID, kAlliance2, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		mCreateShip(	typeid( tAllianceShip ), "G-CM" , kStartPosition_X,
						kNoMetaShipID, kAlliance2, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		mCreateShip(	typeid( tAllianceShip ), "H-DWF" , kStartPosition_X,
						kNoMetaShipID, kAlliance2, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		mCreateShip(	typeid( tAllianceShip ), "Z-DD" , kStartPosition_X,
						kNoMetaShipID, kAlliance2, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	}

	//
	//	Third alliance ship
	//
	switch ( rand() % 7 )
	{
	case 0: //	Federation
		mCreateShip(	typeid( tAllianceShip ), "F-CB" , kStartPosition_W,
						kNoMetaShipID, kAlliance3, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		mCreateShip(	typeid( tAllianceShip ), "K-FQL" , kStartPosition_W,
						kNoMetaShipID, kAlliance3, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		mCreateShip(	typeid( tAllianceShip ), "R-KRC" , kStartPosition_W,
						kNoMetaShipID, kAlliance3, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		mCreateShip(	typeid( tAllianceShip ), "L-CL" , kStartPosition_W,
						kNoMetaShipID, kAlliance3, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		mCreateShip(	typeid( tAllianceShip ), "G-CM" , kStartPosition_W,
						kNoMetaShipID, kAlliance3, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		mCreateShip(	typeid( tAllianceShip ), "H-DWF" , kStartPosition_W,
						kNoMetaShipID, kAlliance3, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		mCreateShip(	typeid( tAllianceShip ), "Z-DD" , kStartPosition_W,
						kNoMetaShipID, kAlliance3, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );	break;
	}

	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}