/********************************************************************************************************\

	Mission:	Diplomatic Immunity
	Script:		"ISC_6Immunity"
	Descript:	Capture a Federation base.

	Created by:		Daniel J. Suleski		03/28/00
	Last Updated:	DJS						03/31/00
					Second draft completed
					DJS						04/11/00
					Fix multiple-player ship random chatter
					Got rid of references to time.h
					DJS						05/15/00
					Initial conversion from old API to new multiplayer API
					DJS						05/20/00
					Asserts added and code cleaned
					DJS						05/26/00
					tShipIterator::mGetFirstShipInPlay() replaced with original form
					DJS						08/23/00
					Map overrides.
					DJS						09/25/00
					mCreateShip() "default-removal" modifications, kCampaignMission, tTeamSpec, etc
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "ISC_6Immunity.h"
#include "BaseVictory.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "NPC1Team.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "VictoryCondition.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tISC_6Immunity );

tISC_6Immunity::tISC_6Immunity()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tISC_6Immunity::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team",
													static_cast< eTeamID >( kPlayerTeam ),
													kMaxChance,
													kTrue,
													kPrimaryTeam,
													kMinChance,
													kSpecISC,
													kAnyRace,
													typeid( tPlayerTeam ),
													kNoTeamTag,
													Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& playerNPCTeam = mCreateTeamSpec(	"Player NPC Team",
														static_cast< eTeamID >( kPlayerNPCTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tPlayerNPCTeam ) );

	const tTeamSpec& enemyTeam = mCreateTeamSpec(	"Enemy Team",
													static_cast< eTeamID >( kEnemyTeam ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak,
													kAnyRace,
													typeid( tEnemyTeam ) );

	const tTeamSpec& npc1Team = mCreateTeamSpec(	"NPC 1 Team",
													static_cast< eTeamID >( kNPC1Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak,
													kAnyRace,
													typeid( tNPC1Team ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(	"Draft Team 1",
												static_cast< eTeamID >( kDraftTeam1 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tDraftTeam ) );

	const tTeamSpec& draft2Team = mCreateTeamSpec(	"Draft Team 2",
												static_cast< eTeamID >( kDraftTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tDraftTeam ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( playerNPCTeam );
	Specs->mSetupTeam( enemyTeam );
	Specs->mSetupTeam( npc1Team );
	Specs->mSetupTeam( draft1Team );
	Specs->mSetupTeam( draft2Team );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tISC_6Immunity::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tISC_6Immunity::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tISC_6Immunity::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tISC_6Immunity::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tISC_6Immunity::mStartMission( void )
{
	tScript::mStartMission();

/*

	HELPER NOTE:	The teamList array that you see below should reflect those constants that have been or you
					have declared in the matching header file (e.g. ISC_6Immunity.h).

*/
	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kNPC1Team,
	};

/*

	HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
					meaning has a meaning???

	Relationship Lookup
	- = kTeamSelfAware	//	use this parameter with itself
	A = kTeamAllied		//	cannot fire upon an ally
	F = kTeamFriendly
	N = kTeamNeutral
	H = kTeamHostile	//	will not fire / capture / cripple until provoked
	W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
	P = kTeamPassive

	Relationship Lookup
	- = kGoalIgnore		//	use this parameter with itself
	I = kGoalIgnore
	C = kGoalCripple
	X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
	D = kGoalDestroy

*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = PlayersTeam,
		//PNPC = PlayerNPCTeam,
		//Nme  = EnemyTeam,
		//NPC1 = NPC1 -- Federation StarBase

		//	     Plyr  PNPC  Nme   NPC1
		/*Plyr*/ "--", "--", "WD", "WX",
		/*PNPC*/ "--", "--", "--", "--",
		/*Nme */ "WD", "--", "--", "AI",
		/*NPC1*/ "WD", "--", "AI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tISC_6Immunity::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_6Immunity::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyBase;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_6Immunity::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}


/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tISC_6Immunity::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}