/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "ISC_6Immunity.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if ( victoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDisconnect_msg );
		return;
	}

/*

	HELPER NOTE:	The following is the analysis of the mission.  You are able to place additional text
					dependant upon the victory outcome in the MissionText.cpp file.  Simply find the
					corresponding values.

*/
	//
	//	Add the appropriate messages to the debriefing list according to the player's performance.
	//
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictory_msg );		break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeat_msg );		break;
	case kUnknownVictoryLevel:
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	You may notice that the mCreateShipsForTeam(...) in this file has a different signature
					and body then the other team files.  This is because the 'tPlayerTeam' is of a
					'tPlayableTeam' class and it would be unknown how many ships the player has control
					of until the game has already begun.  However, if you want to force the player to play
					with a certain amount of ships (perhaps for a special mission), then the body of this
					function would look much more like the other team files.  To get an idea of this, refer
					to the helper notes on those team files.

*/
void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;

	switch( fMissionInfo->mRandomInt32( 1, 20 ) )
	{
	case 1: 
	case 2: 
	case 3: 
	case 4: 
	case 5:
		fMissionScheduler.fNextMissionTitle = "ISC_X3M1BTemple";
		break;
	default:
		fMissionScheduler.fNextMissionTitle = "ISC_7BattleGrave";
		break;
	}
}