/********************************************************************************************************\

	File Name:	AllyShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllyShipBaseState.h"
#include "ISC_7BattleGrave.h"
#include "MissionText.h"
#include "PlayerNPCTeam.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tAllyShipBaseState::mBeginState()
{
    gST.mDebug_fprintf("--- Starting: tAllyShipBaseState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, tAllyShip::kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tAllyShipBaseState::mBeginState\n");
}

void tAllyShipBaseState::mEndState()
{}

void tAllyShipBaseState::mOnDeath( tShipInfo* ship )
{
	Assert( ship );
	if ( ship )
	{
		if ( ship->mGetCustomID() == tPlayerNPCTeam::kAllyShip )
		{
			if ( ship->mGetShipID() == fChildActor->mGetShipID() )
			{
				fChildActor->mSetTimedInterval( kTimerTurnsSet, tAllyShip::kReinforcementsTimerID, 1.5f );
			}
		}
	}
}
void tAllyShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tAllyShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tAllyShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( ( !ship ) || ( !targetShip ) )
		return;

	float	hullDamage = targetShip->mGetDamagePercent();
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	tShipIterator	playerShip = playerTeam->mGetFirstShip();
	tShipInPlayEval	inPlayEval;
	playerShip.mAddEvaluator( &inPlayEval );

	switch ( buttonID )
	{
	case k_ATTACKTARGET_Ally_btn:
		{
			if ( ( hullDamage < 60.0f ) && ( !fChildActor->mAttackedAlly() ) )
			{
				if ( *playerShip != NULL )
					targetShip->mDefendEntity( *playerShip, 50.0f, 10.0f, kAbsolutePriority );
			}
		}
		break;
	}
}
void tAllyShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;

	float	hullDamage = damagedShip->mGetDamagePercent();
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	//
	//	The ally ship was attacked by the player, so the ally ship should no longer respond to the
	//	comm states from the player.
	//
	if (	( hostileShip ) && ( ( fChildActor->mAttackedAlly() ) || 
			( hostileShip->mGetTeam() == kPlayerTeam ) )	)
	{
		fChildActor->mSetAttackedAlly( true );
		playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState1 );
		fChildActor->mSetShipCallbackEvents( kNotifyDamage );;
	}
	//
	//	Change comm states according to the damage taken by the allied ship.
	//
	else if ( damagedShip->mGetCustomID() == fChildActor->mGetCustomID() )
	{
		if ( hullDamage == 0.0f )
		{
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState2 );
		}
		else if ( hullDamage < 30.0f )
		{
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState3 );
		}
		else if ( hullDamage < 60.0f )
		{
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState4 );
		}
		else
		{
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState5 );
		}
		fChildActor->mSetShipCallbackEvents( kNotifyDamage );
	}
}
void tAllyShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tAllyShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tAllyShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllyShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllyShipBaseState::mOnFlashingButtonPressed()
{}
void tAllyShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tAllyShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tAllyShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tAllyShipBaseState::mOnHostChanged()
{}
void tAllyShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch ( timerID )
	{
	case tAllyShip::kReinforcementsTimerID:
		{
			mCreateAllyReinforcements();
		}
		break;
	case tAllyShip::kUpgradeFightersID:
		{
#ifdef FIGHTERS
			if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
			tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
			if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
			if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC. 

			gST.mDebug_fprintf("--- Entering tAllyShipBaseState::mOnTimedInterval -- timer tAllyShip::kUpgradeFightersID triggered\n");

			int32 BPVinc = gST.mUpgradeFighters( fChildActor );
            fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );

			gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
			gST.mDebug_fprintf("--- Finished: tAllyShipBaseState::mOnTimedInterval -- timer tAllyShip::kUpgradeFightersID\n");
#endif
		}
		break;
	}
}
void tAllyShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tAllyShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

void tAllyShipBaseState::mCreateAllyReinforcements()
{
    gST.mDebug_fprintf("--- Entering: tAllyShipBaseState::mCreateAllyReinforcements\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {

		tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		tTeamInfo*	playerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
		Assert( playerNPCTeam && playerTeam );
		if ( ( !playerNPCTeam ) || ( !playerTeam ) )
			return;

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 nPCBPV; 
		switch ( diffLevel )
		{
		case 0:
			nPCBPV = playerBPV * 0.75;	break;
		case 1:
			nPCBPV = playerBPV * 1.0;	break;
		case 2:
			nPCBPV = playerBPV * 1.25;	break;
		}

        int32 fShiplistCount;
		ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
		sl = gST.stripOutRestricted( sl );
		sl = gST.mFilterShipListToSingleRace( sl, playerTeam->mGetRace() );
		sl = gST.selectWarShips(sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassSmallMonster);
		shipClassTypeList.push_back(kEClassMediumMonster);
		shipClassTypeList.push_back(kEClassLargeMonster);
		for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        fShiplistCount = sl.Subset( sl.kSubsetBPV, int(0.825 * nPCBPV), int(1.175 * nPCBPV), true);
        gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
            int(0.825 * nPCBPV), int(1.175 * nPCBPV), fShiplistCount);

		if (sl.size() > 0) {
			Ship s1;
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			playerNPCTeam->mCreateShip(
				typeid( tAllyShip ),
				s1.shipCode.c_str(),
				kStartPosition_H,
				tTeamInfo::kNoMetaShipID,
				tPlayerNPCTeam::kAllyShip,
				0, 0, -1, -1, NULL,
				kDefaultShipOptions );
		} else {
			mOldCreateAllyReinforcements();
		}
	} else {
		mOldCreateAllyReinforcements();
	}

    gST.mDebug_fprintf("--- Finished: tAllyShipBaseState::mCreateAllyReinforcements\n");
}

void tAllyShipBaseState::mOldCreateAllyReinforcements(void)
{
	gST.mDebug_fprintf("---- Entering: tAllyShipBaseState::mOldCreateAllyReinforcements\n");

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tTeamInfo*	playerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
	Assert( playerNPCTeam && playerTeam );
	if ( ( !playerNPCTeam ) || ( !playerTeam ) )
		return;

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	switch ( diffLevel )
	{
	case 0:
		nPCBPV = playerBPV * 0.75;	break;
	case 1:
		nPCBPV = playerBPV * 1.0;	break;
	case 2:
		nPCBPV = playerBPV * 1.25;	break;
	}
	playerNPCTeam->mCreateShip(	typeid( tAllyShip ),
								nPCBPV,
								kClassNewHeavyCruiser,
								kClassHeavyBattlecruiser,
								0,
								kStartPosition_H,
								tPlayerNPCTeam::kAllyShip,
								0, 0, -1, -1, NULL,
								kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tAllyShipBaseState::mOldCreateAllyReinforcements\n");
}
