/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllyShip.h"
#include "BaseVictoryState.h"
#include "EnemyShip.h"
#include "EnemyTeam.h"
#include "ISC_7BattleGrave.h"
#include "MissionText.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ShipList_API.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	//
	//	Determine if the player had lost all of his ships.
	//
	bool playerDied = false;
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tShipIterator ship = team->mGetFirstShip();
	tShipInPlayEval x;
	ship.mAddEvaluator( &x );
	if ( *ship == NULL )
		playerDied = true;

	if ( fChildActor->mDeserted() )
	{
		return kDevastatingDefeat;
	}
	else
	{
		if ( playerDied )
		{
			return kDefeat;
		}
		else
		{
			switch ( fChildActor->mBasesDestroyed() )
			{
			case 3:
				return kDefeat;
				break;

			case 2:
			case 1:
				return kVictory;
				break;

			case 0:
				return kAstoundingVictory;
				break;
			}
		}
	}
	return kDraw;
}


void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
		}
		break;

	case tPlayerNPCTeam::kBase1:
	case tPlayerNPCTeam::kBase2:
	case tPlayerNPCTeam::kBase3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
			fChildActor->mIncreaseBasesDestroyed();

			if ( fChildActor->mBasesDestroyed() == 3 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_2_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
		}
		break;

	case tEnemyTeam::kTransponder:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent9_1_msg );
			fChildActor->mSetTransponderDestroyed( true );
		}
		break;

	case tEnemyTeam::kEnemy1a:
	case tEnemyTeam::kEnemy1b:
	case tEnemyTeam::kEnemy2a:
	case tEnemyTeam::kEnemy2b:
		{
			mAddToBonusPrestige( 25 );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
			fChildActor->mIncreaseAllianceDestroyed();
			fChildActor->mSetCountWave( fChildActor->mWave() + 1 );

			if ( fChildActor->mWave() == 4 && fChildActor->mTransponderDestroyed() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_2_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else if ( fChildActor->mWave() == 4 )
			{

				
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
				fChildActor->mSetCountWave( 0 );

				tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
				Assert( enemyTeam );
				if ( !enemyTeam )
					break;	//	break early if the enemy team does not exist...this is a problem if this occurs
				mCreateEnemyReinforcements();
			}
			fChildActor->mSetCountWave( fChildActor->mWave() % 4 );
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip:
		{}
		break;

	case tPlayerNPCTeam::kBase1:
	case tPlayerNPCTeam::kBase2:
	case tPlayerNPCTeam::kBase3:
		{
			fChildActor->mIncreaseBasesDestroyed();
		}
		break;

	case tEnemyTeam::kTransponder:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kTransponderCaptured_msg );
			fChildActor->mSetTransponderDestroyed( true );
		}
		break;

	case tEnemyTeam::kEnemy1a:
	case tEnemyTeam::kEnemy1b:
	case tEnemyTeam::kEnemy2a:
	case tEnemyTeam::kEnemy2b:
		{
			mAddToBonusPrestige( 25 );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEnemyShipCaptured_msg );
			fChildActor->mIncreaseAllianceDestroyed();
			fChildActor->mSetCountWave( fChildActor->mWave() + 1 );

			if ( fChildActor->mWave() == 4 && fChildActor->mTransponderDestroyed() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEnemyShipsNeutralized_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else if ( fChildActor->mWave() == 4 )
			{		
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
				fChildActor->mSetCountWave( 0 );

				tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
				Assert( enemyTeam );
				if ( !enemyTeam )
					break;	//	break early if the enemy team does not exist...this is a problem if this occurs
				mCreateEnemyReinforcements();
			}
			fChildActor->mSetCountWave( fChildActor->mWave() % 4 );
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
		}
		break;

	case tPlayerNPCTeam::kBase1:
	case tPlayerNPCTeam::kBase2:
	case tPlayerNPCTeam::kBase3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
			fChildActor->mIncreaseBasesDestroyed();

			if ( fChildActor->mBasesDestroyed() == 3 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_2_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip:
		{}
		break;

	case tPlayerNPCTeam::kBase1:
	case tPlayerNPCTeam::kBase2:
	case tPlayerNPCTeam::kBase3:
		{}
		break;

	case tEnemyTeam::kTransponder:
		{}
		break;

	case tEnemyTeam::kEnemy1a:
	case tEnemyTeam::kEnemy1b:
	case tEnemyTeam::kEnemy2a:
	case tEnemyTeam::kEnemy2b:
		{
			//
			//	The alliance has surrendered and was ordered to disengage fromt he system.  Each time an
			//	alliance ship disengages, it is counted as destroyed.
			//
			mAddToBonusPrestige( 25 );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
			fChildActor->mIncreaseAllianceDestroyed();
			fChildActor->mSetCountWave( fChildActor->mWave() + 1 );

			if ( fChildActor->mWave() == 4 && fChildActor->mTransponderDestroyed() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_2_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else if ( fChildActor->mWave() == 4 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
				fChildActor->mSetCountWave( 0 );

				tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
				Assert( enemyTeam );
				if ( !enemyTeam )
					break;	//	break early if the enemy team does not exist...this is a problem if this occurs
				mCreateEnemyReinforcements();
			}
			fChildActor->mSetCountWave( fChildActor->mWave() % 4 );
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent11_1_msg );
			fChildActor->mSetDeserted( true );
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{
/*	HOLD FOR PASTING
	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip:
		{}
		break;

	case tPlayerNPCTeam::kBase1:
	case tPlayerNPCTeam::kBase2:
	case tPlayerNPCTeam::kBase3:
		{}
		break;

	case tEnemyTeam::kTransponder:
		{}
		break;

	case tEnemyTeam::kEnemy1a:
	case tEnemyTeam::kEnemy1b:
	case tEnemyTeam::kEnemy2a:
	case tEnemyTeam::kEnemy2b:
		{}
		break;
	}

	if ( ship->mGetTeam() == kPlayerTeam )
	{
		tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );
		Assert( playerTeam );
		if ( playerTeam )
		{
			if ( playerTeam->mGetFirstShipInPlay() == NULL )
				gScriptInterface.mSetGameStatus( kPlayerDestroyed );
		}
	}
*/
}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );		break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );		break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );		break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );		break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );		break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );		break;
		}
		fChildActor->mSetTimedInterval(	kTimerSecondsSet, timerID, ( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif

	switch ( timerID )
	{
	case tAllyShip::kReinforcementsTimerID:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );
		}
		break;
	}
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


void tBaseVictoryState::mCreateEnemyReinforcements( void )
{
    gST.mDebug_fprintf("--- Entering: tBaseVictoryState::mCreateEnemyReinforcements\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
		Assert( enemyTeam );
		if ( !enemyTeam )
			return;	//	break early if the enemy team does not exist...this is a problem if this occurs

		// CAs and NCAs
		ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
		sl = gST.stripOutRestricted( sl );

		std::list< eEnhancedRoles >  warRolesList;
        warRolesList.push_back(kLineShip);
        warRolesList.push_back(kWarShip);
        warRolesList.push_back(kLeader);
        warRolesList.push_back(kFastCruiser);
        fShiplistCount = sl.Subset(warRolesList);
        gST.mDebug_fprintf("subset for base war classes eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassMediumMonster);
		for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		if (sl.size() > 0) {

			int32 CustomID = tEnemyTeam::kEnemy1a;
			for (int32 ii = 0; ii <= 1; ii++) {
				eRace race_picked = kNeutralRace;
				switch ( rand() % 7 )
				{
				case 0: //	Federation
					race_picked = kFederation; break;
				case 1:	//	Klingon
					race_picked = kKlingon; break;
				case 2:	//	Romulan
					race_picked = kRomulan; break;
				case 3:	//	Lyran
					race_picked = kLyran; break;
				case 4:	//	Gorn
					race_picked = kGorn; break;
				case 5:	//	Hydran
					race_picked = kHydran; break;
				case 6:	//	Mirak
					race_picked = kMirak; break;
				}
				ShipList race_sl = gST.mFilterShipListToSingleRace( sl, race_picked );

				Ship s1;
				race_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("pass %d: ship %s selected. ID: %d  BPV: %d  YFA: %d  YLA %d\n",
					ii, s1.shipCode.c_str(), CustomID, s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				enemyTeam->mCreateShip(
					typeid( tEnemyShip ), 
					s1.shipCode.c_str(), 
					kStartPosition_Y, 
					tTeamInfo::kNoMetaShipID,
					CustomID, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
				CustomID++;

				Ship s2;
				race_sl.SelectRandomShip(s2);
				gST.mDebug_fprintf("pass %d: ship %s selected. ID: %d  BPV: %d  YFA: %d  YLA %d\n",
					ii, s2.shipCode.c_str(), CustomID, s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
				gST.mTrackOrions( s2 );

				enemyTeam->mCreateShip(
					typeid( tEnemyShip ), 
					s2.shipCode.c_str(), 
					kStartPosition_X, 
					tTeamInfo::kNoMetaShipID,
					CustomID, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
				CustomID++;
			} // end for loop

		} else {
			mOldCreateEnemyReinforcements();
		}
	} else {
		mOldCreateEnemyReinforcements();
	}
	gST.mDebug_fprintf("--- Finished: tBaseVictoryState::mCreateEnemyReinforcements\n");
}


void tBaseVictoryState::mOldCreateEnemyReinforcements( void )
{
	gST.mDebug_fprintf("---- Entering: tBaseVictoryState::mOldCreateEnemyReinforcements\n");

	tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	Assert( enemyTeam );
	if ( !enemyTeam )
		return;	//	break early if the enemy team does not exist...this is a problem if this occurs

	//
	//	First set of ships.
	//
	switch ( fMissionInfo->mRandomInt32( 0, 6 ) )
	{
	case 0:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "F-CA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "F-CA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 1:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "K-D7B", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "K-D7B", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;
		
	case 2:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "R-FHK", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "R-FHK", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 3:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "L-NCA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "L-NCA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 4:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "H-RN", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "H-RN", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 5:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "G-CA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "G-CA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 6:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "Z-CA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "Z-CA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy1b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;
	}

	//
	//	Second set of ships.
	//
	switch ( fMissionInfo->mRandomInt32( 0, 6 ) )
	{
	case 0:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "F-CA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "F-CA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 1:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "K-D7B", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "K-D7B", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;
		
	case 2:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "R-FHK", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "R-FHK", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 3:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "L-NCA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "L-NCA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 4:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "H-RN", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "H-RN", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 5:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "G-CA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2a, 0, 0, -1, -1, 
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "G-CA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;

	case 6:
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "Z-CA", kStartPosition_Y,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2a, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		enemyTeam->mCreateShip( typeid( tEnemyShip ), "Z-CA", kStartPosition_X,
								tTeamInfo::kNoMetaShipID, tEnemyTeam::kEnemy2b, 0, 0, -1, -1,
								NULL, kDefaultShipOptions );
		break;
	}

	gST.mDebug_fprintf("---- Finished: tBaseVictoryState::mOldCreateEnemyReinforcements\n");
}