/********************************************************************************************************\

	File Name:	EnemyShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyShip.h"
#include "EnemyShipBaseState.h"
#include "EnemyTeam.h"
#include "ISC_7BattleGrave.h"
#include "MissionText.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tEnemyShipBaseState::mBeginState()
{
    gST.mDebug_fprintf("--- Starting: tEnemyShipBaseState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tEnemyShipBaseState::mBeginState\n");
}

void tEnemyShipBaseState::mEndState()
{}

void tEnemyShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemyShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tEnemyShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tEnemyShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;
	float	hullDamage = damagedShip->mGetDamagePercent();
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	if ( damagedShip->mGetCustomID() == fChildActor->mGetCustomID() )
	{
		if ( hullDamage < 50.0f )
		{
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tEnemyShip::kCommState1 );
		}
		else
		{
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tEnemyShip::kCommState2 );
		}
		fChildActor->mSetShipCallbackEvents( kNotifyDamage );
	}
}
void tEnemyShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipBaseState::mOnFlashingButtonPressed()
{}
void tEnemyShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tEnemyShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tEnemyShipBaseState::mOnHostChanged()
{}

void tEnemyShipBaseState::mOnTimedInterval( int32 timerID )
{
	if (timerID == kUpgradeFightersID) {
#ifdef FIGHTERS
		if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
		tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
		if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
		if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC. 

		gST.mDebug_fprintf("--- Entering tEnemyShipBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

		int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );

		gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
		gST.mDebug_fprintf("--- Finished: tEnemyShipBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
	}
}

void tEnemyShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


