/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyShip.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "transponder.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}


void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		// Listening Post
		ShipList lp_sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        // Listening post platforms
        std::list< eEnhancedClassTypes >  LPClassTypeList;
        LPClassTypeList.push_back(kEClassListeningPost);
        fShiplistCount = lp_sl.Subset(LPClassTypeList, true);
        gST.mDebug_fprintf("subset for kEClassListeningPost class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  LPRolesList;
        LPRolesList.push_back(kListeningPost);
        fShiplistCount = lp_sl.Subset(LPRolesList, true);
        gST.mDebug_fprintf("subset for kListeningPost eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(lp_sl);


		// CAs and NCAs
		ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
		sl = gST.stripOutRestricted( sl );

		std::list< eEnhancedRoles >  warRolesList;
        warRolesList.push_back(kLineShip);
        warRolesList.push_back(kWarShip);
        warRolesList.push_back(kLeader);
        warRolesList.push_back(kFastCruiser);
        fShiplistCount = sl.Subset(warRolesList);
        gST.mDebug_fprintf("subset for base war classes eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassMediumMonster);
		for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		if ((sl.size() > 0) && (lp_sl.size() > 0)) {
			Ship lp1;
			lp_sl.SelectRandomShip(lp1);
			gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
				lp1.shipCode.c_str(), lp1.bpv, lp1.yearFirstAvail, lp1.yearLastAvail);
			gST.mTrackOrions( lp1 );

			mCreateShip(
				typeid( tTransponder ), 
				lp1.shipCode.c_str(), 
				kStartPosition_Z,
				kNoMetaShipID, 
				kTransponder, 
				0, 0, -1, -1,
				NULL, kDefaultShipOptions );

			int32 CustomID = kEnemy1a;
			for (int32 ii = 0; ii <= 1; ii++) {
				eRace race_picked = kNeutralRace;
				switch ( rand() % 7 )
				{
				case 0: //	Federation
					race_picked = kFederation; break;
				case 1:	//	Klingon
					race_picked = kKlingon; break;
				case 2:	//	Romulan
					race_picked = kRomulan; break;
				case 3:	//	Lyran
					race_picked = kLyran; break;
				case 4:	//	Gorn
					race_picked = kGorn; break;
				case 5:	//	Hydran
					race_picked = kHydran; break;
				case 6:	//	Mirak
					race_picked = kMirak; break;
				}
				ShipList race_sl = gST.mFilterShipListToSingleRace( sl, race_picked );

				Ship s1;
				race_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("pass %d: ship %s selected. ID: %d  BPV: %d  YFA: %d  YLA %d\n",
					ii, s1.shipCode.c_str(), CustomID, s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				mCreateShip(
					typeid( tEnemyShip ), 
					s1.shipCode.c_str(), 
					kStartPosition_Y, 
					kNoMetaShipID,
					CustomID, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
				CustomID++;

				Ship s2;
				race_sl.SelectRandomShip(s2);
				gST.mDebug_fprintf("pass %d: ship %s selected. ID: %d  BPV: %d  YFA: %d  YLA %d\n",
					ii, s2.shipCode.c_str(), CustomID, s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
				gST.mTrackOrions( s2 );

				mCreateShip(
					typeid( tEnemyShip ), 
					s2.shipCode.c_str(), 
					kStartPosition_X, 
					kNoMetaShipID,
					CustomID, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
				CustomID++;
			} // end for loop

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tTransponder ), "O-LP", kStartPosition_Z,
					kNoMetaShipID, kTransponder, 0, 0, -1, -1,
					NULL, kDefaultShipOptions );

	//
	//	First set of ships.
	//
	switch ( fMissionInfo->mRandomInt32( 0, 6 ) )
	{
	case 0:
		mCreateShip(	typeid( tEnemyShip ), "F-CA", kStartPosition_Y,
						kNoMetaShipID, kEnemy1a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "F-CA", kStartPosition_X, 
						kNoMetaShipID, kEnemy1b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 1:
		mCreateShip(	typeid( tEnemyShip ), "K-D7B", kStartPosition_Y, 
						kNoMetaShipID, kEnemy1a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "K-D7B", kStartPosition_X,
						kNoMetaShipID, kEnemy1b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;
		
	case 2:
		mCreateShip(	typeid( tEnemyShip ), "R-FHK", kStartPosition_Y, 
						kNoMetaShipID, kEnemy1a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "R-FHK", kStartPosition_X,
						kNoMetaShipID, kEnemy1b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 3:
		mCreateShip(	typeid( tEnemyShip ), "L-NCA", kStartPosition_Y,
						kNoMetaShipID, kEnemy1a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "L-NCA", kStartPosition_X,
						kNoMetaShipID, kEnemy1b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 4:
		mCreateShip(	typeid( tEnemyShip ), "H-RN", kStartPosition_Y,
						kNoMetaShipID, kEnemy1a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "H-RN", kStartPosition_X,
						kNoMetaShipID, kEnemy1b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 5:
		mCreateShip(	typeid( tEnemyShip ), "G-CA", kStartPosition_Y,
						kNoMetaShipID, kEnemy1a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "G-CA", kStartPosition_X,
						kNoMetaShipID, kEnemy1b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 6:
		mCreateShip(	typeid( tEnemyShip ), "Z-CA", kStartPosition_Y,
						kNoMetaShipID, kEnemy1a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "Z-CA", kStartPosition_X, 
						kNoMetaShipID, kEnemy1b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;
	}

	//
	//	Second set of ships.
	//
	switch ( fMissionInfo->mRandomInt32( 0, 6 ) )
	{
	case 0:
		mCreateShip(	typeid( tEnemyShip ), "F-CA", kStartPosition_Y,
						kNoMetaShipID, kEnemy2a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "F-CA", kStartPosition_X,
						kNoMetaShipID, kEnemy2b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 1:
		mCreateShip(	typeid( tEnemyShip ), "K-D7B", kStartPosition_Y, 
						kNoMetaShipID, kEnemy2a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "K-D7B", kStartPosition_X, 
						kNoMetaShipID, kEnemy2b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;
		
	case 2:
		mCreateShip(	typeid( tEnemyShip ), "R-FHK", kStartPosition_Y, 
						kNoMetaShipID, kEnemy2a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "R-FHK", kStartPosition_X,
						kNoMetaShipID, kEnemy2b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 3:
		mCreateShip(	typeid( tEnemyShip ), "L-NCA", kStartPosition_Y,
						kNoMetaShipID, kEnemy2a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "L-NCA", kStartPosition_X,
						kNoMetaShipID, kEnemy2b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 4:
		mCreateShip(	typeid( tEnemyShip ), "H-RN", kStartPosition_Y,
						kNoMetaShipID, kEnemy2a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "H-RN", kStartPosition_X,
						kNoMetaShipID, kEnemy2b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 5:
		mCreateShip(	typeid( tEnemyShip ), "G-CA", kStartPosition_Y,
						kNoMetaShipID, kEnemy2a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "G-CA", kStartPosition_X,
						kNoMetaShipID, kEnemy2b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;

	case 6:
		mCreateShip(	typeid( tEnemyShip ), "Z-CA", kStartPosition_Y,
						kNoMetaShipID, kEnemy2a, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tEnemyShip ), "Z-CA", kStartPosition_X,
						kNoMetaShipID, kEnemy2b, 0, 0, -1, -1,
						NULL, kDefaultShipOptions );
		break;
	}

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}
