/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AllyShip.h"
#include "BaseStation.h"
#include "ISC_7BattleGrave.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}


void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {

		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( playerTeam );
		if ( !playerTeam )
			return;

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 nPCBPV; 
		switch ( diffLevel )
		{
		case 0:
			nPCBPV = playerBPV * 0.75;	break;
		case 1:
			nPCBPV = playerBPV * 1.0;	break;
		case 2:
			nPCBPV = playerBPV * 1.25;	break;
		}

        int32 fShiplistCount;
		ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
		sl = gST.stripOutRestricted( sl );
		sl = gST.mFilterShipListToSingleRace( sl, playerTeam->mGetRace() );
		sl = gST.selectWarShips(sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassSmallMonster);
		shipClassTypeList.push_back(kEClassMediumMonster);
		shipClassTypeList.push_back(kEClassLargeMonster);
		for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        fShiplistCount = sl.Subset( sl.kSubsetBPV, int(0.825 * nPCBPV), int(1.175 * nPCBPV), true);
        gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
            int(0.825 * nPCBPV), int(1.175 * nPCBPV), fShiplistCount);

		if (sl.size() > 0) {
			Ship s1;
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			mCreateShip(
				typeid( tAllyShip ),
				s1.shipCode.c_str(),
				kStartPosition_H,
				kNoMetaShipID,
				kAllyShip,
				0, 0, -1, -1, NULL,
				kDefaultShipOptions );

			mCreateReeducationCamps();
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}

void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tPlayerNPCTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	switch ( diffLevel )
	{
	case 0:
		nPCBPV = playerBPV * 0.75;	break;
	case 1:
		nPCBPV = playerBPV * 1.0;	break;
	case 2:
		nPCBPV = playerBPV * 1.25;	break;
	}

	mCreateShip(	typeid( tAllyShip ),
					nPCBPV,
					kClassNewHeavyCruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_H,
					kAllyShip,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

	mCreateReeducationCamps();

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mCreateReeducationCamps( void ) {
	gST.mDebug_fprintf("---- Entering: tPlayerNPCTeam::mCreateReeducationCamps\n");

#if defined ( _GERMAN )
	mCreateShip(	typeid( tBaseStation ),
					"I-BSN",
					kStartPosition_I,
					kNoMetaShipID,
					kBase1,
					0, 0, -1, -1, "Neuausbildungslager",
					kDefaultShipOptions );
	mCreateShip(	typeid( tBaseStation ),
					"I-BSN",
					kStartPosition_J,
					kNoMetaShipID,
					kBase2,
					0, 0, -1, -1, "Neuausbildungslager",
					kDefaultShipOptions );
	mCreateShip(	typeid( tBaseStation ),
					"I-BSN",
					kStartPosition_K,
					kNoMetaShipID,
					kBase3,
					0, 0, -1, -1, "Neuausbildungslager",
					kDefaultShipOptions );
#else
	mCreateShip(	typeid( tBaseStation ),
					"I-BSN",
					kStartPosition_I,
					kNoMetaShipID,
					kBase1,
					0, 0, -1, -1, "Reeducation Camp",
					kDefaultShipOptions );
	mCreateShip(	typeid( tBaseStation ),
					"I-BSN",
					kStartPosition_J,
					kNoMetaShipID,
					kBase2,
					0, 0, -1, -1, "Reeducation Camp",
					kDefaultShipOptions );
	mCreateShip(	typeid( tBaseStation ),
					"I-BSN",
					kStartPosition_K,
					kNoMetaShipID,
					kBase3,
					0, 0, -1, -1, "Reeducation Camp",
					kDefaultShipOptions );
#endif

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mCreateReeducationCamps\n");
}


void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}


void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}