/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "EnemyShip.h"
#include "Enemy2Ship.h"
#include "Enemy3Ship.h"
#include "EnemyTeam.h"
#include "Enemy2Team.h"
#include "Enemy3Team.h"
#include "Freighter.h"
#include "ISC_8Diplomacy.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if ( fChildActor->mSurrendered() )
	{
		return kVictory;
	}
	else if ( !fChildActor->mDiplomatRescued() )
	{
		if ( fChildActor->mShipsDestroyed() == 3 )
		{
			return kDefeat;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}
	else
	{
		return kAstoundingVictory;
	}
	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kFreighter:
		{
			playerTeam->mGotoCommState( ship->mGetShipID(), tFreighter::kBlankCommState );

			if ( fChildActor->mDiplomatRescued() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
		}
		break;

	case tEnemyTeam::kEnemyShip:
	case tEnemy2Team::kEnemy2Ship:
	case tEnemy3Team::kEnemy3Ship:
		{
			//
			//	Get rid of comm buttons that should not be there.
			//
			if ( ship->mGetCustomID() == tEnemyTeam::kEnemyShip )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemyShip::kBlankCommState );
			if ( ship->mGetCustomID() == tEnemy2Team::kEnemy2Ship )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemy2Ship::kBlankCommState );
			if ( ship->mGetCustomID() == tEnemy3Team::kEnemy3Ship )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemy3Ship::kBlankCommState );

			//
			//	Going to update the number of ships destroyed in each of ship classes.
			//
			tTeamInfo*		freighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
			Assert( freighterTeam );
			if ( !freighterTeam )
				return;
			tShipIterator	freighterIterator = freighterTeam->mGetFirstShip();
			if ( *freighterIterator == NULL )
				return;
			tFreighter*		freighter = static_cast< tFreighter* >( *freighterIterator );

			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );

			fChildActor->mIncreaseShipsDestroyed();
			if ( freighter )
				freighter->mIncreaseShipsDestroyed();

			if ( fChildActor->mShipsDestroyed() == 3 )
			{
				if ( fChildActor->mDiplomatRescued() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_2_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
				}
			}
		}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kFreighter:
		{}
		break;

	case tEnemyTeam::kEnemyShip:
	case tEnemy2Team::kEnemy2Ship:
	case tEnemy3Team::kEnemy3Ship:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent11_1_msg );

			//
			//	Get rid of comm buttons that should not be there.
			//
			if ( ship->mGetCustomID() == tEnemyTeam::kEnemyShip )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemyShip::kBlankCommState );
			if ( ship->mGetCustomID() == tEnemy2Team::kEnemy2Ship )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemy2Ship::kBlankCommState );
			if ( ship->mGetCustomID() == tEnemy3Team::kEnemy3Ship )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemy3Ship::kBlankCommState );

			//
			//	Going to update the number of ships destroyed in each of ship classes.
			//
			tTeamInfo*		freighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
			Assert( freighterTeam );
			if ( !freighterTeam )
				return;
			tShipIterator	freighterIterator = freighterTeam->mGetFirstShip();
			if ( *freighterIterator == NULL )
				return;
			tFreighter*		freighter = static_cast< tFreighter* >( *freighterIterator );

			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );

			fChildActor->mIncreaseShipsDestroyed();
			if ( freighter )
				freighter->mIncreaseShipsDestroyed();

			if ( fChildActor->mShipsDestroyed() == 3 )
			{
				if ( fChildActor->mDiplomatRescued() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_2_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
				}
			}			
		}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kFreighter:
		{
			playerTeam->mGotoCommState( ship->mGetShipID(), tFreighter::kBlankCommState );
		}
		break;

	case tEnemyTeam::kEnemyShip:
	case tEnemy2Team::kEnemy2Ship:
	case tEnemy3Team::kEnemy3Ship:
		{
			//
			//	Get rid of comm buttons that should not be there.
			//
			if ( ship->mGetCustomID() == tEnemyTeam::kEnemyShip )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemyShip::kBlankCommState );
			if ( ship->mGetCustomID() == tEnemy2Team::kEnemy2Ship )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemy2Ship::kBlankCommState );
			if ( ship->mGetCustomID() == tEnemy3Team::kEnemy3Ship )
				playerTeam->mGotoCommState( ship->mGetShipID(), tEnemy3Ship::kBlankCommState );

			//
			//	Going to update the number of ships destroyed in each of ship classes.
			//
			tTeamInfo*		freighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
			Assert( freighterTeam );
			if ( !freighterTeam )
				return;
			tShipIterator	freighterIterator = freighterTeam->mGetFirstShip();
			if ( *freighterIterator == NULL )
				return;
			tFreighter*		freighter = static_cast< tFreighter* >( *freighterIterator );

			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );

			fChildActor->mIncreaseShipsDestroyed();
			if ( freighter )
				freighter->mIncreaseShipsDestroyed();

			if ( fChildActor->mShipsDestroyed() == 3 )
			{
				if ( fChildActor->mDiplomatRescued() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_2_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
				}
			}
		}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent14_1_msg );		
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kFreighter:
		{}
		break;

	case tEnemyTeam::kEnemyShip:
		{}
		break;

	case tEnemy2Team::kEnemy2Ship:
		{}
		break;

	case tEnemy3Team::kEnemy3Ship:
		{}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{}
		break;
	}
}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	switch ( targetShip->mGetCustomID() )
	{
	case tPlayerNPCTeam::kFreighter:
		{}
		break;

	case tEnemyTeam::kEnemyShip:
	case tEnemy2Team::kEnemy2Ship:
	case tEnemy3Team::kEnemy3Ship:
		{
			switch ( buttonID )
			{
			//
			//	The player gives up, the mission ends in a victory, but the player loses a rank, loses
			//	lots of prestige, and moves down a ship class.
			//
			case k_WESURRENDER_PlayerForReal_btn:
				{
					mAddToBonusPrestige( -500 );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				break;

			//
			//	If the enemy is Federation, Lyran, or Hydran, the enemy will surrender.  Otherwise, the
			//	enemy will self-destruct themselves.
			//
			case k_YOUSURRENDER_EnemyForReal_btn:
				{
					eRaceName	enemyRace;
					tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
					tTeamInfo*	enemy2Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy2Team ) );
					tTeamInfo*	enemy3Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy3Team ) );
					Assert( enemyTeam && enemy2Team && enemy3Team );
					if ( ( !enemyTeam ) || ( !enemy2Team ) || ( !enemy3Team ) )
						return;

					if ( targetShip->mGetCustomID() == tEnemyTeam::kEnemyShip )
						enemyRace = enemyTeam->mGetRace();
					else if ( targetShip->mGetCustomID() == tEnemy2Team::kEnemy2Ship )
						enemyRace = enemy2Team->mGetRace();
					else if ( targetShip->mGetCustomID() == tEnemy3Team::kEnemy3Ship )
						enemyRace = enemy3Team->mGetRace();

					//
					//	Enemy ship will either surrender or self-destruct depending upon which race the
					//	enemy is.
					//
					switch ( enemyRace )
					{
					case kFederation:
					case kLyran:
					case kHydran:
						{
							//
							//	Going to update the number of ships destroyed in each of ship classes.
							//
							tTeamInfo*		freighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
							Assert( freighterTeam );
							if ( !freighterTeam )
								return;
							tShipIterator	freighterIterator = freighterTeam->mGetFirstShip();
							if ( *freighterIterator == NULL )
								return;
							tFreighter*		freighter = static_cast< tFreighter* >( *freighterIterator );

							//
							//	Increase in ships is taken care of when the ship finally disengages or
							//	the it is destroyed.
							//
						}
						break;

					default:		//	other races
						{
							targetShip->mDestroyShip();

							//
							//	No need to increase the number of ships destroyed here because it is taken
							//	care of in tState::mOnDeath(...).
							//
						}
						break;
					}

					//
					//	Check if the mission is complete.
					//
					if ( fChildActor->mShipsDestroyed() == 3 )
					{
						if ( fChildActor->mDiplomatRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), k_YOUSURRENDER_Enemy5_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), k_YOUSURRENDER_Enemy4_msg );
						}
					}
				}
				break;
			}
		}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{}
		break;
	}
}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;

	switch ( damagedShip->mGetCustomID() )
	{
	case tPlayerNPCTeam::kFreighter:
		{
			//
			//	Need to set that the player attacked the ally in the freighter's local class.
			//
			tTeamInfo*		freighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
			Assert( freighterTeam );
			if ( !freighterTeam )
				return;
			tShipIterator	freighterIterator = freighterTeam->mGetFirstShip();
			if ( *freighterIterator == NULL )
				return;
			tFreighter*		freighter = static_cast< tFreighter* >( *freighterIterator );

			if ( hostileShip )
			{
				if ( hostileShip->mGetTeam() == kPlayerTeam )
				{
					if ( freighter )
						freighter->mSetAttackedAlly( true );
				}
			}
		}
		break;

	case tEnemyTeam::kEnemyShip:
	case tEnemy2Team::kEnemy2Ship:
	case tEnemy3Team::kEnemy3Ship:
		{}
		break;

	case tNPC1Team::kPlanet:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{}
		break;
	}
}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if ( ( !shipTo ) || ( !shipFrom ) )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	//
	//	If the player is placing something on a ship.
	//
	switch ( shipFrom->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			switch ( shipTo->mGetCustomID() )
			{
			case tPlayerNPCTeam::kFreighter:
				{
					switch ( item )
					{
					case kTransDiplomat:
						{
							shipTo->mRemoveItemFromStores( kTransDiplomat, 1 );
							shipFrom->mAddItemToStores( kTransporterItem, kTransDiplomat, 1 );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
						}
						break;
					}
				}
				break;
			}
		}
		break;
	}

	//
	//	If the player is removing something from a ship.
	//
	switch ( shipTo->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			switch ( shipFrom->mGetCustomID() )
			{
			case tPlayerNPCTeam::kFreighter:
				{
					switch ( item )
					{
					case kTransDiplomat:
						{
							shipFrom->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 1.0f, kTrue );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );
							fChildActor->mSetDiplomatRescued( true );
							if ( fChildActor->mShipsDestroyed() < 3 )
							{
								fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_2_msg );
							}
							else
							{
								fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_3_msg );
								gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							}
						}
						break;
					}
				}
				break;
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );		break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );		break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );		break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );		break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );		break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );		break;
		}
		fChildActor->mSetTimedInterval( kTimerSecondsSet, timerID, ( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

