/********************************************************************************************************\

	File Name:	Enemy2Team.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Enemy2Ship.h"
#include "Enemy2Team.h"
#include "Enemy2TeamBaseState.h"
#include "ISC_8Diplomacy.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tEnemy2Team> gEnemy2TeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemy2Team::mSetupState()
{
	mRegisterState( new tEnemy2TeamBaseState() );
	mGotoState( typeid( tEnemy2TeamBaseState ) );
}

void tEnemy2Team::mSetupCommState()
{}


void tEnemy2Team::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemy2Team::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, kGorn );
		// CAs and NCAs
		sl = gST.stripOutSPECIAL( sl );
		sl = gST.stripOutRestricted( sl );

		std::list< eEnhancedRoles >  warRolesList;
        warRolesList.push_back(kLineShip);
        warRolesList.push_back(kWarShip);
        warRolesList.push_back(kLeader);
        warRolesList.push_back(kFastCruiser);
        fShiplistCount = sl.Subset(warRolesList);
        gST.mDebug_fprintf("subset for basic war classes eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassMediumMonster);
		for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		if (sl.size() > 0) {

			Ship s1;
			sl.SelectRandomShip(s1);
			gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
			gST.mTrackOrions( s1 );

			mCreateShip(
				typeid( tEnemy2Ship ), 
				s1.shipCode.c_str(), 
				kStartPosition_Y, 
				kNoMetaShipID,
				kEnemy2Ship, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemy2Team::mCreateShipsForTeam\n");
}

void tEnemy2Team::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemy2Team::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tEnemy2Ship ), "G-CA", kStartPosition_Y, kNoMetaShipID, kEnemy2Ship,
					0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemy2Team::mOldCreateShipsForTeam\n");
}


void tEnemy2Team::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemy2Team::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tEnemy2Team::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tEnemy2Team::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}
