/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyShip.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "ISC_8Diplomacy.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}


void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, kKlingon );
		sl = gST.stripOutRestricted( sl );

		std::list< eEnhancedRoles >  commandoRolesList;
        commandoRolesList.push_back(kCommando);
        fShiplistCount = sl.Subset(commandoRolesList);
        gST.mDebug_fprintf("subset for commando class eEnhancedRoles completed. ships: %d\n", fShiplistCount);

		// CAs and NCAs
		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassMediumMonster);
		for (int32 i = kEClassLightCruiser; i <= kEClassMediumCruiser; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		if (sl.size() > 0) {

			Ship s1;
			sl.SelectRandomShip(s1);
			gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
			gST.mTrackOrions( s1 );

			mCreateShip(
				typeid( tEnemyShip ), 
				s1.shipCode.c_str(), 
				kStartPosition_Z, 
				kNoMetaShipID,
				kEnemyShip, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tEnemyShip ), "K-D6G", kStartPosition_Z, kNoMetaShipID, kEnemyShip,
					0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}
