/********************************************************************************************************\

	Mission:	The Firm Hand Of Diplomacy
	Script:		"ISC_8Diplomacy"
	Descript:	Rescue a legendary diplomat.

	Created by:		Daniel J. Suleski		04/04/00
	Last Updated:	DJS						04/05/00
					First draft completed
					DJS						04/10/00
					Second draft completed
					DJS						04/11/00
					Fix multiple-player ship random chatter
					Got rid of references to time.h
					DJS						05/15/00
					Initial conversion from old API to new multiplayer API
					DJS						05/20/00
					Asserts added and code cleaned
					DJS						05/26/00
					tShipIterator::mGetFirstShipInPlay() replaced with original form
					DJS						08/23/00
					Map overrides.
					DJS						09/25/00
					mCreateShip() "default-removal" modifications, kCampaignMission, tTeamSpec, etc
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "ISC_8Diplomacy.h"
#include "BaseVictory.h"
#include "EnemyTeam.h"
#include "Enemy2Team.h"
#include "Enemy3Team.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "NPC1Team.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "VictoryCondition.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tISC_8Diplomacy );

tISC_8Diplomacy::tISC_8Diplomacy()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tISC_8Diplomacy::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team",
													static_cast< eTeamID >( kPlayerTeam ),
													kMaxChance,
													kTrue,
													kPrimaryTeam,
													kMinChance,
													kSpecISC,
													kAnyRace,
													typeid( tPlayerTeam ),
													kNoTeamTag,
													Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& playerNPCTeam = mCreateTeamSpec(	"Player NPC Team",
														static_cast< eTeamID >( kPlayerNPCTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tPlayerNPCTeam ) );

	const tTeamSpec& enemyTeam = mCreateTeamSpec(	"Enemy Team",  // Klingon
													static_cast< eTeamID >( kEnemyTeam ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecKling,
													kAnyEnemyOf | kPlayerTeam,
													typeid( tEnemyTeam ) );

	const tTeamSpec& enemy2Team = mCreateTeamSpec(	"Enemy 2 Team",  // Gorn
													static_cast< eTeamID >( kEnemy2Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecGorn,
													kAnyEnemyOf | kPlayerTeam,
													typeid( tEnemy2Team ) );

	const tTeamSpec& enemy3Team = mCreateTeamSpec(	"Enemy 3 Team",  //Hydran
													static_cast< eTeamID >( kEnemy3Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecHydran,
													kAnyEnemyOf | kPlayerTeam,
													typeid( tEnemy3Team ) );

	const tTeamSpec& npc1Team = mCreateTeamSpec(	"NPC 1 Team",
													static_cast< eTeamID >( kNPC1Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecPlayableRace,
													kAnyRace,
													typeid( tNPC1Team ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( playerNPCTeam );
	Specs->mSetupTeam( enemyTeam );
	Specs->mSetupTeam( enemy2Team );
	Specs->mSetupTeam( enemy3Team );
	Specs->mSetupTeam( npc1Team );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tISC_8Diplomacy::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tISC_8Diplomacy::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tISC_8Diplomacy::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tISC_8Diplomacy::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tISC_8Diplomacy::mStartMission( void )
{
	tScript::mStartMission();

/*

	HELPER NOTE:	The teamList array that you see below should reflect those constants that have been or you
					have declared in the matching header file (e.g. ISC_8Diplomacy.h).

*/
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kEnemy2Team,
		kEnemy3Team,
		kNPC1Team,
	};

/*

	HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
					meaning has a meaning???

	Relationship Lookup
	- = kTeamSelfAware	//	use this parameter with itself
	A = kTeamAllied		//	cannot fire upon an ally
	F = kTeamFriendly
	N = kTeamNeutral
	H = kTeamHostile	//	will not fire / capture / cripple until provoked
	W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
	P = kTeamPassive

	Relationship Lookup
	- = kGoalIgnore		//	use this parameter with itself
	I = kGoalIgnore
	C = kGoalCripple
	X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
	D = kGoalDestroy

*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = PlayersTeam,
		//PNPC = PlayerNPCTeam, -- freighter
		//Nme  = EnemyTeam,
		//Nme2 = Enemy2Team,
		//Nme3 = Enemy3Team,
		//NPC1 = NPC1 -- planet

		//	     Plyr  PNPC  Nme   Nme2  Nme3  NPC1
		/*Plyr*/ "--", "NI", "WD", "WD", "WD", "WD",
		/*PNPC*/ "NI", "--", "HD", "HD", "HD", "HD",
		/*Nme */ "WD", "HC", "--", "AI", "AI", "AI",
		/*Nme2*/ "WD", "HC", "AI", "--", "AI", "AI",
		/*Nme3*/ "WD", "HC", "AI", "AI", "--", "AI",
		/*NPC1*/ "WD", "WD", "AI", "AI", "AI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tISC_8Diplomacy::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_8Diplomacy::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['W'] = kObjectPlanet;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_8Diplomacy::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tISC_8Diplomacy::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}