/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Freighter.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "ISC_8Diplomacy.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}


void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( playerTeam );
		if ( !playerTeam )
			return;

        int32 fShiplistCount;
		ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
		sl = gST.stripOutRestricted( sl );
		sl = gST.mFilterShipListToSingleRace( sl, playerTeam->mGetRace() );

		std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(kEClassLargeFreighter));
		fShiplistCount = sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for large freighter filter completed. ships: %d\n", fShiplistCount);

        {
        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kCargo);
        fShiplistCount = sl.Subset(RolesList, true);
        gST.mDebug_fprintf("subset for cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);
        }

        {
        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kQship);
        fShiplistCount = sl.Subset(RolesList, false);
        gST.mDebug_fprintf("filter out qships completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);
        }

		if (sl.size() > 0) {
			Ship s1;
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			mCreateShip(
				typeid( tFreighter ),
				s1.shipCode.c_str(),
				kStartPosition_H,
				kNoMetaShipID,
				kFreighter,
				0, 0, -1, -1, NULL,
				kDefaultShipOptions );
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
	//
	//	Create the freighter that holds the diplomat.
	//
	mCreateShip(	typeid( tFreighter ), "I-FLPh", kStartPosition_H, kNoMetaShipID, kFreighter,
					0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}

void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}
