/********************************************************************************************************\

	File Name:	BaseVictory.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "ScriptInterface.h"
#include "ISC_9B1Homeworld.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "MissionText.h"

tClassAllocator<tBaseVictory> gBaseVictoryAllocator;

/*

	HELPER NOTE:	You must remember to initialize all of your victory attributes here.  You should
					do this for both constructors until informed otherwise.  For example:

	tTeam1BaseVictory::tTeam1BaseVictory( ) : fBSed( false ), field2( value ), field3 ( value ), etc...
	{}

*/
tBaseVictory::tBaseVictory():
	fCapitalTaken(false),
	fDestroyedAlly(0),
	fPlanetTransportsArrived(0),
	fMoonTransportsArrived(0)
	
{}
tBaseVictory::tBaseVictory( tTeamInfo* team ):
	fCapitalTaken(false),
	fDestroyedAlly(0),
	fPlanetTransportsArrived(0),
	fMoonTransportsArrived(0)
{}

/*

	HELPER NOTE:	Really, the only items in this function that you could change are the values passed into
					the tBaseVictoryState constructor.  These values are the capture bonus and prestige
					points awarded according to the player's victory status -- starting at
					kAstoundingVictory and working its way down to kDevastatingDefeat.

*/
void tBaseVictory::mSetupState()
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	tBaseVictoryState* baseVictoryState = new tBaseVictoryState( 300, 225, 75, 0, -75 );
	mRegisterState( baseVictoryState );
	mGotoState( typeid( tBaseVictoryState ) );

	mSetTimedInterval( kTimerSecondsSet, kRandChatID, 20.0f );
}

/*

	HELPER NOTE:	The following is simply an example of setting and modifying the current victory condition
					objectives.

	bool
	tTeam1BaseVictory::mBSed( void )
	{
		return fBSed;
	}
	void
	tTeam1BaseVictory::mSetBSed( bool x )
	{
		fBSed = x;
	}

*/
bool tBaseVictory::mCapitalTaken()
{
	return fCapitalTaken;
}
int32 tBaseVictory::mDestroyedAlly()
{
	return fDestroyedAlly;
}
int32 tBaseVictory::mPlanetTransportsArrived()
{
	return fPlanetTransportsArrived;
}
int32 tBaseVictory::mMoonTransportsArrived()
{
	return fMoonTransportsArrived;
}

void tBaseVictory::mSetCapitalTaken(bool x)
{
	fCapitalTaken = x;
	tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID)kEnemyTeam);
	Assert(enemy);
	if(enemy->mGetRace() == kKlingon || enemy->mGetRace() == kMirak)
	{
		if(enemy->mShipsRemainingInPlay() > 2)
		{
			enemy->mDisengageShips(kByMapEdge, kAbsolutePriority);
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "The remaining enemy ships are disengaging, [PLAYER_RANK].");
		}
	}
	else
	{
		if(enemy->mShipsRemainingInPlay() > 2)
		{
			enemy->mDisengageShips(kByMapEdge, kAbsolutePriority);
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "The remaining enemy ships are disengaging, [PLAYER_RANK].");

		}
	}
	gScriptInterface.mSetGameStatus((eTeamID) kPlayerTeam, kPlayerWins);
}
void tBaseVictory::mIncDestroyedAlly()
{
	fDestroyedAlly++;
}
void tBaseVictory::mIncPlanetTransportsArrived()
{
	fPlanetTransportsArrived++;
	tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID) kEnemyTeam);
	if(enemy->mGetRace() == kKlingon || enemy->mGetRace() == kMirak)
	{
		// if this is the Klingon or Mirak variant
		switch(mPlanetTransportsArrived())
		{
		case 1:
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportReachesPlanet_msg);
			break;
		case 2:
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kPlanetTaken_msg);
			if(mMoonTransportsArrived() == 1)
			{
				mSetCapitalTaken(true);
			}
			break;
		}								
	}
	else
	{
		switch(mPlanetTransportsArrived())
		{
		case 1:
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportReachesPlanet_msg);
			break;
		case 2:
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kPlanetTaken_msg);
			mSetCapitalTaken(true);
			break;									
		}
	}
}
void tBaseVictory::mIncMoonTransportsArrived()
{
	fMoonTransportsArrived++;
	if(mPlanetTransportsArrived() == 2)
	{
		fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kPlanetAndMoon_msg);
		mSetCapitalTaken(true);
	}
	else
	{
		fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kNeedPlanet_msg);
	}
			
}


