/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "BaseVictory.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "PlayerNPC1Team.h"
#include "TransportTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ISC_9B1Homeworld.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus(eTeamID ID)
{
	tISC_9B1Homeworld * script = (tISC_9B1Homeworld *) gScriptInterface.fScript;
	tPlayerTeam * team = (tPlayerTeam *) fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(team);

	if( script->fDiplomatBeamed)
	{
		if((script->fAllianceForged ) && (!team->mAttacker()))
		{
			return kAstoundingVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}

	}
	else
	{
		if(fChildActor->mCapitalTaken())
		{
			return kAstoundingVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}
	
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetTeam() )
	{
	case kPlayerTeam:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );
			}
		}
		break;
	case kPlayerNPC1Team:
		fChildActor->mIncDestroyedAlly();
		if(fChildActor->mDestroyedAlly() == 2)
			fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, "Science Officer: All allied ships are destroyed. We're alone.");
		break;

	
	case kTransportTeam:
	{
		switch(ship->mGetCustomID())
		{
		case tTransportTeam::kPlanetTransport1:
		case tTransportTeam::kPlanetTransport2:	
			if(fChildActor->mPlanetTransportsArrived() != 2)
			{
				fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kTransportDestroyed_msg);
			}
			break;
		case tTransportTeam::kMoonTransport:	
			if(fChildActor->mMoonTransportsArrived() != 1)
			{
				fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kTransportDestroyed_msg);
			}
			break;
		}
	}
	case kEnemyTeam:

		if(ship->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "First Officer: The base has been destroyed.\n\nCrew: Cheers!\n\n");

		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		Assert( team );
		if ( team )
		{
			tShipIterator remaining = team->mGetFirstShip();

			for(remaining; *remaining!= NULL; remaining++)
			{
				if((*remaining)->mGetCustomID() >= tEnemyTeam::kEnemyStarBase && (*remaining)->mGetCustomID() <= tEnemyTeam::kEnemyShip5 )
				{
					// there's an actual ship remaining. Stay in the game
					break;
				}
			}


			if (!team->mShipsRemainingInPlay())
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
		}
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( (eTeamID)kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerCaptured );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDisengaged );
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	switch(ship->mGetCustomID())
	{
	case tTransportTeam::kMoonTransport:
		if(ship->mInPlay())
		{
			fChildActor->mIncMoonTransportsArrived();
		}
		break;
	
	case tTransportTeam::kPlanetTransport1:
	case tTransportTeam::kPlanetTransport2:
		if(ship->mInPlay())
		{
			fChildActor->mIncPlanetTransportsArrived();
		}
		break;
	}
}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	tISC_9B1Homeworld * script = (tISC_9B1Homeworld *) gScriptInterface.fScript;
	tPlayerTeam * PlayerTeam = (tPlayerTeam *) fMissionInfo->mGetTeamHandle((eTeamID)kPlayerTeam);

	if((shipTo->mGetCustomID()== tEnemyTeam::kEnemyPlanet) && (item == kTransDiplomat))
	{
//		shipTo->mAddItemToStores(kTransporterItem, kTransDiplomat, 1);
//		shipFrom->mRemoveItemFromStores(kTransDiplomat, 1);
		

		if(!PlayerTeam->mAttacker())
		{
			script->fAllianceForged = true;
			fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam,
				"First Officer: The diplomats have made it to the planet's surface. Primary objective complete. Let's hope they're as good as command says they are.");
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
		}
		else
		{
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam,
				"First Officer: The diplomats have made it to the planet's surface. Unfortunately, we've received word that our attack against this system has undermined their effort. The mission is a failure.");
					gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );

		}
		
	}
	
	shipTo->mSetShipCallbackEvents( kNotifyTransportedItem );
	shipFrom->mSetShipCallbackEvents( kNotifyTransportedItem );

}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	/*
	int32 X, Y;
	char msg[10];
	if(timerID == tPlayerTeam::kEventTimerID)
	{
		tPlayerTeam * player = (tPlayerTeam *) fMissionInfo->mGetTeamHandle((eTeamID)kPlayerTeam);
		tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID)kEnemyTeam);
		tTeamInfo * transports = fMissionInfo->mGetTeamHandle((eTeamID)kTransportTeam);
		Assert(player);
		Assert(enemy);
		Assert(transports);
	//	return;
		if(player != NULL && enemy != NULL && transports != NULL)
		{
			if(player->mTransportTimer() >= 150)
			{	

				Log("In BaseVictoryState checking transports");
				tShipIterator transport = transports->mGetFirstShip();
				tShipIterator planet = enemy->mGetFirstShip();
				Assert(*planet!=NULL);


				// okay, now we have the planet and the first transport.
				// cycle through all transports, and check them
				for(transport; *transport != NULL; transport++)
				{
					// Check the moon transport first
					if(transport->mGetCustomID() == tTransportTeam::kMoonTransport)
					{
						fMissionInfo->mDebugMessage("Checking Moon transport");
						for(planet; *planet!=NULL; planet++)
						{
							if((*planet)->mGetCustomID() == tEnemyTeam::kEnemyMoon)
							{
								if((*transport)->mGetTacticalDistance(*planet) <= 15)
								{
								}
							}
						}
					}
					else
					{
						Log("Checking Planet Transports");
						planet = enemy->mGetFirstShip();
						for(planet; *planet!=NULL;planet++)
						{
							if((*planet)->mGetCustomID()==tEnemyTeam::kEnemyPlanet)
								break;
						}
						(*planet)->mGetGridPosition(&X, &Y);
						
						sprintf(msg, "Distance: %0.0f", transport->mGetTacticalDistance(X,Y));

						fMissionInfo->mDebugMessage(msg);
						return;
						if(transport->mGetTacticalDistance(*planet) <= 15 )
						{
							// is it still in one piece?
							if( (*transport)->mInPlay() )
							{
								// increment counter
								fChildActor->mIncPlanetTransportsArrived();
								if(enemy->mGetRace() == kKlingon || enemy->mGetRace() == kMirak)
								{
									// if this is the Klingon or Mirak variant
									switch(fChildActor->mPlanetTransportsArrived())
									{
									case 1:
										fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportReachesPlanet_msg);
										break;
									case 2:
										fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kPlanetTaken_msg);
										if(fChildActor->mMoonTransportsArrived() == 1)
										{
											fChildActor->mSetCapitalTaken(true);
										}
										break;
									}								
								}
								else
								{
									switch(fChildActor->mPlanetTransportsArrived())
									{
									case 1:
										
										fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportReachesPlanet_msg);
										break;
									case 2:
										fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kPlanetTaken_msg);
										fChildActor->mSetCapitalTaken(true);
										break;									
									}
								}
							}
						}
					}
				}
			}
		}
	}
	*/
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
