/********************************************************************************************************\

	Mission:	"Homeworld Bound"
	Script:		"ISC_9B1Homeworld"
	Descript:	Diplomatic variation. To get this mission, the player must have 
				beamed aboard the diplomat in ISC_8B, and must be in the Federation,
				Hydran, or Romulan capital sector.

	Created by:		Dutch Blomenkamp	6/28/00
	Last Updated:	NDB		07/16/00
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "PlayerNPC1Team.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerTeam.h"
#include "TransportTeam.h"
#include "ISC_9B1Homeworld.h"
#include "BaseVictory.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tISC_9B1Homeworld );

tISC_9B1Homeworld::tISC_9B1Homeworld()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tISC_9B1Homeworld::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;
	
	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												(eTeamID)kPlayerTeam,
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );
	
	
	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												(eTeamID)kEnemyTeam,
												kMaxChance,
												kTrue,
												kPlayableTeam, // Draftable.
												kMaxChance,
												kSpecRomulan,//PlayableRace & ~kSpecISC,
												kAnyRace,//kAnyEnemyOf | kPlayerTeam,
												typeid( tEnemyTeam ) );


	const tTeamSpec& team3= mCreateTeamSpec(	"Allied Team",
												(eTeamID)kPlayerNPC1Team,
												kMaxChance,
												kTrue,
												kPlayableTeam, 
												kMaxChance,
												kSpecPlayableRace & ~kSpecRomulan,
												kAnyRace,
												typeid( tPlayerNPC1Team ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Transport Team",
												(eTeamID)kTransportTeam,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kSameAs | kPlayerTeam,
												typeid( tTransportTeam ) );

    const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                    static_cast< eTeamID >( kDraft1Team ),
                                                    kMaxChance,
                                                    kTrue,
                                                    kPlayableTeam,
                                                    kMaxChance,
                                                    kSpecPlayableRace,
                                                    kAnyRace,
                                                    typeid( tDraftTeam ) );

    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                    static_cast< eTeamID >( kDraft2Team ),
                                                    kMaxChance,
                                                    kTrue,
                                                    kPlayableTeam,
                                                    kMaxChance,
                                                    kSpecPlayableRace,
                                                    kAnyRace,
                                                    typeid( tDraftTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
    Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions. 
	In multiplayer missions, you could examine which map was picked by the player.
	For example: if ( info.fPreferedMap == 3 );	//	the 3rd map was picked

	Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tISC_9B1Homeworld::mInitializeStart( tDynaverseScriptInfo& info )
{
	//******************************************************
	// This seems like the logical place to setup variables
	// Note that most of this will be changed to use 
	// whatever method the campaign uses.
	//******************************************************
	fDiplomatBeamed = true;		// true for diplomatic variant,
								// false for hostile variant.

	// See MissionText.CPP for the strings involved in setting up the
	// INI entries. Reading in from the ini file allows testing of 
	// various races. Once tested, this variable can be set via
	// the info structure passed into this functionThe diplomatic variant must
	// have a value of 0, 2, or 4 (kFederation, kRomulan, or kHydran

	fAllianceFed = false;		// These three are mutually exclusive.
	fAllianceHydran = false;		// Whichever one is true determines what
	fAllianceRomulan = false;	// race the ally will be.
	fSectorRace = kRomulan;

	fSectorRace = kRomulan;//fMissionInfo->mGetINIFileInt(kIniHeader, kSectorRace);
	
	fAllianceForged = false;		// Set to true for hostile with allies
								// false for no allies.
	fCodeKnown = true;			// True if the player has obtained the code before

	fShieldsLowered = false;	
	info.fEnemyRace = ( eRace ) fSectorRace; 
	return info.fPreferedMap;
	
	
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tISC_9B1Homeworld::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tISC_9B1Homeworld::mInitializeFinished( void )
{
	if(fDiplomatBeamed )
	{
		return kBriefingA_msg;
	}
	else
	{
		if(fAllianceForged)
		{
			if(fSectorRace != kKlingon && fSectorRace != kMirak)
			{
				return kBriefingB_msg;
			}
			else
			{
				return kBriefingC_msg;
			}
		}
		else
		{
			if(fSectorRace != kKlingon && fSectorRace != kMirak)
			{
				return kBriefingD_msg;
			}
			else
			{
				return kBriefingE_msg;
			}
		}

	}
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tISC_9B1Homeworld::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tISC_9B1Homeworld::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kPlayerNPC1Team,
		kTransportTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	if( fDiplomatBeamed == true)
	{
		std::string relationships[kRelationsCount][kRelationsCount] = 
		{
			//Plyr = kPlayerTeam,
			//PNPC = kPlayerNPCTeam,
			//Nme  = kEnemyTeam,
			//NPC1 = kNPC1Team,

			//	     Plyr  Nme
			/*Plyr*/ "--", "NI", "NI", "NI",
			/*Nme */ "NI", "--", "NI", "NI",
			/*Ally*/ "NI", "NI", "--", "NI",
			/*TPrt*/ "NI", "NI", "NI", "--"
		};
		fMissionInfo->mSetTeamRelations( teamList, (std::string*) relationships, kRelationsCount);


	}
	else
	{
		std::string relationships[kRelationsCount][kRelationsCount] = 
		{
			//Plyr = kPlayerTeam,
			//PNPC = kPlayerNPCTeam,
			//Nme  = kEnemyTeam,
			//NPC1 = kNPC1Team,

			//	     Plyr  Nme
			/*Plyr*/ "--", "WD", "NI", "NI",
			/*Nme */ "WD", "--", "WD", "WD",
			/*Ally*/ "NI", "WD", "--", "NI",
			/*TPrt*/ "NI", "WD", "NI", "--"
		};
		fMissionInfo->mSetTeamRelations( teamList, (std::string*) relationships, kRelationsCount);

	}
	
}

char * tISC_9B1Homeworld::GetDynamicMessage( char * buffer, int32 msg, char * fstr, char * rstr)
{
	// Find the "token", and replace it.
	int p = 0;
	p = Messages[msg].find(fstr, p);
	Messages[msg]=Messages[msg].erase(p, strlen(fstr));
	Messages[msg].insert(p, rstr);
	
	// Now stuff the replaced string into buffer.

	if(strlen(buffer) >= Messages[msg].length())
	{
		sprintf(buffer, "%s", Messages[msg].c_str());
	}

	// Now put the message back the way it was for 
	// future use.

	p = Messages[msg].find(rstr, p);
	Messages[msg]=Messages[msg].erase(p, strlen(rstr));
	Messages[msg].insert(p, fstr);
	return buffer;
	
};

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_9B1Homeworld::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['K'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['O'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['P'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Q'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['R'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['S'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['X'] = kObjectEnemyBase;
	Map->fTacticalMapIcon['Y'] = kObjectPlanet;
	Map->fTacticalMapIcon['Z'] = kObjectPlanet;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_9B1Homeworld::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefingA_msg;
}


/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tISC_9B1Homeworld::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}