/********************************************************************************************************\

	File Name:	EnemyPlanet.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyPlanet.h"
#include "EnemyPlanetBaseState.h"
#include "ISC_9B2Homeworld.h"
#include "MissionText.h"
tClassAllocator<tEnemyPlanet> gEnemyPlanetAllocator;

/*

	HELPER NOTE:	If you create any new states on the ship level, you must register them here first.

*/
void tEnemyPlanet::mSetupState()
{
	mRegisterState( new tEnemyPlanetBaseState() );
	mGotoState( typeid( tEnemyPlanetBaseState ) );
}

/*

	HELPER NOTE:	Here is an example of how you could use this function.  Remember that those ships
					that are implicitly created should be done on the team level; those ships that you
					explicitly create could be done here and probably should.  Here is an example:

	void
	tEnemyPlanet::mSetupCommState( void )
	{
		mRegisterCommMessage( kBSCommState1 , kNoStateChange , mGetShipID() , k_BS_1_btn , k_BS_1_msg , kNoRandomMsg );
		mRegisterCommMessage( kBSCommState1 , kBlankCommState , mGetShipID() , k_BS_2_btn , k_BS_2_msg , kNoRandomMsg );
		mRegisterCommMessage( kBSCommState1 , kBSCommState2 , mGetShipID() , k_BS_3_btn , k_BS_3_msg , k_BS_7_msg );

		mRegisterCommMessage( kBSCommState2 , kNoStateChange , mGetShipID() , k_BS_4_btn , k_BS_8_msg , kNoRandomMsg );

		mGotoCommState( kBSCommState1 );
	}
*/
void tEnemyPlanet::mSetupCommState()
{
	tISC_9B2Homeworld * script = (tISC_9B2Homeworld * ) gScriptInterface.fScript;

	tTeamInfo * PlayerTeam = fMissionInfo->mGetTeamHandle((eTeamID)kPlayerTeam);
	Assert(PlayerTeam);

	if(script->fDiplomatBeamed)
	{
		PlayerTeam->mRegisterCommMessage( tISC_9B2Homeworld::kCommState1, kNoStateChange , mGetShipID() , kHail_btn , kCommDummy_msg, kNoRandomMsg );
		PlayerTeam->mRegisterCommMessage( tISC_9B2Homeworld::kCommState2, kNoStateChange , mGetShipID() , kHail_btn , kCommDummy_msg, kNoRandomMsg );
		
	}
	else
	{
		PlayerTeam->mRegisterCommMessage( tISC_9B2Homeworld::kCommState1, kNoStateChange , mGetShipID() , kHail_btn , 	kHailEnemyCapitalHostile_msg, kHailEnemyRandom3_msg );
		PlayerTeam->mRegisterCommMessage( tISC_9B2Homeworld::kCommState2, kNoStateChange , mGetShipID() , kHail_btn , kHailEnemyCapitalTaken_msg, kNoRandomMsg );
	}

	PlayerTeam->mGotoCommState(mGetShipID(), tISC_9B2Homeworld::kCommState1);

/*		

*/


}

/*

	HELPER NOTE:	This function is called once immediately after the ship has been created (perhaps with a
					call to mCreateShip()).  Anyway, in the body of this function, you can setup events that
					you wish to receive that normally are not on by default, change maximum ship speed, change
					the name of the ship, etc.  Below is a list of those events that need to be setup and
					those that do not.

Event							How to setup													Automatic

mAtDestination					tShipInfo::mSetFinalDestination									No
mAtDistance						tShipInfo::mTrackTacticalDistance								No
mOnAlertStatusChange			tShipInfo::mSetShipCallbackEvents( kNotifyAlertStatusChange )	No
mOnBeginOrbit					tShipInfo::mSetShipCallbackEvents( kNotifyBeginOrbit )			No
mOnBeginTractor					tShipInfo::mSetShipCallbackEvents( kNotifyBeginTractor )		No
mOnCapture						On by default													Yes
mOnChangingShip					On by default													Yes
mOnCommButtonPress				Setup a Comm Button State										Yes
mOnCommStateChange				Always happens when a comm state changes						Yes
mOnDamage						tShipInfo::mSetShipCallbackEvents( kNotifyDamage )				No
mOnDeath						On by default													Yes
mOnDeepScan						tShipInfo::mSetShipCallbackEvents( kNotifyDeepScan )			No
mOnDisengaged					On by default													Yes
mOnEndOrbit						tShipInfo::mSetShipCallbackEvents( kNotifyEndOrbit )			No
mOnEndTractor					tShipInfo::mTractorEntity										No
mOnFlashingButtonPressed		tMissionInfo::mStartFlashingButton								No
mOnHostChanged					On by default													Yes
mOnPlayerLeaving				On by default													Yes
mOnProbeLaunch					tShipInfo::mSetShipCallbackEvents( kNotifyProbeLaunch )			No
mOnSpecialTransport				tShipInfo::mSetShipCallbackEvents( kNotifySpecialTransport )	No
mOnStateChange					Always happens when a state changes								Yes
mOnTimedInterval				tEventMgr::mSetTimedInterval									No
mOnTransportedItem				tShipInfo::mSetShipCallbackEvents( kNotifyTransportedItem )		No


	//
	//	Changes the name of the ship, planet, object, etc.  The constant passed into this function should
	//	be declared in MissionText.h and the text for the object should correspond in MissionText.cpp.
	//
	mChangeName( kShipBS_nam );

	//
	//	Sets the maximum speed of the object if you so wish to limit it.
	//
	mSetShipMaxSpeed( 5 );

	//
	//	The following will set up events that normally are not on by default as described above.
	//
	mTrackTacticalDistance( 20.0, 5, 5 );

*/
void tEnemyPlanet::mSetupShip()
{
	mSetShipCallbackEvents( kNotifyTransportedItem );
	mSetShipCallbackEvents( kNotifyDamage );
}
