/********************************************************************************************************\

	File Name:	EnemyTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyTeamBaseState.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "MissionText.h"
#include "ISC_9B2Homeworld.h"
#include "ShipList_API.h"

#if !defined (kTimerConstant)
#define kTimerConstant 2 / 60
#endif

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tEnemyTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tEnemyTeamBaseState::mEndState()
{}

void tEnemyTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	tPlayerTeam * playerteam = (tPlayerTeam *) fMissionInfo->mGetTeamHandle((eTeamID)kPlayerTeam);
	tISC_9B2Homeworld * script = (tISC_9B2Homeworld *) gScriptInterface.fScript;

	if(targetShip->mGetTeam() == kEnemyTeam)
	{
		switch(buttonID)
		{
		case kHail_btn:
			if(targetShip->mGetCustomID() == tEnemyTeam::kEnemyPlanet)
			{
				tISC_9B2Homeworld * script = (tISC_9B2Homeworld *) gScriptInterface.fScript;
				if(!script->fDiplomatBeamed)
					return;
				tPlayerTeam * PlayerTeam = (tPlayerTeam *) fMissionInfo->mGetTeamHandle((eTeamID)kPlayerTeam);
				Assert(PlayerTeam);
				if(!PlayerTeam->mAttacker())
				{
					tShipInfo * playership = *(PlayerTeam->mGetFirstShip());
					Assert(playership);
					if(targetShip->mGetTacticalDistance(playership) <= 10)
					{
						fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kHailEnemyCapitalLowerShields_msg);
						targetShip->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f );
					}
					else
					{
						fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kHailEnemyCapitalDiplomacy_msg);
					}
				}
				else
				{
					fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kHailEnemyCapitalDiplomacyAttacked_msg);
				}
		
			}
			break;

		case kSetAsTarget_btn:
			playerteam->fAITarget = targetShip;
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "Communications Officer: Fleet target set, and ready for assignment, [PLAYER_RANK]");
			break;
		case kDemandSurrender_btn:
			if(targetShip->mGetDamagePercent() <= 50.0)
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kDemandSurrenderGT50_msg );
				break;;
			}
			if(targetShip->mGetDamagePercent() <= 95.0)
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kDemandSurrenderGT5_msg );
				break;
			}
			fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kDemandSurrenderLT5_msg );
			break;
			
		case kLowerShields_btn:
			// check this, though it's really redundant, since
			// only the starbase get's that button
			
			if(targetShip->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
			{
				if(!script->fCodeKnown )
				{
					fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "Communications Officer: We'd need to know their prefix codes, [PLAYER_RANK].");
				}
				else
				{
					if(!script->fShieldsLowered)
					{
						targetShip->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f );
						fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "Communications Officer: Their shields are down, [PLAYER_RANK]\n\n");
						
				// note that using the turns setting with a constant *should* make the 
				// timer constant at all game speeds.
						
						fChildActor->mSetTimedInterval(kTimerTurnsSet, fChildActor->kShieldTimerID, 2 * kTimerConstant);
						script->fShieldsLowered = true;
						
					}
					else
					{
						fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "Communications Officer: They've changed the prefix code. No effect, [PLAYER_RANK].");
					}
				}
			}
		
			break;
		}
	}
}
void tEnemyTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tEnemyTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnFlashingButtonPressed()
{}
void tEnemyTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tEnemyTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tEnemyTeamBaseState::mOnHostChanged()
{}
void tEnemyTeamBaseState::mOnTimedInterval( int32 timerID )
{
	float fShield;
	switch (timerID) {
	case tEnemyTeam::kShieldTimerID:
		if(fChildActor->mShieldTimer() <= 10 )
		{
			fShield = 100.0f;
			
		}
		else
		{
			fShield = 0.0f;
		}
		fChildActor->mIncShieldTimer();
		tShipIterator starbase = fChildActor->mGetFirstShip();
		for(starbase; *starbase != NULL; starbase++)
		{
			if((*starbase)->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
				break;
		}
		Assert(*starbase != NULL);
		if(*starbase != NULL)
			starbase->mSetSystemHealthFromOrig( kAllShieldsHardPoint, fShield );

		if(fChildActor->mShieldTimer() <= 10)
			fChildActor->mSetTimedInterval(kTimerTurnsSet, timerID, (float)2 * kTimerConstant);
		
		break;
	}

    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        tTeamInfo* EnemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kEnemyTeam ) );
        if (EnemyTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        if (EnemyTeam->mIsTeamHumanControlled()) return;               // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tEnemyTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        int32 BPVinc = gST.mUpgradeFightersForTeam(EnemyTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tEnemyTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}
void tEnemyTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

