/********************************************************************************************************\

	File Name:	PlayerNPC1Team.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "PlayerNPC1Team.h"
#include "PlayerNPC1TeamBaseState.h"
#include "PlayerNPC1TeamAllyAttackedState.h"
#include "PlayerNPC1Ship.h"
#include "ShipScriptDescription.h"
#include "ISC_9B2Homeworld.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerNPC1Team> gPlayerNPC1TeamAllocator;

void tPlayerNPC1Team::mSetupState()
{
	mSetAllyAttacked(false);
	mRegisterState( new tPlayerNPC1TeamBaseState() );
	mRegisterState( new tPlayerNPC1TeamAllyAttackedState() );
	mGotoState( typeid( tPlayerNPC1TeamBaseState ) );
}

void tPlayerNPC1Team::mSetupCommState()
{}

void tPlayerNPC1Team::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Started: tPlayerNPC1Team::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		eRaceName race = mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.mFilterShipListToSingleRace( sl, race );

		// Narrow down a ship-only list for CMs, CAs, NCAs.
        ShipList ship_sl = gST.selectWarShips(sl);
		std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassMediumMonster);
        for (int32 i = kEClassMediumCruiser; i <= kEClassNewHeavyCruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(ship_sl);

		if (ship_sl.size() > 0) {
			tISC_9B2Homeworld * script = (tISC_9B2Homeworld * ) gScriptInterface.fScript;
			if(!script->fDiplomatBeamed)
			{
				if(script->fAllianceForged)
				{
					// first ship
					{
					Ship s1;
					ship_sl.SelectRandomShip(s1);
					gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
					mCreateShip( typeid( tPlayerNPC1Ship ),
						s1.shipCode.c_str(),
						kStartPosition_J,
						kNoMetaShipID,
						kPlayerNPC1Ship1,
						0, 0, -1, -1, 
						NULL,
						kDefaultShipOptions );
					}

					// second ship
					{
					Ship s1;
					ship_sl.SelectRandomShip(s1);
					gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
					mCreateShip( typeid( tPlayerNPC1Ship ),
						s1.shipCode.c_str(),
						kStartPosition_K,
						kNoMetaShipID,
						kPlayerNPC1Ship2,
						0, 0, -1, -1, 
						NULL,
						kDefaultShipOptions );
					}
				}
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tPlayerNPC1Team::mCreateShipsForTeam\n");
}

void tPlayerNPC1Team::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Started: tPlayerNPC1Team::mOldCreateShipsForTeam\n");

	tISC_9B2Homeworld * script = (tISC_9B2Homeworld * ) gScriptInterface.fScript;

	if(!script->fDiplomatBeamed)
	{
		if(script->fAllianceForged)
		{
			mCreateShip(	typeid( tPlayerNPC1Ship ),
						kMaxBPV,
						kClassHeavyCruiser,
						0,
						kStartPosition_J,
						kPlayerNPC1Ship1,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );
			mCreateShip(	typeid( tPlayerNPC1Ship ),
						kMaxBPV,
						kClassHeavyCruiser,
						0,
						kStartPosition_K,
						kPlayerNPC1Ship2,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );
		}
	}
	gST.mDebug_fprintf("---- Finished: tPlayerNPC1Team::mOldCreateShipsForTeam\n");
}


void tPlayerNPC1Team::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPC1Team::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPC1Team::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tPlayerNPC1Team::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

bool tPlayerNPC1Team::mAllyAttacked(void)
{
	return fAllyAttacked;
}
void tPlayerNPC1Team::mSetAllyAttacked(bool x)
{
	if(x)
	{
		mGotoState( typeid( tPlayerNPC1TeamAllyAttackedState ) );
	}
	fAllyAttacked = x;
}
