/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ISC_9B2Homeworld.h"
#include "ShipList_API.h"

#define kTimerConstant 2 / 60

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam( void ): fAITarget( NULL )
{}


void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
	fAttacker = false;
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	tISC_9B2Homeworld * script = (tISC_9B2Homeworld * ) gScriptInterface.fScript ;
	if(script->fDiplomatBeamed)
	{
		switch ( victoryLevel )
		{
		case kAstoundingVictory:
			switch(script->fSectorRace)
			{
			case  kFederation:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFedAlly_msg);
				break;
			case kHydran:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebHydranAlly_msg);
				break;
			case kRomulan:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebRomulanAlly_msg);
				break;
			}
			break;
		case kDevastatingDefeat:
			switch(script->fSectorRace)
			{
			case  kFederation:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFedAttack_msg);
				break;
			case kHydran:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebHydranAttack_msg);
				break;
			case kRomulan:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebRomulanAttack_msg);
				break;
			}
			break;
		}
	}
	else
	{
		switch ( victoryLevel )
		{
		case kAstoundingVictory:
			fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebHostileSuccess_msg);
			break;
		case kDevastatingDefeat:
			if(script->fSectorRace = kKlingon || script->fSectorRace==kMirak)
			{
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFailKorM_msg);
			}
			else
			{
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFailGeneric_msg);
			}
			break;
		}
	}

	if ( victoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( mGetTeam(), kDisconnect_msg );
	}
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
	tShipInfo * flagship;

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
			
		}
		flagship = *ship;

		tISC_9B2Homeworld * script = (tISC_9B2Homeworld * ) gScriptInterface.fScript ;
		if(script->fDiplomatBeamed)
		{
			flagship->mAddItemToStores(kTransporterItem,kTransDiplomat,1);
			flagship->mSetShipCallbackEvents( kNotifyTransportedItem );
		}

		flagship->mSetShipThrust(10);
		ship++;
		for(ship; *ship != NULL; ship++)
		{
			ship->mDefendEntity(flagship, 10.0, 4.0, kAbsolutePriority);
			ship->mSetShipThrust(10);

		}
	}
	mSetRandomChat(true);
	mSetTimedInterval( kTimerTurnsSet, kEventTimerID, (float) kTimerConstant * 5);
}


void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;
	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 
				 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}


bool tPlayerTeam::mRandomChat(void)
{
	return fRandomChat;
}
void tPlayerTeam::mSetRandomChat(bool x)
{
	fRandomChat = x;
}

bool tPlayerTeam::mAtEdgeOfMap(void)
{
	return fAtEdgeOfMap;
}

void tPlayerTeam::mSetAtEdgeOfMap(bool x)
{
	fAtEdgeOfMap = x;
}
int32 tPlayerTeam::mTransportTimer()
{
	return fTransportTimer;
}
void tPlayerTeam::mIncTransportTimer()
{
	fTransportTimer++;
}

void tPlayerTeam::mSetTransportTimer(int32 x)
{
	fTransportTimer = x;
}
bool tPlayerTeam::mAttacker()
{
	return fAttacker;
}

void tPlayerTeam::mSetAttacker(bool x)
{
	fAttacker = x;
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;
	fMissionScheduler.fNextMissionTitle = "ISC_10Criminals";

	switch( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
		fMissionScheduler.fMedal = kMedalSpecialFour;
	default:
		break;
	}
}