/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyPlanet.h"
#include "EnemyStarBase.h"
#include "EnemyShip.h"
#include "EnemyDefPlat.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "ISC_9B4Homeworld.h"
#include "ShipList_API.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;


void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}


void tEnemyTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		eRaceName race = mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.stripOutRestricted( gST.narrowed_sl );
        sl = gST.mFilterShipListToSingleRace( sl, race );

		// Narrow down a ship-only list for BBs, DNs and BCHs.
        ShipList ship_sl = gST.selectWarShips(sl);
		ship_sl = gST.stripOutSPECIAL( ship_sl );
		std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassHeavyBattlecruiser; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);


		// BBs
		ShipList BB_sl = ship_sl;
		std::list< eEnhancedClassTypes >  BBClassTypeList;
        BBClassTypeList.push_back(kEClassBattleship);
		fShiplistCount = BB_sl.Subset(BBClassTypeList, true);
		gST.mDebug_fprintf("subset for BB filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(BB_sl);

		// DNs
		ShipList DN_sl = ship_sl;
		std::list< eEnhancedClassTypes >  DNClassTypeList;
        DNClassTypeList.push_back(kEClassDreadnought);
		fShiplistCount = DN_sl.Subset(DNClassTypeList, true);
		gST.mDebug_fprintf("subset for DN filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(DN_sl);

		// BCHs
		ShipList BCH_sl = ship_sl;
		std::list< eEnhancedClassTypes >  BCHClassTypeList;
        BCHClassTypeList.push_back(kEClassHeavyBattlecruiser);
		fShiplistCount = BCH_sl.Subset(BCHClassTypeList, true);
		gST.mDebug_fprintf("subset for BCH filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(BCH_sl);

		// SBs
		ShipList SB_sl = gST.stripOutSPECIAL( sl );
        std::list< eEnhancedClassTypes >  SBClassTypeList;
        SBClassTypeList.push_back(kEClassStarBase);
        fShiplistCount = SB_sl.Subset(SBClassTypeList, true);
        gST.mDebug_fprintf("subset for SB filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(SB_sl);

        // Defense platforms. reload frmo narrowed_sl because DEF platforms are SPECIAL
		ShipList DEF_sl = gST.narrowed_sl;
		DEF_sl = gST.stripOutRestricted( DEF_sl );
        DEF_sl = gST.mFilterShipListToSingleRace( DEF_sl, race );
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. platforms: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. platforms: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);


		// all planets
		ShipList planet_sl = gST.stripOutSPECIAL( sl );
        std::list< eEnhancedClassTypes >  PlanetClassTypeList;
        PlanetClassTypeList.push_back(kEClassPlanet);
        fShiplistCount = planet_sl.Subset(PlanetClassTypeList, true);
        gST.mDebug_fprintf("subset for planet class filter completed. planets:: %d\n", fShiplistCount);

		// homeworlds
		ShipList HW_sl = planet_sl;
        std::list< eEnhancedRoles >  HWRolesList;
        HWRolesList.push_back(kHomeWorld);
        fShiplistCount = HW_sl.Subset(HWRolesList, true);
        gST.mDebug_fprintf("subset for HomeWorld eEnhancedRoles completed. planets: %d\n", fShiplistCount);
        gST.listShiplistContent(HW_sl);

		// Armed Moon
		std::string moonType = "PLPh0";
		switch (race) {
			case kFederation: moonType = "F-PLPh0"; break;
			case kKlingon: moonType = "K-PLPh0"; break;
			case kMirak: moonType = "Z-PLPh0"; break;
			default: break;
		}

		// And now to decide if we want to use shiplist API results or not
		Ship p1;
        if ( 
			(
				// armed moon should only be found for 3 races.
				(
					(planet_sl.Find(moonType, p1)) && 
					( (race == kFederation) || (race == kKlingon) || (race == kMirak) )
				) ||
				( (race != kFederation) && (race != kKlingon) && (race != kMirak) )
			) &&
			(DN_sl.size() > 0) &&
			(HW_sl.size() > 0) &&
			(SB_sl.size() > 0)
		) {

			// create the Homeworld.
			{
				Ship s1;
				HW_sl.SelectRandomShip(s1);
				gST.mTrackOrions( s1 );
				gST.mDebug_fprintf("planet %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
				mCreateShip( typeid( tEnemyPlanet ),
					s1.shipCode.c_str(),
					kStartPosition_Z,
					kNoMetaShipID,
					kEnemyPlanet,
					0, 0, -1, -1, 
					NULL,
					kDefaultShipOptions );
			}

			// create the moon, based on race
			switch (race) {
				case kFederation:
				case kKlingon:
				case kMirak:
					gST.mDebug_fprintf("race: %d  -- creating armed moon: %s.\n", race, p1.shipCode.c_str());
					mCreateShip( typeid( tEnemyPlanet ),
						p1.shipCode.c_str(),
						kStartPosition_Y,
						kNoMetaShipID,
						kEnemyMoon,
						0, 0, -1, -1, 
						NULL,
						kDefaultShipOptions );
					break;
				default: break;
			}

			// create the SB.
			{
				Ship s1;
				SB_sl.SelectRandomShip(s1);
				gST.mTrackOrions( s1 );
				gST.mDebug_fprintf("SB %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
				mCreateShip( typeid( tEnemyStarBase ),
					s1.shipCode.c_str(),
					kStartPosition_X,
					kNoMetaShipID,
					kEnemyStarBase,
					0, 0, -1, -1, 
					NULL,
					kDefaultShipOptions );
			}

			int32 shipSetup = fMissionInfo->mRandomInt32( 0, 2 );
			gST.mDebug_fprintf("ship setup: rolled a %d\n", shipSetup);

			// 0 or 1: first ship is a BB. 2: it's a DN.
			if (
				(
					(shipSetup == 0) || 
					(shipSetup == 1)
				) && 
				(BB_sl.size() > 0)
			) {
				// pick a BB
				Ship s1;
				BB_sl.SelectRandomShip(s1);
				gST.mTrackOrions( s1 );
				gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
				mCreateShip( typeid( tEnemyShip ),
					s1.shipCode.c_str(),
					kStartPosition_W,
					kNoMetaShipID,
					kEnemyShip1,
					0, 0, -1, -1, 
					NULL,
					kDefaultShipOptions );
			} else {
				// pick a DN
				Ship s1;
				DN_sl.SelectRandomShip(s1);
				gST.mTrackOrions( s1 );
				gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
				mCreateShip( typeid( tEnemyShip ),
					s1.shipCode.c_str(),
					kStartPosition_W,
					kNoMetaShipID,
					kEnemyShip1,
					0, 0, -1, -1, 
					NULL,
					kDefaultShipOptions );
			}

			// 0: 2 ships are DNs.
			// or other, but there aren't any BCHs available.
			if (
				(shipSetup == 0) || 
				((shipSetup != 0) && (BCH_sl.size() == 0))
			) {
				// pick a DN
				{
					Ship s1;
					DN_sl.SelectRandomShip(s1);
					gST.mTrackOrions( s1 );
					gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
					mCreateShip( typeid( tEnemyShip ),
						s1.shipCode.c_str(),
						kStartPosition_T,
						kNoMetaShipID,
						kEnemyShip2,
						0, 0, -1, -1, 
						NULL,
						kDefaultShipOptions );
				}

				// pick another DN
				{
					Ship s1;
					DN_sl.SelectRandomShip(s1);
					gST.mTrackOrions( s1 );
					gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
					mCreateShip( typeid( tEnemyShip ),
						s1.shipCode.c_str(),
						kStartPosition_S,
						kNoMetaShipID,
						kEnemyShip3,
						0, 0, -1, -1, 
						NULL,
						kDefaultShipOptions );
				}
			}

			if (shipSetup != 0) {
				if (BCH_sl.size() > 0) {
					// pick a BCH
					{
						Ship s1;
						BCH_sl.SelectRandomShip(s1);
						gST.mTrackOrions( s1 );
						gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
							s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
						mCreateShip( typeid( tEnemyShip ),
							s1.shipCode.c_str(),
							kStartPosition_S,
							kNoMetaShipID,
							kEnemyShip2,
							0, 0, -1, -1, 
							NULL,
							kDefaultShipOptions );
					}

					// pick a 2nd BCH
					{
						Ship s1;
						BCH_sl.SelectRandomShip(s1);
						gST.mTrackOrions( s1 );
						gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
							s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
						mCreateShip( typeid( tEnemyShip ),
							s1.shipCode.c_str(),
							kStartPosition_T,
							kNoMetaShipID,
							kEnemyShip3,
							0, 0, -1, -1, 
							NULL,
							kDefaultShipOptions );
					}

					// pick a 3rd BCH
					{
						Ship s1;
						BCH_sl.SelectRandomShip(s1);
						gST.mTrackOrions( s1 );
						gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
							s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
						mCreateShip( typeid( tEnemyShip ),
							s1.shipCode.c_str(),
							kStartPosition_W,
							kNoMetaShipID,
							kEnemyShip4,
							0, 0, -1, -1, 
							NULL,
							kDefaultShipOptions );
					}

					if (shipSetup == 2) {
						// pick a 4th BCH
						Ship s1;
						BCH_sl.SelectRandomShip(s1);
						gST.mTrackOrions( s1 );
						gST.mDebug_fprintf("ship %s selected. BPV: %d  YFA: %d  YLA %d\n",
							s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
						mCreateShip( typeid( tEnemyShip ),
							s1.shipCode.c_str(),
							kStartPosition_S,
							kNoMetaShipID,
							kEnemyShip5,
							0, 0, -1, -1, 
							NULL,
							kDefaultShipOptions );

					}
				} // else case already handled: 2 DNs.
			}

			// if difficulty is admiral, create the Defense Platforms
			if ((fMissionInfo->mGetDifficultyLevel() == 2) && (DEF_sl.size() > 0)) {
				int32 numDEF = fMissionInfo->mRandomInt32( 2, 7 );
				gST.mDebug_fprintf("We will have %d defense platforms\n", numDEF+1);

				// create the Defense platforms, if they are available. Admiral only.
				for(i = 0; i <= numDEF; i++) {
					Ship s1;
					DEF_sl.SelectRandomShip(s1);
					gST.mTrackOrions( s1 );
					gST.mDebug_fprintf("#%d: DEF platform %s selected. BPV: %d  YFA: %d  YLA %d\n",
						i+1, s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				
					mCreateShip( typeid( tEnemyDefPlat ),
						s1.shipCode.c_str(),
						kStartPosition_P+i, 
						kNoMetaShipID,
						kEnemyDefPlat1 + i, 
						0, 0, -1, -1, 
						NULL, 
						kDefaultShipOptions );
				}
			}

		} else {
			mOldCreateShipsForTeam();
		}
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");
	
//	tISC_9B4Homeworld * script = (tISC_9B4Homeworld * ) gScriptInterface.fScript;
//	fRace = (eRaceName) script->fSectorRace;
	int32 i;

	switch( mGetRace() )
	{
	case kFederation:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kNoMetaShipID,
					kEnemyPlanet,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh0",
					kStartPosition_Y,
					kNoMetaShipID,
					kEnemyMoon,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );


		mCreateShip( typeid( tEnemyStarBase ),
					kMaxBPV,
					kClassStarBase,
					kClassStarBase,
					0,
					kStartPosition_X,
					kEnemyStarBase,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_W,
					kEnemyShip1,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_T,
					kEnemyShip2,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_S,
					kEnemyShip3,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
		break;

	case kRomulan:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kNoMetaShipID,
					kEnemyPlanet,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyStarBase ),
					kMaxBPV,
					kClassStarBase,
					kClassStarBase,
					0,
					kStartPosition_X,
					kEnemyStarBase,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_W,
					kEnemyShip1,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_S,
					kEnemyShip2,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_T,
					kEnemyShip3,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
		break;
	
	case kHydran:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh4",
					kStartPosition_Z,
					kNoMetaShipID,
					kEnemyPlanet,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_X,
					kEnemyShip1,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_W,
					kEnemyShip2,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_S,
					kEnemyShip3,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_T,
					kEnemyShip4,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_T,
					kEnemyShip5,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		for(i = 0; i <= 2; i++)
		{
			mCreateShip( typeid( tEnemyDefPlat ),
				Messages[kDefPlatFed + mGetRace()].c_str(), kStartPosition_P + i, kNoMetaShipID,
				kEnemyDefPlat1 + i, 0, 0, -1, -1, NULL, kDefaultShipOptions );

		}
		break;
	case kLyran:
	case kGorn:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kNoMetaShipID,
					kEnemyPlanet,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );


		mCreateShip( typeid( tEnemyStarBase ),
						kMaxBPV,
						kClassStarBase,
						kClassStarBase,
						0,
						kStartPosition_X,
						kEnemyStarBase,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
						kMaxBPV,
						kClassDreadnought,
						kClassDreadnought,
						0,
						kStartPosition_W,
						kEnemyShip1,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
						kMaxBPV,
						kClassDreadnought,
						kClassDreadnought,
						0,
						kStartPosition_W,
						kEnemyShip2,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );

		for(i = 0; i <= 5; i++)
		{

			mCreateShip( typeid( tEnemyDefPlat ),
				Messages[kDefPlatFed + mGetRace()].c_str(),
				kStartPosition_O + i, kNoMetaShipID,
				kEnemyDefPlat1 + i, 0, 0, -1, -1, NULL,
				kDefaultShipOptions );

		}
		break;
	case kISC:
		break;
	case kKlingon:
	case kMirak:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kNoMetaShipID,
					kEnemyPlanet,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh0",
					kStartPosition_Y,
					kNoMetaShipID,
					kEnemyMoon,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_X,
					kEnemyShip1,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_W,
					kEnemyShip2,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_T,
					kEnemyShip3,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
		
		for(i = 0; i <= 7; i++)
		{
			mCreateShip( typeid( tEnemyDefPlat ),
				Messages[kDefPlatFed + mGetRace()].c_str(), kStartPosition_P+i, kNoMetaShipID,
				kEnemyDefPlat1 + i, 0, 0, -1, -1, NULL, kDefaultShipOptions );

		}
		break;
	}
	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}


void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
	fShieldTimer = 0;
	tShipInfo * starbase = NULL;
	tShipIterator ship = mGetFirstShip();
	for(ship; *ship != NULL; ship++)
	{
		if( ship->mGetCustomID() == kEnemyStarBase)
		{
			starbase = *ship;
		}
	}

	if(starbase != NULL)
	{
		int32 X, Y;
		starbase->mGetGridPosition(&X, &Y);

		ship = mGetFirstShip();
		for(ship; *ship != NULL; ship++)
		{
			if( ship->mGetCustomID() != kEnemyStarBase)
			{
				ship->mDefendEntity(starbase, 10.0f, 10.0f, kMediumPriority);
				ship->mSetFinalDestination(X, Y);
				ship->mSetShipThrust(10);
				ship->mSetShipAIControlled(kTrue);
			}
		}
	}

}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}


int32 tEnemyTeam::mShieldTimer(void)
{
	return fShieldTimer;
}
void tEnemyTeam::mIncShieldTimer(void)
{
	fShieldTimer ++;
}
