/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "TransportTeam.h"
#include "MoonTransport.h"
#include "PlanetTransport.h"
#include "MissionText.h"
#include "ISC_9B4Homeworld.h"
#include "ShipList_API.h"


#define kTimerConstant 2 / 60


void tPlayerTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	tISC_9B4Homeworld * script = (tISC_9B4Homeworld * ) gScriptInterface.fScript;
	if(script->fDiplomatBeamed)
	{
		fChildActor->mSetAttacker(true);
		tTeamInfo * EnemyTeam = fMissionInfo->mGetTeamHandle((eTeamID)kEnemyTeam);
		Assert(EnemyTeam);
		tShipIterator ship= EnemyTeam->mGetFirstShip();
		for(ship; *ship != NULL; ship++)
		{
			fChildActor->mGotoCommState((*ship)->mGetShipID(), tISC_9B4Homeworld::kCommState2);
		}
		return;
	}
	
}
void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if ( deepScanOn )
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kScanning_msg );
}
void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}
void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}
void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	static bool started = false;
	int32 x, y;

	tShipIterator  playership = fChildActor->mGetFirstShip();
	// I'm purposely not doing an assert here, as all I'm checking for is the presence
	// of the player ship, i.e. has it been created yet.

	if(*playership == NULL)
	{
		fChildActor->mSetTimedInterval(	kTimerTurnsSet, timerID, (float) kTimerConstant * 5);
		return;
	}

	// Make sure we get the first valid undestroyed ship.
    // A destroyed ship  will indicate that has left the system.

	for(playership; *playership != NULL; playership++)
	{

		if((*playership)->mInPlay())
			break;
	}

	if(!started)
	{
		tISC_9B4Homeworld * script = (tISC_9B4Homeworld *) gScriptInterface.fScript;

		if(script->fDiplomatBeamed)
		{
			switch( script->fSectorRace)
			{
			case kFederation:
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartFedDiplomacy_msg, kNonCommScriptMessage );
				break;
			case kHydran:
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartHydranDiplomacy_msg, kNonCommScriptMessage );
				break;

			case kRomulan:
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartRomulanDiplomacy_msg, kNonCommScriptMessage );
				break;
			}
		}
		else
		{

			fChildActor->mSetTransportTimer(0);
			if((script->fSectorRace == kKlingon) || (script->fSectorRace == kMirak))
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartHostileKorM_msg, kNonCommScriptMessage );
			}
			else
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartHostileGeneric_msg, kNonCommScriptMessage );
			}
		}
		started = true;
	}
	Assert(*playership != NULL);

	if ( timerID == tPlayerTeam::kEventTimerID )
	{	
		(*playership)->mGetGridPosition(&x, &y);
		if(AtEdgeOfMap(x, y))
		{
			if( !fChildActor->mAtEdgeOfMap() && (*playership)->mInPlay() )
			{
				fChildActor->mSetAtEdgeOfMap(true);
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "Science Officer: Leaving the system, [PLAYER_RANK]\n\n");
			}
		}
		else
		{
			if(fChildActor->mAtEdgeOfMap())
			{
				fChildActor->mSetAtEdgeOfMap(false);
			}
		}
		if(fChildActor->mTransportTimer() <= 150 )
		{
			if(fChildActor->mTransportTimer() == 135)
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportWarning_msg);
			}
			if(fChildActor->mTransportTimer()==150)
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportsArrive_msg);
				mCreateTransportShips();
			}
			fChildActor->mIncTransportTimer();
		}
		fChildActor->mSetTimedInterval(	kTimerTurnsSet, timerID, (float) kTimerConstant * 5);
	}

    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
        if (playerTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        if (playerTeam->mIsTeamHumanControlled()) return;               // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tPlayerTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        int32 BPVinc = gST.mUpgradeFightersForTeam(playerTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tPlayerTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }

}


void tPlayerTeamBaseState::mCreateTransportShips()
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeamBaseState::mCreateTransportShips\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID)kEnemyTeam);
        Assert(enemy);
        tTeamInfo * transports = fMissionInfo->mGetTeamHandle((eTeamID)kTransportTeam);
        Assert(transports);

        eRaceName enemyRace = enemy->mGetRace();
		eRaceName transportRace = transports->mGetRace();
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, transportRace );

        ShipList freighter_sl = sl;
        std::list< eEnhancedClassTypes >  freighterClassList;
        freighterClassList.push_back(kEClassLargeFreighter);
        fShiplistCount = freighter_sl.Subset(freighterClassList);
        gST.mDebug_fprintf("freighter class subset: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  freighterRolesList;
        freighterRolesList.push_back(kCommando);
        fShiplistCount = freighter_sl.Subset(freighterRolesList);
        gST.mDebug_fprintf("freighter roles subset: %d\n", fShiplistCount);
        gST.listShiplistContent(freighter_sl);

        if (freighter_sl.size() == 0) {
            // FAILED TO GET A COMMANDO FREIGHTER --FS
			gST.mDebug_fprintf("Failed to get a commando freighter! Let's do this again for a cargo freighter.\n");

			sl = gST.stripOutSPECIAL( sl );
			sl = gST.stripOutRestricted( sl );
			freighter_sl = sl;

			std::list< eEnhancedClassTypes >  freighterClassList;
			freighterClassList.push_back(kEClassLargeFreighter);
			fShiplistCount = freighter_sl.Subset(freighterClassList);
			gST.mDebug_fprintf("freighter class subset: %d\n", fShiplistCount);

			std::list< eEnhancedRoles >  freighterRolesList;
			freighterRolesList.push_back(kCargo);
			fShiplistCount = freighter_sl.Subset(freighterRolesList);
			gST.mDebug_fprintf("freighter roles subset: %d\n", fShiplistCount);
			gST.listShiplistContent(freighter_sl);
		}

        if (freighter_sl.size() > 0) {
            if(enemy!=NULL && transports!=NULL)
            {
                switch(enemyRace)
                {
				case kFederation:
                case kKlingon:
                case kMirak:
					/* SKIP. DISABLED.
                    {
                        Ship s1;
                        freighter_sl.SelectRandomShip(s1);
                        gST.mDebug_fprintf("freighter %s selected. BPV: %d  YFA: %d  YLA %d\n",
                            s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                        gST.mTrackOrions( s1 );

                    transports->mCreateShip( typeid( tMoonTransport ),
                        s1.shipCode.c_str(),
                        kStartPosition_N,
                        tTeamInfo::kNoMetaShipID,
                        tTransportTeam::kMoonTransport,
                        0, 0, -1, -1,
                        NULL,
                        kDefaultShipOptions );
                    }
					*/
                default:
                    {
                        Ship s1;
                        freighter_sl.SelectRandomShip(s1);
                        gST.mDebug_fprintf("freighter %s selected. BPV: %d  YFA: %d  YLA %d\n",
                            s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                        gST.mTrackOrions( s1 );

                        transports->mCreateShip( typeid( tPlanetTransport ),
                            s1.shipCode.c_str(),
                            kStartPosition_N,
                            tTeamInfo::kNoMetaShipID,
                            tTransportTeam::kPlanetTransport1,
                            0, 0, -1, -1,
                            NULL,
                            static_cast< eShipOptions >( kDefaultShipOptions ) );
                    }

                    {
                        Ship s1;
                        freighter_sl.SelectRandomShip(s1);
                        gST.mDebug_fprintf("freighter %s selected. BPV: %d  YFA: %d  YLA %d\n",
                            s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                        gST.mTrackOrions( s1 );

                        transports->mCreateShip( typeid( tPlanetTransport ),
                            s1.shipCode.c_str(),
                            kStartPosition_N,
                            tTeamInfo::kNoMetaShipID,
                            tTransportTeam::kPlanetTransport2,
                            0, 0, -1, -1,
                            NULL,
                            static_cast< eShipOptions >( kDefaultShipOptions | kStartPositionOffsetE) );
                    }

                    {
                        Ship s1;
                        freighter_sl.SelectRandomShip(s1);
                        gST.mDebug_fprintf("freighter %s selected. BPV: %d  YFA: %d  YLA %d\n",
                            s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                        gST.mTrackOrions( s1 );

                        transports->mCreateShip( typeid( tPlanetTransport ),
                            s1.shipCode.c_str(),
                            kStartPosition_N,
                            tTeamInfo::kNoMetaShipID,
                            tTransportTeam::kPlanetTransport3,
                            0, 0, -1, -1,
                            NULL,
                            static_cast< eShipOptions >( kDefaultShipOptions | kStartPositionOffsetW ) );
                    }
                    break;
                }
            }
        } else {
            mOldCreateTransportShips();
        }
    } else {
        mOldCreateTransportShips();
    }
    gST.mDebug_fprintf("--- Finished: tPlayerTeamBaseState::mCreateTransportShips\n");
}

void tPlayerTeamBaseState::mOldCreateTransportShips()
{
	gST.mDebug_fprintf("---- Entering: tPlayerTeamBaseState::mOldCreateTransportShips\n");

	tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID)kEnemyTeam);
	Assert(enemy);
	tTeamInfo * transports = fMissionInfo->mGetTeamHandle((eTeamID)kTransportTeam);
	Assert(transports);
	
	if(enemy!=NULL && transports!=NULL)
	{
		switch(enemy->mGetRace())
		{
		case kKlingon:
		case kMirak:
		/* DISABLE moon transport.
			transports->mCreateShip( typeid( tMoonTransport ),
				kMaxBPV,
				kClassFreighter,
				kClassFreighter,
				0,
				kStartPosition_N,
				tTransportTeam::kMoonTransport,
				0, 0, -1, -1, NULL, kDefaultShipOptions );
		*/
		default:
			transports->mCreateShip( typeid( tPlanetTransport ),
				kMaxBPV,
				kClassFreighter,
				kClassFreighter,
				0,
				kStartPosition_N,
				tTransportTeam::kPlanetTransport1,
				0, 0, -1, -1, NULL, static_cast< eShipOptions >( kDefaultShipOptions ) );

			transports->mCreateShip( typeid( tPlanetTransport ),
				kMaxBPV,
				kClassFreighter,
				kClassFreighter,
				0,
				kStartPosition_N,
				tTransportTeam::kPlanetTransport2,
				0, 0, -1, -1, NULL, static_cast< eShipOptions >( kDefaultShipOptions | kStartPositionOffsetW ) );

			transports->mCreateShip( typeid( tPlanetTransport ),
				kMaxBPV,
				kClassFreighter,
				kClassFreighter,
				0,
				kStartPosition_N,
				tTransportTeam::kPlanetTransport3,
				0, 0, -1, -1, NULL, static_cast< eShipOptions >( kDefaultShipOptions | kStartPositionOffsetW ) );
			break;
		}
	}

	gST.mDebug_fprintf("---- Finished: tPlayerTeamBaseState::mOldCreateTransportShips\n");
}




void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

bool tPlayerTeamBaseState::AtEdgeOfMap(int32 X, int32 Y)
{
	int32 GridX;
	int32 GridY;
	gScriptInterface.mGetMapDimensions( &GridX, &GridY);

	// kludge to keep a destroyed play ship from indication off the map.
	if( X == 0 && Y == 0)
		return false;

	if( (X < 2 || X > (GridX - 1)) || (Y < 2 || Y > (GridY - 1)))
	{
		return true;
	}
	else
	{
		return false;
	}
}



