/********************************************************************************************************\

	File Name:	BaseVictoryState.h
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#if !defined( BaseVictoryState_h )
#define BaseVictoryState_h

#include "ChildActorVictoryState.h"
#include "BaseVictory.h"


class tBaseVictoryState : public tChildActorVictoryState<tBaseVictory>
{
public:

	//
	//	Serialization events
	//
	virtual void mBeginState();
	virtual void mEndState();
	tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat );
	virtual eVictoryLevels mCalcVictoryStatus( eTeamID ID );

	//
	//	Ship events
	//
	virtual void mOnDeath( tShipInfo* ship );
	virtual void mOnCapture( tShipInfo* ship );
	virtual void mOnDisengaged( tShipInfo* ship );
	virtual void mAtDistance( tShipInfo* ship );
	virtual void mAtDestination( tShipInfo* ship );
	virtual void mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip );
	virtual void mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType );
	virtual void mOnDeepScan( tShipInfo* ship, iTruth deepScanOn );
	virtual void mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus );
	virtual void mOnProbeLaunch( tShipInfo* ship );
	virtual void mOnBeginOrbit( tShipInfo* ship );
	virtual void mOnEndOrbit( tShipInfo* ship );
	virtual void mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip );
	virtual void mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip );
	virtual void mOnFlashingButtonPressed();
	virtual void mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item );
	virtual void mOnChangingShip( tShipInfo* ship );
	virtual void mOnSpecialTransport( tShipInfo* ship );
	virtual void mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value );
	virtual void mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent );
	virtual void mOnShipStoresUpdate( tShipInfo* ship, eShipUpdateType type, 
									  eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value );
	virtual void mOnShipStoresUpdate( tShipInfo* ship, eShipUpdateType type,
									  eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value );

	//
	//	System events
	//
	virtual void mOnPlayerLeaving( int playerNumber );
	virtual void mOnHostChanged();
	virtual void mOnTimedInterval( int32 timerID );
	virtual void mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType );
	virtual void mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType );

	enum eTimerIDs
	{
		kCheckSecretTransmission = 1001,
		kScanTimer,
		kPrimaryObjectiveCompleteID,
		kStartingID,
		kOnDeepScanID,
	};

private:
	int32 iNumEnemyShipsEscapedWithSecret;
	int32 iNumEnemyShipsDefeated;
	iTruth fSecretSecured;
	iTruth fEnemyGotTheSecret;
	iTruth fTalkedToSun;
	iTruth fPlayerDefeated;
	iTruth fTractoringTransport;
	iTruth fScanning;
	iTruth fEnemySetToFollowTransport;

	iTruth fEnemySurrendered;
	iTruth fTransportSurrendered;
	iTruth fTransportCaptured;
	iTruth fTransportHailed;

	bool fDeepScannedSun;
	bool fDeepScannedTransport;

	float CalculateAverageEnemyDamage();
	iTruth OrderEnemyToFollowTransport();
	tShipInfo* GetCurrentShipTarget(tShipInfo* ship);
	void CheckEndMission();
	void EndMission(eGameStatus Status);
	void DoHailEnemy(tShipInfo* ship, tShipInfo* targetShip);
	void DoDemandSurrenderEnemy(tShipInfo* ship, tShipInfo* targetShip);
	void DoHailTransport(tShipInfo* ship, tShipInfo* targetShip);
	void DoDemandSurrenderTransport(tShipInfo* ship, tShipInfo* targetShip);
	void DoHailSun(tShipInfo* ship, tShipInfo* targetShip);

};

#endif
