/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "CommonDefs.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyShip.h"
#include "MissionText.h"
#include "ISC_X3M1BTemple.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
		int32 enemy1BPV = 150;
		int32 enemy2BPV = 50;

		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
		int32 playerBPV = playerTeam->mGetCombatBPV();

		if( diffLevel == 0 ) {
			enemy1BPV = playerBPV * 0.5;
			enemy2BPV = playerBPV * 0.3;
		} else if( diffLevel == 2 )	{
			enemy1BPV = playerBPV * 1.0;
			enemy2BPV = playerBPV * 0.3;
		} else {
			enemy1BPV = playerBPV * 0.6;
			enemy2BPV = playerBPV * 0.4;
		}

		int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
		shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		ShipList list1_sl = sl;
		fShiplistCount = list1_sl.Subset( list1_sl.kSubsetBPV, int(0.825 * enemy1BPV), int(1.175 * enemy1BPV), true);
		gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
			int(0.825 * enemy1BPV), int(1.175 * enemy1BPV), fShiplistCount);

		ShipList list2_sl = sl;
		fShiplistCount = list2_sl.Subset( list2_sl.kSubsetBPV, int(0.825 * enemy2BPV), int(1.175 * enemy2BPV), true);
		gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
			int(0.825 * enemy2BPV), int(1.175 * enemy2BPV), fShiplistCount);


        if ((sl.size() > 0) && (list1_sl.size() > 0) && (list2_sl.size() > 0)) {
			Ship s1, s2;

            list1_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            list2_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            mCreateShip(
                typeid( tEnemyShip ),
                s1.shipCode.c_str(),
                kStartPosition_G,
                kNoMetaShipID,
                kEnemyShip1,
                0, 0, -1, -1, Messages[kEnemyShipName1].c_str(), 
				static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );

			mCreateShip(
                typeid( tEnemyShip ),
                s2.shipCode.c_str(),
                kStartPosition_G,
                kNoMetaShipID,
                kEnemyShip2,
                0, 0, -1, -1, Messages[kEnemyShipName2].c_str(), 
				static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );

		} else {
			mOldCreateShipsForTeam();
		}

	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	int32 enemy1BPV = 0;
	int32 enemy2BPV = 0;

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
	int32 playerBPV = playerTeam->mGetCombatBPV();


	if( diffLevel == 0 )
	{
		enemy1BPV = playerBPV * 0.5;
		enemy2BPV = playerBPV * 0.3;
	}
	else if( diffLevel == 1 )
	{
		enemy1BPV = playerBPV * 0.6;
		enemy2BPV = playerBPV * 0.4;
	}
	else if( diffLevel == 2 )
	{
		enemy1BPV = playerBPV * 1.0;
		enemy2BPV = playerBPV * 0.3;
	}

	mCreateShip( typeid( tEnemyShip ),
							enemy1BPV,
							kClassFrigate,
							kClassBattleship,
							0,
							kStartPosition_G, kEnemyShip1, 0, 0, -1, -1, Messages[kEnemyShipName1].c_str(), static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );			

	mCreateShip( typeid( tEnemyShip ),
							enemy2BPV,
							kClassFrigate,
							kClassBattleship,
							0,
							kStartPosition_G, kEnemyShip2, 0, 0, -1, -1, Messages[kEnemyShipName2].c_str(), static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );			


	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}


void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}

