/********************************************************************************************************\

	File Name:	TransportTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "TransportTeam.h"
#include "TransportTeamBaseState.h"
#include "Transport.h"
#include "ISC_X3M1BTemple.h"
#include "ShipList_API.h"


tClassAllocator<tTransportTeam> gTransportTeamAllocator;


void tTransportTeam::mSetupState()
{
	mRegisterState( new tTransportTeamBaseState() );
	mGotoState( typeid( tTransportTeamBaseState ) );
}


void tTransportTeam::mSetupCommState()
{}


void tTransportTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tTransportTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        std::list< eEnhancedClassTypes >  ClassTypeList;
        ClassTypeList.push_back(kEClassSmallFreighter);
        ClassTypeList.push_back(kEClassCivilianFreighter);
        ClassTypeList.push_back(kEClassPriorityTransport);
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for small freighter classes filter completed. ships: %d\n", fShiplistCount);

		{
        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kCargo);
        fShiplistCount = sl.Subset(RolesList, true);
        gST.mDebug_fprintf("subset for cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);
		}

		{
        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kQship);
        fShiplistCount = sl.Subset(RolesList, false);
        gST.mDebug_fprintf("removed Qship eEnhancedRoles. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);
		}

        if (sl.size() > 0) {
			Ship s1;

            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			mCreateShip(typeid(tTransport), 
				s1.shipCode.c_str(), 
				kStartPosition_I, 
				kNoMetaShipID,	
				kSevrinTransportID, 
				0, 0, -1, -1, 
				Messages[kTransportName].c_str(), 
				kNoShipOptions 
			);

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tTransportTeam::mCreateShipsForTeam\n");
}


void tTransportTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tTransportTeam::mOldCreateShipsForTeam\n");

	mCreateShip(typeid(tTransport), 
		Messages[kTransportClass].c_str(), 
		kStartPosition_I, 
		kNoMetaShipID,	
		kSevrinTransportID, 
		0, 0, -1, -1, 
		Messages[kTransportName].c_str(), 
		kNoShipOptions 
	);

	gST.mDebug_fprintf("---- Finished: tTransportTeam::mOldCreateShipsForTeam\n");
}


void tTransportTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}


void tTransportTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tTransportTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tTransportTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}