/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Constants.h"
#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ISC_X4M1BThief.h"

//
//	Serialization events
//

tBaseVictoryState::tBaseVictoryState( tTeamInfo* team, int32 playerTeamID, int32 enemeyTeamID, int32 captureBouns, int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat ),
	bEnemyHailed(kFalse),
	bEnemySurrendered(kFalse),
	iNumEnemyShipsDestroyed(0),
	iNumEnemyShipsDefeated(0),
	iNumEnemyShipsEscaped(0),
	iNumTransportsLeft(NUM_TRANSPORTS),
	iNumTransportsTaken(0)
{}

/*
	This method awards prestige as appropriate. If the player defeated 
	all enemy ships, then he gets a base amount of prestige for that
	along with a bonus for each transport saved.
*/

void tBaseVictoryState::AwardPrestige()
{
	if (iNumEnemyShipsDefeated == NUM_ENEMY_SHIPS)
	{
		mAddToBonusPrestige(PRESTIGE_ENEMY_DEFEATED);
		mAddToBonusPrestige(iNumTransportsLeft * PRESTIGE_PER_TRANSPORT);
	}
}	

/*
	This method ends the mission in accordance with the supplied status code.
*/

void tBaseVictoryState::EndMission(eGameStatus Status)
{
	AwardPrestige();
	gScriptInterface.mSetGameStatus((eTeamID)kPlayerTeam, Status);
}	

/*
	This method runs the enemy team list of ships and calculates an average
	of their damage.
*/

float tBaseVictoryState::CalculateAverageEnemyDamage()
{
	float fTotalDamage = 0.0f;
	int iNumShips = 0;
	tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

	for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
	{
		if ((*i)->mInPlay())
		{
			fTotalDamage += (*i)->mGetDamagePercent();
			iNumShips++;
		}
	}

	return fTotalDamage / static_cast<float>(iNumShips);
}	

void tBaseVictoryState::mBeginState()
{
#if defined(kRandomChatter)
	mSetTimedInterval(kTimerSecondsSet, kRandChatID, RANDOM_CHATTER_INTERVAL);
#endif
}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus(eTeamID ID)
{
	// TODO: figure out how the team ID plays into this.

	if (iNumEnemyShipsDefeated < NUM_ENEMY_SHIPS)
	{
		return kDevastatingDefeat;
	}
	else
	{
		if (iNumTransportsTaken == 0)
			return kAstoundingVictory;
		else if (iNumTransportsTaken <= MAX_TRANSPORTS_LOST_VICTORY)
			return kVictory;
		else if (iNumTransportsTaken < NUM_TRANSPORTS)
			return kDraw;
		else
			return kDefeat;
	}
}

void tBaseVictoryState::mOnDeath(tShipInfo* ship)
{
	eTeamID teamOriginal = ship->mGetOriginalTeam();

	if (teamOriginal == kTransportTeam)
	{
		if (--iNumTransportsLeft == 0)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kNoTransportsLeft);
	}
	else if (teamOriginal == kEnemyTeam)
	{
		++iNumEnemyShipsDestroyed;
		++iNumEnemyShipsDefeated;

		tTeamInfo*	enemyteam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );

		if ( enemyteam->mShipsRemainingInPlay() < 1 )
		{
			EndMission(kPlayerWins);
		}
	}
	else if (teamOriginal == kPlayerTeam)
	{
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle(ship->mGetTeam());
		if (!team->mShipsRemainingInPlay())
			EndMission(kPlayerDestroyed);
	}
	else if (teamOriginal == kPlatformTeam)
	{
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kDefensePlatformDestroyed);
	}
}

void tBaseVictoryState::mOnCapture(tShipInfo* ship)
{
	eTeamID teamOriginal = ship->mGetOriginalTeam();

	if (teamOriginal == kEnemyTeam)
	{
		++iNumEnemyShipsDefeated;

		tTeamInfo*	enemyteam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );

		if ( enemyteam->mShipsRemainingInPlay() < 1 )
		{
			EndMission(kPlayerWins);
		}
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID teamOriginal = ship->mGetOriginalTeam();

	if (teamOriginal == kTransportTeam)
	{
		++iNumTransportsTaken;
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kTransportDisengaged);
		if (--iNumTransportsLeft == 0)
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kNoTransportsLeft);
	}
	else if (teamOriginal == kEnemyTeam)
	{
		++iNumEnemyShipsEscaped;
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle(ship->mGetTeam());
		if (!team->mShipsRemainingInPlay())
			EndMission(kEnemyDisengaged);
	}
	else if (teamOriginal == kPlayerTeam)
	{
		EndMission(kPlayerDisengaged);
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}

/*
	This method handles all communication with enemy ships in this mission.
*/

void tBaseVictoryState::mOnCommButtonPress(tShipInfo* ship, int32 buttonID, tShipInfo* targetShip)
{
	int32 iMsgID = 0;

	if (buttonID == k_HAIL_Enemy_btn)
	{
		if (targetShip->mGetTeam() == kPlayerTeam)
			iMsgID = k_HAIL_Nme_After_Capture_msg;
		else if (bEnemySurrendered)
			iMsgID = k_HAIL_Nme_After_Surrender_msg;
		else if (!bEnemyHailed)
		{
			iMsgID = k_HAIL_Nme_First_Contact_msg;
			bEnemyHailed = kTrue;
		}
		else
		{
			iMsgID = fMissionInfo->mRandomInt32((int32)k_HAIL_Nme_Random_Response_1_msg, 
												(int32)k_HAIL_Nme_Random_Response_3_msg);
		}
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iMsgID);
	}
	else if (buttonID == k_DS_Enemy_btn)
	{
		if (targetShip->mGetTeam() == kPlayerTeam)
			iMsgID = k_DS_Nme_After_Capture_msg;
		else if (bEnemySurrendered)
			iMsgID = k_DS_Nme_After_Surrender_msg;
		else
		{
			eRace raceEnemy = targetShip->mGetRace();
			float fDamage = CalculateAverageEnemyDamage();
			iTruth bDestruct = kFalse, bSurrender = kFalse;
			tTeamInfo* teamEnemy = fMissionInfo->mGetTeamHandle(targetShip->mGetTeam());

			/*
				Klingons, Romulans and the Mirak will not surrender. They will
				self-destruct, however, if damaged badly enough.
			*/

			if (raceEnemy == kKlingon || raceEnemy == kRomulan || raceEnemy == kMirak)
			{
				if (fDamage >= SELF_DESTRUCT_THRESHOLD)
				{
					bDestruct = kTrue;
					iMsgID = k_DS_Nme_Self_Destruct_msg;
				}
			}
			else if (fDamage >= SURRENDER_THRESHOLD)
			{
				bSurrender = kTrue;
				iMsgID = k_DS_Nme_Surrendering_msg;
				teamEnemy->mSetRelations(static_cast<eTeamID>(kPlayerTeam), kTeamNeutral, kGoalIgnore, 1.0);
			}

			/*
				If the enemy is either self-destructing or surrendering, the
				player has won the scenario. All that remains to be done here
				is to run the list of ships still in play and add to the
				appropriate accumulators for victory calculations. Otherwise,
				we display a random condescending message for the player and
				continue with the game.
			*/

			if (bSurrender || bDestruct)
			{
				for (tShipIterator i = teamEnemy->mGetFirstShip(); *i != NULL; i++)
				{
					/*
						If the ship is to be destroyed, the mOnDeath handler
						will properly update the accumulators. Otherwise, we
						must increment the number of ships defeated.
					*/

					if ((*i)->mInPlay())
					{
						if (bDestruct)
							(*i)->mDestroyShip();
						else
							++iNumEnemyShipsDefeated;
					}
				}

				/*
					If surrendering, we'll need to explicitly call this 
					function here, whereas self-destructing will take care
					of it in the mOnDeath handler.
				*/

				if (bSurrender)
					EndMission(kPlayerWins);
			}
			else
			{
				iMsgID = fMissionInfo->mRandomInt32((int32)k_DS_Nme_Random_Decline_1_msg, 
													(int32)k_DS_Nme_Random_Decline_3_msg);
			}
		}
		fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iMsgID);
	}
}

void tBaseVictoryState::mOnDamage(tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType)
{}

void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat1_msg );	break;
		case 1:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat2_msg);	break;
		case 2:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat3_msg);	break;
		case 3:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat4_msg);	break;
		case 4:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat5_msg);	break;
		case 5:
			fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kRandChat6_msg);	break;
		}

		float fInterval = RANDOM_CHATTER_INTERVAL + 
						  (float)(fMissionInfo->mRandomInt32(0, (int32)RANDOM_CHATTER_RANGE));
		fChildActor->mSetTimedInterval(kTimerSecondsSet, timerID, fInterval);
	}
#endif
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}