/*
	Constants.h

	This module contains the declarations for global constants.
*/

#ifndef _CONSTANTS_H_
#define _CONSTANTS_H_

#include "tbasic.h"

/*
	The number of defense platforms to be created. Note carefully
	that for each platform to be created, there must be a starting
	location defined in the mission map. Further, the starting locations
	of the platforms are assumed to be consecutive.
*/
const int32 NUM_PLATFORMS = 4;

/*
	The number of transports to be created. Note carefully
	that for each transport to be created, there must be a starting
	location defined in the mission map. Further, the starting locations
	of the transports are assumed to be consecutive.
*/
const int32 NUM_TRANSPORTS = 4;

// The minimum number of transports to retain for victory.
const int32 MAX_TRANSPORTS_LOST_VICTORY = 1;

// The maximum length (in characters) of a name for a ship.
const int32 MAX_SHIP_NAME_LENGTH = 64;

// The number of bonus prestige points awarded for each transport saved.
const int32 PRESTIGE_PER_TRANSPORT = 30;

// The number of bonus prestige points awarded for destroying the enemy ships.
const int32 PRESTIGE_ENEMY_DEFEATED = 30;

// The number of enemy ships to be faced.
const int32 NUM_ENEMY_SHIPS = 2;

// The damage percent beyond which the enemy ship will surrender if asked.
const float SURRENDER_THRESHOLD = 80.0f;

// The damage percent beyond which the enemy ship will self destruct.
const float SELF_DESTRUCT_THRESHOLD = 90.0f;

// The interval (in seconds) at which un-captured transports should have their shields lowered.
const float SHIELD_TIMER_INTERVAL = 5.0f;

// Minimum number of seconds between random chatter intervals.
const float RANDOM_CHATTER_INTERVAL = 30.0f;

// Maximum variance for rander chatter interval.
const float RANDOM_CHATTER_RANGE = 10.0f;

#endif	// _CONSTANTS_H_
