/*
	EnemyTeam.cpp

	This module contains the definition of the tEnemyTeam class.
*/

#include "Constants.h"
#include "ISC_X4M1BThief.h"
#include "tclient.h"
#include "ShipScriptDescription.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyShip.h"
#include "ShipList_API.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}


void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		int32 iTotalBPV, iPlayerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
		int32 iVariantPercent = 0;

		if( diffLevel == 0 )
			iTotalBPV = iPlayerBPV * 0.75;
		else if( diffLevel == 1 )
		{
			iTotalBPV = iPlayerBPV * 1.0;
		}
		else if( diffLevel == 2 )
		{
			iTotalBPV = iPlayerBPV * 1.25;
		}

		std::string chosenShips[6];

		gST.mPickFleet(iTotalBPV, NUM_ENEMY_SHIPS, mGetRace(), chosenShips);
		for (int32 i = 0; i < NUM_ENEMY_SHIPS; i++) {

			mCreateShip( typeid(tEnemyShip), 
						 chosenShips[i].c_str(), 
						 kStartPosition_P,
						 kNoMetaShipID,
						 kEnemyShip, 0, 0, -1, -1,
						 NULL,
						 kDefaultShipOptions  );
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	int32 iTotalBPV, iPlayerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	int32 iVariantPercent = 0;

	if( diffLevel == 0 )
		iTotalBPV = iPlayerBPV * 0.75;
	else if( diffLevel == 1 )
	{
		iTotalBPV = iPlayerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		iTotalBPV = iPlayerBPV * 1.25;
	}

	mCreateFleet(typeid(tEnemyShip), NUM_ENEMY_SHIPS, iTotalBPV, 
				 kClassDestroyer, kClassLightCruiser,
				 iVariantPercent, kStartPosition_P, kEnemyShip,
				 0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tDraftTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tDraftTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}

