/*
	ISC_X4M1BThief.cpp

	This module contains the definition of the tISC_X4M1BThief class,
	which is the main class for the script.

	Mission:	A Thief in the Night (from the ISC perspective)
	Script:		"ISC_X4M1BThief"
	Descript:	The player must prevent the enemy from capturing some
				transport ships.

	Created by:		John B. Williston		06/29/00
	Last Updated:	JBW						06/29/00
					DJS						09/26/00
					Meta script modifications

*/

#include "ISC_X4M1BThief.h"
#include "PlayerTeam.h"
#include "TransportTeam.h"
#include "EnemyTeam.h"
#include "PlatformTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tISC_X4M1BThief );

tISC_X4M1BThief::tISC_X4M1BThief()
{}

void tISC_X4M1BThief::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast<eTeamID>(kPlayerTeam),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Transport Team",
												static_cast<eTeamID>(kTransportTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tTransportTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",
												static_cast<eTeamID>(kEnemyTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecGorn | kSpecHydran,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Platform Team",
												static_cast<eTeamID>(kPlatformTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tPlatformTeam ) );

    const tTeamSpec& team5 = mCreateTeamSpec(   "Team 5", 
												static_cast< eTeamID>( kDraftTeam1 ),
                                                kMaxChance,
                                                kTrue,
                                                kPlayableTeam,
                                                kMaxChance,
                                                kSpecAnyRace,
                                                kAnyRace,
                                                typeid( tDraftTeam ) );

    const tTeamSpec& team6 = mCreateTeamSpec(   "Team 6", 
												static_cast< eTeamID>( kDraftTeam2 ),
                                                kMaxChance,
                                                kTrue,
                                                kPlayableTeam,
                                                kMaxChance,
                                                kSpecAnyRace,
                                                kAnyRace,
                                                typeid( tDraftTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

int32 tISC_X4M1BThief::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tISC_X4M1BThief::mCreateExternalShips( int32 Mode )
{}

int32 tISC_X4M1BThief::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tISC_X4M1BThief::mNewInternalShip( tShipInfo* Ship )
{}

void tISC_X4M1BThief::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kTransportTeam,
		kEnemyTeam,
		kPlatformTeam,
	};

	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kTransportTeam,
		//Nme  = kEnemyTeam,
		//Plat = kPlatformTeam,

		//	     Plyr  PNPC  Nme   Plat
		/*Plyr*/ "--", "NI", "WD", "AI",
		/*PNPC*/ "NI", "--", "NI", "NI",
		/*Nme */ "WD", "NX", "--", "WD",
		/*Plat*/ "AI", "NI", "WD", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_X4M1BThief::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['K'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['L'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['M'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['N'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['O'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['P'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Q'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_X4M1BThief::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tISC_X4M1BThief::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}