/********************************************************************************************************\

	File Name:	PlatformTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Constants.h"
#include "MissionText.h"
#include "tclient.h"
#include "ShipScriptDescription.h"
#include "PlatformTeam.h"
#include "PlatformTeamBaseState.h"
#include "Platform.h"
#include "ShipList_API.h"

tClassAllocator<tPlatformTeam> gPlatformTeamAllocator;

void tPlatformTeam::mSetupState()
{
	mRegisterState( new tPlatformTeamBaseState() );
	mGotoState( typeid( tPlatformTeamBaseState ) );
}

void tPlatformTeam::mSetupCommState()
{}


void tPlatformTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlatformTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		// prepare a defense platform shiplist
		// iterate and create them
		int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        // ense platforms
        std::list< eEnhancedClassTypes >  ClassTypeList;
        ClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for platform classes filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kLocalDefense);
        fShiplistCount = sl.Subset(RolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);

		if (sl.size() > 0) {
			long lStartPosition = kStartPosition_L;
			long lID = kPlatform1;

			for (int i = 0; i < NUM_PLATFORMS; i++)
			{
				Ship s1;
				sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				mCreateShip(typeid(tPlatform), s1.shipCode.c_str(), lStartPosition++, kNoMetaShipID,
					lID++, 0, 0, -1, -1, Messages[kPlatformName].c_str(), kDefaultShipOptions );
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tPlatformTeam::mCreateShipsForTeam\n");
}


void tPlatformTeam::mOldCreateShipsForTeam()
{
	// Create the ISC "shooting mines".

	long lStartPosition = kStartPosition_L;
	long lID = kPlatform1;

	for (int i = 0; i < NUM_PLATFORMS; i++)
	{
		mCreateShip(typeid(tPlatform), Messages[kPlatformClass].c_str(), lStartPosition++, kNoMetaShipID,
					lID++, 0, 0, -1, -1, Messages[kPlatformName].c_str(), kDefaultShipOptions );
	}
}

void tPlatformTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlatformTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlatformTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tPlatformTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}
