/*
	Transport.h

	This module contains the declaration for the tTransport class, whose
	contributions are limited essentially to defining the valid transport
	comm states and configuring the transports at start of mission.
*/

#if !defined( Transport_h )
#define Transport_h

#include "ShipInfo.h"

class tTransport : public tShipInfo
{
public:
	tTransport();

	virtual	void mSetupState();
	virtual void mSetupCommState();
	virtual void mSetupShip();

	// Comm state IDs for the transports.
	enum eCommStates
	{
		kCommState1,
		// Transport is being held by the enemy.
		kCommState2,
		// Transport captured; hull at 100%.
		kCommState3,
		// Transport captured; hull at >70%.
		kCommState4,
		// Transport captured; hull at >40%.
		kCommState5,
		// Transport captured; hull at <= 40%.
		kCommState6,

		kBlankCommState,
	};

	iTruth IsInitialCapture() const
		{ return m_bInitialCapture; }
	void SetInitialCapture()
		{ m_bInitialCapture = kFalse; }
	iTruth WasPreviouslyCapturedByPlayer() const
		{ return m_bCapturedByPlayer; }
	void SetCapturedByPlayer()
		{ m_bCapturedByPlayer = kTrue; }

private:
	iTruth m_bInitialCapture;
	iTruth m_bCapturedByPlayer;
};

#endif
