/*
	TransportTeam.cpp

	This module contains the definition for the tTransportTeam class. 
	Its sole contribution is the creation of the transports.
*/

#include "Constants.h"
#include "ShipScriptDescription.h"
#include "MissionText.h"
#include "tclient.h"
#include "Transport.h"
#include "TransportTeam.h"
#include "TransportTeamBaseState.h"
#include "ShipList_API.h"

tClassAllocator<tTransportTeam> gPlayerNPCTeamAllocator;

void tTransportTeam::mSetupState()
{
	mRegisterState( new tTransportTeamBaseState() );
	mGotoState( typeid( tTransportTeamBaseState ) );
}

void tTransportTeam::mSetupCommState()
{}

/*
	This method creates the given number of transports as small-phaser
	equipped freighters. They are created within a simple loop that relies
	upon the transport initial positions to be consecutive. Their names
	are generated via a straightforward C-library sprintf call.
*/


void tTransportTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		// prepare a freighter shiplist with small cargo freighters
		// iterate and create them
		int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        // ense platforms
        std::list< eEnhancedClassTypes >  ClassTypeList;
        ClassTypeList.push_back(kEClassSmallFreighter);
        ClassTypeList.push_back(kEClassCivilianFreighter);
        ClassTypeList.push_back(kEClassPriorityTransport);
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for small freighter classes filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kCargo);
        fShiplistCount = sl.Subset(RolesList, true);
        gST.mDebug_fprintf("subset for cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);

		if (sl.size() > 0) {
			long lStartPosition = kStartPosition_H;
			long lID = kTransport1;

			for (int i = 0; i < NUM_TRANSPORTS; i++)
			{
				Ship s1;
				sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				char szShipName[MAX_SHIP_NAME_LENGTH];
				sprintf(szShipName, Messages[kTransportNameFormat].c_str(), i + 1);
				mCreateShip(typeid(tTransport), s1.shipCode.c_str(), lStartPosition++, kNoMetaShipID,
							lID++, 0, 0, -1, -1, szShipName, kNoShipOptions );
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tTransportTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tTransportTeam::mOldCreateShipsForTeam\n");

	long lStartPosition = kStartPosition_H;
	long lID = kTransport1;

	for (int i = 0; i < NUM_TRANSPORTS; i++)
	{
		char szShipName[MAX_SHIP_NAME_LENGTH];
		sprintf(szShipName, Messages[kTransportNameFormat].c_str(), i + 1);
		mCreateShip(typeid(tTransport), Messages[kTransportClass].c_str(), lStartPosition++, kNoMetaShipID,
					lID++, 0, 0, -1, -1, szShipName, kNoShipOptions );
	}

	gST.mDebug_fprintf("---- Finished: tTransportTeam::mOldCreateShipsForTeam\n");
}


void tTransportTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tTransportTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tDraftTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tDraftTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}

