/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "BaseVictory.h"
#include "EnemyTeam.h"
#include "Freighter.h"
#include "EnemyShip.h"
#include "AsteroidA.h"
#include "AsteroidB.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ISC_X4M2BBomber.h"
#include "ShipList_API.h"

#define kTimerConstant (2.0 / 60)
//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( tTeamInfo* team, int32 playerTeamID, int32 enemeyTeamID, int32 captureBouns, int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
#if defined logging
			Log("CalcVictoryStatus");
#endif


	eVictoryLevels VictoryLevel = kDevastatingDefeat;
	if(fChildActor->mReachedAsteroidA())
	{
		VictoryLevel = kDefeat;
		if(fChildActor->mReachedAsteroidB())
		{
#if defined logging
			Log("Astounding victory");
#endif
			VictoryLevel = kAstoundingVictory;
		}
	}
	return VictoryLevel;
}


void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
/*

	HELPER NOTE:
	If this is not a single player script, you should
	change the team calls here to the proper enums.

*/
	if ( ship->mGetTeam() == kPlayerTeam )
	{
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		if ( !team->mShipsRemainingInPlay() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );
	}
	else if(ship->mGetTeam() == kPlayerNPCTeam)
	{
		if(!fChildActor->mFreighterDestroyed())
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kFreighterDestroyed_msg);
			fChildActor->mSetFreighterDestroyed(true);
		}
	}
	else if(ship->mGetTeam() == kEnemyTeam)
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEnemyDefeated_msg);
		// Start destroyer timer
		if(!fChildActor->mEnemyDefeated())
		{
			mSetTimedInterval(	kTimerTurnsSet, kDestroyerID, (float)kTimerConstant * 10);
		}
		fChildActor->mIncEnemyDefeated();
				tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		if(!team->mShipsRemainingInPlay() )
		{
			if(fChildActor->mReachedAsteroidB() && fChildActor->mEnemyDefeated() > 1)
			{
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
			}
			else if(fChildActor->mFreighterDestroyed())
			{
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );
			}
		}


	}

}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
/*

	HELPER NOTE:
	If this is not a single player script, you should
	change the team calls here to the proper enums.

*/
	if ( ship->mGetOriginalTeam() == kPlayerTeam )
	{
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		if ( !team->mShipsRemainingInPlay() )
		{
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerCaptured );
		}
	}
	else if(ship->mGetOriginalTeam() == kEnemyTeam)
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEnemyDefeated_msg);
			// Start destroyer timer
		if(!fChildActor->mEnemyDefeated())
		{
			mSetTimedInterval(	kTimerTurnsSet, kDestroyerID, (float)kTimerConstant * 10);
		}
		fChildActor->mIncEnemyDefeated();
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		if(!team->mShipsRemainingInPlay() )
		{
			if(fChildActor->mReachedAsteroidB())
			{
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
			}
			else if(fChildActor->mFreighterDestroyed())
			{
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );
			}
		}


	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
/*

	HELPER NOTE:
	If this is not a single player script, you should 
	change the team calls here to the proper enums.

*/
	if ( ship->mGetTeam() == kPlayerTeam )
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kLeftSystem_msg);
		gScriptInterface.mSetGameStatus((eTeamID) kPlayerTeam, kPlayerDisengaged);
	}
	if ( ship->mGetTeam() == kPlayerNPCTeam )
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kFreighterDisengaged);
	}

}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	int32 targetID = targetShip->mGetCustomID();
	tTeamInfo * playerNPCteam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
	tTeamInfo * playerteam =  fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	switch(buttonID)
	{
	case kHail_btn:
		if(targetShip->mGetTeam()==kEnemyTeam)
		{
			if(fChildActor->mTalkedToEnemy())
			{
				playerteam->mGotoCommState( targetShip->mGetShipID(), tEnemyShip::kCommState2);
			}
			else
			{
				fChildActor->mSetTalkedToEnemy(true);
				mSetTimedInterval( kTimerSecondsSet, kDialogID, 1.0f );
			}
		}
	case kDemandSurrender_btn:
		break;
	case kOrbitAsteroidA_btn:
		if(!fChildActor->mOrbitingAsteroidA())
		{
			SendToOrbit(targetShip, playerNPCteam->mFindShipCustomID(tPlayerNPCTeam::kAsteroidA));
			targetShip->mSetShipMaxSpeed(31);
		}
		break;
	case kOrbitAsteroidB_btn:
		if(fChildActor->mOrbitingAsteroidA() && fChildActor->mRepairsMade() && !fChildActor->mReachedAsteroidB())
		{
			SendToOrbit(targetShip, playerNPCteam->mFindShipCustomID(tPlayerNPCTeam::kAsteroidB));
			targetShip->mSetShipMaxSpeed(31);
		}
		break;
	}
	
}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	tTeamInfo * PlayerNPCTeam;
	tTeamInfo * PlayerTeam;
	tTeamInfo * EnemyTeam;
	tShipInfo * freighter;
	tShipInfo * AsteroidA;
	tShipInfo * AsteroidB;
	float kTimerIncrement;


	PlayerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert(PlayerTeam);
	PlayerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
	Assert(PlayerNPCTeam);
	freighter = PlayerNPCTeam->mFindShipCustomID(tPlayerNPCTeam::kFreighter);
//	Assert(freighter);
	AsteroidA =PlayerNPCTeam->mFindShipCustomID(tPlayerNPCTeam::kAsteroidA);
	AsteroidB =PlayerNPCTeam->mFindShipCustomID(tPlayerNPCTeam::kAsteroidB);
	EnemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );

	switch( timerID )
	{
	
	case kDestroyerID:
		mCreateDestroyer();

		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kDestroyerGenerated_msg);
		return;
		break;
	case kDialogID:
		switch(EnemyTeam->mGetRace())
		{
		case kFederation:
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam ), kFedDialog_msg);
			break;
		case kKlingon:
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kKlingonDialog_msg);
			break;
		case kRomulan:
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kRomulanDialog_msg);
			break;
		case kLyran:
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kLyranDialog_msg);
			break;
		case kHydran:
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kHydranDialog_msg);
			break;
		case kGorn:
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kGornDialog_msg);
			break;
		//case kISC:
		//	break;
		case kMirak:
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kMirakDialog_msg);
			break;
		/*
		case kOrion:
			break;
		case kTholian:
			break;
		case kLDR:
			break;
		case kWYN:
			break;
		case kJindarian:
			break;
			*/

		}
		return;
		break;

#if defined( kRandomChatter )
	case kRandChatID:
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam), kRandChat1_msg );	break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam), kRandChat2_msg );	break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam), kRandChat3_msg );	break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam), kRandChat4_msg );	break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam), kRandChat5_msg );	break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam), kRandChat6_msg );	break;
		}
		kTimerIncrement = ( float )( fMissionInfo->mRandomInt32( 20, 30 ) );
#endif
		default:
			return;
	}
	mSetTimedInterval(	kTimerTurnsSet, timerID, kTimerIncrement);


	
}


void tBaseVictoryState::SendToOrbit(tShipInfo * ship, tShipInfo * target)
{
#ifdef logging
	Log(" SendToOrbit commands");
#endif
	ship->mClearCommands();
#ifdef logging
	Log("Clearing commands.");
#endif

#ifdef logging
	Log(" Got target pointer");
#endif
	ship->mFollowEntity(target, kAbsolutePriority);
#ifdef logging
	Log(" Follow");
#endif
}


void tBaseVictoryState::mCreateDestroyer() {
    gST.mDebug_fprintf("--- Entering: tBaseVictoryState::mCreateDestroyer\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		tTeamInfo* EnemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, EnemyTeam->mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        for (int32 i = kEClassDestroyer; i <= kEClassWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD ship classes filter completed. ships: %d\n", fShiplistCount);

		Ship s1;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			EnemyTeam->mCreateShip(
				typeid( tEnemyShip ),
				s1.shipCode.c_str(),
				kStartPosition_V,
				tEnemyTeam::kNoMetaShipID,
				-1,
				0, 0, -1, -1, NULL,
				kDefaultShipOptions );

        } else {
            mOldCreateDestroyer();
        }
    } else {
        mOldCreateDestroyer();
    }

    gST.mDebug_fprintf("--- Finished: tBaseVictoryState::mCreateDestroyer\n");
}

void tBaseVictoryState::mOldCreateDestroyer() 
{
	gST.mDebug_fprintf("---- Entering: tBaseVictoryState::mOldCreateDestroyer\n");

	tTeamInfo* EnemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	EnemyTeam->mCreateShip(
		typeid( tEnemyShip ),
		kMaxBPV,
		kClassDestroyer,
		kClassDestroyer,
		100,
		kStartPosition_Y,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tBaseVictoryState::mOldCreateDestroyer\n");
}