/********************************************************************************************************\

	File Name:	FreighterBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "FreighterBaseState.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "ISC_X4M2BBomber.h"
#include "AsteroidA.h"
#include "AsteroidB.h"
#include "Freighter.h"

#define kTimerConstant (2.0 / 60)
/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tFreighterBaseState::mBeginState()
{
	mSetTimedInterval(	kTimerTurnsSet, kAsteroidID, 2.0f );
}
void tFreighterBaseState::mEndState()
{}

void tFreighterBaseState::mOnDeath( tShipInfo* ship )
{}
void tFreighterBaseState::mOnCapture( tShipInfo* ship )
{}
void tFreighterBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tFreighterBaseState::mAtDistance( tShipInfo* ship )
{}
void tFreighterBaseState::mAtDestination( tShipInfo* ship )
{}
void tFreighterBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tFreighterBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tFreighterBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tFreighterBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tFreighterBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tFreighterBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tFreighterBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tFreighterBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tFreighterBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tFreighterBaseState::mOnFlashingButtonPressed()
{}
void tFreighterBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tFreighterBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tFreighterBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tFreighterBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tFreighterBaseState::mOnHostChanged()
{}
void tFreighterBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tFreighterBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

void tFreighterBaseState::mOnTimedInterval( int32 timerID )
{

#ifdef logging
Log("void tFreighterBaseState::mOnTimedInterval( int32 timerID )");
#endif

	
	tTeamInfo * PlayerNPCTeam;
	tPlayerTeam * PlayerTeam;
	tTeamInfo * EnemyTeam;
	tShipInfo * freighter;
	tShipInfo * AsteroidA;
	tShipInfo * AsteroidB;
	tBaseVictory * victory;
	char msg[1024];

	PlayerTeam = (tPlayerTeam *)fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert(PlayerTeam);
	PlayerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
	Assert(PlayerNPCTeam);
	freighter = PlayerNPCTeam->mFindShipCustomID(tPlayerNPCTeam::kFreighter);
	Assert(freighter);
	AsteroidA =PlayerNPCTeam->mFindShipCustomID(tPlayerNPCTeam::kAsteroidA);
	AsteroidB =PlayerNPCTeam->mFindShipCustomID(tPlayerNPCTeam::kAsteroidB);
	EnemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	victory = PlayerTeam->mVictoryCondition();


	//*** For some reason, after the freighter is destroyed, and I use a
	//*** SetGameStatus to end the game, the timer kicks back through, 
	//*** and sees the freighter as as having reached the asteroids. So,
	//*** the next line takes care of that.

	if(victory->mFreighterDestroyed())
	{
		return;
	}

	switch( timerID )
	{
	case kAsteroidID:
		if(!victory->mReachedAsteroidA())
		{
			// First set up the comm message values
			if(!victory->mOrbitingAsteroidA())
			{
				victory->mIncAsteroidTimer();
				sprintf(msg, "Communications Officer: The freighter reports %i seconds before the bomb explodes.", 135 - victory->mAsteroidTimer());
				Messages[kUtility_msg] = msg;
				if(GetDistanceFromShipToShip(freighter, AsteroidA) <= 5)
				{
					// Reached Asteroid A. Set flags, display message, start repair timer
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kReachAsteroidA_msg);
					victory->mSetReachedAsteroidA(true);
					victory->mSetOrbitingAsteroidA(true);
					PlayerTeam->mGotoCommState( freighter->mGetShipID(), tFreighter::kCommState2);
					PlayerTeam->mGotoCommState( AsteroidA->mGetShipID(), tAsteroidA::kCommState2);
					mSetTimedInterval(	kTimerTurnsSet, kRepairID, (float)kTimerConstant);
				}
			}
			else
			{
#if !defined ( _GERMAN )
				sprintf(msg, "Communications Officer: The freighter reports it's hull is currently %f percent.", (float)(100 - freighter->mGetDamagePercent()));
#endif
				Messages[kUtility_msg] = msg;
			
				mSetTimedInterval(	kTimerTurnsSet, kAsteroidID, 0.2f );
			}
			// Now check where the timer actually is
			switch( victory->mAsteroidTimer())
			{
			case 75:
				if(!victory->mReachedAsteroidA())
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kAsteroidTimer75_msg);
				}
				// One minute
				break;
			case 105:
				if(!victory->mReachedAsteroidA())
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kAsteroidTimer105_msg);
				}
				// 30 seconds
				break;
			case 135:
				if(!victory->mReachedAsteroidA())
				{
					// freighter go boom
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kAsteroidTimer135_msg);
					victory->mSetFreighterDestroyed(true);
					// The Destroy ship fucntion isn't complete,
					fChildActor->mSetSystemHealthFromOrig( kNoWeaponHardPoint, 0.0f, kFalse );
					if ( !EnemyTeam->mShipsRemainingInPlay() )
						gScriptInterface.mSetGameStatus( (eTeamID) kPlayerTeam, kPlayerLoses);
					return;

				}
				break;
			}

		}
		else
		{
			// Check for Asteroid B instead.
			if(GetDistanceFromShipToShip(freighter, AsteroidB) <= 5)
			{
				// Mission accomplished. If no enemies left, end mission
				// If any left, tell the freighter to get out of town.
				victory->mSetReachedAsteroidB(true);
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kReachAsteroidB_msg);
				if ( !EnemyTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( (eTeamID) kPlayerTeam, kPlayerWins);
				}
				else
				{
					freighter->mDisengage(kByMapEdge, kAbsolutePriority);
					freighter->mSetShipMaxSpeed(31);
				}


				return;

			}
		}
		break;
	case kRepairID:
		victory->mIncRepairTimer ();
		sprintf(msg, "Communications Officer: The freighter reports it's hull is currently %f percent.", (float)(100 - freighter->mGetDamagePercent()));
		Messages[kUtility_msg] = msg;

//		fMissionInfo->mDebugMessage("Repair Timer");

		if(victory->mRepairTimer() == 30)
		{
			
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRepairsMade_msg);


			victory->mSetRepairsMade(true);
			PlayerTeam->mGotoCommState( freighter->mGetShipID(), tFreighter::kCommState3);
			PlayerTeam->mGotoCommState( AsteroidA->mGetShipID(), tAsteroidA::kCommState3);
			return;
		}
		break;
	default:
		return;
		
	}
	mSetTimedInterval(	kTimerTurnsSet, timerID, (float)kTimerConstant);


}

float tFreighterBaseState::GetDistanceFromShipToShip(tShipInfo * startship, tShipInfo * destship)
{
#ifdef logging
	Log(" Enter GetDistanceFromShipToShip");
#endif

	return ( startship->mGetTacticalDistance(destship) );

}

