/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"
#include "AsteroidA.h"
#include "AsteroidB.h"
#include "Freighter.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}


void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        std::list< eEnhancedClassTypes >  ClassTypeList;
        ClassTypeList.push_back(kEClassSmallFreighter);
        ClassTypeList.push_back(kEClassCivilianFreighter);
        ClassTypeList.push_back(kEClassPriorityTransport);
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for small freighter classes filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  RolesList;
        RolesList.push_back(kCargo);
        fShiplistCount = sl.Subset(RolesList, true);
        gST.mDebug_fprintf("subset for cargo eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);

		Ship s1;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            mCreateShip(
                typeid( tFreighter ),
                s1.shipCode.c_str(),
                kStartPosition_G,
                kNoMetaShipID,
                kFreighter,
                0, 0, -1, -1, NULL,
                kDefaultShipOptions );

			gST.mDebug_fprintf("Asteroid bases: OP+ 4.1.1 does not have any for ISC. Just create 'em\n");
			mCreateShip(	typeid( tAsteroidA), "N-AB", kStartPosition_I, kNoMetaShipID,
				kAsteroidA, 0, 0, -1, -1, NULL, kDefaultShipOptions );
			mCreateShip(	typeid( tAsteroidB), "N-AB", kStartPosition_J, kNoMetaShipID,
				kAsteroidB, 0, 0, -1, -1, NULL, kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}



void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tPlayerNPCTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tFreighter), "I-FSPh" , kStartPosition_G, kNoMetaShipID,
					kFreighter, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	mCreateShip(	typeid( tAsteroidA), "N-AB", kStartPosition_I, kNoMetaShipID,
					kAsteroidA, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	mCreateShip(	typeid( tAsteroidB), "N-AB", kStartPosition_J, kNoMetaShipID,
					kAsteroidB, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}

