/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "ISC_X4M2BBomber.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

/*

	HELPER NOTE:	The victory condition must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	tBaseVictory *actor = (tBaseVictory *)fVictoryCondition;
	char msg[255];

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebSuccess );
		if(actor->mEnemyDefeated() == 0 )
		{
			return;
		}
		else if(actor->mEnemyDefeated() == 1)
		{
			sprintf(msg, "In addition, you neutralized an enemy vessel. Fine job.\n\n");
			
		}
		else
		{
			sprintf(msg, "In addition, you neutralized %i enemy vessels. Good work!\n\n", actor->mEnemyDefeated());
		}
		Messages[kUtility_msg] = msg;
		break;

	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, 	kDebNoAsteroidB );
		if(actor->mEnemyDefeated() == 0 )
		{
			return;
		}
		else if(actor->mEnemyDefeated() == 1)
		{
			sprintf(msg, "You did manage to neutralize an enemy vessel, but that is small recompense for the loss of the freighter.\n\n");
			
		}
		else
		{
			sprintf(msg, "At least you neutralized %i enemy vessels. \n\n", actor->mEnemyDefeated());
		}
		Messages[kUtility_msg] = msg;
		break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebNoAsteroidA );
				if(actor->mEnemyDefeated() == 0 )
		{
			return;
		}
		else if(actor->mEnemyDefeated() == 1)
		{
			sprintf(msg, "You did manage to destroy an enemy vessel, but that is small recompense for the loss of the freighter.\n\n");
			
		}
		else
		{
			sprintf(msg, "At least you destroyed %i enemy vessels.\n\n", actor->mEnemyDefeated());
		}
		Messages[kUtility_msg] = msg;
		break;
	}

	if ( victoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDisconnect_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, 	kUtility_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, 	kDebDismissed_msg );
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tPlayerTeam::mCreateShipsForTeam()
{
}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	if ( mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static long index = 0;
	/*****************************************
	** This should handle creating ships,
	** and assigning custom IDs
	*****************************************/

	mCreateShip(typeid( tPlayerShip ),
		shipScriptDescription.fClassName.c_str(),
		kStartPosition_H + shipScriptDescription.fTeamID,
		MetaID,
		kPlayerShip + index, 0, 0, -1, -1,
		shipScriptDescription.fCustomName.c_str(),
		static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	index++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

tBaseVictory * tPlayerTeam::mVictoryCondition(void)
{
	return (tBaseVictory *)fVictoryCondition; 
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 450;
	fMissionScheduler.fNextMissionTitle = "ISC_X4M3BFoxtrot";

	switch ( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
		fMissionScheduler.fMedal = kMedalMissionTwo;
		break;
	}
}