/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "Freighter.h"
#include "MissionText.h"
#include "ISC_X4M2BBomber.h"
#include "ShipList_API.h"

#define kTimerConstant (2.0 / 60)

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tPlayerTeamBaseState::mBeginState()
{
	// Give the ships 10 seconds to get up to speed. The freighter is at 
	// is at max speed 10 anyway.
	mSetTimedInterval( kTimerSecondsSet, kStartID, 10.0f );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if ( deepScanOn )
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kScanning_msg );
}
void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}
void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}
void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}



void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	if (timerID == kStartID) {
		static int32 timerinc = kStart_msg;
		float kTimerIncrement;
		int32 X, Y;

		tTeamInfo * PlayerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
		tShipInfo * freighter = PlayerNPCTeam->mFindShipCustomID(tPlayerNPCTeam::kFreighter);
		tPlayerTeam * PlayerTeam = (tPlayerTeam *) fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		tShipInfo * ship = *(PlayerTeam->mGetFirstShip());

		if(timerinc <= kStart4_msg)
		{
			if( timerinc == kStart_msg )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), timerinc, kNonCommScriptMessage );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), timerinc );
			}
			if(timerinc == kStart4_msg)
			{
//					mSetTimedInterval(	kTimerTurnsSet, kAsteroidID, (float) kTimerConstant);
			}
			timerinc++;
		}
		ship->mGetGridPosition(&X, &Y);

		if(AtEdgeOfMap(X, Y))
		{
			if( !PlayerTeam->mVictoryCondition()->mAtEdgeOfMap() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kLeavingSystem_msg);
				PlayerTeam->mVictoryCondition()->mSetAtEdgeOfMap(true);
			}
		} 
		else 
		{
			if( PlayerTeam->mVictoryCondition()->mAtEdgeOfMap() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kLeavingSystem_msg);
				PlayerTeam->mVictoryCondition()->mSetAtEdgeOfMap(false);
			}
		}

		kTimerIncrement = (float) kTimerConstant * 2;
		mSetTimedInterval(	kTimerTurnsSet, timerID, kTimerIncrement);
	}
		
	if (timerID == kUpgradeFightersID)
	{
#ifdef FIGHTERS
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
		if (playerTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
		if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
		if (playerTeam->mIsTeamHumanControlled()) return;               // only for AI teams
		if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC. 

		gST.mDebug_fprintf("--- Entering tPlayerTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

		int32 BPVinc = gST.mUpgradeFightersForTeam(playerTeam);

		gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc );
		gST.mDebug_fprintf("--- Finished: tPlayerTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
	}
		
	
}

bool tPlayerTeamBaseState::AtEdgeOfMap(int32 X, int32 Y)
{
	int32 GridX;
	int32 GridY;
	gScriptInterface.mGetMapDimensions( &GridX, &GridY);
#ifdef logging
    char msg[255];
	sprintf(msg, "In AtEdgeOfMap GridX: %i GridY: %i X: %i Y: %i", GridX, GridY, X, Y);
	Log(msg);
#endif
	if( (X < 2 || X > (GridX - 1)) || (Y < 2 || Y > (GridY - 1)))
	{
		return true;
	}
	else
	{
		return false;
	}
}

