/********************************************************************************************************\

	File Name:	BaseVictory.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "ScriptInterface.h"
#include "ISC_x4m3bFoxtrot.h"

#include "EnemyTeam.h"
#include "TraitorTeam.h"

tClassAllocator<tBaseVictory> gBaseVictoryAllocator;
 
/*

	HELPER NOTE:	You must remember to initialize all of your victory attributes here.  You should
					do this for both constructors until informed otherwise.  For example:

	tTeam1BaseVictory::tTeam1BaseVictory( ) : fBSed( false ), field2( value ), field3 ( value ), etc...
	{}

*/
tBaseVictory::tBaseVictory()
{
	fTraitorStatus = kStillFighting;
	fCommanderStatus = kStillFighting;
	fCannonStatus = kStillFighting;
	fFodderStatus = kStillFighting;
	fCodeKnown = kStillFighting;
	fTalkedToTraitor = false;
	fFriendlyFire = false;
}
tBaseVictory::tBaseVictory( tTeamInfo* team ) : tVictoryCondition( team )
{
	fTraitorStatus = kStillFighting;
	fCommanderStatus = kStillFighting;
	fCannonStatus = kStillFighting;
	fFodderStatus = kStillFighting;
	fCodeKnown = kStillFighting;
	fTalkedToTraitor = false;
	fFriendlyFire = false;
}

/*

	HELPER NOTE:	Really, the only items in this function that you could change are the values passed into
					the tBaseVictoryState constructor.  These values are the capture bonus and prestige
					points awarded according to the player's victory status -- starting at
					kAstoundingVictory and working its way down to kDevastatingDefeat.

*/
void tBaseVictory::mSetupState()
{
	tBaseVictoryState* baseVictoryState = new tBaseVictoryState( 300, 225, 75, 0, -75 );
	mRegisterState( baseVictoryState );
	mGotoState( typeid( tBaseVictoryState ) );

	//	No chatter this mission.
//	mSetTimedInterval( kTimerSecondsSet, kRandChatID, 20.0f );
}

/*

	HELPER NOTE:	The following is simply an example of setting and modifying the current victory condition
					objectives.

	bool
	tTeam1BaseVictory::mBSed( void )
	{
		return fBSed;
	}
	void
	tTeam1BaseVictory::mSetBSed( bool x )
	{
		fBSed = x;
	}

*/
short int tBaseVictory::mGetShipStatus(int32 shipID)
{
	switch (shipID)
	{
		case tTraitorTeam::kTraitor:
			return fTraitorStatus;
			break;
		case tEnemyTeam::kCommander:
			return fCommanderStatus;
			break;
		case tEnemyTeam::kCannon:
			return fCannonStatus;
			break;
		case tEnemyTeam::kFodder:
			return fFodderStatus;
			break;
		default:
			return kEnemyShipDestroyed;
	}
}
void tBaseVictory::mSetShipStatus(int32 shipID, short int status)
{
	switch (shipID)
	{
		case tTraitorTeam::kTraitor:
			fTraitorStatus = status;
			break;
		case tEnemyTeam::kCommander:
			fCommanderStatus = status;
			if (status == kCaptured)
				fCodeKnown = true;
			break;
		case tEnemyTeam::kCannon:
			fCannonStatus = status;
			break;
		case tEnemyTeam::kFodder:
			fFodderStatus = status;
			break;
	}
}

