/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "ISC_x4m3bFoxtrot.h"
#include "PlayerNPCTeam.h"
#include "EnemyTeam.h"
#include "CommonDefs.h"
#include "EnemyShipCapturedState.h"
#include "PlayerNPCShip.h"
#include "TraitorTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	// Very easy victory conditions here.

	//	If traitor taken out, then it's a victory or better
	//	If Codes are Known, it's astounding (assuming traitor was dispatched)
	//	Otherwise it's a failure.  No draws or anything else.

	int traitorStatus = fChildActor->mGetShipStatus(tTraitorTeam::kTraitor);
	int cdrStatus = fChildActor->mGetShipStatus(tEnemyTeam::kCommander);
	int cannonStatus = fChildActor->mGetShipStatus(tEnemyTeam::kCannon);
	int fodderStatus = fChildActor->mGetShipStatus(tEnemyTeam::kFodder);

	//	Calculate bonus prestige for killing enemy ships.
	if (cdrStatus == tBaseVictory::kEnemyShipCaptured || 
			cdrStatus == tBaseVictory::kEnemyShipDestroyed)
		mAddToBonusPrestige( 25 );

	if (cannonStatus == tBaseVictory::kEnemyShipCaptured || 
			cannonStatus == tBaseVictory::kEnemyShipDestroyed)
		mAddToBonusPrestige( 25 );

	if (fodderStatus == tBaseVictory::kEnemyShipCaptured || 
			fodderStatus == tBaseVictory::kEnemyShipDestroyed)
		mAddToBonusPrestige( 25 );

	if (traitorStatus == tBaseVictory::kEnemyShipCaptured ||
		traitorStatus == tBaseVictory::kEnemyShipDestroyed)
	{
		if (fChildActor->mIsCodeKnown())
			return kAstoundingVictory;
		else
			return kVictory;
	}
	else
		return kDefeat;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	if ( !playerTeam )
		return;

	int32 shipID = ship->mGetCustomID();

	switch ( shipID )
	{
		case tPlayerTeam::kPlayerShip1:
		case tPlayerTeam::kPlayerShip2:
		case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			if (!team)
				return;

			if ( !team->mShipsRemainingInPlay() )
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			break;
		}
		case tTraitorTeam::kTraitor:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kKilledTraitor_msg);
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipDestroyed);

			//	Change Comm State of allies to reflect status of traitor
			tTeamInfo* alliedTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerNPCTeam));
			if (alliedTeam)
			{
				tShipInfo* alliedShip;

				alliedShip = alliedTeam->mFindShipCustomID(tPlayerNPCTeam::kEast);
				if (alliedShip && alliedShip->mInPlay())
					playerTeam->mGotoCommState(alliedShip->mGetShipID(), tPlayerNPCShip::kCommState3);

				alliedShip = alliedTeam->mFindShipCustomID(tPlayerNPCTeam::kWest);
				if (alliedShip && alliedShip->mInPlay())
					playerTeam->mGotoCommState(alliedShip->mGetShipID(), tPlayerNPCShip::kCommState3);
			
				alliedShip = alliedTeam->mFindShipCustomID(tPlayerNPCTeam::kSouth);
				if (alliedShip && alliedShip->mInPlay())
					playerTeam->mGotoCommState(alliedShip->mGetShipID(), tPlayerNPCShip::kCommState3);
			}
			break;
		}
		case tEnemyTeam::kCommander:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kKilledCommander_msg);
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipDestroyed);
			break;
		}
		case tEnemyTeam::kCannon:
		case tEnemyTeam::kFodder:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kKilledEnemy_msg);
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipDestroyed);
			break;
		}
	}

	mCheckVictoryConditions();
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	int32 shipID = ship->mGetCustomID();

	switch ( shipID )
	{
		case tPlayerTeam::kPlayerShip1:
		case tPlayerTeam::kPlayerShip2:
		case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
			break;
		}
		case tTraitorTeam::kTraitor:
		{
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipCaptured);
			ship->mGotoState( typeid( tEnemyShipCapturedState ) );
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapturedTraitor_msg);

			break;
		}
		case tEnemyTeam::kCommander:
		{
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipCaptured);
			ship->mGotoState( typeid( tEnemyShipCapturedState ) );
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapturedCommander_msg);
			break;
		}
		case tEnemyTeam::kCannon:
		case tEnemyTeam::kFodder:
		{
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipCaptured);
			ship->mGotoState( typeid( tEnemyShipCapturedState ) );
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapturedEnemy_msg);
			break;
		}
	}
	mCheckVictoryConditions();
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;


	int32 shipID = ship->mGetCustomID();

	switch ( shipID )
	{
		case tPlayerTeam::kPlayerShip1:
		case tPlayerTeam::kPlayerShip2:
		case tPlayerTeam::kPlayerShip3:
		{
			tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();
			if (playerShip && playerShip == ship)
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLeft_msg);
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
			break;
		}
		case tTraitorTeam::kTraitor:
		{
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipEscaped);
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTraitorEscaped_msg);

			//	Change Comm State of allies to reflect status of traitor
			tTeamInfo* alliedTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerNPCTeam));
			if (alliedTeam)
			{
				tShipInfo* alliedShip;

				alliedShip = alliedTeam->mFindShipCustomID(tPlayerNPCTeam::kEast);
				if (alliedShip && alliedShip->mInPlay())
					playerTeam->mGotoCommState(alliedShip->mGetShipID(), tPlayerNPCShip::kCommState3);

				alliedShip = alliedTeam->mFindShipCustomID(tPlayerNPCTeam::kWest);
				if (alliedShip && alliedShip->mInPlay())
					playerTeam->mGotoCommState(alliedShip->mGetShipID(), tPlayerNPCShip::kCommState3);
			
				alliedShip = alliedTeam->mFindShipCustomID(tPlayerNPCTeam::kSouth);
				if (alliedShip && alliedShip->mInPlay())
					playerTeam->mGotoCommState(alliedShip->mGetShipID(), tPlayerNPCShip::kCommState3);
			}
			break;
		}
		case tEnemyTeam::kCommander:
		case tEnemyTeam::kCannon:
		case tEnemyTeam::kFodder:
		{
			fChildActor->mSetShipStatus(shipID, tBaseVictory::kEnemyShipEscaped);
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kEnemyEscaped_msg);
			break;
		}
	}
	mCheckVictoryConditions();
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if (!targetShip)
		return;

	int32 targetShipID = targetShip->mGetCustomID();
	
	switch (targetShipID)
	{
		default:
			break;
		case tTraitorTeam::kTraitor:
		{
			switch (buttonID)
			{
				case kHail_btn:
				{
					if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kStillFighting)
					{
						if (!fChildActor->mDidTraitorTalk())
						{
							fChildActor->mTraitorTalked();
							fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInitTraitorHail_msg);
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTraitorHail_msg);
						}
					}
					else if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kEnemyShipCaptured)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kHailTraitorCaptured_msg);
					}
					break;
				}
				case kDemandSurrender_btn:
				{
					if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kStillFighting)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponse_msg);
					}
					else if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kEnemyShipCaptured)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAlreadyCaptured_msg);
					}
					break;
				}
			}	//	Bottom of switch for processing Traitor Buttons
			break;
		}
		case tEnemyTeam::kCommander:
		case tEnemyTeam::kCannon:
		case tEnemyTeam::kFodder:
		{
			switch (buttonID)
			{
				case kHail_btn:
				{
					if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kStillFighting)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kEnemyHail_msg);
					}
					else if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kEnemyShipCaptured)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapturedEnemyHail_msg);
					}
					break;
				}
				case kDemandSurrender_btn:
				{
					if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kStillFighting)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponse_msg);
					}
					else if (fChildActor->mGetShipStatus(targetShipID) == tBaseVictory::kEnemyShipCaptured)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAlreadyCaptured_msg);
					}
					break;
				}
			}	//	Bottom of switch for processing Buttons
			break;
		}
		case tPlayerNPCTeam::kEast:
		case tPlayerNPCTeam::kWest:
		case tPlayerNPCTeam::kSouth:
		{
			int damageAmt = (int)(targetShip->mGetDamagePercent());
			switch (buttonID)
			{
				case kHail_btn:
				{
					if (fChildActor->mAttackedAlly())
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoAnswer_msg );
					}
					else if (damageAmt == 0)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPriorityTarg_msg);
					}
					else if (damageAmt < 30)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kGoodShape_msg);
					}
					else if (damageAmt < 60)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNotMyDay_msg);
					}
					else
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMayday_msg);
					}
					break;
				}
				case kRequestAssistance_btn:
				{
					if (fChildActor->mAttackedAlly())
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoAnswer_msg);
					}
					else
					{
						tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
						if (playerTeam) 
						{
							tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();
							if (playerShip)
							{
								tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
								if (enemyTeam)
								{
									tShipInfo* killShip = mGetClosestShipByTeam(playerShip, enemyTeam);
									if (killShip)
										targetShip->mSetShipTarget(killShip, kAbsolutePriority);
								}
							}
						}

						if (damageAmt < 70)
						{
							fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAcknowledged_msg);
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), 
								kTwoWoundedVets_msg + fMissionInfo->mRandomInt32(0,2));
						}
					}
					break;
				}
				case kGetTraitor_btn:
				{
					if (fChildActor->mAttackedAlly())
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), 
							kDieWithBlessing_msg);
					}
					else if (fChildActor->mGetShipStatus(tTraitorTeam::kTraitor) == tBaseVictory::kStillFighting)
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kKillTraitor_msg);

						tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));

						if (enemyTeam)
						{
							tShipInfo* traitorShip = enemyTeam->mFindShipCustomID(tTraitorTeam::kTraitor);
							if (traitorShip)
								targetShip->mSetShipTarget(traitorShip, kMediumPriority);
						}
					}
					else
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTraitorDefeated_msg);
					}
					break;
				}
			}	//	Bottom of Switch statement that handles Allied ship buttons.
			break;
		}	//	Bottom of case for allied ships.
	}	//	Bottom of Switch on targetShip Custom ID

}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if (!damagedShip)
		return;

	switch (damagedShip->mGetCustomID())
	{
		case tPlayerNPCTeam::kEast:
		case tPlayerNPCTeam::kSouth:
		case tPlayerNPCTeam::kWest:
		{
			if (!hostileShip)
				return;

			if (hostileShip->mGetTeam() == kPlayerTeam)
			{
				fChildActor->mSetAttackedAlly();
				//	Disengage the entire Allied Team.
				tTeamInfo* alliedTeam = (fMissionInfo->mGetTeamHandle(damagedShip->mGetTeam()));

				if (alliedTeam)
					alliedTeam->mDisengageShips(kByMapEdge, kHighPriority);

				//	Kill Damage Callback.
				tShipInfo* alliedShip;
				for (int i=tPlayerNPCTeam::kEast; i <= tPlayerNPCTeam::kWest; i++)
				{
					alliedShip = alliedTeam->mFindShipCustomID(i);
					if (alliedShip)
						damagedShip->mSetShipCallbackEvents(!kNotifyDamage);
				}
			}
			else
			{
				damagedShip->mSetShipCallbackEvents( kNotifyDamage );
			}
			break;
		}
		case tTraitorTeam::kTraitor:
		{
			if (fChildActor->mGetShipStatus(tTraitorTeam::kTraitor) != tBaseVictory::kStillFighting)
				return;

			float dmg = damagedShip->mGetDamagePercent();
			if (static_cast<int>(dmg) > 70)
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDisabledTraitor_msg);

				fMissionInfo->mSetTeamRelations(static_cast<eTeamID>(kTraitorTeam), static_cast<eTeamID>(kPlayerTeam), kTeamAllied, kGoalIgnore, 1.0, true);
				fMissionInfo->mSetTeamRelations(static_cast<eTeamID>(kTraitorTeam), static_cast<eTeamID>(kPlayerNPCTeam), kTeamAllied, kGoalIgnore, 1.0, true);

				fChildActor->mSetShipStatus(tTraitorTeam::kTraitor, tBaseVictory::kEnemyShipCaptured);
				damagedShip->mGotoState( typeid( tEnemyShipCapturedState ) );
				damagedShip->mSetShipCallbackEvents( !kNotifyDamage );
				mCheckVictoryConditions();
			}
			else
			{
				damagedShip->mSetShipCallbackEvents( kNotifyDamage );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

//
//	Stuff I added
//
void tBaseVictoryState::mCheckVictoryConditions()
{
	short int traitorStatus = fChildActor->mGetShipStatus(tTraitorTeam::kTraitor);

	if (traitorStatus != tBaseVictory::kStillFighting &&
			fChildActor->mGetShipStatus(tEnemyTeam::kCommander)	!= tBaseVictory::kStillFighting	&&
			fChildActor->mGetShipStatus(tEnemyTeam::kCannon)	!= tBaseVictory::kStillFighting &&
			fChildActor->mGetShipStatus(tEnemyTeam::kFodder)	!= tBaseVictory::kStillFighting)	//	The mission is over.
	{
		if (fChildActor->mIsCodeKnown() && traitorStatus != tBaseVictory::kEnemyShipEscaped)
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllObjectivesAchieved_msg);
			gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam),kPlayerWins);
		}
		else
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllWeCanDoHere_msg);

			if (traitorStatus == tBaseVictory::kEnemyShipEscaped)
				gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam),kPlayerLoses);
			else
				gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam),kPlayerWins);
		}
	}
}