/********************************************************************************************************\

	File Name:	CommonDefs.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "ScriptInterface.h"
#include "Assert.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "EnemyTeam.h"
#include "TraitorTeam.h"
#include "ISC_x4m3bFoxtrot.h"

int gMapCenterX;
int gMapCenterY;
int32 gMapSizeX;
int32 gMapSizeY;
float gAbsoluteMapCenterX;
float gAbsoluteMapCenterY;
float gMapRadiusTacDistance;

void mInitMapVars(int32 mapX, int32 mapY)
{
	gMapSizeX = mapX;
	gMapSizeY = mapY;
	gMapCenterX = (gMapSizeX+1)/2;
	gMapCenterY = (gMapSizeY+1)/2;
	gAbsoluteMapCenterX = (static_cast<float>(gMapSizeX))/2;
	gAbsoluteMapCenterY = (static_cast<float>(gMapSizeY))/2;
	gMapRadiusTacDistance = 4*(min(gMapSizeX, gMapSizeY));
}

tShipInfo* mGetClosestShipByTeam(tShipInfo *ship, tTeamInfo* otherTeam)
{
	tShipInfo* closestShip;
	tShipInfo* currentShip;
	float closestDistance = -1;
	float currentDistance;
	int32 lowShip = 0;
	int32 highShip = 0;

	if (!otherTeam || !ship)
		return NULL;

	//	OK, so this function's not 100% portable.  I had some problems getting
	//	otherTeam->->mGetThisShip(i) to work, so I changed my approach.
	switch (otherTeam->mGetTeam())
	{
		case kPlayerTeam:
			lowShip = tPlayerTeam::kPlayerShip1;
			highShip = tPlayerTeam::kPlayerShip3;
			break;
		case kEnemyTeam:
			lowShip = tEnemyTeam::kCommander;
			highShip = tEnemyTeam::kFodder;
			break;
		case kPlayerNPCTeam:
			lowShip = tPlayerNPCTeam::kEast;
			highShip = tPlayerNPCTeam::kWest;
			break;
		case kTraitorTeam:
			lowShip = tTraitorTeam::kTraitor;
			highShip = tTraitorTeam::kTraitor;
			break;
	}

	//	Look through Team for closest ship.
	for (int32 i = lowShip; i <= highShip; i++)
	{
		currentShip = otherTeam->mFindShipCustomID(i);
		
		if (!currentShip || !currentShip->mInPlay())
			continue;	//	Skip ship if null pointer, or if destroyed, captured, or disengaged.

		currentDistance = ship->mGetTacticalDistance(currentShip);

		if (closestDistance == -1 || currentDistance < closestDistance)
		{
			closestDistance = currentDistance;
			closestShip = currentShip;
		}
	}

	//	Return pointer to closest ship
	if (closestDistance == -1)	// Never found a valid ship on team
		return NULL;
	else
		return closestShip;
}
