/********************************************************************************************************\

	File Name:	EnemyShipRunningState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "TraitorTeam.h"
#include "EnemyShipRunningState.h"
#include "EnemyShipDisengageState.h"
#include "ISC_X4M3BFoxtrot.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "CommonDefs.h"
#include "MissionText.h"
/*
	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.
*/
void tEnemyShipRunningState::mBeginState()
{

	fChildActor->mClearCommands();
	fChildActor->mSetShipMaxSpeed(31);

	mSetRunVector(mGetClosestEnemyShip());

	//	Set distance check from planet.

	//	Set Distance from center of map to go to next state
	fChildActor->mTrackTacticalDistance( RUN_RADIUS, gMapCenterX, gMapCenterY, kTrue); 

	//	Set call to capture damage event.
	if (fChildActor->mGetCustomID() == tTraitorTeam::kTraitor)
		fChildActor->mSetShipCallbackEvents( kNotifyDamage );

	//	Set timer for course correction.
	int32 shipID = fChildActor->mGetCustomID();

	switch (shipID)
	{
		case tEnemyTeam::kCommander:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCourseCorrectionCommander, 
																			COURSE_CORRECTION_INTERVAL);
			break;
		case tEnemyTeam::kCannon:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCourseCorrectionCannon, 
																			COURSE_CORRECTION_INTERVAL);
			break;
		case tEnemyTeam::kFodder:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCourseCorrectionFodder,
																			COURSE_CORRECTION_INTERVAL);
			break;
		case tTraitorTeam::kTraitor:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCourseCorrectionTraitor,
																			COURSE_CORRECTION_INTERVAL);
			break;
	}
}

void tEnemyShipRunningState::mEndState()
{
	if (!fChildActor)
		return;

	int32 shipID = fChildActor->mGetCustomID();
	switch (shipID)
	{
		case tEnemyTeam::kCommander:
			fChildActor->mSetTimedInterval(kTimerKillTimer, 
																			kCourseCorrectionCommander, 
																			0);
			break;
		case tEnemyTeam::kCannon:
			fChildActor->mSetTimedInterval(kTimerKillTimer, 
																			kCourseCorrectionCannon, 
																			0);
			break;
		case tEnemyTeam::kFodder:
			fChildActor->mSetTimedInterval(kTimerKillTimer, 
																			kCourseCorrectionFodder,
																			0);
			break;
		case tTraitorTeam::kTraitor:
			fChildActor->mSetTimedInterval(kTimerKillTimer, 
																			kCourseCorrectionTraitor,
																			0);
			break;
	}
}

void tEnemyShipRunningState::mOnDeath( tShipInfo* ship )
{}
void tEnemyShipRunningState::mOnCapture( tShipInfo* ship )
{}
void tEnemyShipRunningState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemyShipRunningState::mAtDistance( tShipInfo* ship )
{
	if (!ship || ship != fChildActor)
		return;

	//	go to DisengageState.
	fChildActor->mGotoState(typeid(tEnemyShipDisengageState));
}
void tEnemyShipRunningState::mAtDestination( tShipInfo* ship )
{
	if (ship && ship == fChildActor)
		fChildActor->mGotoState(typeid(tEnemyShipDisengageState));
}
void tEnemyShipRunningState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tEnemyShipRunningState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tEnemyShipRunningState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyShipRunningState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyShipRunningState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyShipRunningState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyShipRunningState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyShipRunningState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipRunningState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipRunningState::mOnFlashingButtonPressed()
{}
void tEnemyShipRunningState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyShipRunningState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyShipRunningState::mOnSpecialTransport( tShipInfo* ship )
{}
void tEnemyShipRunningState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tEnemyShipRunningState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tEnemyShipRunningState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tEnemyShipRunningState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tEnemyShipRunningState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tEnemyShipRunningState::mOnHostChanged()
{}
void tEnemyShipRunningState::mOnTimedInterval( int32 timerID )
{
	switch (timerID)
	{
		case kCourseCorrectionCommander:
		{
			if (fChildActor->mGetCustomID() == tEnemyTeam::kCommander) 
			{
				mSetRunVector(mGetClosestEnemyShip());
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCourseCorrectionCommander, 
																				COURSE_CORRECTION_INTERVAL);
			}
			break;
		}
		case kCourseCorrectionCannon:
		{
			if (fChildActor->mGetCustomID() == tEnemyTeam::kCannon) 
			{
				mSetRunVector(mGetClosestEnemyShip());
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCourseCorrectionCannon, 
																				COURSE_CORRECTION_INTERVAL);
			}
			break;
		}
		case kCourseCorrectionFodder:
		{
			if (fChildActor->mGetCustomID() != tEnemyTeam::kFodder) 
			{
				mSetRunVector(mGetClosestEnemyShip());
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCourseCorrectionFodder,
																				COURSE_CORRECTION_INTERVAL);
			}
			break;
		}
		case kCourseCorrectionTraitor:
		{
			if (fChildActor->mGetCustomID() != tTraitorTeam::kTraitor) 
			{
				mSetRunVector(mGetClosestEnemyShip());
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCourseCorrectionTraitor,
																				COURSE_CORRECTION_INTERVAL);
			}
			break;
		}
	}
}
void tEnemyShipRunningState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyShipRunningState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

//	Note: Return value can be null.
tShipInfo* tEnemyShipRunningState::mGetClosestEnemyShip()
{
	tShipInfo* closestPlayerShip = NULL;
	tShipInfo* closestAllyShip = NULL;
	tTeamInfo* currentTeam = NULL;

	//	Look through Player Team for closest ship.
	currentTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam ));
	if (!currentTeam)
		return NULL;

	closestPlayerShip = mGetClosestShipByTeam(fChildActor, currentTeam);

	//	Look through Alliance Team for closest ship.
	currentTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerNPCTeam ));
	if (currentTeam)
		closestAllyShip = mGetClosestShipByTeam(fChildActor, currentTeam);

	//	Return pointer to closest ship
	if (!closestAllyShip ||		//	No Allied Ships Remain
		fChildActor->mGetTacticalDistance(closestPlayerShip) <	// PlayerTeamShip Closer
		fChildActor->mGetTacticalDistance(closestAllyShip))
	{
		return closestPlayerShip;
	}
	else
	{
		return closestAllyShip;
	}
}

//	Note that this can receive a NULL pointer.
//	If it does, it'll just set the ship to run to the nearest
//	border.
void tEnemyShipRunningState::mSetRunVector(tShipInfo* hostileShip )
{
	int32 currentX, currentY;	//	Location of this ship

	int32 hostileX, hostileY;	//	Location of hostile ship
	float hostileShipDistance;
	double hostileVectorX;
	double hostileVectorY;

	int32 distanceToEdgeX, distanceToEdgeY;

	int32 mapEdgeX, mapEdgeY;				//	Location of nearest map exit point.
	int32 altMapEdgeX, altMapEdgeY;	//	Location of 2nd-closest map edge
	float mapEdgeDistance;
	double mapVectorX;
	double mapVectorY;

	int32 runX, runY;					//	Location to run to.
	float runDistance;
	double runVectorX;
	double runVectorY;

	//	Get current location.
	fChildActor->mGetGridPosition(&currentX, &currentY);

	//
	//	Get Closest Exit Point.
	//

	if (currentX * 2 > gMapSizeX) // we're on the right side of map
	{
		distanceToEdgeY = gMapSizeX - currentX;
		mapEdgeX = gMapSizeX; // Assuming X edge is further away than Y...
	}
	else
	{
		distanceToEdgeY = currentX - 1;
		mapEdgeX = 1; // Assuming X edge is further away...
	}

	if (currentY * 2 > gMapSizeY) // we're on the bottom of map
	{
		distanceToEdgeX = gMapSizeY - currentY;
		mapEdgeY = gMapSizeY; // Assuming Y edge is further away...
	}
	else
	{
		distanceToEdgeX = currentY -1;
		mapEdgeY = 1; // Assuming Y edge is further away...
	}

	// Which Edge is Closer?
	if (distanceToEdgeX < distanceToEdgeY) // XEdge Closer
	{
		mapEdgeDistance = distanceToEdgeX*8;
		altMapEdgeX = mapEdgeX;
		altMapEdgeY = currentY;
		mapEdgeX = currentX;
	}
	else // YEdge Closer (or equidistant)
	{
		mapEdgeDistance = distanceToEdgeY*8;
		altMapEdgeX = currentX;
		altMapEdgeY = mapEdgeY;
		mapEdgeY = currentY;
	}

	// mapEdgeX and mapEdgeY are now the closest map edge point.

	//	Get coordinates of hostileShip.
	if (hostileShip)
		hostileShip->mGetGridPosition(&hostileX, &hostileY);

	// If hostile ship is in the same grid, then all
	// angles relating to it are meaningless, so we'll just use the 
	// direct-to-edge-of-map vector.
	if (!hostileShip || (hostileX == currentX && hostileY == currentY))
	{
		//	Note: Here's where a NULL pointer value passed in will
		//	get handled, setting runX and runY to the closest map edge.
		runX = mapEdgeX;
		runY = mapEdgeY;
	}
	else
	{
		//	Get distance to hostileShip.
		hostileShipDistance = fChildActor->mGetTacticalDistance(hostileShip);

		//	Get Angle to hostileShip.
		mgetAngle(currentX, currentY, hostileX, hostileY, hostileShipDistance, &hostileVectorX, &hostileVectorY);
		
		//	Get angle to map edge point.
		mgetAngle(currentX, currentY, mapEdgeX, mapEdgeY, mapEdgeDistance, &mapVectorX, &mapVectorY);

		//	Special Case:
		//	If hostile Angle and map Angle are basically the same, then
		//	have the ship pick the 2nd closest map edge.
		//	We're checking to 2 sig digits.
		if (static_cast<int>(100*hostileVectorX) == (static_cast<int>(100*mapVectorX)) &&
				static_cast<int>(100*hostileVectorY) == (static_cast<int>(100*mapVectorY)))
		{
			runX = altMapEdgeX;
			runY = altMapEdgeY;
		}
		else
		{
			//	OK... special cases taken care of.

			//	We want to figure out the half-split between the
			//	converse of hostileVector and the mapVector

			//	Flip hostileVector 180
			hostileVectorX = -hostileVectorX;
			hostileVectorY = -hostileVectorY;

			runVectorX = hostileVectorX - ((hostileVectorX - mapVectorX)/2);
			runVectorY = hostileVectorY - ((hostileVectorY - mapVectorY)/2);

			//	Get largest distance between map edge and hostileShip
			runDistance = max(hostileShipDistance, mapEdgeDistance);

			//	Multiply by run distance, and then normalize for map grid units.
			runX = static_cast<int>((runVectorX * runDistance)/8);
			runY = static_cast<int>((runVectorY * runDistance)/8);


			//	runX/runY are at this point just offsets from 
			//	where the ship currently is.
			runX = currentX + runX;
			runY = currentY + runY;
		}
	}

	//	Boundaries Checking 
	//	this may set the ship to go directly to a map edge if
	//	hostileShip is further away than the map edge.
	runX = min(gMapSizeX, max(1, runX));
	runY = min(gMapSizeY, max(1, runY));

	//	Set ship to go to Running Coordinates.
	fChildActor->mSetFinalDestination(runX, runY);
}

void tEnemyShipRunningState::mgetAngle(int32 currentX, int32 currentY, int32 distantX, int32 distantY, float distance, double *angleX, double *angleY)
{
	int32 mapGridDistance;
	mapGridDistance = static_cast<int>((distance+4)/8);

	if (mapGridDistance == 0)
	{
		*angleX = 0.00;
		*angleY = 0.00;
	}
	else
	{
		*angleX = static_cast<double>(distantX - currentX);
		*angleX /= static_cast<double>(mapGridDistance);

		*angleY = static_cast<double>(distantY - currentY);
		*angleY /= static_cast<double>(mapGridDistance);
	}
}