/********************************************************************************************************\

	File Name:	EnemyShipScatterState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyShipScatterState.h"
#include "EnemyShipRunningState.h"
#include "EnemyShipDisengageState.h"
#include "ISC_X4M3BFoxtrot.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "EnemyTeam.h"
#include "CommonDefs.h"
#include "TraitorTeam.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tEnemyShipScatterState::mBeginState()
{
	double vectorX, vectorY;
	int orderDecision;
	int32 orderSelection;
	int32 runX, runY;

	fChildActor->mClearCommands();
	fChildActor->mSetShipMaxSpeed(20);	// To give players a chance.

	//	Set Distance from center of map to go to second state
	fChildActor->mTrackTacticalDistance( SCATTER_RADIUS, gMapCenterX, gMapCenterY, kTrue);

	//	Set Timer to check enemy ship distances.
	int32 shipID = fChildActor->mGetCustomID();

	switch (shipID)
	{
		case tEnemyTeam::kCommander:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCheckDistanceToEnemyCommander, 
																			DISTANCE_CHECK_INTERVAL);
			break;
		case tEnemyTeam::kCannon:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCheckDistanceToEnemyCannon, 
																			DISTANCE_CHECK_INTERVAL);
			break;
		case tEnemyTeam::kFodder:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCheckDistanceToEnemyFodder,
																			DISTANCE_CHECK_INTERVAL);
			break;
		case tTraitorTeam::kTraitor:
			fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																			kCheckDistanceToEnemyTraitor,
																			DISTANCE_CHECK_INTERVAL);
			break;
	}

	//	Capture interrupt for being fired upon.
	if (fChildActor->mGetCustomID() == tTraitorTeam::kTraitor)
		fChildActor->mSetShipCallbackEvents( kNotifyDamage );


	//	Pick a Start Vector, and head in that direction.
	// Decide how well the subordinate ships will 
	// obey the orders given...
	if (shipID == tEnemyTeam::kCannon || shipID == tEnemyTeam::kFodder)
	{
		orderSelection = fMissionInfo->mRandomInt32(1,100);
		switch (fChildActor->mGetRace())
		{
			default:
			case kFederation:
				if (orderSelection <= 2)
					orderDecision = kBanzai;
				else if (orderSelection <= 6)
					orderDecision = kBeardLion;
				else if (orderSelection <= 21)
					orderDecision = kGloryOrders;
				else if (orderSelection <= 88)
					orderDecision = kOrders;
				else if (orderSelection <= 96)
					orderDecision = kCowardOrders;
				else if (orderSelection <= 97)
					orderDecision = kCoward;
				else
					orderDecision = kBrave;
				break;
			case kKlingon:
				if (orderSelection <= 5)
					orderDecision = kBanzai;
				else if (orderSelection <= 22)
					orderDecision = kBeardLion;
				else if (orderSelection <= 55)
					orderDecision = kGloryOrders;
				else if (orderSelection <= 85)
					orderDecision = kOrders;
				else if (orderSelection <= 89)
					orderDecision = kCowardOrders;
				else if (orderSelection <= 90)
					orderDecision = kCoward;
				else
					orderDecision = kBrave;
				break;
			case kRomulan:
				if (orderSelection <= 2)
					orderDecision = kBanzai;
				else if (orderSelection <= 7)
					orderDecision = kBeardLion;
				else if (orderSelection <= 20)
					orderDecision = kGloryOrders;
				else if (orderSelection <= 50)
					orderDecision = kOrders;
				else if (orderSelection <= 70)
					orderDecision = kCowardOrders;
				else if (orderSelection <= 85)
					orderDecision = kCoward;
				else
					orderDecision = kBrave;
				break;
			case kLyran:
				if (orderSelection <= 10)
					orderDecision = kBanzai;
				else if (orderSelection <= 20)
					orderDecision = kBeardLion;
				else if (orderSelection <= 40)
					orderDecision = kGloryOrders;
				else if (orderSelection <= 80)
					orderDecision = kOrders;
				else if (orderSelection <= 87)
					orderDecision = kCowardOrders;
				else if (orderSelection <= 90)
					orderDecision = kCoward;
				else
					orderDecision = kBrave;
				break;
			case kGorn:
				if (orderSelection <= 4)
					orderDecision = kBanzai;
				else if (orderSelection <= 11)
					orderDecision = kBeardLion;
				else if (orderSelection <= 21)
					orderDecision = kGloryOrders;
				else if (orderSelection <= 81)
					orderDecision = kOrders;
				else if (orderSelection <= 91)
					orderDecision = kCowardOrders;
				else if (orderSelection <= 93)
					orderDecision = kCoward;
				else
					orderDecision = kBrave;
				break;
			case kHydran:
				if (orderSelection <= 7)
					orderDecision = kBanzai;
				else if (orderSelection <= 10)
					orderDecision = kBeardLion;
				else if (orderSelection <= 25)
					orderDecision = kGloryOrders;
				else if (orderSelection <= 80)
					orderDecision = kOrders;
				else if (orderSelection <= 90)
					orderDecision = kCowardOrders;
				else if (orderSelection <= 93)
					orderDecision = kCoward;
				else
					orderDecision = kBrave;
				break;
			case kMirak:
				if (orderSelection <= 10)
					orderDecision = kBanzai;
				else if (orderSelection <= 25)
					orderDecision = kBeardLion;
				else if (orderSelection <= 40)
					orderDecision = kGloryOrders;
				else if (orderSelection <= 80)
					orderDecision = kOrders;
				else if (orderSelection <= 85)
					orderDecision = kCowardOrders;
				else if (orderSelection <= 90)
					orderDecision = kCoward;
				else
					orderDecision = kBrave;
				break;
		}	//	End of switch on race
	}	//	End of if statement on shipID == kCannon or kFodder

	//	Now get the specific coordinates of the Scatter Vector.
	switch (shipID)
	{
		case tTraitorTeam::kTraitor:	//	225 from north
			vectorX = -0.707;
			vectorY = -0.707;
			break;
		case tEnemyTeam::kCommander:	//	135 from north (315 trig, as it bases 0 at due "East", and goes counter-clockwise)
			vectorX =  0.707;
			vectorY = -0.707;
			break;
		case tEnemyTeam::kCannon:	//	This is the ship to the north-west of planet
			switch (orderDecision)
			{
				case kBanzai:	//	0 (90 trig)
					vectorX =  0.000;
					vectorY =  1.000;
					break;
				case kBeardLion:	//	345/105
					vectorX = -0.259;
					vectorY =  0.966;
					break;
				case kGloryOrders:	//	330 / 120
					vectorX = -0.500;
					vectorY =  0.866;
					break;
				case kOrders:	//	315 / 135
					vectorX = -0.707;
					vectorY =  0.707;
					break;
				case kCowardOrders:	//	300 / 150
					vectorX = -0.866;
					vectorY =  0.500;
					break;
				case kCoward:	//	285 / 165
					vectorX = -0.966;
					vectorY =  0.259;
					break;
				case kBrave:	// 270 / 180
					vectorX = -1.000;
					vectorY =  0.000;
					break;
			}
			break;
		case tEnemyTeam::kFodder:	//	This is the ship to the north-east of planet
			switch (orderDecision)
			{
				case kBanzai:	//	0 (90 trig)
					vectorX =  0.000;
					vectorY =  1.000;
					break;
				case kBeardLion:	//	15 / 75
					vectorX =  0.259;
					vectorY =  0.966;
					break;
				case kGloryOrders:	//	30 / 60
					vectorX =  0.500;
					vectorY =  0.866;
					break;
				case kOrders:	//	45 / 45
					vectorX =  0.707;
					vectorY =  0.707;
					break;
				case kCowardOrders:	//	60 / 30
					vectorX =  0.866;
					vectorY =  0.500;
					break;
				case kCoward:	//	75 / 15
					vectorX =  0.966;
					vectorY =  0.259;
					break;
				case kBrave:	// 90 / 0
					vectorX =  1.000;
					vectorY =  0.000;
					break;
			}
			break;
	}	//	End of switch on custom ship ID

	//	Vectors are based on trig coords, which means
	//	that they assume the center of the map is 0,0, when it's really
	//	12,12 or somesuch value.
	//
	//	Need to convert values.

	//	Use non-truncated map center values here.
	runX = static_cast<int>(gAbsoluteMapCenterX + (vectorX*gAbsoluteMapCenterX));
	runY = static_cast<int>(gAbsoluteMapCenterY - (vectorY*gAbsoluteMapCenterY));

	fChildActor->mSetFinalDestination( runX, runY);
}
void tEnemyShipScatterState::mEndState()
{
	switch (fChildActor->mGetCustomID())
	{
		case tEnemyTeam::kCommander:
			fChildActor->mSetTimedInterval(kTimerKillTimer, kCheckDistanceToEnemyCommander, 0);
			break;
		case tEnemyTeam::kCannon:
			fChildActor->mSetTimedInterval(kTimerKillTimer, kCheckDistanceToEnemyCannon, 0);
			break;
		case tEnemyTeam::kFodder:
			fChildActor->mSetTimedInterval(kTimerKillTimer, kCheckDistanceToEnemyFodder, 0);
			break;
		case tTraitorTeam::kTraitor:
			fChildActor->mSetTimedInterval(kTimerKillTimer, kCheckDistanceToEnemyTraitor, 0);
			break;
	}
}
void tEnemyShipScatterState::mOnDeath( tShipInfo* ship )
{}
void tEnemyShipScatterState::mOnCapture( tShipInfo* ship )
{}
void tEnemyShipScatterState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemyShipScatterState::mAtDistance( tShipInfo* ship )
{
	if (!ship || ship != fChildActor) 
		return;

	fChildActor->mGotoState(typeid(tEnemyShipScatterState));
}
void tEnemyShipScatterState::mAtDestination( tShipInfo* ship )
{
	if (!ship || ship != fChildActor) 
		return;

	//	go to Running State
	fChildActor->mGotoState(typeid(tEnemyShipRunningState));
}
void tEnemyShipScatterState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tEnemyShipScatterState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	//	go to Running State
//	if (!damagedShip || damagedShip != fChildActor) 
//		return;

//	fChildActor->mGotoState(typeid(tEnemyShipRunningState));
}
void tEnemyShipScatterState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyShipScatterState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyShipScatterState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyShipScatterState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyShipScatterState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyShipScatterState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipScatterState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyShipScatterState::mOnFlashingButtonPressed()
{}
void tEnemyShipScatterState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyShipScatterState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyShipScatterState::mOnSpecialTransport( tShipInfo* ship )
{}
void tEnemyShipScatterState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tEnemyShipScatterState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tEnemyShipScatterState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tEnemyShipScatterState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tEnemyShipScatterState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tEnemyShipScatterState::mOnHostChanged()
{}
void tEnemyShipScatterState::mOnTimedInterval( int32 timerID )
{
	const int dangerDistance = 15;
	bool tooClose = false;
	tTeamInfo* currentTeam;
	tShipInfo* currentShip;

	//	Dang thing fires off for every ship, no matter what ship sets the timer.
	switch (timerID)
	{
		case kCheckDistanceToEnemyCommander:
			if (fChildActor->mGetCustomID() != tEnemyTeam::kCommander) 
				return;
			break;
		case kCheckDistanceToEnemyCannon:
			if (fChildActor->mGetCustomID() != tEnemyTeam::kCannon) 
				return;
			break;
		case kCheckDistanceToEnemyFodder:
			if (fChildActor->mGetCustomID() != tEnemyTeam::kFodder) 
				return;
			break;
		case kCheckDistanceToEnemyTraitor:
			if (fChildActor->mGetCustomID() != tTraitorTeam::kTraitor) 
				return;
			break;
	}

	//	Check distance of enemy ships.

	//	Look through Player Team for closest ship.
	currentTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam ));

	if (currentTeam)
	{
		currentShip = mGetClosestShipByTeam(fChildActor, currentTeam);
		if (currentShip && fChildActor->mGetTacticalDistance(currentShip) < dangerDistance)
			tooClose = true;
	}

	if (!tooClose)
	{
		//	Look through Alliance Team for closest ship.
		currentTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerNPCTeam ));
		if (currentTeam)
		{
			currentShip = mGetClosestShipByTeam(fChildActor, currentTeam);
			if (currentShip && fChildActor->mGetTacticalDistance(currentShip) < dangerDistance)
				tooClose = true;
		}
	}

	//	If any are too close, go to RunningState
	if (tooClose)
		fChildActor->mGotoState(typeid(tEnemyShipRunningState));
	else	//	Reset Timer to check enemy ship distances.
	{
		switch (fChildActor->mGetCustomID())
		{
			case tEnemyTeam::kCommander:
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCheckDistanceToEnemyCommander, 
																				DISTANCE_CHECK_INTERVAL);
				break;
			case tEnemyTeam::kCannon:
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCheckDistanceToEnemyCannon, 
																				DISTANCE_CHECK_INTERVAL);
				break;
			case tEnemyTeam::kFodder:
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCheckDistanceToEnemyFodder,
																				DISTANCE_CHECK_INTERVAL);
				break;
			case tTraitorTeam::kTraitor:
				fChildActor->mSetTimedInterval(kTimerTurnsSet, 
																				kCheckDistanceToEnemyTraitor,
																				DISTANCE_CHECK_INTERVAL);
				break;
		}
	}
}
void tEnemyShipScatterState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyShipScatterState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}
