/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"
#include "CommonDefs.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "ISC_X4M3BFoxtrot.h"
#include "EnemyShip.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}


void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips( sl );

        std::list< eEnhancedClassTypes >  ClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyBattlecruiser; i++) {
            ClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);


        if (sl.size() > 0) {
			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam) );
			int32 baseBPV = playerTeam->mGetCombatBPV();

			switch (fMissionInfo->mGetDifficultyLevel())
			{
				case 0:		// Novice
					baseBPV = 0.75;
					break;
				case 1:		// Intermediate
				default:	// Unknown Setting
					break;
				case 2:		// Expert
					baseBPV *= 1.25;
					break;
			}

			for (int i = 0; kCommander + i <= kFodder;i++) {
				int32 BPV = (int)(baseBPV/(i==0 ? 4 : 5)); // baseBPV *.25 for Commander, baseBPV * .2 for others

				ShipList this_sl = sl;
				fShiplistCount = this_sl.Subset( this_sl.kSubsetBPV, int(0.9 * BPV), int(1.1 * BPV), true);
				gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
					int(0.9 * BPV), int(1.1 * BPV), fShiplistCount);

				Ship s1;
				sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				mCreateShip(
					typeid( tEnemyShip ),
					s1.shipCode.c_str(),
					kStartPosition_L + i,
					kNoMetaShipID,
					kCommander + i,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
			}
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam) );
	int32 baseBPV = playerTeam->mGetCombatBPV();

	switch (fMissionInfo->mGetDifficultyLevel())
	{
		case 0:		// Novice
			baseBPV = 0.75;
			break;
		case 1:		// Intermediate
		default:	// Unknown Setting
			break;
		case 2:		// Expert
			baseBPV *= 1.25;
			break;
	}

	// Create Enemy Ships
	for (int i = 0; kCommander + i <= kFodder;i++)
	{
		mCreateShip(typeid(tEnemyShip),
						(int)(baseBPV/(i==0 ? 4 : 5)), // baseBPV *.25 for Commander, baseBPV * .2 for others
						kClassLightCruiser,
						kClassHeavyBattlecruiser,
						15,
						kStartPosition_L + i,
						kCommander + i,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );
	}

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}


void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}


void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}


