/********************************************************************************************************\

	File Name:	EnemyTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyTeamBaseState.h"
#include "EnemyShipScatterState.h"
#include "EnemyTeam.h"
#include "TraitorTeam.h"
#include "ISC_x4m3bFoxtrot.h"
#include "ShipList_API.h"

void tEnemyTeamBaseState::mBeginState()
{
	//	Set Timer before starting to run.
	//	Base amount of time on mission difficulty.

	//	For play purposes, I'm going to have the ships hang out before scattering.
	switch (fMissionInfo->mGetDifficultyLevel())
	{
		case 0:		//	Novice
			fChildActor->mSetTimedInterval(kTimerTurnsSet, kInitScatter, 2.0f);
			break;
		case 1:		//	Intermediate
			fChildActor->mSetTimedInterval(kTimerTurnsSet, kInitScatter, 1.5f);
			break;
		case 2:		//	Expert
			fChildActor->mSetTimedInterval(kTimerTurnsSet, kInitScatter, 1.0f);
			break;
	}

	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}
void tEnemyTeamBaseState::mEndState()
{}
void tEnemyTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tEnemyTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tEnemyTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnFlashingButtonPressed()
{}
void tEnemyTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tEnemyTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tEnemyTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tEnemyTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tEnemyTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tEnemyTeamBaseState::mOnHostChanged()
{}
void tEnemyTeamBaseState::mOnTimedInterval( int32 timerID )
{
	//	Go to Scatter State
	if (timerID == kInitScatter) {
		//	Set team ships to new state.
		tShipInfo* currShip = NULL;
		for (int i = tEnemyTeam::kCommander; i <= tEnemyTeam::kFodder; i++)
		{
			currShip = fChildActor->mFindShipCustomID(i);
			if (currShip && currShip->mInPlay())
				currShip->mGotoState(typeid(tEnemyShipScatterState));
		}

		//	Set Traitor Team ship to new state.
		//	(no sense both teams having identical timers)
		tTeamInfo* traitorTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >(kTraitorTeam) );
		if (traitorTeam)
		{
			currShip = traitorTeam->mFindShipCustomID(tTraitorTeam::kTraitor);
			if (currShip && currShip->mInPlay())
				currShip->mGotoState(typeid(tEnemyShipScatterState));
		}
	}

	if (timerID == kUpgradeFightersID)
	{
#ifdef FIGHTERS
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kEnemyTeam ) );
		if (playerTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
		if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
		if (playerTeam->mIsTeamHumanControlled()) return;               // only for AI teams
		if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC. 

		gST.mDebug_fprintf("--- Entering tEnemyTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

		int32 BPVinc = gST.mUpgradeFightersForTeam(playerTeam);

		gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc );
		gST.mDebug_fprintf("--- Finished: tEnemyTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
	}

}
void tEnemyTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}
