/********************************************************************************************************\

	Mission:	<MISSION TITLE>
	Script:		"ISC_x4m3bFoxtrot"
	Descript:	<DESCRIPTION>

	Created by:		<NAME>		mm/dd/yy
	Last Updated:	Richard Glover
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "ISC_x4m3bFoxtrot.h"
#include "EnemyTeam.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "TraitorTeam.h"
#include "DraftTeam.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tISC_x4m3bFoxtrot );

tISC_x4m3bFoxtrot::tISC_x4m3bFoxtrot()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tISC_x4m3bFoxtrot::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player NPC Team",
												static_cast< eTeamID >( kPlayerNPCTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kSameAs | kPlayerTeam,
												typeid( tPlayerNPCTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",			//	Team Name
												static_cast< eTeamID >( kEnemyTeam ),	//	TeamID
												kMaxChance,																	//	Chance Available
												kTrue,																//	Required
												kPlayableTeam,															//	Team Type
												kMaxChance,
#ifdef OP
												kSpecNonPirateRace & kSpecPlayableRace & ~kSpecISC,
#else
												kSpecPlayableRace & ~kSpecISC,
#endif
												kAnyEnemyOf | kPlayerTeam,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Traitor Team",			//	Team Name
												static_cast< eTeamID >( kTraitorTeam ),	//	TeamID
												kMaxChance,																	//	Chance Available
												kTrue,																//	Required
												kPlayableTeam,															//	Team Type
												kMaxChance,																	//	Chance to be played by AI
												kSpecISC,
												kAnyRace,
												typeid( tTraitorTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(   "Team 5", 
												static_cast< eTeamID>( kDraftTeam1 ),
                                                kMaxChance,
                                                kTrue,
                                                kPlayableTeam,
                                                kMaxChance,
                                                kSpecAnyRace,
                                                kAnyRace,
                                                typeid( tDraftTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(   "Team 6", 
												static_cast< eTeamID>( kDraftTeam2 ),
                                                kMaxChance,
                                                kTrue,
                                                kPlayableTeam,
                                                kMaxChance,
                                                kSpecAnyRace,
                                                kAnyRace,
                                                typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tISC_x4m3bFoxtrot::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tISC_x4m3bFoxtrot::mCreateExternalShips( int32 Mode )
{}

/*
	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h
*/
int32 tISC_x4m3bFoxtrot::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tISC_x4m3bFoxtrot::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tISC_x4m3bFoxtrot::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kTraitorTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,	(Allied Ships)
		//Nme  = kEnemyTeam,	(Enemy Ships)
		//Trtr = kTraitorTeam,

		//	     Plyr  PNPC  Nme   Trtr
		/*Plyr*/ "--", "AI", "WD", "WD",
		/*PNPC*/ "AI", "--", "WD", "WD",
		/*Nme */ "WI", "WI", "--", "AI",
		/*Trtr*/ "WI", "WI", "AI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
void tISC_x4m3bFoxtrot::mInitializeDraw( const int32 MapPicked )
{
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tISC_x4m3bFoxtrot::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['K'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['L'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['M'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['N'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tISC_x4m3bFoxtrot::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kISC] = kBriefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tISC_x4m3bFoxtrot::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}