/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"
#include "CommonDefs.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "PlayerNPCShip.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}


void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShips( sl );

        std::list< eEnhancedClassTypes >  ClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassNewHeavyCruiser; i++) {
            ClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for CA classes filter completed. ships: %d\n", fShiplistCount);

		// 150 BPV-ish drafts
		fShiplistCount = sl.Subset( sl.kSubsetBPV, int(0.9 * 150), int(1.1 * 150), true);
		gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
			int(0.9 * 150), int(1.1 * 150), fShiplistCount);


		Ship s1, s2, s3;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

            mCreateShip(
                typeid( tPlayerNPCShip ),
                s1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kEast,
                0, 0, -1, -1, NULL,
                kDefaultShipOptions );

			mCreateShip(
                typeid( tPlayerNPCShip ),
                s1.shipCode.c_str(),
                kStartPosition_I,
                kNoMetaShipID,
                kSouth,
                0, 0, -1, -1, NULL,
                kDefaultShipOptions );

			mCreateShip(
                typeid( tPlayerNPCShip ),
                s1.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kWest,
                0, 0, -1, -1, NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tPlayerNPCTeam::mOldCreateShipsForTeam\n");

	// Create "East" Ally
	mCreateShip(typeid( tPlayerNPCShip ), 
					150,
					kClassNewHeavyCruiser,
					kClassHeavyCruiser,
					20,
					kStartPosition_H,
					kEast,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
	
	// Create "South" Ally
	mCreateShip(typeid( tPlayerNPCShip ), 
					150,
					kClassNewHeavyCruiser,
					kClassHeavyCruiser,
					20,
					kStartPosition_I,
					kSouth,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

	// Create "West" Ally
	mCreateShip(typeid( tPlayerNPCShip ), 
					150,
					kClassNewHeavyCruiser,
					kClassHeavyCruiser,
					20,
					kStartPosition_J,
					kWest,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}

void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}


