/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "ISC_x4m3bFoxtrot.h"
#include "MissionText.h"
#include "CommonDefs.h"
#include "EnemyTeam.h"
#include "TraitorTeam.h"
/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tPlayerShipBaseState::mBeginState()
{
	fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kStart_msg, kNonCommScriptMessage );

	//	Since the player ship won't ever change state, this is OK to set here.
	fTraitorScanned = false;
	fCommanderScanned = false;
	fCannonScanned = false;
	fFodderScanned = false;
	fScanning = false;
	fGotCloseToPlanet = false;

	fChildActor->mSetShipCallbackEvents( kNotifyDeepScan );

	//	One last thing...  Set a distance check for player ships
	//	to give "approaching map edge" message...

	// Set up distance notification for edge of system...
	fChildActor->mTrackTacticalDistance( PLAYER_IN_SYSTEM_RADIUS, gMapCenterX , gMapCenterY);

}
void tPlayerShipBaseState::mEndState()
{}
void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{
	if (!ship)
		return;

	//	If it's this ship, and this ship is the one that's currently
	//	under player control...
	if (ship == fChildActor)
	{
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));

		if (playerTeam)
		{
			tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();

			if (playerShip && playerShip == ship)
			{
				if (fGotCloseToPlanet)
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLeaving_msg);
				}
				else	//	Player is far enough from edge so we'll notify that he's leaving if he gets close again.
				{
					fGotCloseToPlanet = true;
					// Set up distance notification for edge of system...
					fChildActor->mTrackTacticalDistance( PLAYER_LEAVING_RADIUS, gMapCenterX , gMapCenterY, kTrue);
				}
			}
		}
	}
}
void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if (!ship)
		return;

	if (ship == fChildActor)
	{
		//	Turning scan off
		if (!deepScanOn)
		{
			//	Still within scanning distance but not scanned.
			if (fScanning)
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanMovedAway_msg);

			fChildActor->mSetTimedInterval(kTimerKillTimer, kScannerDelay, 0);
			fChildActor->mSetTimedInterval(kTimerKillTimer, kScannerCheck, 0);
		}
		else	//	Turning Scan on.
		{
			if (!mCheckForScanTarget())
			{
				//	Nearest Target Valid, but too far away.
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanningTooFar_msg);
			}
		}
	}
}
void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}
void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch (timerID)
	{
		default:
			return;
			break;
		case kScannerCheck:	//	Look for a valid scan target.
			if (fChildActor->mGetDeepScanStatus())
				mCheckForScanTarget();	//	Automatically restarts kScannerCheck if req'd.
			break;
		case kScannerDelay:
		{
			//	In case player has turned scan off
			if (!fScanning)
				return;

			tTeamInfo* scanTargetTeam = NULL;
			switch (fScanTargetID)
			{
			case tTraitorTeam::kTraitor:
				scanTargetTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTraitorTeam));
				break;
			default:
				scanTargetTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
				break;
			}

			if (scanTargetTeam)
			{
				tShipInfo* scanTargetShip = scanTargetTeam->mFindShipCustomID(fScanTargetID);

				if (scanTargetShip && scanTargetShip->mInPlay() && !mTargetScanned())
				{
					if (fChildActor->mGetTacticalDistance(scanTargetShip) <= SHIP_SCAN_DISTANCE)
					{
						switch (fScanTargetID)
						{
							case tTraitorTeam::kTraitor:
								fTraitorScanned = true;
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanningTraitor_msg);
								break;
							case tEnemyTeam::kCommander:
								fCommanderScanned = true;
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanningCommander_msg);
								break;
							case tEnemyTeam::kCannon:
								fCannonScanned = true;
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanOther_msg);
								break;
							case tEnemyTeam::kFodder:
								fFodderScanned = true;
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanOther_msg);
								break;
						}	//	End of targetShipID switch
					}
					else	//	Moved too far away...
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanningTooFar_msg);
					}
				}
			}
			fScanning = false;
			mCheckForScanTarget();	//	Automatically restarts kScannerCheck if req'd.
			break;
		}	//	End of kScannerDelay case
	}	//	End of switch on timerID
}
void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

iTruth tPlayerShipBaseState::mTargetScanned(void)
{
	switch (fScanTargetID)
	{
		case tTraitorTeam::kTraitor:
			return fTraitorScanned;
		case tEnemyTeam::kCommander:
			return fCommanderScanned;
		case tEnemyTeam::kCannon:
			return fCannonScanned;
		case tEnemyTeam::kFodder:
			return fFodderScanned;
		default:
			return true;	//	We don't want to scan any other targets
	}
}

//	This returns false if the nearest target is valid, but
//	too far away to start scanning.
//	It returns true otherwise.
iTruth tPlayerShipBaseState::mCheckForScanTarget(void)
{
	if (fScanning) 
		return true;

	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kEnemyTeam));
	tTeamInfo* traitorTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTraitorTeam));
	tShipInfo* traitorShip = NULL;
	tShipInfo* enemyShip = NULL;
	tShipInfo* scanTargetShip = NULL;
	
	if (traitorTeam)
	{
		traitorShip = traitorTeam->mFindShipCustomID(tTraitorTeam::kTraitor);
	}

	if (enemyTeam)
	{
		enemyShip = mGetClosestShipByTeam(fChildActor, enemyTeam);
	}

	if (!traitorShip)
	{
		if (!enemyShip)
			return true;
		else
			scanTargetShip = enemyShip;
	}
	else
	{
		if (!enemyShip)
				scanTargetShip = traitorShip;
		else
		{
			//	Traitor Ship and an enemy Ship exist, so find the closest
			//	between the two.
			if (traitorShip->mGetTacticalDistance(fChildActor) <= 
					enemyShip->mGetTacticalDistance(fChildActor))
			{
				scanTargetShip = traitorShip;
			}
			else
			{
				scanTargetShip = enemyShip;
			}
		}
	}	

	fScanTargetID = scanTargetShip->mGetCustomID();
	//	fScanTargetID now has closest ship from either team.

	if (!mTargetScanned())
	{
		if (fChildActor->mGetTacticalDistance(scanTargetShip) <= SHIP_SCAN_DISTANCE)
		{
			fScanning = true;
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kStartScan_msg);
			//	This sets the delay before information is returned from the scan.
			fChildActor->mSetTimedInterval(kTimerTurnsSet, kScannerDelay, SCANNER_DELAY );
			return true;
		}
		else	//	Valid Target, but too far to scan.
		{
			//	ScannerCheck periodically checks to see if the closest target can be scanned.
			fChildActor->mSetTimedInterval(kTimerTurnsSet, kScannerCheck, SCANNER_CHECK );
			return false;
		}
	}
	else	//	Closest Ship not valid scan target... look for another later
	{
		//	ScannerCheck periodically checks to see if the closest target can be scanned.
		fChildActor->mSetTimedInterval(kTimerTurnsSet, kScannerCheck, SCANNER_CHECK );
		return true;
	}
}
