/********************************************************************************************************\

	File Name:	TraitorTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"
#include "TraitorTeam.h"
#include "TraitorTeamBaseState.h"
#include "TraitorShip.h"


tClassAllocator<tTraitorTeam> gTraitorTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tTraitorTeam::mSetupState()
{
	mRegisterState( new tTraitorTeamBaseState() );
	mGotoState( typeid( tTraitorTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tTraitorTeam::mSetupCommState()
{}


void tTraitorTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tTraitorTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        std::list< eEnhancedRoles >  warRolesList;
        warRolesList.push_back(kLineShip);
        warRolesList.push_back(kWarShip);
        warRolesList.push_back(kLeader);
        warRolesList.push_back(kFastCruiser);
        fShiplistCount = sl.Subset(warRolesList);
        gST.mDebug_fprintf("subset for base war classes eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedClassTypes >  ClassTypeList;
        for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++) {
            ClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(ClassTypeList, true);
        gST.mDebug_fprintf("subset for CA classes filter completed. ships: %d\n", fShiplistCount);


		Ship s1;
        if (sl.size() > 0) {
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            mCreateShip(
                typeid( tTraitorShip ),
                s1.shipCode.c_str(),
                kStartPosition_K,
                kNoMetaShipID,
                kTraitor,
                0, 0, -1, -1, NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tTraitorTeam::mCreateShipsForTeam\n");
}


void tTraitorTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tTraitorTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tTraitorShip), "I-CA", kStartPosition_K, kNoMetaShipID,
					kTraitor, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tTraitorTeam::mOldCreateShipsForTeam\n");
}


void tTraitorTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tTraitorTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tTraitorTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
	gST.mInitializeSL();
	gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
   	gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
		shipScriptDescription.fClassName.c_str());

   	gST.mDebug_fprintf("--- Finished: tTraitorTeam::mCreateNewShip: %s\n", 
		shipScriptDescription.fClassName.c_str());
}

