/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "sfcAssert.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "Met_10Patrol.h"


class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	fOrigNumAttackTeams = 0;
	fOrigNumDefendTeams = 0;
	fNumAttackTeams = 0;
	fNumDefendTeams = 0;
	fNumAttackShips = 0;	//The total number of attack ships at game start;
	fRemainingAttackShips = 0;	//The current count of Attack Ships
	fNumDefendShips = 0;
	fRemainingDefendShips = 0;
	fOrionTeamInGame = false;

	fNumTeams = 6;	//Hard coded max number of teams

	for( int32 i = 0; i < 21; ++i )
	{
		fAttackShipsDamageChart[i] = 0;
		fDefendShipsDamageChart[i] = 0;
	}
}
bool tWorldConditions::mGetOrionTeamInGame()
{
	return fOrionTeamInGame;
}
void tWorldConditions::mSetOrionTeamInGame( bool inGame )
{
	fOrionTeamInGame = inGame;
}
void tWorldConditions::mSetDefendDamageCond( int32 index, int32 condition )
{
	fDefendShipsDamageChart[index] = condition;
}
int32 tWorldConditions::mGetDefendDamageCond( int32 index )
{
	return fDefendShipsDamageChart[index];
}
void tWorldConditions::mSetAttackDamageCond( int32 index, int32 condition )
{
	fAttackShipsDamageChart[index] = condition;
}
int32 tWorldConditions::mGetAttackDamageCond( int32 index )
{
	return fAttackShipsDamageChart[index];
}
void tWorldConditions::mAddAttackTeam()
{
	++fNumAttackTeams;
	++fOrigNumAttackTeams;
}
void tWorldConditions::mRemoveAttackTeam()
{
	--fNumAttackTeams;
}
int32 tWorldConditions::mGetAttackTeams()
{
	return fNumAttackTeams;
}
void tWorldConditions::mAddDefendTeam()
{
	++fNumDefendTeams;
	++fOrigNumDefendTeams;
}
void tWorldConditions::mRemoveDefendTeam()
{
	--fNumDefendTeams;
}
int32 tWorldConditions::mGetDefendTeams()
{
	return fNumDefendTeams;
}
void tWorldConditions::mAddDefendShip()
{
	++fNumDefendShips;
	++fRemainingDefendShips;
}
void tWorldConditions::mRemoveDefendShip()
{
	--fRemainingDefendShips;
}
int32 tWorldConditions::mGetNumDefendShips()
{
	return fNumDefendShips;
}
int32 tWorldConditions::mGetRemainingDefendShips()
{
	return fRemainingDefendShips;
}
void tWorldConditions::mAddAttackShip()
{
	++fNumAttackShips;
	++fRemainingAttackShips;
}
int32 tWorldConditions::mGetNumAttackShips()
{
	return fNumAttackShips;
}
void tWorldConditions::mRemoveAttackShip()
{
	--fRemainingAttackShips;
}
int32 tWorldConditions::mGetRemainingAttackShips()
{
	return fRemainingAttackShips;
}
void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo* thisMission = gScriptInterface.mMission();

	fNumAttackTeams = 0;
	fNumDefendTeams = 0;
	fOrionTeamInGame = false;

	tTeamInfo* defendTeam1 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam1 ) );
	tTeamInfo* defendTeam2 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam2 ) );
	tTeamInfo* defendTeam3 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam3 ) );

	tTeamInfo* attackTeam1 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kAttackTeam1 ) );
	tTeamInfo* attackTeam2 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kAttackTeam2 ) );
	tTeamInfo* attackTeam3 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kAttackTeam3 ) );

	tTeamInfo* orionTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( kOrionTeam ) );
		
	if( defendTeam1 && defendTeam1->mShipsRemainingInPlay() )
	{
		++fNumDefendTeams;
	}
	if( defendTeam2 && defendTeam2->mShipsRemainingInPlay() )
	{
		++fNumDefendTeams;
	}
	if( defendTeam3 && defendTeam3->mShipsRemainingInPlay() )
	{
		++fNumDefendTeams;
	}

	if( attackTeam1 && attackTeam1->mShipsRemainingInPlay() )
	{
		++fNumAttackTeams;
	}	
	if( attackTeam2 && attackTeam2->mShipsRemainingInPlay() )
	{
		++fNumAttackTeams;
	}	
	if( attackTeam3 && attackTeam3->mShipsRemainingInPlay() )
	{
		++fNumAttackTeams;
	}
	
	if( orionTeam && orionTeam->mShipsRemainingInPlay() )
	{
		fOrionTeamInGame = true;
	}
	
	for ( int i = 0; i < fNumTeams; i++ )
	{
		tMissionInfo*	thisMission = gScriptInterface.mMission();

		tTeamInfo* currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );

		if ( !currentTeam )
			continue;
		
		
		if( fNumAttackTeams < 1 && fOrionTeamInGame == false )
		{
			if( ( i == 0 ) || ( i == 2 ) | ( i == 3 ) )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
			}
		}
		else if( fNumDefendTeams < 1 && fOrionTeamInGame == false )
		{
			if( ( i == 0 ) || ( i == 2 ) | ( i == 3 ) )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
			}
		}
		else if( fNumDefendTeams == 0 && fNumAttackTeams == 0  )
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
		}
	
	}
}
