/********************************************************************************************************\

	Mission:	Convoy Raid
	Script:		"Met_11ConvoyRaid"
	Descript:	<DESCRIPTION>

	Created by:		David F		06/29/00
	Last Updated:	DF
					converted from Q-ship suprise
					DJS			10/08/00
					Reversed the order in which victory condition is checked when a freighter is
					destroyed and other freighters have already disengaged.

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Met_11ConvoyRaid.h"
#include "DefendTeam.h"
#include "AttackTeam.h"
#include "FreighterTeam.h"
#include "QShipTeam.h"
#include "WorldConditions.h"

DEFINE_SCRIPT_CLASS( tMet_11ConvoyRaid );

tMet_11ConvoyRaid::tMet_11ConvoyRaid()
{
}
void tMet_11ConvoyRaid::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kNeutralRaceHex | kEnemyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( kHalfBand | k3Client );
	Specs->fGenericFlags = kNoGenericFlags | kRequiresBase | kRequiresPlanet;


	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
																		kMaxChance,
																		kTrue,
																		kPrimaryTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyRace,
																		typeid( tAttackTeam ),
																		kNoTeamTag,
																		Messages[kPlayerAttackerTeamTitle_msg] );

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
																		kMaxChance,
																		kTrue,
																		kPrimaryOpponentTeam,
																		kMaxChance,
																		kSpecPlayableRace,
																		kAnyEnemyOf | kAttackTeam1,
																		typeid( tDefendTeam ),
																		kNoTeamTag,
																		Messages[kPlayerDefenderTeamTitle_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
																		kMaxChance,
																		kFalse,
																		kPlayableTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyAllyOf | kAttackTeam1,
																		typeid( tAttackTeam ),
																		kNoTeamTag,
																		Messages[kPlayerAttackerTeamTitle_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Attack Team3", static_cast< eTeamID>( kAttackTeam3 ),
																		kMaxChance,
																		kFalse,
																		kPlayableTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyAllyOf | kAttackTeam1,
																		typeid( tAttackTeam ),
																		kNoTeamTag,
																		Messages[kPlayerAttackerTeamTitle_msg] );


	const tTeamSpec& team5 = mCreateTeamSpec(	"Freighter Team", static_cast< eTeamID>( kFreighterTeam ),
																		kMaxChance,
																		kTrue,
																		kNPCTeam,
																		kMaxChance,
																		kSpecPlayableRace,
																		kSameAs | kDefendTeam1,
																		typeid( tFreighterTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"QShip Team", static_cast< eTeamID>( kQShipTeam ),
																		kMaxChance,
																		kTrue,
																		kNPCTeam,
																		kMaxChance,
																		kSpecPlayableRace,
																		kSameAs | kDefendTeam1,
																		typeid( tQShipTeam ) );
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );

}
int32 tMet_11ConvoyRaid::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_11ConvoyRaid::mCreateExternalShips( int32 Mode )
{
}
int32 tMet_11ConvoyRaid::mInitializeFinished( void )
{
	return kAttackBriefing_msg;
}
void tMet_11ConvoyRaid::mNewInternalShip( tShipInfo* Ship )
{
}
void tMet_11ConvoyRaid::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kAttackTeam2,	//Atk2
		kAttackTeam3,	//Atk3
		kFreighterTeam,	//Frtr
		kQShipTeam,		//QShp
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Atk2	Atk3	Frtr	Qshp
		/*Def1*/	"--",	"WD",	"WD",	"WD",	"AI",	"AI",
		/*Atk1*/	"WI",	"--",	"AI",	"AI",	"WD",	"WI",
		/*Atk2*/	"WI",	"AI",	"--",	"AI",	"WD",	"WI",
		/*Atk3*/	"WI",	"AI",	"AI",	"--",	"WD",	"WI",
		/*Frtr*/	"AI",	"WI",	"WI",	"WI",	"--",	"AI",
		/*QShp*/	"AI",	"WI",	"WI",	"WI",	"AI",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

void tMet_11ConvoyRaid::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{
	tScript::mNewTeam( teamScriptDescription );	
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_11ConvoyRaid::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_11ConvoyRaid::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['X'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['L'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['M'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['N'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_11ConvoyRaid::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.0f );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.4f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
}
void tMet_11ConvoyRaid::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{

		fBriefingIndex[kDefendTeam1][j] = kDefendBriefing_msg;
		fBriefingIndex[kAttackTeam1][j] = kAttackBriefing_msg;
		fBriefingIndex[kAttackTeam2][j] = kAttackBriefing_msg;
		fBriefingIndex[kAttackTeam3][j] = kAttackBriefing_msg;

	}
}