/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "sfcAssert.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "Met_11ConvoyRaid.h"

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	fOrigNumAttackTeams = 0;
	fOrigNumDefendTeams = 0;
	fNumAttackTeams = 0;
	fNumDefendTeams = 0;
	fNumTeams = 6;	//Hard coded max number of teams
	fOrigNumbOfFreighters = 0;	//Number of Freighters at start of game
	fCurrNumbOfFreighters = 0;	//Number of Freighters currently in the game
	fEscpNumbOfFreighters = 0;	//Number of Freighters that have disengaged
	fDestNumbOfFreighters = 0;	//Number of Freighters destroyed
	fNumbOfQShips = 0;	//Number of Q-Ships in game 1 or 2
	fNumbRemainingQShips = 0;
	fAttackerHumanTeam = false;
	fFrtrSizeDice = 0;
}
void  tWorldConditions::mSetFrtrSizeDice( int32 setValue )
{
	fFrtrSizeDice = setValue;
}
int32 tWorldConditions::mGetFrtrSizeDice()
{
	return fFrtrSizeDice;
}
bool tWorldConditions::mIsAttackerHuman()
{
	return	fAttackerHumanTeam;
}
void tWorldConditions::mSetHumanAttacker()
{
	fAttackerHumanTeam = true;
}
void tWorldConditions::mRemoveQShip()
{
	--fNumbRemainingQShips;
}
int32 tWorldConditions::mGetNumQShips()
{
	return fNumbOfQShips;
}
void tWorldConditions::mSetNumQShips( int32 setValue )
{
	fNumbOfQShips = setValue;
	fNumbRemainingQShips = setValue;
}
int32 tWorldConditions::mGetFreightersEscaped()
{
	return fEscpNumbOfFreighters;
}
int32 tWorldConditions::mGetFreightersOriginal()
{
	return fOrigNumbOfFreighters;
}
int32 tWorldConditions::mGetFreightersDestroyed()
{
	return fDestNumbOfFreighters;
}
void tWorldConditions::mAddFreighter()
{
	++fOrigNumbOfFreighters;
	++fCurrNumbOfFreighters;
}
void tWorldConditions::mRemoveFreighter()
{
	--fCurrNumbOfFreighters;
}
void tWorldConditions::mFreighterEscaped()
{
	--fCurrNumbOfFreighters;
	++fEscpNumbOfFreighters;
}
void tWorldConditions::mFreighterDestroyed()
{
	--fCurrNumbOfFreighters;
	++fDestNumbOfFreighters;
}
void tWorldConditions::mAddAttackTeam()
{
	++fNumAttackTeams;
	++fOrigNumAttackTeams;
}
void tWorldConditions::mRemoveAttackTeam()
{
	--fNumAttackTeams;
}
int32 tWorldConditions::mGetAttackTeams()
{
	return fNumAttackTeams;
}
void tWorldConditions::mAddDefendTeam()
{
	++fNumDefendTeams;
	++fOrigNumDefendTeams;
}
void tWorldConditions::mRemoveDefendTeam()
{
	--fNumDefendTeams;
}
int32 tWorldConditions::mGetDefendTeams()
{
	return fNumDefendTeams;
}
void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo* thisMission = gScriptInterface.mMission();

	fNumAttackTeams = 0;
	fNumDefendTeams = 0;
	int32 fCurrNumbOfFreighters = 0;
	fNumbRemainingQShips = 0;

	tTeamInfo* defendTeam1 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam1 ) );

	tTeamInfo* attackTeam1 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kAttackTeam1 ) );
	tTeamInfo* attackTeam2 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kAttackTeam2 ) );
	tTeamInfo* attackTeam3 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kAttackTeam3 ) );

	tTeamInfo* freighterTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( kFreighterTeam ) );
	tTeamInfo* qShipTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( kQShipTeam ) );
		
	if( defendTeam1 && defendTeam1->mShipsRemainingInPlay() )
	{
		++fNumDefendTeams;
	}

	if( attackTeam1 && attackTeam1->mShipsRemainingInPlay() )
	{
		++fNumAttackTeams;
	}	
	if( attackTeam2 && attackTeam2->mShipsRemainingInPlay() )
	{
		++fNumAttackTeams;
	}	
	if( attackTeam3 && attackTeam3->mShipsRemainingInPlay() )
	{
		++fNumAttackTeams;
	}
	
	if( freighterTeam )
	{
		fCurrNumbOfFreighters = freighterTeam->mShipsRemainingInPlay();
	}
	if( qShipTeam )
	{
		fNumbRemainingQShips = qShipTeam->mShipsRemainingInPlay();
	}


	for ( int i = 0; i < fNumTeams; i++ )
	{
		tMissionInfo*	thisMission = gScriptInterface.mMission();

		tTeamInfo* currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );

		if ( !currentTeam )
			continue;
		
		if( ( fNumAttackTeams < 1 ) || 
			( ( fNumDefendTeams < 1 ) && ( ( ( fCurrNumbOfFreighters < 1) && ( fNumbRemainingQShips < 1) ) || (! fAttackerHumanTeam ) ) ) )
		{		
			int32 destroyedFreighters = gWC.mGetFreightersDestroyed();
			int32 attackTeamsRemaining = gWC.mGetAttackTeams();
			int32 escapedFreighters = gWC.mGetFreightersEscaped();
			int32 originalNumberFreighters = gWC.mGetFreightersOriginal();

			if( attackTeamsRemaining > 0 )
			{
				destroyedFreighters = originalNumberFreighters - escapedFreighters;
			}
			
			if( destroyedFreighters < 2 )
			{
				if( i == 0 )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
				}
				else
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
				}
			}
			else
			{
				if( i == 0 )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
				}
				else
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
				}
			}
		}
	}
}
