//	<Name>	DefendVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "DefendVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_12ConvoyEscort.h"
#include "DefendVictoryCondition.h"
#include "MissionText.h"
#include "DefendTeam.h"
//#include "TeamScouted.h"
#include "WorldConditions.h"

tDefendVictoryState::tDefendVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tDefendVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	attackTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam1 ) );
	attackTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam2 ) );
	attackTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam3 ) );
			
	defendTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam1 ) );
	freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	qShipTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kQShipTeam ) );
	
	attackBPV = 0;
	defendBPV = 0;

	fThisTeamShipsInPlay = 1;
	fPlayerLeftAlready = false;
	fThisPlayerDead = false;
}

void tDefendVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kEndingID, 10.0f );
	
	if( attackTeam1 )
	{
		attackBPV += attackTeam1->mGetCombatBPV();
	}
	if( attackTeam2 )
	{
		attackBPV += attackTeam2->mGetCombatBPV();
	}
	if( attackTeam3 )
	{
		attackBPV += attackTeam3->mGetCombatBPV();
	}

	if( defendTeam1 )
	{
		defendBPV += defendTeam1->mGetCombatBPV();
	}
	if( freighterTeam )
	{
		defendBPV += freighterTeam->mGetCombatBPV();
	}
	if( qShipTeam )
	{
		defendBPV += qShipTeam->mGetCombatBPV();
	}
}

void tDefendVictoryState::mEndState()
{
}

eVictoryLevels tDefendVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 destroyedFreighters = gWC.mGetFreightersDestroyed();
	int32 attackTeamsRemaining = gWC.mGetAttackTeams();
	int32 escapedFreighters = gWC.mGetFreightersEscaped();
	int32 originalNumberFreighters = gWC.mGetFreightersOriginal();

	if( attackTeamsRemaining > 0 )
	{
		destroyedFreighters = originalNumberFreighters - escapedFreighters;
	}

	if( ! fThisPlayerDead 
		&& ( ( destroyedFreighters < ( originalNumberFreighters / 2 ) ) || ( attackBPV > defendBPV * 2 ) ) )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 50 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 125 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
	}
	if( fThisPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if( destroyedFreighters == originalNumberFreighters )
	{ 
		if( attackBPV < defendBPV )
		{
			mAddToPrestige( 125 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		return kDevastatingDefeat;
	}
	else if( destroyedFreighters  >  ( originalNumberFreighters / 2 ) )
	{
		return kDefeat;
	}
	else if( destroyedFreighters == 1 )
	{
		return kVictory;
	}
	else if( destroyedFreighters == 0 )
	{
		return kAstoundingVictory;
	}
	else
	{
		return kDraw;
	}
}
int32 tDefendVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tDefendVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetOriginalTeam();


	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	if( fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
		fThisPlayerDead = true;
		gWC.mRemoveDefendTeam();
		fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
		gWC.mCheckForGameEnd();
	}

}
void tDefendVictoryState::mOnDeath( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}
void tDefendVictoryState::mOnCapture( tShipInfo* ship )
{
	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToBonusPrestige( 90 );
	}

	mCheckAllTeams( ship );
}

void tDefendVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveDefendTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
//				fTeamDisengaged = true;
				gWC.mCheckForGameEnd();
			}
		}
	}
}
void tDefendVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tDefendVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	}
}
void tDefendVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveDefendTeam();
		fPlayerLeftAlready = true;
		gWC.mCheckForGameEnd();
	}
}