/********************************************************************************************************\

	File Name:	DefendTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Met_13Monster.h"
#include "DefendTeam.h"
#include "DefendTeamInitialState.h"
#include "DefendTeamShip.h"
#include "ShipScriptDescription.h"
#include "DefendVictoryCondition.h"
#include "MissionText.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

tClassAllocator<tDefendTeam> gDefendTeamAllocator;

tDefendTeam::tDefendTeam()
{
	gWC.mAddDefendTeam();
	fDice = fMissionInfo->mRandomInt32( 0,2 );
}
void tDefendTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tDefendVictoryCondition( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( victoryLevel == kAstoundingVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendAstoundingVictory_msg );
	} 
	else if( victoryLevel == kVictory )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendVictory_msg );
	} 
	else if( victoryLevel == kDraw )
	{
		int32 monstersDestroyed = gWC.mGetNumbMonstersDestroyed();
		int32 originalMonsters = gWC.mGetNumbMonstersOriginal();
		bool isItAPlanetKiller = gWC.mGetIsAPlanetKiller();

		if( monstersDestroyed == originalMonsters || ( isItAPlanetKiller && monstersDestroyed > 0 ) )
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefendAstoundingVictory_msg );
		}
		else
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefendDefeat_msg );
		}
	} 
	else if( victoryLevel == kDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDefeat_msg );
	} 
	else if( victoryLevel == kDevastatingDefeat )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDefendDevastatingDefeat_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
	}
}

void tDefendTeam::mSetupState()
{
	mRegisterState( new tDefendTeamInitialState() );

	mGotoState( typeid( tDefendTeamInitialState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tDefendTeam::mSetupCommState()
{
}

void tDefendTeam::mCreateShipsForTeam()
{
}
void tDefendTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}
/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
		mCreateShip( typeid( tDefendTeamShip ),
						shipScriptDescription.fClassName.c_str(),
						kStartPosition_G + fDice,
						shipScriptDescription.fMetaDatabaseID,
						-1,
						0,
						0,
						-1,
						-1,
						shipScriptDescription.fCustomName.c_str(),
						static_cast< eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );
						
}
void tDefendTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 0;
	fMissionScheduler.fNextMissionTitle = "";
}