//	<Name>	DefendVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "DefendVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_13Monster.h"
#include "DefendVictoryCondition.h"
#include "MissionText.h"
#include "DefendTeam.h"
#include "WorldConditions.h"

tDefendVictoryState::tDefendVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tDefendVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fThisTeamShipsInPlay = 1;
	fPlayerLeftAlready = false;
	fThisPlayerDead = false;
	fTeamDisengaged = false;

	defendBPV =0;
	monsterBPV = 0;

	defendTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam1 ) );
	defendTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam2 ) );
	defendTeam3 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam3 ) );

	monsterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMonsterTeam ) );
}

void tDefendVictoryState::mBeginState()
{
	if( defendTeam1 )
	{
		defendBPV += defendTeam1->mGetCombatBPV();
	}
	if( defendTeam2 )
	{
		defendBPV += defendTeam2->mGetCombatBPV();
	}
	if( defendTeam3 )
	{
		defendBPV += defendTeam3->mGetCombatBPV();
	}
	if( monsterTeam )
	{
		monsterBPV += monsterTeam->mGetCombatBPV();
	}
}

void tDefendVictoryState::mEndState()
{
}

eVictoryLevels tDefendVictoryState::mCalcVictoryStatus( eTeamID ID )
{

	int32 monstersDestroyed = gWC.mGetNumbMonstersDestroyed();
	int32 originalMonsters = gWC.mGetNumbMonstersOriginal();
	bool isItAPlanetKiller = gWC.mGetIsAPlanetKiller();

	if ( fTeamDisengaged )
	{
		return kDefeat;
	}

	if( isItAPlanetKiller )
	{
		if( monstersDestroyed > 0 )
		{
			return kAstoundingVictory;
			mAddToBonusPrestige( 1500 );
		}
		else
		{
			return kDefeat;
		}
	}

	if(	fThisPlayerDead ) 
	{
		return kDevastatingDefeat;
	}
	if( monstersDestroyed == originalMonsters )
	{
		if( ! fThisPlayerDead  )
		{
			if( monsterBPV >= defendBPV )
			{
				mAddToPrestige( 50 * ( ( (float)monsterBPV / (float)defendBPV ) - 1.0f ) );
			}
			else
			{
				mAddToPrestige( 125 * ( ( ( float)monsterBPV / (float)defendBPV ) - 1.0f ) );
			}
		}

		mAddToPrestige( 213 );
		return kDraw;
	}
	else
	{
		return kDraw;
	}
}
int32 tDefendVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tDefendVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetOriginalTeam();


	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	if( fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
		fThisPlayerDead = true;
		gWC.mRemoveDefendTeam();
		fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
		gWC.mCheckForGameEnd();
	}

}
void tDefendVictoryState::mOnDeath( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}
void tDefendVictoryState::mOnCapture( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}

void tDefendVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveDefendTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
				fTeamDisengaged = true;
				gWC.mCheckForGameEnd();
			}
		}
	}
}
void tDefendVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tDefendVictoryState::mOnTimedInterval( int32 timerID )
{
}
void tDefendVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveDefendTeam();
		fPlayerLeftAlready = true;
		gWC.mCheckForGameEnd();
	}
}