/********************************************************************************************************\

	Mission:	Met_13Monster
	Script:		"Met_13Monster"
	Descript:	<DESCRIPTION>

	Created by:		<NAME>			mm/dd/yy
	Last Updated:	<INITIALS>		mm/dd/yy
					<LOG>


\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "DefendTeam2.h"
#include "DefendTeam3.h"
#include "MonsterTeam.h"
#include "WorldConditions.h"

#include "Met_13Monster.h"
#include "VictoryCondition.h"


DEFINE_SCRIPT_CLASS( tMet_13Monster );

tMet_13Monster::tMet_13Monster()
{
}
/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tMet_13Monster::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kPlayerRaceHex | kAlliedRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 6;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k3Band | k3Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase | kRequiresPlanet;

	/*

		HELPER NOTE:	In this case, there are 6 teams setup.  Feel free to change the number of teams
						by deleting them starting with team6 and working your way down to team3, inclusive.
						Also, if the team is to be controlled by the AI and only the AI, then pass the
						constant 'kNPCTeam' instead of 'kPlayableTeam' into the team specification.  Also,
						not noted here, the last parameter of mCreateTeamSpec() can be a team tag such as
						"kTagA" to "kTagF".  This is the default tag for a kNPCTeam that was not chosen
						on the multiplayer UI screen.

	*/

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ), kMaxChance, kTrue,
																		kPrimaryTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyRace,
																		typeid( tDefendTeam ),
																		kNoTeamTag,
																		Messages[kPlayerTeamDefender_msg] );
	
	const tTeamSpec& team2 = mCreateTeamSpec(	"Defend Team2", static_cast< eTeamID>( kDefendTeam2 ), kMaxChance, kFalse,
																		kPlayableTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyAllyOf | kDefendTeam1,
																		typeid( tDefendTeam2 ),
																		kNoTeamTag,
																		Messages[kPlayerTeamDefender_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team3", static_cast< eTeamID>( kDefendTeam3 ), kMaxChance, kFalse,
																		kPlayableTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyAllyOf | kDefendTeam1,
																		typeid( tDefendTeam3 ),
																		kNoTeamTag,
																		Messages[kPlayerTeamDefender_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Monster Team", static_cast< eTeamID>( kMonsterTeam ), kMaxChance, kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecMonster,
																		kAnyRace,
																		typeid( tMonsterTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMet_13Monster::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tMet_13Monster::mCreateExternalShips( int32 Mode )
{
}
/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMet_13Monster::mInitializeFinished( void )
{
	return kBriefing_msg;
}
/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMet_13Monster::mNewInternalShip( tShipInfo* Ship )
{
}
/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tMet_13Monster::mGetDebriefingList( eVictoryLevels VictoryLevel )
{
}
int32 tMet_13Monster::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMet_13Monster::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kDefendTeam2,	//Def2
		kDefendTeam3,	//Def3
		kMonsterTeam,	//Mon
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Def2	Def3	Mon
		/*Def1*/	"--",	"AI",	"AI",	"WD",
		/*Def2*/	"AI",	"--",	"AI",	"WD",
		/*Def3*/	"AI",	"AI",	"--",	"WD",
		/*Mon */	"WD",	"WD",	"WD",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}
tMapTerrain tMet_13Monster::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 4:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_13Monster::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
}
void tMet_13Monster::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, 300 );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, 300 );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, kMinBPV, 300 );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, 200 );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, 200 );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, kMinBPV, 200 );

	mSetTeamShipStrength( kMonsterTeam, kClassMonster, kClassMonster, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kMonsterTeam, kClassMonster, kClassMonster, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kMonsterTeam, kClassMonster, kClassMonster, kMinBPV, kMaxBPV );
}
void tMet_13Monster::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		fBriefingIndex[kDefendTeam1][j] = kBriefing_msg;
		fBriefingIndex[kDefendTeam2][j] = kBriefing_msg;
		fBriefingIndex[kDefendTeam3][j] = kBriefing_msg;
	}
}