/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "sfcAssert.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "Met_13Monster.h"

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	fOrigNumDefendTeams = 0;
	fNumDefendTeams = 0;
	fOrigNumbOfMonsters = 0;	//Number of Monsters at start of game
	fCurrNumbOfMonsters = 0;	//Number of Monsters currently in the game
	fTotalMonstersDestroyed = 0;
	fNumMonstersEscaped = 0;
	fNumTeams = 6;	//Hard coded max number of teams
	fIsAPlanetKiller = false;
}
bool tWorldConditions::mGetIsAPlanetKiller()
{
	return fIsAPlanetKiller;
}
int32 tWorldConditions::mGetNumbMonstersEscaped()
{
	return fNumMonstersEscaped;
}
int32 tWorldConditions::mGetNumbMonstersDestroyed()
{
	return fTotalMonstersDestroyed;
}
int32 tWorldConditions::mGetNumbMonstersOriginal()
{
	return fOrigNumbOfMonsters;
}
int32 tWorldConditions::mGetDefendTeams()
{
	return fNumDefendTeams;
}
int32 tWorldConditions::mGetNumbOrigDefendTeams()
{
	return fOrigNumDefendTeams;
}

void tWorldConditions::mPlanetKillerTrue()
{
	fIsAPlanetKiller = true;
}
void tWorldConditions::mAddDefendTeam()
{
	++fNumDefendTeams;
	++fOrigNumDefendTeams;
}
void tWorldConditions::mRemoveDefendTeam()
{
	--fNumDefendTeams;
}
void tWorldConditions::mAddMonster()
{
	++fOrigNumbOfMonsters;
	++fCurrNumbOfMonsters;
}
void tWorldConditions::mRemoveMonster()
{
	--fCurrNumbOfMonsters;
}
void tWorldConditions::mMonsterEscaped()
{
	++fNumMonstersEscaped;
}
void tWorldConditions::mMonsterDestroyed()
{
	--fCurrNumbOfMonsters;
	++fTotalMonstersDestroyed;
}
void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo*	thisMission = gScriptInterface.mMission();

	fNumDefendTeams = 0;
	fCurrNumbOfMonsters = 0;

	tTeamInfo* defendTeam1 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam1 ) );
	tTeamInfo* defendTeam2 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam2 ) );
	tTeamInfo* defendTeam3 = thisMission->mGetTeamHandle( static_cast< eTeamID >( kDefendTeam3 ) );

	tTeamInfo* monsterTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( kMonsterTeam ) );

	if( defendTeam1 && defendTeam1->mShipsRemainingInPlay() )
	{
		++fNumDefendTeams;
	}
	if( defendTeam2 && defendTeam2->mShipsRemainingInPlay() )
	{
		++fNumDefendTeams;
	}
	if( defendTeam3 && defendTeam3->mShipsRemainingInPlay() )
	{
		++fNumDefendTeams;
	}

	if( monsterTeam )
	{
		fCurrNumbOfMonsters = monsterTeam->mShipsRemainingInPlay();
	}

	for ( int i = 0; i < fNumTeams; i++ )
	{
		tTeamInfo* currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );

		if ( !currentTeam )
			continue;
		
		
		if( ( fNumDefendTeams == 0 ) || ( 0 == fCurrNumbOfMonsters ) )
		{
			int32 destroyedMonsters = gWC.mGetNumbMonstersDestroyed();
			int32 escapedMonsters = gWC.mGetNumbMonstersEscaped();
			int32 orignalMonsters = gWC.mGetNumbMonstersOriginal();

			if( ( orignalMonsters - destroyedMonsters ) ==  0 )
			{
				if( ( i == 0 ) || ( i == 1 ) | ( i == 2 ) )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
				}
			}
			else
			{
				if( ( i == 0 ) || ( i == 1 ) | ( i == 2 ) )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
				}

			}
		}
	}

}
