//	<Name>	AttackVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "AttackVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_14Enigma.h"
#include "AttackVictoryCondition.h"
#include "MissionText.h"
#include "AttackTeam.h"
#include "AttackTeamShip.h"
#include "DefendTeamShip.h"


#include "WorldConditions.h"

tAttackVictoryState::tAttackVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tAttackVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	attackTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackTeam1 ) );

	defendTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefendTeam1 ) );
	
	attackBPV = 0;
	defendBPV = 0;

	fThisTeamShipsInPlay = 0;
	fTeamDisengaged = false;
	fThisPlayerDead = false;
}

void tAttackVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 10.0f );

	if( attackTeam1 )
	{
		attackBPV += attackTeam1->mGetCombatBPV();
	}

	if( defendTeam1 )
	{
		defendBPV += defendTeam1->mGetCombatBPV();
	}
}

void tAttackVictoryState::mEndState()
{
}

eVictoryLevels tAttackVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 attackDamagePoints = 0;
	float attackDamageRatio = 0.0;
	int32 defendDamagePoints = 0;
	float defendDamageRatio = 0.0;
	float damageRatio = 0.0;
	int i;

	for( i = 1; i <= gWC.mGetNumAttackShips(); ++i )
	{
		attackDamagePoints +=	gWC.mGetAttackDamageCond( i ); 
	}
	for( i = 1; i <= gWC.mGetNumDefendShips(); ++i )
	{
		defendDamagePoints +=	gWC.mGetDefendDamageCond( i );
	}
	attackDamageRatio = (float)attackDamagePoints / gWC.mGetNumAttackShips();
	defendDamageRatio = (float)defendDamagePoints / gWC.mGetNumDefendShips();

	if( attackDamageRatio )
	{
		damageRatio = defendDamageRatio / attackDamageRatio;
	}
	else if( defendDamageRatio )
	{
		damageRatio = 3;
	}
	else
	{
		damageRatio = 0.5;
	}

	if( attackBPV > defendBPV && fTeamDisengaged && ( gWC.mGetRemainingDefendShips() > 0 ) )
	{
//		mAddToBonusPrestige( -40 );
	}

	if( fThisPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if ( fTeamDisengaged )			// 20020531 JEK fix disengage wins you a sector bug
	{
		if (attackBPV < defendBPV)
			return kDraw;
		else
			return kDefeat;
	}
	else if( damageRatio > 2 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 50 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 125 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		
		return	kAstoundingVictory;
	}
	else if( damageRatio > 1 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 25 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 62 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return kVictory;
	}
	else if( damageRatio >= 0.5 )
	{
		if( attackBPV <= defendBPV )
		{
			mAddToPrestige( 25 * ( ( (float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}
		else
		{
			mAddToPrestige( 31 * ( ( ( float)defendBPV / (float)attackBPV ) - 1.0f ) );
		}

		return kDraw;
	}
	else if( damageRatio > 0.25 )
	{
		return kDefeat;
	}
	else
	{
		return kDevastatingDefeat;
	}
}
int32 tAttackVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tAttackVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetOriginalTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			gWC.mRemoveAttackShip();
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	if( fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
		gWC.mRemoveAttackTeam();
		fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
		mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
		fThisPlayerDead = true;
	}

	gWC.mCheckForGameEnd();
}
void tAttackVictoryState::mOnDeath( tShipInfo* ship )
{

	int32 thisShipID;

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
		//Empty
		break;
	}

	mCheckAllTeams( ship );
}
void tAttackVictoryState::mOnCapture( tShipInfo* ship )
{
	int32 thisShipID;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tAttackTeamShip* thisShip = dynamic_cast< tAttackTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetAttackDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam1:
			//Break;
		break;
	}

	mCheckAllTeams( ship );
}

void tAttackVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveAttackTeam();
				fTeamDisengaged = true;
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
			}
		}
	}
}
void tAttackVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tAttackVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	}
}
void tAttackVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveAttackTeam();
		fPlayerLeftAlready = true;
		mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
	}
}
void tAttackVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if( shipTo->mGetTeam() == fTeam->mGetTeam() )
	{
		shipFrom->mRemoveItemFromStores(item, 1 );
		mAddToBonusPrestige( 90 );
	}
}