/********************************************************************************************************\

	Mission:	Enigma
	Script:		"Met_14Enigma"
	Descript:	<DESCRIPTION>

	Created by:		David F		10/12/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "AttackTeam.h"
#include "EnigmaTeam.h"
#include "WorldConditions.h"
#include "Met_14Enigma.h"


DEFINE_SCRIPT_CLASS( tMet_14Enigma );

tMet_14Enigma::tMet_14Enigma()
{
}
void tMet_14Enigma::mScriptSpecs( tMissionSpec* Specs )
{

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kNeutralRaceHex | kEnemyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 6;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase | kRequiresPlanet;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
																		kMaxChance,
																		kTrue,
																		kPrimaryTeam,
																		kMinChance,
																		kSpecPlayableRace,
																		kAnyRace,
																		typeid( tDefendTeam ),
																		kNoTeamTag,
																		Messages[kPlayableTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
																		kMaxChance,
																		kFalse,
																		kPrimaryOpponentTeam,
																		kMinChance,
																		kSpecPlayableRace | kSpecOrion | kSpecMonster,
																		kAnyEnemyOf | kDefendTeam1,
																		typeid( tAttackTeam ),
																		kNoTeamTag,
																		Messages[kPlayableTeamTitle_msg] );


	const tTeamSpec& team3 = mCreateTeamSpec(	"Enigma Team", static_cast< eTeamID>( kEnigmaTeam ), kMaxChance, kTrue,
																		kNPCTeam,
																		kMinChance,
													kSpecPlayableRace, kAnyRace, typeid( tEnigmaTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
}
int32 tMet_14Enigma::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_14Enigma::mCreateExternalShips( int32 Mode )
{
}

int32 tMet_14Enigma::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tMet_14Enigma::mNewInternalShip( tShipInfo* Ship )
{
}

void tMet_14Enigma::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kEnigmaTeam,	//Enig
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1
		/*Def1*/	"--",	"WD",	"WD",
		/*Atk1*/	"WD",	"--",	"WD",
		/*Enig*/	"AI",	"AI",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_14Enigma::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 4:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_14Enigma::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;


	Map->fTacticalMapIcon['Z'] = kObjectCorpse;
}
void tMet_14Enigma::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.4f );
}
void tMet_14Enigma::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		fBriefingIndex[kDefendTeam1][j] = kBriefing_msg;
		fBriefingIndex[kAttackTeam1][j] = kBriefing_msg;
	}
}
