/********************************************************************************************************\

	Mission:	Patrol
	Script:		"Met_17Patrol"
	Descript:	This is the Pirate version of Patrol.

	Created by:		David F		06/19/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "DefendTeam2.h"
#include "DefendTeam3.h"
#include "AttackTeam.h"
#include "AttackTeam2.h"
#include "AttackTeam3.h"
#include "WorldConditions.h"
#include "Met_17Patrol.h"


DEFINE_SCRIPT_CLASS( tMet_17Patrol );

tMet_17Patrol::tMet_17Patrol()
{
}
void tMet_17Patrol::mScriptSpecs( tMissionSpec* Specs )
{

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAlliedRaceHex | kPlayerRaceHex | kNeutralRaceHex, kAnyTerrainHex );
	Specs->fWait	= 3;
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kNoGenericFlags | kRequiresBase | kRequiresPlanet;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyRace,
														typeid( tDefendTeam ),
														kNoTeamTag,
														Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryOpponentTeam,
														kMaxChance,
														kSpecOrion,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam ),
														kNoTeamTag,
														Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team2", static_cast< eTeamID>( kDefendTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam2 ),
														kNoTeamTag,
														Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Defend Team3", static_cast< eTeamID>( kDefendTeam3 ),
														kMaxChance, kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,  kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam3 ),
														kNoTeamTag,
														Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace | kSpecOrion,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam2 ),
														kNoTeamTag,
														Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Attack Team3", static_cast< eTeamID>( kAttackTeam3 ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace | kSpecOrion,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam3 ),
														kNoTeamTag,
														Messages[kPlayerTeam1Title_msg] );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}
int32 tMet_17Patrol::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_17Patrol::mCreateExternalShips( int32 Mode )
{
}

int32 tMet_17Patrol::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tMet_17Patrol::mNewInternalShip( tShipInfo* Ship )
{
}

void tMet_17Patrol::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kDefendTeam2,	//Def2
		kDefendTeam3,	//Def3
		kAttackTeam2,	//Atk2
		kAttackTeam3,	//Atk3
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Def2	Def3	Atk2	Atk3
		/*Def1*/	"--",	"WD",	"AI",	"AI",	"WD",	"WD",
		/*Atk1*/	"WD",	"--",	"WD",	"WD",	"AI",	"AI",
		/*Def2*/	"AI",	"WD",	"--",	"AI",	"WD",	"WD",
		/*Def3*/	"AI",	"WD",	"AI",	"--",	"WD",	"WD",
		/*Atk2*/	"WD",	"AI",	"WD",	"WD",	"--",	"AI",
		/*Atk3*/	"WD",	"AI",	"WD",	"WD",	"AI",	"--",
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_17Patrol::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 4:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_17Patrol::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;
}
/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_17Patrol::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.0f );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );

	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kDefendTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.2f );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.0f );

	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
	mSetTeamShipStrength( kAttackTeam3, kClassFrigate, kClassBattleship, 0.7f, 1.0f );
}
void tMet_17Patrol::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		for ( int32 i = kDefendTeam1; i <= kAttackTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefing_msg;
		}
	}
}