/********************************************************************************************************\

	Mission:	Patrol
	Script:		"Met_19Scan"
	Descript:	<DESCRIPTION>

	Created by:		David F		11/11/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "DefendTeam.h"
#include "DefendTeam2.h"
#include "AttackTeam.h"
#include "AttackTeam2.h"
#include "Met_19Scan.h"
#include "WorldConditions.h"
#include "PlanetTeam.h"

DEFINE_SCRIPT_CLASS( tMet_19Scan );

tMet_19Scan::tMet_19Scan()
{
}
void tMet_19Scan::mScriptSpecs( tMissionSpec* Specs )
{

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kPlayerRaceHex | kNeutralRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kNoGenericFlags | kRequiresBase | kRequiresPlanet;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defend Team1", static_cast< eTeamID>( kDefendTeam1 ),
														kMaxChance,
														kTrue,
														kPrimaryTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyRace,
														typeid( tDefendTeam ),
														kNoTeamTag,
														Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team1", static_cast< eTeamID>( kAttackTeam1 ),
														0.5f,
														kFalse,
														kPrimaryOpponentTeam,
														kMinChance,
														kSpecPlayableRace | kSpecOrion | kSpecMonster,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam ),
														kNoTeamTag,
														Messages[kOtherPlayersTitle_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team2", static_cast< eTeamID>( kDefendTeam2 ),
														0.1f,
														kFalse,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace,
														kAnyAllyOf | kDefendTeam1,
														typeid( tDefendTeam2 ),
														kNoTeamTag,
														Messages[kOtherPlayersTitle_msg] );



	const tTeamSpec& team4 = mCreateTeamSpec(	"Attack Team2", static_cast< eTeamID>( kAttackTeam2 ),
														0.1f,
														kFalse,
														kPlayableTeam,
														kMinChance,
														kSpecPlayableRace | kSpecOrion | kSpecMonster,
														kAnyEnemyOf | kDefendTeam1,
														typeid( tAttackTeam2 ),
														kNoTeamTag,
														Messages[kOtherPlayersTitle_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Planet Team", static_cast< eTeamID>( kPlanetTeam ),
															kMaxChance,
															kTrue,
															kNPCTeam,
															kMaxChance,
															kSpecAnyRace,
															kAnyRace,
															typeid( tPlanetTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );

}
int32 tMet_19Scan::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_19Scan::mCreateExternalShips( int32 Mode )
{
}
void tMet_19Scan::mNewInternalShip( tShipInfo* Ship )
{
}

void tMet_19Scan::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 tagList[kRelationsCount] =
	{
		kDefendTeam1,	//Def1
		kAttackTeam1,	//Atk1
		kDefendTeam2,	//Def2
		kAttackTeam2,	//Atk2
		kPlanetTeam,	//Plnt
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//			Def1	Atk1	Def2	Atk2	Plnt
		/*Def1*/	"--",	"WD",	"AI",	"WD",	"AI",
		/*Atk1*/	"WD",	"--",	"WD",	"AI",	"AI",
		/*Def2*/	"AI",	"WD",	"--",	"WD",	"AI",
		/*Atk2*/	"WD",	"AI",	"WD",	"--",	"AI",
		/*Plnt*/	"AI",	"AI",	"AI",	"AI",	"--",

	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_19Scan::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 4:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_19Scan::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;
}
/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_19Scan::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kDefendTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kAttackTeam1, kClassFrigate, kClassBattleship, 0.7f, 1.2f );

	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kDefendTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );

	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	mSetTeamShipStrength( kAttackTeam2, kClassFrigate, kClassBattleship, 0.7f, 1.2f );

}
void tMet_19Scan::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		fBriefingIndex[kDefendTeam1][j] = kScannerBriefing_msg;
		fBriefingIndex[kAttackTeam1][j] = kOtherBriefing_msg;
		fBriefingIndex[kDefendTeam2][j] = kOtherBriefing_msg;
		fBriefingIndex[kAttackTeam2][j] = kOtherBriefing_msg;
	}
}