//	<Name>	ScanShipInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Met_19Scan.h"
#include "ShipInfo.h"
#include "DefendTeamShip.h"
#include "ScanShipInitialState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

tScanShipInitialState::tScanShipInitialState( )
{
	fDeepScanON = false;
}

void tScanShipInitialState::mBeginState()
{
}

void tScanShipInitialState::mEndState()
{
}

void tScanShipInitialState::mOnTimedInterval( int32 timerID )
{
	bool planetScanned = gWC.mGetPlanetScanned();
	tTeamInfo* planetTeam  = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlanetTeam ) );	
	
	tShipIterator pship = planetTeam->mGetFirstShip();
	tShipInfo* planet = *pship;

	if( timerID == kDeepScanOnID && fDeepScanON )
	{
		fMissionInfo->mDisplayMessage( fChildActor->mGetTeam(),  kScanning_msg, kScienceScriptMessage );
		mSetTimedInterval( kTimerTurnsSet, kCheckDistanceID, 0.5f );
	}
	else if( timerID == kCheckDistanceID )
	{
		int32 rangeToPlanetThisInterval = fChildActor->mGetTacticalDistance( planet );


		if( ( planet ) && !planetScanned && ( rangeToPlanetThisInterval < 10 ) && ( fDeepScanON ) && ( ( fChildActor->mGetShipTargetID() ) == planet->mGetShipID() ) )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kDefendTeam1 ), kScannedPlanet_msg, kScienceScriptMessage );
			planetScanned = true;
			gWC.mSetPlanetScanned();
			mSetTimedInterval( kTimerSecondsSet, kGameOverCheckID, 8.0f );
		}
		if( fDeepScanON && ! planetScanned )
		{
			mSetTimedInterval( kTimerTurnsSet, kCheckDistanceID, 0.75f );
		}
	}
	else if( timerID == kGameOverCheckID )
	{
		gWC.mCheckForGameEnd();
	}
	else if( timerID == kFiredProbeID )
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kDefendTeam1 ), kProbe_msg, kScienceScriptMessage );
	}
}
void tScanShipInitialState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	fChildActor->mSetShipCallbackEvents(  kNotifyDamage );
	fThisShipWorldID = fChildActor->mGetThisShipID();

	if( fChildActor->mGetShipID() == damagedShip->mGetShipID() )
	{
		if( damagedShip->mGetDamagePercent() == 100 )
		{
			gWC.mSetDefendDamageCond( fThisShipWorldID, 4 );
		}
		else if( damagedShip->mGetDamagePercent() > 75 )
		{
			gWC.mSetDefendDamageCond( fThisShipWorldID, 3 );
		}
		else if( damagedShip->mGetDamagePercent() > 50 )
		{
			gWC.mSetDefendDamageCond( fThisShipWorldID, 2 );
		}
		else if( damagedShip->mGetDamagePercent() > 25 )
		{
			gWC.mSetDefendDamageCond( fThisShipWorldID, 1 );
		}
	
	}
}
void tScanShipInitialState::mOnDeath( tShipInfo* ship )
{
}
void tScanShipInitialState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fDeepScanON = true;
	}
	else
	{
		fDeepScanON = false;
	}
	ship->mSetShipCallbackEvents(  kNotifyDeepScan );
	mSetTimedInterval( kTimerTurnsSet, kDeepScanOnID, 0.75f );
}
void tScanShipInitialState::mOnProbeLaunch( tShipInfo* ship )
{
	bool planetProbed = gWC.mGetPlanetProbed();

	tShipInfo* planet;
	tTeamInfo* planetTeam  = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlanetTeam ) );	
	
	if( planetTeam )
	{
		tShipIterator pship = planetTeam->mGetFirstShip();
		planet = *pship;
	}

	if( ( !planetProbed ) && ( fChildActor->mGetShipTargetID() == planet->mGetShipID() ) )
	{
		gWC.mSetPlanetProbed();
		planetProbed = true;
		mSetTimedInterval( kTimerTurnsSet, kFiredProbeID, 0.2f );
	}
}