//	<Name>	ScanVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ScanVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_19Scan.h"
#include "DefendVictoryCondition.h"
#include "MissionText.h"
#include "DefendTeam.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "AttackTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"
#include "DefendTeamShip.h"
#include "WorldConditions.h"
#include "ScanShip.h"

tScanVictoryState::tScanVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tScanVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fThisTeamShipsInPlay = 1;
	fTeamDisengaged = false;
	fPlayerLeftAlready = false;
	fThisPlayerDead = false;
}

void tScanVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kEndingID, 10.0f );
}

void tScanVictoryState::mEndState()
{
}

eVictoryLevels tScanVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if( fThisPlayerDead )
	{
		return kDevastatingDefeat;
	}
	else if( ( gWC.mGetPlanetProbed() ) && ( gWC.mGetPlanetScanned() ) )
	{
		return	kAstoundingVictory;
	}
	else if( gWC.mGetPlanetScanned() )
	{
		return kVictory;
	}
	else
	{
		return kDraw;
	}

}
int32 tScanVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tScanVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetOriginalTeam();


	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
		}
	}

	if( fThisTeamShipsInPlay == 0 && fTeam->mGetTeam() == ship->mGetOriginalTeam() )
	{
		fThisPlayerDead = true;
		gWC.mRemoveDefendTeam();
		fMissionInfo->mDisplayMessage( teamID, kAllYourShipsDead_msg, kNonCommScriptMessage );
		mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
	}

}
void tScanVictoryState::mOnDeath( tShipInfo* ship )
{
	int32 thisShipID;

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
		case kAttackTeam2:
			if( ( ship->mGetTeam() == kAttackTeam1 ) || ( ship->mGetTeam() == kAttackTeam2 ) )
			{
				mAddToBonusPrestige( 180 );
			}
		break;
		case kDefendTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tScanShip* thisShip = dynamic_cast< tScanShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		break;
	}

	mCheckAllTeams( ship );
}
void tScanVictoryState::mOnCapture( tShipInfo* ship )
{
	int32 thisShipID;

	switch( ship->mGetOriginalTeam() )
	{
		case kAttackTeam1:
		case kAttackTeam2:
				mAddToBonusPrestige( 270 );
		break;
		case kDefendTeam1:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tScanShip* thisShip = dynamic_cast< tScanShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		break;
		case kDefendTeam2:
			if( fTeam->mGetTeam() == ship->mGetOriginalTeam() )
			{
				tDefendTeamShip* thisShip = dynamic_cast< tDefendTeamShip* >( ship );
				thisShipID = thisShip->mGetThisShipID();
				gWC.mSetDefendDamageCond( thisShipID, 4 );
			}
		break;
	}

	mCheckAllTeams( ship );
}

void tScanVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				gWC.mRemoveDefendTeam();
				fMissionInfo->mDisplayMessage( teamID, kYouHaveDisengaged_msg, kNonCommScriptMessage );
				fTeamDisengaged = true;
				mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
			}
		}
	}
		
}
void tScanVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tScanVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == kEndingID )
	{
		gWC.mCheckForGameEnd();
	}
}
void tScanVictoryState::mOnPlayerLeaving( int playerNumber )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( playerNumber ) );
	Assert( team );
	if ( !team )
		return;

	//
	//	End the game as a loser for this team
	//
	if ( fTeam->mGetTeam() == team->mGetTeam() && ! fPlayerLeftAlready )
	{
		gWC.mRemoveDefendTeam();
		fPlayerLeftAlready = true;
		mSetTimedInterval( kTimerSecondsSet, kEndingID, 3.0 );
	}
}