/********************************************************************************************************\

	Mission:	Scout
	Script:		"Met_Scout"
	Descript:	<DESCRIPTION>

	Created by:		David F		05/30/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Met_1Scout.h"
#include "ScoutTeam.h"
#include "WorldConditions.h"
#include "TeamScouted.h"

DEFINE_SCRIPT_CLASS( tMet_Scout );

tMet_Scout::tMet_Scout()
{}
void tMet_Scout::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kEnemyRaceHex | kNeutralRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kNoGenericFlags | kRequiresBase | kRequiresPlanet;
	

	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1",
												static_cast< eTeamID>( kScoutTeam ),			//const tString& teamName, 
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tScoutTeam ),
												kNoTeamTag,
												Messages[kPlayerTeam1Title_msg] );
		
		
	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2",
												static_cast< eTeamID>( kTeamScouted ),
												kMaxChance, kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace | kSpecOrion,
												kAnyEnemyOf | kScoutTeam,
												typeid( tTeamScouted ),
												kNoTeamTag,
												Messages[kPlayerTeam1Title_msg] );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
}
int32 tMet_Scout::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}
void tMet_Scout::mCreateExternalShips( int32 Mode )
{
}
int32 tMet_Scout::mInitializeFinished( void )
{
	return kScoutTeamBriefing_msg;
}
void tMet_Scout::mNewInternalShip( tShipInfo* Ship )
{
}
void tMet_Scout::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tMet_Scout::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}
void tMet_Scout::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kScoutTeam,
		kTeamScouted,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Scout	= kScoutTeam,
		//Other	= kTeamScouted,


		//			Plyr	Other
		/*Scout*/	"--",	"WD",
		/*Other*/	"WD",	"--",

	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);

	gWC.mCheckForGameEnd();
}
tMapTerrain tMet_Scout::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 4:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}
void tMet_Scout::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;
}
void tMet_Scout::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kScoutTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kScoutTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kScoutTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kTeamScouted, kClassFrigate, kClassBattleship, 1.0f, 1.6f );
	mSetTeamShipStrength( kTeamScouted, kClassFrigate, kClassBattleship, 1.0f, 1.6f );
	mSetTeamShipStrength( kTeamScouted, kClassFrigate, kClassBattleship, 1.0f, 1.6f );
}
void tMet_Scout::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		fBriefingIndex[kScoutTeam][j] = kScoutTeamBriefing_msg;
		fBriefingIndex[kTeamScouted][j] = kTeamScoutedBriefing_msg;
	}
}
