//	<Name>	ScoutTeamVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ScoutTeamVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_1Scout.h"
#include "ScoutTeamVictoryCondition.h"
#include "MissionText.h"
#include "ScoutTeam.h"
#include "TeamScouted.h"
#include "WorldConditions.h"

tScoutTeamVictoryState::tScoutTeamVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat, int32 unknown ) :
	tChildActorVictoryState<tScoutTeamVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat, unknown )
{
	ftotalTeamScoutedShips = 0;
	
	fScoutTeamShipsInPlay = 1;
	fTeamScoutedShipsInPlay = 1;
	fPlayerDied= false;
}

void tScoutTeamVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 10.0f );
}

void tScoutTeamVictoryState::mEndState()
{
}

eVictoryLevels tScoutTeamVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 totalTeamScoutedShips = gWC.mGetTotalTeamScoutedShips();
	int32 shipsScanned = 0;

	for( int32 i = 0; i < 20; ++i )
	{
		if( gWC.mGetScannedStatus( i ) == 1)
		{
			++shipsScanned;
		}
	}

	if( fPlayerDied )
	{
		return kDevastatingDefeat;
	}
	if( shipsScanned == totalTeamScoutedShips )
	{
		return kAstoundingVictory;
	}
	else if( ( shipsScanned > 0 ) || ( fTeamScoutedShipsInPlay == 0 ) )
	{
		return kVictory;
	}
	else if( shipsScanned > 0 )
	{
		return kDraw;
	}
	else
	{
		return kDefeat;
	}
}
int32 tScoutTeamVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tScoutTeamVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eRaceName  scoutRace;
	tTeamInfo* scoutTeam   = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );
	tTeamInfo* teamScouted = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTeamScouted ) );


	scoutRace = scoutTeam->mGetRace();



	fScoutTeamShipsInPlay = mShipsInPlay( scoutTeam );

	fTeamScoutedShipsInPlay = mShipsInPlay(teamScouted );


	if( fTeamScoutedShipsInPlay == 0 )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kScoutTeam, kPlayerWins );
	}
	if( fScoutTeamShipsInPlay == 0 )
	{
		fPlayerDied = true;
		gScriptInterface.mSetGameStatus( (eTeamID)kScoutTeam, kPlayerDestroyed );
	}

}
void tScoutTeamVictoryState::mOnDeath( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}
void tScoutTeamVictoryState::mOnCapture( tShipInfo* ship )
{
	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}
	mCheckAllTeams( ship );
}

void tScoutTeamVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo* scoutTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );
	int32 shipsScanned = 0;
	int32 fThisTeamShipsInPlay;

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{

				for( int32 i = 0; i < 20; ++i )
				{
					if( gWC.mGetScannedStatus( i ) == 1)
					{
						++shipsScanned;
					}
				}
				if( shipsScanned > 0 )
				{
					gScriptInterface.mSetGameStatus( (eTeamID)kScoutTeam, kPlayerWins );
				}
				else
				{
					gScriptInterface.mSetGameStatus( (eTeamID)kScoutTeam, kPlayerLoses );
				}

			}
		}
	}
}
void tScoutTeamVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tScoutTeamVictoryState::mOnTimedInterval( int32 timerID )
{
		if( timerID == kStartingID )
		{
			int32 teamScoutedShips = 0;


			tTeamInfo* teamScouted = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTeamScouted ) );

			tShipIterator ship = teamScouted->mGetFirstShip();

			if( *ship != NULL )
			{
				tShipInPlayEval eval;
				ship.mAddEvaluator( &eval );
			}


			for( ; *ship != NULL; ++ship )
			{
					++teamScoutedShips;
			}

			gWC.mSetTotalTeamScoutedShips( teamScoutedShips );	//Set the total number of kTeamScouted ships in game

			if( teamScoutedShips < 1 )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kScoutTeam, kPlayerWins );
			}

		}
}