//	<Name>	TeamScoutedVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "TeamScoutedVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Met_1Scout.h"
#include "TeamScoutedVictoryCondition.h"
#include "MissionText.h"
#include "ScoutTeam.h"
#include "TeamScouted.h"
#include "WorldConditions.h"

tTeamScoutedVictoryState::tTeamScoutedVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tTeamScoutedVictoryCondition>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	ftotalTeamScoutedShips = 0;
	
	fScoutTeamShipsInPlay = 1;
	fTeamScoutedShipsInPlay = 1;
	fScoutCaptured = false;
	fPlayerDied= false;
}

void tTeamScoutedVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kStartingID, 10.0f );
}

void tTeamScoutedVictoryState::mEndState()
{
}

eVictoryLevels tTeamScoutedVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	int32 totalTeamScoutedShips = gWC.mGetTotalTeamScoutedShips();
	int32 shipsScanned = 0;

	for( int32 i = 0; i < 20; ++i )
	{
		if( gWC.mGetScannedStatus( i ) == 1)
		{
			++shipsScanned;
		}
	}

	if( fPlayerDied )
	{
		return kDevastatingDefeat;
	}
	else if( fScoutCaptured == true && fScoutTeamShipsInPlay == 0 )
	{
		return kAstoundingVictory;
	}
	else if( fScoutTeamShipsInPlay == 0 )
	{
		return kVictory;
	}
	else if( shipsScanned == totalTeamScoutedShips )
	{
		return kDevastatingDefeat;
	}
		else if( 0.5f <= ( (float)shipsScanned / (float)totalTeamScoutedShips ) )
	{
		return kDefeat;
	}
	else if( shipsScanned > 0 )
	{
		return kDraw;
	}
	return kUnknownVictoryLevel;

}
int32 tTeamScoutedVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tTeamScoutedVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eRaceName  scoutRace;
	tTeamInfo* scoutTeam   = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );
	tTeamInfo* teamScouted = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTeamScouted ) );


	scoutRace = scoutTeam->mGetRace();



	fScoutTeamShipsInPlay = mShipsInPlay( scoutTeam );

	fTeamScoutedShipsInPlay = mShipsInPlay(teamScouted );


	if( fScoutTeamShipsInPlay == 0 )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kTeamScouted, kPlayerWins );
	}
	if( fTeamScoutedShipsInPlay == 0 )
	{
		fPlayerDied = true;
		gScriptInterface.mSetGameStatus( (eTeamID)kTeamScouted, kPlayerDestroyed );
	}

}
void tTeamScoutedVictoryState::mOnDeath( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}
void tTeamScoutedVictoryState::mOnCapture( tShipInfo* ship )
{
	tTeamInfo* scoutTeam   = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige(120 );
	}

	mCheckAllTeams( ship );
	if( ship->mGetTeam() == scoutTeam->mGetTeam() )
	{
		fScoutCaptured = true;
	}
}

void tTeamScoutedVictoryState::mOnDisengaged( tShipInfo* ship )
{
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo* teamScouted = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTeamScouted ) );
	tTeamInfo* scoutTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );
	int32 fThisTeamShipsInPlay;

	int32 totalTeamScoutedShips = gWC.mGetTotalTeamScoutedShips();
	int32 shipsScanned = 0;

	if( fTeam )
	{
		if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
		{
			fThisTeamShipsInPlay = mShipsInPlay( fTeam );
			if( fThisTeamShipsInPlay < 1 )
			{
				for( int32 i = 0; i < 20; ++i )
				{
					if( gWC.mGetScannedStatus( i ) == 1)
					{
						++shipsScanned;
					}
				}

				if ( ( ship->mGetTeam() ) == ( scoutTeam->mGetTeam() ) ) 
				{
					if( shipsScanned == 0 )
					{
						gScriptInterface.mSetGameStatus( (eTeamID)kTeamScouted, kPlayerWins );
					}
					else
					{
						gScriptInterface.mSetGameStatus( (eTeamID)kTeamScouted, kPlayerLoses );
					}
				}
				else if ( ( ship->mGetTeam() ) == ( teamScouted->mGetTeam() ) )
				{
					if( shipsScanned == 0 )
					{
						gScriptInterface.mSetGameStatus( (eTeamID)kTeamScouted, kPlayerWins );
					}
					else
					{
						gScriptInterface.mSetGameStatus( (eTeamID)kTeamScouted, kPlayerLoses );
					}
				}
			}
		}
	}
}
void tTeamScoutedVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tTeamScoutedVictoryState::mOnTimedInterval( int32 timerID )
{
			int32 scoutTeamShips = 0;

		if( timerID == kStartingID )
		{
			int32 scoutTeamShips = 0;

			tTeamInfo* scoutTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kScoutTeam ) );

			tShipIterator ship = scoutTeam->mGetFirstShip();

			if( *ship != NULL )
			{
				tShipInPlayEval eval;
				ship.mAddEvaluator( &eval );
			}


			for( ; *ship != NULL; ++ship )
			{
					++scoutTeamShips;
			}

			gWC.mSetTotalScoutShips( scoutTeamShips );	//Set the total number of kScoutTeam ships in game

			if( scoutTeamShips < 1 )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kScoutTeam, kPlayerWins );
			}

		}
}