/********************************************************************************************************\

	File Name:	CommonEnemyShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "CommonEnemyShipBaseState.h"
#include "CommonEnemyTeam.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonEnemyShipBaseState::mBeginState()
{}
void tCommonEnemyShipBaseState::mEndState()
{}

void tCommonEnemyShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mAtDistance( tShipInfo* ship )
{
	//
	//	A ship from the enemy team has made it to the listening post.  So simulate a data transfer
	//	for the team that made it and let the rest of the teams know what has happened and that the
	//	enemy can now successfully disengage.
	//
	if ( !gWC.fListeningPostDestroyed )
	{
		if ( fChildActor->mGetShipID() == ship->mGetShipID() )
		{
			fMissionInfo->mDisplayMessage( ship->mGetTeam(), kDataTransfer_msg );
			for ( int32 i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kReachedListeningPostPlayer_msg );
			}

			for ( i = kEnemyTeam1; i <= kEnemyTeam3; i++ )
			{
				if ( i != ship->mGetTeam() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), kReachedListeningPostEnemy_msg );
				}
			}
		}

		//
		//	Set the victory condition that the data was transferred and that it is a victory if the enemy
		//	team disengages from this point forward.  The player team has already lost, but can still fight
		//	to kill for, maybe, some bonus prestige.
		//
		tCommonEnemyTeam* enemyTeam = dynamic_cast< tCommonEnemyTeam* >( fMissionInfo->mGetTeamHandle( ship->mGetTeam() ) );
		Assert( enemyTeam );
		if ( enemyTeam )
		{
			enemyTeam->mSetReachedListeningPost( kTrue );
			for ( int32 i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
				gWC.mSetTeamWon( i, kFalse );		//	this should probably be done in the victory state, but this will do
		}

		//
		//	If this is an AI ship, tell it to disengage immediately.
		//
		ship->mDisengage( kByMapEdge, kAbsolutePriority );
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonEnemyShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonEnemyShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonEnemyShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonEnemyShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonEnemyShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonEnemyShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonEnemyShipBaseState::mOnFlashingButtonPressed()
{}
void tCommonEnemyShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonEnemyShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonEnemyShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonEnemyShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonEnemyShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonEnemyShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonEnemyShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonEnemyShipBaseState::mOnHostChanged()
{}
void tCommonEnemyShipBaseState::mOnTimedInterval( int32 timerID )
{}
void tCommonEnemyShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonEnemyShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

