/********************************************************************************************************\

	File Name:	CommonVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Met_3Ambushee.h"
#include "DefenderTeam.h"
#include "Attacker1Team.h"
#include "Attacker2Team.h"
#include "WorldConditions.h"
#include "CommonDefs.h"

//
//	Serialization events
//
tCommonVictoryState::tCommonVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tCommonVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}

void tCommonVictoryState::mBeginState()
{}
void tCommonVictoryState::mEndState()
{}

eVictoryLevels tCommonVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefenderTeam ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return kUnknownVictoryLevel;

	tAttacker1Team* attackerTeam1 = dynamic_cast<tAttacker1Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return kUnknownVictoryLevel;

	tAttacker2Team* attackerTeam2 = dynamic_cast<tAttacker2Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam2 ) ) );	
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return kUnknownVictoryLevel;
	
	switch( ID )
	{
	case kDefenderTeam:
		{
			mAddToBonusPrestige( defenderTeam->mGetPrestige() );

			// Destroyed, captured or drove off all enemies
			if( defenderTeam->mHasWon() )
			{
				return kAstoundingVictory;
			}

			// Disengaged...
			if( defenderTeam->mDisengaged() )
			{
				// ...without taking any losses; without inflicting any losses
				if( ( !defenderTeam->mShipsLost() ) && ( !defenderTeam->mShipsKilled() ) )
					mAddToPrestige( 50 );
					return kDraw;
				
				// ...without taking any losses; inflicted some losses
				if( ( !defenderTeam->mShipsLost() ) && ( defenderTeam->mShipsKilled() ) )
					return kAstoundingVictory;

				// ...took losses; inflicted losses; killed more than lost
				if( ( defenderTeam->mShipsLost() ) && ( defenderTeam->mShipsLost() < defenderTeam->mShipsKilled() ) )
					return kVictory;

				// ...took losses; infliccted losses; lost more than killed
				if( ( defenderTeam->mShipsLost() ) && ( defenderTeam->mShipsLost() > defenderTeam->mShipsKilled() ) )
					return kDefeat;

				// ... took losses; inflicted losses; lost & killed are equal
				if( ( defenderTeam->mShipsLost() ) && ( defenderTeam->mShipsLost() == defenderTeam->mShipsKilled() ) )
					return kDefeat;

				// ...took losses; inflicted no losses
				if( ( defenderTeam->mShipsLost() ) && ( !defenderTeam->mShipsKilled() ) )
					return kDevastatingDefeat;

				// anything else
				return kDraw;
			}

			// Died
			return kDevastatingDefeat;
		}
		break;


	case kAttackerTeam1:
		{
			// Lost altogether
			if( defenderTeam->mHasWon() )
			{
				return kDevastatingDefeat;
			}

			// Won outright
			if( attackerTeam1->mHasWon() )
			{
				return kAstoundingVictory;
			}

			// Neither lost nor won, but managed to kill a target
			if( attackerTeam1->mKilledTarget() )
			{
				return kDraw;
			}

			// Failed miserably
			if( !attackerTeam1->mKilledTarget() )
			{
				return kDevastatingDefeat;
			}

			// anything else
			return kDraw;
		}
		break;

	case kAttackerTeam2:
		{
			ASSERT( attackerTeam2 );
			
			if( defenderTeam->mHasWon() )
			{
				return kDevastatingDefeat;
			}

			// Won outright
			if( attackerTeam2->mHasWon() )
			{
				return kAstoundingVictory;
			}

			// Neither lost nor won, but managed to kill a target
			if( attackerTeam2->mKilledTarget() )
			{
				return kDraw;
			}

			// Failed miserably
			if( !attackerTeam2->mKilledTarget() )
			{
				return kDevastatingDefeat;
			}

			// anything else
			return kDraw;

		}
		break;

	default:
		return kUnknownVictoryLevel;
		break;
	}
}

void tCommonVictoryState::mOnDeath( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship )	return;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefenderTeam ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return;

	tAttacker1Team* attackerTeam1 = dynamic_cast<tAttacker1Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return;

	tAttacker2Team* attackerTeam2 = dynamic_cast<tAttacker2Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam2 ) ) );
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return;

	eTeamID thisTeam = fChildActor->mGetTeam()->mGetTeam();

	
	if( thisTeam == ship->mGetTeam() )
	{
		
		switch( ship->mGetTeam() )
		{
		case kDefenderTeam:
			{
				defenderTeam->mIncShipLost();
				attackerTeam1->mIncShipsKilled();
				attackerTeam2->mIncShipsKilled();

				if( !defenderTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerDestroyed );
					
					attackerTeam1->mSetWon( true );
					attackerTeam2->mSetWon( true );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam1 ), kPlayerWins );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam2 ), kPlayerWins );
				}
			}
			break;
			
		case kAttackerTeam1:
			{
				defenderTeam->mIncShipsKilled();
				attackerTeam1->mIncShipLost();
				
				if( !attackerTeam1->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam1 ), kPlayerDestroyed );
					
					if( !attackerTeam2->mShipsRemainingInPlay() )
					{
						defenderTeam->mSetWon( true );
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
					}
				}					
			}
			break;
			
		case kAttackerTeam2:
			{
				defenderTeam->mIncShipsKilled();
				attackerTeam2->mIncShipLost();

				if( !attackerTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam2 ), kPlayerDestroyed );
					
					if( !attackerTeam1->mShipsRemainingInPlay() )
					{						
						defenderTeam->mSetWon( true );
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
					}
				}
			}
			break;
		}
	}
}

void tCommonVictoryState::mOnCapture( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship )	return;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefenderTeam ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return;

	tAttacker1Team* attackerTeam1 = dynamic_cast<tAttacker1Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return;

	tAttacker2Team* attackerTeam2 = dynamic_cast<tAttacker2Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam2 ) ) );
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return;

	eTeamID thisTeam = fChildActor->mGetTeam()->mGetTeam();

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}
	
	if( thisTeam == ship->mGetTeam() )
	{
		
		switch( ship->mGetOriginalTeam() )
		{
		case kDefenderTeam:
			{
				defenderTeam->mIncShipLost();
				attackerTeam1->mIncShipsKilled();
				attackerTeam2->mIncShipsKilled();

				if( !defenderTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerDestroyed );
					
					attackerTeam1->mSetWon( true );
					attackerTeam2->mSetWon( true );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam1 ), kPlayerWins );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam2 ), kPlayerWins );
				}
			}
			break;
			
		case kAttackerTeam1:
			{
				defenderTeam->mIncShipsKilled();
				attackerTeam1->mIncShipLost();
				
				if( !attackerTeam1->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam1 ), kPlayerCaptured );
					
					if( !attackerTeam2->mShipsRemainingInPlay() )
					{
						defenderTeam->mSetWon( true );
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
					}
				}
			}
			break;
			
		case kAttackerTeam2:
			{
				defenderTeam->mIncShipsKilled();
				attackerTeam2->mIncShipLost();

				if( !attackerTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam2 ), kPlayerCaptured );
					
					if( !attackerTeam1->mShipsRemainingInPlay() )
					{						
						defenderTeam->mSetWon( true );
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
					}
				}
			}
			break;
		}
	}
}

void tCommonVictoryState::mOnDisengaged( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship )	return;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefenderTeam ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return;

	tAttacker1Team* attackerTeam1 = dynamic_cast<tAttacker1Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return;

	tAttacker2Team* attackerTeam2 = dynamic_cast<tAttacker2Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam2 ) ) );
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return;

	eTeamID thisTeam = fChildActor->mGetTeam()->mGetTeam();


	if( thisTeam == ship->mGetTeam() )
	{
		switch( ship->mGetTeam() )
		{
		case kDefenderTeam:
			{
				if( !defenderTeam->mShipsRemainingInPlay() )
				{
					defenderTeam->mSetDisengaged();
					
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ),	 kPlayerWins );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam1 ), kPlayerWins );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam2 ), kPlayerWins );
				}
			}
			break;
			
		case kAttackerTeam1:
			{
				defenderTeam->mIncShipsKilled();
				attackerTeam1->mIncShipLost();
				
				if( !attackerTeam1->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam1 ), kPlayerDisengaged );
					
					if( !attackerTeam2->mShipsRemainingInPlay() )
					{
						defenderTeam->mSetWon( true );
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
					}
				}
			}
			break;
			
		case kAttackerTeam2:
			{
				defenderTeam->mIncShipsKilled();
				attackerTeam2->mIncShipLost();
				
				if( !attackerTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttackerTeam2 ), kPlayerDisengaged );
					
					if( !attackerTeam1->mShipsRemainingInPlay() )
					{						
						defenderTeam->mSetWon( true );
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
					}
				}
			}
			break;
		}
	}
}

void tCommonVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnFlashingButtonPressed()
{}
void tCommonVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tCommonVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonVictoryState::mOnHostChanged()
{}
void tCommonVictoryState::mOnTimedInterval( int32 timerID )
{}
void tCommonVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
