/********************************************************************************************************\

	File Name:	DefenderVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "DefenderVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Met_3Ambushee.h"
#include "CommonDefs.h"
#include "DefenderTeam.h"
#include "Attacker1Team.h"
#include "Attacker2Team.h"
#include "WorldConditions.h"

//
//	Serialization events
//
tDefenderVictoryState::tDefenderVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tDefenderVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tDefenderVictoryState::mBeginState()
{}
void tDefenderVictoryState::mEndState()
{}

eVictoryLevels tDefenderVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	return kDraw;
}

void tDefenderVictoryState::mOnDeath( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship ) return;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefenderTeam ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return;

	tAttacker1Team* attackerTeam1 = dynamic_cast<tAttacker1Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return;

	// Attack Team 2 may not exist, so don't assert or bail out if it's not there
	tAttacker2Team* attackerTeam2 = dynamic_cast<tAttacker2Team*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttackerTeam2 ) ) );

	switch( ship->mGetTeam() )
	{
	case kDefenderTeam:
		{
			mAddToPrestige( -kPrestigeShipKilled );

			if( !defenderTeam->mShipsRemainingInPlay() )
			{
				gWC.mSetTeamDead( 1 );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerDestroyed );
			}
		}
		break;

	case kAttackerTeam1:
		{
			mAddToPrestige( kPrestigeShipKilled );

			if( gWC.mIsTeamDead( 2 ) )
			{
				if( attackerTeam2 )
				{
					if( gWC.mIsTeamDead( 3 ) ) 
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
					}
				}
				else
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
				}
			}
		}
		break;
		
	case kAttackerTeam2:
		{
			ASSERT( attackerTeam2 );	// This should never happen

			mAddToPrestige( kPrestigeShipKilled );

			if( !attackerTeam2->mShipsRemainingInPlay() )
			{
				if( !attackerTeam1->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefenderTeam ), kPlayerWins );
				}
			}
		}
		break;
	}
}

void tDefenderVictoryState::mOnCapture( tShipInfo* ship )
{

}

void tDefenderVictoryState::mOnDisengaged( tShipInfo* ship )
{

}

void tDefenderVictoryState::mAtDistance( tShipInfo* ship )
{}
void tDefenderVictoryState::mAtDestination( tShipInfo* ship )
{}
void tDefenderVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tDefenderVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tDefenderVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tDefenderVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tDefenderVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tDefenderVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tDefenderVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tDefenderVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tDefenderVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tDefenderVictoryState::mOnFlashingButtonPressed()
{}
void tDefenderVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tDefenderVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tDefenderVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tDefenderVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tDefenderVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tDefenderVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tDefenderVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tDefenderVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tDefenderVictoryState::mOnHostChanged()
{}
void tDefenderVictoryState::mOnTimedInterval( int32 timerID )
{}
void tDefenderVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tDefenderVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
