/********************************************************************************************************\

	Mission:	Met_3Ambushee
	Script:		"Met_3Ambushee"
	Descript:	Team 1 gets bushwhacked by two enemy teams.
				
	Created by:		Scott Bruno		09/27/00
	Last Updated:	<INITIALS>		mm/dd/yy
					<LOG>


\********************************************************************************************************/


#include "MissionText.h"
#include "MissionMaps.h"
#include "DefenderTeam.h"
#include "Attacker1Team.h"
#include "Attacker2Team.h"
#include "Met_3Ambushee.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


DEFINE_SCRIPT_CLASS( tMet_3Ambushee );

tMet_3Ambushee::tMet_3Ambushee()
{}

int32 tMet_3Ambushee::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tMet_3Ambushee::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kMissionTitle_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kEnemyRaceHex | kNeutralRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 4;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );
	Specs->fGenericFlags = kStopMetaMovement | kRequiresBase | kRequiresPlanet;
	

	const tTeamSpec& team1 = mCreateTeamSpec(	"Defender Team",
												static_cast<eTeamID>( kDefenderTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tDefenderTeam ),
												kNoTeamTag,
												Messages[kTitle_Defender_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team 1",
												static_cast<eTeamID>( kAttackerTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace | kSpecOrion | kSpecMonster,
												kAnyEnemyOf | kPrimaryTeam,
												typeid( tAttacker1Team ),
												kNoTeamTag,
												Messages[kTitle_Attacker_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Attack Team 2",
												static_cast<eTeamID>( kAttackerTeam2 ),
												kMaxChance, 
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace | kSpecOrion | kSpecMonster,
												kAnyEnemyOf | kPrimaryTeam,
												typeid( tAttacker2Team ),
												kNoTeamTag,
												Messages[kTitle_Attacker_msg] );



	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
}

int32 tMet_3Ambushee::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tMet_3Ambushee::mCreateExternalShips( int32 Mode )
{}

int32 tMet_3Ambushee::mInitializeFinished( void )
{
	return kMissionTitle_msg;
}

void tMet_3Ambushee::mNewInternalShip( tShipInfo* Ship )
{}

int32 tMet_3Ambushee::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tMet_3Ambushee::mStartMission()
{
	tScript::mStartMission();


	const int kRelationsCount = 3;
	int32 tagList[kRelationsCount] =
	{
		kDefenderTeam,
		kAttackerTeam1,
		kAttackerTeam2,
	};

	// Def = defender
	// At1 = attacker 1
	// At2 = attacker 2



	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 Def   At1   At2     
		/*Def*/  "--", "WD", "WD", 
		/*At1*/  "WD", "--", "AI", 
		/*At2*/  "WD", "AI", "--", 
	};

	fMissionInfo->mSetTeamRelations( tagList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

void tMet_3Ambushee::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kAttackerTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackerTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackerTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttackerTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackerTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttackerTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefenderTeam,  kClassFrigate, kClassBattleship, 0.5f, 1.2f );
	mSetTeamShipStrength( kDefenderTeam,  kClassFrigate, kClassBattleship, 0.5f, 1.2f );
	mSetTeamShipStrength( kDefenderTeam,  kClassFrigate, kClassBattleship, 0.5f, 1.2f );
}

void tMet_3Ambushee::mDefineBriefings( void )
{
	for( int32 nAttackRace = kFederation; nAttackRace < kMirak + 1; nAttackRace++ )
	{
		fBriefingIndex[kAttackerTeam1][nAttackRace] = kBriefing_Attacker_msg;
		fBriefingIndex[kAttackerTeam2][nAttackRace] = kBriefing_Attacker_msg;
	}

	for( int32 nDefendRace = kFederation; nDefendRace < kMirak + 1; nDefendRace++ )
	{
		fBriefingIndex[kDefenderTeam][nDefendRace] = kBriefing_Defender_msg;
	}
}

void tMet_3Ambushee::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['Z'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;
}

tMapTerrain tMet_3Ambushee::mAssignMapTerrainTypes( int32 Map )
{
	int32 kAllBlackHoles = kTerrainBlackHole1 | kTerrainBlackHole2 | kTerrainBlackHole3
						  | kTerrainBlackHole4 | kTerrainBlackHole5 | kTerrainBlackHole6;

	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace,		kPlanetAnyPlanet,	kBaseNone );
	case 1:		return tMapTerrain( kTerrainAsteroids,	kPlanetAnyPlanet,	kBaseNone );
	case 2:		return tMapTerrain( kTerrainNebula,		kPlanetAnyPlanet,	kBaseNone );
	case 3:		return tMapTerrain( kAllBlackHoles,		kPlanetAnyPlanet,	kBaseNone );
	default:	return tMapTerrain( kTerrainAnyTerrain,	kPlanetAnyPlanet,	kBaseNone );		
	}
	return tMapTerrain();
}