/********************************************************************************************************\

	File Name:	CommonVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "AttackerTeam1.h"
#include "AttackerTeam2.h"
#include "DefenderTeam.h"
#include "Met_4Ambusher.h"

//
//	Serialization events
//
tCommonVictoryState::tCommonVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tCommonVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tCommonVictoryState::mBeginState()
{}
void tCommonVictoryState::mEndState()
{}

eVictoryLevels tCommonVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tAttackerTeam1* attackerTeam1 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return kUnknownVictoryLevel;

	tAttackerTeam1* attackerTeam2 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return kUnknownVictoryLevel;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefender ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return kUnknownVictoryLevel;


	switch( ID )
	{
	case kAttacker1:
		{
			if( !attackerTeam1->mEnemiesKilled() )
				return kDevastatingDefeat;

			if( attackerTeam1->mEnemiesKilled() == attackerTeam1->mShipsLost() )
				return kDraw;

			if( ( attackerTeam1->mShipsLost() ) && ( attackerTeam1->mEnemiesKilled() > attackerTeam1->mShipsLost() ) )
				return kVictory;

			if( ( !attackerTeam1->mShipsLost() ) && ( attackerTeam1->mEnemiesKilled() ) )
				return kAstoundingVictory;
		}
		break;

	case kAttacker2:
		{
			if( !attackerTeam2->mEnemiesKilled() )
				return kDevastatingDefeat;

			if( attackerTeam2->mEnemiesKilled() == attackerTeam2->mShipsLost() )
				return kDraw;

			if( ( attackerTeam2->mShipsLost() ) && ( attackerTeam2->mEnemiesKilled() > attackerTeam2->mShipsLost() ) )
				return kVictory;

			if( ( !attackerTeam2->mShipsLost() ) && ( attackerTeam2->mEnemiesKilled() ) )
				return kAstoundingVictory;
		}
		break;

	case kDefender:
		{
			if( defenderTeam->mShipsLost() > defenderTeam->mEnemiesKilled() )
				return kDevastatingDefeat;

			if( defenderTeam->mShipsLost() == defenderTeam->mEnemiesKilled() )
				return kDraw;

			if( ( !defenderTeam->mShipsLost() ) && ( !defenderTeam->mEnemiesKilled() ) )
				return kDraw;

			if( ( defenderTeam->mShipsLost() ) && ( defenderTeam->mShipsLost() < defenderTeam->mEnemiesKilled() ) )
				return kVictory;

			if( ( !defenderTeam->mShipsLost() ) && ( defenderTeam->mEnemiesKilled() ) )
				return kAstoundingVictory;
		}
		break;
	}

	return kUnknownVictoryLevel;
}

void tCommonVictoryState::mOnDeath( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship ) return;

	tAttackerTeam1* attackerTeam1 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return;

	tAttackerTeam1* attackerTeam2 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefender ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return;

	eTeamID thisTeam = fChildActor->mGetTeam()->mGetTeam();

	if( thisTeam == ship->mGetTeam() )
	{
		switch( ship->mGetTeam() )
		{
		case kAttacker1:
			{
				attackerTeam1->mIncShipsLost();
				defenderTeam->mIncEnemiesKilled();

				if( !attackerTeam1->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker1 ), kPlayerDestroyed );
					
					if( !attackerTeam2->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerWins );
					}
				}
				
			}
			break;
			
		case kAttacker2:
			{
				attackerTeam2->mIncShipsLost();
				defenderTeam->mIncEnemiesKilled();

				if( !attackerTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker2 ), kPlayerDestroyed );
					
					if( !attackerTeam1->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerWins );
					}
				}
				
			}
			break;
			
		case kDefender:
			{
				defenderTeam->mIncShipsLost();
				attackerTeam1->mIncEnemiesKilled();
				attackerTeam2->mIncEnemiesKilled();

				if( !defenderTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerDestroyed );
					
					if( attackerTeam1->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker1 ), kPlayerWins );
					}
					if( attackerTeam2->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker2 ), kPlayerWins );
					}
				}
			}
			break;
			
		default:
			break;
		}
	}
}

void tCommonVictoryState::mOnCapture( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship ) return;

	tAttackerTeam1* attackerTeam1 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return;

	tAttackerTeam1* attackerTeam2 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefender ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	eTeamID thisTeam = fChildActor->mGetTeam()->mGetTeam();

	if( thisTeam == ship->mGetTeam() )
	{
		switch( ship->mGetOriginalTeam() )
		{
		case kAttacker1:
			{
				attackerTeam1->mIncShipsLost();
				defenderTeam->mIncEnemiesKilled();

				if( !attackerTeam1->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker1 ), kPlayerCaptured );
					
					if( !attackerTeam2->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerWins );
					}
				}
			}
			break;
			
		case kAttacker2:
			{
				attackerTeam2->mIncShipsLost();
				defenderTeam->mIncEnemiesKilled();

				if( !attackerTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker2 ), kPlayerCaptured );
					
					if( !attackerTeam1->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerWins );
					}
				}
			}
			break;
			
		case kDefender:
			{
				defenderTeam->mIncShipsLost();
				attackerTeam1->mIncEnemiesKilled();
				attackerTeam2->mIncEnemiesKilled();

				if( !defenderTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerCaptured );
					
					if( attackerTeam1->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker1 ), kPlayerWins );
					}
					if( attackerTeam2->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker2 ), kPlayerWins );
					}
				}
			}
			break;
			
		default:
			break;
		}
	}
}

void tCommonVictoryState::mOnDisengaged( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship ) return;

	tAttackerTeam1* attackerTeam1 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam1 );
	if( !attackerTeam1 ) return;

	tAttackerTeam1* attackerTeam2 = dynamic_cast<tAttackerTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAttacker1 ) ) );
	ASSERT( attackerTeam2 );
	if( !attackerTeam2 ) return;

	tDefenderTeam* defenderTeam = dynamic_cast<tDefenderTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDefender ) ) );
	ASSERT( defenderTeam );
	if( !defenderTeam ) return;


	eTeamID thisTeam = fChildActor->mGetTeam()->mGetTeam();

	if( thisTeam == ship->mGetTeam() )
	{
		switch( ship->mGetTeam() )
		{
		case kAttacker1:
			{
				attackerTeam1->mIncShipsLost();
				defenderTeam->mIncEnemiesKilled();

				if( !attackerTeam1->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker1 ), kPlayerDisengaged );
					
					if( !attackerTeam2->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerWins );
					}
				}
			}
			break;
			
		case kAttacker2:
			{
				attackerTeam2->mIncShipsLost();
				defenderTeam->mIncEnemiesKilled();

				if( !attackerTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker2 ), kPlayerDisengaged );
					
					if( !attackerTeam1->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerWins );
					}
				}
			}
			break;
			
		case kDefender:
			{
				defenderTeam->mIncShipsLost();
				attackerTeam1->mIncEnemiesKilled();
				attackerTeam2->mIncEnemiesKilled();

				if( ! defenderTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kDefender ), kPlayerDisengaged );
					
					if( attackerTeam1->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker1 ), kPlayerWins );
					}
					if( attackerTeam2->mShipsRemainingInPlay() )
					{
						gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kAttacker2 ), kPlayerWins );
					}
				}
			}
			break;
			
		default:
			break;
		}
	}
}

void tCommonVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnFlashingButtonPressed()
{}
void tCommonVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tCommonVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonVictoryState::mOnHostChanged()
{}
void tCommonVictoryState::mOnTimedInterval( int32 timerID )
{}
void tCommonVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
