/********************************************************************************************************\

	Mission:	Met_4Ambusher
	Script:		"Ambush!"
	Descript:	Teams 1 and 2 have come to bushwhack Team 3.

	Created by:		Scott Bruno
	Last Updated:	<INITIALS>		mm/dd/yy
					<LOG>


\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Met_4Ambusher.h"
#include "VictoryCondition.h"
#include "AttackerTeam1.h"
#include "AttackerTeam2.h"
#include "DefenderTeam.h"
#include "WorldConditions.h"

DEFINE_SCRIPT_CLASS( tMet_4Ambusher );

tMet_4Ambusher::tMet_4Ambusher()
{}

int32 tMet_4Ambusher::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tMet_4Ambusher::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kMissionTitle_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kPlayerRaceHex | kEnemyRaceHex | kNeutralRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 4;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );
	Specs->fGenericFlags = kNoGenericFlags | kRequiresBase | kRequiresPlanet;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Attacker Team 1",
												static_cast<eTeamID>( kAttacker1 ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tAttackerTeam1 ),
												kNoTeamTag,
												Messages[kAttackTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Attack Team 2",
												static_cast<eTeamID>( kAttacker2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace | kSpecOrion,
												kSameAs | kAttacker1,
												typeid( tAttackerTeam2 ),
												kNoTeamTag,
												Messages[kAttackTitle_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Defend Team",
												static_cast<eTeamID>( kDefender ),
												kMaxChance, 
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace | kSpecOrion,
												kAnyEnemyOf | kAttacker1,
												typeid( tDefenderTeam ),
												kNoTeamTag,
												Messages[kDefendTitle_msg] );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
}

int32 tMet_4Ambusher::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tMet_4Ambusher::mCreateExternalShips( int32 Mode )
{}

int32 tMet_4Ambusher::mInitializeFinished( void )
{
	return kMissionTitle_msg;
}

void tMet_4Ambusher::mNewInternalShip( tShipInfo* Ship )
{}

void tMet_4Ambusher::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tMet_4Ambusher::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tMet_4Ambusher::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kAttacker1,
		kAttacker2,
		kDefender,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{	
		//		  Att1	Att2  Defr
		/*Att1*/  "--", "AI", "WD",	
		/*Att2*/  "AI", "--", "WD",
		/*Defr*/  "WD", "WD", "--",	
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

void tMet_4Ambusher::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kAttacker1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttacker1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttacker1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kAttacker2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttacker2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kAttacker2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kDefender,  kClassFrigate, kClassBattleship, 0.5f, 1.2f );
	mSetTeamShipStrength( kDefender,  kClassFrigate, kClassBattleship, 0.5f, 1.2f );
	mSetTeamShipStrength( kDefender,  kClassFrigate, kClassBattleship, 0.5f, 1.2f );
}

void tMet_4Ambusher::mDefineBriefings( void )
{
	for( int32 nRace = kFederation; nRace <= kMirak; nRace++ )
	{
		fBriefingIndex[kAttacker1][nRace] = kAttack_Briefing_msg;
		fBriefingIndex[kAttacker2][nRace] = kAttack_Briefing_msg;
		fBriefingIndex[kDefender][nRace]  = kDefend_Briefing_msg;
	}
}

void tMet_4Ambusher::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['Z'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;
}

tMapTerrain tMet_4Ambusher::mAssignMapTerrainTypes( int32 Map )
{
	int32 kAllBlackHoles = kTerrainBlackHole1 | kTerrainBlackHole2 | kTerrainBlackHole3
						  | kTerrainBlackHole4 | kTerrainBlackHole5 | kTerrainBlackHole6;

	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace,		kPlanetAnyPlanet,	kBaseNone );
	case 1:		return tMapTerrain( kTerrainAsteroids,	kPlanetAnyPlanet,	kBaseNone );
	case 2:		return tMapTerrain( kTerrainNebula,		kPlanetAnyPlanet,	kBaseNone );
	case 3:		return tMapTerrain( kAllBlackHoles,		kPlanetAnyPlanet,	kBaseNone );
	default:	return tMapTerrain( kTerrainAnyTerrain,	kPlanetAnyPlanet,	kBaseNone );
	}
	return tMapTerrain();
}