/********************************************************************************************************\

	Mission:	Met_4FleetAction
	Script:		"Met_4FleetAction"
	Descript:	Fleets meet for battle.

	Created by:		Daniel J. Suleski		07/04/00
	Last Updated:	DJS						07/05/00
					Everything
					DJS						10/13/00
					Resurrecting this hellbeast...ugh -- for Beta

\********************************************************************************************************/

#include "Met_5FleetAction.h"
#include "CommonDefs.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


DEFINE_SCRIPT_CLASS( tMet_5FleetAction );

tMet_5FleetAction::tMet_5FleetAction()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tMet_5FleetAction::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fWait	= 3;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kNoGenericFlags | kRequiresBase | kRequiresPlanet;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team 1",
												static_cast< eTeamID >( kPlayerTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player Team 2",
												static_cast< eTeamID >( kPlayerTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												0.5f,
												kSpecPlayableRace | kSpecOrion | kSpecMonster,
												kAnyAllyOf | kPlayerTeam1,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Player Team 3",
												static_cast< eTeamID >( kPlayerTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												0.5f,
												kSpecPlayableRace | kSpecOrion | kSpecMonster,
												kAnyAllyOf | kPlayerTeam1,
												typeid( tCommonPlayerTeam ),
												static_cast< eTeamTag >( kPlayerSide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Enemy Team 1",
												static_cast< eTeamID >( kEnemyTeam1 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace | kSpecOrion | kSpecMonster,
												kAnyEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Enemy Team 2",
												static_cast< eTeamID >( kEnemyTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												0.5f,
												kSpecPlayableRace | kSpecOrion | kSpecMonster,
												kAnyEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Enemy Team 3",
												static_cast< eTeamID >( kEnemyTeam3 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												0.5f,
												kSpecPlayableRace | kSpecOrion | kSpecMonster,
												kAnyEnemyOf | kPlayerTeam1,
												typeid( tCommonEnemyTeam ),
												static_cast< eTeamTag >( kEnemySide ),
												Messages[kTeamTitle_msg] );

	/*

		HELPER NOTE:	Please note that although you setup the teams, you need to create ships and their
						respective files.  Otherwise, an assert will occur.  It is up to you to decide
						whether the ship files should be shared by multiple physical teams or just a
						1-to-1 relationship between the team and the ship files.

	*/
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMet_5FleetAction::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tMet_5FleetAction::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMet_5FleetAction::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMet_5FleetAction::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMet_5FleetAction::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam1,
		kPlayerTeam2,
		kPlayerTeam3,
		kEnemyTeam1,
		kEnemyTeam2,
		kEnemyTeam3,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 PTm1  PTm2  PTm3  ETm1  ETm2  ETm3
		/*PTm1*/ "AI", "AI", "AI", "WD", "WD", "WD",
		/*PTm2*/ "AI", "AI", "AI", "WD", "WD", "WD",
		/*PTm3*/ "AI", "AI", "AI", "WD", "WD", "WD",
		/*ETm1*/ "WD", "WD", "WD", "AI", "AI", "AI",
		/*ETm2*/ "WD", "WD", "WD", "AI", "AI", "AI",
		/*ETm3*/ "WD", "WD", "WD", "AI", "AI", "AI",
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMet_5FleetAction::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace, kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainAsteroids, kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainNebula, kPlanetAnyPlanet, kBaseAnyBase );
	case 3:		return tMapTerrain( kTerrainBlackholes, kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMet_5FleetAction::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['J'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['K'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['L'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tMet_5FleetAction::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		for ( int32 i = kPlayerTeam1; i <= kEnemyTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefing_msg;
		}
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tMet_5FleetAction::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	for ( int32 i = kPlayerTeam2; i <= kEnemyTeam3; i++ )
	{
		mSetTeamShipStrength( i, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
		mSetTeamShipStrength( i, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
		mSetTeamShipStrength( i, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	}
}