/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions()
{
	for ( int32 i = 0; i < kMaxMissionTeams; i++ )
	{
		fWasCaptured[i] = kFalse;
		fTeamWon[i] = kFalse;
	}
}

/*

	HELPER NOTE:	Acts as a sort of victory condition for all of the teams.  This is where you can
					analyze the game state according to the world conditions you have kept.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo* thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	This is the place that the teams that are out of the game, are going to be officially
	//	counted out.  Also, if there is only one side left, then the game will end for everyone
	//	and everyone will receive the proper game status changes.
	//
	iTruth	playerTeamHasShips = kFalse;
	for ( int32 i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
	{
		tCommonPlayerTeam* team = dynamic_cast< tCommonPlayerTeam* >( thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) ) );
		if ( team )
		{
			if ( team->mShipsCurrentlyControlled() )
				playerTeamHasShips = kTrue;
		}
	}

	iTruth	enemyTeamHasShips = kFalse;
	for ( i = kEnemyTeam1; i <= kEnemyTeam3; i++ )
	{
		tCommonEnemyTeam* team = dynamic_cast< tCommonEnemyTeam* >( thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) ) );
		if ( team )
		{
			if ( team->mShipsCurrentlyControlled() )
				enemyTeamHasShips = kTrue;
		}
	}

	iTruth	thereIsAWinner = !( playerTeamHasShips && enemyTeamHasShips );
	if ( thereIsAWinner )
	{
		for ( i = 0; i < kMaxMissionTeams; i++ )
		{
			tTeamInfo* team = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( mTeamWon( i ) )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
				}
				else
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
				}
			}
		}
	}
}


void tWorldConditions::mSetWasCaptured( int32 x, iTruth y )
{
	fWasCaptured[x] = y;
}
void tWorldConditions::mSetTeamWon( int32 x, iTruth y )
{
	fTeamWon[x] = y;
}

iTruth tWorldConditions::mWasCaptured( int32 x )
{
	return fWasCaptured[x];
}
iTruth tWorldConditions::mTeamWon( int32 x )
{
	return fTeamWon[x];
}