/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AttackTeam.h"
#include "BaseVictoryState.h"
#include "DefendTeam.h"
#include "MissionText.h"
#include "Met_8ShipAssault.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "WorldConditions.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fThisPlayerDisengaged = false;	
}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo* team = fMissionInfo->mGetTeamHandle( ID );
	Assert( team );
	if ( team )
	{
		iTruth teamHasShips = kFalse;
		if ( team->mShipsCurrentlyControlled() )
			teamHasShips = kTrue;

		//
		//	Find out which team the argument ID is on.
		//
		if( fThisPlayerDisengaged )
		{
			return kDefeat;
		}

		if ( team->mGetTag() == kAttackTag )
		{
			if( gWC.mAllOTDDestroyed() && teamHasShips )
			{
				return kVictory;
			}
			else
			{
				return kDefeat;
			}
		}
		else if ( team->mGetTag() == kDefendTag )
		{
			//
			//	Find out which teams have ships remaining in play.
			//
			iTruth	attackTeamHasShips = kFalse;		// what a fix...?!?!?!
			for ( int32 k = kAttackTeam1; k <= kAttackTeam3; k++ )
			{
				tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( k ) );
				if ( team )
				{
					if ( team->mShipsCurrentlyControlled() )
						attackTeamHasShips = kTrue;
				}
			}

			if ( !attackTeamHasShips && teamHasShips )
			{
				return kVictory;
			}
			else
			{
				return kDefeat;
			}
		}
	}

	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	if ( team != fChildActor->mGetTeam() )
		return;

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	A shipyard has been destroyed, so the game should end.
			//
			gWC.mSetOTDDestroyed( true );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeDestroyOTD );
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( ( ( int )fTeam == kAttackTeam1 ) || ( ( int )fTeam == kAttackTeam2 ) || ( ( int )fTeam == kAttackTeam3 ) )
				mAddToBonusPrestige( 50 );

			if ( team->mGetTag() == kDefendTag )
				mAddToBonusPrestige( kPrestigeDestroyShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
				gScriptInterface.mSetGameStatus( teamID, kPlayerDestroyed );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( ( ( int )fTeam == kDefendTeam1 ) || ( ( int )fTeam == kDefendTeam2 ) || ( ( int )fTeam == kDefendTeam3 ) )
				mAddToBonusPrestige( 50 );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeDestroyShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
				gScriptInterface.mSetGameStatus( teamID, kPlayerDestroyed );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	if ( team != fChildActor->mGetTeam() )
		return;

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	A shipyard has been destroyed, so the game should end.
			//
			gWC.mSetOTDDestroyed( true );

			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeCaptureOTD );
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( team->mGetTag() == kDefendTag )
				mAddToBonusPrestige( kPrestigeCaptureShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
				gScriptInterface.mSetGameStatus( teamID, kPlayerCaptured );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( team->mGetTag() == kAttackTag )
				mAddToBonusPrestige( kPrestigeCaptureShip );

			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
				gScriptInterface.mSetGameStatus( teamID, kPlayerCaptured );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	if( fTeam->mGetTeam() == ship->mGetOriginalTeam() && fTeam->mIsTeamHumanControlled() )
	{
		int32 fThisTeamShipsInPlay = team->mShipsRemainingInPlay();

		if( fThisTeamShipsInPlay < 1 )
		{
			fThisPlayerDisengaged = true;
		}
	}

	switch ( ship->mGetCustomID() )
	{
	case kOTD:
		{
			//
			//	There is no way in the world that a shipyard is going to disengage.  If it does,
			//	get a good night's rest.
			//			
		}
		break;

	case kAttacker1:
	case kAttacker2:
	case kAttacker3:
		{
			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
				gScriptInterface.mSetGameStatus( teamID, kPlayerDisengaged );
			}
		}
		break;

	case kDefender1:
	case kDefender2:
	case kDefender3:
		{
			if ( !team->mShipsCurrentlyControlled() )
			{
				fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
				gScriptInterface.mSetGameStatus( teamID, kPlayerDisengaged );
			}
		}
		break;
	}

	//
	//	Something has been killed, so check for the game end.
	//
	gWC.mCheckForGameEnd();
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
