/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "Mir_10Search.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "PaladinTeam.h"
#include "PlanetTeam.h"


tPlayerShipBaseState::tPlayerShipBaseState()
{
	fScanning = false;
}

void tPlayerShipBaseState::mBeginState()
{}
void tPlayerShipBaseState::mEndState()
{}

void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fScanning =true;
		mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );
	}

	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, kScan_timer, 0.0f );
	}
}

void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kScan_timer:
		{
			if( !fScanning )
				return;

			mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );

			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
					return;

			tTeamInfo* paladinTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPaladinTeam ) );
			if( !paladinTeam )
					return;

			tShipIterator paladinI = paladinTeam->mGetFirstShip();
			tShipInfo* paladin = *paladinI;

			tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
			if( !planetTeam )
					return;
		
			tShipIterator planetsI = planetTeam->mGetFirstShip();
			tShipInfo* planet1 = *planetsI;
			planetsI++;
			tShipInfo* planet2 = *planetsI;

			int32 rangeToPlanet = 1000;
			int32 minScanRange = 20;
			int32 ScanTargetID = fChildActor->mGetShipTargetID();

			// Scanning planet 1
			if( ( planet1 ) && ( ScanTargetID == planet1->mGetShipID() ) && ( !playerTeam->mScannedPlanet1() ) )
			{
				rangeToPlanet = fChildActor->mGetTacticalDistance( planet1 );
				if( rangeToPlanet > minScanRange ) 
				{
					if( !playerTeam->mGotScanRangeNotice() )
					{
						playerTeam->mSetGotScanRangeNotice();
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanRangeNotice_msg );
					}
					return;
				}

				playerTeam->mSetPlanet1Scanned();
				planet1->mChangeName( kHomeworldName );

				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kPlanet1Scanned_msg );
				return;
			}

			//Scanning planet 2
			if( ( planet2 ) && ( ScanTargetID == planet2->mGetShipID() ) && ( !playerTeam->mScannedPlanet2() ) )
			{
				rangeToPlanet = fChildActor->mGetTacticalDistance( planet2 );
				if( rangeToPlanet > minScanRange ) 
				{
					if( !playerTeam->mGotScanRangeNotice() )
					{
						playerTeam->mSetGotScanRangeNotice();
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanRangeNotice_msg );
					}
					return;
				}

				playerTeam->mSetPlanet2Scanned();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kPlanet2Scanned_msg );
				return;
			}

			// Scan the paladin
			if( ( paladin ) && ( ScanTargetID == paladin->mGetShipID() ) && ( !playerTeam->mScannedPaladin() ) )
			{
				playerTeam->mSetPaladinScanned();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScannedPaladin_msg );
				return;
			}
		}
		break;

	default:
		break;
	}
}

void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}

void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

