/********************************************************************************************************\

	File Name:	LyranTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "LyranTeam.h"
#include "LyranTeamBaseState.h"
#include "LyranShip.h"
#include "Mir_11Wedding.h"
#include "ShipList_API.h"

tClassAllocator<tLyranTeam> gLyranTeamAllocator;

void tLyranTeam::mSetupState()
{
	mRegisterState( new tLyranTeamBaseState() );
	mGotoState( typeid( tLyranTeamBaseState ) );
}

void tLyranTeam::mSetupCommState()
{}

void tLyranTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tLyranTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        ShipList CA_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyCruiser);
        shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CA_sl);
        }

        ShipList CC_sl = CA_sl;
        {
        std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = CC_sl.Subset(leaderRolesList);
        gST.mDebug_fprintf("subset for CC (CA leader) eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CC_sl);
        }
        if (CC_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a CC from shiplist. using CA shiplist\n");
            CC_sl = CA_sl;
        }

		ShipList CL_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CL_sl);
        }

		ShipList DD_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassDestroyer);
        shipClassTypeList.push_back(kEClassWarDestroyer);
        fShiplistCount = DD_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DD_sl);
        }

		// CC, CC, CA, CL, DD
		if (
            (CC_sl.size() > 0) &&
            (CA_sl.size() > 0) &&
            (CL_sl.size() > 0) &&
            (DD_sl.size() > 0)
        ) {
			Ship cc1, cc2, ca1, cl1, dd1;
            CC_sl.SelectRandomShip(cc1);
            gST.mDebug_fprintf("cc1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cc1.shipCode.c_str(), cc1.bpv, cc1.yearFirstAvail, cc1.yearLastAvail);
            gST.mTrackOrions( cc1 );

			CC_sl.SelectRandomShip(cc2);
            gST.mDebug_fprintf("cc2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cc2.shipCode.c_str(), cc2.bpv, cc2.yearFirstAvail, cc2.yearLastAvail);
            gST.mTrackOrions( cc2 );

			CA_sl.SelectRandomShip(ca1);
            gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

			CL_sl.SelectRandomShip(cl1);
            gST.mDebug_fprintf("cl1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cl1.shipCode.c_str(), cl1.bpv, cl1.yearFirstAvail, cl1.yearLastAvail);
            gST.mTrackOrions( cl1 );

			DD_sl.SelectRandomShip(dd1);
            gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
            gST.mTrackOrions( dd1 );

            mCreateShip(
                typeid( tLyranShip ),
                cc1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kLyran1,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

			mCreateShip(
                typeid( tLyranShip ),
                cc2.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kLyran2,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

			mCreateShip(
                typeid( tLyranShip ),
                ca1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kLyran3,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

			mCreateShip(
                typeid( tLyranShip ),
                cl1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kLyran4,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

			mCreateShip(
                typeid( tLyranShip ),
                dd1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kLyran5,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

		} else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tLyranTeam::mCreateShipsForTeam\n");
}


void tLyranTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tLyranTeam::mOldCreateShipsForTeam\n");

	// CC, CC, CA, CL, DD
	mCreateShip( typeid( tLyranShip ),
				 200,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_H,
				 kLyran1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tLyranShip ),
				 200,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_H,
				 kLyran2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tLyranShip ),
				 180,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_H,
				 kLyran3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions  );

	mCreateShip( typeid( tLyranShip ),
				 160,
				 kClassLightCruiser,
				 0,
				 kStartPosition_H,
				 kLyran4,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions  );

	mCreateShip( typeid( tLyranShip ),
				 140,
				 kClassDestroyer,
				 0,
				 kStartPosition_H,
				 kLyran5,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions  );

	gST.mDebug_fprintf("--- Finished: tLyranTeam::mOldCreateShipsForTeam\n");
}

void tLyranTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tLyranTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tLyranTeam", shipScriptDescription );
}

