/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "Mir_11Wedding.h"
#include "MissionText.h"

void tPlayerTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kLog_timer, 0.1f );
	mSetTimedInterval( kTimerTurnsSet, kSitrep_timer, 0.3f );
}

void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanning_msg );
}

void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kLog_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCaptainsLog_msg, kNonCommScriptMessage );
		}
		break;

	case kSitrep_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInitialSitrep_msg );
		}
		break;

	case kDoom_timer:
		{
			tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
			if( !allyTeam )
					return;

			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kDoom_msg );
			allyTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
			fChildActor->mSetRelations( static_cast<eTeamID>( kAllyTeam ), kTeamAtWar, kGoalDestroy, 1.0f );

			mSetTimedInterval( kTimerTurnsSet, kFixDoom_timer, 3.0f );
		}
		break;

	case kFixDoom_timer:
		{
			tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
			if( !allyTeam )
					return;

			tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
			if( !lyranTeam )
					return;

			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kFixDoom_msg );
			
			allyTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAllied, kGoalIgnore, 1.0f );
			allyTeam->mSetRelations( static_cast<eTeamID>( kLyranTeam ), kTeamAllied, kGoalIgnore, 1.0f );

			lyranTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAllied, kGoalIgnore, 1.0f );
			lyranTeam->mSetRelations( static_cast<eTeamID>( kAllyTeam ), kTeamAllied, kGoalIgnore, 1.0f );

			fChildActor->mSetRelations( static_cast<eTeamID>( kAllyTeam ), kTeamAllied, kGoalIgnore, 1.0f );
			fChildActor->mSetRelations( static_cast<eTeamID>( kLyranTeam ), kTeamAllied, kGoalIgnore, 1.0f );

			mSetTimedInterval( kTimerTurnsSet, kVictory_timer, 0.7f );
		}
		break;

	case kVictory_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kVictoryMessage );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
		}
		break;

	default:
		break;
	}
}

void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if( ( !shipTo ) || ( !shipFrom ) || ( item != kTransInfiltrator ) )
		return;

	if( ( shipTo->mGetTeam() == kPlayerTeam ) && ( !fChildActor->mHasAdmiral() ) )
	{
		shipTo->mSetShipCallbackEvents( kNotifyTransportedItem );
		fChildActor->mSetHasAdmiral( true );
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kHaveAdmiral_msg );
		return;
	}

	if( ( shipTo->mGetTeam() == kPlanetTeam ) && ( !fChildActor->mBeamedAdmiralToPlanet() ) )
	{
		fChildActor->mSetBeamedAdmiralToPlanet( kTrue );
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBeamedToPlanet_msg );
		mSetTimedInterval( kTimerTurnsSet, kDoom_timer, 2.0f );
	}

	if( shipTo->mGetTeam() == kPlayerTeam )
	{
		fChildActor->mSetBeamedAdmiralToPlanet( kFalse );
		gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
	}
}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}

void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}

void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}



