/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Mir_1Desecration.h"
#include "PlayerTeam.h"
#include "LyranTeam.h"
#include "CoffinTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam ) return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
	{
		return kDevastatingDefeat;
	}

	// No coffins were destroyed and all the Lyrans are dead. Astounding Victory.
	if ( ( playerTeam->mCoffinsDestroyed() == 0 ) && ( playerTeam->mAllLyransDead() ) )
	{
		return kAstoundingVictory;
	}

	// Less than 3 coffins were destroyed and all enemies are dead. Victory.
	else if( ( playerTeam->mCoffinsDestroyed() < 3 ) && ( playerTeam->mAllLyransDead() ) )
	{
		return kVictory;
	}

	// 3 or more coffins were destroyed. Player loses.
	else if( playerTeam->mCoffinsDestroyed() >= 3 )
	{
		return kDevastatingDefeat;
	}

	// Lyrans survived. Player loses (disengaged, probably)
	else if( !playerTeam->mAllLyransDead() ) 
	{
		return kDevastatingDefeat;
	}


	// All other conditions result in a draw
	return kDraw;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* coffinTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kCoffinTeam ) );
	if( !coffinTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	
	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

	case tCoffinTeam::kCoffin1:
	case tCoffinTeam::kCoffin2:
	case tCoffinTeam::kCoffin3:
	case tCoffinTeam::kCoffin4:
	case tCoffinTeam::kCoffin5:
	case tCoffinTeam::kCoffin6:
		{
			playerTeam->mIncrementCoffinsDestroyed();

			// Player loses if all coffins are dead
			if( !coffinTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllCoffinsDestroyed_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCoffinDestroyed_msg );
			}
		}
		break;

	case tLyranTeam::kLyran1:
	case tLyranTeam::kLyran2:
	case tLyranTeam::kLyran3:
		{
			if( !lyranTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllLyransDead_msg );
				playerTeam->mSetAllLyransDead();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLyranDies_msg );
			}
		}
		break;

	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* coffinTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kCoffinTeam ) );
	if( !coffinTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

	case tCoffinTeam::kCoffin1:
	case tCoffinTeam::kCoffin2:
	case tCoffinTeam::kCoffin3:
	case tCoffinTeam::kCoffin4:
	case tCoffinTeam::kCoffin5:
	case tCoffinTeam::kCoffin6:
		{
			playerTeam->mIncrementCoffinsDestroyed();

			// Player loses if all coffins are dead
			if( !coffinTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllCoffinsDestroyed_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCoffinDestroyed_msg );
			}
		}
		break;

	case tLyranTeam::kLyran1:
	case tLyranTeam::kLyran2:
	case tLyranTeam::kLyran3:
		{
			if( !lyranTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllLyransDead_msg );
				playerTeam->mSetAllLyransDead();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLyranDies_msg );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* coffinTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kCoffinTeam ) );
	if( !coffinTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;

	case tLyranTeam::kLyran1:
	case tLyranTeam::kLyran2:
	case tLyranTeam::kLyran3:
		{
			if( !lyranTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllLyransDead_msg );
				playerTeam->mSetAllLyransDead();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLyranDies_msg );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

