/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Mir_2Dance.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "LyranTeam.h"
#include "PulsarTeam.h"
#include "DilithiumTeam.h"
#include "MineTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}

void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kPulseWarning_timer, 27.0f );
	mSetTimedInterval( kTimerTurnsSet, kPulse_timer, 30.0f );
}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	ASSERT( playerTeam );
	if( !playerTeam ) return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	if( playerTeam->mGetAlliesEscaped() == 3 )
	{
		return kAstoundingVictory;
	}
	else if( playerTeam->mGetAlliesEscaped() == 2 )
	{
		return kVictory;
	}
	else if( playerTeam->mGetAlliesEscaped() == 1 )
	{
		return kDraw;
	}
	else if( playerTeam->mGetAlliesEscaped() == 0 )
	{
		return kDevastatingDefeat;
	}
	
	return kUnknownVictoryLevel;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
		return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
	case tAllyTeam::kAlly3:
		{
			playerTeam->mIncAlliesDead();

			if( !allyTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kAllyDestroyed );
			}
		}
		break;

		
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
			
		}
		break;

	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
	case tAllyTeam::kAlly3:
		{
			playerTeam->mIncAlliesDead();
			if( !allyTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kAllyDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{

	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;

	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
	case tAllyTeam::kAlly3:
		{
			playerTeam->mIncAlliesEscaped();

			if( !allyTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTimeToLeave_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllyDisengages_msg );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kPulseWarning_timer:
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPulsarWarning_msg );
		break;

	// Anomaly pulses. Anyone on the map is doomed.
	case kPulse_timer:
		{
		
			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>(fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) ) );
			ASSERT( playerTeam );
			if( !playerTeam ) return;

			tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kAllyTeam) );
			ASSERT( allyTeam );
			if( !allyTeam ) return;

			tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kLyranTeam) );
			ASSERT( lyranTeam );
			if( !lyranTeam ) return;

			tTeamInfo* pulsarTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPulsarTeam) );
			ASSERT( pulsarTeam );
			if( !pulsarTeam ) return;

			tShipIterator playerShips = playerTeam->mGetFirstShip();
			tShipIterator allyShips = allyTeam->mGetFirstShip();
			tShipIterator enemyShips = lyranTeam->mGetFirstShip();
			tShipIterator pulsar = pulsarTeam->mGetFirstShip();
			
			// Fire the pulsar
			if( *pulsar )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPulsarPulsing_msg );
				fMissionInfo->mPlaySound( kSndStarCollision );
				pulsar->mDestroyShip();
			}
			  

			// Anyone on the map is toast

			// Enemies die first
			while( *enemyShips )
			{
				enemyShips->mDestroyShip();
				enemyShips++;
			}

			// allies next
			while( *allyShips )
			{
				allyShips->mDestroyShip();
				allyShips++;
			}

			// now the player
			while( *playerShips )
			{
				playerShips->mDestroyShip();
				playerShips++;
			}
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

