/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "Mir_2Dance.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "LyranTeam.h"
#include "PulsarTeam.h"
#include "DilithiumTeam.h"
#include "MineTeam.h"
#include "DilithiumShip.h"
#include "MineShip.h"
#include "AllyShip.h"

tPlayerShipBaseState::tPlayerShipBaseState()
{
	fScanning = false;
}

void tPlayerShipBaseState::mBeginState()
{}
void tPlayerShipBaseState::mEndState()
{}

void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fScanning = true;
		mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );
	}

	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, kScan_timer, 0.0f );
	}
}

void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kScan_timer:
		{
			if( !fScanning )
				return;

			mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );
			int32 ScanTargetID = fChildActor->mGetShipTargetID();

			// Teams
			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
				return;

			tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
			if( !allyTeam )
				return;

			tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
			if( !lyranTeam )
				return;

			tTeamInfo* pulsarTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPulsarTeam ) );
			if( !pulsarTeam )
				return;

			tTeamInfo* dilithiumTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kDilithiumTeam ) );
			if( !dilithiumTeam )
				return;

			tTeamInfo* mineTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMineTeam ) );
			if( !mineTeam )
				return;

			// Ships
			tShipInfo* pulsar = pulsarTeam->mGetThisShip(1);

			tShipIterator alliesI = allyTeam->mGetFirstShip();
			tShipInfo* Ally1 = *alliesI;
			alliesI++;
			tShipInfo* Ally2 = *alliesI;
			alliesI++;
			tShipInfo* Ally3 = *alliesI;
		
			tShipInfo* Lyran = lyranTeam->mGetThisShip(1);

			tDilithiumShip* dBox1 = dynamic_cast<tDilithiumShip*>( dilithiumTeam->mGetThisShip(1) );
			tDilithiumShip* dBox2 = dynamic_cast<tDilithiumShip*>( dilithiumTeam->mGetThisShip(2) );
			tDilithiumShip* dBox3 = dynamic_cast<tDilithiumShip*>( dilithiumTeam->mGetThisShip(3) );
			tDilithiumShip* dBox4 = dynamic_cast<tDilithiumShip*>( dilithiumTeam->mGetThisShip(4) );
		
			tMineShip* mBox1 = dynamic_cast<tMineShip*>( mineTeam->mGetThisShip(1) );
			tMineShip* mBox2 = dynamic_cast<tMineShip*>( mineTeam->mGetThisShip(2) );
			tMineShip* mBox3 = dynamic_cast<tMineShip*>( mineTeam->mGetThisShip(3) );
			tMineShip* mBox4 = dynamic_cast<tMineShip*>( mineTeam->mGetThisShip(4) );
			tMineShip* mBox5 = dynamic_cast<tMineShip*>( mineTeam->mGetThisShip(5) );
			tMineShip* mBox6 = dynamic_cast<tMineShip*>( mineTeam->mGetThisShip(6) );
		

			/* 1-Time deals */

			// Pulsar
			if( ( pulsar ) && ( pulsar->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedPulsar() ) )
			{
				playerTeam->mSetPulsarScanned();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPulsarScanned_msg );
				return;
			}


			// Lyran
			if( ( Lyran ) && ( Lyran->mGetShipID() == ScanTargetID ) )
			{
				if( !playerTeam->mLyranRepaired() )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}

				return;
			}



			/* Allies */

			// Ally 1
			if( ( Ally1 ) && ( Ally1->mGetShipID() == ScanTargetID ) )
			{
				if( !playerTeam->mAllyRepaired(1) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}
				return;
			}
			// Ally 2
			if( ( Ally2 ) && ( Ally2->mGetShipID() == ScanTargetID ) )
			{
				if( !playerTeam->mAllyRepaired(2) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}
				return;
			}
			// Ally 3
			if( ( Ally3 ) && ( Ally3->mGetShipID() == ScanTargetID ) )
			{
				if( !playerTeam->mAllyRepaired(3) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBlank_msg );
				}
				return;
			}
		
		
			/* Dilithium boxes */

			// Dbox1
			if( ( dBox1 ) && ( dBox1->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedDBox(1) ) )
			{
				playerTeam->mSetScannedDBox(1);
				dBox1->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDilithiumDetected_msg );
				return;
			}
			// Dbox2
			if( ( dBox2 ) && ( dBox2->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedDBox(2) ) )
			{
				playerTeam->mSetScannedDBox(2);
				dBox2->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDilithiumDetected_msg );
				return;
			}
			// Dbox3
			if( ( dBox3 ) && ( dBox3->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedDBox(3) ) )
			{
				playerTeam->mSetScannedDBox(3);
				dBox3->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDilithiumDetected_msg );
				return;
			}
			// Dbox4
			if( ( dBox4 ) && ( dBox4->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedDBox(4) ) )
			{
				playerTeam->mSetScannedDBox(4);
				dBox4->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDilithiumDetected_msg );
				return;
			}


			/* Mine boxes */

			// Mbox1
			if( ( mBox1 ) && ( mBox1->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedMBox(1) ) )
			{
				playerTeam->mSetScannedMBox(1);
				mBox1->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMinesDetected_msg );
				return;
			}
			// Mbox2
			if( ( mBox2 ) && ( mBox2->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedMBox(2) ) )
			{
				playerTeam->mSetScannedMBox(2);
				mBox2->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMinesDetected_msg );
				return;
			}
			// Mbox3
			if( ( mBox3 ) && ( mBox3->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedMBox(3) ) )
			{
				playerTeam->mSetScannedMBox(3);
				mBox3->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMinesDetected_msg );
				return;
			}
			// Mbox4
			if( ( mBox4 ) && ( mBox4->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedMBox(4) ) )
			{
				playerTeam->mSetScannedMBox(4);
				mBox4->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMinesDetected_msg );
				return;
			}
			// Mbox5
			if( ( mBox5 ) && ( mBox5->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedMBox(5) ) )
			{
				playerTeam->mSetScannedMBox(5);
				mBox5->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMinesDetected_msg );
				return;
			}
			// Mbox6
			if( ( mBox6 ) && ( mBox6->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedMBox(6) ) )
			{
				playerTeam->mSetScannedMBox(6);
				mBox6->mRevealContents();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMinesDetected_msg );
				return;
			}
		}
		break;
	}
}

void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	//fChildActor->mSetShipCallbackEvents( kNotifyTransportedItem );

	
}


void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}

void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}

void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

