/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "Mir_2Dance.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fAlliesDead		= 0;
	fAlliesEscaped	= 0;
	fScannedPulsar	= kFalse;
	fScannedDBox1	= kFalse;
	fScannedDBox2	= kFalse;
	fScannedDBox3	= kFalse;
	fScannedDBox4	= kFalse;
	fScannedMBox1	= kFalse;
	fScannedMBox2	= kFalse;
	fScannedMBox3	= kFalse;
	fScannedMBox4	= kFalse;
	fScannedMBox5	= kFalse;
	fScannedMBox6	= kFalse;
	fAlly1Repaired	= kFalse;
	fAlly2Repaired	= kFalse;
	fAlly3Repaired	= kFalse;
	fLyranRepaired	= kFalse;
	fDied			= kFalse;
}

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( fDied )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_PlayerDied_msg );
		mScheduleNextMission( fVictoryCondition );
		return;
	}

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
			mScheduleNextMission( fVictoryCondition );
			return;
		}
		break;
	}


	if( mGetAlliesEscaped() == 3 )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, k3AlliesEscaped_msg );
	}
	else if( mGetAlliesEscaped() == 2 )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, k2AlliesEscaped_msg );
	}
	else if( mGetAlliesEscaped() == 1 )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, k1AlliesEscaped_msg );
	}
	else if( mGetAlliesEscaped() == 0 )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, k0AlliesEscaped_msg );
	}

	mScheduleNextMission( fVictoryCondition );
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

int32 tPlayerTeam::mGetAlliesDead()
{
	return fAlliesDead;
}

int32 tPlayerTeam::mGetAlliesEscaped()
{
	return fAlliesEscaped;
}

void tPlayerTeam::mIncAlliesDead()
{
	fAlliesDead++;
}

void tPlayerTeam::mIncAlliesEscaped()
{
	fAlliesEscaped++;
}

iTruth tPlayerTeam::mAllyRepaired( int32 ally )
{
	switch( ally )
	{
	case 1:
		return fAlly1Repaired;
		break;

	case 2:
		return fAlly2Repaired;
		break;

	case 3:
		return fAlly3Repaired;
		break;

	default:
		return kFalse;
		break;
	}
}

iTruth	tPlayerTeam::mLyranRepaired()
{
	return fLyranRepaired;
}

iTruth tPlayerTeam::mScannedDBox( int32 box )
{
	switch( box )
	{
	case 1:
		return fScannedDBox1;
		break;

	case 2:
		return fScannedDBox2;
		break;

	case 3:
		return fScannedDBox3;
		break;

	case 4:
		return fScannedDBox4;
		break;

	default:
		return kFalse;
		break;
	}
}


iTruth tPlayerTeam::mScannedMBox( int32 box )
{
	switch( box )
	{
	case 1:
		return fScannedMBox1;
		break;

	case 2:
		return fScannedMBox2;
		break;

	case 3:
		return fScannedMBox3;
		break;

	case 4:
		return fScannedMBox4;
		break;

	case 5:
		return fScannedMBox5;
		break;

	case 6:
		return fScannedMBox6;
		break;

	default:
		return kFalse;
		break;
	}
}

iTruth tPlayerTeam::mScannedPulsar()
{
	return fScannedPulsar;
}

void tPlayerTeam::mSetAllyRepaired( int32 ally )
{
	switch( ally )
	{
	case 1:
		fAlly1Repaired = kTrue;
		break;

	case 2:
		fAlly2Repaired = kTrue;
		break;

	case 3:
		fAlly3Repaired = kTrue;
		break;
	}	
}

void tPlayerTeam::mSetLyranRepaired()
{
	fLyranRepaired = kTrue;
}

void tPlayerTeam::mSetPulsarScanned()
{
	fScannedPulsar = kTrue;
}

void tPlayerTeam::mSetScannedDBox( int32 box )
{
	switch( box )
	{
	case 1:
		fScannedDBox1 = kTrue;
		break;

	case 2:
		fScannedDBox2 = kTrue;
		break;

	case 3:
		fScannedDBox3 = kTrue;
		break;

	case 4:
		fScannedDBox4 = kTrue;
		break;
	}
}

void tPlayerTeam::mSetScannedMBox( int32 box )
{
	switch( box )
	{
	case 1:
		fScannedMBox1 = kTrue;
		break;

	case 2:
		fScannedMBox2 = kTrue;
		break;

	case 3:
		fScannedMBox3 = kTrue;
		break;

	case 4:
		fScannedMBox4 = kTrue;
		break;

	case 5:
		fScannedMBox5 = kTrue;
		break;

	case 6:
		fScannedMBox6 = kTrue;
		break;
	}
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 400;
	fMissionScheduler.fNextMissionTitle = "Mir_3Tug";
	
	if( fAlliesEscaped == 3 )
	{
		fMissionScheduler.fMedal = kMedalMissionOne;
	}
}

