/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Mir_3Tug.h"
#include "PlayerTeam.h"
#include "LyranTeam.h"
#include "KlingonTeam.h"
#include "FreighterTeam.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{

}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam ) return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	// Got freighter: scanned and got logs
	if( ( playerTeam->mGotFreighter() ) && ( playerTeam->mGotLogs() ) && ( playerTeam->mScannedFreighter() ) )
	{
		return kAstoundingVictory;
	}

	// Got freighter, and scanned OR got logs
	else if( ( playerTeam->mGotFreighter() ) && ( ( !playerTeam->mGotLogs() ) || ( playerTeam->mGotLogs() ) ) )
	{
		return kVictory;
	}

	// Got freighter, no scan or logs
	else if( ( playerTeam->mGotFreighter() ) && ( !playerTeam->mGotLogs() ) && ( !playerTeam->mScannedFreighter() ) )
	{
		return kDraw;
	}

	// Everything else is a defeat
	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
					playerTeam->mSetDied();
			}
		}
		break;

	// Lose if freighter buys it
	case tFreighterTeam::kFreighter:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFreighterDies_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerLoses );
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

	// Lose if freighter is captured
	case tFreighterTeam::kFreighter:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFreighterCapped_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerLoses );
		}
		break;

	default:
		{
			ship->mClearAllCommButtons();
		}
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tLyranTeam* lyranTeam = dynamic_cast<tLyranTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) ) );
	if( !lyranTeam )
		return;

	tKlingonTeam* klingonTeam = dynamic_cast<tKlingonTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kKlingonTeam ) ) );
	if( !klingonTeam )
		return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( playerTeam->mHasFreighter() )
			{
				playerTeam->mSetGotFreighter();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;

	// The freighter has left the building: whoever is towing it wins
	case tFreighterTeam::kFreighter:
		{
			if( playerTeam->mHasFreighter() )
			{
				playerTeam->mSetGotFreighter();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPlayerGetsFreighter_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam), kPlayerWins );
			}

			else if( lyranTeam->mHasFreighter() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLyransGetFreighter_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam), kPlayerLoses );
			}

			else if( klingonTeam->mHasFreighter() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kKlingonsGetFreighter_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam), kPlayerLoses );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

