/********************************************************************************************************\

	File Name:	KlingonTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "KlingonTeamBaseState.h"
#include "Mir_3Tug.h"
#include "KlingonTeam.h"
#include "LyranTeam.h"
#include "ShipList_API.h"

void tKlingonTeamBaseState::mBeginState()
{
	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	if( !freighterTeam )
		return;

	tShipInfo* freighter = freighterTeam->mGetThisShip(1);
	if( !freighter )
		return;

	fChildActor->mSetTeamPriority( kTractor, freighter );
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}

void tKlingonTeamBaseState::mEndState()
{}

void tKlingonTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if( !tractorTarget )
		return;

	if( !tractoringShip )
		return;

	if( ( tractorTarget->mGetTeam() != kFreighterTeam ) || ( tractoringShip->mGetTeam() != kKlingonTeam ) )
	{
		return;
	}

	fChildActor->mSetHasFreighter( true );
	fChildActor->mSetTeamPriority( kDisengage );

	tLyranTeam* lyranTeam = dynamic_cast<tLyranTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) ) );
	if( !lyranTeam )
		return;

	lyranTeam->mSetTeamPriority( kKillShip, tractoringShip );
}

void tKlingonTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if( !tractorTarget )
		return;

	if( !tractoringShip )
		return;

	if( ( tractorTarget->mGetTeam() != kFreighterTeam ) || ( tractoringShip->mGetTeam() != kKlingonTeam ) )
	{
		return;
	}

	fChildActor->mSetHasFreighter( false );
	fChildActor->mSetTeamPriority( kTractor, tractorTarget );

	tLyranTeam* lyranTeam = dynamic_cast<tLyranTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) ) );
	if( !lyranTeam )
		return;

	lyranTeam->mSetTeamPriority( kTractor, tractorTarget );
}

void tKlingonTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tKlingonTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tKlingonTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tKlingonTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tKlingonTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}

void tKlingonTeamBaseState::mOnFlashingButtonPressed()
{}
void tKlingonTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tKlingonTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tKlingonTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tKlingonTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tKlingonTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tKlingonTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tKlingonTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tKlingonTeamBaseState::mOnHostChanged()
{}
void tKlingonTeamBaseState::mOnTimedInterval( int32 timerID )
{
    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        tTeamInfo* TheTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kKlingonTeam ) );
        if (TheTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        if (TheTeam->mIsTeamHumanControlled()) return;               // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tKlingonTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        int32 BPVinc = gST.mUpgradeFightersForTeam(TheTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tKlingonTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}
void tKlingonTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tKlingonTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

