/********************************************************************************************************\

	File Name:	LyranTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "LyranTeam.h"
#include "LyranTeamBaseState.h"
#include "LyranShip.h"
#include "Mir_3Tug.h"
#include "ShipList_API.h"

tClassAllocator<tLyranTeam> gLyranTeamAllocator;

tLyranTeam::tLyranTeam()
{
	fHasFreighter = false;
}

void tLyranTeam::mSetupState()
{
	mRegisterState( new tLyranTeamBaseState() );
	mGotoState( typeid( tLyranTeamBaseState ) );
}

void tLyranTeam::mSetupCommState()
{}

void tLyranTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tLyranTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		int32 nStartPos = kStartPosition_H;
		int32 nCurrentID = kLyran1;
		int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
		float nVarPercent;
        if( diffLevel == 0 )
        {
                nVarPercent = 0.65f;
        }
        else if( diffLevel == 1 )
        {
                nVarPercent = 1.0f;
        }
        else if( diffLevel == 2 )
        {
                nVarPercent = 1.25f;
        }

        for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
        {
            tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
            if( currentShip )
            {
                int32 thisShipBPV = int(currentShip->mGetShipBPV() * nVarPercent);

                // now we pick a ship with shiplist API. If it fails to pick one, use the old way.
                ShipList ship_sl = sl;
                fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), true);
                gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), fShiplistCount);
                gST.listShiplistContent(ship_sl);

                Ship s1;
                if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
                    gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                        s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                    gST.mTrackOrions( s1 );

                    mCreateShip(
                        typeid( tLyranShip ),
                        s1.shipCode.c_str(),
                        nStartPos,
                        kNoMetaShipID,
                        nCurrentID,
                        0, 0, 0, 0,
                        NULL,
                        kDefaultShipOptions );
                } else {
                    gST.mDebug_fprintf("Failed to pick ship #%d. Using old method. ShipBPV: %d\n", shipNumber, thisShipBPV);

                    eClassTypes thisShipClass = currentShip->mGetClassType();
                    mCreateShip(
                        typeid( tLyranShip ),
                        thisShipBPV,
                        thisShipClass,
                        0,
                        nStartPos,
                        nCurrentID,
                        0,0,0,0,
                        NULL,
                        kDefaultShipOptions  );
                }
                nCurrentID++;
            }
        }
    } else {
        mOldCreateShipsForTeam(); // no enhanced shiplist at all. Go to old create code. --FS
    }

    gST.mDebug_fprintf("--- Finished: tLyranTeam::mCreateShipsForTeam\n");
}


void tLyranTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tLyranTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
			return;

	int32 nStartPos = kStartPosition_H;
	int32 nCurrentID = kLyran1;
	int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;

	if( diffLevel == 0 )
	{
			nVarPercent = 0.65f;
	}
	else if( diffLevel == 1 )
	{
			nVarPercent = 1.0f;
	}
	else if( diffLevel == 2 )
	{
			nVarPercent = 1.25f;
	}

	for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
	{
		tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );

		if( currentShip )
		{
			eClassTypes thisShipClass = currentShip->mGetClassType();
			int32 thisShipBPV = currentShip->mGetShipBPV();
			mCreateShip(
				typeid( tLyranShip ), 
				thisShipBPV * nVarPercent, 
				thisShipClass, 
				0, 
				nStartPos, 
				nCurrentID, 
				0,0,0,0, 
				NULL, 
				kDefaultShipOptions );
			nCurrentID++;
		}
	}
	gST.mDebug_fprintf("---- Finished: tLyranTeam::mOldCreateShipsForTeam\n");
}

void tLyranTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tLyranTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tLyranTeam", shipScriptDescription );
}

void tLyranTeam::mSetTeamPriority( eTeamPriority priority, tShipInfo* targetShip )
{
	tShipIterator ships = mGetFirstShip();
	tShipInfo* ship1 = *ships;
	ships++;
	tShipInfo* ship2 = *ships;
	ships++;
	tShipInfo* ship3 = *ships;

	switch( priority )
	{
	case kTractor:
		{
			if( !targetShip )
				return;

			if( ship1 )
			{
				ship1->mTractorEntity( targetShip, 0, 0, kAbsolutePriority );

				if( ship2 )
				{
					ship2->mDefendEntity( ship1, 16, 2, kAbsolutePriority );
				}
				if( ship3 )
				{
					ship3->mDefendEntity( ship1, 16, 2, kAbsolutePriority );
				}
			}

			else if ( ( !ship1 ) && ( ship2 ) )
			{
				ship2->mTractorEntity( targetShip, 0, 0, kAbsolutePriority );

				if( ship3 )
				{
					ship3->mDefendEntity( ship2, 16, 2, kAbsolutePriority );
				}
			}

			else if ( ( !ship1 ) && ( !ship2 ) && ( ship3 ) )
			{
				ship3->mTractorEntity( targetShip, 0, 0, kAbsolutePriority );
			}

			else return;
		}
		break;

	case kDisengage:
		{
			if( ship1 )
			{
				ship1->mDisengage( kByMapEdge, kAbsolutePriority );

				if( ship2 )
				{
					ship2->mDefendEntity( ship1, 16, 2, kAbsolutePriority );
				}
				if( ship3 )
				{
					ship3->mDefendEntity( ship1, 16, 2, kAbsolutePriority );
				}
			}

			else if ( ( !ship1 ) && ( ship2 ) )
			{
				ship2->mDisengage( kByMapEdge, kAbsolutePriority );

				if( ship3 )
				{
					ship3->mDefendEntity( ship2, 16, 2, kAbsolutePriority );
				}
			}

			else if ( ( !ship1 ) && ( !ship2 ) && ( ship3 ) )
			{
				ship3->mDisengage( kByMapEdge, kAbsolutePriority );
			}

			else return;
		}
		break;

	case kKillShip:
		{
			if( !targetShip )
				return;

			if( ship1 )
			{
				ship1->mSetShipTarget( targetShip, kAbsolutePriority );
			}

			if( ship2 )
			{
				ship2->mSetShipTarget( targetShip, kAbsolutePriority );
			}

			if( ship3 )
			{
				ship3->mSetShipTarget( targetShip, kAbsolutePriority );
			}
		}
		break;

	default:
		break;
	}
}

			
			
iTruth tLyranTeam::mHasFreighter()
{
	return fHasFreighter;
}

void tLyranTeam::mSetHasFreighter( iTruth x )
{
	fHasFreighter = x;
}
