/********************************************************************************************************\

	File Name:	LyranTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "LyranTeamBaseState.h"
#include "Mir_3Tug.h"
#include "LyranTeam.h"
#include "KlingonTeam.h"
#include "ShipList_API.h"

void tLyranTeamBaseState::mBeginState()
{
	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	if( !freighterTeam )
		return;

	tShipInfo* freighter = freighterTeam->mGetThisShip(1);
	if( !freighter )
		return;

	fChildActor->mSetTeamPriority( kTractor, freighter );

	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}

void tLyranTeamBaseState::mEndState()
{}

void tLyranTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if( !tractorTarget )
		return;

	if( !tractoringShip )
		return;

	if( ( tractorTarget->mGetTeam() != kFreighterTeam ) || ( tractoringShip->mGetTeam() != kLyranTeam ) )
	{
		return;
	}

	fChildActor->mSetHasFreighter( true );
	fChildActor->mSetTeamPriority( kDisengage );

	tKlingonTeam* klingonTeam = dynamic_cast<tKlingonTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kKlingonTeam ) ) );
	if( !klingonTeam )
		return;

	klingonTeam->mSetTeamPriority( kKillShip, tractoringShip );
}

void tLyranTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if( !tractorTarget )
		return;

	if( !tractoringShip )
		return;

	if( ( tractorTarget->mGetTeam() != kFreighterTeam ) || ( tractoringShip->mGetTeam() != kLyranTeam ) )
	{
		return;
	}

	fChildActor->mSetHasFreighter( false );
	fChildActor->mSetTeamPriority( kTractor, tractorTarget );

	tKlingonTeam* klingonTeam = dynamic_cast<tKlingonTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kKlingonTeam ) ) );
	if( !klingonTeam )
		return;

	klingonTeam->mSetTeamPriority( kTractor, tractorTarget );
}

void tLyranTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tLyranTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tLyranTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tLyranTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tLyranTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tLyranTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tLyranTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tLyranTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tLyranTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tLyranTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tLyranTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tLyranTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}

void tLyranTeamBaseState::mOnFlashingButtonPressed()
{}
void tLyranTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tLyranTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tLyranTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tLyranTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tLyranTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tLyranTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tLyranTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tLyranTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tLyranTeamBaseState::mOnHostChanged()
{}
void tLyranTeamBaseState::mOnTimedInterval( int32 timerID )
{
    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        tTeamInfo* TheTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kLyranTeam ) );
        if (TheTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        if (TheTeam->mIsTeamHumanControlled()) return;               // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tLyranTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        int32 BPVinc = gST.mUpgradeFightersForTeam(TheTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tLyranTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}
void tLyranTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tLyranTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

