/********************************************************************************************************\

	File Name:	MonsterBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "MonsterBaseState.h"
#include "ScriptInterface.h"
#include "Mir_4Playtime.h"
#include "PlayerTeam.h"
#include "MissionText.h"
#include "MonsterTeam.h"
#include "CommonDefs.h"

tMonsterBaseState::tMonsterBaseState()
{
	fTimeout = false;
	fFeeding = false;
}

void tMonsterBaseState::mBeginState()
{
}

void tMonsterBaseState::mEndState()
{}

void tMonsterBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{

}

void tMonsterBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	fChildActor->mSetShipCallbackEvents( kNotifyDamage );

	if( ( !damagedShip ) || ( !hostileShip ) )
		return;

	if( ( damagedShip->mGetTeam() != kMonsterTeam ) || ( hostileShip->mGetTeam() != kPlayerTeam ) )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tMonsterTeam* monsterTeam = dynamic_cast<tMonsterTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMonsterTeam ) ) );
	if( !monsterTeam )
		return;

	// Don't bother if player is already at war with the monsters
	if( playerTeam->mAtWarWithMonsters() )
		return;

	// All hell breaks loose if the player fires on a monster during a "time out"
	if( ( fTimeout ) && ( !playerTeam->mAtWarWithMonsters() ) )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMonstersGoToWar_msg );
		monsterTeam->mSetTeamPriority( kKillPlayer );
		playerTeam->mSetAtWarWithMonsters();
		return;
	}


	float damage = damagedShip->mGetDamagePercent();
	if( damage < 50.0f )
		return;

	// a Monster is 50% or more damaged.

	if( monsterTeam->mCurrentPriority() != kFeed )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMonstersTimeout_msg );
	}
	monsterTeam->mSetTeamPriority( kFeed );

	// Set a 5-second timer to ignore any "extra" damage
	mSetTimedInterval( kTimerSecondsSet, kIngoreDamage_timer, 5.0f );
}

void tMonsterBaseState::mAtDestination( tShipInfo* ship )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	if( !playerTeam->mMonstersFeeding() )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMonstersFeeding_msg );
	}
	playerTeam->mSetMonstersFeeding( true );

	mSetTimedInterval( kTimerTurnsSet, kFeed_timer, 1.5f );
}

void tMonsterBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kFeed_timer:
		{
			tMonsterTeam* monsterTeam = dynamic_cast<tMonsterTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMonsterTeam ) ) );
			if( !monsterTeam )
				return;
			
			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
				return;

			fChildActor->mSetSystemHealthFromOrig( kNoWeaponHardPoint, 1.0f, true );

			if( monsterTeam->mCurrentPriority() != kPlay )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMonstersDoneFeeding_msg );
			}
			monsterTeam->mSetTeamPriority( kPlay );

			fTimeout = false;
			playerTeam->mSetMonstersFeeding( false );
		}
		break;

	case kIngoreDamage_timer:
		{
			fTimeout = true;
		}
		break;

	default:
		break;
	}
}


void tMonsterBaseState::mOnDeath( tShipInfo* ship )
{}
void tMonsterBaseState::mOnCapture( tShipInfo* ship )
{}
void tMonsterBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tMonsterBaseState::mAtDistance( tShipInfo* ship )
{}

void tMonsterBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}

void tMonsterBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tMonsterBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tMonsterBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tMonsterBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tMonsterBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tMonsterBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tMonsterBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tMonsterBaseState::mOnFlashingButtonPressed()
{}
void tMonsterBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tMonsterBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tMonsterBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tMonsterBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}

void tMonsterBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tMonsterBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tMonsterBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tMonsterBaseState::mOnHostChanged()
{}

void tMonsterBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tMonsterBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

