/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "Mir_4Playtime.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "MonsterTeam.h"
#include "PlanetTeam.h"
#include "AsteroidTeam.h"


tPlayerShipBaseState::tPlayerShipBaseState()
{
	fScanning = false;
}

void tPlayerShipBaseState::mBeginState()
{}

void tPlayerShipBaseState::mEndState()
{}


void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fScanning = true;
		mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );
	}

	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, kScan_timer, 0.0f );
	}
}

void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kScan_timer:
		{
			if( !fScanning )
				return;

			mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );

			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
				return;

			tMonsterTeam* monsterTeam = dynamic_cast<tMonsterTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMonsterTeam ) ) );
			if( !monsterTeam )
				return;

			tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
			if( !planetTeam )
				return;

			tTeamInfo* asteroidTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAsteroidTeam ) );
			if( !asteroidTeam )
				return;
	
			tShipIterator monsters = monsterTeam->mGetFirstShip();
			tShipInfo* monster1 = *monsters;
			monsters++;
			tShipInfo* monster2 = *monsters;

			tShipIterator rocks = asteroidTeam->mGetFirstShip();
			tShipInfo* specialRock = *rocks;
			tShipInfo* planet = planetTeam->mGetThisShip(1);


			int32 ScanTargetID = fChildActor->mGetShipTargetID();

			// Scan the planet
			if( ( planet ) && ( planet->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedPlanet() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedPlanet_msg );
				playerTeam->mSetPlanetScanned();
				return;
			}


			// Scan special rock
			if( ( specialRock ) && ( specialRock->mGetShipID() == ScanTargetID ) && ( !playerTeam->mScannedSpecialRock() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedSpecialRock_msg );
				playerTeam->mSetSpecialRockScanned();
				return;
			}

			// Scan monster 1 while not feeding
			if( ( monster1 ) && ( monster1->mGetShipID() == ScanTargetID ) && ( playerTeam->mMonstersFeeding() ) && ( !playerTeam->mScannedMonster1() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedMonster1_msg );
				playerTeam->mSetMonster1Scanned();
				return;
			}

			// Scan monster 1 while feeding
			if( ( monster1 ) && ( monster1->mGetShipID() == ScanTargetID ) && ( playerTeam->mMonstersFeeding() ) && ( !playerTeam->mScannedFeedingMonster() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedFeedingMonster_msg );
				playerTeam->mSetFeedingMonsterScanned();
				return;
			}

			// Scan monster 2 while not feeding
			if( ( monster2 ) && ( monster2->mGetShipID() == ScanTargetID ) && ( playerTeam->mMonstersFeeding() ) && ( !playerTeam->mScannedMonster2() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedMonster2_msg );
				playerTeam->mSetMonster2Scanned();
				return;
			}

			// Scan monster 2 while feeding
			if( ( monster2 ) && ( monster2->mGetShipID() == ScanTargetID ) && ( playerTeam->mMonstersFeeding() ) && ( !playerTeam->mScannedFeedingMonster() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedFeedingMonster_msg );
				playerTeam->mSetFeedingMonsterScanned();
				return;
			}
		}
		break;

	default:
		break;
	}
}

void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	
}

void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	
}

void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}

void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}

void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}

void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

