/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fPlayedWithMonsters = false;
	fAtWarWithMonsters = false;
	fScannedRock1 = false;
	fScannedRock2 = false;
	fScannedSpecialRock = false;
	fScannedMonster1 = false;
	fScannedMonster2 = false;
	fScannedFeedingMonster = false;
	fScannedPlanet = false;
	fMonstersFeeding = false;
	fBeamedToPlanet = false;
	fBeamedToRock = false;
	fBeamedSomewhereStupid = false;
	fDied = false;
}

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( fDied )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_PlayerDied_msg );
		mScheduleNextMission( fVictoryCondition );
		return;
	}

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
			mScheduleNextMission( fVictoryCondition );
			return;
		}
		break;
	}

	// Scanned rock and beamed to rock
	if( ( fScannedSpecialRock ) && ( fBeamedToRock ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_Scanned_And_Rock_msg );
	}

	// Beamed to rock but didn't scan rock
	else if( ( !fScannedSpecialRock ) && ( fBeamedToRock ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_NoScan_Rock_msg );
	}

	// Beamed to planet 
	else if( fBeamedToPlanet )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_NoScan_Planet_msg );
	}

	// Beamed team somewhere dumb
	else if( fBeamedSomewhereStupid )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BeamedStupid_msg );
	}

	// Didn't beam anywhere
	else if( ( !fBeamedToPlanet ) && ( !fBeamedToRock ) && ( !fBeamedSomewhereStupid ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BeamedNowhere_msg );
	}

	mScheduleNextMission( fVictoryCondition );
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{

}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}


void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 400;
	fMissionScheduler.fNextMissionTitle = "Mir_5AFunny1";
}