/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "MissionText.h"
#include "Mir_4Playtime.h"
#include "MissionText.h"
#include "Monster.h"
#include "MonsterTeam.h"

void tPlayerTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kLog_timer, 0.1f );
	mSetTimedInterval( kTimerTurnsSet, kSitrep_timer, 0.3f );
}

void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kLog_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCaptainsLog_msg, kNonCommScriptMessage );
		}
		break;

	case kSitrep_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kSitrep_msg );
			
			tTeamInfo* monsterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMonsterTeam ) );
			if( !monsterTeam )
				return;

			tShipIterator monsters = monsterTeam->mGetFirstShip();
			tShipInfo* monster1 = *monsters;
			monsters++;
			tShipInfo* monster2 = *monsters;

			if( monster1 )
			{
				fChildActor->mGotoCommState( monster1->mGetShipID(), tMonster::kState_1 );
			}
			if( monster2 )
			{
				fChildActor->mGotoCommState( monster2->mGetShipID(), tMonster::kState_1 );
			}
		}
		break;

	default:
		break;
	}
}

void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScanning_msg );
	}
}

void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if( ( !shipTo ) || ( !shipFrom ) )
		return;

	if( shipFrom->mGetTeam() != kPlayerTeam ) 
		return;

	if( item != kTransScientists )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
		return;

	tTeamInfo* asteroidTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAsteroidTeam ) );
	if( !asteroidTeam )
		return;

	tShipIterator rocks = asteroidTeam->mGetFirstShip();
	tShipInfo* specialRock = *rocks;
	tShipInfo* rock1 = *rocks;
	tShipInfo* rock2 = *rocks;
	tShipInfo* planet = planetTeam->mGetThisShip(1);

	
	if( planet )
	{
		
		// Beam to planet
		if( shipTo->mGetShipID() == planet->mGetShipID() )
		{
			playerTeam->mSetBeamedToPlanet();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBeamToPlanet_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			return;
		}
	}
	
	if( specialRock )
	{
		
		// Beam to special asteroid
		if ( shipTo->mGetShipID() == specialRock->mGetShipID() )
		{
			playerTeam->mSetBeamedToRock();
			
			if( fChildActor->mScannedSpecialRock() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBeamToSpecialRock_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				playerTeam->mSetBeamedSomewhereStupid();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBeamSomewhereStupid_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}
			return;	
		}
	}

	else
	{
		playerTeam->mSetBeamedSomewhereStupid();
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBeamSomewhereStupid_msg );
		gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
	}
}

void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	tMonsterTeam* monsterTeam = dynamic_cast<tMonsterTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMonsterTeam ) ) );
	ASSERT( monsterTeam );
	if( !monsterTeam ) return;


	monsterTeam->mSetTeamPriority( kPlay );

	tShipIterator monsters = monsterTeam->mGetFirstShip();
	tShipInfo* monster1 = *monsters;
	monsters++;
	tShipInfo* monster2 = *monsters;

	if( monster1 )
	{
		monster1->mClearAllCommButtons();
	}
	if( monster2 )
	{
		monster2->mClearAllCommButtons();
	}
}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}

void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}

void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}

void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}



