/********************************************************************************************************\

	Mission:	"What's So Funny? Part I"
	Script:		"Mir_5AFunny1"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		08/11/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>
					FS				10/05/2012
					Era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "Mir_5AFunny1.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "BaseTeam.h"
#include "PlanetTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tMir_5AFunny1 );

tMir_5AFunny1::tMir_5AFunny1()
{}

int32 tMir_5AFunny1::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tMir_5AFunny1::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Base Team",
												static_cast< eTeamID >( kBaseTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tBaseTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMir_5AFunny1::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tMir_5AFunny1::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMir_5AFunny1::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMir_5AFunny1::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tMir_5AFunny1::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tMir_5AFunny1::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tMir_5AFunny1::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kBaseTeam,
		kPlanetTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Nme  = kEnemyTeam,
		//Base = kBaseTeam,
		//Plan = kPlanetTeam

		//	     Plyr  Nme   Base   Plan
		/*Plyr*/ "--", "WD", "AI", "AI",
		/*Nme*/  "WD", "--", "WD", "WI",
		/*Base*/ "AI", "WD", "--", "AI",
		/*Plan*/ "AI", "WI", "AI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tMir_5AFunny1::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlanet;
	Map->fTacticalMapIcon['I'] = kObjectPlayerBase;
}

tMapTerrain tMir_5AFunny1::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tMir_5AFunny1::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tMir_5AFunny1::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kMirak] = kBriefing_msg;
}