/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "MissionText.h"
#include "Mir_6Wheel.h"
#include "EnemyTeam.h"
#include "ShipList_API.h"


void tPlayerTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kLog_timer, 0.1f );
	mSetTimedInterval( kTimerTurnsSet, kSitrep_timer, 0.3f );
}

void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kLog_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCaptainsLog_msg, kNonCommScriptMessage );
		}
		break;

	case kSitrep_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInitialSitrep_msg );
		}
		break;

	case kExplanation_timer:
		{
			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
			if( !playerTeam )
					return;

			tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );
			if( !enemyTeam )
					return;

			tShipIterator enemyShipsI = enemyTeam->mGetFirstShip();
			for( int32 curShip = 0; curShip < enemyTeam->mGetShipsInPlayCount(); curShip++ )
			{
				tShipInfo* thisShip = *enemyShipsI;
				if( thisShip )
				{
					playerTeam->mGotoCommState( thisShip->mGetShipID(), tEnemyTeam::kCommState_3 );
				}
				enemyShipsI++;
			}
		}
		break;

	default:
		break;
	}
}

void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanning_msg );
	}
}

void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if( !targetShip )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );
	if( !enemyTeam )
			return;

	tTeamInfo* wheelTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kWheelTeam ) );
	if( !wheelTeam ) {
		gST.mDebug_fprintf("FAIL: calling up wheelTeam object before any ships is bad.\n");
		return;
	}

	switch( buttonID )
	{
	case kButton_Hail:
		{
			// Freighter
			if( targetShip->mGetTeam() == kFreighterTeam )
			{
				// First hail
				if( !playerTeam->mHailedFreighter() ) 
				{
					playerTeam->mSetHailedFreighter();
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kHailFreighter_1_msg );
					return;
				}
				// Subsequent hails
				else
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kHailFreighter_2_msg );
					return;
				}
			}

			// Enemy ship
			if( targetShip->mGetTeam() == kEnemyTeam )
			{
				// if the freighter is scanned and hailed
				if( ( playerTeam->mHailedFreighter() ) && ( playerTeam->mScannedFreighter() ) )
				{
					tShipIterator enemyShipsI = enemyTeam->mGetFirstShip();
					for( int32 curShip = 0; curShip < enemyTeam->mGetShipsInPlayCount(); curShip++ )
					{
						tShipInfo* thisShip = *enemyShipsI;
						if( thisShip )
						{
							playerTeam->mGotoCommState( thisShip->mGetShipID(), tEnemyTeam::kCommState_2 );
						}
						enemyShipsI++;
					}
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kEnemyTalks_msg );
					return;
				}
				// Otherwise
				else
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kEnemyDoesntTalk_msg );
					return;
				}
			}
		}
		break;

	case kButton_Explain:
		{
			playerTeam->mSetGotExplanation();
			mSetTimedInterval( kTimerSecondsSet, kExplanation_timer, 5.0f );
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kExplain_1_msg );
		}
		break;

	case kButton_Yes:
		{
			playerTeam->mSetMadeAlliance();
			playerTeam->mSetRelations( static_cast<eTeamID>( kEnemyTeam ), kTeamAllied, kGoalIgnore, 1.0f );
			enemyTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAllied, kGoalIgnore, 1.0f );

			wheelTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
			playerTeam->mSetRelations( static_cast<eTeamID>( kWheelTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
			
			tShipIterator enemyShipsI = enemyTeam->mGetFirstShip();
			for( int32 curShip = 0; curShip < enemyTeam->mGetShipsInPlayCount(); curShip++ )
			{
				tShipInfo* thisShip = *enemyShipsI;
				if( thisShip )
				{
					thisShip->mClearAllCommButtons();
				}
				enemyShipsI++;
			}

			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kMakeAlliance_msg );
		}
		break;

	case kButton_No:
		{
			enemyTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
			playerTeam->mSetRelations( static_cast<eTeamID>( kEnemyTeam ), kTeamAtWar, kGoalDestroy, 1.0f );

			tShipIterator enemyShipsI = enemyTeam->mGetFirstShip();
			for( int32 curShip = 0; curShip < enemyTeam->mGetShipsInPlayCount(); curShip++ )
			{
				tShipInfo* thisShip = *enemyShipsI;
				if( thisShip )
				{
					thisShip->mClearAllCommButtons();
				}
				enemyShipsI++;
			}

			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNoAlliance_msg );
		}
		break;

	default:
		break;
	}
}


void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}

void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}
void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}

void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}



