/********************************************************************************************************\

	File Name:	WheelTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "WheelTeam.h"
#include "WheelTeamBaseState.h"
#include "WheelShip.h"
#include "ShipList_API.h"


tClassAllocator<tWheelTeam> gWheelTeamAllocator;

void tWheelTeam::mSetupState()
{
	mRegisterState( new tWheelTeamBaseState() );
	mGotoState( typeid( tWheelTeamBaseState ) );
}

void tWheelTeam::mSetupCommState()
{}

void tWheelTeam::mCreateShipsForTeam()
{}

void tWheelTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tWheelTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tWheelTeam", shipScriptDescription );
}


// It is my opinion that if it cannot get the national guard ships, it should just go ahead 
// and use a F-CA and F-DD pair. -- FS, 20121009
void tWheelTeam::mCreateReinforcements()
{
    gST.mDebug_fprintf("--- Entering: tWheelTeam::mCreateReinforcements\n");
	gST.mInitializeSL();
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutRestricted( sl );
		// don't strip out special if we want local_defense national guard ships

        ShipList CA_sl = sl;
		{
        std::list< eEnhancedClassTypes >  CAClassList;
		for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++)
		{
			CAClassList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
        fShiplistCount = CA_sl.Subset(CAClassList);
        gST.mDebug_fprintf("CA class subset: %d\n", fShiplistCount);
		}

		ShipList DD_sl = sl;
		{
        std::list< eEnhancedClassTypes >  DDClassList;
		for (int32 i = kEClassDestroyer; i <= kEClassWarDestroyer; i++)
		{
			DDClassList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
        fShiplistCount = DD_sl.Subset(DDClassList);
        gST.mDebug_fprintf("DD class subset: %d\n", fShiplistCount);
		}

		{
        std::list< eEnhancedRoles >  LDRolesList;
        LDRolesList.push_back(kLocalDefense);
        fShiplistCount = CA_sl.Subset(LDRolesList);
        gST.mDebug_fprintf("CA Local Defense roles subset: %d\n", fShiplistCount);
        gST.listShiplistContent(CA_sl);

		fShiplistCount = DD_sl.Subset(LDRolesList);
        gST.mDebug_fprintf("DD Local Defense roles subset: %d\n", fShiplistCount);
        gST.listShiplistContent(DD_sl);
		}

        if ((CA_sl.size() > 0) && (DD_sl.size() > 0)) 
		{
            Ship ca1, ca2, dd1, dd2;
            CA_sl.SelectRandomShip(ca1);
            gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

			CA_sl.SelectRandomShip(ca2);
            gST.mDebug_fprintf("ca2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca2.shipCode.c_str(), ca2.bpv, ca2.yearFirstAvail, ca2.yearLastAvail);
            gST.mTrackOrions( ca2 );

			DD_sl.SelectRandomShip(dd1);
            gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
            gST.mTrackOrions( dd1 );

			DD_sl.SelectRandomShip(dd2);
            gST.mDebug_fprintf("dd2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd2.shipCode.c_str(), dd2.bpv, dd2.yearFirstAvail, dd2.yearLastAvail);
            gST.mTrackOrions( dd2 );

			mCreateShip( typeid( tWheelShip ),
						 ca1.shipCode.c_str(),
						 kStartPosition_J,
						 kNoMetaShipID,
						 kWheel1,
						 0,0,0,0,
						 "Cult Ship",
						 kDefaultShipOptions );

			mCreateShip( typeid( tWheelShip ),
						 ca2.shipCode.c_str(),
						 kStartPosition_J,
						 kNoMetaShipID,
						 kWheel2,
						 0,0,0,0,
						 "Cult Ship",
						 kDefaultShipOptions );

			mCreateShip( typeid( tWheelShip ),
						 dd1.shipCode.c_str(),
						 kStartPosition_J,
						 kNoMetaShipID,
						 kWheel3,
						 0,0,0,0,
						 "Cult Ship",
						 kDefaultShipOptions );

			mCreateShip( typeid( tWheelShip ),
						 dd2.shipCode.c_str(),
						 kStartPosition_J,
						 kNoMetaShipID,
						 kWheel4,
						 0,0,0,0,
						 "Cult Ship",
						 kDefaultShipOptions );
        } else {
            mOldCreateReinforcements();
        }
    } else {
        mOldCreateReinforcements();
    }
    gST.mDebug_fprintf("--- Finished: tWheelTeam::mCreateReinforcements\n");
}

void tWheelTeam::mOldCreateReinforcements()
{
	gST.mDebug_fprintf("--- Starting: tWheelTeam::mOldCreateReinforcements\n");

	mCreateShip( typeid( tWheelShip ),
				 "F-CA",
				 kStartPosition_J,
				 kNoMetaShipID,
				 kWheel1,
				 0,0,0,0,
				 "Cult Ship",
				 kDefaultShipOptions );

	mCreateShip( typeid( tWheelShip ),
				 "F-DD",
				 kStartPosition_J,
				 kNoMetaShipID,
				 kWheel2,
				 0,0,0,0,
				 "Cult Ship",
				 kDefaultShipOptions );

	mCreateShip( typeid( tWheelShip ),
				 "F-DD",
				 kStartPosition_J,
				 kNoMetaShipID,
				 kWheel3,
				 0,0,0,0,
				 "Cult Ship",
				 kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tWheelTeam::mOldCreateReinforcements\n");
}
