/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Mir_7Gates.h"
#include "PlayerTeam.h"
#include "EnemyTeam1.h"
#include "EnemyTeam2.h"
#include "PlanetTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kBorderCheck_timer, 0.5f );
}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	if( playerTeam->mSurrendered() )
		return kAstoundingVictory;

	// All other conditions are failures
	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kBorderCheck_timer:
		{
			mSetTimedInterval( kTimerSecondsSet, kBorderCheck_timer, 0.5f );

			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
			if( !playerTeam )
					return;

			tShipIterator playerShipsI = playerTeam->mGetFirstShip();
			for( int32 curShip = 1; curShip < 4; curShip++ )
			{
				tShipInfo* thisShip = *playerShipsI;
				if( thisShip )
				{
					int32 mapMaxX, mapMaxY;
					int32 positionX, positionY;
					fMissionInfo->mGetMapDimensions( &mapMaxX, &mapMaxY);
					thisShip->mGetGridPosition( &positionX, &positionY );
					
					if( ( positionX == 0 ) || ( positionY == 0 ) )
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kDeadEnd_msg );
						thisShip->mDestroyShip();
					}
					if( ( positionX == mapMaxX ) || ( positionY == mapMaxY ) )
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kDeadEnd_msg );
						thisShip->mDestroyShip();
					}
				}
				playerShipsI++;
			}
		}
		break;

	default:
		break;
	}
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	ASSERT( ship );
	if ( !ship ) return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	ASSERT( playerTeam );
	if( !playerTeam ) return;

	tEnemyTeam1* enemyTeam1 = dynamic_cast<tEnemyTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam1 ) ) );
	ASSERT( enemyTeam1 );
	if( !enemyTeam1 ) return;

	tEnemyTeam2* enemyTeam2 = dynamic_cast<tEnemyTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam2 ) ) );
	ASSERT( enemyTeam2 );
	if( !enemyTeam2 ) return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

	case tPlanetTeam::kPlanet:
		{
			playerTeam->mKillTeam();
		}
		break;

	case tEnemyTeam1::kEnemy1_1:
	case tEnemyTeam1::kEnemy1_2:
	case tEnemyTeam1::kEnemy1_3:
		{
			enemyTeam1->mRespawnShip( ship->mGetCustomID(), ship->mGetClassName() );

			if( !playerTeam->mNoticedRespawn() )
			{
				playerTeam->mSetNoticedRespawn();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNoticeRespawn_msg );
			}
		}
		break;

	case tEnemyTeam2::kEnemy2_1:
	case tEnemyTeam2::kEnemy2_2:
	case tEnemyTeam2::kEnemy2_3:
		{
			enemyTeam2->mRespawnShip( ship->mGetCustomID(), ship->mGetClassName() );

			if( !playerTeam->mNoticedRespawn() )
			{
				playerTeam->mSetNoticedRespawn();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNoticeRespawn_msg );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

	case tPlanetTeam::kPlanet:
		break;

	default:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kShipCapped_msg );
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

