/********************************************************************************************************\

	File Name:	CommonDefs.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "EnemyTeam1.h"
#include "EnemyTeam2.h"
#include "ShipList_API.h"

common_defs CD;

common_defs::setup()
{
	for (int32 i = 0; i <= 7; i++)
	{
		first_spawn_complete[i] = false;
	}

	custom_ID_map[0] = tEnemyTeam1::kEnemy1_1;
	custom_ID_map[1] = tEnemyTeam1::kEnemy1_2;
	custom_ID_map[2] = tEnemyTeam1::kEnemy1_3;
	custom_ID_map[3] = tEnemyTeam2::kEnemy2_1;
	custom_ID_map[4] = tEnemyTeam2::kEnemy2_2;
	custom_ID_map[5] = tEnemyTeam2::kEnemy2_3;

	previous_fighter_selection[6] = "N-SH";
}

void common_defs::mSetFighterSelection( int32 customID, std::string fighter_selection )
{
	gST.mDebug_fprintf("---- Entering common_defs::mSetFighterSelection\n");

	previous_fighter_selection[ mGetCustomID_map(customID) ] = fighter_selection;
	first_spawn_complete[ mGetCustomID_map(customID) ] = true;

	gST.mDebug_fprintf("---- Finished common_defs::mSetFighterSelection\n");
}


std::string common_defs::mGetFighterSelection( int32 customID )
{
	gST.mDebug_fprintf("---- Entering common_defs::mGetFighterSelection\n");

	std::string selected_fighter = previous_fighter_selection[ mGetCustomID_map(customID) ].c_str();

	gST.mDebug_fprintf("---- Finished common_defs::mGetFighterSelection\n");
	return selected_fighter;
}


int32 common_defs::mGetCustomID_map( int32 customID ) 
{
	switch(customID)
	{
	case tEnemyTeam1::kEnemy1_1: return 0; break;
	case tEnemyTeam1::kEnemy1_2: return 1; break;
	case tEnemyTeam1::kEnemy1_3: return 2; break;
	case tEnemyTeam2::kEnemy2_1: return 3; break;
	case tEnemyTeam2::kEnemy2_2: return 4; break;
	case tEnemyTeam2::kEnemy2_3: return 5; break;
	};
	return 6;
}

bool common_defs::mGetFightersAlreadyDone( int32 customID )
{
	return first_spawn_complete[ mGetCustomID_map(customID) ];
}

int32 common_defs::mCustomUpgradeFighters( tShipInfo *ship )
{
	gST.mDebug_fprintf("---- Entering: common_defs::mCustomUpgradeFighters\n");

#ifdef FIGHTERS
	if (! ship) return 0;

	// compensate for the orion number chomping silliness.
	std::string fShipClassName = gST.mGetTrackedOrion(ship->mGetClassName());
	gST.mCompleteNarrowedList(); // just in case;

    Ship s1;
	gST.mDebug_fprintf("---- Checking for fighters on: %s.\n", fShipClassName.c_str());
	if (! gST.narrowed_sl.Find(fShipClassName.c_str(), s1)) {
		gST.mDebug_fprintf("Ship %s not found in narrowed shiplist! Aborting fighter upgrade.\n", fShipClassName.c_str());
		return 0;
	}

	// check for fighters.
	// up to 4 fighter bays
	int num_bays = 0;
	int32 count = 0;
	int32 total_num_fighters = 0;
	Ship selected_fighter;
	BaseShip fighter;
	bool found_fighter = false;

	for (int k = 0; k < 4; k++)
	{
		// assume all fighters are the same in the ship
		// load only the first one.
		if (! found_fighter) {
			fighter = s1.fighters[k].first;
			found_fighter = true;
		}
		count = s1.fighters[k].second;
		if (count)
		{
			num_bays++;
			total_num_fighters += count;
		}
	}
	gST.mDebug_fprintf("Total number of Fighters: %d\n", total_num_fighters);

	if (total_num_fighters)
	{
		// We have fighters!
		// print out: fighter.shipCode
		gST.mDebug_fprintf("We have fighters! Bays: %d Count: %d  Type: %s\n",
			num_bays, total_num_fighters, fighter.shipCode.c_str());

		int32 avg_fighters = total_num_fighters / num_bays;
		if (total_num_fighters % num_bays)
			avg_fighters++;

		// check to see if we've already had a spawn with this fighter.
		if ( CD.mGetFightersAlreadyDone( ship->mGetCustomID() ) )
		{
			std::string original_fighter_selection = CD.mGetFighterSelection( ship->mGetCustomID() );
			gST.mDebug_fprintf("This is a respawn of %s. Original fighters were: %s\n", fShipClassName.c_str(), original_fighter_selection.c_str());
			ship->mSetFighters(num_bays, avg_fighters, original_fighter_selection);

			gST.mDebug_fprintf("---- Finished: common_defs::mCustomUpgradeFighters\n");
			return 0;
		} else {
			ShipList tmpFighter_sl;
			gST.mDebug_fprintf("Removing fighter selection which are less than %d BPV.\n", fighter.bpv);

			{
			SHIP_LIST_ITER(gST.narrowed_fighter_sl)
				if (
					(iter->race == s1.race)
					&& (iter->bpv >= fighter.bpv)
				) {
					tmpFighter_sl.push_back(*iter);
				}
			SHIP_LIST_ITER_END
			}

			if (tmpFighter_sl.size() == 0) {
				gST.mDebug_fprintf("weird! empty fighter shiplist. Aborting!\n");
				return 0;
			}
			gST.listShiplistContent(tmpFighter_sl);

			tmpFighter_sl.SelectRandomShip(selected_fighter);

			// print out: fighter.shipCode
			gST.mDebug_fprintf("New Fighter type: Bays: %d   #/bay: %d  Type: %s\n",
				num_bays, avg_fighters, selected_fighter.shipCode.c_str());
				
 			ship->mSetFighters(num_bays, avg_fighters, selected_fighter.shipCode);
			gST.mDebug_fprintf("---- Done: fighters on: %s.\n", ship->mGetClassName());

			// And now to remember what we chose.
			CD.mSetFighterSelection( ship->mGetCustomID(), selected_fighter.shipCode );

			return (num_bays * avg_fighters * selected_fighter.bpv) - (num_bays * avg_fighters * fighter.bpv);
		}
	}

#endif // #ifdef FIGHTERS

	gST.mDebug_fprintf("---- Finished: common_defs::mCustomUpgradeFighters\n");
	return 0;
}
