/********************************************************************************************************\

	File Name:	EnemyTeam2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyTeam2.h"
#include "EnemyTeam2BaseState.h"
#include "Mir_7Gates.h"
#include "EnemyShip2.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam2> gEnemyTeam2Allocator;

void tEnemyTeam2::mSetupState()
{
	mRegisterState( new tEnemyTeam2BaseState() );
	mGotoState( typeid( tEnemyTeam2BaseState ) );
}

void tEnemyTeam2::mSetupCommState()
{}

void tEnemyTeam2::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam2::mCreateShipsForTeam\n");
    gST.mInitializeSL();
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		int32 nStartPos = kStartPosition_I;
		int32 nCurrentID = kEnemy2_1;
		int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
		float nVarPercent;
        if( diffLevel == 0 )
        {
                nVarPercent = 0.65f;
        }
        else if( diffLevel == 1 )
        {
                nVarPercent = 1.0f;
        }
        else if( diffLevel == 2 )
        {
                nVarPercent = 1.25f;
        }

        for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
        {
            tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
            if( currentShip )
            {
                int32 thisShipBPV = int(currentShip->mGetShipBPV() * nVarPercent);

                // now we pick a ship with shiplist API. If it fails to pick one, use the old way.
                ShipList ship_sl = sl;
                fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), true);
                gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), fShiplistCount);
                gST.listShiplistContent(ship_sl);

                Ship s1;
                if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
                    gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                        s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                    gST.mTrackOrions( s1 );

                    mCreateShip(
                        typeid( tEnemyShip2 ),
                        s1.shipCode.c_str(),
                        nStartPos,
                        kNoMetaShipID,
                        nCurrentID,
                        0, 0, 0, 0,
                        NULL,
                        kDefaultShipOptions );
                } else {
                    gST.mDebug_fprintf("Failed to pick ship #%d. Using old method. ShipBPV: %d\n", shipNumber, thisShipBPV);

                    eClassTypes thisShipClass = currentShip->mGetClassType();
                    mCreateShip(
                        typeid( tEnemyShip2 ),
                        thisShipBPV,
                        thisShipClass,
                        0,
                        nStartPos,
                        nCurrentID,
                        0,0,0,0,
                        NULL,
                        kDefaultShipOptions  );
                }
                nCurrentID++;
            }
        }
    } else {
        mOldCreateShipsForTeam(); // no enhanced shiplist at all. Go to old create code. --FS
    }

    gST.mDebug_fprintf("--- Finished: tEnemyTeam2::mCreateShipsForTeam\n");
}

void tEnemyTeam2::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tEnemyTeam1::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
		return;
	
	int32 nStartPos = kStartPosition_I;
	int32 nCurrentID = kEnemy2_1;
	int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;
	
	if( diffLevel == 0 )
	{
		nVarPercent = 0.65f;
	}
	else if( diffLevel == 1 )
	{
		nVarPercent = 1.0f;
	}
	else if( diffLevel == 2 )
	{
		nVarPercent = 1.25f;
	}
	
	for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
	{
		tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
		
		if( currentShip )
		{
			eClassTypes thisShipClass = currentShip->mGetClassType();
			int32 thisShipBPV = currentShip->mGetShipBPV();
			mCreateShip( 
				typeid( tEnemyShip2 ), 
				thisShipBPV * nVarPercent, 
				thisShipClass, 
				0, 
				nStartPos, 
				nCurrentID, 
				0,0,0,0, 
				NULL, 
				static_cast<eShipOptions>( kDefaultShipOptions | kDeathUponCapture ) );
			nCurrentID++;
		}
	}
	gST.mDebug_fprintf("--- Finished: tEnemyTeam2::mOldCreateShipsForTeam\n");
}

void tEnemyTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam2", shipScriptDescription );
}

void tEnemyTeam2::mRespawnShip( int32 shipID, const char* className )
{
	mCreateShip( typeid( tEnemyShip2 ),
				 className,
				 kStartPosition_Z,
				 kNoMetaShipID,
				 shipID,
				 0,0,0,0,
				 NULL,
				 static_cast<eShipOptions>( kDefaultShipOptions | kDeathUponCapture ) );
}

void tEnemyTeam2::mUpdateCommStatus( int32 status )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
			return;

	int32 CommState;
	
	switch( status )
	{
	case 1:
		CommState = tEnemyShip2::kCommState_1;
		break;

	case 2:
		CommState = tEnemyShip2::kCommState_2;
		break;

	case 3:
		CommState = tEnemyShip2::kCommState_3;
		break;

	default:
		CommState = -1;
		break;
	}


	tShipIterator myShipsI = mGetFirstShip();
	for( int32 curShip = 1; curShip < 4; curShip++ )
	{
		tShipInfo* thisShip = *myShipsI;
		if( thisShip )
		{
			playerTeam->mGotoCommState( thisShip->mGetShipID(), CommState );
		}
		myShipsI++;
	}			
}