/********************************************************************************************************\

	File Name:	AllyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AllyTeam.h"
#include "AllyTeamBaseState.h"
#include "AllyShip.h"
#include "Mir_8Massacre.h"
#include "ShipList_API.h"

tClassAllocator<tAllyTeam> gAllyTeamAllocator;

void tAllyTeam::mSetupState()
{
	mRegisterState( new tAllyTeamBaseState() );
	mGotoState( typeid( tAllyTeamBaseState ) );
}

void tAllyTeam::mSetupCommState()
{}

void tAllyTeam::mCreateShipsForTeam()
{
	
}

void tAllyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tAllyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tAllyTeam", shipScriptDescription );
}

void tAllyTeam::mCreateReinforcements()
{
    gST.mDebug_fprintf("---- Starting: tAllyTeam::mCreateReinforcements\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		int32 fShiplistCount;
        ShipList sl = gST.stripOutRestricted( gST.narrowed_sl );
        sl = gST.mFilterShipListToSingleRace( sl, mGetRace() );

        ShipList commando_sl = sl;
        ShipList ship_sl = sl;
        ship_sl = gST.stripOutSPECIAL( ship_sl );
		ship_sl = gST.selectWarShipRoles( ship_sl );

		// CA_sl
		ShipList CA_sl = ship_sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
        }
        gST.listShiplistContent(CA_sl);

		// CL_sl
		ShipList CL_sl = ship_sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        }
        gST.listShiplistContent(CL_sl);

		// DD_sl
		ShipList DD_sl = ship_sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassDestroyer; i <= kEClassWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = DD_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD filter completed. ships: %d\n", fShiplistCount);
        }
        gST.listShiplistContent(DD_sl);

		// commando_sl
        {
		// We need to strip out WYN ships because we're choosing SPECIALs.  FS, 20121012
		std::list< eEnhancedRoles >  WYNList;
		WYNList.push_back(kRoleWYN);
		fShiplistCount = gST.narrowed_sl.Subset(WYNList, false);
		gST.mDebug_fprintf("stripping out WYN ships completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = commando_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for commando filter completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedRoles > enhRolesList;
        enhRolesList.push_back(kCommando);
        fShiplistCount = commando_sl.Subset(enhRolesList);
        gST.mDebug_fprintf("subset for commando role completed. ships: %d\n", fShiplistCount);
        }
        gST.listShiplistContent(commando_sl);

		if (
			(CA_sl.size() > 0) && 
			(CL_sl.size() > 0) && 
			(DD_sl.size() > 0) && 
			(commando_sl.size() > 0)
		) {
			Ship ca1, cl1, dd1, com1, com2;
            CA_sl.SelectRandomShip(ca1);
            gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

            CL_sl.SelectRandomShip(cl1);
            gST.mDebug_fprintf("cl1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cl1.shipCode.c_str(), cl1.bpv, cl1.yearFirstAvail, cl1.yearLastAvail);
            gST.mTrackOrions( cl1 );

            DD_sl.SelectRandomShip(dd1);
            gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
            gST.mTrackOrions( dd1 );

            commando_sl.SelectRandomShip(com1);
            gST.mDebug_fprintf("com1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                com1.shipCode.c_str(), com1.bpv, com1.yearFirstAvail, com1.yearLastAvail);
            gST.mTrackOrions( com1 );

            commando_sl.SelectRandomShip(com2);
            gST.mDebug_fprintf("com2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                com2.shipCode.c_str(), com2.bpv, com2.yearFirstAvail, com2.yearLastAvail);
            gST.mTrackOrions( com2 );

			mCreateShip( 
				 typeid( tAllyShip ),
				 ca1.shipCode.c_str(),
				 kStartPosition_G,
				 0,
				 kAlly1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

			mCreateShip( 
				 typeid( tAllyShip ),
				 cl1.shipCode.c_str(),
				 kStartPosition_G,
				 0,
				 kAlly2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

			mCreateShip( 
				 typeid( tAllyShip ),
				 dd1.shipCode.c_str(),
				 kStartPosition_G,
				 0,
				 kAlly3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

			mCreateShip( 
				 typeid( tAllyShip ),
				 com1.shipCode.c_str(),
				 kStartPosition_G,
				 0,
				 kAlly4,
				 0,0,0,0,
				 "Invasion Force",
				 kDefaultShipOptions );

			mCreateShip( 
				 typeid( tAllyShip ),
				 com2.shipCode.c_str(),
				 kStartPosition_G,
				 0,
				 kAlly5,
				 0,0,0,0,
				 "Invasion Force",
				 kDefaultShipOptions );
		} else {
				mOldCreateReinforcements();
		}
	} else {
		mOldCreateReinforcements();
	}

    gST.mDebug_fprintf("---- Finished: tAllyTeam::mCreateReinforcements\n");
}

void tAllyTeam::mOldCreateReinforcements()
{

    gST.mDebug_fprintf("---- Starting: tAllyTeam::mOldCreateReinforcements\n");

	mCreateShip( typeid( tAllyShip ),
				 180,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_G,
				 kAlly1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 150,
				 kClassLightCruiser,
				 0,
				 kStartPosition_G ,
				 kAlly2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 140,
				 kClassDestroyer,
				 0,
				 kStartPosition_G,
				 kAlly3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 "Z-MCG",
				 kStartPosition_G,
				 0,
				 kAlly4,
				 0,0,0,0,
				 "Invasion Force",
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 "Z-MCG",
				 kStartPosition_G,
				 0,
				 kAlly5,
				 0,0,0,0,
				 "Invasion Force",
				 kDefaultShipOptions );

    gST.mDebug_fprintf("---- Finished: tAllyTeam::mOldCreateReinforcements\n");
}

iTruth tAllyTeam::mSetTeamPriority( eTeamPriority priority )
{
	switch( priority )
	{
	case kKillTransport:
		{
			tTeamInfo* transportTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kTransportTeam ) );
			if( !transportTeam )
					return false;

			tShipIterator transportI = transportTeam->mGetFirstShip();
			tShipInfo* transport = *transportI;
			if( !transport )
				return false;

			tShipIterator myShips = mGetFirstShip();

			for ( int32 curShip = 0; curShip < 5; curShip++ )
			{
				if( *myShips )
				{
					tShipInfo* thisShip = *myShips;
					thisShip->mSetShipTarget( transport, kAbsolutePriority );
				}
				myShips++;
			}
			return true;
		}
		break;

	case kCaptureBase:
		{
			tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) );
			if( !baseTeam )
					return false;

			tShipIterator baseI = baseTeam->mGetFirstShip();
			tShipInfo* base = *baseI;
			if( !base )
				return false;

			tShipIterator myShips = mGetFirstShip();

			for ( int32 curShip = 0; curShip < 5; curShip++ )
			{
				if( *myShips )
				{
					tShipInfo* thisShip = *myShips;
					thisShip->mSetShipTarget( base, kAbsolutePriority );
				}
				myShips++;
			}
			return true;
		}
		break;

	case kFight:
		{
			// AI should attack any remaining Lyran ships on its own
			return true;
		}
		break;

	default:
		{
			return false;
		}
		break;
	}
}
