/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Mir_8Massacre.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "LyranTeam.h"
#include "LyranTeam2.h"
#include "PlanetTeam.h"
#include "BaseTeam.h"
#include "TransportTeam.h"
#include "CommonDefs.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}

void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kEnemyLaunch_timer, 7.0f );
	mSetTimedInterval( kTimerTurnsSet, kFriendlyReinforcement_timer, 8.0f );
	mSetTimedInterval( kTimerTurnsSet, kEnemyReinforcement_timer, 9.0f );
}

void tBaseVictoryState::mEndState()
{}


void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kEnemyLaunch_timer:
		{
			tTransportTeam* transportTeam = dynamic_cast<tTransportTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kTransportTeam ) ) );
			if( !transportTeam )
					return;

			tBaseTeam* baseTeam = dynamic_cast<tBaseTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) ) );
			if( !baseTeam )
					return;

			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
					return;

			if( ( !playerTeam->mBaseCaptured() ) || ( !playerTeam->mBaseDestroyed() ) )
			{
				transportTeam->mCreateTransport();
				playerTeam->mSetTransportLaunched();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTransportLaunched_msg );
			}
		}
		break;

	case kFriendlyReinforcement_timer:
		{
			tAllyTeam* allyTeam = dynamic_cast<tAllyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) ) );
			if( !allyTeam )
					return;

			allyTeam->mCreateReinforcements();
			
			if( !allyTeam->mSetTeamPriority( kKillTransport ) )
			{
				if( !allyTeam->mSetTeamPriority( kCaptureBase ) )
				{
					allyTeam->mSetTeamPriority( kFight );
				}
			}

			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kReinforcementsArrive_msg );
		}
		break;

	case kEnemyReinforcement_timer:
		{
			tLyranTeam2* lyranTeam2 = dynamic_cast<tLyranTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam2 ) ) );
			if( !lyranTeam2 )
					return;

			lyranTeam2->mCreateReinforcements();

			if( !lyranTeam2->mSetTeamPriority( kDefendTransport ) )
			{
				if( !lyranTeam2->mSetTeamPriority( kDefendBase ) ) 
				{
					lyranTeam2->mSetTeamPriority( kFight );
				}
			}

			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kEnemyReinforcementsArrive_msg );
		}
		break;
	
	default:
		break;
	}
}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	// Transport escaped: player loses
	if( !playerTeam->mTransportDead() )
	{
		return kDevastatingDefeat;
	}

	// Transport dead but base not capped or killed: player loses
	else if ( ( playerTeam->mTransportDead() ) && ( !playerTeam->mBaseDestroyed() ) )
	{
		return kDefeat;
	}

	// Transport dead, base dead: player wins
	else if ( ( playerTeam->mTransportDead() ) && ( playerTeam->mBaseDestroyed() ) )
	{
		return kAstoundingVictory;
	}

	return kUnknownVictoryLevel;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tAllyTeam* allyTeam = dynamic_cast<tAllyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) ) );
	if( !allyTeam )
			return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
			return;

	tLyranTeam2* lyranTeam2 = dynamic_cast<tLyranTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam2 ) ) );
	if( !lyranTeam2 )
			return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
			return;

	tBaseTeam* baseTeam = dynamic_cast<tBaseTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) ) );
	if( !baseTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
					playerTeam->mSetDied();
			}
		}
		break;

	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
	case tAllyTeam::kAlly3:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllyDestroyed_msg );
		}
		break;

	case tTransportTeam::kTransport:
		{
			playerTeam->mSetTransportDead();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTransportDestroyed_msg );

			// If the base is also dead or captured, the player has won
			if( ( playerTeam->mBaseCaptured() ) || ( playerTeam->mBaseDestroyed() ) )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}

			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNeedToGetBase_msg );

				if( !allyTeam->mSetTeamPriority( kCaptureBase ) )
				{	
					allyTeam->mSetTeamPriority( kFight );
				}

				if( !lyranTeam2->mSetTeamPriority( kDefendBase ) )
				{
					lyranTeam2->mSetTeamPriority( kFight );
				}
			}
		}
		break;

	case tBaseTeam::kBase:
		{
			playerTeam->mSetBaseDestroyed();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBaseDestroyed_msg );

			// If the transport hasn't launched yet, the player wins
			if( !playerTeam->mTransportLaunched() )
			{
				playerTeam->mSetTransportDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBaseDeadBeforeLaunch_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				return;
			}

			// If the transport is launched but dead then the player wins
			else if( playerTeam->mTransportDead() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				return;
			}

			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNeedToGetTransport_msg );
				allyTeam->mSetTeamPriority( kFight );
				lyranTeam2->mSetTeamPriority( kFight );
			}
		}
		break;
	}

}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tAllyTeam* allyTeam = dynamic_cast<tAllyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) ) );
	if( !allyTeam )
			return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
			return;

	tLyranTeam2* lyranTeam2 = dynamic_cast<tLyranTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam2 ) ) );
	if( !lyranTeam2 )
			return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
			return;

	tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) );
	if( !baseTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;


	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
	case tAllyTeam::kAlly3:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllyCapped_msg );
		}
		break;

	case tTransportTeam::kTransport:
		{
			playerTeam->mSetTransportDead();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTransportCapped_msg );
			ship->mDestroyShip();
			

			// If the base is also dead or captured, the player wins here
			if( ( playerTeam->mBaseCaptured() ) || ( playerTeam->mBaseDestroyed() ) )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNeedToGetBase_msg );

				if( !allyTeam->mSetTeamPriority( kCaptureBase ) )
				{
					allyTeam->mSetTeamPriority( kFight );
				}
				
				if( !lyranTeam2->mSetTeamPriority( kDefendBase ) )
				{
					lyranTeam2->mSetTeamPriority( kFight );
				}
			}
		}
		break;

	case tBaseTeam::kBase:
		{
			playerTeam->mSetBaseDestroyed();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBaseCapped_msg );

			if( !playerTeam->mTransportLaunched() )
			{	
				playerTeam->mSetTransportDead();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				return;
			}

			// If the transport is also dead then the player wins
			if( playerTeam->mTransportDead() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNeedToGetTransport_msg );

				if( !allyTeam->mSetTeamPriority( kKillTransport ) )
				{
					allyTeam->mSetTeamPriority( kFight );
				}

				if( !lyranTeam2->mSetTeamPriority( kDefendTransport ) )
				{
					lyranTeam2->mSetTeamPriority( kFight );
				}
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
			return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
			return;

	tTeamInfo* lyranTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam2 ) );
	if( !lyranTeam2 )
			return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
			return;

	tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) );
	if( !baseTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;

	case tTransportTeam::kTransport:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTransportEscaped_msg );

			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

