/********************************************************************************************************\

	File Name:	LyranTeam2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "LyranTeam2.h"
#include "LyranTeam2BaseState.h"
#include "LyranShip2.h"
#include "PlayerTeam.h"
#include "Mir_8Massacre.h"
#include "ShipList_API.h"


tClassAllocator<tLyranTeam2> gLyranTeam2Allocator;

void tLyranTeam2::mSetupState()
{
	mRegisterState( new tLyranTeam2BaseState() );
	mGotoState( typeid( tLyranTeam2BaseState ) );
}

void tLyranTeam2::mSetupCommState()
{}

void tLyranTeam2::mCreateShipsForTeam()
{}

void tLyranTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tLyranTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tLyranTeam2", shipScriptDescription );
}


void tLyranTeam2::mCreateReinforcements()
{
	mCreateShipsBasedOnPlayerTeam(kLyran2_1, kStartPosition_H);
}

iTruth tLyranTeam2::mSetTeamPriority( eTeamPriority priority )
{
	switch( priority )
	{
	case kDefendTransport:
		{
			tTeamInfo* transportTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kTransportTeam ) );
			if( !transportTeam )
					return false;

			tShipIterator transportI = transportTeam->mGetFirstShip();
			tShipInfo* transport = *transportI;
			if( !transport )
				return false;

			tShipIterator myShips = mGetFirstShip();

			for( int32 curShip = 0; curShip < 3; curShip++ );
			{
				
				if( *myShips )
				{
					tShipInfo* thisShip = *myShips;
					thisShip->mClearCommands();

					// Leaving this mDefendEntity in without any corrections .. 
					// it'll auto-correct if/when the transport leaves or dies. --FS, 20121012
					thisShip->mDefendEntity( transport, 20, 6, kAbsolutePriority );
				}
				myShips++;
			}
			return true;
		}
		break;

	case kDefendBase:
		{
			tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) );
			if( !baseTeam )
					return false;

			tShipIterator baseI = baseTeam->mGetFirstShip();
			tShipInfo* base = *baseI;
			if( !base )
				return false;

			int32 X, Y;
			base->mGetGridPosition(&X, &Y);

			tShipIterator myShips = mGetFirstShip();

			for( int32 curShip = 0; curShip < 3; curShip++ )
			{
				if( *myShips )
				{
					tShipInfo* thisShip = *myShips;
					thisShip->mClearCommands();
					thisShip->mDefendEntity( base, 25, 10, kAbsolutePriority );

					// AI is dumb and stops with mDefendEntity. Forcing Destination! -- FS
					thisShip->mSetFinalDestination(X, Y);
					thisShip->mSetShipThrust( 10 );
					thisShip->mSetShipAIControlled(kTrue);

				}
				myShips++;
			}
			return true;
		}
		break;

	case kFight:
		{
			// AI should automatically attack any targets in the area
			tShipIterator myShips = mGetFirstShip();

			for( int32 curShip = 0; curShip < 3; curShip++ )
			{
				if( *myShips )
				{
					tShipInfo* thisShip = *myShips;
					thisShip->mClearCommands();

				}
				myShips++;
			}

			return true;
		}
		break;

	default:
		return false;
		break;
	}
}


void tLyranTeam2::mCreateShipsBasedOnPlayerTeam(int32 startCustomID, int32 position)
{
	gST.mDebug_fprintf("--- Entering: tLyranTeam2::mCreateShipsForTeam\n");
    gST.mInitializeSL();
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        int32 nStartPos = position;
        int32 nCurrentID = startCustomID;
        int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
        int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
        float nVarPercent;
        if( diffLevel == 0 )
        {
                nVarPercent = 0.65f;
        }
        else if( diffLevel == 1 )
        {
                nVarPercent = 1.0f;
        }
        else if( diffLevel == 2 )
        {
                nVarPercent = 1.25f;
        }

        for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
        {
            tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
            if( currentShip )
            {
                int32 thisShipBPV = int(currentShip->mGetShipBPV() * nVarPercent);

                // now we pick a ship with shiplist API. If it fails to pick one, use the old way.
                ShipList ship_sl = sl;
                fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), true);
                gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), fShiplistCount);
                gST.listShiplistContent(ship_sl);

                Ship s1;
                if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
                    gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                        s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                    gST.mTrackOrions( s1 );

                    mCreateShip(
                        typeid( tLyranShip2 ),
                        s1.shipCode.c_str(),
                        nStartPos,
                        kNoMetaShipID,
                        nCurrentID,
                        0, 0, 0, 0,
                        NULL,
                        kDefaultShipOptions );
                } else {
                    gST.mDebug_fprintf("Failed to pick ship #%d. Using old method. ShipBPV: %d\n", shipNumber, thisShipBPV);

                    eClassTypes thisShipClass = currentShip->mGetClassType();
                    mCreateShip(
                        typeid( tLyranShip2 ),
                        thisShipBPV,
                        thisShipClass,
                        0,
                        nStartPos,
                        nCurrentID,
                        0,0,0,0,
                        NULL,
                        kDefaultShipOptions  );
                }
                nCurrentID++;
            }
        }
    } else {
        mOldCreateShipsBasedOnPlayerTeam(startCustomID, position);
    }

    gST.mDebug_fprintf("--- Finished: tKlingonTeam::mCreateShipsForTeam\n");
}

void tLyranTeam2::mOldCreateShipsBasedOnPlayerTeam(int32 startCustomID, int32 position)
{
	gST.mDebug_fprintf("--- Starting: tLyranTeam2::mOldCreateShipsBasedOnPlayerTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
		return;

	int32 nStartPos = position;
	int32 nCurrentID = startCustomID;
	int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;

	if( diffLevel == 0 )
	{
		nVarPercent = 0.65f;
	}
	else if( diffLevel == 1 )
	{
		nVarPercent = 1.0f;
	}
	else if( diffLevel == 2 )
	{
		nVarPercent = 1.25f;
	}

	for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
	{
		tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
		if( currentShip )
		{
			eClassTypes thisShipClass = currentShip->mGetClassType();
			int32 thisShipBPV = currentShip->mGetShipBPV();

			mCreateShip( typeid( tLyranShip2 ),
						 thisShipBPV * nVarPercent,
						 thisShipClass,
						 0,
						 nStartPos,
						 nCurrentID,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );

			nCurrentID++;
		}
	}

	gST.mDebug_fprintf("--- Finished: tLyranTeam2::mOldCreateShipsBasedOnPlayerTeam\n");
}



