/********************************************************************************************************\

	Mission:	"Massacre"
	Script:		"Mir_8Massacre"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		08/05/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>
					FS				10/10/2012
					era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Mir_8Massacre.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "LyranTeam.h"
#include "LyranTeam2.h"
#include "PlanetTeam.h"
#include "BaseTeam.h"
#include "TransportTeam.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tMir_8Massacre );

tMir_8Massacre::tMir_8Massacre()
{}

int32 tMir_8Massacre::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tMir_8Massacre::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;
	
const tTeamSpec& team1 = mCreateTeamSpec(		"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Ally Team2",
												static_cast< eTeamID >( kAllyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tAllyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Lyran Team",
												static_cast< eTeamID >( kLyranTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tLyranTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Lyran Team2",
												static_cast< eTeamID >( kLyranTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tLyranTeam2 ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Base Team",
												static_cast< eTeamID >( kBaseTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tBaseTeam ) );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Transport Team",
												static_cast< eTeamID >( kTransportTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tTransportTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMir_8Massacre::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tMir_8Massacre::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMir_8Massacre::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMir_8Massacre::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tMir_8Massacre::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tMir_8Massacre::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tMir_8Massacre::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 7;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kAllyTeam,
		kLyranTeam,
		kLyranTeam2,
		kPlanetTeam,
		kBaseTeam,
		kTransportTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Ally = kAllyTeam,
		//Lyrn  = kLyranTeam,
		//Lyr2 = kLyranTeam2,
		//Plan = kPlanetTeam
		//Base = kBaseTeam
		//Tran = kTransportTeam

		//	     Plyr  Ally  Lyrn  Lyr2	 Plan  Base  Tran
		/*Plyr*/ "--", "AI", "WD", "WD", "NI", "WD", "WD",
		/*Ally*/ "AI", "--", "WI", "WI", "NI", "WX", "WD",
		/*Lyrn*/ "WD", "WD", "--", "AI", "AI", "AI", "AI",
		/*Lyr2*/ "WD", "WD", "AI", "--", "AI", "AI", "AI",
		/*Plan*/ "WI", "WI", "AI", "AI", "--", "AI", "AI",
		/*Base*/ "WD", "WD", "AI", "AI", "AI", "--", "AI",
		/*Tran*/ "PI", "PI", "AI", "AI", "AI", "AI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tMir_8Massacre::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlanet;
	Map->fTacticalMapIcon['I'] = kObjectEnemyBase;
}

tMapTerrain tMir_8Massacre::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tMir_8Massacre::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tMir_8Massacre::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kMirak] = kBriefing_msg;
}