/********************************************************************************************************\

	File Name:	BackupTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BackupTeam.h"
#include "BackupTeamBaseState.h"
#include "BackupShip.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"

tClassAllocator<tBackupTeam> gBackupTeamAllocator;

void tBackupTeam::mSetupState()
{
	mRegisterState( new tBackupTeamBaseState() );
	mGotoState( typeid( tBackupTeamBaseState ) );
}

void tBackupTeam::mSetupCommState()
{}

void tBackupTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tBackupTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShipRoles( sl );

        ShipList CA_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyCruiser);
        shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CA_sl);
        }

		ShipList BCH_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyBattlecruiser);
        fShiplistCount = BCH_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for BCH filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(BCH_sl);
        }

        if (BCH_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a BCH from shiplist. using CA shiplist\n");
            BCH_sl = CA_sl;
        }

        ShipList CL_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CL_sl);
        }

        ShipList DD_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassDestroyer);
        shipClassTypeList.push_back(kEClassWarDestroyer);
        fShiplistCount = DD_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DD_sl);
        }

		if (
			(BCH_sl.size() > 0) &&
			(CA_sl.size() > 0) &&
			(CL_sl.size() > 0) &&
			(DD_sl.size() > 0)
		) {
			Ship bch1, ca1, cl1, dd1, dd2;
			BCH_sl.SelectRandomShip(bch1);
			gST.mDebug_fprintf("bch1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				bch1.shipCode.c_str(), bch1.bpv, bch1.yearFirstAvail, bch1.yearLastAvail);
            gST.mTrackOrions( bch1 );

			CA_sl.SelectRandomShip(ca1);
			gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

			CL_sl.SelectRandomShip(cl1);
			gST.mDebug_fprintf("cl1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				cl1.shipCode.c_str(), cl1.bpv, cl1.yearFirstAvail, cl1.yearLastAvail);
            gST.mTrackOrions( cl1 );

			DD_sl.SelectRandomShip(dd1);
			gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
            gST.mTrackOrions( dd1 );

			DD_sl.SelectRandomShip(dd2);
			gST.mDebug_fprintf("dd2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				dd2.shipCode.c_str(), dd2.bpv, dd2.yearFirstAvail, dd2.yearLastAvail);
            gST.mTrackOrions( dd2 );

			mCreateShip(
				typeid( tBackupShip ),
				bch1.shipCode.c_str(),
				kStartPosition_H,
				kNoMetaShipID,
				kBackup1,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );

			mCreateShip(
				typeid( tBackupShip ),
				ca1.shipCode.c_str(),
				kStartPosition_H,
				kNoMetaShipID,
				kBackup2,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );

			mCreateShip(
				typeid( tBackupShip ),
				cl1.shipCode.c_str(),
				kStartPosition_H,
				kNoMetaShipID,
				kBackup3,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );

			mCreateShip(
				typeid( tBackupShip ),
				dd1.shipCode.c_str(),
				kStartPosition_H,
				kNoMetaShipID,
				kBackup4,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );

			mCreateShip(
				typeid( tBackupShip ),
				dd2.shipCode.c_str(),
				kStartPosition_H,
				kNoMetaShipID,
				kBackup5,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tBackupTeam::mCreateShipsForTeam\n");
}

void tBackupTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tBackupTeam::mOldCreateShipsForTeam\n");

	// BCH, CA, CL, DD, DD
	mCreateShip( typeid( tBackupShip ),
				 200,
				 kClassHeavyBattlecruiser,
				 0,
				 kStartPosition_H,
				 kBackup1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tBackupShip ),
				 170,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_H,
				 kBackup2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tBackupShip ),
				 150,
				 kClassLightCruiser,
				 0,
				 kStartPosition_H,
				 kBackup3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tBackupShip ),
				 150,
				 kClassLightCruiser,
				 0,
				 kStartPosition_H,
				 kBackup4,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tBackupShip ),
				 120,
				 kClassDestroyer,
				 0,
				 kStartPosition_H,
				 kBackup5,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tBackupTeam::mOldCreateShipsForTeam\n");
}

void tBackupTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tBackupTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tBackupTeam", shipScriptDescription );
}

