/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Mir_9Edge.h"
#include "PlayerTeam.h"
#include "BackupTeam.h"
#include "LyranTeam.h"
#include "PlanetTeam.h"
#include "InvasionTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}

void tBaseVictoryState::mBeginState()
{}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	if( !playerTeam->mPlanetCapped() )
	{
		return kAstoundingVictory;
	}

	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;
	
	tTeamInfo* backupTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBackupTeam ) );
	if( !backupTeam )
		return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	tTeamInfo* invasionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kInvasionTeam ) );
	if( !invasionTeam )
		return;



	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					playerTeam->mSetDied();
					playerTeam->mSetPlanetCapped();
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );					
			}
		}
		break;

	case tInvasionTeam::kInvasion1:
	case tInvasionTeam::kInvasion2:
		{
			if( !invasionTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInvasionTeamDead_msg );
				playerTeam->mSetInvasionTeamStopped();

				if( !lyranTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
			}
		}
		break;

	// Player wins if both Lyran teams are gone
	case tBackupTeam::kBackup1:
	case tBackupTeam::kBackup2:
	case tBackupTeam::kBackup3:
	case tBackupTeam::kBackup4:
	case tBackupTeam::kBackup5:
	case tBackupTeam::kBackup6:
		{
			if( !backupTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAlliesDead_msg );
			}
		}
		break;

	// Player wins if both Lyran teams are gone
	case tLyranTeam::kLyran1:
	case tLyranTeam::kLyran2:
	case tLyranTeam::kLyran3:
	case tLyranTeam::kLyran4:
	case tLyranTeam::kLyran5:
	case tLyranTeam::kLyran6:
		{
			if( !lyranTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLyransDead_msg );

				if( !invasionTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
			}
		}
		break;

	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;
	
	tTeamInfo* backupTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBackupTeam ) );
	if( !backupTeam )
		return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	tTeamInfo* invasionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kInvasionTeam ) );
	if( !invasionTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
				{
					playerTeam->mSetPlanetCapped();
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
				}
			}
		}
		break;

	// Player loses if planet is capped
	case tPlanetTeam::kPlanet:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPlanetCapped_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerLoses );
		}
		break;
		

	// Player wins if both Lyran teams are gone
	case tInvasionTeam::kInvasion1:
	case tInvasionTeam::kInvasion2:
		{
			if( !invasionTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInvasionTeamDead_msg );
				playerTeam->mSetInvasionTeamStopped();

				if( !lyranTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
			}
		}
		break;

	// Player wins if both Lyran teams are gone
	case tLyranTeam::kLyran1:
	case tLyranTeam::kLyran2:
	case tLyranTeam::kLyran3:
	case tLyranTeam::kLyran4:
	case tLyranTeam::kLyran5:
	case tLyranTeam::kLyran6:
		{
			if( !lyranTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLyransDead_msg );

				if( !invasionTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;
	
	tTeamInfo* backupTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBackupTeam ) );
	if( !backupTeam )
		return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	tTeamInfo* invasionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kInvasionTeam ) );
	if( !invasionTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetPlanetCapped();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;

	// Player wins if both Lyran teams are gone
	case tInvasionTeam::kInvasion1:
	case tInvasionTeam::kInvasion2:
		{
			if( !invasionTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInvasionTeamDead_msg );
				playerTeam->mSetInvasionTeamStopped();

				if( !lyranTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
			}
		}
		break;

	// Player wins if both Lyran teams are gone
	case tLyranTeam::kLyran1:
	case tLyranTeam::kLyran2:
	case tLyranTeam::kLyran3:
	case tLyranTeam::kLyran4:
	case tLyranTeam::kLyran5:
	case tLyranTeam::kLyran6:
		{
			if( !lyranTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLyransDead_msg );

				if( !invasionTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
			}
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

