/********************************************************************************************************\

	File Name:	LyranTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "LyranTeam.h"
#include "LyranTeamBaseState.h"
#include "LyranShip.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tLyranTeam> gLyranTeamAllocator;

void tLyranTeam::mSetupState()
{
	mRegisterState( new tLyranTeamBaseState() );
	mGotoState( typeid( tLyranTeamBaseState ) );
}

void tLyranTeam::mSetupCommState()
{}

void tLyranTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tLyranTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShipRoles( sl );

        ShipList CA_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyCruiser);
        shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CA_sl);
        }

		ShipList CC_sl = CA_sl;
		{
		std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = CC_sl.Subset(leaderRolesList);
        gST.mDebug_fprintf("subset for CC (CA leader) eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(CC_sl);
		}
		if (CC_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a CC from shiplist. using CA shiplist\n");
            CC_sl = CA_sl;
        }

		ShipList BCH_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyBattlecruiser);
        fShiplistCount = BCH_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for BCH filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(BCH_sl);
        }
        if (BCH_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a BCH from shiplist. using CA shiplist\n");
            BCH_sl = CA_sl;
        }

        ShipList CL_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CL_sl);
        }

		ShipList CM_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		for (int32 i = kEClassHeavyWarDestroyer; i <= kEClassMediumCruiser; i++) {
			shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
		}
        fShiplistCount = CM_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CM filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CM_sl);
        }
        if (CM_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a CM from shiplist. using CL shiplist\n");
            CM_sl = CL_sl;
        }

        ShipList DD_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassDestroyer);
        shipClassTypeList.push_back(kEClassWarDestroyer);
        fShiplistCount = DD_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DD_sl);
        }

        ShipList FF_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassFrigate);
        fShiplistCount = FF_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for FF filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(FF_sl);
        }

		// 0: CA, DD, DD
		// 1: CC, CA, DD, DD
		// 2: BCH, CC, CM, DD, FF
		if (
			(BCH_sl.size() > 0) &&
			(CC_sl.size() > 0) &&
			(CA_sl.size() > 0) &&
			(CM_sl.size() > 0) &&
			(DD_sl.size() > 0) &&
			(FF_sl.size() > 0)
		) {

			int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

			// ship1:  0: nada   1: CC  2: BCH
			switch( diffLevel )
			{
			case 1:
				{
				Ship cc1;
				CC_sl.SelectRandomShip(cc1);
				gST.mDebug_fprintf("cc1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					cc1.shipCode.c_str(), cc1.bpv, cc1.yearFirstAvail, cc1.yearLastAvail);
				gST.mTrackOrions( cc1 );

				mCreateShip(
					typeid( tLyranShip ),
					cc1.shipCode.c_str(),
					kStartPosition_I,
					kNoMetaShipID,
					kLyran1,
					0,0,0,0,
					NULL,
					kDefaultShipOptions );
				break;
				}
			case 2:
				{
				Ship bch1;
				BCH_sl.SelectRandomShip(bch1);
				gST.mDebug_fprintf("bch1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					bch1.shipCode.c_str(), bch1.bpv, bch1.yearFirstAvail, bch1.yearLastAvail);
				gST.mTrackOrions( bch1 );

				mCreateShip(
					typeid( tLyranShip ),
					bch1.shipCode.c_str(),
					kStartPosition_I,
					kNoMetaShipID,
					kLyran1,
					0,0,0,0,
					NULL,
					kDefaultShipOptions );
				break;
				}
			}

			// Ship2: 0: CA   1: CA   2: CA   -- in all 3 difficulty cases
			Ship ca1;
			CA_sl.SelectRandomShip(ca1);
			gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

			mCreateShip(
					typeid( tLyranShip ),
					ca1.shipCode.c_str(),
					kStartPosition_I,
					kNoMetaShipID,
					kLyran2,
					0,0,0,0,
					NULL,
					kDefaultShipOptions );

			// Ship3: 0: DD   1: DD   2: CM
			switch( diffLevel )
			{
			case 0:
			case 1:
				{
				Ship dd1;
				DD_sl.SelectRandomShip(dd1);
				gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
				gST.mTrackOrions( dd1 );

				mCreateShip(
					typeid( tLyranShip ),
					dd1.shipCode.c_str(),
					kStartPosition_I,
					kNoMetaShipID,
					kLyran3,
					0,0,0,0,
					NULL,
					kDefaultShipOptions );
				break;
				}
			case 2:
				{
				Ship cm1;
				CM_sl.SelectRandomShip(cm1);
				gST.mDebug_fprintf("cm1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					cm1.shipCode.c_str(), cm1.bpv, cm1.yearFirstAvail, cm1.yearLastAvail);
				gST.mTrackOrions( cm1 );

				mCreateShip(
					typeid( tLyranShip ),
					cm1.shipCode.c_str(),
					kStartPosition_I,
					kNoMetaShipID,
					kLyran3,
					0,0,0,0,
					NULL,
					kDefaultShipOptions );
				break;
				}
			}

			// Ship4:  0: DD   1: DD   2: DD
			Ship dd2;
			DD_sl.SelectRandomShip(dd2);
			gST.mDebug_fprintf("dd2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				dd2.shipCode.c_str(), dd2.bpv, dd2.yearFirstAvail, dd2.yearLastAvail);
            gST.mTrackOrions( dd2 );

			mCreateShip(
					typeid( tLyranShip ),
					dd2.shipCode.c_str(),
					kStartPosition_I,
					kNoMetaShipID,
					kLyran4,
					0,0,0,0,
					NULL,
					kDefaultShipOptions );

			// Ship5: 0: nada   1: nada   2: FF
			switch( diffLevel )
			{
			case 2:
				{
				Ship ff1;
				FF_sl.SelectRandomShip(ff1);
				gST.mDebug_fprintf("ff1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					ff1.shipCode.c_str(), ff1.bpv, ff1.yearFirstAvail, ff1.yearLastAvail);
				gST.mTrackOrions( ff1 );

				mCreateShip(
					typeid( tLyranShip ),
					ff1.shipCode.c_str(),
					kStartPosition_I,
					kNoMetaShipID,
					kLyran5,
					0,0,0,0,
					NULL,
					kDefaultShipOptions );
				break;
				}
			}

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tLyranTeam::mCreateShipsForTeam\n");
}

void tLyranTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tLyranTeam::mOldCreateShipsForTeam\n");

	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	switch( diffLevel )
	{

	// Light fleet
	case 0:
		{
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassHeavyCruiser,
				0,
				kStartPosition_I,
				kLyran2,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassDestroyer,
				0,
				kStartPosition_I,
				kLyran3,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassDestroyer,
				0,
				kStartPosition_I,
				kLyran4,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
		}
		break;
		
		// Moderate fleet
	case 1:
		{
			mCreateShip( typeid( tLyranShip ),
				200,
				kClassHeavyCruiser,
				0,
				kStartPosition_I,
				kLyran1,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassHeavyCruiser,
				0,
				kStartPosition_I,
				kLyran2,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassDestroyer,
				0,
				kStartPosition_I,
				kLyran3,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassDestroyer,
				0,
				kStartPosition_I,
				kLyran4,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
		}
		break;
		
		
		//Heavy doom fleet
	case 2:
		{
			mCreateShip( typeid( tLyranShip ),
				200,
				kClassHeavyBattlecruiser,
				0,
				kStartPosition_I,
				kLyran1,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassHeavyCruiser,
				0,
				kStartPosition_I,
				kLyran2,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassLightCruiser,
				0,
				kStartPosition_I,
				kLyran3,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassDestroyer,
				0,
				kStartPosition_I,
				kLyran4,
				0,0,0,0,
				NULL,
				kDefaultShipOptions );
			
			mCreateShip( typeid( tLyranShip ),
				170,
				kClassFrigate,
				0,
				kStartPosition_I,
				kLyran5,
				0,0,0,0,
				NULL,
				kDefaultShipOptions);
		}
		break;
	}

	gST.mDebug_fprintf("--- Finished: tLyranTeam::mOldCreateShipsForTeam\n");
}

void tLyranTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tLyranTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tLyranTeam", shipScriptDescription );
}

