/********************************************************************************************************\

	Mission:	"Battle of Claw's Edge"
	Script:		"Mir_9Edge"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		08/05/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>
					FS				10/12/2012
					era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Mir_9Edge.h"
#include "PlayerTeam.h"
#include "BackupTeam.h"
#include "LyranTeam.h"
#include "PlanetTeam.h"
#include "InvasionTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tMir_9Edge );

tMir_9Edge::tMir_9Edge()
{}

int32 tMir_9Edge::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tMir_9Edge::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Backup Team",
												static_cast< eTeamID >( kBackupTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tBackupTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Lyran Team",
												static_cast< eTeamID >( kLyranTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tLyranTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Invasion Team",
												static_cast< eTeamID >( kInvasionTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tInvasionTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( draft1Team );
}

int32 tMir_9Edge::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tMir_9Edge::mCreateExternalShips( int32 Mode )
{}

int32 tMir_9Edge::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tMir_9Edge::mNewInternalShip( tShipInfo* Ship )
{}

void tMir_9Edge::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tMir_9Edge::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tMir_9Edge::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kBackupTeam,
		kLyranTeam,
		kPlanetTeam,
		kInvasionTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Bckp = kBackupTeam,
		//Lyr  = kLyranTeam,
		//Plan = kPlanetTeam,
		//Inva = kInvasionTeam
	
		//	     Plyr  Bckp  Lyr   Plan  Inva
		/*Plyr*/ "--", "AI", "WD", "AI", "WD",
		/*Bckp*/ "AI", "--", "WD", "AI", "WD",
		/*Lyr*/  "WD", "WD", "--", "WI", "AI",
		/*Plan*/ "AI", "AI", "WI", "--", "WI",
		/*Inva*/ "WD", "WD", "AI", "WX", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


void tMir_9Edge::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlanet;
	Map->fTacticalMapIcon['Z'] = kObjectPlanet;
	Map->fTacticalMapIcon['I'] = kObjectEnemyShip;
}

tMapTerrain tMir_9Edge::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tMir_9Edge::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tMir_9Edge::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kMirak] = kBriefing_msg;
}