/********************************************************************************************************\

	File Name:	CommonInterface.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "CommonInterface.h"
#include "SFCAssert.h"

class tCommonInterface gCI;

/*

	Purpose:	This function will tell all AI controlled-ship teams to chase after the puck
				and tractor it into the opposing starbase.

*/
void tCommonInterface::mAIGetPuck()
{
	//
	//	Run through each possible human-controlled team and tell each to tractor to their
	//	opposing goals.
	//
	for ( int32 i = kPlayerTeam1; i <= kAITeam2Team; i++ )
	{
		tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( team && !team->mIsTeamHumanControlled() )
		{
			//
			//	Get a pointer to the puck.
			//
			tTeamInfo*	puckTeam = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( kPuckTeam ) );
			Assert( puckTeam );
			if ( !puckTeam )
				continue;
			tShipIterator	puck = puckTeam->mGetFirstShip();
			if ( *puck == NULL )
				continue;

			switch ( team->mGetTag() )
			{
			case kTeam1Tag:
				{
					//
					//	Get a pointer to the opposing team's starbase.
					//
					tTeamInfo*		starbase2Team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( kStarbase2Team ) );
					Assert( starbase2Team );
					if ( !starbase2Team )
						continue;
					tShipIterator	starbase2 = starbase2Team->mGetFirstShip();
					if ( *starbase2 == NULL )
						continue;

					//
					//	Get the position of the above starbase.
					//
					int32	destX = 0, destY = 0;
					starbase2->mGetGridPosition( &destX, &destY );

					//
					//	Now, run through the list of ships on the team and tell all of the AI
					//	teams to chase that puck.
					//
					tShipIterator	ship = team->mGetFirstShip();
					if ( *ship == NULL )
						continue;
					tShipInPlayEval	y;
					ship.mAddEvaluator( &y );
					for ( tShipIterator j = ship; *j != NULL; j++ )
					{
						j->mTractorEntity( *puck, destX, destY, kHighPriority );
					}
				}
				break;

			case kTeam2Tag:
				{
					//
					//	Get a pointer to the opposing team's starbase.
					//
					tTeamInfo*		starbase1Team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( kStarbase1Team ) );
					Assert( starbase1Team );
					if ( !starbase1Team )
						continue;
					tShipIterator	starbase1 = starbase1Team->mGetFirstShip();
					if ( *starbase1 == NULL )
						continue;

					//
					//	Get the position of the above starbase.
					//
					int32	destX = 0, destY = 0;
					starbase1->mGetGridPosition( &destX, &destY );

					//
					//	Now, run through the list of ships on the team and tell all of the AI
					//	teams to chase that puck.
					//
					tShipIterator	ship = team->mGetFirstShip();
					if ( *ship == NULL )
						continue;
					tShipInPlayEval	y;
					ship.mAddEvaluator( &y );
					for ( tShipIterator j = ship; *j != NULL; j++ )
					{
						j->mTractorEntity( *puck, destX, destY, kHighPriority );
					}
				}
				break;
			}
		}
	}
}

