/********************************************************************************************************\

	Mission:	Mul_10Hockey
	Script:		"Mul_10Hockey"
	Descript:	<DESCRIPTION>

	Created by:		Daniel J. Suleski		06/21/00
	Last Updated:	DJS						06/22/00
					Created starbase team, all victory conditions, fixed bugs.
					DJS						07/26/00
					Second pass.
					DJS						08/23/00
					Map overrides.
					DJS						08/26/00
					Now has 6 playable teams.  Also checked through text.  Disengage now destroys ship.
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "CommonDefs.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "Mul_10Hockey.h"
#include "Team1.h"
#include "Team2.h"
#include "PuckTeam.h"
#include "StarbaseTeam.h"
#include "VictoryCondition.h"


DEFINE_SCRIPT_CLASS( tMul_10Hockey );

tMul_10Hockey::tMul_10Hockey()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tMul_10Hockey::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fHelpTags = Messages[kHelpTags_msg];
	Specs->fMissionType = kMultiplayerMission;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecOneShip;
	Specs->fMenuStates = kDisableTagC | kDisableTagD | kDisableTagE | kDisableTagF | kEnableSupplyDock;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );
	Specs->fGenericFlags = kDisableCapture;

	/*

		HELPER NOTE:	In this case, there are 6 teams setup.  Feel free to change the number of teams
						by deleting them starting with team6 and working your way down to team3, inclusive.
						Also, if the team is to be controlled by the AI and only the AI, then pass the
						constant 'kNPCTeam' instead of 'kPlayableTeam' into the team specification.  Also,
						not noted here, the last parameter of mCreateTeamSpec() can be a team tag such as
						"kTagA" to "kTagF".  This is the default tag for a kNPCTeam that was not chosen
						on the multiplayer UI screen.

	*/
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1",
												static_cast< eTeamID >( kPlayerTeam1 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTeam1 ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2",
												static_cast< eTeamID >( kPlayerTeam2 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTeam1 ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3",
												static_cast< eTeamID >( kPlayerTeam3 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTeam1 ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4",
												static_cast< eTeamID >( kPlayerTeam4 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTeam1 ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5",
												static_cast< eTeamID >( kPlayerTeam5 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTeam1 ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6",
												static_cast< eTeamID >( kPlayerTeam6 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTeam1 ) );

	const tTeamSpec& team7 = mCreateTeamSpec(	"AI Team 1",
												static_cast< eTeamID >( kAITeam1Team ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTeam1 ),
												static_cast< eTeamTag >( kTeam1Tag ) );

	const tTeamSpec& team8 = mCreateTeamSpec(	"AI Team 2",
												static_cast< eTeamID >( kAITeam2Team ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kKlingon | kRomulan,
												kAnyRace,
												typeid( tTeam2 ),
												static_cast< eTeamTag >( kTeam2Tag ) );

	const tTeamSpec& team9 = mCreateTeamSpec(	"Team 1 Starbase",
												static_cast< eTeamID >( kStarbase1Team ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tStarbaseTeam ),
												static_cast< eTeamTag >( kStarbase1Tag ) );

	const tTeamSpec& team10 = mCreateTeamSpec(	"Team 2 Starbase",
												static_cast< eTeamID >( kStarbase2Team ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tStarbaseTeam ),
												static_cast< eTeamTag >( kStarbase2Tag ) );

	const tTeamSpec& team11 = mCreateTeamSpec(	"Puck",
												static_cast< eTeamID >( kPuckTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tPuckTeam ),
												static_cast< eTeamTag >( kPuckTag ) );

	/*

		HELPER NOTE:	Please note that although you setup the teams, you need to create ships and their
						respective files.  Otherwise, an assert will occur.  It is up to you to decide
						whether the ship files should be shared by multiple physical teams or just a
						1-to-1 relationship between the team and the ship files.

	*/
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
	Specs->mSetupTeam( team8 );
	Specs->mSetupTeam( team9 );
	Specs->mSetupTeam( team10 );
	Specs->mSetupTeam( team11 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMul_10Hockey::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tMul_10Hockey::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMul_10Hockey::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMul_10Hockey::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMul_10Hockey::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 tagList[kRelationsCount] =
	{
		kTeam1Tag,		//	A
		kTeam2Tag,		//	B
		kStarbase1Tag,	//	C
		kStarbase2Tag,	//	D
		kPuckTag		//	E
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 A     B     C     D     E
		/*A   */ "AI", "WD", "AI", "FI", "FI",
		/*B   */ "WD", "AI", "FI", "AI", "FI",
		/*C   */ "AI", "FI", "AI", "FI", "FI",
		/*D   */ "FI", "AI", "FI", "AI", "FI",
		/*E   */ "AI", "AI", "AI", "AI", "AI",
	};

	fMissionInfo->mSetTagRelations( tagList, ( std::string* )relationships, kRelationsCount );

	/*

		HELPER NOTE:	You may wish to check if the game has ended here in the case, for instance, in
						the case that all physical teams share the same team tag letter and therefore, have
						no common enemies.

	*/
}

/*

	HELPER NOTE:	Since you really do not know which teams are going to be associated with which team
					tages until the game starts, you must create the teams here.  This function may look
					like the following:

					void tMul_10Hockey::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
					{
						switch ( teamScriptDescription.fTeamTag )
						{
						case kTagA:
						case kTagB:
						case kTagC:
						case kTagD:
						case kTagE:
						case kTagF:
							{
								mMission()->mSetupTeam(	teamScriptDescription.fTeam,
														teamScriptDescription.fTeamTag,
														teamScriptDescription.fRace,
														teamScriptDescription.fHumanControlled,
														typeid( tCommonTeam ) );
							}
							break;
						}
					}

*/
void tMul_10Hockey::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{
	//
	//	The team tag that comes through here must be valid.
	//
	Assert(	teamScriptDescription.fTeamTag == kTeam1Tag || teamScriptDescription.fTeamTag == kTeam2Tag ||
			teamScriptDescription.fTeamTag == kStarbase1Tag || teamScriptDescription.fTeamTag == kStarbase2Tag ||
			teamScriptDescription.fTeamTag == kPuckTag );
	if ( !( teamScriptDescription.fTeamTag == kTeam1Tag || teamScriptDescription.fTeamTag == kTeam2Tag ||
			teamScriptDescription.fTeamTag == kStarbase1Tag || teamScriptDescription.fTeamTag == kStarbase2Tag ||
			teamScriptDescription.fTeamTag == kPuckTag ) )
	{
		return;
	}

	//
	//	If a team has not been assigned a race, let's assign it to Orion.
	//
	eRace race = teamScriptDescription.fRace;
	if ( ( race == kUndefinedRace ) || ( race == kNoRace ) || ( race == kNoRaceType ) )
	{
		race = static_cast< eRace >( rand() % ( ( kMirak - kFederation ) + 1 ) + kFederation );
	}

	//
	//	Rather than create a team dependant upon its team mentioned in the teamSpec, we use the
	//	team tag to determine which actual class to assign to this team.
	//
	switch ( teamScriptDescription.fTeamTag )
	{
	case kTeam1Tag:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									race,
									teamScriptDescription.fHumanControlled,
									typeid( tTeam1 ) );
		}
		break;
	
	case kTeam2Tag:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									race,
									teamScriptDescription.fHumanControlled,
									typeid( tTeam2 ) );
		}
		break;

	case kStarbase1Tag:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									race,
									teamScriptDescription.fHumanControlled,
									typeid( tStarbaseTeam ) );
		}
		break;

	case kStarbase2Tag:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									race,
									teamScriptDescription.fHumanControlled,
									typeid( tStarbaseTeam ) );
		}
		break;

	case kPuckTag:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									race,
									teamScriptDescription.fHumanControlled,
									typeid( tPuckTeam ) );
		}
		break;
	}
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMul_10Hockey::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['L'] = kObjectPlayerShip;
	
	Map->fTacticalMapIcon['M'] = kObjectPlayerBase;

	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyBase;

	Map->fTacticalMapIcon['Q'] = kObjectShipEnd;
}