/********************************************************************************************************\

	File Name:	PuckBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "CommonInterface.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "PuckBaseState.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tPuckBaseState::mBeginState()
{}
void tPuckBaseState::mEndState()
{}

void tPuckBaseState::mOnDeath( tShipInfo* ship )
{
	if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
	{
		if ( fChildActor->mGetTeam() == ship->mGetTeam() )
		{
			//
			//	Create a new puck in the center of the "field".
			//
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				team->mCreateShip(	typeid( tPuck ),
									ship->mGetClassName(),
									kStartPosition_Q,
									tTeamInfo::kNoMetaShipID,
									ship->mGetCustomID(),
									0, 0, -1, -1, NULL,
									kDefaultShipOptions );
			}

			char temp[128];
			sprintf( temp, Messages[kScore_msg].c_str(), gWC.mTeam1Score(), gWC.mTeam2Score() );
			for ( int32 i = kPlayerTeam1; i <= kPlayerTeam6; i++ )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( i ), temp );
			}

			if ( ( gWC.mTeam1Score() >= kNumHitsForWin ) || ( gWC.mTeam2Score() >= kNumHitsForWin ) )
			{
				if ( gScriptInterface.mIsHost() )
				{
					gWC.mCheckForGameEnd();
				}
			}
		}
	}
}
void tPuckBaseState::mOnCapture( tShipInfo* ship )
{}
void tPuckBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
	{
		ship->mDestroyShipSilently();
	}
}
void tPuckBaseState::mAtDistance( tShipInfo* ship )
{}
void tPuckBaseState::mAtDestination( tShipInfo* ship )
{}
void tPuckBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPuckBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tPuckBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tPuckBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPuckBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPuckBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPuckBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPuckBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPuckBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPuckBaseState::mOnFlashingButtonPressed()
{}
void tPuckBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPuckBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPuckBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPuckBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPuckBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPuckBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPuckBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPuckBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPuckBaseState::mOnHostChanged()
{}
void tPuckBaseState::mOnTimedInterval( int32 timerID )
{}
void tPuckBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPuckBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

