/********************************************************************************************************\

	File Name:	PuckTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "CommonInterface.h"
#include "PuckTeamBaseState.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tPuckTeamBaseState::mBeginState()
{
	fDist1 = static_cast< float >( kPuckDistanceForScore ) + 1.0f;
	fDist2 = static_cast< float >( kPuckDistanceForScore ) + 1.0f;
}
void tPuckTeamBaseState::mEndState()
{}

void tPuckTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPuckTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPuckTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPuckTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tPuckTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tPuckTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPuckTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPuckTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPuckTeamBaseState::mOnFlashingButtonPressed()
{}
void tPuckTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPuckTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPuckTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPuckTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPuckTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPuckTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPuckTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPuckTeamBaseState::mOnHostChanged()
{}
void tPuckTeamBaseState::mOnTimedInterval( int32 timerID )
{
	switch ( timerID )
	{
	case kCheckInGoalTimerID:
		{
			//
			//	Find out if either team has gotten the puck close enough to a starbase to score.
			//
			iTruth team1Scores = kFalse;
			iTruth team2Scores = kFalse;
			if ( ( fDist1 >= 0.0f ) && ( fDist1 <= static_cast< float >( kPuckDistanceForScore ) ) )
			{
				team2Scores = kTrue;
			}
			if ( ( fDist2 >= 0.0f ) && ( fDist2 <= static_cast< float >( kPuckDistanceForScore ) ) )
			{
				team1Scores = kTrue;
			}

			//
			//	Get pointers to both starbases so that the we can use their positions to determine
			//	the distance to the puck.
			//
			tTeamInfo*	starTeam1 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kStarbase1Team ) );
			tTeamInfo*	starTeam2 = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kStarbase2Team ) );
			Assert( starTeam1 && starTeam2 );
			if ( ! ( starTeam1 && starTeam2 ) )
				return;
			tShipIterator	starbase1 = starTeam1->mGetFirstShip();
			tShipIterator	starbase2 = starTeam2->mGetFirstShip();
			tShipIterator	puck = fChildActor->mGetFirstShip();
			if ( ! ( ( *starbase1 != NULL ) && ( *starbase2 != NULL ) && ( *puck != NULL ) ) )
				return;

			//
			//	If either team has scored, then we must increment the appropriate score and destroy
			//	the puck.
			//
			if ( team1Scores || team2Scores )
			{
				if ( team1Scores )
				{
					gWC.mIncrementTeam1Score();
					if ( gWC.mTeam1Score() >= kNumHitsForWin )
					{
						gWC.mSetStarbase2Invincible( false );
						starbase2->mDestroyShip();
					}
				}
				else if ( team2Scores )
				{
					gWC.mIncrementTeam2Score();
					if ( gWC.mTeam2Score() >= kNumHitsForWin )
					{
						gWC.mSetStarbase1Invincible( false );
						starbase1->mDestroyShip();
					}
				}

				//
				//	This may need only "host"-care...let's see...
				//
				if ( gScriptInterface.mIsHost() )
				{
					puck->mDestroyShip();
				}

				//
				//	Reinitialize both distances so that the puck does not blow up again.
				//
				fDist1 = static_cast< float >( kPuckDistanceForScore ) + 1.0f;
				fDist2 = static_cast< float >( kPuckDistanceForScore ) + 1.0f;
			}
			//
			//	Otherwise, let us recalculate the fDist1 and fDist2 with the puck's new position.
			//
			else
			{
				fDist1 = puck->mGetTacticalDistance( *starbase1 );
				fDist2 = puck->mGetTacticalDistance( *starbase2 );
			}

			//
			//	Check for the end of the game if needed and reset the timer if needed.
			//
			if ( ( gWC.mTeam1Score() >= kNumHitsForWin ) || ( gWC.mTeam2Score() >= kNumHitsForWin ) )
			{
				;	//	doing the following in the mOnDeath() event now...
//				if ( gScriptInterface.mIsHost() )
//				{
//					gWC.mCheckForGameEnd();
//				}
			}
			else
			{
				fChildActor->mSetTimedInterval( kTimerSecondsSet, kCheckInGoalTimerID, kPuckLimboTime );

				//
				//	Tell the AI guys to chase after the new puck again.
				//
				gCI.mAIGetPuck();
			}
		}
		break;
	}
}
void tPuckTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPuckTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

