/********************************************************************************************************\

	File Name:	StarbaseBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "ScriptInterface.h"
#include "StarbaseBaseState.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tStarbaseBaseState::mBeginState()
{}
void tStarbaseBaseState::mEndState()
{}

void tStarbaseBaseState::mOnDeath( tShipInfo* ship )
{
	if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
	{
		if ( fChildActor->mGetTeam() == ship->mGetTeam() )
		{
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				switch ( team->mGetTag() )
				{
				case kStarbase1Tag:
					{
						gWC.mSetStarbase1Destroyed( true );
					}
					break;

				case kStarbase2Tag:
					{
						gWC.mSetStarbase2Destroyed( true );
					}
					break;
				}
			}

			//
			//	A starbase was destroyed, so check if the game has ended.
			//
			gWC.mCheckForGameEnd();
		}
	}
}
void tStarbaseBaseState::mOnCapture( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
	{
		ship->mDestroyShipSilently();
	}
}
void tStarbaseBaseState::mAtDistance( tShipInfo* ship )
{}
void tStarbaseBaseState::mAtDestination( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tStarbaseBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( fChildActor->mGetCustomID() == damagedShip->mGetCustomID() )
	{
		if ( fChildActor->mGetTeam() == damagedShip->mGetTeam() )
		{
			//
			//	Reset the starbase back to full health.  The starbase should only be destroyed
			//	when it is not invincible.
			//
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( damagedShip->mGetTeam() );
			Assert( team );
			if ( team )
			{
				switch ( team->mGetTag() )
				{
				case kStarbase1Tag:
					{
						if ( gWC.mStarbase1Invincible() )
						{
							fChildActor->mSetSystemHealthFromOrig( kExcessDamageHardPoint, 100.0f );
						}
					}
					break;

				case kStarbase2Tag:
					{
						if ( gWC.mStarbase2Invincible() )
						{
							fChildActor->mSetSystemHealthFromOrig( kExcessDamageHardPoint, 100.0f );
						}
					}
					break;
				}
			}

			//
			//	Tell the starbase that we still want to find out about damage.
			//
			fChildActor->mSetShipCallbackEvents( kNotifyDamage );
		}
	}
}
void tStarbaseBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tStarbaseBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tStarbaseBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tStarbaseBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tStarbaseBaseState::mOnFlashingButtonPressed()
{}
void tStarbaseBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tStarbaseBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tStarbaseBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tStarbaseBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tStarbaseBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tStarbaseBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tStarbaseBaseState::mOnHostChanged()
{}
void tStarbaseBaseState::mOnTimedInterval( int32 timerID )
{}
void tStarbaseBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tStarbaseBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

