/********************************************************************************************************\

	File Name:	StarbaseTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "StarbaseTeam.h"
#include "StarbaseTeamBaseState.h"
#include "StarbaseTeamBaseVictory.h"
#include "Starbase.h"
#include "ShipScriptDescription.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


tClassAllocator<tStarbaseTeam> gStarbaseTeamAllocator;

tStarbaseTeam::tStarbaseTeam()
{}

/*

	HELPER NOTE:	The victory condition for this team is specific to this team and only to this team.

*/
void tStarbaseTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tStarbaseTeamBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tStarbaseTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tStarbaseTeam::mSetupState()
{
	mRegisterState( new tStarbaseTeamBaseState() );
	mGotoState( typeid( tStarbaseTeamBaseState ) );
}

/*

	HELPER NOTE:	The comm states should include the team they apply for when registered.

*/
void tStarbaseTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the team's BPV.
	//
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_G,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tStarbaseTeam::mCreateShipsForTeam()
{
	//
	//	Figure out the starting position of the starbase.
	//
	iMapPosition startPos = kStartPosition_M;
	int32 customID = kStarbase1;
	if ( mGetTag() == kStarbase1Tag )
	{
		startPos = kStartPosition_M;
		customID = kStarbase1;
	}
	else if ( mGetTag() == kStarbase2Tag )
	{
		startPos = kStartPosition_Z;
		customID = kStarbase2;
	}

	//
	//	Get the race of the first attacking team and use that race to determine the race of the
	//	attacking team's starbase regardless of what the other defender races are.
	//
	eRace race = kNoRace;
	tTeamInfo* team = NULL;
	if ( mGetTag() == kStarbase1Tag )
	{
		for ( int32 i = kPlayerTeam1; i <= kAITeam2Team; i++ )
		{
			team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( !team )
				continue;

			if ( team->mGetTag() == kTeam1Tag )
				break;
		}
		if ( team )
			race = team->mGetRace();
	}
	else if ( mGetTag() == kStarbase2Tag )
	{
		for ( int32 i = kPlayerTeam1; i <= kAITeam2Team; i++ )
		{
			team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( !team )
				continue;

			if ( team->mGetTag() == kTeam2Tag )
				break;
		}
		if ( team )
			race = team->mGetRace();
	}

	switch ( race )
	{
	case kFederation:
		mCreateShip(	typeid( tStarbase ), "F-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case kKlingon:
		mCreateShip(	typeid( tStarbase ), "K-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case kRomulan:
		mCreateShip(	typeid( tStarbase ), "R-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case kLyran:
		mCreateShip(	typeid( tStarbase ), "L-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case kGorn:
		mCreateShip(	typeid( tStarbase ), "G-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case kHydran:
		mCreateShip(	typeid( tStarbase ), "H-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case kMirak:
		mCreateShip(	typeid( tStarbase ), "Z-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case kISC:
	default:
		mCreateShip(	typeid( tStarbase ), "I-SB", startPos, kNoMetaShipID,
						customID, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	}
}

/*

	HELPER NOTE:	This is where the event notification messages can be set for the ships that have
					been supplied by the multiplayer engine.  Also, if the team is controlled by a
					player, then one of that team's ships should be set to player control.  Also, the
					parent function must ALWAYS be called at the start of this function as you see
					below.  For example:

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	for( ;
		 *ship != NULL;
		 ++ship )
	{
		ship->mSetShipCallbackEvents( kNotifyDamage );
	}

*/
void tStarbaseTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	If the ships that the player chooses for the team from the multiplayer engine are
					not the ships that that team should start with, then leave the function overidden
					and create the desired ships in tTeamInfo::mCreateShipsForTeam().  Otherwise, delete
					this function and the respective prototype in the class header or simply undefine
					the constant.

*/
void tStarbaseTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{}
