/********************************************************************************************************\

	File Name:	Team2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonDefs.h"
#include "CommonInterface.h"
#include "MissionText.h"
#include "Team2.h"
#include "Team2BaseState.h"
#include "Team2BaseVictory.h"
#include "Team2Ship.h"
#include "ShipScriptDescription.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


tClassAllocator<tTeam2> gTeam2Allocator;

tTeam2::tTeam2()
{
	gWC.mIncrementTotalTeam2Teams();
}

/*

	HELPER NOTE:	The victory condition for this team is specific to this team and only to this team.

*/
void tTeam2::mSetupVictoryCondition()
{
	fVictoryCondition = new tTeam2BaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tTeam2::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;

	case kDraw:
	default:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tTeam2::mSetupState()
{
	mRegisterState( new tTeam2BaseState() );
	mGotoState( typeid( tTeam2BaseState ) );
}

/*

	HELPER NOTE:	The comm states should include the team they apply for when registered.

*/
void tTeam2::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the team's BPV.
	//
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_G,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tTeam2::mCreateShipsForTeam()
{
	//
	//	Add up the total BPV of the ships on the team 1
	//
	int32 totalBPV = 0;
	for ( int32 i = kPlayerTeam1; i <= kPlayerTeam6; i++ )  // get total of team 1-6 bpv
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( team && team->mGetTag() == kTeam1Tag )
		{
			totalBPV += team->mGetCombatBPV();
		}
	}


	//
	//	If this is a team that is not human controlled, then we need to check if there are any
	//	human controlled players on the team.  If not, then we need to create some AI ships in
	//	its place.
	//
	if ( !( gWC.mTotalTeam2Teams() - kNumHardCodedNPCTeams ))
	{	
		for ( int32 i = 0; i < gWC.mTotalTeam1Teams() - kNumHardCodedNPCTeams; i++ )
		{
			mCreateShip(	typeid( tTeam2Ship ),
							totalBPV / gWC.mTotalTeam1Teams(),
							kClassLightCruiser,
							100,
							kStartPosition_T + i,
							kPlayerShip,
							0, 0, -1, -1, NULL,
							kDefaultShipOptions );
//			mCreateShip( typeid( tTeam2Ship ), "O-CR", kStartPosition_T + i, kNoMetaShipID, kPlayerShip,
//						 0, 0, -1, -1, NULL, kDefaultShipOptions );
		}
	}
}

/*

	HELPER NOTE:	This is where the event notification messages can be set for the ships that have
					been supplied by the multiplayer engine.  Also, if the team is controlled by a
					player, then one of that team's ships should be set to player control.  Also, the
					parent function must ALWAYS be called at the start of this function as you see
					below.  For example:

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	for( ;
		 *ship != NULL;
		 ++ship )
	{
		ship->mSetShipCallbackEvents( kNotifyDamage );
	}

*/
void tTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	if ( mIsTeamHumanControlled() )
	{	
		tShipIterator ship = mGetFirstShip();

		Assert ( *ship != NULL );

		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	//
	//	Tell the AI (if present on this team) to tractor the puck into the opposing goal.
	//
	gCI.mAIGetPuck();
}

/*

	HELPER NOTE:	If the ships that the player chooses for the team from the multiplayer engine are
					not the ships that that team should start with, then leave the function overidden
					and create the desired ships in tTeamInfo::mCreateShipsForTeam().  Otherwise, delete
					this function and the respective prototype in the class header or simply undefine
					the constant.

*/
void tTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	if ( mGetTeam() != kAITeam2Team )
	{
		mCreateShip(	typeid( tTeam2Ship ), 
						shipScriptDescription.fClassName.c_str(), 
						kStartPosition_T + shipScriptDescription.fTeamID,
						kNoMetaShipID,
						kPlayerShip, 0, 0, -1, -1,
						shipScriptDescription.fCustomName.c_str(),
						kDefaultShipOptions );
	}
}