/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "CommonDefs.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions():	fTotalTeam1Teams( 0 ), fTotalTeam2Teams( 0 ),
										fTeam1Score( 0 ), fTeam2Score( 0 ),
										fStarbase1Destroyed( false ), fStarbase2Destroyed( false ),
										fStarbase1Invincible( true ), fStarbase2Invincible( true )
{}

/*

	Purpose:	The condition for the game ending will occur at the following times:

				1. The attacking starbase is destroyed.
				1. The defending starbase is destroyed.
				
				The game will not be checked if all human players are on one team in the beginning
				because if this is the case, then the AI will take over controlling the opposing side.
				In addition, the game end will not be checked if all of the ships on one team are
				destroyed because the starbase should still be in play.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	//
	//	Let's check if the starbase 1 is destroyed.  If so, then all of the team 1 teams
	//	get a losing condition and all of the team 2 teams get a winning condition set.
	//
	if ( mStarbase1Destroyed() )
	{
		for ( int32 i = kPlayerTeam1; i <= kAITeam2Team; i++ )
		{
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( team->mGetGameStatus() == kGameInProgress )
				{
					if ( team->mGetTag() == kTeam2Tag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerWins );
					}
					else if ( team->mGetTag() == kTeam1Tag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerLoses );
					}
				}
			}
		}

		//
		//	No need to go on.
		//
		return;
	}

	//
	//	Let's check if the starbase 2 is destroyed.  If so, then all of the team 2 teams
	//	get a losing condition and all of the team 1 teams get a winning condition set.
	//
	if ( mStarbase2Destroyed() )
	{
		for ( int32 i = kPlayerTeam1; i <= kAITeam2Team; i++ )
		{
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( team->mGetGameStatus() == kGameInProgress )
				{
					if ( team->mGetTag() == kTeam1Tag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerWins );
					}
					else if ( team->mGetTag() == kTeam2Tag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerLoses );
					}
				}
			}
		}

		//
		//	No need to go on.
		//
		return;
	}

	//
	//	If we got this far, then the game is not considered to be completed yet.
	//
}

/*

	Accessors and Modifiers

*/
void tWorldConditions::mIncrementTotalTeam1Teams()
{
	fTotalTeam1Teams++;
}
void tWorldConditions::mIncrementTotalTeam2Teams()
{
	fTotalTeam2Teams++;
}
void tWorldConditions::mIncrementTeam1Score()
{
	fTeam1Score++;
}
void tWorldConditions::mIncrementTeam2Score()
{
	fTeam2Score++;
}
void tWorldConditions::mSetStarbase1Destroyed( bool x )
{
	fStarbase1Destroyed = x;
}
void tWorldConditions::mSetStarbase2Destroyed( bool x )
{
	fStarbase2Destroyed = x;
}
void tWorldConditions::mSetStarbase1Invincible( bool x )
{
	fStarbase1Invincible = x;
}
void tWorldConditions::mSetStarbase2Invincible( bool x )
{
	fStarbase2Invincible = x;
}

int32 tWorldConditions::mTotalTeam1Teams()
{
	return fTotalTeam1Teams;
}
int32 tWorldConditions::mTotalTeam2Teams()
{
	return fTotalTeam2Teams;
}
int32 tWorldConditions::mTeam1Score()
{
	return fTeam1Score;
}
int32 tWorldConditions::mTeam2Score()
{
	return fTeam2Score;
}
bool tWorldConditions::mStarbase1Destroyed()
{
	return fStarbase1Destroyed;
}
bool tWorldConditions::mStarbase2Destroyed()
{
	return fStarbase2Destroyed;
}
bool tWorldConditions::mStarbase1Invincible()
{
	return fStarbase1Invincible;
}
bool tWorldConditions::mStarbase2Invincible()
{
	return fStarbase2Invincible;
}