/********************************************************************************************************\

	File Name:	CommonTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonBaseVictory.h"
#include "CommonDefs.h"
#include "CommonShip1.h"
#include "CommonTeam.h"
#include "CommonTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


tClassAllocator<tCommonTeam> gCommonTeamAllocator;

tCommonTeam::tCommonTeam( void ): fShipCustomID( kShip1 ), fNumShips( 0 )
{
	gWC.fNumTeams++;
	fStartID = gWC.mGenerateRandomStartPosition();
}

void tCommonTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );
		} 
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;

	case kDraw:
	default:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );
		}
		break;
	}
}

void tCommonTeam::mSetupState()
{
	mRegisterState( new tCommonTeamBaseState() );
	mGotoState( typeid( tCommonTeamBaseState ) );
}

void tCommonTeam::mSetupCommState()
{}

void tCommonTeam::mCreateShipsForTeam()
{
	if( !gWC.mChoseShips() )
	{
		gWC.mChooseRandomShipType();
		gWC.mSetChoseShips();
	}

	eClassTypes		shipClass	= gWC.mGetChosenShip();
	int32			shipBPV		= gWC.mGetChosenBPV();
	iMapPosition	startpos	= gWC.mGenerateStartPosition( mGetTag() );
	eShipOptions	fullOptions = static_cast<eShipOptions>( kCanBeRestricted | kCanBeCarrier |
															 kCanBeEscort | kCanBeSublight	|
															 kCanBeScout | kCanBeMarine | kCanBeTournament
															 | kStartPositionCanBeOffset);

	mCreateShip( typeid( tCommonShip1 ),
				 shipBPV,
				 shipClass,
				 100,
				 startpos,
				 fShipCustomID,
				 0,0,0,0,
				 NULL,
				 fullOptions );	
	
	fShipCustomID++;
	fNumShips++;
}

void tCommonTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	for( ; *ship != NULL; ++ship )
	{
		ship->mSetShipCallbackEvents( kNotifyDamage );
	}

}

void tCommonTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{

}
