/********************************************************************************************************\

	File Name:	CommonBaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonBaseVictoryState.h"
#include "CommonDefs.h"
#include "CommonShip1.h"
#include "CommonTeam.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"


//
//	Serialization events
//
tCommonBaseVictoryState::tCommonBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tCommonBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fNoWinner = kFalse;
	fDisconnect = kFalse;
}


void tCommonBaseVictoryState::mBeginState()
{
	if( gScriptInterface.mIsHost() )
	{
		float TimeLimit = gWC.mGetTimeLimit() * 60;
		mSetTimedInterval( kTimerSecondsSet, kGame_timer, TimeLimit );
	}
}

void tCommonBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kGame_timer:
		{
			for( int32 i = kTeam1; i < kTeam7; i++ )
			{
				if( i == fChildActor->mGetTeam()->mGetTeam() )
				{
					fNoWinner = kTrue;
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( i ), kNoWinner_msg );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( i ), kPlayerLoses );
				}
			}
		}
		break;

	default:
		break;
	}
}


eVictoryLevels tCommonBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tCommonTeam*	team = static_cast< tCommonTeam* >( fMissionInfo->mGetTeamHandle( ID ) );
	Assert( team );
	if ( team )
	{
		if( fNoWinner || fDisconnect )
		{
			return kDraw;
		}

		if ( gWC.mShipsDestroyed( ID ) == team->mNumShips() )
		{
			return kDefeat;
		}
		else
		{
			return kVictory;
		}
	}
	return kDraw;
}

void tCommonBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	switch ( ship->mGetCustomID() )
	{
	//
	//	A ship was lost, so increment the number of ships destroyed on that team and check if the
	//	game is over.
	//
	case kShip1:
	case kShip2:
	case kShip3:
		{
			if ( fTeam->mGetTeam() == team->mGetTeam() )
			{
				gWC.mIncrementShipsDestroyed( teamID );

				//
				//	If all of the ships are gone on a team, then the game has ended for that team.
				//
				if ( gWC.mShipsDestroyed( teamID ) == ( static_cast< tCommonTeam* >( team )->mNumShips() ) )
				{
					fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
					gScriptInterface.mSetGameStatus( teamID, kPlayerLoses );
					gWC.mSetTeamWon( teamID, false );
					gWC.mCheckForGameEnd();
				}
			}
		}
		break;
	}
}
void tCommonBaseVictoryState::mOnCapture( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	switch ( ship->mGetCustomID() )
	{
	//
	//	A ship was lost, so increment the number of ships destroyed on that team and check if the
	//	game is over.
	//
	case kShip1:
	case kShip2:
	case kShip3:
		{
			if ( fTeam->mGetTeam() == team->mGetTeam() )
			{
				gWC.mIncrementShipsDestroyed( teamID );

				//
				//	If all of the ships are gone on a team, then the game has ended for that team.
				//
				if ( gWC.mShipsDestroyed( teamID ) == ( static_cast< tCommonTeam* >( team )->mNumShips() ) )
				{
					fMissionInfo->mDisplayMessage( teamID, kYouAreDead_msg, kNonCommScriptMessage );
					gScriptInterface.mSetGameStatus( teamID, kPlayerLoses );
					gWC.mSetTeamWon( teamID, false );
					gWC.mCheckForGameEnd();
				}
			}
		}
		break;
	}
}

void tCommonBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{
	for( int32 i = kTeam1; i < kTeam7; i++ )
	{
		if( i == fChildActor->mGetTeam()->mGetTeam() )
		{
			fDisconnect = kTrue;
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( i ), kDisconnected_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( i ), kPlayerLoses );
		}
	}
}

void tCommonBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonBaseVictoryState::mOnFlashingButtonPressed()
{}
void tCommonBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//

void tCommonBaseVictoryState::mOnHostChanged()
{}

void tCommonBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
void tCommonBaseVictoryState::mEndState()
{}
