/********************************************************************************************************\

	File Name:	CommonTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonBaseVictory.h"
#include "CommonDefs.h"
#include "CommonShip1.h"
#include "CommonTeam.h"
#include "CommonTeamBaseState.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


tClassAllocator<tCommonTeam> gCommonTeamAllocator;

tCommonTeam::tCommonTeam( void ): fShipCustomID( kShip1 ), fNumShips( 0 )
{
	gWC.fNumTeams++;
}

void tCommonTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );
		} 
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;

	case kDraw:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );
		}
		break;

	default:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );
		}
		break;
	}
}

void tCommonTeam::mSetupState()
{
	mRegisterState( new tCommonTeamBaseState() );
	mGotoState( typeid( tCommonTeamBaseState ) );
}

void tCommonTeam::mSetupCommState()
{}

void tCommonTeam::mCreateShipsForTeam()
{

}

void tCommonTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	for( ; *ship != NULL; ++ship )
	{
		ship->mSetShipCallbackEvents( kNotifyDamage );
	}
}

void tCommonTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	iMapPosition startpos = gWC.mGenerateStartPosition( mGetTag() );

	mCreateShip( typeid( tCommonShip1 ), 
				 shipScriptDescription.fClassName.c_str(), 
				 startpos,
				 kNoMetaShipID,
				 fShipCustomID, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 kDefaultShipOptions );

	fShipCustomID++;
	fNumShips++;
}
