/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "MissionInfo.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"
#include "WorldConditions.h"
#include <fstream.h>
#include <string>

class tWorldConditions gWC;

tWorldConditions::tWorldConditions()
{
	for ( int i = 0; i < 6; i++ )
	{
		fShipsDestroyed[i] = 0;
		fTeamWon[i] = true;
	}
	
	fNumTeams = 0;

	for( int32 thisPos = kFirstStartPosition; thisPos < kLastStartPosition; thisPos++ )
	{
		fPositionUsed[thisPos] = kFalse;
	}
}

void tWorldConditions::mCheckForGameEnd( void )
{

	iTruth	remainingShipsOnSameTeam = true;
	int32	lastWinningTeam = -1;

	tMissionInfo*	thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	Find out if all the ships left standing are from the same team.
	//
	for ( int i = 0; i < kMaxTeams; i++ )
	{
		//
		//	If we already have found a winning team, then lastWinningTeam must be the most recent winning
		//	team we have found.
		//
		if ( lastWinningTeam != -1 )
		{
			tTeamInfo* currentTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( !currentTeam )
				continue;
			tTeamInfo* previousTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( lastWinningTeam ) );
			if ( !previousTeam )
				continue;

			//
			//	If the two ships do not match team tags, then we know that a fight is still brewing and
			//	we should not end any games as winners yet.
			//
			if ( mTeamWon( i ) )
			{
				if ( currentTeam->mGetTag() != previousTeam->mGetTag() )
				{
					remainingShipsOnSameTeam = false;
					break;
				}
			}
		}

		//
		//	If the current team still has the won flag standing, then save the ID so that we can compare
		//	on the next pass through.
		//
		if ( mTeamWon( i ) )
		{
			lastWinningTeam = i;
		}
	}

	//
	//	If we have hit a winning condition (all ships remaining on same team), then we should end the game
	//	for all remaining players with a winning condition.
	//
	if ( remainingShipsOnSameTeam )
	{
		const int maxPlayers = 6;
		i = 0;
		int TeamCounter = 0;
		while( i < fNumTeams && TeamCounter < kMaxTeams )
		{
			tTeamInfo* Team = gScriptInterface.fScript->mMission()->mGetTeamHandle( eTeamID(TeamCounter) );
			if ( mTeamWon( i ) )
			{
				if(Team)
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( TeamCounter ), kPlayerWins );
					i++;
				}
			}
			else
			{
				if(Team)
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( TeamCounter ), kPlayerLoses );
					i++;
				}
			}
			TeamCounter++;
		}
	}
}


int32 tWorldConditions::mShipsDestroyed( int x )
{
	return ( x >= 0 && x <= 5 )
		? fShipsDestroyed[x] : 0 ;
}
bool tWorldConditions::mTeamWon( int x )
{
	return ( x >= 0 && x <= 5 )
		? fTeamWon[x] : 0;
}

void tWorldConditions::mIncrementShipsDestroyed( int x )
{
	if ( x < 0 || x > 5 )
		return;
	fShipsDestroyed[x]++;
}

void tWorldConditions::mSetTeamWon( int x, bool y )
{
	if ( x < 0 || x > 5 )
		return;
	fTeamWon[x] = y;
}

void tWorldConditions::mGetTimeFromIni()
{
	const char filename[]		= "assets\\scripts\\scripts.ini";
	const char sectionname[]	= "[Mul_14TimedBattle]";
	char *line					= new char[256];
	int32 bufsize				= 256;
	int32 linesize, numstart;
	std::string	setting;



	ifstream inifile( filename, ios::nocreate | ios::in );
	if( inifile.fail() )
	{
		fTimeLimit = 20;
		delete[] line;
		return;
	}

	while( strcmp(line, sectionname) && inifile.good() )
	{
		inifile.getline( line, bufsize );

		if( !strcmp(line, sectionname) )
		{
			inifile.getline( line, bufsize );
			if( line )
			{	
				setting = line;
				::ZeroMemory( line, sizeof(line) );
				linesize = setting.size();
				numstart = setting.find("=") + 1;
				setting.copy( line, linesize - numstart, numstart );
				fTimeLimit = atoi( line );
			}
			break;
		}
	}
	
	if( fTimeLimit <= 0 || fTimeLimit > 99 )
	{
		fTimeLimit = 20;
	}

	delete[] line;
}
 
int32 tWorldConditions::mGetTimeLimit()
{
	if( !gScriptInterface.mIsHost() )
		return -1;

	mGetTimeFromIni();
	return fTimeLimit;
}
	
iMapPosition tWorldConditions::mGenerateStartPosition( eTeamTag teamTag )
{
	iMapPosition position = kStartPosition_G;

	switch( teamTag )
	{
	case kTagA:
		if( !fPositionUsed[kStartPosition_G] )
		{
			position = kStartPosition_G;
			fPositionUsed[kStartPosition_G] = kTrue;
		}
		else if( !fPositionUsed[kStartPosition_H] )
		{
			position = kStartPosition_H;
			fPositionUsed[kStartPosition_H] = kTrue;
		}
		else
		{
			position = kStartPosition_I;
			fPositionUsed[kStartPosition_I] = kTrue;
		}
		break;

	case kTagB:
		if( !fPositionUsed[kStartPosition_J] )
		{
			position = kStartPosition_J;
			fPositionUsed[kStartPosition_J] = kTrue;
		}
		else if( !fPositionUsed[kStartPosition_K] )
		{
			position = kStartPosition_K;
			fPositionUsed[kStartPosition_K] = kTrue;
		}
		else
		{
			position = kStartPosition_L;
			fPositionUsed[kStartPosition_L] = kTrue;
		}
		break;

	case kTagC:
		if( !fPositionUsed[kStartPosition_M] )
		{
			position = kStartPosition_M;
			fPositionUsed[kStartPosition_M] = kTrue;
		}
		else if( !fPositionUsed[kStartPosition_N] )
		{
			position = kStartPosition_N;
			fPositionUsed[kStartPosition_N] = kTrue;
		}
		else
		{
			position = kStartPosition_O;
			fPositionUsed[kStartPosition_O] = kTrue;
		}
		break;

	case kTagD:
	if( !fPositionUsed[kStartPosition_P] )
		{
			position = kStartPosition_P;
			fPositionUsed[kStartPosition_P] = kTrue;
		}
		else if( !fPositionUsed[kStartPosition_Q] )
		{
			position = kStartPosition_Q;
			fPositionUsed[kStartPosition_Q] = kTrue;
		}
		else
		{
			position = kStartPosition_R;
			fPositionUsed[kStartPosition_R] = kTrue;
		}
		break;

	case kTagE:
		if( !fPositionUsed[kStartPosition_S] )
		{
			position = kStartPosition_S;
			fPositionUsed[kStartPosition_S] = kTrue;
		}
		else if( !fPositionUsed[kStartPosition_T] )
		{
			position = kStartPosition_T;
			fPositionUsed[kStartPosition_T] = kTrue;
		}
		else
		{
			position = kStartPosition_U;
			fPositionUsed[kStartPosition_U] = kTrue;
		}
		break;

	case kTagF:
		if( !fPositionUsed[kStartPosition_V] )
		{
			position = kStartPosition_V;
			fPositionUsed[kStartPosition_V] = kTrue;
		}
		else if( !fPositionUsed[kStartPosition_W] )
		{
			position = kStartPosition_W;
			fPositionUsed[kStartPosition_W] = kTrue;
		}
		else
		{
			position = kStartPosition_X;
			fPositionUsed[kStartPosition_X] = kTrue;
		}
		break;

	default:
		position = kStartPosition_Y;
		fPositionUsed[kStartPosition_Y] = kTrue;
		break;
	}

	return position;
}