/********************************************************************************************************\

	Mission:	Multiplayer Tourney
	Script:		"Mul_1Free4All"
	Descript:	This is a battle between a max of 6 teams.  Each team has 1 and only 1 specially
				designed cruiser dependent upon the race of the team.

	Created by:		Daniel J. Suleski	06/14/00
	Last Updated:	DJS					07/17/00
					Second pass
					DJS					08/23/00
					Map overrides.
					DJS					08/26/00
					Text check.

\********************************************************************************************************/
#include "MissionText.h"
#include "MissionMaps.h"
#include "Mul_1Free4All.h"
#include "CommonTeam.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"

#include "teamtracker.h"
#include "scriptstructs.h"

DEFINE_SCRIPT_CLASS( tMul_1Free4All );



tMul_1Free4All::tMul_1Free4All()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tMul_1Free4All::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fHelpTags = Messages[kHelpTags_msg];
	Specs->fMissionType = kMultiplayerMission;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMenuStates = kEnableSupplyDock;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );
	Specs->fGenericFlags = kDisableCapture;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1",
												kTeam1,
												kMinChance,
												kTrue,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2",
												kTeam2,
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3",
												kTeam3,
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4",
												kTeam4,
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5",
												kTeam5,
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6",
												kTeam6,
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tMul_1Free4All::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tMul_1Free4All::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tMul_1Free4All::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tMul_1Free4All::mNewInternalShip( tShipInfo* Ship )
{}

void tMul_1Free4All::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 tagList[kRelationsCount] =
	{
		kTagA,
		kTagB,
		kTagC,
		kTagD,
		kTagE,
		kTagF,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 A     B     C     D     E     F
		/*A   */ "AI", "WD", "WD", "WD", "WD", "WD",
		/*B   */ "WD", "AI", "WD", "WD", "WD", "WD",
		/*C   */ "WD", "WD", "AI", "WD", "WD", "WD",
		/*D   */ "WD", "WD", "WD", "AI", "WD", "WD",
		/*E   */ "WD", "WD", "WD", "WD", "AI", "WD",
		/*F   */ "WD", "WD", "WD", "WD", "WD", "AI",
	};

	fMissionInfo->mSetTagRelations( tagList, ( std::string* )relationships, kRelationsCount );

	//
	//	The last thing that should be done before the mission actually starts is to check if the game
	//	has already been won.  This can most likely happen as a result of all teams having the same
	//	team tag.  Therefore, everyone is allied.
	//
	gWC.mCheckForGameEnd();
}

void tMul_1Free4All::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{
	switch ( teamScriptDescription.fTeamTag )
	{
	case kTagA:
	case kTagB:
	case kTagC:
	case kTagD:
	case kTagE:
	case kTagF:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									teamScriptDescription.fRace,
									teamScriptDescription.fHumanControlled,
									typeid( tCommonTeam ) );
		}
		break;
	}
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMul_1Free4All::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['L'] = kObjectPlayerShip;
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tMul_1Free4All::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}