/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"

#if defined ( _GERMAN )

/*
	G-L are fixed starting positions for up to 6 teams on the attacking side.
	T-Y are fixed starting positions for up to 6 teams on the defending side.
	Z is the fixed position for the defending team starbase.
*/
int32 TotalMaps = 15;
const char* MapTypes[] = {	"Offener Weltraum - K",
							"Offener Weltraum - M",
							"Offener Weltraum - G",
							"Planeten - K",
							"Planeten - M",
							"Planeten - G",
							"Asteroidenfeld - K",
							"Asteroidenfeld - M",
							"Asteroidenfeld - G",
							"Asteroidengrtel - K",
							"Asteroidengrtel - M",
							"Asteroidengrtel - G",
							"Nebel - K",
							"Nebel - M",
							"Nebel - G" };

/*
	Legend:
			Starting Positions			Ending Positions			Planets			
			Start1 --> G				End1 --> g					Earth --> 1
			Start2 --> H				End2 --> h					Ringed Earth --> 2
			Start3 --> I				End3 --> i					Mars --> 3
			Start4 --> J				End4 --> j					Jupiter --> 4
			Start5 --> K				End5 --> k					Organia --> 5
			Start6 --> L				End6 --> l					Saturn --> 6
			Start7 --> M				End7 --> m					Night City --> 7
			Start8 --> N				End8 --> n					Fire --> 8
			Start9 --> O				End9 --> o					Ice --> 9
			Start10 --> P				End10 --> p					Gas --> 0
			Start11 --> Q				End11 --> q					
			Start12 --> R				End12 --> r					
			Start13 --> S				End13 --> s					
			Start14 --> T				End14 --> t					
			Start15 --> U				End15 --> u					
			Start16 --> V				End16 --> v					
			Start17 --> W				End17 --> w					
			Start18 --> X				End18 --> x					
			Start19 --> Y				End19 --> y					
			Start20 --> Z				End20 --> z					

			Celestial Bodies			Asteroids					Black Holes
			Moon --> @					SmallAsteroid --> [			SmallBlackHole --> :
			Sun1 --> (					LargeAsteroid --> ]			LargeBlackHole --> ,
			Sun2 --> )					IceAsteroid --> *			
			Sun3 --> !
			
			Nebulas						Dust Clouds					Other
			Nebula1 --> &				LightDustCloud --> <		WormHole --> %
			Nebula2 --> ?				HeavyDustCloud --> >		Fissure --> $
																	Corpse --> #
																	Organian --> ^
*/
const char* MapLines[]=
{
   "Mul_2BaseAssault",

   "// Offener Weltraum - K",
   " +____1____2____3+",
   " |...............|",
   " |...............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   "1|...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |.......Z.......|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   "// Offener Weltraum - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J..........|",
   "4|.........................|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Offener Weltraum - G",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   
   
    "// Planeten - K",
   " +____1____2____3+",
   " |...............|",
   " |....X..Z..Y....|",
   " |....xV.z.Wy....|",
   " |.....vT.Uw.....|",
   "1|......t.u......|",
   " |...............|",
   " |...............|",
   " |..0....1....7..|",
   " |...............|",
   "2|...............|",
   " |....ghi.jkl....|",
   " |....GHI.JKL....|",
   " |...............|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   
   "// Planeten - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |................wW.......|",
   "1|....9...........xX..Z....|",
   " |................yY.......|",
   " |.........................|",
   " |...................TUV...|",
   " |...................tuv...|",
   "2|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.......2.................|",
   "4|.........................|",
   " |...ghi...................|",
   " |...jkl...................|",
   " |...GHI...................|",
   " |...JKL...................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   
   "// Planeten - G",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y.......7.......|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........1...............................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |................i.G.H.j.................|",
   " |..............k.I.....J.l......3........|",
   " |..............K.........L...............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",


   "// Asteroidenfeld - K",
   " +____1____2____3+",
   " |...............|",
   " |.A...a...a...A.|",
   " |...............|",
   " |.GHI...*...JKL.|",
   "1|.ghi.......jkl.|",
   " |.a..a...a....a.|",
   " |...............|",
   " |.......tuvwxy..|",
   " |.......TUVWXY..|",
   "2|.a...........a.|",
   " |....a.....a....|",
   " |.......A...z...|",
   " |.........a..Z..|",
   " |...A..a........|",
   "3|..........A....|",
   " +---------------+",
   "      1    2    3",

   "// Asteroidenfeld - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |...X.................U...|",
   " |..V.x...............u.W..|",
   " |.T.v.................w.Y.|",
   "1|..t...................y..|",
   " |...........A...a.........|",
   " |........A................|",
   " |.a...A............A......|",
   " |.........a...............|",
   "2|................A........|",
   " |A....*.............a..A..|",
   " |.........................|",
   " |....a......Z....*........|",
   " |...........z........a....|",
   "3|.................A.......|",
   " |....A...a................|",
   " |..............a........A.|",
   " |..a........A....a........|",
   " |......A..................|",
   "4|..............a...A......|",
   " |..g.......a...........l..|",
   " |.G.i.................j.L.|",
   " |..I.k...............h.J..|",
   " |...K.................H...|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Asteroidenfeld - G",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........*.....................*.........|",
   " |.....................................*..|",
   "1|...................Z....................|",
   " |.....[.........[...z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y..........[....|",
   "2|................v.T.U.w.................|",
   " |..*......]........t.u...................|",
   " |.................................]......|",
   " |........................................|",
   " |........................................|",
   "3|.....[..................................|",
   " |........................................|",
   " |..............[...................[.....|",
   " |......................[.................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |.......[.....................*..........|",
   " |.................*......................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |....]...........i.G.H.j.................|",
   " |..............k.I.....J.l......[....*...|",
   " |..............K.........L...............|",
   "6|.........*..............................|",
   " |........................................|",
   " |........................................|",
   " |...................]....................|",
   " |...]............................].......|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |..*................................[....|",
   " |........].................].............|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   
   
   "// Asteroidengrtel - K",
   " +____1____2____3+",
   " |............a..|",
   " |.Z.........Y...|",
   " |..z......X..ya.|",
   " |.......W..x.A..|",
   "1|.....V..w.a....|",
   " |...U..v..A...a.|",
   " |.T..u..a..a....|",
   " |..t..a......l..|",
   " |...a....a.i..L.|",
   "2|...A.a....Ik...|",
   " |.a......h...K..|",
   " |...a..a..Hj....|",
   " |..A.....g..J...|",
   " |.a...a..G......|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   "// Asteroidengrtel - M",
   " +____1____2____3____4____5+",
   " |..A............a.........|",
   " |....a.......G.....a......|",
   " |............g...A........|",
   " |.Tt....a.......A......hH.|",
   "1|....A..............A.....|",
   " |................a........|",
   " |.......A...........a.....|",
   " |.Uu..a..........*.....iI.|",
   " |.......*..........A......|",
   "2|....A................A...|",
   " |........a.......A........|",
   " |.Vv..A...........a..a.jJ.|",
   " |.........................|",
   " |.........a.......*....A..|",
   "3|......a............A.....|",
   " |.Ww..A...A............kK.|",
   " |..................A..a...|",
   " |.......a..*..............|",
   " |..................a......|",
   "4|.Xx....A..a..........AlL.|",
   " |......A..........*.......|",
   " |...z...a...........a.....|",
   " |..Z.......A....y......a..|",
   " |.......A...a...Y....A....|",
   "5|.........a..........a..A.|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Asteroidengrtel - G",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |[[[[[[[[[...............................|",
   " |*********[[[[[[[[[[[[...................|",
   " |]]]]]]]]]************[[[[[[[[[[[[[......|",
   " |.........]]]]]]]]]]]]*************[[[[[[|",
   "4|.....................]]]]]]]]]]]]]******|",
   " |..................................]]]]]]|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   
   
   "// Nebel - K",
   " +____1____2____3+",
   " |...............|",
   " |.&.............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   "1|...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |.......Z.......|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   "// Nebel - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.&.......................|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J..........|",
   "4|.........................|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Nebel - G",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |.&......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |...................g.h..................|",
   " |.................i.G.H.j................|",
   " |...............k.I.....J.l..............|",
   " |...............K.........L..............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "///",
   0
};

#else

/*
	G-L are fixed starting positions for up to 6 teams on the attacking side.
	T-Y are fixed starting positions for up to 6 teams on the defending side.
	Z is the fixed position for the defending team starbase.
*/
int32 TotalMaps = 15;
const char* MapTypes[] = {	"Open Space - S",
							"Open Space - M",
							"Open Space - L",
							"Planets - S",
							"Planets - M",
							"Planets - L",
							"Asteroid Field - S",
							"Asteroid Field - M",
							"Asteroid Field - L",
							"Asteroid Belt - S",
							"Asteroid Belt - M",
							"Asteroid Belt - L",
							"Nebula - S",
							"Nebula - M",
							"Nebula - L" };

/*
	Legend:
			Starting Positions			Ending Positions			Planets			
			Start1 --> G				End1 --> g					Earth --> 1
			Start2 --> H				End2 --> h					Ringed Earth --> 2
			Start3 --> I				End3 --> i					Mars --> 3
			Start4 --> J				End4 --> j					Jupiter --> 4
			Start5 --> K				End5 --> k					Organia --> 5
			Start6 --> L				End6 --> l					Saturn --> 6
			Start7 --> M				End7 --> m					Night City --> 7
			Start8 --> N				End8 --> n					Fire --> 8
			Start9 --> O				End9 --> o					Ice --> 9
			Start10 --> P				End10 --> p					Gas --> 0
			Start11 --> Q				End11 --> q					
			Start12 --> R				End12 --> r					
			Start13 --> S				End13 --> s					
			Start14 --> T				End14 --> t					
			Start15 --> U				End15 --> u					
			Start16 --> V				End16 --> v					
			Start17 --> W				End17 --> w					
			Start18 --> X				End18 --> x					
			Start19 --> Y				End19 --> y					
			Start20 --> Z				End20 --> z					

			Celestial Bodies			Asteroids					Black Holes
			Moon --> @					SmallAsteroid --> [			SmallBlackHole --> :
			Sun1 --> (					LargeAsteroid --> ]			LargeBlackHole --> ,
			Sun2 --> )					IceAsteroid --> *			
			Sun3 --> !
			
			Nebulas						Dust Clouds					Other
			Nebula1 --> &				LightDustCloud --> <		WormHole --> %
			Nebula2 --> ?				HeavyDustCloud --> >		Fissure --> $
																	Corpse --> #
																	Organian --> ^
*/
const char* MapLines[]=
{
   "Mul_2BaseAssault",

   "// Open Space - S",
   " +____1____2____3+",
   " |...............|",
   " |...............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   "1|...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |.......Z.......|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   "// Open Space - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J..........|",
   "4|.........................|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Open Space - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   
   
    "// Planets - S",
   " +____1____2____3+",
   " |...............|",
   " |....X..Z..Y....|",
   " |....xV.z.Wy....|",
   " |.....vT.Uw.....|",
   "1|......t.u......|",
   " |...............|",
   " |...............|",
   " |..0....1....7..|",
   " |...............|",
   "2|...............|",
   " |....ghi.jkl....|",
   " |....GHI.JKL....|",
   " |...............|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   
   "// Planets - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |................wW.......|",
   "1|....9...........xX..Z....|",
   " |................yY.......|",
   " |.........................|",
   " |...................TUV...|",
   " |...................tuv...|",
   "2|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.......2.................|",
   "4|.........................|",
   " |...ghi...................|",
   " |...jkl...................|",
   " |...GHI...................|",
   " |...JKL...................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   
   "// Planets - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y.......7.......|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........1...............................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |................i.G.H.j.................|",
   " |..............k.I.....J.l......3........|",
   " |..............K.........L...............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",


   "// Asteroid Field - S",
   " +____1____2____3+",
   " |...............|",
   " |.A...a...a...A.|",
   " |...............|",
   " |.GHI...*...JKL.|",
   "1|.ghi.......jkl.|",
   " |.a..a...a....a.|",
   " |...............|",
   " |.......tuvwxy..|",
   " |.......TUVWXY..|",
   "2|.a...........a.|",
   " |....a.....a....|",
   " |.......A...z...|",
   " |.........a..Z..|",
   " |...A..a........|",
   "3|..........A....|",
   " +---------------+",
   "      1    2    3",

   "// Asteroid Field - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |...X.................U...|",
   " |..V.x...............u.W..|",
   " |.T.v.................w.Y.|",
   "1|..t...................y..|",
   " |...........A...a.........|",
   " |........A................|",
   " |.a...A............A......|",
   " |.........a...............|",
   "2|................A........|",
   " |A....*.............a..A..|",
   " |.........................|",
   " |....a......Z....*........|",
   " |...........z........a....|",
   "3|.................A.......|",
   " |....A...a................|",
   " |..............a........A.|",
   " |..a........A....a........|",
   " |......A..................|",
   "4|..............a...A......|",
   " |..g.......a...........l..|",
   " |.G.i.................j.L.|",
   " |..I.k...............h.J..|",
   " |...K.................H...|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Asteroid Field - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........*.....................*.........|",
   " |.....................................*..|",
   "1|...................Z....................|",
   " |.....[.........[...z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y..........[....|",
   "2|................v.T.U.w.................|",
   " |..*......]........t.u...................|",
   " |.................................]......|",
   " |........................................|",
   " |........................................|",
   "3|.....[..................................|",
   " |........................................|",
   " |..............[...................[.....|",
   " |......................[.................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |.......[.....................*..........|",
   " |.................*......................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |....]...........i.G.H.j.................|",
   " |..............k.I.....J.l......[....*...|",
   " |..............K.........L...............|",
   "6|.........*..............................|",
   " |........................................|",
   " |........................................|",
   " |...................]....................|",
   " |...]............................].......|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |..*................................[....|",
   " |........].................].............|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   
   
   "// Asteroid Belt - S",
   " +____1____2____3+",
   " |............a..|",
   " |.Z.........Y...|",
   " |..z......X..ya.|",
   " |.......W..x.A..|",
   "1|.....V..w.a....|",
   " |...U..v..A...a.|",
   " |.T..u..a..a....|",
   " |..t..a......l..|",
   " |...a....a.i..L.|",
   "2|...A.a....Ik...|",
   " |.a......h...K..|",
   " |...a..a..Hj....|",
   " |..A.....g..J...|",
   " |.a...a..G......|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   "// Asteroid Belt - M",
   " +____1____2____3____4____5+",
   " |..A............a.........|",
   " |....a.......G.....a......|",
   " |............g...A........|",
   " |.Tt....a.......A......hH.|",
   "1|....A..............A.....|",
   " |................a........|",
   " |.......A...........a.....|",
   " |.Uu..a..........*.....iI.|",
   " |.......*..........A......|",
   "2|....A................A...|",
   " |........a.......A........|",
   " |.Vv..A...........a..a.jJ.|",
   " |.........................|",
   " |.........a.......*....A..|",
   "3|......a............A.....|",
   " |.Ww..A...A............kK.|",
   " |..................A..a...|",
   " |.......a..*..............|",
   " |..................a......|",
   "4|.Xx....A..a..........AlL.|",
   " |......A..........*.......|",
   " |...z...a...........a.....|",
   " |..Z.......A....y......a..|",
   " |.......A...a...Y....A....|",
   "5|.........a..........a..A.|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Asteroid Belt - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |[[[[[[[[[...............................|",
   " |*********[[[[[[[[[[[[...................|",
   " |]]]]]]]]]************[[[[[[[[[[[[[......|",
   " |.........]]]]]]]]]]]]*************[[[[[[|",
   "4|.....................]]]]]]]]]]]]]******|",
   " |..................................]]]]]]|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |..................g.h...................|",
   " |................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   
   
   "// Nebula - S",
   " +____1____2____3+",
   " |...............|",
   " |.&.............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   "1|...............|",
   " |...............|",
   " |...............|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |.......Z.......|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",

   "// Nebula - M",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.&.......................|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J..........|",
   "4|.........................|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",

   "// Nebula - L",
   " +____1____2____3____4____5____6____7____8+",
   " |........................................|",
   " |.&......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |...................g.h..................|",
   " |.................i.G.H.j................|",
   " |...............k.I.....J.l..............|",
   " |...............K.........L..............|",
   "6|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "7|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "8|........................................|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "///",
   0
};

#endif

