/********************************************************************************************************\

	Mission:	Mul_2BaseAssault
	Script:		"Mul_2BaseAssault"
	Descript:	<DESCRIPTION>

	Created by:		Daniel J. Suleski & Warren Shultz		06/16/00
	Last Updated:	DJS & WS								06/17/00
					Made it so that humans can play on either the attacking or defending team.
					DJS										06/19/00
					Added world conditions and fixed various team-related bugs.
					DJS										06/20/00
					Added victory conditions; therefore, completed the script.
					DJS										07/18/00
					Second pass.
					DJS										08/23/00
					Map overrides.
					DJS										08/26/00
					Text check.  Re-enable AI.
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"

#include "Mul_2BaseAssault.h"
#include "VictoryCondition.h"

#include "AttackTeam.h"
#include "DefendTeam.h"


DEFINE_SCRIPT_CLASS( tMul_2BaseAssault );

tMul_2BaseAssault::tMul_2BaseAssault()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tMul_2BaseAssault::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fHelpTags = Messages[kHelpTags_msg];
	Specs->fMissionType = kMultiplayerMission;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMenuStates = kDisableTagC | kDisableTagD | kDisableTagE | kDisableTagF | kEnableSupplyDock;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k2Band | k4Client );
	Specs->fGenericFlags = kDisableCapture;

	/*

		HELPER NOTE:	In this case, there are 6 teams setup.  Feel free to change the number of teams
						by deleting them starting with team6 and working your way down to team3, inclusive.
						Also, if the team is to be controlled by the AI and only the AI, then pass the
						constant 'kNPCTeam' instead of 'kPlayableTeam' into the team specification.  Also,
						not noted here, the last parameter of mCreateTeamSpec() can be a team tag such as
						"kTagA" to "kTagF".  This is the default tag for a kNPCTeam that was not chosen
						on the multiplayer UI screen.

	*/
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1",
												static_cast< eTeamID >( kPlayerTeam1 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAttackTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2",
												static_cast< eTeamID >( kPlayerTeam2 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAttackTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3",
												static_cast< eTeamID >( kPlayerTeam3 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAttackTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4",
												static_cast< eTeamID >( kPlayerTeam4 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAttackTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5",
												static_cast< eTeamID >( kPlayerTeam5 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAttackTeam ) );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Defense Team",
												static_cast< eTeamID >( kDefenseTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kDefendTag ) );

	const tTeamSpec& team8 = mCreateTeamSpec(	"Attack Team",
												static_cast< eTeamID >( kAttackTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAttackTeam ),
												static_cast< eTeamTag >( kAttackTag ) );


	/*

		HELPER NOTE:	Please note that although you setup the teams, you need to create ships and their
						respective files.  Otherwise, an assert will occur.  It is up to you to decide
						whether the ship files should be shared by multiple physical teams or just a
						1-to-1 relationship between the team and the ship files.

	*/
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team7 );
	Specs->mSetupTeam( team8 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMul_2BaseAssault::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tMul_2BaseAssault::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMul_2BaseAssault::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMul_2BaseAssault::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMul_2BaseAssault::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 tagList[kRelationsCount] =
	{
		kTagA,
		kTagB,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 A     B    
		/*A   */ "AI", "WD",
		/*B   */ "WD", "AI",

	};

	fMissionInfo->mSetTagRelations( tagList, ( std::string* )relationships, kRelationsCount );

	/*

		HELPER NOTE:	You may wish to check if the game has ended here in the case, for instance, in
						the case that all physical teams share the same team tag letter and therefore, have
						no common enemies.

	*/
}

/*

	HELPER NOTE:	Since you really do not know which teams are going to be associated with which team
					tages until the game starts, you must create the teams here.  This function may look
					like the following:

					void tMul_2BaseAssault::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
					{
						switch ( teamScriptDescription.fTeamTag )
						{
						case kTagA:
						case kTagB:
						case kTagC:
						case kTagD:
						case kTagE:
						case kTagF:
							{
								mMission()->mSetupTeam(	teamScriptDescription.fTeam,
														teamScriptDescription.fTeamTag,
														teamScriptDescription.fRace,
														teamScriptDescription.fHumanControlled,
														typeid( tCommonTeam ) );
							}
							break;
						}
					}

*/
void tMul_2BaseAssault::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{
	//
	//	The team that comes through here must be either tag A or tag B -- no other.
	//
	Assert( ( teamScriptDescription.fTeamTag == kAttackTag ) ||
			( teamScriptDescription.fTeamTag == kDefendTag ) );
	if ( !( ( teamScriptDescription.fTeamTag == kAttackTag ) ||
			( teamScriptDescription.fTeamTag == kDefendTag ) ) )
	{
		return;
	}

	//
	//	Figure out what the defending team should be.
	//
	eRace defendingRace = kNoRace;
	for ( int32 i = kPlayerTeam1; i <= kPlayerTeam6; i++ )
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( team )
		{
			if ( team->mGetTag() == kDefendTag )
			{
				defendingRace = team->mGetRace();
			}
		}
	}
	if ( defendingRace == kNoRace )
	{
		defendingRace = mChooseRandomStarbaseRace( kAttackTag );
	}
	if ( defendingRace == kNoRace )
	{
		defendingRace = kFederation;
	}

	//
	//	Figure out what the attacking team should be.
	//
	eRace attackingRace = kNoRace;
	for ( i = kPlayerTeam1; i <= kPlayerTeam6; i++ )
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( team )
		{
			if ( team->mGetTag() == kAttackTag )
			{
				attackingRace = team->mGetRace();
			}
		}
	}
	if ( attackingRace == kNoRace )
	{
		attackingRace = mChooseRandomStarbaseRace( kDefendTag );
	}
	if ( attackingRace == kNoRace )
	{
		attackingRace = kKlingon;
	}

	switch ( teamScriptDescription.fTeamTag )
	{
	case kAttackTag:
		{
			if ( teamScriptDescription.fTeam == kAttackTeam )
				mMission()->mSetupTeam( teamScriptDescription.fTeam, teamScriptDescription.fTeamTag, attackingRace, teamScriptDescription.fHumanControlled, typeid( tAttackTeam ) );
			else
				mMission()->mSetupTeam( teamScriptDescription.fTeam, teamScriptDescription.fTeamTag, teamScriptDescription.fRace, teamScriptDescription.fHumanControlled, typeid( tAttackTeam ) );
		}
		break;
	
	case kDefendTag:
		{
			if ( teamScriptDescription.fTeam == kDefenseTeam )
				mMission()->mSetupTeam( teamScriptDescription.fTeam, teamScriptDescription.fTeamTag, defendingRace, teamScriptDescription.fHumanControlled, typeid( tDefendTeam ) );
			else
				mMission()->mSetupTeam( teamScriptDescription.fTeam, teamScriptDescription.fTeamTag, teamScriptDescription.fRace, teamScriptDescription.fHumanControlled, typeid( tDefendTeam ) );
		}
		break;
	}
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMul_2BaseAssault::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['L'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyBase;
}

eRace tMul_2BaseAssault::mChooseRandomStarbaseRace( int32 tag )
{
	eRace chosenRace = kNoRace;

	//
	//	Find a valid, random race that is unique from any other attacking race.
	//
	int32 timesAround = 0;
	while ( ( chosenRace == kNoRace ) || ( timesAround <= kNumActiveRaces ) )
	{
		chosenRace = static_cast< eRace >( rand() % ( kNumActiveRaces - kFirstRace ) + kFirstRace );
		
		iTruth isDuplicated = kFalse;
		for ( int32 i = kPlayerTeam1; i <= kPlayerTeam6; i++ )
		{
			tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( team->mGetRace() == chosenRace && team->mGetTag() == tag )
				{
					isDuplicated = kTrue;
					break;
				}
			}
		}

		if ( isDuplicated )
			chosenRace = kNoRace;

		timesAround++;
	}

	return chosenRace;
}