/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "Mul_2BaseAssault.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions():	fTotalAttackTeams ( 0 ), fTotalDefendTeams ( 0 ),
										fTotalAttackShips ( 0 ), fTotalDefendShips ( 0 ),
										fAttackShipsDestroyed( 0 ), fDefendShipsDestroyed( 0 ),
										fStarbaseDestroyed( false )
{}

/*

	Purpose:	The condition for the game ending will occur at the following times:

				1. The starbase is destroyed.
				2. All of the ships on the attacking team are destroyed.
				
				The game will not be check if all human players are on one team in the beginning
				because if this is the case, then the AI will take over controlling the opposing side.
				In addition, the game end will not be checked if all of the defending ships are
				destroyed because the starbase should still be in play.  This should allow the
				defending players to watch until one of the above conditions are met.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	//
	//	Let's check if the starbase is destroyed.  If so, then all of the defending teams get a
	//	losing condition and all of the attacking teams get a winning condition set.
	//
	if ( mStarbaseDestroyed() )
	{
		for ( int32 i = kPlayerTeam1; i <= kAttackTeam; i++ )
		{
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( team->mGetGameStatus() == kGameInProgress )
				{
					if ( team->mGetTag() == kAttackTag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerWins );
					}
					else if ( team->mGetTag() == kDefendTag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerLoses );
					}
				}
			}
		}

		//
		//	No need to go on.
		//
		return;
	}

	//
	//	Let's check if all of the attacking ships are destroyed.
	//
	if ( mAttackShipsDestroyed() >= mTotalAttackShips() )
	{
		for ( int32 i = kPlayerTeam1; i <= kAttackTeam; i++ )
		{
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( team->mGetTag() == kAttackTag )
				{
					gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerLoses );
				}
				else if ( team->mGetTag() == kDefendTag )
				{
					gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerWins );
				}
			}
		}

		//
		//	No need to go on.
		//
		return;
	}

	//
	//	If we got this far, then the game is not considered to be completed yet.
	//
}

void tWorldConditions::mIncrementTotalAttackTeams()
{
	fTotalAttackTeams++;
}
void tWorldConditions::mIncrementTotalDefendTeams()
{
	fTotalDefendTeams++;
}
void tWorldConditions::mIncrementTotalAttackShips()
{
	fTotalAttackShips++;
}
void tWorldConditions::mIncrementTotalDefendShips()
{
	fTotalDefendShips++;
}
void tWorldConditions::mIncrementAttackShipsDestroyed()
{
	fAttackShipsDestroyed++;
}
void tWorldConditions::mIncrementDefendShipsDestroyed()
{
	fDefendShipsDestroyed++;
}
void tWorldConditions::mSetStarbaseDestroyed( bool x )
{
	fStarbaseDestroyed = x;
}

int32 tWorldConditions::mTotalAttackTeams()
{
	return fTotalAttackTeams;
}
int32 tWorldConditions::mTotalDefendTeams()
{
	return fTotalDefendTeams;
}	
int32 tWorldConditions::mTotalAttackShips()
{
	return fTotalAttackShips;
}
int32 tWorldConditions::mTotalDefendShips()
{
	return fTotalDefendShips;
}
int32 tWorldConditions::mAttackShipsDestroyed()
{
	return fAttackShipsDestroyed;
}
int32 tWorldConditions::mDefendShipsDestroyed()
{
	return fDefendShipsDestroyed;
}
bool tWorldConditions::mStarbaseDestroyed()
{
	return fStarbaseDestroyed;
}

