/********************************************************************************************************\

	File Name:	CommonHeavyBattlecruiserBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "MissionText.h"
#include "CommonDefs.h"
#include "CommonHeavyBattlecruiserBaseState.h"
#include "CommonDreadnought.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonHeavyBattlecruiserBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kInvincibility_TID, 0.5f );
}
void tCommonHeavyBattlecruiserBaseState::mEndState()
{}

void tCommonHeavyBattlecruiserBaseState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );

	Assert( team );
	if ( !team )
		return;

	if ( fChildActor->mGetTeam() == teamID )
	{
		if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
		{

			//
			//	The HeavyBattlecruiser is lost, so spawn a Dreadnought.
			//
			gWC.mSetHeavyBattlecruiserDestroyed( teamID, true );
			team->mCreateShip(	typeid( tCommonDreadnought ),
								kMaxBPV,
								kClassDreadnought,
								100,
								gWC.mGenerateRandomStartPosition(),
								kDreadnought,
								0, 0, -1, -1, NULL,
								kDefaultShipOptions );//static_cast< eShipOptions >( kDefaultShipOptions | kDeathUponDisengage ) );
		}
	}
}
void tCommonHeavyBattlecruiserBaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mAtDistance( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonHeavyBattlecruiserBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonHeavyBattlecruiserBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonHeavyBattlecruiserBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonHeavyBattlecruiserBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonHeavyBattlecruiserBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonHeavyBattlecruiserBaseState::mOnFlashingButtonPressed()
{}
void tCommonHeavyBattlecruiserBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonHeavyBattlecruiserBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonHeavyBattlecruiserBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tCommonHeavyBattlecruiserBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonHeavyBattlecruiserBaseState::mOnHostChanged()
{}
void tCommonHeavyBattlecruiserBaseState::mOnTimedInterval( int32 timerID )
{
	switch ( timerID )
	{
	case kInvincibility_TID:
		{
			fChildActor->mSetTargetable( kTrue );
		}
		break;
	}
}
void tCommonHeavyBattlecruiserBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonHeavyBattlecruiserBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


