/********************************************************************************************************\

	File Name:	CommonLightCruiserBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "MissionText.h"
#include "CommonDefs.h"
#include "CommonLightCruiserBaseState.h"
#include "CommonHeavyCruiser.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonLightCruiserBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kInvincibility_TID, 0.5f );
}
void tCommonLightCruiserBaseState::mEndState()
{}

void tCommonLightCruiserBaseState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );

	Assert( team );
	if ( !team )
		return;

	if ( fChildActor->mGetTeam() == teamID )
	{
		if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
		{

			//
			//	The LightCruiser is lost, so spawn a Heavy Cruiser.
			//
			gWC.mSetLightCruiserDestroyed( teamID, true );
			team->mCreateShip(	typeid( tCommonHeavyCruiser ),
								kMaxBPV,
								kClassHeavyCruiser,
								100,
								gWC.mGenerateRandomStartPosition(),
								kHeavyCruiser,
								0, 0, -1, -1, NULL,
								static_cast< eShipOptions >( kDefaultShipOptions | kDeathUponDisengage ) );
		}
	}
}
void tCommonLightCruiserBaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mAtDistance( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonLightCruiserBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonLightCruiserBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonLightCruiserBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonLightCruiserBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonLightCruiserBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonLightCruiserBaseState::mOnFlashingButtonPressed()
{}
void tCommonLightCruiserBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonLightCruiserBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonLightCruiserBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tCommonLightCruiserBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonLightCruiserBaseState::mOnHostChanged()
{}
void tCommonLightCruiserBaseState::mOnTimedInterval( int32 timerID )
{
	switch ( timerID )
	{
	case kInvincibility_TID:
		{
			fChildActor->mSetTargetable( kTrue );
		}
		break;
	}
}
void tCommonLightCruiserBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonLightCruiserBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


