/********************************************************************************************************\

	File Name:	CommonHeavyCruiserBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "MissionText.h"
#include "CommonDefs.h"
#include "CommonDreadnought.h"
#include "CommonHeavyCruiserBaseState.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonHeavyCruiserBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kInvincibility_TID, 0.5f );
}
void tCommonHeavyCruiserBaseState::mEndState()
{}

void tCommonHeavyCruiserBaseState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );

	Assert( team );
	if ( !team )
		return;

	if ( fChildActor->mGetTeam() == teamID )
	{
		if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
		{

			//
			//	The Heavy Cruiser is lost, so spawn a Dreadnought.
			//
			gWC.mSetHeavyCruiserDestroyed( teamID, true );
			team->mCreateShip(typeid( tCommonDreadnought ),
									kMaxBPV,
									kClassDreadnought,
									100,
									gWC.mGenerateRandomStartPosition(),
									kDreadnought,
									0, 0, -1, -1, NULL,
									kDefaultShipOptions );
		}
	}
}
void tCommonHeavyCruiserBaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonHeavyCruiserBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );

	Assert( team );
	if ( !team )
		return;

	if ( fChildActor->mGetTeam() == teamID )
	{
		if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
		{

			//
			//	The Heavy Cruiser is lost, so spawn a Dreadnought.
			//
			gWC.mSetHeavyCruiserDestroyed( teamID, true );
		}
	}
}
void tCommonHeavyCruiserBaseState::mAtDistance( tShipInfo* ship )
{}
void tCommonHeavyCruiserBaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonHeavyCruiserBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonHeavyCruiserBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonHeavyCruiserBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonHeavyCruiserBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonHeavyCruiserBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonHeavyCruiserBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonHeavyCruiserBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonHeavyCruiserBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonHeavyCruiserBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonHeavyCruiserBaseState::mOnFlashingButtonPressed()
{}
void tCommonHeavyCruiserBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonHeavyCruiserBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonHeavyCruiserBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tCommonHeavyCruiserBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonHeavyCruiserBaseState::mOnHostChanged()
{}
void tCommonHeavyCruiserBaseState::mOnTimedInterval( int32 timerID )
{
	switch ( timerID )
	{
	case kInvincibility_TID:
		{
			fChildActor->mSetTargetable( kTrue );
		}
		break;
	}
}
void tCommonHeavyCruiserBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonHeavyCruiserBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


