/********************************************************************************************************\

	File Name:	CommonShip2BaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "MissionText.h"
#include "CommonDefs.h"
#include "CommonShip2BaseState.h"
#include "CommonShip3.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonShip2BaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kInvincibility_TID, 0.5f );
}
void tCommonShip2BaseState::mEndState()
{}

void tCommonShip2BaseState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );

	Assert( team );
	if ( !team )
		return;

	if ( fChildActor->mGetTeam() == teamID )
	{
		if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
		{

			//
			//	The Ship2 is lost, so spawn a Ship3.
			//
			gWC.mSetShip2Destroyed( teamID, true );
			team->mCreateShip(	typeid( tCommonShip3 ),
								gClassNames[static_cast< int32 >( team->mGetRace() )][2],
								kStartPosition_G + ship->mGetTeam(),
								tTeamInfo::kNoMetaShipID,
								kShip3,
								0, 0, -1, -1, NULL,
								kDefaultShipOptions );
		}
	}
}
void tCommonShip2BaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonShip2BaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );

	Assert( team );
	if ( !team )
		return;

	if ( fChildActor->mGetTeam() == teamID )
	{
		if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
		{

			//
			//	The Ship2 is lost, so spawn a Ship3.
			//
			gWC.mSetShip2Destroyed( teamID, true );
		}
	}
}
void tCommonShip2BaseState::mAtDistance( tShipInfo* ship )
{}
void tCommonShip2BaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonShip2BaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonShip2BaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonShip2BaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonShip2BaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonShip2BaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonShip2BaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonShip2BaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonShip2BaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonShip2BaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonShip2BaseState::mOnFlashingButtonPressed()
{}
void tCommonShip2BaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonShip2BaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonShip2BaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tCommonShip2BaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonShip2BaseState::mOnHostChanged()
{}
void tCommonShip2BaseState::mOnTimedInterval( int32 timerID )
{
	switch ( timerID )
	{
	case kInvincibility_TID:
		{
			fChildActor->mSetTargetable( kTrue );
		}
		break;
	}
}
void tCommonShip2BaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonShip2BaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


