/********************************************************************************************************\

	File Name:	CommonTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionText.h"
#include "CommonBaseVictory.h"
#include "CommonDefs.h"
#include "CommonShip1.h"
#include "CommonTeam.h"
#include "CommonTeamBaseState.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


tClassAllocator<tCommonTeam> gCommonTeamAllocator;

tCommonTeam::tCommonTeam( void )
{
	gWC.fNumTeams++;
}

void tCommonTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;

	case kDraw:
	default:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tCommonTeam::mSetupState()
{
	mRegisterState( new tCommonTeamBaseState() );
	mGotoState( typeid( tCommonTeamBaseState ) );
}

void tCommonTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tCommonTeam::mCreateShipsForTeam()
{
	mCreateShip(	typeid( tCommonShip1 ),
					gClassNames[static_cast< int32 >( mGetRace() )][fMissionInfo->mRandomInt32(0, 2 )],
					gWC.mGenerateRandomStartPosition(),
					kNoMetaShipID,
					kShip1,
					0, 0, -1, -1, NULL,
					kDefaultShipOptions );
}

void tCommonTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	for( ; *ship != NULL; ++ship )
	{
		ship->mSetShipCallbackEvents( kNotifyDamage );
	}
}

void tCommonTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{}
