/********************************************************************************************************\

	File Name:	DefendTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "DefendShip.h"
#include "DefendTeam.h"
#include "DefendTeamBaseState.h"
#include "MissionText.h"
#include "Mul_9TeamAssault.h"
#include "ShipScriptDescription.h"
#include "Starbase.h"
#include "WorldConditions.h"


tClassAllocator<tDefendTeam> gDefendTeamAllocator;


tDefendTeam::tDefendTeam(): fCustomID( kDefender1 )
{
	gWC.mIncrementTotalDefendTeams();
}

void tDefendTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tDefendTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;

	case kDraw:
	default:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tDefendTeam::mSetupState()
{
	mRegisterState( new tDefendTeamBaseState() );
	mGotoState( typeid( tDefendTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tDefendTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tDefendTeam::mCreateShipsForTeam()
{
	//
	//	Add up the total BPV of the ships on the attacking team.
	//
	int32 totalBPV = 0;
	for ( int32 i = kTeam1; i <= kTeam8; i++ )  // get total of team 1-6 bpv
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast < eTeamID > ( i ) );
		if ( team && team->mGetTag() == kTagA )
		{
			totalBPV += team->mGetCombatBPV();
		}
	}

	//
	//	Get the race of the first defending team and use that race to determine the race of the
	//	defending team's starbase regardless of what the other defender races are.
	//
	eRace defendingRace = kNoRace;
	tTeamInfo* team = NULL;
	for ( i = kPlayerTeam1; i <= kDefenseTeam; i++ )
	{
		team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( i ) );
		if ( !team )
			continue;

		if ( team->mGetTag() == kDefendTag )
			break;
	}
	if ( team )
		defendingRace = team->mGetRace();

	//
	//	Create AI controlled ships if a human is not on the defending team.
	//
	if ( !( gWC.mTotalDefendTeams() - kNumHardCodedNPCTeams ) )
	{	
		for ( int32 i = 0; i < gWC.mTotalAttackTeams() - kNumHardCodedNPCTeams; i++ )
		{
			mCreateShip(	typeid( tDefendShip ),
							totalBPV / gWC.mTotalAttackTeams(),
							kClassFrigate,
							100,
							kStartPosition_T + i,
							kDefender1,
							0, 0, -1, -1, NULL,
							kDefaultShipOptions,
							( ( mGetTeam() == kDefenseTeam ) ? defendingRace : kNoRace ) );
		}
	}

	//
	//	We need 1 starbase for the defending team, so since there is always going to be 1 AI controlled
	//	defending team, we should create that 1 starbase here.
	//
	if ( mGetTeam() == kDefenseTeam )
	{
		switch ( defendingRace )
		{
		case kFederation:
			mCreateShip(	typeid( tStarbase ), "F-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		case kKlingon:
			mCreateShip(	typeid( tStarbase ), "K-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		case kRomulan:
			mCreateShip(	typeid( tStarbase ), "R-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		case kLyran:
			mCreateShip(	typeid( tStarbase ), "L-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		case kGorn:
			mCreateShip(	typeid( tStarbase ), "G-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		case kHydran:
			mCreateShip(	typeid( tStarbase ), "H-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		case kMirak:
			mCreateShip(	typeid( tStarbase ), "Z-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		case kISC:
			mCreateShip(	typeid( tStarbase ), "I-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		default:
			mCreateShip(	typeid( tStarbase ), "O-SB", kStartPosition_Z, kNoMetaShipID,
							kDefenderStarbase, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
		}
	}
}

void tDefendTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
	if ( mIsTeamHumanControlled() )
	{
		tShipIterator ship = mGetFirstShip();

		Assert ( *ship != NULL );

		if ( *ship != NULL )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tDefendTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	if ( mGetTeam() != kDefenseTeam )
	{
		mCreateShip(	typeid( tDefendShip ), 
						shipScriptDescription.fClassName.c_str(), 
						kStartPosition_T + shipScriptDescription.fTeamID,
						kNoMetaShipID,
						fCustomID, 0, 0, -1, -1, //Default data,so we can set the custom name
						shipScriptDescription.fCustomName.c_str(),
						kDefaultShipOptions );

		fCustomID++;
	}
}

