/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"

#if defined ( _GERMAN )

/*
	G-L are fixed starting positions for up to 6 teams on the attacking side.
	T-Y are fixed starting positions for up to 6 teams on the defending side.
	Z is the fixed position for the defending team starbase.
*/
int32 TotalMaps = 15;
const char* MapTypes[] = {	"Offener Weltraum - G",
							"Offener Weltraum - M",
							"Offener Weltraum - K",
							"Planeten - G",
							"Planeten - M",
							"Planeten - K",
							"Asteroidenfeld - G",
							"Asteroidenfeld - M",
							"Asteroidenfeld - K",
							"Asteroidengrtel - G",
							"Asteroidengrtel - M",
							"Asteroidengrtel - K",
							"Nebel - G",
							"Nebel - M",
							"Nebel - K" };

/*
	Legend:
			Starting Positions			Ending Positions			Planets			
			Start1 --> G				End1 --> g					Earth --> 1
			Start2 --> H				End2 --> h					Ringed Earth --> 2
			Start3 --> I				End3 --> i					Mars --> 3
			Start4 --> J				End4 --> j					Jupiter --> 4
			Start5 --> K				End5 --> k					Organia --> 5
			Start6 --> L				End6 --> l					Saturn --> 6
			Start7 --> M				End7 --> m					Night City --> 7
			Start8 --> N				End8 --> n					Fire --> 8
			Start9 --> O				End9 --> o					Ice --> 9
			Start10 --> P				End10 --> p					Gas --> 0
			Start11 --> Q				End11 --> q					
			Start12 --> R				End12 --> r					
			Start13 --> S				End13 --> s					
			Start14 --> T				End14 --> t					
			Start15 --> U				End15 --> u					
			Start16 --> V				End16 --> v					
			Start17 --> W				End17 --> w					
			Start18 --> X				End18 --> x					
			Start19 --> Y				End19 --> y					
			Start20 --> Z				End20 --> z					

			Celestial Bodies			Asteroids					Black Holes
			Moon --> @					SmallAsteroid --> [			SmallBlackHole --> :
			Sun1 --> (					LargeAsteroid --> ]			LargeBlackHole --> ,
			Sun2 --> )					IceAsteroid --> *			
			Sun3 --> !
			
			Nebulas						Dust Clouds					Other
			Nebula1 --> &				LightDustCloud --> <		WormHole --> %
			Nebula2 --> ?				HeavyDustCloud --> >		Fissure --> $
																	Corpse --> #
																	Organian --> ^
*/
const char* MapLines[]=
{
   "Mul_9TeamAssault",
   "// Offener Weltraum - G",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Offener Weltraum - M",
   " +____1____2____3____4____5+",
   " |S........................|",
   " |.s.......................|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J...m......|",
   "4|...................M.....|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.......................o.|",
   "5|........................O|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Offener Weltraum - K",
   " +____1____2____3+",
   " |O..............|",
   " |.o.............|",
   " |.......M.......|",
   " |.......m.......|",
   "1|...............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |...............|",
   " |............Zs.|",
   "3|..............S|",
   " +---------------+",
   "      1    2    3",
   "// Planeten - G",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |...............................6........|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|.......7................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Planeten - M",
   " +____1____2____3____4____5+",
   " |........................S|",
   " |.8.....................s.|",
   " |.........................|",
   " |................wW.......|",
   "1|................xX..Z....|",
   " |................yY.......|",
   " |.........................|",
   " |...................TUV...|",
   " |...................tuv...|",
   "2|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.......2.................|",
   "4|.........................|",
   " |...ghi...................|",
   " |...jkl...................|",
   " |...GHI....m..............|",
   " |...JKL....M............o.|",
   "5|........................O|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Planeten - S",
   " +____1____2____3+",
   " |..............S|",
   " |....X..Z..Y..s.|",
   " |....xV.z.Wy....|",
   " |.....vT.Uw.....|",
   "1|......t.u......|",
   " |...............|",
   " |...............|",
   " |.......1.......|",
   " |...............|",
   "2|...............|",
   " |....ghi.jkl....|",
   " |....GHI.JKL....|",
   " |.......m.......|",
   " |.......M.....o.|",
   "3|..............O|",
   " +---------------+",
   "      1    2    3",
   "// Asteroidenfeld - G",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |..........[.................[...........|",
   "1|...................Z....................|",
   " |......[............z....................|",
   " |..............................[.........|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|...[............v.T.U.w..............[..|",
   " |..................t.u...................|",
   " |........................................|",
   " |.............................[..........|",
   " |......[.................................|",
   "3|........................................|",
   " |............................[...........|",
   " |...................[....................|",
   " |.[......................................|",
   " |..........[.............................|",
   "4|..........................[.............|",
   " |...[..............[..............[......|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................[...............|",
   " |..........[.....................[.......|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |...[..........K.........L...............|",
   " |........................................|",
   " |...................m..............[.....|",
   "7|...................M....................|",
   " |...........[............................|",
   " |........................................|",
   " |.....[.....................[............|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Asteroidenfeld - M",
   " +____1____2____3____4____5+",
   " |.........S...............|",
   " |...X.....s...........U...|",
   " |..V.x.......Z.......u.W..|",
   " |.T.v........z........w.Y.|",
   "1|..t...................y..|",
   " |...........A...a.........|",
   " |........A................|",
   " |.a...A............A......|",
   " |.........a...............|",
   "2|................A........|",
   " |A....*.............a..A..|",
   " |.........................|",
   " |....a...........*........|",
   " |....................a....|",
   "3|.................A.......|",
   " |....A...a................|",
   " |..............a........A.|",
   " |..a........A....a........|",
   " |......A..................|",
   "4|..............a...A......|",
   " |..g.......a...........l..|",
   " |.G.i.................j.L.|",
   " |..I.k.......m.......h.J..|",
   " |...K........M...o....H...|",
   "5|................O........|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroidenfeld - S",
   " +____1____2____3+",
   " |..........O....|",
   " |.A........o..A.|",
   " |.......M.......|",
   " |.GHI...m...JKL.|",
   "1|.ghi.......jkl.|",
   " |.a..a...a....a.|",
   " |...............|",
   " |.......tuvwxy..|",
   " |.......TUVWXY..|",
   "2|.a...........a.|",
   " |....a.....a....|",
   " |.......A...z...|",
   " |Ss.......a..Z..|",
   " |...A..a........|",
   "3|..........A....|",
   " +---------------+",
   "      1    2    3",
   "// Asteroidengrtel - G",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |]]]]]]]]]]]]]]]]]]]]]]].................|",
   " |.......................]]]]]]]]]]]]]]]]]|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |]]]]]]]]]]]]]]]]]]]]]]].................|",
   " |.......................]]]]]]]]]]]]]]]]]|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Asteroidengrtel - M",
   " +____1____2____3____4____5+",
   " |..A.....S......a.........|",
   " |....a...s...G........M...|",
   " |............g...A...m....|",
   " |.Tt....a.......A......hH.|",
   "1|....A..............A.....|",
   " |................a........|",
   " |.......A...........a.....|",
   " |.Uu..a..........*.....iI.|",
   " |.......*..........A......|",
   "2|....A................A...|",
   " |........a.......A........|",
   " |.Vv..A...........a..a.jJ.|",
   " |.........................|",
   " |.........a.......*....A..|",
   "3|......a............A.....|",
   " |.Ww..A...A............kK.|",
   " |..................A..a...|",
   " |.......a..*..............|",
   " |..................a......|",
   "4|.Xx....A..a..........AlL.|",
   " |......A..........*.......|",
   " |...z...a...........a.....|",
   " |..Z.......A....y......a..|",
   " |.......A...a..oY....A....|",
   "5|.........a....O.....a..A.|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroidengrtel - K",
   " +____1____2____3+",
   " |.......S....a..|",
   " |.Z.....s...Y...|",
   " |..z......X..ya.|",
   " |.......W..x.A..|",
   "1|.....V..w.a....|",
   " |...U..v..A...a.|",
   " |.T..u..a..a....|",
   " |..t..a......l..|",
   " |...a....a.i..L.|",
   "2|...A.a....Ik...|",
   " |.a......h...K..|",
   " |...a..a..Hj....|",
   " |..A.....g..Jm..|",
   " |.a...a.oG....M.|",
   "3|.......O.......|",
   " +---------------+",
   "      1    2    3",
   "// Nebel - G",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |...C....................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Nebel - M",
   " +____1____2____3____4____5+",
   " |........................S|",
   " |.C.....................s.|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J...m......|",
   "4|...................M.....|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.o.......................|",
   "5|O........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Nebel - K",
   " +____1____2____3+",
   " |..............O|",
   " |.C...........o.|",
   " |.......M.......|",
   " |.......m.......|",
   "1|...............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |.......Z.......|",
   " |.............s.|",
   "3|..............S|",
   " +---------------+",
   "      1    2    3",
   "///",
   0
};

#else

/*
	G-L are fixed starting positions for up to 6 teams on the attacking side.
	T-Y are fixed starting positions for up to 6 teams on the defending side.
	Z is the fixed position for the defending team starbase.
*/
int32 TotalMaps = 15;
const char* MapTypes[] = {	"Open Space - L",
							"Open Space - M",
							"Open Space - S",
							"Planets - L",
							"Planets - M",
							"Planets - S",
							"Asteroid Field - L",
							"Asteroid Field - M",
							"Asteroid Field - S",
							"Asteroid Belt - L",
							"Asteroid Belt - M",
							"Asteroid Belt - S",
							"Nebula - L",
							"Nebula - M",
							"Nebula - S" };

/*
	Legend:
			Starting Positions			Ending Positions			Planets			
			Start1 --> G				End1 --> g					Earth --> 1
			Start2 --> H				End2 --> h					Ringed Earth --> 2
			Start3 --> I				End3 --> i					Mars --> 3
			Start4 --> J				End4 --> j					Jupiter --> 4
			Start5 --> K				End5 --> k					Organia --> 5
			Start6 --> L				End6 --> l					Saturn --> 6
			Start7 --> M				End7 --> m					Night City --> 7
			Start8 --> N				End8 --> n					Fire --> 8
			Start9 --> O				End9 --> o					Ice --> 9
			Start10 --> P				End10 --> p					Gas --> 0
			Start11 --> Q				End11 --> q					
			Start12 --> R				End12 --> r					
			Start13 --> S				End13 --> s					
			Start14 --> T				End14 --> t					
			Start15 --> U				End15 --> u					
			Start16 --> V				End16 --> v					
			Start17 --> W				End17 --> w					
			Start18 --> X				End18 --> x					
			Start19 --> Y				End19 --> y					
			Start20 --> Z				End20 --> z					

			Celestial Bodies			Asteroids					Black Holes
			Moon --> @					SmallAsteroid --> [			SmallBlackHole --> :
			Sun1 --> (					LargeAsteroid --> ]			LargeBlackHole --> ,
			Sun2 --> )					IceAsteroid --> *			
			Sun3 --> !
			
			Nebulas						Dust Clouds					Other
			Nebula1 --> &				LightDustCloud --> <		WormHole --> %
			Nebula2 --> ?				HeavyDustCloud --> >		Fissure --> $
																	Corpse --> #
																	Organian --> ^
*/
const char* MapLines[]=
{
   "Mul_9TeamAssault",
   "// Open Space - L",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Open Space - M",
   " +____1____2____3____4____5+",
   " |S........................|",
   " |.s.......................|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J...m......|",
   "4|...................M.....|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.......................o.|",
   "5|........................O|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Open Space - S",
   " +____1____2____3+",
   " |O..............|",
   " |.o.............|",
   " |.......M.......|",
   " |.......m.......|",
   "1|...............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |...............|",
   " |............Zs.|",
   "3|..............S|",
   " +---------------+",
   "      1    2    3",
   "// Planets - L",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |...............................6........|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|.......7................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Planets - M",
   " +____1____2____3____4____5+",
   " |........................S|",
   " |.8.....................s.|",
   " |.........................|",
   " |................wW.......|",
   "1|................xX..Z....|",
   " |................yY.......|",
   " |.........................|",
   " |...................TUV...|",
   " |...................tuv...|",
   "2|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.......2.................|",
   "4|.........................|",
   " |...ghi...................|",
   " |...jkl...................|",
   " |...GHI....m..............|",
   " |...JKL....M............o.|",
   "5|........................O|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Planets - S",
   " +____1____2____3+",
   " |..............S|",
   " |....X..Z..Y..s.|",
   " |....xV.z.Wy....|",
   " |.....vT.Uw.....|",
   "1|......t.u......|",
   " |...............|",
   " |...............|",
   " |.......1.......|",
   " |...............|",
   "2|...............|",
   " |....ghi.jkl....|",
   " |....GHI.JKL....|",
   " |.......m.......|",
   " |.......M.....o.|",
   "3|..............O|",
   " +---------------+",
   "      1    2    3",
   "// Asteroid Field - L",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |..........[.................[...........|",
   "1|...................Z....................|",
   " |......[............z....................|",
   " |..............................[.........|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|...[............v.T.U.w..............[..|",
   " |..................t.u...................|",
   " |........................................|",
   " |.............................[..........|",
   " |......[.................................|",
   "3|........................................|",
   " |............................[...........|",
   " |...................[....................|",
   " |.[......................................|",
   " |..........[.............................|",
   "4|..........................[.............|",
   " |...[..............[..............[......|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................[...............|",
   " |..........[.....................[.......|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |...[..........K.........L...............|",
   " |........................................|",
   " |...................m..............[.....|",
   "7|...................M....................|",
   " |...........[............................|",
   " |........................................|",
   " |.....[.....................[............|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Asteroid Field - M",
   " +____1____2____3____4____5+",
   " |.........S...............|",
   " |...X.....s...........U...|",
   " |..V.x.......Z.......u.W..|",
   " |.T.v........z........w.Y.|",
   "1|..t...................y..|",
   " |...........A...a.........|",
   " |........A................|",
   " |.a...A............A......|",
   " |.........a...............|",
   "2|................A........|",
   " |A....*.............a..A..|",
   " |.........................|",
   " |....a...........*........|",
   " |....................a....|",
   "3|.................A.......|",
   " |....A...a................|",
   " |..............a........A.|",
   " |..a........A....a........|",
   " |......A..................|",
   "4|..............a...A......|",
   " |..g.......a...........l..|",
   " |.G.i.................j.L.|",
   " |..I.k.......m.......h.J..|",
   " |...K........M...o....H...|",
   "5|................O........|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroid Field - S",
   " +____1____2____3+",
   " |..........O....|",
   " |.A........o..A.|",
   " |.......M.......|",
   " |.GHI...m...JKL.|",
   "1|.ghi.......jkl.|",
   " |.a..a...a....a.|",
   " |...............|",
   " |.......tuvwxy..|",
   " |.......TUVWXY..|",
   "2|.a...........a.|",
   " |....a.....a....|",
   " |.......A...z...|",
   " |Ss.......a..Z..|",
   " |...A..a........|",
   "3|..........A....|",
   " +---------------+",
   "      1    2    3",
   "// Asteroid Belt - L",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |........................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |]]]]]]]]]]]]]]]]]]]]]]].................|",
   " |.......................]]]]]]]]]]]]]]]]]|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |]]]]]]]]]]]]]]]]]]]]]]].................|",
   " |.......................]]]]]]]]]]]]]]]]]|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Asteroid Belt - M",
   " +____1____2____3____4____5+",
   " |..A.....S......a.........|",
   " |....a...s...G........M...|",
   " |............g...A...m....|",
   " |.Tt....a.......A......hH.|",
   "1|....A..............A.....|",
   " |................a........|",
   " |.......A...........a.....|",
   " |.Uu..a..........*.....iI.|",
   " |.......*..........A......|",
   "2|....A................A...|",
   " |........a.......A........|",
   " |.Vv..A...........a..a.jJ.|",
   " |.........................|",
   " |.........a.......*....A..|",
   "3|......a............A.....|",
   " |.Ww..A...A............kK.|",
   " |..................A..a...|",
   " |.......a..*..............|",
   " |..................a......|",
   "4|.Xx....A..a..........AlL.|",
   " |......A..........*.......|",
   " |...z...a...........a.....|",
   " |..Z.......A....y......a..|",
   " |.......A...a..oY....A....|",
   "5|.........a....O.....a..A.|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroid Belt - S",
   " +____1____2____3+",
   " |.......S....a..|",
   " |.Z.....s...Y...|",
   " |..z......X..ya.|",
   " |.......W..x.A..|",
   "1|.....V..w.a....|",
   " |...U..v..A...a.|",
   " |.T..u..a..a....|",
   " |..t..a......l..|",
   " |...a....a.i..L.|",
   "2|...A.a....Ik...|",
   " |.a......h...K..|",
   " |...a..a..Hj....|",
   " |..A.....g..Jm..|",
   " |.a...a.oG....M.|",
   "3|.......O.......|",
   " +---------------+",
   "      1    2    3",
   "// Nebula - L",
   " +____1____2____3____4____5____6____7____8+",
   " |S.......................................|",
   " |.s......................................|",
   " |........................................|",
   " |...C....................................|",
   "1|...................Z....................|",
   " |...................z....................|",
   " |........................................|",
   " |..............X.........Y...............|",
   " |..............x.V.....W.y...............|",
   "2|................v.T.U.w.................|",
   " |..................t.u...................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "3|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "4|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   "5|........................................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |..................g.h...................|",
   "6|................i.G.H.j.................|",
   " |..............k.I.....J.l...............|",
   " |..............K.........L...............|",
   " |........................................|",
   " |...................m....................|",
   "7|...................M....................|",
   " |........................................|",
   " |........................................|",
   " |........................................|",
   " |......................................o.|",
   "8|.......................................O|",
   " +----------------------------------------+",
   "      1    2    3    4    5    6    7    8",
   "// Nebula - M",
   " +____1____2____3____4____5+",
   " |........................S|",
   " |.C.....................s.|",
   " |.........X...............|",
   " |..........x..............|",
   "1|....Z....................|",
   " |.....z...V...............|",
   " |..........v..............|",
   " |.........................|",
   " |.........T...............|",
   "2|..Y..W..U.t..............|",
   " |...y..w..u...............|",
   " |.........................|",
   " |.........................|",
   " |..............g..i..k....|",
   "3|.............h.G..I..K...|",
   " |..............H..........|",
   " |.........................|",
   " |.............j...........|",
   " |..............J...m......|",
   "4|...................M.....|",
   " |.............l...........|",
   " |..............L..........|",
   " |.........................|",
   " |.o.......................|",
   "5|O........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Nebula - S",
   " +____1____2____3+",
   " |..............O|",
   " |.C...........o.|",
   " |.......M.......|",
   " |.......m.......|",
   "1|...............|",
   " |....GHI.JKL....|",
   " |....ghi.jkl....|",
   " |...............|",
   " |....tuv.wxy....|",
   "2|....TUV.WXY....|",
   " |...............|",
   " |.......z.......|",
   " |.......Z.......|",
   " |.............s.|",
   "3|..............S|",
   " +---------------+",
   "      1    2    3",
   "///",
   0
};

#endif

