/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "Mul_9TeamAssault.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions():	fTotalAttackTeams ( 0 ), fTotalDefendTeams ( 0 ),
										fAttackStarbaseDestroyed( false ), fDefendStarbaseDestroyed( false )
{}

/*

	Purpose:	The condition for the game ending will occur at the following times:

				1. The attacking starbase is destroyed.
				1. The defending starbase is destroyed.
				
				The game will not be checked if all human players are on one team in the beginning
				because if this is the case, then the AI will take over controlling the opposing side.
				In addition, the game end will not be checked if all of the ships on one team are
				destroyed because the starbase should still be in play.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	//
	//	Let's check if the defending starbase is destroyed.  If so, then all of the defending teams
	//	get a losing condition and all of the attacking teams get a winning condition set.
	//
	if ( mDefendStarbaseDestroyed() )
	{
		for ( int32 i = kPlayerTeam1; i <= kAttackTeam; i++ )
		{
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( team->mGetGameStatus() == kGameInProgress )
				{
					if ( team->mGetTag() == kAttackTag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerWins );
					}
					else if ( team->mGetTag() == kDefendTag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerLoses );
					}
				}
			}
		}

		//
		//	No need to go on.
		//
		return;
	}

	//
	//	Let's check if the attacking starbase is destroyed.  If so, then all of the attacking teams
	//	get a losing condition and all of the defending teams get a winning condition set.
	//
	if ( mAttackStarbaseDestroyed() )
	{
		for ( int32 i = kPlayerTeam1; i <= kAttackTeam; i++ )
		{
			tTeamInfo* team = gScriptInterface.mMission()->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( team->mGetGameStatus() == kGameInProgress )
				{
					if ( team->mGetTag() == kDefendTag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerWins );
					}
					else if ( team->mGetTag() == kAttackTag )
					{
						gScriptInterface.mSetGameStatus( team->mGetTeam(), kPlayerLoses );
					}
				}
			}
		}

		//
		//	No need to go on.
		//
		return;
	}

	//
	//	If we got this far, then the game is not considered to be completed yet.
	//
}

void tWorldConditions::mIncrementTotalAttackTeams()
{
	fTotalAttackTeams++;
}
void tWorldConditions::mIncrementTotalDefendTeams()
{
	fTotalDefendTeams++;
}
void tWorldConditions::mSetAttackStarbaseDestroyed( bool x )
{
	fAttackStarbaseDestroyed = x;
}
void tWorldConditions::mSetDefendStarbaseDestroyed( bool x )
{
	fDefendStarbaseDestroyed = x;
}

int32 tWorldConditions::mTotalAttackTeams()
{
	return fTotalAttackTeams;
}
int32 tWorldConditions::mTotalDefendTeams()
{
	return fTotalDefendTeams;
}	
bool tWorldConditions::mAttackStarbaseDestroyed()
{
	return fAttackStarbaseDestroyed;
}
bool tWorldConditions::mDefendStarbaseDestroyed()
{
	return fDefendStarbaseDestroyed;
}

