/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyPlanet.h"
#include "EnemyStarBase.h"
#include "EnemyShip.h"
#include "EnemyDefPlat.h"
#include "MissionText.h"
#include "ISC_9B3Homeworld.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tEnemyTeam::mCreateShipsForTeam()
{
	
	tISC_9B3Homeworld * script = (tISC_9B3Homeworld * ) gScriptInterface.fScript;
	fRace = (eRaceName) script->fSectorRace;
	int32 i;

	switch(fRace)
	{
	case kFederation:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kEnemyPlanet);

		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh0",
					kStartPosition_Y,
					kEnemyMoon);


		mCreateShip( typeid( tEnemyStarBase ),
					kMaxBPV,
					kClassStarBase,
					kClassStarBase,
					0,
					kStartPosition_X,
					kEnemyStarBase );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_W,	kEnemyShip1 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_T, kEnemyShip2 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_S, kEnemyShip3 );
		break;

	case kRomulan:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kEnemyPlanet);

		mCreateShip( typeid( tEnemyStarBase ),
					kMaxBPV,
					kClassStarBase,
					kClassStarBase,
					0,
					kStartPosition_X,
					kEnemyStarBase );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_W,	kEnemyShip1 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_S, kEnemyShip2 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_T, kEnemyShip3 );
		break;
	
	case kHydran:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh4",
					kStartPosition_Z,
					kEnemyPlanet);

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_X,	kEnemyShip1 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_W, kEnemyShip2 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_S, kEnemyShip3 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_T, kEnemyShip4 );
		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassHeavyBattlecruiser,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_T, kEnemyShip5 );

		for(i = 0; i <= 2; i++)
		{
			mCreateShip( typeid( tEnemyDefPlat ),
				Messages[kDefPlatFed + mGetRace()].c_str(), kStartPosition_P + i, kEnemyDefPlat1 + i );

		}
		break;
	case kLyran:
	case kGorn:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kEnemyPlanet);


		mCreateShip( typeid( tEnemyStarBase ),
						kMaxBPV,
						kClassStarBase,
						kClassStarBase,
						0,
						kStartPosition_X,
						kEnemyStarBase );

		mCreateShip( typeid( tEnemyShip ),
						kMaxBPV,
						kClassDreadnought,
						kClassDreadnought,
						0,
						kStartPosition_W, kEnemyShip1 );

		mCreateShip( typeid( tEnemyShip ),
						kMaxBPV,
						kClassDreadnought,
						kClassDreadnought,
						0,
						kStartPosition_W, kEnemyShip2 );

		for(i = 0; i <= 5; i++)
		{

			mCreateShip( typeid( tEnemyDefPlat ),
				Messages[kDefPlatFed + mGetRace()].c_str(),
				kStartPosition_O + i, kEnemyDefPlat1 + i );

		}
		break;
	case kISC:
		break;
	case kKlingon:
	case kMirak:
		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh1",
					kStartPosition_Z,
					kEnemyPlanet);

		mCreateShip( typeid( tEnemyPlanet ),
					"PlPh0",
					kStartPosition_Y,
					kEnemyMoon);

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassBattleship,
					kClassBattleship,
					0,
					kStartPosition_X,	kEnemyShip1 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_W, kEnemyShip2 );

		mCreateShip( typeid( tEnemyShip ),
					kMaxBPV,
					kClassDreadnought,
					kClassDreadnought,
					0,
					kStartPosition_T, kEnemyShip3 );
		
		for(i = 0; i <= 7; i++)
		{
			mCreateShip( typeid( tEnemyDefPlat ),
				Messages[kDefPlatFed + mGetRace()].c_str(), kStartPosition_P+i, kEnemyDefPlat1 + i );

		}
		break;
	}
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
	fShieldTimer = 0;
	tShipInfo * starbase = NULL;
	tShipIterator ship = mGetFirstShip();
	for(ship; *ship != NULL; ship++)
	{
		if( ship->mGetCustomID() == kEnemyStarBase)
		{
			starbase = *ship;
		}
	}

	if(starbase != NULL)
	{

		ship = mGetFirstShip();
		for(ship; *ship != NULL; ship++)
		{
			if( ship->mGetCustomID() != kEnemyStarBase)
			{
				ship->mDefendEntity(starbase, 10.0, 10.0, kMediumPriority);
			}
		}
	}

}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{}


int32 tEnemyTeam::mShieldTimer(void)
{
	return fShieldTimer;
}
void tEnemyTeam::mIncShieldTimer(void)
{
	fShieldTimer ++;
}
