/********************************************************************************************************\

	Mission:	"Homeworld Bound"
	Script:		"ISC_9B3Homeworld"
	Descript:	Capital Assault with allies, no shield code. To get this mission, 
				must have successfully completed ISC_9B1, but not completed the mission
				to get the prefix code to lower the shield.
	Created by:		Dutch Blomenkamp	6/28/00
	Last Updated:	NDB 07/16/00

\********************************************************************************************************/

#include "PlayerNPC1Team.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerTeam.h"
#include "TransportTeam.h"
#include "ISC_9B3Homeworld.h"
#include "BaseVictory.h"

DEFINE_SCRIPT_CLASS( tISC_9B3Homeworld );

tISC_9B3Homeworld::tISC_9B3Homeworld()
{}

/*

	HELPER NOTE:	You may be able to register states here; however it is suggested that you do not.  Use
					only if you must and know how.

*/
void tISC_9B3Homeworld::mSetupState()
{}

/*

	HELPER NOTE:	You may be able to register comm states here; however it is suggested that you do not.
					Use only if you must and know how.

*/
void tISC_9B3Homeworld::mSetupCommState()
{}

/*

	HELPER NOTE:	This function should return whom you consider to be the primary enemy.  Perhaps useful
					in multiplayer scripts, but single player missions do not seem to actively use it.

*/
int32 tISC_9B3Homeworld::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tISC_9B3Homeworld::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fEnemyRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 6;
	Specs->fMenuStates = kEnableSupplyDock;

	
	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												(eTeamID)kPlayerTeam,
												1.0f,
												kFalse,
												kPlayableTeam, 1.0f,
												tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
												kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ) );
	
	
	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												(eTeamID)kEnemyTeam,
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tEnemyTeam ) );


	const tTeamSpec& team3= mCreateTeamSpec(	"Allied Team",
												(eTeamID)kPlayerNPC1Team,
												1.0f,
												kFalse,
												kNPCTeam, 1.0f,
												tStrength(	0, 0,
																		kClassFrigate,
																		kClassFrigate,
																		0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlayerNPC1Team ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Transport Team",
												(eTeamID)kTransportTeam,
												1.0f,
												kFalse,
												kNPCTeam, 1.0f,
												tStrength(	0, 0,
																		kClassFreighter,
																		kClassFreighter,
																		0, 300 ),
												kSpecAnyRace,
												kSameAs | kPlayerTeam,
												typeid( tTransportTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions. 
	In multiplayer missions, you could examine which map was picked by the player.
	For example: if ( info.fPreferedMap == 3 );	//	the 3rd map was picked

	Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tISC_9B3Homeworld::mInitializeStart( tDynaverseScriptInfo& info )
{
	//******************************************************
	// This seems like the logical place to setup variables
	// Note that most of this will be changed to use 
	// whatever method the campaign uses.
	//******************************************************
	fDiplomatBeamed = false;		// true for diplomatic variant,
								// false for hostile variant.

	// See MissionText.CPP for the strings involved in setting up the
	// INI entries. Reading in from the ini file allows testing of 
	// various races. Once tested, this variable can be set via
	// the info structure passed into this function
	fSectorRace = fMissionInfo->mGetINIFileInt(kIniHeader, kSectorRace);
	fAllianceForged = true;		// Set to true for hostile with allies
								// false for no allies.
	
	fAllianceFed = false;		// These three are mutually exclusive.
	fAllianceHydran = true;		// Whichever one is true determines what
	fAllianceRomulan = false;	// race the ally will be.
	
	fCodeKnown = false;			// True if the player has obtained the code before

	fShieldsLowered = false;	
	info.fEnemyRace = ( eRace ) fSectorRace; 
	return info.fPreferedMap;
	
	
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tISC_9B3Homeworld::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tISC_9B3Homeworld::mInitializeFinished( void )
{
	if(fDiplomatBeamed )
	{
		return kBriefingA_msg;
	}
	else
	{
		if(fAllianceForged)
		{
			if(fSectorRace != kKlingon && fSectorRace != kMirak)
			{
				return kBriefingB_msg;
			}
			else
			{
				return kBriefingC_msg;
			}
		}
		else
		{
			if(fSectorRace != kKlingon && fSectorRace != kMirak)
			{
				return kBriefingD_msg;
			}
			else
			{
				return kBriefingE_msg;
			}
		}

	}
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tISC_9B3Homeworld::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tISC_9B3Homeworld::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tISC_9B3Homeworld::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tISC_9B3Homeworld::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kPlayerNPC1Team,
		kTransportTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	if( fDiplomatBeamed == true)
	{
		std::string relationships[kRelationsCount][kRelationsCount] = 
		{
			//Plyr = kPlayerTeam,
			//PNPC = kPlayerNPCTeam,
			//Nme  = kEnemyTeam,
			//NPC1 = kNPC1Team,

			//	     Plyr  Nme
			/*Plyr*/ "--", "NI", "NI", "NI",
			/*Nme */ "NI", "--", "NI", "NI",
			/*Ally*/ "NI", "NI", "--", "NI",
			/*TPrt*/ "NI", "NI", "NI", "--"
		};
		fMissionInfo->mSetTeamRelations( teamList, (std::string*) relationships, kRelationsCount);


	}
	else
	{
		std::string relationships[kRelationsCount][kRelationsCount] = 
		{
			//Plyr = kPlayerTeam,
			//PNPC = kPlayerNPCTeam,
			//Nme  = kEnemyTeam,
			//NPC1 = kNPC1Team,

			//	     Plyr  Nme
			/*Plyr*/ "--", "WD", "NI", "NI",
			/*Nme */ "WD", "--", "WD", "WD",
			/*Ally*/ "NI", "WD", "--", "NI",
			/*TPrt*/ "NI", "WD", "NI", "--"
		};
		fMissionInfo->mSetTeamRelations( teamList, (std::string*) relationships, kRelationsCount);

	}
	
}

char * tISC_9B3Homeworld::GetDynamicMessage( char * buffer, int32 msg, char * fstr, char * rstr)
{
	// Find the "token", and replace it.
	int p = 0;
	p = Messages[msg].find(fstr, p);
	Messages[msg]=Messages[msg].erase(p, strlen(fstr));
	Messages[msg].insert(p, rstr);
	
	// Now stuff the replaced string into buffer.

	if(strlen(buffer) >= Messages[msg].length())
	{
		sprintf(buffer, "%s", Messages[msg].c_str());
	}

	// Now put the message back the way it was for 
	// future use.

	p = Messages[msg].find(rstr, p);
	Messages[msg]=Messages[msg].erase(p, strlen(rstr));
	Messages[msg].insert(p, fstr);
	return buffer;
	
};
