/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "TransportTeam.h"
#include "MoonTransport.h"
#include "PlanetTransport.h"
#include "MissionText.h"
#include "ISC_9B3Homeworld.h"


#define kTimerConstant 2 / 60

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tPlayerTeamBaseState::mBeginState()
{}
void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	tISC_9B3Homeworld * script = (tISC_9B3Homeworld * ) gScriptInterface.fScript;
	if(script->fDiplomatBeamed)
	{
		fChildActor->mSetAttacker(true);
		tTeamInfo * EnemyTeam = fMissionInfo->mGetTeamHandle((eTeamID)kEnemyTeam);
		Assert(EnemyTeam);
		tShipIterator ship= EnemyTeam->mGetFirstShip();
		for(ship; *ship != NULL; ship++)
		{
			fChildActor->mGotoCommState((*ship)->mGetShipID(), tISC_9B3Homeworld::kCommState2);
		}
		return;
	}
	
}
void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}
void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}
void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	static bool started = false;
	int32 x, y;

	tShipIterator  playership = fChildActor->mGetFirstShip();
	// I'm purposely not doing an assert here, as all I'm checking for is the presence
	// of the player ship, i.e. has it been created yet.

	if(*playership == NULL)
	{
		fChildActor->mSetTimedInterval(	kTimerTurnsSet, timerID, (float) kTimerConstant * 5);
		return;
	}

	// Make sure we get the first valid undestroyed ship.
    // A destroyed ship  will indicate that has left the system.

	for(playership; *playership != NULL; playership++)
	{

		if((*playership)->mInPlay())
			break;
	}

	if(!started)
	{
		tISC_9B3Homeworld * script = (tISC_9B3Homeworld *) gScriptInterface.fScript;

		if(script->fDiplomatBeamed)
		{
			switch( script->fSectorRace)
			{
			case kFederation:
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartFedDiplomacy_msg );
				break;
			case kHydran:
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartHydranDiplomacy_msg );
				break;

			case kRomulan:
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartRomulanDiplomacy_msg );
				break;
			}
		}
		else
		{

			fChildActor->mSetTransportTimer(0);
			if((script->fSectorRace == kKlingon) || (script->fSectorRace == kMirak))
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartHostileKorM_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kStartHostileGeneric_msg );
			}
		}
		started = true;
	}
	Assert(*playership != NULL);

	if ( timerID == tPlayerTeam::kEventTimerID )
	{	
		(*playership)->mGetGridPosition(&x, &y);
		if(AtEdgeOfMap(x, y))
		{
			if( !fChildActor->mAtEdgeOfMap() && (*playership)->mInPlay() )
			{
				fChildActor->mSetAtEdgeOfMap(true);
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, "Science Officer: Leaving the system, [PLAYER_RANK]\n\n");
			}
		}
		else
		{
			if(fChildActor->mAtEdgeOfMap())
			{
				fChildActor->mSetAtEdgeOfMap(false);
			}
		}
		if(fChildActor->mTransportTimer() <= 150 )
		{
			if(fChildActor->mTransportTimer() == 135)
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportWarning_msg);
			}
			if(fChildActor->mTransportTimer()==150)
			{
				fMissionInfo->mDisplayMessage((eTeamID)kPlayerTeam, kTransportsArrive_msg);
		
				tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID)kEnemyTeam);
				Assert(enemy);
				tTeamInfo * transports = fMissionInfo->mGetTeamHandle((eTeamID)kTransportTeam);
				Assert(transports);
				
				if(enemy!=NULL && transports!=NULL)
				{
					switch(enemy->mGetRace())
					{
					case kKlingon:
					case kMirak:
						transports->mCreateShip( typeid( tMoonTransport ),
							kMaxBPV,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_N,
							tTransportTeam::kMoonTransport);
					default:
						transports->mCreateShip( typeid( tPlanetTransport ),
							kMaxBPV,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_N,
							tTransportTeam::kPlanetTransport1);

						transports->mCreateShip( typeid( tPlanetTransport ),
							kMaxBPV,
							kClassFreighter,
							kClassFreighter,
							0,
							kStartPosition_N,
							tTransportTeam::kPlanetTransport2);
						break;
					}
				}
			}
			fChildActor->mIncTransportTimer();
		}
		fChildActor->mSetTimedInterval(	kTimerTurnsSet, timerID, (float) kTimerConstant * 5);
	}
}
void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

bool tPlayerTeamBaseState::AtEdgeOfMap(int32 X, int32 Y)
{
	int32 GridX;
	int32 GridY;
	gScriptInterface.mGetMapDimensions( &GridX, &GridY);

	// kludge to keep a destroyed play ship from indication off the map.
	if( X == 0 && Y == 0)
		return false;

	if( (X < 2 || X > (GridX - 1)) || (Y < 2 || Y > (GridY - 1)))
	{
		return true;
	}
	else
	{
		return false;
	}
}



