/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ISC_9B5Homeworld.h"

#define kTimerConstant 2 / 60

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam( void ): fAITarget( NULL )
{}

/*

	HELPER NOTE:	The victory condition must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
	fAttacker = false;
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	tISC_9B5Homeworld * script = (tISC_9B5Homeworld * ) gScriptInterface.fScript ;
	if(script->fDiplomatBeamed)
	{
		switch ( victoryLevel )
		{
		case kAstoundingVictory:
			switch(script->fSectorRace)
			{
			case  kFederation:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFedAlly_msg);
				break;
			case kHydran:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebHydranAlly_msg);
				break;
			case kRomulan:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebRomulanAlly_msg);
				break;
			}
			break;
		case kDevastatingDefeat:
			switch(script->fSectorRace)
			{
			case  kFederation:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFedAttack_msg);
				break;
			case kHydran:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebHydranAttack_msg);
				break;
			case kRomulan:
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebRomulanAttack_msg);
				break;
			}
			break;
		}
	}
	else
	{
		switch ( victoryLevel )
		{
		case kAstoundingVictory:
			fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebHostileSuccess_msg);
			break;
		case kDevastatingDefeat:
			if(script->fSectorRace = kKlingon || script->fSectorRace==kMirak)
			{
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFailKorM_msg);
			}
			else
			{
				fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebFailGeneric_msg);
			}
			break;
		}
	}

	if ( victoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( mGetTeam(), kDisconnect_msg );
	}
}

/*
                                                                                                              
	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tPlayerTeam::mCreateShipsForTeam()
{}

/*

	HELPER NOTE:	This is where you want to set the player ship as player controlled.  Also, you have
					the ability to setup state callback events here such as:

						for( ; *ship != NULL; ++ship )
						{
							ship->mSetShipCallbackEvents( kNotifyDamage );
						}

*/
void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
	tShipInfo * flagship;

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
			
		}
		flagship = *ship;
			tISC_9B5Homeworld * script = (tISC_9B5Homeworld * ) gScriptInterface.fScript ;
		if(script->fDiplomatBeamed)
		{
			flagship->mAddItemToStores(kTransporterItem,kTransDiplomat,1);
			flagship->mSetShipCallbackEvents( kNotifyTransportedItem );
		}

		flagship->mSetShipThrust(10);
		ship++;
		for(ship; *ship != NULL; ship++)
		{
			ship->mDefendEntity(flagship, 10.0, 4.0, kAbsolutePriority);
			ship->mSetShipThrust(10);

		}
	}
	mSetRandomChat(true);
	mSetTimedInterval( kTimerTurnsSet, kEventTimerID, (float) kTimerConstant * 5);
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
		static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 shipScriptDescription.fMetaDatabaseID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;
}

bool tPlayerTeam::mRandomChat(void)
{
	return fRandomChat;
}
void tPlayerTeam::mSetRandomChat(bool x)
{
	fRandomChat = x;
}

bool tPlayerTeam::mAtEdgeOfMap(void)
{
	return fAtEdgeOfMap;
}

void tPlayerTeam::mSetAtEdgeOfMap(bool x)
{
	fAtEdgeOfMap = x;
}
int32 tPlayerTeam::mTransportTimer()
{
	return fTransportTimer;
}
void tPlayerTeam::mIncTransportTimer()
{
	fTransportTimer++;
}

void tPlayerTeam::mSetTransportTimer(int32 x)
{
	fTransportTimer = x;
}
bool tPlayerTeam::mAttacker()
{
	return fAttacker;
}

void tPlayerTeam::mSetAttacker(bool x)
{
	fAttacker = x;
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	

	switch( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		{
			fMissionScheduler.fNextMissionTitle = "ISC_10Criminals";
			fMissionScheduler.fNextMissionScore = 1000;
		}
		break;

	default:
		{
			fMissionScheduler.fCampaignEvent = tTeamInfo::tMissionScheduler::kLost;
			fMissionScheduler.fNextMissionScore = -1;
			fMissionScheduler.fNextMissionTitle = "ISC_10Criminals";
		}
		break;
	}
}