/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "VictoryCondition.h"
#include "ISC_X3M3BStar.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::CreateAlliedShip()
{
	mCreateShip(typeid(tPlayerShip), Messages[kAlliedShipClass].c_str(), kStartPosition_H, kNoMetaShipID,
				kAlliedShip, 0, 0, -1, -1, Messages[kAlliedShipName].c_str(), kDefaultShipOptions );
}

/*

	HELPER NOTE:	The victory condition must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
		case kAstoundingVictory:
			// This ought *never* happen.
			assert(0);
			fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;

		case kVictory:
		{
			tPlayerTeamBaseState* pState = static_cast<tPlayerTeamBaseState*>(fCurState);
			assert(NULL != pState);
			eVictoryChoice vc = pState->GetVictoryChoice();
			eRaceName playerRace = mGetRace();
			int32 iCustomMsg;

			switch (vc)
			{
				case kChooseYouth:
					iCustomMsg = kVictoryYouth_msg;
					break;
				case kChooseKnowledge:
					if (playerRace == kKlingon)
						iCustomMsg = kVictoryYouthKlingon_msg;
					else
						iCustomMsg = kVictoryKnowledge_msg;
					break;
				case kChoosePrestige:
					iCustomMsg = kVictoryPrestige_msg;
					break;
				case kChooseVictory:
					iCustomMsg = kVictoryVictory_msg;
					break;
			}

			fMissionInfo->mAddToDebriefingList( fTeam, iCustomMsg );			
			fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );
			break;
		}

		case kDraw:
			fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
		case kDefeat:
			fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
		case kDevastatingDefeat:
			fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
		case kUnknownVictoryLevel:
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );			break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
	fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), kStart_msg, kNonCommScriptMessage);
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	static int32 shipNumber = kPlayerShip1;
	static iTruth fCreatedAlly;

	//	Create the ship according to the description given by the engine.

	mCreateShip( typeid(tPlayerShip), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_H,
				 shipScriptDescription.fMetaDatabaseID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	//	Increment the ship number for each ship that comes through here on the player team

	shipNumber++;

	if (!fCreatedAlly)
	{
		CreateAlliedShip();
		fCreatedAlly = kTrue;
	}
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 1000;
	fMissionScheduler.fNextMissionTitle = "ISC_7BattleGrave.scr";
}