/********************************************************************************************************\

	File Name:	CheckPointTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BeamGenerator.h"
#include "CommonDefs.h"
#include "CheckPoint.h"
#include "CheckPointTeam.h"
#include "CheckPointTeamBaseState.h"
#include "Draw.h"
#include "MissionText.h"
#include "WorldConditions.h"
#include <sstream>


tClassAllocator<tCheckPointTeam> gCheckPointTeamAllocator;

tCheckPointTeam::tCheckPointTeam(): fStartPos( kStartPosition_K )
{}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tCheckPointTeam::mSetupState()
{
	mRegisterState( new tCheckPointTeamBaseState() );
	mGotoState( typeid( tCheckPointTeamBaseState ) );
}

/*

	HELPER NOTE:	Comm states for ships that you have created should not be set up in here.

*/
void tCheckPointTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tCheckPointTeam::mCreateShipsForTeam()
{
	//
	//	Create each of the checkpoints.
	//
	int32 numCheckPoints = 0;
	int32 x = 0, y = 0;
	gWC.mInitializeCheckPoints();
	for ( char j = kStartLetterForCheckPoints; j <= kStartLetterForCheckPoints + kMaxNumCheckPoints; j++ )
	{
		fMissionInfo->mGetGridPosition( j, &x, &y );
		if ( x >= 0 && y >= 0 )
			numCheckPoints++;
		else
			break;
	}
	gWC.mSetTotalCheckPoints( numCheckPoints );
	
	for ( int32 i = 0; i < gWC.mTotalCheckPoints(); i++ )
	{
		char tempClassName[64];
		char tempCustomName[64];
		sprintf( tempClassName, Messages[kCheckPointClass_nam].c_str() );
		sprintf( tempCustomName, Messages[kCheckPointCustom_nam].c_str(), i + 1 );
		mCreateShip(	typeid( tCheckPoint ), tempClassName, fStartPos, kNoMetaShipID,
						-1, 0, 0, -1, -1, tempCustomName, kDefaultShipOptions );

		fStartPos++;
	}

	//
	//	Create the border objects and display the track borders if any.
	//
	int32 numBorderObjects = 0;
	int32 mapPicked = gWC.fMapPicked;
	switch ( mapPicked )
	{
	case 1:
		{
			numBorderObjects = kMaxVertices_BorderTrack;
			for ( i = 0; i < numBorderObjects; i++ )
				gVertices[i] = gVertices_BorderTrack[i];
			break;
		}
	case 2:
		{
			numBorderObjects = kMaxVertices_SpiderTrack;
			for ( i = 0; i < numBorderObjects; i++ )
				gVertices[i] = gVertices_SpiderTrack[i];
			break;
		}
	case 3:
		{
			numBorderObjects = kMaxVertices_8Track;
			for ( i = 0; i < numBorderObjects; i++ )
				gVertices[i] = gVertices_8Track[i];
			break;
		}
	case 4:
		{
			numBorderObjects = kMaxVertices_PlanetTrack;
			for ( i = 0; i < numBorderObjects; i++ )
				gVertices[i] = gVertices_PlanetTrack[i];
			break;
		}
	case 5:
		{
			numBorderObjects = kMaxVertices_RelayTrack;
			break;
		}
	default: numBorderObjects = 0;	break;
	}
	for ( i = 0; i < numBorderObjects; i++ )
	{
		mCreateShip(	typeid( tBeamGenerator ), Messages[kBoundaryObject_nam].c_str(), kStartPosition_Specified,
						kNoMetaShipID, -1, gVertices[i].x, gVertices[i].y, -1, -1, NULL, kDefaultShipOptions );
	}

	for ( i = 0; i < tDraw::kMaxDrawObjects; i++ )
	{
		gDraw.mDisplayDrawObject( i );
	}
}

void tCheckPointTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

/*

	HELPER NOTE:	This function must be overridden with nothing exactly how it is for the single
					player missions.

*/
void tCheckPointTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{}

