/********************************************************************************************************\

	File Name:	CommonShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "CommonShipBaseState.h"
#include "CommonTeam.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

#include <sstream>


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCommonShipBaseState::mBeginState()
{}
void tCommonShipBaseState::mEndState()
{}

void tCommonShipBaseState::mOnDeath( tShipInfo* ship )
{
	if ( fChildActor->mGetTeam() == ship->mGetTeam() )
	{
		tCommonTeam* team = dynamic_cast< tCommonTeam* >( fMissionInfo->mGetTeamHandle( ship->mGetTeam() ) );
		Assert( team );
		if ( !team )
			return;

		//
		//	Figure out the checkpoint position to start the ship off at again.  We know that the
		//	first checkpoint was at kStartPosition_H and could possibly go up to kStartPosition_Z.
		//
		iMapPosition spawnPosition = kStartPosition_K + ( team->mNumCheckPointsPassed() - 1/*offset*/ );
		if ( !team->mNumCheckPointsPassed() )
			spawnPosition = ship->mGetStartPositionIndex();

		team->mCreateShip(	typeid( tCommonShip ),
							ship->mGetClassName(),
							spawnPosition,
							tTeamInfo::kNoMetaShipID,
							kPlayerShip,
							0, 0, -1, -1, NULL,
							kDefaultShipOptions );
	}
}
void tCommonShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tCommonShipBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	if ( fChildActor->mGetTeam() == ship->mGetTeam() )
	{
		ship->mDestroyShipSilently();
	}
}
void tCommonShipBaseState::mAtDistance( tShipInfo* ship )
{
	if ( fChildActor->mGetTeam() == ship->mGetTeam() )
	{
		//
		//	The ship has reached a checkpoint, so increment the current checkpoint and track the
		//	distance to the next one in line.  We also want to check if this was the last checkpoint.
		//	If so, then the losers should blow up and the game should end.
		//
		tCommonTeam* team = dynamic_cast< tCommonTeam* >( fMissionInfo->mGetTeamHandle( ship->mGetTeam() ) );
		Assert( team );
		if ( !team )
			return;

		team->mIncrementNumCheckPointsPassed();

		//
		//	Random message display on the fly.  Add more if wanted.
		//
		char tempString[128];
		if ( team->mNumCheckPointsPassed() < gWC.mTotalCheckPoints() )
		{
			sprintf( tempString, Messages[kCheckPointPassed_msg].c_str() );
		}
		else
		{
			for ( int32 i = kPlayerTeam1; i <= kPlayerTeam4; i++ )
			{
				if ( team->mGetTeam() == static_cast< eTeamID >( i ) )
				{
					sprintf( tempString, Messages[kYouWon_msg].c_str() );
				}
				else
				{
					sprintf( tempString, Messages[kYouLost_msg].c_str() );
				}
			}
		}
		fMissionInfo->mDisplayMessage( ship->mGetTeam(), tempString );
			

		//
		//	Set the goal indicator to the next checkpoint and start tracking its distance.
		//
		if ( team->mNumCheckPointsPassed() < gWC.mTotalCheckPoints() )
		{
			fMissionInfo->mSetGoalIndicator(	ship, kTrue,
												gCheckPointPositions[team->mNumCheckPointsPassed()].x,
												gCheckPointPositions[team->mNumCheckPointsPassed()].y );
			fChildActor->mTrackTacticalDistance(	static_cast< float >( kValidDistanceFromCheckPoint ),
													gCheckPointPositions[team->mNumCheckPointsPassed()].x,
													gCheckPointPositions[team->mNumCheckPointsPassed()].y,
													kTrue );
		}
		
		gWC.mCheckForGameEnd();
	}
}
void tCommonShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tCommonShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonShipBaseState::mOnFlashingButtonPressed()
{}
void tCommonShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tCommonShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonShipBaseState::mOnHostChanged()
{}
void tCommonShipBaseState::mOnTimedInterval( int32 timerID )
{}
void tCommonShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

