/********************************************************************************************************\

	File Name:	CommonTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonDefs.h"
#include "CommonShip.h"
#include "CommonTeam.h"
#include "CommonTeamBaseState.h"
#include "CommonTeamBaseVictory.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


tClassAllocator<tCommonTeam> gCommonTeamAllocator;

tCommonTeam::tCommonTeam(): fNumCheckPointsPassed( 0 )
{
	gWC.mIncrementNumPlayers();
}

/*

	HELPER NOTE:	The victory condition for this team is specific to this team and only to this team.

*/
void tCommonTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tCommonTeamBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;

	case kDraw:
	default:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tCommonTeam::mSetupState()
{
	mRegisterState( new tCommonTeamBaseState() );
	mGotoState( typeid( tCommonTeamBaseState ) );
}

/*

	HELPER NOTE:	The comm states should include the team they apply for when registered.

*/
void tCommonTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the team's BPV.
	//
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_G,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tCommonTeam::mCreateShipsForTeam()
{}

/*

	HELPER NOTE:	This is where the event notification messages can be set for the ships that have
					been supplied by the multiplayer engine.  Also, if the team is controlled by a
					player, then one of that team's ships should be set to player control.  Also, the
					parent function must ALWAYS be called at the start of this function as you see
					below.  For example:

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	for( ;
		 *ship != NULL;
		 ++ship )
	{
		ship->mSetShipCallbackEvents( kNotifyDamage );
	}

*/
void tCommonTeam::mTeamSetupComplete()
{
	gWC.mInitializeCheckPoints();

	tTeamInfo::mTeamSetupComplete();

	//
	//	If there are no checkpoints spawned, then end the game since the objective is not available.
	//
	if ( gWC.mTotalCheckPoints() <= 0 )
		gWC.mCheckForGameEnd();
}

/*

	HELPER NOTE:	If the ships that the player chooses for the team from the multiplayer engine are
					not the ships that that team should start with, then leave the function overidden
					and create the desired ships in tTeamInfo::mCreateShipsForTeam().  Otherwise, delete
					this function and the respective prototype in the class header or simply undefine
					the constant.

*/
void tCommonTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	//
	//	This team better be human controlled or something is wrong.
	//
	if ( mIsTeamHumanControlled() )
	{
		//
		//	Determine where the player's start.
		//
		eShipOptions options = kNoShipOptions;
		switch ( shipScriptDescription.fTeamID )
		{
		case kPlayerTeam1:
			options = static_cast< eShipOptions >( kStartPositionCanBeOffset | kStartPositionOffsetN ); break;
		case kPlayerTeam2:
			options = static_cast< eShipOptions >( kStartPositionCanBeOffset | kStartPositionOffsetE ); break;
		case kPlayerTeam3:
			options = static_cast< eShipOptions >( kStartPositionCanBeOffset | kStartPositionOffsetS ); break;
		case kPlayerTeam4:
			options = static_cast< eShipOptions >( kStartPositionCanBeOffset | kStartPositionOffsetW ); break;
		}

		mCreateShip(	typeid( tCommonShip ), 
						shipScriptDescription.fClassName.c_str(), 
						kStartPosition_G,
						kNoMetaShipID,
						kPlayerShip, 0, 0, -1, -1,
						shipScriptDescription.fCustomName.c_str(),
						options );
	}
}


void tCommonTeam::mIncrementNumCheckPointsPassed()
{
	fNumCheckPointsPassed++;
}
int32 tCommonTeam::mNumCheckPointsPassed()
{
	return fNumCheckPointsPassed;
}