/********************************************************************************************************\

	File Name:	CommonTeamBaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonTeam.h"
#include "CommonTeamBaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipScriptDescription.h"
#include "WorldConditions.h"


//
//	Serialization events
//
tCommonTeamBaseVictoryState::tCommonTeamBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tCommonTeamBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tCommonTeamBaseVictoryState::mBeginState()
{}
void tCommonTeamBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the team did once you end the mission with
					a call to mSetGameStatus() for this specific team.  It is not necessary to have every
					victory condition handled here (e.g. kAstoundingVictory...kDevastatingDefeat),
					especially if the victory condition is simply a boolean value (e.g. killed or
					has killed).  Also notice that the example is using the world condition for analysis.
					An example follows.

	eVictoryLevels
	tCommonTeamBaseVictoryState::mCalcVictoryStatus()
	{
		if ( gWC.mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tCommonTeamBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tCommonTeam* team = dynamic_cast< tCommonTeam* >( fMissionInfo->mGetTeamHandle( ID ) );
	Assert( team );
	if ( team )
	{
		if ( team->mGetGameStatus() == kPlayerWins )
		{
			return kVictory;
		}
		else
		{
			return kDefeat;
		}
	}
	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!

	void
	tCommonTeamBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tCommonTeamBaseVictoryState::mOnDeath( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnCapture( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonTeamBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonTeamBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonTeamBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonTeamBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonTeamBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonTeamBaseVictoryState::mOnFlashingButtonPressed()
{}
void tCommonTeamBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonTeamBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonTeamBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonTeamBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonTeamBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonTeamBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//

/*

	HELPER NOTE:	When this event is called, you need to pretend that the team associated with the
					playerNumber has been defeated.  So call what you need to, so that you may determine
					if the game has ended.

*/
void tCommonTeamBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonTeamBaseVictoryState::mOnHostChanged()
{}
void tCommonTeamBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tCommonTeamBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonTeamBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

