/********************************************************************************************************\

	File Name:	Draw.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Draw.h"

class tDraw gDraw;

/*

	HELPER NOTE:	This is where you will define all of your drawing objects for this script.
					Use tDrawCanvas::mCreateDrawObject() for each drawing object.  Before you
					do this though, you must define a set of "tScript3DPoint"s as static memory.
					If static memory is not used, you WILL crash!  The following is an example:

					static	tScript3DPoint	localPoints[3];
							tMapPoint		localAnchor;

					localAnchor.x = 2;
					localAnchor.y = 2;	
					int32 localVertices = mPredefinedTriangle( localPoints );
					eScriptPrimitiveFlags flags = static_cast< eScriptPrimitiveFlags >( kPolygon | kCW | kSolid | kDoubleSide | kBlend );

					mCreateDrawObject( 0, fPrimitives, localPoints, localVertices, localAnchor, flags );

*/

/*

	PURPOSE:	Goals.

*/
int32 tDraw::mGoalTop(	tScript3DPoint	Save[],
						float			R,
						float			G,
						float			B )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		0.4f,	0.4f,	0.05f,	R,	G,	B	),
		tScript3DPoint(		0.4f,	-0.4f,	0.05f,	R,	G,	B	),
		tScript3DPoint(		-0.4f,	-0.4f,	0.05f,	R,	G,	B	),
		tScript3DPoint(		-0.4f,	0.4f,	0.05f,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
		Save[i] = localPoints[i];

	return kNumPoints;
}

int32 tDraw::mGoalMiddle(	tScript3DPoint	Save[],
							float			R,
							float			G,
							float			B )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		0.4f,	0.4f,	0.0f,	R,	G,	B	),
		tScript3DPoint(		0.4f,	-0.4f,	0.0f,	R,	G,	B	),
		tScript3DPoint(		-0.4f,	-0.4f,	0.0f,	R,	G,	B	),
		tScript3DPoint(		-0.4f,	0.4f,	0.0f,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
		Save[i] = localPoints[i];

	return kNumPoints;
}

int32 tDraw::mGoalBottom(	tScript3DPoint	Save[],
							float			R,
							float			G,
							float			B )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		0.4f,	0.4f,	-0.05f,	R,	G,	B	),
		tScript3DPoint(		0.4f,	-0.4f,	-0.05f,	R,	G,	B	),
		tScript3DPoint(		-0.4f,	-0.4f,	-0.05f,	R,	G,	B	),
		tScript3DPoint(		-0.4f,	0.4f,	-0.05f,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
		Save[i] = localPoints[i];

	return kNumPoints;
}

/*

	PURPOSE:	Border track.

*/
void tDraw::mDefineDrawObjectsForMap_BorderTrack( void )
{
	//
	//	Boundaries.
	//
	eScriptPrimitiveFlags boundaryFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint boundaryAnchor;
	boundaryAnchor.x = 1;
	boundaryAnchor.y = 1;

	static	tScript3DPoint	boundary1Points[6];
	int32 boundary1Vertices = mMapBoundary1_BorderTrack( boundary1Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary1, fPrimitives, boundary1Points, boundary1Vertices, boundaryAnchor, boundaryFlags );
#if defined( k2Borders )
	static	tScript3DPoint	boundary1aPoints[6];
	int32 boundary1aVertices = mMapBoundary1_BorderTrack( boundary1aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary1a, fPrimitives, boundary1aPoints, boundary1aVertices, boundaryAnchor, boundaryFlags );
#endif

	static	tScript3DPoint	boundary2Points[6];
	int32 boundary2Vertices = mMapBoundary2_BorderTrack( boundary2Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary2, fPrimitives, boundary2Points, boundary2Vertices, boundaryAnchor, boundaryFlags );
#if defined( k2Borders )
	static	tScript3DPoint	boundary2aPoints[6];
	int32 boundary2aVertices = mMapBoundary2_BorderTrack( boundary2aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary2a, fPrimitives, boundary2aPoints, boundary2aVertices, boundaryAnchor, boundaryFlags );
#endif



	//
	//	Goal lines.
	//
	eScriptPrimitiveFlags goalFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint goalAnchor;
	goalAnchor.x = 2;
	goalAnchor.y = 10;

	static	tScript3DPoint	goalTopPoints[4];
	int32 goalTopVertices = mGoalTop( goalTopPoints );
	mCreateDrawObject( kMapGoalTop, fPrimitives, goalTopPoints, goalTopVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalMiddlePoints[4];
	int32 goalMiddleVertices = mGoalMiddle( goalMiddlePoints );
	mCreateDrawObject( kMapGoalMiddle, fPrimitives, goalMiddlePoints, goalMiddleVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalBottomPoints[4];
	int32 goalBottomVertices = mGoalBottom( goalBottomPoints );
	mCreateDrawObject( kMapGoalBottom, fPrimitives, goalBottomPoints, goalBottomVertices, goalAnchor, goalFlags );
}

int32 tDraw::mMapBoundary1_BorderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 6;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		3.0f,	2.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		12.0f,	2.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		12.0f,	9.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		9.0f,	12.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		3.0f,	12.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		3.0f,	2.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary2_BorderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 6;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		12.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		13.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		13.0f,	13.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		10.0f,	13.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		10.0f,	12.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		12.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}







/*

	PURPOSE:	Spider track.

*/

void tDraw::mDefineDrawObjectsForMap_SpiderTrack( void )
{
	eScriptPrimitiveFlags boundaryFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint boundaryAnchor;
	boundaryAnchor.x = 1;
	boundaryAnchor.y = 1;

	static	tScript3DPoint	boundary1Points[4];
	int32 boundary1Vertices = mMapBoundary1_SpiderTrack( boundary1Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary1, fPrimitives, boundary1Points, boundary1Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary2Points[4];
	int32 boundary2Vertices = mMapBoundary2_SpiderTrack( boundary2Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary2, fPrimitives, boundary2Points, boundary2Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary3Points[4];
	int32 boundary3Vertices = mMapBoundary3_SpiderTrack( boundary3Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary3, fPrimitives, boundary3Points, boundary3Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary4Points[4];
	int32 boundary4Vertices = mMapBoundary4_SpiderTrack( boundary4Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary4, fPrimitives, boundary4Points, boundary4Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary5Points[4];
	int32 boundary5Vertices = mMapBoundary5_SpiderTrack( boundary5Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary5, fPrimitives, boundary5Points, boundary5Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary6Points[4];
	int32 boundary6Vertices = mMapBoundary6_SpiderTrack( boundary6Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary6, fPrimitives, boundary6Points, boundary6Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary7Points[4];
	int32 boundary7Vertices = mMapBoundary7_SpiderTrack( boundary7Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary7, fPrimitives, boundary7Points, boundary7Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary8Points[4];
	int32 boundary8Vertices = mMapBoundary8_SpiderTrack( boundary8Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary8, fPrimitives, boundary8Points, boundary8Vertices, boundaryAnchor, boundaryFlags );

#if defined( k2Borders )
	static	tScript3DPoint	boundary1aPoints[4];
	int32 boundary1aVertices = mMapBoundary1_SpiderTrack( boundary1aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary1a, fPrimitives, boundary1aPoints, boundary1aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary2aPoints[4];
	int32 boundary2aVertices = mMapBoundary2_SpiderTrack( boundary2aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary2a, fPrimitives, boundary2aPoints, boundary2aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary3aPoints[4];
	int32 boundary3aVertices = mMapBoundary3_SpiderTrack( boundary3aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary3a, fPrimitives, boundary3aPoints, boundary3aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary4aPoints[4];
	int32 boundary4aVertices = mMapBoundary4_SpiderTrack( boundary4aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary4a, fPrimitives, boundary4aPoints, boundary4aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary5aPoints[4];
	int32 boundary5aVertices = mMapBoundary5_SpiderTrack( boundary5aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary5a, fPrimitives, boundary5aPoints, boundary5aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary6aPoints[4];
	int32 boundary6aVertices = mMapBoundary6_SpiderTrack( boundary6aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary6a, fPrimitives, boundary6aPoints, boundary6aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary7aPoints[4];
	int32 boundary7aVertices = mMapBoundary7_SpiderTrack( boundary7aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary7a, fPrimitives, boundary7aPoints, boundary7aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary8aPoints[4];
	int32 boundary8aVertices = mMapBoundary8_SpiderTrack( boundary8aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary8a, fPrimitives, boundary8aPoints, boundary8aVertices, boundaryAnchor, boundaryFlags );
#endif

	
	eScriptPrimitiveFlags goalFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint goalAnchor;
	goalAnchor.x = 22;
	goalAnchor.y = 13;

	static	tScript3DPoint	goalTopPoints[4];
	int32 goalTopVertices = mGoalTop( goalTopPoints );
	mCreateDrawObject( kMapGoalTop, fPrimitives, goalTopPoints, goalTopVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalMiddlePoints[4];
	int32 goalMiddleVertices = mGoalMiddle( goalMiddlePoints );
	mCreateDrawObject( kMapGoalMiddle, fPrimitives, goalMiddlePoints, goalMiddleVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalBottomPoints[4];
	int32 goalBottomVertices = mGoalBottom( goalBottomPoints );
	mCreateDrawObject( kMapGoalBottom, fPrimitives, goalBottomPoints, goalBottomVertices, goalAnchor, goalFlags );
}

int32 tDraw::mMapBoundary1_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		5.0f,	1.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		10.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		10.0f,	3.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		5.0f,	1.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary2_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		14.0f,	3.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		14.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		19.0f,	1.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		14.0f,	3.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		Save[i] = localPoints[i];
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary3_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		23.0f,	5.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		17.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		21.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		23.0f,	5.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		Save[i] = localPoints[i];
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary4_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		21.0f,	14.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		17.0f,	14.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		23.0f,	19.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		21.0f,	14.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary5_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		19.0f,	23.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		14.0f,	17.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		14.0f,	21.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		19.0f,	23.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary6_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		10.0f,	21.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		10.0f,	17.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		5.0f,	23.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		10.0f,	21.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary7_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		1.0f,	19.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		7.0f,	14.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		3.0f,	14.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		1.0f,	19.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary8_SpiderTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		3.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		7.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		1.0f,	5.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		3.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}







/*

	PURPOSE:	8 track.

*/

void tDraw::mDefineDrawObjectsForMap_8Track( void )
{
	eScriptPrimitiveFlags boundaryFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint boundaryAnchor;
	boundaryAnchor.x = 1;
	boundaryAnchor.y = 1;

	static	tScript3DPoint	boundary1Points[15];
	int32 boundary1Vertices = mMapBoundary1_8Track( boundary1Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary1, fPrimitives, boundary1Points, boundary1Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary2Points[5];
	int32 boundary2Vertices = mMapBoundary2_8Track( boundary2Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary2, fPrimitives, boundary2Points, boundary2Vertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary3Points[5];
	int32 boundary3Vertices = mMapBoundary3_8Track( boundary3Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary3, fPrimitives, boundary3Points, boundary3Vertices, boundaryAnchor, boundaryFlags );

#if defined( k2Borders )
	static	tScript3DPoint	boundary1aPoints[15];
	int32 boundary1aVertices = mMapBoundary1_8Track( boundary1aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary1a, fPrimitives, boundary1aPoints, boundary1aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary2aPoints[5];
	int32 boundary2aVertices = mMapBoundary2_8Track( boundary2aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary2a, fPrimitives, boundary2aPoints, boundary2aVertices, boundaryAnchor, boundaryFlags );
	static	tScript3DPoint	boundary3aPoints[5];
	int32 boundary3aVertices = mMapBoundary3_8Track( boundary3aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary3a, fPrimitives, boundary3aPoints, boundary3aVertices, boundaryAnchor, boundaryFlags );
#endif

	
	eScriptPrimitiveFlags goalFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint goalAnchor;
	goalAnchor.x = 10;
	goalAnchor.y = 19;

	static	tScript3DPoint	goalTopPoints[4];
	int32 goalTopVertices = mGoalTop( goalTopPoints );
	mCreateDrawObject( kMapGoalTop, fPrimitives, goalTopPoints, goalTopVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalMiddlePoints[4];
	int32 goalMiddleVertices = mGoalMiddle( goalMiddlePoints );
	mCreateDrawObject( kMapGoalMiddle, fPrimitives, goalMiddlePoints, goalMiddleVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalBottomPoints[4];
	int32 goalBottomVertices = mGoalBottom( goalBottomPoints );
	mCreateDrawObject( kMapGoalBottom, fPrimitives, goalBottomPoints, goalBottomVertices, goalAnchor, goalFlags );
}

int32 tDraw::mMapBoundary1_8Track(	tScript3DPoint	Save[],
									float			R,
									float			G,
									float			B,
									float			offX,
									float			offY,
									float			offZ )
{
	const int32 kNumPoints = 15;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		6.0f,	3.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		18.0f,	3.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		22.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		22.0f,	9.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		19.0f,	12.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		22.0f,	15.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		22.0f,	17.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		18.0f,	21.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		6.0f,	21.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		2.0f,	17.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		2.0f,	15.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		5.0f,	12.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		2.0f,	9.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		2.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		6.0f,	3.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary2_8Track(	tScript3DPoint	Save[],
									float			R,
									float			G,
									float			B,
									float			offX,
									float			offY,
									float			offZ )
{
	const int32 kNumPoints = 5;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		9.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		15.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		15.0f,	9.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		9.0f,	9.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		9.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}

int32 tDraw::mMapBoundary3_8Track(	tScript3DPoint	Save[],
									float			R,
									float			G,
									float			B,
									float			offX,
									float			offY,
									float			offZ )
{
	const int32 kNumPoints = 5;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		9.0f,	15.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		15.0f,	15.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		15.0f,	17.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		9.0f,	17.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		9.0f,	15.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}



/*

	PURPOSE:	Planet track.

*/

void tDraw::mDefineDrawObjectsForMap_PlanetTrack( void )
{
	eScriptPrimitiveFlags boundaryFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint boundaryAnchor;
	boundaryAnchor.x = 1;
	boundaryAnchor.y = 1;

	static	tScript3DPoint	boundary1Points[21];
	int32 boundary1Vertices = mMapBoundary1_PlanetTrack( boundary1Points, 0.0f, 0.0f, 1.0f );
	mCreateDrawObject( kMapBoundary1, fPrimitives, boundary1Points, boundary1Vertices, boundaryAnchor, boundaryFlags );
#if defined( k2Borders )
	static	tScript3DPoint	boundary1aPoints[21];
	int32 boundary1aVertices = mMapBoundary1_PlanetTrack( boundary1aPoints, 0.0f, 0.0f, 1.0f, fBorderSpacing, fBorderSpacing, 0.0f );
	mCreateDrawObject( kMapBoundary1a, fPrimitives, boundary1aPoints, boundary1aVertices, boundaryAnchor, boundaryFlags );
#endif

	
	eScriptPrimitiveFlags goalFlags = static_cast< eScriptPrimitiveFlags >( kLineLoop | kCW | kSolid | kDoubleSide | kBlend );
	tMapPoint goalAnchor;
	goalAnchor.x = 24;
	goalAnchor.y = 21;

	static	tScript3DPoint	goalTopPoints[4];
	int32 goalTopVertices = mGoalTop( goalTopPoints );
	mCreateDrawObject( kMapGoalTop, fPrimitives, goalTopPoints, goalTopVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalMiddlePoints[4];
	int32 goalMiddleVertices = mGoalMiddle( goalMiddlePoints );
	mCreateDrawObject( kMapGoalMiddle, fPrimitives, goalMiddlePoints, goalMiddleVertices, goalAnchor, goalFlags );
	static	tScript3DPoint	goalBottomPoints[4];
	int32 goalBottomVertices = mGoalBottom( goalBottomPoints );
	mCreateDrawObject( kMapGoalBottom, fPrimitives, goalBottomPoints, goalBottomVertices, goalAnchor, goalFlags );
}

int32 tDraw::mMapBoundary1_PlanetTrack(	tScript3DPoint	Save[],
										float			R,
										float			G,
										float			B,
										float			offX,
										float			offY,
										float			offZ )
{
	const int32 kNumPoints = 21;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		1.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		11.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		11.0f,	14.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		9.0f,	16.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		4.0f,	16.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		2.0f,	18.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		2.0f,	22.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		19.0f,	22.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		19.0f,	11.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		15.0f,	11.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		15.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		18.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		21.0f,	7.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		21.0f,	22.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		23.0f,	22.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		23.0f,	1.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		14.0f,	1.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		11.0f,	4.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		8.0f,	1.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		1.0f,	1.0f,	fBorderHeight,	R,	G,	B	),
		tScript3DPoint(		1.0f,	10.0f,	fBorderHeight,	R,	G,	B	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
	{
		localPoints[i].fX += offX;
		localPoints[i].fY += offY;
		localPoints[i].fZ += offZ;
		Save[i] = localPoints[i];
	}

	return kNumPoints;
}