/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"

/*
	G-L is the start position of the teams.
	K-T are the checkpoints.  Do not skip letters.  Start at K and end at any letter up to T.
	U-Z are the points where boxes will spawn.  If none specified, then is random.

	If you add, delete, or change anything in this map list, you must update "CommonDefs.cpp", "CommonDefs.h",
	"CheckpointTeam.cpp", "Draw.cpp", and "Draw.h" -- whew!
*/
int32 TotalMaps = 5;
const char* MapTypes[] = {	"Border Track",
							"Spider Track",
							"8 Track",
							"Planet Track",
							"Relay Track" };

/*
	Legend:
			Starting Positions			Ending Positions			Planets			
			Start1 --> G				End1 --> g					Earth (Organia) --> 1
			Start2 --> H				End2 --> h					Ringed Earth --> 2
			Start3 --> I				End3 --> i					Mars --> 3
			Start4 --> J				End4 --> j					Jupiter --> 4
			Start5 --> K				End5 --> k					Dark Earth --> 5
			Start6 --> L				End6 --> l					Saturn --> 6
			Start7 --> M				End7 --> m					Night City --> 7
			Start8 --> N				End8 --> n					Fire --> 8
			Start9 --> O				End9 --> o					Ice --> 9
			Start10 --> P				End10 --> p					Gas --> 0
			Start11 --> Q				End11 --> q					
			Start12 --> R				End12 --> r					
			Start13 --> S				End13 --> s					
			Start14 --> T				End14 --> t					
			Start15 --> U				End15 --> u					
			Start16 --> V				End16 --> v					
			Start17 --> W				End17 --> w					
			Start18 --> X				End18 --> x					
			Start19 --> Y				End19 --> y					
			Start20 --> Z				End20 --> z					

			Celestial Bodies			Asteroids					Black Holes
			Moon --> @					SmallAsteroid --> [			SmallBlackHole --> :
			Sun1 --> (					LargeAsteroid --> ]			LargeBlackHole --> ,
			Sun2 --> )					IceAsteroid --> *			
			Sun3 --> !
			
			Nebulas						Dust Clouds					Other
			Nebula1 --> &				LightDustCloud --> <		WormHole --> %
			Nebula2 --> ?				HeavyDustCloud --> >		Fissure --> $
																	Corpse --> #
																	Organian --> ^
*/
const char* MapLines[]=
{
   "Mul_8Race",
   "// Border Track",
   " +____1____2____3+",
   " |...............|",
   " |.K..L....M...N.|",
   " |...............|",
   " |.ij.<<<<<<<<...|",
   "1|.gh.<<<<<<<<.O.|",
   " |.GH.<<<<<<<<...|",
   " |.IJ.<<<<<<<<...|",
   " |....<<<<<<<<...|",
   " |....<<<<<<<<...|",
   "2|.T..<<<<<<<<.P.|",
   " |....<<<<<<<....|",
   " |....<<<<<<..<..|",
   " |...........<<..|",
   " |.S..R...Q......|",
   "3|...............|",
   " +---------------+",
   "      1    2    3",
   "// Spider Track",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |......<<<<.....<<<<......|",
   " |...Q...<<<..O..<<<...M...|",
   "1|........<<.....<<........|",
   " |.........<.....<.........|",
   " |..<...................<..|",
   " |..<<.................<<..|",
   " |..<<<...............<<<..|",
   "2|..<<<<.............<<<<..|",
   " |.........................|",
   " |.........................|",
   " |...L......I...J......R...|",
   " |..........iG.Hj..........|",
   "3|...........g.h...........|",
   " |..<<<<.............<<<<..|",
   " |..<<<...............<<<..|",
   " |..<<.................<<..|",
   " |..<...................<..|",
   "4|.........<.....<.........|",
   " |........<<.....<<........|",
   " |...N...<<<..K..<<<...P...|",
   " |......<<<<.....<<<<......|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// 8 Track",
   " +____1____2____3____4____5+",
   " |<<<<<<<<<<<<<<<<<<<<<<<<<|",
   " |<<<<<<<<<<<<<<<<<<<<<<<<<|",
   " |<<<<<<<<<<<<<<<<<<<<<<<<<|",
   " |<<<<<<.............<<<<<<|",
   "1|<<<<<.......P.......<<<<<|",
   " |<<<<.................<<<<|",
   " |<<<.........L.........<<<|",
   " |<<.....................<<|",
   " |<<........*****........<<|",
   "2|<<.....................<<|",
   " |<<<.........O.........<<<|",
   " |<<<<........K........<<<<|",
   " |<<<<<...............<<<<<|",
   " |<<<<........M........<<<<|",
   "3|<<<.........Q.........<<<|",
   " |<<.....................<<|",
   " |<<........*****........<<|",
   " |<<................gh...<<|",
   " |<<<.........N.....ij..<<<|",
   "4|<<<<..............GH.<<<<|",
   " |<<<<<.......R.....IJ<<<<<|",
   " |<<<<<<.............<<<<<<|",
   " |<<<<<<<<<<<<<<<<<<<<<<<<<|",
   " |<<<<<<<<<<<<<<<<<<<<<<<<<|",
   "5|<<<<<<<<<<<<<<<<<<<<<<<<<|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Planet Track",
   " +____1____2____3____4____5+",
   " |<<<<<<<<<<<<<<<<<<<<<<<<<|",
   " |<........<<<<<...P......<|",
   " |<.N.......<<<...........<|",
   " |<..........<............<|",
   "1|<....7.............8...Q<|",
   " |<.......................<|",
   " |<........M..............<|",
   " |<.......................<|",
   " |<..O..............<<<...<|",
   "2|<................<<<<...<|",
   " |<...............<<<<<...<|",
   " |<<<<<<<<<<<.........<.R.<|",
   " |<<<<<<<<<<<.........<...<|",
   " |<<<<<<<<<<<.........<...<|",
   "3|<<<<<<<<<<<......K..<...<|",
   " |<<<<<<<<<<..........<...<|",
   " |<<<<................<.S.<|",
   " |<<<.............9...<...<|",
   " |<<....jg............<...<|",
   "4|<<...JGih...........<...<|",
   " |<<....IH.....L......<...<|",
   " |<<..................<.T.<|",
   " |<<..................<...<|",
   " |<<<<<<<<<<<<<<<<<<<<<<<<<|",
   "5|<<<<<<<<<<<<<<<<<<<<<<<<<|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Relay Track",
   " +____+",
   " |>>>>|",
   " |>T>>|",
   " |>>>>|",
   " |>>>>|",
   "1|>>>>|",
   " |>>S>|",
   " |>>>>|",
   " |>>>>|",
   " |>>>>|",
   "2|.K.M|",
   " |Q.O.|",
   " |....|",
   " |....|",
   " |jghi|",
   "3|JGHI|",
   " |....|",
   " |....|",
   " |.P.R|",
   " |N.L.|",
   "4|>>>>|",
   " +----+",
   "///",
   0
};