/********************************************************************************************************\

	Mission:	Mul_8Race
	Script:		"Mul_8Race"
	Descript:	A race around a large map.  Checkpoints determine "bounds"

	Created by:		Daniel J. Suleski		06/24/00
	Last Updated:	DJS						06/25/00
					Initial setup.  Map creation.
					DJS						06/26/00
					World Conditions.  Additional Text.
					DJS						07/19/00
					Second pass.
					DJS						07/20/00
					Adding wormholes and boxes.
					DJS						07/30/00
					Added boundaries with dust clouds using new tDraw.
					DJS						08/23/00
					Map overrides.
					DJS						08/26/00
					Text check.  Disengage now destroys ships.  Can now set sides.
					DJS						08/28/00
					Added the relay track.  Made primitives double lined -- no longer solid masses.
					DJS						09/26/00
					Meta script modifications
					DJS						10/02/00
					Vertex objects added and polygon code is now just for lines.


\********************************************************************************************************/

#include "CheckPointTeam.h"
#include "CommonDefs.h"
#include "CommonTeam.h"
#include "Draw.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "Mul_8Race.h"
#include "SFCAssert.h"
#include "Strength.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


DEFINE_SCRIPT_CLASS( tMul_8Race );

tMul_8Race::tMul_8Race()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tMul_8Race::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fHelpTags = Messages[kHelpTags_msg];
	Specs->fMissionType = kMultiplayerMission;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecOneShip;
	Specs->fMenuStates = kDisableTagE | kDisableTagF | kDisableAI | kEnableSupplyDock;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k2Band | k4Client );

	/*

		HELPER NOTE:	In this case, there are 6 teams setup.  Feel free to change the number of teams
						by deleting them starting with team6 and working your way down to team3, inclusive.
						Also, if the team is to be controlled by the AI and only the AI, then pass the
						constant 'kNPCTeam' instead of 'kPlayableTeam' into the team specification.  Also,
						not noted here, the last parameter of mCreateTeamSpec() can be a team tag such as
						"kTagA" to "kTagF".  This is the default tag for a kNPCTeam that was not chosen
						on the multiplayer UI screen.

	*/
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1",
												static_cast< eTeamID >( kPlayerTeam1 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ),
												static_cast< eTeamTag >( kTeam1Tag ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2",
												static_cast< eTeamID >( kPlayerTeam2 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ),
												static_cast< eTeamTag >( kTeam2Tag ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3",
												static_cast< eTeamID >( kPlayerTeam3 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ),
												static_cast< eTeamTag >( kTeam3Tag ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4",
												static_cast< eTeamID >( kPlayerTeam4 ),
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ),
												static_cast< eTeamTag >( kTeam4Tag ) );

	const tTeamSpec& checkpointTeam = mCreateTeamSpec(	"Checkpoint Team",
														static_cast< eTeamID >( kCheckPointTeam ),
													kMaxChance,
													kTrue,
													kNPCTeam,
													kMaxChance,
													kSpecAnyRace,
													kAnyRace,
													typeid( tCheckPointTeam ),
													static_cast< eTeamTag >( kCheckPointTeamTag ) );

	/*

		HELPER NOTE:	Please note that although you setup the teams, you need to create ships and their
						respective files.  Otherwise, an assert will occur.  It is up to you to decide
						whether the ship files should be shared by multiple physical teams or just a
						1-to-1 relationship between the team and the ship files.

	*/
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( checkpointTeam );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tMul_8Race::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tMul_8Race::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tMul_8Race::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tMul_8Race::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tMul_8Race::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 tagList[kRelationsCount] =
	{
		kTeam1Tag,
		kTeam2Tag,
		kTeam3Tag,
		kTeam4Tag,
		kCheckPointTeamTag,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 Tm1   Tm2   Tm3   Tm4   CkPt
		/*Tm1 */ "AI", "WD", "WD", "WD", "FI",
		/*Tm2 */ "WD", "AI", "WD", "WD", "FI",
		/*Tm3 */ "WD", "WD", "AI", "WD", "FI",
		/*Tm4 */ "WD", "WD", "WD", "AI", "FI",
		/*CkPt*/ "FI", "FI", "FI", "FI", "AI",
	};

	fMissionInfo->mSetTagRelations( tagList, ( std::string* )relationships, kRelationsCount );

	/*

		HELPER NOTE:	You may wish to check if the game has ended here in the case, for instance, in
						the case that all physical teams share the same team tag letter and therefore, have
						no common enemies.

	*/
}

/*

	HELPER NOTE:	Since you really do not know which teams are going to be associated with which team
					tages until the game starts, you must create the teams here.  This function may look
					like the following:

					void tMul_8Race::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
					{
						switch ( teamScriptDescription.fTeamTag )
						{
						case kTagA:
						case kTagB:
						case kTagC:
						case kTagD:
						case kTagE:
						case kTagF:
							{
								mMission()->mSetupTeam(	teamScriptDescription.fTeam,
														teamScriptDescription.fTeamTag,
														teamScriptDescription.fRace,
														teamScriptDescription.fHumanControlled,
														typeid( tCommonTeam ) );
							}
							break;
						}
					}

*/
void tMul_8Race::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{
	switch ( teamScriptDescription.fTeamTag )
	{

	//
	//	This is a human controlled team.
	//
	case kTeam1Tag:
	case kTeam2Tag:
	case kTeam3Tag:
	case kTeam4Tag:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									teamScriptDescription.fRace,
									teamScriptDescription.fHumanControlled,
									typeid( tCommonTeam ) );
		}
		break;

	//
	//	This is the team put aside for the checkpoints.
	//
	case kCheckPointTeamTag:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									teamScriptDescription.fRace,
									teamScriptDescription.fHumanControlled,
									typeid( tCheckPointTeam ) );
		}
		break;
	}
}

/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
void tMul_8Race::mInitializeDraw( const int32 MapPicked )
{
	gWC.fMapPicked = MapPicked;

	switch ( MapPicked )
	{
	case 1:
		gDraw.mDefineDrawObjectsForMap_BorderTrack();		break;
	case 2:
		gDraw.mDefineDrawObjectsForMap_SpiderTrack();		break;
	case 3:
		gDraw.mDefineDrawObjectsForMap_8Track();			break;
	case 4:
		gDraw.mDefineDrawObjectsForMap_PlanetTrack();		break;
	case 5:
	default:
		break;		//	do nothing
	}
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMul_8Race::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['K'] = kObjectShipEnd;
	Map->fTacticalMapIcon['L'] = kObjectShipEnd;
	Map->fTacticalMapIcon['M'] = kObjectShipEnd;
	Map->fTacticalMapIcon['N'] = kObjectShipEnd;
	Map->fTacticalMapIcon['O'] = kObjectShipEnd;
	Map->fTacticalMapIcon['P'] = kObjectShipEnd;
	Map->fTacticalMapIcon['Q'] = kObjectShipEnd;
	Map->fTacticalMapIcon['R'] = kObjectShipEnd;
	Map->fTacticalMapIcon['S'] = kObjectShipEnd;
	Map->fTacticalMapIcon['T'] = kObjectShipEnd;
}
