/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "CommonDefs.h"
#include "CommonTeam.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions(): fNumPlayers( 0 ), fTotalCheckPoints( 0 ), fMapPicked( 0 )
{}

void tWorldConditions::mCheckForGameEnd( void )
{
	eTeamID	teamThatWon = kNoTeam;
	tMissionInfo* thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	First, find out if anyone has passed the last checkpoint.
	//
	for ( int32 i = kPlayerTeam1; i <= kPlayerTeam4; i++ )
	{
		tCommonTeam* team = dynamic_cast< tCommonTeam* >( thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) ) );
		if ( !team )
			continue;

		if ( team->mNumCheckPointsPassed() >= fTotalCheckPoints )
			teamThatWon = static_cast< eTeamID >( i );
	}

	//
	//	If there is no winner yet, let us see if all of the teams are still in play.
	//
	if ( teamThatWon == kNoTeam )
	{
		for ( i = kPlayerTeam1; i <= kPlayerTeam4; i++ )
		{
			tTeamInfo* team = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( !team )
				continue;

			if ( team->mGetGameStatus() == kGameInProgress )
			{
				if ( teamThatWon != kNoTeam )
				{
					teamThatWon = kNoTeam;
					break;
				}
				teamThatWon = static_cast< eTeamID >( i );
			}
		}
	}

	if ( teamThatWon != kNoTeam )
	{

		//
		//	If a team has passed the last checkpoint, then that team has won so everyone else blows up.
		//
		for ( i = kPlayerTeam1; i <= kPlayerTeam4; i++ )
		{
			tTeamInfo* team = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( !team )
				continue;

#if 0		///DANNF -- do not take out till certain that this script works
			if ( i != teamThatWon )
			{
				tShipIterator	ship = team->mGetFirstShip();
				tShipInPlayEval	x;
				ship.mAddEvaluator( &x );
				for ( ; *ship != NULL; ship++ )
				{
					///??DAN -- do not kill players when mission is over -- bad crash
					;//ship->mDestroyShip();
				}
			}
#endif
		}

		//
		//	The teams should receive their repective debriefings.
		//
		tTeamInfo* winningTeam = thisMission->mGetTeamHandle( static_cast< eTeamID >( teamThatWon ) );
		if ( !winningTeam )
			return;

		for ( i = kPlayerTeam1; i <= kPlayerTeam4; i++ )
		{
			tTeamInfo* team = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( !team )
				continue;

			if ( team->mGetTag() == winningTeam->mGetTag() )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
			}
		}
	}
}

void tWorldConditions::mInitializeCheckPoints()
{
	tMissionInfo* thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	Save the positions of the checkpoints.
	//
	int32 x = 0, y = 0;
	for ( char j = kStartLetterForCheckPoints; j <= kStartLetterForCheckPoints + kMaxNumCheckPoints; j++ )
	{
		thisMission->mGetGridPosition( j, &x, &y );
		if ( x >= 0 && y >= 0 )
		{
			gCheckPointPositions[j - kStartLetterForCheckPoints].x = x;
			gCheckPointPositions[j - kStartLetterForCheckPoints].y = y;
		}
		else
			break;
	}
}


void tWorldConditions::mIncrementNumPlayers()
{
	fNumPlayers++;
}
void tWorldConditions::mSetTotalCheckPoints( int32 x )
{
	fTotalCheckPoints = x;
}

int32 tWorldConditions::mNumPlayers()
{
	return fNumPlayers;
}
int32 tWorldConditions::mTotalCheckPoints()
{
	return fTotalCheckPoints;
}