/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_9C2Welcome.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus(eTeamID ID)
{
	if(fChildActor->mBaseDestroyed())
		return kAstoundingVictory;
	else
		return kDevastatingDefeat;

}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( (eTeamID) kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( (eTeamID) kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetTeam() )
	{
	case kPlayerTeam: 
		if ( !playerTeam->mShipsRemainingInPlay() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );
		break;
	case kPlayerNPC1Team:
	case kPlayerNPC2Team:
	case kPlayerNPC3Team:
	case kPlayerNPC4Team:
		fChildActor->mIncAlliesDestroyed();
		if(fChildActor->mAlliesDestroyed() == 4)
			fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Science Officer: All allied ships are destroyed. We're alone.");
		break;


	case kEnemyTeam:
		if( ship->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
		{
			fChildActor->mSetBaseDestroyed(true);
		}
	}

}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetOriginalTeam() )
	{
	case kPlayerTeam: 
		if ( !playerTeam->mShipsRemainingInPlay() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerCaptured );
		break;
	case kPlayerNPC1Team:
	case kPlayerNPC2Team:
	case kPlayerNPC3Team:
	case kPlayerNPC4Team:
		fChildActor->mIncAlliesDestroyed();
		if(fChildActor->mAlliesDestroyed() == 4)
			fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Science Officer: All allied ships are destroyed. We're alone.");
		break;
	case kEnemyTeam:
		if( ship->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
		{
			fChildActor->mSetBaseDestroyed(true);
		
		}
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle(  (eTeamID) kEnemyTeam );
		Assert( team );
		if ( team )
		{
			if ( !team->mShipsRemainingInPlay() )
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
		}
		break;
	}
	

}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	
	
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	tShipInfo * playership;
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		// Get the first undestroyed ship, so that we make sure
		// it's the flagship disengaging.

		playership = *(playerTeam->mGetFirstShip());
		for(playership; playership != NULL; playership++)
		{
			if(playership->mInPlay())
				break;
		}
		if(playership->mGetCustomID() == ship->mGetCustomID() )
		{
			fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Science Officer: We have left the system, [PLAYER_RANK].\n\n");
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDisengaged );
		}
		break;

	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
