/********************************************************************************************************\

	File Name:	EnemyTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "EnemyTeamBaseState.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "MissionText.h"
#include "Uni_9C2Welcome.h"

#if !defined (kTimerConstant)
#define kTimerConstant 2 / 60
#endif
/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tEnemyTeamBaseState::mBeginState()
{}
void tEnemyTeamBaseState::mEndState()
{}

void tEnemyTeamBaseState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	if( ship->mGetTeam() == kEnemyTeam)
	{
		if(ship->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
			fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "First Officer: The base has been destroyed. Mission Accomplished.\n\nCrew: Cheers!\n\n");

		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		Assert( team );
		if ( team )
		{
			if ( !team->mShipsRemainingInPlay() )
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
		}
	}
	
}
void tEnemyTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	tPlayerTeam * playerteam = (tPlayerTeam *) fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam);
	tUni_9C2Welcome * script = (tUni_9C2Welcome *) gScriptInterface.fScript;

	if(targetShip->mGetTeam() == kEnemyTeam)
	{
		switch(buttonID)
		{
		case kLowerShields_btn:
			// check this, though it's really redundant, since
			// only the starbase get's that button
			if(targetShip->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
			{
				if(!script->gCodeKnown )
				{
					fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Communications Officer: We'd need to know their prefix codes, [PLAYER_RANK].");
				}
				else
				{
					if(!script->gShieldsLowered)
					{
						targetShip->mSetSystemHealthFromCurr(kAllShieldsHardPoint, 50.0f);
						fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Communications Officer: Their shields are down, [PLAYER_RANK]\n\n");
				// note that using the turns setting with a constant *should* make the 
				// timer constant at all game speeds.
						fChildActor->mSetTimedInterval(kTimerTurnsSet, fChildActor->kShieldTimerID, 2 * kTimerConstant);
						script->gShieldsLowered = true;
					}
					else
					{
						fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Communications Officer: They've changed the prefix code. No effect, [PLAYER_RANK].");
					}
				}
			}
			break;
		}
	}
}
void tEnemyTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tEnemyTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnFlashingButtonPressed()
{}
void tEnemyTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tEnemyTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tEnemyTeamBaseState::mOnHostChanged()
{}
void tEnemyTeamBaseState::mOnTimedInterval( int32 timerID )
{
	tShipInfo * starbase;
	float fShield;
	switch(timerID)
	{
	case tEnemyTeam::kShieldTimerID:
		if(fChildActor->mShieldTimer() <= 10 )
		{
			fShield = 100.0f;
			
		}
		else
		{
			fShield = 0.0f;
		}
		fChildActor->mIncShieldTimer();
		starbase = *(fChildActor->mGetFirstShip());
		for(starbase; starbase != NULL; starbase++)
		{
			if(starbase->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
				break;
		}
		Assert(starbase);
		if(starbase != NULL)
			starbase->mSetSystemHealthFromCurr(kAllShieldsHardPoint, fShield);

		if(fChildActor->mShieldTimer() <= 10)
			fChildActor->mSetTimedInterval(kTimerTurnsSet, timerID, (float)2 * kTimerConstant);
		
		break;
	}
}
void tEnemyTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

