/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "Uni_9C2Welcome.h"


#define kTimerConstant 2 / 60

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tPlayerTeamBaseState::mBeginState()
{}
void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{

	if(!hostileShip)
		return;
	if( (hostileShip->mGetTeam() == kPlayerTeam ) && (damagedShip->mGetTeam() == kEnemyTeam) && (fChildActor->mRandomChat()) )
	{
		fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "[PLAYER_NAME]: Quiet on the bridge!");
		fChildActor->mSetRandomChat(false);
	}
}
void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}
void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}
void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	static bool started = false;
	int32 x, y;

	tShipInfo  * playership = *(fChildActor->mGetFirstShip());
	// I'm purposely not doing an assert here, as all I'm checking for is the presence
	// of the player ship, i.e. has it been created yet.

	if(playership == NULL)
	{
		fChildActor->mSetTimedInterval(	kTimerTurnsSet, timerID, (float) kTimerConstant * 5);
		return;
	}

	// Make sure we get the first valid undestroyed ship.
    // A destroyed ship  will indicate that has left the system.

	for(playership; playership != NULL; playership++)
	{

		if(playership->mInPlay())
			break;
	}

	if(!started)
	{
		fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, kFedStart_msg + fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam)->mGetRace());
		started = true;
	}
	Assert(playership);

	if ( timerID == tPlayerTeam::kEventTimerID )
	{
		if( fChildActor->mRandomChat() )
		{

			// Is counter at 0?
			if( fChildActor->mChatterCounter() == 0 )  
			{
				// display message and reset counter
				fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, kRandChat1_msg + fMissionInfo->mRandomInt32(0, 5) );
				fChildActor->mSetChatterCounter( fMissionInfo->mRandomInt32(3, 6) );
				
			}
			else
			{
				// decrement chatter counter
				fChildActor->mDecChatterCounter();
			}

			// Now check to see if we're within 100k of the starbase. If so, shut up.
			
			tShipIterator starbase = fMissionInfo->mGetTeamHandle( (eTeamID) kEnemyTeam)->mGetFirstShip();
			for(starbase; *starbase != NULL; starbase++)
			{
				if( (*starbase)->mGetCustomID()== tEnemyTeam::kEnemyStarBase)
					break;
			}
			Assert(*starbase != NULL);
			if(playership != NULL && *starbase != NULL)
			{

				if(playership->mGetTacticalDistance(*starbase) < 100)
				{
					fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "[PLAYER_NAME]: All right, quiet on the bridge! \n\n");
					fChildActor->mSetRandomChat(false);
				}
			}
	
		}
		playership->mGetGridPosition(&x, &y);
		if(AtEdgeOfMap(x, y))
		{
			if( !fChildActor->mAtEdgeOfMap() && !playership->mTestShipStatus(kShipDestroyed) )
			{
				fChildActor->mSetAtEdgeOfMap(true);
				fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Science Officer: Leaving the system, [PLAYER_RANK]\n\n");
			}
		}
		else
		{
			if(fChildActor->mAtEdgeOfMap())
			{
				fChildActor->mSetAtEdgeOfMap(false);
			}
		}

		fChildActor->mSetTimedInterval(	kTimerTurnsSet, timerID, (float) kTimerConstant * 5);
	}

}
void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

bool tPlayerTeamBaseState::AtEdgeOfMap(int32 X, int32 Y)
{
	int32 GridX;
	int32 GridY;
	gScriptInterface.mGetMapDimensions( &GridX, &GridY);

	// kludge to keep a destroyed play ship from indication off the map.
	if( X == 0 && Y == 0)
		return false;

	if( (X < 2 || X > (GridX - 1)) || (Y < 2 || Y > (GridY - 1)))
	{
		return true;
	}
	else
	{
		return false;
	}
}



