/********************************************************************************************************\

	Mission:	Where You're Not Welcome
	Script:		"Uni_9C2Welcome"
	Descript:	Assault on an ISC starbase, without allies. In this version
				the player has NOT obtained a prefix code to lower the ISC starbase
				shields.

	Created by:		Dutch Blomenkamp mm/dd/yy
	Last Updated:	NDB	07/18/00
					

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerTeam.h"
#include "Uni_9C2Welcome.h"



DEFINE_SCRIPT_CLASS( tUni_9C2Welcome );

tUni_9C2Welcome::tUni_9C2Welcome() : 
	gShieldsLowered(false),
		gCodeKnown(false)
{}

/*

	HELPER NOTE:	You may be able to register states here; however it is suggested that you do not.  Use
					only if you must and know how.

*/
void tUni_9C2Welcome::mSetupState()
{}

/*

	HELPER NOTE:	You may be able to register comm states here; however it is suggested that you do not.
					Use only if you must and know how.

*/
void tUni_9C2Welcome::mSetupCommState()
{}

/*

	HELPER NOTE:	This function should return whom you consider to be the primary enemy.  Perhaps useful
					in multiplayer scripts, but single player missions do not seem to actively use it.

*/
int32 tUni_9C2Welcome::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tUni_9C2Welcome::mScriptSpecs( tMissionSpec* Specs )
{

	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fEnemyRace = kSpecISC;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMinBPV = 600;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 6;
	Specs->fMenuStates = kEnableSupplyDock;


	
	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												(eTeamID)kPlayerTeam,
												1.0f,
												kFalse,
												kPlayableTeam, 1.0f,
												tStrength(	0, 0,
																		kClassHeavyCruiser,
																		kClassHeavyCruiser,
																		0, 300 ),
												kSpecAnyRace & ~kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ) );

			
	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												(eTeamID)kEnemyTeam,
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												tStrength(	0, 0,
																		kClassStarBase,
																		kClassStarBase,
																		0, 999 ),
												kSpecISC,
												kAnyRace,
												typeid( tEnemyTeam ) );

	
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_9C2Welcome::mInitializeStart( tDynaverseScriptInfo& info )
{
	// These are global variables that
	// will be passed back and forth to the campaign,
	// if I'm understanding things correctly.
	// I'm initializing them as they would be if the 
	// player had successfully completed the pre-requisites.
	gCodeKnown = false;
	gShieldsLowered = false;


	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_9C2Welcome::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_9C2Welcome::mInitializeFinished( void )
{
	tTeamInfo * PlayerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam);
	//Assert(PlayerTeam);
	if(!PlayerTeam)
	{
		return kBriefing_msg;
	}
	else
	{
		return kFedBriefing_msg + PlayerTeam->mGetRace();
	}
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_9C2Welcome::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tUni_9C2Welcome::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_9C2Welcome::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_9C2Welcome::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC1 = kPlayerNPC1Team,
		//PNPC2 = kPlayerNPC2Team,
		//PNPC3 = kPlayerNPC3Team,
		//PNPC4 = kPlayerNPC4Team,
		//Nme  = kEnemyTeam,
		

		//			Plyr	Nme
		/*Plyr*/	"--",	"WD",
		/*Nme */	"WD",	"--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}
