/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_X3M2AGlass.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if(fChildActor->mTalkedToBlackHole())
		return kAstoundingVictory;
	else
		return kDevastatingDefeat;
		
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tBaseVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == kPlayerTeam )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{


	tTeamInfo * player = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	ship = *(player->mGetFirstShip());
	Assert(ship);
	if(ship->mGetTacticalDistance(targetShip) > 15.0)
	{
		Messages[kUtility_msg].operator =(Messages[kDistanceOver15K_msg]);
	}
	else
	{
		switch(targetShip->mGetCustomID())
		{

		case tPlayerNPCTeam::kStar1:
			if(!fChildActor->mTalkedToStar1())
			{
				Messages[kUtility_msg].operator =(Messages[kStar1Dialog_msg]);
				mAddToPrestige(50);
				fChildActor->mSetTalkedToStar1(true);
			}
			else
			{
				Messages[kUtility_msg].operator =(Messages[fMissionInfo->mRandomInt32(kStar1Rand1_msg, kStar1Rand3_msg)]);
			}
			break;
		case tPlayerNPCTeam::kStar2:
			if(!fChildActor->mTalkedToStar2())
			{
				Messages[kUtility_msg].operator =(Messages[kNoIntelligencePossible_msg]);
				fChildActor->mSetTalkedToStar2(true);
				player->mGotoCommState(targetShip->mGetShipID(), kBlankCommState);
				fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kUtility_msg);
			}
			break;
		case tPlayerNPCTeam::kStar3:
			if(!fChildActor->mTalkedToStar3())
			{
				Messages[kUtility_msg].operator =(Messages[kNoIntelligencePossible_msg]);
				fChildActor->mSetTalkedToStar3(true);
				player->mGotoCommState(targetShip->mGetShipID(), kBlankCommState);
				fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kUtility_msg);
			}
			break;
		
		case tPlayerNPCTeam::kStar4:
			if(!fChildActor->mTalkedToStar4())
			{
				Messages[kUtility_msg].operator =(Messages[kStar4Dialog_msg]);
				mAddToPrestige(50);
				fChildActor->mSetTalkedToStar4(true);
			}
			else
			{
				Messages[kUtility_msg].operator =(Messages[fMissionInfo->mRandomInt32(kStar4Rand1_msg, kStar4Rand3_msg)]);
			}
			break;
		case tPlayerNPCTeam::kHole:
			if(ship->mGetTacticalDistance(targetShip) > 5.0)
			{
				Messages[kUtility_msg].operator =(Messages[kDistanceOver5K_msg]);
			}
			else
			{
				if(!fChildActor->mTalkedToBlackHole())
				{
					Messages[kUtility_msg].operator =(Messages[kHoleDialog_msg]);
					fChildActor->mSetTalkedToBlackHole(true);
					player->mGotoCommState(targetShip->mGetShipID(), kBlankCommState);
					fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kUtility_msg);
				}
			}
			break;

		default:
			fMissionInfo->mDebugMessage("No case found!");

		}
	}
}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn)
	{
		mSetTimedInterval( kTimerTurnsSet, fChildActor->kScanID, 0.75f );
	}
	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, fChildActor->kScanID, 0.0f );
	}

}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	int32 kMessage;
	int32 X, Y;
	tShipInfo * target;
	bool validtarget = false;
	static int32 kHoleFlag = -1;

	tTeamInfo * enemy = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerNPCTeam);
	tTeamInfo * player = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(enemy);
	Assert(player);
	tShipIterator ship = player->mGetFirstShip();
	tShipIterator hole = enemy->mGetFirstShip();
	for(hole; (*hole != NULL); hole++)
	{
		if((*hole)->mGetCustomID() == tPlayerNPCTeam::kHole)
			break;
	}

	switch(timerID)
	{
	case tBaseVictory::kStartID:
		// Display starting message, and start the general
		// timer for periodic checks.
		fChildActor->mSetTimedInterval( kTimerTurnsSet, tBaseVictory::kEdgeCheckID, 0.5f );
		fScanning = false;
		break;

	case tBaseVictory::kEndTimer:
		// provides a chance for the final messages to be displayed and 
		// read.
		gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
		break;

	case tBaseVictory::kBlackHoleID:
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), Messages[kWaitingTooLong_msg + kHoleFlag].c_str() );
		kHoleFlag ++;
		if( ship->mGetTacticalDistance((*hole)) > 15)
		{
			mSetTimedInterval(kTimerSecondsSet, tBaseVictory::kEndTimer, 10.0);
			break;
		}
		mSetTimedInterval(kTimerTurnsSet, tBaseVictory::kBlackHoleID, 0.25f );
		if(kHoleFlag == 5)
			(*ship)->mDestroyShip();
		break;

	case tBaseVictory::kScanID:
		fScanning = true;
		break;

	case tBaseVictory::kEdgeCheckID:
		(*ship)->mGetGridPosition(&X, &Y);
		if(mAtEdgeOfMap(X, Y))
		{
			fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kLeavingSystem_msg);
		}
		if(fChildActor->mTalkedToBlackHole())
		{
			// The player has reached the black hole, talked to it,
			// and gotten his warnings. Now the fun starts.
			mSetTimedInterval(kTimerTurnsSet, tBaseVictory::kBlackHoleID,  0.5f );
			mSetTimedInterval(kTimerKillTimer, tBaseVictory::kEdgeCheckID,  0.5f );
			break;
		}
		else
		{
			if( ship->mGetTacticalDistance((*hole)) <= 10)
			{
				if(kHoleFlag == -1)
				{
					fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kHoleWarning_msg);
					kHoleFlag++;
				}
			}
			mSetTimedInterval(kTimerTurnsSet, tBaseVictory::kEdgeCheckID, .5f );
		}
	
		if( !fScanning )
			return;

		target= fMissionInfo->mFindShipID(ship->mGetShipTargetID());
		if( target == NULL)
			return;
	
		if((*ship)->mGetTacticalDistance(target) <= 30.0)
		{
			validtarget = true;
			switch(target->mGetCustomID())
			{
			case tPlayerNPCTeam::kStar1:
				if(!fChildActor->mScannedStar1())
				{
					Messages[kUtility_msg].operator =(Messages[kScanningStars1and2_msg]);
					fChildActor->mSetScannedStar1(true);
				}
				else
					Messages[kUtility_msg].operator =(Messages[kScanNoFurtherInfo_msg]);
				break;

			case tPlayerNPCTeam::kStar2:
				if(!fChildActor->mScannedStar2())
				{
					Messages[kUtility_msg].operator =(Messages[kScanningStars1and2_msg]);
					fChildActor->mSetScannedStar2(true);
				}
				else
					Messages[kUtility_msg].operator =(Messages[kScanNoFurtherInfo_msg]);
				break;

			case tPlayerNPCTeam::kStar3:
				if(!fChildActor->mScannedStar3())
				{
					Messages[kUtility_msg].operator =(Messages[kScanningStars3and4_msg]);
					fChildActor->mSetScannedStar3(true);
				}
				else
					Messages[kUtility_msg].operator =(Messages[kScanNoFurtherInfo_msg]);
				break;

			case tPlayerNPCTeam::kStar4:

				if(!fChildActor->mScannedStar4())
				{
					fChildActor->mSetScannedStar4(true);
					Messages[kUtility_msg].operator =(Messages[kScanningStars3and4_msg]);
				}
				else
					Messages[kUtility_msg].operator =(Messages[kScanNoFurtherInfo_msg]);
				break;

			case tPlayerNPCTeam::kHole:
				if(!fChildActor->mScannedBlackHole())
				{
					Messages[kUtility_msg].operator =(Messages[kScanningHole_msg]);
					fChildActor->mSetScannedBlackHole(true);
				}
				break;

			}
		}
		if(!validtarget)
		{
			char distance[255];
			sprintf(distance, "Science Officer: Range to target %f tactical units.", (*ship)->mGetTacticalDistance(target));
			Messages[kUtility_msg].operator =(distance);
			Messages[kUtility_msg].operator +=(Messages[kScanOutOfRange_msg]);
		}
		
		kMessage = kUtility_msg;
		fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kMessage);
		break;

	}

}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

bool tBaseVictoryState::mAtEdgeOfMap(int32 X, int32 Y)
{
	int32 GridX;
	int32 GridY;
	gScriptInterface.mGetMapDimensions( &GridX, &GridY);

	// kludge to keep a destroyed play ship from indication off the map.
	if( X == 0 && Y == 0)
		return false;

	if( (X < 2 || X > (GridX - 1)) || (Y < 2 || Y > (GridY - 1)))
	{
		return true;
	}
	else
	{
		return false;
	}
}


