/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "Uni_X3M2AGlass.h"
tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

/*

	HELPER NOTE:	The victory condition must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	// kDebNoHoleNoStar4_msg
//	"Talked to hole is false, talked to star4 is false",
	// kDebNoHoleStar4_msg,
//	"Talked to hole is false, talked to star4 is true",
	// kDebTalkedToHole_msg,
//	"Talked to hole is true",
	// kDebNoTalkToAllStars_msg,
//	"Talked to star1 is false or talked to star4 is false",
	// kDebTalkedToStars_msg
//	"Talked to star1 and star4",
	// kDebKlingon_msg,
	tBaseVictory * victory = (tBaseVictory *) fVictoryCondition;

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebTalkedToHole_msg );
		if(fRace == kKlingon)
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebKlingon_msg );
		}
		if(victory->mTalkedToStar1() && victory->mTalkedToStar4())
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebTalkedToAllStars_msg );
		}
		else
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebNoTalkToStars_msg );
		}
		break;
	case kDevastatingDefeat:
		if(victory->mTalkedToStar4())
			fMissionInfo->mAddToDebriefingList( fTeam, kDebNoHoleStar4_msg );
		else
			fMissionInfo->mAddToDebriefingList( fTeam, kDebNoHoleNoStar4_msg );
		break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );			break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tPlayerTeam::mCreateShipsForTeam()
{
/*
	mCreateShip(	typeid( tPlayerShip ),
					mGetCombatBPV(),
					kClassHeavyCruiser,
					0,
					kStartPosition_G,
					kPlayerShip1 );
*/
}

/*

	HELPER NOTE:	This is where you want to set the player ship as player controlled.  Also, you have
					the ability to setup state callback events here such as:

						for( ; *ship != NULL; ++ship )
						{
							ship->mSetShipCallbackEvents( kNotifyDamage );
						}

*/
void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
	fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kStartFed_msg  + mGetRace(), kNonCommScriptMessage);

}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 shipScriptDescription.fMetaDatabaseID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;
	
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 1000;
	fMissionScheduler.fNextMissionTitle = "Uni_X3M3AStar.scr";
}