/********************************************************************************************************\

	Mission:	<MISSION TITLE>
	Script:		"Uni_X3M2AGlass"
	Descript:	<DESCRIPTION>

	Created by:		Blomenkamp
	Last Updated:	DJS						09/26/00
					Meta script modifications


\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "Uni_X3M2AGlass.h"


DEFINE_SCRIPT_CLASS( tUni_X3M2AGlass );

tUni_X3M2AGlass::tUni_X3M2AGlass()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tUni_X3M2AGlass::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyHex, kAnyRaceHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecPlayableRace & ~kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player NPC Team",
												static_cast< eTeamID >( kPlayerNPCTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tPlayerNPCTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_X3M2AGlass::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_X3M2AGlass::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_X3M2AGlass::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_X3M2AGlass::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_X3M2AGlass::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,
		//Nme  = kEnemyTeam,
		//NPC1 = kNPC1Team,

		//	     Plyr  PNPC  
		/*Plyr*/ "--", "NI",
		/*PNPC*/ "NI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);

}

/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
void tUni_X3M2AGlass::mInitializeDraw( const int32 MapPicked )
{
	// gDraw.mDefineDrawObjects();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_X3M2AGlass::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['V'] = kObjectFissure;
	Map->fTacticalMapIcon['W'] = kObjectFissure;
	Map->fTacticalMapIcon['X'] = kObjectFissure;
	Map->fTacticalMapIcon['Y'] = kObjectFissure;
	Map->fTacticalMapIcon['Z'] = kObjectBlackHole;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_X3M2AGlass::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFed_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] = kBriefingRomulan_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefingLyran_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kBriefingGorn_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefingHydran_msg;
	fBriefingIndex[kPlayerTeam][kMirak] = kBriefingMirak_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_X3M2AGlass::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
}