/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "VictoryCondition.h"
#include "Uni_X3M3AStar.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::CreateAlliedShip()
{
	char* pszShipClass;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	eRaceName playerRace = playerTeam->mGetRace();

	switch (playerRace)
	{
		case kFederation:
			pszShipClass = "F-CC+";
			break;
		case kKlingon:
			pszShipClass = "K-D7D";
			break;
		case kHydran:
			pszShipClass = "H-LB";
			break;
		case kGorn:
			pszShipClass = "G-CC+";
			break;
	};

	mCreateShip(typeid(tPlayerShip), pszShipClass, kStartPosition_G, kNoMetaShipID, kAlliedShip,
				0, 0, -1, -1, Messages[kAlliedShipName].c_str(), kDefaultShipOptions );
}

/*

	HELPER NOTE:	The victory condition must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
		case kAstoundingVictory:
			// This ought *never* happen.
			assert(0);
			fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;

		case kVictory:
		{
			tPlayerTeamBaseState* pState = static_cast<tPlayerTeamBaseState*>(fCurState);
			assert(NULL != pState);
			eVictoryChoice vc = pState->GetVictoryChoice();
			eRaceName playerRace = mGetRace();
			int32 iCustomMsg;

			switch (vc)
			{
				case kChooseYouth:
					iCustomMsg = kVictoryYouth_msg;
					break;
				case kChooseKnowledge:
					if (playerRace == kKlingon)
						iCustomMsg = kVictoryYouthKlingon_msg;
					else
						iCustomMsg = kVictoryKnowledge_msg;
					break;
				case kChoosePrestige:
					iCustomMsg = kVictoryPrestige_msg;
					break;
				case kChooseVictory:
					iCustomMsg = kVictoryVictory_msg;
					break;
			}

			fMissionInfo->mAddToDebriefingList( fTeam, iCustomMsg );			
			fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );
			break;
		}

		case kDraw:
			fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
		case kDefeat:
			fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
		case kDevastatingDefeat:
			fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
		case kUnknownVictoryLevel:
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );			break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );

	int32 iStartMsg;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	eRaceName playerRace = playerTeam->mGetRace();

	switch (playerRace)
	{
		case kFederation:
			iStartMsg = kFederation_start_msg;
			break;
		case kKlingon:
			iStartMsg = kKlingon_start_msg;
			break;
		case kHydran:
			iStartMsg = kHydran_start_msg;
			break;
		case kGorn:
			iStartMsg = kGorn_start_msg;
			break;
	};

	fMissionInfo->mDisplayMessage(static_cast<eTeamID>(kPlayerTeam), iStartMsg, kNonCommScriptMessage);
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tPlayerTeam::mCreateShipsForTeam()
{}

/*

	HELPER NOTE:	This is where you want to set the player ship as player controlled.  Also, you have
					the ability to setup state callback events here such as:

						for( ; *ship != NULL; ++ship )
						{
							ship->mSetShipCallbackEvents( kNotifyDamage );
						}

*/
void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	static int32 shipNumber = kPlayerShip1;
	static iTruth fCreatedAlly;

	//	Create the ship according to the description given by the engine.

	mCreateShip( typeid(tPlayerShip), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 shipScriptDescription.fMetaDatabaseID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	//	Increment the ship number for each ship that comes through here on the player team

	shipNumber++;

	if (!fCreatedAlly)
	{
		CreateAlliedShip();
		fCreatedAlly = kTrue;
	}
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 1000;
	fMissionScheduler.fNextMissionTitle = "Uni_3IronCage.scr";
}