/********************************************************************************************************\

	Mission:	Wishing Upon A Star
	Script:		Uni_X3M3AStar
	Descript:	?

	Created by:		John Williston		?
	Last Updated:	DJS					09/06/00
					Added tactical icon code and kPrimaryTeam
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerTeam.h"
#include "NPC1Team.h"
#include "Uni_X3M3AStar.h"


DEFINE_SCRIPT_CLASS( tUni_X3M3AStar );

tUni_X3M3AStar::tUni_X3M3AStar()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tUni_X3M3AStar::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kFederation_briefing_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyHex, kAnyRaceHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecGorn | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"NPC 1 Team",
												static_cast< eTeamID >( kNPC1Team ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tNPC1Team ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_X3M3AStar::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_X3M3AStar::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_X3M3AStar::mInitializeFinished( void )
{
	int32 iBriefingMsg;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	eRaceName playerRace = playerTeam->mGetRace();

	switch (playerRace)
	{
		case kFederation:
			iBriefingMsg = kFederation_briefing_msg;
			break;
		case kKlingon:
			iBriefingMsg = kKlingon_briefing_msg;
			break;
		case kHydran:
			iBriefingMsg = kHydran_briefing_msg;
			break;
		case kGorn:
			iBriefingMsg = kGorn_briefing_msg;
			break;
	};

	return iBriefingMsg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_X3M3AStar::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_X3M3AStar::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kNPC1Team,
	};

	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Nme  = kEnemyTeam,
		//NPC1 = kNPC1Team,

		//	     Plyr  Nme   NPC1
		/*Plyr*/ "--", "WD", "FI",
		/*Nme */ "WD", "--", "FI",
		/*NPC1*/ "FI", "FI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_X3M3AStar::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['I'] = kObjectFissure;
	Map->fTacticalMapIcon['J'] = kObjectFissure;
	Map->fTacticalMapIcon['K'] = kObjectFissure;
	Map->fTacticalMapIcon['L'] = kObjectFissure;

	Map->fTacticalMapIcon['M'] = kObjectShipEnd;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_X3M3AStar::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kFederation_briefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kKlingon_briefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kGorn_briefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kHydran_briefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_X3M3AStar::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
}