/********************************************************************************************************\

	Mission:	Time Crisis
	Script:		"Uni_XTimeCrisis"
	Descript:	Prevent the enemy from changing the events of the past

	Created by:		Scott Bruno		08/24/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_XTimeCrisis.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "TimeholeTeam.h"

DEFINE_SCRIPT_CLASS( tUni_XTimeCrisis );

tUni_XTimeCrisis::tUni_XTimeCrisis()
{}

/*

	HELPER NOTE:	This function should return whom you consider to be the primary enemy.  Perhaps useful
					in multiplayer scripts, but single player missions do not seem to actively use it.

*/
int32 tUni_XTimeCrisis::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tUni_XTimeCrisis::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fEnemyRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecOneShip; //kMissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 6;
	Specs->fMenuStates = kNoMenuButtons;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( kAllBand );

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												1.0f,
												kFalse,
												kPlayableTeam,
												1.0f,
												tStrength(	1, 1,
															kClassFrigate,
															kClassFrigate,
															0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlayerTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												tStrength(	1, 1,
															kClassFrigate,
															kClassFrigate,
															0, 300 ),
												kSpecAnyRace,
												kDifferentThan | kPlayerTeam,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Timehole Team",
												static_cast< eTeamID >( kTimeholeTeam ),
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												tStrength(	1, 1,
															kClassFrigate,
															kClassFrigate,
															0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tTimeholeTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_XTimeCrisis::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_XTimeCrisis::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_XTimeCrisis::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_XTimeCrisis::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tUni_XTimeCrisis::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_XTimeCrisis::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_XTimeCrisis::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kTimeholeTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Nme  = kEnemyTeam,
		//Tmhl = kTimeholeTeam,

		//	     Plyr  Nme   Tmhl
		/*Plyr*/ "--", "WD", "NI",
		/*Nme */ "WD", "--", "NI",
		/*Tmhl*/ "PI", "PI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
