//	<Name>	DeepScanTracker.cpp
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#include "DeepScanTracker.h"
#include "MissionInfo.h"
#include "ShipInfo.h"
#include "sfcAssert.h"

tDeepScanTracker::tDeepScanTracker(void) :
	fMissionInfo( NULL ),
	fShip( NULL ),
	fCurrentSlotInRange( kMaxSlots ),
	fLastResult( kOutOfRange_ScanOff )
{
	int32 i = kMaxSlots;
	while( i-- )
	{
		fSlots[i] = kEmptySlot;
	}
}

// must be called first
void tDeepScanTracker::mSetTimer( tMissionInfo* missionInfo, tShipInfo* ship, eTimerIntervalTypes timerMode, int32 timerID, int32 secondsInterval)
{
	fMissionInfo = missionInfo;
	fShip = ship;
	fTimerID = timerID;
	fTimerMode = timerMode;
	fSecondsInterval = secondsInterval;
	//This does not work yet
	Assert( kFalse );
	//fMissionInfo->mSetTimedInterval( fTimerMode, fTimerID, fSecondsInterval );
}

// returns kTrue if object added to list
iTruth tDeepScanTracker::mSetTarget( tShipInfo* Object, float Distance, int32 TurnDuration )
{
	int32 i = kMaxSlots;
	while( i-- )
	{
		if( fSlots[i] == kEmptySlot )
		{
			fSlots[i] = kObjectInSlot;
			fObjects[i] = Object;
			fDistance[i] = Distance;
			fDuration[i] = TurnDuration;
			return kTrue;
		}
	}
	return kFalse;
}

// returns kTrue if location added to list
// GridX and GridY are 1 based
iTruth tDeepScanTracker::mSetLocation( int32 GridX, int32 GridY, float Distance, int32 TurnDuration )
{
	int32 i = kMaxSlots;
	while( i-- )
	{
		if( fSlots[i] == kEmptySlot )
		{
			fSlots[i] = kLocationInSlot;
			fLocations[i].x = GridX;
			fLocations[i].y = GridY;
			fDistance[i] = Distance;
			fDuration[i] = TurnDuration;
			return kTrue;
		}
	}
	return kFalse;
}

tShipInfo* tDeepScanTracker::mTestForDeepScan( eDeepScanFlags* Flags, tMapPoint* MapPoint )
{
	tShipInfo* Object;
	
	// test to scan the current target
	if (fLastResult == kOutOfRange_ScanOff
		|| fLastResult == kOutOfRange_ScanOn
		|| fLastResult == kInRange_ScanOff)
	{
		Object = fMissionInfo->mGetShipTarget( fShip );
		if( Object )
		{
			int32 i = kMaxSlots;
			float distance;
			while( i-- )
			{
				if (fSlots[i] == kObjectInSlot
					&& fObjects[i] == Object)
				{
					distance = fShip->mGetTacticalDistance( fObjects[i] );
					if( distance <= fDistance[i] )
					{
						fCurrentSlotInRange = i;
						break;
					}
					else	// reset polling timer and wait for target to come into range
					{
						//This does not work yet
						Assert( kFalse );
						//fMissionInfo->mSetTimedInterval( fTimerMode, fTimerID, fSecondsInterval );
						*Flags = fLastResult;
						return NULL;
					}
				}
			}
		}
	}

	// test to find a new object in range
	if( fCurrentSlotInRange == kMaxSlots )
	{
		int32 i = kMaxSlots;
		while( i-- )
		{
			if( fSlots[i] != kEmptySlot )
			{
				float distance;
				if( fSlots[i] == kObjectInSlot )
				{
					distance = fShip->mGetTacticalDistance( fObjects[i] );
				}
				else
				{
					distance = fShip->mGetTacticalDistance( fLocations[i].x, fLocations[i].y );
				}
				if( distance <= fDistance[i] )
				{
					fCurrentSlotInRange = i;
					break;
				}
			}
		}
	}

	// determine which result to return
	Object = NULL;
	if( fCurrentSlotInRange < kMaxSlots )
	{
		// test for player leaving objects range
		float distance;
		if( fSlots[fCurrentSlotInRange] == kObjectInSlot )
		{
			// return pointer to object
			Object = fObjects[fCurrentSlotInRange];
			distance = fShip->mGetTacticalDistance( Object );
		}
		else
		{
			distance = fShip->mGetTacticalDistance (fLocations[fCurrentSlotInRange].x,
																   fLocations[fCurrentSlotInRange].y);
			if( MapPoint )
			{
				*MapPoint = fLocations[fCurrentSlotInRange];
			}
		}
		if( distance > fDistance[fCurrentSlotInRange] + 0.5 )
		{
			// turn off count down timer
			fCurrentSlotInRange = kMaxSlots;
			//This does not work yet
			Assert( kFalse );
			//fMissionInfo->mSetTimedInterval( kTimerCountdownSet, fTimerID, 0);
			*Flags =( fShip->mGetDeepScanStatus() ) ? kLeavingRange_ScanOn_TimerStopped : kLeavingRange_ScanOff;
		}
		else	// object in range
		{
			// test for entering range
			if (fLastResult == kOutOfRange_ScanOn
				|| fLastResult == kOutOfRange_ScanOff
				|| fLastResult == kLeavingRange_ScanOn_TimerStopped
				|| fLastResult == kLeavingRange_ScanOff)
			{
				*Flags =( fShip->mGetDeepScanStatus() ) ? kEnteringRange_ScanOn_TimerStarted : kEnteringRange_ScanOff;
			}
			else	// still in range
			{
				*Flags =( fShip->mGetDeepScanStatus() ) ? kInRange_ScanOn : kInRange_ScanOff;

				// start timer if turning scan on
				if( *Flags == kInRange_ScanOn &&
					(fLastResult == kInRange_ScanOff ||
					 fLastResult == kInRange_ScanOff_TimerStopped ||
					 fLastResult == kEnteringRange_ScanOff ) )
				{
					*Flags = kInRange_ScanOn_TimerStarted;
				}
				// stop timer if turning scan off
				if( *Flags == kInRange_ScanOff &&
					(fLastResult == kInRange_ScanOn || 
					 fLastResult == kInRange_ScanOn_TimerStarted || 
					 fLastResult == kEnteringRange_ScanOn_TimerStarted) )
				{
					*Flags = kInRange_ScanOff_TimerStopped;
				}
			}

			// turn timer on if entering range with scan on or already in range with scan on
			if (*Flags == kEnteringRange_ScanOn_TimerStarted
				|| *Flags == kInRange_ScanOn_TimerStarted)
			{
				//This does not work yet
				Assert( kFalse );
				//fMissionInfo->mSetTimedInterval( kTimerCountdownSet, fTimerID, fDuration[fCurrentSlotInRange] );
			}

			// turn timer off if leaving range or turning scan off
			if (*Flags == kLeavingRange_ScanOn_TimerStopped
				|| *Flags == kInRange_ScanOff_TimerStopped)
			{
				//This does not work yet
				Assert( kFalse );
				//fMissionInfo->mSetTimedInterval( kTimerCountdownSet, fTimerID, 0 );
			}
		}
	}
	else	// object is still out of range
	{
		*Flags =( fShip->mGetDeepScanStatus() ) ? kOutOfRange_ScanOn : kOutOfRange_ScanOff;
	}

	// reset polling timer
	//This does not work yet
	Assert( kFalse );
	//fMissionInfo->mSetTimedInterval( fTimerMode, fTimerID, fSecondsInterval );
	fLastResult = *Flags;
	return Object;
}

// call with NULL to remove all objects
void tDeepScanTracker::mRemoveTarget( tShipInfo* Object )
{
	int32 i = kMaxSlots;
	while( i-- )
	{
		if (Object == NULL
			||( fSlots[i] == kObjectInSlot && fObjects[i] == Object) )
		{
			fSlots[i] = kEmptySlot;
			if( i == fCurrentSlotInRange )
			{
				fCurrentSlotInRange = kMaxSlots;
				fLastResult = kOutOfRange_ScanOff;
			}
		}
	}
}

// GridX and GridY are 1 based
void tDeepScanTracker::mRemoveLocation( int32 GridX, int32 GridY )
{
	int32 i = kMaxSlots;
	while( i-- )
	{
		if (fSlots[i] == kLocationInSlot
			&& fLocations[i].x == GridX
			&& fLocations[i].y == GridY)
		{
			fSlots[i] = kEmptySlot;
			if( i == fCurrentSlotInRange )
			{
				fCurrentSlotInRange = kMaxSlots;
				fLastResult = kOutOfRange_ScanOff;
			}
		}
	}
}


// 4Nov99	zdrummond	Added Comment Header and Footer's