//	<Name>	DrawCanvas.cpp
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#include "ScriptInterface.h"
#pragma optimize  ( "g", off)
#pragma optimize  ( "p", off)



/*
	Purpose:	Constructor for the tDrawCanvas object.  This object allows the scripter to display primitives.

	Inputs:		N/A
	Outputs:	N/A
*/
tDrawCanvas::tDrawCanvas( void )
{
	for ( int32 i = 0; i < kScriptMaxDrawObjects; i++ )
	{
		fExists[i] = kFalse;
	}
}

/*
	Purpose:	Destructor for the tDrawCanvasObject.

	Inputs:		N/A
	Outputs:	N/A
*/
tDrawCanvas::~tDrawCanvas( void )
{
	for ( int i = 0; i < kScriptMaxDrawObjects; i++ )
	{
		mKillScriptPrimitive( i );
	}
}

/*
	Purpose:	Displays the drawing object associated with the ID.

	Inputs:		( int32 )				-	The ID associated with the primitive to be drawn.
	Outputs:	N/A
*/
void
tDrawCanvas::mDisplayDrawObject( int32 ID )
{
	if ( ( ID >= 0 && ID < kScriptMaxDrawObjects ) && fExists[ID] )
	{
		mDisplayScriptPrimitive( ID );
	}
}

/*
	Purpose:	Erases the drawing object associated with the ID.

	Inputs:		( int32 )	-	The ID associated with the primitive to be drawn.
	Outputs:	N/A
*/
void
tDrawCanvas::mEraseDrawObject( int32 ID )
{
	if ( ( ID >= 0 && ID < kScriptMaxDrawObjects ) && fExists[ID] )
		mEraseScriptPrimitive( ID );
}

/*
	Purpose:	Predefined points that make up a square on the XZ-plane.

	Inputs:		( tScript3DPoint[] )	-	The array that will contain the points of the draw object
				( float )				-	The red component
				( float )				-	The green component
				( float )				-	The blue component
	Outputs:	N/A
*/
int32
tDrawCanvas::mPredefinedSquare(	tScript3DPoint	Save[],
								float			Red,
								float			Green,
								float			Blue )
{
	const int32 kNumPoints = 4;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		1.0f,	1.0f,	0.0f,	Red,	Green,	Blue	),
		tScript3DPoint(		1.0f,	-1.0f,	0.0f,	Red,	Green,	Blue	),
		tScript3DPoint(		-1.0f,	-1.0f,	0.0f,	Red,	Green,	Blue	),
		tScript3DPoint(		-1.0f,	1.0f,	0.0f,	Red,	Green,	Blue	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
		Save[i] = localPoints[i];

	return kNumPoints;
}

/*
	Purpose:	Predefined points that make up a triangle on the XZ-plane.

	Inputs:		( tScript3DPoint[] )	-	The array that will contain the points of the draw object
				( float )				-	The red component
				( float )				-	The green component
				( float )				-	The blue component
	Outputs:	N/A
*/
int32
tDrawCanvas::mPredefinedTriangle(	tScript3DPoint	Save[],
									float			Red,
									float			Green,
									float			Blue )
{
	const int32 kNumPoints = 3;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		0.5f,	0.0f,	0.0f,	Red,	Green,	Blue	),
		tScript3DPoint(		-0.5f,	-0.5f,	0.0f,	Red,	Green,	Blue	),
		tScript3DPoint(		-0.5f,	0.5f,	0.0f,	Red,	Green,	Blue	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
		Save[i] = localPoints[i];

	return kNumPoints;
}

/*
	Purpose:	Predefined points that make up a line on the XZ-plane.

	Inputs:		( tScript3DPoint[] )	-	The array that will contain the points of the draw object
				( float )				-	The red component
				( float )				-	The green component
				( float )				-	The blue component
	Outputs:	N/A
*/
int32
tDrawCanvas::mPredefinedLine(	tScript3DPoint	Save[],
								float			Red,
								float			Green,
								float			Blue )
{
	const int32 kNumPoints = 2;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		0.5f,	0.0f,	0.0f,	Red,	Green,	Blue	),
		tScript3DPoint(		-0.5f,	0.0f,	0.0f,	Red,	Green,	Blue	),
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
		Save[i] = localPoints[i];

	return kNumPoints;
}

/*
	Purpose:	Predefined point that make up a point on the XZ-plane.

	Inputs:		( tScript3DPoint[] )	-	The array that will contain the points of the draw object
				( float )				-	The red component
				( float )				-	The green component
				( float )				-	The blue component
	Outputs:	N/A
*/
int32
tDrawCanvas::mPredefinedPoint(	tScript3DPoint	Save[],
								float			Red,
								float			Green,
								float			Blue )
{
	const int32 kNumPoints = 1;
	tScript3DPoint localPoints[kNumPoints] =
	{
		tScript3DPoint(		0.0f,	0.0f,	0.0f,	Red,	Green,	Blue	)
	};
	for ( int32 i = 0; i < kNumPoints; i++ )
		Save[i] = localPoints[i];

	return kNumPoints;
}

/*
	Purpose:	Sets the drawing objects into a structure recognizable by the source.

	Inputs:		( int32 )					-	The ID associated with the primitive to create
				( tScript3DPrimitive* )		-	Address of the memory to hold the primitive info
				( tScript3DPoint )			-	List of points describing the primitive
				( int32 )					-	Number of points
				( tMapPoint )				-	Anchor position in 2D space of the ASCII map
				( eScriptPrimitiveFlags )	-	Describes how to draw the primitive
	Outputs:	N/A
*/
void
tDrawCanvas::mCreateDrawObject(	int32					ID,
								tScript3DPrimitive*		Primitive,
								tScript3DPoint			Points[kScriptMaxPoints],
								int32					NumPoints,
								tMapPoint				Anchor,
								eScriptPrimitiveFlags	Options )
{
	if ( ID >= 0 && ID < kScriptMaxDrawObjects )
	{
		if ( !fExists[ID] )
		{
			for ( int32 i = 0; i < NumPoints; i++ )
			{
				Primitive[ID].fVertexList[i] = Points[i];
			}
			Primitive[ID].fNumVertices = NumPoints;
			Primitive[ID].fAnchor.x = Anchor.x - 1;
			Primitive[ID].fAnchor.y = Anchor.y - 1;
			Primitive[ID].fFlags = Options;
			
			fExists[ID] = kTrue;

			//
			//	Let the source create this primitive.
			//
			mCreateScriptPrimitive( ID, Primitive[ID] );
		}
	}
}

/*
	Purpose:	Destroys a draw object.  This function should be called for each draw object at the end of
				the script.

	Inputs:		( int32 )	-	The ID that is associated with this primitive
	Outputs:	N/A
*/
void
tDrawCanvas::mDestroyDrawObject( int32 ID )
{
	if ( fExists[ID] )
	{
		mKillScriptPrimitive( ID );
		fExists[ID] = kFalse;
	}
}

/*
	Purpose:	Renders a primitive according to an ID.

	Inputs:		( int32 )	-	The ID that is associated with this primitive
	Outputs:	N/A
*/
void
tDrawCanvas::mDisplayScriptPrimitive( int32 ID )
{
	tScript3DPrimitive tempPrimitive;
	gScriptInterface.mPrimitive( ID, tempPrimitive, kDisplayPrimitive );
}

/*
	Purpose:	Erases a primitive according to an ID.

	Inputs:		( int32 )	-	The ID that is associated with this primitive
	Outputs:	N/A
*/
void
tDrawCanvas::mEraseScriptPrimitive( int32 ID )
{
	tScript3DPrimitive tempPrimitive;
	gScriptInterface.mPrimitive( ID, tempPrimitive, kErasePrimitive );
}

/*
	Purpose:	Creates a primitive given the structure used for rendering.  This must be called from each
				client's script if any of the other functions in this file are to function as intended.  This
				function is called from "tDrawCanvas::mCreateDrawObject()".

	Inputs:		( int32 )				-	The ID that will be associated with this primitive
				( tScript3DPrimitive )	-	A structure containing the info about the polygon to be drawn
	Outputs:	N/A
*/
void
tDrawCanvas::mCreateScriptPrimitive(	int32					ID,
										tScript3DPrimitive		Primitive )
{
	gScriptInterface.mPrimitive( ID, Primitive, kCreatePrimitive );
}

/*
	Purpose:	Destroys a primitive according to an ID.

	Inputs:		( int32 )	-	The ID that is associated with this primitive
	Outputs:	N/A
*/
void
tDrawCanvas::mKillScriptPrimitive( int32 ID )
{
	tScript3DPrimitive tempPrimitive;
	gScriptInterface.mPrimitive( ID, tempPrimitive, kKillPrimitive );
}

//	26Jul00		dsuleski		Created.
//	27Jul00		dsuleski		Updated for Dumas code in source.