//	<Name>	DrawCanvas.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//
#pragma once

#if !defined( DrawCanvas_h )
#define DrawCanvas_h

#include "scriptstructs.h"
#include "tmissionmap.h"


class tDrawCanvas
{
public:

	//
	//	The maximum number of points that a drawing object can contain.
	//	The maximum number of drawing objects allowed per script.
	//
	enum
	{
		kScriptMaxPoints = 1 << 7,
		kScriptMaxDrawObjects = 1 << 5,
	};

public:

	//
	//	Constructor for initialization.
	//
	tDrawCanvas( void );
	~tDrawCanvas( void );

public:

	//
	//	Displays and erases the drawing object associated with the ID.
	//
	void	mEraseDrawObject( int32 ID );
	void	mDisplayDrawObject( int32 ID );

protected:

	//
	//	Predefined points.
	//
	virtual int32	mPredefinedSquare( tScript3DPoint Save[], float Red = 1.0f, float Green = 1.0f, float Blue = 1.0f );
	virtual int32	mPredefinedTriangle( tScript3DPoint Save[], float Red = 1.0f, float Green = 1.0f, float Blue = 1.0f );
	virtual int32	mPredefinedLine( tScript3DPoint Save[], float Red = 1.0f, float Green = 1.0f, float Blue = 1.0f );
	virtual int32	mPredefinedPoint( tScript3DPoint Save[], float Red = 1.0f, float Green = 1.0f, float Blue = 1.0f );

	//
	//	Sets the drawing objects into a structure recognizable by the source.  These are protected to prevent
	//	a script from creating additional draw objects after a script has started.
	//
	void	mCreateDrawObject( int32 ID, tScript3DPrimitive* Primitive, tScript3DPoint Points[kScriptMaxPoints], int32 NumPoints, tMapPoint Anchor, eScriptPrimitiveFlags Options = kDefaultScriptPrimitiveFlags );
	void	mDestroyDrawObject( int32 ID );

protected:

	//
	//	Sends over to the source.
	//
	void	mDisplayScriptPrimitive( int32 ID );
	void	mEraseScriptPrimitive( int32 ID );
	void	mCreateScriptPrimitive( int32 ID, tScript3DPrimitive Primitive );
	void	mKillScriptPrimitive( int32 ID );

protected:

	iTruth	fExists[kScriptMaxDrawObjects];
};

#endif


//	26Jul00		dsuleski		Created.
//	28Jul00		dsuleski		DrawCanvas object now dynamically allocates memory for vertices.