//	<Name>	DynamicAllocator.cpp
//
//
//	<Info>	DESCRIPTION
//

#include <BaseTypes.h>
#include "DynamicAllocator.h"
#include <superassert.h>

#if !defined( USE_PCH )

#include <Algorithm>

#endif


tAllocator::~tAllocator()
{_func_start( tAllocator::~tAllocator );_func_end;}

tAllocatedClass::tAllocatedClass() :
	fAllocID( kInvalidAllocatorID )
{_func_start( tAllocatedClass::tAllocatedClass );_func_end;}
	
void tAllocatedClass::mSetAllocatorID( iAllocatorID allocID )
{
	_func_start( tAllocatedClass::mSetAllocatorID );
	Assert( fAllocID == kInvalidAllocatorID );
	
	fAllocID = allocID;
	_func_end;
}

tAllocatedClass::~tAllocatedClass()
{_func_start( tAllocatedClass::~tAllocatedClass );_func_end;}

iAllocatorID tAllocatedClass::mGetAllocatorID() const
{
	_func_start( tAllocatedClass::mGetAllocatorID );
	return fAllocID;
	_func_end;
}

int32 tAllocatedClass::operator==( const type_info& type )
{
	_func_start( tAllocatedClass::operator == );
	int32 valid = ( typeid( *this ) == type );
	if( !valid )
	{
		valid = typeid( this ) == type;
	}

	return valid;
	_func_end;
}

int32 tDynamicAllocator::tAllocatorInfo::operator<( const tAllocatorInfo& allocInfo ) const
{
	_func_start( tDynamicAllocator::tAllocatorInfo::operator < );
	if( fTypeID )
	{
		if( allocInfo.fTypeID )
		{
			//If we are before or equal to alloc then return true
			if( strcmp( fTypeID->raw_name(), allocInfo.fTypeID->raw_name() ) <= 0 )
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		else
		{
			return true;
		}
	}
	
	return false;
	_func_end;
}

int32 tDynamicAllocator::tAllocatorInfo::operator==( const type_info& type ) const
{
	_func_start( tDynamicAllocator::tAllocatorInfo::operator == );
	Assert( fTypeID );
	if( fTypeID )
	{
		return *fTypeID == type;
	}
	else
	{
		return false;
	}
	_func_end;
}

int32 tDynamicAllocator::tAllocatorInfo::operator==( const char* raw_name ) const
{
	_func_start( tDynamicAllocator::tAllocatorInfo::operator == );
	Assert( fTypeID );
	if( fTypeID )
	{
		if( strcmp( fTypeID->raw_name(), raw_name ) == 0 )
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else
	{
		return false;
	}
	_func_end;
}


tDynamicAllocator* tDynamicAllocator::fpDynamicAllocator = NULL;
tCleanupGlobalPointer< tDynamicAllocator > tDynamicAllocator::fCleanupSingletion( &tDynamicAllocator::fpDynamicAllocator );

tDynamicAllocator* tDynamicAllocator::Instance()
{
	_func_start( tDynamicAllocator::Instance );
	if( fpDynamicAllocator == NULL )
	{
		fpDynamicAllocator = new tDynamicAllocator; 
	}

	return fpDynamicAllocator; 
	_func_end;
}

tDynamicAllocator::tDynamicAllocator() :
	fAllocatorList( NULL ),
	fAllocatorListLocked( false )	
{_func_start( tDynamicAllocator::tDynamicAllocator );_func_end;}

tDynamicAllocator::~tDynamicAllocator()
{
	_func_start( tDynamicAllocator::~tDynamicAllocator );
	if( fAllocatorList )
	{
		delete fAllocatorList;
	}
	_func_end;
}

void tDynamicAllocator::LockDownAllocator()
{
	_func_start( tDynamicAllocator::LockDownAllocator );
	if( fAllocatorListLocked )
	{
		return;
	}

	fAllocatorListLocked = true;

	for( int32u i = 0; i < fAllocatorList->size(); i++ )
	{
		(*fAllocatorList)[ i ].fAllocator->mAssignAllocID( i );
	}
	_func_end;
}

void tDynamicAllocator::mAddToAllocList( const type_info& type, tAllocator* alloc )
{
	_func_start( tDynamicAllocator::mAddToAllocList );
	Assert( fAllocatorListLocked == false );
	if( fAllocatorListLocked )
	{
		return;
	}

	if( fAllocatorList == NULL )
	{
		fAllocatorList = new tAllocatorList;	
	}
	
	tAllocatorInfo info;
	info.fTypeID = &type;
	info.fAllocator = alloc;

	fAllocatorList->push_back( info );
	std::sort( fAllocatorList->begin(), fAllocatorList->end() );
	_func_end;
}

tAllocatedClass* tDynamicAllocator::mAllocateClass( const type_info& typeToAlloc )
{
	_func_start( tDynamicAllocator::mAllocateClass );
	iAllocatorID id = mGetAllocatorID( typeToAlloc );

	return mAllocateClass( id );
	_func_end;
}

tAllocatedClass* tDynamicAllocator::mAllocateClass( iAllocatorID typeAllocID )
{
	_func_start( tDynamicAllocator::mAllocateClass );
	LockDownAllocator();

	int32 maxAllocSize = static_cast<int32>( fAllocatorList->size() );
	if( 0 <= typeAllocID && typeAllocID < maxAllocSize )
	{
		const tAllocatorInfo& allocatorInfo = fAllocatorList->at( typeAllocID );
		return allocatorInfo.fAllocator->mAllocate();				
	}
	else
	{
		Assert( false );
		return NULL;
	}
	_func_end;
}

iAllocatorID tDynamicAllocator::mGetAllocatorID( const type_info& type )
{
	_func_start( tDynamicAllocator::mGetAllocatorID );
	LockDownAllocator();

	if( fAllocatorList )
	{
		for( int32u i = 0; i < fAllocatorList->size(); i++ )
		{
			const tAllocatorInfo& allocatorInfo = fAllocatorList->at( i );
			
			if( allocatorInfo == type )
			{
				return i;
			}
		}
	}

	return kInvalidAllocatorID;
	_func_end;
}

iAllocatorID tDynamicAllocator::mGetAllocatorID( const char* raw_name )
{
	_func_start( tDynamicAllocator::mGetAllocatorID );
	LockDownAllocator();

	if( fAllocatorList )
	{
		for( int32u i = 0; i < fAllocatorList->size(); i++ )
		{
			const tAllocatorInfo& allocatorInfo = fAllocatorList->at( i );
			
			if( allocatorInfo == raw_name )
			{
				return i;
			}
		}
	}

	return kInvalidAllocatorID;
	_func_end;
}


const type_info* tDynamicAllocator::mGetType( iAllocatorID allocID )
{
	_func_start( tDynamicAllocator::mGetType );
	LockDownAllocator();

	int32 maxAllocSize = static_cast<int32u>( fAllocatorList->size() );
	if( 0 <= allocID && allocID < maxAllocSize )
	{
		const tAllocatorInfo& allocatorInfo = fAllocatorList->at( allocID );

		return allocatorInfo.fTypeID;
	}
	else
	{
		Assert( false );
		return NULL;
	}
		
	_func_end;
}

//	16Dec99	zdrummond	Added Comment Header and Footer's
//	12Jun00	dsuleski	Changed some returns types of bool to int32 because of performance warning
//	8Jul00	dsuleski	Documentation update
