//	<Name>	DynamicAllocator.h
//
//
//	<Info>	DESCRIPTION
//
#pragma once

#if !defined( DynamicAllocator_h )
#define DynamicAllocator_h

#if !defined( USE_PCH )

#include <typeinfo>
#include <vector>

#endif

#include <CleanUpGlobalPointer.h>

typedef int32 iAllocatorID;
const int8 kInvalidAllocatorID = -1;

class tAllocatedClass
{
public:
	tAllocatedClass();
	virtual ~tAllocatedClass();
	
	void mSetAllocatorID( iAllocatorID allocID );
	virtual iAllocatorID mGetAllocatorID() const;
	
	template< class tChildClass >
	bool mCastTo( tChildClass** childClass ); 

	virtual int32 operator==( const type_info& type );

protected:
	iAllocatorID fAllocID;
};

template< class tChildClass >
bool tAllocatedClass::mCastTo( tChildClass** childClass )
{
	_func_start( tAllocatedClass::mCastTo );
	*childClass = dynamic_cast<tChildClass>( this );
	if( *childClass )
	{
		return true;
	}
	else
	{
		return false;
	}
	_func_end;
}

class tAllocator
{
public:
	virtual tAllocatedClass* mAllocate() = 0;

protected:
	virtual ~tAllocator();

	friend class tDynamicAllocator;
	virtual void mAssignAllocID( iAllocatorID allocID ) = 0;
};

class tDynamicAllocator
{
public:
//Setup
	static tDynamicAllocator* Instance();
	// Once this function is called, no more classes can be added to 
	// the allocation list. This is to preserver the integrity of the 
	// Allocator ID's
	void LockDownAllocator();


	void mAddToAllocList( const type_info& type, tAllocator* alloc );
	tAllocatedClass* mAllocateClass( const type_info& typeToAlloc );
	tAllocatedClass* mAllocateClass( iAllocatorID typeAllocID );

	//Asking for an ID, automaticaly locks down the allocator
	iAllocatorID mGetAllocatorID( const type_info& type );
	iAllocatorID mGetAllocatorID( const char* raw_name );
	template< typename tnClass >
	static iAllocatorID mGetAllocatorID();

	const type_info* mGetType( iAllocatorID allocID );

private:
//Types
	class tAllocatorInfo
	{
	public:
		int32	operator<( const tAllocatorInfo& allocInfo ) const;
		int32	operator==( const type_info& type ) const;
		int32	operator==( const char* raw_name ) const;

		const type_info* fTypeID;
		tAllocator* fAllocator;
	};

	typedef std::vector< tAllocatorInfo > tAllocatorList;

//Setup
	//Singleton
	tDynamicAllocator();
	~tDynamicAllocator();

	
//Fields
	friend tCleanupGlobalPointer< tDynamicAllocator >;
	static tCleanupGlobalPointer< tDynamicAllocator > fCleanupSingletion;
	static tDynamicAllocator* fpDynamicAllocator;
	
	bool fAllocatorListLocked;
	tAllocatorList* fAllocatorList;

};



template< class tClass >
class tClassAllocator : public tAllocator 
{
public:
	tClassAllocator();

	virtual tAllocatedClass* mAllocate();
	static iAllocatorID mGetAllocatorID();

protected:
	virtual void mAssignAllocID( iAllocatorID allocID );

	static iAllocatorID fAllocID;
};


template< class tClass >
iAllocatorID tClassAllocator<tClass>::fAllocID = kInvalidAllocatorID;

template< class tClass >
tClassAllocator<tClass>::tClassAllocator<tClass>()
{
	_func_start( tClassAllocator::tClassAllocator );
	tDynamicAllocator::Instance()->mAddToAllocList( typeid( tClass ), this );
	_func_end;
}

template< class tClass >
tAllocatedClass* tClassAllocator<tClass>::mAllocate()
{
	_func_start( tClassAllocator::mAllocate );
	tClass* pClass = new tClass();
	
	pClass->mSetAllocatorID( fAllocID );
	
	return pClass; 
	_func_end;
}

template< class tClass >
iAllocatorID tClassAllocator<tClass>::mGetAllocatorID()
{
	_func_start( tClassAllocator::mGetAllocatorID );
	return fAllocID;
	_func_end;
}

template< class tClass >
void tClassAllocator<tClass>::mAssignAllocID( iAllocatorID allocID )
{
	_func_start( tClassAllocator::mAssignAllocID );
	fAllocID = allocID;
	_func_end;
}

//Global Allocator ID function
template< class tClass >
iAllocatorID GetAllocID()
{
	_func_start( GetAllocID );
	return tClassAllocator<tClass>::mGetAllocatorID();
	_func_end;
}


//Used to force VC to link in the class, otherwise it might optmize it out
#define USE_CLASS( UseClassType, AllocName )\
	{\
		extern tClassAllocator<UseClassType> AllocName;\
		int8u* hold = (int8u*)&AllocName;\
	}



#endif


//	16Dec99	zdrummond	Added Comment Header and Footer's
//	12Jun00	dsuleski	Changed some returns types of bool to int32 because of performance warning
//	8Jul00	dsuleski	Documentation update
