//	<Name>	ScriptInterface.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#ifndef scriptinterface_h
#define scriptinterface_h

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the SCRIPTDLL_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// SCRIPTDLL_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef SCRIPTDLL_EXPORTS
#define SCRIPTDLL_API __declspec(dllexport)
#else
#define SCRIPTDLL_API __declspec(dllimport)
#endif


#include "windows.h"
#include "math.h"
#include "tbasic.h"
#include "soundnums.h"
#include "scriptstructs.h"
#include "scriptentry.h"
#include "tmissionmap.h"
#include "ShipInfo.h"
#include "TeamInfo.h"
#include "MissionInfo.h"
#include "Script.h"


extern tScript* AllocNewScript();
extern SCRIPTDLL_API int kMaxUnits;
extern SCRIPTDLL_API int32 TotalMaps;
extern SCRIPTDLL_API char* MapTypes[];
extern SCRIPTDLL_API char* MapLines[];
extern SCRIPTDLL_API char* Messages[];



#define DEFINE_SCRIPT_CLASS( tScriptClass )		\
	tScript* AllocNewScript()					\
	{											\
		return &gScript;						\
	}

//-------------------------------------------------------
// max range constants
const int32 kUniqueMission = 32767;
const int32 kCaptureBonus = 30;		// prestige bonus for capturing ships
const int32 kTechnicalBonus = 40;	// 1-time prestige bonus for completing technical

enum eMedalsAwarded
{
	kMissionMedal_1 = 1,
	kMissionMedal_2,
	kMissionMedal_3,
	kMissionMedal_4,
	kSpecialMedal_1,
	kSpecialMedal_2,
	kSpecialMedal_3,
	kSpecialMedal_4,
};

class tVictoryCondition;
class tScript;
class tTeamScriptDescription;
class tShipScriptDescription;

//class tScriptInterface : public tScriptEntry
// This class is exported from the ScriptInterface.dll
class SCRIPTDLL_API tScriptInterface : public tScriptEntry
{
	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:

	tScriptInterface( void );
	~tScriptInterface( void );

	virtual void				mCreateScript( char* ScriptName );
	virtual void				mInitialize ( tDynaverseScriptInfo& Info );
#if 1	///DANHM -- this block was changed to reflect host migration
	virtual void				mScriptStartMission( void );
	virtual void				mSetNewHost( void );
#endif
	virtual iTruth				mSaveEntityID( eTeamID Team, int32 CallbackID, int32 EntityID, iAllocatorID allocID );
	virtual void				mEntityEvent( tShipSystemsInfo& Info );
	virtual void				mInitMapForRead( void );
	virtual const char*			mGetNextMapLine( void );
	virtual void				mBriefingText( char* briefing );
	virtual const tMissionSpec*	mGetScriptSpecs( char* ScriptName ); 
	virtual iTruth				mValidateVersion( float Version );
	virtual eTeamRelationship	mGetTeamRelations( eTeamID Team1, eTeamID Team2 );
	virtual void				mNewTeam( const tTeamScriptDescription& TeamScriptDescription );
	virtual void				mNewShip( const tShipScriptDescription& ShipScriptDescription );
	virtual void				mBeginMission( void );
	
	virtual void	mEntityMgrCallback (cbEntityMgr* Func);
	virtual void	mCreateShipCallback (cbCreateShip* Func);
	virtual void	mMessageCallback (cbMessages* Func);
	virtual void	mRandomCallback (cbRandom* Func);
	virtual void	mPrimitiveCallback (cbPrimitive* Func);

	void	mSetGameStatus( eTeamID TeamID, eGameStatus Status );
	void	mSetDeepScanStatus( iTruth On );

	tMissionInfo*	mMission( void ) const;
	iTruth			mIsScriptCreated( void ) const;
	tScript*		mScript( void ) const;
	std::string		mGetCampaignTitle( void );

public:

	cbCreateShip*		mCreateSpecShip;
	cbEntityMgr*		mEntityMgr;
	cbMessages*			mPopupMessage;
	cbRandom*			mRandomInt;
	cbPrimitive*		mPrimitive;

#if 0
	int32			mRandomInt( int32 lo, int32 hi );
#endif
	void			mFindMapObject (unsigned char MapObject, int32* GridX, int32* GridY);
	void			mGetMapDimensions (int32* GridX, int32* GridY);

	iTruth		fMissionBegun;

	iTruth		mIsHost( void );
	void		mSetHost( iTruth x = kTrue );

	typedef tScript*( *tNewDLLEntry )( void );
	tScript*	fScript;

protected:

	void		mFinishSetupShips( void );
	eRaceName	mChooseRace( eRaceName RaceArray[], int32 ValidRaces );
	eSystems	mChooseSystem( eSystems SystemsArray[] );
	iTruth		mInitMapForRead( int32 Map );

	int32			fBriefingIndex;
	int32			fMapLineIndex;
	int32			fMapPicked;
	tMissionSpec*	fMissionSpec;
	iTruth			fHost;
	HINSTANCE		fScriptHandle;
	
};

extern SCRIPTDLL_API tScriptInterface gScriptInterface;


#endif		// #ifndef scriptinterface_h

//	8Jul00	dsuleski	Documentation update
//	10Jul00	dsuleski	mRandomInt() is now its own function