//	<Name>	MissionInfo.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//
#pragma once

#if !defined( MissionInfo_h )
#define MissionInfo_h

#include "tbasic.h"
#include "drawcanvas.h"
#include "scriptstructs.h"
#include "tmissionmap.h"
#include "typeinfo.h"
#include <string>
#include <map>
#include "DynaverseScriptInfo.h"

enum eMulti_TeamTypes
{
	kHuman_AlliedTeams,
	kHuman_EnemyTeams,
	kCPU_AlliedTeams,
	kCPU_EnemyTeams,
	kMaxMultiTeams
};

class tTeamInfo;
class tShipInfo;
class tScript;

extern std::string Messages[];

class tMissionInfo
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:

	enum eBitRequest
	{
		kOnBits,
		kOffBits,
		kAllBits,
	};

	tMissionInfo( tScript* script );

	iTruth				mCheckForMissionEnd();

	void				mSetTagRelations( int32* TagList, const std::string* RelationTable, int32 Size );
	void				mSetTeamRelations( int32* TeamList, const std::string* RelationTable, int32 Size ); 
	void				mSetTeamRelations( eTeamID Team1, eTeamID Team2, eTeamRelationship Relations, eTargetGoals Goals, float EPVFactor, iTruth MutualRelations = kFalse );
	void				mSetTeamRelations( eMulti_TeamTypes Team1, int32 Team2, eTeamRelationship Relations, eTargetGoals Goals, float EPVFactor, iTruth MutualRelations = kFalse );

	void				mSetupTeam( int32 Team, eTeamTag Tag, eRaceName Race, iTruth HumanControlled, const type_info& teamClass = typeid( tTeamInfo ) );
	void				mSetMultiplayerTeams( tDynaverseScriptInfo& Info );
	void				mSetSkirmishTeams( const tDynaverseScriptInfo& Info );
	void				mSetPrestigePoints (int32 AstoundingVictory, int32 Victory, int32 Draw, int32 Defeat, int32 DevastatingDefeat);
	void				mSetGoalIndicator( eTeamID teamID, tShipInfo* TargetShip );
	void				mSetGoalIndicator( eTeamID teamID, iTruth IndicatorOn, int32 GridX, int32 GridY );		// Grid X/Y are 1 based
	void				mSetGoalIndicator( tShipInfo* FromShip, tShipInfo* TargetShip );
	void				mSetGoalIndicator( tShipInfo* FromShip, iTruth IndicatorOn, int32 GridX, int32 GridY );		// Grid X/Y are 1 based

	tTeamInfo*			mGetTeamHandle( eTeamID Team );
	iTruth				mGetGridPosition( unsigned char MapObject, int32* GridX, int32* GridY ) const;		// Grid XY are 1 based
	float				mGetTacticalDistance( unsigned char MapObject1, unsigned char MapObject2 ) const;
	int32				mGetPlayerRank( void ) const;
	int32				mGetEliteOrganization( void ) const;
	int32				mGetDifficultyLevel( void ) const;
	int32				mGetEra( void ) const;
	ePoliticalRelation	mGetRaceRelationship( eRaceName Race1, eRaceName Race2 ) const;
	tShipInfo*			mGetShipTarget( tShipInfo* Ship ) const;
	int32				mGetINIFileInt( int32 HeaderID, int32 LabelID ) const;
	void				mGetMapDimensions( int32* GridX, int32* GridY ) const;
	iTruth				mGetHexType( tMetaLocationSpec& Hex );
	
	int32				mRandomInt32( int32 Lo, int32 Hi ) const;
	int32				mGetRandomBitValue( int32 String, int32 NumBits, eBitRequest BitRequest = kAllBits );
	void				mDisplayMessage( eTeamID MsgForTeam, int32 MsgID, eScriptMessageTypes Type = kMissionScriptMessage, tShipInfo* Ship = NULL );
	void				mDisplayMessage( eTeamID MsgForTeam, const char* Msg, eScriptMessageTypes Type = kMissionScriptMessage, tShipInfo* Ship = NULL );
	void				mDisplayMessage( eTeamID MsgForTeam, std::string Msg, eScriptMessageTypes Type = kMissionScriptMessage, tShipInfo* Ship = NULL );
	void				mReplaceTokens( char* Msg );
	void				mAddToDebriefingList( eTeamID teamID, int32 DebriefingIndex );
	void				mAddToDebriefingList( eTeamID teamID, const char* Debriefing );
	void				mAddToDebriefingList( eTeamID teamID, std::string Debriefing );
	void				mCreateMapObject( unsigned char MapObject, int32 GridX, int32 GridY );		// Grid XY are 1 based
	void				mPlaySound( int32 SoundID );
	void				mPlayVoiceSample( int32 VoiceID, iTruth StopPreviousVoice = kTrue );
	int32				mVoiceSampleStillPlaying( void );
	void				mAddToCareerTotals( eClassTypes Type, eCareerEvents Event, int32 Amount );
	tShipInfo*			mFindShipID( int32 ShipID ) const;
	void				mScheduleMission( int32 MsgID);			// file name of mission to schedule (including extension )
	void				mStartFlashingButton( const char* ButtonID );
	void				mStopFlashingButton( const char* ButtonID );

	void				mAddMapIcon( eMapObjectType Type, int32 X, int32 Y, int32 SubType = 0 );
	iTruth				mGetCampaignType( void );

	void				mAdjustDynaverse( eDynaverseTableType Type, int32 Modifier, eRaceName Race );

	int32				fTotalAlliedHumanTeams;
	int32				fTotalEnemyHumanTeams;
	int32				fTotalAlliedCPUTeams;
	int32				fTotalEnemyCPUTeams;

	eGameStatus			fStatusArray[kMaxTeams];

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:

	void		mDebugMessage( const char* Message );
	void		mSetDate( const tDynaverseScriptInfo::tMissionDate& Date );
	void		mSetLocation( eSectorName Location );
	void		mSetSystemName( eSystems SystemName );
	iTruth		mAllTeamsReady( void );
	
	eGameStatus			mGetGameStatus( eTeamID ID );
	void				mRemoveShipAsTarget( tShipInfo* TargetShip );
	eTeamRelationship	mGetTeamRelations( eTeamID Team1, eTeamID Team2 ) const;
	void				mResetDebriefingCount( eTeamID TeamID );
	iTruth				mIsTeamLeaving( eTeamID Team );
	void				mInvokeGameStatus( iTruth UponHostChange = kFalse, eTeamID TeamID = kNoTeam, iTruth PlayerLeaving = kFalse, iTruth AmIHost = kFalse );



	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:

	tMissionInfo( void );
	~tMissionInfo( void );

protected:


	typedef std::map<char, int32> tCharLookup;
	
	static void mSetupRelationshipLookup();
	
	static iTruth		fLookupInitialized;
	static tCharLookup	fRelationshipLookup;
	static tCharLookup	fGoalLookup;
	
	tTeamInfo*	fTeams[kMaxTeams];
	eRaceName	fRaces[kMaxTeams];

	eSectorName		fLocation;
	eSystems		fSystemName;

	tDynaverseScriptInfo::tMissionDate	fDate;

	tScript* fScript;

	iTruth fMissionEnd;
};

#endif


//	4Nov99	zdrummond	Added Comment Header and Footer's
//	06Jun00	dsuleski	mGetGameStatus() now takes a team ID
//	8Jul00	dsuleski	Documentation update
//	25Jul00	dsuleski	Drawing primitive functionality