//	<Name>	Script.cpp
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//

#pragma warning( disable:4800 )
#include "Script.h"
#include "sfcAssert.h"
#include "TeamInfo.h"

/*
	Purpose:	Constructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tScript::tScript( void ) :
	fThisScriptIsTheHost( kFalse ),
	fReadyToStartMission( kFalse ),
	fMissionBegun( kFalse )
{
	fMissionInfo = new tMissionInfo( this );

	for ( int32 i = 0; i < kMaxTeams; i++ )
	{
		for ( int32 j = 0; j < kNumActiveRaces; j++ )
		{
			fBriefingIndex[i][j] = kNoBriefing;
			fBriefing[i][j] = "";
		}
	}
}

/*
	Purpose:	Destructor.

	Inputs:		N/A
	Outputs:	N/A
*/
tScript::~tScript( void )
{
	if( fMissionInfo )
	{
		delete fMissionInfo;
	}
}

/*
	Purpose:	

	Inputs:		
	Outputs:	
*/
void
tScript::mCreateExternalShips( int32 Mode )
{}

/*
	Purpose:	Will determine how all starting positions show up on the tactical map.  Will also assign
				models for #0 - #9 and their icons on the tactical map.  The default for the starting
				positions is that they will not show up at all on the tactical map.  The default for #0 - #9
				is that they will show up as planets on the tactical map and will be assigned "PL1" - "PL10".

	Inputs:		( tMapOverrides* )	-	Structure that holds the info on the meaning of the ASCII map
	Outputs:	N/A
*/
void
tScript::mASCIIOverrides( tMapOverrides* Map )
{}

/*
	Purpose:	Accessor for the current mission.

	Inputs:		N/A
	Outputs:	( tMissionInfo* )	-	The requested mission
*/
tMissionInfo*
tScript::mMission( void ) const
{
	Assert( fMissionInfo ); 
	return fMissionInfo;
}

/*
	Purpose:	Evaluates whether or not the local script is hosting.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the local script is hosting;
								kFalse, otherwise
*/
iTruth
tScript::mHosting( void ) const
{
	return fThisScriptIsTheHost;
}

/*
	Purpose:	Called when the mission is ready to start.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScript::mReadyToStartMission( void )
{
	fReadyToStartMission = kTrue;

	if( mIsMissionReadyToStart() )
	{
		mStartMission();
	}
}

/*
	Purpose:	Determines whether or not a mission is ready to start.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the mission is ready; kFalse, otherwise
*/
iTruth
tScript::mIsMissionReadyToStart( void )
{
	if( fReadyToStartMission && fMissionInfo->mAllTeamsReady() )
	{
		return kTrue;
	}
	else
	{
		return kFalse;
	}
}

/*
	Purpose:	Initialization for a script.  Please note that this function must be called
				first if it is overridden in a script.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScript::mStartMission( void )
{
	//
	//	Tell the even manager that the fScript is ready for events
	//
	gEventMgr.mRegisterActor( this );

	//
	//	Finish team setup, after the ships in the game, and the teams are ready
	//
	for( int32 teamIndex = 0; teamIndex < kMaxTeams; teamIndex++ )
	{
		tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( teamIndex ) );
		if( team )
		{
			team->mTeamSetupComplete();			
		}
	}

	fMissionBegun = kTrue;
}

/*
	Purpose:	Evaluates whether a mission is under way.

	Inputs:		N/A
	Outputs:	( iTruth )	-	kTrue if the mission has begun; kFalse, otherwise
*/
iTruth
tScript::mHasMissionBegun( void )
{
	return fMissionBegun;
}

/*
	Purpose:	Initializes script data that was not known during its constructor.

	Inputs:		( iTruth )			-	kTrue if the local script is the host; kFalse, otherwise
				( tMissionData& )	-	Constant reference to a structure containing the date
				( eSectorName )		-	One of several constants that describe the current location
				( eSystems )		-	One of several constants that describe the current system

	Outputs:	N/A
*/
void
tScript::mSetupScript(	iTruth										ThisScriptHosting,
						eTeamID										PlayerTeam,
						const tDynaverseScriptInfo::tMissionDate&	MissionDate,
						eSectorName									MissionLocation,
						eSystems									SystemName )
{
	fThisScriptIsTheHost = ThisScriptHosting;

	Assert( fMissionInfo ); 
	fMissionInfo->mSetDate( MissionDate );
	fMissionInfo->mSetLocation( MissionLocation );
	fMissionInfo->mSetSystemName( SystemName );
}

/*
	Purpose:	The traits of a new team must be determined in this way.

	Inputs:		( string& )			-	The name of the team
				( eTeamID )			-	The ID to be assigned to the new team
				( float )			-	Ranges from 0.0 to 1.0; 1.0 means that
										the team is always available to play
				( iTruth )			-	kTrue means that the team must be available;
										kFalse, otherwise
				( eTeamType )		-	Can be kNPCTeam, kPlayableTeam, etc
				( float )			-	Ranges from 0.0 to 1.0; 1.0 means that
										the team will always be taken over by AI
										if a human did not take the team first
				( tStrength& )		-	A class that contains information on what
										types of ships and how many ships exist
										on this team by default
				( iMissSpecRace )	-	One of several constants that determine
										which races are valid for this team
				( iRaceCriteria )	-	One of several constants that determine
										which race should be chosen for this team
				( type_info& )		-	Constant reference to a "typeid()"
				( eTeamTag )		-	A default tag used for multiplayer
				( string )			-	The title that the Meta will display for this team
				( string )			-	An array of race-specific briefings that the Meta will display
	Outputs:	( tTeamSpec )		-	A class that contains the above information
*/
tTeamSpec
tScript::mCreateTeamSpec(	const std::string&			TeamName, 
							eTeamID						TeamID, 
							float						ChanceTeamIsAvailable, 
							iTruth						Requried, 
							eTeamType					TeamType, 
							float						ChanceTeamCanBePlayedByAI, 
							iMissSpecRace				ValidRaces, 
							iRaceCriteria				RaceCriteria, 
							const type_info&			TeamAllocType,
							eTeamTag					TeamTag,
							const std::string&			TeamMissionTitle ) 
{								
	iAllocatorID allocID = tDynamicAllocator::Instance()->mGetAllocatorID( TeamAllocType );
	tTeamSpec teamSpec( TeamName,
						TeamID,
						ChanceTeamIsAvailable,
						Requried,
						TeamType,
						ChanceTeamCanBePlayedByAI,
						tStrength( mGetTeamShipStrengths( TeamID ) ),
						ValidRaces,
						RaceCriteria,
						allocID,
						TeamTag,
						TeamMissionTitle,
						mGetBriefingList( TeamID ) );

	return teamSpec;
}

/*
	Purpose:	When a new team is created, it goes through something like this.  However, if the
				teams are being created off of team tags, rather than their IDs, then it important
				to override this with the correct team class.

	Inputs:		( tTeamScriptDescription& )	-	Constant reference of a data structure that contains
												trait information on the team in question
	Outputs:	N/A
*/
void
tScript::mNewTeam( const tTeamScriptDescription& TeamScriptDescription )
{
	const type_info* teamClass = tDynamicAllocator::Instance()->mGetType( TeamScriptDescription.fTeamActorID );
	mMission()->mSetupTeam( TeamScriptDescription.fTeam, TeamScriptDescription.fTeamTag, TeamScriptDescription.fRace, TeamScriptDescription.fHumanControlled, *teamClass );
}

/*
	Purpose:	Functionality passed to the script so that primitives can be created and associated
				with the draw object.

	Inputs:		( int32 )	-	An index that describes the map chosen
	Outputs:	N/A
*/
void
tScript::mInitializeDraw( const int32 MapPicked )
{}

/*
	Purpose:	Functionality passed to the script so that primitives can be created and associated
				with the draw object.

	Inputs:		( int32 )	-	An index that describes the map chosen
	Outputs:	N/A
*/
tMapTerrain
tScript::mAssignMapTerrainTypes( int32 Map )
{
	return tMapTerrain( kTerrainAnyTerrain, kPlanetAnyPlanet, kBaseAnyBase );
}

/*
	Purpose:	Initializes the index into the mission text that each race briefing points.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScript::mSetBriefings( void )
{
	//
	//	Copy over the mission text briefings from the individual script to the parent script.
	//
	for ( int32 i = 0; i < kMaxTeams; i++ )
	{
		for ( int32 j = 0; j < kNumActiveRaces; j++ )
		{
			if ( fBriefingIndex[i][j] != kNoBriefing )
			{
				fBriefing[i][j] = Messages[fBriefingIndex[i][j]];
			}
		}
	}
}

/*
	Purpose:	Returns the list of race briefings for a particular team.

	Inputs:		( int32 )	-	The ID of the team in question
	Outputs:	( string* )	-	The list of strings
*/
std::string*
tScript::mGetBriefingList( int32 teamID )
{
	if ( teamID > kNoTeam && teamID < kMaxTeams )
		return fBriefing[teamID];
	return NULL;
}

/*
	Purpose:	The script should override this function for meta missions in each individual script.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScript::mDefineBriefings( void )
{}

/*
	Purpose:	Returns the list of ship strengths for a particular team.

	Inputs:		( int32 )	-	The ID of the team in question
	Outputs:	( string* )	-	The list of strings
*/
std::vector< tShipStrength >
tScript::mGetTeamShipStrengths( int32 teamID )
{
	if ( teamID > kNoTeam && teamID < kMaxTeams )
		return fTeamShipStrengths[teamID];
	return NULL;
}

/*
	Purpose:	The script should override this function to for meta missions in each individual script.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScript::mDefineTeamShipStrengths( void )
{}

/*
	Purpose:	Sets up the ship strength of a particular team.  This function takes a min and max BPV range.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScript::mSetTeamShipStrength(	int32		TeamID,
								eClassTypes	MinClass,
								eClassTypes	MaxClass,
								int32		MinBPV,
								int32		MaxBPV )
{
	fTeamShipStrengths[TeamID].push_back( tShipStrength( MinClass, MaxClass, MinBPV, MaxBPV ) );
}

/*
	Purpose:	Sets up the ship strength of a particular team.  This function takes a min and max ratio range
				as compared with kPrimaryTeam.

	Inputs:		N/A
	Outputs:	N/A
*/
void
tScript::mSetTeamShipStrength(	int32		TeamID,
								eClassTypes	MinClass,
								eClassTypes	MaxClass,
								float		MinRatio,
								float		MaxRatio )
{
	fTeamShipStrengths[TeamID].push_back( tShipStrength( MinClass, MaxClass, MinRatio, MaxRatio ) );
}
//	04Nov99	zdrummond	Added Comment Header and Footer's
//	19Apr00	dsuleski	Changed mCreateTeamSpec(...) to include meta ship creation flag
//	27Apr00	dsuleski	Changed name of script ship functions (mCreateMissionShips and mNewMissionShip)
//	11Jun00	dsuleski	mNewTeam() added for tag teams in multiplayer
//	08Jul00	dsuleski	Documentation update
//	09Oct00	dsuleski	Briefing list helper functions added
//	10Oct00	dsuleski	Team ship strengths helper functions added