//	<Name>	Script.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//
#pragma once

#if !defined( Script_h )
#define Script_h

#include "tbasic.h"
#include "scriptstructs.h"
#include "tmissionmap.h"
#include "MissionInfo.h"
#include "Actor.h"
#include "VictoryCondition.h"
#include "DynaverseScriptInfo.h"
#include "Race.h"

#if 0

#pragma message( "Script.h: Warning! This is legacy info, it needs to be removed!" )
			//  Bit masks for number of ships
#define MissSpecOneShip			1
#define MissSpecTwoShip			2
#define MissSpecThreeShip		4
#define MissSpecAllShips		( MissSpecOneShip | MissSpecTwoShip | MissSpecThreeShip)

enum eMissSpecRace { kFedMission    =   1,	 kKlingMission     =   2,	    kRomulanMission		=   4,
					 kLyranMission  =   8,	 kHydranMission    =  16,	    kGornMission		=  32,
					 kTholianMission=	64,  kISCMission	   =  128,		kMirakMission		= 256,
					 kLDRMission	=	512,	 kWYNMission   =  1024,		kJindarianMission	= 2048,
					 kOrionMission  =  4096, kNonPirateMission =  4095,		kAllRaceMission		= 8191,
					 kEliteRaceMission = 8192
					};
#endif

class tShipInfo;

class tScript : public tActor
{
	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:
	
	tScript( void );
	virtual ~tScript( void );

	virtual int32			mInitializeStart( tDynaverseScriptInfo& Info ) = 0;	
	virtual void			mNewInternalShip( tShipInfo* Ship ) = 0;
	virtual void			mScriptSpecs( tMissionSpec* Specs ) = 0;

	virtual void			mCreateExternalShips( int32 Mode );
	virtual void			mASCIIOverrides( tMapOverrides* Map );
	virtual void			mNewTeam( const tTeamScriptDescription& TeamScriptDescription );
	virtual void			mInitializeDraw( const int32 MapPicked );
	virtual tMapTerrain		mAssignMapTerrainTypes( int32 Map );
	virtual void			mDefineBriefings( void );
	virtual void			mDefineTeamShipStrengths( void );
	
	//
	//	Accessors
	//
	virtual tMissionInfo*	mMission() const;
	virtual iTruth			mHosting() const;

	virtual void	mReadyToStartMission( void );
	virtual iTruth	mIsMissionReadyToStart( void );
	virtual iTruth	mHasMissionBegun( void );

	void			mSetBriefings( void );
	std::string*	mGetBriefingList( int32 TeamID );
	std::vector< tShipStrength >	mGetTeamShipStrengths( int32 TeamID );
	void	mSetTeamShipStrength( int32 TeamID, eClassTypes MinClass, eClassTypes MaxClass, int32 MinBPV = kMinBPV, int32 MaxBPV = kMaxBPV );
	void	mSetTeamShipStrength( int32 TeamID, eClassTypes MinClass, eClassTypes MaxClass, float MinRatio = 0.8f, float MaxRatio = 1.2f );

	//
	//	If you override this, you MUST call it at the beginning of the overridden function body
	//
	virtual void mStartMission( void );

protected:
	friend class tScriptInterface;
	void mSetupScript( iTruth ThisScriptHosting, eTeamID PlayerTeam, const tDynaverseScriptInfo::tMissionDate& MissionDate, eSectorName MissionLocation, eSystems SystemName );

	enum { kNoBriefing = -1, };
	tTeamSpec mCreateTeamSpec(	const std::string& TeamName,
								eTeamID TeamID,
								float ChanceTeamIsAvailable,
								iTruth Requried,
								eTeamType TeamType,
								float ChanceTeamCanBePlayedByAI,
								iMissSpecRace ValidRaces,
								iRaceCriteria RaceCriteria,
								const type_info& TeamAllocType = typeid( tTeamInfo ),
								eTeamTag TeamTag = kNoTeamTag,
								const std::string&	TeamMissionTitle = "Mystery?"
								);

	iTruth		fThisScriptIsTheHost;
	iTruth		fReadyToStartMission;
	iTruth		fMissionBegun;

	int32		fBriefingIndex[kMaxTeams][kNumActiveRaces];
	std::string	fBriefing[kMaxTeams][kNumActiveRaces];

	std::vector< tShipStrength > fTeamShipStrengths[kMaxTeams];
};

#endif


//	04Nov99	zdrummond	Added Comment Header and Footer's
//	08Jul00	dsuleski	Documentation update
//	03Oct00	dsuleski	Added mAssignMapTerrainTypes()