//	<Name>	ShipInfo.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//
#pragma once

#if !defined( ShipInfo_h )
#define ShipInfo_h

#include "tbasic.h"
#include "scriptstructs.h"
#include "tmissionmap.h"
#include "actor.h"
#include "DynamicAllocator.h"
#include "SysBaseEnums.h"

//-------------------------------------------------------
// values to use with mSetShipStatus and mTestShipStatus
enum eShipStatus
{
	kUndefined					= 1 << 0,
	kShipCreated				= 1 << 1,
	kShipAtFinalDestination		= 1 << 2,
	kShipCaptured				= 1 << 3,
	kShipDisengaged				= 1 << 4,
	kShipDestroyed				= 1 << 5,
	kShipInternalsTaken			= 1 << 6,
};

//
//	must match enum eDisengageType in aiShipCommands.h
//
enum eDisengageMethods
{
	kByAcceleration,
	kByMapEdge,
};


//
//	values to use with mSetWeaponState
//
enum eWeaponStates
{
	kWeaponState_None,
	kWeaponState_Phasers,
	kWeaponState_All,

	kNumWeaponStates,

	kWeaponState_Default = kWeaponState_None,
};

class tShipInfo	: public tActor
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:

	virtual	void	mSetupState( void );
	virtual void	mSetupCommState( void );
	virtual void	mSetupShip( void );

	void	mSetShipCallbackEvents( iNotifyEvents NewEvents );
	void	mSetShipThrust( int32 NewThrust );
	void	mSetShipMaxSpeed( int32 MaxSpeed );
	void	mSetShipImmediateSpeed( int32 ImmediateSpeed );
	int32	mGetShipSpeed( void );
	void	mSetShipPlayerControlled( void );
	void	mSetShipAIControlled( iTruth AIControlled );
	void	mSetFinalDestination( int32 EndX = -1, int32 EndY = -1 );
	void	mTrackTacticalDistance( float Distance, int32 GridX, int32 GridY, iTruth NotifyInOut = kFalse );	// Grid XY are 1 based
	void	mDestroyShip( void );
	void	mDestroyShipSilently( void );
	void	mChangeName( int32 MsgID );

	//
	//	Systems access calls
	//	No feedback requests (Setting values)
	//
	void	mSetSystemHealthFromOrig( eSystemHardPoint System, float Percentage, iTruth AllowHeal = kFalse );
	void	mSetSystemHealthFromCurr( eSystemHardPoint System, float Percentage, iTruth RoundUp = kFalse );
	void	mDamageRandomInternals( float percentage );
	void	mDamageRandomInternals( int numHits );

	//
	//	Functions requiring feedback requests 
	//	meaning that these functions will send a feedback msg
	//
	void	mGetHealthOfSystem( eSystemHardPoint System );
	void	mGetMaxHealthOfSystem( eSystemHardPoint System );
	
	//
	//	Sets trigger when a trigger is tripped, a feedback msg will be sent
	//	If the Orlower flag is true then the msg will trip on that level or  lower, if not, it will trip on that level or higher
	//	Note: each eSystemHardPoint system can only have one trigger placed on it.  Any subsequent trigger request cancels the previous request.
	//
	void	mGetPercentAtMarkOriginal_Trigger( eSystemHardPoint System, float AtThisMark, iTruth OrLower = kTrue );

	//
	//	BPV values
	//
	int32	mGetBPV( void );
	int32	mGetTotalBPV( void );

	//
	//	Clears trigger, no feedback
	//
	void	mClearPreviousTriggers( eSystemHardPoint System );

	//
	//	Supply access calls
	//	No feedback requests (Setting values)
	//
	void	mRemoveItemFromStores( eAccessShipStores_ItemTypes Item, int32 Amount );
	void	mRemoveItemFromStores( eScriptTransportItems TransporterItem, int32 Amount );
	void	mAddItemToStores( eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes Subtype, int32 Amount );
	void	mAddItemToStores( eAccessShipStores_ItemTypes Item, eScriptTransportItems TransporterItem, int32 Amount);
	void	mSetItemInStores( eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes Subtype, int32 Amount );				
	void	mSetItemInStores( eAccessShipStores_ItemTypes Item, eScriptTransportItems TransporterItem, int32 Amount);
	void	mSetShipLoadout( int32 Marines = -1, int32 SpareParts = -1, int32 Mines = -1, int32 Shuttles = -1, int32 FighterBays = -1, int32 FightersPerBay = -1, std::string FighterSpec = "", int32 MissileTypeIStandard = -1, int32 MissileTypeIMedium = -1, int32 MissileTypeIFast = -1, int32 MissileTypeIVStandard = -1, int32 MissileTypeIVMedium = -1, int32 MissileTypeIVFast = -1 );
	void	mSetShuttles( int32 Shuttles );
	void	mSetFighters( int32 FighterBays, int32 FightersPerBay, std::string FighterSpec = "" );


	//
	//	Functions requiring feedback requests 
	//	meaning that these functions will send a feedback msg
	//
	void	mGetAmountOfStore( eAccessShipStores_ItemTypes Item );
	void	mGetAmountOfStore( eScriptTransportItems TransporterItem );
	void	mGetMaxAmountOfStore( eAccessShipStores_ItemTypes Item );
	void	mGetMaxAmountOfStore( eScriptTransportItems Item );

	//
	//	Sets trigger when a trigger is tripped, a feedback msg will be sent
	//	If the Orlower flag is true then the msg will trip on that level or  lower, if not, it will trip on that level or higher
	//	Note: each eSystemHardPoint system can only have one trigger placed on it.  Any subsequent trigger request cancels the previous request.
	//
	void	mGetAmountAtMark_Trigger( eAccessShipStores_ItemTypes Item, float AtThisMark, iTruth OrLower = kTrue);

	//
	//	Clears trigger, no feedback
	//
	void	mClearPreviousTriggers( eAccessShipStores_ItemTypes Item );

	
	eRaceName		mGetRace( void );
	eClassTypes		mGetClassType( void );
	const char*		mGetClassName( void );
	int32			mGetStartPositionIndex( void );
	int32			mGetCustomID( void );
	const char*		mGetCustomName( void );
	int32			mGetStartX( void );
	int32			mGetStartY( void );
	int32			mGetEndX( void );
	int32			mGetEndY( void );
	float			mGetTrackingDistance( void );
	int32			mAssessDamage( eRaceName EnemyRace );
	void			mGetGridPosition( int32* GridX, int32* GridY );		// Grid XY are 1 based
	float			mGetTacticalDistance( tShipInfo* TargetShip );
	float			mGetTacticalDistance( unsigned char MapObject );
	float			mGetTacticalDistance( int32 GridX, int32 GridY );	// Grid XY are 1 based
	eAlertLevels	mGetAlertStatus( void );
	void			mSetAlertStatus( eAlertLevels Status );
	float			mGetDamagePercent( void );
	eOfficerRank	mGetOfficerRank( eOfficerTypes Officer );
	void			mSetOfficerRank( eOfficerTypes Officer, eOfficerRank Rank );
	void			mSetOfficerName( eOfficerTypes Officer, int32 MsgID );
	void			mSetTargetable( iTruth State );
	void			mSetDeepScanStatus( iTruth state );
	void			mSetCustomID( int32 x );
	void			mSetCustomName( const char* Name );
	iTruth			mGetDeepScanStatus( void ) const;

	//
	//	AI specific commands
	//
	void			mSetShipTarget( tShipInfo* TargetShip, eAIPriority TargetPriority );
	void			mRemoveTargets( void );
	void			mClearCommands( void );
	void			mDisengage( eDisengageMethods Method, eAIPriority Priority );
	void			mPatrol( float DetectionRange, int32 PatrolPointCount, tMapPoint* PatrolPoints, eAIPriority Priority );
	void			mChangeAIOfficerState( eAIOfficers Officer, iTruth State );
	void			mCloak( iTruth State );
	void			mTractorEntity( tShipInfo* Entity, int32 EndX, int32 EndY, eAIPriority Priority ); // End XY are 1 based
	void			mDefendEntity( tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority );
	void			mFollowEntity( tShipInfo* Entity, eAIPriority Priority );
	void			mFireProbe( tShipInfo* Target, eAIPriority Priority );
	void			mSetWeaponState( eWeaponStates weaponState );

	void			mSetCommButton( eTeamID TeamID, int32 MsgID, iTruth SetAsTarget = kFalse, int32 ButtonNumber = -1 );
	void			mClearCommButton( eTeamID TeamID, int32 ButtonNumber );
	void			mClearCommMsg( eTeamID TeamID, int32 MsgID );
	void			mClearAllCommButtons( void );

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:

	void		mSetShipStatus( eShipStatus Status, iTruth SetState = kTrue );
	void		mSetShipID( int32 ShipID );
	void		mSetShipBPV( int32 ShipBPV );
	void		mSetClassType( eClassTypes ClassType );
	int32		mGetShipTargetID( void );
	void		mReplaceShipCallbackEvents( iNotifyEvents NewEvents );

	eTeamID			mGetTeam( void );
	eTeamID			mGetOriginalTeam( void );
	int32			mGetShipID( void );
	iTruth			mInPlay( void );
	iTruth			mTestShipStatus( int32u Status );
	int32			mGetShipBPV( void );
	iNotifyEvents	mGetShipCallbackEvents( void );
	int32			mGetCallbackID( void );
	void			mSetTeam( eTeamID NewTeam );
	eShipOptions	mGetShipOptions( void );

	//-------------------------------------------------------
	// private routines - not for script use
	//-------------------------------------------------------
public:

	tShipInfo( void );
	~tShipInfo( void );

	void mSetupShipInfo( eTeamID Team, eRaceName Race, int32 CallbackID, int32 Position, int32 CustomID, const char* CustomName, int32 StartX, int32 StartY, int32 EndX, int32 EndY, eShipOptions ShipOptions );

protected:

	eTeamID			fTeam;
	eTeamID			fOriginalTeam;
	eRaceName		fRace;
	int32			fShipID;
	eClassTypes		fClassType;
	iMapPosition	fStartPositionIndex;
	int32			fStartX;
	int32			fStartY;
	int32			fEndX;
	int32			fEndY;
	float			fTrackingDistance;
	int32u			fStatus;
	int32			fShipBPV;
	iNotifyEvents	fShipEvents;
	int32			fCallbackID;
	iTruth			fDeepScanStatus;
	int32			fCustomID;
	std::string		fCustomName;
	eShipOptions	fShipOptions;
};
typedef std::vector<tShipInfo*> tShipContainer;

class tShipEvaluator
{
public:

	virtual iTruth mEvaluate( tShipContainer::iterator ShipIter ) const = 0;
};

class tShipIterator
{
public:

	tShipIterator( tShipContainer::iterator ShipIter, tShipContainer::iterator End );

	void mAddEvaluator( tShipEvaluator* Eval ); 

	iTruth mValid();
	
	tShipIterator&	operator++(); //Prefix operator
	tShipIterator	operator++( int ); //Postfix operator
	tShipInfo*		operator*();
	tShipInfo*		operator->();
	iTruth			operator==( tShipContainer::iterator ShipIter );
	iTruth			operator!();

protected:

	typedef std::vector<tShipEvaluator*> tEvaluators;
	iTruth mEvaluate( tShipContainer::iterator Ship );
	
	tEvaluators fEvaluators;

	tShipContainer::iterator fCurShip;
	tShipContainer::iterator fEndOfShipList;
};

class tShipInPlayEval : public tShipEvaluator
{
public:

	iTruth mEvaluate( tShipContainer::iterator ShipIter ) const;
};

class tShipCustomIDEval : public tShipEvaluator
{
public:

	tShipCustomIDEval( int32 CustomID );

	virtual iTruth mEvaluate( tShipContainer::iterator ShipIter ) const;

protected:

	int32 fCustomID;
};

class tShipClassEval : public tShipEvaluator
{
public:

	tShipClassEval( const type_info& ShipClass );
	virtual	iTruth mEvaluate( tShipContainer::iterator ShipIter ) const;

protected:
	const type_info& fShipClass;
};




#endif


//	4Nov99	zdrummond	Added Comment Header and Footer's
//	29Apr00	dsuleski	Added mSetCustomID(...) for ships given to the script through the meta
//	29Jun00	dsuleski	New accessor and modifier for the ship custom name
//	8Jul00	dsuleski	Documentation update