//	<Name>	State.h
//
//	<Info>	DESCRIPTION
//
#pragma once

#if !defined( State_h )
#define State_h

#include "scriptstructs.h"
#include "EventMgr.h"
#include "Actor.h"
#include "SysBaseEnums.h"

class tShipInfo;
class tMissionInfo;
class tTeamInfo;

class tState
{
public:
	enum
	{
		kInvalidStateID = -1,
	};
	
	tState();
	
	//
	//	Events
	//
	void mHandleShipEvent( tShipInfo* Ship, const tEventMgr::tEventInfo& EventInfo );
	
	virtual void	mBeginState( void );
	virtual void	mEndState( void );

	//
	//	Ship events
	//
	virtual void	mOnDeath( tShipInfo* Ship );
	virtual void	mOnCapture( tShipInfo* Ship );
	virtual void	mOnDisengaged( tShipInfo* Ship );
	virtual void	mAtDistance( tShipInfo* Ship );
	virtual void	mAtDestination( tShipInfo* Ship );
	virtual void	mOnCommButtonPress( tShipInfo* Ship, int32 ButtonID, tShipInfo* TargetShip );
	virtual void	mOnDamage( tShipInfo* DamagedShip, tShipInfo* HostileShip, eDamageToShip DamageType );
	virtual void	mOnDeepScan( tShipInfo* Ship, iTruth DeepScanOn );
	virtual void	mOnAlertStatusChange( tShipInfo* Ship, eAlertLevels AlertStatus );
	virtual void	mOnProbeLaunch( tShipInfo* Ship );
	virtual void	mOnBeginOrbit( tShipInfo* Ship );
	virtual void	mOnEndOrbit( tShipInfo* Ship );
	virtual void	mOnBeginTractor( tShipInfo* TractorTarget, tShipInfo* TractoringShip );
	virtual void	mOnEndTractor( tShipInfo* TractorTarget, tShipInfo* TractoringShip );
	virtual void	mOnFlashingButtonPressed();
	virtual void	mOnTransportedItem( tShipInfo* ShipTo, tShipInfo* ShipFrom, eScriptTransportItems Item );
	virtual void	mOnChangingShip( tShipInfo* Ship );
	virtual void	mOnSpecialTransport( tShipInfo* Ship );
	virtual void	mOnShipSystemUpdate( tShipInfo* Ship, eShipUpdateType Type, eSystemHardPoint System, int32 Value );
	virtual void	mOnShipSystemUpdate( tShipInfo* Ship, eShipUpdateType Type, eSystemHardPoint System, float Percent );

	enum eSubItemType
	{
		kSupplySubItem,
		kTransporterSubItem,
	};

	virtual void	mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value );
	virtual void	mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eScriptTransportItems SubItem, int32 Value );
	
	//
	//	System events
	//
	virtual void	mOnPlayerLeaving( int32 PlayerNumber, eTeamID TeamID );
	virtual void	mOnHostChanged( void );
	virtual void	mOnTimedInterval( int32 TimerID );
	virtual void	mOnNewTeam( tTeamInfo* Team );
	
	enum eStateEvent
	{
		kEndingState,
		kStartingNewState,
	};

	virtual void	mOnStateChange( tActor* Actor, tState* StateID, eStateEvent EventType );
	virtual void	mOnCommStateChange( tActor* Actor, int32 State, eStateEvent EventType );
	virtual void	mOnCommStateChange( tActor* Actor, int32 State, int32 buttonID, eStateEvent EventType );

	//
	//	Operations
	//
	iTruth operator==( const type_info& StateType );
	virtual const char* mGetName( void );
	virtual iStateID mGetStateID( void );

	virtual void mSetTimedInterval( eTimerIntervalTypes Type, int32 TimerID, float Interval );

protected:

	friend class tActor;
	virtual void mSetup( tActor* Actor, iStateID StateID );

	tActor* fActor;
	iStateID fStateID; 
	tMissionInfo* fMissionInfo;
};

#endif


//	8Nov99	zdrummond	Added Comment Header and Footer's
//	8Jul00	dsuleski	Documentation update