//	<Name>	TeamInfo.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	DESCRIPTION
//
#pragma once

#if !defined( TeamInfo_h )
#define TeamInfo_h

#include <typeinfo.h>
#include "ShipInfo.h"
#include "MissionInfo.h"

#include "tbasic.h"
#include "scriptstructs.h"
#include "tmissionmap.h"
#include "DynamicAllocator.h"
#include "Medals.h"	


struct tAITargetInfo
{
	eTeamRelationship	fRelationships[kMaxTeams];
	eTargetGoals		fTargetGoals[kMaxTeams];
	float				fEPVFactors[kMaxTeams];
};

class tShipScriptDescription;
class tVictoryCondition;

class tTeamInfo	: public tActor
{
	//-------------------------------------------------------
	// public routines for normal script use
	//-------------------------------------------------------
public:
	enum
	{
		kMaxShips = 12,
	};

					tTeamInfo( void );
	virtual			~tTeamInfo( void );
	virtual void	mSetTeamInfoData( eTeamID Team, eRaceName Race, iTruth humanControlled, eTeamTag tag );
	virtual void	mSetTeamFlag( int32 Flag );

	virtual void	mSetupVictoryCondition( void );
	virtual void	mSetupState( void );
	virtual void	mSetupCommState( void );
	virtual void	mCreateShipsForTeam( void );
	
	//
	//	If you overload this, you MUST call it before you define your behavior!
	//	This functions registers the team as an actor, sets up all the ships, and 
	//	then registers them as actors
	//
	virtual void	mTeamSetupComplete();
	
	virtual void	mCreateNewShip( const tShipScriptDescription& shipScriptDescription );
	virtual void	mCreateNewShip( void );

	virtual void		mHandleShipEvent( tShipInfo* ship, const tEventMgr::tEventInfo& EventInfo );
	virtual void		mSetGameStatus( eGameStatus Status );
	virtual eGameStatus	mGetGameStatus( void );
	virtual void		mSetupDebriefingList( eVictoryLevels VictoryLevel );
								  

	eTeamID		mGetTeam( void );
	eTeamTag	mGetTag( void );
	eRaceName	mGetRace( void );
	int32		mGetShipsInPlayCount( void );
	int32		mGetTeamFlag( void );
	iTruth		mIsTeamHumanControlled( void );
	iTruth		mIsTeamSetup( void );

	//
	//	AI specific commands
	//
	void		mClearCommands( void );
	void		mDisengageShips( eDisengageMethods Method, eAIPriority Priority );
	void		mDefendEntity( tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority );

	//
	//	Start XY/End XY are 1 based
	//
	void	mCreateShip( const type_info& shipClass, const tShipInGame& ShipData);

	// These are the only CreateShip routines used so far.
	// Only the one with the metaDatabseID is compatable with the Metaverse.
	// ALL meta scripts should switch to the new function
	// A compile warning will be issued if you don't use the MetaDatabaseID version
	
	enum { kNoMetaShipID };
	void	mCreateShip( const type_info& ShipClass, const char* ClassName, int32 Position, int32 MetaDatabaseID, int32 CustomID, int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName, eShipOptions ShipOptions ); // The only one used
	void	mCreateShip( const type_info& ShipClass, int32 TotalBPV, eClassTypes ShipType, int32 VariantPercent, int32 Position, /*int32 MetaDatabaseID,*/ int32 CustomID, int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName, eShipOptions ShipOptions, eRace Race = kNoRace );

	void	mCreateShip( const type_info& ShipClass, int32 TotalBPV, eClassTypes SmallestHullSize, eClassTypes LargestHullSize, int32 VariantPercent, int32 Position, int32 CustomID, int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName, eShipOptions ShipOptions );

	
	void	mCreateFleet( const type_info& ShipClass, int32 TotalShips, int32 TotalBPV, eClassTypes	SmallestHullSize, eClassTypes LargestHullSize, int32 VariantPercent, int32 Position, int32 CustomID, int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName, eShipOptions ShipOptions, eRace Race = kNoRace );

	void	mDuplicateShip( tShipInfo* Ship, int32 Position, int32 StartX, int32 StartY, int32 EndX, int32 EndY, const char* CustomName, eShipOptions ShipOptions );
	void	mCreateCPUFleet( const type_info& ShipClass, int32 VariantPercent, int32 Position, int32 CustomID, int32 StartX, int32 StartY, int32 EndX, int32 EndY );

	tShipIterator	mGetFirstShip( void );
	tShipInfo*		mGetThisShip( int32 ShipIndex );		// return ship at given index - index is 1 based
	iTruth			mAllInPosition( void );
	int32			mGetCombatBPV( void );
	int32			mGetTotalCombatBPV( void );
	int32			mAssessDamages( tTeamInfo* EnemyTeam );
	float			mGetTotalDamagePercent( void );		// must be called before ending mission!!!
	eClassTypes		mGetMaxClassType( void );

	int32			mFindCommIndex( int32 ID, iTruth overwrite = kFalse );
	virtual void	mRegisterCommMessage(  int32 StateID, int32 NextState, int32 ShipID, int32 ButtonID, int32 MessageIDStart, int32 MessageIDEnd = kNoRandomMsg, eScriptMessageTypes MsgType = kMissionScriptMessage );
	virtual void	mGotoCommState( int32 ID, int32 StateID, int32 ButtonID = -1 );
	virtual int32	mGetCommState( int32 ID );

	//-------------------------------------------------------
	// public routines for internal script use or debugging
	//-------------------------------------------------------
public:

	tShipInfo*			mFindShipIndex( int32 CallbackID );
	tShipInfo*			mFindShipID( int32 ShipID );
	tShipInfo*			mFindShipCustomID( int32 customID );
	iTruth				mAllShipsReady( void );
	void				mAddToCombatBPV( int32 ShipBPV );
	void				mRemoveShipAsTarget( tShipInfo* TargetShip );
	iNotifyEvents		mGetTeamStatus( void );
	void				mSetRelations( eTeamID Team, eTeamRelationship Relations, eTargetGoals Goals, float EPVFactor );
	eTeamRelationship	mGetRelations( eTeamID Team );
	eTargetGoals		mGetGoal( eTeamID Team );
	float				mGetEPV( eTeamID Team );
	int32				mGetShipCount( eShipStatus Status );
	iTruth				mAddShipToTeamList( tCreateShipInfo& Info );
	void				mRemoveShipFromTeamList( tShipInfo* Ship );
	void				mSetTeamLeaving( void );
	iTruth				mIsTeamLeaving( void );
	int32				mShipsRemainingInPlay( void );
	int32				mShipsRemainingInPlayFromOriginalTeam( void );
	int32				mShipsCurrentlyControlled( void );
	void				mSetCurrentlyControlledShip( tShipInfo* Ship );
	tShipInfo*			mCurrentlyControlledShip( void );


	class tMissionScheduler
	{
	public:
		enum eCampaignEvents		// must match to tDisplayCampaignEvent
		{
			kNone		=	0 << 0,
			kRetire		=	1 << 1,
			kLost		=	1 << 2,
		};

	public:
		tMissionScheduler():	fVictoryLevel( kUnknownVictoryLevel ), fPrestige( 0 ),
								fBonusPrestige( 0 ), fNextMissionScore( 0 ),
								fMedal( kNoMedals ), fCampaignEvent( kNone )
		{
			fNextMissionTitle = "";
		}

		eVictoryLevels	fVictoryLevel;
		int32			fPrestige;
		int32			fBonusPrestige;
		int32			fNextMissionScore;
		std::string		fNextMissionTitle;
		int32			fMedal;
		eCampaignEvents	fCampaignEvent;
	};
	tMissionScheduler*	mGetNextMissionInfo( void );
	virtual void		mScheduleNextMission( tVictoryCondition* VictoryCondition );

public:

	iTruth				fHasEndedHost;
	iTruth				fHasEndedClient;
	tVictoryCondition*	fVictoryCondition;
	eVictoryLevels		fVictoryLevel;
	eGameStatus			fStatus;
	iTruth				fTeamLeaving;

protected:

	tMissionScheduler	fMissionScheduler;

	enum
	{
		kMaxCommStates = tEventMgr::kMaxActors
	};

	class tCommMessage
	{
	public:
		enum eMessageMode
		{
			kShipMode,
			kTeamMode
		};

		tCommMessage();
		tCommMessage( int32 NextState, int32 SourceTeamID, int32 ShipID, int32 ButtonNameID, int32 MessageIDStart, int32 MessageIDEnd, eScriptMessageTypes MsgType );
		tCommMessage( int32 NextState, int32 SourceTeamID, eTeamID TeamID, int32 ButtonNameID, int32 MessageIDStart, int32 MessageIDEnd, eScriptMessageTypes MsgType );

		eMessageMode fMessageMode;
		int32 fSourceTeamID;
		int32 fShipID;
		eTeamID fTeamID;
		
		int32 fButtonNameID;
		int32 fMessageIDStart;
		int32 fMessageIDEnd;
		int32 fNextState; 
		eScriptMessageTypes fMsgType;
	};
	typedef std::vector< tCommMessage > tCommMsgList;

	void	mCreateTeamShip( const type_info& ShipClass, tCreateShipInfo& Info, int32 Position, int32 CustomID, int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1, eRace Race = kNoRace );
	void	mCreateTeamFleet( const type_info& ShipClass, int32 ShipCount, tCreateShipInfo Info[], int32 Position, int32 CustomID, int32 StartX = 0, int32 StartY = 0, int32 EndX = -1, int32 EndY = -1, const char* CustomName = NULL, eShipOptions ShipOptions = kDefaultShipOptions, eRace Race = kNoRace );
	void	mDetermineStartPositionOffset( eShipOptions* Options, iMapPosition Position );

	enum eCommObjects
	{
		kMaxCommObjects = tEventMgr::kMaxActors,//1 << 4, //	32
	};

	int32			fCurCommState[kMaxCommObjects];
	tCommMsgList	fCommStates[kMaxCommStates][kMaxCommObjects];
	int32			fIDField[kMaxCommObjects];

	eTeamID			fTeam;	 
	eTeamTag		fTag; 
	eRaceName		fRace;
	int32			fNextCallbackID;
	tShipContainer	fShipList;
	iTruth			fHumanControlled; 	
	
	int32				fCombatBPV;

	tAITargetInfo		fAITargets;

	iTruth				fTeamSetup;

	//
	//	Used to search the team ship list
	//
	enum eSearchType
	{
		kNoSearch,
		kCustomIDSearch,
		kClassSearch,
	};

	tShipInfo*			fCurrentlyControlledShip;
	int32				fShipIndex;
	eSearchType			fSearchType;
	const type_info*	fShipClassType;
	int32				fCustomID;
	int32				fFlag;
};


#endif


//	4Nov99	zdrummond	Added Comment Header and Footer's
//	18Apr00	dsuleski	new mCreateNewShip(...) added to help with old API conversion
//	28Apr00	dsuleski	new mShipsRemaining() on team level counts ships in play
//	29Apr00	dsuleski	can now get the current commstate of a team
//	26May00	dsuleski	mGetFirstShipInPlay() causes crash with local evaluator -- got rid of it
//	13Jun00	dsuleski	Various enums are were changed from 32 to 64 (the new kMaxActors value)
//	15Jun00	dsuleski	Added function to remove a ship from a team list
//	16Jun00	dsuleski	Added accessor for fCurrentlyControlledShip
//	3Jul00	dsuleski	Comm messages no longer need to display messages if not requested to do so
//	8Jul00	dsuleski	Documentation update
//	25Sep00	dsuleski	Created tMissionScheduler and appropriate helper functions within tTeamInfo