//	<Name>	VictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "VictoryState.h"

/*
	Purpose:	Initializes the prestige points for each of the victory levels.

	Inputs:		( int32 )	-	The value rewarded for achieving an astounding victory
				( int32 )	-	The value rewarded for achieving a victory
				( int32 )	-	The value rewarded for achieving a draw
				( int32 )	-	The value rewarded for achieving a defeat
				( int32 )	-	The value rewarded for achieving a devastating defeat
				( int32 )	-	The value rewarded for an unknown victory level (upon forfeit)
	Outputs:	N/A
*/
tVictoryState::tVictoryState(	int32	AstoundingVictory,
								int32	Victory,
								int32	Draw,
								int32	Defeat,
								int32	DevastatingDefeat,
								int32	Unknown ) :
	fTeam( NULL ),
	fBonusPrestige( 0 )
{
	if( AstoundingVictory >= 0 )
	{ 
		fPrestigePoints[kAstoundingVictory] = AstoundingVictory;
		fPrestigePoints[kVictory] = Victory;
		fPrestigePoints[kDraw] = Draw;
		fPrestigePoints[kDefeat] = Defeat;
		fPrestigePoints[kDevastatingDefeat] = DevastatingDefeat;
	}
	fUnknownVictoryPoints = Unknown;
}

/*
	Purpose:	Since each team can have its own victory state, this associates that team with that
				victory state.

	Inputs:		( tTeamInfo* )	-	The team in question
	Outputs:	N/A
*/
void
tVictoryState::mSetTeam( tTeamInfo* Team )
{
	fTeam = Team;
}

/*
	Purpose:	Returns the value of the prestige contained in the victory level determined by 'level'.

	Inputs:		( eVictoryLevels )	-	The level of prestige points requested
	Outputs:	( int32 )			-	The amount of prestige point for the requested level
*/
int32
tVictoryState::mGetPrestigePoints( eVictoryLevels Level )
{
	return fPrestigePoints[Level]; 
}

/*
	Purpose:	An accessor for the value of the bonus prestige.

	Inputs:		N/A
	Outputs:	( int32 )	-	The amount of bonus prestige
*/
int32
tVictoryState::mGetBonusPrestige( void )
{
	return fBonusPrestige; 
}

/*
	Purpose:	Will return the prestige value associated with a special victory level.

	Inputs:		( eVictoryLevels )	-	The level of victory in question
	Outputs:	( int32 )			-	The number of prestige points associated
										with the level of victory in question
*/
int32
tVictoryState::mLookupPrestigeValue( eVictoryLevels Level )
{
	switch ( Level )
	{
	case kUnknownVictoryLevel:
		return fUnknownVictoryPoints;
	}
	return 0;
}

/*
	Purpose:	Increments all prestige victory levels the amount of 'x'.

	Inputs:		( int32 )	-	The amount to increase the prestige of each victory level
	Outputs:	N/A
*/
void
tVictoryState::mAddToPrestige( int32 X )
{
	for ( int32 i = 0; i < kMaxVictoryLevels; i++ )
	{
		fPrestigePoints[i] += X;
	}
}

/*
	Purpose:	Increments the prestige of the victory level, 'level', the amount in 'x'.

	Inputs:		( eVictoryLevels )	-	The requested level for modification
				( int32	)			-	The amount to increase the prestige for the requested level
	Outputs:	
*/
void
tVictoryState::mAddToPrestige(	eVictoryLevels	Level,
								int32			X )
{
	fPrestigePoints[Level] += X;
}

/*
	Purpose:	Adds the amount in 'x' to the bonus prestige.

	Inputs:		( int32 )	-	The amount to increase the bonus prestige
*/
void
tVictoryState::mAddToBonusPrestige( int32 X )
{
	fBonusPrestige += X;
}


//	15Nov99	zdrummond	Added Comment Header and Footer's
//	29Apr00	dsuleski	Added mAddToBonusPrestige(...) for functionality of SFC1 API
//	5Jun00	dsuleski	Prestige points now defaulted to 0
//	8Jul00	dsuleski	Documentation update