/********************************************************************************************************\

	Mission:	<MISSION TITLE>
	Script:		"Tem_Single"
	Descript:	<DESCRIPTION>

	Created by:		<NAME>		mm/dd/yy
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "CommonDefs.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "NPC1Team.h"
#include "Tem_Single.h"


DEFINE_SCRIPT_CLASS( tTem_Single );

tTem_Single::tTem_Single()
{}

/*

	HELPER NOTE:	Below set up the mission specifications for this script.  Each type of mission can require
					various fields to be set.  These types are as follows:

					kCampaignMission
						- for general Metaverse missions
					kUniqueCampaignMission
						- for "elite" Metaverse missions
						- cannot have kPrimaryOpponentTeam or kPlayableTeam
					kTutorialMission
						- the fWait field determines where this script appears in the list
					kMultiplayerMission
						- cannot have kPrimaryTeam or kPrimaryOppenentTeam

*/
void tTem_Single::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kMissionTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player NPC Team",
												static_cast< eTeamID >( kPlayerNPCTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tPlayerNPCTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"NPC 1 Team",
												static_cast< eTeamID >( kNPC1Team ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tNPC1Team ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
}

/*

	HELPER NOTE:	Return the map index you want to use for only kUniqueCampaignMission.

*/
int32 tTem_Single::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all ships have been created for the script.

*/
void tTem_Single::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	kCampaignMission may require an overall briefing in the future, at which point, you would
					want to return the briefing index.

*/
int32 tTem_Single::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tTem_Single::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tTem_Single::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kNPC1Team,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,
		//Nme  = kEnemyTeam,
		//NPC1 = kNPC1Team,

		//	     Plyr  PNPC  Nme   NPC1
		/*Plyr*/ "--", "AI", "WD", "FI",
		/*PNPC*/ "AI", "--", "WD", "FI",
		/*Nme */ "WD", "WD", "--", "FI",
		/*NPC1*/ "FI", "FI", "FI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
void tTem_Single::mInitializeDraw( const int32 MapPicked )
{
//	gDraw.mDefineDrawObjects();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.
					Your custom values may look something like the following:

*/
void tTem_Single::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

*/
void tTem_Single::mDefineBriefings( void )
{
	for ( int32 i = kFederation; i <= kMirak; i++ )
	{
		fBriefingIndex[kPlayerTeam][i] = kBriefing_msg;
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam.  You can have no
					more than three (3) calls for each team.  Any additional will be ignored.

*/
void tTem_Single::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}