/********************************************************************************************************\

	Mission:	<MISSION TITLE>
	Script:		"Tst_AI"
	Descript:	<DESCRIPTION>

	Created by:		<NAME>		mm/dd/yy
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "NPC1Team.h"
#include "Tst_AI.h"


DEFINE_SCRIPT_CLASS( tTst_AI );

tTst_AI::tTst_AI()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tTst_AI::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fEnemyRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 6;
	Specs->fMenuStates = kNoMenuButtons;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( kAllBand );

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												1.0f,
												kFalse,
												kPrimaryTeam,
												1.0f,
												tStrength(	1, 3,
															kClassCarrier,
															kClassBattleship,
															0, kMaxBPV ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlayerTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player NPC Team",
												static_cast< eTeamID >( kPlayerNPCTeam ),
												1.0f,
												kTrue,
												kNPCTeam,
												1.0f,
												tStrength(	0, 0,
															kClassFrigate,
															kClassFrigate,
															0, kMaxBPV ),
												kSpecOrion,
												kAnyRace,
												typeid( tPlayerNPCTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												0.0f,
												kFalse,
												kNPCTeam,
												0.0f,
												tStrength(	0, 0,
															kClassFrigate,
															kClassFrigate,
															0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"NPC 1 Team",
												static_cast< eTeamID >( kNPC1Team ),
												0.0f,
												kFalse,
												kNPCTeam,
												0.0f,
												tStrength(	0, 0,
															kClassFrigate,
															kClassFrigate,
															0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tNPC1Team ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
//	Specs->mSetupTeam( team3 );
//	Specs->mSetupTeam( team4 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tTst_AI::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tTst_AI::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tTst_AI::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tTst_AI::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tTst_AI::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
//		kEnemyTeam,
//		kNPC1Team,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,
		//Nme  = kEnemyTeam,
		//NPC1 = kNPC1Team,

		//	     Plyr  PNPC  Nme   NPC1
		/*Plyr*/ "--", "AI",// "WD", "FI",
		/*PNPC*/ "AI", "--",// "WD", "FI",
//		/*Nme */ "WD", "WD", "--", "FI",
//		/*NPC1*/ "FI", "FI", "FI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
void tTst_AI::mInitializeDraw( const int32 MapPicked )
{
	;
}
