/********************************************************************************************************\

	Mission:	Hardpoint Test
	Script:		"Tst_HardpointTest"
	Descript:	Test of damage allocation / hardpoint system

	Created by:		Scott Bruno	07/26/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "NPC1Team.h"
#include "Tst_HardpointTest.h"


DEFINE_SCRIPT_CLASS( tTst_HardpointTest );

tTst_HardpointTest::tTst_HardpointTest()
{}

/*

	HELPER NOTE:	This function should return whom you consider to be the primary enemy.  Perhaps useful
					in multiplayer scripts, but single player missions do not seem to actively use it.

*/
int32 tTst_HardpointTest::mGetEnemyTeamID( void )
{
	return fPlayersTeam;
}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tTst_HardpointTest::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fEnemyRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 6;
	Specs->fMenuStates = kNoMenuButtons;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( kAllBand );

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												1.0f,
												kFalse,
												kPlayableTeam,
												1.0f,
												tStrength(	1, 1,
															kClassFrigate,
															kClassFrigate,
															0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlayerTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												tStrength(	1, 1,
															kClassFrigate,
															kClassFrigate,
															0, 300 ),
												kSpecAnyRace,
												kAnyRace,
												typeid( tEnemyTeam ) );

	
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tTst_HardpointTest::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tTst_HardpointTest::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tTst_HardpointTest::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tTst_HardpointTest::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tTst_HardpointTest::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tTst_HardpointTest::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tTst_HardpointTest::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Nme  = kEnemyTeam,
	
		//	     Plyr  NME
		/*Plyr*/ "--", "WD", 
		/*NME*/  "WD", "--", 
	
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


