/********************************************************************************************************\

	File Name:	CommonVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "CommonVictoryState.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"
#include "MissionText.h"

//
//	Serialization events
//
tCommonVictoryState::tCommonVictoryState(	int32 astoundingVictory,
											int32 victory,
											int32 draw,
											int32 defeat,
											int32 devastatingDefeat ) : tChildActorVictoryState< tCommonVictory >(	astoundingVictory,
																													victory,
																													draw,
																													defeat,
																													devastatingDefeat )
{}
void tCommonVictoryState::mBeginState()
{}
void tCommonVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tCommonVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tCommonVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	//
	//	Now figure out which victory level this team should be.
	//
	if ( gWC.mTeamWon( ID ) )
	{
		if ( !gWC.mWasCaptured( ID ) )
			return kAstoundingVictory;
		else
			return kVictory;			
	}
	else
	{
		if ( !gWC.mWasCaptured( ID ) )
			return kDefeat;
		else
			return kDevastatingDefeat;
	}

	return kDraw;
}

//
//	Ship events
//
/*

	HELPER NOTE:	Inside each of the following functions, the ship parameter can be examined to determine
					which ship received the corresponding event.  You can find out which ship it is by
					determining its custom ID.  Remember, since this is a victory condition, there is no
					reason to check the ship against 'fChildActor'.  Simply examine the team and ship as
					described below and end the game appropriately, if needed with a call to mSetGameStatus().

					Please REMEMBER TO TAKE CARE OF onCapture, onDeath, and onDisengage events for the player
					ship since not doing so will freeze the game.  In other words, you MUST tell the game that
					it is OVER!  It has been started for you; however, if the player has multiple ships under
					control, you must run the player's ship list for any other ships that could be in play.

	void
	tCommonVictoryState::mOnBS( tShipInfo* ship )
	{
		switch ( ship->mGetTeam() )
		{
		case kMadeUpTeam:
			{
				switch ( ship->mGetCustomID() )
				{
				case kMadeUpID:
					{
						//
						//	the ship with custom ID of 'kMadeUpID' has been BSed and in this case, the
						//	player should lose for whatever reason.  Remember, the call to mSetGameStatus()
						//	will call mCalcVictoryStatus() as described above.
						//
						fChildActor->mSetBSed( true );
						gScriptInterface.mSetGameStatus( kPlayerLoses );
					}
					break;
				}
			}
			break;

		case etc...
		}

		if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
		{
			//
			//	the player BSed, so the player has accomplished the primary objective.
			//
			gScriptInterface.mSetGameStatus( kPlayerWins );
		}
	}

*/
void tCommonVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	if ( ship->mGetCustomID() == kListeningPost )
	{
		for ( int32 i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
		{
			gWC.mSetTeamWon( i, kFalse );
		}
		gWC.fListeningPostDestroyed = kTrue;
	}

	if ( !team->mShipsCurrentlyControlled() )
	{
		gWC.mSetTeamWon( teamID, kFalse );
		gScriptInterface.mSetGameStatus( teamID, kPlayerDestroyed );
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	if( ship->mGetTeam() == fTeam->mGetTeam() )
	{
		this->mAddToPrestige( 120 );
	}

	if ( !team->mShipsCurrentlyControlled() )
	{
		gWC.mSetTeamWon( teamID, kFalse );
		gScriptInterface.mSetGameStatus( teamID, kPlayerCaptured );
	}

	//
	//	Let's subtract prestige points from this team's score and add them to the other team.
	//
	gWC.mSetWasCaptured( teamID, kTrue );

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	eTeamID		teamID = ship->mGetTeam();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( teamID );
	Assert( team );
	if ( !team )
		return;

	//
	//	The exception to the above rule is if the ship was on the enemy side and already transferred the data.
	//
	bool gotData = false;
	if ( teamID == kEnemyTeam1 || teamID == kEnemyTeam2 || teamID == kEnemyTeam3 )
	{
		tCommonEnemyTeam* enemyTeam = dynamic_cast< tCommonEnemyTeam* >( team );
		if ( enemyTeam )
		{
			if ( enemyTeam->mReachedListeningPost() )
			{
				gWC.mSetTeamWon( teamID, kTrue );
				gotData = true;
			}
		}
	}

	if ( !team->mShipsCurrentlyControlled() )
	{
		if ( !gotData )
		{
			gWC.mSetTeamWon( teamID, kFalse );
			gScriptInterface.mSetGameStatus( teamID, kPlayerDisengaged );
		}
		else
		{
			gScriptInterface.mSetGameStatus( teamID , kPlayerWins );
		}
	}

	//
	//	The game could possibly end here, so let's check.
	//
	gWC.mCheckForGameEnd();
}
void tCommonVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnFlashingButtonPressed()
{}
void tCommonVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tCommonVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tCommonVictoryState::mOnHostChanged()
{}
void tCommonVictoryState::mOnTimedInterval( int32 timerID )
{}
void tCommonVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
