/********************************************************************************************************\

	Mission:	Tst_HoldingAction
	Script:		"Tst_HoldingAction"
	Descript:	Small force is assigned to delay a larger force.

	Created by:		Daniel J. Suleski		07/05/00
	Last Updated:	DJS						07/06/00
					Added extra traits to mission that were not found in Met_4FleetAction.  This
					included the event catching for an enemy ship making it off of the 'far' edge
					of the map and the event catching for the the holding team destroying half of
					the enemy ships.
					DJS						10/13/00
					Resurrecting this hellbeast...ugh -- for Beta


\********************************************************************************************************/

#include "Tst_HoldingAction.h"
#include "CommonDefs.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "GoalTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"


DEFINE_SCRIPT_CLASS( tTst_HoldingAction );

tTst_HoldingAction::tTst_HoldingAction()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tTst_HoldingAction::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionDescription = Messages[kMissionTitle_msg];
	Specs->fHelpTags = Messages[kMissionTitle_msg];
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMenuStates = kEnableSupplyDock | kDisableTagC | kDisableTagD | kDisableTagE | kDisableTagF;
	Specs->fGenericFlags = kDisableCapture;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kEnemyRaceHex, kAnyTerrainHex );
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );

	const tTeamSpec& team1 = mCreateTeamSpec(	"Enemy Team 1",
												static_cast< eTeamID >( kEnemyTeam1 ),
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonEnemyTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team 2",
												static_cast< eTeamID >( kEnemyTeam2 ),
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team 3",
												static_cast< eTeamID >( kEnemyTeam3 ),
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonEnemyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Holding Team 1",
												static_cast< eTeamID >( kPlayerTeam1 ),
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonPlayerTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Holding Team 2",
												static_cast< eTeamID >( kPlayerTeam2 ),
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonPlayerTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Holding Team 3",
												static_cast< eTeamID >( kPlayerTeam3 ),
												kMinChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonPlayerTeam ) );

	const tTeamSpec& goalTeam = mCreateTeamSpec(	"Goal Team",
													static_cast< eTeamID >( kGoalTeam ),
													kMaxChance,
													kFalse,
													kNPCTeam,
													kMaxChance,
													kSpecAnyRace,
													kAnyRace,
													typeid( tGoalTeam ),
													static_cast< eTeamTag >( kGoalSide ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( goalTeam );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tTst_HoldingAction::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when all of the team ships are created.  This is your last chance to create
					ships before the mission starts.  You can tell that this is the initial call if
					the "Mode" parameter is set to 0.

*/
void tTst_HoldingAction::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tTst_HoldingAction::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you would normally set up the teams' relationships and goals.  However,
					since this is a multiplayer mission you will set up the relations between team tags
					rather than the teams themselves.

*/
void tTst_HoldingAction::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 tagList[kRelationsCount] =
	{
		kEnemySide,
		kPlayerSide,
		kGoalSide,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 ETm1  PTm1  Goal
		/*ETm1*/ "AI", "WD", "FI",
		/*PTm1*/ "WD", "AI", "FI",
		/*Goal*/ "FI", "FI", "AI",
	};

	fMissionInfo->mSetTagRelations( tagList, ( std::string* )relationships, kRelationsCount );
}

/*

	HELPER NOTE:	Create a 'tMapTerrain' object, assign it a meta location, and return based on the given
					map index of the ASCII maps in MissionText.cpp.

*/
tMapTerrain tTst_HoldingAction::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace,		kPlanetAnyPlanet, kBaseAnyBase );
	case 1:		return tMapTerrain( kTerrainAsteroids,	kPlanetAnyPlanet, kBaseAnyBase );
	case 2:		return tMapTerrain( kTerrainNebula,		kPlanetAnyPlanet, kBaseAnyBase );
	default:	break;
	}
	return tMapTerrain();
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tTst_HoldingAction::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['I'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['L'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['M'] = kObjectShipEnd;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tTst_HoldingAction::mDefineBriefings( void )
{
	for ( int32 j = kFederation; j <= kMirak; j++ )
	{
		for ( int32 i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefingPlayer_msg;
		}
		for ( i = kEnemyTeam1; i <= kEnemyTeam3; i++ )
		{
			fBriefingIndex[i][j] = kBriefingEnemy_msg;
		}
	}
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tTst_HoldingAction::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kEnemyTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam1, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );

	for ( int32 i = kEnemyTeam2; i <= kPlayerTeam3; i++ )
	{
		mSetTeamShipStrength( i, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
		mSetTeamShipStrength( i, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
		mSetTeamShipStrength( i, kClassFrigate, kClassBattleship, 0.7f, 1.2f );
	}
}

void tTst_HoldingAction::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{
	switch ( teamScriptDescription.fTeamTag )
	{
	case kEnemySide:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									teamScriptDescription.fRace,
									teamScriptDescription.fHumanControlled,
									typeid( tCommonEnemyTeam ) );
		}
		break;

	case kPlayerSide:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									teamScriptDescription.fRace,
									teamScriptDescription.fHumanControlled,
									typeid( tCommonPlayerTeam ) );
		}
		break;

	case kGoalSide:
		{
			mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									teamScriptDescription.fRace,
									teamScriptDescription.fHumanControlled,
									typeid( tGoalTeam ) );
		}
		break;
	}
}