/********************************************************************************************************\

	File Name:	WorldConditions.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "CommonEnemyTeam.h"
#include "CommonPlayerTeam.h"
#include "GoalTeam.h"
#include "ScriptInterface.h"
#include "WorldConditions.h"

/*

	HELPER NOTE:	This global reference to the world condition is here twofold.  One, it can be easily
					accessed from the all victory states; two, it maintains the consistancy relatively
					easily throughout the script.

*/
class tWorldConditions gWC;

/*

	HELPER NOTE:	Remember to instantiate your world conditions here.

*/
tWorldConditions::tWorldConditions()
{
	for ( int32 i = kEnemyTeam1; i <= kPlayerTeam3; i++ )
	{
		fWasCaptured[i] = kFalse;
		fTeamWon[i] = kFalse;
	}
	fListeningPostDestroyed = kFalse;
}

/*

	HELPER NOTE:	Acts as a sort of victory condition for all of the teams.  This is where you can
					analyze the game state according to the world conditions you have kept.

*/
void tWorldConditions::mCheckForGameEnd( void )
{
	tMissionInfo* thisMission = gScriptInterface.mMission();
	if ( !thisMission )
		return;

	//
	//	This is the place that the teams that are out of the game, are going to be officially
	//	counted out.  Also, if there is only one side left, then the game will end for everyone
	//	and everyone will receive the proper game status changes.
	//
	iTruth	playerTeamHasShips = kFalse;
	for ( int32 i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
	{
		tCommonPlayerTeam* team = dynamic_cast< tCommonPlayerTeam* >( thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) ) );
		if ( team )
		{
			if ( team->mShipsCurrentlyControlled() )
				playerTeamHasShips = kTrue;
		}
	}

	iTruth	enemyTeamHasShips = kFalse;
	for ( i = kEnemyTeam1; i <= kEnemyTeam3; i++ )
	{
		tCommonEnemyTeam* team = dynamic_cast< tCommonEnemyTeam* >( thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) ) );
		if ( team )
		{
			if ( team->mShipsCurrentlyControlled() )
				enemyTeamHasShips = kTrue;
		}
	}

	iTruth	thereIsAWinner = !( playerTeamHasShips && enemyTeamHasShips );
	if ( !thereIsAWinner )
	{
		iTruth playerTeamHasWon = kFalse;
		iTruth enemyTeamHasWon = kFalse;
		for ( i = kPlayerTeam1; i <= kPlayerTeam3; i++ )
		{
			tTeamInfo* team = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( fTeamWon[i] )
					playerTeamHasWon = kTrue;
			}
		}
		for ( i = kEnemyTeam1; i <= kEnemyTeam3; i++ )
		{
			tTeamInfo* team = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				if ( fTeamWon[i] )
					enemyTeamHasWon = kTrue;
			}
		}
		thereIsAWinner = !( playerTeamHasWon && enemyTeamHasWon );
	}
	if ( thereIsAWinner )
	{
		//
		//	Need to take care of a special case where:
		//		1. the player ships have been neutralized
		//		2. the listening post is still around
		//		3. the player has not disengaged with the data from the listening post (optional -- not implemented yet)
		//
		if ( !playerTeamHasShips && !fListeningPostDestroyed )
		{
			thereIsAWinner = kFalse;
		}
	}

	//
	//	End the game if there is a winner because one side destroyed another side's ships.
	//
	if ( thereIsAWinner )
	{
		for ( i = kEnemyTeam1; i < kPlayerTeam3; i++ )
		{
			tTeamInfo* team = thisMission->mGetTeamHandle( static_cast< eTeamID >( i ) );
			if ( team )
			{
				//
				//	If there are no more player ships, and only enemy ships remain, then just end the game
				//	as a losers for the player team regardless of whether the enemy team got to the listening
				//	post or not.  The remaining enemy teams should win by default because there is no longer
				//	a threat.
				//
				if ( !playerTeamHasShips && enemyTeamHasShips )
				{
					for ( int32 j = kPlayerTeam1; j <= kPlayerTeam3; j++ )
						mSetTeamWon( j, kFalse );
					for ( j = kEnemyTeam1; j <= kEnemyTeam3; j++ )
						mSetTeamWon( j, kTrue );
				}
				else if ( playerTeamHasShips && !enemyTeamHasShips )
				{
					for ( int32 j = kPlayerTeam1; j <= kPlayerTeam3; j++ )
						mSetTeamWon( j, kTrue );
					for ( j = kEnemyTeam1; j <= kEnemyTeam3; j++ )
						mSetTeamWon( j, kFalse );
				}

				//
				//	Now act on whether the team won or lost.
				//
				if ( mTeamWon( i ) )
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerWins );
				}
				else
				{
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( i ), kPlayerLoses );
				}
			}
		}
	}
}


void tWorldConditions::mSetWasCaptured( int32 x, iTruth y )
{
	fWasCaptured[x] = y;
}
void tWorldConditions::mSetTeamWon( int32 x, iTruth y )
{
	fTeamWon[x] = y;
}

iTruth tWorldConditions::mWasCaptured( int32 x )
{
	return fWasCaptured[x];
}
iTruth tWorldConditions::mTeamWon( int32 x )
{
	return fTeamWon[x];
}