/********************************************************************************************************\

	Mission:	<MISSION TITLE>
	Script:		"Tst_Models"
	Descript:	<DESCRIPTION>

	Created by:		<NAME>		mm/dd/yy
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "NPC1Team.h"
#include "Tst_Models.h"


DEFINE_SCRIPT_CLASS( tTst_Models );

tTst_Models::tTst_Models()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tTst_Models::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fEnemyRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecNoShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 6;
	Specs->fMenuStates = kNoMenuButtons;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( kAllBand );

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												1.0f,
												kFalse,
												kPlayableTeam,
												1.0f,
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlayerTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player NPC Team",
												static_cast< eTeamID >( kPlayerNPCTeam ),
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlayerNPCTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												kSpecAnyRace,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"NPC 1 Team",
												static_cast< eTeamID >( kNPC1Team ),
												1.0f,
												kFalse,
												kNPCTeam,
												1.0f,
												kSpecAnyRace,
												kAnyRace,
												typeid( tNPC1Team ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tTst_Models::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tTst_Models::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tTst_Models::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tTst_Models::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tTst_Models::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kNPC1Team,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,
		//Nme  = kEnemyTeam,
		//NPC1 = kNPC1Team,

		//	     Plyr  PNPC  Nme   NPC1
		/*Plyr*/ "--", "AI", "WD", "FI",
		/*PNPC*/ "AI", "--", "WD", "FI",
		/*Nme */ "WD", "WD", "--", "FI",
		/*NPC1*/ "FI", "FI", "FI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
void tTst_Models::mInitializeDraw( const int32 MapPicked )
{
	;
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tTst_Models::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectShipEnd;
	Map->fTacticalMapIcon['I'] = kObjectShipEnd;
	Map->fTacticalMapIcon['J'] = kObjectPlanet;
	Map->fTacticalMapIcon['K'] = kObjectPlanet;
	Map->fTacticalMapIcon['L'] = kObjectPlanet;
	Map->fTacticalMapIcon['M'] = kObjectPlanet;
	Map->fTacticalMapIcon['N'] = kObjectPlanet;
	Map->fTacticalMapIcon['O'] = kObjectPlanet;
}
