/********************************************************************************************************\

	Mission:	"Object Test"
	Script:		"Tst_ObjectTest"
	Descript:	Used to quickly check out terrain, spec file objects, etc;
				Use tTestTeam::mCreateShipsForTeam()

	Created by:		Scott Bruno		08/10/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Tst_ObjectTest.h"
#include "PlayerTeam.h"
#include "TestTeam.h"
#include "ObjectTeam.h"

DEFINE_SCRIPT_CLASS( tTst_ObjectTest );

tTst_ObjectTest::tTst_ObjectTest()
{}

int32 tTst_ObjectTest::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tTst_ObjectTest::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fHelpTags = Messages[kDescription_msg];
	Specs->fMissionType = kHistoricalMission;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecAllShips;
	Specs->fMenuStates = kEnableSupplyDock;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												1.0f,
												kTrue,
												kPrimaryTeam,
												1.0f,
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlayerTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Test Team",
												static_cast< eTeamID >( kTestTeam ),
												1.0f,
												kTrue,
												kNPCTeam,
												1.0f,
												kSpecAnyRace,
												kAnyRace,
												typeid( tTestTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Object Team",
												static_cast< eTeamID >( kObjectTeam ),
												1.0f,
												kTrue,
												kNPCTeam,
												1.0f,
												kSpecAnyRace,
												kAnyRace,
												typeid( tObjectTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
}

int32 tTst_ObjectTest::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tTst_ObjectTest::mCreateExternalShips( int32 Mode )
{}

int32 tTst_ObjectTest::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tTst_ObjectTest::mNewInternalShip( tShipInfo* Ship )
{}

void tTst_ObjectTest::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tTst_ObjectTest::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tTst_ObjectTest::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kTestTeam,
		kObjectTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Test = kTestTeam

		//	     Plyr  Test  OBJ
		/*Plyr*/ "--", "NI", "WD", 
		/*Test*/ "NI", "--", "WD",
		/*OBJ*/  "WD", "NI", "--",
	
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}
