/********************************************************************************************************\

	File Name:	CommonVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "PriTeam.h"
#include "EnemyTeam.h"
#include "EnemyTeam2.h"
#include "Tst_ScottsTest.h"
#include "worldconditions.h"

//
//	Serialization events
//
tCommonVictoryState::tCommonVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tCommonVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tCommonVictoryState::mBeginState()
{}
void tCommonVictoryState::mEndState()
{}

eVictoryLevels tCommonVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPriTeam* priTeam = dynamic_cast<tPriTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPriTeam ) ) );
	ASSERT( priTeam );
	if( !priTeam ) return kUnknownVictoryLevel;

	tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
	ASSERT( enemyTeam );
	if( !enemyTeam ) return kUnknownVictoryLevel;

	tEnemyTeam2* enemyTeam2 = dynamic_cast<tEnemyTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam2 ) ) );
	ASSERT( enemyTeam2 );
	if( !enemyTeam2 ) return kUnknownVictoryLevel;

	switch( ID )
	{
	case kPriTeam:
		{
			if( priTeam->mDied() )
			{
				return kDevastatingDefeat;
			}
		
			if( priTeam->mBaseDied() )
			{
				if( !enemyTeam->mDied() || !enemyTeam2->mDied() )
				{
					return kDevastatingDefeat;
				}
				return kDefeat;
			}

			if( enemyTeam2->mDied() )
			{
				return kAstoundingVictory;
			}

			return kDraw;
		}
		break;

	case kEnemyTeam2:
		{
			if( enemyTeam2->mDied() )
			{
				return kDevastatingDefeat;
			}

			if( priTeam->mDied() )
			{
				return kAstoundingVictory;
			}

			if( priTeam->mBaseDied() && !priTeam->mDied() )
			{
				return kVictory;
			}

			return kDraw;
		}
		break;

	default:
		return kUnknownVictoryLevel;
	}
}

void tCommonVictoryState::mOnDeath( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship ) return;

	tPriTeam* priTeam = dynamic_cast<tPriTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPriTeam ) ) );
	ASSERT( priTeam );
	if( !priTeam ) return;

	tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
	ASSERT( enemyTeam );
	if( !enemyTeam ) return;

	tEnemyTeam2* enemyTeam2 = dynamic_cast<tEnemyTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam2 ) ) );
	ASSERT( enemyTeam2 );
	if( !enemyTeam2 ) return;

	switch( ship->mGetTeam() )
	{
	case kPriTeam:
		{
			if( ship->mGetCustomID() == tPriTeam::kBase )
			{
				priTeam->mSetBaseDied();
				gWC.mSetBaseDied();
			}

			if( !priTeam->mShipsRemainingInPlay() )
			{
				priTeam->mSetDied();

				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerDestroyed );
			
				if( enemyTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kEnemyTeam2 ), kPlayerWins );
				}
			}
		}
		break;

	case kEnemyTeam:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				enemyTeam->mSetDied();

				if( !enemyTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerWins );
				}
			}
		}
		break;

	case kEnemyTeam2:
		{
			if( !enemyTeam2->mShipsRemainingInPlay() )
			{
				enemyTeam2->mSetDied();

				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kEnemyTeam2 ), kPlayerDestroyed );

				if( !enemyTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerWins );
				}
			}
		}
		break;
	}
}

void tCommonVictoryState::mOnCapture( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship ) return;

	tPriTeam* priTeam = dynamic_cast<tPriTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPriTeam ) ) );
	ASSERT( priTeam );
	if( !priTeam ) return;

	tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
	ASSERT( enemyTeam );
	if( !enemyTeam ) return;

	tEnemyTeam2* enemyTeam2 = dynamic_cast<tEnemyTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam2 ) ) );
	ASSERT( enemyTeam2 );
	if( !enemyTeam2 ) return;

	switch( ship->mGetOriginalTeam() )
	{
	case kPriTeam:
		{
			if( ship->mGetCustomID() == tPriTeam::kBase )
			{
				priTeam->mSetBaseDied();
				gWC.mSetBaseDied();
			}

			if( !priTeam->mShipsRemainingInPlay() )
			{
				priTeam->mSetDied();

				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerDestroyed );
			
				if( enemyTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kEnemyTeam2 ), kPlayerWins );
				}
			}
		}
		break;

	case kEnemyTeam:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				enemyTeam->mSetDied();

				if( !enemyTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerWins );
				}
			}
		}
		break;

	case kEnemyTeam2:
		{
			if( !enemyTeam2->mShipsRemainingInPlay() )
			{
				enemyTeam2->mSetDied();

				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kEnemyTeam2 ), kPlayerDestroyed );

				if( !enemyTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerWins );
				}
			}
		}
		break;
	}	
}

void tCommonVictoryState::mOnDisengaged( tShipInfo* ship )
{
	ASSERT( ship );
	if( !ship ) return;

	tPriTeam* priTeam = dynamic_cast<tPriTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPriTeam ) ) );
	ASSERT( priTeam );
	if( !priTeam ) return;

	tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
	ASSERT( enemyTeam );
	if( !enemyTeam ) return;

	tEnemyTeam2* enemyTeam2 = dynamic_cast<tEnemyTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam2 ) ) );
	ASSERT( enemyTeam2 );
	if( !enemyTeam2 ) return;

	switch( ship->mGetTeam() )
	{
	case kPriTeam:
		{
			if( ship->mGetCustomID() == tPriTeam::kBase )
			{
				priTeam->mSetBaseDied();
				gWC.mSetBaseDied();
			}

			if( !priTeam->mShipsRemainingInPlay() )
			{
				priTeam->mSetDied();

				if( !gWC.mBaseDied() )
				{
					gWC.mSetBaseDied();
				}

				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerDestroyed );
			
				if( enemyTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kEnemyTeam2 ), kPlayerWins );
				}
			}
		}
		break;

	case kEnemyTeam:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				enemyTeam->mSetDied();

				if( !enemyTeam2->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerWins );
				}
			}
		}
		break;

	case kEnemyTeam2:
		{
			if( !enemyTeam2->mShipsRemainingInPlay() )
			{
				enemyTeam2->mSetDied();

				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kEnemyTeam2 ), kPlayerDestroyed );

				if( !enemyTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPriTeam ), kPlayerWins );
				}
			}
		}
		break;
	}
}

void tCommonVictoryState::mAtDistance( tShipInfo* ship )
{}
void tCommonVictoryState::mAtDestination( tShipInfo* ship )
{}
void tCommonVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCommonVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCommonVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCommonVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCommonVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tCommonVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCommonVictoryState::mOnFlashingButtonPressed()
{}
void tCommonVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tCommonVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tCommonVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tCommonVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tCommonVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
													eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tCommonVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCommonVictoryState::mOnHostChanged()
{}
void tCommonVictoryState::mOnTimedInterval( int32 timerID )
{}
void tCommonVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCommonVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
