/********************************************************************************************************\

	Mission:	Tst_ScottsTest
	Script:		"Tst_ScottsTest"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		10/22/00
	Last Updated:	<INITIALS>		mm/dd/yy
					<LOG>


\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Tst_ScottsTest.h"
#include "PriTeam.h"
#include "PropTeam.h"
#include "EnemyTeam.h"
#include "EnemyTeam2.h"
#include "WorldConditions.h"


DEFINE_SCRIPT_CLASS( tTst_ScottsTest );

tTst_ScottsTest::tTst_ScottsTest()
{}

int32 tTst_ScottsTest::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tTst_ScottsTest::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kMissionTitle_msg];
	Specs->fMissionType = kCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kPlayerRaceHex, kEmptySpaceHex | kAsteroidHex
																	 | kShippingLaneHex | kDustCloudsHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = static_cast<eBandwidthPart>( k1Band | k5Client );
	Specs->fGenericFlags = kPlaceBaseMission | kRequiresPlanet | kRequiresBase;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Primary team",
												static_cast<eTeamID>( kPriTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tPriTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy team",
												static_cast<eTeamID>( kEnemyTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kWorstEnemyOf | kPriTeam,
												typeid( tEnemyTeam ),
												kNoTeamTag );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Prop team",
												static_cast<eTeamID>( kPropTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecPlayableRace,
												kSameAs | kPriTeam,
												typeid( tPropTeam ),
												kNoTeamTag);

	const tTeamSpec& team4 = mCreateTeamSpec(	"Enemy Team 2",
												static_cast<eTeamID>( kEnemyTeam2 ),
												kMaxChance,
												kTrue,
												kPrimaryOpponentTeam,
												kMaxChance,
												kSpecPlayableRace,
												kWorstEnemyOf | kPriTeam,
												typeid( tEnemyTeam2 ),
												kNoTeamTag,
												Messages[kTitle3] );

	
	
	
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );

}

int32 tTst_ScottsTest::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tTst_ScottsTest::mCreateExternalShips( int32 Mode )
{}

int32 tTst_ScottsTest::mInitializeFinished( void )
{
	return 0;
}

void tTst_ScottsTest::mNewInternalShip( tShipInfo* Ship )
{}

void tTst_ScottsTest::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tTst_ScottsTest::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tTst_ScottsTest::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPriTeam,
		kEnemyTeam,
		kPropTeam,
		kEnemyTeam2,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//				Pri   Nme   Prop  NME2  
		/*kPriTeam*/	"--", "WD", "AI", "WD",
		/*kEnemyTeam*/	"WD", "--", "HI", "AI",
		/*kPropTeam */	"AI", "HI", "--", "HI",
		/*EnemyTeam2*/  "WD", "AI", "WD", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, ( std::string* )relationships, kRelationsCount );

	gWC.mCheckForGameEnd();
}

void tTst_ScottsTest::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPriTeam, kClassFrigate, kClassStarBase, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPriTeam, kClassFrigate, kClassStarBase, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPriTeam, kClassFrigate, kClassStarBase, kMinBPV, kMaxBPV );

	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kEnemyTeam2, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}

void tTst_ScottsTest::mDefineBriefings( void )
{
	for( int32 nRace = kFederation; nRace <= kMirak; nRace++ )
	{
		fBriefingIndex[kEnemyTeam2][nRace] = kBriefing_Attacker;
	}

	fBriefingIndex[kPriTeam][kFederation]	= kBriefing_Fed;
	fBriefingIndex[kPriTeam][kKlingon]		= kBriefing_Kli;
	fBriefingIndex[kPriTeam][kRomulan]		= kBriefing_Rom;

	fBriefingIndex[kPriTeam][kGorn]			= kBriefing_Other;
	fBriefingIndex[kPriTeam][kHydran]		= kBriefing_Other;
	fBriefingIndex[kPriTeam][kLyran]		= kBriefing_Other;
	fBriefingIndex[kPriTeam][kISC]			= kBriefing_Other;
	fBriefingIndex[kPriTeam][kMirak]		= kBriefing_Other;
}

void tTst_ScottsTest::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );
	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
}

tMapTerrain tTst_ScottsTest::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:		return tMapTerrain( kTerrainSpace,			kPlanetNone,	kBaseNone );
	case 1:		return tMapTerrain( kTerrainAsteroids,		kPlanetNone,	kBaseNone );
	case 2:		return tMapTerrain( kTerrainDustclouds,		kPlanetNone,	kBaseNone );
	}
	return tMapTerrain();
}