//	<Name>	PlayerTeamInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ShipInfo.h"
#include "Tut_Weapons380.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamInitialState.h"
#include "PlayerTeamLoserState.h"
#include "PlayerTeamFiredEarlyState.h"
#include "PlayerTeamWinnerState.h"

#include "MissionText.h"
#include "ScriptInterface.h"


tPlayerTeamInitialState::tPlayerTeamInitialState( )
{
	fChapter = 0;
	fTargetDamaged = false;
	fOrderedToDamage = false;
	fOrderedToDestroy = false;
	fFlashingButtonPressed = false;
	fEnteredTrainingGround = false;
	fTargetDestroyed = false;
	fTookToLong = false;
	fNumberOfMines = 0;
	fNumberOfMinesDecreased = false;
}

void tPlayerTeamInitialState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kDisplayMessageID, 2.0f );
}

void tPlayerTeamInitialState::mEndState()
{
}
void tPlayerTeamInitialState::mOnTimedInterval( int32 timerID )
{
	tShipInfo* playerShip;
	tShipInfo* firstTarget;

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(  static_cast< eTeamID>( kPlayerTeam ) );
	if( playerTeam )
	{
		tShipIterator ship = playerTeam->mGetFirstShip();
		tShipInPlayEval inPlayEval;
		ship.mAddEvaluator( &inPlayEval );
		playerShip = *ship;	
	}
	tTeamInfo* targetTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTargetTeam ) );
	if( targetTeam )
	{
		if ( targetTeam->mShipsRemainingInPlay() )
		{
			tShipIterator targetship = targetTeam->mGetFirstShip();
			tShipInPlayEval inPlayEval2;
			targetship.mAddEvaluator( &inPlayEval2 );
			firstTarget = *targetship;
		}
	}

	if( timerID == kCheckDistanceID ) 
	{
		if( ( playerShip->mGetTacticalDistance( firstTarget ) ) < 55 )
		{
			fEnteredTrainingGround = true;
			mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 3.0f );
			fChapter = 16;
		}
		else
		{
			mSetTimedInterval( kTimerSecondsSet, kCheckDistanceID, 3.0f );
		}
	}
	else if( timerID == kTookToLongID )
	{
		fTookToLong = true;
	}
	else if( timerID == kDisplayMessageID )
	{
		switch( fChapter )
		{
			case 0:
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kWhere_msg, kNonCommScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kDisplayMessageID, 3.0f );
				fChapter++;
			break;
			case 1:
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kWhere2_msg, kNonCommScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kDisplayMessageID, 3.0f );
				fChapter++;
			break;
			case 2:
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kWhere3_msg, kNonCommScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kDisplayMessageID, 4.0f );
				fChapter++;
			break;
			case 3:
				fMissionInfo->mPlayVoiceSample( kSndTut4_Welcome, kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_Welcome_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
				fChapter++;
			break;
			case 4:
				fMissionInfo->mPlayVoiceSample( kSndTut4_Revere, kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_Revere_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 12.0f );
				fChapter++;
			break;
			case 5:
				fMissionInfo->mPlayVoiceSample( kSndTut4_Observe, kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_Observe_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 6.0f );
				fChapter++;
			break;
			case 6:
				fMissionInfo->mPlayVoiceSample( kSndTut4_RedAlert , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_RedAlert_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 12.0f );
				fChapter++;
			break;
			case 7:
				fMissionInfo->mPlayVoiceSample( kSndTut4_FirstTopic , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_FirstTopic_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
				fMissionInfo->mStartFlashingButton( "A5_WEAPONS_SELECT" );
				fChapter++;
			break;
			case 8:
				if( fFlashingButtonPressed )
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_WeaponsGrouping , kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_WeaponsGrouping_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
					fChapter++;
					fFlashingButtonPressed = false;
				}
				else
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 1.0f );
				}
			break;
			case 9:
				fMissionInfo->mPlayVoiceSample( kSndTut4_FourDifferentGroupings , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kTut4_FourDifferentGroups_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 10.0f );
				fChapter++;
			break;
			case 10:
				fMissionInfo->mPlayVoiceSample( kSndTut4_SetPhotonGroup , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_SetPhotonGroup_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 16.0f );
				fMissionInfo->mStartFlashingButton( "PlayerSchem.Weapon1" );
				fChapter++;
			break;
			case 11:
				if( fFlashingButtonPressed )
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_SelectPortBank , kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_SelectPortBank_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 5.0f );
					fMissionInfo->mStartFlashingButton( "PlayerSchem.Weapon2" );
					fChapter++;
					fFlashingButtonPressed = false;
				}
				else
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
				}
			break;
			case 12:
				if( fFlashingButtonPressed )
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_PhotonGroupDone , kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_PhotonGroupDone_msg, kScienceScriptMessage );
///////////////////////////////////////////////////////
					mSetTimedInterval( kTimerSecondsSet, kCheckDistanceID, 20.0f );
					fChapter++;
					fFlashingButtonPressed = false;
				}
				else
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 1.0f );
				}
			break;
			case 13:
				if( fEnteredTrainingGround )
				{
					fChapter += 3;
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 6.0f );
				}
				else
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_EnterTrainingGroundsNow , kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_EnterTrainingGroundNow_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerTurnsSet, kCheckVoicePlayingID, 2.0f );
					fChapter++;
				}
			break;
			case 14:
				if( fEnteredTrainingGround )
				{
					fChapter += 2;
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 6.0f );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_ImWaiting_msg, kScienceScriptMessage );
					fMissionInfo->mPlayVoiceSample( kSndTut4_ImWaiting, kTrue );
					mSetTimedInterval( kTimerTurnsSet, kCheckVoicePlayingID, 1.5f );
					fChapter++;
				}
			break;
			case 15:
				if( fEnteredTrainingGround )
				{
					fChapter++;
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 3.0f );
				}
				else
				{
					fChildActor->mGotoState( typeid( tPlayerTeamLoserState ) );
				}
			break;
			case 16:
				fMissionInfo->mPlayVoiceSample( kSndTut4_TrainingGrounds , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_TrainingGrounds_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 9.0f );
				fChapter++;
			break;
			case 17:
				fMissionInfo->mPlayVoiceSample( kSndTut4_BeforeTalkPhotons , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_BeforeTalkPhotons_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
				fChapter++;
			break;
			case 18:
				fMissionInfo->mPlayVoiceSample( kSndTut4_IntroPhotons , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_IntroPhotons_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 17.0f );
				fChapter++;
			break;
			case 19:
				fMissionInfo->mPlayVoiceSample( kSndTut4_PhotonSettings , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_PhotonSettings_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 16.0f );
				fChapter++;
			break;
			case 20:
				fMissionInfo->mPlayVoiceSample( kSndTut4_RegularPhotons , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_RegularPhotons_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 12.0f );
				fChapter++;
			break;
			case 21:
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 0.5f );
				fChapter++;
			break;
			case 22:
				fMissionInfo->mPlayVoiceSample( kSndTut4_FireSpread , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_FireSpread_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 12.0f );
				fOrderedToDamage = true;
				fChapter++;
			break;
			case 23:
				if( fTargetDamaged )
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_Overload, kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_Overload_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 14.0f );
					fMissionInfo->mStartFlashingButton( "A6_PHOTON_OVERLOAD" );
					fChapter++;
					fFlashingButtonPressed = false;
				}
				else
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
				}
			break;
			case 24:
				if( fFlashingButtonPressed )
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_OverloadMyopic , kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_OverloadMyopic_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
					fChapter++;
					fFlashingButtonPressed = false;
				}
				else
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 3.0f );
				}
			break;
			case 25:
				fOrderedToDestroy = true;
				fMissionInfo->mPlayVoiceSample( kSndTut4_FinishOffFreighter , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_FinishOffFreighter_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 9.0f );
				fChapter++;
			break;
			case 26:
				if( fTargetDestroyed )
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_StandardMine , kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_StandardMine_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 17.0f );
					fMissionInfo->mStartFlashingButton( "A3_OFFICER_WEAPONS" );
					fChapter++;
					fFlashingButtonPressed = false;
				}
				else
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 3.0f );
				}
			break;
			case 27:
				if( fFlashingButtonPressed )
				{
					fChapter += 3;
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 1.0f );
				}
				else
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_WeaponsOfficerNow, kTrue );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_WeaponsOfficerNow_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerTurnsSet, kCheckVoicePlayingID, 1.0f );
					mSetTimedInterval( kTimerTurnsSet, kTookToLongID, 1.0f );
					fChapter++;
				}
			break;
			case 28:
				if( fFlashingButtonPressed )
				{
					fChapter += 2;
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 1.0f );
				}
				else if( ! fTookToLong )
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 1.0f );
				}
				else
				{
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 1.0f );
					fChapter++;
					fTookToLong = false;
				}
			break;
			case 29:
				if( fFlashingButtonPressed )
				{
					fChapter++;
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 3.0f );
				}
				else
				{
					fChildActor->mGotoState( typeid( tPlayerTeamLoserState ) );
				}
			break;			
			case 30:
				fMissionInfo->mPlayVoiceSample( kSndTut4_DroppingOutOfBay , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_DroppingOutOfBay_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
				fChapter++;
				fFlashingButtonPressed = false;
				playerShip->mGetAmountOfStore( kSmall_Mines );
			break;
			case 31:
				fMissionInfo->mPlayVoiceSample( kSndTut4_LayMine , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_LayMine_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerTurnsSet, kCheckVoicePlayingID, 0.5f );
				fMissionInfo->mStartFlashingButton( "A3_WEAPONS_LAYMINE" );
				fChapter++;
			break;
			case 32:
				if( fNumberOfMinesDecreased )//( fFlashingButtonPressed )
				{
					fMissionInfo->mPlayVoiceSample( kSndTut4_MineExplained , kTrue );
					fMissionInfo->mStopFlashingButton( "A3_WEAPONS_LAYMINE" );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_MineExplained_msg, kScienceScriptMessage );
					mSetTimedInterval( kTimerTurnsSet, kCheckVoicePlayingID, 1.0f );
					fChapter++;
				}
				else
				{
					playerShip->mGetAmountOfStore( kSmall_Mines );
					mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 2.0f );
				}
			break;
			case 33:
				fMissionInfo->mPlayVoiceSample( kSndTut4_MineArmed , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_MineArmed_msg, kScienceScriptMessage );
				mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 7.0f );
				fChapter++;
			break;
			case 34:
				fMissionInfo->mPlayVoiceSample( kSndTut4_Congrats , kTrue );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ),  kTut4_Congrats_msg, kScienceScriptMessage );
				fChildActor->mGotoState( typeid( tPlayerTeamWinnerState ) );
			break;
		}

	}
	else if ( timerID == kCheckVoicePlayingID )
	{
		if( fMissionInfo->mVoiceSampleStillPlaying() )// != 0 )//**********************check when voice samples are in
		{
			mSetTimedInterval( kTimerSecondsSet, kCheckVoicePlayingID, 1.0f );
		}
		else
		{
			mSetTimedInterval( kTimerSecondsSet, kDisplayMessageID, 0.2f );
		}
	}

	
}
void tPlayerTeamInitialState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	tTeamInfo* targetTeam= fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTargetTeam ) );
	
	if( !fOrderedToDamage )
	{
		fMissionInfo->mPlayVoiceSample( kSndTut4_FailedFiredEarly , kTrue );
		fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kTut4_FailedFiredEarly_msg );
		fChildActor->mGotoState( typeid( tPlayerTeamFiredEarlyState ) );
	}
	else if ( damagedShip->mGetTeam() == targetTeam->mGetTeam() )
	{
		fTargetDamaged = true;
	}
}
void tPlayerTeamInitialState::mOnFlashingButtonPressed()
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
	tShipIterator ship = playerTeam->mGetFirstShip();
	tShipInPlayEval inPlayEval;
	ship.mAddEvaluator( &inPlayEval );
	tShipInfo* playerShip = *ship;	

	fFlashingButtonPressed = true;
	playerShip->mSetShipCallbackEvents( kNotifyFlashingButtonPressed );
}
void tPlayerTeamInitialState::mOnDeath( tShipInfo* ship )
{
	tTeamInfo* targetTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTargetTeam ) );

	if( ship->mGetTeam() == targetTeam->mGetTeam() )
	{
		if( !fOrderedToDestroy )
		{
			fMissionInfo->mPlayVoiceSample( kSndTut4_FailedFiredEarly , kTrue );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID>( kPlayerTeam ), kTut4_FailedFiredEarly_msg );
			fChildActor->mGotoState( typeid( tPlayerTeamFiredEarlyState ) );
		}
		else
		{
			fTargetDestroyed = true;
		}
	}

}
void tPlayerTeamInitialState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	tTeamInfo* propTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPropTeam ) );
	if( propTeam )
	{
		if( tractorTarget->mGetTeam() == propTeam->mGetTeam() ) 
		{
			fMissionInfo->mPlayVoiceSample( kSndTut5_Failed, kTrue );
			fMissionInfo->mDisplayMessage(  static_cast< eTeamID>( kPlayerTeam ), kTut_Failed_msg );
			fChildActor->mGotoState( typeid( tPlayerTeamLoserState ) );
		}
	}
}
void tPlayerTeamInitialState::mOnShipStoresUpdate( tShipInfo* Ship, eShipUpdateType Type, eAccessShipStores_ItemTypes Item, eAccessShipStores_SubTypes SubItem, int32 Value )
{
	if( Item == kSmall_Mines )
	{
		if( Value < fNumberOfMines )
		{
			fNumberOfMinesDecreased = true;
		}
		else
		{
			fNumberOfMines = Value;
		}
	}
}