/********************************************************************************************************\

	File Name:	BaseVictory.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "Uni_10Criminals.h"


tClassAllocator<tBaseVictory> gBaseVictoryAllocator;

/*

	HELPER NOTE:	You must remember to initialize all of your victory attributes here.  You should
					do this for both constructors until informed otherwise.  For example:

	tTeam1BaseVictory::tTeam1BaseVictory( ) : fBSed( false ), field2( value ), field3 ( value ), etc...
	{}

*/
tBaseVictory::tBaseVictory() :
		fFreighterArrived(false),
		fISCShipsDestroyed(0),
		fSurrenderAccepted(false),
		fSurrenderOffered(false),
		fSignaled(false),
		fISCTalk(false)

{}
tBaseVictory::tBaseVictory( tTeamInfo* team ) : tVictoryCondition( team ),
		fFreighterArrived(false),
		fISCShipsDestroyed(0),
		fSurrenderAccepted(false),
		fSurrenderOffered(false),
		fSignaled(false),
		fISCTalk(false)
{}

/*

	HELPER NOTE:	Really, the only items in this function that you could change are the values passed into
					the tBaseVictoryState constructor.  These values are the capture bonus and prestige
					points awarded according to the player's victory status -- starting at
					kAstoundingVictory and working its way down to kDevastatingDefeat.

*/
void tBaseVictory::mSetupState()
{
	tBaseVictoryState* baseVictoryState = new tBaseVictoryState( 300, 225, 75, 0, -75 );
	mRegisterState( baseVictoryState );
	mGotoState( typeid( tBaseVictoryState ) );

//	mSetTimedInterval( kTimerSecondsSet, kRandChatID, 20.0f );
}

/*

	HELPER NOTE:	The following is simply an example of setting and modifying the current victory condition
					objectives.

	bool
	tTeam1BaseVictory::mBSed( void )
	{
		return fBSed;
	}
	void
	tTeam1BaseVictory::mSetBSed( bool x )
	{
		fBSed = x;
	}

*/

bool tBaseVictory::mFreighterArrived()
{
	return fFreighterArrived;
}
int32 tBaseVictory::mISCShipsDestroyed()
{
	return fISCShipsDestroyed;
}
bool tBaseVictory::mSurrenderAccepted()
{
	return fSurrenderAccepted;
}
bool tBaseVictory::mSurrenderOffered()
{
	return fSurrenderOffered;
}
bool tBaseVictory::mSignaled()
{
	return fSignaled;
}
bool tBaseVictory::mISCTalk()
{
	return fISCTalk;
}


//
//	Modifiers.
//
void tBaseVictory::mSetFreighterArrived(bool x)
{
	fFreighterArrived = x;
	fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kFreighterArrived_msg);
	tTeamInfo * playerteam = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(playerteam);
	tTeamInfo * organianteam = fMissionInfo->mGetTeamHandle((eTeamID) kOrganianTeam);
	Assert(organianteam);
	tShipIterator ship = organianteam->mGetFirstShip();
	playerteam->mGotoCommState((*ship)->mGetShipID(), kFreighterArrived);

}
void tBaseVictory::mIncISCShipsDestroyed()
{
	fISCShipsDestroyed++;
	if(fISCShipsDestroyed == 2)
	{
		tTeamInfo * playerteam = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
		Assert(playerteam);
		tTeamInfo * enemyteam = fMissionInfo->mGetTeamHandle((eTeamID) kEnemyTeam);
		Assert(enemyteam);
		tShipIterator ship = enemyteam->mGetFirstShip();
		for(ship; *ship!=NULL; ship++)
		{
			playerteam->mGotoCommState((*ship)->mGetShipID(), kOfferSurrender);
		}
	}
	
}
void tBaseVictory::mSetSurrenderAccepted(bool x)
{
	fSurrenderAccepted = x;
	CreateOrganian();
	mDisableWeapons();
	mSetTimedInterval( kTimerSecondsSet, kCountDownID, 30.0f );	
}
void tBaseVictory::mSetSurrenderOffered(bool x)
{
	fSurrenderOffered = x;
}
void tBaseVictory::mSetSignaled(bool x)
{
	fSignaled = x;
	CreateOrganian();
	mDisableWeapons();
	mSetTimedInterval( kTimerSecondsSet, kCountDownID, 10.0f );	
	tTeamInfo * organianteam = fMissionInfo->mGetTeamHandle((eTeamID) kOrganianTeam);
	tTeamInfo * playerteam = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(organianteam);
	Assert(playerteam);
	tShipIterator organia = organianteam->mGetFirstShip();
	playerteam->mGotoCommState(organia->mGetShipID(), kBlankCommState);

}
void tBaseVictory::mSetISCTalk(bool x)
{
	fISCTalk = x;
	tTeamInfo * playerteam = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(playerteam);
	tTeamInfo * enemyteam = fMissionInfo->mGetTeamHandle((eTeamID) kEnemyTeam);
	Assert(enemyteam);
	tShipIterator ship = enemyteam->mGetFirstShip();
	for(ship; *ship!=NULL; ship++)
	{
		playerteam->mGotoCommState((*ship)->mGetShipID(), kISCTalk);
	}
}

void tBaseVictory::mDisableWeapons()
{
	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kOrganianTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam,
		//Nme  = kEnemyTeam,
		//Org  = kOrganianTeam

		//	     Plyr  PNPC  Nme   Org
		/*Plyr*/ "--", "AI", "FI", "FI",
		/*PNPC*/ "AI", "--", "FI", "FI",
		/*Nme */ "FI", "FI", "--", "FI",
		/*Org */ "FI", "FI", "FI", "--",


	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
	
}

void tBaseVictory::CreateOrganian()
{
	int32 OrganiaX, OrganiaY;
	int32 PlayerX, PlayerY;
	int32 NewX, NewY;

	tTeamInfo * util = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerNPCTeam);
	Assert(util);
	tShipIterator organia = util->mGetFirstShip();
	(*organia)->mGetGridPosition(&OrganiaX, &OrganiaY);
	util = fMissionInfo->mGetTeamHandle((eTeamID) kPlayerTeam);
	Assert(util);

	tShipIterator player = util->mGetFirstShip();
	(*player)->mGetGridPosition(&PlayerX, &PlayerY);
	NewX = (PlayerX > OrganiaX) ? PlayerX - 1 : PlayerX + 1;
	NewY = (PlayerY > OrganiaY) ? PlayerY - 1 : PlayerY + 1;

//	fMissionInfo->mDebugMessage("Creating Organian");
	fMissionInfo->mCreateMapObject(*"^", NewX, NewY);
}