/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "CommonDefs.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "Freighter.h"
#include "ShipScriptDescription.h"
#include "Uni_10Criminals.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;


void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}

void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        // we're going to randomly pick 3 ships from the 7 non-ISC races --FS
        // sizes: large qship/aux to BCH.

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
		sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

		ShipList freighter_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassLargeFreighter);
        fShiplistCount = freighter_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for freighter filter completed. ships: %d\n", fShiplistCount);

		std::list< eEnhancedRoles >  freighterRolesList;
        freighterRolesList.push_back(kCargo);
        fShiplistCount = freighter_sl.Subset(freighterRolesList);
        gST.mDebug_fprintf("freighter roles subset: %d\n", fShiplistCount);
        gST.listShiplistContent(freighter_sl);
		}

		sl = gST.selectWarShipRoles( sl );
		ShipList CA_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyCruiser);
		shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(CA_sl);
		}

		ShipList BCH_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyBattlecruiser);
        fShiplistCount = BCH_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for BCH filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(BCH_sl);
		}

		if (BCH_sl.size() == 0) {
			gST.mDebug_fprintf("FAIL! Unable to select a BCH from shiplist. using CA shiplist\n");
			BCH_sl = CA_sl;
		}

		ShipList DN_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassDreadnought);
		shipClassTypeList.push_back(kEClassHeavyDreadnought);
        fShiplistCount = DN_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DN filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent(DN_sl);
		}

		if (DN_sl.size() == 0) {
			gST.mDebug_fprintf("FAIL! Unable to select a DN from shiplist. using BCH shiplist (which may be a copy of CA shiplist)\n");
			DN_sl = BCH_sl;
		}

        if ( (freighter_sl.size() > 0) && (BCH_sl.size() > 0) && (DN_sl.size() > 0) ) 
		{
			Ship freighter1, bch1, bch2, carrier_bch1, dn1;

			// freighter1
			freighter_sl.SelectRandomShip(freighter1);
            gST.mDebug_fprintf("freighter1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                freighter1.shipCode.c_str(), freighter1.bpv, freighter1.yearFirstAvail, freighter1.yearLastAvail);
            gST.mTrackOrions( freighter1 );

			mCreateShip(
                typeid( tFreighter ),
                freighter1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kFreighter,
                0, 0, -1, -1,
                NULL,
                kCanBeLargeQShip );

			// bch1
			BCH_sl.SelectRandomShip(bch1);
            gST.mDebug_fprintf("bch1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                bch1.shipCode.c_str(), bch1.bpv, bch1.yearFirstAvail, bch1.yearLastAvail);
            gST.mTrackOrions( bch1 );

			mCreateShip(
                typeid( tShipInfo ),
                bch1.shipCode.c_str(),
                kStartPosition_I,
                kNoMetaShipID,
                kNewBCHShip,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

			// carrier_bch1 -- not gonna bother, not with OP+'s style of doing things. --FS
			BCH_sl.SelectRandomShip(carrier_bch1);
            gST.mDebug_fprintf("carrier_bch1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                carrier_bch1.shipCode.c_str(), carrier_bch1.bpv, carrier_bch1.yearFirstAvail, carrier_bch1.yearLastAvail);
            gST.mTrackOrions( carrier_bch1 );

			mCreateShip(
                typeid( tShipInfo ),
                carrier_bch1.shipCode.c_str(),
                kStartPosition_I,
                kNoMetaShipID,
                kNewCarrier,
                0, 0, -1, -1,
                NULL,
                static_cast<eShipOptions>( kCanBeCarrier | kStartPositionCanBeOffset )  );

			// dn1
			DN_sl.SelectRandomShip(dn1);
            gST.mDebug_fprintf("dn1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dn1.shipCode.c_str(), dn1.bpv, dn1.yearFirstAvail, dn1.yearLastAvail);
            gST.mTrackOrions( dn1 );

			mCreateShip(
                typeid( tShipInfo ),
                dn1.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kNewDNShip,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

			// bch2
			BCH_sl.SelectRandomShip(bch2);
            gST.mDebug_fprintf("bch2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                bch2.shipCode.c_str(), bch2.bpv, bch2.yearFirstAvail, bch2.yearLastAvail);
            gST.mTrackOrions( bch2 );

			mCreateShip(
                typeid( tShipInfo ),
                bch2.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kNewBCH2Ship,
                0, 0, -1, -1,
                NULL,
                kStartPositionCanBeOffset );
			
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tPlayerNPCTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	
		typeid( tFreighter ),
		kMaxBPV,
		kClassFreighter,
		0,
		kStartPosition_H,
		kFreighter,
		0, 0, -1, -1, NULL,
		kCanBeLargeQShip );

	mCreateShip(	
		typeid( tShipInfo ),
		kMaxBPV,
		kClassHeavyBattlecruiser,
		0,
		kStartPosition_I,
		kNewBCHShip,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );

	mCreateShip(	
		typeid( tShipInfo ),
		kMaxBPV,
		kClassCarrier,
		0,
		kStartPosition_I,
		kNewCarrier,
		0, 0, -1, -1, NULL,
		static_cast<eShipOptions>( kCanBeCarrier | kStartPositionCanBeOffset ) );

	mCreateShip(	
		typeid( tShipInfo ),
		kMaxBPV,
		kClassDreadnought,
		0,
		kStartPosition_J,
		kNewDNShip,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );

	mCreateShip(	
		typeid( tShipInfo ),
		kMaxBPV,
		kClassHeavyBattlecruiser,
		0,
		kStartPosition_J,
		kNewBCH2Ship,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}

void tPlayerNPCTeam::mTeamSetupComplete()
{
	// have to hard code these, because of problems in
	// mGetGridPosition with the planet.
	int32 X = 20;
	int32 Y = 15;
	tShipIterator ship = mGetFirstShip();
	if(*ship != NULL)
	{
		tShipInfo * Organia = *(fMissionInfo->mGetTeamHandle((eTeamID) kOrganianTeam)->mGetFirstShip());
		if(Organia != NULL)
		{
			Organia->mGetGridPosition(&X, &Y);
			(*ship)->mSetFinalDestination(X, Y);
			(*ship)->mTrackTacticalDistance(SIGNAL_RADIUS, X, Y);
		}
		(*ship)->mSetShipThrust(15);
		(*ship)->mSetShipMaxSpeed(15);
		(*ship)->mSetShipAIControlled(kTrue);
	}

	
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlayerNPCTeam", shipScriptDescription );
}

