/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "Uni_10Criminals.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;


void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList(fTeam,  kDebDismissed_msg);
		break;

	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeat_msg );
		fMissionInfo->mAddToDebriefingList(fTeam,  kDebDismissed_msg);
		break;

	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( fTeam, kDisconnect_msg );
		break;
	}	
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{
	/*
		mCreateShip(	typeid( tPlayerShip ),
					kMaxBPV,
					kClassLightCruiser,
					0,
					kStartPosition_G,
					kPlayerShip4 );
		mCreateShip(	typeid( tPlayerShip ),
					kMaxBPV,
					kClassLightCruiser,
					0,
					kStartPosition_G,
					kPlayerShip5 );
		mCreateShip(	typeid( tPlayerShip ),
					kMaxBPV,
					kClassLightCruiser,
					0,
					kStartPosition_G,
					kPlayerShip6 );

  */
}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
	fMissionInfo->mDisplayMessage((eTeamID) kPlayerTeam, kFedStart_msg + mGetRace(), kNonCommScriptMessage );
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = -1;

	switch( fMissionScheduler.fVictoryLevel )
	{
		case kAstoundingVictory:
		case kVictory:
		case kDraw:
			fMissionScheduler.fCampaignEvent = tTeamInfo::tMissionScheduler::kRetire;
			break;
		default:
			fMissionScheduler.fCampaignEvent = tTeamInfo::tMissionScheduler::kLost;
			break;
	}	

	switch ( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
		fMissionScheduler.fMedal = kMedalMissionOne;
		break;
	}
	fMissionScheduler.fNextMissionTitle = "";
}