/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "MissionText.h"
#include "Uni_10Search.h"
#include "PaladinTeam.h"
#include "PlanetTeam.h"

void tPlayerTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kLog_timer, 0.1f );
	mSetTimedInterval( kTimerTurnsSet, kSitrep_timer, 0.3f );
}

void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanning_msg );
	}
}

void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if( !targetShip )
		return;

	if( targetShip->mGetTeam() != kPaladinTeam )
		return;

	switch( buttonID )
	{
	case kButton_ClaimSystem:
		{
			tTeamInfo* paladinTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPaladinTeam ) );
			if( !paladinTeam )
					return;

			tShipIterator paladinI = paladinTeam->mGetFirstShip();
			tShipInfo* paladin = *paladinI;
			if( !paladin )
				return;
			paladin->mClearAllCommButtons();

			mSetTimedInterval( kTimerTurnsSet, kPaladinThinks_timer, 0.4f );
		}
		break;

	default:
		break;
	}
}

void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if( ( !shipTo ) || ( !shipFrom ) )
		return;

	if( item != kTransScientists )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;
	
	fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), BeamingTeam_msg );

	// Player beamed scientists to some silly place
	if( ( shipTo->mGetTeam() != kPlanetTeam ) && ( !playerTeam->mBeamedToPlanet() ) )
	{
		playerTeam->mSetBeamedSilly();
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBeamedSomewhereSilly_msg );
		gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		return;
	}

	// Player beams to planet 2.
	if( shipTo->mGetCustomID() == tPlanetTeam::kPlanet2 ) 
	{
		playerTeam->mSetBeamedToPlanet();
		mSetTimedInterval( kTimerTurnsSet, kNothingHere_timer, 0.2f );
		return;
	}

	// Player beams to planet 1. Planet not scanned.
	if( ( shipTo->mGetCustomID() == tPlanetTeam::kPlanet1 ) && ( !playerTeam->mScannedPlanet1() ) )
	{
		playerTeam->mSetBeamedToPlanet();
		mSetTimedInterval( kTimerTurnsSet, kNothingHere_timer, 0.2f );
		return;
	}

	// Player beams to planet 1. Planet has been scanned.
	if( ( shipTo->mGetCustomID() == tPlanetTeam::kPlanet1 ) && ( playerTeam->mScannedPlanet1() ) )
	{
		mSetTimedInterval( kTimerTurnsSet, kFoundHomeworld_timer, 0.2f );
		return;
	}

	// Player beams team back up for some reason
	if( shipTo->mGetTeam() == kPlayerTeam )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBeamedBackUp_msg );
		gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		return;
	}
}

void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kLog_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCaptainsLog_msg, kNonCommScriptMessage );
		}
		break;

	case kSitrep_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInitialSitrep_msg );
		}
		break;

	case kNothingHere_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kNothingHere_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			return;
		}
		break;

	case kFoundHomeworld_timer:
		{
			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
					return;

			playerTeam->mSetDiscoveredHomeworld();
			
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kFoundHomeworld_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
		}
		break;

	case kPaladinThinks_timer:
		{
			tTeamInfo* paladinTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPaladinTeam ) );
			if( !paladinTeam )
					return;

			paladinTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAllied, kGoalIgnore, 1.0f );
			fChildActor->mSetRelations( static_cast<eTeamID>( kPaladinTeam ), kTeamAllied, kGoalIgnore, 1.0f );

			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kClaimSystem_2_msg );

			tShipIterator paladinI = paladinTeam->mGetFirstShip();
			tShipInfo* paladin = *paladinI;
			if( !paladin )
				return;

			tShipIterator playerShipsI = fChildActor->mGetFirstShip();
			tShipInfo* ship1 = *playerShipsI;
			if( !ship1 )
				return;

			paladin->mFollowEntity( ship1, kAbsolutePriority );
		}
		break;

	default:
		break;
	}
}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}

void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}

void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}

void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}



