/********************************************************************************************************\

	File Name:	AllyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AllyTeam.h"
#include "AllyTeamBaseState.h"
#include "AllyShip.h"
#include "ShipList_API.h"

tClassAllocator<tAllyTeam> gAllyTeamAllocator;

void tAllyTeam::mSetupState()
{
	mRegisterState( new tAllyTeamBaseState() );
	mGotoState( typeid( tAllyTeamBaseState ) );
}

void tAllyTeam::mSetupCommState()
{}

void tAllyTeam::mCreateShipsForTeam()
{
    // DN, CC, CA, CL, CL
    gST.mDebug_fprintf("--- Entering: tLyranTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        ShipList CA_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyCruiser);
        shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CA_sl);
        }

        ShipList CC_sl = CA_sl;
        {
        std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = CC_sl.Subset(leaderRolesList);
        gST.mDebug_fprintf("subset for CC (CA leader) eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CC_sl);
        }
        if (CC_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a CC from shiplist. using CA shiplist\n");
            CC_sl = CA_sl;
        }

        ShipList DN_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassDreadnought);
		shipClassTypeList.push_back(kEClassHeavyDreadnought);
        fShiplistCount = DN_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DN filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DN_sl);
        }
        if (DN_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a DN from shiplist. using CC shiplist\n");
            DN_sl = CC_sl;
        }

        ShipList CL_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CL_sl);
        }

        if (
            (DN_sl.size() > 0) &&
            (CC_sl.size() > 0) &&
            (CA_sl.size() > 0) &&
            (CL_sl.size() > 0)
        ) {
            Ship dn1, cc1, ca1, cl1, cl2;
            DN_sl.SelectRandomShip(dn1);
            gST.mDebug_fprintf("dn1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dn1.shipCode.c_str(), dn1.bpv, dn1.yearFirstAvail, dn1.yearLastAvail);
            gST.mTrackOrions( dn1 );

            CC_sl.SelectRandomShip(cc1);
            gST.mDebug_fprintf("cc1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cc1.shipCode.c_str(), cc1.bpv, cc1.yearFirstAvail, cc1.yearLastAvail);
            gST.mTrackOrions( cc1 );

            CA_sl.SelectRandomShip(ca1);
            gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

            CL_sl.SelectRandomShip(cl1);
            gST.mDebug_fprintf("cl1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cl1.shipCode.c_str(), cl1.bpv, cl1.yearFirstAvail, cl1.yearLastAvail);
            gST.mTrackOrions( cl1 );

            CL_sl.SelectRandomShip(cl2);
            gST.mDebug_fprintf("cl2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cl2.shipCode.c_str(), cl2.bpv, cl2.yearFirstAvail, cl2.yearLastAvail);
            gST.mTrackOrions( cl2 );

            mCreateShip(
                typeid( tAllyShip ),
                dn1.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kAlly1,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip ),
                cc1.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kAlly2,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip ),
                ca1.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kAlly3,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip ),
                cl1.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kAlly4,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip ),
                cl2.shipCode.c_str(),
                kStartPosition_J,
                kNoMetaShipID,
                kAlly5,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tAllyTeam::mCreateShipsForTeam\n");
}


void tAllyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tAllyTeam::mOldCreateShipsForTeam\n");

	// DN, CA, CA, CL, CL
	mCreateShip( typeid( tAllyShip ),
				 220,
				 kClassDreadnought,
				 0,
				 kStartPosition_J,
				 kAlly1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 200,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_J,
				 kAlly2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 200,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_J,
				 kAlly3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 160,
				 kClassLightCruiser,
				 0,
				 kStartPosition_J,
				 kAlly4,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip ),
				 160,
				 kClassLightCruiser,
				 0,
				 kStartPosition_J,
				 kAlly5,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tAllyTeam::mOldCreateShipsForTeam\n");
}

void tAllyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tAllyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tAllyTeam", shipScriptDescription );
}

