/********************************************************************************************************\

	File Name:	MirakTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MirakTeam.h"
#include "MirakTeamBaseState.h"
#include "MirakShip.h"
#include "Uni_11Wedding.h"
#include "ShipList_API.h"


tClassAllocator<tMirakTeam> gMirakTeamAllocator;

void tMirakTeam::mSetupState()
{
	mRegisterState( new tMirakTeamBaseState() );
	mGotoState( typeid( tMirakTeamBaseState ) );
}

void tMirakTeam::mSetupCommState()
{}

void tMirakTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tMirakTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        ShipList CA_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyCruiser);
        shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CA_sl);
        }

        ShipList CL_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CL_sl);
        }

        ShipList DD_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassDestroyer);
        shipClassTypeList.push_back(kEClassWarDestroyer);
        fShiplistCount = DD_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DD_sl);
        }

		// CA, CL, DD, DD
        if (
            (CA_sl.size() > 0) &&
            (CL_sl.size() > 0) &&
            (DD_sl.size() > 0)
        ) {
            Ship ca1, cl1, dd1, dd2;
            CA_sl.SelectRandomShip(ca1);
            gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

            CL_sl.SelectRandomShip(cl1);
            gST.mDebug_fprintf("cl1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cl1.shipCode.c_str(), cl1.bpv, cl1.yearFirstAvail, cl1.yearLastAvail);
            gST.mTrackOrions( cl1 );

            DD_sl.SelectRandomShip(dd1);
            gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
            gST.mTrackOrions( dd1 );

            DD_sl.SelectRandomShip(dd2);
            gST.mDebug_fprintf("dd2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd2.shipCode.c_str(), dd2.bpv, dd2.yearFirstAvail, dd2.yearLastAvail);
            gST.mTrackOrions( dd2 );

            mCreateShip(
                typeid( tMirakShip ),
                ca1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kMirak1,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tMirakShip ),
                cl1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kMirak4,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tMirakShip ),
                dd1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kMirak5,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tMirakShip ),
                dd2.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kMirak6,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tMirakTeam::mCreateShipsForTeam\n");
}

void tMirakTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tMirakTeam::mOldCreateShipsForTeam\n");

	// CA, CL, DD, DD
	mCreateShip( typeid( tMirakShip ),
				 200,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_H,
				 kMirak1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tMirakShip ),
				 160,
				 kClassLightCruiser,
				 0,
				 kStartPosition_H,
				 kMirak4,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tMirakShip ),
				 140,
				 kClassDestroyer,
				 0,
				 kStartPosition_H,
				 kMirak5,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );	

	mCreateShip( typeid( tMirakShip ),
				 140,
				 kClassDestroyer,
				 0,
				 kStartPosition_H,
				 kMirak6,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );	

	gST.mDebug_fprintf("--- Finished: tMirakTeam::mOldCreateShipsForTeam\n");
}

void tMirakTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tMirakTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tMirakTeam", shipScriptDescription );
}

