/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_1Desecration.h"
#include "AsteroidTeam.h"
#include "AsteroidShip.h"
#include "EnemyTeam.h"
#include "CoffinTeam.h"
#include "EnemyShip.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fGateCreated = false;	
}


void tBaseVictoryState::mBeginState()
{
	
}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

		// Found gate & all asteroids scanned
		if( playerTeam->mGateTransported() )
		{
			return kAstoundingVictory;
		}

		// Any other condition
		return kDevastatingDefeat;
}


void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;


	tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
	if( !enemyTeam )
		return;


	tTeamInfo* coffinTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kCoffinTeam ) );
	if( !coffinTeam )
		return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					playerTeam->mSetDied();
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

	case tEnemyTeam::kEnemy1:
	case tEnemyTeam::kEnemy2:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
				playerTeam->mSetAllEnemiesDead();
		}
		break;

	case tCoffinTeam::kCoffin1:
	case tCoffinTeam::kCoffin2:
	case tCoffinTeam::kCoffin3:
	case tCoffinTeam::kCoffin4:
	case tCoffinTeam::kCoffin5:
	case tCoffinTeam::kCoffin6:
		{
			if( !enemyTeam->mAtWar() )
			{
				if( enemyTeam->mShipsRemainingInPlay() )
				{
					tShipIterator enemiesI = enemyTeam->mGetFirstShip();
					tShipInfo* enemy1 = *enemiesI;
					enemiesI++;
					tShipInfo* enemy2 = *enemiesI;
					
					enemyTeam->mSetAtWar();
					enemyTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
					playerTeam->mSetRelations( static_cast<eTeamID>( kEnemyTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
					
					if( enemy1 ) {	playerTeam->mGotoCommState( enemy1->mGetShipID(), tEnemyShip::kCommState_4 ); }
					if( enemy2 ) {	playerTeam->mGotoCommState( enemy2->mGetShipID(), tEnemyShip::kCommState_4 ); }
					
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMirakGoToWar_msg );
				}
			}

			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCoffinDestroyed_msg );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
	if( !enemyTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
		}
		break;

	case tEnemyTeam::kEnemy1:
	case tEnemyTeam::kEnemy2:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllEnemiesDead_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kEnemyCapped_msg );
			}
		}
		break;

	case tCoffinTeam::kCoffin1:
	case tCoffinTeam::kCoffin2:
	case tCoffinTeam::kCoffin3:
	case tCoffinTeam::kCoffin4:
	case tCoffinTeam::kCoffin5:
	case tCoffinTeam::kCoffin6:
		{
			if( !enemyTeam->mAtWar() )
			{
				if( enemyTeam->mShipsRemainingInPlay() )
				{
					tShipIterator enemiesI = enemyTeam->mGetFirstShip();
					tShipInfo* enemy1 = *enemiesI;
					enemiesI++;
					tShipInfo* enemy2 = *enemiesI;
					
					enemyTeam->mSetAtWar();
					enemyTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
					playerTeam->mSetRelations( static_cast<eTeamID>( kEnemyTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
					
					if( enemy1 ) {	playerTeam->mGotoCommState( enemy1->mGetShipID(), tEnemyShip::kCommState_4 ); }
					if( enemy2 ) {	playerTeam->mGotoCommState( enemy2->mGetShipID(), tEnemyShip::kCommState_4 ); }
					
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMirakGoToWar_msg );
				}
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );
	if( !enemyTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}

void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if( ( !damagedShip ) || ( !hostileShip ) )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* asteroidTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAsteroidTeam ) );
	if( !asteroidTeam )
		return;

	tAsteroidShip* asteroid3 = dynamic_cast<tAsteroidShip*>( asteroidTeam->mGetThisShip(3) );

	if( damagedShip->mGetShipID() != asteroid3->mGetShipID() )
		return;

	if( ( asteroid3 ) && ( damagedShip->mGetShipID() == asteroid3->mGetShipID() ) && ( playerTeam->mScannedAsteroid(3) ) && ( !fGateCreated ) )
	{
		fGateCreated = true;
		asteroid3->mCreateGate();
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAsteroidPhasered_msg );
	}
}


void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}

void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

