/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "PlayerTeamBaseState.h"
#include "Uni_1Desecration.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "AsteroidTeam.h"
#include "AsteroidShip.h"
#include "CoffinTeam.h"
#include "PlanetTeam.h"
#include "EnemyShip.h"

#define ScanRangeAsteroid	7


tPlayerShipBaseState::tPlayerShipBaseState()
{
	fScanning = false;	
}

void tPlayerShipBaseState::mBeginState()
{
}

void tPlayerShipBaseState::mEndState()
{}

void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fScanning = true;
		mSetTimedInterval( kTimerTurnsSet, kScanTimer, 0.75f );
	}

	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, kScanTimer, 0.0f );
	}
}

void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* asteroidTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAsteroidTeam ) );
	if( !asteroidTeam )
		return;

	switch( timerID )
	{

	case kScanTimer:
		{
			if( !fScanning )
				return;
			mSetTimedInterval( kTimerTurnsSet, kScanTimer, 0.75f );
									
			// Target of scan
			int32 ScanTargetID = fChildActor->mGetShipTargetID();
			
			// Scanning asteroids
			tShipIterator asteroidsI = asteroidTeam->mGetFirstShip();
			tShipInfo* asteroid1 = *asteroidsI;
			asteroidsI++;
			tShipInfo* asteroid2 = *asteroidsI;
			asteroidsI++;
			//tAsteroidShip* asteroid3 = dynamic_cast<tAsteroidShip*>( asteroidTeam->mGetThisShip(3) );
			tShipInfo* asteroid3 = *asteroidsI;
			asteroidsI++;
			tShipInfo* asteroid4 = *asteroidsI;
			int32 RangeToAsteroid1, RangeToAsteroid2, RangeToAsteroid3, RangeToAsteroid4;
			
			if( asteroid1 ) { RangeToAsteroid1 = fChildActor->mGetTacticalDistance( asteroid1 ); }
			if( asteroid2 ) { RangeToAsteroid2 = fChildActor->mGetTacticalDistance( asteroid2 ); }
			if( asteroid3 ) { RangeToAsteroid3 = fChildActor->mGetTacticalDistance( asteroid3 ); }
			if( asteroid4 ) { RangeToAsteroid4 = fChildActor->mGetTacticalDistance( asteroid4 ); }
			
			// Asteroid 1
			if( ( asteroid1 ) && ( asteroid1->mGetShipID() == ScanTargetID ) && ( RangeToAsteroid1 < ScanRangeAsteroid ) && ( !playerTeam->mScannedAsteroid(1) ) )
			{
				playerTeam->mSetAsteroidScanned(1);
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedNormalAsteroid_msg );
				
				if( ( playerTeam->mGateTransported() ) && ( playerTeam->mGetNumAsteroidsScanned() == 4 ) && ( playerTeam->mGetRace() != kLyran ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPrimaryComplete_msg );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
				return;
			}
			
			// Asteroid 2
			if( ( asteroid2 ) && ( asteroid2->mGetShipID() == ScanTargetID ) && ( RangeToAsteroid2 < ScanRangeAsteroid ) && ( !playerTeam->mScannedAsteroid(2) ) )
			{
				playerTeam->mSetAsteroidScanned(2);
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedNormalAsteroid_msg );
				
				if( ( playerTeam->mGateTransported() ) && ( playerTeam->mGetNumAsteroidsScanned() == 4 ) && ( playerTeam->mGetRace() != kLyran ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPrimaryComplete_msg );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
				return;
			}
			
			// Asteroid 3 - special asteroid
			if( ( asteroid3 ) && ( asteroid3->mGetShipID() == ScanTargetID ) && ( RangeToAsteroid3 < ScanRangeAsteroid ) && ( !playerTeam->mScannedAsteroid(3) ) )
			{
				playerTeam->mSetAsteroidScanned(3);
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedSpecialAsteroid_msg );
				
				if( ( playerTeam->mGateTransported() ) && ( playerTeam->mGetNumAsteroidsScanned() == 4 ) && ( playerTeam->mGetRace() != kLyran ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPrimaryComplete_msg );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
				return;
			}
			
			// Asteroid 4
			if( ( asteroid4 ) && ( asteroid4->mGetShipID() == ScanTargetID ) && ( RangeToAsteroid4 < ScanRangeAsteroid ) && ( !playerTeam->mScannedAsteroid(4) ) )
			{
				playerTeam->mSetAsteroidScanned(4);
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedNormalAsteroid_msg );
				
				if( ( playerTeam->mGateTransported() ) && ( playerTeam->mGetNumAsteroidsScanned() == 4 ) && ( playerTeam->mGetRace() != kLyran ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kPrimaryComplete_msg );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
				}
				return;
			}
		}
		break;
		
	}
}

void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	
}

void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
		return;

	tEnemyTeam* enemyTeam = dynamic_cast<tEnemyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) ) );
	if( !enemyTeam )
		return;

	tShipIterator enemiesI = enemyTeam->mGetFirstShip();
	tShipInfo* enemy1 = *enemiesI;
	enemiesI++;
	tShipInfo* enemy2 = *enemiesI;

	if( !enemyTeam->mAtWar() )
	{
		enemyTeam->mSetAtWar();
		enemyTeam->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
		playerTeam->mSetRelations( static_cast<eTeamID>( kEnemyTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
		if( enemy1 ) {	playerTeam->mGotoCommState( enemy1->mGetShipID(), tEnemyShip::kCommState_4 ); }
		if( enemy2 ) {	playerTeam->mGotoCommState( enemy2->mGetShipID(), tEnemyShip::kCommState_4 ); }
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMirakGoToWar_msg );
	}

}

void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}

void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}

void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}

void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

