/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fScannedAsteroid1 = false;
	fScannedAsteroid2 = false;
	fScannedAsteroid3 = false;
	fScannedAsteroid4 = false;
	fAsteroidsScanned = 0;
	fEnemiesKilled = 0;
	fCoffinsKilled = 0;
	fGateTransported = false;
	fScanRangeNoticeGiven = false;
	fAllEnemiesDead = false;
	fDied = false;
}

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}


void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( fDied )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_PlayerDied_msg );
		mScheduleNextMission( fVictoryCondition );
		return;
	}

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictory_msg );		break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeat_msg );		break;
	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDeb_LeftEarly_msg );
			mScheduleNextMission( fVictoryCondition );
			return;
		}
		break;
	}

	if( mGateTransported() )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_GateFound );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_GateNotFound );
	}

	if( mGetRace() == kLyran )
	{
		if( fAllEnemiesDead )
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDeb_MirakDead_msg );
		}
	}

	mScheduleNextMission( fVictoryCondition );
}


void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}


void tPlayerTeam::mCreateShipsForTeam()
{
	
}


void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}


iTruth tPlayerTeam::mScannedAsteroid(int32 asteroid)
{
	switch( asteroid )
	{
	case 1:
		return fScannedAsteroid1;
		break;

	case 2:
		return fScannedAsteroid2;
		break;

	case 3:
		return fScannedAsteroid3;
		break;

	case 4:
		return fScannedAsteroid4;
		break;

	default:
		return false;
		break;
	}
}

void tPlayerTeam::mSetAsteroidScanned(int32 asteroid)
{
	fAsteroidsScanned++;

	switch( asteroid )
	{
	case 1:
		fScannedAsteroid1 = true;
		break;

	case 2:
		fScannedAsteroid2 = true;
		break;

	case 3:
		fScannedAsteroid3 = true;
		break;

	case 4:
		fScannedAsteroid4 = true;
		break;
	}
}

int32 tPlayerTeam::mGetNumAsteroidsScanned()
{
	return fAsteroidsScanned;
}

void tPlayerTeam::mIncrementAsteroidsScanned()
{
	fAsteroidsScanned++;
}

int32 tPlayerTeam::mGetEnemiesKilled()
{
	return fEnemiesKilled;
}

void tPlayerTeam::mIncrementEnemiesKilled()
{
	fEnemiesKilled++;
}

int32 tPlayerTeam::mGetCoffinsKilled()
{
	return fCoffinsKilled;
}

void tPlayerTeam::mIncrementCoffinsKilled()
{
	fCoffinsKilled++;
}

iTruth tPlayerTeam::mGateTransported()
{
	return fGateTransported;
}

void tPlayerTeam::mSetGateTransported()
{
	fGateTransported = true;
}

iTruth tPlayerTeam::mScanRangeNoticeGiven()
{
	return fScanRangeNoticeGiven;
}

void tPlayerTeam::mSetScanRangeNoticeGiven()
{
	fScanRangeNoticeGiven = true;
}

int32 tPlayerTeam::mGetAsteroidsScanned()
{
	return fAsteroidsScanned;
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 400;
	fMissionScheduler.fNextMissionTitle = "Uni_2Dance";
}