/********************************************************************************************************\

	Mission:	"Desecration"
	Script:		"Uni_1Desecration"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno
	Last Updated:	FS				10/14/2012
					era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_1Desecration.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "AsteroidTeam.h"
#include "CoffinTeam.h"
#include "PlanetTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_1Desecration );

tUni_1Desecration::tUni_1Desecration()
{}

int32 tUni_1Desecration::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_1Desecration::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kMissionTitle_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );


	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Asteroid Team",
												static_cast< eTeamID >( kAsteroidTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tAsteroidTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Coffin Team",
												static_cast< eTeamID >( kCoffinTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tCoffinTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( draft1Team );
}

int32 tUni_1Desecration::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_1Desecration::mCreateExternalShips( int32 Mode )
{}

int32 tUni_1Desecration::mInitializeFinished( void )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	ASSERT( playerTeam );
	if( !playerTeam ) return kBriefing_Other_msg;

	if( playerTeam->mGetRace() == kLyran )
		return kBriefing_Lyran_msg;

	else
		return kBriefing_Other_msg;
}

void tUni_1Desecration::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_1Desecration::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_1Desecration::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_1Desecration::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kAsteroidTeam,
		kCoffinTeam,
		kPlanetTeam,
	};


	// If player == Lyran, Mirak (enemy) team begins at war

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
		return;
	if( playerTeam->mGetRace() == kLyran )
	{
		std::string relationships[kRelationsCount][kRelationsCount] = 
		{
			//Plyr = kPlayerTeam,
			//Nme  = kEnemyTeam,
			//Astr = kAsteroidTeam,
			//Coff = kCoffinTeam,
			//Plan = kPlanetTeam,
			
			//	     Plyr  Nme   Astr  Coff  Plan
			/*Plyr*/ "--", "WD", "NI", "WD", "AI",
			/*Nme*/  "WD", "--", "NI", "AI", "NI",
			/*Astr*/ "PI", "PI", "--", "PI", "PI",
			/*Coff*/ "WD", "AI", "PI", "--", "PI",
			/*Plan*/ "AI", "AI", "PI", "PI", "--",
		};

		fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
	}

	else
	{
		std::string relationships[kRelationsCount][kRelationsCount] = 
		{
			//Plyr = kPlayerTeam,
			//Nme  = kEnemyTeam,
			//Astr = kAsteroidTeam,
			//Coff = kCoffinTeam,
			//Plan = kPlanetTeam,
			
			//	     Plyr  Nme   Astr  Coff  Plan
			/*Plyr*/ "--", "HI", "NI", "HI", "AI",
			/*Nme*/  "HI", "--", "NI", "AI", "NI",
			/*Astr*/ "PI", "PI", "--", "PI", "PI",
			/*Coff*/ "HI", "AI", "PI", "--", "PI",
			/*Plan*/ "AI", "AI", "PI", "PI", "--",
		};

		fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
	}
	
	
}


void tUni_1Desecration::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlanet;
	Map->fTacticalMapIcon['K'] = kObjectAsteroid;
	Map->fTacticalMapIcon['L'] = kObjectAsteroid;
	Map->fTacticalMapIcon['M'] = kObjectAsteroid;
	Map->fTacticalMapIcon['N'] = kObjectAsteroid;
}

tMapTerrain tUni_1Desecration::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_1Desecration::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tUni_1Desecration::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_Lyran_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_Other_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_Other_msg;
}