//	<Name>	ISCTeamInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ISCTeamInitialState.h"
#include "ISCTeamArrivalState.h"
#include "ScriptInterface.h"
#include "MissionInfo.h"
#include "TeamInfo.h"
#include "PlayerTeamShipInitialState.h"
#include "PlayerTeamShipOrgArriveState.h"
#include "ISCTeamShips.h"
#include "Uni_1Peace.h"
#include "ShipList_API.h"

void tISCTeamInitialState::mBeginState()
{}

void tISCTeamInitialState::mOnTimedInterval( int32 timerID )
{}

void tISCTeamInitialState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{
	if( 
		( *stateID == typeid( tPlayerTeamShipOrgArriveState ) ) &&				// Create the ISC Ships
		(eventType == kStartingNewState) 
	) {
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 iSCBPV;
		int32 iSCSmallBPV;

		if( diffLevel == 0 )
		{
			iSCBPV = playerBPV * 1.75;
		}
		else if( diffLevel == 1 )
		{
			iSCBPV = playerBPV * 2.0;
		}
		else if( diffLevel == 2 )
		{
			iSCBPV = playerBPV * 2.0;
		}
		iSCSmallBPV = ( iSCBPV * 0.75 ); 

		mCreateISCShips(iSCSmallBPV, iSCBPV);
		fChildActor->mGotoState( typeid( tISCTeamArrivalState() ) ); // to upgrade the fighters, if any
	}
}

void tISCTeamInitialState::mCreateISCShips( int32 iSCSmallBPV, int32 iSCBPV )
{
	gST.mDebug_fprintf("--- Starting: tISCTeamInitialState::mCreateISCShips\n");

    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = fChildActor->mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, race );
        sl = gST.stripOutRestricted( sl );
        sl = gST.stripOutSPECIAL( sl );
        ShipList ship_sl = gST.selectWarShips(sl);

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        ShipList small_sl = sl;
        ShipList large_sl = sl;

        fShiplistCount = small_sl.Subset( small_sl.kSubsetBPV, int(iSCSmallBPV * 0.825), int(iSCSmallBPV * 1.075), true);
        gST.mDebug_fprintf("small_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(iSCSmallBPV * 0.825), int(iSCSmallBPV * 1.075), fShiplistCount);
		gST.listShiplistContent(small_sl);

        fShiplistCount = large_sl.Subset( large_sl.kSubsetBPV, int(iSCBPV * 0.825), int(iSCBPV * 1.075), true);
        gST.mDebug_fprintf("large_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(iSCBPV * 0.825), int(iSCBPV * 1.075), fShiplistCount);
		gST.listShiplistContent(large_sl);

        if ((small_sl.size() > 0) && (large_sl.size() > 0)) {
			Ship s1, s2;
            small_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            large_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

		fChildActor->mCreateShip( typeid( tISCTeamShips ), 
			s1.shipCode.c_str(),
			kStartPosition_W,
			tTeamInfo::kNoMetaShipID,
			-1, 
			0, 0, -1, -1, 
			NULL, 
			kDefaultShipOptions );

		fChildActor->mCreateShip( typeid( tISCTeamShips ), 
			s2.shipCode.c_str(), 
			kStartPosition_X,
			tTeamInfo::kNoMetaShipID,
			-1, 
			0, 0, -1, -1, 
			NULL, 
			kDefaultShipOptions );
		} else {
			mOldCreateISCShips(iSCSmallBPV, iSCBPV);
		}
	} else {
		mOldCreateISCShips(iSCSmallBPV, iSCBPV);
	}

	gST.mDebug_fprintf("--- Finished: tISCTeamInitialState::mCreateISCShips\n");
}

void tISCTeamInitialState::mOldCreateISCShips( int32 iSCSmallBPV, int32 iSCBPV )
{
	gST.mDebug_fprintf("---- Starting: tISCTeamInitialState::mOldCreateISCShips\n");

	fChildActor->mCreateFleet( typeid( tISCTeamShips ), 
		1, 
		iSCSmallBPV, 
		kClassFrigate, 
		kClassHeavyCruiser, 
		0, 
		kStartPosition_W,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	fChildActor->mCreateFleet( typeid( tISCTeamShips ), 
		1, 
		iSCBPV, 
		kClassLightCruiser, 
		kClassDreadnought, 
		0, 
		kStartPosition_X,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tISCTeamInitialState::mOldCreateISCShips\n");
}