#include "Uni_1Peace.h"

#include "MissionText.h"
#include "MissionMaps.h"

#include "VictoryCondition.h"

#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
//#include "eracename.h"
#include "NPC1Team.h"
#include "NPC2Team.h"
#include "NPC3Team.h"
#include "OrganianTeam.h"
#include "ISCTeam.h"
#include "PeaceVictory.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tUni_1Peace );

tUni_1Peace::tUni_1Peace()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tUni_1Peace::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

		// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID >( kPlayerTeam ),
															kMaxChance,
															kTrue,
															kPrimaryTeam,
															kMinChance,
															kSpecPlayableRace & ~kSpecISC,
															kAnyRace,
															typeid( tPlayerTeam ), kNoTeamTag, Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID >( kEnemyTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecISC,
															kAnyRace,
															typeid( tISCTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", static_cast< eTeamID >( kNPC1Team ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecFed | kSpecLyran| kSpecHydran,
															kDifferentThan | kPlayerTeam,
															typeid( tNPC1Team ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", static_cast< eTeamID >( kNPC2Team ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecRomulan| kSpecGorn,
															kDifferentThan | kPlayerTeam,
															typeid( tNPC2Team ) );


	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5", static_cast< eTeamID >( kNPC3Team ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecKling | kSpecMirak,
															kDifferentThan | kPlayerTeam,
															typeid( tNPC3Team ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6", static_cast< eTeamID >( kOrganianTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecISC,
															kAnyRace,
															typeid( tOrganianTeam ) );


	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );

}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tUni_1Peace::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tUni_1Peace::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tUni_1Peace::mCreateExternalShips( int32 Mode )
{

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tUni_1Peace::mInitializeFinished( void )
{
	eRaceName  playerRace;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
    playerRace = playerTeam->mGetRace();

	// return the index to the briefing message we want to use
	switch( playerRace )
	{
		case kFederation:
			return( kFedBriefing_msg );
		break;
		case kKlingon:
			return( kKliBriefing_msg );
		break;
		case kRomulan:
			return( kRomBriefing_msg );
		break;
		case kLyran:
			return( kLyrBriefing_msg );
		break;
		case kGorn:
			return( kGorBriefing_msg );
		break;
		case kMirak:
			return( kMirakBriefing_msg );
		break;
		case kHydran:
			return( kHydBriefing_msg );
		break;
		default:
			Assert( 0 );
			return( kHydBriefing_msg );
		break;
	}

}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tUni_1Peace::mStartMission( void )
{

	tScript::mStartMission();

	// setup team relations and target goals
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kNPC1Team,
		kNPC2Team,
		kNPC3Team,
		kOrganianTeam
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr	= kPlayerTeam,
		//Enemy = kEnemyTeam,
		//NPC1  = kNPC1Team,
		//NPC2  = kNPC2Team,
		//NPC3  = kNPC3Team,
		//Orgn	= kOrganian

		//			Plyr	Enemy	NPC1	NPC2	NPC3	Organian  
		/*Plyr*/	"--",	"HI",	"HI",	"HI",	"HI",	"AI",
		/*Enemy*/	"HI",	"--",	"HI",	"HI",	"HI",	"AI",
		/*NPC1*/	"HI",	"HI",	"--",	"HI",	"HI",	"AI",
		/*NPC2*/	"HI",	"HI",	"HI",	"--",	"HI",	"AI",
		/*NPC3*/	"HI",	"HI",	"HI",	"HI",	"--",	"AI",
		/*Orgn*/	"AI",	"AI",	"AI",	"AI",	"AI",	"--",	
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tUni_1Peace::mNewInternalShip( tShipInfo* Ship )
{
}

void tUni_1Peace::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;


	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;
}
void tUni_1Peace::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tUni_1Peace::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] =	kFedBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] =		kKliBriefing_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] =		kRomBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] =		kLyrBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] =		kHydBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] =		kGorBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] =		kMirakBriefing_msg;
}

