/********************************************************************************************************\

	Mission:	"Dilithium Dance"
	Script:		"Uni_2Dance"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		08/02/00
	Last Updated:	FS				10/14/2012
					era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_2Dance.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "LyranTeam.h"
#include "PulsarTeam.h"
#include "DilithiumTeam.h"
#include "MineTeam.h"
#include "MapConstants.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tUni_2Dance );

tUni_2Dance::tUni_2Dance()
{}

int32 tUni_2Dance::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_2Dance::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Ally Team",
												static_cast< eTeamID >( kAllyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tAllyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Mirak Team",
												static_cast< eTeamID >( kLyranTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tLyranTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Pulsar Team",
												static_cast< eTeamID >( kPulsarTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tPulsarTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Dilithium Team",
												static_cast< eTeamID >( kDilithiumTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tDilithiumTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Mine Team",
												static_cast< eTeamID >( kMineTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tMineTeam ) );

	

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

int32 tUni_2Dance::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_2Dance::mCreateExternalShips( int32 Mode )
{}

int32 tUni_2Dance::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_2Dance::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_2Dance::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_2Dance::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_2Dance::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kAllyTeam,
		kLyranTeam,
		kDilithiumTeam,
		kDilithiumTeam,
		kMineTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Ally = kAllyTeam,
		//Lyrn = kLyranTeam,
		//Puls = kPulsarTeam,
		//Dili = kDilithiumTeam,
		//Mine = kMineTeam,

		//	     Plyr  Ally  Lyrn  Puls  Dili  Mine
		/*Plyr*/ "--", "AI", "WD", "NI", "AI", "AI",
		/*Ally*/ "AI", "--", "HI", "NI", "NI", "NI",
		/*Lyrn*/ "WD", "WD", "--", "NI", "NI", "NI",
		/*Puls*/ "NI", "NI", "NI", "--", "NI", "NI",
		/*Dili*/ "AI", "AI", "NI", "NI", "--", "AI",
		/*Mine*/ "AI", "AI", "NI", "NI", "AI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tUni_2Dance::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['X'] = kObjectWormHole;
	Map->fTacticalMapIcon['W'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['Y'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
}

tMapTerrain tUni_2Dance::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_2Dance::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
	
}

void tUni_2Dance::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_msg;
}