//	<Name>	DefPlatformTeamShipInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Uni_2Promises.h"
#include "ShipInfo.h"
#include "DefPlatformTeamShip.h"
#include "DefPlatformTeamShipInitialState.h"
#include "DefPlatformTeamShipScannedState.h"
#include "DefPlatformTeamShipDiplomacyState.h"

#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


tDefPlatformTeamShipInitialState::tDefPlatformTeamShipInitialState( )
{
	fDeepScanON = false;
}

void tDefPlatformTeamShipInitialState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kCheckDistanceDefensePlatID, 0.75f );

    gST.mDebug_fprintf("--- Starting: tDefPlatformTeamShipInitialState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tDefPlatformTeamShipInitialState::mBeginState\n");
}

void tDefPlatformTeamShipInitialState::mEndState()
{}

void tDefPlatformTeamShipInitialState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fDeepScanON = true;
	}
	else
	{
		fDeepScanON = false;
	}
}

void tDefPlatformTeamShipInitialState::mOnTimedInterval( int32 timerID )
{
	int32 rangeThisSecond = 0;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tShipIterator ship = playerTeam->mGetFirstShip();
	tShipInfo* playerShip = *ship;
	bool dimplomacyStarted = fChildActor->mGetDiplomacyStarted();

	if( timerID == kCheckDistanceDefensePlatID )
	{
		for( playerShip; playerShip != NULL; playerShip = *( ++ship ) )
		{
			rangeThisSecond = fChildActor->mGetTacticalDistance( playerShip );

			if( !dimplomacyStarted && ( rangeThisSecond < 10 ) && ( fDeepScanON ) && ( ( fChildActor->mGetShipID() ) == ( playerShip->mGetShipTargetID() ) ) )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kScannedDefPlatform_msg, kScienceScriptMessage );
				fChildActor->mGotoState( typeid( tDefPlatformTeamShipScannedState ) );
			}
			else if( ( rangeThisSecond < 4 ) && dimplomacyStarted && ( ( playerShip->mGetAlertStatus() ) == kAlertGreen ) )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kDiplomacyPrevails_msg );
				mSetTimedInterval( kTimerSecondsSet, kDiplomacyID, 3.0f );
			}
		}// end for playerShip

		mSetTimedInterval( kTimerTurnsSet, kCheckDistanceDefensePlatID, 0.5f );

	}//timerID = kCheckDistanceID
	else if ( timerID == kDiplomacyID )
	{
		fChildActor->mGotoState( typeid( tDefPlatformTeamShipDiplomacyState ) );
	}

	if (timerID == kUpgradeFightersID) {
    #ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
        if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tDefPlatformTeamShipInitialState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
        int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
        gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tDefPlatformTeamShipInitialState::mOnTimedInterval -- timer kUpgradeFightersID\n");
    #endif
    }

}
void tDefPlatformTeamShipInitialState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if( damagedShip->mGetShipID() == fChildActor->mGetShipID() )
	{
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), playerTeam->mGetTeam(), kTeamAtWar, kGoalDestroy, 1.0, kTrue );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kMineControllerTeam ), playerTeam->mGetTeam(), kTeamHostile, kGoalIgnore, 1.0, kTrue );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kDefPlatformTeam ), playerTeam->mGetTeam(), kTeamAtWar, kGoalDestroy, 1.0, kTrue );

		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kFreighterTeam ), kTeamAtWar, kGoalDestroy, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kMineControllerTeam ), static_cast< eTeamID >( kFreighterTeam ), kTeamHostile, kGoalIgnore, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kDefPlatformTeam ), static_cast< eTeamID >( kFreighterTeam ), kTeamAtWar, kGoalDestroy, 1.0, kFalse );

		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kFreighterTeam ), static_cast< eTeamID >( kEnemyTeam ), kTeamAtWar, kGoalIgnore, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kFreighterTeam ), static_cast< eTeamID >( kMineControllerTeam ), kTeamAtWar, kGoalIgnore, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kFreighterTeam ), static_cast< eTeamID >( kDefPlatformTeam ), kTeamAtWar, kGoalIgnore, 1.0, kFalse );
	}
}
