//	<Name>	EnemyTeam.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Uni_2Promises.h"
#include "EnemyTeam.h"
#include "EnemyTeamShip.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"




tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

tEnemyTeam::tEnemyTeam() : tTeamInfo()
{}

void tEnemyTeam::mSetupState()
{}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tEnemyTeam::mCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {

		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 nPCBPV; 
		if( diffLevel == 0 )
		{
			nPCBPV = playerBPV * 0.6;
		}
		else if( diffLevel == 1 )
		{
			nPCBPV = playerBPV * 0.8;
		}
		else if( diffLevel == 2 )
		{
			nPCBPV = playerBPV * 1.0;
		}

        eRaceName race = mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, race );
        sl = gST.stripOutRestricted( sl );
        sl = gST.stripOutSPECIAL( sl );
        ShipList ship_sl = gST.selectWarShips(sl);

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(nPCBPV * 0.825), int(nPCBPV * 1.075), true);
        gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(nPCBPV * 0.825), int(nPCBPV * 1.075), fShiplistCount);
        gST.listShiplistContent(ship_sl);

        if (ship_sl.size() > 0) {

            Ship s1, s2;
            ship_sl.SelectRandomShip(s1);
            gST.mTrackOrions( s1 );
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

            ship_sl.SelectRandomShip(s2);
            gST.mTrackOrions( s2 );
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);

            mCreateShip(
                typeid( tEnemyTeamShip),
                s1.shipCode.c_str(),
                kStartPosition_S,
                kNoMetaShipID,
				kEnemyOneID, 
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tEnemyTeamShip ),
                s2.shipCode.c_str(),
                kStartPosition_T,
                kNoMetaShipID,
                kEnemyTwoID,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.6;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 0.8;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	mCreateShip(
		typeid( tEnemyTeamShip ), 
		nPCBPV, 
		kClassFrigate, 
		kClassHeavyBattlecruiser, 
		0, 
		kStartPosition_S,
		kEnemyOneID, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	mCreateShip(
		typeid( tEnemyTeamShip ), 
		nPCBPV, 
		kClassFrigate, 
		kClassHeavyBattlecruiser, 
		0, 
		kStartPosition_T,
		kEnemyTwoID, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}



