//	<Name>	FreighterTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "tclient.h"
#include "Uni_2Promises.h"
#include "FreighterTeam.h"
#include "FreighterTeamShip.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tFreighterTeam> gFreighterTeamAllocator;

void tFreighterTeam::mSetupState()
{}

void tFreighterTeam::mSetupCommState()
{}

void tFreighterTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tFreighterTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: tFreighterTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tFreighterTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tFreighterTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );

        int32 playerBPV = playerTeam->mGetCombatBPV();
        int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

        std::list< eEnhancedClassTypes >  freighterClassList;
        std::list< eEnhancedRoles >  freighterRolesList;

        if( diffLevel == 0 )
        {
            freighterClassList.push_back(kEClassLargeFreighter);
            freighterClassList.push_back(kEClassLargeAuxiliary);

            freighterRolesList.push_back(kQship);
            freighterRolesList.push_back(kLineShip);
            freighterRolesList.push_back(kWarShip);
            freighterRolesList.push_back(kLeader);
            freighterRolesList.push_back(kDroneBombardmentShip);
            freighterRolesList.push_back(kEscort);
            freighterRolesList.push_back(kCarrier);
            freighterRolesList.push_back(kPFTender);
            freighterRolesList.push_back(kFastCruiser);
            freighterRolesList.push_back(kMauler);
            freighterRolesList.push_back(kPolice);
        }
        else if( diffLevel == 1 )
        {
            freighterClassList.push_back(kEClassSmallFreighter);
            freighterClassList.push_back(kEClassLargeFreighter);
            freighterClassList.push_back(kEClassPriorityTransport);

            freighterRolesList.push_back(kCargo);
            freighterRolesList.push_back(kQship);
        }
        else if( diffLevel == 2 )
        {
		    freighterClassList.push_back(kEClassSmallFreighter);
            freighterClassList.push_back(kEClassPriorityTransport);

            freighterRolesList.push_back(kCargo);
        }

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        ShipList freighter_sl = sl;
        sl = gST.selectWarShips(sl);

        fShiplistCount = freighter_sl.Subset(freighterClassList);
        gST.mDebug_fprintf("freighter class subset: %d\n", fShiplistCount);
        fShiplistCount = freighter_sl.Subset(freighterRolesList);
        gST.mDebug_fprintf("freighter roles subset: %d\n", fShiplistCount);
		gST.listShiplistContent(freighter_sl);

		if (freighter_sl.size() > 0) {
			Ship s1;
            freighter_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            mCreateShip(
                typeid( tFreighterTeamShip ),
                s1.shipCode.c_str(),
                kStartPosition_G,
                kNoMetaShipID,
                -1,
                0, 0, -1, -1, 
				NULL,
                static_cast<eShipOptions>( kStartPositionCanBeOffset | kStartPositionOffsetNE ) );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("---- Finished: tFreighterTeam::mCreateShipsForTeam\n");
}

void tFreighterTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tFreighterTeam::mOldCreateShipsForTeam\n");

	int32 freighterBPV = 0;
	switch(fMissionInfo->mGetDifficultyLevel()) {
		case 0:
			freighterBPV = kMaxBPV; break;
		case 1:
			freighterBPV = 65; break;
		default: break;
	}

	mCreateShip(
		typeid( tFreighterTeamShip ), 
		freighterBPV, 
		kClassFreighter, 
		kClassFreighter, 
		0, 
		kStartPosition_G,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		static_cast<eShipOptions>( kStartPositionCanBeOffset | kStartPositionOffsetNE ) );

	gST.mDebug_fprintf("---- Finished: tFreighterTeam::mOldCreateShipsForTeam\n");
}


