//	<Name>	FreighterTeamShipInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Uni_2Promises.h"
#include "ShipInfo.h"
#include "FreighterTeamShip.h"
#include "FreighterTeamShipInitialState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "PlayerTeam.h"
#include "ShipList_API.h"



tFreighterTeamShipInitialState::tFreighterTeamShipInitialState( )
{
	fWarning = false;
	fLetLooseAtDestination = false;
}

void tFreighterTeamShipInitialState::mBeginState()
{
	mSetTimedInterval( kTimerSecondsSet, kFreighterMovingID, 3.0f );		// Freighter will starting moving

    gST.mDebug_fprintf("--- Starting: tFreighterTeamShipInitialState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tFreighterTeamShipInitialState::mBeginState\n");
}

void tFreighterTeamShipInitialState::mEndState()
{}

void tFreighterTeamShipInitialState::mOnTimedInterval( int32 timerID )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tShipIterator ship = playerTeam->mGetFirstShip();
	tShipInfo* playerShip = *ship;
	int32 rangeThisSecond = 0;


	if ( timerID == kFreighterMovingID )
	{
		fChildActor->mSetShipMaxSpeed( 15 );
		fChildActor->mSetFinalDestination( 16, 15 );
		mSetTimedInterval( kTimerSecondsSet, kCheckDistancePlayerToFreighterID, 5.0f );
	}
	else if( timerID == kCheckDistancePlayerToFreighterID )
	{
		mSetTimedInterval( kTimerTurnsSet, kCheckDistancePlayerToFreighterID, 0.05f );

		for( playerShip; playerShip != NULL; playerShip = *( ++ship) )
		{		
			rangeThisSecond = fChildActor->mGetTacticalDistance( playerShip );

			if( ( rangeThisSecond > 14 ) && ( fChildActor != NULL) )
			{
				playerShip->mSetSystemHealthFromOrig( kNoWeaponHardPoint, 0.0 ); //destroys ship
			}
			else if( ( fWarning == false) && ( rangeThisSecond > 10 ) && (fChildActor != NULL) )
			{
				fWarning = true;
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kApproachingProtectionEdge_msg, kScienceScriptMessage );
			}
			else if( ( rangeThisSecond < 9 ) && (fChildActor != NULL) )
			{
				fWarning = false;
			}
		}// for playerShip
	}//timerID = kCheckDistanceID

    if (timerID == kUpgradeFightersID) {
    #ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
        if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tFreighterTeamShipInitialState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
        int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
        gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tFreighterTeamShipInitialState::mOnTimedInterval -- timer kUpgradeFightersID\n");
    #endif
    }

}
void tFreighterTeamShipInitialState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{
}
void tFreighterTeamShipInitialState::mAtDestination( tShipInfo* ship )
{
	if ( ( ship->mGetTeam() == kFreighterTeam ) && ( fLetLooseAtDestination ) )
	{
		fLetLooseAtDestination = false;
		fChildActor->mClearCommands();
		fChildActor->mSetShipMaxSpeed( 15 );
	}
}
void tFreighterTeamShipInitialState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if( buttonID == kDisengage_btn )
	{
		fLetLooseAtDestination = false;
		fChildActor->mClearCommands();
		fChildActor->mSetShipMaxSpeed( 15 );
		fChildActor->mDisengage( kByMapEdge , kAbsolutePriority  );
	}
	else if( buttonID == kSetPlanetCourse_btn )// Set freighter course for planet
	{
		fLetLooseAtDestination = false;
		fChildActor->mClearCommands();
		fChildActor->mSetShipMaxSpeed( 15 );
		fChildActor->mSetFinalDestination( 16, 15 );
	}
	else if( buttonID == kSetDefPlatCourse_btn )// Set freighter course for Defense Platform
	{
		fChildActor->mClearCommands();
		fChildActor->mSetShipMaxSpeed( 15 );
		fChildActor->mSetFinalDestination( 22, 21 );
		fLetLooseAtDestination = true;
	}
	else if( buttonID == kSetMineContCourse_btn )// Set freighter course for Mine Controller
	{
		fChildActor->mClearCommands();
		fChildActor->mSetShipMaxSpeed( 15 );
		fChildActor->mSetFinalDestination( 24, 19 );
		fLetLooseAtDestination = true;
	}
}
