//	<Name>	MineControllerTeamShipInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Uni_2Promises.h"
#include "ShipInfo.h"
#include "MineControllerTeamShip.h"
#include "MineControllerTeamShipInitialState.h"
#include "MineControllerTeamShipScannedState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipList_API.h"


tMineControllerTeamShipInitialState::tMineControllerTeamShipInitialState( )
{
	fDeepScanON = false;
}

void tMineControllerTeamShipInitialState::mBeginState()
{
    gST.mDebug_fprintf("--- Starting: tMineControllerTeamShipInitialState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tMineControllerTeamShipInitialState::mBeginState\n");
}

void tMineControllerTeamShipInitialState::mEndState()
{}

void tMineControllerTeamShipInitialState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fDeepScanON = true;
		mSetTimedInterval( kTimerTurnsSet, kCheckDistanceToMineControllerID, 0.75f );	
	}
	else
	{
		fDeepScanON = false;
	}
}

void tMineControllerTeamShipInitialState::mOnTimedInterval( int32 timerID )
{
	int32 rangeThisSecond = 0;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tShipIterator ship = playerTeam->mGetFirstShip();
	tShipInfo* playerShip = *ship;


	if( timerID == kCheckDistanceToMineControllerID )
	{
		for( playerShip; playerShip != NULL; playerShip = *( ++ship ) )
		{
			rangeThisSecond = fChildActor->mGetTacticalDistance( playerShip );

			if( ( rangeThisSecond < 10 ) && ( fDeepScanON ) && ( ( fChildActor->mGetShipID() ) == ( playerShip->mGetShipTargetID() ) ) )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kScannedMineController_msg, kScienceScriptMessage );
				fChildActor->mGotoState( typeid( tMineControllerTeamShipScannedState ) );
			}
			if ( rangeThisSecond < 15 )
			{
				fMissionInfo->mSetTeamRelations( fChildActor->mGetTeam(), playerTeam->mGetTeam(), kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			}
		}

		if( fDeepScanON )
		{
			mSetTimedInterval( kTimerTurnsSet, kCheckDistanceToMineControllerID, 0.5f );
		}

	}//timerID = kCheckDistanceID

    if (timerID == kUpgradeFightersID) {
    #ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
        if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tMineControllerTeamShipInitialState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
        int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
        gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tMineControllerTeamShipInitialState::mOnTimedInterval -- timer kUpgradeFightersID\n");
    #endif
    }

}
void tMineControllerTeamShipInitialState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if( damagedShip->mGetShipID() == fChildActor->mGetShipID() )
	{
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), playerTeam->mGetTeam(), kTeamAtWar, kGoalDestroy, 1.0, kTrue );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kMineControllerTeam ), playerTeam->mGetTeam(), kTeamHostile, kGoalIgnore, 1.0, kTrue );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kDefPlatformTeam ), playerTeam->mGetTeam(), kTeamAtWar, kGoalDestroy, 1.0, kTrue );

		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kFreighterTeam ), kTeamAtWar, kGoalDestroy, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kMineControllerTeam ), static_cast< eTeamID >( kFreighterTeam ), kTeamHostile, kGoalIgnore, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kDefPlatformTeam ), static_cast< eTeamID >( kFreighterTeam ), kTeamAtWar, kGoalDestroy, 1.0, kFalse );

		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kFreighterTeam ), static_cast< eTeamID >( kEnemyTeam ), kTeamAtWar, kGoalIgnore, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kFreighterTeam ), static_cast< eTeamID >( kMineControllerTeam ), kTeamAtWar, kGoalIgnore, 1.0, kFalse );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kFreighterTeam ), static_cast< eTeamID >( kDefPlatformTeam ), kTeamAtWar, kGoalIgnore, 1.0, kFalse );
	}
}
