//	<Name>	PromisesVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "PromisesVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Uni_2Promises.h"
#include "PromisesVictory.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamShipPriObjCompState.h"
#include "DefPlatformTeamShip.h"
#include "DefPlatformTeamShipScannedState.h"
#include "MineControllerTeamShip.h"
#include "MineControllerTeamShipScannedState.h"
#include "DefPlatformTeamShipDiplomacyState.h"
              
tPromisesVictoryState::tPromisesVictoryState( tTeamInfo* team, int32 playerTeamID, int32 enemeyTeamID, int32 captureBouns,
												 int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tPromisesVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fiSCOneDestroyed = false;
	fiSCTwoDestroyed = false;
	fiSCOneCaptured  = false;
	fiSCTwoCaptured  = false;
	fMineControllerCaptured = false;
	fMineControllerDestroyed = false;
	fDefensePlatformDestroyed = false;
	fDefensePlatformCaptured = false;
	fPlayerShipInPlay = 1;

}

void tPromisesVictoryState::mBeginState()
{

}

void tPromisesVictoryState::mEndState()
{
}

eVictoryLevels tPromisesVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	eRaceName  playerRace;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
    playerRace = playerTeam->mGetRace();

	if( playerRace == kFederation)
	{
		if( fChildActor->mGetFedDiplomacy() )
		{
			return kAstoundingVictory;
		}
		else if( fChildActor->mGetPrimaryObjectiveComp() ) 
		{
			return kVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}
	else if ( playerRace == kKlingon || playerRace == kLyran)
	{
		if( ( fChildActor->mGetPrimaryObjectiveComp() ) && 
			( fDefensePlatformDestroyed || fDefensePlatformCaptured ) )
		{
			return kAstoundingVictory;
		}
		else if( fChildActor->mGetPrimaryObjectiveComp() ) 
		{
			return kVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}
	else if ( playerRace == kRomulan)
	{
		if( ( fChildActor->mGetPrimaryObjectiveComp() ) && 
			( fChildActor->mGetDefPlatformScanned() ) && 
			( fChildActor->mGetMineControllerScanned() ) )
		{
			return kAstoundingVictory;
		}
		else if( fChildActor->mGetPrimaryObjectiveComp() ) 
		{
			return kVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}	
	}
	else if ( playerRace == kGorn || playerRace == kMirak )
	{
		if( ( fChildActor->mGetPrimaryObjectiveComp() ) && 
			( /*fMineControllerDestroyed ||*/ fMineControllerCaptured ) ||
			( fiSCOneCaptured || fiSCTwoCaptured || fMineControllerCaptured || fDefensePlatformCaptured ) )
		{
			return kAstoundingVictory;
		}
		else if( fChildActor->mGetPrimaryObjectiveComp() ) 
		{
			return kVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}	
	}
	else //kHydran
	{
		if( ( fChildActor->mGetPrimaryObjectiveComp() ) && 
			( fMineControllerDestroyed || fMineControllerCaptured ) ||
			( fiSCOneDestroyed || fiSCTwoDestroyed || fMineControllerDestroyed ) )
		{
			return kAstoundingVictory;
		}
		else if( fChildActor->mGetPrimaryObjectiveComp() ) 
		{
			return kVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}	
	}

}
int32 tPromisesVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tPromisesVictoryState::mOnDeath( tShipInfo* ship )
{
	bool iSCOneOut = false;
	bool iSCTwoOut = false;
	bool MineContOut = false;
	bool DefPlatOut  = false;

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tShipIterator pship = playerTeam->mGetFirstShip();
	tShipInfo* playerShip = *pship;

	// Check to see how many player ships left alive
	if ( ( ship->mGetOriginalTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		int32 playerShipsAlive = playerTeam->mShipsRemainingInPlay();

		if ( playerShipsAlive == 0 )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerDestroyed );
		}
	}

	else if( ( ship->mGetTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyOneID ) ) )
	{
		fiSCOneDestroyed = true;
	}
	else if ( ( ship->mGetTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyTwoID ) ) )
	{
		fiSCTwoDestroyed = true;
	}
	else if ( ship->mGetTeam() == kMineControllerTeam )
	{
		fMineControllerDestroyed = true;
	}
	else if ( ship->mGetTeam() == kDefPlatformTeam )
	{
		fDefensePlatformDestroyed = true;
	}


	if ( fiSCOneDestroyed || fiSCOneCaptured )
	{
		iSCOneOut = true;
	}
	if ( fiSCTwoDestroyed || fiSCTwoCaptured )
	{
		iSCTwoOut = true;
	}
	if ( fMineControllerCaptured || fMineControllerDestroyed  )
	{
		MineContOut = true;
	}
	if ( fDefensePlatformDestroyed || fDefensePlatformCaptured )
	{
		DefPlatOut  = true;
	}
	if ( ( fChildActor->mGetPrimaryObjectiveComp() ) && iSCOneOut && iSCTwoOut && MineContOut && DefPlatOut )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
	}
}

void tPromisesVictoryState::mOnCapture( tShipInfo* ship )
{
	bool iSCOneOut = false;
	bool iSCTwoOut = false;
	bool MineContOut = false;
	bool DefPlatOut  = false;

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	if( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyOneID ) ) )
	{
		fiSCOneCaptured = true;
	}
	else if ( ( ship->mGetOriginalTeam() == kEnemyTeam ) && ( ship->mGetCustomID() == ( tEnemyTeam::kEnemyTwoID ) ) )
	{
		fiSCTwoCaptured = true;
	}
	else if ( ship->mGetOriginalTeam() == kMineControllerTeam )
	{
		fMineControllerCaptured = true;
	}
	else if ( ship->mGetOriginalTeam() == kDefPlatformTeam )
	{
		fDefensePlatformCaptured = true;
	}
	else if ( ( ship->mGetOriginalTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		int32 playerShipsAlive = playerTeam->mShipsRemainingInPlay();

		if( playerShipsAlive < 1 )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerDestroyed );
		}
	}

	if ( fiSCOneDestroyed || fiSCOneCaptured )
	{
		iSCOneOut = true;
	}
	if ( fiSCTwoDestroyed || fiSCTwoCaptured )
	{
		iSCTwoOut = true;
	}
	if ( fMineControllerCaptured || fMineControllerDestroyed  )
	{
		MineContOut = true;
	}
	if ( fDefensePlatformDestroyed || fDefensePlatformCaptured )
	{
		DefPlatOut  = true;
	}
	if ( ( fChildActor->mGetPrimaryObjectiveComp() ) && iSCOneOut && iSCTwoOut && MineContOut && DefPlatOut )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
	}
}

void tPromisesVictoryState::mOnDisengaged( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	fPlayerShipInPlay = mShipsInPlay( playerTeam );

	if ( ( ship->mGetTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		if( fPlayerShipInPlay < 1 )
		{
			if ( fChildActor->mGetPrimaryObjectiveComp() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
			}
			else
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
			}
		}
	}
}
void tPromisesVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{
	bool iSCOneOut = false;
	bool iSCTwoOut = false;
	bool MineContOut = false;
	bool DefPlatOut  = false;

	if( (*actor == typeid( tPlayerTeam ) ) &&  // Check to see if Primary Objective is Complete
		(*state == typeid( tPlayerTeamShipPriObjCompState ) ) && 
		(eventType == kStartingNewState ) )
	{
		fChildActor->mSetPrimaryObjectiveComp( true );
	}

	else if( (*actor == typeid( tDefPlatformTeamShip ) ) &&  // Check to see if Defense Platform has been scanned
		(*state == typeid(tDefPlatformTeamShipScannedState) ) && 
		(eventType == kStartingNewState ) )
	{
		fChildActor->mSetDefPlatformScanned( true );
	}
	else if( (*actor == typeid( tMineControllerTeamShip ) ) &&  // Check to see if Defense Platform has been scanned
		(*state == typeid( tMineControllerTeamShipScannedState) ) && 
		(eventType == kStartingNewState ) )
	{
		fChildActor->mSetMineControllerScanned( true );
	}
	else if( (*actor == typeid( tDefPlatformTeamShip ) ) &&  // Check to see if the Fed used dimplomacy to win
		(*state == typeid( tDefPlatformTeamShipDiplomacyState) ) && 
		(eventType == kStartingNewState ) )
	{
		fChildActor->mSetFedDiplomacy( true );
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
	}
	if ( fChildActor->mGetPrimaryObjectiveComp() )
	{
		if ( fiSCOneDestroyed || fiSCOneCaptured )
		{
			iSCOneOut = true;
		}
		if ( fiSCTwoDestroyed || fiSCTwoCaptured )
		{
			iSCTwoOut = true;
		}
		if ( fMineControllerCaptured || fMineControllerDestroyed  )
		{
			MineContOut = true;
		}
		if ( fDefensePlatformDestroyed || fDefensePlatformCaptured )
		{
			DefPlatOut  = true;
		}
		if ( iSCOneOut && iSCTwoOut && MineContOut && DefPlatOut )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
		}
	}
	
}
void tPromisesVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	tTeamInfo* planetTeam  = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlanetTeam ) );
	tShipIterator ship = planetTeam->mGetFirstShip();
	tShipInfo* planet = *ship;
	ship = playerTeam->mGetFirstShip();
	tShipInfo* playerShip = *ship;


	// Check for Transport of Med supplies to the Planet
	ship = playerTeam->mGetFirstShip();
	playerShip = *ship;
	for( playerShip; playerShip != NULL; playerShip = *( ++ship) )
	{
		if ( ( shipTo == playerShip ) && ( item == kTransHarryMudd ) && ( shipFrom == planet ) )
		{
			
			mAddToBonusPrestige( 20 );
			planet->mRemoveItemFromStores( kTransHarryMudd, 1 );
		}
	}// end for playership

}