#include "Uni_2Promises.h"

#include "MissionText.h"
#include "MissionMaps.h"

#include "VictoryCondition.h"

#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "PlanetTeam.h"
#include "FreighterTeam.h"
#include "MineControllerTeam.h"
#include "DefPlatformTeam.h"
#include "PromisesVictory.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tUni_2Promises );

tUni_2Promises::tUni_2Promises()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tUni_2Promises::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID >( kPlayerTeam ),
																		kMaxChance,
																		kFalse,
																		kPrimaryTeam,
																		kMaxChance,
																		kSpecPlayableRace & ~kSpecISC,
																		kAnyRace, typeid( tPlayerTeam ), kNoTeamTag, Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID >( kEnemyTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace, typeid( tEnemyTeam ) );


;	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", static_cast< eTeamID >( kPlanetTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecMirak | kSpecHydran,
																		kAnyRace, typeid( tPlanetTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", static_cast< eTeamID >( kFreighterTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecAnyRace,
																		kSameAs | kPlayerTeam,
																		typeid( tFreighterTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5", static_cast< eTeamID >( kMineControllerTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace, typeid( tMineControllerTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6", static_cast< eTeamID >( kDefPlatformTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace, typeid( tDefPlatformTeam ) );


	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tUni_2Promises::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tUni_2Promises::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tUni_2Promises::mCreateExternalShips( int32 Mode )
{
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tUni_2Promises::mInitializeFinished( void )
{
	eRaceName  playerRace;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
    playerRace = playerTeam->mGetRace();

	// return the index to the briefing message we want to use
	switch( playerRace )
	{
		case kFederation:
			return( kFedBriefing_msg );
		break;
		case kKlingon:
			return( kKlingonBriefing_msg );
		break;
		case kRomulan:
			return( kRomulanBriefing_msg );
		break;
		case kLyran:
			return( kLyranBriefing_msg );
		break;
		case kGorn:
			return( kGornBriefing_msg );
		break;
		case kMirak:
			return( kMirakBriefing_msg );
		break;
		case kHydran:
			return( kHydranBriefing_msg );
		break;
	}

	return( kFedBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tUni_2Promises::mStartMission( void )
{
	tScript::mStartMission();

	// setup team relations and target goals
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kPlanetTeam,
		kFreighterTeam,
		kMineControllerTeam,
		kDefPlatformTeam	
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr	= fPlayersTeam,
		//Enemy = kEnemyTeam,
		//PLNT  = kPlanetTeam,
		//Frgh  = kFreighterTeam,
		//Mine  = kMineControllerTeam,
		//DefP  = kDefPlatformTeam,

		//			Plyr	Enemy	PLNT	Frgh	Mine	DefP  
		/*Plyr*/	"--",	"WD",	"AI",	"AI",	"WD",	"WD",
		/*Enemy*/	"WD",	"--",	"AI",	"WD",	"AI",	"AI",
		/*PLNT*/	"AI",	"AI",	"--",	"AI",	"AI",	"AI",
		/*Frgh*/	"AI",	"HI",	"AI",	"--",	"WD",	"WD",
		/*Mine*/	"HI",	"AI",	"AI",	"HI",	"--",	"AI",
		/*DefP*/	"WD",	"AI",	"AI",	"WD",	"AI",	"--",

	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tUni_2Promises::mNewInternalShip( tShipInfo* Ship )
{
}
void tUni_2Promises::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;


	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['U'] = kObjectEnemyBase;

	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
}
void tUni_2Promises::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tUni_2Promises::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] =	kFedBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] =		kKlingonBriefing_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] =		kRomulanBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] =		kLyranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] =		kHydranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] =		kGornBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] =		kMirakBriefing_msg;

}