//	<Name>	BaseVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "FreighterTeam.h"
#include "ISCFreighter.h"
#include "ISCTeam.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"
#include "Uni_3IronCage.h"

#include "BaseVictoryState.h"


tBaseVictoryState::tBaseVictoryState( tTeamInfo* team, int32 captureBonus, int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{

}

void tBaseVictoryState::mBeginState()
{

}

void tBaseVictoryState::mEndState()
{}


eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return kDraw;
	eRaceName	playerRace = playerTeam->mGetRace();

	//
	//	Set the victory level according to the objectives accomplished.
	//
	switch ( playerRace )
	{
	case kFederation:
	case kMirak:
		{
			if ( fChildActor->mPrisonersRescued() )
			{
				if ( fChildActor->mSuppliesRescued() && !fChildActor->mSupplyShipDestroyed() )
				{
					return kAstoundingVictory;
				}
				else
				{
					return kVictory;
				}
			}
			else
			{
				if ( fChildActor->mSuppliesRescued() && !fChildActor->mSupplyShipDestroyed() )
				{
					return kDefeat;
				}
				else
				{
					return kDevastatingDefeat;
				}
			}
		}
		break;

	case kKlingon:
	case kLyran:
	case kGorn:
		{
			if ( fChildActor->mPrisonersRescued() )
			{
				if ( fChildActor->mISCCLShipsDestroyed() == 2 )
				{
					return kAstoundingVictory;
				}
				else
				{
					return kVictory;
				}
			}
			else
			{
				if ( fChildActor->mISCCLShipsDestroyed() == 2 )
				{
					return kDefeat;
				}
				else
				{
					return kDevastatingDefeat;
				}
			}
		}
		break;

	case kRomulan:
	case kHydran:
		{
			if ( fChildActor->mPrisonShipDestroyed() )
			{
				if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
				{
					return kAstoundingVictory;
				}
				else
				{
					return kVictory;
				}
			}
			else
			{
				if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
				{
					return kDefeat;
				}
				else
				{
					return kDevastatingDefeat;
				}
			}
		}
		break;

	default:
		{
			return kDraw;
		}
		break;
	}
	return kDraw;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( ship->mGetCustomID() )
	{
	case tFreighterTeam::kPrisonFreighter:
		{			
			fChildActor->mSetPrisonShipDestroyed( true );

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
				}
				break;

			case kKlingon:
			case kLyran:
			case kGorn:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_3_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
				}
				break;

			case kRomulan:
			case kHydran:
				{
					if ( fChildActor->mISCDefensePlatformsDestroyed() < 2 )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
					}
					else if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
				}
				break;
			}
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{
			fChildActor->mSetSupplyShipDestroyed( true );

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					if ( fChildActor->mSuppliesRetrieved() )
					{
						mAddToBonusPrestige( -150 );
					}
				}
				break;

			case kKlingon:
			case kLyran:
			case kGorn:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom14_msg );
				}
				break;

			case kRomulan:
			case kHydran:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom14_msg );
				}
				break;
			}
		}
		break;

	case tISCTeam::kISCCL1:
	case tISCTeam::kISCCL2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom10_msg );
			fChildActor->mIncreaseISCCLShipsDestroyed();

			switch ( playerRace )
			{
			case kKlingon:
			case kLyran:
			case kGorn:
				{
					mAddToBonusPrestige( 120 );
					if ( fChildActor->mISCCLShipsDestroyed() == 2 && !fChildActor->mPrisonersRetrieved() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_1_msg );
					}
					else if ( fChildActor->mISCCLShipsDestroyed() == 2 && !fChildActor->mPrisonersRescued() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_2_msg );
					}
					else if ( fChildActor->mISCCLShipsDestroyed() == 2 && fChildActor->mPrisonersRescued() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_3_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
				}
				break;
			}
		}
		break;

	case tISCTeam::kDefensePlatform1:
	case tISCTeam::kDefensePlatform2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_1_msg );
			fChildActor->mIncreaseISCDefensePlatformsDestroyed();

			switch ( playerRace )
			{
			case kRomulan:
			case kHydran:
				{
					mAddToBonusPrestige( 40 );
				}
				break;
			}

			if ( fChildActor->mISCDefensePlatformsDestroyed() == 2 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_2_msg );

				switch ( playerRace )
				{
				case kRomulan:
				case kHydran:
					{						
						if ( !fChildActor->mPrisonShipDestroyed() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_3_msg );
						}
						else if ( fChildActor->mPrisonShipDestroyed() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_4_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
					}
					break;
				}
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( ship->mGetCustomID() )
	{
	case tFreighterTeam::kPrisonFreighter:
		{
			fChildActor->mSetPrisonersRetrieved( true );

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );

					if ( !fChildActor->mSuppliesRetrieved() && !fChildActor->mSupplyShipDestroyed() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_2_msg );
					}
					else if ( fChildActor->mSupplyShipDestroyed())
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_3_msg );
					}
				}
				break;

			case kKlingon:
			case kLyran:
			case kGorn:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
				}
				break;

			case kRomulan:
			case kHydran:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom5_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
				}
				break;
			}
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{
			fChildActor->mSetSuppliesRetrieved( true );

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );
				}
				break;

			case kKlingon:
			case kLyran:
			case kGorn:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
				}
				break;

			case kRomulan:
			case kHydran:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
				}
				break;
			}
		}
		break;

	case tISCTeam::kISCCL1:
	case tISCTeam::kISCCL2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom12_msg );
			fChildActor->mIncreaseISCCLShipsDestroyed();
		}
		break;

	case tISCTeam::kDefensePlatform1:
	case tISCTeam::kDefensePlatform2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom11_msg );
			fChildActor->mIncreaseISCDefensePlatformsDestroyed();
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	tTeamInfo*	freighterTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kFreighterTeam ) );
	Assert( freighterTeam );
	if ( !freighterTeam )
		return;
	tShipIterator prisonIterator = freighterTeam->mGetFirstShip();
	tShipIterator supplyIterator = freighterTeam->mGetFirstShip();
//	tShipInPlayEval x, y;
//	prisonIterator.mAddEvaluator( &x );
//	supplyIterator.mAddEvaluator( &y );
	tShipCustomIDEval p( tFreighterTeam::kPrisonFreighter ), q( tFreighterTeam::kSupplyFreighter );
	prisonIterator.mAddEvaluator( &p );
	supplyIterator.mAddEvaluator( &q );
	tShipInfo*	prisonShip = *prisonIterator;
	tShipInfo*	supplyShip = *supplyIterator;

	switch ( ship->mGetCustomID() )
	{
	case tFreighterTeam::kPrisonFreighter:
		{
			fChildActor->mSetPrisonersRescued( true );			

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					if ( ( ship ) && ( static_cast< tISCFreighter* >( ship ) )->mIsTractored() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
					else if ( !fChildActor->mSupplyShipDestroyed() )
					{
						if ( fChildActor->mSuppliesRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent7_4_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent7_1_msg );
						}
					}
					else if ( fChildActor->mSupplyShipDestroyed() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent7_2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
				}
				break;

			case kKlingon:
			case kLyran:
			case kGorn:
				{
					if ( ( ship ) && ( static_cast< tISCFreighter* >( ship ) )->mIsTractored() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
					else if ( fChildActor->mISCCLShipsDestroyed() < 2 )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent7_3_msg );
					}
					else if ( fChildActor->mISCCLShipsDestroyed() == 2 )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent7_4_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
				}
				break;

			case kRomulan:
			case kHydran:
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom9_msg );
					fChildActor->mSetPrisonShipDestroyed( true );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				break;
			}

			( static_cast< tISCFreighter* >( ship ) )->mSetIsTractored( false );
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{
			fChildActor->mSetSuppliesRescued( true );			

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					mAddToBonusPrestige( 50 );

					if ( ( ship ) && ( static_cast< tISCFreighter* >( ship ) )->mIsTractored() )
					{
						if ( fChildActor->mPrisonersRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom7_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
						}
					}
					else
					{
						if ( !fChildActor->mPrisonersRetrieved() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );
						}
						else if ( !fChildActor->mPrisonersRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_2_msg );
						}
						else if ( fChildActor->mPrisonersRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_3_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
					}
				}
				break;

			case kKlingon:
			case kLyran:
			case kGorn:
				{
					if ( ( ship ) && ( static_cast< tISCFreighter* >( ship ) )->mIsTractored() )
					{
						if ( fChildActor->mPrisonersRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom8_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
						}
					}
				}
				break;

			case kRomulan:
			case kHydran:
				{
					if ( ( ship ) && ( static_cast< tISCFreighter* >( ship ) )->mIsTractored() )
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom8_msg );
						if ( fChildActor->mPrisonShipDestroyed() )
						{
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
						else
						{
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
						}
					}
				}
				break;
			}

			( static_cast< tISCFreighter* >( ship ) )->mSetIsTractored( false );
		}
		break;

	case tISCTeam::kISCCL1:
	case tISCTeam::kISCCL2:
		{}
		break;

	case tISCTeam::kDefensePlatform1:
	case tISCTeam::kDefensePlatform2:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if (	( ( prisonShip && !( static_cast< tISCFreighter* >( prisonShip ) )->mIsTractored() ) ) &&
					( ( supplyShip && !( static_cast< tISCFreighter* >( supplyShip ) )->mIsTractored() ) )	)
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_1_msg );
			}

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					//
					//	The prison ship was in tow when the player left the map.
					//
					if ( ( prisonShip ) && ( static_cast< tISCFreighter* >( prisonShip ) )->mIsTractored() )
					{
						fChildActor->mSetPrisonersRescued( true );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
					}
					//
					//	The supply ship was in tow when the player left the map.  If the prison ship had
					//	already disengaged, then the mission was a success; otherwise, it was a failure.
					//
					else if ( ( supplyShip ) && ( ( static_cast< tISCFreighter* >( supplyShip ) )->mIsTractored() ) )
					{
						fChildActor->mSetSuppliesRescued( true );
						if ( fChildActor->mPrisonersRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom7_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
						}
					}
					else if ( fChildActor->mPrisonersRescued() && !fChildActor->mPrisonShipDestroyed() )
					{
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
					}
					else
					{
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
					}
				}
				break;

			case kKlingon:
			case kLyran:
			case kGorn:
				{
					//
					//	The prison ship was in tow when the player left the map.
					//
					if ( ( prisonShip ) && ( static_cast< tISCFreighter* >( prisonShip ) )->mIsTractored() )
					{
						fChildActor->mSetPrisonersRescued( true );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
					//
					//	The supply ship was in tow when the player left the map.  If the prison ship had
					//	already disengaged, then the mission was a success; otherwise, it was a failure.
					//
					else if ( ( supplyShip ) && ( ( static_cast< tISCFreighter* >( supplyShip ) )->mIsTractored() ) )
					{
						fChildActor->mSetSuppliesRescued( true );
						if ( fChildActor->mPrisonersRescued() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom8_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
						}
					}
					else if ( fChildActor->mPrisonersRescued() && !fChildActor->mPrisonShipDestroyed() )
					{
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
					else
					{
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
					}
				}
				break;

			case kRomulan:
			case kHydran:
				{
					if ( ( prisonShip ) && ( static_cast< tISCFreighter* >( prisonShip ) )->mIsTractored() )
					{
						fChildActor->mSetPrisonersRescued( true );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom9_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
					else if ( ( supplyShip ) && ( static_cast< tISCFreighter* >( supplyShip ) )->mIsTractored() )
					{
						fChildActor->mSetSuppliesRescued( true );
						if ( fChildActor->mPrisonShipDestroyed() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom8_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
						}
					}
					else if (	fChildActor->mPrisonShipDestroyed() && !fChildActor->mPrisonersRescued() )
					{
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
					}
					else
					{
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
					}
				}
				break;
			}
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( ship->mGetCustomID() )
	{
	case tFreighterTeam::kPrisonFreighter:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent8_1_msg );
			fChildActor->mSetPrisonersReachPlanet( true );
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
		}
		break;

	case tFreighterTeam::kSupplyFreighter:
		{}
		break;

	case tISCTeam::kISCCL1:
	case tISCTeam::kISCCL2:
		{}
		break;

	case tISCTeam::kDefensePlatform1:
	case tISCTeam::kDefensePlatform2:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{}
		break;
	}
}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );		break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );		break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );		break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );		break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );		break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );		break;
		}
		mSetTimedInterval(	kTimerSecondsSet, timerID, ( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif
}