/********************************************************************************************************\

	File Name:	FreighterTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "FreighterTeam.h"
#include "ISCFreighter.h"
#include "ShipList_API.h"
#include "FreighterTeamBaseState.h"


tClassAllocator<tFreighterTeam> gFreighterTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tFreighterTeam::mSetupState()
{
    mRegisterState( new tFreighterTeamBaseState() );
    mGotoState( typeid( tFreighterTeamBaseState ) );
}

void tFreighterTeam::mSetupCommState()
{}

void tFreighterTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tFreighterTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        ShipList freighter_sl = sl;

        std::list< eEnhancedClassTypes >  freighterClassList;
		freighterClassList.push_back(kEClassSmallFreighter);
        freighterClassList.push_back(kEClassPriorityTransport);
        fShiplistCount = freighter_sl.Subset(freighterClassList);
        gST.mDebug_fprintf("freighter class subset: %d\n", fShiplistCount);

		std::list< eEnhancedRoles >  freighterRolesList;
		freighterRolesList.push_back(kCargo);
        fShiplistCount = freighter_sl.Subset(freighterRolesList);
        gST.mDebug_fprintf("freighter roles subset: %d\n", fShiplistCount);
        gST.listShiplistContent(freighter_sl);

        if (freighter_sl.size() > 0) {
            Ship s1, s2;
            freighter_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			freighter_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

			mCreateShip(
				typeid( tISCFreighter ), 
				s1.shipCode.c_str(), 
				kStartPosition_X, 
				kNoMetaShipID,
				kPrisonFreighter, 
				0, 0, -1, -1, 
				NULL, 
				static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );
			mCreateShip(
				typeid( tISCFreighter ), 
				s2.shipCode.c_str(), 
				kStartPosition_W, 
				kNoMetaShipID,
				kSupplyFreighter, 
				0, 0, -1, -1, 
				NULL, 
				static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }
	gST.mDebug_fprintf("---- Finished: tFreighterTeam::mCreateShipsForTeam\n");
}


void tFreighterTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tFreighterTeam::mOldCreateShipsForTeam\n");

	mCreateShip(
		typeid( tISCFreighter ), 
		"I-FSPh", 
		kStartPosition_X, 
		kNoMetaShipID,
		kPrisonFreighter, 
		0, 0, -1, -1, 
		NULL, 
		static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );
	mCreateShip(
		typeid( tISCFreighter ), 
		"I-FSPh", 
		kStartPosition_W, 
		kNoMetaShipID,
		kSupplyFreighter, 
		0, 0, -1, -1, 
		NULL, 
		static_cast< eShipOptions >( kDefaultShipOptions & ~kCanAddToFleetUponCapture ) );

	gST.mDebug_fprintf("---- Finished: tFreighterTeam::mOldCreateShipsForTeam\n");
}


void tFreighterTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tFreighterTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tFreighterTeam", shipScriptDescription );
}