//	<Name>	ISCFreighterBaseState.cpp
//
//
//	<Info>	DESCRIPTION
//


#include "ISCTeam.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "Uni_3IronCage.h"

#include "ISCFreighterBaseState.h"


/*

	Purpose:	Examines the freighter and determines what actions to take dependant upon what is in the
				freighter (prisoners or supplies) and which race the player team is when the freighter has
				been captured.

*/
void tISCFreighterBaseState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
    eRaceName	playerRace = playerTeam->mGetRace();

	if ( ship->mGetShipID() == fChildActor->mGetShipID() )
	{
		if ( fChildActor->mIsPrisonShip() )
		{
			fChildActor->mDisengage( kByMapEdge, kAbsolutePriority );
			switch ( playerRace )
			{
			case kFederation:
			case kKlingon:
			case kLyran:
			case kGorn:
			case kMirak:
				{
					/***

						The player was supposed to save the prisoners and did, so inform the player so and
						adjust their victory state accordingly.
					
					***/
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tISCFreighter::kDemand2CommState );
				}
				break;

			case kRomulan:
			case kHydran:
				{
					/***

						The player has captured the ship rather than destroying it, text can be added here
						informing the player that the prisoner ship should be destroyed, not captured.

					***/
				}
				break;
			}
		}
		else
		{
			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					/***

						The player was supposed to save the supply ship and did, so inform the player so and
						adjust their victory state accordingly.
					
					***/
				}
				break;

			case kKlingon:
			case kRomulan:
			case kLyran:
			case kGorn:
			case kHydran:
				{
					/***

						The player has no reason to capture the supply ship, so just inform the player that
						time is being wasted.

					***/
				}
				break;
			}
		}
	}
}

/*

	Purpose:	Examines the freighter and determines what actions to take dependant upon what is in the
				freighter (prisoners or supplies) and which race the player team is when the freighter has
				been destroyed.

*/
void tISCFreighterBaseState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
    eRaceName	playerRace = playerTeam->mGetRace();

	if ( ship->mGetShipID() == fChildActor->mGetShipID() )
	{
		if ( fChildActor->mIsPrisonShip() )
		{
			switch ( playerRace )
			{
			case kFederation:
			case kKlingon:
			case kLyran:
			case kGorn:
			case kMirak:
				{
					/***

						The player was supposed to save the prisoners, but blew them up instead, so inform
						the player so and end the mission as incomplete.
					
					***/
				}
				break;

			case kRomulan:
			case kHydran:
				{
					/***

						The player has destroyed the ship rather than capturing it, text can be added here
						informing the player the primary object has been achieved.  The mission can continue
						on waiting for the secondary objective to be complete if it has not already done so.

					***/
				}
				break;
			}
		}
		else
		{
			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					/***

						The player was supposed to save the supplies inside of the this freighter, but instead
						it was destroyed, so the secondary objective has failed.  Inform the player that this
						is so by displaying a message and end the mission if the primary object was completed.

					***/
				}
				break;

			case kKlingon:
			case kRomulan:
			case kLyran:
			case kGorn:
			case kHydran:
				{
					/***

						The player was allowed to blow this freighter up.  If it was blown up, then just
						inform the player, through the use of a message, that they have done a good job.

					***/
				}
				break;
			}
		}
	}
}

/*

	Purpose:	When either the prison ship or supply ship reach a small distance from the planet, the
				mission is over because the primary objective of either capturing or destroying the
				prison ship has failed.

*/
void tISCFreighterBaseState::mAtDistance( tShipInfo* ship )
{}

/*

	Purpose:	Will be called when the prison ship or supply ship reach the planet.

*/
void tISCFreighterBaseState::mAtDestination( tShipInfo* ship )
{
	/***

		This message does not need to be handled since the mission will end when the ship is within some
		distance of the planet.

	***/
}

/*

	Purpose:	Dependant on which freighter is scanned (prison or supply), a description of the contents
				of the ship will be displayed.

*/
void tISCFreighterBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if ( !ship )
		return;

	if ( ship->mGetShipTargetID() == fChildActor->mGetShipID() )
	{
		if( deepScanOn)
		{
			fScanning = true;
			mSetTimedInterval( kTimerTurnsSet, tISCFreighter::kScanID, 0.75f );
		}
		else
		{
			fScanning = false;
			mSetTimedInterval( kTimerKillTimer, tISCFreighter::kScanID, 0.0f );
		}
	}
}

/*

	Purpose:	If either of the freighters are under control of the player's team when disengaged, then
				the primary or secondary objectives have been completed depending upon the race of the
				player's team.

*/
void tISCFreighterBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
    eRaceName	playerRace = playerTeam->mGetRace();
}

void tISCFreighterBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if ( ( !tractorTarget ) || ( !tractoringShip ) )
		return;
	if ( fChildActor->mGetCustomID() == tractorTarget->mGetCustomID() )
	{
		( static_cast< tISCFreighter* >( tractorTarget ) )->mSetIsTractored( true );
	}

	fChildActor->mSetShipCallbackEvents( kNotifyEndTractor );
}

void tISCFreighterBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if ( ( !tractorTarget ) || ( !tractoringShip ) )
		return;
	if ( fChildActor->mGetCustomID() == tractorTarget->mGetCustomID() )
	{
		( static_cast< tISCFreighter* >( tractorTarget ) )->mSetIsTractored( false );
	}

	fChildActor->mSetShipCallbackEvents( kNotifyEndTractor );
}

void tISCFreighterBaseState::mOnTimedInterval( int32 timerID )
{
	switch ( timerID )
	{
	case tISCFreighter::kScanID:
		{
			if ( fScanning )
			{
				if ( fChildActor->mIsPrisonShip() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent9_1_msg );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent10_1_msg );
				}
			}
		}
	}
}