//	<Name>	ISCTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ISCDefensePlatform.h"
#include "ISCFreighter.h"
#include "ISCLightCruiserCL.h"
#include "ISCTeam.h"
#include "ISCTeamBaseState.h"
#include "ScriptInterface.h"
#include "tclient.h"
#include "Uni_3IronCage.h"
#include "ShipList_API.h"

tClassAllocator<tISCTeam> gISCTeamAllocator;


void tISCTeam::mSetupState()
{
    mRegisterState( new tISCTeamBaseState() );
    mGotoState( typeid( tISCTeamBaseState ) );
}

void tISCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tISCTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

		// ships
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( playerTeam );
		int32 playerBPV = playerTeam->mGetCombatBPV();

		ShipList ship_sl = sl;
		ship_sl = gST.stripOutRestricted( ship_sl );
        ship_sl = gST.stripOutSPECIAL( ship_sl );
        ship_sl = gST.selectWarShips(ship_sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		int32 nPCBPV = ( playerBPV >> 1 );  // same BPV hack as what Taldren did. --FS
        fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(nPCBPV * 0.825), int(nPCBPV * 1.075), true);
        gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(nPCBPV * 0.825), int(nPCBPV * 1.075), fShiplistCount);
        gST.listShiplistContent(ship_sl);


        // Defense platforms
		ShipList DEF_sl = sl;
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);


        if ((ship_sl.size() > 0) && (DEF_sl.size() > 0)) {
			// Create ship 
            Ship ship1, ship2;
            ship_sl.SelectRandomShip(ship1);
            gST.mDebug_fprintf("ship1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship1.shipCode.c_str(), ship1.bpv, ship1.yearFirstAvail, ship1.yearLastAvail);
            gST.mTrackOrions( ship1 );

            ship_sl.SelectRandomShip(ship2);
            gST.mDebug_fprintf("ship2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship2.shipCode.c_str(), ship2.bpv, ship2.yearFirstAvail, ship2.yearLastAvail);
            gST.mTrackOrions( ship2 );

			mCreateShip(
				typeid( tISCLightCruiserCL ), 
				ship1.shipCode.c_str(), 
				kStartPosition_Z, 
				kNoMetaShipID,
				kISCCL1, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions  );
			mCreateShip(
				typeid( tISCLightCruiserCL ), 
				ship2.shipCode.c_str(), 
				kStartPosition_Y, 
				kNoMetaShipID,
				kISCCL2, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );


			// Create DEF platforms
            Ship def1, def2;
            DEF_sl.SelectRandomShip(def1);
            gST.mDebug_fprintf("def1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                def1.shipCode.c_str(), def1.bpv, def1.yearFirstAvail, def1.yearLastAvail);
            gST.mTrackOrions( def1 );

            DEF_sl.SelectRandomShip(def2);
            gST.mDebug_fprintf("def2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                def2.shipCode.c_str(), def2.bpv, def2.yearFirstAvail, def2.yearLastAvail);
            gST.mTrackOrions( def2 );

			mCreateShip(
				typeid( tISCDefensePlatform ), 
				def1.shipCode.c_str(), 
				kStartPosition_V, 
				kNoMetaShipID,
				kDefensePlatform1, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions  );
			mCreateShip(
				typeid( tISCDefensePlatform ), 
				def2.shipCode.c_str(), 
				kStartPosition_U, 
				kNoMetaShipID,
				kDefensePlatform2, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tISCTeam::mCreateShipsForTeam\n");
}

void tISCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tISCTeam::mOldCreateShipsForTeam\n");

	int32 playerBPV = 0;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	if ( playerTeam )
		playerBPV = playerTeam->mGetCombatBPV();

	mCreateShip(
		typeid( tISCLightCruiserCL ), 
		( playerBPV >> 1 ), // some cheap way of roughly doing 50%?
		kClassFrigate, 
		kClassHeavyCruiser, 
		100,
		kStartPosition_Z, 
		kISCCL1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );
	mCreateShip(
		typeid( tISCLightCruiserCL ), 
		( playerBPV >> 1 ), // some cheap way of roughly doing 50%?
		kClassFrigate, 
		kClassNewHeavyCruiser, 
		100,
		kStartPosition_Y, 
		kISCCL2, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	mCreateShip(
		typeid( tISCDefensePlatform ), 
		"I-DEF", 
		kStartPosition_V, 
		kNoMetaShipID,
		kDefensePlatform1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions  );
	mCreateShip(
		typeid( tISCDefensePlatform ), 
		"I-DEF", 
		kStartPosition_U, 
		kNoMetaShipID,
		kDefensePlatform2, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tISCTeam::mOldCreateShipsForTeam\n");
}

void tISCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tISCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tISCTeam", shipScriptDescription );
}
