//	<Name>	PlayerTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"
#include "ShipScriptDescription.h"
#include "Uni_3IronCage.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if ( victoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDisconnect_msg );
		return;
	}

	fMissionInfo->mAddToDebriefingList( fTeam, victoryLevel + kAstoundingVictory_msg );

	//
	//	Add the appropriate messages to the debriefing list according to the player's performance.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
    eRaceName	playerRace = playerTeam->mGetRace();

	// return the index to the briefing message we want to use
	switch (playerRace)
	{
	case kFederation:
	case kMirak:
		{
			switch ( victoryLevel )
			{
			case kDefeat:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebFederationMirak1_msg );	break;
			case kDevastatingDefeat:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebFederationMirak2_msg );	break;
			case kVictory:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebFederationMirak3_msg );	break;
			case kAstoundingVictory:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebFederationMirak4_msg );	break;
			}
		}
		break;

	case kKlingon:
	case kLyran:
	case kGorn:
		{
			switch ( victoryLevel )
			{
			case kDefeat:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebKlingonLyranGorn1_msg );	break;
			case kDevastatingDefeat:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebKlingonLyranGorn2_msg );	break;
			case kVictory:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebKlingonLyranGorn3_msg );	break;
			case kAstoundingVictory:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebKlingonLyranGorn4_msg );	break;
			}
		}
		break;

	case kRomulan:
	case kHydran:
		{
			switch ( victoryLevel )
			{
			case kDevastatingDefeat:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebRomulanHydran1_msg );	break;
			case kVictory:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebRomulanHydran2_msg );	break;
			case kAstoundingVictory:
				fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDebRomulanHydran3_msg );	break;
			}
		}
		break;

	default:
		{
			; // do nothing more
		}
		break;
	}
}

void tPlayerTeam::mSetupState()
{}

void tPlayerTeam::mSetupCommState()
{}


void tPlayerTeam::mCreateShipsForTeam()
{}


void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	//
	//	Create the ship according to the description given by the engine.
	//
	mCreateShip( typeid( tPlayerTeamShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 
				 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	//
	//	Increment the ship number for each ship that comes through here on the player team
	//
	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;

#ifdef SLTOOLSDEBUGOUT
	// I am testing: force Uni_X4M1AThief as next mission -- FS
	fMissionScheduler.fNextMissionTitle = "Uni_X4M1AThief";
#else
	switch( fMissionInfo->mRandomInt32( 1, 20 ) )
	{
	case 1: case 2: case 3: case 4: case 5:
	case 6: case 7: case 8: case 9: case 10:
		fMissionScheduler.fNextMissionTitle = "Uni_X4M1AThief";
		break;
	default:
		fMissionScheduler.fNextMissionTitle = "Uni_4Resolve";
		break;
	}
#endif

	switch ( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
		fMissionScheduler.fMedal = kMedalSpecialTwo;
		break;
	}
}