
#include "ShipList_API.h"
#include "MissionInfo.h"
#include "MissionText.h"
//#include "ScriptInterface.h"
#include "ShipScriptDescription.h"
#include "Tokenizer.h"
#include <sys/stat.h>

shiplist_tools gST;

ShipList global_sl;
ShipList narrowed_sl;
ShipList selected_drafted_sl;
#ifdef FIGHTERS
ShipList fighter_sl;
ShipList narrowed_fighter_sl;
#endif


shiplist_tools::setup()
{
	fEarliestYear = -999;
	fLatestYear = 999;
	fDraftedShipsSetUp = 0;
	fNarrowedListFullySetUp = 0;

	fTeam1DeltaFighterBPV = 0;
	fTeam2DeltaFighterBPV = 0;
	fTeam3DeltaFighterBPV = 0;
}


// Add me to really something early, like the Main mission script's mScriptSpecs()
void shiplist_tools::mOpenDebugFile() 
{
#ifdef SLTOOLSDEBUGOUT
	std::string fileName = "C:\\"+Messages[kMissionTitle_msg]+".txt";
	FILE *fp = fopen(fileName.c_str(), "w");
	fprintf (fp, "Starting debug logfile: %s\n", fileName.c_str() );
	fflush(fp);
	fclose(fp);
	ZeroTimeCount = GetTickCount();
#endif
}


void shiplist_tools::mDebug_fprintf ( const char* format, ... )
{
#ifdef SLTOOLSDEBUGOUT
        va_list argptr ;
        va_start ( argptr, format ) ;
        int32 CurrentTimeCount = GetTickCount();

        // Insert timer counter here.
        char buf[1024];
        sprintf(buf, "%d: %s\0", CurrentTimeCount - ZeroTimeCount, format);

        // actual printf here.
		std::string fileName = "C:\\"+Messages[kMissionTitle_msg]+".txt";
		FILE *fp = fopen(fileName.c_str(), "a");
        vfprintf(fp, buf, argptr);
        va_end ( argptr ) ;
        fflush(fp);
		fclose(fp);
#endif
}


/*
	// Add this to tPlayerTeam::mCreateShipsForTeam !
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();
	gST.fHumanShipsSetUp = 1;
	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateShipsForTeam\n");
 */


// Add me just before the players' ships are created. 
// ie: at the start of PlayerTeam::mCreateShip
void shiplist_tools::mInitializeSL( void )
{
	if (global_sl.size() > 0) {
		return;
	}

	mDebug_fprintf("--- Entering: shiplist_tools::mInitializeSL\n");
	int32 fShiplistCount = 0;

	bool is_meta = true;
	// Load ShipList
	if (global_sl.size() == 0) {
		mDebug_fprintf("loading shiplist\n");
#ifdef OP
		global_sl.CreateMaster(ShipList::kOrionPirates, is_meta);
#else
		global_sl.CreateMaster(ShipList::kEmpiresAtWar, is_meta);
#endif
		mDebug_fprintf("shiplist loaded. ships: %d\n", global_sl.size());
		narrowed_sl = global_sl;
	}
	

#ifdef FIGHTERS
	if (fighter_sl.size() == 0) {
		mDebug_fprintf("loading fighter list\n");
#ifdef OP
		fighter_sl.CreateFighterList(ShipList::kOrionPirates, is_meta);
#else
		fighter_sl.CreateFighterList(ShipList::kEmpiresAtWar, is_meta);
#endif
		mDebug_fprintf("fighter list loaded. ships: %d\n", fighter_sl.size());
		narrowed_fighter_sl = fighter_sl;
	}
#endif

	mDebug_fprintf("--- Finishing: shiplist_tools::mInitializeSL\n");
}

void shiplist_tools::listShiplistContent( ShipList this_sl ) 
{
#ifdef SLTOOLSDEBUGOUT
		{
		SHIP_LIST_ITER(this_sl)
		  for (;;)
		  {
			mDebug_fprintf("   ship %s:  %d BPV.  YFA: %d   YLA: %d\n", 
				iter->shipCode.c_str(), iter->bpv, iter->yearFirstAvail, iter->yearLastAvail);
			break;
		  }
		SHIP_LIST_ITER_END
		}
#endif
}


void shiplist_tools::mDraftedShipCensus( const char* shipCode )
{
    mDebug_fprintf("--- Entering: shiplist_tools::mDraftedShipCensus - %s\n", shipCode);

	fDraftedShipsSetUp++;

	Ship aShip;
    global_sl.Find( shipCode, aShip);
    selected_drafted_sl.push_back(aShip);

	// kOrion ships have last digit stripped. ANNOYING.
	// Gotta create a map to track them for later tracking.
	mTrackOrions(aShip);

    mDebug_fprintf("Human/Drafted Ship: %s. YFA: %d  YLA: %d\n",
             shipCode, aShip.yearFirstAvail, aShip.yearLastAvail);
    mDebug_fprintf("--- Finished: shiplist_tools::mDraftedShipCensus\n");
}


ShipList shiplist_tools::stripOutSPECIAL( ShipList this_sl )
{
	// Strip out ships with SPECIAL
	std::list< eClassTypes >  classTypeList;
	classTypeList.push_back(kClassSpecial);
	int32 fShiplistCount = this_sl.Subset(classTypeList, false);
	mDebug_fprintf("subset for eClassTypes (no SPECIAL) completed. ships: %d\n", fShiplistCount);
	return this_sl;
}


ShipList shiplist_tools::stripOutRestricted( ShipList this_sl )
{
	// Strip out Qships with 'R' restricted.
	{
	SHIP_LIST_ITER(this_sl)
	  for (;;)
	  {
		if (iter->specialRole == kSpecialRoleR) {
		  iter = this_sl.erase(iter);
		  if (iter != this_sl.end()) continue; else break;
		}
		break;
	  }
	SHIP_LIST_ITER_END
	}
	mDebug_fprintf("subset for eSpecialRole (no R) completed. ships: %d\n", this_sl.size());
	return this_sl;
}


ShipList shiplist_tools::mFilterShipListToSingleRace( ShipList this_sl, eRace race ) 
{
	std::list< eRace >  raceList;
	raceList.push_back(race);
	int32 fShiplistCount = this_sl.Subset(raceList);
	mDebug_fprintf("subset for race (%d) filter completed. entries: %d\n", race, fShiplistCount);

	return this_sl;
}


ShipList shiplist_tools::selectWarShips( ShipList this_sl )
{
    if (this_sl.mIsEnhancedShiplist()) {
        int32 fShiplistCount = 0;

        std::list< eEnhancedRoles >  warRolesList;
        warRolesList.push_back(kLineShip);
        warRolesList.push_back(kWarShip);
        warRolesList.push_back(kLeader);
        warRolesList.push_back(kDroneBombardmentShip);
        warRolesList.push_back(kEscort);
        warRolesList.push_back(kCarrier);
        warRolesList.push_back(kPFTender);
        warRolesList.push_back(kFastCruiser);
        warRolesList.push_back(kMauler);
        warRolesList.push_back(kPolice);
        // and a monster
        warRolesList.push_back(kRoleMonster);
        fShiplistCount = this_sl.Subset(warRolesList);
        gST.mDebug_fprintf("subset for warship eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  doNotWantRolesList;
        doNotWantRolesList.push_back(kLocalDefense);
        fShiplistCount = this_sl.Subset(doNotWantRolesList, false);
        gST.mDebug_fprintf("filtered out roles we do not want for warship encounters. ships: %d\n", fShiplistCount);
    }
    return this_sl;
}


// Add me to the first AI team to be created. Or all. Doesn't matter
// ie: tNPC1Team::mCreateShipsForTeam
void shiplist_tools::mCompleteNarrowedList( void )
{
	if (! fNarrowedListFullySetUp) {
		mDebug_fprintf("--- Entering shiplist_tools::mCompleteNarrowedList\n");

		// We need a delay until the human ships are loaded
		int32 waitForHumans=0;
		while (
			(fDraftedShipsSetUp == 0) && (waitForHumans < 200)
		) {
			waitForHumans++;
			Sleep(50); //MilliSeconds
		}
		mDebug_fprintf("Waited for humans to be set up: %d milliseconds\n", waitForHumans * 50);

		if ((narrowed_sl.size() != 0) && (selected_drafted_sl.size() != 0)) {
			bool XShip_allowed = 0;
			fEarliestYear = -999;
			fLatestYear = 999;
				
			SHIP_LIST_ITER(selected_drafted_sl)
			mDebug_fprintf("parsing human/drafted ship(%s): YFA: %d  YLA %d\n", iter->shipCode.c_str(), iter->yearFirstAvail, iter->yearLastAvail);
			if (iter->yearFirstAvail > fEarliestYear) {
				mDebug_fprintf("replacing fEarliestYear: %d to %d\n", fEarliestYear, iter->yearFirstAvail);
				fEarliestYear = iter->yearFirstAvail;
			}
			if (iter->yearLastAvail < fLatestYear) {
				mDebug_fprintf("replacing fLatestYear: %d to %d\n", fLatestYear, iter->yearLastAvail);
				fLatestYear = iter->yearLastAvail;
			}
			if (selected_drafted_sl.mIsEnhancedShiplist()) {
				if (iter->enhancedXtech != kNoXtech) {
					XShip_allowed = 1;
					mDebug_fprintf("Xship (%d) on player team: %s\n", iter->enhancedXtech, iter->shipCode.c_str());
				}
			}
			SHIP_LIST_ITER_END

			if (fEarliestYear > fLatestYear) {  // I've seen this happen! --FS
				mDebug_fprintf("WTF! fEarliestYear > fLatestYear! Swapping the 2 around.\n");
				int32 temp = fEarliestYear;
				fEarliestYear = fLatestYear;
				fLatestYear = temp;
			}

			// no player has an x-ship, so AI isn't allowed one.
			// This fixes the fact that xships can exist in late era and appear too early.
			if (XShip_allowed == 0) {
				if (fLatestYear > kX1Begin) {
					fLatestYear = kX1Begin - 1;
				}
				mDebug_fprintf("There are no Xships on player team. No AI X-ships allowed.\n");
			}

			// we alread have a subset function for cleaning up the shiplist based on years of availability
			mDebug_fprintf("Estimated years of play: %d to %d\n", fEarliestYear, fLatestYear);
			int32 fShiplistCount = narrowed_sl.Subset(ShipList::kSubsetYearRangeExists, fEarliestYear, fLatestYear);
			mDebug_fprintf("subset for year range completed. ships: %d\n", fShiplistCount);

#ifdef FIGHTERS
			fShiplistCount = narrowed_fighter_sl.Subset(ShipList::kSubsetYearRangeExists, fEarliestYear, fLatestYear);
			mDebug_fprintf("fighter subset for year range completed. fighters: %d\n", fShiplistCount);
#endif
		}
		fNarrowedListFullySetUp = true;

		mDebug_fprintf("--- Finished shiplist_tools::mCompleteNarrowedList\n");
	}
}


int32 shiplist_tools::mGetModelSize( std::string geometry )
{
	FILE* modelSizFP;
	std::string modelSizFileName;
	bool registry_location_success;

#ifdef OP
	registry_location_success = FindModelsDirectory(kOrionPirates, modelSizFileName);
#else
	registry_location_success = FindModelsDirectory(kEmpiresAtWar, modelSizFileName);
#endif
	modelSizFileName += "model.siz";


	if ( ! ( registry_location_success && FileExists(modelSizFileName) ) ) {
		mDebug_fprintf("File not Found! %s\n", modelSizFileName.c_str());
		return -1;
	}

	modelSizFP = fopen(modelSizFileName.c_str(), "r");
    if (!modelSizFP)
        return -1;

    for (;;)
    {
        char inputBuf[1024];
        if (!fgets(inputBuf, 1024, modelSizFP))
            break;      // assume EOF

		inputBuf[strlen(inputBuf) - 1] = 0x00;      /* trip EOL-CR */
		Tokenizer   tok(inputBuf, "\t");

		std::string model = tok.getNextTokenC();
		float modelSize = tok.getNextTokenF();

		std::transform(model.begin(), model.end(), model.begin(), (int(*)(int)) toupper);
		std::replace( model.begin(), model.end(), '\\', '/' );
		std::transform(geometry.begin(), geometry.end(), geometry.begin(), (int(*)(int)) toupper);
		std::replace( geometry.begin(), geometry.end(), '\\', '/' );
		if (model.compare(geometry) == 0) {
			return modelSize;
			break;
		}
	}

	return -1;
}


bool shiplist_tools::FindModelsDirectory(eGame game, std::string &location)
{
    HKEY    hKey;
    LONG    lval;
    DWORD   type;
    char    path[256];
    DWORD   siz = 256;

    if (game == ShipList::kOrionPirates)
    {
        lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                            "Software\\Taldren\\Starfleet Command Orion Pirates",
                            0,
                            KEY_QUERY_VALUE,
                            &hKey);

        if (lval != ERROR_SUCCESS)
            return false;

        lval = RegQueryValueEx( hKey,
                                "Directory",
                                0,
                                &type,
                                (unsigned char *)path,
                                &siz );

        RegCloseKey(hKey);

        if (lval != ERROR_SUCCESS)
            return false;
    }
    else if (game == ShipList::kEmpiresAtWar)
    {
        lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                            "Software\\Taldren\\Starfleet Command II",
                            0,
                            KEY_QUERY_VALUE,
                            &hKey);

        if (lval != ERROR_SUCCESS)
            return false;

        lval = RegQueryValueEx( hKey,
                                "Path",
                                0,
                                &type,
                                (unsigned char *)path,
                                &siz );

        RegCloseKey(hKey);

        if (lval != ERROR_SUCCESS)
            return false;
    }
    else
    {
        DebugBreak();
    }

    path[siz] = 0;
    if (path[siz - 1] != '\\')
        strcat(path, "\\");

    strcat(path, "Assets\\Models\\");

    location = path;
    return true;
}


bool shiplist_tools::FileExists( std::string strFilename ) {
  struct stat stFileInfo;
  int intStat;

  // Attempt to get the file attributes
  intStat = stat(strFilename.c_str(),&stFileInfo);
  if(intStat == 0) {
    return true;
  } else {
    return false;
  }
  
  return false;
}


int32 shiplist_tools::mUpgradeFighters( tShipInfo *ship )
{
#ifdef FIGHTERS
	if (! ship) return 0;

	// compensate for the orion number chomping silliness.
	std::string fShipClassName = mGetTrackedOrion(ship->mGetClassName());

	mCompleteNarrowedList(); // just in case;

    Ship s1;
	mDebug_fprintf("---- Checking for fighters on: %s.\n", fShipClassName.c_str());
	if (! narrowed_sl.Find(fShipClassName.c_str(), s1)) {
		mDebug_fprintf("Ship %s not found! Aborting fighter upgrade.\n", fShipClassName.c_str());
		return 0;
	}

	// check for fighters.
	// up to 4 fighter bays
	int num_bays = 0;
	int32 count = 0;
	int32 total_num_fighters = 0;
	Ship selected_fighter;
	BaseShip fighter;
	bool found_fighter = false;

	for (int k = 0; k < 4; k++)
	{
		// assume all fighters are the same in the ship
		// load only the first one.
		if (! found_fighter) {
			fighter = s1.fighters[k].first;
			found_fighter = true;
		}
		count = s1.fighters[k].second;
		if (count)
		{
			num_bays++;
			total_num_fighters += count;
		}
	}
	mDebug_fprintf("Total number of Fighters: %d\n", total_num_fighters);

	if (total_num_fighters)
	{
		// We have fighters!
		// print out: fighter.shipCode
		mDebug_fprintf("We have fighters! Bays: %d Count: %d  Type: %s\n",
			num_bays, total_num_fighters, fighter.shipCode.c_str());

		ShipList tmpFighter_sl;

		mDebug_fprintf("Removing fighter selection which are less than %d BPV.\n", fighter.bpv);

		{
		SHIP_LIST_ITER(narrowed_fighter_sl)
			if (
				(iter->race == s1.race)
				&& (iter->bpv >= fighter.bpv)
			) {
				tmpFighter_sl.push_back(*iter);
			}
		SHIP_LIST_ITER_END
		}

		if (tmpFighter_sl.size() == 0) {
			mDebug_fprintf("weird! empty fighter shiplist. Aborting!\n");
			return 0;
		}
		listShiplistContent(tmpFighter_sl);

		tmpFighter_sl.SelectRandomShip(selected_fighter);

		int32 avg_fighters = total_num_fighters / num_bays;
		if (total_num_fighters % num_bays)
			avg_fighters++;
		// print out: fighter.shipCode

		mDebug_fprintf("New Fighter type: Bays: %d   #/bay: %d  Type: %s\n",
			num_bays, avg_fighters, selected_fighter.shipCode.c_str());
			
 		ship->mSetFighters(num_bays, avg_fighters, selected_fighter.shipCode);
		mDebug_fprintf("---- Done: fighters on: %s.\n", ship->mGetClassName());

		return (num_bays * avg_fighters * selected_fighter.bpv) - (num_bays * avg_fighters * fighter.bpv);
	}
#endif // FIGHTERS
	return 0;
}


int32 shiplist_tools::mUpgradeFightersForTeam( tTeamInfo *ti )
{
	int32 fTeam_BPV_Increase = 0;
#ifdef FIGHTERS
	if (ti) {
		mDebug_fprintf("--- Entering shiplist_tools::mUpgradeFightersForTeam: Team: %d.\n", ti->mGetTeam());
		tShipIterator tsi = ti->mGetFirstShip();
		for (;;)
		{
			Ship aShip;
			tShipInfo *si = *tsi;
			if (!si)
				break;

			fTeam_BPV_Increase += mUpgradeFighters( si );

			tsi++;
		}
		mDebug_fprintf("--- Finished shiplist_tools::mUpgradeFightersForTeam: Team: %d.\n", ti->mGetTeam());
	}
#endif
	return fTeam_BPV_Increase;
}


void shiplist_tools::mPickShip(int32 BPV, eRace race, Ship &s1, ShipList provided_sl )
{
    mDebug_fprintf("---- Entering: mPickShip( BPV: %d, race: %d, Ship &s1\n", BPV, race);

    int32 fShiplistCount = 0;
    ShipList this_sl;

    if (
        ( provided_sl.size() > 0 ) && ( provided_sl.mIsEnhancedShiplist() )
    ) {
        this_sl = provided_sl;
    } else {
        this_sl = narrowed_sl;

        std::list< eRace >  raceList;
        raceList.push_back(race);
        fShiplistCount = this_sl.Subset(raceList);
        mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", race, fShiplistCount);

        this_sl = stripOutRestricted( this_sl );
        this_sl = stripOutSPECIAL( this_sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = this_sl.Subset(shipClassTypeList, true);
        mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);
    }

    fShiplistCount = this_sl.Subset( this_sl.kSubsetBPV, int(0.9 * BPV), int(1.1 * BPV), true);
    mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
        int(0.9 * BPV), int(1.1 * BPV), fShiplistCount);

    if ((rand() % 37) < 23) {
        mDebug_fprintf("picking a ship of the line\n");
        ShipList lineShip_sl = this_sl;
        std::list< eEnhancedRoles >  RolesList;
        mDebug_fprintf("Selecting kLineShip role.\n");
        RolesList.push_back(kLineShip);
        fShiplistCount = lineShip_sl.Subset(RolesList);
        mDebug_fprintf("subset for kLineShip eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        listShiplistContent(lineShip_sl);

        lineShip_sl.SelectRandomShip(s1);
    } else {
        mDebug_fprintf("picking a random warship\n");
        ShipList warShip_sl = selectWarShips( this_sl );
        listShiplistContent(warShip_sl);

        warShip_sl.SelectRandomShip(s1);
    }
    mDebug_fprintf("ship selected! %s:  %d BPV\n", s1.shipCode.c_str(), s1.bpv);

    mDebug_fprintf("---- Finished: mPickShip( BPV: %d, race: %d, Ship &s1\n", BPV, race);
}


int32 shiplist_tools::mPickFleet(int32 bpv, int32 numships, eRace race, std::string chosenShips[])
{
	mDebug_fprintf("---- Entering: mPickFleet( bpv: %d, numships: %d, race: %d, std::string chosenShips[] )\n", 
		bpv, numships, race);

	// bail out if they give us impossible options
	if (
		((race < 0) || (race > kNeutralRace)) ||
		((numships < 1) || (bpv < 1) || (numships > 6)) 
	) {
		mDebug_fprintf("---- Finished: mPickFleet( bpv: %d, numships: %d, race: %d, std::string chosenShips[] )\n", 
			bpv, numships, race);
		return 0;
	}


	int32 fShiplistCount = 0;

	ShipList this_sl = narrowed_sl;
	std::list< eRace >  raceList;
	raceList.push_back(race);
	fShiplistCount = this_sl.Subset(raceList);
	mDebug_fprintf("subset for race (%d) filter completed. ships: %d\n", race, fShiplistCount);

	this_sl = stripOutRestricted( this_sl );
	this_sl = stripOutSPECIAL( this_sl );

    std::list< eEnhancedClassTypes >  shipClassTypeList;
    shipClassTypeList.push_back(kEClassSmallMonster);
    shipClassTypeList.push_back(kEClassMediumMonster);
    shipClassTypeList.push_back(kEClassLargeMonster);
    for (int32 x = kEClassFirstShip; x <= kEClassLastShip; x++) {
        shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
    }
    fShiplistCount = this_sl.Subset(shipClassTypeList, true);
    mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

	this_sl = selectWarShips( this_sl );
	

	// find the range of available ship bpvs
	int32 smallval = this_sl.MinimumBPV();
	int32 largeval = this_sl.MaximumBPV();

	if (smallval > bpv) {
		mDebug_fprintf("---- Finished: mPickFleet( bpv: %d, numships: %d, race: %d, std::string chosenShips[] )\n", 
			bpv, numships, race);
		return 0;
	}
	if (smallval < 1) smallval = 1;
	if (largeval < smallval) largeval = smallval;

	// using the tree created by mReadShiplist,
	// create a fleet of ships with the target race, bpv, and number of ships
	//    and return the actual bpv of the resulting fleet
	// the ship types are stored in the chosenShips array,
	//    so it is up to the caller to then call the mCreateShip methods

	// initialize the ship names, 
	//   just to make sure they didn't pass us something
	//   that had values in it already
	int32 i;
	int32 bpvremaining = bpv;
	int32 actualbpv = 0;
	int32 shipsremaining = numships;
	for (i = 0; i < 6; i++) chosenShips[i] = "";

	// otherwise look for a reasonable breakdown of bpvs into the number of ships
	int32 currship = 0;
	while ((shipsremaining > 0) && (bpvremaining > smallval)) {
		// pick an upper bound for the bpv of the next ship to add
		int32 thisbpv = bpvremaining;
		switch (shipsremaining) {
		   case 6: // if this is the sixth to last ship, it gets 15-25% of the remaining bpv
			   thisbpv = (thisbpv * (15 + (rand() % 10))) / 100; 
			   break;
		   case 5: // if this is the fifth to last ship, it gets 20-30% of the remaining bpv
			   thisbpv = (thisbpv * (20 + (rand() % 10))) / 100; 
			   break;
		   case 4: // if this is the fourth to last ship, it gets 25-35% of the remaining bpv
			   thisbpv = (thisbpv * (25 + (rand() % 10))) / 100; 
			   break;
		   case 3: // if this is the third to last ship, it gets 35-50% of the remaining bpv
			   thisbpv = (thisbpv * (35 + (rand() % 15))) / 100; 
			   break;
		   case 2: // if this is the second to last ship, it gets 50-70% of the remaining bpv
			   thisbpv = (thisbpv * (50 + (rand() % 20))) / 100; 
			   break;
		   default: // in general this ship gets all the remaining bpv
			   break;
		}
		Ship s1;
		mPickShip(thisbpv, race, s1, this_sl);
		chosenShips[currship] = s1.shipCode.c_str();
		thisbpv = s1.bpv;

		actualbpv += thisbpv;
		bpvremaining = bpvremaining - thisbpv;
		shipsremaining--;
		if (actualbpv > 0) currship++;
	}

	mDebug_fprintf("---- Finished: mPickFleet( bpv: %d, numships: %d, race: %d, std::string chosenShips[] )\n", 
		bpv, numships, race);

	return actualbpv;
}

void shiplist_tools::mTrackOrions( Ship aShip ) 
{
	if (aShip.race != kOrion) return;

	mDebug_fprintf("---- Entering shiplist_tools::mTrackOrions( %s )\n", aShip.shipCode.c_str());

	std::string fChompedShipCode = mChompDigit(aShip.shipCode.c_str());
	mDebug_fprintf("chomped shipCode : %s\n", fChompedShipCode.c_str());

	tOrionShipMap::iterator iter = fOrionShipMap.begin();
	iter = fOrionShipMap.find(fChompedShipCode.c_str());

	if ((fOrionShipMap.size() == 0) || (iter == fOrionShipMap.end())) {
		mDebug_fprintf("Adding ship %s -> %s to fOrionShipMap...\n", 
			fChompedShipCode.c_str(), aShip.shipCode.c_str());
		fOrionShipMap.insert(tOrionShipMap::value_type(fChompedShipCode, aShip.shipCode));

	} else {
		mDebug_fprintf("Ship %s -> %s already in fOrionShipMap\n", 
			fChompedShipCode.c_str(), aShip.shipCode.c_str());
	}

	mDebug_fprintf("---- Finished shiplist_tools::mTrackOrions( %s )\n", aShip.shipCode.c_str());
}

std::string shiplist_tools::mChompDigit( std::string shipCode )
{
   char fStripChars[] = "1234567890";

   for (unsigned int i = 0; i < sizeof(fStripChars); ++i)
   {
      // you need include <algorithm> to use general algorithms like std::remove()
      shipCode.erase(std::remove(shipCode.begin(), shipCode.end(), fStripChars[i]), shipCode.end());
   }
   return shipCode;
}

std::string shiplist_tools::mGetTrackedOrion( std::string shipCode ) 
{
	tOrionShipMap::iterator iter = fOrionShipMap.begin();
	iter = fOrionShipMap.find(shipCode.c_str());
	if ((fOrionShipMap.size() > 0) && (iter != fOrionShipMap.end())) {
		// found
		mDebug_fprintf("Oh hey! You asked for %s.. you meant %s!\n", shipCode.c_str(), iter->second.c_str());
		return iter->second;
	}
	return shipCode;
}

void shiplist_tools::mCensusOnlyDraft( std::string originFunction, const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: %s::mCreateNewShip: %s\n",
        originFunction.c_str(), shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

   // don't actually draft it. We're just collecting era data!
    gST.mDebug_fprintf("We don't actually draft ship %s. We're just collecting data!\n",
        shipScriptDescription.fClassName.c_str());

    gST.mDebug_fprintf("--- Finished: %s::mCreateNewShip: %s\n",
        originFunction.c_str(), shipScriptDescription.fClassName.c_str());
}