/********************************************************************************************************\

	Mission:	"Nor Iron Bars A Cage"
	Script:		"Uni_3IronCage"
	Descript:	The player attempts to intercept a prison ship, board it, and return it to their space.

	Created by:		Daniel J. Suleski	3/10/00
	Last Updated:	DJS					3/14/00
					DJS					3/31/00
					Second draft completed
					DJS						04/11/00
					Fix multiple-player ship random chatter
					Got rid of references to time.h
					DJS						05/10/00
					Initial conversion from old API to new multiplayer API
					DJS						05/20/00
					Asserts added and code cleaned
					DJS						05/26/00
					tShipIterator::mGetFirstShipInPlay() replaced with original form
					DJS						08/23/00
					Map overrides.
					DJS						09/25/00
					mCreateShip() "default-removal" modifications, kCampaignMission, tTeamSpec, etc
					DJS						09/26/00
					Meta script modifications
					FS                      09/09/12
					Update for OP+ shiplist, upgraded fighters, drone speeds checked.

	Thanks:		Zachary Drummond and Dave Ferrell

\********************************************************************************************************/

#include "Uni_3IronCage.h"
#include "BaseVictory.h"
#include "FreighterTeam.h"
#include "ISCTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "NPC1Team.h"
#include "PlayerTeam.h"
#include "SFCAssert.h"
#include "VictoryCondition.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_3IronCage );

tUni_3IronCage::tUni_3IronCage()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tUni_3IronCage::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team",
													static_cast< eTeamID >( kPlayerTeam ),
													kMaxChance,
													kTrue,
													kPrimaryTeam,
													kMinChance,
													kSpecPlayableRace & ~kSpecISC,
													kAnyRace,
													typeid( tPlayerTeam ),
													kNoTeamTag,
													Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& iscTeam = mCreateTeamSpec(	"ISC Team",
												static_cast< eTeamID >( kISCTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tISCTeam ) );

	const tTeamSpec& npc1Team = mCreateTeamSpec(	"NPC 1 Team",
													static_cast< eTeamID >( kNPC1Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace,
													kAnyRace,
													typeid( tNPC1Team ) );

	const tTeamSpec& freighterTeam = mCreateTeamSpec(	"Freighter Team",
														static_cast< eTeamID >( kFreighterTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kSpecISC,
														kAnyRace,
														typeid( tFreighterTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
   const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( iscTeam );
	Specs->mSetupTeam( npc1Team );
	Specs->mSetupTeam( freighterTeam );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tUni_3IronCage::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map -- base 1 index (not base 0!)
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tUni_3IronCage::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tUni_3IronCage::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tUni_3IronCage::mInitializeFinished( void )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return kBriefing_msg;
    eRaceName	playerRace = playerTeam->mGetRace();

	// return the index to the briefing message we want to use
	switch (playerRace)
	{
	case kFederation:
		{
			return ( kBriefingFederation_msg );
		}
		break;

	case kKlingon:
		{
			return ( kBriefingKlingon_msg );
		}
		break;

	case kRomulan:
		{
			return ( kBriefingRomulan_msg );
		}
		break;

	case kLyran:
		{
			return ( kBriefingLyran_msg );
		}
		break;

	case kGorn:
		{
			return ( kBriefingGorn_msg );
		}
		break;

	case kMirak:
		{
			return ( kBriefingMirak_msg );
		}
		break;

	case kHydran:
		{
			return ( kBriefingHydran_msg );
		}
		break;

	default:
		{
			return ( kBriefing_msg );
		}
		break;
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tUni_3IronCage::mStartMission( void )
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kISCTeam,
		kNPC1Team,
		kFreighterTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = fPlayersTeam,
		//ISC = ISCTeam,
		//NPC1 = NPC1,
		//Frei = Freighter Team

		//	     Plyr  ISC   NPC1  Frei
		/*Plyr*/ "--", "WD", "FI", "WX",
		/*ISC */ "WD", "--", "FI", "AI",
		/*NPC1*/ "FI", "FI", "--", "FI",
		/*Frei*/ "HI", "AI", "FI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tUni_3IronCage::mNewInternalShip( tShipInfo* Ship )
{
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_3IronCage::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['T'] = kObjectPlanet;

	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_3IronCage::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] = kBriefingRomulan_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefingLyran_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kBriefingGorn_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefingHydran_msg;
	fBriefingIndex[kPlayerTeam][kMirak] = kBriefingMirak_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_3IronCage::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}