/********************************************************************************************************\

	Mission:	"Tug of War"
	Script:		"Uni_3Tug"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		
	Last Updated:	FS				10/15/2012
					era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_3Tug.h"
#include "PlayerTeam.h"
#include "LyranTeam.h"
#include "KlingonTeam.h"
#include "FreighterTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_3Tug );

tUni_3Tug::tUni_3Tug()
{}

int32 tUni_3Tug::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_3Tug::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	// "Lyran" team is actually Mirak
	const tTeamSpec& team2 = mCreateTeamSpec(	"Lyran Team",
												static_cast< eTeamID >( kLyranTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tLyranTeam ) );

	// "Klingon" team is actually Gorn
	const tTeamSpec& team3 = mCreateTeamSpec(	"Klingon Team",
												static_cast< eTeamID >( kKlingonTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecGorn,
												kAnyRace,
												typeid( tKlingonTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Freighter Team",
												static_cast< eTeamID >( kFreighterTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran | kSpecMirak,
												kAnyRace,
												typeid( tFreighterTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_3Tug::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_3Tug::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_3Tug::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_3Tug::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tUni_3Tug::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_3Tug::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_3Tug::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kLyranTeam,
		kKlingonTeam,
		kFreighterTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Lyr  = kLyranTeam,
		//Kli  = kKlingonTeam,
		//Frt  = kFreighterTeam,

		//	     Plyr  Lyr   Kli   Frt
		/*Plyr*/ "--", "WD", "WD", "FI",
		/*Lyr*/  "WD", "--", "WD", "FI",
		/*Kli*/  "WD", "WD", "--", "FI",
		/*Frt*/  "PI", "PI", "PI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


void tUni_3Tug::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['Z'] = kObjectShipStart;
}

tMapTerrain tUni_3Tug::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_3Tug::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tUni_3Tug::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_msg;
}