/********************************************************************************************************\

	File Name:	MonsterTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MonsterTeam.h"
#include "MonsterTeamBaseState.h"
#include "Monster.h"
#include "Uni_4Playtime.h"
#include "PlayerTeam.h"
#include "PlanetTeam.h"
#include "AsteroidTeam.h"
#include "ShipList_API.h"


tClassAllocator<tMonsterTeam> gMonsterTeamAllocator;

tMonsterTeam::tMonsterTeam()
{
	fCurrentPriority = kNothing;
}

void tMonsterTeam::mSetupState()
{
	mRegisterState( new tMonsterTeamBaseState() );
	mGotoState( typeid( tMonsterTeamBaseState ) );
}

void tMonsterTeam::mSetupCommState()
{
}

void tMonsterTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tMonsterTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        ShipList monster_sl = sl;

        // 2 era-correct small monsters, picked randomly.
        std::list< eEnhancedClassTypes >  monsterClassList;
        monsterClassList.push_back(kEClassSmallMonster);
        fShiplistCount = monster_sl.Subset(monsterClassList);
        gST.mDebug_fprintf("monster class subset: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  monsterRolesList;
        monsterRolesList.push_back(kRoleMonster);
        fShiplistCount = monster_sl.Subset(monsterRolesList);
        gST.mDebug_fprintf("monster roles subset: %d\n", fShiplistCount);
        gST.listShiplistContent(monster_sl);

        if (monster_sl.size() > 0) {
            Ship s1, s2;
            monster_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            monster_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            mCreateShip( typeid( tMonster ),
                         s1.shipCode.c_str(),
                         kStartPosition_H,
                         kNoMetaShipID,
                         kMonster1,
                         0,0,0,0,
                         NULL,
                         kDefaultShipOptions );

            mCreateShip( typeid( tMonster ),
                         s2.shipCode.c_str(),
                         kStartPosition_I,
                         kNoMetaShipID,
                         kMonster2,
                         0,0,0,0,
                         NULL,
                         kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tMonsterTeam::mCreateShipsForTeam\n");
}


void tMonsterTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tMonsterTeam::mOldCreateShipsForTeam\n");

	mCreateShip( typeid( tMonster ),
				 "M-AMS",
				 kStartPosition_H,
				 kNoMetaShipID,
				 kMonster1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tMonster ),
				 "M-SGS",
				 kStartPosition_I,
				 kNoMetaShipID,
				 kMonster2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tMonsterTeam::mOldCreateShipsForTeam\n");
}

void tMonsterTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tMonsterTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tMonsterTeam", shipScriptDescription );
}

void tMonsterTeam::mSetTeamPriority( eTeamPriority priority )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
		return;

	tTeamInfo* asteroidTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAsteroidTeam ) );
	if( !asteroidTeam )
		return;

	switch( priority )
	{
	case kNothing:
		{
			fCurrentPriority = priority;
			return;
		}
		break;

	case kPlay:
		{
			fCurrentPriority = priority;
			mSetRelations( static_cast<eTeamID>(kPlayerTeam), kTeamAtWar, kGoalDestroy, 1.0f );

			tShipIterator playerShips = playerTeam->mGetFirstShip();
		
			tShipIterator monsters = mGetFirstShip();
			tShipInfo* monster1 = *monsters;
			monsters++;
			tShipInfo* monster2 = *monsters;

			if( monster1 )
			{
				if( *playerShips )
				{
					monster1->mSetShipTarget( *playerShips, kAbsolutePriority );
				}
			}

			if( monster2 )
			{
				if( *playerShips )
				{
					monster1->mSetShipTarget( *playerShips, kAbsolutePriority );
				}
			}
		}
		break;

	case kFeed:
		{
			fCurrentPriority = priority;

			mSetRelations( static_cast<eTeamID>(kPlayerTeam), kTeamFriendly, kGoalIgnore, 1.0f );

			tShipIterator monsters = mGetFirstShip();
			tShipInfo* monster1 = *monsters;
			monsters++;
			tShipInfo* monster2 = *monsters;

			tShipInfo* specialAsteroid = asteroidTeam->mFindShipCustomID( tAsteroidTeam::kRockSpecial );
			if( !specialAsteroid )
				return;

			int32 nX, nY;
			specialAsteroid->mGetGridPosition( &nX, &nY );
			nX++;
			nY++;

			if( monster1 ) monster1->mSetFinalDestination( nX, nY );
			if( monster2 ) monster2->mSetFinalDestination( nX, nY );
		}
		break;

	case kKillPlayer:
		{
			mSetRelations( static_cast<eTeamID>(kPlayerTeam), kTeamAtWar, kGoalDestroy, 1.0f );
		}
		break;
		
	default:
		break;
	}
}