/********************************************************************************************************\

	Mission:	"Playtime"
	Script:		"Uni_4Playtime"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		08/05/00
	Last Updated:	FS				10/16/2012
					era-correct ships, drones and fighters.. well.. ok. Fine. Monsters.

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_4Playtime.h"
#include "PlayerTeam.h"
#include "MonsterTeam.h"
#include "PlanetTeam.h"
#include "AsteroidTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"

DEFINE_SCRIPT_CLASS( tUni_4Playtime );

tUni_4Playtime::tUni_4Playtime()
{}

int32 tUni_4Playtime::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_4Playtime::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag, 
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Monster Team",
												static_cast< eTeamID >( kMonsterTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMonster,
												kAnyRace,
												typeid( tMonsterTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran | kSpecMirak,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Asteroid Team",
												static_cast< eTeamID >( kAsteroidTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tAsteroidTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

int32 tUni_4Playtime::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_4Playtime::mCreateExternalShips( int32 Mode )
{}

int32 tUni_4Playtime::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_4Playtime::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_4Playtime::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_4Playtime::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_4Playtime::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kMonsterTeam,
		kPlanetTeam,
		kAsteroidTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Mons = kMonsterTeam,
		//Plan = kPlanetTeam,
		//Astr = kAsteroidTeam,

		//	     Plyr  Mons  Plan  Astr
		/*Plyr*/ "--", "NI", "NI", "NI",
		/*Mons*/ "HI", "--", "NI", "NI",
		/*Plan*/ "PI", "PI", "--", "PI",
		/*Astr*/ "PI", "PI", "PI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tUni_4Playtime::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['M'] = kObjectPlanet;
	Map->fTacticalMapIcon['J'] = kObjectAsteroid;
	Map->fTacticalMapIcon['K'] = kObjectAsteroid;
	Map->fTacticalMapIcon['L'] = kObjectAsteroid;
}

tMapTerrain tUni_4Playtime::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_4Playtime::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tUni_4Playtime::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kLyran]		= kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon]	= kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran]	= kBriefing_msg;
}