//	<Name>	EnemyTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "tclient.h"
#include "ScriptInterface.h"
#include "EnemyTeam.h"
#include "EnemyTeamShips.h"
#include "Uni_4Resolve.h"
#include "ShipList_API.h"
#include "EnemyTeamBaseState.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

void tEnemyTeam::mSetupState()
{
    mRegisterState( new tEnemyTeamBaseState() );
    mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {

		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 nPCBPV; 
		if( diffLevel == 0 )
		{
			nPCBPV = playerBPV * 0.4;
		}
		else if( diffLevel == 1 )
		{
			nPCBPV = playerBPV * 0.6;
		}
		else if( diffLevel == 2 )
		{
			nPCBPV = playerBPV * 0.8;
		}

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutRestricted( sl );
        sl = gST.stripOutSPECIAL( sl );
		sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        fShiplistCount = sl.Subset( sl.kSubsetBPV, int(nPCBPV * 0.825), int(nPCBPV * 1.075), true);
        gST.mDebug_fprintf("sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(nPCBPV * 0.825), int(nPCBPV * 1.075), fShiplistCount);
        gST.listShiplistContent(sl);

        if (sl.size() > 0) {

            Ship s1;
            sl.SelectRandomShip(s1);
            gST.mTrackOrions( s1 );
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

            mCreateShip(
                typeid( tEnemyTeamShips ),
                s1.shipCode.c_str(),
                kStartPosition_L,
                kNoMetaShipID,
                -1,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }

	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.4;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 0.6;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 0.8;
	}

	mCreateShip(	
		typeid( tEnemyTeamShips ), 
		nPCBPV, 
		kClassFrigate, 
		kClassLightCruiser, 
		100, 
		kStartPosition_L,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}
