//	<Name>	LegendaryCaptainTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "tclient.h"
#include "ScriptInterface.h"
#include "LegendaryCaptainTeam.h"
#include "LegendaryCaptainTeamShip.h"
#include "Uni_4Resolve.h"
#include "ShipList_API.h"
#include "LegendaryCaptainTeamBaseState.h"


tClassAllocator<tLegendaryCaptainTeam> gLegendaryCaptainTeamAllocator;

void tLegendaryCaptainTeam::mSetupState()
{
    mRegisterState( new tLegendaryCaptainTeamBaseState() );
    mGotoState( typeid( tLegendaryCaptainTeamBaseState ) );
}

void tLegendaryCaptainTeam::mSetupCommState()
{}

void tLegendaryCaptainTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tLegendaryCaptainTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* PlayerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
        if (!PlayerTeam)
            return;
        int32 PlayerTeamBPV = PlayerTeam->mGetCombatBPV();
        int32 LegendaryCaptainBPV = PlayerTeamBPV;

        switch (fMissionInfo->mGetDifficultyLevel()) {
        case 0:
            LegendaryCaptainBPV *= 0.6; break;
        case 1:
            LegendaryCaptainBPV *= 0.8; break;
        case 2:
			LegendaryCaptainBPV *= 1.2; break;
        default:
            break;
        }

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

        ShipList ship_sl = sl;
        ship_sl = gST.stripOutRestricted( ship_sl );
        ship_sl = gST.stripOutSPECIAL( ship_sl );

        // Role will be LEADER.
        std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = ship_sl.Subset(leaderRolesList);
        gST.mDebug_fprintf("leader roles subset: %d\n", fShiplistCount);

        ShipList leader_sl = ship_sl;
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 x = kClassLightCruiser; x <= kEClassHeavyBattlecruiser; x++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
        }
        fShiplistCount = leader_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		fShiplistCount = leader_sl.Subset( leader_sl.kSubsetBPV, int(0.875 * LegendaryCaptainBPV), int(1.075 * LegendaryCaptainBPV), true);
		gST.mDebug_fprintf("subset for BPV range (%d - %d) completed. ships: %d\n",
			int(0.875 * LegendaryCaptainBPV), int(1.075 * LegendaryCaptainBPV), fShiplistCount);
		gST.listShiplistContent(leader_sl);

        if (leader_sl.size() > 0) {
            // Create ship
            Ship ship1;
            leader_sl.SelectRandomShip(ship1);
            gST.mDebug_fprintf("ship1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship1.shipCode.c_str(), ship1.bpv, ship1.yearFirstAvail, ship1.yearLastAvail);
            gST.mTrackOrions( ship1 );

            mCreateShip(
                typeid( tLegendaryCaptainTeamShip ),
                ship1.shipCode.c_str(),
                kStartPosition_M,
                kNoMetaShipID,
                -1,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tLegendaryCaptainTeam::mCreateShipsForTeam\n");
}


void tLegendaryCaptainTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tLegendaryCaptainTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 LegendBPV; 
	if( diffLevel == 0 )
	{
		LegendBPV = playerBPV * 0.6;
	}
	else if( diffLevel == 1 )
	{
		LegendBPV = playerBPV * 0.8;
	}
	else if( diffLevel == 2 )
	{
		LegendBPV = playerBPV * 1.2;
	}
	mCreateShip(
		typeid( tLegendaryCaptainTeamShip ), 
		LegendBPV, 
		kClassLightCruiser, 
		kClassHeavyBattlecruiser, 
		50, 
		kStartPosition_M,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tLegendaryCaptainTeam::mOldCreateShipsForTeam\n");
}

void tLegendaryCaptainTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tLegendaryCaptainTeam", shipScriptDescription );
}
