//	<Name>	PlanetTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "tclient.h"
#include "PlanetTeam.h"
#include "Planet.h"
#include "ShipList_API.h"
#include "PlanetTeamBaseState.h"

tClassAllocator<tPlanetTeam> gPlanetTeamAllocator;

void tPlanetTeam::mSetupState()
{
	mRegisterState( new tPlanetTeamBaseState() );
    mGotoState( typeid( tPlanetTeamBaseState ) );
}

void tPlanetTeam::mSetupCommState()
{
}

void tPlanetTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tPlanetTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();
        int32 fShiplistCount = 0;
        ShipList planet_sl = gST.mFilterShipListToSingleRace(gST.narrowed_sl, race);

        // Planet classes
        std::list< eEnhancedClassTypes >  planetClassTypeList;
        planetClassTypeList.push_back(kEClassPlanet);
        fShiplistCount = planet_sl.Subset(planetClassTypeList, true);
        gST.mDebug_fprintf("subset for planet class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  planetRolesList;
        // colony world eEnhancedRole
        gST.mDebug_fprintf("Filtering out homeworlds and armed worlds: colony planet.\n");
        planetRolesList.push_back(kHomeWorld);
        planetRolesList.push_back(kArmedWorld);
        fShiplistCount = planet_sl.Subset(planetRolesList, false);
        gST.mDebug_fprintf("subset for planet eEnhancedRoles completed. ships: %d\n", fShiplistCount);

        // remove bad planet choices and planets that are too large
        {
        SHIP_LIST_ITER(planet_sl)
            std::string badplanet = "";
            switch (race) {
                case kKlingon: badplanet = "K-PE"; break;
                case kRomulan: badplanet = "R-PE"; break;
                case kLyran: badplanet = "L-PE"; break;
                case kGorn: badplanet = "G-PE"; break;
                case kHydran: badplanet = "H-PE"; break;
                case kMirak: badplanet = "Z-PE"; break;
                case kISC: badplanet = "I-PE"; break;
                default: break;
            }
            for (;;) {
                if (gST.mGetModelSize(iter->geometry) > 151) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                if (badplanet.compare(iter->shipCode) == 0) {
                    iter = planet_sl.erase(iter);
                    if (iter != planet_sl.end()) continue; else break;
                }
                break;
            }
        SHIP_LIST_ITER_END
        }
        gST.mDebug_fprintf("Filtered planets that are too large for mission. planets: %d\n", planet_sl.size());
        gST.listShiplistContent(planet_sl);


        // And now to decide if we want to use shiplist API results or not
        Ship p1;
        if ((planet_sl.size() > 0) && (planet_sl.SelectRandomShip(p1))) {
            gST.mDebug_fprintf("p1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                p1.shipCode.c_str(), p1.bpv, p1.yearFirstAvail, p1.yearLastAvail);
            gST.mTrackOrions( p1 );

            mCreateShip(
                typeid( tPlanet ),
                p1.shipCode.c_str(),
                kStartPosition_Z,
                kNoMetaShipID,
                -1,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tPlanetTeam::mCreateShipsForTeam\n");
}

void tPlanetTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tPlanetTeam::mOldCreateShipsForTeam\n");

	mCreateShip(
		typeid( tPlanet ), 
		"PL9", 
		kStartPosition_Z,
		kNoMetaShipID, 
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tPlanetTeam::mOldCreateShipsForTeam\n");
}

void tPlanetTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlanetTeam", shipScriptDescription );
}
