//	<Name>	PlayerTeam.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Uni_4Resolve.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamInitialState.h"
#include "ResolveVictory.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fEnemyBaseScanned = false;
	fRaidSuccessful = false;
	fRaidFailed = false;
}
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tResolveVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels VictoryLevel )
{
	eRaceName  playerRace;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
    playerRace = playerTeam->mGetRace();

	// return the index to the debreifing message we want to use
	if( VictoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kLeftEarly_msg );
	}
	else if( ( playerRace == kKlingon ) || ( playerRace == kGorn ) || ( playerRace == kMirak ) )
	{
		if( VictoryLevel == kAstoundingVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kKliGorMirakAstoundingVictory_msg );
		}
		else if ( VictoryLevel == kVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kKliGorMirakVictory_msg );
		}
		else if ( VictoryLevel == kDraw )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kKliGorMirakDraw_msg );
		}
		else
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kKliGorMirakDevastatingDefeat_msg );
		}
	}
	else if( playerRace == kRomulan )
	{
		if( VictoryLevel == kAstoundingVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kRomAstoundingVictory_msg );
		}
		else if ( VictoryLevel == kVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kRomVictory_msg );
		}
		else if ( VictoryLevel == kDraw )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kRomDraw_msg );
		}
		else
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kRomDevastatingDefeat_msg );
		}
	}
	else if( playerRace == kLyran )
	{
		if( VictoryLevel == kAstoundingVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kLyrAstoundingVictory_msg );
		}
		else if ( VictoryLevel == kVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kLyrVictory_msg );
		}
		else if ( VictoryLevel == kDraw )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kLyrDraw_msg );
		}
		else
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kLyrDevastatingDefeat_msg );
		}
	}
	else
	{
		if( VictoryLevel == kAstoundingVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kFedHydAstoundingVictory_msg );
		}
		else if ( VictoryLevel == kVictory )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kFedHydVictory_msg );
		}
		else if ( VictoryLevel == kDraw )
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kFedHydDraw_msg );
		}
		else
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kFedHydDkDevastatingDefeat_msg );
		}
	}
	//
	//	Schedule the next mission.
	//
	mScheduleNextMission( fVictoryCondition );

}

void tPlayerTeam::mCreateShipsForTeam( int count, tShipInGame* ships[] )
{}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamInitialState( ) );
	mGotoState( typeid( tPlayerTeamInitialState ) );
}

void tPlayerTeam::mSetEnemyBaseScanned(bool setValue )
{
	fEnemyBaseScanned = setValue;
}
bool tPlayerTeam::mGetEnemyBaseScanned()
{
	return fEnemyBaseScanned;
}
void tPlayerTeam::mSetRaidSuccessful(bool setValue )
{
	fRaidSuccessful = setValue;
}
bool tPlayerTeam::mGetRaidSuccessful()
{
	return fRaidSuccessful;
}
void tPlayerTeam::mSetRaidFailed(bool setValue )
{
	fRaidFailed = setValue;
}
bool tPlayerTeam::mGetRaidFailed()
{
	return fRaidFailed;
}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	mCreateShip( typeid( tPlayerTeamShip ), 
			 shipScriptDescription.fClassName.c_str(), 
			 kStartPosition_G + shipScriptDescription.fTeamID,
			 MetaID,
			 -1, 
			 0, 0, -1, -1, //Default data,so we can set the custom name
			 shipScriptDescription.fCustomName.c_str(),
			 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	eRaceName playerRace = mGetRace();

	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;

#ifdef SLTOOLSDEBUGOUT
	gST.mDebug_fprintf("Debug mode on. Next mission scheduled: Uni_X1M1ASmuggler\n");
	fMissionScheduler.fNextMissionTitle = "Uni_X1M1ASmuggler";
#else
	switch( playerRace )
	{
		case kFederation:
		case kGorn:
		case kHydran:
		case kRomulan:
			{
				int32 missionDice = fMissionInfo->mRandomInt32( 1, 20 );
				
				if( missionDice < 16 )
				{
					fMissionScheduler.fNextMissionTitle = "Uni_X1M1ASmuggler";
				}
				else
				{
					fMissionScheduler.fNextMissionTitle = "Uni_5Accuse";
				}
			}
		break;
		case kMirak:
				fMissionScheduler.fNextMissionTitle = "Uni_X2M1Interceptor";
		break;
		default:
				fMissionScheduler.fNextMissionTitle = "Uni_5Accuse";
		break;
	}
#endif
}
