//	<Name>	PlayerTeamShipInitialState.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Uni_4Resolve.h"
#include "ShipInfo.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamShipInitialState.h"
#include "Planet.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "SFCAssert.h"

tPlayerTeamShipInitialState::tPlayerTeamShipInitialState( )
{
	fDeepScanON = false;
	fBaseLocated = false;
	fInfiltratorTeamDead = false;
	fOrderedInfiltratorTeamInTime = false;
}

void tPlayerTeamShipInitialState::mBeginState()
{
//	mSetTimedInterval( kTimerTurnsSet, kCheckDistanceID, 2.0f );
}

void tPlayerTeamShipInitialState::mEndState()
{
}
void tPlayerTeamShipInitialState::mOnTimedInterval( int32 timerID )
{
	int32 rangeToPlanetThisInterval = 100;

	tTeamInfo* planetTeam  = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlanetTeam ) );
	tShipIterator ship = planetTeam->mGetFirstShip();

	tShipInfo* planet = *ship;

	eRaceName  playerRace;

 	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );
	if( playerTeam == NULL )
	{
		return;
	}
	playerRace = playerTeam->mGetRace();

	fBaseLocated = playerTeam->mGetEnemyBaseScanned();

	if( timerID == kDeepScanONID )
	{
		if( fDeepScanON )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kBeginningScan_msg, kScienceScriptMessage );
			mSetTimedInterval( kTimerTurnsSet, kCheckDistanceID, 1.0f );

		}
	}
	// Only scan the planet if that is the players goal
	if( ( timerID == kCheckDistanceID ) && ( ( playerRace == kFederation) || ( playerRace == kRomulan) || ( playerRace == kLyran ) || ( playerRace == kHydran ) ) )
	{
	
		rangeToPlanetThisInterval = fChildActor->mGetTacticalDistance( planet );

		if( ( planet ) && !fBaseLocated && ( rangeToPlanetThisInterval < 10 ) && ( fDeepScanON ) && ( ( fChildActor->mGetShipTargetID() ) == planet->mGetShipID() ) )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kLocatedBase_msg, kScienceScriptMessage );
			playerTeam->mSetEnemyBaseScanned( true );
		}
		else
		{
			mSetTimedInterval( kTimerTurnsSet, kCheckDistanceID, 1.0f );
		}
	}
	else if( ( timerID == kWaitedToLongID ) && ( ! ( fOrderedInfiltratorTeamInTime ) ) )
	{
		playerTeam->mGotoCommState( planet->mGetShipID(), tPlanet::kBlankCommState );
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kWaitedToLong_msg );
		fInfiltratorTeamDead = true;
		playerTeam->mSetRaidFailed( true );
	}
	else if( ( timerID == kJobDoneID ) && ( ! ( fInfiltratorTeamDead ) ) )
	{
		playerTeam->mSetRaidSuccessful( true );
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kRaidComplete_msg );
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kSecondaryObjectiveComplete );
	}
}
void tPlayerTeamShipInitialState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fDeepScanON = true;
	}
	else
	{
		fDeepScanON = false;
	}
	ship->mSetShipCallbackEvents(  kNotifyDeepScan );
	mSetTimedInterval( kTimerTurnsSet, kDeepScanONID, 0.75f );
}
void tPlayerTeamShipInitialState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	tTeamInfo* planetTeam  = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlanetTeam ) );
	tShipIterator ship = planetTeam->mGetFirstShip();
	tShipInfo* planet = *ship;
	eRaceName  playerRace;
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );
	if( playerTeam == NULL )
	{
		return;
	}
	ship = playerTeam->mGetFirstShip();
	tShipInfo* playersFirstShip = *ship;

	playerRace = playerTeam->mGetRace();

	if ( ( playerTeam->mGetEnemyBaseScanned() ) && ( item == kTransInfiltrator ) && ( shipTo == planet ) )
	{
		if( playerRace == kRomulan )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kRomInfiltratorArrived_msg );
		}
		else if( playerRace == kLyran )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kLyrInfiltratorArrived_msg );
		}
		else
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kFedHydInfiltratorArrived_msg );
		}

		playerTeam->mGotoCommState( planet->mGetShipID(), tPlanet::kRaidDataBanksCommState );
		planet->mRemoveItemFromStores( kTransInfiltrator, 1 );	
		mSetTimedInterval( kTimerTurnsSet, kWaitedToLongID, 2.0f );

	}
	shipFrom->mSetShipCallbackEvents( kNotifyTransportedItem );  //from players ship TO another
	shipTo->mSetShipCallbackEvents( kNotifyTransportedItem );
}
void tPlayerTeamShipInitialState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{
}
void tPlayerTeamShipInitialState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if( buttonID == kRaid_btn )
	{
		mSetTimedInterval( kTimerTurnsSet, kJobDoneID, 1.0f );
		fOrderedInfiltratorTeamInTime = true;
	}
}