//	<Name>	ResolveVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ResolveVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Uni_4Resolve.h"
#include "ResolveVictory.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
//#include "FreighterTeamShipReachedPlanetState.h"

              
tResolveVictoryState::tResolveVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tResolveVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fEnemyShipsInPlay = 1;
	fLegendaryCaptainShipsInPlay = 1;
	fDryDockShipsInPlay = 4;
	fPlayerShipsInPlay = 1;
	fSpyInfilitrationComplete = false;
	fLegendaryCaptainDestroyedAsSecondary = false;
}

void tResolveVictoryState::mBeginState()
{

}

void tResolveVictoryState::mEndState()
{
}

eVictoryLevels tResolveVictoryState::mCalcVictoryStatus( eTeamID ID )
{
//	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( gScriptInterface.mPlayersTeam() );
//	Assert( playerTeam );

	eRaceName  playerRace;
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	playerRace = playerTeam->mGetRace();

	fSpyInfilitrationComplete = playerTeam->mGetRaidSuccessful();

	if( ( playerRace == kFederation) || ( playerRace == kRomulan) || ( playerRace == kLyran ) || ( playerRace == kHydran ) )
	{
		if( ( fDryDockShipsInPlay == 0  ) && ( fSpyInfilitrationComplete ) )
		{
			return kAstoundingVictory;
		}
		else if( fDryDockShipsInPlay == 0  )
		{
			return kVictory;
		}
		else if( ( fSpyInfilitrationComplete ) && ( fDryDockShipsInPlay < 4 ) )
		{
			return kDraw;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}
	else
	{
		if( ( fDryDockShipsInPlay == 0  ) && ( fLegendaryCaptainShipsInPlay == 0 ) )
		{
			return kAstoundingVictory;
		}
		else if( fDryDockShipsInPlay == 0  )
		{
			return kVictory;
		}
		else if( ( fLegendaryCaptainShipsInPlay == 0 ) && ( fDryDockShipsInPlay < 4 ) )
		{
			return kDraw;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}


}
int32 tResolveVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tResolveVictoryState::mCheckAllTeams( tShipInfo* ship )
{
}
void tResolveVictoryState::mOnDeath( tShipInfo* ship )
{
	eRaceName  playerRace;
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	tTeamInfo* legendaryCaptainTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kLegendaryCaptainTeam ) );
	tTeamInfo* dryDockTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kMainDryDockTeam ) );
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	playerRace = playerTeam->mGetRace();
	fSpyInfilitrationComplete = playerTeam->mGetRaidSuccessful();


	if( ship->mGetTeam() == kEnemyTeam )
	{
		fEnemyShipsInPlay = mShipsInPlay( enemyTeam );
	}
	else if( ship->mGetTeam() == kLegendaryCaptainTeam )
	{
		fLegendaryCaptainShipsInPlay = mShipsInPlay( legendaryCaptainTeam );
		if( ( playerRace == kKlingon) || ( playerRace == kGorn ) || ( playerRace == kMirak ) )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kSecondaryObjectiveComplete );
			fLegendaryCaptainDestroyedAsSecondary = true;
		}
	}
	else if( ship->mGetTeam() == kMainDryDockTeam )
	{
		fDryDockShipsInPlay = mShipsInPlay( dryDockTeam );
		if ( fDryDockShipsInPlay == 0 )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kPrimaryObjectiveComplete );
		}
	}
	else if( ship->mGetTeam() == ( playerTeam->mGetTeam() ) )
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );
	}
	if( fEnemyShipsInPlay == 0 
		&& fLegendaryCaptainShipsInPlay == 0 
		&& fDryDockShipsInPlay == 0 
		&& ( fSpyInfilitrationComplete || fLegendaryCaptainDestroyedAsSecondary || ( playerTeam->mGetRaidFailed() ) ) )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
	}
	if( fPlayerShipsInPlay == 0 )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerDestroyed );
	}

	mCheckAllTeams( ship );
}
void tResolveVictoryState::mOnCapture( tShipInfo* ship )
{
	eRaceName  playerRace;
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	tTeamInfo* legendaryCaptainTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kLegendaryCaptainTeam ) );
	tTeamInfo* dryDockTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kMainDryDockTeam ) );
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	playerRace = playerTeam->mGetRace();
	fSpyInfilitrationComplete = playerTeam->mGetRaidSuccessful();

	if( ship->mGetTeam() == playerTeam->mGetTeam() )
	{
		mAddToPrestige( 30 );
	}

	if( ship->mGetOriginalTeam() == kEnemyTeam )
	{
		fEnemyShipsInPlay = mShipsInPlay( enemyTeam );
	}
	else if( ship->mGetOriginalTeam() == kLegendaryCaptainTeam )
	{
		fLegendaryCaptainShipsInPlay = mShipsInPlay( legendaryCaptainTeam );
		if( ( playerRace == kKlingon) || ( playerRace == kGorn ) || ( playerRace == kMirak ) )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kSecondaryObjectiveComplete );
			fLegendaryCaptainDestroyedAsSecondary = true;
		}
	}
	else if( ship->mGetOriginalTeam() == kMainDryDockTeam )
	{
		fDryDockShipsInPlay = mShipsInPlay( dryDockTeam );
		if ( fDryDockShipsInPlay == 0 )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ),  kPrimaryObjectiveComplete );
		}
	}
	else if( ship->mGetOriginalTeam() == ( playerTeam->mGetTeam() ) )
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );
	}
	if( fEnemyShipsInPlay == 0 
		&& fLegendaryCaptainShipsInPlay == 0 
		&& fDryDockShipsInPlay == 0 
		&& ( fSpyInfilitrationComplete || fLegendaryCaptainDestroyedAsSecondary || ( playerTeam->mGetRaidFailed() ) ) )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
	}
	else if( fPlayerShipsInPlay == 0 )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerDestroyed );
	}
	else
	{
		mCheckAllTeams( ship );
	}


}

void tResolveVictoryState::mOnDisengaged( tShipInfo* ship )
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	fPlayerShipsInPlay = mShipsInPlay( playerTeam );

	if ( ( ship->mGetTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		if ( fPlayerShipsInPlay < 1 ) 
		{
			if( fLegendaryCaptainShipsInPlay == 0 && fDryDockShipsInPlay == 0 )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
			}
			else
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerDestroyed );
			}
		}
	}

}
void tResolveVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{

}
void tResolveVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if( buttonID == kRaid_btn )
	{
		mSetTimedInterval( kTimerTurnsSet, kJobDoneID, 1.2f );
	}
}
void tResolveVictoryState::mOnTimedInterval( int32 timerID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	if( timerID == kJobDoneID )
	{
		fSpyInfilitrationComplete = true;
	}

	if( fEnemyShipsInPlay == 0 
		&& fLegendaryCaptainShipsInPlay == 0 
		&& fDryDockShipsInPlay == 0 
		&& ( fSpyInfilitrationComplete || fLegendaryCaptainDestroyedAsSecondary || ( playerTeam->mGetRaidFailed() ) ) )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
	}
}