#include "Uni_4Resolve.h"

#include "MissionText.h"
#include "MissionMaps.h"

#include "VictoryCondition.h"
#include "MainDryDockTeam.h"
#include "PlanetTeam.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "FreighterTeam.h"
#include "LegendaryCaptainTeam.h"
#include "ResolveVictory.h"
#include "SFCAssert.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tUni_4Resolve );

tUni_4Resolve::tUni_4Resolve()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tUni_4Resolve::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;
	
	// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID >( kPlayerTeam ),
																		kMaxChance,
																		kFalse,
																		kPrimaryTeam,
																		kMaxChance,
																		kSpecPlayableRace & ~kSpecISC,
																		kAnyRace,
																		typeid( tPlayerTeam ), kNoTeamTag, Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID >( kEnemyTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace,
																		typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", static_cast< eTeamID >( kMainDryDockTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace, typeid( tMainDryDockTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", static_cast< eTeamID >( kPlanetTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace,
																		typeid( tPlanetTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5", static_cast< eTeamID >( kLegendaryCaptainTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace, typeid( tLegendaryCaptainTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6", static_cast< eTeamID >( kFreighterTeam ),
																		kMaxChance,
																		kTrue,
																		kPlayableTeam,
																		kMaxChance,
																		kSpecISC,
																		kAnyRace,
																		typeid( tFreighterTeam ) );
	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tUni_4Resolve::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tUni_4Resolve::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}
//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tUni_4Resolve::mStartMission( void )
{

	tScript::mStartMission();

	// setup team relations and target goals
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kMainDryDockTeam,
		kPlanetTeam,
		kLegendaryCaptainTeam,
		kFreighterTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr  = kPlayerTeam,
		//Enemy = kEnemyTeam,
		//DryD  = kMainDryDockTeam,
		//PLNT	= kPlanetTeam,
		//Legd	= kLegendaryCaptainTeam,
		//Frtr	= kFreighterTeam,

		//			Plyr	Enemy	DryD	PLNT	Legd	Frtr 
		/*Plyr*/	"--",	"WD",	"WD",	"WD",	"WD",	"AI",
		/*Enemy*/	"WD",	"--",	"AI",	"AI",	"AI",	"AI",
		/*DryD*/	"WD",	"AI",	"--",	"AI",	"AI",	"AI",
		/*PLNT*/	"WD",	"AI",	"AI",	"--",	"AI",	"AI",
		/*Legd*/	"WD",	"AI",	"AI",	"AI",	"--",	"AI",
		/*Frtr*/	"AI",	"AI",	"AI",	"AI",	"AI",	"--",

	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tUni_4Resolve::mNewInternalShip( tShipInfo* Ship )
{
}
void tUni_4Resolve::mCreateExternalShips( int32 Mode )
{

}
void tUni_4Resolve::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;


	Map->fTacticalMapIcon['L'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['M'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['V'] = kObjectEnemyBase;
	Map->fTacticalMapIcon['W'] = kObjectEnemyBase;
	Map->fTacticalMapIcon['X'] = kObjectEnemyBase;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyBase;
}
void tUni_4Resolve::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tUni_4Resolve::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] =	kFedBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] =		kKlingonBriefing_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] =		kRomulanBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] =		kLyranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] =		kHydranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] =		kGornBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] =		kMirakBriefing_msg;

}

