/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_5AFunny1.h"
#include "EnemyTeam.h"
#include "BaseTeam.h"
#include "Base.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	if( ( playerTeam->mBeamedDiplomat() ) & ( !playerTeam->mBaseDestroyed() ) )
		return kAstoundingVictory;

	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;
	
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );
	if( !enemyTeam )
			return;

	tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) );
	if( !baseTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					playerTeam->mSetDied();
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

	case tEnemyTeam::kEnemy1:
	case tEnemyTeam::kEnemy2:
	case tEnemyTeam::kEnemy3:
	case tEnemyTeam::kEnemy4:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kEnemiesDead_msg );
				
				tShipInfo* base = baseTeam->mGetThisShip(1);
				if( base )
				{
					playerTeam->mGotoCommState( base->mGetShipID(), tBase::kState2 );
				}

				if( playerTeam->mBeamedDiplomat() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				}
			}
		}
		break;

	case tBaseTeam::kBase:
		{
			playerTeam->mSetBaseDestroyed();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBaseDestroyed_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );
	if( !enemyTeam )
			return;

	tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kBaseTeam ) );
	if( !baseTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

	case tEnemyTeam::kEnemy1:
	case tEnemyTeam::kEnemy2:
	case tEnemyTeam::kEnemy3:
	case tEnemyTeam::kEnemy4:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kEnemiesDead_msg );
				
				tShipInfo* base = baseTeam->mGetThisShip(1);
				if( base )
				{
					playerTeam->mGotoCommState( base->mGetShipID(), tBase::kState2 );
				}

				if( playerTeam->mBeamedDiplomat() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				}
			}
		}
		break;
/*
	case tEnemyTeam::kEnemy1:
	case tEnemyTeam::kEnemy2:
	case tEnemyTeam::kEnemy3:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kEnemiesDead_msg );
			}
		}
		break;
*/
	case tBaseTeam::kBase:
		{
			playerTeam->mSetBaseDestroyed();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBaseDestroyed_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		}
		break;

	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

