/********************************************************************************************************\

	File Name:	AllyShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllyShip.h"
#include "AllyShipBaseState.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "ScriptInterface.h"
#include "Uni_5Accuse.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tAllyShipBaseState::mBeginState()
{}
void tAllyShipBaseState::mEndState()
{}

void tAllyShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tAllyShipBaseState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;

	//
	//	If an offer has not been made yet, then set the timer for it.
	//
	if ( !fChildActor->mFirstOfferMade() )
	{
		fChildActor->mSetTimedInterval( kTimerTurnsSet, tAllyShip::kFirstOfferID, 1.0 );
	}
	//
	//	If the ISC offer was not taken by the ally and the player has not attacked the ally, then a second
	//	offer will come through from the ISC.
	//
	else if ( !fChildActor->mSecondOfferMade() )
	{
		if ( !fChildActor->mFirstOfferISC() )
		{
			fChildActor->mSetTimedInterval( kTimerTurnsSet, tAllyShip::kSecondOfferID, 1.0 );
		}
	}
}
void tAllyShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( !targetShip )
		return;

	if ( fChildActor->mFirstOfferMade() && !fChildActor->mFirstDecisionFinished() )
	{
		if ( buttonID == k_REINFORCEALLIANCE_Ally_btn )
		{
			if ( !fChildActor->mAttackedAlly() )
			{
				fChildActor->mSetFirstOfferISC( false );
			}
		}
	}
	else if ( fChildActor->mSecondOfferMade() && !fChildActor->mSecondDecisionFinished() )
	{
		if ( buttonID == k_REINFORCEALLIANCE_Ally_btn )
		{
			if ( !fChildActor->mAttackedAlly() )
			{
				fChildActor->mSetSecondOfferISC( false );
			}
		}
	}
	else if ( fChildActor->mSecondDecisionFinished() && !fChildActor->mSecondOfferISC() )
	{
		if ( buttonID == k_REQUESTASSISTANCE_Ally_btn )
		{
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kAllyTeam ), static_cast< eTeamID >( kEnemyTeam ), kTeamAtWar, kGoalDestroy, 1.0 );
		}
	}
}
void tAllyShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	//
	//	Check for the player attacking the ally.  Also need to put a check in if the hostileShip
	//	exists because dust clouds can cause damage and are not identified with tShipInfo*.
	//
	if ( hostileShip )
	{
		if ( hostileShip->mGetTeam() == kPlayerTeam )
		{
			fChildActor->mSetAttackedAlly( true );
			if ( !fChildActor->mFirstOfferMade() )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_HAIL_AttackedAllyCommState );
			}
			else if ( !fChildActor->mSecondOfferMade() )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_REINFORCEALLIANCE_EnemiesCommState );
			}
			else
			{
				;	//	do nothing
			}
		}
	}
}
void tAllyShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tAllyShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tAllyShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllyShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllyShipBaseState::mOnFlashingButtonPressed()
{}
void tAllyShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tAllyShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tAllyShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tAllyShipBaseState::mOnHostChanged()
{}
void tAllyShipBaseState::mOnTimedInterval( int32 timerID )
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( timerID )
	{
	case tAllyShip::kFirstOfferID:
		{
			fChildActor->mSetFirstOfferMade( true );
			playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_REINFORCEALLIANCE_BaseCommState );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
			fChildActor->mSetTimedInterval( kTimerTurnsSet, tAllyShip::kFirstOfferDecisionID, 2.0 );
			if ( fChildActor->mAttackedAlly() )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_REINFORCEALLIANCE_EnemiesCommState );
			}
			else
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_REINFORCEALLIANCE_BaseCommState );
			}
		}
		break;

	case tAllyShip::kFirstOfferDecisionID:
		{
			fChildActor->mSetFirstDecisionFinished( true );

			//
			//	If the ISC bait was not taken, then track the distance of the player to the planet again
			//	for a second offer.
			//
			if ( !fChildActor->mFirstOfferISC() )
			{
				int32		gridX = 0 , gridY = 0;
				tTeamInfo*	organiaTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNPC1Team ) );
				Assert( organiaTeam );
				if ( !organiaTeam )
					return;
				tShipInfo*	organia = organiaTeam->mFindShipCustomID( tNPC1Team::kOrgania );

				if ( organia )
				{
					organia->mGetGridPosition( &gridX , &gridY );
					fChildActor->mSetFinalDestination( gridX, gridY );
					fChildActor->mTrackTacticalDistance( 200.0, gridX, gridY );
				}
			}
			//
			//	Otherwise, if the ISC bait was taken, then the ally should turn against the player.
			//
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_1_msg );
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kBlankCommState );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kAllyTeam ), static_cast< eTeamID >( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0 );
			}
		}
		break;

	case tAllyShip::kSecondOfferID:
		{
			fChildActor->mSetSecondOfferMade( true );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );
			fChildActor->mSetTimedInterval( kTimerTurnsSet, tAllyShip::kSecondOfferDecisionID, 2.0 );
			if ( fChildActor->mAttackedAlly() )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_REINFORCEALLIANCE_EnemiesCommState );
			}
			else
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_REINFORCEALLIANCE_FriendsCommState );
			}
		}
		break;

	case tAllyShip::kSecondOfferDecisionID:
		{
			fChildActor->mSetSecondDecisionFinished( true );

			//
			//	If the ISC bait was not taken, then set the relationships between the player and ally as
			//	being allied.  The ally will try to destroy the ISC once the comm button for assistance
			//	is depressed.
			//
			if ( !fChildActor->mSecondOfferISC() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::k_REQUESTASSISTANCE_FriendsCommState );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerTeam), static_cast< eTeamID >( kAllyTeam ), kTeamAllied, kGoalIgnore, 1.0 );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kAllyTeam ), static_cast< eTeamID >( kPlayerTeam ), kTeamAllied, kGoalIgnore, 1.0 );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kAllyTeam ), kTeamAtWar, kGoalDestroy, 1.0 );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kAllyTeam ), static_cast< eTeamID >( kEnemyTeam ), kTeamAtWar, kGoalDestroy, 1.0 );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0 );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerTeam ), static_cast< eTeamID >( kEnemyTeam ), kTeamAtWar, kGoalDestroy, 1.0 );				
				fChildActor->mSetShipMaxSpeed( 15 );
			}
			//
			//	Otherwise, if the ISC bait was taken, then the ally should turn against the player.
			//
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kBlankCommState );
				fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kAllyTeam ), static_cast< eTeamID >( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0 );
			}
		}
		break;
	}
}
void tAllyShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tAllyShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


