/********************************************************************************************************\

	File Name:	BaseVictory.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"
#include "Uni_5Accuse.h"


tClassAllocator<tBaseVictory> gBaseVictoryAllocator;

/*

	HELPER NOTE:	You must remember to initialize all of your victory attributes here.  For example:

	tBaseVictory::tBaseVictory( ) : fBSed( false ), field2( value ), field3 ( value ), etc...
	{}

*/
tBaseVictory::tBaseVictory( ) :
	fReachedOrgania ( false ) ,
	fAllyBetrayed ( false ) ,
	fAttackedAlly ( false ) ,
	fISCShipsDestroyed ( 0 ) ,
	fISCDefensePlatformsDestroyed ( 0 ),
	fFirstOfferMade ( false ) ,
	fFirstOfferISC ( false ) ,
	fSecondOfferMade ( false ) ,
	fSecondOfferISC ( false )
{}

tBaseVictory::tBaseVictory( tTeamInfo* team ) : tVictoryCondition( team ),
	fReachedOrgania ( false ) ,
	fAllyBetrayed ( false ) ,
	fAttackedAlly ( false ) ,
	fISCShipsDestroyed ( 0 ) ,
	fISCDefensePlatformsDestroyed ( 0 ),
	fFirstOfferMade ( false ) ,
	fFirstOfferISC ( false ) ,
	fSecondOfferMade ( false ) ,
	fSecondOfferISC ( false )
{}
/*

	HELPER NOTE:	Really, the only item in this function that you could change are the values passed into
					the tBaseVictoryState constructor.  These values are the prestige points awarded according
					to the player's victory status -- starting at kAstoundingVictory and working its way down
					to kDevastatingDefeat.

*/
void tBaseVictory::mSetupState()
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();
	tBaseVictoryState* baseVictoryState;

	switch ( playerRace )
	{
	case kFederation:
	case kRomulan:
	case kHydran:
		{
			baseVictoryState = new tBaseVictoryState( 300, 225, 75, 0, -75 );
		}
		break;

	case kKlingon:
	case kGorn:
	case kLyran:
	case kMirak:
		{
			baseVictoryState = new tBaseVictoryState( 300, 225, 75, 0, -75 );
		}
		break;
	}

	mRegisterState( baseVictoryState );	
	mGotoState( typeid( tBaseVictoryState ) );

	mSetTimedInterval( kTimerSecondsSet, kRandChatID, 20.0f );
}

/*

	HELPER NOTE:	The following is simply an example of setting and modifying the current victory condition
					objectives.

	bool
	tBaseVictory::mBSed( void )
	{
		return fBSed;
	}
	void
	tBaseVictory::mSetBSed( bool x )
	{
		fBSed = x;
	}

*/
//
//	Accessors.
//
bool tBaseVictory::mReachedOrgania( void )
{
	return fReachedOrgania;
}
bool tBaseVictory::mAllyBetrayed( void )
{
	return fAllyBetrayed;
}
bool tBaseVictory::mAttackedAlly( void )
{
	return fAttackedAlly;
}
int32 tBaseVictory::mISCShipsDestroyed( void )
{
	return fISCShipsDestroyed;
}
int32 tBaseVictory::mISCDefensePlatformsDestroyed( void )
{
	return fISCDefensePlatformsDestroyed;
}
bool tBaseVictory::mFirstOfferMade( void )
{
	return fFirstOfferMade;
}
bool tBaseVictory::mFirstOfferISC( void )
{
	return fFirstOfferISC;
}
bool tBaseVictory::mSecondOfferMade( void )
{
	return fSecondOfferMade;
}
bool tBaseVictory::mSecondOfferISC( void )
{
	return fSecondOfferISC;
}

//
//	Modifiers.
//
void tBaseVictory::mSetReachedOrgania( bool x )
{
	fReachedOrgania = x;
}
void tBaseVictory::mSetAllyBetrayed( bool x )
{
	fAllyBetrayed = x;
}
void tBaseVictory::mSetAttackedAlly( bool x )
{
	fAttackedAlly = x;
}
void tBaseVictory::mIncreaseISCShipsDestroyed( void )
{
	fISCShipsDestroyed++;
}
void tBaseVictory::mIncreaseISCDefensePlatformsDestroyed( void )
{
	fISCDefensePlatformsDestroyed++;
}
void tBaseVictory::mSetFirstOfferMade( bool x )
{
	fFirstOfferMade = x;
}
void tBaseVictory::mSetFirstOfferISC( bool x )
{
	fFirstOfferISC = x;
}
void tBaseVictory::mSetSecondOfferMade( bool x )
{
	fSecondOfferMade = x;
}
void tBaseVictory::mSetSecondOfferISC( bool x )
{
	fSecondOfferISC = x;
}