/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "ISCDefensePlatform.h"
#include "ISCShip.h"
#include "ScriptInterface.h"
#include "Uni_5Accuse.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");

    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
		int32 nPCBPV;
		switch ( fMissionInfo->mGetDifficultyLevel() )
		{
		case 0:
			nPCBPV = playerBPV * 0.75;	break;
		case 1:
			nPCBPV = playerBPV * 1.0;	break;
		case 2:
			nPCBPV = playerBPV * 1.25;	break;
		default:
			break;
		}

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );


		// Defense platforms
		ShipList DEF_sl = sl;
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);


		// ship selections
		sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		if ((sl.size() > 0) && (DEF_sl.size() > 0)) {

			// 3 ships.
			for (int32 i = 0; i < 3; i++) {
				Ship s1;
				int32 thisShipBPV = nPCBPV;

				switch(i) {
				case 2:
					thisShipBPV *= 0.60;
					break;
				case 1:
					thisShipBPV *= 0.80;
					break;
				default:
					break;
				}

				ShipList ship_sl = sl;
                fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.925), int(thisShipBPV * 1.075), true);
                gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(thisShipBPV * 0.925), int(thisShipBPV * 1.075), fShiplistCount);
                gST.listShiplistContent(ship_sl);

				if (ship_sl.size() == 0) {
					// too much BPV in player fleet for this to work. Let's redo this differently.
					gST.mDebug_fprintf("FAIL! Cannot select a ship with %d BPV. Trying to just pick a big ship instead.\n", thisShipBPV);
					ship_sl = sl;

					std::list< eEnhancedClassTypes >  bigShipClassTypeList;
					bigShipClassTypeList.push_back(kEClassLargeMonster);
					for (int32 i = kEClassDreadnought; i <= kEClassBattleship; i++) {
						bigShipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
					}
					fShiplistCount = ship_sl.Subset(bigShipClassTypeList, true);
					gST.mDebug_fprintf("subset for big ship classes filter completed. ships: %d\n", fShiplistCount);
					gST.listShiplistContent(ship_sl);
				}

				if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
					gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
					gST.mTrackOrions( s1 );

					mCreateShip(
						typeid( tISCShip ),
						s1.shipCode.c_str(),
						kStartPosition_X - i,
						kNoMetaShipID,
						kISCShip1 + i,
						0, 0, -1, -1,
						NULL,
						kDefaultShipOptions );
				} else {
					gST.mDebug_fprintf("FAIL AGAIN. Using old method\n");
					mCreateShip( typeid( tISCShip ),
						thisShipBPV,
						kClassFrigate,
						kClassDreadnought,
						0,
						kStartPosition_X - i,
						kISCShip1 + i,
						0, 0, -1, -1, 
						NULL,
						kDefaultShipOptions );
				}
			}

			// 2 Defense platforms
			for (int32 j = 0; j < 2; j++) {
				Ship def;

				DEF_sl.SelectRandomShip(def);
				gST.mDebug_fprintf("def %s selected. BPV: %d  YFA: %d  YLA %d\n",
					def.shipCode.c_str(), def.bpv, def.yearFirstAvail, def.yearLastAvail);
				gST.mTrackOrions( def );

				mCreateShip( typeid( tISCDefensePlatform ), 
					def.shipCode.c_str(), 
					kStartPosition_Z - j, 
					kNoMetaShipID,
					kISCDefensePlatform1 + j, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
			}

        } else {
            mOldCreateShipsForTeam();
        }
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	switch ( diffLevel )
	{
	case 0:
		nPCBPV = playerBPV * 0.75;	break;
	case 1:
		nPCBPV = playerBPV * 1.0;	break;
	case 2:
		nPCBPV = playerBPV * 1.25;	break;
	}

	// ships
	mCreateShip( typeid( tISCShip ),
		nPCBPV,
		kClassFrigate,
		kClassWarDestroyer,
		0,
		kStartPosition_X,
		kISCShip1,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );
	mCreateShip( typeid( tISCShip ),
		( nPCBPV * 0.8),
		kClassFrigate,
		kClassWarDestroyer,
		0,
		kStartPosition_W,
		kISCShip2,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );
	mCreateShip(
		typeid( tISCShip ),
		( nPCBPV * 0.6),
		kClassFrigate,
		kClassWarDestroyer,
		0,
		kStartPosition_V,
		kISCShip3,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );
/*
	mCreateShip( typeid( tISCShip ),
		nPCBPV,
		kClassFrigate,
		kClassWarDestroyer,
		0,
		kStartPosition_U,
		kISCShip4,
		0, 0, -1, -1, NULL,
		kDefaultShipOptions );
*/

	// defense platforms
	mCreateShip( typeid( tISCDefensePlatform ), 
		"I-DEF", 
		kStartPosition_Z, 
		kNoMetaShipID,
		kISCDefensePlatform1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );
	mCreateShip( typeid( tISCDefensePlatform ),
		"I-DEF", 
		kStartPosition_Y, 
		kNoMetaShipID,
		kISCDefensePlatform2, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}