/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "Uni_5Accuse.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if ( victoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDisconnect_msg );
		return;
	}

	fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( playerRace )
	{
	case kFederation:
	case kRomulan:
	case kHydran:
		{
			switch ( victoryLevel )
			{
			case kAstoundingVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictoryFederationRomulanHydran_msg );	break;
			case kVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebVictoryFederationRomulanHydran_msg );			break;
			case kDraw:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );										break;
			case kDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeatFederationRomulanHydran_msg );				break;
			case kDevastatingDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeatFederationRomulanHydran_msg );	break;
			}
		}
		break;

	case kKlingon:
	case kGorn:
	case kLyran:
	case kMirak:
		{
			switch ( victoryLevel )
			{
			case kAstoundingVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictoryKlingonGornLyranMirak_msg );	break;
			case kVictory:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebVictoryKlingonGornLyranMirak_msg );				break;
			case kDraw:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );										break;
			case kDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeatKlingonGornLyranMirak_msg );				break;
			case kDevastatingDefeat:
				fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeatKlingonGornLyranMirak_msg );	break;
			}
		}
		break;
	}
}


void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	eRaceName playerRace = mGetRace();

	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;

	switch( playerRace )
	{
		case kLyran:
				fMissionScheduler.fNextMissionTitle = "Uni_X2M1Interceptor";
		break;
		default:
				fMissionScheduler.fNextMissionTitle = "Uni_6Immunity";
		break;
	}	
}
