/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "Uni_5BFunny2.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "LyranTeam.h"
#include "KlingonTeam.h"
#include "HydranTeam.h"
#include "MirakTeam.h"
#include "PlanetTeam.h"

tPlayerShipBaseState::tPlayerShipBaseState()
{
	fScanning = false;
}


void tPlayerShipBaseState::mBeginState()
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
		return;

	switch( playerTeam->mGetRace() )
	{
	case kLyran:
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLog_Lyran_msg, kNonCommScriptMessage );
		break;

	case kKlingon:
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLog_Klink_msg, kNonCommScriptMessage );
		break;

	case kHydran:
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLog_Hydran_msg, kNonCommScriptMessage );
		break;
	}

	mSetTimedInterval( kTimerTurnsSet, kInitalMsg_Timer, 0.3f );
}

void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fScanning = true;
		mSetTimedInterval( kTimerTurnsSet, kScanTimer, 0.75f );
	}

	else
	{
		fScanning =false;
	}
}

void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	tTeamInfo* klingonTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kKlingonTeam ) );
	if( !klingonTeam )
		return;

	tTeamInfo* hydranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kHydranTeam ) );
	if( !hydranTeam )
		return;

	tTeamInfo* mirakTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam ) );
	if( !mirakTeam )
		return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
		return;

	tShipInfo* lMoon = lyranTeam->mGetThisShip(1);
	tShipInfo* kMoon = klingonTeam->mGetThisShip(1);
	tShipInfo* hMoon = hydranTeam->mGetThisShip(1);
	tShipInfo* mMoon = mirakTeam->mGetThisShip(1);
	tShipInfo* planet = planetTeam->mGetThisShip(1);
	
	switch( timerID )
	{
	case kInitalMsg_Timer:
		{
			switch( playerTeam->mGetRace() )
			{
			case kLyran:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInitialMsg_Lyran );
				break;

			case kKlingon:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInitialMsg_Klink );
				break;

			case kHydran:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInitialMsg_Hydran );
				break;
			}
		}
		break;

	case kScanTimer:
		{
			if( !fScanning )
				return;

			int32 ScanTargetID = fChildActor->mGetShipTargetID();

			// Lyran moon
			if( (lMoon) && (ScanTargetID == lMoon->mGetShipID()) && (!playerTeam->mScannedMoon(1)) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedMoon_msg );
				playerTeam->mSetScannedMoon(1);
				return;
			}
			
			// Klingon moon
			if( (kMoon) && (ScanTargetID == kMoon->mGetShipID()) && (!playerTeam->mScannedMoon(2)) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedMoon_msg );
				playerTeam->mSetScannedMoon(2);
				return;
			}
			
			//Hydran moon
			if( (hMoon) && (ScanTargetID == hMoon->mGetShipID()) && (!playerTeam->mScannedMoon(3)) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedMoon_msg );
				playerTeam->mSetScannedMoon(3);
				return;
			}
			
			// Mirak moon
			if( (mMoon) && (ScanTargetID == mMoon->mGetShipID()) && (!playerTeam->mScannedMoon(4)) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedMoon_msg );
				playerTeam->mSetScannedMoon(4);
				return;
			}
			
			//Planet
			if( (planet) && (ScanTargetID == planet->mGetShipID()) && (!playerTeam->mScannedPlanet()) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kScannedPlanet_msg );
				playerTeam->mSetScannedPlanet();
				return;
			}

			if( (!playerTeam->mScannedMoon(1)) || (!playerTeam->mScannedMoon(2)) || (!playerTeam->mScannedMoon(3)) || (!playerTeam->mScannedMoon(4)) || (!playerTeam->mScannedPlanet()) )
			{
				mSetTimedInterval( kTimerTurnsSet, kScanTimer, 0.75f );
			}
		}
		break;
	}
}

void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	fChildActor->mSetShipCallbackEvents( kNotifyTransportedItem );


	if( item != kTransDiplomat )
		return;

	if( !shipTo )
		return;

	if( !shipFrom )
		return;
	
	if( shipFrom->mGetTeam() != kPlayerTeam )
		return;


	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* lyranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kLyranTeam ) );
	if( !lyranTeam )
		return;

	tTeamInfo* klingonTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kKlingonTeam ) );
	if( !klingonTeam )
		return;

	tTeamInfo* hydranTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kHydranTeam ) );
	if( !hydranTeam )
		return;

	tTeamInfo* mirakTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam ) );
	if( !mirakTeam )
		return;

	tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
	if( !planetTeam )
		return;

	tShipInfo* lMoon = lyranTeam->mGetThisShip(1);
	tShipInfo* kMoon = klingonTeam->mGetThisShip(1);
	tShipInfo* hMoon = hydranTeam->mGetThisShip(1);
	tShipInfo* mMoon = mirakTeam->mGetThisShip(1);
	tShipInfo* planet = planetTeam->mGetThisShip(1);

	int32 TransTargetID = shipTo->mGetShipID();

	fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDiplomat_Delivered_msg );

	// Find beam target

	// Lyran moon
	if( (lMoon) && (TransTargetID == lMoon->mGetShipID()) )
	{
		playerTeam->mSetDiplomatDelivered( tPlayerTeam::kDestLyran );
		if( playerTeam->mGetRace() == kLyran )
		{
			playerTeam->mBeamedToOwnMoon();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBeamedToOwnMoon_msg );
			gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerLoses );
		}
		else
		{
			gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
		}
		return;
	}

	// Klingon moon
	if( (kMoon) && (TransTargetID == kMoon->mGetShipID()) )
	{
		playerTeam->mSetDiplomatDelivered( tPlayerTeam::kDestKlingon );
		if( playerTeam->mGetRace() == kKlingon )
		{
			playerTeam->mBeamedToOwnMoon();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBeamedToOwnMoon_msg );
			gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerLoses );
		}
			else
		{
			gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
		}
		return;
	}

	//Hydran moon
	if( (hMoon) && (TransTargetID == hMoon->mGetShipID()) )
	{
		playerTeam->mSetDiplomatDelivered(tPlayerTeam::kDestHydran );
		if( playerTeam->mGetRace() == kHydran )
		{
			playerTeam->mBeamedToOwnMoon();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBeamedToOwnMoon_msg );
			gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerLoses );
		}
			else
		{
			gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
		}
		return;
	}

	// Mirak moon
	if( (mMoon) && (TransTargetID == mMoon->mGetShipID()) )
	{
		playerTeam->mSetDiplomatDelivered( tPlayerTeam::kDestMirak );
		gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerWins );
	}

	//Planet
	if( (planet) && (TransTargetID == planet->mGetShipID()) )
	{
		playerTeam->mSetDiplomatDelivered( tPlayerTeam::kDestPlanet );
		gScriptInterface.mSetGameStatus(static_cast<eTeamID>(kPlayerTeam), kPlayerLoses );
	}
}



void tPlayerShipBaseState::mEndState()
{}

void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}

void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}

void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

