/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "Uni_5BFunny2.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fDiplomatDelivered = false;
	fDiplomatDelivered_Lyran = false;
	fDiplomatDelivered_Klingon = false;
	fDiplomatDelivered_Hydran = false;
	fDiplomatDelivered_Mirak = false;
	fDiplomatDestination = -1;
	fScannedMoon1 = false;
	fScannedMoon2 = false;
	fScannedMoon3 = false;
	fScannedMoon4 = false;
	fScannedPlanet = false;;
	fAttackedPlanet = false;
	fAttackedMoon = false;
	fBeamedToOwnMoon = false;
	fDied = false;
}

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( fDied )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_PlayerDied_msg );
		mScheduleNextMission( fVictoryCondition );
		return;
	}

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictory_msg );	break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeat_msg );	break;
	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDebLeftEarly_msg );
			mScheduleNextMission( fVictoryCondition );
			return;
		}
		break;
	}

	// Not beamed at all
	if( !mDiplomatDelivered() )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kD_NotBeamed_msg );
	}

	// Beamed, but to own moon (duh)
	if( ( mDiplomatDelivered() ) && ( mBeamedToOwnMoon() ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam,  kD_BeamedToOwn_msg );
	}

	// Beamed to the planet
	if( (mDiplomatDelivered() ) && ( mGetDiplomatDestination() == kDestPlanet ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam,  kD_BeamedToPlanet_msg );
	}

	// Beamed to somewhere other than your own moon or the planet
	if( (mDiplomatDelivered() ) && ( !mBeamedToOwnMoon() ) && ( mGetDiplomatDestination() != kDestPlanet ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam,  KD_Beamed_Success_msg );
	}

	// Lyran beams to Mirak planet
	if( (mDiplomatDelivered() ) && ( !mBeamedToOwnMoon() ) && ( mGetDiplomatDestination() == kDestMirak ) && ( mGetRace() == kLyran ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam,  KD_Lyran_Beamed_To_Mirak_msg );
	}

	mScheduleNextMission( fVictoryCondition );
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{
	
}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}



iTruth tPlayerTeam::mDiplomatDelivered()
{
	return fDiplomatDelivered;
}

int32 tPlayerTeam::mGetDiplomatDestination()
{
	return fDiplomatDestination;
}

void tPlayerTeam::mSetDiplomatDelivered(int32 Destination)
{
	fDiplomatDelivered = true;
	fDiplomatDestination = Destination;
}

iTruth tPlayerTeam::mScannedMoon(int32 x)
{
	switch( x )
	{
	case 1:
		return fScannedMoon1;
		break;

	case 2:	
		return fScannedMoon2;
		break;

	case 3:
		return fScannedMoon3;
		break;

	case 4:
		return fScannedMoon4;
		break;

	default:
		return true;
	}
}

iTruth tPlayerTeam::mScannedPlanet()
{
	return	fScannedPlanet;
}

iTruth tPlayerTeam::mAttackedMoon()
{
	return fAttackedMoon;
}

iTruth tPlayerTeam::mAttackedPlanet()
{
	return fAttackedPlanet;
}

void tPlayerTeam::mSetScannedMoon(int32 x)
{
	switch( x )
	{
	case 1:
		fScannedMoon1 = true;
		break;

	case 2:	
		fScannedMoon2 = true;
		break;

	case 3:
		fScannedMoon3 = true;
		break;

	case 4:
		fScannedMoon4 = true;
		break;
	}
}

void tPlayerTeam::mSetScannedPlanet()
{
	fScannedPlanet = true;
}


void tPlayerTeam::mSetAttackedMoon()
{
	fAttackedMoon = true;
}

void tPlayerTeam::mSetAttackedPlanet()
{
	fAttackedPlanet = true;
}

iTruth tPlayerTeam::mBeamedToOwnMoon()
{
	return fBeamedToOwnMoon;
}

void tPlayerTeam::mSetBeamedToOwnMoon()
{
	fBeamedToOwnMoon = true;
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 400;
	fMissionScheduler.fNextMissionTitle = "Uni_6Wheel";

	if( fDied )
	{
		fMissionScheduler.fNextMissionTitle = "";
		fMissionScheduler.fCampaignEvent = tMissionScheduler::kLost;
		return;
	}
}