/********************************************************************************************************\

	Mission:	"What's So Funny, Part II"
	Script:		"Uni_5BFunny2"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno
	Last Updated:	FS				10/18/2012
					era-correct ships, drones and fighters.. well.. ok. Fine. Monsters.

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerTeam.h"
#include "LyranTeam.h"
#include "KlingonTeam.h"
#include "HydranTeam.h"
#include "MirakTeam.h"
#include "PlanetTeam.h"
#include "Uni_5BFunny2.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tUni_5BFunny2 );

tUni_5BFunny2::tUni_5BFunny2()
{}

int32 tUni_5BFunny2::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_5BFunny2::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag, 
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Lyran Team",
												static_cast< eTeamID >( kLyranTeam ),
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMaxChance,
												kSpecLyran,
												kAnyRace,
												typeid( tLyranTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Klingon Team",
												static_cast< eTeamID >( kKlingonTeam ),
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMaxChance,
												kSpecKling,
												kAnyRace,
												typeid( tKlingonTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Hydran Team",
												static_cast< eTeamID >( kHydranTeam ),
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMaxChance,
												kSpecHydran,
												kAnyRace,
												typeid( tHydranTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Mirak Team",
												static_cast< eTeamID >( kMirakTeam ),
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tMirakTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak | kSpecISC,
												kAnyRace,
												typeid( tPlanetTeam ) );




	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

int32 tUni_5BFunny2::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_5BFunny2::mCreateExternalShips( int32 Mode )
{}

int32 tUni_5BFunny2::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_5BFunny2::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_5BFunny2::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_5BFunny2::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_5BFunny2::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kLyranTeam,
		kKlingonTeam,
		kHydranTeam,
		kMirakTeam,
		kPlanetTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Lyr  = kLyranTeam,
		//Kli  = kKlingonTeam,
		//Hyd  = kHydranTeam,
		//Mir  = kMirakTeam,
		//Pla  = kPlanetTeam

		//	     Plyr  Lyr    Kli   Hyd	 Mir   Pla
		/*Plyr*/ "--", "NI", "NI", "NI", "NI", "AI",
		/*Lyr*/  "AI", "--", "NI", "NI", "NI", "AI",
		/*Kli */ "AI", "NI", "--", "NI", "NI", "AI",
		/*Hyd*/  "AI", "NI", "NI", "--", "NI", "AI",
		/*Mir*/	 "AI", "NI", "NI", "NI", "--", "AI",
		/*Pla*/	 "AI", "NI", "NI", "NI", "NI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


void tUni_5BFunny2::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlanet;
	Map->fTacticalMapIcon['I'] = kObjectPlanet;
	Map->fTacticalMapIcon['J'] = kObjectPlanet;
	Map->fTacticalMapIcon['K'] = kObjectPlanet;
	Map->fTacticalMapIcon['L'] = kObjectPlanet;
}

tMapTerrain tUni_5BFunny2::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_5BFunny2::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tUni_5BFunny2::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_msg;
}