/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "Cruiser.h"
#include "DamagedShip1.h"
#include "EnemyTeam.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "PlayerNPCTeam.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "Starbase.h"
#include "Uni_6Immunity.h"
#include "ShipList_API.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	if ( fChildActor->mVictimsExtracted() )
	{
		if ( fChildActor->mBoom() )
		{
			return kAstoundingVictory;
		}
		else
		{
			return kVictory;
		}
	}
	else
	{
		if ( fChildActor->mISCKilled() < 4 )
		{
			return kDevastatingDefeat;
		}
		else
		{
			return kDefeat;
		}
	}
	return kDraw;
}

//
//	Ship events
//
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	starbaseTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNPC1Team ) );
	Assert( starbaseTeam );
	if ( !starbaseTeam )
		return;
	tShipInfo*	starbase = starbaseTeam->mFindShipCustomID( tNPC1Team::kStarbase );

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kDamagedShip1:
	case tPlayerNPCTeam::kDamagedShip2:
	case tPlayerNPCTeam::kDamagedShip3:
		{
			//
			//	If the hostages have not been rescued and the rescue team was onboard, then
			//	the mission was a failure.
			//
			if ( starbase )
			{
				if	(	( !( static_cast< tStarbase* >( starbase ) )->mHasHostages() ) &&
						( ( static_cast< tDamagedShip1* >( ship ) )->mHasRescueTeam() )	)
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom5_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
				}
			}
		}
		break;

	case tEnemyTeam::kCruiser1:
	case tEnemyTeam::kCruiser2:
	case tEnemyTeam::kCruiser3:
		{
			fChildActor->mIncreaseISCKilled();
		}
		break;

	case tNPC1Team::kStarbase:
		{
			fChildActor->mSetBoom( true );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent16_1_msg );
			
			//
			//	If the hostages have not been rescued ,then	the mission was a failure.
			//
			if ( ( static_cast< tDamagedShip1* >( ship ) )->mHasHostages() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom7_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
			else
			{
				mAddToBonusPrestige( 300 );
				if ( fChildActor->mISCBoarded() )
				{
					mAddToBonusPrestige( 300 );
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
				}
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent21_1_msg );
			if ( ( static_cast< tPlayerShip* >( ship ) )->mHasHostages() )
			{
				fChildActor->mSetVictimsExtracted( true );
			}

			if ( fChildActor->mVictimsExtracted() )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;

	static bool	sISCBoarded = false;

	switch ( ship->mGetCustomID() )
	{
	case tEnemyTeam::kCruiser1:
	case tEnemyTeam::kCruiser2:
	case tEnemyTeam::kCruiser3:
	case tEnemyTeam::kCruiser4:
		{
			if ( !sISCBoarded )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent15_1_msg );
				sISCBoarded = true;
			}
			fChildActor->mSetISCBoarded( true );
		}
		break;
	}
}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if ( ( !shipTo ) || ( !shipFrom ) )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	tShipIterator	playerShip = playerTeam->mGetFirstShip();
	tShipInPlayEval	x;
	playerShip.mAddEvaluator( &x );

	//
	//	If the player is placing something on a ship.
	//
	switch ( shipFrom->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			switch ( shipTo->mGetCustomID() )
			{
			case tPlayerNPCTeam::kDamagedShip1:
			case tPlayerNPCTeam::kDamagedShip2:
			case tPlayerNPCTeam::kDamagedShip3:
				{}
				break;

			case tEnemyTeam::kCruiser1:
			case tEnemyTeam::kCruiser2:
			case tEnemyTeam::kCruiser3:
				{}
				break;
			
			case tNPC1Team::kStarbase:
				{}
				break;
			}
		}
		break;
	}
	//
	//	If the player is removing something from a ship.
	//
	switch ( shipTo->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			switch ( shipTo->mGetCustomID() )
			{
			case tPlayerNPCTeam::kDamagedShip1:
			case tPlayerNPCTeam::kDamagedShip2:
			case tPlayerNPCTeam::kDamagedShip3:
				{}
				break;

			case tEnemyTeam::kCruiser1:
			case tEnemyTeam::kCruiser2:
			case tEnemyTeam::kCruiser3:
				{}
				break;
			
			case tNPC1Team::kStarbase:
				{
					//
					//	The hostages are being beamed onboard the player ship.
					//
					;	//	do nothing
				}
				break;
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );	break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );	break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );	break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );	break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );	break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );	break;
		}
//		fChildActor->mSetTimedInterval(	kTimerSecondsSet, timerID,
//										( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif

	if ( timerID == tBaseVictory::kReinforcementsTimerID)
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent2_1_msg );

		mCreateEnemyShips();
	}
}

void tBaseVictoryState::mCreateEnemyShips()
{
    gST.mDebug_fprintf("--- Entering: tBaseVictoryState::mCreateEnemyShips\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
		tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
		Assert( enemyTeam && playerTeam );
		if ( ( !enemyTeam ) || ( !playerTeam ) )
			return;

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 nPCBPV; 
		switch ( fMissionInfo->mGetDifficultyLevel() )
		{
		case 0:
			nPCBPV = playerBPV / playerTeam->mGetShipsInPlayCount() * 0.95;	break;
		case 1:
			nPCBPV = playerBPV / playerTeam->mGetShipsInPlayCount() * 1.2;	break;
		case 2:
			nPCBPV = playerBPV / playerTeam->mGetShipsInPlayCount() * 1.45;	break;
		}


        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, enemyTeam->mGetRace() );

        // ship selections
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
		shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        ShipList ship_sl = sl;
		// It's actually BPV UP TO PlayerTeamBPV * difficulty modifier.  Making it a range.
        fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(nPCBPV * 0.625), int(nPCBPV * 1.075), true);
        gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(nPCBPV * 0.625), int(nPCBPV * 1.075), fShiplistCount);
        gST.listShiplistContent(ship_sl);

		if (ship_sl.size() > 0) {
			Ship s1;
			ship_sl.SelectRandomShip(s1);
			gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			Ship s2;
			ship_sl.SelectRandomShip(s2);
			gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

			Ship s3;
			ship_sl.SelectRandomShip(s3);
			gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

			Ship s4;
			ship_sl.SelectRandomShip(s4);
			gST.mDebug_fprintf("s4 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s4.shipCode.c_str(), s4.bpv, s4.yearFirstAvail, s4.yearLastAvail);
            gST.mTrackOrions( s4 );

			enemyTeam->mCreateShip(	typeid( tCruiser ),
									s1.shipCode.c_str(),
									kStartPosition_Z,
									tEnemyTeam::kNoMetaShipID,
									tEnemyTeam::kCruiser1,
									0, 0, -1, -1, 
									NULL,
									kDefaultShipOptions );
			enemyTeam->mCreateShip(	typeid( tCruiser ),
									s2.shipCode.c_str(),
									kStartPosition_Y,
									tEnemyTeam::kNoMetaShipID,
									tEnemyTeam::kCruiser2,
									0, 0, -1, -1, 
									NULL,
									kDefaultShipOptions );
			enemyTeam->mCreateShip(	typeid( tCruiser ),
									s3.shipCode.c_str(),
									kStartPosition_X,
									tEnemyTeam::kNoMetaShipID,
									tEnemyTeam::kCruiser3,
									0, 0, -1, -1, 
									NULL,
									kDefaultShipOptions );
			enemyTeam->mCreateShip(	typeid( tCruiser ),
									s4.shipCode.c_str(),
									kStartPosition_W,
									tEnemyTeam::kNoMetaShipID,
									tEnemyTeam::kCruiser4,
									0, 0, -1, -1, 
									NULL,
									kDefaultShipOptions );

		} else {
			mOldCreateEnemyShips();
		}

	} else {
		mOldCreateEnemyShips();
	}

	gST.mDebug_fprintf("--- Finished: tBaseVictoryState::mCreateEnemyShips\n");
}

void tBaseVictoryState::mOldCreateEnemyShips()
{
	gST.mDebug_fprintf("---- Starting: tBaseVictoryState::mOldCreateEnemyShips\n");

	tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( enemyTeam && playerTeam );
	if ( ( !enemyTeam ) || ( !playerTeam ) )
		return;

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	int32 nPCBPV; 
		switch ( fMissionInfo->mGetDifficultyLevel() )
		{
		case 0:
			nPCBPV = playerBPV / playerTeam->mGetShipsInPlayCount() * 0.95;	break;
		case 1:
			nPCBPV = playerBPV / playerTeam->mGetShipsInPlayCount() * 1.2;	break;
		case 2:
			nPCBPV = playerBPV / playerTeam->mGetShipsInPlayCount() * 1.45;	break;
		}

	//
	//	Create the 4 ISC ships that come in after the 4 minutes into the mission.
	//
	enemyTeam->mCreateShip(	typeid( tCruiser ),
							nPCBPV,
							kClassFrigate,
							kClassHeavyBattlecruiser,
							0,
							kStartPosition_Z,
							tEnemyTeam::kCruiser1,
							0, 0, -1, -1, NULL,
							kDefaultShipOptions );
	enemyTeam->mCreateShip(	typeid( tCruiser ),
							nPCBPV,
							kClassFrigate,
							kClassHeavyBattlecruiser,
							0,
							kStartPosition_Y,
							tEnemyTeam::kCruiser2,
							0, 0, -1, -1, NULL,
							kDefaultShipOptions );
	enemyTeam->mCreateShip(	typeid( tCruiser ),
							nPCBPV,
							kClassFrigate,
							kClassHeavyBattlecruiser,
							0,
							kStartPosition_X,
							tEnemyTeam::kCruiser3,
							0, 0, -1, -1, NULL,
							kDefaultShipOptions );
	enemyTeam->mCreateShip(	typeid( tCruiser ),
							nPCBPV,
							kClassFrigate,
							kClassHeavyBattlecruiser,
							0,
							kStartPosition_W,
							tEnemyTeam::kCruiser4,
							0, 0, -1, -1, NULL,
							kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tBaseVictoryState::mOldCreateEnemyShips\n");
}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

