/********************************************************************************************************\

	File Name:	CruiserBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CruiserBaseState.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"
#include "Uni_6Immunity.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tCruiserBaseState::mBeginState()
{
    gST.mDebug_fprintf("--- Starting: tCruiserBaseState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tCruiserBaseState::mBeginState\n");
}
void tCruiserBaseState::mEndState()
{}

void tCruiserBaseState::mOnDeath( tShipInfo* ship )
{}
void tCruiserBaseState::mOnCapture( tShipInfo* ship )
{}
void tCruiserBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tCruiserBaseState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;

	if ( fChildActor->mGetCustomID() == ship->mGetCustomID() )
	{
		//
		//	Shut down the ISC ship.
		//
		fChildActor->mSetSystemHealthFromOrig( kRandomInternalsHardPoint, 0.0f );
		fChildActor->mSetShipMaxSpeed( 0 );
		
		//
		//	Since the ISC has boarded the starbase, the starbase should turn against the player.
		//
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kNPC1Team ), static_cast< eTeamID >( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kNPC1Team ), static_cast< eTeamID >( kPlayerNPCTeam ), kTeamAtWar, kGoalDestroy, 1.0f );

		//
		//	The ISC ships should also start attacking the player ships.
		//
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
		fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kPlayerNPCTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
	}
}
void tCruiserBaseState::mAtDestination( tShipInfo* ship )
{}
void tCruiserBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tCruiserBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tCruiserBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tCruiserBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tCruiserBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tCruiserBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tCruiserBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tCruiserBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCruiserBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tCruiserBaseState::mOnFlashingButtonPressed()
{}
void tCruiserBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if ( ( !shipTo ) || ( !shipFrom ) )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	tShipIterator	playerShip = playerTeam->mGetFirstShip();
	tShipInPlayEval	x;
	playerShip.mAddEvaluator( &x );

	//
	//	If the player is placing something on a ship.
	//
	switch ( shipFrom->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			switch ( shipTo->mGetTeam() )
			{
			case kEnemyTeam:
				{
					switch ( item )
					{
					case kTransEngineers:
						{
							( static_cast< tCruiser* >( shipTo ) )->mSetHasEngineers( true );
							( static_cast< tPlayerShip* >( shipFrom ) )->mSetHasEngineers( false );
						}
						break;

					case kTransMarineDude:
						{
							( static_cast< tCruiser* >( shipTo ) )->mSetHasRescueTeam( true );
							( static_cast< tPlayerShip* >( shipFrom ) )->mSetHasRescueTeam( false );
						}
						break;
					}
				}
				break;
			}
		}
		break;
	}
	//
	//	If the player is removing something from a ship.
	//
	switch ( shipTo->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			switch ( shipFrom->mGetTeam() )
			{
			case kEnemyTeam:
				{
					switch ( item )
					{
					case kTransEngineers:
						{
							( static_cast< tPlayerShip* >( shipTo ) )->mSetHasEngineers( true );
							( static_cast< tCruiser* >( shipFrom ) )->mSetHasEngineers( false );
						}
						break;

					case kTransMarineDude:
						{
							( static_cast< tPlayerShip* >( shipTo ) )->mSetHasRescueTeam( true );
							( static_cast< tCruiser* >( shipFrom ) )->mSetHasRescueTeam( false );
						}
						break;
					}
				}
				break;
			}
		}
		break;
	}
}
void tCruiserBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tCruiserBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tCruiserBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tCruiserBaseState::mOnHostChanged()
{}
void tCruiserBaseState::mOnTimedInterval( int32 timerID )
{
	if (timerID == kUpgradeFightersID) {
#ifdef FIGHTERS
		if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
		tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
		if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
		if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC. 

		gST.mDebug_fprintf("--- Entering tCruiserBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
		int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
		gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
		gST.mDebug_fprintf("--- Finished: tCruiserBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
	}
}
void tCruiserBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tCruiserBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


