//	<Name>	DamagedShip3.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "DamagedShip3.h"
#include "DamagedShip3BaseState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "Uni_6Immunity.h"

tClassAllocator<tDamagedShip3> gDamagedShip31Allocator;

tDamagedShip3::tDamagedShip3(): fCanRepair( false ), fNumScrews( 0 ),
								fHasHostages( false ), fHasRescueTeam( false ), fHasEngineers( false )
{}
void tDamagedShip3::mSetupState()
{
	mRegisterState( new tDamagedShip3BaseState() );
	mGotoState( typeid( tDamagedShip3BaseState ) );
}
void tDamagedShip3::mSetupCommState()
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	playerTeam->mRegisterCommMessage(	kCommState1, kNoStateChange, mGetShipID(),
										k_HAIL_btn, k_HAIL_msg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState1, kNoStateChange, mGetShipID(),
										k_DROPSHIELDS_Ship3_btn, kNoMsg, kNoRandomMsg );

	playerTeam->mGotoCommState( mGetShipID(), kCommState1 );
}
void tDamagedShip3::mSetupShip()
{
	//
	//	Take away its repair supplies and damage the starbase.
	//
	mSetNumScrews( mDamageShip() );

	mSetShipCallbackEvents( kNotifyTransportedItem );
}

/*

	Purpose:	Will damage said ship considerably.  Will also not allow the ship to move.  Also, the
				ship's repair screws will be eliminated, but counted first and returned.

*/
int32 tDamagedShip3::mDamageShip()
{
	int32	numScrews = 25;

	//
	//	Have the ship just sit there.
	//
	mSetShipMaxSpeed( 0 );

	//
	//	Damage the internals and shields of the ship.
	//
	mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f );
	mSetSystemHealthFromOrig( kRandomInternalsHardPoint, 0.0f );

	//
	//	Get the number of screws on the ship, then get rid of all the screws.
	//
	mRemoveItemFromStores( kTransSpareParts, numScrews );

	return numScrews;
}
/*

	Purpose:	Will bring a ship back up to 50%.

*/
void tDamagedShip3::mRepairShip()
{
	//
	//	Set the ship's max speed.
	//
	mSetShipMaxSpeed( fMissionInfo->mRandomInt32( 7, 15 ) );

	//
	//	Repair the engines, weapons, and shields to 100%.
	//
	mSetSystemHealthFromOrig( kAllShieldsHardPoint, 1.0f, kTrue );
	mSetSystemHealthFromOrig( kRandomInternalsHardPoint, 1.0f, kTrue );
}

bool tDamagedShip3::mCanRepair( void )
{
	return fCanRepair;
}
int32 tDamagedShip3::mNumScrews( void )
{
	return fNumScrews;
}
bool tDamagedShip3::mHasHostages( void )
{
	return fHasHostages;
}
bool tDamagedShip3::mHasRescueTeam( void )
{
	return fHasRescueTeam;
}
bool tDamagedShip3::mHasEngineers( void )
{
	return fHasEngineers;
}
void tDamagedShip3::mSetCanRepair( bool x )
{
	fCanRepair = x;
}
void tDamagedShip3::mSetNumScrews( int32 x )
{
	fNumScrews = x;
}
void tDamagedShip3::mSetHasHostages( bool x )
{
	fHasHostages = x;
}
void tDamagedShip3::mSetHasRescueTeam( bool x )
{
	fHasRescueTeam = x;
}
void tDamagedShip3::mSetHasEngineers( bool x )
{
	fHasEngineers = x;
}