/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "DamagedShip1.h"
#include "DamagedShip2.h"
#include "DamagedShip3.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}

void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		// we're going to randomly pick 3 ships from the 7 non-ISC races --FS
		// sizes: large qship/aux to BCH.

		int32 fShiplistCount;
        ShipList sl = gST.stripOutSPECIAL( gST.narrowed_sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
		shipClassTypeList.push_back(kEClassSmallAuxiliary);
		shipClassTypeList.push_back(kEClassLargeAuxiliary);
        for (int32 i = kEClassFrigate; i <= kEClassHeavyBattlecruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);


		ShipList ship1_sl = gST.mFilterShipListToSingleRace( sl, mChooseRace() );
		ShipList ship2_sl = gST.mFilterShipListToSingleRace( sl, mChooseRace() );
		ShipList ship3_sl = gST.mFilterShipListToSingleRace( sl, mChooseRace() );

		if (
			(ship1_sl.size() > 0) &&
			(ship2_sl.size() > 0) &&
			(ship3_sl.size() > 0)
		) {
			Ship s1;
			ship1_sl.SelectRandomShip(s1);
			gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			Ship s2;
			ship2_sl.SelectRandomShip(s2);
			gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

			Ship s3;
			ship3_sl.SelectRandomShip(s3);
			gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

			mCreateShip(
				typeid( tDamagedShip1 ), 
				s1.shipCode.c_str(), 
				kStartPosition_I, 
				kNoMetaShipID,
				kDamagedShip1, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );

			mCreateShip(
				typeid( tDamagedShip2), 
				s2.shipCode.c_str(), 
				kStartPosition_J,
				kNoMetaShipID,
				kDamagedShip2,
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );

			mCreateShip(
				typeid( tDamagedShip3), 
				s3.shipCode.c_str(), 
				kStartPosition_K,
				kNoMetaShipID,
				kDamagedShip3,
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}

eRaceName tPlayerNPCTeam::mChooseRace()
{
	eRaceName picked_race;
	switch ( fMissionInfo->mRandomInt32( 0 , 6 ) ) {
		case 0: picked_race = kFederation; break;
		case 1: picked_race = kKlingon; break;
		case 2: picked_race = kRomulan; break;
		case 3: picked_race = kLyran; break;
		case 4: picked_race = kGorn; break;
		case 5: picked_race = kHydran; break;
		case 6: picked_race = kMirak; break;
	}
	return picked_race;
}

void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tPlayerNPCTeam::mOldCreateShipsForTeam\n");

	//
	//	First alliance ship
	//
	switch ( fMissionInfo->mRandomInt32( 0 , 6 ) )
	{
	case 0: //	Federation
		mCreateShip(	typeid( tDamagedShip1 ), "F-CB" , kStartPosition_I, kNoMetaShipID,
						kDamagedShip1, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		mCreateShip( typeid( tDamagedShip1 ), "K-FQL" , kStartPosition_I, kNoMetaShipID,
						kDamagedShip1, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		mCreateShip( typeid( tDamagedShip1 ), "R-KRC" , kStartPosition_I, kNoMetaShipID,
						kDamagedShip1, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		mCreateShip( typeid( tDamagedShip1 ), "L-CL" , kStartPosition_I, kNoMetaShipID,
						kDamagedShip1, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		mCreateShip( typeid( tDamagedShip1 ), "G-CM" , kStartPosition_I, kNoMetaShipID,
						kDamagedShip1, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;;
	case 5:	//	Hydran
		mCreateShip( typeid( tDamagedShip1 ), "H-DWF" , kStartPosition_I, kNoMetaShipID,
						kDamagedShip1, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		mCreateShip( typeid( tDamagedShip1 ), "Z-DD" , kStartPosition_I, kNoMetaShipID,
						kDamagedShip1, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	}

	//
	//	Second alliance ship
	//
	switch ( fMissionInfo->mRandomInt32( 0 , 6 ) )
	{
	case 0: //	Federation
		mCreateShip( typeid( tDamagedShip2 ), "F-CB" , kStartPosition_J, kNoMetaShipID,
						kDamagedShip2, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		mCreateShip( typeid( tDamagedShip2 ), "K-FQL" , kStartPosition_J, kNoMetaShipID,
						kDamagedShip2, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		mCreateShip( typeid( tDamagedShip2 ), "R-KRC" , kStartPosition_J, kNoMetaShipID,
						kDamagedShip2, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		mCreateShip( typeid( tDamagedShip2 ), "L-CL" , kStartPosition_J, kNoMetaShipID,
						kDamagedShip2, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		mCreateShip( typeid( tDamagedShip2 ), "G-CM" , kStartPosition_J, kNoMetaShipID,
						kDamagedShip2, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		mCreateShip( typeid( tDamagedShip2 ), "H-DWF" , kStartPosition_J, kNoMetaShipID,
						kDamagedShip2, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		mCreateShip( typeid( tDamagedShip2 ), "Z-DD" , kStartPosition_J, kNoMetaShipID,
						kDamagedShip2, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	}

	//
	//	Thrid alliance ship
	//
	switch ( fMissionInfo->mRandomInt32( 0 , 6 ) )
	{
	case 0: //	Federation
		mCreateShip( typeid( tDamagedShip3 ), "F-CB" , kStartPosition_K, kNoMetaShipID,
						kDamagedShip3, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 1:	//	Klingon
		mCreateShip( typeid( tDamagedShip3 ), "K-FQL" , kStartPosition_K, kNoMetaShipID,
						kDamagedShip3, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 2:	//	Romulan
		mCreateShip( typeid( tDamagedShip3 ), "R-KRC" , kStartPosition_K, kNoMetaShipID,
						kDamagedShip3, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 3:	//	Lyran
		mCreateShip( typeid( tDamagedShip3 ), "L-CL" , kStartPosition_K, kNoMetaShipID,
						kDamagedShip3, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 4:	//	Gorn
		mCreateShip( typeid( tDamagedShip3 ), "G-CM" , kStartPosition_K, kNoMetaShipID,
						kDamagedShip3, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 5:	//	Hydran
		mCreateShip( typeid( tDamagedShip3 ), "H-DWF" , kStartPosition_K, kNoMetaShipID,
						kDamagedShip3, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	case 6:	//	Mirak
		mCreateShip( typeid( tDamagedShip3 ), "Z-DD" , kStartPosition_K, kNoMetaShipID,
						kDamagedShip3, 0, 0, -1, -1, NULL, kDefaultShipOptions );	break;
	}

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}

void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlayerNPCTeam", shipScriptDescription );
}
