/********************************************************************************************************\

	File Name:	PlayerShip.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShip.h"
#include "PlayerShipBaseState.h"
#include "MissionText.h"
#include "ShipList_API.h"


bool tPlayerShip::sIsOriginal = true;




tClassAllocator<tPlayerShip> gPlayerShipAllocator;

tPlayerShip::tPlayerShip() :	fIsOriginal( sIsOriginal ), 
								fHasHostages( false ),
								fHasRescueTeam( false ), 
								fHasEngineers( false)
{
	sIsOriginal = false;
}

/*

	HELPER NOTE:	If you create any new states on the ship level, you must register them here first.

*/
void tPlayerShip::mSetupState()
{
	mRegisterState( new tPlayerShipBaseState() );
	mGotoState( typeid( tPlayerShipBaseState ) );
}

/*

	HELPER NOTE:	This is the initialization function for the comm buttons of ships.  You could also set
					comm buttons on the team level, but its usefulness currently appears here on the ship
					level.  The best way to describe how to create comm buttons, is to simply give an example.
					Those constants that are passed in the following API calls can be found in the matching
					header file or should be created in the MissionText.h file.  Try these calls out and see
					what results to get a better idea of what every parameter means.

	void
	tPlayerShip::mSetupCommState( void )
	{
		mRegisterCommMessage( kBSCommState1 , kNoStateChange , mGetShipID() , k_BS_1_btn , k_BS_1_msg , kNoRandomMsg );
		mRegisterCommMessage( kBSCommState1 , kBlankCommState , mGetShipID() , k_BS_2_btn , k_BS_2_msg , kNoRandomMsg );
		mRegisterCommMessage( kBSCommState1 , kBSCommState2 , mGetShipID() , k_BS_5_btn , k_BS_5_msg , k_BS_7_msg );

		mRegisterCommMessage( kBSCommState2 , kNoStateChange , mGetShipID() , k_BS_4_btn , k_BS_8_msg , kNoRandomMsg );

		mGotoCommState( kBSCommState1 );
	}
*/
void tPlayerShip::mSetupCommState()
{}

/*

	HELPER NOTE:	This function is called once immediately after the ship has been created (perhaps with a
					call to mCreateShip()).  Anyway, in the body of this function, you can setup events that
					you wish to receive that normally are not on by default, change maximum ship speed, change
					the name of the ship, etc.  Below is a list of those events that need to be setup and
					those that do not.

Event							How to setup													Automatic

mAtDestination					tShipInfo::mSetFinalDestination									No
mAtDistance						tShipInfo::mTrackTacticalDistance								No
mOnAlertStatusChange			tShipInfo::mSetShipCallbackEvents( kNotifyAlertStatusChange )	No
mOnBeginOrbit					tShipInfo::mSetShipCallbackEvents( kNotifyBeginOrbit )			No
mOnBeginTractor					tShipInfo::mSetShipCallbackEvents( kNotifyBeginTractor )		No
mOnCapture						On by default													Yes
mOnChangingShip					On by default													Yes
mOnCommButtonPress				Setup a Comm Button State										Yes
mOnCommStateChange				Always happens when a comm state changes						Yes
mOnDamage						tShipInfo::mSetShipCallbackEvents( kNotifyDamage )				No
mOnDeath						On by default													Yes
mOnDeepScan						tShipInfo::mSetShipCallbackEvents( kNotifyDeepScan )			No
mOnDisengaged					On by default													Yes
mOnEndOrbit						tShipInfo::mSetShipCallbackEvents( kNotifyEndOrbit )			No
mOnEndTractor					tShipInfo::mTractorEntity										No
mOnFlashingButtonPressed		tMissionInfo::mStartFlashingButton								No
mOnHostChanged					On by default													Yes
mOnPlayerLeaving				On by default													Yes
mOnProbeLaunch					tShipInfo::mSetShipCallbackEvents( kNotifyProbeLaunch )			No
mOnSpecialTransport				tShipInfo::mSetShipCallbackEvents( kNotifySpecialTransport )	No
mOnStateChange					Always happens when a state changes								Yes
mOnTimedInterval				tEventMgr::mSetTimedInterval									No
mOnTransportedItem				tShipInfo::mSetShipCallbackEvents( kNotifyTransportedItem )		No


	//
	//	Changes the name of the ship, planet, object, etc.  The constant passed into this function should
	//	be declared in MissionText.h and the text for the object should correspond in MissionText.cpp.
	//
	mChangeName( kShipBS_nam );

	//
	//	Sets the maximum speed of the object if you so wish to limit it.
	//
	mSetShipMaxSpeed( 5 );

	//
	//	The following will set up events that normally are not on by default as described above.
	//
	mTrackTacticalDistance( 20.0, 5, 5 );

*/
void tPlayerShip::mSetupShip()
{	
	mSetShipCallbackEvents( kNotifyTransportedItem );

	//
	//	Place the engineers and rescue team on one of the player's ships.
	//
	if ( mIsOriginal() )
	{
		mAddItemToStores( kTransporterItem, kTransEngineers, 1 );
		mAddItemToStores( kTransporterItem, kTransMarineDude, 1 );		
		mSetHasEngineers( true );
		mSetHasRescueTeam( true );
	}
}

bool tPlayerShip::mIsOriginal( void )
{
	return fIsOriginal;
}
bool tPlayerShip::mHasHostages( void )
{
	return fHasHostages;
}
bool tPlayerShip::mHasRescueTeam( void )
{
	return fHasRescueTeam;
}
bool tPlayerShip::mHasEngineers( void )
{
	return fHasEngineers;
}
void tPlayerShip::mSetIsOriginal( bool x )
{
	fIsOriginal = x;
}
void tPlayerShip::mSetHasHostages( bool x )
{
	fHasHostages = x;
}
void tPlayerShip::mSetHasRescueTeam( bool x )
{
	fHasRescueTeam = x;
}
void tPlayerShip::mSetHasEngineers( bool x )
{
	fHasEngineers = x;
}

