/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "ShipScriptDescription.h"
#include "Uni_6Immunity.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam() :	fHasSeenkEvent6_1_msg( false ),
								fHasSeenkEvent6_2_msg( false ),
								fHasSeenkEvent8_1_msg( false ),
								fHasSeenkEvent9_1_msg( false ),
								fHasSeenkEvent9_2_msg( false ),
								fHasSeenkEvent11_1_msg( false ),
								fHasSeenkEvent12_1_msg( false ),
								fHasSeenkEvent12_2_msg( false ),
								fHasSeenkEvent14_1_msg( false ),
								fHasSeenkEvent5_1_msg( false ),
								fHasSeenkEvent17_1_msg( false ),
								fHasSeenkEvent17_2_msg( false ),
								fHasSeenkEvent19_1_msg( false ),
								fHasSeenkEvent19_2_msg( false ),
								fHasSeenkEvent7_1_msg( false ),
								fHasSeenkEvent10_1_msg( false ),
								fHasSeenkEvent13_1_msg( false ),
								fHasSeenkEvent18_1_msg( false ),
								fHasSeenkEvent22_1_msg( false ),
								fHasSeenkEvent3_1_msg( false ),
								fHasSeenkEvent3_2_msg( false )
{
}


void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{

/*

	HELPER NOTE:	The following is the analysis of the mission.  You are able to place additional text
					dependant upon the victory outcome in the MissionText.cpp file.  Simply find the
					corresponding values.

*/
	//
	//	Add the appropriate messages to the debriefing list according to the player's performance.
	//
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictory_msg );
		break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebVictory_msg );
		break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );
		break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeat_msg );
		break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeat_msg );
		break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( fTeam, kDisconnect_msg );
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	You may notice that the mCreateShipsForTeam(...) in this file has a different signature
					and body then the other team files.  This is because the 'tPlayerTeam' is of a
					'tPlayableTeam' class and it would be unknown how many ships the player has control
					of until the game has already begun.  However, if you want to force the player to play
					with a certain amount of ships (perhaps for a special mission), then the body of this
					function would look much more like the other team files.  To get an idea of this, refer
					to the helper notes on those team files.

*/
void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 
				 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;

#ifdef SLTOOLSDEBUGOUT
	// currently am testing as Gorn anyway.. force next mission to be Uni_X2M1Interceptor.
	fMissionScheduler.fNextMissionTitle = "Uni_X2M1Interceptor";
#else
	switch( mGetRace() )
	{
		case kRomulan:
		case kFederation:
		case kLyran:
		case kMirak:
			fMissionScheduler.fNextMissionTitle = "Uni_X5M1AMoon";
		break;
		case kGorn:
		case kKlingon:
			fMissionScheduler.fNextMissionTitle = "Uni_X2M1Interceptor";
		break;
		default:
			fMissionScheduler.fNextMissionTitle = "Uni_7BattleGrave";
		break;
	}
#endif

	switch ( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
		fMissionScheduler.fMedal = kMedalSpecialThree;
		break;
	}
}

bool tPlayerTeam::mHasSeenMessage( int32 messageID )
{
	gST.mDebug_fprintf("---- tPlayerTeam::mHasSeenMessage called for message ID %d\n", messageID);
	switch( messageID ) {
	// kDamagedShip1
	case kEvent6_1_msg: if ( ! fHasSeenkEvent6_1_msg ) { fHasSeenkEvent6_1_msg = true; return false; } else { return true; }; break;
	case kEvent6_2_msg: if ( ! fHasSeenkEvent6_2_msg ) { fHasSeenkEvent6_2_msg = true; return false; } else { return true; }; break;
	case kEvent8_1_msg: if ( ! fHasSeenkEvent8_1_msg ) { fHasSeenkEvent8_1_msg = true; return false; } else { return true; }; break;
	case kEvent7_1_msg: if ( ! fHasSeenkEvent7_1_msg ) { fHasSeenkEvent7_1_msg = true; return false; } else { return true; }; break;

	// kDamagedShip2
	case kEvent9_1_msg: if ( ! fHasSeenkEvent9_1_msg ) { fHasSeenkEvent9_1_msg = true; return false; } else { return true; }; break;
	case kEvent9_2_msg: if ( ! fHasSeenkEvent9_2_msg ) { fHasSeenkEvent9_2_msg = true; return false; } else { return true; }; break;
	case kEvent11_1_msg: if ( ! fHasSeenkEvent11_1_msg ) { fHasSeenkEvent11_1_msg = true; return false; } else { return true; }; break;
	case kEvent10_1_msg: if ( ! fHasSeenkEvent10_1_msg ) { fHasSeenkEvent10_1_msg = true; return false; } else { return true; }; break;

	// kDamagedShip5
	case kEvent12_1_msg: if ( ! fHasSeenkEvent12_1_msg ) { fHasSeenkEvent12_1_msg = true; return false; } else { return true; }; break;
	case kEvent12_2_msg: if ( ! fHasSeenkEvent12_2_msg ) { fHasSeenkEvent12_2_msg = true; return false; } else { return true; }; break;
	case kEvent14_1_msg: if ( ! fHasSeenkEvent14_1_msg ) { fHasSeenkEvent14_1_msg = true; return false; } else { return true; }; break;
	case kEvent13_1_msg: if ( ! fHasSeenkEvent13_1_msg ) { fHasSeenkEvent13_1_msg = true; return false; } else { return true; }; break;

	// kStarbase
	case kEvent5_1_msg: if ( ! fHasSeenkEvent5_1_msg ) { fHasSeenkEvent5_1_msg = true; return false; } else { return true; }; break;
	case kEvent17_1_msg: if ( ! fHasSeenkEvent17_1_msg ) { fHasSeenkEvent17_1_msg = true; return false; } else { return true; }; break;
	case kEvent17_2_msg: if ( ! fHasSeenkEvent17_2_msg ) { fHasSeenkEvent17_2_msg = true; return false; } else { return true; }; break;
	case kEvent19_1_msg: if ( ! fHasSeenkEvent19_1_msg ) { fHasSeenkEvent19_1_msg = true; return false; } else { return true; }; break;
	case kEvent19_2_msg: if ( ! fHasSeenkEvent19_2_msg ) { fHasSeenkEvent19_2_msg = true; return false; } else { return true; }; break;
	case kEvent18_1_msg: if ( ! fHasSeenkEvent18_1_msg ) { fHasSeenkEvent18_1_msg = true; return false; } else { return true; }; break;
	case kEvent22_1_msg: if ( ! fHasSeenkEvent22_1_msg ) { fHasSeenkEvent22_1_msg = true; return false; } else { return true; }; break;
	case kEvent3_1_msg: if ( ! fHasSeenkEvent3_1_msg ) { fHasSeenkEvent3_1_msg = true; return false; } else { return true; }; break;
	case kEvent3_2_msg: if ( ! fHasSeenkEvent3_2_msg ) { fHasSeenkEvent3_2_msg = true; return false; } else { return true; }; break;
	}
	return false;
}