//	<Name>	Starbase.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "MissionText.h"
#include "ScriptInterface.h"
#include "Starbase.h"
#include "StarbaseBaseState.h"
#include "TeamInfo.h"
#include "Uni_6Immunity.h"

tClassAllocator<tStarbase> gStarbaseAllocator;

tStarbase::tStarbase() :	fCanRepair( false ), fNumScrews( 0 ),
							fHasHostages( true ), fHasRescueTeam( false ), fHasEngineers( false )
{}
void tStarbase::mSetupState()
{
	mRegisterState( new tStarbaseBaseState() );
	mGotoState( typeid( tStarbaseBaseState ) );
}
void tStarbase::mSetupCommState()
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

//	playerTeam->mRegisterCommMessage(	kCommState1, kNoStateChange, mGetShipID(),
//										k_HAIL_btn, k_HAIL_msg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState1, kNoStateChange, mGetShipID(),
										k_DROPSHIELDS_Base_btn, kNoMsg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState1, kNoStateChange, mGetShipID(),
										k_SENDDESTRUCTCODE_Starbase1_btn, k_SENDDESTRUCTCODE_Starbase1_msg, kNoRandomMsg );

//	playerTeam->mRegisterCommMessage(	kCommState2, kNoStateChange, mGetShipID(),
//										k_HAIL_btn, k_HAIL_msg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState2, kNoStateChange, mGetShipID(),
										k_DROPSHIELDS_Base_btn, kNoMsg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState2, kNoStateChange, mGetShipID(),
										k_SENDDESTRUCTCODE_Starbase2_btn, k_SENDDESTRUCTCODE_Starbase2_msg, kNoRandomMsg );

//	playerTeam->mRegisterCommMessage(	kCommState3, kNoStateChange, mGetShipID(),
//										k_HAIL_btn, k_HAIL_msg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState3, kNoStateChange, mGetShipID(),
										k_DROPSHIELDS_Base_btn, kNoMsg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState3, kCommState4, mGetShipID(),
										k_SENDDESTRUCTCODE_Starbase3_btn, k_SENDDESTRUCTCODE_Starbase3_msg, kNoRandomMsg );

	playerTeam->mRegisterCommMessage(	kCommState4, kNoStateChange, mGetShipID(),
										k_DROPSHIELDS_Base_btn, kNoMsg, kNoRandomMsg );
	playerTeam->mRegisterCommMessage(	kCommState4, kNoStateChange, mGetShipID(),
										k_HAIL_btn, k_HAIL_msg, kNoRandomMsg );

	playerTeam->mGotoCommState( mGetShipID(), kCommState1 );
}
void tStarbase::mSetupShip()
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( playerRace )
	{
	case kFederation:
		mChangeName( kStarbaseFederation_nam );		break;
	case kKlingon:
		mChangeName( kStarbaseKlingon_nam );		break;
	case kRomulan:
		mChangeName( kStarbaseRomulan_nam );		break;
	case kLyran:
		mChangeName( kStarbaseLyran_nam );			break;
	case kGorn:
		mChangeName( kStarbaseGorn_nam );			break;
	case kHydran:
		mChangeName( kStarbaseHydran_nam );			break;
	case kMirak:
		mChangeName( kStarbaseMirak_nam );			break;
	}

	//
	//	Take away its repair supplies and damage the starbase.
	//
	mSetNumScrews( mDamageShip() );

	mSetShipCallbackEvents( kNotifyTransportedItem );
}

/*

	Purpose:	Will damage said ship considerably.  Will also not allow the ship to move.  Also, the
				ship's repair screws will be eliminated, but counted first and returned.

*/
int32 tStarbase::mDamageShip()
{
	int32 numScrews = 25;

	//
	//	Have the ship just sit there.
	//
	mSetShipMaxSpeed( 0 );

	//
	//	Damage the internals and shields of the ship.
	//
	mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f );
	mSetSystemHealthFromOrig( kRandomInternalsHardPoint, 0.0f );

	//
	//	Get the number of screws on the ship, then get rid of all the screws.
	//
	mGetAmountOfStore( kSpareParts );
	mRemoveItemFromStores( kTransSpareParts, numScrews );
	
	return numScrews;
}
/*

	Purpose:	Will bring a ship back up to 50%.

*/
void tStarbase::mRepairShip()
{
	//
	//	Set the ship's max speed.
	//
	mSetShipMaxSpeed( 0 );

	//
	//	Repair the engines, weapons, and shields to 100%.
	//
	mSetSystemHealthFromOrig( kAllShieldsHardPoint, 1.0f, kTrue );
	mSetSystemHealthFromOrig( kAllWeaponsHardPoint, 1.0f, kTrue );
//	mSetSystemHealthFromOrig( kRandomInternalsHardPoint, 1.0f, kTrue );
}

bool tStarbase::mCanRepair( void )
{
	return fCanRepair;
}
int32 tStarbase::mNumScrews( void )
{
	return fNumScrews;
}
bool tStarbase::mHasHostages( void )
{
	return fHasHostages;
}
bool tStarbase::mHasRescueTeam( void )
{
	return fHasRescueTeam;
}
bool tStarbase::mHasEngineers( void )
{
	return fHasEngineers;
}
void tStarbase::mSetCanRepair( bool x )
{
	fCanRepair = x;
}
void tStarbase::mSetNumScrews( int32 x )
{
	fNumScrews = x;
}
void tStarbase::mSetHasHostages( bool x )
{
	fHasHostages = x;
}
void tStarbase::mSetHasRescueTeam( bool x )
{
	fHasRescueTeam = x;
}
void tStarbase::mSetHasEngineers( bool x )
{
	fHasEngineers = x;
}