/********************************************************************************************************\

	File Name:	StarbaseBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "StarbaseBaseState.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "Uni_6Immunity.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tStarbaseBaseState::mBeginState()
{}
void tStarbaseBaseState::mEndState()
{}

void tStarbaseBaseState::mOnDeath( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnCapture( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tStarbaseBaseState::mAtDistance( tShipInfo* ship )
{}
void tStarbaseBaseState::mAtDestination( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( targetShip->mGetShipID() == fChildActor->mGetShipID() )
	{
		tPlayerTeam* playerTeam = static_cast< tPlayerTeam* >( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
		Assert( playerTeam );
		if ( !playerTeam )
			return;

		switch ( buttonID )
		{
		case k_SENDDESTRUCTCODE_Starbase2_btn:
			{
				if ( !fMissionInfo->mRandomInt32( 0, 3 ) )
				{
					playerTeam->mGotoCommState( fChildActor->mGetShipID(), tStarbase::kCommState3 );
				}
			}
			break;

		case k_SENDDESTRUCTCODE_Starbase3_btn:
			{
				fChildActor->mDestroyShip();
			}
			break;

		case k_DROPSHIELDS_Base_btn:
			{
				fChildActor->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f );
			}
			break;	
		}
	}
}
void tStarbaseBaseState::mOnDamage( tShipInfo* Starbase, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tStarbaseBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tStarbaseBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tStarbaseBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tStarbaseBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tStarbaseBaseState::mOnFlashingButtonPressed()
{}
void tStarbaseBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if ( ( !shipTo ) || ( !shipFrom ) )
		return;
	tPlayerTeam* playerTeam = static_cast< tPlayerTeam* >( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	tShipIterator	playerShip = playerTeam->mGetFirstShip();
	tShipInPlayEval	x;
	playerShip.mAddEvaluator( &x );

	//
	//	If the player is placing something on a ship.
	//
	if ( fChildActor->mGetTeam() == shipTo->mGetTeam() )
	{
		switch ( shipFrom->mGetCustomID() )
		{
		case tPlayerTeam::kPlayerShip1:
		case tPlayerTeam::kPlayerShip2:
		case tPlayerTeam::kPlayerShip3:
			{
				switch ( shipTo->mGetCustomID() )
				{
				case tNPC1Team::kStarbase:
					{
						switch ( item )
						{
						case kTransEngineers:
							{
								fChildActor->mRemoveItemFromStores( kTransEngineers, 1 );
								//
								//	The hostages should be rescued before the engineers go over to the starbase.
								//
								if ( fChildActor->mHasHostages() )
								{
									gST.mDebug_fprintf("SB: engineers! from shipID %d to shipID %d\n", 
										shipFrom->mGetCustomID(), shipTo->mGetCustomID());
									fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );
									shipFrom->mAddItemToStores( kTransporterItem, kTransEngineers, 1 );
								}
								//
								//	If the starbase is not functional yet, then it is fine for the engineers
								//	to transport over to the starbase.
								//
								else if ( !fChildActor->mCanRepair() )
								{
									gST.mDebug_fprintf("SB: engineers! from shipID %d to shipID %d\n", 
										shipFrom->mGetCustomID(), shipTo->mGetCustomID());
									if (! playerTeam->mHasSeenMessage( kEvent3_2_msg )) {
										fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_2_msg );
									}
									fChildActor->mSetTimedInterval( kTimerTurnsSet, tStarbase::kStarbaseRepairTimerID, 1.0f );
									( static_cast< tStarbase* >( shipTo ) )->mSetHasEngineers( true );
									( static_cast< tPlayerShip* >( shipFrom ) )->mSetHasEngineers( false );
								}
								//
								//	There is no need to send over the engineers if the starbase is already
								//	repairing itself.
								//
								else
								{
									gST.mDebug_fprintf("SB: engineers! from shipID %d to shipID %d\n", 
										shipFrom->mGetCustomID(), shipTo->mGetCustomID());

									if (! playerTeam->mHasSeenMessage( kEvent3_1_msg )) {
										fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent3_1_msg );
									}
									shipFrom->mAddItemToStores( kTransporterItem, kTransEngineers, 1 );
								}
							}
							break;

						case kTransMarineDude:
							{
								fChildActor->mRemoveItemFromStores( kTransMarineDude, 1 );
								//
								//	If the starbase has the hostages, then the rescue team can go over.
								//
								if ( fChildActor->mHasHostages() )
								{
									gST.mDebug_fprintf("SB: marines! from shipID %d to shipID %d\n", 
										shipFrom->mGetCustomID(), shipTo->mGetCustomID());

									if (! playerTeam->mHasSeenMessage( kEvent17_1_msg )) {
										fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent17_1_msg );
									}
									fChildActor->mSetTimedInterval( kTimerTurnsSet, tStarbase::kStarbaseRescuedTimerID, 1.5f );
									( static_cast< tStarbase* >( shipTo ) )->mSetHasRescueTeam( true );
									( static_cast< tPlayerShip* >( shipFrom ) )->mSetHasRescueTeam( false );
								}
								//
								//	The rescue team has already saved the hostages, there is no reason to
								//	beam them aboard the starbase.
								//
								else
								{
									gST.mDebug_fprintf("SB: marines! from shipID %d to shipID %d\n", 
										shipFrom->mGetCustomID(), shipTo->mGetCustomID());

									if (! playerTeam->mHasSeenMessage( kEvent17_2_msg )) {
										fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent17_2_msg );
									}
									shipFrom->mAddItemToStores( kTransporterItem, kTransMarineDude, 1 );
								}
							}
							break;
						}
					}
					break;
				}
			}
			break;
		}
	}
	//
	//	If the player is removing something from a ship.
	//
	else if ( fChildActor->mGetTeam() == shipFrom->mGetTeam() )
	{
		switch ( shipTo->mGetCustomID() )
		{
		case tPlayerTeam::kPlayerShip1:
		case tPlayerTeam::kPlayerShip2:
		case tPlayerTeam::kPlayerShip3:
			{
				switch ( shipFrom->mGetCustomID() )
				{
				case tNPC1Team::kStarbase:
					{
						switch ( item )
						{
						case kTransEngineers:
							{
								gST.mDebug_fprintf("SB: returning engineers! from shipID %d to shipID %d\n", 
										shipFrom->mGetCustomID(), shipTo->mGetCustomID());

								if (! playerTeam->mHasSeenMessage( kEvent5_1_msg )) {
									fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );
								}
								fChildActor->mSetCanRepair( true );
								fChildActor->mRepairShip();
								fChildActor->mDefendEntity( *playerShip, 100.0f, 10.0f, kAbsolutePriority );
								( static_cast< tPlayerShip* >( shipTo ) )->mSetHasEngineers( true );
								( static_cast< tStarbase* >( shipFrom ) )->mSetHasEngineers( false );
								playerTeam->mGotoCommState( fChildActor->mGetShipID(), tStarbase::kCommState2 );
							}
							break;

						case kTransMarineDude:
							{
								gST.mDebug_fprintf("SB: returning marines! from shipID %d to shipID %d\n", 
										shipFrom->mGetCustomID(), shipTo->mGetCustomID());

								if (! playerTeam->mHasSeenMessage( kEvent19_1_msg )) {
									fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent19_1_msg );
								}
								if (! playerTeam->mHasSeenMessage( kEvent19_2_msg )) {
									fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent19_2_msg );
								}
								( static_cast< tPlayerShip* >( shipTo ) )->mSetHasRescueTeam( true );
								( static_cast< tStarbase* >( shipFrom ) )->mSetHasRescueTeam( false );
								( static_cast< tPlayerShip* >( shipTo ) )->mSetHasHostages( true );
								( static_cast< tStarbase* >( shipFrom ) )->mSetHasHostages( false );
							}
							break;
						}
					}
					break;
				}
			}
			break;
		}
	}
}
void tStarbaseBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tStarbaseBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tStarbaseBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tStarbaseBaseState::mOnHostChanged()
{}
void tStarbaseBaseState::mOnTimedInterval( int32 timerID )
{
	tPlayerTeam* playerTeam = static_cast< tPlayerTeam* >( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( timerID )
	{
	case tStarbase::kStarbaseNoShieldTimerID:
		{
			if ( !fChildActor->mCanRepair() )
			{
				fChildActor->mSetSystemHealthFromOrig( kAllShieldsHardPoint, 0.0f );
			}
		}
		break;

	case tStarbase::kStarbaseRescuedTimerID:
		{
			fChildActor->mAddItemToStores( kTransporterItem, kTransMarineDude, 1 );
			if (! playerTeam->mHasSeenMessage( kEvent18_1_msg )) {
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent18_1_msg );
			}
		}
		break;

	case tStarbase::kStarbaseRepairTimerID:
		{
#if 0
//			fChildActor->mAddItemToStores( kTransporterItem, kTransSpareParts, fChildActor->mNumScrews() );
#endif
			fChildActor->mAddItemToStores( kTransporterItem, kTransEngineers, 1 );
			if (! playerTeam->mHasSeenMessage( kEvent22_1_msg )) {
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent22_1_msg );
			}
		}
		break;
	}
}
void tStarbaseBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tStarbaseBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


void tStarbaseBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tStarbaseBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tStarbaseBaseState::mOnShipStoresUpdate( tShipInfo* ship, eShipUpdateType type, eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tStarbaseBaseState::mOnShipStoresUpdate( tShipInfo* ship, eShipUpdateType type, eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}
