/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_6Wheel.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "FreighterTeam.h"
#include "WheelTeam.h"
#include "PlanetTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}

void tBaseVictoryState::mBeginState()
{
	//mSetTimedInterval( kTimerTurnsSet, kWheel_timer, 8.0f );
}

void tBaseVictoryState::mEndState()
{}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kWheel_timer:
		{
			/*
			tWheelTeam* wheelTeam = dynamic_cast<tWheelTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kWheelTeam ) ) );
			if( !wheelTeam )
					return;

			wheelTeam->mCreateReinforcements();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kWheelTeamArrives_msg );
			*/
		}
		break;

	default:
		break;
	}
}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	// Made an alliance and captured the planet
	if( ( playerTeam->mMadeAlliance() ) && ( playerTeam->mCappedPlanet() ) )
	{
		return kAstoundingVictory;
	}

	// All other conditions are losers
	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );
	if( !enemyTeam )
			return;

	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	if( !freighterTeam )
			return;

	tWheelTeam* wheelTeam = dynamic_cast<tWheelTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kWheelTeam ) ) );
	if( !wheelTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					playerTeam->mSetDied();
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

	case tEnemyTeam::kEnemy1:
	case tEnemyTeam::kEnemy2:
	case tEnemyTeam::kEnemy3:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetEnemyTeamDead();

				// If all "enemy" ships are killed without having obtained an explanation
				// from them first, the player loses.
				if( !playerTeam->mGotExplanation() )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kStoppedEnemyNoExplanation_msg );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
				}
			}
		}
		break;

	case tFreighterTeam::kFreighter:
		{
			playerTeam->mSetFreighterDead();

			// Player loses if the freighter is destroyed without having obtained an
			// explanation from the enemy team
			if( !playerTeam->mMadeAlliance() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kFreighterKilled_NoExplanation_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}

			// If an alliance was made, then the Wheel ships appear at this point
			else
			{
				wheelTeam->mCreateReinforcements();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kWheelTeamArrives_msg );
			}
		}
		break;

	case tWheelTeam::kWheel1:
	case tWheelTeam::kWheel2:
	case tWheelTeam::kWheel3:
	case tWheelTeam::kWheel4:
		{
			if( !wheelTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kWheelTeamDead_msg );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam ) );
	if( !enemyTeam )
			return;

	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	if( !freighterTeam )
			return;

	tWheelTeam* wheelTeam = dynamic_cast<tWheelTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kWheelTeam ) ) );
	if( !wheelTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;


	case tEnemyTeam::kEnemy1:
	case tEnemyTeam::kEnemy2:
	case tEnemyTeam::kEnemy3:
		{
			if( !enemyTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetEnemyTeamDead();

				// If all "enemy" ships are killed without having obtained an explanation
				// from them first, the player loses.
				if( !playerTeam->mGotExplanation() )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kStoppedEnemyNoExplanation_msg );
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
				}
			}
		}
		break;

	case tWheelTeam::kWheel1:
	case tWheelTeam::kWheel2:
	case tWheelTeam::kWheel3:
	case tWheelTeam::kWheel4:
		{
			ship->mDestroyShip();
			if( !wheelTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kWheelTeamDead_msg );
			}
		}
		break;

	case tFreighterTeam::kFreighter:
		{
			ship->mDestroyShip();
			playerTeam->mSetFreighterDead();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCapFreighter_msg );

			// Player loses if the freighter is destroyed without having obtained an
			// explanation from the enemy team
			if( !playerTeam->mMadeAlliance() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kFreighterKilled_NoExplanation_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}

			// If an alliance was made, then the Wheel ships appear at this point
			else
			{
				wheelTeam->mCreateReinforcements();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kWheelTeamArrives_msg );
			}
		}
		break;

	case tPlanetTeam::kPlanet:
		{
			playerTeam->mSetCappedPlanet();

			// This is a bad idea unless the player knows the story and has
			// made an alliance with the enemy team. Otherwise they're just
			// hijacking the planet.
			if( !playerTeam->mMadeAlliance() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_NoReason_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

