/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "Uni_6Wheel.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"

tPlayerShipBaseState::tPlayerShipBaseState()
{
	fScanning = false;
}

void tPlayerShipBaseState::mBeginState()
{}
void tPlayerShipBaseState::mEndState()
{}

void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fScanning = true;
		mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );
	}
	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, kScan_timer, 0.0f );
	}
}

void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kScan_timer:
		{
			if( !fScanning )
				return;
			mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );
			

			// Teams and ships

			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
					return;

			tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
			if( !freighterTeam )
					return;

			tShipIterator freighterI = freighterTeam->mGetFirstShip();
			tShipInfo* freighter = *freighterI;

			tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlanetTeam ) );
			if( !planetTeam )
					return;

			tShipIterator planetI = planetTeam->mGetFirstShip();
			tShipInfo* planet = *planetI;

			// Get range to planet
			int32 MinScanRangeToPlanet = 30;
			int32 RangeToPlanet;
			if( planet )
			{
				RangeToPlanet = fChildActor->mGetTacticalDistance( planet );
			}

			// ID of object being scanned.
			int32 ScanTargetID = fChildActor->mGetShipTargetID();


			// Scanning the freighter
			if( ( freighter ) && ( ScanTargetID == freighter->mGetShipID() ) && ( !playerTeam->mScannedFreighter() ) )
			{
				playerTeam->mSetScannedFreighter();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanFreighter_msg );
			}

			// Scanning the planet: only matters if the player got an explanation from the
			// enemy team
			if( ( planet ) && ( ScanTargetID == planet->mGetShipID() ) && ( playerTeam->mGotExplanation() ) && ( !playerTeam->mScannedPlanet() ) )
			{
				// Out of range
				if( ( RangeToPlanet > MinScanRangeToPlanet ) && ( !playerTeam->mScanRangeNotice() ) )
				{
					playerTeam->mSetScanRangeNotice();
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kPlanetOutOfScanRange_msg );
				}
				// In range
				else
				{
					playerTeam->mSetScannedPlanet();
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScannedPlanet_msg );
				}
			}		
		}
		break;

	default:
		break;
	}
}

void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	
}

void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}

void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}

void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

