/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fHailedFreighter = false;
	fScannedFreighter = false;
	fGotExplanation = false;
	fMadeAlliance = false;
	fCappedPlanet = false;
	fScannedPlanet = false;
	fWheelTeamArrived = false;
	fFreighterDead = false;
	fEnemyTeamDead = false;
	fScanRangeNotice = false;
	fDied = false;
}

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( fDied )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_PlayerDied_msg );
		mScheduleNextMission( fVictoryCondition );
		return;
	}

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
			mScheduleNextMission( fVictoryCondition );
			return;
		}
		break;
	}


	// Freighter destroyed without having made an alliance
	if( ( !fMadeAlliance ) && ( fFreighterDead ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_NoAllianceFrtDead_msg );
	}

	// Enemy team destroyed without having made an alliance 
	if( ( !fMadeAlliance  ) && ( fEnemyTeamDead ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_NoAllianceEnemyDead_msg );
	}

	// Capped the planet without an explanation
	if( ( !fGotExplanation ) && ( fCappedPlanet ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_NoExplanationCappedPlanet_msg );
	}

	// Made an alliance but didn't cap the planet
	if( ( fMadeAlliance ) && ( !fCappedPlanet ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_AllianceNoCapture_msg );
	}

	// Made an alliance and captured the planet
	if( ( fMadeAlliance ) && ( fCappedPlanet ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_AllianceCapture_msg );
	}

	mScheduleNextMission( fVictoryCondition );
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{

}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 400;
	fMissionScheduler.fNextMissionTitle = "Uni_7Gates";
}
