/********************************************************************************************************\

	File Name:	AllianceListeningPostBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllianceListeningPostBaseState.h"
#include "AllyShip.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "NPC2Team.h"
#include "PlayerNPCTeam.h"
#include "Transport.h"
#include "Uni_7BattleGrave.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tAllianceListeningPostBaseState::mBeginState()
{
	gST.mDebug_fprintf("--- Starting: tAllianceListeningPostShipBaseState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tAllianceListeningPostShipBaseState::mBeginState\n");
}
void tAllianceListeningPostBaseState::mEndState()
{}

void tAllianceListeningPostBaseState::mOnDeath( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mOnCapture( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mAtDistance( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mAtDestination( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( fChildActor->mGetCustomID() == targetShip->mGetCustomID() )
	{
		switch ( buttonID )
		{
		case k_REQUESTCRUISER_RelayStationForReal_btn:
			{
				tTeamInfo* playerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
				Assert( playerNPCTeam );
				if ( !playerNPCTeam )
					return;
				mSetTimedInterval( kTimerTurnsSet, kSpawnCruiserTimerID, 1.0f );

				tTeamInfo*	allyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
				Assert( allyTeam );
				if ( !allyTeam )
					return;
				eRaceName	allyRace = allyTeam->mGetRace();

				int32	customID =	( gBaseVictory->mAllianceWave() == 1 ) ?
									tPlayerNPCTeam::kAllyShip2 :
									tPlayerNPCTeam::kAllyShip1;
			}
			break;

		case k_REQUESTTRANSPORT_RelayStationForReal_btn:
			{
				tTeamInfo* NPC1Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNPC1Team ) );
				Assert( NPC1Team );
				if ( !NPC1Team )
					return;

				if ( gBaseVictory->mTransportAvailable() )
				{
					mSetTimedInterval( kTimerTurnsSet, kSpawnTransportTimerID, 1.0f );
				}
				gBaseVictory->mSetTransportAvailable( false );
			}
			break;

		case k_MOREMARINES_RelayStation_btn:
			{
				//
				//	Add more marines onto the player's ship so that the base stations can be captured.
				//
				tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
				Assert( team );
				if ( !team )
					return;

				float	dist = team->mCurrentlyControlledShip()->mGetTacticalDistance( targetShip );
				if ( dist <= 5.0f )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
					team->mCurrentlyControlledShip()->mAddItemToStores( kMarines, kStandard, 32 );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom7_msg );
				}
			}
			break;

		case k_SELFDESTRUCTBASE_RelayStation_btn:
			{
				//
				//   Tell all *3* captured & vacant bases to self-destruct.
				//

				tTeamInfo* PlayerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
				Assert( PlayerTeam );
				if ( !PlayerTeam )
					return;

				tTeamInfo* Enemy2Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy2Team ) );
				tTeamInfo* Enemy3Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy3Team ) );
				tTeamInfo* Enemy4Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy4Team ) );

				tShipInfo* base1 = Enemy2Team->mFindShipCustomID( kBase1 );
				tShipInfo* base2 = Enemy3Team->mFindShipCustomID( kBase2 );
				tShipInfo* base3 = Enemy4Team->mFindShipCustomID( kBase3 );

				if (base1)
					base1->mDestroyShip();
				if (base2)
					base2->mDestroyShip();
				if (base3)
					base3->mDestroyShip();

			}
		}
		fChildActor->mChangeCommState();
	}
}
void tAllianceListeningPostBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;
	static int count = 0;
	if ( fChildActor->mGetCustomID() == damagedShip->mGetCustomID() )
	{
		//
		//	Give a warning to the player that the transponder is being attacked.
		//
		if ( !count )
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent18_1_msg );
		count = 1;
		fChildActor->mSetShipCallbackEvents( kNotifyDamage );
	}
}
void tAllianceListeningPostBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tAllianceListeningPostBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tAllianceListeningPostBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllianceListeningPostBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllianceListeningPostBaseState::mOnFlashingButtonPressed()
{}
void tAllianceListeningPostBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tAllianceListeningPostBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tAllianceListeningPostBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tAllianceListeningPostBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tAllianceListeningPostBaseState::mOnHostChanged()
{}
void tAllianceListeningPostBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tAllianceListeningPostBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


void tAllianceListeningPostBaseState::mOnTimedInterval( int32 timerID )
{
	bool found_ally1 = false;
	bool found_ally2 = false;

	tPlayerNPCTeam *	playerNPCTeam = dynamic_cast<tPlayerNPCTeam*>(  fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) ) );
	Assert( playerNPCTeam );
	if ( playerNPCTeam )
    {
		tShipIterator tsi = playerNPCTeam->mGetFirstShip();
		for (;;)
		{
			Ship aShip;
			tShipInfo *si = *tsi;
			if (!si)
				break;

			if (si->mGetCustomID() == tPlayerNPCTeam::kAllyShip1)
				found_ally1 = true;
			if (si->mGetCustomID() == tPlayerNPCTeam::kAllyShip2)
				found_ally2= true;

			tsi++;
		}
	} else {
		return;
	}

	switch ( timerID )
	{
	case kSpawnCruiserTimerID:
		{
			if (! found_ally1) {
				playerNPCTeam->mCreateShip1();
			} else if ( ! found_ally2) {
				playerNPCTeam->mCreateShip2();
			}
			gBaseVictory->mIncreaseAllianceWave();
		}
		break;

	case kSpawnTransportTimerID:
		{
			eRaceName	playerRace;
			tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
			playerRace = playerTeam->mGetRace();
			tNPC1Team* NPC1Team = dynamic_cast<tNPC1Team*>(  fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNPC1Team ) ) );

			NPC1Team->mCreateTransport();

			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
		}
		break;

	case kUpgradeFightersID:
		{
#ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
        if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tAllianceListeningPostBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
        int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
        gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tAllianceListeningPostBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
		}
		break;
	}

}

