/********************************************************************************************************\

	File Name:	AllyShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllyShipBaseState.h"
#include "MissionText.h"
#include "Uni_7BattleGrave.h"
#include "ShipList_API.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tAllyShipBaseState::mBeginState()
{
    gST.mDebug_fprintf("--- Starting: tAllyShipBaseState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tAllyShipBaseState::mBeginState\n");
}
void tAllyShipBaseState::mEndState()
{}

void tAllyShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tAllyShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tAllyShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( targetShip->mGetCustomID() == fChildActor->mGetCustomID() )
	{
		switch ( buttonID )
		{
		case k_DEFENDME_AllyForReal_btn:
			{
				tTeamInfo* team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
				Assert( team );
				if ( !team )
					return;

				targetShip->mDefendEntity( team->mCurrentlyControlledShip(), 25.0f, 5.0f, kAbsolutePriority );
			}
		}
	}
}
void tAllyShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if ( ( !damagedShip ) || ( !hostileShip ) )
		return;
	float	hullDamage = damagedShip->mGetDamagePercent();
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	if ( fChildActor->mGetCustomID() == damagedShip->mGetCustomID() )
	{
		if ( hostileShip )
		{
			if ( hullDamage == 0.0f )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState1 );
			}
			else if ( hullDamage < 30.0f )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState2 );
			}
			else if ( hullDamage < 60.0f )
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState3 );
			}
			else
			{
				playerTeam->mGotoCommState( fChildActor->mGetShipID(), tAllyShip::kCommState4 );
			}
		}
	}

	fChildActor->mSetShipCallbackEvents( kNotifyDamage );
}
void tAllyShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tAllyShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tAllyShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllyShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tAllyShipBaseState::mOnFlashingButtonPressed()
{}
void tAllyShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tAllyShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tAllyShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tAllyShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tAllyShipBaseState::mOnHostChanged()
{}
void tAllyShipBaseState::mOnTimedInterval( int32 timerID )
{
    if (timerID == kUpgradeFightersID) {
#ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
        if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tAllyShipBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
        int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
        gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tAllyShipBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}
void tAllyShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tAllyShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


