/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllianceListeningPost.h"
#include "BaseVictoryState.h"
#include "EnemyTeam.h"
#include "Enemy2Team.h"
#include "Enemy3Team.h"
#include "Enemy4Team.h"
#include "Enemy5Team.h"
#include "Enemy6Team.h"
#include "ISCShip.h"
#include "MissionText.h"
#include "NPC1Team.h"
#include "NPC2Team.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "TeamInfo.h"
#include "Transport.h"
#include "Uni_7BattleGrave.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}


eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return kDraw;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( playerRace )
	{
	case kFederation:
	case kMirak:
		{
			if (	!fChildActor->mBase1PrisonersRescued() || !fChildActor->mBase2PrisonersRescued() ||
					!fChildActor->mBase3PrisonersRescued() )
			{
				return kDefeat;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && !fChildActor->mISCListeningPostDestroyed()	)
			{
				return kVictory;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && fChildActor->mISCListeningPostDestroyed()	)
			{
				return kAstoundingVictory;
			}
		}
		break;

	case kKlingon:
		{
			if (	!fChildActor->mBase1PrisonersRescued() || !fChildActor->mBase2PrisonersRescued() ||
					!fChildActor->mBase3PrisonersRescued() )
			{
				return kDefeat;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && fChildActor->mISCShipsDestroyed() < 15		)
			{
				return kVictory;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && fChildActor->mISCShipsDestroyed() >= 15	)
			{
				return kAstoundingVictory;
			}
		}
		break;

	case kRomulan:
	case kGorn:
		{
			if (	!fChildActor->mBase1PrisonersRescued() || !fChildActor->mBase2PrisonersRescued() ||
					!fChildActor->mBase3PrisonersRescued() )
			{
				return kDefeat;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && !fChildActor->mCBCaptured()	)
			{
				return kVictory;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && fChildActor->mCBCaptured()	)
			{
				return kAstoundingVictory;
			}
		}
		break;

	case kLyran:
	case kHydran:
		{
			if (	!fChildActor->mBase1PrisonersRescued() || !fChildActor->mBase2PrisonersRescued() ||
					!fChildActor->mBase3PrisonersRescued() )
			{
				return kDefeat;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && ( !fChildActor->mBase1Destroyed() ||
						  !fChildActor->mBase2Destroyed() || !fChildActor->mBase3Destroyed() )	)
			{
				return kVictory;
			}
			else if (	( fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
						  fChildActor->mBase3PrisonersRescued() ) && ( fChildActor->mBase1Destroyed() &&
						  fChildActor->mBase2Destroyed() && fChildActor->mBase3Destroyed() )	)
			{
				return kAstoundingVictory;
			}
		}
		break;
	}

	return kDraw;
}

//
//	Ship events
//

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	//
	//	Get a pointer to the alliance listening post since that comm state will be changing often here.
	//
	tTeamInfo*	NPC2Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNPC2Team ) );
	Assert( NPC2Team );
	if ( !NPC2Team )
		return;
	tShipIterator	allianceLP = NPC2Team->mGetFirstShip();

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );

			//
			//	Adjust counts of ISC ships.
			//
			gBaseVictory->mDecreaseAllianceWave();
			gBaseVictory->mIncreaseAllianceShipsDestroyed();

			//
			//	Change the comm state for the alliance listening post.
			//
			if ( *allianceLP != NULL )
				( static_cast< tAllianceListeningPost* >( *allianceLP ) )->mChangeCommState();
		}
		break;

	case tEnemyTeam::kISC1:
	case tEnemyTeam::kISC2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom8_msg );

			tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
			Assert( enemyTeam );
			if ( !enemyTeam )
				return;

			//
			//	Adjust counts of ISC ships.
			//
			gBaseVictory->mDecreaseEnemyWave();
			gBaseVictory->mIncreaseISCShipsDestroyed();

			//
			//	If there are no more enemy cruisers, spawn 2 more.
			//
			if ( gBaseVictory->mEnemyWave() == 0 && gBaseVictory->mISCShipsDestroyed() < 10 )
			{
				enemyTeam->mSetTimedInterval( kTimerTurnsSet, tEnemyTeam::kSpawnTimerID, 1.0f );
			}

			switch ( playerRace )
			{
			case kKlingon:
				{
					//
					//	The Klingon secondary objective has been completed.  End the mission if the
					//	primary objective has also been completed.
					//
					if ( gBaseVictory->mISCShipsDestroyed() >= 10 )
					{
						mAddToBonusPrestige( 300 );
						if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase2PrisonersRescued() &&
								gBaseVictory->mBase3PrisonersRescued()	)
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_3_msg );
						}
					}
				}
				break;
			}
		}
		break;

	case tEnemy2Team::kBase1:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom14_msg );
			gBaseVictory->mSetBase1Destroyed( true );
			if ( !gBaseVictory->mBase1PrisonersLoaded() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent14_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}

			switch ( playerRace )
			{
			case kLyran:
			case kHydran:
				{
					mAddToBonusPrestige( 100 );
					if (	gBaseVictory->mBase1Destroyed() && gBaseVictory->mBase2Destroyed() &&
							gBaseVictory->mBase3Destroyed()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent14_3_msg );
					}
					else if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase1Destroyed() &&
								gBaseVictory->mBase2PrisonersRescued() && gBaseVictory->mBase2Destroyed() &&
								gBaseVictory->mBase3PrisonersRescued() && gBaseVictory->mBase3Destroyed()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					}
				}
				break;
			}
		}
		break;

	case tEnemy3Team::kBase2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom14_msg );
			gBaseVictory->mSetBase2Destroyed( true );
			if ( !gBaseVictory->mBase2PrisonersLoaded() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent15_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}

			switch ( playerRace )
			{
			case kLyran:
			case kHydran:
				{
					mAddToBonusPrestige( 100 );
					if (	gBaseVictory->mBase1Destroyed() && gBaseVictory->mBase2Destroyed() &&
							gBaseVictory->mBase3Destroyed()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent15_3_msg );
					}
					else if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase1Destroyed() &&
								gBaseVictory->mBase2PrisonersRescued() && gBaseVictory->mBase2Destroyed() &&
								gBaseVictory->mBase3PrisonersRescued() && gBaseVictory->mBase3Destroyed()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					}
				}
				break;
			}
		}
		break;

	case tEnemy4Team::kBase3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom14_msg );
			mAddToBonusPrestige( 100 );
			gBaseVictory->mSetBase3Destroyed( true );
			if ( !gBaseVictory->mBase3PrisonersLoaded() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent16_1_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}

			switch ( playerRace )
			{
			case kLyran:
			case kHydran:
				{
					if (	gBaseVictory->mBase1Destroyed() && gBaseVictory->mBase2Destroyed() &&
							gBaseVictory->mBase3Destroyed()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent16_3_msg );
					}
					else if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase1Destroyed() &&
								gBaseVictory->mBase2PrisonersRescued() && gBaseVictory->mBase2Destroyed() &&
								gBaseVictory->mBase3PrisonersRescued() && gBaseVictory->mBase3Destroyed()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					}
				}
				break;
			}
		}
		break;

	case tEnemy5Team::kListeningPostShip:
		{
			gBaseVictory->mSetISCListeningPostDestroyed( true );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom11_msg );

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					mAddToBonusPrestige( 300 );
					if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase2PrisonersRescued() &&
							gBaseVictory->mBase3PrisonersRescued()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					}
					else
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent17_1_msg );
					}
				}
				break;
			}
		}
		break;

	case tEnemy6Team::kFlagship:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom15_msg );
			gBaseVictory->mSetCBDestroyed( true );

			switch ( playerRace )
			{
			case kRomulan:
			case kGorn:
				{
					//
					//	The secondary objective was accomplished.  Should the mission end?
					//
					mAddToBonusPrestige( 300 );
					if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase2PrisonersRescued() &&
							gBaseVictory->mBase3PrisonersRescued()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					}
					else
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_2_msg );
					}
				}
				break;
			}
		}
		break;

	case tNPC1Team::kTransport:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_1_msg );
			if ( gBaseVictory->mBase1PrisonersLoaded() && !gBaseVictory->mBase1PrisonersRescued() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_2_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
			else if ( gBaseVictory->mBase2PrisonersLoaded() && !gBaseVictory->mBase2PrisonersRescued() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_3_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
			else if ( gBaseVictory->mBase3PrisonersLoaded() && !gBaseVictory->mBase3PrisonersRescued() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent13_4_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
			
			//
			//	Change the comm state for the alliance listening post.
			//
			( static_cast< tAllianceListeningPost* >( *allianceLP ) )->mChangeCommState();
		}
		break;

	case tNPC2Team::kAllianceListeningPost:
		{
			//
			//	Check the status of the rescued prisoners.  If it is not possible to complete the
			//	primary objective with a transport that is already in play or if there is no transport
			//	in play any longer, then stop the mission as failed.
			//
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent19_1_msg );
			if ( gBaseVictory->mTransportAvailable() )
			{
				if (	!gBaseVictory->mBase1PrisonersRescued() || !gBaseVictory->mBase2PrisonersRescued() ||
						!gBaseVictory->mBase3PrisonersRescued()	)
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent19_2_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
				}
			}
			else
			{
				if (	( !gBaseVictory->mBase1PrisonersRescued() && !gBaseVictory->mBase2PrisonersRescued() ) ||
						( !gBaseVictory->mBase1PrisonersRescued() && !gBaseVictory->mBase3PrisonersRescued() ) ||
						( !gBaseVictory->mBase2PrisonersRescued() && !gBaseVictory->mBase3PrisonersRescued() )	)
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom5_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
				}
			}
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	tTeamInfo*	NPC2Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNPC2Team ) );
	Assert( NPC2Team );
	if ( !NPC2Team )
		return;
	tShipIterator	allianceLP = NPC2Team->mGetFirstShip();

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{}
		break;

	case tEnemyTeam::kISC1:
	case tEnemyTeam::kISC2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom12_msg );

			tTeamInfo*	enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
			Assert( enemyTeam );
			if ( !enemyTeam )
				return;

			//
			//	Adjust counts of ISC ships.
			//
			gBaseVictory->mDecreaseEnemyWave();
			gBaseVictory->mIncreaseISCShipsDestroyed();

			//
			//	If there are no more enemy cruisers, spawn 2 more.
			//
			if ( gBaseVictory->mEnemyWave() == 0 && gBaseVictory->mISCShipsDestroyed() < 10 )
			{
				enemyTeam->mSetTimedInterval( kTimerTurnsSet, tEnemyTeam::kSpawnTimerID, 1.0f );
			}

			switch ( playerRace )
			{
			case kKlingon:
				{
					//
					//	The Klingon secondary objective has been completed.  End the mission if the
					//	primary objective has also been completed.
					//
					if ( gBaseVictory->mISCShipsDestroyed() >= 10 )
					{
						mAddToBonusPrestige( 300 );
						if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase2PrisonersRescued() &&
								gBaseVictory->mBase3PrisonersRescued()	)
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent4_3_msg );
						}
					}
				}
				break;
			}
		}
		break;

	case tEnemy2Team::kBase1:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent6_1_msg );
			gBaseVictory->mSetBase1Captured( true );

			//
			//	Change the comm state for the alliance listening post.
			//
			( static_cast< tAllianceListeningPost* >( *allianceLP ) )->mChangeCommState();

		}
		break;

	case tEnemy3Team::kBase2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent8_1_msg );
			gBaseVictory->mSetBase2Captured( true );

			//
			//	Change the comm state for the alliance listening post.
			//
			( static_cast< tAllianceListeningPost* >( *allianceLP ) )->mChangeCommState();
		}
		break;

	case tEnemy4Team::kBase3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent10_1_msg );
			gBaseVictory->mSetBase3Captured( true );

			//
			//	Change the comm state for the alliance listening post.
			//
			( static_cast< tAllianceListeningPost* >( *allianceLP ) )->mChangeCommState();
		}
		break;

	case tEnemy5Team::kListeningPostShip:
		{
			gBaseVictory->mSetISCListeningPostDestroyed( true );
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom11_msg );

			switch ( playerRace )
			{
			case kFederation:
			case kMirak:
				{
					mAddToBonusPrestige( 300 );
					if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase2PrisonersRescued() &&
							gBaseVictory->mBase3PrisonersRescued()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					}
					else
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
					}
				}
				break;
			}
		}
		break;

	case tEnemy6Team::kFlagship:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );
			gBaseVictory->mSetCBCaptured( true );

			switch ( playerRace )
			{
			case kRomulan:
			case kGorn:
				{
					//
					//	Secondary objective achieved for Romulan and Gorn.
					//
					mAddToBonusPrestige( 300 );
					if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase2PrisonersRescued() &&
							gBaseVictory->mBase3PrisonersRescued()	)
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
						gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
					}
					else
					{
						fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_2_msg );
					}
				}
				break;
			}
		}
		break;

	case tNPC1Team::kTransport:
		{}
		break;

	case tNPC2Team::kAllianceListeningPost:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;
	eRaceName	playerRace = playerTeam->mGetRace();

	tTeamInfo*	NPC2Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kNPC2Team ) );
	Assert( NPC2Team );
	if ( !NPC2Team )
		return;
	tShipIterator	allianceLP = NPC2Team->mGetFirstShip();

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );

			//
			//	Adjust counts of ISC ships.
			//
			gBaseVictory->mDecreaseAllianceWave();
			gBaseVictory->mIncreaseAllianceShipsDestroyed();

			//
			//	Change the comm state for the alliance listening post.
			//
			if ( *allianceLP != NULL )
				( static_cast< tAllianceListeningPost* >( *allianceLP ) )->mChangeCommState();
		}
		break;

	case tEnemyTeam::kISC1:
	case tEnemyTeam::kISC2:
		{}
		break;

	case tEnemy2Team::kBase1:
		{}
		break;

	case tEnemy3Team::kBase2:
		{}
		break;

	case tEnemy4Team::kBase3:
		{}
		break;

	case tEnemy5Team::kListeningPostShip:
		{}
		break;

	case tEnemy6Team::kFlagship:
		{}
		break;

	case tNPC1Team::kTransport:
		{
			//
			//	The transport has left the system, so the alliance will send another now if requested.
			//
			gBaseVictory->mSetTransportAvailable( true );

			if ( ( static_cast< tTransport* >( ship ) )->mSaving1() )
			{
				gBaseVictory->mSetBase1PrisonersRescued( true );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_1_msg );
				( static_cast< tTransport* >( ship ) )->mSetSaving1( false );
			}
			else if ( ( static_cast< tTransport* >( ship ) )->mSaving2() )
			{
				gBaseVictory->mSetBase2PrisonersRescued( true );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_4_msg );
				( static_cast< tTransport* >( ship ) )->mSetSaving2( false );
			}
			else if ( ( static_cast< tTransport* >( ship ) )->mSaving3() )
			{
				gBaseVictory->mSetBase3PrisonersRescued( true );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_7_msg );
				( static_cast< tTransport* >( ship ) )->mSetSaving3( false );
			}
			
			//
			//	If all of the prisoners were saved, check if the secondary objective was complete and
			//	if so, continue on; otherwise, end the mission.
			//
			if (	gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase2PrisonersRescued() &&
					gBaseVictory->mBase3PrisonersRescued()	)
			{
				switch ( playerRace )
				{
				case kFederation:
				case kMirak:
					{
						if ( gBaseVictory->mISCListeningPostDestroyed() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_2_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom10_msg );
						}
					}
					break;

				case kKlingon:
					{
						if ( gBaseVictory->mISCShipsDestroyed() >= 10 )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_2_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
						}
						else if ( gBaseVictory->mISCListeningPostDestroyed() )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_2_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom10_msg );
						}
					}
					break;

				case kRomulan:
				case kGorn:
					{
						if ( gBaseVictory->mCBCaptured() || gBaseVictory->mCBDestroyed()  )
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_2_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom10_msg );
						}
					}
					break;

				case kLyran:
				case kHydran:
					{
						if (	gBaseVictory->mBase1Destroyed() && gBaseVictory->mBase2Destroyed() &&
								gBaseVictory->mBase3Destroyed()		)
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_2_msg );
							gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom13_msg );
						}
						else
						{
							fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom10_msg );
						}
					}
					break;
				}
			}

			//
			//	Change the comm state for the alliance listening post.
			//
			( static_cast< tAllianceListeningPost* >( *allianceLP ) )->mChangeCommState();
		}
		break;

	case tNPC2Team::kAllianceListeningPost:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent21_1_msg );
			if (	fChildActor->mBase1PrisonersRescued() && fChildActor->mBase2PrisonersRescued() &&
					fChildActor->mBase3PrisonersRescued()	)
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{}
		break;

	case tEnemyTeam::kISC1:
	case tEnemyTeam::kISC2:
		{}
		break;

	case tEnemy2Team::kBase1:
		{}
		break;

	case tEnemy3Team::kBase2:
		{}
		break;

	case tEnemy4Team::kBase3:
		{}
		break;

	case tEnemy5Team::kListeningPostShip:
		{}
		break;

	case tEnemy6Team::kFlagship:
		{}
		break;

	case tNPC1Team::kTransport:
		{
			if ( ( static_cast< tTransport* >( ship ) )->mSaving1() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent7_1_msg );
				gBaseVictory->mSetBase1PrisonersLoaded( true );			
			}
			else if ( ( static_cast< tTransport* >( ship ) )->mSaving2() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent9_1_msg );
				gBaseVictory->mSetBase2PrisonersLoaded( true );
			}
			else if ( ( static_cast< tTransport* >( ship ) )->mSaving3() )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent11_1_msg );
				gBaseVictory->mSetBase3PrisonersLoaded( true );
			}
			ship->mSetShipMaxSpeed( 31 );  // a kick in the butt to disengage. Mission's long enough already. -- FS
			ship->mDisengage( kByMapEdge, kAbsolutePriority );
			playerTeam->mGotoCommState( ship->mGetShipID(), tTransport::kBlankCommState ); // blank the comm messages. -- FS
		}
		break;

	case tNPC2Team::kAllianceListeningPost:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{}
		break;
	}
}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );	break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );	break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );	break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );	break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );	break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );	break;
		}
		fChildActor->mSetTimedInterval(	kTimerSecondsSet, timerID, ( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
