/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "ISCShip.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        }
		gST.listShiplistContent(sl);

		// 2 CL/CM like ships coming up!
		if (sl.size() > 0)
		{
			Ship s1, s2;
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

			mCreateShip(	
				typeid( tISCShip ), 
				s1.shipCode.c_str(), 
				kStartPosition_X, 
				kNoMetaShipID,
				kISC1, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );

			mCreateShip(	
				typeid( tISCShip ), 
				s2.shipCode.c_str(), 
				kStartPosition_Y, 
				kNoMetaShipID,
				kISC2, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );
			} else {
				mOldCreateShipsForTeam();
			}
    } else {
        mOldCreateShipsForTeam();
    }
    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	
		typeid( tISCShip ), 
		"I-CM", 
		kStartPosition_X, 
		kNoMetaShipID,
		kISC1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	mCreateShip(	
		typeid( tISCShip ), 
		"I-CM", 
		kStartPosition_Y, 
		kNoMetaShipID,
		kISC2, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}
