/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "Race.h"
#include "tclient.h"
#include "AllyShip.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "Uni_7BattleGrave.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}

void tPlayerNPCTeam::mCreateShipsForTeam()
{
	    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");

		bool found_ally1 = false;
		bool found_ally2 = false;
		tShipIterator tsi = mGetFirstShip();
		for (;;)
		{
			Ship aShip;
			tShipInfo *si = *tsi;
			if (!si)
				break;
			if (si->mGetCustomID() == kAllyShip1)
				found_ally1 = true;
			if (si->mGetCustomID() == kAllyShip2)
				found_ally2 = true;
			tsi++;
		}
		if (! found_ally1)
			mCreateShip1();
		if (! found_ally2)
			mCreateShip2();


	    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}

void tPlayerNPCTeam::mCreateShip1()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShip1\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		bool found_ally1 = false;
		tShipIterator tsi = mGetFirstShip();
		for (;;)
		{
			Ship aShip;
			tShipInfo *si = *tsi;
			if (!si)
				break;
			if (si->mGetCustomID() == kAllyShip1)
				found_ally1 = true;
			tsi++;
		}

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassHeavyCruiser; i <= kEClassHeavyBattlecruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA classes filter completed. ships: %d\n", fShiplistCount);
        }

		ShipList leader_sl = sl;
		std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = leader_sl.Subset(leaderRolesList, true);
        gST.mDebug_fprintf("subset for Leader eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(leader_sl);

		if (leader_sl.size() > 0)
		{
			Ship s1;
			if (! found_ally1) {
				leader_sl.SelectRandomShip(s1);
				gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
				gST.mTrackOrions( s1 );

				mCreateShip( 
					typeid( tAllyShip ), 
					s1.shipCode.c_str(), 
					kStartPosition_H, 
					kNoMetaShipID, 
					kAllyShip1, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
			}

		} else {
			mOldCreateShip1();
		}
    } else {
        mOldCreateShip1();
    }
    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShip1\n");
}

void tPlayerNPCTeam::mCreateShip2()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShip2\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

		bool found_ally2 = false;
		tShipIterator tsi = mGetFirstShip();
		for (;;)
		{
			Ship aShip;
			tShipInfo *si = *tsi;
			if (!si)
				break;
			if (si->mGetCustomID() == kAllyShip2)
				found_ally2= true;

			tsi++;
		}

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );

        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassHeavyCruiser; i <= kEClassHeavyBattlecruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA classes filter completed. ships: %d\n", fShiplistCount);
        }

		ShipList lineship_sl = sl;
		std::list< eEnhancedRoles >  lineshipRolesList;
        lineshipRolesList.push_back(kLineShip);
        fShiplistCount = lineship_sl.Subset(lineshipRolesList, true);
        gST.mDebug_fprintf("subset for lineship eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(lineship_sl);

		if (lineship_sl.size() > 0)
		{
			Ship s2;
			if (! found_ally2) {
				lineship_sl.SelectRandomShip(s2);
				gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
				gST.mTrackOrions( s2 );

				mCreateShip( 
					typeid( tAllyShip ), 
					s2.shipCode.c_str(), 
					kStartPosition_I, 
					kNoMetaShipID, 
					kAllyShip2, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
			}

		} else {
			mOldCreateShip2();
		}
    } else {
        mOldCreateShip2();
    }
    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShip2\n");
}

void tPlayerNPCTeam::mOldCreateShip1()
{
	gST.mDebug_fprintf("---- Starting: tPlayerNPCTeam::mOldCreateShip1\n");

	tTeamInfo*	allyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( allyTeam );
	if ( !allyTeam )
		return;

	std::string allyShip1_type = mSelectShipType(1);
	std::string allyShip2_type = mSelectShipType(2);

	bool found_ally1 = false;
	tShipIterator tsi = mGetFirstShip();
	for (;;)
	{
		Ship aShip;
		tShipInfo *si = *tsi;
		if (!si)
			break;
		if (si->mGetCustomID() == kAllyShip1)
			found_ally1 = true;
		tsi++;
	}

	if (! found_ally1 ) {
		mCreateShip( 
			typeid( tAllyShip ), 
			allyShip1_type.c_str(), 
			kStartPosition_H, 
			kNoMetaShipID, 
			kAllyShip1, 
			0, 0, -1, -1, 
			NULL, 
			kDefaultShipOptions );
	}

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShip1\n");
}

void tPlayerNPCTeam::mOldCreateShip2()
{
	gST.mDebug_fprintf("---- Starting: tPlayerNPCTeam::mOldCreateShip2\n");

	tTeamInfo*	allyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( allyTeam );
	if ( !allyTeam )
		return;

	std::string allyShip2_type = mSelectShipType(2);
	bool found_ally2 = false;
	tShipIterator tsi = mGetFirstShip();
	for (;;)
	{
		Ship aShip;
		tShipInfo *si = *tsi;
		if (!si)
			break;
		if (si->mGetCustomID() == kAllyShip2)
			found_ally2= true;
		tsi++;
	}


	if (! found_ally2 ) {
		mCreateShip( 
			typeid( tAllyShip ), 
			allyShip2_type.c_str(), 
			kStartPosition_I, 
			kNoMetaShipID, 
			kAllyShip2, 
			0, 0, -1, -1, 
			NULL, 
			kDefaultShipOptions );
	}

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShip2\n");
}

void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlayerNPCTeam", shipScriptDescription );
}

std::string tPlayerNPCTeam::mSelectShipType(int which_ship) {
	std::string allyShip1_type = "F-BCG";
	std::string allyShip2_type = "F-BCJ";
	
	tTeamInfo*	allyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( allyTeam );
	if ( !allyTeam )
		return allyShip1_type;


	switch ( allyTeam->mGetRace() )
	{
	case kFederation:
		allyShip1_type = "F-BCG";
		allyShip2_type = "F-BCJ";
		break;
	case kKlingon:
		allyShip1_type = "K-D6D";
		allyShip2_type = "K-D7C";
		break;
	case kRomulan:
		allyShip1_type = "R-K7R";
		allyShip2_type = "R-KCR";
		break;
	case kLyran:
		allyShip1_type = "L-BC";
		allyShip2_type = "L-BCPP";
		break;
	case kGorn:
		allyShip1_type = "G-BC";
		allyShip2_type = "G-BCH";
		break;
	case kHydran:
		allyShip1_type = "H-OV";
		allyShip2_type = "H-D7H";
		break;
	case kMirak:
		allyShip1_type = "Z-BCH";
		allyShip2_type = "Z-BC";
		break;
	}

	switch(which_ship) {
	case 1: 
		return allyShip1_type;
		break;
	case 2:
		return allyShip2_type;
		break;
	default:
		return allyShip1_type;
		break;
	}

	return allyShip1_type;
}