/********************************************************************************************************\

	File Name:	TransportBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "MissionText.h"
#include "TeamInfo.h"
#include "TransportBaseState.h"
#include "Uni_7BattleGrave.h"
#include "ShipList_API.h"


/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tTransportBaseState::mBeginState()
{
    gST.mDebug_fprintf("--- Starting: tTransportBaseState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tTransportBaseState::mBeginState\n");
}
void tTransportBaseState::mEndState()
{}

void tTransportBaseState::mOnDeath( tShipInfo* ship )
{}
void tTransportBaseState::mOnCapture( tShipInfo* ship )
{}
void tTransportBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tTransportBaseState::mAtDistance( tShipInfo* ship )
{}
void tTransportBaseState::mAtDestination( tShipInfo* ship )
{}
void tTransportBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if ( fChildActor->mGetCustomID() == targetShip->mGetCustomID() )
	{
		switch ( buttonID )
		{
		case k_GOTOSTATION_Transport_btn:
			{
				tTeamInfo*	base1Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy2Team ) );
				tTeamInfo*	base2Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy3Team ) );
				tTeamInfo*	base3Team = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy4Team ) );
				Assert( base1Team && base2Team && base3Team );
				if ( ( !base1Team ) || ( !base2Team ) || ( !base3Team ) )
					return;

				tShipIterator	base1 = base1Team->mGetFirstShip();
				tShipIterator	base2 = base2Team->mGetFirstShip();
				tShipIterator	base3 = base3Team->mGetFirstShip();
				if ( ( *base1 == NULL ) || ( *base2 == NULL ) || ( *base3 == NULL ) )
					return;				
				int32	base1X, base1Y, base2X, base2Y, base3X, base3Y;
				base1->mGetGridPosition( &base1X, &base1Y );
				base2->mGetGridPosition( &base2X, &base2Y );
				base3->mGetGridPosition( &base3X, &base3Y );

				if ( !gBaseVictory->mBase1PrisonersRescued() && gBaseVictory->mBase1Captured() )
				{
					fChildActor->mSetFinalDestination( base1X, base1Y );
					fChildActor->mTrackTacticalDistance( 5.0f, base1X, base1Y );
					fChildActor->mSetSaving1( true );
				}
				else if ( !gBaseVictory->mBase2PrisonersRescued() && gBaseVictory->mBase2Captured() )
				{
					fChildActor->mSetFinalDestination( base2X, base2Y );
					fChildActor->mTrackTacticalDistance( 5.0f, base2X, base2Y );
					fChildActor->mSetSaving2( true );
				}
				else if ( !gBaseVictory->mBase3PrisonersRescued() && gBaseVictory->mBase3Captured() )
				{
					fChildActor->mSetFinalDestination( base3X, base3Y );
					fChildActor->mTrackTacticalDistance( 5.0f, base3X, base3Y );
					fChildActor->mSetSaving3( true );
				}
			}
			break;
		}
	}
}
void tTransportBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tTransportBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tTransportBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tTransportBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tTransportBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tTransportBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tTransportBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportBaseState::mOnFlashingButtonPressed()
{}
void tTransportBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tTransportBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tTransportBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tTransportBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tTransportBaseState::mOnHostChanged()
{}
void tTransportBaseState::mOnTimedInterval( int32 timerID )
{
	if (timerID == kUpgradeFightersID) {
#ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
        if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tTransportBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
        int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
        gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tTransportBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}
void tTransportBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tTransportBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


