/********************************************************************************************************\

	File Name:	PlayerTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerTeamBaseState.h"
#include "MissionText.h"
#include "Uni_7Gates.h"
#include "EnemyTeam1.h"
#include "EnemyTeam2.h"

tPlayerTeamBaseState::tPlayerTeamBaseState()
{
	fTeamReadyToBeamUp = false;
}

void tPlayerTeamBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kLog_timer, 0.1f );
	mSetTimedInterval( kTimerTurnsSet, kSitrep_timer, 0.3f );
}

void tPlayerTeamBaseState::mEndState()
{}

void tPlayerTeamBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kLog_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCaptainsLog_msg, kNonCommScriptMessage );
		}
		break;

	case kSitrep_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kSitrep_msg );
		}
		break;

	case kLandingParty_timer:
		{
			fTeamReadyToBeamUp = true;
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTeamReadyToBeamUp_msg );
		}
		break;

	default:
		break;
	}
}

void tPlayerTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	if( ( !shipTo ) || ( !shipFrom ) )
		return;

	if( item != kTransScientists )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;


	// Beam team to planet
	if( ( shipFrom->mGetTeam() == kPlayerTeam ) && ( shipTo->mGetTeam() == kPlanetTeam ) && ( !playerTeam->mBeamedTeamToPlanet() ) )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTeamBeamedToPlanet_msg );
		mSetTimedInterval( kTimerTurnsSet, kLandingParty_timer, 1.5f );
	}

	// Beam team to somewhere else. Player loses.
	if( ( shipFrom->mGetTeam() == kPlayerTeam ) && ( shipTo->mGetTeam() != kPlanetTeam ) && ( !playerTeam->mBeamedTeamToPlanet() ) )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBeamedTeamNowhere_msg );
		gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam), kPlayerLoses );
	}

	// Beam team home if they signal ready
	if( ( shipFrom->mGetTeam() == kPlanetTeam ) && ( shipTo->mGetTeam() == kPlayerTeam ) && ( fTeamReadyToBeamUp ) )
	{
		fChildActor->mSetCurrentCommState( 3 );
		fChildActor->mSetHasInformation();

		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBeamedTeamHomeReady_msg );

		tEnemyTeam1* enemyTeam1 = dynamic_cast<tEnemyTeam1*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam1 ) ) );
		if( !enemyTeam1 )
				return;

		tEnemyTeam2* enemyTeam2 = dynamic_cast<tEnemyTeam2*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kEnemyTeam2 ) ) );
		if( !enemyTeam2 )
				return;

		enemyTeam1->mUpdateCommStatus( 3 );
		enemyTeam2->mUpdateCommStatus( 3 );
	}

	// Beam team home if they have not signaled ready
	if( ( shipFrom->mGetTeam() == kPlanetTeam ) && ( shipTo->mGetTeam() == kPlayerTeam) && ( ! fTeamReadyToBeamUp ) )
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kBeamedTeamHomeNotReady_msg );
		gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam), kPlayerLoses );
	}
}

void tPlayerTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	if( !targetShip )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	switch( buttonID )
	{
	case kButton_Hail:
		{
			// Hail enemy
			if( ( targetShip->mGetTeam() == kEnemyTeam1 ) || ( targetShip->mGetTeam() == kEnemyTeam2 ) )
			{
				// Have not noticed respawn
				if( !playerTeam->mNoticedRespawn() ) 
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kHail_NoScanOrSpawn );
					return;
				}

				// Have noticed respawn but doesn't have the info
				if( ( playerTeam->mNoticedRespawn() ) && ( !playerTeam->mHasInformation() ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kHail_SpawnNoInfo );
					return;
				}

				// Has the info
				if(  playerTeam->mHasInformation() )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kHail_Info );
					return;
				}
			}
		}
		break;

	case kButton_WeSurrender:
		{
			fChildActor->mSetSurrendered();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kHail_WeSurrender );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
		}
		break;

	default:
		break;
	}
}

void tPlayerTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mAtDestination( tShipInfo* ship )
{}


void tPlayerTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tPlayerTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tPlayerTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerTeamBaseState::mOnFlashingButtonPressed()
{}

void tPlayerTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerTeamBaseState::mOnHostChanged()
{}

void tPlayerTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}



