/********************************************************************************************************\

	Mission:	"The Gates of Hell"
	Script:		"Uni_7Gates"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		
	Last Updated:	FS				10/19/2012
					era-correct ships, drones and fighters.

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_7Gates.h"
#include "PlayerTeam.h"
#include "EnemyTeam1.h"
#include "EnemyTeam2.h"
#include "PlanetTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_7Gates );

tUni_7Gates::tUni_7Gates()
{}

int32 tUni_7Gates::mGetEnemyTeamID( void ) 
{
	return kTeam1;
}

void tUni_7Gates::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag, 
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team 1",
												static_cast< eTeamID >( kEnemyTeam1 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tEnemyTeam1 ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team 2",
												static_cast< eTeamID >( kEnemyTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecRomulan,
												kAnyRace,
												typeid( tEnemyTeam2 ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran | kSpecMirak, // kSpecAnyRace,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

int32 tUni_7Gates::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_7Gates::mCreateExternalShips( int32 Mode )
{}

int32 tUni_7Gates::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_7Gates::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_7Gates::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_7Gates::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_7Gates::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam1,
		kEnemyTeam2,
		kPlanetTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Nme1  = kEnemyTeam1,
		//Nme2  = kEnemyTeam2,
		//Plan = kPlanetTeam,

		//	     Plyr  Nme1  Nme2   Plan
		/*Plyr*/ "--", "HI", "HI", "AI",
		/*Nme1*/ "HI", "--", "WD", "NI",
		/*Nme2*/ "HI", "WD", "--", "NI",
		/*Plan*/ "AI", "NI", "NI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tUni_7Gates::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlanet;
}

tMapTerrain tUni_7Gates::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_7Gates::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tUni_7Gates::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_msg;
}