/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "AllyShip.h"
#include "BaseVictoryState.h"
#include "EnemyTeam.h"
#include "Enemy2Team.h"
#include "Enemy3Team.h"
#include "MissionText.h"
#include "PlayerNPCTeam.h"
#include "PlayerTeam.h"
#include "Uni_8Hammer.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID TeamID )
{
	if ( fChildActor->mISCWave() < 2 )
	{
		if ( fChildActor->mPlanetCaptured() || fChildActor->mPlanetDestroyed() )
		{
			return kVictory;
		}
		else
		{
			return kDevastatingDefeat;
		}
	}
	else
	{
		if ( fChildActor->mPlanetCaptured() || fChildActor->mPlanetDestroyed() )
		{
			return kAstoundingVictory;
		}
		else
		{
			return kDraw;
		}
	}

	return kDefeat;
}

//
//	Ship events
//

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	tEnemy2Team* enemy2Team = static_cast< tEnemy2Team* >( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy2Team ) ) );
	Assert( enemy2Team );
	if ( !enemy2Team )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{
			fChildActor->mIncreaseAllyShipsDestroyed();

			if ( fChildActor->mAllyShipsDestroyed() == 1 )
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom1_msg );
			else
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom2_msg );
		}
		break;

	case tEnemyTeam::kPlanet:
		{
			fChildActor->mSetPlanetDestroyed( true );

			//
			//	Find out if the mission should end here.
			//
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent9_1_msg );
			if ( fChildActor->mISCWave() >= 2 )
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent9_2_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent9_3_msg );
			}
		}
		break;

	case tEnemy2Team::kISC1:
	case tEnemy2Team::kISC2:
	case tEnemy2Team::kISC3:
	case tEnemy2Team::kISC4:
	case tEnemy2Team::kISCCB:
		{
			fChildActor->mIncreaseISCShipsDestroyed();
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom3_msg );

			//
			//	Change the comm state of your allies if they are still around.
			//
			tTeamInfo*	playerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
			Assert( playerNPCTeam );
			if ( !playerNPCTeam )
				return;
			for ( tShipIterator tempShip = playerNPCTeam->mGetFirstShip();/*InPlay() ;*/ *tempShip != NULL ; ++tempShip )
			{
				( static_cast< tAllyShip* >( *tempShip ) )->mChangeCommState();
			}

			//
			//	If the ship that was killed is the last of the current wave, then increase the current
			//	wave and create another fleet if so needed.
			//
			if ( !( fChildActor->mISCShipsDestroyed() % 3 ) )
			{
				fChildActor->mIncreaseISCWave();
				if ( fChildActor->mISCWave() < 2 )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom8_msg );
					enemy2Team->mCreateWedgeFormation();					
				}
			}

			//
			//	Check if the game should end.
			//
			if ( fChildActor->mISCWave() >= 2 )
			{
				if ( fChildActor->mPlanetCaptured() || fChildActor->mPlanetDestroyed() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_2_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );					
				}
			}
		}
		break;

	case tEnemy3Team::kPlatform1:
	case tEnemy3Team::kPlatform2:
	case tEnemy3Team::kPlatform3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom5_msg );
		}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{}
		break;

	case tEnemyTeam::kPlanet:
		{
			fChildActor->mSetPlanetCaptured( true );

			//
			//	Change the team relations so that the enemy has surrendered.
			//
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kPlayerTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemy2Team ), static_cast< eTeamID >( kPlayerTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemy3Team ), static_cast< eTeamID >( kPlayerTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemyTeam ), static_cast< eTeamID >( kPlayerNPCTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemy2Team ), static_cast< eTeamID >( kPlayerNPCTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kEnemy3Team ), static_cast< eTeamID >( kPlayerNPCTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerTeam ), static_cast< eTeamID >( kEnemyTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerTeam ), static_cast< eTeamID >( kEnemy2Team ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerTeam ), static_cast< eTeamID >( kEnemy3Team ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerNPCTeam ), static_cast< eTeamID >( kEnemyTeam ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerNPCTeam ), static_cast< eTeamID >( kEnemy2Team ), kTeamFriendly, kGoalIgnore, 1.0 );
			fMissionInfo->mSetTeamRelations( static_cast< eTeamID >( kPlayerNPCTeam ), static_cast< eTeamID >( kEnemy3Team ), kTeamFriendly, kGoalIgnore, 1.0 );

			if ( fChildActor->mISCWave() < 2 )
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent10_1_msg );
			else
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom9_msg );
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
		}
		break;

	case tEnemy2Team::kISC1:
	case tEnemy2Team::kISC2:
	case tEnemy2Team::kISC3:
	case tEnemy2Team::kISC4:
	case tEnemy2Team::kISCCB:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom6_msg );
		}
		break;

	case tEnemy3Team::kPlatform1:
	case tEnemy3Team::kPlatform2:
	case tEnemy3Team::kPlatform3:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tEnemy2Team* enemy2Team = static_cast< tEnemy2Team* >( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy2Team ) ) );
	Assert( enemy2Team );
	if ( !enemy2Team )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{}
		break;

	case tEnemyTeam::kPlanet:
		{
			fChildActor->mIncreaseISCShipsDestroyed();
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom7_msg );

			//
			//	Change the comm state of your allies if they are still around.
			//
			tTeamInfo*	playerNPCTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
			Assert( playerNPCTeam );
			if ( !playerNPCTeam )
				return;
			tShipIterator	tempShip = playerNPCTeam->mGetFirstShip();
			tShipInPlayEval	x;
			tempShip.mAddEvaluator( &x );

			for ( ; *tempShip != NULL ; tempShip++ )
			{
				( static_cast< tAllyShip* >( *tempShip ) )->mChangeCommState();
			}

			//
			//	If the ship that was killed is the last of the current wave, then increase the current
			//	wave and create another fleet if so needed.
			//
			if ( !( fChildActor->mISCShipsDestroyed() % 3 ) )
			{
				fChildActor->mIncreaseISCWave();
				if ( fChildActor->mISCWave() < 2 )
				{
					enemy2Team->mCreateWedgeFormation();
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom8_msg );
				}
			}

			//
			//	Check if the game should end.
			//
			if ( fChildActor->mISCWave() >= 2 )
			{
				if ( fChildActor->mPlanetCaptured() || fChildActor->mPlanetDestroyed() )
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_2_msg );
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
				}
				else
				{
					fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent5_1_msg );					
				}
			}
		}
		break;

	case tEnemy2Team::kISC1:
	case tEnemy2Team::kISC2:
	case tEnemy2Team::kISC3:
	case tEnemy2Team::kISC4:
	case tEnemy2Team::kISCCB:
		{}
		break;

	case tEnemy3Team::kPlatform1:
	case tEnemy3Team::kPlatform2:
	case tEnemy3Team::kPlatform3:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent12_1_msg );
			if ( fChildActor->mPlanetCaptured() || fChildActor->mPlanetDestroyed() )
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
			else
			{
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if ( !ship )
		return;
	tEnemy2Team* enemy2Team = static_cast< tEnemy2Team* >( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemy2Team ) ) );
	Assert( enemy2Team );
	if ( !enemy2Team )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{
			if ( !fChildActor->mWarningApproach() )
			{
				fChildActor->mSetWarningApproach( true );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent8_1_msg );

				//
				//	Create the first echelon of enemy cruisers.
				//
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
				enemy2Team->mCreateWedgeFormation();

				//
				//	Tell your allied ships to stop following the player.
				//
				tTeamInfo*	allyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
				Assert( allyTeam );
				if ( !allyTeam )
					return;
				tShipIterator ally1Iterator = allyTeam->mGetFirstShip();
				tShipIterator ally2Iterator = allyTeam->mGetFirstShip();
				tShipInPlayEval x, y;
				tShipCustomIDEval p( tPlayerNPCTeam::kAllyShip1 ), q( tPlayerNPCTeam::kAllyShip2 );
				ally1Iterator.mAddEvaluator( &x ), ally1Iterator.mAddEvaluator( &y );
				ally2Iterator.mAddEvaluator( &p ), ally2Iterator.mAddEvaluator( &q );
				tShipInfo*	ally1 = *ally1Iterator;
				tShipInfo*	ally2 = *ally2Iterator;
				if ( ally1 )
					ally1->mClearCommands();
				if ( ally2 )
					ally2->mClearCommands();
			}
		}
		break;

	case tEnemyTeam::kPlanet:
		{}
		break;

	case tEnemy2Team::kISC1:
	case tEnemy2Team::kISC2:
	case tEnemy2Team::kISC3:
	case tEnemy2Team::kISC4:
	case tEnemy2Team::kISCCB:
		{}
		break;

	case tEnemy3Team::kPlatform1:
	case tEnemy3Team::kPlatform2:
	case tEnemy3Team::kPlatform3:
		{}
		break;

	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !fChildActor->mWarningApproach() )
			{
				fChildActor->mSetWarningApproach( true );
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvent8_1_msg );

				//
				//	Create the first echelon of enemy cruisers.
				//
				fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCustom4_msg );
				enemy2Team->mCreateWedgeFormation();

				//
				//	Tell your allied ships to stop following the player.
				//
				tTeamInfo*	allyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerNPCTeam ) );
				Assert( allyTeam );
				if ( !allyTeam )
					return;
				tShipIterator ally1Iterator = allyTeam->mGetFirstShip();
				tShipIterator ally2Iterator = allyTeam->mGetFirstShip();
				tShipInPlayEval x, y;
				tShipCustomIDEval p( tPlayerNPCTeam::kAllyShip1 ), q( tPlayerNPCTeam::kAllyShip2 );
				ally1Iterator.mAddEvaluator( &x ), ally1Iterator.mAddEvaluator( &y );
				ally2Iterator.mAddEvaluator( &p ), ally2Iterator.mAddEvaluator( &q );
				tShipInfo*	ally1 = *ally1Iterator;
				tShipInfo*	ally2 = *ally2Iterator;
				if ( ally1 )
					ally1->mClearCommands();
				if ( ally2 )
					ally2->mClearCommands();
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )
	if ( timerID == tBaseVictory::kRandChatID )
	{
		switch ( fMissionInfo->mRandomInt32( 0, 5 ) )
		{
		case 0:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat1_msg );	break;
		case 1:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat2_msg );	break;
		case 2:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat3_msg );	break;
		case 3:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat4_msg );	break;
		case 4:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat5_msg );	break;
		case 5:
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kRandChat6_msg );	break;
		}

		fChildActor->mSetTimedInterval(	kTimerSecondsSet, timerID, ( float )( fMissionInfo->mRandomInt32( 20, 30 ) ) );
	}
#endif
}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}


/*

	switch ( ship->mGetCustomID() )
	{
	case tPlayerNPCTeam::kAllyShip1:
	case tPlayerNPCTeam::kAllyShip2:
		{}
		break;

	case tEnemyTeam::kPlanet:
		{}
		break;

	case tEnemy2Team::kISC1:
	case tEnemy2Team::kISC2:
	case tEnemy2Team::kISC3:
	case tEnemy2Team::kISC4:
	case tEnemy2Team::kISCCB:
		{}
		break;

	case tEnemy3Team::kPlatform1:
	case tEnemy3Team::kPlatform2:
	case tEnemy3Team::kPlatform3:
		{}
		break;
	}

	if ( ship->mGetTeam() == ( gScriptInterface.mPlayersTeam() )->mGetTeam() )
	{
		if ( ( gScriptInterface.mPlayersTeam() )->mGetFirstShipInPlay() == NULL )
			gScriptInterface.mSetGameStatus( kPlayerDestroyed );
	}

*/