/********************************************************************************************************\

	File Name:	Enemy2Team.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Enemy2Team.h"
#include "Enemy2TeamBaseState.h"
#include "ISCCruiser.h"
#include "Uni_8Hammer.h"
#include "ShipList_API.h"


tClassAllocator<tEnemy2Team> gEnemy2TeamAllocator;

tEnemy2Team::tEnemy2Team()
{
	wave = 0;
}
/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemy2Team::mSetupState()
{
	mRegisterState( new tEnemy2TeamBaseState() );
	mGotoState( typeid( tEnemy2TeamBaseState ) );
}

void tEnemy2Team::mSetupCommState()
{}

void tEnemy2Team::mCreateShipsForTeam()
{}

void tEnemy2Team::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemy2Team::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemy2Team", shipScriptDescription );
}


/*

	Purpose:	Creates an echelon of ships in a wedge formation.  These ships are the ISC ships that
				are responding to the attacks on their planet.

*/

void tEnemy2Team::mCreateWedgeFormation()
{
	gST.mDebug_fprintf("--- Entering: tEnemy2Team::mCreateWedgeFormation\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		if ( wave == 0 )
		{
			mCreateWaveShip( "I-CC", kEClassHeavyCruiser, kEClassHeavyBattlecruiser, kStartPosition_R, kISCCB, true );
			mCreateWaveShip( "I-CM", kEClassLightCruiser, kEClassHeavyWarDestroyer, kStartPosition_S, kISC1, false );
			mCreateWaveShip( "I-CSZ", kEClassLightCruiser, kEClassHeavyWarDestroyer, kStartPosition_T, kISC2, false );

			++wave;
		} else {
			mCreateWaveShip( "I-CA", kEClassHeavyCruiser, kEClassNewHeavyCruiser, kStartPosition_U, kISC3, false );
			mCreateWaveShip( "I-DDG", kEClassDestroyer, kEClassWarDestroyer, kStartPosition_V, kISC4, false );
			mCreateWaveShip( "I-DN", kEClassDreadnought, kEClassHeavyDreadnought, kStartPosition_R, kISCCB, true );
		}
	} else {
		mOldCreateWedgeFormation();
	}
	gST.mDebug_fprintf("--- Finished: tEnemy2Team::mCreateWedgeFormation\n");
}

void tEnemy2Team::mOldCreateWedgeFormation()
{
	gST.mDebug_fprintf("--- Starting: tEnemy2Team::mOldCreateWedgeFormation\n");

	if( wave == 0 )
	{
		mCreateShip(	typeid( tISCCruiser ), "I-CC", kStartPosition_R, kNoMetaShipID,
						kISCCB, 0, 0, -1, -1, NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tISCCruiser ), "I-CM", kStartPosition_S, kNoMetaShipID,
						kISC1, 0, 0, -1, -1, NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tISCCruiser ), "I-CSZ", kStartPosition_T, kNoMetaShipID,
						kISC2, 0, 0, -1, -1, NULL, kDefaultShipOptions );
		++wave;
	}
	else
	{
		mCreateShip(	typeid( tISCCruiser ), "I-CA", kStartPosition_U, kNoMetaShipID,
						kISC3, 0, 0, -1, -1, NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tISCCruiser ), "I-DDG", kStartPosition_V, kNoMetaShipID,
						kISC4, 0, 0, -1, -1, NULL, kDefaultShipOptions );
		mCreateShip(	typeid( tISCCruiser ), "I-DN", kStartPosition_R, kNoMetaShipID,
						kISCCB, 0, 0, -1, -1, NULL, kDefaultShipOptions );
	}

	gST.mDebug_fprintf("--- Finished: tEnemy2Team::mOldCreateWedgeFormation\n");
}

void tEnemy2Team::mCreateWaveShip( char* oldCodeShipType, int32 startEnhClass, int32 endEnhClass, int32 shipPosition, int32 customID, bool isLeader )
{
	gST.mDebug_fprintf("---- Entering: tEnemy2Team::mCreateWaveShip\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShipRoles( sl );

        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = startEnhClass; i <= endEnhClass; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for hull classes filter completed. ships: %d\n", fShiplistCount);
        }

		// isLeader used here:
		{
        std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = sl.Subset(leaderRolesList, isLeader);
        gST.mDebug_fprintf("subset for Leader eEnhancedRoles completed. ships: %d\n", fShiplistCount);
		}
		gST.listShiplistContent(sl);

		if (sl.size() > 0) {
            Ship s1;
            sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			mCreateShip(
				typeid( tISCCruiser ), 
				s1.shipCode.c_str(), 
				shipPosition, 
				kNoMetaShipID,
				customID, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );
		} else {
			mCreateShip(
				typeid( tISCCruiser ), 
				oldCodeShipType, 
				shipPosition, 
				kNoMetaShipID,
				customID, 
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );
		}
	} else {
		mCreateShip(
			typeid( tISCCruiser ), 
			oldCodeShipType, 
			shipPosition, 
			kNoMetaShipID,
			customID, 
			0, 0, -1, -1, 
			NULL, 
			kDefaultShipOptions );
	}
	gST.mDebug_fprintf("---- Finished: tEnemy2Team::mCreateWaveShip\n");
}