/********************************************************************************************************\

	File Name:	Enemy3Team.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Enemy3Team.h"
#include "Enemy3TeamBaseState.h"
#include "ISCPlatform.h"
#include "ShipList_API.h"


tClassAllocator<tEnemy3Team> gEnemy3TeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemy3Team::mSetupState()
{
	mRegisterState( new tEnemy3TeamBaseState() );
	mGotoState( typeid( tEnemy3TeamBaseState ) );
}

void tEnemy3Team::mSetupCommState()
{}

void tEnemy3Team::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tEnemy3Team::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

        // Defense platforms
        ShipList DEF_sl = sl;
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);

		if (DEF_sl.size() > 0) {
            // Create DEF platforms
            Ship def0, def1, def2;
			switch (fMissionInfo->mGetDifficultyLevel())
			{
			case 2:
				{
				DEF_sl.SelectRandomShip(def2);
				gST.mDebug_fprintf("def2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					def2.shipCode.c_str(), def2.bpv, def2.yearFirstAvail, def2.yearLastAvail);
		        gST.mTrackOrions( def2 );

				mCreateShip(	
					typeid( tISCPlatform ), 
					def2.shipCode.c_str(), 
					kStartPosition_W, 
					kNoMetaShipID, 
					kPlatform3, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
				}
			case 1:
				{
				DEF_sl.SelectRandomShip(def1);
				gST.mDebug_fprintf("def1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					def1.shipCode.c_str(), def1.bpv, def1.yearFirstAvail, def1.yearLastAvail);
		        gST.mTrackOrions( def1 );

				mCreateShip(	
					typeid( tISCPlatform ), 
					def1.shipCode.c_str(), 
					kStartPosition_X, 
					kNoMetaShipID, 
					kPlatform2, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
				}
			case 0:
				{
				DEF_sl.SelectRandomShip(def0);
				gST.mDebug_fprintf("def0 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					def0.shipCode.c_str(), def0.bpv, def0.yearFirstAvail, def0.yearLastAvail);
		        gST.mTrackOrions( def0 );

				mCreateShip(	
					typeid( tISCPlatform ), 
					def0.shipCode.c_str(), 
					kStartPosition_Y, 
					kNoMetaShipID, 
					kPlatform1, 
					0, 0, -1, -1, 
					NULL, 
					kDefaultShipOptions );
				}
				break;
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemy3Team::mCreateShipsForTeam\n");
}

void tEnemy3Team::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tEnemy3Team::mOldCreateShipsForTeam\n");

	int32	diffLevel = fMissionInfo->mGetDifficultyLevel();

	if ( diffLevel >= 0 )
		mCreateShip(	typeid( tISCPlatform ), "I-DEF", kStartPosition_Y, kNoMetaShipID, 
						kPlatform1, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	if ( diffLevel >= 1 )
		mCreateShip(	typeid( tISCPlatform ), "I-DEF", kStartPosition_X, kNoMetaShipID,
						kPlatform2, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	if ( diffLevel >= 2 )
		mCreateShip(	typeid( tISCPlatform ), "I-DEF", kStartPosition_W, kNoMetaShipID,
						kPlatform3, 0, 0, -1, -1, NULL, kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tEnemy3Team::mOldCreateShipsForTeam\n");
}


void tEnemy3Team::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemy3Team::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemy3Team", shipScriptDescription );
}
