/********************************************************************************************************\

	File Name:	PlayerNPCTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AllyShip.h"
#include "PlayerNPCTeam.h"
#include "PlayerNPCTeamBaseState.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPCTeam> gPlayerNPCTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPCTeam::mSetupState()
{
	mRegisterState( new tPlayerNPCTeamBaseState() );
	mGotoState( typeid( tPlayerNPCTeamBaseState ) );
}

void tPlayerNPCTeam::mSetupCommState()
{}

void tPlayerNPCTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPCTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShipRoles( sl );

		ShipList DN_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassDreadnought);
		shipClassTypeList.push_back(kEClassHeavyDreadnought);
        fShiplistCount = DN_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DN/DNH classes filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent( DN_sl );
        }

		ShipList BB_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassBattleship);
        fShiplistCount = BB_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for BB classes filter completed. ships: %d\n", fShiplistCount);
		gST.listShiplistContent( BB_sl );
        }

		if (BB_sl.size() == 0) {
			gST.mDebug_fprintf("No BBs to select with. Going with DNs for the BB\n");
			BB_sl = DN_sl;
		}

        if ((DN_sl.size() > 0) && (BB_sl.size() > 0))
        {
            Ship s1, s2;
            DN_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

            BB_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

            mCreateShip(
                typeid( tAllyShip ),
                s1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kAllyShip1,
                0, 0, -1, -1,
                NULL,
                static_cast<eShipOptions>( kCanBeCarrier | kStartPositionCanBeOffset ) );

            mCreateShip(
                typeid( tAllyShip ),
                s2.shipCode.c_str(),
                kStartPosition_I,
                kNoMetaShipID,
                kAllyShip2,
                0, 0, -1, -1,
                NULL,
                static_cast<eShipOptions>( kCanBeCarrier | kStartPositionCanBeOffset ) );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }
    gST.mDebug_fprintf("--- Finished: tPlayerNPCTeam::mCreateShipsForTeam\n");
}


void tPlayerNPCTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tPlayerNPCTeam::mOldCreateShipsForTeam\n");

	mCreateShip(	
		typeid( tAllyShip ), 
		kMaxBPV, 
		kClassDreadnought, 
		100, 
		kStartPosition_H,
		kAllyShip1, 
		0, 0, -1, -1, 
		NULL, 
		static_cast<eShipOptions>( kCanBeCarrier | kStartPositionCanBeOffset ) );

	mCreateShip(
		typeid( tAllyShip ), 
		kMaxBPV, 
		kClassBattleship, 
		100, 
		kStartPosition_I,
		kAllyShip2, 
		0, 0, -1, -1, 
		NULL, 
		static_cast<eShipOptions>( kCanBeCarrier | kStartPositionCanBeOffset ) );

	gST.mDebug_fprintf("---- Finished: tPlayerNPCTeam::mOldCreateShipsForTeam\n");
}

void tPlayerNPCTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPCTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlayerNPCTeam", shipScriptDescription );
}

