/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "shipScriptDescription.h"
#include "Uni_8Hammer.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
	gBaseVictory = static_cast< tBaseVictory* >( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
/*

	HELPER NOTE:	The following is the analysis of the mission.  You are able to place additional text
					dependant upon the victory outcome in the MissionText.cpp file.  Simply find the
					corresponding values.

*/
	//
	//	Add the appropriate messages to the debriefing list according to the player's performance.
	//
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebAstoundingVictory_msg );
		break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebVictory_msg );
		break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDraw_msg );
		break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDefeat_msg );
		break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), victoryLevel + kAstoundingVictory_msg );
		fMissionInfo->mAddToDebriefingList( fTeam, kDebDevastatingDefeat_msg );
		break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( static_cast< eTeamID >( kPlayerTeam ), kDisconnect_msg );
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;

	switch( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
	case kDraw:
		fMissionScheduler.fNextMissionTitle = "Uni_9A1Tyr";
		break;
	default:
		fMissionScheduler.fNextMissionTitle = "Uni_9C1Welcome";
		break;
	}
}