/********************************************************************************************************\

	Mission:	Dropping The Hammer
	Script:		"Uni_8Hammer"
	Descript:	Divert the ISC by attacking an ISC planet.

	Created by:		Daniel J. Suleski	04/08/00
	Last Updated:	DJS					04/10/00
					First draft complete
					DJS					04/11/00
					Second draft complete
					DJS					05/15/00
					Initial conversion from old API to new multiplayer API
					DJS					05/20/00
					Asserts added and code cleaned
					DJS						05/26/00
					tShipIterator::mGetFirstShipInPlay() replaced with original form
					DJS						08/23/00
					Map overrides.
					DJS						09/25/00
					mCreateShip() "default-removal" modifications, kCampaignMission, tTeamSpec, etc
					DJS						09/26/00
					Meta script modifications
					FS						09/25/2012
					era-correct ships, drones and upgraded fighters


\********************************************************************************************************/

#include "Uni_8Hammer.h"
#include "BaseVictory.h"
#include "EnemyTeam.h"
#include "Enemy2Team.h"
#include "Enemy3Team.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "NPC1Team.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "SFCAssert.h"
#include "VictoryCondition.h"
#include "ShipList_API.h"


DEFINE_SCRIPT_CLASS( tUni_8Hammer );

tBaseVictory* gBaseVictory;

tUni_8Hammer::tUni_8Hammer()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tUni_8Hammer::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team",
													static_cast< eTeamID >( kPlayerTeam ),
													kMaxChance,
													kTrue,
													kPrimaryTeam,
													kMinChance,
													kSpecPlayableRace & ~kSpecISC,
													kAnyRace,
													typeid( tPlayerTeam ),
													kNoTeamTag,
													Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& playerNPCTeam = mCreateTeamSpec(	"Player NPC Team",
														static_cast< eTeamID >( kPlayerNPCTeam ),
														kMaxChance,
														kTrue,
														kPlayableTeam,
														kMaxChance,
														kAnyRace,
														kSameAs | kPlayerTeam,
														typeid( tPlayerNPCTeam ) );

	const tTeamSpec& enemyTeam = mCreateTeamSpec(	"Enemy Team",
													static_cast< eTeamID >( kEnemyTeam ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecISC,
													kAnyRace,
													typeid( tEnemyTeam ) );

	const tTeamSpec& enemy2Team = mCreateTeamSpec(	"Enemy 2 Team",
													static_cast< eTeamID >( kEnemy2Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecISC,
													kAnyRace,
													typeid( tEnemy2Team ) );

	const tTeamSpec& enemy3Team = mCreateTeamSpec(	"Enemy 3 Team",
													static_cast< eTeamID >( kEnemy3Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecISC,
													kAnyRace,
													typeid( tEnemy3Team ) );

	const tTeamSpec& npc1Team = mCreateTeamSpec(	"NPC 1 Team",
													static_cast< eTeamID >( kNPC1Team ),
													kMaxChance,
													kTrue,
													kPlayableTeam,
													kMaxChance,
													kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC,
													kAnyRace,
													typeid( tNPC1Team ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( playerNPCTeam );
	Specs->mSetupTeam( enemyTeam );
	Specs->mSetupTeam( enemy2Team );
	Specs->mSetupTeam( enemy3Team );
	Specs->mSetupTeam( npc1Team );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tUni_8Hammer::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tUni_8Hammer::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tUni_8Hammer::mCreateExternalShips( int32 Mode )
{}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tUni_8Hammer::mInitializeFinished( void )
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	Assert( playerTeam );
	if ( !playerTeam )
		return kBriefing_msg;
	eRaceName	playerRace = playerTeam->mGetRace();

	switch ( playerRace )
	{
	case kFederation:
	case kRomulan:
	case kHydran:
		return kBriefingFederationRomulanHydran_msg;
	case kKlingon:
	case kLyran:
		return kBriefingKlingonLyran_msg;
	case kGorn:
	case kMirak:
		return kBriefingGornMirak_msg;
	}

	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tUni_8Hammer::mStartMission( void )
{
	tScript::mStartMission();

/*

	HELPER NOTE:	The teamList array that you see below should reflect those constants that have been or you
					have declared in the matching header file (e.g. Uni_8Hammer.h).

*/
	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kEnemy2Team,
		kEnemy3Team,
		kNPC1Team,
	};

/*

	HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
					meaning has a meaning???

	Relationship Lookup
	- = kTeamSelfAware	//	use this parameter with itself
	A = kTeamAllied		//	cannot fire upon an ally
	F = kTeamFriendly
	N = kTeamNeutral
	H = kTeamHostile	//	will not fire / capture / cripple until provoked
	W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
	P = kTeamPassive

	Relationship Lookup
	- = kGoalIgnore		//	use this parameter with itself
	I = kGoalIgnore
	C = kGoalCripple
	X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
	D = kGoalDestroy

*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = PlayersTeam,
		//PNPC = PlayerNPCTeam,
		//Nme  = EnemyTeam,
		//Nme2 = Enemy2Team,
		//Nme3 = Enemy3Team,
		//NPC1 = NPC1

		//	     Plyr  PNPC  Nme   Nme2  Nme3  NPC1
		/*Plyr*/ "--", "AI", "WD", "WD", "WD", "FI",
		/*PNPC*/ "AI", "--", "WD", "WD", "WD", "FI",
		/*Nme */ "WD", "WD", "--", "AI", "AI", "FI",
		/*Nme2*/ "WD", "WD", "AI", "--", "AI", "FI",
		/*Nme3*/ "WD", "WD", "AI", "AI", "--", "FI",
		/*NPC1*/ "FI", "FI", "FI", "FI", "FI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tUni_8Hammer::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_8Hammer::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['Z'] = kObjectPlanet;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_8Hammer::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederationRomulanHydran_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingonLyran_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] = kBriefingFederationRomulanHydran_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefingKlingonLyran_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kBriefingGornMirak_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefingFederationRomulanHydran_msg;
	fBriefingIndex[kPlayerTeam][kMirak] = kBriefingGornMirak_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_8Hammer::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}