/********************************************************************************************************\

	File Name:	BaseTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseTeam.h"
#include "BaseTeamBaseState.h"
#include "Uni_8Massacre.h"
#include "BaseShip.h"
#include "Planet.h"
#include "ShipList_API.h"

tClassAllocator<tBaseTeam> gBaseTeamAllocator;

void tBaseTeam::mSetupState()
{
	mRegisterState( new tBaseTeamBaseState() );
	mGotoState( typeid( tBaseTeamBaseState ) );
}

void tBaseTeam::mSetupCommState()
{}

void tBaseTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tBaseTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.stripOutRestricted( gST.narrowed_sl );
        sl = gST.mFilterShipListToSingleRace( sl, race );
        sl = gST.stripOutSPECIAL( sl );

		{
        std::list< eEnhancedClassTypes >  BSClassTypeList;
        BSClassTypeList.push_back(kEClassBaseStation);
        fShiplistCount = sl.Subset(BSClassTypeList, true);
        gST.mDebug_fprintf("subset for BS filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(sl);
		}

        ShipList planet_sl = gST.narrowed_sl;
        std::string worldType = "PL13";
        switch (race) {
            case kFederation: worldType = "F-PL13"; break;
            case kKlingon: worldType = "K-PL13"; break;
            case kRomulan: worldType = "R-PL13"; break;
            case kGorn: worldType = "G-PL13"; break;
            case kLyran: worldType = "L-PL13"; break;
            case kHydran: worldType = "H-PL13"; break;
            case kISC: worldType = "I-PL13"; break;
            case kMirak: worldType = "Z-PL13"; break;

            default: break;
        }

		Ship p1;

        if ((sl.size() > 0) && (planet_sl.Find(worldType, p1)))
        {
            Ship s1;
            sl.SelectRandomShip(s1);
            gST.mTrackOrions( s1 );
            gST.mDebug_fprintf("BATS %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

            mCreateShip( typeid( tBaseShip ),
                         s1.shipCode.c_str(),
                         kStartPosition_I,
                         kNoMetaShipID,
                         kBase1,
                         0,0,0,0,
                         "Medical Outpost",
                         kDefaultShipOptions );

            gST.mDebug_fprintf("Planet chosen: %s\n", p1.shipCode.c_str());
            mCreateShip(
                typeid( tPlanet ),
                p1.shipCode.c_str(),
                kStartPosition_K,
                kNoMetaShipID,
                kPlanet,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tBaseTeam::mCreateShipsForTeam\n");
}

void tBaseTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tBaseTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
		return;

	char *szShipString;
	switch( playerTeam->mGetRace() )
	{
	case kLyran:
		szShipString = "L-BS";
		break;

	case kKlingon:
		szShipString = "K-BS";
		break;

	case kHydran:
		szShipString = "H-BS";
		break;

	// Should never happen
	default:
		szShipString = "F-BS";
		break;
	}

	mCreateShip( typeid( tBaseShip ),
				 szShipString,
				 kStartPosition_I,
				 kNoMetaShipID,
				 kBase1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid ( tPlanet ),
				 "PL13",
				 kStartPosition_K,
				 kNoMetaShipID,
				 kPlanet,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tBaseTeam::mOldCreateShipsForTeam\n");
}

void tBaseTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tBaseTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tBaseTeam", shipScriptDescription );
}

