/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_8Massacre.h"
#include "PlayerTeam.h"
#include "MirakTeam.h"
#include "BaseTeam.h"
#include "FriendlyTeam.h"
#include "FreighterTeam.h"
#include "InvasionTeam.h"


tBaseVictoryState::tBaseVictoryState()
{
	fWinEarly = false;
}

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}

void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kCaptainsLog_timer, 0.2f );
	mSetTimedInterval( kTimerTurnsSet, kInitialMessage_timer, 0.4f );
	mSetTimedInterval( kTimerTurnsSet, kEnemyReinforcements_timer, 9.0f );
	mSetTimedInterval( kTimerTurnsSet, kFriendlyReinforcements_timer, 10.0f );
	mSetTimedInterval( kTimerTurnsSet, kLaunchTransport_timer, 11.0f );
}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	// Win if the enemy team was crunched before the base launched a transport
	if( playerTeam->mKilledEnemyBeforeLaunch() )
	{
		return kAstoundingVictory;
	}

	// Win if the transport escapes and the base is saved
	else if( ( playerTeam->mTransportEscaped() ) && ( !playerTeam->mBaseDestroyed() ) )
	{
		return kAstoundingVictory;
	}

	// Win if the transport was escaping (BUT NOT DESTROYED) when all enemy were destroyed. base was saved.
	else if( ( !playerTeam->mTransportEscaped() ) && ( !playerTeam->mTransportDestroyed() ) && ( !playerTeam->mBaseDestroyed() ) && ( playerTeam->mKilledAllEnemies() ) )
	{
		return kAstoundingVictory;
	}

	// Fail mission if transport is destroyed/capped or base is destroyed/capped
	else if( ( playerTeam->mTransportDestroyed() ) || ( playerTeam->mBaseDestroyed() ) ) 
	{
		return kDevastatingDefeat;
	}

	// No other combination of conditions should be reached
	return kUnknownVictoryLevel;
}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;


	switch( timerID )
	{
		case kCaptainsLog_timer:
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCaptainsLog_msg, kNonCommScriptMessage );
			}
			break;

		case kInitialMessage_timer:
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInitialSitrep_msg );
			}
			break;

		case kEnemyReinforcements_timer:
			{
				//if( !fWinEarly )
				//{
					
					tInvasionTeam* invasionTeam = dynamic_cast<tInvasionTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kInvasionTeam ) ) );
					if( !invasionTeam )
						return;
					
					tMirakTeam* mirakTeam = dynamic_cast<tMirakTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam ) ) );
					if( !mirakTeam )
						return;
					
					mirakTeam->mCreateReinforcements();
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kEnemyReinforcements_msg );
				//}
			}
			break;

		case kFriendlyReinforcements_timer:
			{
				//if( !fWinEarly )
				//{
					tFriendlyTeam* friendlyTeam = dynamic_cast<tFriendlyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFriendlyTeam ) ) );
					if( !friendlyTeam )
						return;
					
					friendlyTeam->mCreateReinforcements();
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFriendlyReinforcements_msg );
					if( playerTeam->mGetRace() == kKlingon )
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kKlingonForcesArrive_msg );
					}
				//}
			}
			break;

		case kLaunchTransport_timer:
			{
				tFreighterTeam* freighterTeam = dynamic_cast<tFreighterTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) ) );
				if( !freighterTeam )
					return;

				tTeamInfo* mirakTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam ) );
				if( !mirakTeam )
					return;

				freighterTeam->mCreateFreighter();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTransportLaunched_msg );
				playerTeam->mSetTransportLaunched();

				tShipIterator mships = mirakTeam->mGetFirstShip();
				tShipInfo* currentShip;

				while( *mships )
				{
					currentShip = *mships;
					currentShip->mSetShipTarget( freighterTeam->mGetThisShip(1), kAbsolutePriority );
					mships++;
				}


			}
			break;
	}
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* mirakTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam ) );
	if( !mirakTeam )
			return;

	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	ASSERT( freighterTeam );
	if( !freighterTeam ) return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					playerTeam->mSetDied();
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

		// Lose here if the transport is destroyed
	case tFreighterTeam::kFreighter1:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFreighterDies_msg );
			playerTeam->mSetTransportDestroyed();
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		}
		break;

	case tBaseTeam::kBase1:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBaseDestroyed_msg );
			playerTeam->mSetBaseDestroyed();

			// Lose here if the transport hasn't launched yet
			if( !playerTeam->mTransportLaunched() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFreighterDies_msg );
				playerTeam->mSetTransportDestroyed();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}

		}
		break;

	case tMirakTeam::kMirak1:
	case tMirakTeam::kMirak2:
	case tMirakTeam::kMirak3:
	case tMirakTeam::kMirak4:
	case tMirakTeam::kMirak5:
	case tMirakTeam::kMirak6:	
	case tMirakTeam::kMirak7:
	case tMirakTeam::kMirak8:
		{
			if( !mirakTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInvadersStopped_msg );

				if( !playerTeam->mTransportLaunched() )
				{
					playerTeam->mSetKilledEnemyBeforeLaunch();
					fWinEarly = true;
					mSetTimedInterval( kTimerKillTimer, kEnemyReinforcements_timer,		0.0f );
					mSetTimedInterval( kTimerKillTimer, kFriendlyReinforcements_timer,  0.0f );
				}
				
				else if( ( !playerTeam->mTransportDestroyed() ) && ( ! playerTeam->mTransportEscaped() ) )
				{
					playerTeam->mSetTransportEscaped();
				}
				
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				playerTeam->mSetKilledAllEnemies();
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* mirakTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam ) );
	if( !mirakTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

		// Transport captured. Oops.
	case tFreighterTeam::kFreighter1:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFreighterCapped_msg );
			playerTeam->mSetTransportDestroyed();
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		}
		break;

	case tBaseTeam::kBase1:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBaseCapped_msg );
			playerTeam->mSetBaseCaptured();

			// Lose here if the transport hasn't launched yet
			if( !playerTeam->mTransportLaunched() )
			{
				playerTeam->mSetTransportDestroyed();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}
		}
		break;

	case tMirakTeam::kMirak1:
	case tMirakTeam::kMirak2:
	case tMirakTeam::kMirak3:
	case tMirakTeam::kMirak4:
	case tMirakTeam::kMirak5:
	case tMirakTeam::kMirak6:
	case tMirakTeam::kMirak7:
	case tMirakTeam::kMirak8:	
		{
			if( !mirakTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInvadersStopped_msg );

				if( !playerTeam->mTransportLaunched() )
				{
					playerTeam->mSetKilledEnemyBeforeLaunch();
					fWinEarly = true;
					mSetTimedInterval( kTimerKillTimer, kEnemyReinforcements_timer,		0.0f );
					mSetTimedInterval( kTimerKillTimer, kFriendlyReinforcements_timer,  0.0f );
				}
				
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				playerTeam->mSetKilledAllEnemies();
			}
		}
		break;

	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* mirakTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam ) );
	if( !mirakTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetBaseDestroyed();
				playerTeam->mSetTransportDestroyed();
				playerTeam->mSetPlanetInvaded();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
			}
		}
		break;

		// Win if the transport escapes and if there are no enemies to kill
	case tFreighterTeam::kFreighter1:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFreighterEscapes_msg );
			playerTeam->mSetTransportEscaped();

			if( !mirakTeam->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tMirakTeam::kMirak1:
	case tMirakTeam::kMirak2:
	case tMirakTeam::kMirak3:
	case tMirakTeam::kMirak4:
	case tMirakTeam::kMirak5:
	case tMirakTeam::kMirak6:
	case tMirakTeam::kMirak7:
	case tMirakTeam::kMirak8:	
		{
			if( !mirakTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInvadersStopped_msg );

				if( !playerTeam->mTransportLaunched() )
				{
					fWinEarly = true;
					playerTeam->mSetKilledEnemyBeforeLaunch();
					mSetTimedInterval( kTimerKillTimer, kEnemyReinforcements_timer,		0.0f );
					mSetTimedInterval( kTimerKillTimer, kFriendlyReinforcements_timer,  0.0f );
				}
				
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				playerTeam->mSetKilledAllEnemies();
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

