/********************************************************************************************************\

	File Name:	FriendlyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "FriendlyTeam.h"
#include "FriendlyTeamBaseState.h"
#include "FriendlyShip.h"
#include "Uni_8Massacre.h"
#include "ShipList_API.h"


tClassAllocator<tFriendlyTeam> gFriendlyTeamAllocator;

void tFriendlyTeam::mSetupState()
{
	mRegisterState( new tFriendlyTeamBaseState() );
	mGotoState( typeid( tFriendlyTeamBaseState ) );
}

void tFriendlyTeam::mSetupCommState()
{}

void tFriendlyTeam::mCreateShipsForTeam()
{}

void tFriendlyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tFriendlyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tFriendlyTeam", shipScriptDescription );
}

void tFriendlyTeam::mCreateReinforcements()
{
	gST.mDebug_fprintf("--- Entering: tFriendlyTeam::mCreateReinforcements\n");

    gST.mInitializeSL();
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

		ShipList CA_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassMediumMonster);
        for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);
		}

		ShipList CC_sl = CA_sl;
        {
        std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = CC_sl.Subset(leaderRolesList);
        gST.mDebug_fprintf("subset for CC (CA leader) eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CC_sl);
        }
        if (CC_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a CC from shiplist. using CA shiplist\n");
            CC_sl = CA_sl;
        }

		if ( (CC_sl.size() > 0) && (CA_sl.size() > 0) ) 
		{
            Ship cc1, ca1, ca2;
            CC_sl.SelectRandomShip(cc1);
            gST.mDebug_fprintf("cc1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cc1.shipCode.c_str(), cc1.bpv, cc1.yearFirstAvail, cc1.yearLastAvail);
            gST.mTrackOrions( cc1 );

            CA_sl.SelectRandomShip(ca1);
            gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

            CA_sl.SelectRandomShip(ca2);
            gST.mDebug_fprintf("ca2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca2.shipCode.c_str(), ca2.bpv, ca2.yearFirstAvail, ca2.yearLastAvail);
            gST.mTrackOrions( ca2 );

            mCreateShip(
                typeid( tFriendlyShip ),
                cc1.shipCode.c_str(),
                kStartPosition_L,
                kNoMetaShipID,
                kFriendly1,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tFriendlyShip ),
                ca1.shipCode.c_str(),
                kStartPosition_L,
                kNoMetaShipID,
                kFriendly2,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tFriendlyShip ),
                ca2.shipCode.c_str(),
                kStartPosition_L,
                kNoMetaShipID,
                kFriendly3,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );
		} else {
			mOldCreateReinforcements();
		}
	} else {
		mOldCreateReinforcements();
	}

	gST.mDebug_fprintf("--- Finished: tFriendlyTeam::mCreateReinforcements\n");
}

void tFriendlyTeam::mOldCreateReinforcements()
{
	gST.mDebug_fprintf("--- Entering: tFriendlyTeam::mOldCreateReinforcements\n");

	if( mGetRace() != kKlingon )
	{
		mCreateShip( typeid( tFriendlyShip ),
				 180,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_L,
				 kFriendly1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

		mCreateShip( typeid( tFriendlyShip ),
				 170,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_L,
				 kFriendly2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );


		mCreateShip( typeid( tFriendlyShip ),
				 160,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_L,
				 kFriendly3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );
	}
	else
	{
		mCreateShip( typeid( tFriendlyShip ),
					 "K-D7W",
					 kStartPosition_L,
					 kNoMetaShipID,
					 kFriendly1,
					 0,0,0,0,
					 "IKV Death's Hand",
					 kDefaultShipOptions );

		mCreateShip( typeid( tFriendlyShip ),
					 "K-D7C",
					 kStartPosition_L,
					 kNoMetaShipID,
					 kFriendly1,
					 0,0,0,0,
					 "IKV Blue Fire",
					 kDefaultShipOptions );

		mCreateShip( typeid( tFriendlyShip ),
					 "K-D7C",
					 kStartPosition_L,
					 kNoMetaShipID,
					 kFriendly1,
					 0,0,0,0,
					 "IKV Two Fingers",
					 kDefaultShipOptions );
	}

	gST.mDebug_fprintf("--- Finished: tFriendlyTeam::mOldCreateReinforcements\n");
}