/********************************************************************************************************\

	File Name:	MirakTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MirakTeam.h"
#include "MirakTeamBaseState.h"
#include "MirakShip.h"
#include "Uni_8Massacre.h"
#include "ShipList_API.h"


tClassAllocator<tMirakTeam> gMirakTeamAllocator;

void tMirakTeam::mSetupState()
{
	mRegisterState( new tMirakTeamBaseState() );
	mGotoState( typeid( tMirakTeamBaseState ) );
}

void tMirakTeam::mSetupCommState()
{}

void tMirakTeam::mCreateShipsForTeam()
{
	mCreateShipsBasedOnPlayerTeam(kMirak1, kStartPosition_H);
}

void tMirakTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tMirakTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tMirakTeam", shipScriptDescription );
}


void tMirakTeam::mCreateReinforcements()
{
	mCreateShipsBasedOnPlayerTeam(kMirak5, kStartPosition_H);
}

void tMirakTeam::mCreateShipsBasedOnPlayerTeam(int32 startCustomID, int32 position)
{
    gST.mDebug_fprintf("--- Entering: tMirakTeam::mCreateShipsBasedOnPlayerTeam\n");
    gST.mInitializeSL();
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

		ShipList CL_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassMediumMonster);
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
		}

		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);
		}

        int32 nStartPos = position;
        int32 nCurrentID = startCustomID;
        int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
        int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
        float nVarPercent;
        if( diffLevel == 0 )
        {
                nVarPercent = 0.65f;
        }
        else if( diffLevel == 1 )
        {
                nVarPercent = 1.0f;
        }
        else if( diffLevel == 2 )
        {
                nVarPercent = 1.25f;
        }

		if (CL_sl.size() > 0) {
			for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
			{
				tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
				if( currentShip )
				{
					int32 thisShipBPV = int(currentShip->mGetShipBPV() * nVarPercent);

					// now we pick a ship with shiplist API. If it fails to pick one, use the old way.
					ShipList ship_sl = sl;
					fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), true);
					gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
						int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), fShiplistCount);
					gST.listShiplistContent(ship_sl);

					Ship s1;
					if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
						gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
							s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
						gST.mTrackOrions( s1 );

						mCreateShip(
							typeid( tMirakShip ),
							s1.shipCode.c_str(),
							nStartPos,
							kNoMetaShipID,
							nCurrentID,
							0, 0, 0, 0,
							NULL,
							kDefaultShipOptions );
					} else {
						gST.mDebug_fprintf("Failed to pick ship #%d. Using old method. ShipBPV: %d\n", shipNumber, thisShipBPV);

						eClassTypes thisShipClass = currentShip->mGetClassType();
						mCreateShip(
							typeid( tMirakShip ),
							thisShipBPV,
							thisShipClass,
							0,
							nStartPos,
							nCurrentID,
							0,0,0,0,
							NULL,
							kDefaultShipOptions  );
					}
					nCurrentID++;
				}
			}

			Ship cl1;
			CL_sl.SelectRandomShip(cl1);
			gST.mDebug_fprintf("cl1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
				cl1.shipCode.c_str(), cl1.bpv, cl1.yearFirstAvail, cl1.yearLastAvail);
			gST.mTrackOrions( cl1 );

			mCreateShip(
				typeid( tMirakShip ),
				cl1.shipCode.c_str(),
				nStartPos,
				kNoMetaShipID,
				nCurrentID,
				0, 0, 0, 0,
				NULL,
				kDefaultShipOptions );
		} else {
			mOldCreateShipsBasedOnPlayerTeam(startCustomID, position);
		}
    } else {
        mOldCreateShipsBasedOnPlayerTeam(startCustomID, position);
    }

    gST.mDebug_fprintf("--- Finished: tMirakTeam::mCreateShipsBasedOnPlayerTeam\n");
}


void tMirakTeam::mOldCreateShipsBasedOnPlayerTeam(int32 startCustomID, int32 position)
{
	gST.mDebug_fprintf("--- Starting: tMirakTeam::mOldCreateShipsBasedOnPlayerTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
			return;

	int32 nStartPos = position;
	int32 nCurrentID = startCustomID;
	int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;

	if( diffLevel == 0 )
	{
			nVarPercent = 0.65f;
	}
	else if( diffLevel == 1 )
	{
			nVarPercent = 1.0f;
	}
	else if( diffLevel == 2 )
	{
			nVarPercent = 1.25f;
	}

	for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
	{
		tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );

		if( currentShip )
		{
			eClassTypes thisShipClass = currentShip->mGetClassType();
			int32 thisShipBPV = currentShip->mGetShipBPV();
			mCreateShip( 
				typeid( tMirakShip ), 
				thisShipBPV * nVarPercent, 
				thisShipClass, 
				0, 
				nStartPos, 
				nCurrentID, 
				0,0,0,0, 
				NULL, 
				kDefaultShipOptions );
			nCurrentID++;
		}
	}

	mCreateShip( typeid( tMirakShip ),
				 150,
				 kClassLightCruiser,
				 0,
				 nStartPos,
				 nCurrentID,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tMirakTeam::mOldCreateShipsBasedOnPlayerTeam\n");
}