/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fTransportLaunched = false;
	fTransportDestroyed = false;
	fTransportEscaped = false;
	fBaseDestroyed = false;
	fPlanetInvaded = false;
	fInvasionForceDestroyed = false;
	fKilledEnemyBeforeLaunch = false;
	fBaseCaptured = false;
	fDied = false;
	fEnemyReinforcementsArrived = false;
	fKilledAllEnemies = false;
}

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( fDied )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_PlayerDied_msg );
		mScheduleNextMission( fVictoryCondition );
		return;
	}

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
			mScheduleNextMission( fVictoryCondition );
			return;
		}
		break;
	}


	 // Enemy was stopped before the base needed to launch an evac transport
	if( fKilledEnemyBeforeLaunch )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_KilledEnemyBeforeLaunch );
	}

	// Transport was launched; transport and base survived
	else if( ( fTransportLaunched) && ( fTransportEscaped ) && ( !fBaseDestroyed ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BaseAndTransportSurvived );
	}

	// Base was destroyed, transport escaped
	else if( ( fBaseDestroyed ) && ( fTransportEscaped ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BaseKilled_TransportEscaped );
	}

	// Base was destroyed, transport dead
	else if( ( fBaseDestroyed ) && ( fTransportDestroyed ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BaseKilled_TransportKilled );
	}

	// Base was captured, transport escaped
	else if( ( fBaseCaptured ) && ( fTransportEscaped ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BaseCapped_TransportEscaped );
	}

	// Base was captured, transport dead
	else if( ( fBaseCaptured ) && ( fTransportDestroyed ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BaseCapped_TransportKilled );
	}

	// Base destroyed or capped before transport launched
	else if( ( (fBaseDestroyed) || (fBaseCaptured) ) && ( !fTransportLaunched ) )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_BaseGone_NoLaunch );
	}

	mScheduleNextMission( fVictoryCondition );
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{

}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 400;
	fMissionScheduler.fNextMissionTitle = "Uni_9Edge";
}

iTruth tPlayerTeam::mTransportLaunched()
{
	return fTransportLaunched;
}

void tPlayerTeam::mSetTransportLaunched()
{
	fTransportLaunched = true;
}

iTruth tPlayerTeam::mTransportDestroyed()
{
	return fTransportDestroyed;
}

void tPlayerTeam::mSetTransportDestroyed()
{
	fTransportDestroyed = true;
}

iTruth tPlayerTeam::mBaseDestroyed()
{
	return fBaseDestroyed;
}

void tPlayerTeam::mSetBaseDestroyed()
{
	fBaseDestroyed = true;
}

iTruth tPlayerTeam::mTransportEscaped()
{
	return fTransportEscaped;
}

void tPlayerTeam::mSetTransportEscaped()
{
	fTransportEscaped = true;
}

iTruth tPlayerTeam::mPlanetInvaded()
{
	return fPlanetInvaded;
}

void tPlayerTeam::mSetPlanetInvaded()
{
	fPlanetInvaded = true;
}

iTruth tPlayerTeam::mInvasionForceDestroyed()
{	
	return fInvasionForceDestroyed;
}

void tPlayerTeam::mSetInvasionForceDestroyed()
{
	fInvasionForceDestroyed = true;
}

iTruth	tPlayerTeam::mKilledEnemyBeforeLaunch()
{
	return fKilledEnemyBeforeLaunch;
}

void tPlayerTeam::mSetKilledEnemyBeforeLaunch()
{
	fKilledEnemyBeforeLaunch = true;
}

iTruth tPlayerTeam::mBaseCaptured()
{
	return fBaseCaptured;
}

void tPlayerTeam::mSetBaseCaptured()
{
	fBaseCaptured = true;
}

iTruth	tPlayerTeam::mKilledAllEnemies()
{
	return fKilledAllEnemies;
}

void tPlayerTeam::mSetKilledAllEnemies()
{
	fKilledAllEnemies = true;
}