/********************************************************************************************************\

	Mission:	"Massacre"
	Script:		"Uni_8Massacre"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno
	Last Updated:	FS				10/20/2012
					era-correct ships, drones and fighters.

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_8Massacre.h"
#include "PlayerTeam.h"
#include "MirakTeam.h"
#include "BaseTeam.h"
#include "FriendlyTeam.h"
#include "FreighterTeam.h"
#include "InvasionTeam.h"
#include "ShipList_API.h"

DEFINE_SCRIPT_CLASS( tUni_8Massacre );

tUni_8Massacre::tUni_8Massacre()
{}

int32 tUni_8Massacre::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_8Massacre::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag, 
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Mirak Team",
												static_cast< eTeamID >( kMirakTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tMirakTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Base Team",
												static_cast< eTeamID >( kBaseTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tBaseTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Friendly Team",
												static_cast< eTeamID >( kFriendlyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tFriendlyTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Freighter Team",
												static_cast< eTeamID >( kFreighterTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tFreighterTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Invasion Team",
												static_cast< eTeamID >( kInvasionTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tInvasionTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_8Massacre::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_8Massacre::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_8Massacre::mInitializeFinished( void )
{
	return kBriefing_msg;
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_8Massacre::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	Leave blank until informed otherwise.

*/
void tUni_8Massacre::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_8Massacre::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_8Massacre::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kMirakTeam,
		kBaseTeam,
		kFriendlyTeam,
		kFreighterTeam,
		kInvasionTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Mirk = kMirakTeam,
		//Base = kBaseTeam,
		//Fren = kFriendlyTeam,
		//Frt  = kFreighterTeam,
		//Inv  = kInvasionTeam,

		//	     Plyr  Mirk  Base  Fren  Frt   Inv
		/*Plyr*/ "--", "WD", "AI", "AI", "AI", "WD",
		/*Mirk*/ "WD", "--", "WX", "WD", "WD", "AI",
		/*Base*/ "AI", "WD", "--", "AI", "AI", "WD",
		/*Fren*/ "AI", "WD", "AI", "--", "AI", "WD",
		/*Frt*/	 "AI", "NI", "AI", "AI", "--", "NI",
		/*Inv*/	 "HI", "AI", "WX", "HI", "WD", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


void tUni_8Massacre::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerBase;
	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;
}


tMapTerrain tUni_8Massacre::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_8Massacre::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}

void tUni_8Massacre::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_msg;
}