/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyStarBase.h"
#include "EnemyShip.h"
#include "EnemyDefPlat.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

tEnemyTeam::tEnemyTeam(): fShieldTimer( 0 )
{
}


void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    // make the SB!
    mCreateStarBase();

	std::string DNShipType = "I-CA";
	std::string BBShipType = "I-CC";

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		switch (fMissionInfo->mGetDifficultyLevel()) {
		case 2:
			DNShipType = gST.mImprovedHardcodedShip( "I-DN", mGetRace(), kEClassDreadnought, kEClassHeavyDreadnought );
			BBShipType = gST.mImprovedHardcodedShip( "I-BB", mGetRace(), kEClassHeavyDreadnought, kEClassBattleship, kLeader );
			break;
		case 1:
			DNShipType = gST.mImprovedHardcodedShip( "I-CA", mGetRace(), kEClassHeavyCruiser, kEClassNewHeavyCruiser );
			BBShipType = gST.mImprovedHardcodedShip( "I-DN", mGetRace(), kEClassDreadnought, kEClassHeavyDreadnought, kLeader );
			break;
		case 0:
		default:
			DNShipType = gST.mImprovedHardcodedShip( "I-CA", mGetRace(), kEClassHeavyCruiser, kEClassNewHeavyCruiser );
			BBShipType = gST.mImprovedHardcodedShip( "I-CC", mGetRace(), kEClassHeavyCruiser, kEClassHeavyBattlecruiser, kLeader );
			break;
		}

		gST.mDebug_fprintf("Selected DN: %s   Selected BB: %s\n", DNShipType.c_str(), BBShipType.c_str());

		mCreateShip( typeid( tEnemyShip ),
			DNShipType.c_str(),
			kStartPosition_Y, 
			kNoMetaShipID, 
			kEnemyDreadnought,
			0, 0, -1, -1, 
			NULL, 
			kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
			BBShipType.c_str(), 
			kStartPosition_X, 
			kNoMetaShipID, 
			kEnemyBattleship,
			0, 0, -1, -1, 
			NULL, 
			kDefaultShipOptions );

	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	if( diffLevel == 2 )
	{
		mCreateShip( typeid( tEnemyShip ),
			"I-DN", kStartPosition_Y, kNoMetaShipID, kEnemyDreadnought,
			0, 0, -1, -1, NULL, kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
			"I-BB", kStartPosition_X, kNoMetaShipID, kEnemyBattleship,
			0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else if( diffLevel == 1 )
	{
		mCreateShip( typeid( tEnemyShip ),
			"I-CA", kStartPosition_Y, kNoMetaShipID, kEnemyDreadnought,
			0, 0, -1, -1, NULL, kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
			"I-DN", kStartPosition_X, kNoMetaShipID, kEnemyBattleship,
			0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	else
	{
		mCreateShip( typeid( tEnemyShip ),
			"I-CA", kStartPosition_Y, kNoMetaShipID, kEnemyDreadnought,
			0, 0, -1, -1, NULL, kDefaultShipOptions );

		mCreateShip( typeid( tEnemyShip ),
			"I-CC", kStartPosition_X, kNoMetaShipID, kEnemyBattleship,
			0, 0, -1, -1, NULL, kDefaultShipOptions );
	}
	gST.mDebug_fprintf("--- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
	fShieldTimer = 0;
	tShipInfo * starbase;
	tShipIterator ship = mGetFirstShip();
	for(ship; *ship != NULL; ship++)
	{
		if( ship->mGetCustomID() == kEnemyStarBase)
		{
			starbase = *ship;
		}
	}
	Assert(starbase);
	if(starbase != NULL)
	{
        int32 X, Y;
        starbase->mGetGridPosition(&X, &Y);

		ship = mGetFirstShip();
		for(ship; *ship != NULL; ship++)
		{
			if( ship->mGetCustomID() != kEnemyStarBase)
			{
				ship->mDefendEntity(starbase, 10.0, 10.0, kAbsolutePriority);  // AI is dumb and stops. Forcing Destination! -- FS
                ship->mSetFinalDestination(X, Y);
                ship->mSetShipThrust( 10 );
                //ship->mSetShipMaxSpeed( 10 );
                ship->mSetShipAIControlled(kTrue);
			}
		}
	}
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}


int32 tEnemyTeam::mShieldTimer(void)
{
	return fShieldTimer;
}
void tEnemyTeam::mIncShieldTimer(void)
{
	fShieldTimer ++;
}

void tEnemyTeam::mCreateStarBase()
{
    gST.mDebug_fprintf("--- Starting: tEnemyTeam::mCreateDefSats()\n");

    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

        // Defense platforms
        ShipList DEF_sl = sl;
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);

        // StarBase
        ShipList SB_sl = gST.stripOutSPECIAL( sl );
        SB_sl = gST.stripOutRestricted( SB_sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassStarBase);
        fShiplistCount = SB_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for SB filter completed. ships: %d\n", fShiplistCount);


        if ((DEF_sl.size() > 0) && (SB_sl.size() > 0))
        {
            Ship def, sb;
            SB_sl.SelectRandomShip(sb);
            gST.mDebug_fprintf("sb %s selected. BPV: %d  YFA: %d  YLA %d\n",
                sb.shipCode.c_str(), sb.bpv, sb.yearFirstAvail, sb.yearLastAvail);
            gST.mTrackOrions( sb );

            mCreateShip( typeid( tEnemyStarBase ),
                sb.shipCode.c_str(), kStartPosition_Z, kNoMetaShipID, kEnemyStarBase,
                0, 0, -1, -1, NULL, kDefaultShipOptions );

            for(int32 i = 0; i < 2; i++)
            {
                DEF_sl.SelectRandomShip(def);
                gST.mDebug_fprintf("def %s selected. BPV: %d  YFA: %d  YLA %d\n",
                    def.shipCode.c_str(), def.bpv, def.yearFirstAvail, def.yearLastAvail);
                gST.mTrackOrions( def );

                mCreateShip( typeid( tEnemyDefPlat ),
                    def.shipCode.c_str(), kStartPosition_P + i, kNoMetaShipID, kEnemyDefPlat1 + i,
                    0, 0, -1, -1, NULL, kDefaultShipOptions );
            }

        } else {
            mOldCreateStarBase();
        }
    } else {
        mOldCreateStarBase();
    }

    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateStarBase()\n");
}

void tEnemyTeam::mOldCreateStarBase()
{
    gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateStarBase()\n");

	mCreateShip( typeid( tEnemyStarBase ),
		"I-SB", kStartPosition_Z, kNoMetaShipID, kEnemyStarBase,
		0, 0, -1, -1, NULL, kDefaultShipOptions );

	for(int32 i = 0; i < 2; i++)
	{
		mCreateShip( typeid( tEnemyDefPlat ),
			"I-DEF", kStartPosition_P + i, kNoMetaShipID,
			kEnemyDefPlat1 + i, 0, 0, -1, -1, NULL,
			kDefaultShipOptions );
	}

    gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateStarBase()\n");
}


