/********************************************************************************************************\

	File Name:	PlayerNPC2Team.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "PlayerNPC2Team.h"
#include "PlayerNPC2TeamBaseState.h"
#include "PlayerNPC2TeamAllyAttackedState.h"
#include "PlayerNPC2Ship.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPC2Team> gPlayerNPC2TeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPC2Team::mSetupState()
{
	mSetAllyAttacked(false);
	mRegisterState( new tPlayerNPC2TeamBaseState() );
	mRegisterState( new tPlayerNPC2TeamAllyAttackedState() );
	mGotoState( typeid( tPlayerNPC2TeamBaseState ) );
}

void tPlayerNPC2Team::mSetupCommState()
{}

void tPlayerNPC2Team::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPC2Team::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

		ShipList ship_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		if ( diffLevel == 2 )
		{
			shipClassTypeList.push_back(kEClassHeavyCruiser);
			shipClassTypeList.push_back(kEClassNewHeavyCruiser);
		} else {
			shipClassTypeList.push_back(kEClassHeavyBattlecruiser);
		}
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for hull classes filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent( ship_sl );
        }

		if ((diffLevel != 2) && (ship_sl.size() == 0)) 
        {
			gST.mDebug_fprintf("FAIL: BCH selection empty. Falling back to CAs.\n");
			ship_sl = sl;
			{
			std::list< eEnhancedClassTypes >  shipClassTypeList;
			shipClassTypeList.push_back(kEClassHeavyCruiser);
			shipClassTypeList.push_back(kEClassNewHeavyCruiser);
			fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
			}
			gST.mDebug_fprintf("subset for hull classes filter completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent( ship_sl );
        }

        if (ship_sl.size() > 0)
        {
            Ship ship1;
            ship_sl.SelectRandomShip(ship1);
            gST.mDebug_fprintf("ship1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship1.shipCode.c_str(), ship1.bpv, ship1.yearFirstAvail, ship1.yearLastAvail);
            gST.mTrackOrions( ship1 );

			mCreateShip( typeid( tPlayerNPC2Ship ),
				ship1.shipCode.c_str(),
				kStartPosition_G + mGetTeam(), 
				kNoMetaShipID, 
				kPlayerNPC2Ship1,
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }
    gST.mDebug_fprintf("--- Finished: tPlayerNPC2Team::mCreateShipsForTeam\n");
}

void tPlayerNPC2Team::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tPlayerNPC2Team::mOldCreateShipsForTeam\n");

	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	if( diffLevel == 2 )
	{
		mCreateShip(	typeid( tPlayerNPC2Ship ),
						kMaxBPV,
						kClassHeavyCruiser,
						0,
						kStartPosition_G + mGetTeam(),
						kPlayerNPC2Ship1,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );
	}
	else
	{
		mCreateShip(	typeid( tPlayerNPC2Ship ),
						kMaxBPV,
						kClassHeavyBattlecruiser,
						0,
						kStartPosition_G + mGetTeam(),
						kPlayerNPC2Ship1,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );
	}

	gST.mDebug_fprintf("--- Finished: tPlayerNPC2Team::mOldCreateShipsForTeam\n");
}

void tPlayerNPC2Team::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPC2Team::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlayerNPC2Team", shipScriptDescription );
}

bool tPlayerNPC2Team::mAllyAttacked(void)
{
	return fAllyAttacked;
}
void tPlayerNPC2Team::mSetAllyAttacked(bool x)
{
	if(x)
	{
		mGotoState( typeid( tPlayerNPC2TeamAllyAttackedState ) );
	}
	fAllyAttacked = x;
}
