/********************************************************************************************************\

	File Name:	PlayerNPC4Team.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "PlayerNPC4Team.h"
#include "PlayerNPC4TeamBaseState.h"
#include "PlayerNPC4TeamAllyAttackedState.h"
#include "PlayerNPC4Ship.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerNPC4Team> gPlayerNPC4TeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerNPC4Team::mSetupState()
{
	mSetAllyAttacked(false);
	mRegisterState( new tPlayerNPC4TeamBaseState() );
	mRegisterState( new tPlayerNPC4TeamAllyAttackedState() );
	mGotoState( typeid( tPlayerNPC4TeamBaseState ) );
}

void tPlayerNPC4Team::mSetupCommState()
{}

void tPlayerNPC4Team::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tPlayerNPC4Team::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

		ShipList ship_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
		shipClassTypeList.push_back(kEClassHeavyCruiser);
		shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for hull classes filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent( ship_sl );
        }

        if (ship_sl.size() > 0)
        {
            Ship ship1;
            ship_sl.SelectRandomShip(ship1);
            gST.mDebug_fprintf("ship1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship1.shipCode.c_str(), ship1.bpv, ship1.yearFirstAvail, ship1.yearLastAvail);
            gST.mTrackOrions( ship1 );

			mCreateShip( typeid( tPlayerNPC4Ship ),
				ship1.shipCode.c_str(),
				kStartPosition_G + mGetTeam(), 
				kNoMetaShipID, 
				kPlayerNPC4Ship1,
				0, 0, -1, -1, 
				NULL, 
				kDefaultShipOptions );
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }
    gST.mDebug_fprintf("--- Finished: tPlayerNPC4Team::mCreateShipsForTeam\n");
}

void tPlayerNPC4Team::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tPlayerNPC4Team::mOldCreateShipsForTeam\n");

	mCreateShip(	typeid( tPlayerNPC4Ship ),
					kMaxBPV,
					kClassHeavyCruiser,
					0,
					kStartPosition_G + mGetTeam(),
					kPlayerNPC4Ship1,
					0, 0, -1,  -1, NULL,
					kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tPlayerNPC4Team::mOldCreateShipsForTeam\n");
}

void tPlayerNPC4Team::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerNPC4Team::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlayerNPC4Team", shipScriptDescription );
}

bool tPlayerNPC4Team::mAllyAttacked(void)
{
	return fAllyAttacked;
}
void tPlayerNPC4Team::mSetAllyAttacked(bool x)
{
	if(x)
	{
		mGotoState( typeid( tPlayerNPC4TeamAllyAttackedState ) );
	}
	fAllyAttacked = x;
}
