/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "Uni_9A1Tyr.h"
#include "ShipList_API.h"

#define kTimerConstant 2 / 60

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam(): fAITarget( NULL )
{}

/*

	HELPER NOTE:	The victory condition must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebBaseDestroyedTrue_msg);
		fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebDismissed_msg);
		break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebBaseDestroyedFalse_msg);
		fMissionInfo->mAddToDebriefingList(mGetTeam(), kDebDismissed_msg);
		break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( mGetTeam(), kDisconnect_msg );
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
	tShipInfo * flagship;

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
			
		}
		flagship = *ship;
		flagship->mSetShipThrust(10);
		ship++;
		for(ship; *ship != NULL; ship++)
		{
			// ship->mDefendEntity(flagship, 10.0, 4.0, kAbsolutePriority);  // mDefendEntity is broken. --FS
			ship->mSetShipThrust(10);
		}
	}
	mSetRandomChat(true);
	mSetChatterCounter(fMissionInfo->mRandomInt32(3, 6));
	mSetTimedInterval( kTimerTurnsSet, kEventTimerID, (float) kTimerConstant * 5);
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

bool tPlayerTeam::mRandomChat(void)
{
	return fRandomChat;
}
void tPlayerTeam::mSetRandomChat(bool x)
{
	fRandomChat = x;
}

bool tPlayerTeam::mAtEdgeOfMap(void)
{
	return fAtEdgeOfMap;
}

void tPlayerTeam::mSetAtEdgeOfMap(bool x)
{
	fAtEdgeOfMap = x;
}

int32 tPlayerTeam::mChatterCounter(void)
{
	return fChatterCounter;
}
void tPlayerTeam::mSetChatterCounter(int32 x)
{
	fChatterCounter = x;
}

void tPlayerTeam::mDecChatterCounter(void)
{
	fChatterCounter--;
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 250;
	fMissionScheduler.fNextMissionTitle = "Uni_10Criminals";
}