/********************************************************************************************************\

	Mission:	Operation: Tyr
	Script:		"Uni_9A1Tyr"
	Descript:	Assault on an ISC starbase, with allies. In this version
				the player has obtained a prefix code to lower the ISC starbase
				shields.

	Created by:		Dutch Blomenkamp mm/dd/yy
	Last Updated:	NDB	07/17/00
					DJS						09/26/00
					Meta script modifications
					

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerNPC1Team.h"
#include "PlayerNPC2Team.h"
#include "PlayerNPC3Team.h"
#include "PlayerNPC4Team.h"

#include "PlayerTeam.h"
#include "Uni_9A1Tyr.h"
#include "ShipList_API.h"



DEFINE_SCRIPT_CLASS( tUni_9A1Tyr );

tUni_9A1Tyr::tUni_9A1Tyr() : 
	gShieldsLowered(false),
		gCodeKnown(true)

{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tUni_9A1Tyr::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												(eTeamID)kPlayerTeam,
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
												kSpecPlayableRace & ~kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player NPC 1 Team ",
												(eTeamID)kPlayerNPC1Team,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC,,
												kBestAllyOf | kPlayerTeam,
												typeid( tPlayerNPC1Team ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Player NPC 2 Team ",
												(eTeamID)kPlayerNPC2Team,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC,,
												kBestAllyOf | kPlayerTeam,
												typeid( tPlayerNPC2Team ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Player NPC 3 Team ",
												(eTeamID)kPlayerNPC3Team,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC,,
												kSameAs | kPlayerTeam,
												typeid( tPlayerNPC3Team ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Player NPC 4 Team ",
												(eTeamID)kPlayerNPC4Team,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC,,
												kAnyAllyOf | kPlayerTeam,
												typeid( tPlayerNPC4Team ) );

			
	const tTeamSpec& team6 = mCreateTeamSpec(	"Enemy Team",
												(eTeamID)kEnemyTeam,
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tEnemyTeam ) );

	
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
  


}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_9A1Tyr::mInitializeStart( tDynaverseScriptInfo& info )
{
	// These are global variables that
	// will be passed back and forth to the campaign,
	// if I'm understanding things correctly.
	// I'm initializing them as they would be if the 
	// player had successfully completed the pre-requisites.
	gCodeKnown = true;
	gShieldsLowered = false;


	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_9A1Tyr::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_9A1Tyr::mInitializeFinished( void )
{
	tTeamInfo * PlayerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam);
	//Assert(PlayerTeam);
	if(!PlayerTeam)
	{
		return kBriefing_msg;
	}
	else
	{
		return kFedBriefing_msg + PlayerTeam->mGetRace();
	}
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_9A1Tyr::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_9A1Tyr::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPC1Team,
		kPlayerNPC2Team,
		kPlayerNPC3Team,
		kPlayerNPC4Team,
		kEnemyTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC1 = kPlayerNPC1Team,
		//PNPC2 = kPlayerNPC2Team,
		//PNPC3 = kPlayerNPC3Team,
		//PNPC4 = kPlayerNPC4Team,
		//Nme  = kEnemyTeam,
		

		//			Plyr	PNPC1	PNPC2	PNPC3	PNPC4	Nme
		/*Plyr*/	"--",	"AI",	"AI",	"AI",	"AI",	"WD",
		/*PNPC1*/	"AI",	"--",	"AI",	"AI",	"AI",	"WD",
		/*PNPC2*/	"AI",	"AI",	"--",	"AI",	"AI",	"WD",
		/*PNPC3*/	"AI",	"AI",	"AI",	"--",	"AI",	"WD",
		/*PNPC4*/	"AI",	"AI",	"AI",	"AI",	"--",	"WD",
		/*Nme */	"WD",	"WD",	"WD",	"WD",	"WD",	"--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_9A1Tyr::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['J'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['K'] = kObjectAlliedShip;

	Map->fTacticalMapIcon['P'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Q'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['R'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['S'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['T'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['U'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['V'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['W'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['X'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Z'] = kObjectEnemyBase;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_9A1Tyr::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kFedBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kKlingonBriefing_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] = kRomulanBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kLyranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kGornBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kHydranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] = kMirakBriefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_9A1Tyr::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}