/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_9C1Welcome.h"


//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}
/*

	HELPER NOTE:	The following function will calculate how the player did once you end the mission with
					a call to mSetGameStatus().  It is not necessary to have every victory condition handled
					here (e.g. kAstoundingVictory...kDevastatingDefeat), especially if the victory condition
					is simply a boolean value (e.g. killed or has killed).  An example follows.

	eVictoryLevels
	tBaseVictoryState::mCalcVictoryStatus()
	{
		if ( fChildActor->mBSed() )
		{
			//
			//	objective was completed.
			//
			return kAstoundingVictory;
		}
		else
		{
			//
			//	objective was not completed.
			//
			return kDevastatingDefeat;
		}
	}

*/
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus(eTeamID ID)
{
	if(fChildActor->mBaseDestroyed())
		return kAstoundingVictory;
	else
		return kDevastatingDefeat;

}

//
//	Ship events
//
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetTeam() )
	{
	case kPlayerTeam: 
		if ( !playerTeam->mShipsRemainingInPlay() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );
		break;

	case kEnemyTeam:
		if( ship->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
		{
			fChildActor->mSetBaseDestroyed(true);

			tTeamInfo *ti = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
			tShipIterator tsi = ti->mGetFirstShip();
			for (;;)
			{
				tShipInfo *si = *tsi;
				if (!si)
					break;

				si->mClearCommands();
				si->mSetShipAIControlled(kTrue);
				tsi++;
			}
		}
	}

}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetOriginalTeam() )
	{
	case kPlayerTeam: 
		if ( !playerTeam->mShipsRemainingInPlay() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerCaptured );
		break;
	case kEnemyTeam:
		if( ship->mGetCustomID() == tEnemyTeam::kEnemyStarBase)
		{
			fChildActor->mSetBaseDestroyed(true);

			tTeamInfo *ti = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
			tShipIterator tsi = ti->mGetFirstShip();
			for (;;)
			{
				tShipInfo *si = *tsi;
				if (!si)
					break;

				si->mClearCommands();
				si->mSetShipAIControlled(kTrue);
				tsi++;
			}
		}
		tTeamInfo*	team = fMissionInfo->mGetTeamHandle(  (eTeamID) kEnemyTeam );
		Assert( team );
		if ( team )
		{
			if ( !team->mShipsRemainingInPlay() )
				gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerWins );
		}
		break;
	}
	

}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	
	
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	tShipInfo * playership;
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( (eTeamID) kPlayerTeam );
	Assert( playerTeam );
	if ( !playerTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		// Get the first undestroyed ship, so that we make sure
		// it's the flagship disengaging.

		playership = *(playerTeam->mGetFirstShip());
		for(playership; playership != NULL; playership++)
		{
			if(playership->mInPlay())
				break;
		}
		if(playership->mGetCustomID() == ship->mGetCustomID() )
		{
			fMissionInfo->mDisplayMessage( (eTeamID) kPlayerTeam, "Science Officer: We have left the system, [PLAYER_RANK].\n\n");
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDisengaged );
		}
		break;

	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
