/********************************************************************************************************\

	File Name:	AllyTeam1.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "AllyTeam1.h"
#include "AllyTeam1BaseState.h"
#include "AllyShip1.h"
#include "CommonDefs.h"
#include "Uni_9Edge.h"
#include "ShipList_API.h"


tClassAllocator<tAllyTeam1> gAllyTeam1Allocator;

void tAllyTeam1::mSetupState()
{
	mRegisterState( new tAllyTeam1BaseState() );
	mGotoState( typeid( tAllyTeam1BaseState ) );
}

void tAllyTeam1::mSetupCommState()
{}

void tAllyTeam1::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tAllyTeam1::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        ShipList CA_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyCruiser);
        shipClassTypeList.push_back(kEClassNewHeavyCruiser);
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CA_sl);
        }

        ShipList BCH_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassHeavyBattlecruiser);
        fShiplistCount = BCH_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for BCH filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(BCH_sl);
        }
        if (BCH_sl.size() == 0) {
            gST.mDebug_fprintf("FAIL! Unable to select a BCH from shiplist. using CA shiplist\n");
            BCH_sl = CA_sl;
        }

        ShipList DD_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassDestroyer);
        shipClassTypeList.push_back(kEClassWarDestroyer);
        fShiplistCount = DD_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DD_sl);
        }

		if (
            (BCH_sl.size() > 0) &&
            (CA_sl.size() > 0) &&
            (DD_sl.size() > 0)
        ) {
            Ship bch1, bch2, ca1, dd1, dd2;
            BCH_sl.SelectRandomShip(bch1);
            gST.mDebug_fprintf("bch1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                bch1.shipCode.c_str(), bch1.bpv, bch1.yearFirstAvail, bch1.yearLastAvail);
            gST.mTrackOrions( bch1 );

            BCH_sl.SelectRandomShip(bch2);
            gST.mDebug_fprintf("bch2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                bch2.shipCode.c_str(), bch2.bpv, bch2.yearFirstAvail, bch2.yearLastAvail);
            gST.mTrackOrions( bch2 );

            CA_sl.SelectRandomShip(ca1);
            gST.mDebug_fprintf("ca1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ca1.shipCode.c_str(), ca1.bpv, ca1.yearFirstAvail, ca1.yearLastAvail);
            gST.mTrackOrions( ca1 );

            DD_sl.SelectRandomShip(dd1);
            gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
            gST.mTrackOrions( dd1 );

            DD_sl.SelectRandomShip(dd2);
            gST.mDebug_fprintf("dd2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd2.shipCode.c_str(), dd2.bpv, dd2.yearFirstAvail, dd2.yearLastAvail);
            gST.mTrackOrions( dd2 );

            mCreateShip(
                typeid( tAllyShip1 ),
                bch1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kAlly1_1,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip1 ),
                bch2.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kAlly1_2,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip1 ),
                ca1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kAlly1_3,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip1 ),
                dd1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kAlly1_4,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tAllyShip1 ),
                dd2.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kAlly1_5,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tAllyTeam1::mCreateShipsForTeam\n");
}

void tAllyTeam1::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tAllyTeam1::mOldCreateShipsForTeam\n");

	// BCH, BCH, CA, DD, DD
	mCreateShip( typeid( tAllyShip1 ),
			     400,
				 kClassHeavyBattlecruiser,
				 0,
				 kStartPosition_H,
				 kAlly1_1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip1 ),
			     400,
				 kClassHeavyBattlecruiser,
				 0,
				 kStartPosition_H,
				 kAlly1_2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip1 ),
			     400,
				 kClassHeavyCruiser,
				 0,
				 kStartPosition_H,
				 kAlly1_3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip1 ),
			     400,
				 kClassDestroyer,
				 0,
				 kStartPosition_H,
				 kAlly1_4,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tAllyShip1 ),
			     400,
				 kClassDestroyer,
				 0,
				 kStartPosition_H,
				 kAlly1_5,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tAllyTeam1::mOldCreateShipsForTeam\n");
}

void tAllyTeam1::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tAllyTeam1::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tAllyTeam1", shipScriptDescription );
}

