/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ShipInfo.h"
#include "Uni_9Edge.h"
#include "ScriptInterface.h"
#include "PlayerTeam.h"
#include "AllyTeam1.h"
#include "AllyTeam2.h"
#include "MirakTeam1.h"
#include "MirakTeam2.h"
#include "LPTeam.h"
#include "AlliedPlanetTeam.h"
#include "MirakPlanetTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kCaptainsLog_timer, 0.2f );
	mSetTimedInterval( kTimerTurnsSet, kInitialUpdate_timer, 0.4f );
}

void tBaseVictoryState::mEndState()
{}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kCaptainsLog_timer:
		{
			tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
			if( !playerTeam )
				return;

			switch( playerTeam->mGetRace() )
			{
			case kLyran:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCaptainsLog_Lyran_msg, kNonCommScriptMessage );
				break;

			case kKlingon:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCaptainsLog_Klingon_msg, kNonCommScriptMessage );
				break;

			case kHydran:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCaptainsLog_Hydran_msg, kNonCommScriptMessage );
				break;
			}
		}
		break;

	case kInitialUpdate_timer:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kInitialReport_msg );
		}
		break;

	}
}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return kDraw;

	if( playerTeam->mDied() )
		return kDevastatingDefeat;

	if( playerTeam->mCappedPlanet() )
	{
		return kAstoundingVictory;
	}

	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* allyTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam1 ) );
	if( !allyTeam1 )
		return;
	
	tTeamInfo* allyTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam2 ) );
	if( !allyTeam2 )
		return;

	tTeamInfo* mirakTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam1 ) );
	if( !mirakTeam1 )
		return;

	tTeamInfo* mirakTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam2 ) );
	if( !mirakTeam2 )
		return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
					playerTeam->mSetDied();
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}
		}
		break;

	// A "primary" ally dies. Not really important
	case tAllyTeam1::kAlly1_1:
	case tAllyTeam1::kAlly1_2:
	case tAllyTeam1::kAlly1_3:
		{
			if( !allyTeam1->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllAlliesDead_msg );
			}

			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllyDies_msg );
			}
		}
		break;
		
	// A transport dies. Lose here if no more transports are left and
	// the planet is not captured	
	case tAllyTeam2::kAlly_2_1:
	case tAllyTeam2::kAlly_2_2:
	case tAllyTeam2::kAlly_2_3:
		{
			if( !allyTeam2->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCannotCapPlanet_msg );
			}
			
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTransportDies_msg );
			}		
		}
		break;
		
	case tMirakTeam1::kMirak_1_1:
	case tMirakTeam1::kMirak_1_2:
	case tMirakTeam1::kMirak_1_3:
	case tMirakTeam1::kMirak_1_4:
	case tMirakTeam1::kMirak_1_5:
	case tMirakTeam1::kMirak_1_6:
		{
			if( !mirakTeam1->mShipsRemainingInPlay() && !mirakTeam2->mShipsRemainingInPlay() )
			{
				playerTeam->mSetCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;		
		
	case tMirakTeam2::kMirak_2_1:
	case tMirakTeam2::kMirak_2_2:
	case tMirakTeam2::kMirak_2_3:
	case tMirakTeam2::kMirak_2_4:
	case tMirakTeam2::kMirak_2_5:
	case tMirakTeam2::kMirak_2_6:
		{
			if( !mirakTeam1->mShipsRemainingInPlay() && !mirakTeam2->mShipsRemainingInPlay() )
			{
				playerTeam->mSetCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;			
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* allyTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam1 ) );
	if( !allyTeam1 )
		return;
	
	tTeamInfo* allyTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam2 ) );
	if( !allyTeam2 )
		return;

	tTeamInfo* mirakTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam1 ) );
	if( !mirakTeam1 )
		return;

	tTeamInfo* mirakTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam2 ) );
	if( !mirakTeam2 )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

	// Planet captured. Player wins.
	case tMirakPlanetTeam::kMirakPlanet:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCappedPlanet_msg );
			playerTeam->mSetCappedPlanet();
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
		}
		break;


	// A "primary" ally is captured. Not really important
	case tAllyTeam1::kAlly1_1:
	case tAllyTeam1::kAlly1_2:
	case tAllyTeam1::kAlly1_3:
		{
			if( !allyTeam1->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllAlliesDead_msg );
			}

			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAllyDies_msg );
			}
		}
		break;
		
	// A transport is captured. Lose here if no mroe transports are left and
	// the planet is not captured	
	case tAllyTeam2::kAlly_2_1:
	case tAllyTeam2::kAlly_2_2:
	case tAllyTeam2::kAlly_2_3:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTransportDies_msg );

			if( ( playerTeam->mNumTransportsCalled() == 3 ) && ( !playerTeam->mCappedPlanet() ) )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCannotCapPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
			}
		}
		break;

	case tMirakTeam1::kMirak_1_1:
	case tMirakTeam1::kMirak_1_2:
	case tMirakTeam1::kMirak_1_3:
	case tMirakTeam1::kMirak_1_4:
	case tMirakTeam1::kMirak_1_5:
	case tMirakTeam1::kMirak_1_6:
		{
			if( !mirakTeam1->mShipsRemainingInPlay() && !mirakTeam2->mShipsRemainingInPlay() )
			{
				playerTeam->mSetCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;		
		
	case tMirakTeam2::kMirak_2_1:
	case tMirakTeam2::kMirak_2_2:
	case tMirakTeam2::kMirak_2_3:
	case tMirakTeam2::kMirak_2_4:
	case tMirakTeam2::kMirak_2_5:
	case tMirakTeam2::kMirak_2_6:
		{
			if( !mirakTeam1->mShipsRemainingInPlay() && !mirakTeam2->mShipsRemainingInPlay() )
			{
				playerTeam->mSetCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;			
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
		return;

	tTeamInfo* mirakTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam1 ) );
	if( !mirakTeam1 )
		return;

	tTeamInfo* mirakTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam2 ) );
	if( !mirakTeam2 )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;

	case tMirakTeam1::kMirak_1_1:
	case tMirakTeam1::kMirak_1_2:
	case tMirakTeam1::kMirak_1_3:
	case tMirakTeam1::kMirak_1_4:
	case tMirakTeam1::kMirak_1_5:
	case tMirakTeam1::kMirak_1_6:
		{
			if( !mirakTeam1->mShipsRemainingInPlay() && !mirakTeam2->mShipsRemainingInPlay() )
			{
				playerTeam->mSetCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;		
		
	case tMirakTeam2::kMirak_2_1:
	case tMirakTeam2::kMirak_2_2:
	case tMirakTeam2::kMirak_2_3:
	case tMirakTeam2::kMirak_2_4:
	case tMirakTeam2::kMirak_2_5:
	case tMirakTeam2::kMirak_2_6:
		{
			if( !mirakTeam1->mShipsRemainingInPlay() && !mirakTeam2->mShipsRemainingInPlay() )
			{
				playerTeam->mSetCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;			
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

