/********************************************************************************************************\

	File Name:	LPShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "LPShipBaseState.h"
#include "Uni_9Edge.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "ShipList_API.h"

#define LP_DETECTION_RANGE	50



void tLPShipBaseState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kRangeCheck_timer, 1.0f );

	gST.mDebug_fprintf("--- Starting: tLPShipBaseState::mBeginState\n");
#ifdef FIGHTERS
    fChildActor->mChangeAIOfficerState( kFighterOfficer, kFalse );
    mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
#endif
    gST.mDebug_fprintf("--- Finished: tLPShipBaseState::mBeginState\n");
}

void tLPShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	// In detection range of Mirak LP?
	case kRangeCheck_timer:
		{
			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
				return;
			
			tShipInfo* ship = playerTeam->mGetThisShip(1);
			if( !ship )
				return;

			int32 RangeToPlayer = fChildActor->mGetTacticalDistance( ship );

			if( ( RangeToPlayer < LP_DETECTION_RANGE ) && ( !playerTeam->mDetected() ) )
			{
				tTeamInfo* mirakTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kMirakTeam2 ) );
				if( !mirakTeam2 )
					return;

				tTeamInfo* allyTeam1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam1 ) );
				if( !allyTeam1 )
					return;

				tTeamInfo* allyTeam2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam2 ) );
				if( !allyTeam2 )
					return;

				mirakTeam2->mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamAtWar, kGoalDestroy, 1.0f );
				mirakTeam2->mSetRelations( static_cast<eTeamID>( kAllyTeam1 ), kTeamAtWar, kGoalDestroy, 1.0f );
				mirakTeam2->mSetRelations( static_cast<eTeamID>( kAllyTeam2 ), kTeamAtWar, kGoalDestroy, 1.0f );

				allyTeam1->mSetRelations( static_cast<eTeamID>( kMirakTeam2 ), kTeamAtWar, kGoalDestroy, 1.0f );
				allyTeam1->mSetRelations( static_cast<eTeamID>( kMirakTeam2 ), kTeamAtWar, kGoalDestroy, 1.0f );
				allyTeam1->mSetRelations( static_cast<eTeamID>( kMirakTeam2 ), kTeamAtWar, kGoalDestroy, 1.0f );

				allyTeam2->mSetRelations( static_cast<eTeamID>( kMirakTeam2 ), kTeamNeutral, kGoalIgnore, 1.0f );
				allyTeam2->mSetRelations( static_cast<eTeamID>( kMirakTeam2 ), kTeamNeutral, kGoalIgnore, 1.0f );
				allyTeam2->mSetRelations( static_cast<eTeamID>( kMirakTeam2 ), kTeamNeutral, kGoalIgnore, 1.0f );


				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kGotDetected_msg );
				playerTeam->mSetDetected();
			}

			else
			{
				mSetTimedInterval( kTimerTurnsSet, kRangeCheck_timer, 1.0f );
			}
		
		}
		break;

	case kUpgradeFightersID:
		{
#ifdef FIGHTERS
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        tTeamInfo* shipTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( fChildActor->mGetTeam() ) );
        if (shipTeam->mIsTeamHumanControlled()) return;    // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tLPShipBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");
        int32 BPVinc = gST.mUpgradeFighters( fChildActor );
        fChildActor->mChangeAIOfficerState( kFighterOfficer, kTrue );
        gST.mDebug_fprintf("Ship BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tLPShipBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
		}
		break;
    }
}


void tLPShipBaseState::mEndState()
{}

void tLPShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tLPShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tLPShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tLPShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tLPShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tLPShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tLPShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tLPShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tLPShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tLPShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tLPShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tLPShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tLPShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tLPShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tLPShipBaseState::mOnFlashingButtonPressed()
{}
void tLPShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tLPShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tLPShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tLPShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tLPShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tLPShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tLPShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tLPShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tLPShipBaseState::mOnHostChanged()
{}

void tLPShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tLPShipBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

