/********************************************************************************************************\

	File Name:	MirakTeam2.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MirakTeam2.h"
#include "MirakTeam2BaseState.h"
#include "MirakShip2.h"
#include "ShipList_API.h"


tClassAllocator<tMirakTeam2> gMirakTeam2Allocator;

void tMirakTeam2::mSetupState()
{
	mRegisterState( new tMirakTeam2BaseState() );
	mGotoState( typeid( tMirakTeam2BaseState ) );
}

void tMirakTeam2::mSetupCommState()
{}

void tMirakTeam2::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tMirakTeam2::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        ShipList CL_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(CL_sl);
        }

        ShipList DD_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassDestroyer);
        shipClassTypeList.push_back(kEClassWarDestroyer);
        fShiplistCount = DD_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for DD filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DD_sl);
        }

        ShipList FF_sl = sl;
        {
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassFrigate);
        fShiplistCount = FF_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for FF filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(FF_sl);
        }

		if (
            (CL_sl.size() > 0) &&
            (DD_sl.size() > 0) &&
            (FF_sl.size() > 0)
        ) {
            Ship cl1, cl2, dd1, ff1;
            CL_sl.SelectRandomShip(cl1);
            gST.mDebug_fprintf("cl1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cl1.shipCode.c_str(), cl1.bpv, cl1.yearFirstAvail, cl1.yearLastAvail);
            gST.mTrackOrions( cl1 );

            CL_sl.SelectRandomShip(cl2);
            gST.mDebug_fprintf("cl2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                cl2.shipCode.c_str(), cl2.bpv, cl2.yearFirstAvail, cl2.yearLastAvail);
            gST.mTrackOrions( cl2 );

            DD_sl.SelectRandomShip(dd1);
            gST.mDebug_fprintf("dd1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                dd1.shipCode.c_str(), dd1.bpv, dd1.yearFirstAvail, dd1.yearLastAvail);
            gST.mTrackOrions( dd1 );

            FF_sl.SelectRandomShip(ff1);
            gST.mDebug_fprintf("ff1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ff1.shipCode.c_str(), ff1.bpv, ff1.yearFirstAvail, ff1.yearLastAvail);
            gST.mTrackOrions( ff1 );

            mCreateShip(
                typeid( tMirakShip2 ),
                cl1.shipCode.c_str(),
                kStartPosition_M,
                kNoMetaShipID,
                kMirak_2_1,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tMirakShip2 ),
                cl2.shipCode.c_str(),
                kStartPosition_M,
                kNoMetaShipID,
                kMirak_2_2,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tMirakShip2 ),
                dd1.shipCode.c_str(),
                kStartPosition_M,
                kNoMetaShipID,
                kMirak_2_3,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );

            mCreateShip(
                typeid( tMirakShip2 ),
                ff1.shipCode.c_str(),
                kStartPosition_M,
                kNoMetaShipID,
                kMirak_2_4,
                0,0,0,0,
                NULL,
                kDefaultShipOptions );
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tMirakTeam2::mCreateShipsForTeam\n");
}

void tMirakTeam2::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("--- Starting: tMirakTeam2::mOldCreateShipsForTeam\n");

	// CL, CL, DD, FF
	mCreateShip( typeid( tMirakShip2 ),
				 150,
				 kClassLightCruiser,
				 0,
				 kStartPosition_M,
				 kMirak_2_1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tMirakShip2 ),
				 150,
				 kClassLightCruiser,
				 0,
				 kStartPosition_M,
				 kMirak_2_2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tMirakShip2 ),
				 150,
				 kClassDestroyer,
				 0,
				 kStartPosition_M,
				 kMirak_2_3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tMirakShip2 ),
				 150,
				 kClassFrigate,
				 0,
				 kStartPosition_M,
				 kMirak_2_4,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("--- Finished: tMirakTeam2::mOldCreateShipsForTeam\n");
}

void tMirakTeam2::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tMirakTeam2::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tMirakTeam2", shipScriptDescription );
}

