/********************************************************************************************************\

	Mission:	"Battle of Claw's Edge"
	Script:		"Uni_9Edge"
	Descript:	

	Created by:		Scott Bruno		07/29/00
	Last Updated:	FS				10/20/2012
					era-correct ships, drones and fighters.

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_9Edge.h"
#include "PlayerTeam.h"
#include "AllyTeam1.h"
#include "AllyTeam2.h"
#include "MirakTeam1.h"
#include "MirakTeam2.h"
#include "LPTeam.h"
#include "AlliedPlanetTeam.h"
#include "MirakPlanetTeam.h"
#include "ShipList_API.h"



DEFINE_SCRIPT_CLASS( tUni_9Edge );
tUni_9Edge::tUni_9Edge()
{}

int32 tUni_9Edge::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_9Edge::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag, 
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Ally Team 1",
												static_cast< eTeamID >( kAllyTeam1 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tAllyTeam1 ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Ally Team 2",
												static_cast< eTeamID >( kAllyTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecHydran,
												kSameAs | kPlayerTeam,
												typeid( tAllyTeam2 ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Mirak Team 1",
												static_cast< eTeamID >( kMirakTeam1 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tMirakTeam1 ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Mirak Team 2",
												static_cast< eTeamID >( kMirakTeam2 ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tMirakTeam2 ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Listening Post Team",
												static_cast< eTeamID >( kLPTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tLPTeam ) );

	const tTeamSpec& team7 = mCreateTeamSpec(	"Allied Planet Team",
												static_cast< eTeamID >( kAlliedPlanetTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak | kSpecISC,
												kBestAllyOf | kPlayerTeam,
												typeid( tAlliedPlanetTeam ) );

	const tTeamSpec& team8 = mCreateTeamSpec(	"Mirak Planet Team",
												static_cast< eTeamID >( kMirakPlanetTeam ),
												kMaxChance,
												kTrue,
												kNPCTeam,
												kMaxChance,
												kSpecMirak,
												kAnyRace,
												typeid( tMirakPlanetTeam ) );


	

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );
	Specs->mSetupTeam( team7 );
	Specs->mSetupTeam( team8 );
}

int32 tUni_9Edge::mInitializeStart( tDynaverseScriptInfo& info )
{
	/*
	FILE *fp = fopen("c:\\edge_date.txt", "w");
	fprintf(fp, "Date --  Year: %d  Day: %d\n", 
		info.fMissionDate.fYear, info.fMissionDate.fDay);
	fprintf(fp, "Campaign Year: %d\n", 
		info.fCampaignYear);
	fflush(fp);
	fclose(fp);
	*/

	return 1;
}

void tUni_9Edge::mCreateExternalShips( int32 Mode )
{}

int32 tUni_9Edge::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_9Edge::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_9Edge::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_9Edge::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_9Edge::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 8;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kAllyTeam1,
		kAllyTeam2,
		kMirakTeam1,
		kMirakTeam2,
		kLPTeam,
		kAlliedPlanetTeam,
		kMirakPlanetTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//All1 = kAllyTeam1
		//All2 = kAllyTeam2
		//Mir1 = kMirakTeam1
		//Mir2 = kMirakTeam2
		//LP   = kLPTeam
		//APT  = kAlliedPlanetTeam
		//MPT  = kMirakPlanetTeam

		//	     Plyr  All1  All2  Mir1  Mir2  LP    APT   MPT 
		/*Plyr*/ "--", "AI", "AI", "WD", "WD", "HI", "AI", "HI",
		/*All1*/ "AI", "--", "AI", "WD", "WD", "HI", "AI", "HI",
		/*All2*/ "AI", "AI", "--", "WD", "WD", "HI", "AI", "HI",
		/*Mir1*/ "WD", "WD", "WD", "--", "AI", "AI", "HI", "AI",
		/*Mir2*/ "WD", "WD", "WD", "AI", "--", "AI", "HI", "AI",
		/*LP*/   "PI", "PI", "PI", "AI", "AI", "--", "PI", "AI",
		/*APT*/  "AI", "AI", "AI", "HI", "WI", "PI", "--", "PI",
		/*MPT*/  "HI", "HI", "HI", "AI", "AI", "AI", "PI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


void tUni_9Edge::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlanet;
	Map->fTacticalMapIcon['J'] = kObjectPlanet;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['L'] = kObjectEnemyShip;
}	

tMapTerrain tUni_9Edge::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_9Edge::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );	
}


void tUni_9Edge::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_msg;
}


