/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_X1M1ASmuggler.h"
#include "PlayerTeam.h"
#include "SmugglerTeam1.h"
#include "SmugglerTeam2.h"
#include "FreighterTeam.h"
#include "OrionTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	// Failed to get the drug smuggler
	if( ( !playerTeam->mCappedSmuggler2() ) && ( !playerTeam->mKilledSmuggler2() ) )
	{
		// Didn't get the artifacts smuggler either
		if( ( !playerTeam->mCappedSmuggler1() ) && ( !playerTeam->mKilledSmuggler1() ) )
		{
			return kDevastatingDefeat;
		}

		// At least got the artifacts smuggler
		else
			return kDefeat;
	}

	// Got the drug smuggler
	if( ( playerTeam->mCappedSmuggler2() ) || ( playerTeam->mKilledSmuggler2() ) )
	{
		// Also got the artifacts smuggler
		if( ( playerTeam->mCappedSmuggler1() ) || ( playerTeam->mKilledSmuggler1() ) )
		{
			return kAstoundingVictory;
		}

		// Didn't get the artifacts smuggler
		else
			return kVictory;
	}

	// All other conditions are a defeat
	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* smuggler1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam1 ) );
	if( !smuggler1 )
		return;

	tTeamInfo* smuggler2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam2 ) );
	if( !smuggler2 )
		return;

	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	if( !freighterTeam )
		return;

	tOrionTeam* orionTeam = dynamic_cast<tOrionTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) ) );
	if( !orionTeam )
			return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}			
		}
		break;

	case tFreighterTeam::kFreighter:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kFreighterDead_msg );
		}
		break;

	case tSmugglerTeam1::kSmuggler1:
		{
			playerTeam->mSetKilledSmuggler1();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kKilledSmuggler1_msg );

			// Orions disengage when their buddy is gone
			if( orionTeam->mShipsRemainingInPlay() )
			{
				orionTeam->mRunAway();
			}

			// Player wins if both smugglers are gone
			if( !smuggler2->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tSmugglerTeam2::kSmuggler2:
		{
			playerTeam->mSetKilledSmuggler2();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kKilledSmuggler2_msg );

			// Player wins if both smugglers are gone
			if( !smuggler1->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* smuggler1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam1 ) );
	if( !smuggler1 )
		return;

	tTeamInfo* smuggler2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam2 ) );
	if( !smuggler2 )
		return;

	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	if( !freighterTeam )
		return;

	tOrionTeam* orionTeam = dynamic_cast<tOrionTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) ) );
	if( !orionTeam )
			return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;
	

	case tFreighterTeam::kFreighter:
		{
			// If the player caps the innocent freighter, the game is over
			if( ship->mGetTeam() == kPlayerTeam )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedInnocentFreighter_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
				return;
			}
		}
		break;

	case tSmugglerTeam1::kSmuggler1:
		{
			playerTeam->mSetCappedSmuggler1();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedSmuggler1_msg );

			// Orions disengage when their buddy is gone
			if( orionTeam->mShipsRemainingInPlay() )
			{
				orionTeam->mRunAway();
			}

			// Player wins if both smugglers are gone
			if( !smuggler2->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tSmugglerTeam2::kSmuggler2:
		{
			playerTeam->mSetCappedSmuggler2();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kCappedSmuggler2_msg );

			// Player wins if both smugglers are gone
			if( !smuggler1->mShipsRemainingInPlay() )
			{
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* smuggler1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam1 ) );
	if( !smuggler1 )
		return;

	tTeamInfo* smuggler2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam2 ) );
	if( !smuggler2 )
		return;

	tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
	if( !freighterTeam )
		return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

