/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "OrionTeam.h"
#include "OrionTeamBaseState.h"
#include "OrionShip.h"
#include "Uni_X1M1ASmuggler.h"
#include "MissionText.h"
#include "ShipList_API.h"


tClassAllocator<tOrionTeam> gOrionTeamAllocator;

void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamBaseState() );
	mGotoState( typeid( tOrionTeamBaseState ) );
}

void tOrionTeam::mSetupCommState()
{}

void tOrionTeam::mCreateShipsForTeam()
{
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tOrionTeam", shipScriptDescription );
}

void tOrionTeam::mCreateOrions()
{
	gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateOrions\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
		if( !playerTeam )
				return;

        int32 fShiplistCount;

		ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
		sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		int32 nCurrentID = kOrion1;
		int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();  // possible bug: ships that player has captured will count. oh well. --FS
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
		float nVarPercent;

		if( diffLevel == 0 )
		{
				nVarPercent = 0.65f;
		}
		else if( diffLevel == 1 )
		{
				nVarPercent = 1.0f;
		}
		else if( diffLevel == 2 )
		{
				nVarPercent = 1.25f;
		}

		for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
		{
			tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
			if( currentShip )
			{
				int32 thisShipBPV = int(currentShip->mGetShipBPV() * nVarPercent);

				// now we pick a ship with shiplist API. If it fails to pick one, use the old way.
				ShipList ship_sl = sl;
				fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.925), int(thisShipBPV * 1.075), true);
				gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
					int(thisShipBPV * 0.925), int(thisShipBPV * 1.075), fShiplistCount);
				gST.listShiplistContent(ship_sl);

				Ship s1;
				if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
					gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
						s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
					gST.mTrackOrions( s1 );

					mCreateShip( 
						typeid( tOrionShip ), 
						s1.shipCode.c_str(),
						kStartPosition_K,
						kNoMetaShipID,
						nCurrentID,
						0, 0, 0, 0,
						NULL,
						kDefaultShipOptions );
				} else {
					gST.mDebug_fprintf("Failed to pick ship #%d. Using old method. ShipBPV: %d\n", shipNumber, thisShipBPV);

					eClassTypes thisShipClass = currentShip->mGetClassType();
					mCreateShip( 
						typeid( tOrionShip ), 
						thisShipBPV, 
						thisShipClass, 
						0, 
						kStartPosition_K, 
						nCurrentID, 
						0, 0, 0, 0, 
						NULL, 
						kDefaultShipOptions );
				}
				nCurrentID++;
			}
		}
	} else {
		mOldCreateOrions(); // no enhanced shiplist at all. Go to old create code. --FS
	}
	fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kOrionsAppear_msg );

	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateOrions\n");
}

void tOrionTeam::mOldCreateOrions()
{
	gST.mDebug_fprintf("---- Entering: tOrionTeam::mOldCreateOrions\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
			return;

	int32 nStartPos = kStartPosition_K;
	int32 nCurrentID = kOrion1;
	int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();  // possible bug: ships that player has captured will count. oh well. --FS
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;

	if( diffLevel == 0 )
	{
			nVarPercent = 0.65f;
	}
	else if( diffLevel == 1 )
	{
			nVarPercent = 1.0f;
	}
	else if( diffLevel == 2 )
	{
			nVarPercent = 1.25f;
	}

	for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
	{
		tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );

		if( currentShip )
		{
			eClassTypes thisShipClass = currentShip->mGetClassType();
			int32 thisShipBPV = currentShip->mGetShipBPV();

			mCreateShip( 
				typeid( tOrionShip ), 
				thisShipBPV * nVarPercent, 
				thisShipClass, 
				0, 
				nStartPos, 
				nCurrentID, 
				0,0,0,0, 
				NULL, 
				kDefaultShipOptions );
			nCurrentID++;
		}
	}

	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateOrions\n");
}

void tOrionTeam::mRunAway()
{
	tShipIterator orionsI = mGetFirstShip();

	while( *orionsI )
	{
		orionsI->mDisengage( kByMapEdge, kAbsolutePriority );
		orionsI++;
	}

	mSetRelations( static_cast<eTeamID>( kPlayerTeam ), kTeamNeutral, kGoalIgnore, 1.0f );
	fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kOrionsRunAway_msg );
}
