/********************************************************************************************************\

	File Name:	PlayerShipBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "PlayerShipBaseState.h"
#include "Uni_X1M1ASmuggler.h"
#include "MissionText.h"
#include "PlayerTeam.h"

tPlayerShipBaseState::tPlayerShipBaseState()
{
	fScanning = false;
}

void tPlayerShipBaseState::mBeginState()
{}
void tPlayerShipBaseState::mEndState()
{}

void tPlayerShipBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	fChildActor->mSetShipCallbackEvents( kNotifyBeginTractor );
}

void tPlayerShipBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{
	if( deepScanOn )
	{
		fScanning = true;
		mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );
	}

	else
	{
		fScanning = false;
		mSetTimedInterval( kTimerKillTimer, kScan_timer, 0.0f );
	}
}

void tPlayerShipBaseState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kScan_timer:
		{
			if( !fScanning )
				return;

			mSetTimedInterval( kTimerTurnsSet, kScan_timer, 0.75f );

			tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
			if( !playerTeam )
					return;

			tTeamInfo* smuggler1 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam1 ) );
			if( !smuggler1 )
					return;

			tTeamInfo* smuggler2 = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kSmugglerTeam2 ) );
			if( !smuggler2 )
					return;

			tTeamInfo* freighterTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kFreighterTeam ) );
			if( !freighterTeam )
					return;


			int32 ScanTargetID = fChildActor->mGetShipTargetID();
			int32 ScanRange = 20;

			tShipIterator smuggler1I = smuggler1->mGetFirstShip();
			tShipIterator smuggler2I = smuggler2->mGetFirstShip();
			tShipIterator freighterI = freighterTeam->mGetFirstShip();

			// smuggler1
			if( ( *smuggler1I ) && ( ScanTargetID == smuggler1I->mGetShipID() ) && ( !playerTeam->mScannedSmuggler1() ) )
			{
				int32 range = fChildActor->mGetTacticalDistance( *smuggler1I );

				if( range < ScanRange )
				{
					playerTeam->mSetScannedSmuggler1();
					smuggler1I->mChangeName( kArtifacts_smuggler_name );
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScannedSmuggler1_msg );
					return;
				}

				else if( ( range > ScanRange ) && ( !playerTeam->mNotifiedScanRange() ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanRange_msg );
					return;
				}

				else if( ( range > ScanRange ) && ( playerTeam->mNotifiedScanRange() ) )
				{
					return;
				}
			}

			// smuggler2
			if( ( *smuggler2I ) && ( ScanTargetID == smuggler2I->mGetShipID() ) && ( !playerTeam->mScannedSmuggler2() ) )
			{
				int32 range = fChildActor->mGetTacticalDistance( *smuggler2I );
				
				if( range < ScanRange )
				{
					playerTeam->mSetScannedSmuggler2();
					smuggler2I->mChangeName( kDrugSmuggler_name );
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScannedSmuggler2_msg );
					return;
				}

				else if( ( range > ScanRange ) && ( !playerTeam->mNotifiedScanRange() ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanRange_msg );
					return;
				}

				else if( ( range > ScanRange ) && ( playerTeam->mNotifiedScanRange() ) )
				{
					return;
				}
			}

			// freighter
			if( ( *freighterI ) && ( ScanTargetID == freighterI->mGetShipID() ) && ( !playerTeam->mScannedFreighter() ) )
			{
				int32 range = fChildActor->mGetTacticalDistance( *freighterI );

				if( range < ScanRange )
				{
					playerTeam->mSetScannedFreighter();
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScannedFreighter_msg );
					return;
				}

				else if( ( range > ScanRange ) && ( !playerTeam->mNotifiedScanRange() ) )
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kScanRange_msg );
					return;
				}

				else if( ( range > ScanRange ) && ( playerTeam->mNotifiedScanRange() ) )
				{
					return;
				}
			}
		}
		break;

	default:
		break;
	}
}

void tPlayerShipBaseState::mOnDeath( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCapture( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDistance( tShipInfo* ship )
{}
void tPlayerShipBaseState::mAtDestination( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tPlayerShipBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}

void tPlayerShipBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tPlayerShipBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnEndOrbit( tShipInfo* ship )
{}

void tPlayerShipBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tPlayerShipBaseState::mOnFlashingButtonPressed()
{}
void tPlayerShipBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tPlayerShipBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tPlayerShipBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tPlayerShipBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tPlayerShipBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tPlayerShipBaseState::mOnHostChanged()
{}

void tPlayerShipBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tPlayerShipBaseState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

