/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "CommonDefs.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fCappedSmuggler1 = false;
	fCappedSmuggler2 = false;
	fKilledSmuggler1 = false;
	fKilledSmuggler2 = false;
	fSmuggler1GotAway = false;
	fSmuggler2GotAway = false;
	fScannedSmuggler1 = false;
	fScannedSmuggler2 = false;
	fScannedFreighter = false;
	fNotifiedScanRange = false;
	fTractoredSmuggler1 = false;
	fTractoredSmuggler2 = false;
	fCappedFreighter = false;
	fFiredOnFreighter = false;
	fDied = false;
}

void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if( fDied )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_PlayerDied_msg );
		mScheduleNextMission( fVictoryCondition );
		return;
	}

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kAstoundingVictory_msg );	break;
	case kVictory:
		fMissionInfo->mAddToDebriefingList( fTeam, kVictory_msg );				break;
	case kDraw:
		fMissionInfo->mAddToDebriefingList( fTeam, kDraw_msg );					break;
	case kDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDefeat_msg );				break;
	case kDevastatingDefeat:
		fMissionInfo->mAddToDebriefingList( fTeam, kDevastatingDefeat_msg );	break;
	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
			mScheduleNextMission( fVictoryCondition );
			return;
		}
		break;
	}


	// Smuggler 1
	// capped 
	if( fCappedSmuggler1 )
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_Capped1_msg );

	// killed
	else if( fKilledSmuggler1 )
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_Killed1_msg );

	// didn't get him 
	else if( ( !fCappedSmuggler1 ) && ( !fKilledSmuggler1 ) )
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_Missed1_msg );

	// Smuggler 2
	// capped 
	if( fCappedSmuggler2 )
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_Capped2_msg );

	// killed
	else if( fKilledSmuggler2 )
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_Killed2_msg );

	// didn't get him 
	else if( ( !fCappedSmuggler2 ) && ( !fKilledSmuggler2 ) )
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_Missed2_msg );

		
	// Fired on or attacked a friendly freighter
	if( ( fFiredOnFreighter ) || ( fCappedFreighter ) )
		fMissionInfo->mAddToDebriefingList( fTeam, kDeb_AttackedFreighter_msg );

	mScheduleNextMission( fVictoryCondition );
}

void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G,
				 MetaID,
				 shipNumber, 0, 0, -1, -1,
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast<eShipOptions>( kCanAddToFleetUponCapture | kCanBeCarrier | kStartPositionCanBeOffset ) );

	shipNumber++;

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}


void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 450;

	if( ( fMissionScheduler.fVictoryLevel == kAstoundingVictory ) || ( fMissionScheduler.fVictoryLevel == kVictory ) )
	{
		fMissionScheduler.fNextMissionTitle = "Uni_X1M2Trial";
	}
	else
	{
		fMissionScheduler.fNextMissionTitle = "Uni_X1M1FInsurrection";
	}
}