/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_X1M2Trial.h"
#include "PlayerTeam.h"
#include "TransportTeam.h"
#include "AllyTeam.h"
#include "OrionTeam.h"
#include "PlanetTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{
	mSetTimedInterval( kTimerTurnsSet, kOrion_timer, 5.0f );
}

void tBaseVictoryState::mEndState()
{}

void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	switch( timerID )
	{
	case kOrion_timer:
		{
			tOrionTeam* orionTeam = dynamic_cast<tOrionTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) ) );
			if( !orionTeam )
				return;

			orionTeam->mCreateOrions();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kOrionsArrive_msg );

			tAllyTeam* allyTeam = dynamic_cast<tAllyTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) ) );
			if( !allyTeam )
				return;
			allyTeam->mClearDefenseCommands();
		}
		break;

	default:
		break;
	}
}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	// Prison transport reached planet
	if( playerTeam->mTransportReachedPlanet() )
		return kAstoundingVictory;

	// all other conditions are failures
	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	ASSERT( playerTeam );
	if( !playerTeam ) return;

	tOrionTeam* orionTeam = dynamic_cast<tOrionTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) ) );
	ASSERT( orionTeam );
	if( !orionTeam ) return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}			
		}
		break;

	case tTransportTeam::kTransport:
		{
			playerTeam->mSetTransportDead();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTransportDead_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		}
		break;

	case tOrionTeam::kOrion1:
	case tOrionTeam::kOrion2:
	case tOrionTeam::kOrion3:
	case tOrionTeam::kOrion4:
	case tOrionTeam::kOrion5:
	case tOrionTeam::kOrion6:
		{
			if( !orionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetTransportReachedPlanet();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	ASSERT( playerTeam );
	if( !playerTeam ) return;

	tOrionTeam* orionTeam = dynamic_cast<tOrionTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) ) );
	ASSERT( orionTeam );
	if( !orionTeam ) return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

	case tTransportTeam::kTransport:
		{
			playerTeam->mSetTransportCapped();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kTransportCapped_msg );
			gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
		}
		break;

	case tOrionTeam::kOrion1:
	case tOrionTeam::kOrion2:
	case tOrionTeam::kOrion3:
	case tOrionTeam::kOrion4:
	case tOrionTeam::kOrion5:
	case tOrionTeam::kOrion6:
		{
			if( !orionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetTransportReachedPlanet();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	ASSERT( playerTeam );
	if( !playerTeam ) return;

	tOrionTeam* orionTeam = dynamic_cast<tOrionTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) ) );
	ASSERT( orionTeam );
	if( !orionTeam ) return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;

	case tOrionTeam::kOrion1:
	case tOrionTeam::kOrion2:
	case tOrionTeam::kOrion3:
	case tOrionTeam::kOrion4:
	case tOrionTeam::kOrion5:
	case tOrionTeam::kOrion6:
		{
			if( !orionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetTransportReachedPlanet();
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

