/********************************************************************************************************\

	Mission:	Trial Run
	Script:		"Uni_X1M2Trial"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		08/30/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_X1M2Trial.h"
#include "PlayerTeam.h"
#include "TransportTeam.h"
#include "AllyTeam.h"
#include "OrionTeam.h"
#include "PlanetTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X1M2Trial );

tUni_X1M2Trial::tUni_X1M2Trial()
{}

int32 tUni_X1M2Trial::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_X1M2Trial::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecFed | kSpecRomulan | kSpecHydran | kSpecGorn,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Transport Team",
												static_cast< eTeamID >( kTransportTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecRomulan | kSpecHydran | kSpecGorn,
												kSameAs | kPlayerTeam,
												typeid( tTransportTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Ally Team",
												static_cast< eTeamID >( kAllyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecRomulan | kSpecHydran | kSpecGorn,
												kSameAs | kPlayerTeam,
												typeid( tAllyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Orion Team",
												static_cast< eTeamID >( kOrionTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecOrion,
												kAnyRace,
												typeid( tOrionTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
	const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

int32 tUni_X1M2Trial::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_X1M2Trial::mCreateExternalShips( int32 Mode )
{}

int32 tUni_X1M2Trial::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_X1M2Trial::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_X1M2Trial::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_X1M2Trial::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_X1M2Trial::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kTransportTeam,
		kAllyTeam,
		kOrionTeam,
		kPlanetTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Tran = ktransportTeam,
		//Ally = kAllyTeam,
		//Orin = kOrionTeam,
		//Plan = kPlanetTeam,

		//	     Plyr  Tran  Ally  Orin  Plan
		/*Plyr*/ "--", "AI", "AI", "WD", "AI",
		/*Tran*/ "AI", "--", "AI", "WI", "AI",
		/*Ally*/ "AI", "AI", "--", "WD", "AI",
		/*Orin*/ "WD", "WX", "WD", "--", "HI",
		/*Plan*/ "AI", "AI", "AI", "HI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tUni_X1M2Trial::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['I'] = kObjectAlliedShip;
	Map->fTacticalMapIcon['Z'] = kObjectPlayerBase;
}

tMapTerrain tUni_X1M2Trial::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_X1M2Trial::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}

void tUni_X1M2Trial::mDefineBriefings( void )
{
	for( int32 nRace = kFederation; nRace < kMirak + 1; nRace++ )
	{
		fBriefingIndex[kPlayerTeam][nRace] = kBriefing_msg;
	}
}