//	<Name>	InterceptorVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "InterceptorVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Uni_X2M1Interceptor.h"
#include "InterceptorVictory.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
//#include "PlayerTeamShipGameOverState.h"

              
tInterceptorVictoryState::tInterceptorVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tInterceptorVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fEnemyShipsInPlay = 1;
	fPlayerShipsInPlay = 1;
	fFreighterDestroyed = false;
	fDestroyedByHole = false;
	fGotInfo = false;
}

void tInterceptorVictoryState::mBeginState()
{

}

void tInterceptorVictoryState::mEndState()
{
}

eVictoryLevels tInterceptorVictoryState::mCalcVictoryStatus( eTeamID ID )
{

	if(  fGotInfo && fDestroyedByHole )
	{
		return kAstoundingVictory;
	}
	else if( fGotInfo )
	{
		return kDraw;
	}
	else if( fDestroyedByHole )
	{
		return kDefeat;
	}
	else
	{
		return kDevastatingDefeat;
	}

}
int32 tInterceptorVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}
void tInterceptorVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eRaceName  playerRace;
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	playerRace = playerTeam->mGetRace();


	if( ship->mGetTeam() == kEnemyTeam )
	{
		fEnemyShipsInPlay = mShipsInPlay( enemyTeam );
	}
	else if( ship->mGetTeam() == ( playerTeam->mGetTeam() ) )
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );
	}

	if( fPlayerShipsInPlay == 0 )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerDestroyed );
	}

}
void tInterceptorVictoryState::mOnDeath( tShipInfo* ship )
{
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );

	if( ship->mGetOriginalTeam() == enemyTeam->mGetTeam() )
	{
		fFreighterDestroyed = true;

		if( ship->mGetTacticalDistance( 13, 12 ) < 10 )
		{
			fDestroyedByHole = true;
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kHoleDestroyedFreighter_msg );
		}
		else
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kFreighterDestroyed_msg );
		}
		mSetTimedInterval( kTimerTurnsSet, fGameOverID, 0.5f );

	}

	mCheckAllTeams( ship );
}
void tInterceptorVictoryState::mOnCapture( tShipInfo* ship )
{
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );

	if( ship->mGetOriginalTeam() == enemyTeam->mGetTeam() )
	{
		ship->mSetShipMaxSpeed( 0 );
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kCapturedFreighter_msg );
		mSetTimedInterval( kTimerTurnsSet, fDescrambledID, 0.75f );
	}

}

void tInterceptorVictoryState::mOnDisengaged( tShipInfo* ship )
{
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kEnemyTeam ) );
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	fPlayerShipsInPlay = mShipsInPlay( playerTeam );

	if ( ( ship->mGetTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		if ( fPlayerShipsInPlay < 1 ) 
		{
			if( fDestroyedByHole && fGotInfo )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
			}
			else
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
			}
		}
	}
	else if ( ship->mGetTeam() == ( enemyTeam->mGetTeam() ) )
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kFreighterEscaped_msg );
		mSetTimedInterval( kTimerTurnsSet, fGameOverID, 0.5f );
	}
}
void tInterceptorVictoryState::mOnTimedInterval( int32 timerID )
{
	if( timerID == fDescrambledID )
	{
		if( !fFreighterDestroyed )
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kDescrambled_msg );
			fGotInfo = true;
		}
	}
	else if ( timerID == fGameOverID )
	{
		if( fDestroyedByHole && fGotInfo )
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerWins );
		}
		else 
		{
			gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam, kPlayerLoses );
		}
	}
}
