/********************************************************************************************************\

	Mission:	
	Script:		"Uni_X2M1Interceptor"
	Descript:	<DESCRIPTION>

	Created by:		David Ferrell 04/10/2000
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG> log its big its heavy its wood

\********************************************************************************************************/

#include "Uni_X2M1Interceptor.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "InterceptorVictory.h"
#include "ShipList_API.h"
#include "DraftTeam.h"

DEFINE_SCRIPT_CLASS( tUni_X2M1Interceptor );

tUni_X2M1Interceptor::tUni_X2M1Interceptor()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tUni_X2M1Interceptor::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;
	
/*	
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyHex, kAnyRaceHex );
	Specs->fSectorType = kEnemyBorder | kWarzone | kUnexplored | kEnemyColonial;
	Specs->fPlayerRace =  kSpecAnyRace;//kSpecKling | kSpecLyran | kSpecGorn | kSpecMirak;
	Specs->fEnemyRace = kSpecISC;
	Specs->fNumShips = kMissSpecOneShip;
	Specs->fMinBPV = 100;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 0;
	Specs->fMaxDate = 99;
	Specs->fWait = kUniqueMission;
	Specs->fMenuStates = kNoMenuButtons;
*/

		// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID >( kPlayerTeam ),
															kMaxChance,
															kTrue,
															kPrimaryTeam,
															kMinChance,
															kSpecKling | kSpecLyran | kSpecGorn | kSpecMirak,
															kAnyRace,
															typeid( tPlayerTeam ), kNoTeamTag, Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID >( kEnemyTeam ),
															kMaxChance,
															kTrue,
															kPlayableTeam,
															kMaxChance,
															kSpecISC,
															kAnyRace,
															typeid( tEnemyTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	const tTeamSpec& draft3Team = mCreateTeamSpec(  "Draft 3 Team",
                                                   static_cast< eTeamID >( kDraft3Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft4Team = mCreateTeamSpec(  "Draft 4 Team",
                                                   static_cast< eTeamID >( kDraft4Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );



	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
	Specs->mSetupTeam( draft3Team );
    Specs->mSetupTeam( draft4Team );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tUni_X2M1Interceptor::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tUni_X2M1Interceptor::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}
//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tUni_X2M1Interceptor::mStartMission( void )
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
	};


	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = PlayersTeam,
		//Nme  = EnemyTeam,

		//	     Plyr  Nme
		/*Plyr*/ "--", "WC",
		/*Nme */ "WI", "--",
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tUni_X2M1Interceptor::mNewInternalShip( tShipInfo* Ship )
{
}
void tUni_X2M1Interceptor::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
}
void tUni_X2M1Interceptor::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tUni_X2M1Interceptor::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kKlingon] =		kKliBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] =		kLyrBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] =		kGorBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] =		kMirakBriefing_msg;

}


