//	<Name>	EnemyTeam.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "tclient.h"
#include "ScriptInterface.h"
#include "EnemyTeam.h"
#include "EnemyTeamShip.h"
#include "EnemyVisitorShip.h"
#include "MissionText.h"
#include "Uni_X2M2Reclamation.h"
#include "ShipList_API.h"

tEnemyTeam::tEnemyTeam()
{
	fPlayerShipDetected = false;
}
tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

void tEnemyTeam::mSetupState()
{}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");

    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 playerBPV = playerTeam->mGetCombatBPV();
        int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
        int32 nPCBPV;
        switch ( fMissionInfo->mGetDifficultyLevel() )
        {
        case 0:
            nPCBPV = playerBPV * 0.2;  break;
        case 1:
            nPCBPV = playerBPV * 0.4;   break;
        case 2:
            nPCBPV = playerBPV * 0.6;  break;
        default:
            break;
        }

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		// this kinda has a lower chance because it's player BPV * 0.2
		ShipList ship1_sl = sl;
		fShiplistCount = ship1_sl.Subset( ship1_sl.kSubsetBPV, int(nPCBPV * 0.925), int(nPCBPV * 1.075), true);
        gST.mDebug_fprintf("ship1_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(nPCBPV * 0.925), int(nPCBPV * 1.075), fShiplistCount);
        gST.listShiplistContent(ship1_sl);

		ShipList ship2_sl = sl;
		int32 ship2BPV = nPCBPV * 1.2;
		fShiplistCount = ship2_sl.Subset( ship2_sl.kSubsetBPV, int(ship2BPV * 0.925), int(ship2BPV * 1.075), true);
        gST.mDebug_fprintf("ship2_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(ship2BPV * 0.925), int(ship2BPV * 1.075), fShiplistCount);
        gST.listShiplistContent(ship2_sl);

		ShipList ship3_sl = sl;
		int32 ship3BPV = nPCBPV * 1.3;
		fShiplistCount = ship3_sl.Subset( ship3_sl.kSubsetBPV, int(ship3BPV * 0.925), int(ship3BPV * 1.075), true);
        gST.mDebug_fprintf("ship3_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(ship3BPV * 0.925), int(ship3BPV * 1.075), fShiplistCount);
        gST.listShiplistContent(ship3_sl);

		// BPV ranges are too small to work. Load up from a bigger range
		if ((ship2_sl.size() == 0) && (ship3_sl.size() != 0)) {
			gST.mDebug_fprintf("ship2_sl is empty. Copying from ship3_sl.\n");
			ship2_sl = ship3_sl;
		}
        if ((ship1_sl.size() == 0) && (ship2_sl.size() != 0)) {
			gST.mDebug_fprintf("ship3_sl is empty. Copying from ship2_sl.\n");
			ship1_sl = ship2_sl;
		}

		if ((ship1_sl.size() > 0) && (ship1_sl.size() > 0) && (ship1_sl.size() > 0)) {

			Ship s1;
			ship1_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			Ship s2;
			ship2_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

			Ship s3;
			ship3_sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

            mCreateShip(
                typeid( tEnemyTeamShip ),
                s1.shipCode.c_str(),
                kStartPosition_L,
                kNoMetaShipID,
                kEnemyShip1,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

			mCreateShip(
                typeid( tEnemyTeamShip ),
                s2.shipCode.c_str(),
                kStartPosition_M,
                kNoMetaShipID,
                kEnemyShip2,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

		    mCreateShip(
                typeid( tEnemyVisitorShip ),
                s3.shipCode.c_str(),
                kStartPosition_O,
                kNoMetaShipID,
                kEnemyShip3,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Entering: tEnemyTeam::mCreateShipsForTeam\n");
}


void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.2;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 0.4;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 0.6;
	}
	mCreateShip(
		typeid( tEnemyTeamShip ), 
		nPCBPV, 
		kClassFrigate, 
		kClassHeavyCruiser, 
		100, 
		kStartPosition_L,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );
	mCreateShip(
		typeid( tEnemyTeamShip ), 
		nPCBPV * 1.2, 
		kClassFrigate, 
		kClassHeavyCruiser, 
		100, 
		kStartPosition_M,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );
	mCreateShip(
		typeid( tEnemyVisitorShip ), 
		nPCBPV * 1.3, 
		kClassFrigate, 
		kClassHeavyCruiser, 
		100, 
		kStartPosition_O,
		-1, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}
void tEnemyTeam::mSetPlayerShipDetected( bool setValue )
{
	fPlayerShipDetected = setValue;
}
bool tEnemyTeam::mGetPlayerShipDetected()
{
	return fPlayerShipDetected;
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}
