//	<Name>	PlayerTeam.cpp
//
//
//	<Info>	DESCRIPTION
//
#include "Uni_X2M2Reclamation.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamInitialState.h"
#include "ReclamationVictory.h"
#include "MissionText.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

tPlayerTeam::tPlayerTeam()
{
	fBuoyScanned = false;
	fSentCode = false;
	fLegendRescued = false;
}
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tReclamationVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels VictoryLevel )
{
	eRaceName  playerRace;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
    playerRace = playerTeam->mGetRace();

	// return the index to the briefing message we want to use
	if( VictoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kLeftEarly_msg );
	}
	else
	{
		switch( playerRace )
		{
			case kKlingon:
				if( VictoryLevel == kAstoundingVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kKliAstoundingVictory_msg );
				}
				else if( VictoryLevel == kVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kKliVictory_msg );
				}
				else
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kKliDevastatingDefeat_msg );
				}
			break;
			case kLyran:
				if( VictoryLevel == kAstoundingVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kLyrAstoundingVictory_msg );
				}
				else if( VictoryLevel == kVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kLyrVictory_msg );
				}
				else
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kLyrDevastatingDefeat_msg );
				}
			break;
			case kGorn:
				if( VictoryLevel == kAstoundingVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kGorAstoundingVictory_msg );
				}
				else if( VictoryLevel == kVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kGorVictory_msg );
				}
				else
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kGorDevastatingDefeat_msg );
				}
			break;
			case kMirak:
				if( VictoryLevel == kAstoundingVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kMirakAstoundingVictory_msg );
				}
				else if( VictoryLevel == kVictory )
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kMirakVictory_msg );
				}
				else
				{
					fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam,  kMirakDevastatingDefeat_msg );
				}
			break;
		}//end switch
	}
}
void tPlayerTeam::mCreateShipsForTeam( int count, tShipInGame* ships[] )
{
}
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamInitialState( ) );
	mGotoState( typeid( tPlayerTeamInitialState ) );
}
void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();
	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}
}
void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int32 shipNumber = kPlayerShip1;

	mCreateShip( typeid( tPlayerTeamShip ), 
				 shipScriptDescription.fClassName.c_str(), 
				 kStartPosition_G + shipScriptDescription.fTeamID,
				 MetaID,
				 shipNumber, 
				 0, 0, -1, -1, //Default data,so we can set the custom name
				 shipScriptDescription.fCustomName.c_str(),
				 static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	shipNumber++;

	gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mSetBuoyScanned( bool setValue )
{
	fBuoyScanned = setValue;
}
bool tPlayerTeam::mGetBuoyScanned()
{
	return fBuoyScanned;
}
void tPlayerTeam::mSetSentCode( bool setValue )
{
	fSentCode = setValue;
}
bool tPlayerTeam::mGetSentCode()
{
	return fSentCode;
}
void tPlayerTeam::mSetLegendRescued( bool setValue )
{
	fLegendRescued = setValue;
}
bool tPlayerTeam::mGetLegendRescued()
{
	return fLegendRescued;
}
void tPlayerTeam::mSetScientistsArrived( bool setValue )
{
	fScientistsArrived = setValue;
}
bool tPlayerTeam::mGetScientistsArrived()
{
	return fScientistsArrived;
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 450;
	fMissionScheduler.fNextMissionTitle = "Uni_X2M3Apes";
}