//	<Name>	ReclamationVictoryState.cpp
//
//
//	<Info>	DESCRIPTION
//

#include "ReclamationVictoryState.h"
#include "ShipInfo.h"
#include "ScriptInterface.h"

#include "Uni_X2M2Reclamation.h"
#include "ReclamationVictory.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "PlayerTeamShip.h"
#include "PlayerTeamShipGameOverState.h"

              
tReclamationVictoryState::tReclamationVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tReclamationVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{
	fEnemyShipsInPlay = 4;
	fBaseInPlay = 1;
	fPlayerShipsInPlay = 1;
}

void tReclamationVictoryState::mBeginState()
{

}

void tReclamationVictoryState::mEndState()
{
}

eVictoryLevels tReclamationVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	bool goalAcomplished = playerTeam->mGetLegendRescued();
	bool usedCode = playerTeam->mGetSentCode();

	if(  goalAcomplished && usedCode )
	{
		return kAstoundingVictory;
	}
	else if( goalAcomplished )
	{
		return kVictory;
	}
	else
	{
		return kDevastatingDefeat;
	}
}

int32 tReclamationVictoryState::mShipsInPlay( tTeamInfo* teamID )
{
	int32 teamShipsInPlay = teamID->mShipsRemainingInPlay();

	return teamShipsInPlay;
}

void tReclamationVictoryState::mCheckAllTeams( tShipInfo* ship )
{
	eRaceName  playerRace;
	tTeamInfo* enemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kEnemyTeam ) );
	tTeamInfo* baseTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kBaseTeam ) );
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	playerRace = playerTeam->mGetRace();


	if( ship->mGetOriginalTeam() == kEnemyTeam )
	{
		fEnemyShipsInPlay = mShipsInPlay( enemyTeam );
	}
	else if( ship->mGetOriginalTeam() == kBaseTeam )
	{
		fBaseInPlay = mShipsInPlay( baseTeam );
	}
	else if( ship->mGetOriginalTeam() == ( playerTeam->mGetTeam() ) )
	{
		fPlayerShipsInPlay = mShipsInPlay( playerTeam );
	}

	if( fEnemyShipsInPlay == 0 && ( playerTeam->mGetLegendRescued() ) )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
	}
	if( fEnemyShipsInPlay == 0 && ( ! playerTeam->mGetLegendRescued() ) )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerLoses );
	}
	if( fPlayerShipsInPlay == 0 )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerDestroyed );
	}

}
void tReclamationVictoryState::mOnDeath( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}
void tReclamationVictoryState::mOnCapture( tShipInfo* ship )
{
	mCheckAllTeams( ship );
}

void tReclamationVictoryState::mOnDisengaged( tShipInfo* ship )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) ) );
	Assert( playerTeam );

	fPlayerShipsInPlay = mShipsInPlay( playerTeam );

	if ( ( ship->mGetTeam() ) == ( playerTeam->mGetTeam() ) ) 
	{
		if ( fPlayerShipsInPlay < 1 ) 
		{
			if( playerTeam->mGetLegendRescued() )
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
			}
			else
			{
				gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerLoses );
			}
		}
	}

}
void tReclamationVictoryState::mOnStateChange( tActor* actor, tState* state, eStateEvent eventType )
{
	if( *actor == typeid( tPlayerTeamShip ) &&
		*state == typeid( tPlayerTeamShipGameOverState ) &&
		eventType == kStartingNewState )
	{
		gScriptInterface.mSetGameStatus( (eTeamID)kPlayerTeam,  kPlayerWins );
	}
}