/********************************************************************************************************

	Mission:	Reclamation Series 2 Mission 2
	Script:		"Uni_X2M2Reclamation"
	Descript:	<DESCRIPTION>

	Created by:		David Ferrell		03/27/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>...Log, it's big it's heavy it's wood
					FS  09/22/2012
					Era correct ships, drones, and upgraded fighters

********************************************************************************************************/

#include "Uni_X2M2Reclamation.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "VictoryCondition.h"
#include "PlayerTeam.h"
#include "EnemyTeam.h"
#include "BaseTeam.h"
#include "BuoyTeam.h"
#include "ReclamationVictory.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X2M2Reclamation );

tUni_X2M2Reclamation::tUni_X2M2Reclamation()
{
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tUni_X2M2Reclamation::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kShortBriefing_msg];
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

		// must create the teams in the script
	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1", static_cast< eTeamID >( kPlayerTeam ),
																kMaxChance,
																kFalse,
																kPrimaryTeam,
																kMaxChance,
																kSpecAnyRace,
																kAnyRace,
																typeid( tPlayerTeam ), 
																kNoTeamTag, 
																Messages[kPlayerTeam1Title_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2", static_cast< eTeamID >( kEnemyTeam ),
																kMaxChance,
																kTrue,
																kPlayableTeam,
																kMaxChance,
																kSpecAnyRace,
																kSameAs | kPlayerTeam,
																typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3", static_cast< eTeamID >( kBaseTeam ),
																kMaxChance,
																kTrue,
																kPlayableTeam,
																kMaxChance,
																kSpecAnyRace,
																kSameAs | kPlayerTeam,
																typeid( tBaseTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4", static_cast< eTeamID >( kBuoyTeam ),
																kMaxChance,
																kTrue,
																kPlayableTeam,
																kMaxChance,
																kSpecAnyRace,
																kSameAs | kPlayerTeam,
																typeid( tBuoyTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	//After teams are created set them up 
	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tUni_X2M2Reclamation::mInitializeStart( tDynaverseScriptInfo& info )
{


	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tUni_X2M2Reclamation::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}
//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tUni_X2M2Reclamation::mInitializeFinished( void )
{
	eRaceName  playerRace;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );//gScriptInterface.mPlayersTeam();//static_cast< eTeamID >( kPlayerTeam )
    playerRace = playerTeam->mGetRace();

	// return the index to the briefing message we want to use
	switch( playerRace )
	{
		case kKlingon:
			return( kKliBriefing_msg );
		break;
		case kLyran:
			return( kLyrBriefing_msg );
		break;
		case kGorn:
			return( kGorBriefing_msg );
		break;
		case kMirak:
			return( kMirakBriefing_msg );
		break;
		default:
			Assert( 0 );
			return( 0 );
		break;
	}
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tUni_X2M2Reclamation::mStartMission( void )
{
	tScript::mStartMission();
	
	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kBaseTeam,
		kBuoyTeam,
	};


	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = PlayersTeam,
		//Nme  = EnemyTeam,
		//Base = kBaseTeam,
		//Buoy = kBuoyTeam,

		//			Plyr	Nme		Base	Buoy
		/*Plyr*/	"--",	"HI",	"HI",	"NI",
		/*Nme */	"HI",	"--",	"AI",	"AI",
		/*Base*/	"HI",	"AI",	"--",	"AI",
		/*Buoy*/	"AI",	"AI",	"AI",	"--"
	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);

}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tUni_X2M2Reclamation::mNewInternalShip( tShipInfo* Ship )
{
}
void tUni_X2M2Reclamation::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['Z'] = kObjectEnemyBase;

	Map->fTacticalMapIcon['M'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['N'] = kObjectEnemyShip;

	Map->fTacticalMapIcon['O'] = kObjectEnemyShip;
}
void tUni_X2M2Reclamation::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}
void tUni_X2M2Reclamation::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kKlingon] =		kKliBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] =		kLyrBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] =		kGorBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] =		kMirakBriefing_msg;

}


