/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_X2M3Apes.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "ISCTeam.h"
#include "PlanetTeam.h"
#include "InvasionTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}

void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	if( playerTeam->mDied() )
	{
		return kDevastatingDefeat;
	}

	if( playerTeam->mEscortsDead() && ( playerTeam->mInvadersDead() ) )
	{
		return kAstoundingVictory;
	}
	if( playerTeam->mInvadersDead() && !playerTeam->mEscortsDead() )
	{
		return kVictory;
	}
	if( !playerTeam->mInvadersDead() )
	{
		return kDevastatingDefeat;
	}

	return kDevastatingDefeat;
}


void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
			return;

	tTeamInfo* iscTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kISCTeam ) );
	if( !iscTeam )
			return;

	tTeamInfo* invasionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kInvasionTeam ) );
	if( !invasionTeam )
			return;

	
	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				playerTeam->mSetISCCappedPlanet();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}			
		}
		break;
	
	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
		{
			if( !allyTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetAlliesDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAlliesDead_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllyDead_msg );
			}
		}
		break;

	case tISCTeam::kISC1:
	case tISCTeam::kISC2:
	case tISCTeam::kISC3:
		{
			if( !iscTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kISCTeamDead_msg );
				playerTeam->mSetEscortsDead();

				if( !invasionTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				}
			}

			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kISCDead_msg );
			}
		}
		break;

	case tInvasionTeam::kInvasion1:
	case tInvasionTeam::kInvasion2:
		{
			if( !invasionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetISCInvasionForceDead();
				playerTeam->mSetInvadersDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInvasionForceDead_msg );

				if( !iscTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				}
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInvasionShipDead_msg );
			}
		}
		break;
	}
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
			return;

	tTeamInfo* iscTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kISCTeam ) );
	if( !iscTeam )
			return;

	tTeamInfo* invasionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kInvasionTeam ) );
	if( !invasionTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				playerTeam->mSetISCCappedPlanet();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}			
		}
		break;

	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
		{
			if( !allyTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetAlliesDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAlliesDead_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllyDead_msg );
			}
		}
		break;

	case tISCTeam::kISC1:
	case tISCTeam::kISC2:
	case tISCTeam::kISC3:
		{
			if( !iscTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kISCTeamDead_msg );
				playerTeam->mSetEscortsDead();

				if( !invasionTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				}
			}

			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kISCDead_msg );
			}
		}
		break;

	case tInvasionTeam::kInvasion1:
	case tInvasionTeam::kInvasion2:
		{
			if( !invasionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetISCInvasionForceDead();
				playerTeam->mSetInvadersDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInvasionForceDead_msg );

				if( !iscTeam->mShipsRemainingInPlay() )
				{
					gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
				}
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInvasionShipDead_msg );
			}
		}
		break;

	case tPlanetTeam::kPlanet:
		{
			// Who got the planet?
			
			// Player or Ally team
			if( ( ship->mGetTeam() == kPlayerTeam ) || ( ship->mGetTeam() == kAllyTeam ) )
			{
				playerTeam->mSetCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kPlanetCapped_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}

			//The bad guys got it
			if( ( ship->mGetTeam() == kISCTeam ) || ( ship->mGetTeam() == kInvasionTeam ) )
			{
				playerTeam->mSetISCCappedPlanet();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kEnemyCapsPlanet_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerLoses );
			}
			break;
		}
	}
}

void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;

	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
			return;

	tTeamInfo* iscTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kISCTeam ) );
	if( !iscTeam )
			return;

	tTeamInfo* invasionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kInvasionTeam ) );
	if( !invasionTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if( invasionTeam->mShipsRemainingInPlay() )
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kInvasionByDefault_msg );
				playerTeam->mSetISCCappedPlanet();
			}
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}

void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

