/********************************************************************************************************\

	File Name:	PlanetTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "PlanetTeam.h"
#include "PlanetTeamBaseState.h"
#include "Planet.h"
#include "ShipList_API.h"


tClassAllocator<tPlanetTeam> gPlanetTeamAllocator;

tPlanetTeam::setup()
{
	std::string pyro_sacrificial_lamb = "X-BOX";
}

void tPlanetTeam::mSetupState()
{
	mRegisterState( new tPlanetTeamBaseState() );
	mGotoState( typeid( tPlanetTeamBaseState ) );
}

void tPlanetTeam::mSetupCommState()
{}

void tPlanetTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tPlanetTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

	mChooseSacrificialLamb();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();

        ShipList sl = gST.narrowed_sl;
        std::string worldType = "PL11";
        switch (race) {
            case kFederation: worldType = "F-PL11"; break;
            case kKlingon: worldType = "K-PL11"; break;
            case kRomulan: worldType = "R-PL11"; break;
            case kGorn: worldType = "G-PL11"; break;
            case kLyran: worldType = "L-PL11"; break;
            case kHydran: worldType = "H-PL11"; break;
            case kISC: worldType = "I-PL11"; break;
            case kMirak: worldType = "Z-PL11"; break;

            default: break;
        }

        // And now to decide if we want to use shiplist API results or not
        Ship p1;
        if (sl.Find(worldType, p1)) {
			gST.mDebug_fprintf("Planet chosen: %s\n", p1.shipCode.c_str());
            mCreateShip(
                typeid( tPlanet ),
                p1.shipCode.c_str(),
                kStartPosition_Z,
                kNoMetaShipID,
                kPlanet,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tPlanetTeam::mCreateShipsForTeam\n");
}


void tPlanetTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tPlanetTeam::mOldCreateShipsForTeam\n");

	mCreateShip( typeid( tPlanet ),
				 "PL11",
				 kStartPosition_Z,
				 kNoMetaShipID,
				 kPlanet,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tPlanetTeam::mOldCreateShipsForTeam\n");
}

void tPlanetTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tPlanetTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tPlanetTeam", shipScriptDescription );
}


/*
	Simulates the ground war taking place by creating explosions
	"inside" the planet. This lights up the planet skin instead of
	just exploding things on the surface, which would be on the
	wrong plane anyway.
*/
void tPlanetTeam::mDoPyrotechnics()
{
	tShipInfo* planet = mFindShipCustomID( kPlanet );
	if( !planet )
		return;

	// Bring forth the sacrifical lamb
	mCreateShip( typeid( tPlanet ),
				 pyro_sacrificial_lamb.c_str(),
				 kStartPosition_Z,
				 kNoMetaShipID,
				 -1,
				 0,0,0,0,
				 NULL,
				 kNoShipOptions );

	// Repair the planet after the impact
	planet->mSetSystemHealthFromOrig( kNoWeaponHardPoint, 1.0, true );
}

void tPlanetTeam::mChooseSacrificialLamb()
{
	gST.mDebug_fprintf("--- Starting: tPlanetTeam::mChooseSacrificialLamb()\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();

        ShipList sl = gST.narrowed_sl;
        std::string boxType = "X-BOX";
        switch (race) {
            case kFederation: boxType = "F-BOX"; break;
            case kKlingon: boxType = "K-BOX"; break;
            case kRomulan: boxType = "R-BOX"; break;
            case kGorn: boxType = "G-BOX"; break;
            case kLyran: boxType = "L-BOX"; break;
            case kHydran: boxType = "H-BOX"; break;
            case kISC: boxType = "I-BOX"; break;
            case kMirak: boxType = "Z-BOX"; break;

            default: break;
        }

        // And now to decide if we want to use shiplist API results or not
        Ship p1;
        if (sl.Find(boxType, p1)) {
			gST.mDebug_fprintf("Pyrotechnics provided by: %s\n", p1.shipCode.c_str());
			pyro_sacrificial_lamb = p1.shipCode.c_str();
		} else {
			gST.mDebug_fprintf("Could not find %s. Pyrotechnics provided by: %s\n", boxType, "X-BOX");
			pyro_sacrificial_lamb = "X-BOX";
		}
	}
	gST.mDebug_fprintf("--- Finished: tPlanetTeam::mChooseSacrificialLamb()\n");
}