/********************************************************************************************************\

	Mission:	Bleeding the Apes  (...and that means what??)
	Script:		"Uni_X2M3Apes"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		09/01/00
	Last Updated:	<INITIALS>		mm/dd/yy
					<LOG>
					FS              09/22/2012
					Era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_X2M3Apes.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "ISCTeam.h"
#include "PlanetTeam.h"
#include "InvasionTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X2M3Apes );

tUni_X2M3Apes::tUni_X2M3Apes()
{}

int32 tUni_X2M3Apes::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_X2M3Apes::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecKling | kSpecLyran | kSpecGorn | kSpecMirak,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Ally Team",
												static_cast< eTeamID >( kAllyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecAnyRace,
												kSameAs | kPlayerTeam,
												typeid( tAllyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"ISC Team",
												static_cast< eTeamID >( kISCTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tISCTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecAnyRace,
												kSameAs | kPlayerTeam,
												typeid( tPlanetTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Invasion Team",
												static_cast< eTeamID >( kInvasionTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tInvasionTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( draft1Team );
}

int32 tUni_X2M3Apes::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_X2M3Apes::mCreateExternalShips( int32 Mode )
{}

int32 tUni_X2M3Apes::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_X2M3Apes::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_X2M3Apes::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_X2M3Apes::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_X2M3Apes::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 5;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kAllyTeam,
		kISCTeam,
		kPlanetTeam,
		kInvasionTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Ally = kAllyTeam
		//ISC  = kISCTeam
		//Plan = kPlanetTeam
		//Inva = kInvasionTeam

		//	     Plyr  Ally  ISC   Plan  Inva
		/*Plyr*/ "--", "AI", "WD", "WX", "WD",
		/*Ally*/ "AI", "--", "HI", "WX", "HI",
		/*ISC */ "WD", "WD", "--", "HI", "AI",
		/*Plan*/ "NI", "NI", "NI", "--", "NI",
		/*Inva*/ "HI", "HI", "AI", "WX", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


void tUni_X2M3Apes::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['Z'] = kObjectPlanet;
}

tMapTerrain tUni_X2M3Apes::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_X2M3Apes::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}

void tUni_X2M3Apes::mDefineBriefings( void )
{
	for( int32 nRace = kFederation; nRace < kMirak + 1; nRace++ )
	{
		fBriefingIndex[kPlayerTeam][nRace] = kBriefing_msg;
	}
}