/********************************************************************************************************\

	File Name:	BaseVictoryState.h
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#if !defined( BaseVictoryState_h )
#define BaseVictoryState_h

#include "ChildActorVictoryState.h"
#include "BaseVictory.h"


class tBaseVictoryState : public tChildActorVictoryState<tBaseVictory>
{
public:

	//
	//	Serialization events
	//
	virtual void mBeginState();
	virtual void mEndState();
	tBaseVictoryState(	tTeamInfo* team, int32 playerTeamID, int32 enemeyTeamID, int32 captureBouns,
						int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat );
	virtual eVictoryLevels mCalcVictoryStatus(eTeamID ID);

	//
	//	Ship events
	//
	virtual void mOnDeath( tShipInfo* ship );
	virtual void mOnCapture( tShipInfo* ship );
	virtual void mOnDisengaged( tShipInfo* ship );
	virtual void mAtDistance( tShipInfo* ship );
	virtual void mAtDestination( tShipInfo* ship );
	virtual void mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip );
	virtual void mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType );
	virtual void mOnDeepScan( tShipInfo* ship, iTruth deepScanOn );
	virtual void mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus );
	virtual void mOnProbeLaunch( tShipInfo* ship );
	virtual void mOnBeginOrbit( tShipInfo* ship );
	virtual void mOnEndOrbit( tShipInfo* ship );
	virtual void mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip );
	virtual void mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip );
	virtual void mOnFlashingButtonPressed();
	virtual void mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item );
	virtual void mOnChangingShip( tShipInfo* ship );
	virtual void mOnSpecialTransport( tShipInfo* ship );

	//
	//	System events
	//
	virtual void mOnPlayerLeaving( int playerNumber );
	virtual void mOnHostChanged();
	virtual void mOnTimedInterval( int32 timerID );
	virtual void mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType );
	virtual void mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType );

	enum eTimerIDs
	{
		kCheckSecretTransmission = 1001,
		kScanTimer,
		kPrimaryObjectiveCompleteID,
		kCheckEndID,
		kRandChatID,
		kOKToHailID,
	};

private:
	iTruth fISCSurrendered;
	iTruth fISCDestroyed;
	iTruth fISCDefeated;

	iTruth fTransportSurrendered;
	iTruth fTransportCaptured;
	iTruth fTransportDestroyed;
	iTruth fTransportHailed;

	iTruth fSecretSecured;
	iTruth fTalkedToSun;
	iTruth fISCGotTheSecret;
	iTruth fProximityWarning;
	iTruth fISCSetToFollowTransport;
	iTruth fPlayerDefeated;

	iTruth fScanning;
	bool fDeepScannedSun;
	bool fDeepScannedTransport;
	iTruth fTractoringTransport;

	int32 iNumISCShipsEscapedWithSecret;
	int32 iNumISCShipsDefeated;

	float CalculateAverageEnemyDamage();
	iTruth OrderISCToFollowTransport();
	tShipInfo* GetCurrentShipTarget(tShipInfo* ship);
	void CheckEndMission();
	void EndMission(eGameStatus Status);
	void DoHailISC(tShipInfo* ship, tShipInfo* targetShip);
	void DoDemandSurrenderISC(tShipInfo* ship, tShipInfo* targetShip);
	void DoHailTransport(tShipInfo* ship, tShipInfo* targetShip);
	void DoDemandSurrenderTransport(tShipInfo* ship, tShipInfo* targetShip);
	void DoHailSun(tShipInfo* ship, tShipInfo* targetShip);
};

#endif
