/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "Uni_X3M1ATemple.h"
#include "MissionText.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyShip.h"
#include "ShipList_API.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		int32 enemy1BPV = 0;
		int32 enemy2BPV = 0;

		tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
		int32 playerBPV = playerTeam->mGetCombatBPV();

		switch (fMissionInfo->mGetDifficultyLevel()) {
		case 2:
			enemy1BPV = playerBPV * 1.0;
			enemy2BPV = playerBPV * 0.3;
			break;
		case 1:
			enemy1BPV = playerBPV * 0.6;
			enemy2BPV = playerBPV * 0.4;
			break;
		case 0:
		default:
			enemy1BPV = playerBPV * 0.5;
			enemy2BPV = playerBPV * 0.3;
			break;
		}

        eRaceName race = mGetRace();
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, race );
        sl = gST.stripOutRestricted( sl );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.selectWarShips(sl);

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		ShipList large_sl = sl;
        fShiplistCount = large_sl.Subset( large_sl.kSubsetBPV, int(enemy1BPV * 0.825), int(enemy1BPV * 1.075), true);
        gST.mDebug_fprintf("large_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(enemy1BPV * 0.825), int(enemy1BPV * 1.075), fShiplistCount);
        gST.listShiplistContent(large_sl);

		ShipList small_sl = sl;
        fShiplistCount = small_sl.Subset( small_sl.kSubsetBPV, int(enemy2BPV * 0.825), int(enemy2BPV * 1.075), true);
        gST.mDebug_fprintf("small_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(enemy2BPV * 0.825), int(enemy2BPV * 1.075), fShiplistCount);
        gST.listShiplistContent(small_sl);

        if ((small_sl.size() > 0) && (large_sl.size() > 0)) {
            Ship s1, s2;
            large_sl.SelectRandomShip(s1);
            gST.mTrackOrions( s1 );
            gST.mDebug_fprintf("large ship (s1) %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

            small_sl.SelectRandomShip(s2);
            gST.mTrackOrions( s2 );
            gST.mDebug_fprintf("small ship (s2) %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);

            mCreateShip(
                typeid( tEnemyShip ),
                s1.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kEnemyShip1,
                0, 0, -1, -1,
                NULL,
                static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );

            mCreateShip(
                typeid( tEnemyShip ),
                s2.shipCode.c_str(),
                kStartPosition_H,
                kNoMetaShipID,
                kEnemyShip2,
                0, 0, -1, -1,
                NULL,
                static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );
        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

    gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Entering: tEnemyTeam::mOldCreateShipsForTeam\n");

	int32 enemy1BPV = 0;
	int32 enemy2BPV = 0;

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kPlayerTeam ) );
	int32 playerBPV = playerTeam->mGetCombatBPV();

	switch (fMissionInfo->mGetDifficultyLevel()) {
	case 2:
		enemy1BPV = playerBPV * 1.0;
		enemy2BPV = playerBPV * 0.3;
		break;
	case 1:
		enemy1BPV = playerBPV * 0.6;
		enemy2BPV = playerBPV * 0.4;
		break;
	case 0:
	default:
		enemy1BPV = playerBPV * 0.5;
		enemy2BPV = playerBPV * 0.3;
		break;
	}

	mCreateShip( typeid( tEnemyShip ),
		enemy1BPV,
		kClassFrigate,
		kClassBattleship,
		0,
		kStartPosition_H, 
		kEnemyShip1, 
		0, 0, -1, -1, 
		NULL, 
		static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );			

	mCreateShip( typeid( tEnemyShip ),
		enemy2BPV,
		kClassFrigate,
		kClassBattleship,
		0,
		kStartPosition_H, 
		kEnemyShip2, 
		0, 0, -1, -1, 
		NULL, 
		static_cast<eShipOptions>( kStartPositionCanBeOffset | kCanBeCarrier ) );			

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}

