/********************************************************************************************************\

	Mission:	Temple In The Shy
	Script:		"Uni_X3M1ATemple"
	Descript:	?

	Created by:		John Williston		?
	Last Updated:	DJS					09/06/00
					Added tactical icon code and kPrimaryTeam
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerTeam.h"
#include "TransportTeam.h"
#include "SunTeam.h"
#include "Uni_X3M1ATemple.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X3M1ATemple );

tUni_X3M1ATemple::tUni_X3M1ATemple()
{}

void tUni_X3M1ATemple::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kFederation_briefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast<eTeamID>(kPlayerTeam),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecGorn | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Enemy Team",
												static_cast<eTeamID>(kEnemyTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Transport Team",
												static_cast<eTeamID>(kTransportTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed,
												kAnyRace,
												typeid( tTransportTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Sun Team",
												static_cast<eTeamID>(kSunTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tSunTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
	const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

int32 tUni_X3M1ATemple::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_X3M1ATemple::mCreateExternalShips( int32 Mode )
{}

int32 tUni_X3M1ATemple::mInitializeFinished( void )
{
	int32 iBriefingMsg;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	eRaceName playerRace = playerTeam->mGetRace();

	switch (playerRace)
	{
		case kFederation:
			iBriefingMsg = kFederation_briefing_msg;
			break;
		case kKlingon:
			iBriefingMsg = kKlingon_briefing_msg;
			break;
		case kHydran:
			iBriefingMsg = kHydran_briefing_msg;
			break;
		case kGorn:
			iBriefingMsg = kGorn_briefing_msg;
			break;
	};

	return iBriefingMsg;
}

void tUni_X3M1ATemple::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_X3M1ATemple::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kEnemyTeam,
		kTransportTeam,
		kSunTeam,
	};

	/*
		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Nme  = kEnemyTeam,
		//Trns = kTransportTeam,
		//Sun  = kSunTeam,

		//	     Plyr  Nme   Trns  Sun
		/*Plyr*/ "--", "NI", "NI", "NI",
		/*Nme */ "NH", "--", "FI", "NI",
		/*Trns*/ "NI", "FI", "--", "NI",
		/*Sun*/  "NI", "NI", "NI", "--",

	};

	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_X3M1ATemple::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['I'] = kObjectShipEnd;

	Map->fTacticalMapIcon['J'] = kObjectFissure;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_X3M1ATemple::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kFederation_briefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kKlingon_briefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kGorn_briefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kHydran_briefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_X3M1ATemple::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}