/*
	BaseVictoryState.h

	This module contains the declaration for the tBaseVictoryState class.
*/

#if !defined( BaseVictoryState_h )
#define BaseVictoryState_h

#include "ChildActorVictoryState.h"
#include "BaseVictory.h"

/*
	This class is the state responsible for handling victory-related events
	and performing calculations related to victory levels at the end of the
	scenario. All calls that end the present scenario are made by the methods
	of this class; i.e., if the mission is ending, it's ending because of a
	call made by this class.
*/

class tBaseVictoryState : public tChildActorVictoryState<tBaseVictory>
{
public:

	//	Serialization events
	virtual void mBeginState();
	virtual void mEndState();
	tBaseVictoryState(	tTeamInfo* team, int32 playerTeamID, int32 enemeyTeamID, int32 captureBouns,
						int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat );
	virtual eVictoryLevels mCalcVictoryStatus(eTeamID ID);

	//	Ship events
	virtual void mOnDeath( tShipInfo* ship );
	virtual void mOnCapture( tShipInfo* ship );
	virtual void mOnDisengaged( tShipInfo* ship );
	virtual void mAtDistance( tShipInfo* ship );
	virtual void mAtDestination( tShipInfo* ship );
	virtual void mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip );
	virtual void mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType );
	virtual void mOnDeepScan( tShipInfo* ship, iTruth deepScanOn );
	virtual void mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus );
	virtual void mOnProbeLaunch( tShipInfo* ship );
	virtual void mOnBeginOrbit( tShipInfo* ship );
	virtual void mOnEndOrbit( tShipInfo* ship );
	virtual void mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip );
	virtual void mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip );
	virtual void mOnFlashingButtonPressed();
	virtual void mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item );
	virtual void mOnChangingShip( tShipInfo* ship );
	virtual void mOnSpecialTransport( tShipInfo* ship );

	//	System events
	virtual void mOnPlayerLeaving( int playerNumber );
	virtual void mOnHostChanged();
	virtual void mOnTimedInterval( int32 timerID );
	virtual void mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType );
	virtual void mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType );

	// Timer ID for the random chatter.
	enum eTimerIDs
	{
		kRandChatID = 1000,
	};

private:
	// The number of transport ships that have successfully disengaged (i.e., left the playing field).
	int32 fNumTransportsCaptured;
	// The number of transport ships presently left in play, whether captured by the player or not.
	int32 fNumTransportsLeft;		
	// Has the enemy ship been destroyed?
	iTruth fEnemyDestroyed;			
	// Has the enemy ship been captured?
	iTruth fEnemyCaptured;			
	// Has the enemy ship surrendered?
	iTruth fEnemySurrendered;		
	// Has the enemy ship been attacked after having surrendered?
	iTruth fEnemyAttackedAfterSurrender;	
	// Is the player ship presently tractoring a transport ship?
	iTruth fTractoringATransport;	

	// This method returns kTrue if the enemy ship has been defeated or kFalse otherwise.
	iTruth IsEnemyDefeated();
	// This method adds a prestige bonus for use of non-lethal force depending upon player race.
	void AddNonLethalBonus();
};

#endif
