/*
	EnemyShip.cpp

	This module contains the definition of the tEnemyShip class, which does
	little more than initially configure the enemy ship correctly.
*/

#include "Uni_X4M1AThief.h"
#include "TeamInfo.h"
#include "MissionText.h"
#include "EnemyShip.h"
#include "EnemyShipBaseState.h"

tClassAllocator<tEnemyShip> gEnemyShipAllocator;

void tEnemyShip::mSetupState()
{
	mRegisterState( new tEnemyShipBaseState() );
	mGotoState( typeid( tEnemyShipBaseState ) );
}

/*
	This method sets up the comm states for the enemy ship. The initial hail
	message, as seen below, is set based on the player's race, and each
	comm state is described briefly as follows:

		(1) Attacked after having surrendered.
		(2) Surrendered or been disabled.
		(3) Ship has been captured.
		(4) Not yet hailed, still able to fight.
		(5) Not yet hailed, beaten up rather badly.
		(6) Hailed, still able to fight.
		(7) Hailed, beaten up rather badly.

	These seven states must be set for each of two valid buttons, the HAIL
	button corresponding to k_HAIL_ISC_btn, and the DEMAND SURRENDER button
	corresponding to k_DS_ISC_btn.
*/

void tEnemyShip::mSetupCommState()
{
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	eRaceName playerRace = playerTeam->mGetRace();
	int32 iInitialContactMsg;

	// Set the initial response to hail message on the basis of the
	// player's race.

	switch (playerRace)
	{
		case kFederation:
			iInitialContactMsg = k_HAIL_ISC4_msg;
			break;
		case kKlingon:
			iInitialContactMsg = k_HAIL_ISC5_msg;
			break;
		case kRomulan:
			iInitialContactMsg = k_HAIL_ISC6_msg;
			break;
		case kLyran:
			iInitialContactMsg = k_HAIL_ISC7_msg;
			break;
		case kGorn:
			iInitialContactMsg = k_HAIL_ISC8_msg;
			break;
		case kHydran:
			iInitialContactMsg = k_HAIL_ISC9_msg;
			break;
		case kMirak:
			iInitialContactMsg = k_HAIL_ISC10_msg;
			break;
	}

	int32 iShipID = mGetShipID();
	playerTeam->mRegisterCommMessage(kCommState1, kNoStateChange, iShipID, k_HAIL_ISC_btn, k_HAIL_ISC1_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState2, kNoStateChange, iShipID, k_HAIL_ISC_btn, k_HAIL_ISC2_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState3, kNoStateChange, iShipID, k_HAIL_ISC_btn, k_HAIL_ISC3_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState4, kNoStateChange, iShipID, k_HAIL_ISC_btn, iInitialContactMsg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState5, kNoStateChange, iShipID, k_HAIL_ISC_btn, iInitialContactMsg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState6, kNoStateChange, iShipID, k_HAIL_ISC_btn, k_HAIL_ISC11_msg, k_HAIL_ISC13_msg);
	playerTeam->mRegisterCommMessage(kCommState7, kNoStateChange, iShipID, k_HAIL_ISC_btn, k_HAIL_ISC11_msg, k_HAIL_ISC13_msg);

	playerTeam->mRegisterCommMessage(kCommState1, kNoStateChange, iShipID, k_DS_ISC_btn, k_DS_ISC1_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState2, kNoStateChange, iShipID, k_DS_ISC_btn, k_DS_ISC2_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState3, kNoStateChange, iShipID, k_DS_ISC_btn, k_DS_ISC3_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState4, kNoStateChange, iShipID, k_DS_ISC_btn, k_DS_ISC4_msg, k_DS_ISC6_msg);
	playerTeam->mRegisterCommMessage(kCommState5, kNoStateChange, iShipID, k_DS_ISC_btn, k_DS_ISC7_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState6, kNoStateChange, iShipID, k_DS_ISC_btn, k_DS_ISC4_msg, k_DS_ISC6_msg);
	playerTeam->mRegisterCommMessage(kCommState7, kNoStateChange, iShipID, k_DS_ISC_btn, k_DS_ISC7_msg, kNoRandomMsg);

	playerTeam->mGotoCommState(mGetShipID(), kCommState4);
}

// This method needs only to configure the ship callback events to enable
// the damage handler.

void tEnemyShip::mSetupShip()
{
	mSetShipCallbackEvents(kNotifyDamage);
}
