/*
	EnemyTeam.cpp

	This module contains the definition of the tEnemyTeam class.
*/

#include "Constants.h"
#include "MissionText.h"
#include "tclient.h"
#include "ScriptInterface.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "EnemyShip.h"
#include "EnemyPlatform.h"
#include "Uni_X4M1AThief.h"
#include "ShipList_API.h"


tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

/*
	This method creates the enemy ship and defense platforms.
*/

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

        // ships
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
        Assert( playerTeam );
        int32 playerBPV = playerTeam->mGetCombatBPV();

		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 nPCBPV; 
		if( diffLevel == 0 )
		{
			nPCBPV = playerBPV * 0.7;
		}
		else if( diffLevel == 1 )
		{
			nPCBPV = playerBPV * 1.0;
		}
		else if( diffLevel == 2 )
		{
			nPCBPV = playerBPV * 1.4;
		}

        ShipList ship_sl = sl;
        ship_sl = gST.stripOutRestricted( ship_sl );
        ship_sl = gST.stripOutSPECIAL( ship_sl );
        ship_sl = gST.selectWarShips(ship_sl);

		std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFirstShip; i <= kEClassLastShip; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
		gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(nPCBPV * 0.825), int(nPCBPV * 1.075), true);
        gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(nPCBPV * 0.825), int(nPCBPV * 1.075), fShiplistCount);
        gST.listShiplistContent(ship_sl);


        // Defense platforms
        ShipList DEF_sl = sl;
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);


        if ((ship_sl.size() > 0) && (DEF_sl.size() > 0)) {
            // Create ship
            Ship ship1;
            ship_sl.SelectRandomShip(ship1);
            gST.mDebug_fprintf("ship1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship1.shipCode.c_str(), ship1.bpv, ship1.yearFirstAvail, ship1.yearLastAvail);
            gST.mTrackOrions( ship1 );

            mCreateShip(
                typeid( tEnemyShip ),
                ship1.shipCode.c_str(),
                kStartPosition_P,
                kNoMetaShipID,
                kEnemyShip,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions  );

			// Create the ISC "shooting mines".
			long lStartPosition = kStartPosition_L;
			long lID = kEnemyPlatform1;

			for (int i = 0; i < NUM_ENEMY_PLATFORMS; i++)
			{
				Ship def;
				DEF_sl.SelectRandomShip(def);
				gST.mDebug_fprintf("def %s selected. BPV: %d  YFA: %d  YLA %d\n",
					def.shipCode.c_str(), def.bpv, def.yearFirstAvail, def.yearLastAvail);
				gST.mTrackOrions( def );

				mCreateShip(
					typeid(tEnemyPlatform), 
					def.shipCode.c_str(), 
					lStartPosition++, 
					kNoMetaShipID,
					lID++, 
					0, 0, -1, -1, 
					Messages[kEnemyPlatformName].c_str(), 
					kDefaultShipOptions );
			}
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("--- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
    gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.7;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.4;
	}


	// Create the enemy ship.
	mCreateShip(
		typeid( tEnemyShip ), 
		nPCBPV, 
		kClassFrigate, 
		kClassHeavyCruiser, 
		100, 
		kStartPosition_P,
		kEnemyShip, 
		0, 0, -1, -1, 
		NULL, 
		kNoShipOptions );


	// Create the ISC "shooting mines".
	long lStartPosition = kStartPosition_L;
	long lID = kEnemyPlatform1;

	for (int i = 0; i < NUM_ENEMY_PLATFORMS; i++)
	{
		mCreateShip(
			typeid(tEnemyPlatform), 
			Messages[kEnemyPlatformClass].c_str(), 
			lStartPosition++, 
			kNoMetaShipID,
			lID++, 
			0, 0, -1, -1, 
			Messages[kEnemyPlatformName].c_str(), 
			kDefaultShipOptions );
	}
	
    gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}

