/*
	Transport.cpp

	This module contains the definition for the tTransport class, whose
	contributions are limited essentially to defining the valid transport
	comm states and configuring the transports at start of mission.
*/

#include "Uni_X4M1AThief.h"
#include "TeamInfo.h"
#include "MissionText.h"
#include "TransportTeam.h"
#include "Transport.h"
#include "TransportBaseState.h"

tClassAllocator<tTransport> gTransportAllocator;

void tTransport::mSetupState()
{
	mRegisterState( new tTransportBaseState() );
	mGotoState( typeid( tTransportBaseState ) );
}

/*
	This method initializes the valid comm states for the transports. They
	are described briefly as follows:

		(1) Transport hasn't yet been captured.
		(2) Transport captured; ship integrity at 100%.
		(3) Transport captured; ship integrity at >= 70%.
		(4) Transport captured; ship integrity at >= 40%.
		(5) Transport captured; ship integrity at <= 30%.
*/

void tTransport::mSetupCommState()
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	playerTeam->mRegisterCommMessage(kCommState1, kNoStateChange, mGetShipID(), k_HAIL_Transport_btn, k_HAIL_Transport1_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState2, kNoStateChange, mGetShipID(), k_HAIL_Transport_btn, k_HAIL_Transport2_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState3, kNoStateChange, mGetShipID(), k_HAIL_Transport_btn, k_HAIL_Transport3_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState4, kNoStateChange, mGetShipID(), k_HAIL_Transport_btn, k_HAIL_Transport4_msg, kNoRandomMsg);
	playerTeam->mRegisterCommMessage(kCommState5, kNoStateChange, mGetShipID(), k_HAIL_Transport_btn, k_HAIL_Transport5_msg, kNoRandomMsg);
	playerTeam->mGotoCommState(mGetShipID(), kCommState1);
}

/*
	This method is responsible for the initial configuration of each 
	transport. At the outset, each transport should be at a dead stop,
	completely unmanned and without any shields.

	NB: For some reason, the mSetPercentageFromOrig method called
		below does not actually lower the shields at startup. To ensure
		that the shields are lowered, a timer is set and managed
		separately. For details, see the tTransportTeamBaseState class.
*/

void tTransport::mSetupShip()
{
	mSetShipMaxSpeed(0);
	mSetItemInStores(kMarines, kStandard, 0);
	mSetSystemHealthFromOrig(kAllShieldsHardPoint, 0.0f);
	mSetShipCallbackEvents(kNotifyDamage);
}
