

#include "Constants.h"
#include "Uni_X4M1AThief.h"
#include "TransportTeamBaseState.h"
#include "ShipList_API.h"

/*
	Set a timer to be used in keeping the shields of the transports down
	while they are unowned by the player. I don't know why, but this
	approach seems required as a single call to the mSetPercentageFromOrig
	method at ship setup time does not suffice.
*/

void tTransportTeamBaseState::mBeginState()
{
	mSetTimedInterval(kTimerSecondsSet, kTransportShieldsID, SHIELD_TIMER_INTERVAL);
	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
}

void tTransportTeamBaseState::mEndState()
{}
void tTransportTeamBaseState::mOnDeath( tShipInfo* ship )
{}
void tTransportTeamBaseState::mOnCapture( tShipInfo* ship )
{}
void tTransportTeamBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tTransportTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tTransportTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tTransportTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tTransportTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tTransportTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tTransportTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tTransportTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tTransportTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tTransportTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tTransportTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTransportTeamBaseState::mOnFlashingButtonPressed()
{}
void tTransportTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tTransportTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tTransportTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tTransportTeamBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tTransportTeamBaseState::mOnHostChanged()
{}

/*
	When the transport shields timer goes off, we need to check
	each transport on the team, lowering its shields if it is not
	yet captured by the player. I don't know why this is necessary,
	but lowering the shields in the initial ship setup doesn't
	seem to do the job.
*/

void tTransportTeamBaseState::mOnTimedInterval( int32 timerID )
{
	// TODO: If ever lowering the shields will work in the ship setup, then
	//       the timer may be removed altogether.

	if (timerID == kTransportShieldsID)
	{
		tTeamInfo* transportTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTransportTeam));
		tShipIterator i = transportTeam->mGetFirstShip();
		
		while (*i != NULL)
		{
			// I don't understand why a captured transport, whose mGetTeam
			// method will return kPlayerTeam, is still enumerated as part
			// of the transport team, but it is. Thus, it is necessary to 
			// check whether the transport at hand has been captured and
			// avoid dropping the shields if it has.

			if (i->mGetTeam() != kPlayerTeam)
				i->mSetSystemHealthFromOrig(kAllShieldsHardPoint, 0.0f);
			i++;
		}

		// Reset the timer whenever it goes off to make sure the
		// shields on those transports stay down.

		mSetTimedInterval(kTimerSecondsSet, kTransportShieldsID, SHIELD_TIMER_INTERVAL);
	}

    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        tTeamInfo* EnemyTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kTransportTeam ) );
        if (EnemyTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        if (EnemyTeam->mIsTeamHumanControlled()) return;               // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tTransportTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        int32 BPVinc = gST.mUpgradeFightersForTeam(EnemyTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tTransportTeamBaseState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }


}

void tTransportTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tTransportTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

