/*
	Uni_X4M1AThief.cpp

	This module contains the definition of the tUni_X4M1AThief class,
	which is the main class for the script.

	Mission:	A Thief in the Night
	Script:		"Uni_X4M1AThief"
	Descript:	The player must capture some enemy transports while fending
				off an enemy ship and some defense platforms.

	Created by:		John B. Williston		06/12/00
	Last Updated:	JBW						06/18/00
					DJS						09/06/00
					Added tactical icon code and kPrimaryTeam
					DJS						09/26/00
					Meta script modifications
					FS						09/09/12
					OP+ enhanced shiplist, upgraded fighters, era-correct drones.

*/

#include "EnemyTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "TransportTeam.h"
#include "PlayerTeam.h"
#include "Uni_X4M1AThief.h"
#include "ShipList_API.h"
#include "DraftTeam.h"

DEFINE_SCRIPT_CLASS( tUni_X4M1AThief );

tUni_X4M1AThief::tUni_X4M1AThief()
{}

void tUni_X4M1AThief::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_Federation_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast<eTeamID>(kPlayerTeam),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecGorn | kSpecHydran,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Transport Team",
												static_cast<eTeamID>(kTransportTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tTransportTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",
												static_cast<eTeamID>(kEnemyTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
	const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	const tTeamSpec& draft3Team = mCreateTeamSpec(  "Draft 3 Team",
                                                   static_cast< eTeamID >( kDraft3Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam(team1);
	Specs->mSetupTeam(team2);
	Specs->mSetupTeam(team3);
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
	Specs->mSetupTeam( draft3Team );
}

int32 tUni_X4M1AThief::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_X4M1AThief::mCreateExternalShips( int32 Mode )
{}

/*
	This method returns the ID of the text string to be used as a briefing
	message, which varies depending upon the player's race.
*/

int32 tUni_X4M1AThief::mInitializeFinished( void )
{
	int32 iBriefingMsg;
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
	eRaceName playerRace = playerTeam->mGetRace();

	switch (playerRace)
	{
		case kFederation:
			iBriefingMsg = kBriefing_Federation_msg;
			break;
		case kKlingon:
			iBriefingMsg = kBriefing_Klingon_msg;
			break;
		case kRomulan:
			iBriefingMsg = kBriefing_Romulan_msg;			
			break;
		case kLyran:
			iBriefingMsg = kBriefing_Lyran_msg;
			break;
		case kHydran:
			iBriefingMsg = kBriefing_Hydran_msg;
			break;
		case kGorn:
			iBriefingMsg = kBriefing_Gorn_msg;
			break;
		case kMirak:
			iBriefingMsg = kBriefing_Mirak_msg;
			break;
	};

	return iBriefingMsg;
}

void tUni_X4M1AThief::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_X4M1AThief::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 3;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kTransportTeam,
		kEnemyTeam,
	};

	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kTransportTeam,
		//Nme  = kEnemyTeam,

		//	     Plyr  PNPC  Nme   NPC1
		/*Plyr*/ "--", "FI", "WD",
		/*PNPC*/ "NI", "--", "NI",
		/*Nme */ "WD", "FI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_X4M1AThief::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['H'] = kObjectShipEnd;
	Map->fTacticalMapIcon['I'] = kObjectShipEnd;
	Map->fTacticalMapIcon['J'] = kObjectShipEnd;
	Map->fTacticalMapIcon['K'] = kObjectShipEnd;

	Map->fTacticalMapIcon['L'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['M'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['N'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['O'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['P'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_X4M1AThief::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kBriefing_Federation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefing_Klingon_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] = kBriefing_Romulan_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kBriefing_Lyran_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kBriefing_Gorn_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kBriefing_Hydran_msg;
	fBriefingIndex[kPlayerTeam][kMirak] = kBriefing_Mirak_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_X4M1AThief::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}