/********************************************************************************************************\

	File Name:	BaseVictory.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "Uni_X4M2ACuckoo.h"
#include "MissionText.h"


tClassAllocator<tBaseVictory> gBaseVictoryAllocator;

/*

	HELPER NOTE:
	You must remember to initialize all of your victory
	attributes here. You should do this for both constructors
	until informed otherwise.
	For example:

	tTeam1BaseVictory::tTeam1BaseVictory( ) : fBSed( false ), field2( value ), field3 ( value ), etc...
	{}

*/
tBaseVictory::tBaseVictory() :
		fDetectionRadius(0),
		fDetected( false ),
		fCruiserDisabled(false),
		fCruiserDestroyed(false),
		fFreighterDestroyed(false),
		fFreighterBoarded( false),
		fOperationComplete( false),
		fOperationsTimer(0),
		fTeamLifted( false),
		fInDustCloud( false),
		fAtMapEdge(false)
{}
tBaseVictory::tBaseVictory( tTeamInfo* team ) :
		fDetectionRadius(0),
		fDetected( false ),
		fCruiserDisabled(false),
		fCruiserDestroyed(false),
		fFreighterDestroyed(false),
		fFreighterBoarded( false),
		fOperationComplete( false),
		fOperationsTimer(0),
		fTeamLifted( false),
		fInDustCloud( false),
		fAtMapEdge( false)
{}

/*

	HELPER NOTE:
	Really, the only items in this function that
	you could change are the values passed into the
	tBaseVictoryState constructor.  These values are 
	the capture bonus and prestige points awarded
	according to the player's victory status -- starting at
	kAstoundingVictory and working its way down to kDevastatingDefeat.

*/
void tBaseVictory::mSetupState()
{
	//tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( kPlayerTeam );

	tBaseVictoryState* baseVictoryState = new tBaseVictoryState( 300, 225, 75, 0, -75 );
	mRegisterState( baseVictoryState );
	/****************************************************
	** Here's one way to vary the difficulty level. The 
	** greater the detection radius, the tougher the 
	** mission will be.
	****************************************************/
	fDetectionRadius = 100;
	mGotoState( typeid( tBaseVictoryState ) );
}


/************************************************
** Calculate victory status
************************************************/
eVictoryLevels tBaseVictory::mCalcVictoryStatus()
{
	tTeamInfo * PlayerTeam = 	gScriptInterface.mMission()->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam));
	tShipIterator ship = PlayerTeam->mGetFirstShip();
	if( *ship != NULL && mOperationComplete() && mTeamLifted() && !mDetected() && !mCruiserDisabled() && !mCruiserDestroyed() && !mFreighterDestroyed())
	{
		return kAstoundingVictory;
	}
	else
	{
		return kDevastatingDefeat;
	}
}
/*

	HELPER NOTE:
	The following is simply an example of setting and 
	modifying the current victory condition objectives.

	bool
	tTeam1BaseVictory::mBSed( void )
	{
		return fBSed;
	}
	void
	tTeam1BaseVictory::mSetBSed( bool x )
	{
		fBSed = x;
	}

*/
bool tBaseVictory::mDetected( void)
{
	return fDetected;
}
bool tBaseVictory::mCruiserDisabled( void)
{
	return fCruiserDisabled;
}
bool tBaseVictory::mFreighterBoarded(void)
{
	return fFreighterBoarded;
}
bool tBaseVictory::mOperationComplete(void)
{
	return fOperationComplete;
}
int32 tBaseVictory::mOperationsTimer(void)
{
	return fOperationsTimer;
}
bool tBaseVictory::mTeamLifted(void)
{
	return fTeamLifted;
}

bool tBaseVictory::mInDustCloud(void)
{
	return fInDustCloud;
}
bool tBaseVictory::mAtMapEdge(void)
{
	return fAtMapEdge;
}

bool tBaseVictory::mCruiserDestroyed( void)
{
	return fCruiserDestroyed;
}
bool tBaseVictory::mFreighterDestroyed(void)
{
	return fFreighterDestroyed;
}
//
//	Modifiers.
//
/*

	void mSetBSed( bool x );

*/
void tBaseVictory::mSetDetected(bool x)
{
	fDetected = x;
}
void tBaseVictory::mSetCruiserDisabled(bool x)
{
	fCruiserDisabled = x;
}

void tBaseVictory::mSetFreighterBoarded(bool x)
{
	fFreighterBoarded = x;
}
void tBaseVictory::mSetOperationComplete(bool x)
{
	fOperationComplete = x;
}
void tBaseVictory::mIncOperationsTimer(void)
{
	fOperationsTimer++;
}
void tBaseVictory::mSetTeamLifted(bool x)
{
	fTeamLifted = x;
}
void tBaseVictory::mSetInDustCloud(bool x)
{
	if(x)
	{
		fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kEvt_14);
	}
	else
	{
		fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kEvt_15);
	}
	fInDustCloud = x;
}
void tBaseVictory::mSetAtMapEdge(bool x)
{
	if(x)
	{
		fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kEvt_12);
	}
	fAtMapEdge = x;
}
float tBaseVictory::mDetectionRadius(void)
{
	return fDetectionRadius;
}

void tBaseVictory::mSetFreighterDestroyed(bool x)
{

	fFreighterDestroyed = x;
}
void tBaseVictory::mSetCruiserDestroyed(bool x)
{
	fCruiserDestroyed = x;
}