/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "BaseVictory.h"
#include "PlayerTeam.h"
#include "PlayerDecoyShip.h"
#include "Freighter.h"
#include "PlayerShip.h"
#include "MissionText.h"
#include "Uni_X4M2ACuckoo.h"

/*

	HELPER NOTE:
	There are so many things that can be done here depending upon which mission you are
	coding.  So rather than list out possible bodies for each of the events below, the
	signatures have been given to save time cutting and pasting.  However, if you do need
	examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
	which is the latest mission and the mission that contains files that most closely reflect
	this file and others in this script.

*/
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory , int32 victory , int32 draw , int32 defeat , int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory , victory , draw , defeat , devastatingDefeat )
{}
eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	return fChildActor->mCalcVictoryStatus();
}

void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	int32 ID = ship->mGetCustomID();
	tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
	switch(ID)
	{
	case kISCLightCruiser:
		fChildActor->mSetCruiserDestroyed(true);
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvt_4);
		gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );
		break;
	case kFreighter:
		fChildActor->mSetFreighterDestroyed(true);
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvt_3);
		if(!fChildActor->mDetected())
		{
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );
		}
		break;


    case kPlayerShip:
	case kPlayerShip2:
	case kPlayerShip3:
		if ( !team->mShipsRemainingInPlay() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );

		break;
	case kPlayerShip4:
		break;
		
	}


}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	int32 ID = ship->mGetCustomID();
	switch(ID)
	{
	case kFreighter:
		fChildActor->mSetFreighterDestroyed(true);
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kAborted_msg);
		if(!fChildActor->mDetected())
		{
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );
		}
		break;
/*
		if(! fChildActor->mFreighterBoarded())
		{
			fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvt_7);
			fChildActor->mSetFreighterBoarded(true);
			fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) )->mGotoCommState(ship->mGetShipID(), tFreighter::kCommState2 );
			ship->mAddItemToStores(kTransporterItem,kTransInfiltrator,0);
			mSetTimedInterval( kTimerSecondsSet, kOpTimerID, 1.0 );
		}
		break;
*/
	case kISCLightCruiser:
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvt_6);
		fChildActor->mSetCruiserDestroyed(true);
		gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );
		break;

    case kPlayerShip:
	case kPlayerShip2:
	case kPlayerShip3:
	case kPlayerShip4:

		tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
		if ( !team->mShipsRemainingInPlay() )
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerDestroyed );
		break;
		
	}


}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	int32 ID = ship->mGetCustomID();
	tTeamInfo * PlayerTeam =fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ));
	Assert(PlayerTeam);
	tShipInfo * playership = *(PlayerTeam->mGetFirstShip());
	Assert( playership);
	if(!playership==NULL)
	{
		if(playership->mTestShipStatus(kShipDisengaged))
		{
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kEvt_13);
			gScriptInterface.mSetGameStatus((eTeamID) kPlayerTeam, kPlayerDisengaged);
		}
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	if(damagedShip->mGetCustomID()==kISCLightCruiser)
	{
		if((damagedShip->mGetDamagePercent() >= 50) && (!fChildActor->mCruiserDisabled()))
		{
			fChildActor->mSetCruiserDisabled(true);
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kEvt_5);
			gScriptInterface.mSetGameStatus( ( eTeamID )kPlayerTeam, kPlayerLoses );
		}
	}
	damagedShip->mSetShipCallbackEvents( kNotifyDamage );
}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{

	if((shipTo->mGetCustomID()== kFreighter) && (item == kTransInfiltrator))
	{
		if(!fChildActor->mFreighterBoarded())
		{
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam ), kEvt_10);
			fChildActor->mSetFreighterBoarded(true);
			fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam))->mGotoCommState(shipTo->mGetShipID(), tFreighter::kCommState2 );
//			shipTo->mAddItemToStores(kTransporterItem,kTransInfiltrator,0);			
			mSetTimedInterval( kTimerSecondsSet, kOpTimerID, 1.0 );
		}
		else
		{
			shipTo->mRemoveItemFromStores( kTransInfiltrator, 1 );
//			shipFrom->mAddItemToStores( kTransporterItem, kTransInfiltrator, 1 );
		}
	}
	if((shipTo->mGetCustomID()== kPlayerShip) && (item == kTransInfiltrator))
	{
		fMissionInfo->mDisplayMessage( static_cast< eTeamID >( kPlayerTeam ), kEvt_11);
		fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam))->mGotoCommState( shipFrom->mGetShipID(), tFreighter::kCommState1 );
//		shipTo->mAddItemToStores(kTransporterItem,kTransInfiltrator,1);
		if(!fChildActor->mTeamLifted())
		{
			fChildActor->mSetTeamLifted(true);
		}
	}

	shipTo->mSetShipCallbackEvents( kNotifyTransportedItem );
	shipFrom->mSetShipCallbackEvents( kNotifyTransportedItem );
}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{
}

void tBaseVictoryState::mOnPlayerLeaving( int playerNumber )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
	float distance = 0;

    int32 X = 0;
	int32 Y = 0;
	tTeamInfo * FreighterTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kFreighterTeam));
	if ( !FreighterTeam )
		return;
//	Assert(FreighterTeam);
	tTeamInfo * PlayerTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam));
	if ( !PlayerTeam )
		return;
//	Assert(PlayerTeam);
	tShipIterator playership = PlayerTeam->mGetFirstShip();	
	if ( *playership == NULL )
		return;
//	Assert(*playership != NULL);
	tTeamInfo * EnemyTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kEnemyTeam));
	if ( !EnemyTeam )
		return;
//	Assert(EnemyTeam);
	tShipIterator enemyship = EnemyTeam->mGetFirstShip();
	if ( *enemyship == NULL )
		return;
//	Assert(*enemyship != NULL);
	tShipIterator freighter = FreighterTeam->mGetFirstShip();
	if ( *freighter == NULL )
		return;
	tShipInfo * enemy = *enemyship;
	switch(timerID)
	{

	case kOpTimerID:
		fChildActor->mIncOperationsTimer();
		if( fChildActor->mOperationsTimer() == 10)
		{
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), kEvt_8);
			PlayerTeam->mGotoCommState( (*freighter)->mGetShipID(), tFreighter::kCommState3 );
		}
		if( fChildActor->mOperationsTimer() == 20)
		{
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam ),kEvt_9);
			fChildActor->mSetOperationComplete(true);
			PlayerTeam->mGotoCommState( freighter->mGetShipID(), tFreighter::kCommState4 );
			freighter->mAddItemToStores(kTransporterItem,kTransInfiltrator,1);
		}
		mSetTimedInterval(	kTimerSecondsSet, timerID, 1.0);
		break;
	}

}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}
