/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "FreighterTeam.h"
#include "ISCLightCruiser.h"
#include "Uni_X4M2ACuckoo.h"
#include "ShipList_API.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

void tEnemyTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tEnemyTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

        ShipList ship_sl = sl;
        ship_sl = gST.stripOutRestricted( ship_sl );
        ship_sl = gST.stripOutSPECIAL( ship_sl );
        ship_sl = gST.selectWarShipRoles(ship_sl);

		// The mission description is very clear: It's a light cruiser. --FS
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        if (ship_sl.size() > 0) {
            // Create ship
            Ship ship1;
            ship_sl.SelectRandomShip(ship1);
            gST.mDebug_fprintf("ship1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship1.shipCode.c_str(), ship1.bpv, ship1.yearFirstAvail, ship1.yearLastAvail);
            gST.mTrackOrions( ship1 );

            mCreateShip(
                typeid( tISCLightCruiser ),
                ship1.shipCode.c_str(),
                kStartPosition_Z,
                kNoMetaShipID,
                kISCLightCruiser,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mCreateShipsForTeam\n");
}

void tEnemyTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tEnemyTeam::mOldCreateShipsForTeam\n");

	mCreateShip(
		typeid( tISCLightCruiser ), 
		"I-CL", 
		kStartPosition_Z,
		kNoMetaShipID,
		kISCLightCruiser, 
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tEnemyTeam::mOldCreateShipsForTeam\n");
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo * FreighterTeam = gScriptInterface.mMission()->mGetTeamHandle(static_cast< eTeamID >( kFreighterTeam ));
	
	tShipIterator Freighter = FreighterTeam->mGetFirstShip();
    tShipInfo * ship = (tShipInfo *) &Freighter;
	mDefendEntity(ship, 20.0, 5.0, kMediumPriority);
	tTeamInfo::mTeamSetupComplete();

}

void tEnemyTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tEnemyTeam", shipScriptDescription );
}

