/********************************************************************************************************\

	File Name:	ISCLightCruiserBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "ISCLightCruiserBaseState.h"
#include "ISCLightCruiser.h"
#include "EnemyTeam.h"
#include "PlayerTeam.h"
#include "MissionText.h"
#include "BaseVictory.h"
#include "Uni_X4M2ACuckoo.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tISCLightCruiserBaseState::mBeginState()
{

}
void tISCLightCruiserBaseState::mEndState()
{}

void tISCLightCruiserBaseState::mOnDeath( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mOnCapture( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mOnDisengaged( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mAtDistance( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mAtDestination( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	tPlayerTeam * PlayerTeam = (tPlayerTeam *)fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam));
	Assert(PlayerTeam);
	tBaseVictory * victory = PlayerTeam->mVictoryCondition();
	if(targetShip->mGetCustomID() == kISCLightCruiser)
	{
		if(!victory->mDetected())
		{
			victory->mSetDetected(true);
			fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam ), kEvt_2);
		}
	}

}
void tISCLightCruiserBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tISCLightCruiserBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tISCLightCruiserBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tISCLightCruiserBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tISCLightCruiserBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tISCLightCruiserBaseState::mOnFlashingButtonPressed()
{}
void tISCLightCruiserBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tISCLightCruiserBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tISCLightCruiserBaseState::mOnSpecialTransport( tShipInfo* ship )
{}

void tISCLightCruiserBaseState::mOnPlayerLeaving( int playerNumber )
{}
void tISCLightCruiserBaseState::mOnHostChanged()
{}
void tISCLightCruiserBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tISCLightCruiserBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

void tISCLightCruiserBaseState::mOnTimedInterval( int32 timerID )
{

	int32 X, Y;
	float distance; 
	tPlayerTeam * PlayerTeam = (tPlayerTeam *)fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam ));
	if ( !PlayerTeam )
		return;
//	Assert(PlayerTeam);
	tShipIterator playership = PlayerTeam->mGetFirstShip();	
	if ( *playership == NULL )
		return;
//	Assert(*playership != NULL);
	tTeamInfo * EnemyTeam = fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kEnemyTeam ));
	if ( !EnemyTeam )
		return;
//	Assert(EnemyTeam);
	tShipIterator enemyship = EnemyTeam->mGetFirstShip();
	if ( *enemyship == NULL )
		return;
//	Assert(*enemyship != NULL);
	tShipInfo * enemy = *enemyship;

	tBaseVictory * victory = PlayerTeam->mVictoryCondition();

	// ** Check that no invalidating defeat conditions have happened

	if(victory->mCruiserDestroyed() || victory->mCruiserDisabled() || victory->mFreighterDestroyed() || victory->mDetected())
	{
		// shut down the timer, because it just doesn't matter any more.
		return;
	}

	switch(timerID)
	{
	case kDistChkID:
		
		enemy->mGetGridPosition(&X, &Y);

		// Is enemy ship in the dust cloud?
		if(CheckInDustCloud( X,  Y))
		{
			// if it is, and we haven't set the flag, set it now.
			// and notify the player.
			if( !victory->mInDustCloud())
			{
				victory->mSetInDustCloud(true);
			}
		}
		else		// Not in dust cloud
		{
			// if not, and we haven't cleared the flag, clear 
			// it now. Also, give the player 1 timer interval to
			// get out of range. (probably meaningless, but will
			// at least keep the screen from getting overwhelmed
			// with messages.
			if( victory->mInDustCloud())
			{
				victory->mSetInDustCloud(false);
			}
			else
			{
				// check every player ship except the decoy
				for(playership; *playership != NULL; *playership++)
				{

					if(!(playership->mGetCustomID() == kPlayerShip4))
					{
						// Check distance
						distance = GetDistanceFromShipToShip(*enemyship, *playership);
												
						if(distance <= victory->mDetectionRadius())
						{
							victory->mSetDetected(true);
							fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam ), kEvt_2);
							return;
						}
					}
				}
			}

		}
	}
	mSetTimedInterval( kTimerSecondsSet, kDistChkID, 5.0 );

}

bool tISCLightCruiserBaseState::CheckInDustCloud(int32 X, int32 Y)
{
	return ( X > 14 && X < 25 && Y > 14 && Y < 25);
}
float tISCLightCruiserBaseState::GetDistanceFromShipToShip(tShipInfo * startship, tShipInfo * destship)
{
	return ( startship->mGetTacticalDistance(destship) );

}