/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerDecoyShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "Uni_X4M2ACuckoo.h"
#include "ShipScriptDescription.h"
#include "ShipList_API.h"


tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;

/*

	HELPER NOTE:
	The victory condition must be part of the player's team.
	Simply keep this the way it is for single player missions.

*/
void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

/*

	HELPER NOTE:
	The set of debriefings must be part of the player's team.
	Simply keep this the way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	
	// If I'm not mistaken, victory level won't matter here, since 
	// success is based on the flags. So the only thing that gets
	// added to the debriefing message is the flag debriefs.
	tBaseVictory *actor = (tBaseVictory *)fVictoryCondition;
		
	switch ( victoryLevel )
	{
	case kAstoundingVictory:
		fMissionInfo->mAddToDebriefingList( (eTeamID )kPlayerTeam, 	kDebSuccess_msg);
		break;
 	case kDevastatingDefeat:
		if(actor->mDetected() == true)
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID) kPlayerTeam, kDebIfDetected_msg );
			return;

		}
		if(actor->mCruiserDisabled() || actor->mCruiserDestroyed() || actor->mFreighterDestroyed())
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID) kPlayerTeam, 	kDebGenericFailure_msg );
			return;

		}
		if(!actor->mOperationComplete())
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID) kPlayerTeam, 	kDebOperationCompleteFalse_msg );
			return;
		}
		if(actor->mFreighterBoarded() && !actor->mTeamLifted())
		{
			fMissionInfo->mAddToDebriefingList( (eTeamID )kPlayerTeam, 	kDebIfTeamLiftedFalse_msg);
			return;
		
		}
		break;
	case kUnknownVictoryLevel:
		fMissionInfo->mAddToDebriefingList( (eTeamID)kPlayerTeam, kDisconnect_msg );
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level,
	you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );

}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	
//	eCustomIDs ID;

	tShipIterator ship = mGetFirstShip();
	Assert(*ship != NULL);

	if ( mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
	tShipInfo * flagship = *ship;
	flagship->mAddItemToStores(kTransporterItem,kTransInfiltrator,1);
	flagship->mSetFinalDestination(5,5);
	ship->mSetShipCallbackEvents( kNotifyTransportedItem );


	switch(fMissionInfo->mGetTeamHandle(static_cast< eTeamID >( kPlayerTeam))->mGetRace())
	{
	case kFederation:
		fStartMsg = kFedStart_msg;
		break;
	case kKlingon:
		fStartMsg = kKlingonStart_msg;
		break;
	case kRomulan:
		fStartMsg = kRomulanStart_msg;
		break;
	case kLyran:
		fStartMsg = kLyranStart_msg;
		break;
	case kHydran:
		fStartMsg = kHydranStart_msg;
		break;
	case kGorn:
		fStartMsg = kGornStart_msg;
		break;
	case kMirak:
		fStartMsg = kMirakStart_msg;
		break;
	}
	fMissionInfo->mDisplayMessage(static_cast< eTeamID >( kPlayerTeam), fStartMsg, kNonCommScriptMessage);
	mSetTimedInterval(	kTimerSecondsSet, kMapEdgeID, 5);
	tTeamInfo::mTeamSetupComplete();
}

void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static long index = 0;

	mCreateShip(typeid( tPlayerShip ),
		shipScriptDescription.fClassName.c_str(),
		kStartPosition_G + shipScriptDescription.fTeamID,
		MetaID,
		kPlayerShip + index, 
		0, 0, -1, -1,
		shipScriptDescription.fCustomName.c_str(),
		static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );
	index++;

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

tBaseVictory * tPlayerTeam::mVictoryCondition(void)
{
	return (tBaseVictory *)fVictoryCondition; 
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 450;
	fMissionScheduler.fNextMissionTitle = "Uni_X4M3ADoors";
}