/********************************************************************************************************\

	Mission:	<MISSION TITLE>
	Script:		"Uni_X4M2ACuckoo"
	Descript:	<DESCRIPTION>

	Created by:		N.D. Blomenkamp 05/25/00
	Last Updated:	NDB	06/05/00
	Localized code to actors, as per Dave Ferrell. Lowered detection
	range to 100K. Suggest when single player is implemented, this be
	used to adjust for difficulty level. Also changed dust cloud to
	asteroids, as the ISC cruiser kept crippling itself in the dust
	cloud.
					DJS						09/26/00
					Meta script modifications


\********************************************************************************************************/

#include "EnemyTeam.h"
#include "FreighterTeam.h"
#include "MissionText.h"
#include "MissionMaps.h"
#include "PlayerTeam.h"
#include "PlayerNPCTeam.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "Uni_X4M2ACuckoo.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X4M2ACuckoo );

tUni_X4M2ACuckoo::tUni_X4M2ACuckoo()
{}

/*

	HELPER NOTE:	Set the mission specs here.  For the time being, every mission type must be
					kMultiplayerMission until informed otherwise.

*/
void tUni_X4M2ACuckoo::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
												kSpecPlayableRace & ~kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Player NPC Team",
												static_cast< eTeamID >( kPlayerNPCTeam), 
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace & ~kSpecISC,
												kSameAs | kPlayerTeam,
												typeid( tPlayerNPCTeam ) );
	
	const tTeamSpec& team3 = mCreateTeamSpec(	"Enemy Team",
												static_cast< eTeamID >( kEnemyTeam),
												kMaxChance,
												kTrue, 
												kPlayableTeam,
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tEnemyTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Freighter Team",
												static_cast< eTeamID >( kFreighterTeam),
												kMaxChance,
												kTrue,
												kPlayableTeam, 
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tFreighterTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
	const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
	Specs->mSetupTeam( draft2Team );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_X4M2ACuckoo::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

iTruth tUni_X4M2ACuckoo::mUsePlayerShip( tShipInGame& Ship )
{
	return kTrue;
}
void tUni_X4M2ACuckoo::mCreateExternalShips( int32 Mode )
{
}

/*

	HELPER NOTE:	Return the briefing message index that
	you want to use from MissionText.h

*/
int32 tUni_X4M2ACuckoo::mInitializeFinished( void )
{
	
	tTeamInfo * PlayerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	switch(PlayerTeam->mGetRace())
	{
	case kFederation:
		return kFedBriefing_msg;
	case kKlingon:
		return kKlingonBriefing_msg;
	case kRomulan:
		return kRomulanBriefing_msg;
	case kLyran:
		return kLyranBriefing_msg;
	case kHydran:
		return kHydranBriefing_msg;
	case kGorn:
		return kGornBriefing_msg;
	case kMirak:
		return kMirakBriefing_msg;
	}
	
	return kBriefing_msg;

}

/*

	HELPER NOTE:
	Called when a new ship enters the mission while in-game.
	The Ship parameter holds the ship information for this
	specified ship.

*/
void tUni_X4M2ACuckoo::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:
	
	  This is where you set up the teams' relationships and goals.
	  Read on and edit as you see necessary for this mission.

*/
void tUni_X4M2ACuckoo::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kPlayerNPCTeam,
		kEnemyTeam,
		kFreighterTeam,
	};


	/*

		HELPER NOTE:
		Below is the initial relationship / goal table.
		Each letter has a meaning and each meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//PNPC = kPlayerNPCTeam
		//Nme  = kEnemyTeam,
		//Frtr = kFreighterTeam

		//	     Plyr  PNPC  Nme   Frtr
		/*Plyr*/ "--", "AI", "HI", "WI",
		/*PNPC*/ "AI", "--", "NI", "WI",
		/*Nme */ "WD", "WD", "--", "AI",
		/*Frtr*/ "WI", "WI", "AI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_X4M2ACuckoo::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerBase;
	Map->fTacticalMapIcon['J'] = kObjectPlayerBase;
	Map->fTacticalMapIcon['Y'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Z'] = kObjectEnemyShip;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_X4M2ACuckoo::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kFedBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kKlingonBriefing_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] = kRomulanBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kLyranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kGornBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kHydranBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] = kMirakBriefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_X4M2ACuckoo::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}