/********************************************************************************************************\

	File Name:	BaseVictory.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "tclient.h"
#include "BaseVictory.h"
#include "BaseVictoryState.h"
#include "ScriptInterface.h"
#include "MissionText.h"

#include "TraitorTeamAtWarState.h"
#include "FleetTeamExistsState.h"
#include "TraitorTeamShowState.h"

#include "Uni_X4M3ADoors.h"
#include "CommonDefs.h"
#include "PlayerTeam.h"
#include "TraitorTeam.h"
#include "PlanetTeam.h"

tClassAllocator<tBaseVictory> gBaseVictoryAllocator;

/*

	HELPER NOTE:	You must remember to initialize all of your victory attributes here.  You should
					do this for both constructors until informed otherwise.  For example:

	tTeam1BaseVictory::tTeam1BaseVictory( ) : fBSed( false ), field2( value ), field3 ( value ), etc...
	{}

*/
tBaseVictory::tBaseVictory()
{
	fDecisionState = kNoContact;

	fHostile = false;
	fCodeKnown = false;
	fDefectorDestroyed = false;
	fTraitorReachedPlanet = false;
	fPlayerReachedPlanet = false;
	fTimerSecs = kNever;
	fFleetAction = false;
	fISCDestroyed = 0;
}
tBaseVictory::tBaseVictory( tTeamInfo* team ) : tVictoryCondition( team )
{
	fDecisionState = kNoContact;

	fHostile = false;
	fCodeKnown = false;
	fDefectorDestroyed = false;
	fTraitorReachedPlanet = false;
	fPlayerReachedPlanet = false;
	fTimerSecs = kNever;
	fFleetAction = false;
	fISCDestroyed = 0;
}

/*

	HELPER NOTE:	Really, the only items in this function that you could change are the values passed into
					the tBaseVictoryState constructor.  These values are the capture bonus and prestige
					points awarded according to the player's victory status -- starting at
					kAstoundingVictory and working its way down to kDevastatingDefeat.

*/
void tBaseVictory::mSetupState()
{
	tBaseVictoryState* baseVictoryState = new tBaseVictoryState( 300, 225, 75, 0, -75 );
	mRegisterState( baseVictoryState );
	mGotoState( typeid( tBaseVictoryState ) );

	mSetTimedInterval( kTimerSecondsSet, kRandChatID, 20.0f );
}

/*

	HELPER NOTE:	The following is simply an example of setting and modifying the current victory condition
					objectives.

	bool
	tTeam1BaseVictory::mBSed( void )
	{
		return fBSed;
	}
	void
	tTeam1BaseVictory::mSetBSed( bool x )
	{
		fBSed = x;
	}

*/

void tBaseVictory::mSetExitGoal(void)
{
	int32 mapEdgeX, mapEdgeY;

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

	if (!playerTeam)
		return;

	tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();

	if (!playerShip)
		return;

	mGetClosestMapEdge(playerShip, &mapEdgeX, &mapEdgeY);

	fMissionInfo->mSetGoalIndicator(playerShip->mGetTeam(), true, mapEdgeX, mapEdgeY);
}

void tBaseVictory::mUpdateFleetArrival(int newTime)
{
	if (mFleetAction())
		return;

	if (newTime == kRightNow && mGetFleetArrivalTime() != kRightNow)
	{
		mSetFleetArrivalTime(newTime);
		mSetTimedInterval(kTimerSecondsSet, kCallTheFleetID, kRightNow);
	}
	else if (mGetFleetArrivalTime() > newTime)	// newTime is quicker
	{
		mSetFleetArrivalTime(newTime);
		mSetTimedInterval(kTimerTurnsSet, kCallTheFleetID, newTime);
	}
}

void tBaseVictory::mFleetHasArrived(void)
{
	// Set FleetAction Flag.
	fFleetAction = true;

	tTeamInfo* fleetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kFleetTeam) );
	if (!fleetTeam)
		return;

	// Set to ExistsState, which will create the fleet ships.
	fleetTeam->mGotoState(typeid( tFleetTeamExistsState ));

	// Tell the player that the fleet is here...
	fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFleetArrives_msg);

	tTeamInfo* traitorTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kTraitorTeam) );
	tShipInfo* traitorShip = traitorTeam->mFindShipCustomID( tTraitorTeam::kTraitor);
	
	if (traitorTeam && traitorShip)
	{
		if (mGetDecision() == kAccepted && traitorShip->mInPlay())
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFleetArrivesAgreed_msg);
			// Change state of Traitor to attack at half strength
			traitorTeam->mGotoState( typeid( tTraitorTeamShowState ) );
		}
		else if (traitorShip->mInPlay() &&
							mGetDecision() <= kNoDecision &&
							!mHostileTraitor())
		{
			// Player took too long to decide...
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTooLate_msg);
			mSetHostileTraitor();
		}
	}

	if (mCodeKnown())
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFleetArrivesHaveCodes_msg);
	}
}

void tBaseVictory::mAnotherISCDestroyed(void)
{
	++fISCDestroyed;
	tTeamInfo* fleetTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kFleetTeam));

	if (fleetTeam && fleetTeam->mGetShipsInPlayCount() == 0)
	{
		// Fleet has been wiped out... what does traitor do?
		// If not hostile, or captured, then have it try to escape.
		
		// How could it not be hostile, yet have been captured you ask?
		// Well, if it goes into "Putting On A Show" state, the
		// player can fire at it without setting the hostiletraitor flag.
		// this is because the player might think he needs to also put on
		// a show - besides, it's not right for the player to not be able
		// to fire back - as long as he doesn't kill the traitor, he should
		// be fine in ShowState...  but he could also capture the traitor
		// during that state, which we have to check against, otherwise
		// you get some weird messages that don't apply.
		// Let's also make sure the traitor hasn't yet been destroyed...
		// say the player captures it (as above),then the ISC fleet blows it up
		if (!mHostileTraitor() && !mDefectorCaptured() && !mDefectorDestroyed())
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFleetDestroyedTraitor_msg );

			tTeamInfo* traitorTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kTraitorTeam) );
			if (traitorTeam)
			{
				traitorTeam->mSetRelations( static_cast<eTeamID>(kPlayerTeam), kTeamHostile, kGoalIgnore, 1.0);

				tShipInfo* traitorShip = traitorTeam->mFindShipCustomID(tTraitorTeam::kTraitor);
				if (traitorShip)
				{
					traitorShip->mDisengage( kByMapEdge, kAbsolutePriority );
				}
			}
		}
		else
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFleetDestroyed_msg );
		}
	}

	// let's take this opportunity to update the player's exit vector.
	if (mCodeKnown() || mDefectorDestroyed())
		mSetExitGoal();

}

void tBaseVictory::mSetHostileTraitor(void)
{
	fHostile = true;
	if (!fFleetAction)
		mUpdateFleetArrival(tBaseVictory::kFastTime);

	tTeamInfo* traitorTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kTraitorTeam ));
	if (!traitorTeam)
		return;

	traitorTeam->mGotoState(typeid( tTraitorTeamAtWarState ) );
}

void tBaseVictory::mPlayerReachedPlanet(tShipInfo* playerShip, tShipInfo* traitorShip)
{
	fPlayerReachedPlanet = true;

	if (!traitorShip || !playerShip)
		return;

	if (mGetTraitorAtPlanet())
	{
		// We've arrived message.
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kReachedPlanet_msg );

		// Additional message that you're both there.
		if (playerShip->mGetTacticalDistance(traitorShip) <= SECURE_CHANNEL_DISTANCE)
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBothAtPlanet_msg );
		}
	}
	else
	{
		// We've arrived, but have to wait.
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kReachedPlanetFirst_msg );
	}

	// Set up distance notification for edge of system...
	//	Do this for all Player Ships in play.

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );
	Assert(playerTeam);

	tShipInfo* aPlayerTeamShip = NULL;
	for (int32 i = tPlayerTeam::kPlayer; i <= tPlayerTeam::kPlayer3; i++)
	{
		aPlayerTeamShip = playerTeam->mFindShipCustomID(i);
		if (aPlayerTeamShip && aPlayerTeamShip->mInPlay())
		{
			aPlayerTeamShip->mTrackTacticalDistance( PLAYER_LEAVING_RADIUS, gMapCenterX, gMapCenterY, kTrue);
		}
	}
}


//	Return true if you have to reset the check.
//	return false otherwise.
void tBaseVictory::mCheckHostileAct(void)
{
	//	If the player has accepted the offer AND the fleet is here,
	//	then it's not _really_ hostile - the player is also putting on a show.
	if (!(mGetDecision() == kAccepted && mFleetAction()))
	{
		mSetHostileTraitor();
	}
}

void tBaseVictory::mCheckForMissionEnd(void)
{
	if (mISCDestroyed() == 7)
	{
		if (fDefectorCaptured)
		{
			// All ISC Fleet destroyed, Traitor captured, end mission with victory.
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerWins );
		}
		else if (fDefectorDestroyed)
		{
			//	End mission with failure.
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerLoses );
		}
		//	Else defector not captured or destroyed - either fight it out, 
		//	or let the player leave the map, which gives him opportunity to 
		//	try to capture the traitor, even if he's made a deal.
	}
	//	If there are fleet ships left, then continue the mission.
}

void tBaseVictory::mGetClosestMapEdge(tShipInfo* ship, int32* mapEdgeX, int32* mapEdgeY)
{
	int32 currentX, currentY;
	int32 distanceToEdgeX, distanceToEdgeY;

	if (!ship)
		return;

	ship->mGetGridPosition(&currentX, &currentY);

	if (currentX * 2 > gMapSizeX) // we're on the right side of map
	{
		distanceToEdgeY = gMapSizeX - currentX;
		*mapEdgeX = gMapSizeX; // Assuming X edge is further away than Y...
	}
	else
	{
		distanceToEdgeY = currentX - 1;
		*mapEdgeX = 1; // Assuming X edge is further away...
	}

	if (currentY * 2 > gMapSizeY) // we're on the bottom of map
	{
		distanceToEdgeX = gMapSizeY - currentY;
		*mapEdgeY = gMapSizeY; // Assuming Y edge is further away...
	}
	else
	{
		distanceToEdgeX = currentY -1;
		*mapEdgeY = 1; // Assuming Y edge is further away...
	}

	// Which Edge is Closer?
	if (distanceToEdgeX < distanceToEdgeY) // XEdge Closer
	{
		*mapEdgeX = currentX;
	}
	else // YEdge Closer (or equidistant)
	{
		*mapEdgeY = currentY;
	}

	// mapEdgeX and mapEdgeY are now the closest map edge point.

}