/********************************************************************************************************\

	File Name:	BaseVictory.h
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#if !defined( BaseVictory_h )
#define BaseVictory_h

#include "VictoryCondition.h"


/*

HELPER NOTE:	Inside the C-style comments, you will find an example of the accessor, modifier, and
					attributes for the victory condition objectives.  In this example, the player must
					become BSed to achieve the primary objective.

*/


class tBaseVictory : public tVictoryCondition
{
public:

	tBaseVictory( );
	tBaseVictory( tTeamInfo* team );
	void mSetupState( );

	/*

		HELPER NOTE:	If you want random chatter, do not remove kRandChatID

	*/
	enum eTimerIDs
	{
		kRandChatID = 5100,
		kCallTheFleetID,
		kRatTraitorOopsID,
	};

	enum eDecisionStates 
	{
		kNoContact,
		kNoOffer,
		kNoDecision,
		kAccepted,
		kRejected, 
	};

	// We use two different times for quick fleet arrival.
	// "Right Now" is used when the traitor is destroyed, captured
	// the offer is accepted, or the player otherwise has no
	// obstacles to exiting with a victory.
	//
	// FastTime is to give Hydrans and Feds a little extra time
	// to get ahead and try to capture the traitor before the fleet
	// overwhelms them.
	// one turn from refusal (or firing on the traitor),
	// is probably not enough time to get done before the fleet
	// is there - which forces a fight through the fleet, which
	// is absolutely required for this mission to be any sort of 
	// challenge.

	//	kRightNow is measured in seconds, the rest in turns.
	enum eFleetArrivalTimes
	{
		kRightNow = 5,		// Time to arrive after making agreement.
		kFastTime = 1,		// Time to arrive after rejecting offer.
		kSlowTime = 6,		// Time to arrive after initial contact.
		kNever    = 30,		// Inital Setting.
	};

	//
	//	Misc Methods
	//
	void mSetExitGoal(void);
	void mUpdateFleetArrival(int newTime);
	void mFleetHasArrived(void);
	void mAnotherISCDestroyed(void);
	void mSetHostileTraitor(void);
	void mPlayerReachedPlanet(tShipInfo* playerShip, tShipInfo* traitorShip);
	void mCheckHostileAct(void);
	void mCheckForMissionEnd(void);
	void mGetClosestMapEdge(tShipInfo* ship, int32* mapEdgeX, int32* mapEdgeY);

	//
	//	Accessors.
	//

	int  mGetDecision(void) {return fDecisionState;}
	
	bool mCodeKnown(void) {return fCodeKnown;}
	bool mDefectorDestroyed(void) {return fDefectorDestroyed;}
	bool mDefectorCaptured(void)  {return fDefectorCaptured;}
	bool mGetTraitorAtPlanet(void) {return fTraitorReachedPlanet;}
	bool mGetPlayerAtPlanet(void) {return fPlayerReachedPlanet;}
	bool mHostileTraitor(void) {return fHostile;}

	int  mGetFleetArrivalTime(void) {return fTimerSecs;};
	bool mFleetAction(void) {return fFleetAction;}
	int  mISCDestroyed(void) {return fISCDestroyed;}
	
	//
	//	Modifiers.
	//
	void mMadeContact(void) {fDecisionState = kNoOffer;}
	void mMadeOffer(void) {fDecisionState = kNoDecision;}
	void mMadeDecision(int choice) { fDecisionState = choice; }
	void mSetCodeKnown(bool HaveCode) { fCodeKnown = HaveCode;}

	void mTraitorKilled(void) {fDefectorDestroyed = true;}
	void mSetDefectorCaptured(void)  { fDefectorCaptured = true;}

	void mTraitorReachedPlanet(void) {fTraitorReachedPlanet = true;}

	void mPlayerLeftPlanet(void) {fPlayerReachedPlanet = false;}

	void mSetFleetArrivalTime(int arriveTime) {fTimerSecs = arriveTime;};

private:

	//
	//	Victory attributes.
	//

	int  fDecisionState;

	bool fHostile;
	bool fCodeKnown;
	bool fDefectorDestroyed;
	bool fDefectorCaptured;
	bool fTraitorReachedPlanet;
	bool fPlayerReachedPlanet;

	bool fFleetAction;
	int  fISCDestroyed;
	int  fTimerSecs;
};

#endif
