/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "PlayerTeam.h"
#include "TraitorTeam.h"
#include "PlanetTeam.h"
#include "FleetTeam.h"
#include "FleetShip.h"

#include "TraitorShip.h"
#include "Planet.h"
#include "ShipInfo.h"

#include "ScriptInterface.h"
#include "Uni_X4M3ADoors.h"

#include "FleetTeamExistsState.h"
#include "TraitorTeamShowState.h"

#include "CommonDefs.h"

//
//	Serialization events
//

tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	//	For multiplayer, we'd deal with the passed parm.
	//	I'm not altogether sure it's working yet and this is a
	//	single-player script, so we'll deal with grabbing the 
	//	player TeamID instead of using the passed parm.
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

	//	If playerTeam doesn't exist, we've got other problems, 
	//	but return a defeat message.
	if (!playerTeam)
		return kDefeat;

	switch (playerTeam->mGetRace())
	{
		case kFederation:
		case kHydran: 
		{
			// RKG:	Added another possibility for Fed/Hydran
			//			victory conditions, as defined states don't
			//			handle all possibilities.
			// i.e. offer made, player never accepted/rejected offer, and
			// Traitor was killed but not captured.

			if (fChildActor->mGetDecision() == tBaseVictory::kAccepted)	// gave info on ally.
				return kDevastatingDefeat;
			else if (fChildActor->mCodeKnown())	// Got code, didn't give info.
				return kAstoundingVictory;
			else if (fChildActor->mGetDecision() == tBaseVictory::kRejected)	// rejected, didn't get code.
				return kVictory;
			else if (fChildActor->mGetDecision() < tBaseVictory::kNoDecision)	// Didn't listen to offer.
				return kDefeat;
			else																// Didn't make decision.
				return kDraw;
			  
			break;
		}
		default:
		{
			// RKG: Split Accepted and Stole Code conditions
			//  Otherwise, debriefing wouldn't have made sense
			//  if player captured the ISC - as there would no
			//  longer be a contact in the ISC insurrection.

			// RKG:	Added another possibility for other races
			//			Victory conditions.
			// i.e. Offer made, no decision made, and code not known.

			if (fChildActor->mGetDecision() == tBaseVictory::kAccepted	&&
					!fChildActor->mDefectorCaptured())	// code, provided info.
				return kAstoundingVictory;
			else if (fChildActor->mCodeKnown())			// Code, but didn't provide info.
				return kVictory;
			else if (fChildActor->mGetDecision() == tBaseVictory::kRejected)	// No Code, rejected.
				return kDevastatingDefeat;
			else if (fChildActor->mGetDecision() < tBaseVictory::kNoDecision)	// Didn't listen to offer.
				return kDraw;
			else																		// Didn't make decision.
				return kDefeat;

			break;
		}
	}
}

//
//	Ship events
//
void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if (!ship)
		return;


	int32 shipID = ship->mGetCustomID();

	switch (shipID)
	{
		case tPlayerTeam::kPlayer:
		case tPlayerTeam::kPlayer2:
		case tPlayerTeam::kPlayer3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			if (!team)
				return;

			if ( !team->mShipsRemainingInPlay() )
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			break;

		}

		case tTraitorTeam::kTraitor:
		{
			fChildActor->mTraitorKilled();

			if (fChildActor->mGetDecision() == tBaseVictory::kAccepted)	// Accepted codes.
			{
				// send kDestroyedAgreed_msg
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDestroyedAgreed_msg );
				fChildActor->mSetCodeKnown(false);
			}
			else if (!fChildActor->mCodeKnown()) // codes known.
			{
				// send kDestroyedNoCodes_msg
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDestroyedNoCodes_msg );

				// Original Specs indicated to End Scenario.
				// I'd rather the player fight his/her way off of the map,
				// THEN lose because he doesn't have the code.
				// gScriptInterface.mSetGameStatus( kPlayerLoses );
			}
			else if (fChildActor->mDefectorCaptured())
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapturedTraitorDestroyed_msg);
			}
			else if (fChildActor->mFleetAction()) // stole codes, fleet is in the area.
			{
				// Is this dead code?  Not absolutely sure... best to leave in in the event
				//	of an odd turn of events.
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDestroyedOther_msg );
			}
			else // codes taken, no fleet.
			{
				// send kDestroyedNoDecisionWithCodesNoFleet_msg
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDestroyedNoDecisionWithCodesNoFleet_msg );
			}

			// If traitor is destroyed, and fleet isn't here yet,
			// then bring them in to make the player fight his way out.
			// this is rather unlikely to be needed, simply because
			// it would require the traitor to get killed and yet have
			// the player not fire on him or tractor him...
			fChildActor->mUpdateFleetArrival(tBaseVictory::kRightNow);

			// No matter what, if the traitor is dead, there's little
			// reason to hang around...
			fChildActor->mSetExitGoal();
			break;
		}

		case tFleetTeam::kFleet1:
		case tFleetTeam::kFleet2:
		case tFleetTeam::kFleet3:
		case tFleetTeam::kFleet4:
		case tFleetTeam::kFleet5:
		case tFleetTeam::kFleet6:
		case tFleetTeam::kFleet7:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDestroyedFleetShip_msg);
			fChildActor->mAnotherISCDestroyed();
			mAddToBonusPrestige( 30 );
			break;
		}
		default:
			break;
	}

	fChildActor->mCheckForMissionEnd();
}

void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	/*
		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.
	*/
	if ( !ship )
		return;

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	if ( !playerTeam )
		return;

	int32 shipID = ship->mGetCustomID();

	switch ( shipID )
	{
		case tPlayerTeam::kPlayer:
		case tPlayerTeam::kPlayer2:
		case tPlayerTeam::kPlayer3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
			break;
		}

		case tTraitorTeam::kTraitor:
		{
			int32 mapEdgeX, mapEdgeY;
			fChildActor->mGetClosestMapEdge(ship, &mapEdgeX, &mapEdgeY);
			ship->mClearCommands();
			ship->mSetFinalDestination(mapEdgeX, mapEdgeY);
			ship->mSetShipMaxSpeed(31);
			ship->mSetShipThrust(31);
			fMissionInfo->mSetTeamRelations(static_cast<eTeamID>(kTraitorTeam), static_cast<eTeamID>(kFleetTeam), kTeamHostile, kGoalIgnore, 1.0, true);

			ship->mDisengage(kByMapEdge, kAbsolutePriority);
			
			fChildActor->mSetCodeKnown(true);
			fChildActor->mSetExitGoal();
			fChildActor->mSetDefectorCaptured();

			if (fChildActor->mGetDecision() <= tBaseVictory::kNoOffer)
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCaptureNoOffer_msg );
			}
			else if (fChildActor->mGetDecision() == tBaseVictory::kAccepted)
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCaptureAccepted_msg );
			}
			else
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapture_msg );
			}

			// If traitor is captured, and fleet isn't on the way yet,
			// then bring them in to make the player fight his way out.

			// When will this happen?
			// If traitor gets a shield knocked down,
			// perhaps by running into the planet, and the player then captures
			// the traitor, all without firing a single shot or getting the
			// initial secure channel contact...
			// Not bloody likely, but stranger things have happened.
			fChildActor->mUpdateFleetArrival(tBaseVictory::kRightNow);

			// Set to blank comm state.
			tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

			if (playerTeam) //	Why would this ever happen?  But best to check.
			{
				playerTeam->mGotoCommState(ship->mGetShipID(), tTraitorShip::kBlankCommState );
			}
			

			break;
		}
		case tFleetTeam::kFleet1:
		case tFleetTeam::kFleet2:
		case tFleetTeam::kFleet3:
		case tFleetTeam::kFleet4:
		case tFleetTeam::kFleet5:
		case tFleetTeam::kFleet6:
		case tFleetTeam::kFleet7:
		{
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapturedFleetShip_msg );
			fChildActor->mAnotherISCDestroyed();
			mAddToBonusPrestige( 30 );

			// Set to blank comm state.
			tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

			if (playerTeam) //	Why would this ever happen?  But best to check.
			{
				playerTeam->mGotoCommState(ship->mGetShipID(), tFleetShip::kBlankCommState );
			}
			break;
		}
		default:
			break;
	}

	fChildActor->mCheckForMissionEnd();

}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	/*

		HELPER NOTE:	If this is not a single player script, you should change the team calls here to
						the proper enums.

	*/
	if ( !ship )
		return;
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );
	if ( !playerTeam )
		return;

	int32 shipID = ship->mGetCustomID();

	switch ( shipID )
	{
		case tPlayerTeam::kPlayer:
		case tPlayerTeam::kPlayer2:
		case tPlayerTeam::kPlayer3:
		{
			tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();
			if (playerShip && playerShip == ship)
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLeftSystem_msg);
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>(kPlayerTeam), kPlayerDisengaged );
			}
			break;
		}
		case tTraitorTeam::kTraitor:
		{
			if (fChildActor->mDefectorCaptured())
			{
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kCapturedTraitorLeftSystem_msg);
			}
			break;
		}
		default:
			break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{
	if (!ship)
		return;

	tTeamInfo* traitorTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kTraitorTeam) );
	tTeamInfo* playerTeam  = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

	if ( !playerTeam || !traitorTeam)
		return;

	tShipInfo* traitorShip = traitorTeam->mFindShipCustomID( tTraitorTeam::kTraitor);

	if (!traitorShip)
		return;

	int32 shipID = ship->mGetCustomID();

	switch ( shipID )
	{
		case tPlayerTeam::kPlayer:
		case tPlayerTeam::kPlayer2:
		case tPlayerTeam::kPlayer3:
		{
			// Skip notification for any ship that does not currently
			// have the player piloting it.
			//
			//	This leaves a potential hole whereby one AI-controlled 
			//	player ship could get to the planet, then the player could
			//	switch to that and never get the notification or flag set.
			//
			//	But see below at mOnChangingShip
			tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();
			if (!playerShip || playerShip != ship)
				return;

			if (fChildActor->mGetPlayerAtPlanet())
			{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLeavingSystem_msg);
			}
			else
			{
				fChildActor->mPlayerReachedPlanet(playerShip, traitorShip);
			}
			break;
		}
		case tTraitorTeam::kTraitor:
		{
			// Only do these things once.
			if (fChildActor->mGetTraitorAtPlanet()) 
				break;

			fChildActor->mTraitorReachedPlanet();
			ship->mSetShipMaxSpeed(10);	// To give players a chance to keep up with traitor.

			// Announce arrival of Traitor
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kTraitorAtPlanet_msg );

			if (fChildActor->mGetPlayerAtPlanet())
			{
				tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();
				if (!playerShip || playerShip != ship)
					break;

				if (ship->mGetTacticalDistance(playerShip) <= SECURE_CHANNEL_DISTANCE)
				{
					fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kBothAtPlanet_msg );
				}
			}
			
			break;
		}
		default:
			break;
	}
}

void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}

void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{
	// ship is currently being passed in as a null value...
	// we'll assume that everything comes from the player's ship.

	if (!targetShip)
		return;

	int32 shipID = targetShip->mGetCustomID();
	tTeamInfo* targetTeam = fMissionInfo->mGetTeamHandle(targetShip->mGetTeam());
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

	if (!targetTeam || !playerTeam)
		return;

	switch (shipID)
	{
		case tTraitorTeam::kTraitor:
		{
			tShipInfo* playerShip = playerTeam->mCurrentlyControlledShip();
			if (!playerShip)
				return;
			bool secureChannel = (playerShip->mGetTacticalDistance(targetShip) <= SECURE_CHANNEL_DISTANCE ? true : false);
			eRaceName	playerRace = playerTeam->mGetRace();
			switch (buttonID)
			{
				// Would be nice if tracktacticaldistance would
				// accept a tShipInfo* parm... then could
				// handle this all with standard comm states
				// and changing comm states when distance changed.
				// but if wishes were fishes....

				case kHail_CS2_btn: // First Contact
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
					}
					else // First contact.
					{
						switch (playerRace)
						{
							case kGorn:
							case kMirak:
							case kKlingon:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kGloryContact_msg);
								break;
							default:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kContact_msg);
								break;
						}
						fChildActor->mMadeContact();
						fChildActor->mUpdateFleetArrival(tBaseVictory::kSlowTime);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState3 );
					}
					break;
				}
				case kHail_CS3_btn:	//	Hail after 1st Contact
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
					}
					else // Getting Details
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kDetails_msg);
						fChildActor->mMadeOffer();
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					break;
				}
				case kAboutOffer_btn: // About that Offer Main Menu
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
					}
					else 
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kAboutOffer_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState5);
					}
					break;
				}
				case kRequestInfo_btn: // Request Offer Info Sub menu
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					else 
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRequestInfo_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState7);
					}
					break;
				}
				case kRepeat_btn:	//	Repeat Offer
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					else 
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRepeatOffer_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					break;
				}
				case kMoreInfo_btn:	//	Ask for more Information
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					else 
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMoreInfo_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					break;
				}
				case kRespond_btn: // Respond Option on Submenu
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					else 
					{
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kOfferRepsonse_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState6);
					}
					break;
				}
				case kAccept_btn:
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					else 
					{
						switch (playerRace)
						{
							case kFederation:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFAccept_msg);
								break;
							case kKlingon:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kKAccept_msg);
								break;
							case kRomulan:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRAccept_msg);
								break;
							case kLyran:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLAccept_msg);
								break;
							case kHydran:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kHAccept_msg);
								break;
							case kGorn:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kGAccept_msg);
								break;
							case kMirak:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMAccept_msg);
								break;
						}
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState8);
						fChildActor->mMadeDecision(tBaseVictory::kAccepted);
						fChildActor->mSetCodeKnown(true);
						fChildActor->mSetExitGoal();
						fChildActor->mUpdateFleetArrival(tBaseVictory::kRightNow);
					}
					break;
				}
				case kReject_btn:
				{
					if (!secureChannel)
					{
						// Too far away
						fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kNoResponseDistance_msg);
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState4);
					}
					else 
					{
						switch (playerRace)
						{
							case kFederation:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kFReject_msg);
								break;
							case kKlingon:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kKReject_msg);
								break;
							case kRomulan:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRReject_msg);
								break;
							case kLyran:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kLReject_msg);
								break;
							case kHydran:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kHReject_msg);
								break;
							case kGorn:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kGReject_msg);
								break;
							case kMirak:
								fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kMReject_msg);
								break;
						}
						playerTeam->mGotoCommState(targetShip->mGetShipID(), tTraitorShip::kCommState9);
						fChildActor->mMadeDecision(tBaseVictory::kRejected);

						fChildActor->mSetHostileTraitor();

					}
					break;
				}
				default:
					break;
			}	// end of switch on buttonID
			break;
		} // end of case for kTraitor

		case tFleetTeam::kFleet1:
		case tFleetTeam::kFleet2:
		case tFleetTeam::kFleet3:
		case tFleetTeam::kFleet4:
		case tFleetTeam::kFleet5:
		case tFleetTeam::kFleet6:
		case tFleetTeam::kFleet7:
		{
			switch (buttonID)
			{
				case kRatOutTraitor_btn:
				{
					if (!fChildActor->mDefectorDestroyed() 
						&& fChildActor->mGetDecision() == tBaseVictory::kAccepted)
					{
						playerTeam->mSetTimedInterval(kTimerSecondsSet, tBaseVictory::kRatTraitorOopsID, 2);
						// there's a potential hole here - the message displays two seconds later,
						// perhaps the traitor could be killed in that much time.
						// however, it's not a huge issue, as it'll come in the msg log as if
						// it were done nearly simultaniously as the RatOutTraitor button.
						// doing it this way because I want it to show up at the end of the Rat dialog,
						// and didn't want to have SIX possible dialogs (one for each 3 options, then
						// two sets, one with and one without the "Sorry, Charlie - you shouldn't have done that"
						// message.
						fChildActor->mSetCodeKnown(false);
						fChildActor->mSetExitGoal();
					}
					break;
				}
				default:
					break;
			}	//	Bottom of switch on button ID
			break;
		}	//	End of Case for fleetTeam handling.
		default:
			break;
	}	// end of switch on ShipID
}

void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{
	//	We're ONLY looking for damage to the Traitor.
	if (!hostileShip || !damagedShip)
		return;

	if (damagedShip->mGetTeam() == kTraitorTeam)
	{
		if (hostileShip->mGetTeam() == kPlayerTeam)
		{
			fChildActor->mCheckHostileAct();
		}
		else
		{
			damagedShip->mSetShipCallbackEvents(kNotifyDamage);
		}
	}
}

void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{
	if (!tractoringShip | !tractorTarget)
		return;

	if (tractoringShip->mGetTeam() == kPlayerTeam)
	{
		if (tractorTarget->mGetTeam() == kTraitorTeam)
		{
			fChildActor->mCheckHostileAct();
		}
		else
		{
			tractoringShip->mSetShipCallbackEvents( kNotifyBeginTractor );
		}
	}
}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{
//	Check here for player ship distances to/from planet.

	if (!ship)
		return;

	int32 shipID = ship->mGetCustomID();

	switch (shipID)
	{
		default:
			break;
		case tPlayerTeam::kPlayer:
		case tPlayerTeam::kPlayer2:
		case tPlayerTeam::kPlayer3:
		{
			//	We're going to see if we need to display messages because the player
			//	may have switched to a ship that's "close enough" to the planet.
			if (!fChildActor->mGetPlayerAtPlanet())	//	Not reported at planet yet
			{
				tTeamInfo* planetTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlanetTeam) );
				if (!planetTeam)
					return;

				tShipInfo *planet = planetTeam->mFindShipCustomID(tPlanetTeam::kPlanet);
				if (!planet)
					return;

				//	Distance to planet is appropriate - so we must've jumped across the boundary
				if (ship->mGetTacticalDistance(planet) <= AT_PLANET_DISTANCE)
				{
					tTeamInfo* traitorTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kTraitorTeam) );
					if (traitorTeam)
					{
						tShipInfo* traitorShip = traitorTeam->mFindShipCustomID(tTraitorTeam::kTraitor);
						if (traitorShip)
							fChildActor->mPlayerReachedPlanet(ship, traitorShip);
					}
				}
			}

			//	Reset Exit Goal if required
			if (fChildActor->mCodeKnown() || 
					fChildActor->mDefectorDestroyed() ||
					fChildActor->mDefectorCaptured())	//	Is it possible to have captured him but not know the code?
			{
				fChildActor->mSetExitGoal();
			}

			break;
		}	//	Bottom of Case statement.
	}	//	bottom of switch
}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{
#if defined( kRandomChatter )

	static boolean msgDisplayed[6] = {false,false,false,false,false,false};

	if ( timerID == tBaseVictory::kRandChatID  &&	
		fChildActor->mGetDecision() == tBaseVictory::kNoContact &&		// Stop chatter if we've mad contact
		!fChildActor->mHostileTraitor())	// Stop chatter if we attack.
	{
		int32 whichMsg = fMissionInfo->mRandomInt32( 0, 5 );

		if (!msgDisplayed[whichMsg])
		{
			msgDisplayed[whichMsg] = true;

			switch ( whichMsg )
			{
			case 0:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat1_msg );	
				break;
			case 1:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat2_msg );	
				break;
			case 2:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat3_msg );	
				break;
			case 3:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat4_msg );	
				break;
			case 4:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat5_msg );	
				break;
			case 5:
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRandChat6_msg );	
				break;
			}
		}

		//	If all messages haven't been displayed, then reset the timer.
		if (msgDisplayed[0] == false || msgDisplayed[1] == false || 
				msgDisplayed[2] == false || msgDisplayed[3] == false || 
				msgDisplayed[4] == false || msgDisplayed[5] == false)
		{
			fChildActor->mSetTimedInterval(	kTimerSecondsSet, timerID,
										( float )( fMissionInfo->mRandomInt32( 30, 45 ) ) );
		}
	}
#endif

	// Fleet is here... time to party. :-)
	if ( timerID == tBaseVictory::kCallTheFleetID &&
		!fChildActor->mFleetAction()) // don't do this twice (in case the timer gets fired off more than once).
	{
		fChildActor->mFleetHasArrived();
	}
	else if (timerID == tBaseVictory::kRatTraitorOopsID)
	{
		fMissionInfo->mDisplayMessage( static_cast<eTeamID>(kPlayerTeam), kRat_Response_msg);
	}
}

void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}
