/********************************************************************************************************\

	File Name:	FleetTeamExistsState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/
#include "FleetTeamExistsState.h"
#include "PlayerTeam.h"
#include "Uni_X4M3ADoors.h"
#include "FleetShip.h"
#include "ShipList_API.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tFleetTeamExistsState::mBeginState()
{
	// I find it amazingly annoying that, when you tell an actor to
	// go to a state that it's already in, that it fires up BeginState...
	// but it probably has it's uses, for some missions...
	// so I can short-circuit it with a static boolean for this one.
	static bool fleetCreated = false;

	if (fleetCreated)
		return;

	fleetCreated = true;
	mCreateFleetTeamShips();
}

void tFleetTeamExistsState::mCreateFleetTeamShips()
{
    gST.mDebug_fprintf("--- Starting: tFleetTeamExistsState::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
		tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

		//	If playerTeam doesn't exist, fleet won't either
		//	(can this happen if player is killed right as fleet's coming in?)
		if (!playerTeam)
			return;	

		int32 playerBPV = playerTeam->mGetCombatBPV();
		int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

		int32 fleetBPV;
		const int32 variantPct = 15;
		if( diffLevel == 0 )
		{
			fleetBPV = (playerBPV * 0.75)/4;
		}
		else if( diffLevel == 1 )
		{
			fleetBPV = (playerBPV)/4;
		}
		else if( diffLevel == 2 )
		{
			fleetBPV = (playerBPV * 1.25)/4;
		}

		if (fleetBPV < 120) {
			fleetBPV = 120; // Set minimum to 120
		}

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, fChildActor->mGetRace() );

        ShipList ship_sl = sl;
        ship_sl = gST.stripOutRestricted( ship_sl );
        ship_sl = gST.stripOutSPECIAL( ship_sl );
		ship_sl = gST.selectWarShipRoles( ship_sl );

		// limit hull sizes
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 x = kEClassLightCruiser; x <= kEClassHeavyBattlecruiser; x++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
        }
        fShiplistCount = ship_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

		// limit further based on player BPV. +- 15%
        fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(fleetBPV * 0.85), int(fleetBPV * 1.15), true);
        gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
            int(fleetBPV * 0.85), int(fleetBPV * 1.15), fShiplistCount);
        gST.listShiplistContent(ship_sl);

		if (ship_sl.size() > 0) {
			for (int i =0; tFleetTeam::kFleet1 + i <= tFleetTeam::kFleet7;i++)
			{	
				Ship s1;
				ship_sl.SelectRandomShip(s1);
				gST.mTrackOrions( s1 );
				gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
					s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);

				fChildActor->mCreateShip(
					typeid( tFleetShip ),
					s1.shipCode.c_str(),
					kStartPosition_I + i,
					fChildActor->kNoMetaShipID,
					tFleetTeam::kFleet1 + i,
					0, 0, -1, -1,
					NULL,
					kDefaultShipOptions );

			}
		} else {
			mOldCreateFleetTeamShips();
		}
	} else {
		mOldCreateFleetTeamShips();
	}

	mSetTimedInterval( kTimerSecondsSet, kUpgradeFightersID, 2.0f + (0.2f * fChildActor->mGetTeam()) );
	gST.mDebug_fprintf("--- Finished: tFleetTeamExistsState::mCreateShipsForTeam\n");
}

void tFleetTeamExistsState::mOldCreateFleetTeamShips()
{
	gST.mDebug_fprintf("---- Starting: tFleetTeamExistsState::mOldCreateShipsForTeam\n");

	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );

	//	If playerTeam doesn't exist, fleet won't either
	//	(can this happen if player is killed right as fleet's coming in?)
	if (!playerTeam)
		return;	

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 fleetBPV;
	const int32 variantPct = 15;
	if( diffLevel == 0 )
	{
		fleetBPV = (playerBPV * 0.75)/4;
	}
	else if( diffLevel == 1 )
	{
		fleetBPV = (playerBPV)/4;
	}
	else if( diffLevel == 2 )
	{
		fleetBPV = (playerBPV * 1.25)/4;
	}
	fleetBPV = min(fleetBPV, 120); // Set minimum to 120

	// Gets BPV for range, ~120-160 BPV as minimums,
	// more if players fleet is larger than ~560 (assuming diffLevel 1)
	for (int i =0; tFleetTeam::kFleet1 + i <= tFleetTeam::kFleet7;i++)
	{
		fChildActor->mCreateShip(	typeid( tFleetShip ),
						fleetBPV,
						kClassLightCruiser,
						kClassHeavyBattlecruiser,
						variantPct,
						kStartPosition_I + i,
						tFleetTeam::kFleet1 + i,
						0, 0, -1, -1, NULL,
						kDefaultShipOptions );
	}

	gST.mDebug_fprintf("---- Finished: tFleetTeamExistsState::mOldCreateShipsForTeam\n");
}

void tFleetTeamExistsState::mEndState()
{}

void tFleetTeamExistsState::mOnDeath( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnCapture( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnDisengaged( tShipInfo* ship )
{}
void tFleetTeamExistsState::mAtDistance( tShipInfo* ship )
{}
void tFleetTeamExistsState::mAtDestination( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tFleetTeamExistsState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tFleetTeamExistsState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tFleetTeamExistsState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tFleetTeamExistsState::mOnProbeLaunch( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnBeginOrbit( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnEndOrbit( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tFleetTeamExistsState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tFleetTeamExistsState::mOnFlashingButtonPressed()
{}
void tFleetTeamExistsState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tFleetTeamExistsState::mOnChangingShip( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnSpecialTransport( tShipInfo* ship )
{}
void tFleetTeamExistsState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tFleetTeamExistsState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tFleetTeamExistsState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tFleetTeamExistsState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tFleetTeamExistsState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tFleetTeamExistsState::mOnHostChanged()
{}
void tFleetTeamExistsState::mOnTimedInterval( int32 timerID )
{
    if (timerID == kUpgradeFightersID)
    {
#ifdef FIGHTERS
        tTeamInfo* FleetTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID>( kFleetTeam ) );
        if (FleetTeam->mGetTeam() != fChildActor->mGetTeam()) return;  // only do for current team.
        if (fMissionInfo->mGetDifficultyLevel() < 2) return;            // only for admiral difficulty.
        if (FleetTeam->mIsTeamHumanControlled()) return;               // only for AI teams
        if (! gScriptInterface.mIsHost()) return;                       // only applied by the host PC.

        gST.mDebug_fprintf("--- Entering tFleetTeamExistsState::mOnTimedInterval -- timer kUpgradeFightersID triggered\n");

        int32 BPVinc = gST.mUpgradeFightersForTeam(FleetTeam);

        gST.mDebug_fprintf("Team BPV increased by: %d BPV\n", BPVinc);
        gST.mDebug_fprintf("--- Finished: tFleetTeamExistsState::mOnTimedInterval -- timer kUpgradeFightersID\n");
#endif
    }
}
void tFleetTeamExistsState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tFleetTeamExistsState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}