/********************************************************************************************************\

	File Name:	PlayerTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "BaseVictory.h"
#include "Uni_X4M3ADoors.h"
#include "MissionText.h"
#include "PlayerShip.h"
#include "PlayerTeam.h"
#include "PlayerTeamBaseState.h"
#include "VictoryCondition.h"
#include "ShipScriptDescription.h"
#include "CommonDefs.h"
#include "ShipList_API.h"

tClassAllocator<tPlayerTeam> gPlayerTeamAllocator;


void tPlayerTeam::mSetupVictoryCondition()
{
	fVictoryCondition = new tBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition);
}

/*

	HELPER NOTE:	The set of debriefings must be part of the player's team.  Simply keep this the
					way it is for single player missions.

*/
void tPlayerTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{
	if ( victoryLevel == kUnknownVictoryLevel )
	{
		fMissionInfo->mAddToDebriefingList( fTeam, kDisconnect_msg );
	}
	else
	{
		fMissionInfo->mAddToDebriefingList( fTeam, victoryLevel + kAstoundingVictory_msg );
	}

	switch (mGetRace())
	{
		case kFederation:
		case kHydran:

			switch ( victoryLevel )
			{
				case kAstoundingVictory:
					fMissionInfo->mAddToDebriefingList( fTeam, kFH_Code_msg );
					break;

				case kVictory:
					fMissionInfo->mAddToDebriefingList( fTeam, kFH_RefusedNoCode_msg );
					break;

				case kDraw:
					fMissionInfo->mAddToDebriefingList( fTeam, kFH_NoResponseToOfferNoCode_msg );
					break;

				case kDefeat:
					fMissionInfo->mAddToDebriefingList( fTeam, kFH_NoOffer_msg );
					break;

				case kDevastatingDefeat:
					fMissionInfo->mAddToDebriefingList( fTeam, kFH_Accepted_msg );
					break;
			}
			break;

		default:
			switch ( victoryLevel )
			{
				case kAstoundingVictory:
					fMissionInfo->mAddToDebriefingList( fTeam, kAccepted_msg );
					break;

				case kVictory:
					fMissionInfo->mAddToDebriefingList( fTeam, kCode_msg );
					break;

				case kDraw:
					fMissionInfo->mAddToDebriefingList( fTeam, kNoOffer_msg );
					break;

				case kDefeat:
					fMissionInfo->mAddToDebriefingList( fTeam, kNoResponseToOfferNoCode_msg );
					break;

				case kDevastatingDefeat:
					fMissionInfo->mAddToDebriefingList( fTeam, kRefusedNoCode_msg );
					break;
			}
			break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tPlayerTeam::mSetupState()
{
	mRegisterState( new tPlayerTeamBaseState() );
	mGotoState( typeid( tPlayerTeamBaseState ) );
}

void tPlayerTeam::mSetupCommState()
{}

void tPlayerTeam::mCreateShipsForTeam()
{}

void tPlayerTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL && mIsTeamHumanControlled() )
	{
		ship->mSetShipPlayerControlled();
	}
	if (gMapCenterX == 0)
	{
		//	Initialize Global Map Variables.
		int32 mapX, mapY;
		fMissionInfo->mGetMapDimensions(&mapX, &mapY);
		mInitMapVars(mapX, mapY);
	}
}

// Called once for each ship the player has.
void tPlayerTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
    gST.mDebug_fprintf("--- Entering: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
    gST.mInitializeSL();
    gST.mDraftedShipCensus( shipScriptDescription.fClassName.c_str() );

    // Hack that enforces AIs to NOT be a Meta ship, and thus get era drones!
    int32 MetaID = shipScriptDescription.fMetaDatabaseID;
#ifdef ERA_BASED_DRONE_SPEEDS
    if ((! mIsTeamHumanControlled()) && (fMissionInfo->mGetDifficultyLevel() >= 1)) {
        gST.mDebug_fprintf("Setting drones to era-correct speed. Team %d\n", mGetTeam());
        MetaID = kNoMetaShipID;
    }
#endif

	static int customShipNumber = kPlayer;

	mCreateShip( typeid( tPlayerShip ), 
				shipScriptDescription.fClassName.c_str(), 
				kStartPosition_H,
				MetaID,
				customShipNumber, 
				0, 0, -1, -1,
				shipScriptDescription.fCustomName.c_str(),
				static_cast< eShipOptions >( kDefaultShipOptions | kCanBeCarrier ) );

	customShipNumber++;

    gST.mDebug_fprintf("--- Finished: tPlayerTeam::mCreateNewShip: %s\n",
        shipScriptDescription.fClassName.c_str());
}

void tPlayerTeam::mScheduleNextMission( tVictoryCondition* VictoryCondition )
{
	fMissionScheduler.fVictoryLevel = VictoryCondition->mCalcVictoryStatus( mGetTeam() );
	fMissionScheduler.fPrestige = VictoryCondition->mGetPrestigePoints( fMissionScheduler.fVictoryLevel );
	fMissionScheduler.fBonusPrestige = VictoryCondition->mGetBonusPrestige();
	fMissionScheduler.fNextMissionScore = 450;
	fMissionScheduler.fNextMissionTitle = "Uni_4Resolve";

	switch ( fMissionScheduler.fVictoryLevel )
	{
	case kAstoundingVictory:
		{
			switch ( mGetRace() )
			{
			case kFederation:
			case kRomulan:
			case kLyran:
			case kGorn:
			case kMirak:
				{
					fMissionScheduler.fMedal = kMedalMissionTwo;
				}
				break;
			case kKlingon:
			case kHydran:
				{
					fMissionScheduler.fMedal = kMedalMissionThree;
				}
				break;
			}
		}
		break;
	}
}