/********************************************************************************************************\

	File Name:	TraitorTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "TraitorTeam.h"
#include "TraitorTeamBaseState.h"
#include "TraitorTeamShowState.h"
#include "TraitorTeamAtWarState.h"
#include "TraitorShip.h"
#include "Uni_X4M3ADoors.h"
#include "ShipList_API.h"
#include "ShipScriptDescription.h"

tClassAllocator<tTraitorTeam> gTraitorTeamAllocator;


void tTraitorTeam::mSetupState()
{
	mRegisterState( new tTraitorTeamBaseState() );
	mRegisterState( new tTraitorTeamShowState() );
	mRegisterState( new tTraitorTeamAtWarState() );
	mGotoState( typeid( tTraitorTeamBaseState ) );
}


void tTraitorTeam::mSetupCommState()
{}

void tTraitorTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Starting: tTraitorTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* PlayerTeam = fMissionInfo->mGetTeamHandle(static_cast<eTeamID>(kPlayerTeam));
		if (!PlayerTeam)
			return;
        int32 PlayerTeamBPV = PlayerTeam->mGetCombatBPV();
		int32 TraitorBPV = PlayerTeamBPV;

		switch (fMissionInfo->mGetDifficultyLevel()) {
		case 0:
			TraitorBPV *= 0.80; break;
		case 1:
			TraitorBPV *= 0.90; break;
		case 2:
		default:
			break;
		}

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

        ShipList ship_sl = sl;
        ship_sl = gST.stripOutRestricted( ship_sl );
        ship_sl = gST.stripOutSPECIAL( ship_sl );

		// Role will be LEADER.
		std::list< eEnhancedRoles >  leaderRolesList;
        leaderRolesList.push_back(kLeader);
        fShiplistCount = ship_sl.Subset(leaderRolesList);
        gST.mDebug_fprintf("leader roles subset: %d\n", fShiplistCount);

        // Let's get all the LEADER ships of same size or 1 size larger of your Team's BPV.
		ShipList leader_sl = ship_sl;
		fShiplistCount = leader_sl.Subset( leader_sl.kSubsetBPV, 0, (TraitorBPV-1), false);
		gST.mDebug_fprintf("remove all ships smaller than TraitorBPV (%d): %d\n", TraitorBPV, fShiplistCount);

		// figure out the smallest enhancedShipClass of the leader ships that has more BPV than PlayerTeam * difficulty modifier --FS
		int32 lowestLeaderBPV = kMaxBPV;
		eEnhancedClassTypes smallestLeaderHullClass = kEClassHeavyCruiser;
		{
		SHIP_LIST_ITER(leader_sl)
			if (iter->bpv < lowestLeaderBPV) {
				lowestLeaderBPV = iter->bpv;
				smallestLeaderHullClass = iter->enhancedShipClass;
			}
		SHIP_LIST_ITER_END
		}

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(smallestLeaderHullClass);
        fShiplistCount = leader_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(leader_sl);

		if (leader_sl.size() == 0) {
			gST.mDebug_fprintf("FAIL! Ok. Let's just get the best CC-like ships we can!\n");

			// start over
			leader_sl = ship_sl;
			std::list< eEnhancedClassTypes >  shipClassTypeList;
			for (int32 x = kEClassHeavyCruiser; x <= kEClassHeavyBattlecruiser; x++) {
				shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(x));
			}
			fShiplistCount = leader_sl.Subset(shipClassTypeList, true);
			gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);
			gST.listShiplistContent(leader_sl);
		}

        if (leader_sl.size() > 0) {
            // Create ship
            Ship ship1;
            leader_sl.SelectRandomShip(ship1);
            gST.mDebug_fprintf("ship1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                ship1.shipCode.c_str(), ship1.bpv, ship1.yearFirstAvail, ship1.yearLastAvail);
            gST.mTrackOrions( ship1 );

            mCreateShip(
                typeid( tTraitorShip),
                ship1.shipCode.c_str(),
                kStartPosition_G,
                kNoMetaShipID,
                kTraitor,
                0, 0, -1, -1,
                NULL,
                kDefaultShipOptions );

        } else {
            mOldCreateShipsForTeam();
        }
    } else {
        mOldCreateShipsForTeam();
    }

	gST.mDebug_fprintf("--- Finished: tTraitorTeam::mCreateShipsForTeam\n");
}

void tTraitorTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tTraitorTeam::mOldCreateShipsForTeam\n");

	mCreateShip(
		typeid( tTraitorShip ), 
		"I-CC", 
		kStartPosition_G, 
		kNoMetaShipID, 
		kTraitor,
		0, 0, -1, -1, 
		NULL, 
		kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tTraitorTeam::mOldCreateShipsForTeam\n");
}

void tTraitorTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tTraitorTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tTraitorTeam", shipScriptDescription );
}

