/********************************************************************************************************\

	File Name:	TraitorTeamShowState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "TraitorTeamShowState.h"
#include "Uni_X4M3ADoors.h"
#include "TraitorShip.h"
#include "PlayerTeam.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.  However, if you do need
					examples of the bodies of these functions, please feel free to refer back to ISC_5Accuse,
					which is the latest mission and the mission that contains files that most closely reflect
					this file and others in this script.

*/
void tTraitorTeamShowState::mBeginState()
{
	// Change state of Traitor to attack at half strength
	// somehow....
	
	tShipInfo* traitorShip = fChildActor->mFindShipCustomID(tTraitorTeam::kTraitor);
	if (!traitorShip)
		return;

	tTeamInfo* playerTeam  = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );
	if (!playerTeam)
		return;

	// Set Traitor to New Comm State - the chance for talking is over...
	playerTeam->mGotoCommState(traitorShip->mGetShipID(), tTraitorShip::kCommState9 );

	// How about setting all officers to rookies?
	for (int i = 0; i < kMaxOfficers; i++)
	{
		traitorShip->mSetOfficerRank( static_cast<eOfficerTypes>(i), kRookie );
	}

	// Then we'll set the goal to capture...
	fChildActor->mSetRelations( playerTeam->mGetTeam(), kTeamAtWar, kGoalCapture, 1.0);
	

	// And disable most of the AI Officers, including Transporter...
	for (i = 0; i <= kPlanetKillerOfficer; i++)
	{
		switch (i)
		{
			case kShieldReinforcementOfficer:
			case kECCMOfficer:
			case kTractorOfficer:
			case kHitAndRunOfficer:
			case kTBombOfficer:
			case kHETOfficer:
			case kCloakOfficer: // doesn't really matter
			case kFighterOfficer: // neither does this
			case kPlanetKillerOfficer: // or this
			{
				traitorShip->mChangeAIOfficerState(static_cast<eAIOfficers>(i), false);
				break;
			}
			// We want to keep these around, as they'll help keep the
			// traitor alive.
			case kECMOfficer:
			case kRepairOfficer:
			case kSeekingWeaponDefenseOfficer:
			default:
				break;
		}
	}

	// Clear previous orders
	fChildActor->mClearCommands();

	//	Set max speed low enough so player can get away...
	traitorShip->mSetShipMaxSpeed(20);

	// Set proper comm state.
	playerTeam->mGotoCommState(traitorShip->mGetShipID(), tTraitorShip::kCommState9 );

	//	Kill the ondamage notification
	traitorShip->mSetShipCallbackEvents( ~kNotifyDamage );

	//	Kill the Tractor Notification
	tShipInfo* playerShip = NULL;
	for (i = tPlayerTeam::kPlayer; i <=tPlayerTeam::kPlayer3; i++)
	{
		playerShip = playerTeam->mFindShipCustomID(i);
		if (playerShip && playerShip->mInPlay())
			playerShip->mSetShipCallbackEvents(~kNotifyBeginTractor);
	}


}
void tTraitorTeamShowState::mEndState()
{
	// Change All of the officers back.
	tShipInfo* traitorShip = fChildActor->mFindShipCustomID(tTraitorTeam::kTraitor);
	if (!traitorShip)
		return;

	for (int i = 0; i < kMaxOfficers; i++)
	{
		switch (fMissionInfo->mGetDifficultyLevel())
		{
		case 0:
			traitorShip->mSetOfficerRank( static_cast<eOfficerTypes>(i), kJunior );
			break;
		case 1:
		default: // shouldn't be req'd, but I dislike switches without a default option.
			traitorShip->mSetOfficerRank( static_cast<eOfficerTypes>(i), kSenior );
			break;
		case 2:
			traitorShip->mSetOfficerRank( static_cast<eOfficerTypes>(i), kVeteran );
			break;
		}
	}

	// re-enable the AI Officers
	for (i = 0; i <= kPlanetKillerOfficer; i++)
	{
		traitorShip->mChangeAIOfficerState(static_cast<eAIOfficers>(i), true);
	}
}

void tTraitorTeamShowState::mOnDeath( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnCapture( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnDisengaged( tShipInfo* ship )
{}
void tTraitorTeamShowState::mAtDistance( tShipInfo* ship )
{}
void tTraitorTeamShowState::mAtDestination( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tTraitorTeamShowState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tTraitorTeamShowState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tTraitorTeamShowState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tTraitorTeamShowState::mOnProbeLaunch( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnBeginOrbit( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnEndOrbit( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTraitorTeamShowState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tTraitorTeamShowState::mOnFlashingButtonPressed()
{}
void tTraitorTeamShowState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tTraitorTeamShowState::mOnChangingShip( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnSpecialTransport( tShipInfo* ship )
{}
void tTraitorTeamShowState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tTraitorTeamShowState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tTraitorTeamShowState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tTraitorTeamShowState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tTraitorTeamShowState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tTraitorTeamShowState::mOnHostChanged()
{}
void tTraitorTeamShowState::mOnTimedInterval( int32 timerID )
{}
void tTraitorTeamShowState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tTraitorTeamShowState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

