/********************************************************************************************************\

	Mission:	All Doors Must Open
	Script:		"Uni_X4M3ADoors"
	Descript:	Meet with an ISC Traitor and listen to his offer.

	Created by:		Richard K. Glover 07/04/2000
	Last Updated:	RKG 09/01/2000 
	
		09/01/2000: RKG; Updated to API v5.0
					DJS						09/26/00
					Meta script modifications


\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_X4M3ADoors.h"
#include "PlayerTeam.h"
#include "FleetTeam.h"
#include "PlanetTeam.h"
#include "TraitorTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X4M3ADoors );

tUni_X4M3ADoors::tUni_X4M3ADoors()
{}

void tUni_X4M3ADoors::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kFBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 1, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& playerTeam = mCreateTeamSpec(	"Player Team", 
												static_cast<eTeamID>(kPlayerTeam), 
												kMaxChance,
												kTrue, 
												kPrimaryTeam, 
												kMaxChance,
												kSpecPlayableRace & ~kSpecISC,
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kPlayerTeamTitle_msg] );

	const tTeamSpec& traitorTeam = mCreateTeamSpec(	"Traitor Team", 
												static_cast<eTeamID>(kTraitorTeam), 
												kMaxChance, 
												kTrue, 
												kPlayableTeam, 
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tTraitorTeam ) );

	const tTeamSpec& fleetTeam = mCreateTeamSpec(	"Fleet Team", 
												static_cast<eTeamID>(kFleetTeam), 
												kMaxChance, 
												kTrue, 
												kPlayableTeam, 
												kMaxChance,
												kSpecISC,
												kAnyRace,
												typeid( tFleetTeam ) );

	const tTeamSpec& planetTeam = mCreateTeamSpec(	"Planet Team", 
												static_cast<eTeamID>(kPlanetTeam),
												kMaxChance, 
												kTrue, 
												kPlayableTeam, 
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecMirak | kSpecHydran,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
	const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( playerTeam );
	Specs->mSetupTeam( traitorTeam );
	Specs->mSetupTeam( fleetTeam );
	Specs->mSetupTeam( planetTeam );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

/*

	HELPER NOTE:	Return the map you want to use for single player missions.  In multiplayer missions,
					you could examine which map was picked by the player.  For example:

						if ( info.fPreferedMap == 3 )
							;	//	the 3rd map was picked

					Also note that the index into the array of maps starts at 1, NOT 0!

*/
int32 tUni_X4M3ADoors::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

/*

	HELPER NOTE:	Called when the player starts the mission.  It will be further explained, if necessary,
					when it is better fleshed out.

*/
void tUni_X4M3ADoors::mCreateExternalShips( int32 Mode )
{}

/*

	HELPER NOTE:	Return the briefing message index that you want to use from MissionText.h

*/
int32 tUni_X4M3ADoors::mInitializeFinished( void )
{
	tTeamInfo*	playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>(kPlayerTeam) );


	if (!playerTeam)
		return kFBriefing_msg;

	switch (playerTeam->mGetRace())
	{
		case kFederation:
		default:
			return kFBriefing_msg;
		case kKlingon:
			return kKBriefing_msg;
		case kRomulan: 
			return kRBriefing_msg;
		case kLyran: 
			return kLBriefing_msg;
		case kHydran: 
			return kHBriefing_msg;
		case kGorn: 
			return kGBriefing_msg;
		case kMirak:
			return kMBriefing_msg;
	}
}

/*

	HELPER NOTE:	Called when a new ship enters the mission while in-game.  The Ship parameter holds
					the ship information for this specified ship.

*/
void tUni_X4M3ADoors::mNewInternalShip( tShipInfo* Ship )
{}

/*

	HELPER NOTE:	This is where you set up the teams' relationships and goals.  Read on and edit as
					you see necessary for this mission.

*/
void tUni_X4M3ADoors::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kTraitorTeam,
		kFleetTeam,
		kPlanetTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Trai = kTraitorTeam,
		//Flee = kFleetTeam,
		//Plan = kPlanetTeam,

		//	     Plyr  Trai  Flee  Plan
		/*Plyr*/ "--", "NI", "HI", "PI",
		/*Trai*/ "HD", "--", "AI", "PI",
		/*Flee*/ "WD", "AI", "--", "PI",
		/*Plan*/ "PI", "PI", "PI", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


/*

	HELPER NOTE:	Initialize your definitions of any drawing objects here.

*/
void tUni_X4M3ADoors::mInitializeDraw( const int32 MapPicked )
{
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tUni_X4M3ADoors::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['P'] = kObjectPlanet;
}

/*

	HELPER NOTE:	Fill in the briefing index for each team and race similar to the following manner:

  	fBriefingIndex[kPlayerTeam][kFederation] = kBriefingFederation_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kBriefingKlingon_msg;

	fBriefingIndex[kEnemyTeam][kISC] = kBriefingISC_msg;

*/
void tUni_X4M3ADoors::mDefineBriefings( void )
{
	fBriefingIndex[kPlayerTeam][kFederation] = kFBriefing_msg;
	fBriefingIndex[kPlayerTeam][kKlingon] = kKBriefing_msg;
	fBriefingIndex[kPlayerTeam][kRomulan] = kRBriefing_msg;
	fBriefingIndex[kPlayerTeam][kLyran] = kLBriefing_msg;
	fBriefingIndex[kPlayerTeam][kGorn] = kGBriefing_msg;
	fBriefingIndex[kPlayerTeam][kHydran] = kHBriefing_msg;
	fBriefingIndex[kPlayerTeam][kMirak] = kMBriefing_msg;
}

/*

	HELPER NOTE:	This is where you set the ship strengths for each non-kNPCTeam as follows:

  	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassCarrier, kClassBattleship, kMinBPV, kMaxBPV );

*/
void tUni_X4M3ADoors::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}