/********************************************************************************************************\

	Mission:	Orion Moon
	Script:		"Uni_X5M1AMoon"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		09/05/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>
					FS				09/23/2012
					era-correct ships, drones. Upgraded fighters.

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_X5M1AMoon.h"
#include "PlayerTeam.h"
#include "OrionTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X5M1AMoon );

tUni_X5M1AMoon::tUni_X5M1AMoon()
{}

int32 tUni_X5M1AMoon::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_X5M1AMoon::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMinChance,
#ifdef OP
												// OP is full of pirates. Everyone should get to enjoy these missions. Except ISC.
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC
#else
												// in EAW, Orions are only located near/in fed space.
												kSpecFed | kSpecRomulan | kSpecLyran | kSpecMirak,
#endif
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Orion Team",
												static_cast< eTeamID >( kOrionTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecOrion,
												kAnyRace,
												typeid( tOrionTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	const tTeamSpec& draft3Team = mCreateTeamSpec(  "Draft 3 Team",
                                                   static_cast< eTeamID >( kDraft3Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft4Team = mCreateTeamSpec(  "Draft 4 Team",
                                                   static_cast< eTeamID >( kDraft4Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
	Specs->mSetupTeam( draft3Team );
    Specs->mSetupTeam( draft4Team );
}

int32 tUni_X5M1AMoon::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_X5M1AMoon::mCreateExternalShips( int32 Mode )
{}

int32 tUni_X5M1AMoon::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_X5M1AMoon::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_X5M1AMoon::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_X5M1AMoon::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_X5M1AMoon::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kOrionTeam,
	};

	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Orin = kOrionTeam,

		//	     Plyr  Orin
		/*Plyr*/ "--", "WD",
		/*Orin*/ "WD", "--",
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}


void tUni_X5M1AMoon::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['L'] = kObjectPlanet;
}


tMapTerrain tUni_X5M1AMoon::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_X5M1AMoon::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}

void tUni_X5M1AMoon::mDefineBriefings( void )
{
	for( int32 nRace = kFederation; nRace < kMirak + 1; nRace++ )
	{
		fBriefingIndex[kPlayerTeam][nRace] = kBriefing_msg;
	}
}