/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "OrionTeam.h"
#include "OrionTeamBaseState.h"
#include "Defsat.h"
#include "Moon.h"
#include "OrionShip.h"
#include "Uni_X5M1FPyre.h"
#include "ShipList_API.h"

tClassAllocator<tOrionTeam> gOrionTeamAllocator;

tOrionTeam::tOrionTeam()
{
    std::string pyro_sacrificial_lamb = "X-BOX";
}

void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamBaseState() );
	mGotoState( typeid( tOrionTeamBaseState ) );
}

void tOrionTeam::mSetupCommState()
{}

void tOrionTeam::mCreateShipsForTeam()
{
	mChooseSacrificialLamb();
	mCreateMoon();
	mCreateDefSats();

    gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateShipsForTeam\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        int32 nStartPos = kStartPosition_L;
        int32 nCurrentID = kShip1;
        int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
        int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
        float nVarPercent;

        if( diffLevel == 0 )
        {
                nVarPercent = 0.65f;
        }
        else if( diffLevel == 1 )
        {
                nVarPercent = 1.0f;
        }
        else if( diffLevel == 2 )
        {
                nVarPercent = 1.25f;
        }

        for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
        {
            tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
            if( currentShip )
            {
                int32 thisShipBPV = int(currentShip->mGetShipBPV() * nVarPercent);

                // now we pick a ship with shiplist API. If it fails to pick one, use the old way.
                ShipList ship_sl = sl;
                fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), true);
                gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), fShiplistCount);
                gST.listShiplistContent(ship_sl);

                Ship s1;
                if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
                    gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                        s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                    gST.mTrackOrions( s1 );

                    mCreateShip(
                        typeid( tOrionShip ),
                        s1.shipCode.c_str(),
                        nStartPos,
                        kNoMetaShipID,
                        nCurrentID,
                        0, 0, 0, 0,
                        NULL,
                        kDefaultShipOptions );
                } else {
                    gST.mDebug_fprintf("Failed to pick ship #%d. Using old method. ShipBPV: %d\n", shipNumber, thisShipBPV);

                    eClassTypes thisShipClass = currentShip->mGetClassType();
                    mCreateShip(
                        typeid( tOrionShip ),
                        thisShipBPV,
                        thisShipClass,
                        0,
                        nStartPos,
                        nCurrentID,
                        0,0,0,0,
                        NULL,
                        kDefaultShipOptions );
                }
                nCurrentID++;
            }
        }
    } else {
        mOldCreateShipsForTeam(); // no enhanced shiplist at all. Go to old create code. --FS
    }

    gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
}


void tOrionTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tOrionTeam::mOldCreateShipsForTeam\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
			return;

	int32 nStartPos = kStartPosition_L;
	int32 nCurrentID = kShip1;
	int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;

	if( diffLevel == 0 )
	{
			nVarPercent = 0.65f;
	}
	else if( diffLevel == 1 )
	{
			nVarPercent = 1.0f;
	}
	else if( diffLevel == 2 )
	{
			nVarPercent = 1.25f;
	}

	for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
	{
		tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );

		if( currentShip )
		{
			eClassTypes thisShipClass = currentShip->mGetClassType();
			int32 thisShipBPV = currentShip->mGetShipBPV();
			mCreateShip(
				typeid( tOrionShip ), 
				thisShipBPV * nVarPercent, 
				thisShipClass, 
				0, 
				nStartPos, 
				nCurrentID, 
				0,0,0,0, 
				NULL, 
				kDefaultShipOptions );
			nCurrentID++;
		}
	}
	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateShipsForTeam\n");
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tOrionTeam", shipScriptDescription );
}

void tOrionTeam::mDestroyMoonBase()
{
	
}

void tOrionTeam::mDoPyrotechnics( int32 numReps )
{
	int32 reps = 0;
	while( reps != numReps )
	{
		mCreateShip( typeid( tOrionShip ),
					 pyro_sacrificial_lamb.c_str(),
					 kStartPosition_H,
					 kNoMetaShipID,
					 -1,
					 0,0,0,0,
				     NULL,
					 kNoShipOptions );

		reps++;
	}
}

void tOrionTeam::mCreateReinforcements()
{
    gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateReinforcements\n");
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
        if( !playerTeam )
                return;

        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassSmallMonster);
        shipClassTypeList.push_back(kEClassMediumMonster);
        shipClassTypeList.push_back(kEClassLargeMonster);
        for (int32 i = kEClassFrigate; i <= kEClassBattleship; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for ship classes filter completed. ships: %d\n", fShiplistCount);

        int32 nStartPos = kStartPosition_Z;
		int32 nCurrentID = kShip4;
        int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
        int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
        float nVarPercent;

        if( diffLevel == 0 )
        {
                nVarPercent = 0.65f;
        }
        else if( diffLevel == 1 )
        {
                nVarPercent = 1.0f;
        }
        else if( diffLevel == 2 )
        {
                nVarPercent = 1.25f;
        }

        for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
        {
            tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );
            if( currentShip )
            {
                int32 thisShipBPV = int(currentShip->mGetShipBPV() * nVarPercent);

                // now we pick a ship with shiplist API. If it fails to pick one, use the old way.
                ShipList ship_sl = sl;
                fShiplistCount = ship_sl.Subset( ship_sl.kSubsetBPV, int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), true);
                gST.mDebug_fprintf("ship_sl subset for BPV range (%d - %d) completed. ships: %d\n",
                    int(thisShipBPV * 0.825), int(thisShipBPV * 1.075), fShiplistCount);
                gST.listShiplistContent(ship_sl);

                Ship s1;
                if ((ship_sl.size() > 0) && (ship_sl.SelectRandomShip(s1))) {
                    gST.mDebug_fprintf("s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                        s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
                    gST.mTrackOrions( s1 );

                    mCreateShip(
                        typeid( tOrionShip ),
                        s1.shipCode.c_str(),
                        nStartPos,
                        kNoMetaShipID,
                        nCurrentID,
                        0, 0, 0, 0,
                        NULL,
                        kDefaultShipOptions );
                } else {
                    gST.mDebug_fprintf("Failed to pick ship #%d. Using old method. ShipBPV: %d\n", shipNumber, thisShipBPV);

                    eClassTypes thisShipClass = currentShip->mGetClassType();
                    mCreateShip(
                        typeid( tOrionShip ),
                        thisShipBPV,
                        thisShipClass,
                        0,
                        nStartPos,
                        nCurrentID,
                        0,0,0,0,
                        NULL,
                        kDefaultShipOptions );
                }
                nCurrentID++;
            }
        }
    } else {
        mOldCreateReinforcements(); // no enhanced shiplist at all. Go to old create code. --FS
    }

    gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateReinforcements\n");
}

void tOrionTeam::mOldCreateReinforcements()
{
	gST.mDebug_fprintf("---- Starting: tOrionTeam::mOldCreateReinforcements\n");

	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) );
	if( !playerTeam )
			return;

	int32 nStartPos = kStartPosition_Z;
	int32 nCurrentID = kShip4;
	int32 nPlayerShips = playerTeam->mGetShipsInPlayCount();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();
	float nVarPercent;

	if( diffLevel == 0 )
	{
			nVarPercent = 0.65f;
	}
	else if( diffLevel == 1 )
	{
			nVarPercent = 1.0f;
	}
	else if( diffLevel == 2 )
	{
			nVarPercent = 1.25f;
	}

	for( int32 shipNumber = 1; shipNumber < ( nPlayerShips + 1 ); shipNumber++ )
	{
		tShipInfo* currentShip = playerTeam->mGetThisShip( shipNumber );

		if( currentShip )
		{
			eClassTypes thisShipClass = currentShip->mGetClassType();
			int32 thisShipBPV = currentShip->mGetShipBPV();
			mCreateShip( 
				typeid( tOrionShip ), 
				thisShipBPV * nVarPercent, 
				thisShipClass, 
				0, 
				nStartPos, 
				nCurrentID, 
				0,0,0,0, 
				NULL, 
				kDefaultShipOptions );
			nCurrentID++;
		}
	}
	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateReinforcements\n");
}

void tOrionTeam::mCreateMoon()
{
    // Bah. not messing with this. --FS

    // The moon ( | )
	mCreateShip( typeid( tMoon ),
				 "PL0",
				 kStartPosition_H,
				 kNoMetaShipID,
				 kMoon,
				 0,0,0,0,
				 NULL,
				 kNoShipOptions );
}

void tOrionTeam::mCreateDefSats()
{
    gST.mDebug_fprintf("--- Starting: tOrionTeam::mCreateDefSats()\n");

    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();

        int32 fShiplistCount;
        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );

        // Defense platforms
        ShipList DEF_sl = sl;
        std::list< eEnhancedClassTypes >  DEFClassTypeList;
        DEFClassTypeList.push_back(kEClassPlatform);
        fShiplistCount = DEF_sl.Subset(DEFClassTypeList, true);
        gST.mDebug_fprintf("subset for Platform class filter completed. ships: %d\n", fShiplistCount);

        std::list< eEnhancedRoles >  DEFRolesList;
        DEFRolesList.push_back(kLocalDefense);
        fShiplistCount = DEF_sl.Subset(DEFRolesList, true);
        gST.mDebug_fprintf("subset for Local Defense eEnhancedRoles completed. ships: %d\n", fShiplistCount);
        gST.listShiplistContent(DEF_sl);

        if (DEF_sl.size() > 0) {
            Ship def1, def2, def3;
            DEF_sl.SelectRandomShip(def1);
            gST.mDebug_fprintf("def1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                def1.shipCode.c_str(), def1.bpv, def1.yearFirstAvail, def1.yearLastAvail);
            gST.mTrackOrions( def1 );

            DEF_sl.SelectRandomShip(def2);
            gST.mDebug_fprintf("def2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                def2.shipCode.c_str(), def2.bpv, def2.yearFirstAvail, def2.yearLastAvail);
            gST.mTrackOrions( def2 );

            DEF_sl.SelectRandomShip(def3);
            gST.mDebug_fprintf("def3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                def3.shipCode.c_str(), def3.bpv, def3.yearFirstAvail, def3.yearLastAvail);
            gST.mTrackOrions( def3 );

			mCreateShip( typeid( tDefsat ),
						 def1.shipCode.c_str(),
						 kStartPosition_I,
						 kNoMetaShipID,
						 kSat1,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );

			mCreateShip( typeid( tDefsat ),
						 def2.shipCode.c_str(),
						 kStartPosition_J,
						 kNoMetaShipID,
						 kSat2,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );

			mCreateShip( typeid( tDefsat ),
						 def3.shipCode.c_str(),
						 kStartPosition_K,
						 kNoMetaShipID,
						 kSat3,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );

        } else {
            mOldCreateDefSats();
        }
    } else {
        mOldCreateDefSats();
    }

    gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateDefSats()\n");
}

void tOrionTeam::mOldCreateDefSats()
{
    gST.mDebug_fprintf("---- Starting: tOrionTeam::mOldCreateDefSats\n");

    // Das defsats
	mCreateShip( typeid( tDefsat ),
				 "O-DEF",
				 kStartPosition_I,
				 kNoMetaShipID,
				 kSat1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tDefsat ),
				 "O-DEF",
				 kStartPosition_J,
				 kNoMetaShipID,
				 kSat2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tDefsat ),
				 "O-DEF",
				 kStartPosition_K,
				 kNoMetaShipID,
				 kSat3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

    gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateDefSats\n");
}

void tOrionTeam::mChooseSacrificialLamb()
{
    gST.mDebug_fprintf("---- Starting: tOrionTeam::mChooseSacrificialLamb()\n");
    gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        eRaceName race = mGetRace();

        ShipList sl = gST.narrowed_sl;
        std::string boxType = "X-BOX";
        switch (race) {
            case kFederation: boxType = "F-BOX"; break;
            case kKlingon: boxType = "K-BOX"; break;
            case kRomulan: boxType = "R-BOX"; break;
            case kGorn: boxType = "G-BOX"; break;
            case kLyran: boxType = "L-BOX"; break;
            case kHydran: boxType = "H-BOX"; break;
            case kISC: boxType = "I-BOX"; break;
            case kMirak: boxType = "Z-BOX"; break;

            default: break;
        }

        // And now to decide if we want to use shiplist API results or not
        Ship p1;
        if (sl.Find(boxType, p1)) {
            gST.mDebug_fprintf("Pyrotechnics provided by: %s\n", p1.shipCode.c_str());
            pyro_sacrificial_lamb = p1.shipCode.c_str();
        } else {
            gST.mDebug_fprintf("Could not find %s. Pyrotechnics provided by: %s\n", boxType, "X-BOX");
            pyro_sacrificial_lamb = "X-BOX";
        }
    }
    gST.mDebug_fprintf("---- Finished: tOrionTeam::mChooseSacrificialLamb()\n");
}