/********************************************************************************************************\

	Mission:	The Pyre
	Script:		"Uni_X5M1FPyre"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		09/08/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>
					FS				09/23/2012
					Era-correct ships, drones and upgraded fighters

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_X5M1FPyre.h"
#include "PlayerTeam.h"
#include "OrionTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X5M1FPyre );

tUni_X5M1FPyre::tUni_X5M1FPyre()
{}

int32 tUni_X5M1FPyre::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_X5M1FPyre::mScriptSpecs( tMissionSpec* Specs )
{
	gST.mOpenDebugFile();

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
										 		kPrimaryTeam,
												kMinChance,
#ifdef OP
                                                // OP is full of pirates. Everyone should get to enjoy these missions. Except ISC.
                                                kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC
#else
                                                // in EAW, Orions are only located near/in fed space.
                                                kSpecFed | kSpecRomulan | kSpecLyran | kSpecMirak,
#endif
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Orion Team",
												static_cast< eTeamID >( kOrionTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecOrion,
												kAnyRace,
												typeid( tOrionTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	const tTeamSpec& draft3Team = mCreateTeamSpec(  "Draft 3 Team",
                                                   static_cast< eTeamID >( kDraft3Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft4Team = mCreateTeamSpec(  "Draft 4 Team",
                                                   static_cast< eTeamID >( kDraft4Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
	Specs->mSetupTeam( draft3Team );
    Specs->mSetupTeam( draft4Team );
}

int32 tUni_X5M1FPyre::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_X5M1FPyre::mCreateExternalShips( int32 Mode )
{}

int32 tUni_X5M1FPyre::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_X5M1FPyre::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_X5M1FPyre::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_X5M1FPyre::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_X5M1FPyre::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 2;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kOrionTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Orin = kOrionTeam,
	
		//	     Plyr  Orin  
		/*Plyr*/ "--", "WD", 
		/*Orin*/ "WD", "--", 
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}

void tUni_X5M1FPyre::mASCIIOverrides( tMapOverrides* Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlanet;
}

tMapTerrain tUni_X5M1FPyre::mAssignMapTerrainTypes( int32 Map )
{
	return tMapTerrain( kTerrainAnyTerrain, kPlanetAnyPlanet, kBaseAnyBase );
}

void tUni_X5M1FPyre::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}

void tUni_X5M1FPyre::mDefineBriefings( void )
{
	for( int32 nRace = kFederation; nRace < kMirak + 1; nRace++ )
	{
		fBriefingIndex[kPlayerTeam][nRace] = kBriefing_msg;
	}
}