/********************************************************************************************************\

	File Name:	BaseVictoryState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "BaseVictoryState.h"
#include "MissionText.h"
#include "ScriptInterface.h"
#include "ShipInfo.h"
#include "Uni_X5M2Sake.h"
#include "PlayerTeam.h"
#include "AllyTeam.h"
#include "OrionTeam.h"
#include "PlanetTeam.h"

//
//	Serialization events
//
tBaseVictoryState::tBaseVictoryState( int32 astoundingVictory, int32 victory, int32 draw, int32 defeat, int32 devastatingDefeat ) :
	tChildActorVictoryState<tBaseVictory>( astoundingVictory, victory, draw, defeat, devastatingDefeat )
{}
void tBaseVictoryState::mBeginState()
{}
void tBaseVictoryState::mEndState()
{}

eVictoryLevels tBaseVictoryState::mCalcVictoryStatus( eTeamID ID )
{
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return kUnknownVictoryLevel;

	if( playerTeam->mResolvedPeacefully() )
		return kAstoundingVictory;

	else if( ( playerTeam->mOrionsDead() ) && ( !playerTeam->mAlliesDead() ) )
		return kVictory;

	else if( ( playerTeam->mOrionsDead() ) && ( playerTeam->mAlliesDead() ) )
		return kDraw;

	return kDevastatingDefeat;
}

void tBaseVictoryState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
			return;

	tTeamInfo* orionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) );
	if( !orionTeam )
			return;


	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			if ( !playerTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetDied();
				gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDestroyed );
			}			
		}
		break;

	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
	case tAllyTeam::kAlly3:
		{
			if( !allyTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetAlliesDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAlliesDead_msg );
			}
		}
		break;

	case tOrionTeam::kOrionFlagship:
		{
			playerTeam->mSetFlagshipDead();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kFlagshipDead_msg );

			if( !orionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetOrionsDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllOrionsDead_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tOrionTeam::kOrion1:
	case tOrionTeam::kOrion2:
	case tOrionTeam::kOrion3:
	case tOrionTeam::kOrion4:
		{
			if( !orionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetOrionsDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllOrionsDead_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;
	}
}
void tBaseVictoryState::mOnCapture( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
			return;

	tTeamInfo* orionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) );
	if( !orionTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			tTeamInfo*	team = fMissionInfo->mGetTeamHandle( ship->mGetTeam() );
			Assert( team );
			if ( team )
			{
				if ( !team->mShipsRemainingInPlay() )
					gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerCaptured );
			}
		}
		break;

	case tAllyTeam::kAlly1:
	case tAllyTeam::kAlly2:
	case tAllyTeam::kAlly3:
		{
			if( !allyTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetAlliesDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAlliesCapped_msg );
			}
		}
		break;

	case tOrionTeam::kOrionFlagship:
		{
			playerTeam->mSetFlagshipDead();
			fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kFlagshipCapped_msg );

			if( !orionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetOrionsDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllOrionsDead_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	case tOrionTeam::kOrion1:
	case tOrionTeam::kOrion2:
	case tOrionTeam::kOrion3:
	case tOrionTeam::kOrion4:
		{
			if( !orionTeam->mShipsRemainingInPlay() )
			{
				playerTeam->mSetOrionsDead();
				fMissionInfo->mDisplayMessage( static_cast<eTeamID>( kPlayerTeam ), kAllOrionsDead_msg );
				gScriptInterface.mSetGameStatus( static_cast<eTeamID>( kPlayerTeam ), kPlayerWins );
			}
		}
		break;

	}
}
void tBaseVictoryState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	
	tPlayerTeam* playerTeam = dynamic_cast<tPlayerTeam*>( fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kPlayerTeam ) ) );
	if( !playerTeam )
			return;

	tTeamInfo* allyTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kAllyTeam ) );
	if( !allyTeam )
			return;

	tTeamInfo* orionTeam = fMissionInfo->mGetTeamHandle( static_cast<eTeamID>( kOrionTeam ) );
	if( !orionTeam )
			return;

	switch ( ship->mGetCustomID() )
	{
	case tPlayerTeam::kPlayerShip1:
	case tPlayerTeam::kPlayerShip2:
	case tPlayerTeam::kPlayerShip3:
		{
			gScriptInterface.mSetGameStatus( static_cast< eTeamID >( kPlayerTeam ), kPlayerDisengaged );
		}
		break;
	}
}
void tBaseVictoryState::mAtDistance( tShipInfo* ship )
{}
void tBaseVictoryState::mAtDestination( tShipInfo* ship )
{}
void tBaseVictoryState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tBaseVictoryState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tBaseVictoryState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tBaseVictoryState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tBaseVictoryState::mOnProbeLaunch( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnEndOrbit( tShipInfo* ship )
{}
void tBaseVictoryState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tBaseVictoryState::mOnFlashingButtonPressed()
{}
void tBaseVictoryState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tBaseVictoryState::mOnChangingShip( tShipInfo* ship )
{}
void tBaseVictoryState::mOnSpecialTransport( tShipInfo* ship )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tBaseVictoryState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{}
void tBaseVictoryState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
												eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

//
//	System events
//
void tBaseVictoryState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tBaseVictoryState::mOnHostChanged()
{}
void tBaseVictoryState::mOnTimedInterval( int32 timerID )
{}
void tBaseVictoryState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tBaseVictoryState::mOnCommStateChange( tActor* actor, int32 state, int32 buttonID, eStateEvent eventType )
{}

