/********************************************************************************************************\

	File Name:	OrionTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "OrionTeam.h"
#include "OrionTeamBaseState.h"
#include "OrionShip.h"
#include "OrionFlagship.h"
#include "ShipList_API.h"


tClassAllocator<tOrionTeam> gOrionTeamAllocator;

void tOrionTeam::mSetupState()
{
	mRegisterState( new tOrionTeamBaseState() );
	mGotoState( typeid( tOrionTeamBaseState ) );
}

void tOrionTeam::mSetupCommState()
{}

void tOrionTeam::mCreateShipsForTeam()
{
    gST.mDebug_fprintf("--- Entering: tOrionTeam::mCreateShipsForTeam\n");
	gST.mInitializeSL();  // just to be safe
    gST.mCompleteNarrowedList();

    if ((gST.narrowed_sl.size() > 0) && (gST.narrowed_sl.mIsEnhancedShiplist())) {
        int32 fShiplistCount;

        ShipList sl = gST.mFilterShipListToSingleRace( gST.narrowed_sl, mGetRace() );
        sl = gST.stripOutSPECIAL( sl );
        sl = gST.stripOutRestricted( sl );
        sl = gST.selectWarShipRoles( sl );

		// Frigates
		ShipList FF_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassFrigate);
        fShiplistCount = FF_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for FF classes filter completed. ships: %d\n", fShiplistCount);
		}

		// Light Cruiser
		ShipList CL_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        shipClassTypeList.push_back(kEClassMediumMonster);
        for (int32 i = kEClassLightCruiser; i <= kEClassHeavyWarDestroyer; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CL_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CL classes filter completed. ships: %d\n", fShiplistCount);
		}

		// Heavy Cruiser
		ShipList CA_sl = sl;
		{
        std::list< eEnhancedClassTypes >  shipClassTypeList;
        for (int32 i = kEClassHeavyCruiser; i <= kEClassNewHeavyCruiser; i++) {
            shipClassTypeList.push_back(static_cast<eEnhancedClassTypes>(i));
        }
        fShiplistCount = CA_sl.Subset(shipClassTypeList, true);
        gST.mDebug_fprintf("subset for CA classes filter completed. ships: %d\n", fShiplistCount);
		}

		if ( (FF_sl.size() > 0) && (CL_sl.size() > 0) && (CA_sl.size() > 0) ) {
			Ship s1, s2, s3, s4, s5;

			CA_sl.SelectRandomShip(s1);
            gST.mDebug_fprintf("CA s1 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s1.shipCode.c_str(), s1.bpv, s1.yearFirstAvail, s1.yearLastAvail);
            gST.mTrackOrions( s1 );

			CL_sl.SelectRandomShip(s2);
            gST.mDebug_fprintf("CL s2 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s2.shipCode.c_str(), s2.bpv, s2.yearFirstAvail, s2.yearLastAvail);
            gST.mTrackOrions( s2 );

			FF_sl.SelectRandomShip(s3);
            gST.mDebug_fprintf("FF s3 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s3.shipCode.c_str(), s3.bpv, s3.yearFirstAvail, s3.yearLastAvail);
            gST.mTrackOrions( s3 );

			FF_sl.SelectRandomShip(s4);
            gST.mDebug_fprintf("FF s4 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s4.shipCode.c_str(), s4.bpv, s4.yearFirstAvail, s4.yearLastAvail);
            gST.mTrackOrions( s4 );

			FF_sl.SelectRandomShip(s5);
            gST.mDebug_fprintf("FF s5 %s selected. BPV: %d  YFA: %d  YLA %d\n",
                s5.shipCode.c_str(), s5.bpv, s5.yearFirstAvail, s5.yearLastAvail);
            gST.mTrackOrions( s5 );

			mCreateShip( typeid( tOrionFlagship ),
						 s1.shipCode.c_str(),
						 kStartPosition_H,
						 kNoMetaShipID,
						 kOrionFlagship,
						 0,0,0,0,
						 "Orion Flagship",
						 kDefaultShipOptions );

			mCreateShip( typeid( tOrionShip ),
						 s2.shipCode.c_str(),
						 kStartPosition_H,
						 kNoMetaShipID,
						 kOrion1,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );

			mCreateShip( typeid( tOrionShip ),
						 s3.shipCode.c_str(),
						 kStartPosition_H,
						 kNoMetaShipID,
						 kOrion2,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );

			mCreateShip( typeid( tOrionShip ),
						 s4.shipCode.c_str(),
						 kStartPosition_H,
						 kNoMetaShipID,
						 kOrion3,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );

			mCreateShip( typeid( tOrionShip ),
						 s5.shipCode.c_str(),
						 kStartPosition_H,
						 kNoMetaShipID,
						 kOrion4,
						 0,0,0,0,
						 NULL,
						 kDefaultShipOptions );
		} else {
			mOldCreateShipsForTeam();
		}
	} else {
		mOldCreateShipsForTeam();
	}
	gST.mDebug_fprintf("--- Finished: tOrionTeam::mCreateShipsForTeam\n");
}

void tOrionTeam::mOldCreateShipsForTeam()
{
	gST.mDebug_fprintf("---- Starting: tOrionTeam::mOldCreateShipsForTeam\n");

	mCreateShip( typeid( tOrionFlagship ),
				 "O-CA1",
				 kStartPosition_H,
				 kNoMetaShipID,
				 kOrionFlagship,
				 0,0,0,0,
				 "Orion Flagship",
				 kDefaultShipOptions );

	mCreateShip( typeid( tOrionShip ),
				 "O-CR1",
				 kStartPosition_H,
				 kNoMetaShipID,
				 kOrion1,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tOrionShip ),
				 "O-LR1",
				 kStartPosition_H,
				 kNoMetaShipID,
				 kOrion2,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tOrionShip ),
				 "O-LR1",
				 kStartPosition_H,
				 kNoMetaShipID,
				 kOrion3,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	mCreateShip( typeid( tOrionShip ),
				 "O-LR1",
				 kStartPosition_H,
				 kNoMetaShipID,
				 kOrion4,
				 0,0,0,0,
				 NULL,
				 kDefaultShipOptions );

	gST.mDebug_fprintf("---- Finished: tOrionTeam::mOldCreateShipsForTeam\n");
}

void tOrionTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}

void tOrionTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{
	gST.mCensusOnlyDraft( "tOrionTeam", shipScriptDescription );
}

