/********************************************************************************************************\

	Mission:	<MISSION TITLE>
	Script:		"Uni_X5M2Sake"
	Descript:	<DESCRIPTION>

	Created by:		Scott Bruno		09/12/00
	Last Updated:	<INITIALS>	mm/dd/yy
					<LOG>

\********************************************************************************************************/

#include "MissionText.h"
#include "MissionMaps.h"
#include "Uni_X5M2Sake.h"
#include "PlayerTeam.h"
#include "OrionTeam.h"
#include "PlanetTeam.h"
#include "AllyTeam.h"
#include "ShipList_API.h"
#include "DraftTeam.h"


DEFINE_SCRIPT_CLASS( tUni_X5M2Sake );

tUni_X5M2Sake::tUni_X5M2Sake()
{}

int32 tUni_X5M2Sake::mGetEnemyTeamID( void )
{
	return kTeam1;
}

void tUni_X5M2Sake::mScriptSpecs( tMissionSpec* Specs )
{
	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kBriefing_msg];
	Specs->fMissionType = kUniqueCampaignMission;
	Specs->fMetaLocationSpec = tMetaLocationSpec( 0, kAnyRaceHex, kAnyTerrainHex );
	Specs->fMinDate = kMinDate;
	Specs->fMaxDate = kMaxDate;
	Specs->fBandwidthPart = kAllBand;
	Specs->fGenericFlags = kStopMetaMovement;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Player Team",
												static_cast< eTeamID >( kPlayerTeam ),
												kMaxChance,
												kTrue,
												kPrimaryTeam,
												kMaxChance,
#ifdef OP
                                                // OP is full of pirates. Everyone should get to enjoy these missions. Except ISC.
                                                kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC
#else
                                                // in EAW, Orions are only located near/in fed space.
                                                kSpecFed | kSpecRomulan | kSpecLyran | kSpecMirak,
#endif
												kAnyRace,
												typeid( tPlayerTeam ),
												kNoTeamTag,
												Messages[kTitle2] );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Orion Team",
												static_cast< eTeamID >( kOrionTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecOrion,
												kAnyRace,
												typeid( tOrionTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Planet Team",
												static_cast< eTeamID >( kPlanetTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC,
												kAnyRace,
												typeid( tPlanetTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Ally Team",
												static_cast< eTeamID >( kAllyTeam ),
												kMaxChance,
												kTrue,
												kPlayableTeam,
												kMaxChance,
												kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecMirak, // kSpecPlayableRace && ~kSpecISC,
												kSameAs | kPlayerTeam,
												typeid( tAllyTeam ) );

	const tTeamSpec& draft1Team = mCreateTeamSpec(  "Draft 1 Team",
                                                   static_cast< eTeamID >( kDraft1Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );
   
    const tTeamSpec& draft2Team = mCreateTeamSpec(  "Draft 2 Team",
                                                   static_cast< eTeamID >( kDraft2Team ),
                                                   kMaxChance,
                                                   kTrue,
                                                   kPlayableTeam,
                                                   kMaxChance,
                                                   kSpecPlayableRace,
                                                   kAnyRace,
                                                   typeid( tDraftTeam ) );


	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( draft1Team );
    Specs->mSetupTeam( draft2Team );
}

int32 tUni_X5M2Sake::mInitializeStart( tDynaverseScriptInfo& info )
{
	return 1;
}

void tUni_X5M2Sake::mCreateExternalShips( int32 Mode )
{}

int32 tUni_X5M2Sake::mInitializeFinished( void )
{
	return kBriefing_msg;
}

void tUni_X5M2Sake::mNewInternalShip( tShipInfo* Ship )
{}

void tUni_X5M2Sake::mGetDebriefingList( eVictoryLevels VictoryLevel )
{}

int32 tUni_X5M2Sake::mGetTechnicalSuccessLevel( tDynaverseScriptInfo& info )
{
	return 0;
}

void tUni_X5M2Sake::mStartMission()
{
	tScript::mStartMission();

	const int kRelationsCount = 4;
	int32 teamList[kRelationsCount] =
	{
		kPlayerTeam,
		kOrionTeam,
		kPlanetTeam,
		kAllyTeam,
	};


	/*

		HELPER NOTE:	Below is the initial relationship / goal table.  Each letter has a meaning and each
						meaning has a meaning???

		Relationship Lookup
		- = kTeamSelfAware	//	use this parameter with itself
		A = kTeamAllied		//	cannot fire upon an ally
		F = kTeamFriendly
		N = kTeamNeutral
		H = kTeamHostile	//	will not fire / capture / cripple until provoked
		W = kTeamWar		//	will seek out and destroy / capture / cripple the enemy
		P = kTeamPassive

		Relationship Lookup
		- = kGoalIgnore		//	use this parameter with itself
		I = kGoalIgnore
		C = kGoalCripple
		X = kGoalCapture	//	do not have too many ships target a ship for capturing (accidently destroys it).
		D = kGoalDestroy

	*/
	std::string relationships[kRelationsCount][kRelationsCount] = 
	{
		//Plyr = kPlayerTeam,
		//Orin = kOrionTeam,
		//Plan = kPlanetTeam,
		//Ally = kAllyTeam,

		//	     Plyr  Orin  Plan  Ally
		/*Plyr*/ "--", "HI", "AI", "AI",
		/*Orin*/ "HI", "--", "HI", "HI",
		/*Plan*/ "AI", "HI", "--", "AI",
		/*Ally*/ "AI", "HI", "AI", "--"
	};
	fMissionInfo->mSetTeamRelations( teamList, (std::string*)relationships, kRelationsCount);
}



tMapTerrain tUni_X5M2Sake::mAssignMapTerrainTypes( int32 Map )
{
	switch ( Map )
	{
	case 0:
		{
			return tMapTerrain( kTerrainSpace, kPlanetHomeWorld, kBaseStarbase );
		}
		break;
	default:
		{}
		break;
	}
	return tMapTerrain();
}

void tUni_X5M2Sake::mDefineTeamShipStrengths( void )
{
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
	mSetTeamShipStrength( kPlayerTeam, kClassFrigate, kClassBattleship, kMinBPV, kMaxBPV );
}

void tUni_X5M2Sake::mDefineBriefings( void )
{
	for( int32 nRace = kFederation; nRace < kMirak + 1; nRace++ )
	{
		fBriefingIndex[kPlayerTeam][nRace] = kBriefing_msg;
	}
}


void tUni_X5M2Sake::mASCIIOverrides( tMapOverrides *Map )
{
	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlanet;
	Map->fTacticalMapIcon['H'] = kObjectEnemyShip;
}