//	<Name>	tdebug.h

//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.

//	<Info>	Abstract Debug output part definition.
//
//  Note:	This debug part has been designed so that (at least on Windows) you
//			can use it at -any- time. Thought it is a part, you can use it
//			regardless of whether or not the Tiger part mechanism has been
//			activated or shut down. This allows us to have debug output
//			statements in other part mActivate() methods. In fact you can even
//			use it inside of constructors of statically instantiated objects.
//			To do this safely you must use the 'mSendLineAlways()' method.
//
//			For now this method is only available for Win32 with MSVC.
//			If this is possible on MacOS, then perhaps we'll make it standard.
//
//
#ifndef	tdebug_h
#define	tdebug_h

#include "tpart.h"
#include "tlearn.h"

class	tDebugPart: public tPart
{
protected:
	// The following data member should be declared in the Win32 or MacOS
	// specific part .CPP file. It should not be assigned any value but NULL.
	//
	static	tDebugPart	*fTheDebugPart;

public:
	tDebugPart()			{	fTheDebugPart = this;	}
	virtual	~tDebugPart()	{	/* nothing	*/			};
				
	GenAbstractBinding( tDebugPart );

	// Targets
	//
	enum
	{
		// Predefined targets
		//
		kDefault	= 0x0001,
		kDebugger	= 0x0002,
		kComPort	= 0x0004,
		kWindow		= 0x0008,
		k2ndMonitor	= 0x0010,
		kMonoMonitor= 0x0020,

			// If new predefined targets are needed, place them here
			// (before _kPredefinedMax)

			// Client code should ignore following two
			//
			_kPredefinedMax,
			_kPredefinedMask	= (_kPredefinedMax-1) | (_kPredefinedMax-2)
	};

	virtual void	mSendLine(const char *str, const int32u kTargets=kDefault)	= 0;
	virtual int32u	mCreateLogFileTarget(
						const char	*filename,
						iTruth		bAppend	= kFalse,		
						iTruth		bCreateIfNecessary = kTrue)					= 0;
	virtual void	mCloseLogFileTarget(const char *filename)					= 0;
	virtual void	mCloseLogFileTarget(const int32u kTargetID)					= 0;

	// Since the tDebugPart is designed so you can call it at -any- time
	// (even if tPart::mActivate() has not been called, or the constructor
	// for the tDebugPart has not been called), you should -not-
	// use the 'aDebugPart->mSendLine()', instead use the following method
	//
	//
	static void mSendLineAlways(const char *str, const int32u kTargets=kDefault)
	{
		// fTheDebugPart should be NULL if the part has not been constructed yet
		//
		if (fTheDebugPart)
		{
			fTheDebugPart->mSendLine(str,kTargets);
		}
	}
	static	tDebugPart	*mGet(void)
	{
		return( fTheDebugPart );
	}
};

// Following macro is the standard Tiger method of accessing this part
//
#define	aDebugPart	(BindPart(tDebugPart))

// Following macro allows simple one line output to debugger
//
#define	_DEBUGGERSAY(a)	aDebugPart->mSendLine(a,tDebugPart::kDebugger)

// Following macro allows simple one line output to default output
//
#define	_DEFAULTSAY(a)	aDebugPart->mSendLine(a,tDebugPart::kDefault)


#endif	// #ifndef tdebug_h


//	12Feb97 mhertogh	created.
//	30Jul97	dmandap		Made destructor virtual, changed interface of mSendLine()
//	03Dec97	mhertogh	brought up to Style spec.
//	09Dec97	dmandap		Removed superfluous comma in enumeration list (kill warning)
//	17Dec97	dmandap		Added new pre-defined target, kMonoMonitor
//	28May98	dmandap		Changed ::mCreateLogFileTarget() to accept a second parameter,
//						this parameter specifies whether to append or overwrite the file.
//						Added the ::mGet() method.
//

