//	<Name>	tdmpBase.h
//
//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//
//	<Info>	Provides the following:
//			1) Definition of 'tErrInfo' class.
//			   This is such an essential data structure which should really
//			   be defined in 'tbasic.h'. I use it to throw exceptions with.
//			   
//			2) Macros to access improved memory management functions
//			   The current tiger memory parts provide no mechanism to
//			   track memory allocation. Hence these macros. When compiling
//			   under MSVC with _DEBUG defined, the macros provide access
//			   to the extensive (read super-coolio) debug heap management
//			   routines that MSVC can provide.
//			   
//			3) A macro to provide conditional 'say' statements.
//			   The 'tsay.h' header file in tiger provides a 'tSay' object
//			   for this purpose. However that header file includes
//			   system header files (such as <stdio.h> and <conio.h>) that
//			   I do NOT want my code to accidentally include.
//
//

#ifndef _TDMPBASE_H_
#define _TDMPBASE_H_

#include "tlearn.h"		// for CompilerMicrosoft, CompilerMetrowerks etc
#include "tbasic.h"		// for int8u, int16u etc

/*--------------------------------[ tErrInfo ]------------------------------*/
// This is a class that you can use to perform more descriptive exception
// handling with. By throwing this data type, the catch block can examine
// the 'fLocale' and 'fMsg' fields to determine where and why the exception
// occurred.
//
// use like this:
//	void tFoo::mExampleMethod(void)
//	{
//		if (bad thing happens)
//		{
//			throw tErrInfo(__FILE__,__LINE__,
//				"tFoo::mExampleMethod(): Something terrible happend"
//			);
//		}
//	}
//
//
class tErrInfo
{
public:
    char    fLocale[512];
    char    fMsg[1024];

    tErrInfo() { fLocale[0] = 0; fMsg[0] = 0; }

    tErrInfo(
        const char  *pFileName,
        int         lineNum,
        const char  *fmt,
        ...);
};

/*---------------------------------[ Macros ]-------------------------------*/
extern void _dmpSay(const char *fmt,...);
#if defined(_DEBUG)
	#define SAY(a)      _dmpSay a
#else
	#define	SAY(a)		// whitespace
#endif

#undef MALLOC
#undef CALLOC
#undef FREE
#if CompilerMicrosoft && defined(_DEBUG) && defined(_CRTDBG_MAP_ALLOC)
	extern void *_dmpMalloc_dbg(unsigned size, const char *fname, int lineNum);
	extern void *_dmpCalloc_dbg(unsigned size, const char *fname, int lineNum);
	extern void _dmpFree_dbg(void *p, const char *fname, int lineNum);

	#define MALLOC(a)   _dmpMalloc_dbg((a),__FILE__,__LINE__)
	#define CALLOC(a)   _dmpCalloc_dbg((a),__FILE__,__LINE__)
	#define FREE(a)     _dmpFree_dbg((a),__FILE__,__LINE__)
#else
	extern void *_dmpMalloc(unsigned size);
	extern void *_dmpCalloc(unsigned size);
	extern void _dmpFree(void *p);

	#define MALLOC(a)   _dmpMalloc((a))
	#define CALLOC(a)   _dmpCalloc((a))
	#define FREE(a)     _dmpFree((a))
#endif


#endif  // _TDMPBASE_H_

//	<History>
//	05Jun97	dmandap	Moved from the 'dmp' project folder. Based off of 'dmpBase.h'


/* End of file: tdmpBase.h
 */


