//	<Name>	tsay.h
//
//	<Legal>	Copyright (c) 1997,1996 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.
//  
//	<Info>	Debug say class

#ifndef tsay_h
#define tsay_h

#include "tbasic.h"
#include "tdebug.h"

class tSay
{
private:
	struct tMasterControl
	{
		int		fNumberActiveInstances;	// total number of tSay instances
		iTruth	fOnceOnly;
		iTruth	fOnOff;					// kFalse if all tSay objects disabled
		iTruth	fNamesOnOff;			// kTrue if names should be displayed
		iTruth	fLevelOnOff;			// 

		inline tMasterControl()
		{
			fNumberActiveInstances 	= 0;
			fOnceOnly				= kTrue;
			fOnOff 					= kTrue;
			fNamesOnOff				= kTrue;
			fLevelOnOff				= kTrue;
		}
	}; 

private:
	static tMasterControl 	fMasterControl;

	enum	{	kMaxUserName = 80	};
	char	fUserName[kMaxUserName];	// this instance's user defined name
	int		fLevel;						// priority level
	char	fIndentChar;				// character used to display indentation
	int		fIndentAmount;				// amount of characters to indent
	int		fSkipRate;					// output on every fSkipRate iteration
	int		fSkipCntr;					// used to keep track of skip rate iteration
	int32u	fTargets;					// output target(s)
	int32u	fLogFileTarget;				// target ID for logfile
	iTruth	fOnOff;						// TRUE if output enabled

protected:
	void	mHandleOutput(const int kLevel, const char *stringMessage);

public:
	tSay(const char *userName=NULL, const int32u kTargetID=tDebugPart::kDebugger);
	virtual ~tSay();

	// Master Control functions
	//
	static	void mSetMasterOnOff(const iTruth bOnOff);
	static	void mSetShowNamesOnOff(const iTruth bOnOff);
	static	void mSetShowLevelsOnOff(const iTruth bOnOff);

	// Enable/Disable output, set debug (priority) level
	//
	void	mSetOnOff(const iTruth bOnOff);
	void	mSetLevel(const int kLevel);

	void	mSetIndentChar(const char c);
	void	mSetIndent(const int kIndent);
	int		mIncrementIndent(const int kAmount);

	// Sets the number calls to ::mSay() that will be ignored
	//
	void	mSetSkipRate(const int kSkipRate);

	// Enable/Disable debug output target (e.g. tDebugPart::kComPort etc)
	// 
	void	mEnableTarget(const int32u kTargetID);
	void	mDisableTarget(const int32u kTargetID);

	// Specify logfile target, close logfile target
	//
	iTruth	mSetLogFile(const char *name);
	iTruth	mCloseLogFile(void);

	// 		
	void	mJustSayIt(const char *format,...);
	void	mSay(const char *format,...);
	void	mSay(const int kLevel, const char *format,...);
};

// Use following object (_gGlobalSay) when you don't wish to create a
// tSay object just to provide simple debug output. A macro is also provided
// below which allows you to easily use this global tSay object
//
// Note:	Do -NOT- use this object or the macro unless Tiger parts have
//			been initialized. So, don't use them in your constructors or
//			destructors!!! Sorry, but that's just how it is.
// 
// 
extern tSay	_gGlobalSay;
#ifndef TGSAY
	#if defined(_DEBUG)
		#define TGSAY(a)      _gGlobalSay.mJustSayIt a
		#define TGSAY2(a)      { TGSAY(a); TGSAY((" -- %s %d\n", __FILE__, __LINE__)); }
	#else
		#define	TGSAY(a)		// whitespace
		#define TGSAY2(a)     // whitespace
	#endif
#endif

#endif	// #ifdef _TSAY_H_

//<HISTORY>
// ?????96  gmarster	created
// 14Feb97	mhertogh	Adapted to use tdebug msendLine
// 18Feb97	mhertogh	Made more tiger codebook compliant
// 06Aug97	dmandap		Adapted to use new tDebugPart. Also made truly OS neutral
// 03Oct97	dmandap		Added global (_gGlobalSay) object and TGSAY macro
// 03Dec97	mhertogh	Removed end of file comment
// 23Jun98	oschlunk	Added TGSAY2 macro -- for those people who want file and line info...