//	<Name>	tsequence.h

//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.

//	<Info>	Generic interface of a number sequence generator

#ifndef tsequence_h
#define tsequence_h 1

#include "tbasic.h"

class tSequencePart
{
protected:

	int32u fNumberCount;		// the number of numbers generated so far

	const int32u kHistorySize;	// the number of numbers to keep track of that have been generated
	int32u *fHistory;			// the history of numbers previously generated

	int32u fSeed;				// the seed for the number generator
	int32u fPreviousResult;		// the result previously generated by the number generator
	
	// the number generator for the sequence - to be defined by a sub-class
	virtual int32u		mGenerateNumber(void)		= 0;

public:

	//--------------------------------------------------------------------------------
	// Constructors and deconstructors
	//
	
	tSequencePart(int32u initialSeed = 1, int32u historySize = 0) 
		: fSeed(initialSeed), kHistorySize(historySize), fNumberCount(0)
	{
		if (kHistorySize > 0)
		{
			fHistory = new int32u[kHistorySize];
		}
		else
		{
			fHistory = NULL;
		}

		fPreviousResult = fSeed;
	};

	virtual ~tSequencePart()
	{
		if (fHistory)
		{
			delete [] fHistory;
		}
	};
	
	//--------------------------------------------------------------------------------
	// Setting and getting the seed for the number generator
	//
	
	// sets a new seed for the number generator
	virtual void	mSetSeed(int32u newSeed)
	{
		fSeed = newSeed;
		fPreviousResult = newSeed;
	}
	
	// returns the current initial seed being used
	virtual int32u	mGetSeed(void) { return fSeed; }
	
	//--------------------------------------------------------------------------------
	// Getting numbers from the generator
	//
	
	// returns the next number in the sequence
	virtual int32u	mGetNumber(void);
	
	// returns the specified number in the sequence (1-based)
	virtual int32u	mGetNumber(int32u index);
	
	// return a number from the number generator with as little overhead as possible
	virtual int32u	mQuickGet(void) { return mGenerateNumber(); }
	
};

#endif

//	<History>
//	26Nov97	vtagle		Created.
//	10Dec97	vtagle		Added function to quickly get a number from the number generator
//						Updated comments to conform to new standards
//	02Apr98	vtagle		Sequence history is now optional
//	13Jul98	oschlunk	Made most of the fuctions virtual



