//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	this is a structure for streaming things out

// Primary Precompiled Header
#include "Precompiled.h"

#include "tsay.h"
#include "sfcstreams.h"
#include "tvectormath.h"
#include "tstring.h"



tSFCStream::tSFCStream(int8u *buffer, int32 x) 
{ 
	_func_start( tSFCStream::tSFCStream );
	fBuffer = buffer; 
	fOwned = kFalse; 
	fGetPtr = fBuffer; 
	fPutPtr = fBuffer; 
	fMaxPtr = fBuffer + x; 
	fReadLock = kFalse;
	fWriteLock = kFalse;
	if (fBuffer == NULL)
	{
		TGSAY2(("Major Trouble!"));
		fMaxPtr = NULL; 
	}
	fTabNumber = 0;
	_func_end;
};

tSFCStream::tSFCStream(int32 x) 
{ 
	_func_start( tSFCStream::tSFCStream );
	fOwned = kTrue;
	fBuffer = new int8u[x]; 
	fMaxPtr = fBuffer + x; 
	if (fBuffer == NULL)
	{
		TGSAY2(("Major Trouble!"));
		fMaxPtr = NULL; 
	}
	fGetPtr = fBuffer; 
	fPutPtr = fBuffer; 
	fReadLock = kFalse;
	fWriteLock = kFalse;
	fTabNumber = 0;
	fVersionNumber = -1;
	_func_end;
};


void tSFCStream::mReset()
{
	_func_start( tSFCStream::mReset );
	fGetPtr = fBuffer; 
	fPutPtr = fBuffer; 
	fReadLock = kFalse;
	fWriteLock = kFalse;
	fTabNumber = 0;
	_func_end;
}

void tSFCStream::mSetVersionNumber( int32 versionNumber )
{
	_func_start( tSFCStream::mSetVersionNumber );
	fVersionNumber = versionNumber;
	_func_end;
}

int tSFCStream::mGetVersionNumber()
{
	_func_start( tSFCStream::mGetVersionNumber );
	return fVersionNumber;
	_func_end;
}

tSFCStream::~tSFCStream() 
{ 
	_func_start( tSFCStream::~tSFCStream );
	if (fBuffer != NULL) 
	{ 
		if (fOwned == kTrue) 
		{ 
			delete fBuffer; 
		} 
		fBuffer = NULL; 
		fGetPtr = NULL; 
		fPutPtr = NULL; 
	}; 
	_func_end;
};

tSFCStream& tSFCStream::operator<<(tPoint3& num) 
{ 
	_func_start( tSFCStream::operator << );
	*this << num.x;
	*this << num.y;
	*this << num.z;
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator<<(tPoint4& num) 
{ 
	_func_start( tSFCStream::operator << );
	*this << num.x;
	*this << num.y;
	*this << num.z;
	*this << num.w;
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator<<(tPoint6& num) 
{ 
	_func_start( tSFCStream::operator << );
	*this << num.fXYZ;
	*this << num.fNormal;
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator<<(tPoint2& num) 
{ 
	_func_start( tSFCStream::operator << );
	*this << num.x;
	*this << num.y;
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator<<(tMatrix& num) 
{ 
	_func_start( tSFCStream::operator << );
	tMtx4x4&	mtx = num.mGetMtx4x4();
//	inline void	mSet3x4Flag(const iTruth bIs3x4);
	iTruth	val = num.mGet3x4Flag();
	*this << val;
	if ((mtx[0][0] == 1.0f) && (mtx[0][1] == 0.f) && (mtx[0][2] == 0.f) && (mtx[0][3] == 0.f) && 
    (mtx[1][0] == 0.f) && (mtx[1][1] == 1.0f) && (mtx[1][2] ==0.f) &&  (mtx[1][3] == 0.f) && 
    (mtx[2][0] == 0.f) && (mtx[2][1] == 0.f) && (mtx[2][2] == 1.0f) && (mtx[2][3] == 0.f) && 
    (mtx[3][0] == 0.f) && (mtx[3][1] == 0.f) && (mtx[3][2] == 0.f) && (mtx[3][3] == 1.0f))
	{
		val = kTrue;
	}
	else
	{
		val = kFalse;
	}
	*this << val;

	if (val == kTrue)
	{
		return (*this);
	}

	int32 i,j;
	for(i=0;i<4;i++)
	{
		for (j=0;j<4;j++)
		{
			*this << mtx[i][j];
		}
	}

	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator<<(tString& num) 
{ 
	_func_start( tSFCStream::operator << );
	iBasicStringElement *wString = num.mWideStr();
	int	len = num.mStrLen()+1;

	int32 i;
	for (i=0;i<len;i++)
	{
		*this << wString[i];
	}
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator<<( std::string& value ) 
{ 
	_func_start( tSFCStream::operator << );
	int len = value.length()+1;
	for( int i = 0; i < len; i++ )
	{
		*this << value[i];
	}

	return (*this);
	_func_end;
};


tSFCStream& tSFCStream::operator>>(tPoint3& num) 
{ 
	_func_start( tSFCStream::operator >> );
	*this >> num.x;
	*this >> num.y;
	*this >> num.z;
	return (*this);
	_func_end;
};


tSFCStream& tSFCStream::operator>>(tPoint4& num) 
{ 
	_func_start( tSFCStream::operator >> );
	*this >> num.x;
	*this >> num.y;
	*this >> num.z;
	*this >> num.w;
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator>>(tPoint6& num) 
{ 
	_func_start( tSFCStream::operator >> );
	*this >> num.fXYZ;
	*this >> num.fNormal;
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator>>(tPoint2& num) 
{ 
	_func_start( tSFCStream::operator >> );
	*this >> num.x;
	*this >> num.y;
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator>>(tMatrix& num) 
{ 
	_func_start( tSFCStream::operator >> );
	num.mIdentity();

	iTruth	val;
	*this >> val;
	num.mSet3x4Flag(val);
 	*this >> val;
	if (val == kTrue)
	{
		return (*this);
	}

	tMtx4x4& mtx = num.mGetMtx4x4();


	int32 i,j;
	for(i=0;i<4;i++)
	{
		for (j=0;j<4;j++)
		{
			*this >> mtx[i][j];
		}
	}

	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator>>(tString& num) 
{ 
	_func_start( tSFCStream::operator >> );
	num = "";
	iBasicStringElement wString;
	*this >> wString;
	while (wString != (iBasicStringElement)0)
	{
		num += wString; 
		*this >> wString;
	}
	return (*this);
	_func_end;
};

tSFCStream& tSFCStream::operator>>( std::string& value)
{
	_func_start( tSFCStream::operator >> );
	std::string::value_type element;
	*this >> element;
	
	while( element != (std::string::value_type)0 )
	{
		value += element;
		*this >> element;
	}

	return (*this);
	_func_end;
}

void tSFCStream::mWriteOutTabs()
{
	_func_start( tSFCStream::mWriteOutTabs );
	char c = '\t';
	int32 i;
	for (i=0;i<fTabNumber;i++)
	{
		mWriteToStream((int8u *)(&c),1);
	}
	_func_end;
};

void tSFCStream::mReadInTabs()
{
	_func_start( tSFCStream::mReadInTabs );
	char c;
	int32 i;
	for (i=0;i<fTabNumber;i++)
	{
		mReadFromStream((int8u *)(&c),1);
	}
	_func_end;
}

// these need to become endian aware...
void tSFCStream::mWriteToStream(int8u *x, int32 size, iTruth ordered) 
{ 
	_func_start( tSFCStream::mWriteToStream );
	if (fReadLock == kTrue)
	{
		TGSAY2(("Trying to write to a read-only stream!"));
		return;
	}

	if (x == NULL)
	{
		TGSAY2(("Trying to write to a Null ptr!"));
		return;
	}

#if	EndianBig
	if (ordered == kTrue)
	{
		x = x+size-1;
		while ((size > 0) && (fPutPtr < fMaxPtr)) 
		{ 
			*fPutPtr = *x; fPutPtr++; x--; size--; 
		}; 
	}
	else
	{
		while ((size > 0) && (fPutPtr < fMaxPtr)) 
		{ 
			*fPutPtr = *x; fPutPtr++; x++;  size--; 
		}; 
	}
#else
	while ((size > 0) && (fPutPtr < fMaxPtr)) 
	{ 
		*fPutPtr = *x; fPutPtr++; x++;  size--; 
	}; 
#endif
	if (size > 0) 
	{ 
		TGSAY2(("Buffer trouble!")); 
	}
	_func_end;
};


int32 tSFCStream::mReadFromStream(int8u *x, int32 size, iTruth ordered) 
{ 
	_func_start( tSFCStream::mReadFromStream );
	int32 initialSize = size;

	if (fWriteLock == kTrue)
	{
		TGSAY2(("Trying to read from a write-only stream!"));
		return 0;
	}

	if (x == NULL)
	{
		TGSAY2(("Trying to read from a Null ptr!"));
		return 0;
	}

#if	EndianBig
	if (ordered == kTrue)
	{
		x = x+size-1;
		while ((size > 0) && (fGetPtr < fPutPtr)) 
		{ 
			*x = *fGetPtr; fGetPtr++; x--;  size--;  
		}; 
	}
	else
	{
		while ((size > 0) && (fGetPtr < fPutPtr)) 
		{ 
			*x = *fGetPtr; fGetPtr++; x++;  size--; 
		}; 
	}
#else
	while ((size > 0) && (fGetPtr < fPutPtr)) 
	{ 
		*x = *fGetPtr; fGetPtr++; x++; size--;  
	}; 
#endif
	if (size > 0) 
	{ 
		TGSAY2(("Buffer trouble!")); 
	}

	return initialSize - size;
	_func_end;
};

void tSFCStream::mSetReadSize(int32 readSize)	
{	
	_func_start( tSFCStream::mSetReadSize );
	fPutPtr = fGetPtr+readSize;	
	if (fPutPtr > fMaxPtr)
	{
		fPutPtr = fMaxPtr;
		TGSAY2(("Trying to set the write ptr too far up!"));
	}
	_func_end;
};

void tSFCStream::mSetReadPtr(int8u *ptr)	
{	
	_func_start( tSFCStream::mSetReadPtr );
	fGetPtr = ptr;	
	if (fGetPtr > fPutPtr)
	{
		fGetPtr = fPutPtr;
		TGSAY2(("Trying to set the read ptr too far up!"));
	}
	if (fGetPtr < fBuffer)
	{
		fGetPtr = fBuffer;
		TGSAY2(("Trying to set the read ptr too far down!"));
	}
	_func_end;
};

void tSFCStream::mSetWritePtr(int8u *ptr)	
{	
	_func_start( tSFCStream::mSetWritePtr );
	fPutPtr = ptr;	
	if (fPutPtr > fMaxPtr)
	{
		fPutPtr = fMaxPtr;
		TGSAY2(("Trying to set the write ptr too far up!"));
	}
	if (fPutPtr < fGetPtr)
	{
		fPutPtr = fGetPtr;
		TGSAY2(("Trying to set the write ptr too far down!"));
	}
	_func_end;
};

void TextStreamCompress(char *text, tSFCStream &s)
{
	_func_start( TextStreamCompress );
	s.mWriteStringToStream(text);
	_func_end;
}
void TextStreamUncompress(char *text, tSFCStream &s)
{
	_func_start( TextStreamUncompress );
	s.mReadStringFromStream(text);
	_func_end;
}

void Int32StreamCompress(int32& num, tSFCStream &s)
{
	_func_start( Int32StreamCompress );
	int8u overflowMask = 0x80;
	int8u negativeMask = 0x40;
	int8u smallMask = 0x3F;
	int8u bigMask = 0x7F;
	int32u valueToSend;
	int8u temp;

	if (num < 0)
	{
		valueToSend = (int32u)(-num);
		if (valueToSend < 64)
		{
			temp = (int8u)valueToSend;
			temp = 	temp | negativeMask;
			s << temp;
			return;
		}
		else
		{
 			temp = (int8u)(valueToSend&smallMask);
			temp = (temp | overflowMask | negativeMask);
		}
	}
	else 
	{
		valueToSend = (int32u)num;
		if (valueToSend < 64)
		{
			temp = (int8u)valueToSend;
			s << temp;
			return;
		}
		else
		{
 			temp = (int8u)(valueToSend&smallMask);
			temp = (temp | overflowMask);
		}
	}

	s << temp;
	valueToSend = (valueToSend>>6);
	while (valueToSend >= 128)
	{
 		temp = (int8u)(valueToSend&bigMask);
		temp = (temp | overflowMask);
		s << temp;
		valueToSend = (valueToSend>>7);
	}
	temp = (int8u)valueToSend;
	s << temp;

	_func_end;
}

void Int32StreamUncompress(int32& num, tSFCStream &s)
{
	_func_start( Int32StreamUncompress );
	int8u overflowMask = 0x80;
	int8u negativeMask = 0x40;
	int8u smallMask = 0x3F;
	int8u bigMask = 0x7F;
	int32 value;
	int32 multiplier;
	int8u temp;
	iTruth bNegative = kFalse;

	s >> temp;
	num = (temp&smallMask);
	if ((temp&negativeMask) > 0)
	{
		bNegative = kTrue;
	}

	// we are negative
	if ((temp&overflowMask) > 0)
	{
		multiplier = 64;
		s >> temp;
		while ((temp&overflowMask) > 0)
		{
			int32 value = (temp&bigMask);
			num += multiplier*value;
			multiplier *= 128;
			s >> temp;
		}
		value = (temp&bigMask);
		num += multiplier*value;
	}

	if (bNegative == kTrue)
	{
		num = -num;
	}

	_func_end;
}

void Int32uStreamCompress(int32u& num, tSFCStream &s)
{
	_func_start( Int32uStreamCompress );
	int8u overflowMask = 0x80;
	int8u bigMask = 0x7F;
	int32u valueToSend = num;
	int8u temp;

	while (valueToSend >= 128)
	{
 		temp = (int8u)(valueToSend&bigMask);
		temp = (temp | overflowMask);
		s << temp;
		valueToSend = (valueToSend>>7);
	}
	temp = (int8u)valueToSend;
	s << temp;

	_func_end;
}

void Int32uStreamUncompress(int32u& num, tSFCStream &s)
{
	_func_start( Int32uStreamUncompress );
	int8u overflowMask = 0x80;
	int8u bigMask = 0x7F;
	int32 value;
	int32 multiplier = 1;
	int8u temp;

	s >> temp;
	num = 0;
	while ((temp&overflowMask) > 0)
	{
		int32 value = (temp&bigMask);
		num += multiplier*value;
		multiplier *= 128;
		s >> temp;
	}
	value = (temp&bigMask);
	num += multiplier*value;
	_func_end;
}

void FloatStreamCompress(float& num, tSFCStream &s)
{
	_func_start( FloatStreamCompress );

	//
	// 20011016 JEK This function produces rounding errors when the
	//				value is > 999.5f
	//				Therefore with the agreement of Marc I decided
	//				to disable compression and stream the float
	//				to the network verbatim.
	//

	s << num;
	
/*	iTruth bNegative = kFalse;
	float val = num;

	if (num < 0)
	{
		bNegative = kTrue;
		val = -num;
	}
	int32 multiplier = 0;
	if (val < .00000000000009995f)
	{
		multiplier = 0;
		val = 0;
	}
	else if (val > 999.5)
	{
		val = .1*val;
		multiplier++;
		while (val > 999.5)
		{
			val = .1*val;
			multiplier++;
		}
	}
	else if (val < 99.95)
	{
		val = 10*val;
		multiplier--;
		while (val < 99.95)
		{
			val = 10*val;
			multiplier--;
		}

	}

	val += .5;
	int16 value = (int32)((int16)(val))+((multiplier+16)<<10);
	if ((multiplier < -16) || (multiplier > 15))
	{
		TGSAY2(("multiplier out of range!"));
		value = 0;
	}
	if (bNegative == kTrue)
	{
		value = -value;
	}
	s << value; */
	_func_end;
}

/*
float gFloatUncompressConstant[32] = 
{
	.0000000000000001f,
	.000000000000001f,
	.00000000000001f,
	.0000000000001f,
	.000000000001f,
	.00000000001f,
	.0000000001f,
	.000000001f,
	.00000001f,
	.0000001f,
	.000001f,
	.00001f,
	.0001f,
	.001f,
	.01f,
	.1f,
	1.f,
	10.f,
	100.f,
	1000.f,
	10000.0f,
	100000.0f,
	1000000.0f,
	10000000.0f,
	100000000.0f,
	1000000000.0f,
	10000000000.0f,
	100000000000.0f,
	1000000000000.0f,
	10000000000000.0f,
	100000000000000.0f,
	1000000000000000.0f,
};
*/

void FloatStreamUncompress(float& num, tSFCStream &s)
{
	_func_start( FloatStreamUncompress );

	//
	// 20011016 JEK This function produces rounding errors when the
	//				value is > 999.5f
	//				Therefore with the agreement of Marc I decided
	//				to disable compression and stream the float
	//				from the network verbatim.
	//

	s >> num;
	
/*	int16 value;
	s >> value;
	iTruth bNegative = kFalse;
	if (value < 0)
	{
		bNegative = kTrue;
		value = -value;
	}
	int32 multiplier = (value>>10);
	value = value&(0x000003FF);
	if (value == 0)
	{
		num = 0.f;
		return;
	}

	num = (float)value * gFloatUncompressConstant[multiplier];
	if (bNegative == kTrue)
	{
		num = -num;
	} */
	_func_end;
}

void TPoint3StreamCompress(tPoint3& num, tSFCStream &s)
{
	_func_start( TPoint3StreamCompress );
	float val = num.x;
	FloatStreamCompress(val,s);
	val = num.y;
	FloatStreamCompress(val,s);
	val = num.z;
	FloatStreamCompress(val,s);
	_func_end;
}

void TPoint3StreamUncompress(tPoint3& num, tSFCStream &s)
{
	_func_start( TPoint3StreamUncompress );
	float val; 
	FloatStreamUncompress(val,s);
	num.x = val;
	FloatStreamUncompress(val,s);
	num.y = val;
	FloatStreamUncompress(val,s);
	num.z = val;
	_func_end;
}



void TPoint4StreamCompress(tPoint4& num, tSFCStream &s)
{
	_func_start( TPoint4StreamCompress );
	float val = num.x;
	FloatStreamCompress(val,s);
	val = num.y;
	FloatStreamCompress(val,s);
	val = num.z;
	FloatStreamCompress(val,s);
	val = num.w;
	FloatStreamCompress(val,s);
	_func_end;
}

void TPoint4StreamUncompress(tPoint4& num, tSFCStream &s)
{
	_func_start( TPoint4StreamUncompress );
	float val; 
	FloatStreamUncompress(val,s);
	num.x = val;
	FloatStreamUncompress(val,s);
	num.y = val;
	FloatStreamUncompress(val,s);
	num.z = val;
	FloatStreamUncompress(val,s);
	num.w = val;
	_func_end;
}

void TPoint6StreamCompress(tPoint6& num, tSFCStream &s)
{
	_func_start( TPoint6StreamCompress );
	TPoint3StreamCompress(num.fXYZ, s);
	TPoint3StreamCompress(num.fNormal, s);
	_func_end;
}

void TPoint6StreamUncompress(tPoint6& num, tSFCStream &s)
{
	_func_start( TPoint6StreamUncompress );
	TPoint3StreamUncompress(num.fXYZ, s);
	TPoint3StreamUncompress(num.fNormal, s);
	_func_end;
}

void TPoint2StreamCompress(tPoint2& num, tSFCStream &s)
{
	_func_start( TPoint2StreamCompress );
	float val = num.x;
	FloatStreamCompress(val,s);
	val = num.y;
	FloatStreamCompress(val,s);
	_func_end;
}

void TPoint2StreamUncompress(tPoint2& num, tSFCStream &s)
{
	_func_start( TPoint2StreamUncompress );
	float val; 
	FloatStreamUncompress(val,s);
	num.x = val;
	FloatStreamUncompress(val,s);
	num.y = val;
	_func_end;
}


void TMatrixStreamCompress(tMatrix& num, tSFCStream &s)
{
	_func_start( TMatrixStreamCompress );
	tMtx4x4&	mtx = num.mGetMtx4x4();

	iTruth	val = num.mGet3x4Flag();
	ITruthStreamCompress(val,s);

	if ((mtx[0][0] == 1.0f) && (mtx[0][1] == 0.f) && (mtx[0][2] == 0.f) && (mtx[0][3] == 0.f) && 
    (mtx[1][0] == 0.f) && (mtx[1][1] == 1.0f) && (mtx[1][2] ==0.f) &&  (mtx[1][3] == 0.f) && 
    (mtx[2][0] == 0.f) && (mtx[2][1] == 0.f) && (mtx[2][2] == 1.0f) && (mtx[2][3] == 0.f) && 
    (mtx[3][0] == 0.f) && (mtx[3][1] == 0.f) && (mtx[3][2] == 0.f) && (mtx[3][3] == 1.0f))
	{
		val = kTrue;
	}
	else
	{
		val = kFalse;
	}
	ITruthStreamCompress(val,s);

	if (val == kTrue)
	{
		return;
	}

	int32 i,j;
	for(i=0;i<4;i++)
	{
		for (j=0;j<4;j++)
		{
			FloatStreamCompress(mtx[i][j],s);
		}
	}

	return;
	_func_end;
}

void TMatrixStreamUncompress(tMatrix& num, tSFCStream &s)
{
	_func_start( TMatrixStreamUncompress );
	num.mIdentity();

	iTruth	val;
	ITruthStreamUncompress(val,s);

	num.mSet3x4Flag(val);
	ITruthStreamUncompress(val,s);

	if (val == kTrue)
	{
		return;
	}

	tMtx4x4& mtx = num.mGetMtx4x4();


	int32 i,j;
	for(i=0;i<4;i++)
	{
		for (j=0;j<4;j++)
		{
			FloatStreamUncompress(mtx[i][j],s);
		}
	}

	return;
	_func_end;
}


void TStringStreamCompress(tString& num, tSFCStream &s )
{
	_func_start( TStringStreamCompress );
	s << num;
	_func_end;
}

void TStringStreamUncompress(tString& num, tSFCStream &s )
{
	_func_start( TStringStreamUncompress );
	s >> num;
	_func_end;
}

void TStdStringStreamCompress( std::string& str, tSFCStream &s )
{
	_func_start( TStdStringStreamCompress );
	s << str;
	_func_end;
}

void TStdStringStreamUncompress( std::string& str, tSFCStream &s )
{
	_func_start( TStdStringStreamUncompress );
	s >> str;
	_func_end;
}





// Beware these convert a double to a float and then do the above -- if that's not appropriate,
//	do not use these
void DoubleStreamCompress(double& num, tSFCStream &s)
{
	_func_start( DoubleStreamCompress );
	float val = (float)num;
	FloatStreamCompress(val,s);
	_func_end;
}

void DoubleStreamUncompress(double& num, tSFCStream &s)
{
	_func_start( DoubleStreamUncompress );
	float val;
	FloatStreamUncompress(val,s);
	num = (double)val;
	_func_end;
}

void Int16StreamCompress(int16& num, tSFCStream &s)
{
	_func_start( Int16StreamCompress );
	int32 i = (int32)num;
	Int32StreamCompress(i,s);
	_func_end;
}

void Int16StreamUncompress(int16& num, tSFCStream &s)
{
	_func_start( Int16StreamUncompress );
	int32 i;
	Int32StreamUncompress(i,s);
	num = (int16)i;
	_func_end;
}


void Int16uStreamCompress(int16u& num, tSFCStream &s)
{
	_func_start( Int16uStreamCompress );
	int32u i = (int32u)num;
	Int32uStreamCompress(i,s);
	_func_end;
}

void Int16uStreamUncompress(int16u& num, tSFCStream &s)
{
	_func_start( Int16uStreamUncompress );
	int32u i;
	Int32uStreamUncompress(i,s);
	num = (int16u)i;
	_func_end;
}



void Int8StreamCompress(int8& num, tSFCStream &s)
{
	_func_start( Int8StreamCompress );
	s << num;
	_func_end;
}

void Int8StreamUncompress(int8& num, tSFCStream &s)
{
	_func_start( Int8StreamUncompress );
	s >> num;
	_func_end;
}


void Int8uStreamCompress(int8u& num, tSFCStream &s)
{
	_func_start( Int8uStreamCompress );
	s << num;
	_func_end;
}

void Int8uStreamUncompress(int8u& num, tSFCStream &s)
{
	_func_start( Int8uStreamUncompress );
	s >> num;
	_func_end;
}


void ITruthStreamCompress(iTruth& num, tSFCStream &s)
{
	_func_start( ITruthStreamCompress );
	s << num;
	_func_end;
}


void ITruthStreamUncompress(iTruth& num, tSFCStream &s)
{
	_func_start( ITruthStreamUncompress );
	s >> num;
	_func_end;
}


// you've got some kind of integer and you don't know what size it is
void IntegerStreamCompress(int32 size, void *num, tSFCStream &s)
{
	_func_start( IntegerStreamCompress );
	switch (size)
	{
	case 1:
		{
			int8 val = (*(int8 *)num);
			Int8StreamCompress(val,s);
		}
		break;
	case 2:
		{
			int16 val = (*(int16 *)num);
			Int16StreamCompress(val,s);
		}
		break;
	case 4:
		{
			int32 val = (*(int32 *)num);
			Int32StreamCompress(val,s);
		}
		break;
	default:
		{
			TGSAY2(("Unknown Size!"));
		}
 		break;
	}
	_func_end;
}

void IntegerStreamUncompress(int32 size, void *num, tSFCStream &s)
{
	_func_start( IntegerStreamUncompress );
	switch (size)
	{
	case 1:
		{
			int8 val;
			Int8StreamUncompress(val,s);
			*((int8 *)num) = val;
		}
		break;
	case 2:
		{
			int16 val;
			Int16StreamUncompress(val,s);
			*((int16 *)num) = val;
		}
		break;
	case 4:
		{
			int32 val;
			Int32StreamUncompress(val,s);
			*((int32 *)num) = val;
		}
		break;
	default:
		{
			TGSAY2(("Unknown Size!"));
		}
 		break;
	}
	_func_end;
}




// this compresses an int32 -- numbers from -63 to 63 will take only 1 byte...
void Int32StreamOutText(char *text, int32& num, tSFCStream &s)
{
	_func_start( Int32StreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %ld",text,num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void Int32StreamInText(char *text, int32& num, tSFCStream &s)
{
	_func_start( Int32StreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %ld");
	s.mReadTextLineFromStream(buffer);
	sscanf(buffer,format,&num);
	_func_end;
}




void Int32uStreamOutText(char *text, int32u& num, tSFCStream &s)
{
	_func_start( Int32uStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %lu",text,num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void Int32uStreamInText(char *text, int32u& num, tSFCStream &s)
{
	_func_start( Int32uStreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %lu");
	s.mReadTextLineFromStream(buffer);
	sscanf(buffer,format,&num);
	_func_end;
}



void Int16StreamOutText(char *text, int16& num, tSFCStream &s)
{
	_func_start( Int16StreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %ld",text,(int32)num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void Int16StreamInText(char *text, int16& num, tSFCStream &s)
{
	_func_start( Int16StreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %ld");
	s.mReadTextLineFromStream(buffer);
	int32 val;
	sscanf(buffer,format,&val);
	num = (int16)val;
	_func_end;
}



void Int16uStreamOutText(char *text, int16u& num, tSFCStream &s)
{
	_func_start( Int16uStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %lu",text,(int32u)num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void Int16uStreamInText(char *text, int16u& num, tSFCStream &s)
{
	_func_start( Int16uStreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %lu");
	s.mReadTextLineFromStream(buffer);
	int32u val;
	sscanf(buffer,format,&val);
	num = (int16u)val;
	_func_end;
}



void Int8StreamOutText(char *text, int8& num, tSFCStream &s)
{
	_func_start( Int8StreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %ld",text,(int32)num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void Int8StreamInText(char *text, int8& num, tSFCStream &s)
{
	_func_start( Int8StreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %ld");
	s.mReadTextLineFromStream(buffer);
	int32 val;
	sscanf(buffer,format,&val);
	num = (int8)val;
	_func_end;
}



void Int8uStreamOutText(char *text, int8u& num, tSFCStream &s)
{
	_func_start( Int8uStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %lu",text,(int32u)num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void Int8uStreamInText(char *text, int8u& num, tSFCStream &s)
{
	_func_start( Int8uStreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %lu");
	s.mReadTextLineFromStream(buffer);
	int32u val;
	sscanf(buffer,format,&val);
	num = (int8u)val;
	_func_end;
}



void ITruthStreamOutText(char *text, iTruth& num, tSFCStream &s)
{
	_func_start( ITruthStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %lu",text,(int32u)num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void ITruthStreamInText(char *text, iTruth& num, tSFCStream &s)
{
	_func_start( ITruthStreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %lu");
	s.mReadTextLineFromStream(buffer);
	int32u val;
	sscanf(buffer,format,&val);
	num = (iTruth)val;
	_func_end;
}


void FloatStreamOutText(char *text, float& num, tSFCStream &s)
{
	_func_start( FloatStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %f",text,num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}

void FloatStreamInText(char *text, float& num, tSFCStream &s)
{
	_func_start( FloatStreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %f");
	s.mReadTextLineFromStream(buffer);
	sscanf(buffer,format,&num);
	_func_end;
}

void TPoint3StreamOutText(char *text, tPoint3& num, tSFCStream &s)
{
	_func_start( TPoint3StreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = x=%f y=%f z=%f",text,num.x,num.y,num.z);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}

void TPoint3StreamInText(char *text, tPoint3& num, tSFCStream &s)
{
	_func_start( TPoint3StreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = x=%f y=%f z=%f");
	s.mReadTextLineFromStream(buffer);
	float x,y,z;
	sscanf(buffer,format,&x,&y,&z);
	num.x = x;
	num.y = y;
	num.z = z;
	_func_end;
}

void TPoint2StreamOutText(char *text, tPoint2& num, tSFCStream &s)
{
	_func_start( TPoint2StreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = x=%f y=%f",text,num.x,num.y);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}

void TPoint2StreamInText(char *text, tPoint2& num, tSFCStream &s)
{
	_func_start( TPoint2StreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = x=%f y=%f");
	s.mReadTextLineFromStream(buffer);
	float x,y;
	sscanf(buffer,format,&x,&y);
	num.x = x;
	num.y = y;
	_func_end;
}

void TPoint4StreamOutText(char *text, tPoint4& num, tSFCStream &s)
{
	_func_start( TPoint4StreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = x=%f y=%f z=%f w=%f",text,num.x,num.y,num.z,num.w);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}

void TPoint4StreamInText(char *text, tPoint4& num, tSFCStream &s)
{
	_func_start( TPoint4StreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = x=%f y=%f z=%f w=%f");
	s.mReadTextLineFromStream(buffer);
	float x,y,z,w;
	sscanf(buffer,format,&x,&y,&z,&w);
	num.x = x;
	num.y = y;
	num.z = z;
	num.w = w;
	_func_end;
}

void TPoint6StreamOutText(char *text, tPoint6& num, tSFCStream &s)
{
	_func_start( TPoint6StreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s",text);
	s.mWriteTextLineToStream(buffer);
	s.mIncrementTabNumber(); 
	TPoint3StreamOutText("fXYZ", num.fXYZ, s);
	TPoint3StreamOutText("fNormal", num.fNormal, s);
	s.mDecrementTabNumber(); 
	_func_end;
}

void TPoint6StreamInText(char *text, tPoint6& num, tSFCStream &s)
{
	_func_start( TPoint6StreamInText );
	s.mReadInTabs();
	char buffer[256];
	s.mReadTextLineFromStream(buffer);
	s.mIncrementTabNumber(); 
	TPoint3StreamInText("fXYZ", num.fXYZ, s);
	TPoint3StreamInText("fNormal", num.fNormal, s);
	s.mDecrementTabNumber(); 
	_func_end;
}

void TStringStreamOutText( tString& text, tString& num, tSFCStream &s)
{
	_func_start( TStringStreamOutText );
	s.mWriteOutTabs();
	tString composite = text;
	composite += " = ";
	composite += num;
//	wchar_t buffer[256];
//	iBasicStringElement *wString = composite.mWideStr();
	s << composite;
	s << '\n';
//	swprintf(buffer,"%s = %s",text,wString);
//	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void TStringStreamInText( tString& text, tString& num, tSFCStream &s)
{
	_func_start( TStringStreamInText );
	s.mReadInTabs();
	tString composite = text;
	composite += " = ";
//	composite += num;
	tString wString;
//	wchar_t buffer[256];
//	iBasicStringElement *wString = composite.mWideStr();
	s >> wString;
	num = wString.substr(composite.mStrLen());
	char c;
	s >> c;

//	wchar_t buffer[256];
//	wchar_t format[256];
//	wcscpy(format,text);
//	wcscat(format," = ");
//	s.mReadTextLineFromStream(buffer);
//	int32 len = wcslen(format);
//	num = &buffer[len];
	_func_end;
}

void TStdStringStreamOutText( std::string& text, std::string& num, tSFCStream &s)
{
	_func_start( TStdStringStreamOutText );
	s.mWriteOutTabs();
	std::string composite = text;
	text += " = ";
	text += num;

	s << composite;
	s << '\n';
	_func_end;
}


void TStdStringStreamInText( std::string& text, std::string& num, tSFCStream &s)
{
	_func_start( TStdStringStreamInText );
	s.mReadInTabs();

	std::string wString;
	s >> wString;
	
	num = text;
	num += " = "; 
	num += wString;
	_func_end;
}




void TMatrixStreamOutText(char *text, tMatrix& num, tSFCStream &s)
{
	_func_start( TMatrixStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s",text);
	s.mWriteTextLineToStream(buffer);

	s.mIncrementTabNumber(); 

	tMtx4x4& mtx = num.mGetMtx4x4();
	iTruth val;
	iTruth	val3x4 = num.mGet3x4Flag();

	if ((mtx[0][0] == 1.0f) && (mtx[0][1] == 0.f) && (mtx[0][2] == 0.f) && (mtx[0][3] == 0.f) && 
    (mtx[1][0] == 0.f) && (mtx[1][1] == 1.0f) && (mtx[1][2] ==0.f) &&  (mtx[1][3] == 0.f) && 
    (mtx[2][0] == 0.f) && (mtx[2][1] == 0.f) && (mtx[2][2] == 1.0f) && (mtx[2][3] == 0.f) && 
    (mtx[3][0] == 0.f) && (mtx[3][1] == 0.f) && (mtx[3][2] == 0.f) && (mtx[3][3] == 1.0f))
	{
		val = kTrue;
	}
	else
	{
		val = kFalse;
	}

	s.mWriteOutTabs();
	sprintf(buffer,"identity =%ld 3x4=%ld",(int32)val,(int32)val3x4);
	s.mWriteTextLineToStream(buffer);

	if (val == kFalse)
	{
		s.mWriteOutTabs();
		sprintf(buffer,"mtx[0] = [0]=%f [1]=%f [2]=%f [3]=%f",mtx[0][0],mtx[0][1],mtx[0][2],mtx[0][3]);
		s.mWriteTextLineToStream(buffer);

		s.mWriteOutTabs();
		sprintf(buffer,"mtx[1] = [0]=%f [1]=%f [2]=%f [3]=%f",mtx[1][0],mtx[1][1],mtx[1][2],mtx[1][3]);
		s.mWriteTextLineToStream(buffer);

		s.mWriteOutTabs();
		sprintf(buffer,"mtx[2] = [0]=%f [1]=%f [2]=%f [3]=%f",mtx[2][0],mtx[2][1],mtx[2][2],mtx[2][3]);
		s.mWriteTextLineToStream(buffer);

		s.mWriteOutTabs();
		sprintf(buffer,"mtx[3] = [0]=%f [1]=%f [2]=%f [3]=%f",mtx[3][0],mtx[3][1],mtx[3][2],mtx[3][3]);
		s.mWriteTextLineToStream(buffer);
	}
	
	s.mDecrementTabNumber(); 
	_func_end;
}

void TMatrixStreamInText(char *text, tMatrix& num, tSFCStream &s)
{
	_func_start( TMatrixStreamInText );
	num.mIdentity();

	s.mReadInTabs();
	char buffer[256];
//	char format[256];
	s.mReadTextLineFromStream(buffer);
 
	s.mIncrementTabNumber(); 

	int32 val;
	int32 val3x4;

	s.mReadInTabs();
	s.mReadTextLineFromStream(buffer);
	sscanf(buffer,"identity =%ld 3x4=%ld",&val,&val3x4);
	if (val3x4 > 0)
	{
  		num.mSet3x4Flag(kTrue);
	}
	else
	{
 		num.mSet3x4Flag(kFalse);
	}

	if (val == 0)
	{
		tMtx4x4& mtx = num.mGetMtx4x4();
		s.mReadInTabs();
		s.mReadTextLineFromStream(buffer);
		sscanf(buffer,"mtx[0] = [0]=%f [1]=%f [2]=%f [3]=%f",&mtx[0][0],&mtx[0][1],&mtx[0][2],&mtx[0][3]);

		s.mReadInTabs();
		s.mReadTextLineFromStream(buffer);
		sscanf(buffer,"mtx[1] = [0]=%f [1]=%f [2]=%f [3]=%f",&mtx[1][0],&mtx[1][1],&mtx[1][2],&mtx[1][3]);

		s.mReadInTabs();
		s.mReadTextLineFromStream(buffer);
		sscanf(buffer,"mtx[2] = [0]=%f [1]=%f [2]=%f [3]=%f",&mtx[2][0],&mtx[2][1],&mtx[2][2],&mtx[2][3]);

		s.mReadInTabs();
		s.mReadTextLineFromStream(buffer);
		sscanf(buffer,"mtx[3] = [0]=%f [1]=%f [2]=%f [3]=%f",&mtx[3][0],&mtx[3][1],&mtx[3][2],&mtx[3][3]);
	}
	
	s.mDecrementTabNumber(); 

	
	


	_func_end;
}


void DoubleStreamOutText(char *text, double& num, tSFCStream &s)
{
	_func_start( DoubleStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %lf",text,num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void DoubleStreamInText(char *text, double& num, tSFCStream &s)
{
	_func_start( DoubleStreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = %lf");
	s.mReadTextLineFromStream(buffer);
	sscanf(buffer,format,&num);
	_func_end;
}


// you've got some kind of integer and you don't know what size it is
void IntegerStreamOutText(char *text, int32 size, void *num, tSFCStream &s)
{
	_func_start( IntegerStreamOutText );
	switch (size)
	{
	case 1:
		{
			int8 val = (*(int8 *)num);
			Int8StreamOutText(text,val,s);
		}
		break;
	case 2:
		{
			int16 val = (*(int16 *)num);
			Int16StreamOutText(text,val,s);
		}
		break;
	case 4:
		{
			int32 val = (*(int32 *)num);
			Int32StreamOutText(text,val,s);
		}
		break;
	default:
		{
			TGSAY2(("Unknown Size!"));
		}
 		break;
	}
	_func_end;
}

void IntegerStreamInText(char *text, int32 size, void *num, tSFCStream &s)
{
	_func_start( IntegerStreamInText );
	switch (size)
	{
	case 1:
		{
			int8 val;
			Int8StreamInText(text, val,s);
			*((int8 *)num) = val;
		}
		break;
	case 2:
		{
			int16 val;
			Int16StreamInText(text,val,s);
			*((int16 *)num) = val;
		}
		break;
	case 4:
		{
			int32 val;
			Int32StreamInText(text,val,s);
			*((int32 *)num) = val;
		}
		break;
	default:
		{
			TGSAY2(("Unknown Size!"));
		}
 		break;
	}
	_func_end;
}



void TextStreamOutText(char *text, char *num, tSFCStream &s)
{
	_func_start( TextStreamOutText );
	s.mWriteOutTabs();
	char buffer[256];
	sprintf(buffer,"%s = %s",text,num);
	s.mWriteTextLineToStream(buffer);
	_func_end;
}


void TextStreamInText(char *text, char *num, tSFCStream &s)
{
	_func_start( TextStreamInText );
	s.mReadInTabs();
	char buffer[256];
	char format[256];
	strcpy(format,text);
	strcat(format," = ");
	s.mReadTextLineFromStream(buffer);
	int32 len = strlen(format);
	strcpy(num,&buffer[len]);
	_func_end;
}

void TextStreamOutTitle(char *text, tSFCStream &s)
{
	_func_start( TextStreamOutTitle );
	s.mWriteOutTabs();
	s.mWriteTextLineToStream(text);
	_func_end;
}


void TextStreamInTitle(char *text, tSFCStream &s)
{
	_func_start( TextStreamInTitle );
	s.mReadInTabs();
	s.mReadTextLineFromStream(text);
	_func_end;
}


void TextStreamOutSeperator(tSFCStream &s)
{
	_func_start( TextStreamOutSeperator );
	char str[1];
	str[0] = '\0';
	s.mWriteTextLineToStream(str);
	_func_end;
}

void TextStreamInSeperator(tSFCStream &s)
{
	_func_start( TextStreamInSeperator );
	char str[32];
	s.mReadTextLineFromStream(str);
	_func_end;
}


void IncrementStreamTab(tSFCStream &s) 
{ 
	_func_start( IncrementStreamTab );
	s.mIncrementTabNumber(); 
	_func_end;
};

void DecrementStreamTab(tSFCStream &s) 
{ 
	_func_start( DecrementStreamTab );
	s.mDecrementTabNumber(); 
	_func_end;
};

void ResetStreamTab(tSFCStream &s) 
{ 
	_func_start( ResetStreamTab );
	s.mResetTabNumber(); 
	_func_end;
};




//--------------------------------------------------------------------------------
//
//

int32 tSFCStream::mReadStringFromStream(char *str)
{
	_func_start( tSFCStream::mReadStringFromStream );
	int8u *x = (int8u *)str;
	int32 i = 0;
	int32 amtRead = mReadFromStream(x, 1);
	while ((amtRead == 1) && (*x != '\0'))
	{
		x++;
		i++;
		amtRead = mReadFromStream(x, 1);
	}
	if ((amtRead == 0) && (x != NULL))
	{
		*x = '\0';
	}
	return i;
	_func_end;
}

//--------------------------------------------------------------------------------
//
//
void tSFCStream::mWriteStringToStream(char *str)
{
	_func_start( tSFCStream::mWriteStringToStream );
	int32u	len;
	if (str)
	{
		len = strlen(str)+1;
	}
	else
	{
		len = 0;
	}

	int8u *x = (int8u *)str;
	if (len > 0)
	{
		mWriteToStream(x,len);
	}
	else
	{
		*this << '\0';
	}
	_func_end;
}
//--------------------------------------------------------------------------------
//
//

int32 tSFCStream::mReadTextLineFromStream(char *str)
{
	_func_start( tSFCStream::mReadTextLineFromStream );
	int8u *x = (int8u *)str;
	int32 i = 0;
	int32 amtRead = mReadFromStream(x, 1);
	while ((amtRead == 1) && (*x != '\n'))
	{
		x++;
		i++;
		amtRead = mReadFromStream(x, 1);
	}
	if (x != NULL)
	{
		*x = '\0';
	}
	return i;
	_func_end;
}

//--------------------------------------------------------------------------------
//
//
void tSFCStream::mWriteTextLineToStream(char *str)
{
	_func_start( tSFCStream::mWriteTextLineToStream );
	int32u	len;
	if (str)
	{
		len = strlen(str);
	}
	else
	{
		len = 0;
	}

	int8u *x = (int8u *)str;
	if (len > 0)
	{
		mWriteToStream(x,len);
	}
	*this <<'\n';

	_func_end;
}


void ArchiveInt32(int32 &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveInt32 );
	if (mode & kArchiveStreamCompress)
	{
		Int32StreamCompress(num, s);
	}
	else
	{
		Int32StreamUncompress(num, s);
	}
	_func_end;
}

void ArchiveInt32u(int32u &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveInt32u );
	if (mode & kArchiveStreamCompress)
	{
		Int32uStreamCompress(num, s);
	}
	else
	{
		Int32uStreamUncompress(num, s);
	}
	_func_end;
}

void ArchiveInt16(int16 &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveInt16 );
	if (mode & kArchiveStreamCompress)
	{
		Int16StreamCompress(num, s);
	}
	else
	{
		Int16StreamUncompress(num, s);
	}
	_func_end;
}

void ArchiveInt16u(int16u &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveInt16u );
	if (mode & kArchiveStreamCompress)
	{
		Int16uStreamCompress(num, s);
	}
	else
	{
		Int16uStreamUncompress(num, s);
	}
	_func_end;
}

void ArchiveInt8(int8 &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveInt8 );
	if (mode & kArchiveStreamCompress)
	{
		Int8StreamCompress(num, s);
	}
	else
	{
		Int8StreamUncompress(num, s);
	}
	_func_end;
}

void ArchiveInt8u(int8u &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveInt8u );
	if (mode & kArchiveStreamCompress)
	{
		Int8uStreamCompress(num, s);
	}
	else
	{
		Int8uStreamUncompress(num, s);
	}
	_func_end;
}

void ArchiveITruth(iTruth &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveITruth );
	if (mode & kArchiveStreamCompress)
	{
		ITruthStreamCompress(num, s);
	}
	else
	{
		ITruthStreamUncompress(num, s);
	}
	_func_end;
}

void ArchiveFloat(float &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveFloat );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			FloatStreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			FloatStreamUncompress(num, s);
		}
	}
	_func_end;
}


void ArchiveTPoint3(tPoint3 &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveTPoint3 );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			TPoint3StreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			TPoint3StreamUncompress(num, s);
		}
	}
	_func_end;
}

void ArchiveTMatrix(tMatrix &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveTMatrix );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			TMatrixStreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			TMatrixStreamUncompress(num, s);
		}
	}
	_func_end;
}

void ArchiveTString(tString &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveTString );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			TStringStreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			TStringStreamUncompress(num, s);
		}
	}
	_func_end;
}

void ArchiveStdString( std::string& str, tSFCStream &s, int32u mode )
{
	_func_start( ArchiveStdString );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << str;
		}
		else
		{
			TStdStringStreamCompress( str, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> str;
		}
		else
		{
			TStdStringStreamUncompress( str, s);
		}
	}
	_func_end;
}

void ArchiveTPoint4(tPoint4 &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveTPoint4 );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			TPoint4StreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			TPoint4StreamUncompress(num, s);
		}
	}
	_func_end;
}

void ArchiveTPoint6(tPoint6 &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveTPoint6 );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			TPoint6StreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			TPoint6StreamUncompress(num, s);
		}
	}
	_func_end;
}

void ArchiveTPoint2(tPoint2 &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveTPoint2 );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			TPoint2StreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			TPoint2StreamUncompress(num, s);
		}
	}
	_func_end;
}

void ArchiveDouble(double &num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveDouble );
	if (mode & kArchiveStreamCompress)
	{
		if (mode & kNoCompression)
		{
			s << num;
		}
		else
		{
			DoubleStreamCompress(num, s);
		}
	}
	else
	{
		if (mode & kNoCompression)
		{
			s >> num;
		}
		else
		{
			DoubleStreamUncompress(num, s);
		}
	}
	_func_end;
}

void ArchiveInteger(int32 size, void *num, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveInteger );
	if (mode & kArchiveStreamCompress)
	{
		IntegerStreamCompress(size, num, s);
	}
	else
	{
		IntegerStreamUncompress(size, num, s);
	}
	_func_end;
}

void ArchiveString(char *text, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveString );
	if (mode & kArchiveStreamCompress)
	{
		TextStreamCompress(text, s);
	}
	else
	{
		TextStreamUncompress(text, s);
	}
	_func_end;
}




#ifdef _NO_ARCHIVE_TEXT_MODE_
	#undef ArchiveInt32TM
	#undef ArchiveInt32uTM 
	#undef ArchiveInt16TM
	#undef ArchiveInt16uTM
	#undef ArchiveInt8TM	
	#undef ArchiveInt8uTM
	#undef ArchiveITruthTM
	#undef ArchiveFloatTM
	#undef ArchiveMinMaxFloatTM
	#undef ArchiveTPoint3TM
	#undef ArchiveTMatrixTM
	#undef ArchiveTStringTM
	#undef ArchiveTPoint4TM
	#undef ArchiveTPoint6TM
	#undef ArchiveTPoint2TM
	#undef ArchiveMinMaxIntTM
	#undef ArchiveDoubleTM
	#undef ArchiveIntegerTM 
	#undef ArchiveStringTM
	#undef ArchiveTitleTM
	#undef ArchiveSeperatorTM	
#endif

void ArchiveInt32TM(int32&num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveInt32TM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			Int32StreamOutText(text, num, s);
		}
		else
		{
			Int32StreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveInt32(num, s, mode);
	}

	_func_end;
}

void ArchiveInt32uTM(int32u &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveInt32uTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			Int32uStreamOutText(text, num, s);
		}
		else
		{
			Int32uStreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveInt32u(num, s, mode);
	}
	_func_end;
}

void ArchiveInt16TM(int16 &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveInt16TM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			Int16StreamOutText(text, num, s);
		}
		else
		{
			Int16StreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveInt16(num, s, mode);
	}
	_func_end;
}

void ArchiveInt16uTM(int16u &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveInt16uTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			Int16uStreamOutText(text, num, s);
		}
		else
		{
			Int16uStreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveInt16u(num, s, mode);
	}
	_func_end;
}

void ArchiveInt8TM(int8 &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveInt8TM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			Int8StreamOutText(text, num, s);
		}
		else
		{
			Int8StreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveInt8(num, s, mode);
	}
	_func_end;
}

void ArchiveInt8uTM(int8u &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveInt8uTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			Int8uStreamOutText(text, num, s);
		}
		else
		{
			Int8uStreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveInt8u(num, s, mode);
	}
	_func_end;
}

void ArchiveITruthTM(iTruth &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveITruthTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			ITruthStreamOutText(text, num, s);
		}
		else
		{
			ITruthStreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveITruth(num, s, mode);
	}
	_func_end;
}

void ArchiveFloatTM(float &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveFloatTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			FloatStreamOutText(text, num, s);
		}
		else
		{
			FloatStreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveFloat(num, s, mode);
	}
	_func_end;
}


void ArchiveTPoint3TM(tPoint3 &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveTPoint3TM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TPoint3StreamOutText(text, num, s);
		}
		else
		{
			TPoint3StreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveTPoint3(num, s, mode);
	}
	_func_end;
}

void ArchiveTMatrixTM(tMatrix &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveTMatrixTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TMatrixStreamOutText(text, num, s);
		}
		else
		{
			TMatrixStreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveTMatrix(num, s, mode);
	}
	_func_end;
}


void ArchiveTStringTM(tString &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveTStringTM );
	if (mode &kArchiveTextMode)
	{
		tString textString(text);
		if (mode & kArchiveStreamCompress)
		{
			TStringStreamOutText(textString, num, s);
		}
		else
		{
			TStringStreamInText(textString, num, s);
		}
	}
	else
	{
		ArchiveTString(num, s, mode);
	}
	_func_end;
}

void ArchiveStdStringTM( std::string& str, tSFCStream &s, int32u mode, char *text )
{
	_func_start( ArchiveStdStringTM );
	if (mode &kArchiveTextMode)
	{
		std::string textString( text );
		if (mode & kArchiveStreamCompress)
		{
			TStdStringStreamOutText( textString, str, s);
		}
		else
		{
			TStdStringStreamInText( textString, str, s);
		}
	}
	else
	{
		ArchiveStdString(str, s, mode);
	}
	_func_end;
}

void ArchiveTPoint4TM(tPoint4 &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveTPoint4TM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TPoint4StreamOutText(text, num, s);
		}
		else
		{
			TPoint4StreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveTPoint4(num, s, mode);
	}
	_func_end;
}

void ArchiveTPoint6TM(tPoint6 &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveTPoint6TM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TPoint6StreamOutText(text, num, s);
		}
		else
		{
			TPoint6StreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveTPoint6(num, s, mode);
	}
	_func_end;
}

void ArchiveTPoint2TM(tPoint2 &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveTPoint2TM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TPoint2StreamOutText(text, num, s);
		}
		else
		{
			TPoint2StreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveTPoint2(num, s, mode);
	}
	_func_end;
}

void ArchiveDoubleTM(double &num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveDoubleTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			DoubleStreamOutText(text, num, s);
		}
		else
		{
			DoubleStreamInText(text, num, s);
		}
	}
	else
	{
		ArchiveDouble(num, s, mode);
	}
	_func_end;
}

void ArchiveIntegerTM(int32 size, void *num, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveIntegerTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			IntegerStreamOutText(text, size, num, s);
		}
		else
		{
			IntegerStreamInText(text, size, num, s);
		}
	}
	else
	{
		ArchiveInteger(size, num, s, mode);
	}
	_func_end;
}

void ArchiveStringTM(char *string, tSFCStream &s, int32u mode, char *text)
{
	_func_start( ArchiveStringTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TextStreamOutText(text, string, s);
		}
		else
		{
			TextStreamInText(text, string, s);
		}
	}
	else
	{
		ArchiveString(string, s, mode);
	}
	_func_end;
}


void ArchiveTitleTM(char *title, tSFCStream &s, int32u mode)
{
	_func_start( ArchiveTitleTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TextStreamOutTitle(title, s);
		}
		else
		{
			TextStreamInTitle(title, s);
		}
	}
	_func_end;
}

void ArchiveSeperatorTM(tSFCStream &s, int32u mode)
{
	_func_start( ArchiveSeperatorTM );
	if (mode &kArchiveTextMode)
	{
		if (mode & kArchiveStreamCompress)
		{
			TextStreamOutSeperator(s);
		}
		else
		{
			TextStreamInSeperator(s);
		}
	}
	_func_end;
}

//	<History>
//	14Jul98		oschlunk Created
//	28Jul98		oschlunk added floating point compression
//	3Sept98		oschlunk	added archiving stuff -- greatly expanded everything
//	18May99	dmandap		Removed references to tFlt2Int4 and tLineSegment data types (they no longer exist)
//




