//	<Name>	MissionMatchLocation.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>
//
#pragma once

#if ! defined ( MissionMatchLocation_H )
#define  MissionMatchLocation_H

// This is a helper class that stores a location expression for meta mission matching
// this replaces the cruder form of description like kEnemyBorder from before

#include "BaseTypes.h"
#include "BaseException.h"
//#include <vector>
#include <DatabaseObject.h>
//#include <MetaDatabase\Objects\MapObjects\MetaMapHex.h>

typedef int16u iLocationObjects;
enum eLocationObjects
{
	kTerrainShift				= 0,
	kAsteroidHex				= 1 << 0,
	kNebulaHex					= 1 << 1,
	kBlackHoleHex				= 1 << 2,
	kShippingLaneHex			= 1 << 3,
	kDustCloudsHex				= 1 << 4,
	kEmptySpaceHex				= 1 << 5,
	kAnyTerrainHex				= kAsteroidHex | kNebulaHex | kBlackHoleHex | kShippingLaneHex | kDustCloudsHex | kEmptySpaceHex,		

	kBaseShift					= 6,
	kBaseStationHex				= 1 << 6,
	kBattleStationHex			= 1 << 7,
	kStarbaseHex				= 1 << 8,
	kBaseWeaponsPlatformHex		= 1 << 9,
	kBaseListeningPostHex		= 1 << 10,
	kAnyBaseHex					= kBaseStationHex | kBattleStationHex | kStarbaseHex | kBaseWeaponsPlatformHex | kBaseListeningPostHex,

	kPlanetShift				= 11,
	kHomeworldHex				= 1 << 11,
	kCoreworldHex				= 1 << 12,
	kAsteroidBaseHex			= 1 << 13,
	kColonyHex					= 1 << 14,
	kAnyPlanetHex				= kHomeworldHex | kCoreworldHex | kAsteroidBaseHex | kColonyHex,

	// If you really need to put more then 16 items
	// in the enum, then you will have to change iLocationObjects to a int32u


};  // still have another 2 left

enum eLocationWhoHex
{
	kPlayerRaceHex		= 1 << 0,
	kEnemyRaceHex		= 1 << 1,
	kAlliedRaceHex		= 1 << 2,
	kFederationRaceHex	= 1 << 3,
	kKlingonRaceHex		= 1 << 4,
	kRomulanRaceHex		= 1 << 5,
	kGornRaceHex		= 1 << 6,
	kLyranRaceHex		= 1 << 7,
	kHydranRaceHex		= 1 << 8,
	kMirakRaceHex		= 1 << 9,
	kISCRaceHex			= 1 << 10,
	kNeutralRaceHex		= 1 << 11,
	kOrionOrionRaceHex		= 1 << 12,
	kOrionKorgathRaceHex		= 1 << 13,
	kOrionPrimeRaceHex			= 1 << 14,
	kOrionTigerHeartRaceHex		= 1 << 15,
	kOrionBeastRaidersRaceHex	= 1 << 16,
	kOrionSyndicateRaceHex		= 1 << 17,
	kOrionWyldeFireRaceHex		= 1 << 18,
	kOrionCamboroRaceHex		= 1 << 19,
	kAnyRaceHex			=	kPlayerRaceHex | kEnemyRaceHex | kAlliedRaceHex | 
							kFederationRaceHex| kKlingonRaceHex	| kRomulanRaceHex |
							kGornRaceHex | kLyranRaceHex | kHydranRaceHex |
							kMirakRaceHex | kISCRaceHex | kNeutralRaceHex,
	kAnyCartelRaceHex	=	kPlayerRaceHex | kOrionOrionRaceHex | kOrionKorgathRaceHex | kOrionPrimeRaceHex |
							kOrionTigerHeartRaceHex | kOrionBeastRaidersRaceHex | kOrionSyndicateRaceHex | 
							kOrionWyldeFireRaceHex | kOrionCamboroRaceHex,
}; // still have another 10 left

class tMetaLocationSpec : public nDataStore::tStreamable
{
public:
// types
	typedef std::vector< tMetaLocationSpec > iQueryResult;

// setup
	tMetaLocationSpec();
	tMetaLocationSpec( const tMetaLocationSpec& aMetaLocationSpec );
	tMetaLocationSpec& operator= ( const tMetaLocationSpec& aMetaLocationSpec );
	tMetaLocationSpec( int32 range, int32 who, int32 object );

//Operations
	virtual	void StreamOut( nDataStore::tOutput& output ) const;
	virtual	void StreamIn( nDataStore::tInput& input );

// fields
	int32 fRange;
	int32 fLocationWhoHexFlags;
	int32 fLocationObjectFlags;
};

// Thus you can create "Location Expressions" such as

// Nearest enemy homeworld with a range no greater than 20
// tMetaLocationSpec location( 20, kEnemyRace, kHomeworld )

// Nearest home base with a range no greater than 5
// tMetaLocationSpec location( 5, kPlayerRace, kAnyBase )

// Nearest Romulan, Gorn or ISC, asteroid hex with a planet, no greater than 0 (have to be right on it)
// tMetaLocationSpec location( 0, kRomulanRaceHex|kGornRaceHex|kISCRaceHex, kAsteroidHex|kAnyPlanet )



#endif