//	<Name>	NetMsgClass.h
//
//
//	<Info>	DESCRIPTION
//
#pragma once

#include <tbasic.h>

#if !defined( NetMsgClass_h )
#define NetMsgClass_h

//--------------------------------------------------------------------------------
//
//
// Computes offset of a data member field relative to start of structure
//
#ifndef QSI_FIELD_OFFSET
	#define QSI_FIELD_OFFSET(type, field)     ((long)&(((type *)0)->field))
#endif

// Computes size of data member field for a given structure
//
#ifndef QSI_FIELD_SIZE
	#define QSI_FIELD_SIZE(type,field)        (sizeof( ((type *)0)->field ) )
#endif

// Macro to initialize a tNetMsg::tMapItem
//
#define	MAKE_NMMI(type,field)	{	QSI_FIELD_SIZE(type,field), QSI_FIELD_OFFSET(type,field) }


#define kThereIsNoZeroNext  1
#define kWatchOutZeroIsNext 2
#define kSomeNonZeroNumber  254

//--------------------------------------------------------------------------------
// Base class that all message packets should be based from.
// Derived classes must provide an ::mGetMap() method
//
struct tNetMsg
{
public:
	enum eCommandID
	{
		kAddPlayer = 0,
		kAdjustSpeedAbs,
		kAdjustSpeedRel,
		kBreakDown,
		kCorrectPos,
		kCaptureShip,
		kCreateDirect,
		kCreatePPDFire,
		kCreateESG,
		kCreateESGLance,
		kCreatePhysicalEntity,
		kCreateSeeker,
		kCreateSeekerAtPoint,
		kCreateShipFromSpec,
		kCreateTractor,
		kDamage,
		kEnableShipModeChange,
		kTractorChangeMessage,
		kEndGame,
		kEscortCommands,
		kFireWeapons,
		kKillEntity,
		kHoldPosition,
		kLayMine,
		kMoveInDirection,
		kMoveTo,
//		kNewPlayer,
		kOrbit,
		kPlayerNumber,
		kRotateTractor,
		kSelectTarget,
		kSetEnityRange,
		kSetGameSpeed,
		kSetTurnAndFireSpeed,
		kSlam,
		kStop,
		kTransportBomb,
		kTransportMen,
//		kPing,
//		kPingAck,
		kTimeAdjust,
		kUpdateInternals,
		kDeepScan,
		kSelectMap,
//		kStartPosition,
		kSetPlayerReady,
		kAICalculation,
		kSetPlayerTeam,
		kSetPlayerTag,
		kSetPlayerColor,
		kSetPlayerFleetInfo,
		kPlayerInfo,
		kSessionChat,
		kReadyToStart,
		kRejectPlayer,
		kAIChangeTeamRelationshipToWar,
		kSetPlayerRace,
		kSetPlayerRaceCheck,
		kSetPlayerBPVLimit,
		kChangeHost,
		kWarpOut,
		kReadyPolling,
		// Mission specific functions
		kNotifyScriptEvent,
		kScriptEvent,
		kCreateMissionMap,
		kSetPlayerShipID,
		kShipTargetable,
		kShipCanTarget,
		kRemoveTargets,
		kTeamScriptDescription,
		kShipScriptDescription,
		kTeamReady,
#if 1	///DANHM -- this block was changed to reflect host migration
		kScriptBeginMission,
#endif
		kDisplayInGameMessage,
		kSetCommButton,	
		kResetDebriefing,
		kDebriefingMessage,
		kPlayerStatusChange,
		kSetTeamRelation,
		kScriptPrimitive,
		kTrackDistanceFromTarget,
		kSetFinalDestination,
		kSetGoalIndicator,
		kScriptStartMission,
		kMissionEnd,
		// End mission specific functions
		kSendMapInfo,
		kSetShipPlayerControlled,

		kDamageSystem,
		kHealSystem,
		kSetPercentageFromOrig,
		kSetPercentageFromCurr,
		kGetHealthOfSystem,
		kGetMaxHealthOfSystem,
		kGetPercentAtMarkOriginal_Trigger,
		kGetPercentAtMarkCurrent_Trigger,
		kClearPreviousSystemTriggers,

		kQUERRYREQUEST_HEALTH_VALUE,
		kQUERRYREQUEST_MAX_HEALTH_VALUE,
		kQUERRYREQUEST_HEALTH_PERCENTAGE,		

		kRemoveItemFromStores,
		kAddItemToStores,
		kSetItemInStores,

		kGetAmountOfStores,
		kGetMaxAmountOfStores,

		kGetHealthOfHardpoint,

		kGetAmountOfStoresAtMark,

		kClearPreviousStoresTriggers,

		kSendVoice,

		kQUERRYREQUEST_ITEM_AMOUNT,
		kQUERRYREQUEST_ITEM_MAX_AMOUNT,
		kScriptFinished,
		kUpdateCloakStatus,
		kCueRecording,

		kNotifyScreenshot,
	};

public:
	enum
	{
		kSizeInt8		= 1,
		kSizeInt16		= 2,
		kSizeInt32		= 4,
		kSizeInt64		= 8,		// __int64 or double
		kSizeTPoint3	= 12,		// tPoint3

		kSizeTString	= -1,		// tString
		kSizeString		= -2,		// single-byte string
		kSizeSFCStream	= -3		// a tSFCStream	
	};

	struct	tMapItem
	{
		int16	fSize;
		int16u	fOffset;
	};

	enum 
	{
		kNoMsgTime		= -1
	};

public:
	virtual	const tNetMsg::tMapItem *mGetMap(void) const	= 0;

	virtual	int32u	mStreamTo(int8u *pStreamBuffer);
	virtual	int32u	mStreamFrom(const int8u *pStreamBuffer);
	// These will allocate more mem so make sure you clean them up
	static int8u*	mMakeStringy(const int length,const int8u *pStreamBuffer);
	static int8u*	mMakeUnStringy(const int length,int8u *pStreamBuffer);

protected:
	int32u mStream(const void *pStreamBuffer, iTruth bOutput, tNetMsg *pMsg);
};

#endif


//	14Apr00	zdrummond	Added Comment Header and Footer's
//	21Jul00	dsuleski	Added new message, "kSetTeamRelation"
//	23Jul00	dsuleski	Added new message, "kSetGoalIndicator"