//	<Name>	strength.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>
//
#pragma once

#if ! defined ( Strength_H )
#define Strength_H

// tStrength this class is used both inside of tMetaSpec|tTeamInfo as a class to
// hold the definitions for a team.  And in the mission matcher server code.  See
// tMissionMatcherCtlrS.cpp where it is used inside of tMissionProfile's to store
// information about the team slots.  Specifically the ship strength of a given team

#include "BaseTypes.h"
#include "BaseException.h"
#include <DatabaseObject.h>
#include "ScriptsData.h"

//
//	This class describes the strength of an individual ship.
//
class tShipStrength : public nDataStore::tStreamable
{
public:

	typedef std::vector< tShipStrength > iVector;

	tShipStrength( void );
	tShipStrength( eClassTypes minClass, eClassTypes maxClass, int32 minBPV, int32 maxBPV );
	tShipStrength( eClassTypes minClass, eClassTypes maxClass, float minRatio, float maxRatio );
	tShipStrength( const tShipStrength& rhs );
	operator=( tShipStrength rhs );

public:

	virtual	void StreamOut( nDataStore::tOutput& output ) const;
	virtual	void StreamIn( nDataStore::tInput& input );

	eClassTypes fMinClass, fMaxClass;
	int32 fMinBPV;
	int32 fMaxBPV;
	float fMinRatio;
	float fMaxRatio;
	bool fRelativeToPrimaryTeam;
};


class tStrength : public nDataStore::tStreamable
{
public:
// Types
	typedef std::vector< tStrength > iQueryResult;

	enum
	{
		kInfinite = 99999,
	};

// Setup
	tStrength();
	~tStrength( void );
	tStrength( std::vector< tShipStrength > shipList );
	tStrength& operator= ( const tStrength& aStrength );

//Operations
	virtual	void StreamOut( nDataStore::tOutput& output ) const;
	virtual	void StreamIn( nDataStore::tInput& input );

// Fields	
	
	int32 fMinCount;
	int32 fMaxCount;

	//List of ships that encompass above traits
	tShipStrength::iVector fShipStrengthList;
};

#endif