//	<Name>	entitymgr.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	Entity Manager
//
#pragma once

#ifndef entitymgr_h
#define entitymgr_h

#include "tvectormath.h"

#include <FunctionWrapper.h>

typedef int32 tID;

class tEntity;
class tEntity_Weapon;
class tEntityShip;
class tPhysicalEntity;
class tBasicFile;
class tSFCStream;
class tEntity_Astral;
class tObfuscatedNumber;
class tEntityMgr
{
public:
	// ID constants
	enum {
		kMaxNumberofEntitiesInTheGame  = 16384,
		kMaxNumberofServerControlledEntities = 1024,
		kMaxNumberofPlayerControlledEntities = 2560,
		kMaxNumberofPlayers					 = 6,
		kMaxNumberofAstralEntities		=	16
	};

	enum
	{
		kShipEntity			= 0,
		kAstronomical,
		kSeekingWeapon ,
		kInstantEffects,
		kExplosions
	};

	tEntityMgr();
	~tEntityMgr();

	iTruth		mInit(void);
	void		mDone(void);

	// Cheat reporting code
	void	mReportModifiedShip(int32 ID);

	iTruth		mControlledByThisMachine(const tID kValue);
	int			mControlledByWhichPlayer(const tID kValue);
	iTruth		mControlledByHost(const tID kValue);
	tEntity		*mFindID(const tID kIDToFind) const {_func_start( tEntityMgr::mFindID ); if(kIDToFind>=0 && kIDToFind<kMaxNumberofEntitiesInTheGame) return(fIDLookup[kIDToFind]); else return NULL;_func_end;}

	// Following is similar to mFindID() except when the entity can not be located; if a tEntity_Placeholder exists for the requested ID then
	// that a pointer to the placeholder object will be returned instead
	//
	tEntity		*mGetEquivalentEntity(const tID kIDToFind);

	// this is not a ID lookup like mFindID,  this just gets the index into the array

	int				mGetMaxAstralEntityArraySize(void) {_func_start( tEntityMgr::mGetMaxAstralEntityArraySize );return kMaxNumberofAstralEntities;_func_end;};
	int				mGetNextAstralEntityArrayLocation(void) {_func_start( tEntityMgr::mGetNextAstralEntityArrayLocation );return fNextAstralIndexFree;_func_end;};
	tEntity_Astral	*mGetAstralEntity(const int kAstralToGet) const {_func_start( tEntityMgr::mGetAstralEntity ); if(kAstralToGet>=0 && kAstralToGet<kMaxNumberofAstralEntities) return(fAstralEntityArray[kAstralToGet]); else return NULL;_func_end;}
	void			mAddAstralEntity(tEntity_Astral *pNew,int index);

	void			mAddEntity(tEntity *pNew);

	void		mKillID(const tID kIDToKill);
	void		mKillAll(void);

	tEntity		*mGetEntityList(void);
	tEntity		*mGetLastEntity(void);
	// Player Number lets the Entman know what range to give player
	//
	void		mSetPlayerNumber(const int player);
	int32		mGetPlayerNumber(void){_func_start( tEntityMgr::mGetPlayerNumber ); return fPlayer;_func_end;};
	tID			mGetNextPlayerID(void);
	tID			mGetNextHostID(void);
	tEntity		*mDetectAnyCollisionWithPoint(tPoint3 point, float radius);
	void		mMakeContectWithAnyPhysicalObject(int32 ContactID);
	iTruth		mDetectCollision(tID EntID1,tID EntID2,iTruth ForceCollisionCheck = kFalse);
	iTruth		mDetectCollision(tEntity*,tEntity*,iTruth ForceCollisionCheck = kFalse) ;
	iTruth		mDetectImmediateCollision(tID EntID1,tID EntID2);
	iTruth		mDetectImmediateCollision(tEntity*,tEntity*);

	// Added this for "proximity" weapon behavior; ProximityRadius is the radius for collision. Do _NOT_ multiply
	// ProximityRadius by kDistancePerHex; the function will do that itself; e.g., for range 1 say 1.0 not 10.0
	iTruth		mDetectProximityCollision(tEntity*,tEntity*,float ProximityRadius,iTruth ForceCollisionCheck = kFalse) ;

	//
	//	Used for offsets of entities when first spawning.
	//
	enum eOffsetDirections
	{
		kXPlus,
		kXPlusYMinus,
		kYMinus,
		kXMinusYMinus,
		kXMinus,
		kXMinusYPlus,
		kYPlus,
		kXPlusYMPlus,

		kNumOffsetDirections,
	};
	enum eMaxOffsets
	{
		kX = 75,
		kY = 75,
		kZ = 0,
	};
	tPoint3		mSpawnOffsetPosition( tPoint3 initPos, float collRadius );

	float		mGetDistanceFromEntities(const tID EntID1,const tID EntID2);
	float		mGetDistanceFromEntities(const tEntity*,const tEntity*);
	float		mGetDistanceFromEntityToPoint(const tEntity*,const tPoint3& location);
	float		mGetDistanceFromEntityToPoint(const tID EntID,const tPoint3& location);

	int32u		mFixUpSeed(int32u seed);
	void		mDecodeSeed(int32u seed,int32 Who);
	// From the Second Entities point of view.
	//
	tAngle		mDirectionOfLastCollision(void){_func_start( tEntityMgr::mDirectionOfLastCollision );return fLastCollisionDir;_func_end;};
	tAngle		mDirectionToOtherEntity(tEntity* From,tEntity* Target);
	tAngle		mDirectionToOtherEntity(int32 IDFrom,int32  IDTarget);
	tAngle		mDirectionToPoint(tEntity* From,tPoint3 location);
	// Command Selection and Targeting functions
	//
	iTruth		mMarkIDasControledByPlayer(const tID kID);
	tID			mGetPlayersControledEntityID(void);
	tEntity		*mGetPlayersControledEntity(void);

	tEntity_Weapon	*mGetWeaponEntity(const tID kID) const;
	tEntityShip	*mGetShipEntity(const tID kID) const;
	tPhysicalEntity *mGetPhysicalEntity(const tID kID) const;

	tEntity_Weapon	*mCastToWeapon(tEntity *pEntity);
	const tEntity_Weapon	*mCastToWeapon(const tEntity *pEntity) const;
	tEntityShip	*mCastToShip(tEntity *pEntity);
	const tEntityShip	*mCastToShip(const tEntity *pEntity) const;
	tPhysicalEntity	*mCastToPhysical(tEntity *pEntity);
	const tPhysicalEntity	*mCastToPhysical(const tEntity *pEntity) const;
	void	mConsiderThisMachineHost(void) {_func_start( tEntityMgr::mConsiderThisMachineHost ); fHost = kTrue;_func_end;};
	void	mConsiderThisMachineClient(void){_func_start( tEntityMgr::mConsiderThisMachineClient );fHost = kFalse;_func_end;};


	virtual void mArchive(tSFCStream &s, int32u mode);

	int32		mGetBaseID(void);
protected:
	
	int32		fPlayerIDBase;
	int32		fPlayer;
	tEntity		*fEntities;				// linked list of entities
	tEntity		*fTail;
	iTruth		fHost;
	int fNextAstralIndexFree;
	tID			fNextPlayerID;
	tID			fNextHostID;
	tID			fPlayersControledShip;
	tObfuscatedNumber*	fEntityStrength;
	tAngle			fLastCollisionDir;
	tEntity			*fIDLookup[kMaxNumberofEntitiesInTheGame];
	tEntity_Astral	*fAstralEntityArray[kMaxNumberofAstralEntities];
	int8u			fFreshnessID[kMaxNumberofEntitiesInTheGame];
};

extern	tEntityMgr	gEntityMgr;

#endif	// #ifndef entitymgr_h


// 2/27/98	<mhertogh>	Added Comment Header and Footer's
//	6May98		mhertogh	Separated many Ship related functions from base Entity
// 28Jul98	dmandap		Made some methods 'const', added mCastToShip() methods
// 10Aug98	mhertogh	Added mCast to Physical and Get Physical Entity!
//	12Aug98	mhertogh	Increased the number of Asteral Entities.
//	3Sept98	mhertogh	Altered mDetectCollision to support a kCollide property as well as a force check.
// 08Sep98	mhertogh	Changed mCreatedByThisMachine to mControlledByThisMachine
// 07Dec98	mhertogh	Added Astral entity list and all the support that goes with it.
// 23Dec98	mhertogh	Added entity to point distance functions
//	30Dec98	mhertogh	Added mDirectionToPoint.
//	02Mar99	dmandap		Fixed "one off" bug in mFindID(). Added new method mGetEquivalentEntity()
//	16Mar99	mhertogh	Added mControlledByWhichPlayer.
//	19May99	rliu		Added function mControlledByHost()
//	29Jun00	dsuleski	Added ability to detect collision without incremental algorithm (immediate)
