//	<Name>	missionspec.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
#pragma once

#ifndef missionspec_h
#define missionspec_h

#include <BaseTypes.h>
#include <DynamicAllocator.h>

#include "scriptsdata.h"
#include "TeamData.h"

#include "Strength.h"
#include "MissionMatchLocation.h"

#include "ScriptMapData.h"

//  Bit masks for number of ships
typedef int32 iMissSpecShips;
enum
{
	kMissSpecNoShip		= 0,
	kMissSpecOneShip	= 1, 
	kMissSpecTwoShip	= 2, 
	kMissSpecThreeShip	= 3, 
	kMissSpecAllShips	= kMissSpecThreeShip
};

// Bit masks for race (assumed to be in same order as eRaceName
typedef int32 iMissSpecRace;
enum 
{ 
	kSpecNoRace			= 0,
	kSpecFed			= 1 <<	kFederation,
	kSpecKling			= 1 <<	kKlingon,
	kSpecRomulan		= 1 <<	kRomulan,
	kSpecLyran			= 1 <<	kLyran,
	kSpecHydran			= 1 <<	kHydran,
	kSpecGorn			= 1 <<	kGorn,
	kSpecTholian		= 1 <<	kTholian,
	kSpecISC			= 1 <<	kISC,
	kSpecMirak			= 1 <<	kMirak,
	kSpecLDR			= 1 <<	kLDR,
	kSpecWYN			= 1 <<	kWYN,
	kSpecJindarian		= 1 <<	kJindarian,
	kSpecOrion			= 1 <<	kOrion,

	// Orion Races
	kSpecOrionOrion			= 1 << kOrionOrion,
	kSpecOrionKorgath		= 1 << kOrionKorgath,
	kSpecOrionPrime			= 1 << kOrionPrime,
	kSpecOrionTigerHeart	= 1 << kOrionTigerHeart,
	kSpecOrionBeastRaiders	= 1 << kOrionBeastRaiders,
	kSpecOrionSyndicate		= 1 << kOrionSyndicate,
	kSpecOrionWyldeFire		= 1 << kOrionWyldeFire,
	kSpecOrionCamboro		= 1 << kOrionCamboro,

	kSpecMonster		= 1 <<	kMonster,
	
	kSpecAnyRace		= ( kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecHydran | kSpecGorn | kSpecTholian | kSpecISC | kSpecMirak | kSpecLDR | kSpecWYN | kSpecJindarian | kSpecOrion | kSpecOrionOrion | kSpecOrionKorgath | kSpecOrionPrime | kSpecOrionTigerHeart | kSpecOrionBeastRaiders | kSpecOrionSyndicate | kSpecOrionWyldeFire | kSpecOrionCamboro),
	kSpecNonPirateRace	= kSpecAnyRace & ( ~kSpecOrion | ~kSpecOrionOrion | ~kSpecOrionKorgath | ~kSpecOrionPrime | ~kSpecOrionTigerHeart | ~kSpecOrionBeastRaiders | ~kSpecOrionSyndicate | ~kSpecOrionWyldeFire | ~kSpecOrionCamboro ),
	kSpecPlayableRace	= ( kSpecFed | kSpecKling | kSpecRomulan | kSpecLyran | kSpecGorn | kSpecHydran | kSpecISC | kSpecMirak | kSpecOrionOrion | kSpecOrionKorgath | kSpecOrionPrime | kSpecOrionTigerHeart | kSpecOrionBeastRaiders | kSpecOrionSyndicate | kSpecOrionWyldeFire | kSpecOrionCamboro),
};

// Bit masks for the how the race will be chossen
// iRaceCriteria is broken into two, the higher 16 bits are used to state a relationship. The lower 16 bits are used to 
// designate what the relation ship refers to. So you can same the team should be the same race as team 2, or the enemy of the player

typedef int32 iRaceCriteria;
enum 
{
	//Team number -- as specified in the script...for example, kFederation is not valid!
	kTeamMask		= 0x0000FFFF,
	
	
	//Relationships
	kRelationshipMask = 0xFFFF0000,

	kSameAs			= 1 << 16,
	kAnyEnemyOf		= 2 << 16,
	kWorstEnemyOf	= 3 << 16,
	kAnyAllyOf		= 4 << 16,
	kBestAllyOf		= 5 << 16,
	kAnyRace		= 6 << 16, //Doesn't need an object
	kDifferentThan	= 7 << 16,
};

enum eMissionTypes
{ 
	kCampaignMission,
	kUniqueCampaignMission, 
	kTutorialMission, 
	kTechnicalMission, 
	kHistoricalMission, 
	kMultiplayerMission,
	kSkirmishMission,
	kSkirmishVOMission,
	kDebugMission,
	kBonusMission,
};

enum eTeamType
{
	kNPCTeam,				// Only run by the AI
	
	//The following teams are intended for human play, although there is a chance 
	//that an AI can fill the slot if the scripter wants it
	kPlayableTeam, 
	kPrimaryTeam,			// Used by Meta to decide how it will choose sides
	kPrimaryOpponentTeam,	// Used by Meta to decide how it will choose sides
	kDeadTeam,				// Used to indicate a team that has left the universe and must be removed
};

enum eBandwidthPart
{
	kNumClientsMask = 0x00FF,
	kForgotClientInitialization = 0x0000,	//	this constant is used if the scripter forgot to set the clients
	k0Client = 0x0001,
	k1Client = 0x0002,
	k2Client = 0x0004,
	k3Client = 0x0008,
	k4Client = 0x0010,
	k5Client = 0x0020,

	kHostBandwidthMask = 0xFF00,
	kForgotHostInitialization = 0x0000,	//	this constant is used if the scripter forgot to set the host bandwidth
	k1Band = 0x0100,
	k2Band = 0x0200,
	k3Band = 0x0400,
	k4Band = 0x0800,
	k5Band = 0x1000,
	k6Band = 0x2000,
	
	kThirdBand = k2Band,
	kHalfBand = k3Band,
	kAllBand = k6Band,
};

enum eGenericFlags
{
	kNoGenericFlags			=	0,

	kStopMetaMovement		=	1 << 0,
	kPlaceBaseMission		=	1 << 1,
	kResupplyMission		=	1 << 2,
	kShippingLaneMission	=	1 << 3,
	kDisableCapture			=	1 << 4,
	kRequiresBase			=	1 << 5,
	kRequiresPlanet			=	1 << 6,

	kNextGenericFlag		=	1 << 7,
	kMaxGenericFlags		=	1 << 31,
};

class tTeamSpec
{
public:
	tTeamSpec(	const std::string&	teamName,								// Team name identifier
				int32				teamID,									// One of the eTeamID values to identify this team
				float				chanceTeamIsAvailable,					// Chance this team will be a part of the mission
				bool				required,								// If this team is available, this designates that the team MUST be played by someone
				eTeamType			teamType,								// Whether or not this team can be played by a human.
				float				chanceTeamCanBePlayedByAI,				// If the is a playable team, the chance if AI will fill the slot, if no human is avaliable
				tStrength			strengthLimits,							// The strength restrictions on the team
				iMissSpecRace		validRaces,								// What races can be play this team,
				iRaceCriteria		raceCriteria,							// How the race is chosen
				iAllocatorID		teamAllocTypeID,						// The id used to alloc the object that will run the team
				eTeamTag			teamTag = kNoTeamTag,					// The letter associated with this team for multiplayer
				const std::string&	teamMissionTitle = "Mystery Mission",	// The title of the mission as you want this particular team to see as the mission title
				std::string			briefings[kNumActiveRaces] = NULL,		// The briefing of the mission as you want this particular team to see as the mission briefing
				bool				allowJoinInProgress = false				// true if you want people to be able to join this mission in this team slot after mission starts
			 );  
				
	tTeamSpec();
	virtual ~tTeamSpec();

	std::string			fTeamName;
	int32				fTeamID;
	float				fChanceTeamIsAvailable; //Anything greater then 1.0 will always be available
	eTeamType			fTeamType;
	float				fChanceTeamCanBePlayedByAI;
	bool				fRequired; //This team MUST be played, IF it is available to be played
	bool				fAllowJoinInProgress;	
	tStrength			fStrengthLimits;
	iRaceCriteria		fRaceCriteria; //Race Criteria has presidence over fValidRaces. If you spec the same race as team1, that is s what you get
	iMissSpecRace		fValidRaces;
	std::string			fTeamMissionTitle;
	std::string			fBriefings[kNumActiveRaces];
	iAllocatorID		fTeamAllocTypeID;
	eTeamTag			fTeamTag;
};


typedef std::vector< int32 > iTeamIDs;

class tMissionSpec
{
public:
// setup
	tMissionSpec();
	tMissionSpec( const tMissionSpec& spec );
	tMissionSpec& operator=( const tMissionSpec& spec );
	~tMissionSpec();

// operations
	void mSetupTeam( const tTeamSpec& teamSpec );
	int32 mTeamCount();
	int32 mPlayableCount();
	tTeamSpec* mGetTeamSpec( int32 index );

// fields
	float				fVersion;
	std::string			fFileName;
	std::string			fMissionTitle;
	std::string			fMissionDescription;
	std::string			fHelpTags;
	eMissionTypes		fMissionType;	// Set with one of the eMissionType values
	iMissSpecRace		fPlayerRace;	// Logical OR the MissSpecRace values
	tMetaLocationSpec	fMetaLocationSpec;  // an expression of what location requirements this script has see MetaLocationSpec.h
	iMissSpecRace		fEnemyRace;		// Logical OR the MissSpecRace values
	iMissSpecShips		fNumShips;		// Logical OR the number of ship values
	int32				fMinBPV;
	int32				fMaxBPV;
	int32				fMinDate;
	int32				fMaxDate;
	int32				fWait;			// also indicates menu position for Tutorial and Technical missions
	int32				fTutorialOrder;	//	0 or negative indicates error, least number indicates 1st tutorial, greatest number indicates last tutorial
	eMenuButtonStates	fMenuStates;
	eBandwidthPart		fBandwidthPart;	//	host is kPrimaryTeam in Meta and is first human slot in Multi
	tScriptMapList		fScriptMapData;
	tMapOverrides		fMapOverrides;
	tMapTerrainList		fMapTerrainList;
	int32				fGenericFlags;	//	see eGenericFlags for constants

protected:
	void copyData( const tMissionSpec& spec );
	tTeamSpec* fTeam[kMaxTeams];
	int32 fTeamCount;
	int32 fPlayableTeamCount;
};

#endif	// #ifndef missionspec_h

//	History
//	12Oct98	sroney	Started
//	14Oct98	dboulanger	added new enums and #ifndef
//	03Nov98	dboulanger	added mission title and supply dock params
//	04Nov98	dboulanger	added enable supply dock param
//	10Nov98	dboulanger	added mission type for academy style missions
//	08Dec98	dboulanger	changed enum type to eMissionTypes
//	06Jan99	dboulanger	added kSpecialMission to eMissionTypes
//	15Feb99	dboulanger	added kEliteRaceMission to eMissSpecRace. removed kSpecialMission
//	06May99	dboulanger	changed fEnableSupplyDock to use eMenuButtons values
//	19Apr00 dsuleski	changed the constructor for tTeamSpec to incorporate new meta ship creation flag
//	12Jul00	dsuleski	fBandwidthPart added
