//	<Name>	scriptstructs.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	common classes and structs used to pass data back and forth between 
//          the DLL and the game.
//
#pragma once

#ifndef scriptstructs_h
#define scriptstructs_h

#include "tstring.h"
#include "race.h"
#include "raceenums.h"
#include "eshiptypes.h"
#include "esectorname.h"
#include "missionspec.h"
#include "entitymgr.h"
#include "team.h"
#include "scriptsdata.h"
#include "NetMsgClass.h"
#include "tmissionmap.h"

typedef struct tScript3DPoint
{
public:
	tScript3DPoint(	float x = 0.0f, float y = 0.0f, float z = 0.0f,
					float r = 0.0f, float g = 0.0f, float b = 0.0f );
	void operator=( tScript3DPoint rhs );

public:
	float	fX;
	float	fY;
	float	fZ;

	float	fR;
	float	fG;
	float	fB;

} tScript3DPoint;

typedef struct tScript3DPrimitive
{
public:

	enum{ kMaxVertices = 1 << 7 };
	tScript3DPrimitive(	int32 vertices = 0, int16 anchorX = 0, int16 anchorY = 0,
						eScriptPrimitiveFlags flags = kDefaultScriptPrimitiveFlags );
	void operator=( tScript3DPrimitive rhs );	

public:

	tScript3DPoint			fVertexList[kMaxVertices];

	int32					fNumVertices;
	tMapPoint				fAnchor;	
	eScriptPrimitiveFlags	fFlags;

} tScript3DPrimitive;

struct tShipSystemsInfo : public tNetMsg
{
public:
	iNotifyEvents	fAction;
	int32			fShipEntityID;
	eTeamID			fTeam;

	int32			fGridX;
	int32			fGridY;

	int32			fInt32[5];
	float			fFloat[5];
	tString			fText;
	const char*		fTextPtr;

public:
	virtual	const tNetMsg::tMapItem *mGetMap(void) const
	{
		_func_start( mGetMap );
		static	tNetMsg::tMapItem map[] =
		{
			MAKE_NMMI(tShipSystemsInfo,fAction),
			MAKE_NMMI(tShipSystemsInfo,fShipEntityID),
			MAKE_NMMI(tShipSystemsInfo,fTeam),
			MAKE_NMMI(tShipSystemsInfo,fGridX),
			MAKE_NMMI(tShipSystemsInfo,fGridY),

			MAKE_NMMI(tShipSystemsInfo,fInt32[0]),
			MAKE_NMMI(tShipSystemsInfo,fInt32[1]),
			MAKE_NMMI(tShipSystemsInfo,fInt32[2]),
			MAKE_NMMI(tShipSystemsInfo,fInt32[3]),
			MAKE_NMMI(tShipSystemsInfo,fInt32[4]),

			MAKE_NMMI(tShipSystemsInfo,fFloat[0]),
			MAKE_NMMI(tShipSystemsInfo,fFloat[1]),
			MAKE_NMMI(tShipSystemsInfo,fFloat[2]),
			MAKE_NMMI(tShipSystemsInfo,fFloat[3]),
			MAKE_NMMI(tShipSystemsInfo,fFloat[4]),

			{ kSizeTString, QSI_FIELD_OFFSET(tShipSystemsInfo,fText)},
			{ kSizeString, QSI_FIELD_OFFSET(tShipSystemsInfo,fTextPtr) },

			{	0, 0	}	// Terminator
		};
		return(map);
		_func_end;
	}
};

struct tShipInGame
{
public:
	tShipInGame();

	void mSetShipPosition( iMapPosition shipPos );

	iMapPosition mGetShipPosition() const;

	const char*		fClassName;
	const char*		fCustomName;
	eClassTypes		fClassType;
	int32			fShipBPV;
	int32			fStartX;
	int32			fStartY;
	int32			fEndX;
	int32			fEndY;
	eTeamID			fTeam;
	int32			fScriptCallbackID;
	int32			fShadowShipID;
	iTruth			fCreateShadowShip;
	iTruth			fDisplayOnMap;
	int32			fCustomID;
	iAllocatorID	fShipInfoClassID;
	iTruth			fHumanControlled;
	eShipOptions	fShipOptions;
	int32			fMetaDatabaseID;

protected:
	iMapPosition	fShipPosition;	
};
typedef std::vector<tShipInGame> tShipList;

struct tCreateShipInfo
{
public:
	tShipInGame fShipData;

	eClassTypes		fClassTypeLast;		// specifies last ship type for fleet creation
	eRaceName		fShipRace;
	int32			fVariant;
	int32			fTotalBPV;
	eClassTypes		fSelectedClassType;
	char*			fCustomName;
};

enum eMultiplayerMembers
{
	kPlayerTypeNoOne		=	0,
	kPlayerTypeAI			=	1,
	kPlayerTypeHuman		=	2,
	kPlayerTypeWatch		=	3
};

#endif	// #ifndef scriptstructs_h

// 17Sep98	dboulanger	created
// 24Sep98	dboulanger	Expanded structs to pass more info back and forth.
// 02Oct98  dboulanger	expanded structs for ship events and Dynaverse data
// 02Oct98	sroney		Change some strings to enums, make allies,enemies arrays
// 05Oct98  dboulanger	added target and fleet size to script structs
// 07Oct98  dboulanger	moved script version and map struct into dynaverse struct
// 07Oct98	dboulanger	added player died tag to event enum
// 08Oct98	dboulanger	changed fSelectedClassType data type
// 14Oct98	dboulanger	added MissionSpecs include
// 15Oct98	dboulanger	added ship position to player fleet struct
// 16Oct98	dboulanger	added new enum for end game status reporting
// 20Oct98	dboulanger	expanded Dynaverse class members - added systems enum
//						added default size to arrays
// 23Oct98	dboulanger	added struct members for timer interval and distance from center
// 26Oct98	dboulanger	added star base class
// 02Nov98	dboulanger	changed order of game status enums
// 04Nov98	dboulanger	added mission title to Dynaverse struct
// 06Nov98	dboulanger	added supply dock param
// 09Nov98  dboulanger	added extra params to tShipInGame struct
// 11Nov98  dboulanger	moved team type enum from script header file
//						added extra params to ShipInGame struct
// 11Nov98	dboulanger	revised certain structs
// 19Nov98	dboulanger	changed version num
// 20Nov98	dboulanger	added fTechnicalSuccess to Dynaverse struct
// 24Nov98	dboulanger	added functions for new script control
// 02Dec98	dboulanger	added kPlanetTeam and AITargetInfo struct
// 04Dec98	dboulanger	added new team relationships for scripts
// 08Dec98	dboulanger	added extra map info to tDynaverseScriptInfo struct
// 15Dec98	dboulanger	added index to thePlayers ship arrays to tShipInGame struct
// 04Jan99	dboulanger	added extra controls to work with Comm menu buttons
// 06Jan99	dboulanger	added enum for career events
// 08Jan99	dboulanger	added kNotifyDamage
// 10Jan99	ctaylor		added basestation and battlestation class types
// 20Jan99	ebethke		added support for the monster race; updated script version #
// 01Feb99	dboulanger	added new enum event: kAddToCareerTotal
// 04Feb99	dboulanger	moved in eVictoryLevels enum
// 12Feb99  dboulanger	added enums for ship damage and player changing ships
// 12Feb99  dboulanger	added deep scan notify to script events enum
// 18Feb99  dboulanger	added kGetTacticalDistance to notify events
// 25Feb99  dboulanger	split iNotifyEvents enum into two enums - one for game events and one for script callback
// 03Mar99  dboulanger	modified various enums and structs to work with new script features
// 09Mar99  dboulanger	added script transport items enum
// 10Mar99  dboulanger	added shields and internals to damage enum
// 10Mar99  dboulanger	added enums to access ship supplies
// 11Mar99	dboulanger	increased limit of timer callbacks to 10
//	22Mar99	zdrummond	New callback event
// 23Mar99	boulanger	changed to a new script interface standard.
//	02Apr99	dboulanger	modified code to work with multiplayer scripts
//	09Apr99	dboulanger	added params for multi/skirmish play
//	05May99	dboulanger	added enum to enable menu buttons and script function to read INI file
//	06May99	dboulanger	added campaign completed to eGameStatus enum
//	01Jun99	dboulanger	added new interface params for damage and scritp control
//	01Jun99	dboulanger	added extra script values array for script interface
//	04Jun99	dboulanger	added damage by probe and mine to eDamageToShip enum
//	17Jun99	dboulanger	added kNotifyPlayerLeaving for multiplayer scripts and kMakeShipTargetable
//	07Jul99	dboulanger	added fighter enums
//	5Oct99	zdrummond	New Trasnporter items
//	27Jun00	dsuleski	tCreateShipInfo now contains data for the ship custom name
//	25Jul00	dsuleski	tExtraData struct added for kDrawPolygon among others


