//	<Name>	SFCStreams.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	  This should contain the data structures for streaming out stuff
//
//
#pragma once

#ifndef	sfcstreams_h
#define	sfcstreams_h	1

#include "tclient.h"

#include <FunctionWrapper.h>

struct tPoint3;
class tMatrix;
class tString;
struct tFlt2Int4;
struct tPoint2;
struct tLineSegment;
struct tPoint4;
struct tPoint6;


// use this #define to turn off a lot of excess baggage these calls carry that won't
//	be needed in the final release of this code
//#define _NO_ARCHIVE_TEXT_MODE_

class tSFCStream
{
public:
	tSFCStream(int8u *buffer = NULL, int32 x = 0);
	tSFCStream(int32 x);
	~tSFCStream();

	// these need to become endian aware...
	void mWriteToStream(int8u *x, int32 size, iTruth ordered = kFalse);
	// this returns the number of bytes read
	int32 mReadFromStream(int8u *x, int32 size, iTruth ordered = kFalse);

	int32 mGetSize()	{_func_start( tSFCStream::mGetSize );	return (int32)(fMaxPtr-fBuffer);	_func_end;};
	int8u *mGetReadPtr() const	{_func_start( tSFCStream::mGetReadPtr );	return fGetPtr;	_func_end;};
	int8u *mGetBasePtr() const 	{_func_start( tSFCStream::mGetBasePtr );	return fBuffer;	_func_end;};
	int8u *mGetWritePtr() const	{_func_start( tSFCStream::mGetWritePtr );	return fPutPtr;	_func_end;};
	void mSetReadPtr(int8u *ptr);
	void mSetWritePtr(int8u *ptr);
 	int32 mGetReadSize() const {_func_start( tSFCStream::mGetReadSize );	return (int32)(fPutPtr-fGetPtr);	_func_end;};
 	void mSetReadSize(int32 readSize);
	void mReset();
	void mSetVersionNumber( int32 versionNumber );
	int mGetVersionNumber();

	// These are safety features that can catch a person writing to a stream when they
	//	ought only to be reading from it and vice-versa
	void mSetReadLock() {_func_start( tSFCStream::mSetReadLock ); fReadLock = kTrue; fWriteLock = kFalse; _func_end;};
	void mSetWriteLock() {_func_start( tSFCStream::mSetWriteLock ); fWriteLock = kTrue; fReadLock = kFalse; _func_end;};
	void mClearLocks() {_func_start( tSFCStream::mClearLocks ); fWriteLock = kFalse; fReadLock = kFalse; _func_end;};


	inline tSFCStream& operator<<(char *x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)x, 1); return (*this); _func_end;};
	inline tSFCStream& operator<<(unsigned char *x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)x, 1); return (*this);_func_end;};
	inline tSFCStream& operator<<(signed char *x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)x, 1); return (*this);_func_end;};
	inline tSFCStream& operator<<(char x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), 1); return (*this);_func_end;};
	inline tSFCStream& operator<<(unsigned char x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), 1); return (*this);_func_end;};
	inline tSFCStream& operator<<(signed char x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), 1); return (*this);_func_end;};
	inline tSFCStream& operator<<(short x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(short), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(unsigned short& x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(unsigned short), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(int x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(int), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(unsigned int x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(unsigned int), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(long x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(long), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(unsigned long x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(unsigned long), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(float x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(float), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(double x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(double), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator<<(long double x) {_func_start( tSFCStream::operator << ); mWriteToStream((int8u *)(&x), sizeof(long double), kTrue); return (*this);_func_end;};
	tSFCStream& operator<<(tPoint3& num);
	tSFCStream& operator<<(tMatrix& matrix);
	tSFCStream& operator<<(tString& theString);
	tSFCStream& operator<<(tPoint4& num);
	tSFCStream& operator<<(tPoint6& num);
	tSFCStream& operator<<(tFlt2Int4& num);
	tSFCStream& operator<<(tPoint2& num);
	tSFCStream& operator<<(tLineSegment& num);
	tSFCStream& operator<<(std::string& value);



	inline tSFCStream& operator>>(char *x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)x, 1); return (*this); _func_end;};
	inline tSFCStream& operator>>(unsigned char *x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)x, 1); return (*this);_func_end;};
	inline tSFCStream& operator>>(signed char *x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)x, 1); return (*this);_func_end;};
	inline tSFCStream& operator>>(char& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), 1); return (*this);_func_end;};
	inline tSFCStream& operator>>(unsigned char& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), 1); return (*this);_func_end;};
	inline tSFCStream& operator>>(signed char& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), 1); return (*this);_func_end;};
	inline tSFCStream& operator>>(short& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(short), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(unsigned short& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(unsigned short), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(int& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(int), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(unsigned int& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(unsigned int), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(long& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(long), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(unsigned long& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(unsigned long), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(float& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(float), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(double& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(double), kTrue); return (*this);_func_end;};
	inline tSFCStream& operator>>(long double& x) {_func_start( tSFCStream::operator >> ); mReadFromStream((int8u *)(&x), sizeof(long double), kTrue); return (*this);_func_end;};
	tSFCStream& operator>>(tPoint3& num);
	tSFCStream& operator>>(tMatrix& matrix);
	tSFCStream& operator>>(tString& theString);
	tSFCStream& operator>>(tPoint4& num);
	tSFCStream& operator>>(tPoint6& num);
	tSFCStream& operator>>(tFlt2Int4& num);
	tSFCStream& operator>>(tPoint2& num);
	tSFCStream& operator>>(tLineSegment& num);
	tSFCStream& operator>>(std::string& value);


	void mWriteStringToStream(char *str);
	// this returns the string length (excluding the trailing zero)
	int32 mReadStringFromStream(char *str);
	void mWriteTextLineToStream(char *str);
	// this returns the string length (excluding the trailing zero)
	int32 mReadTextLineFromStream(char *str);

	void mIncrementTabNumber() {_func_start( tSFCStream::mIncrementTabNumber ); fTabNumber++; _func_end;};
	void mDecrementTabNumber() {_func_start( tSFCStream::mDecrementTabNumber ); fTabNumber--; _func_end;};
	void mResetTabNumber() {_func_start( tSFCStream::mResetTabNumber ); fTabNumber = 0; _func_end;};
	void mWriteOutTabs();
	void mReadInTabs();
	int32 mGetTabNumber() {_func_start( tSFCStream::mGetTabNumber ); return fTabNumber; _func_end;};
	void mSetTabNumber(int32 tabNumber) {_func_start( tSFCStream::mSetTabNumber ); fTabNumber = tabNumber; _func_end;};

	//Generic Stream function
	static inline void mIO(void *p, const void *src, int size);

protected:
	int32 fVersionNumber;
	int8u *fBuffer;
	int8u *fGetPtr;
	int8u *fPutPtr;
	int8u *fMaxPtr;
	iTruth fOwned;
	iTruth	fReadLock;
	iTruth	fWriteLock;

	int32 fTabNumber;
};

enum 
{
	kArchiveStreamUncompress = 1,
	kArchiveStreamCompress = 2,
	kArchiveTextMode = 4,
	kNoCompression = 8
};

//--------------------------------------------------------------------------------
// func:	Serves as both stream input, and stream output.
// note:	When stream input,
//				src == byte stream (least significant byte first)
//				p == destination
//			When stream output
//				p == byte stream (least significant byte first)
//				src == source
//
inline void tSFCStream::mIO(void *p, const void *src, int size)
{
	_func_start( tSFCStream::mIO );
	const int8u	*s = (int8u*)src;
	int8u		*d = (int8u*)p;

#if EndianBig
	s += size-1;
#endif

	for (int i=0; i<size; i++)
	{
		*d++ = s[0];
#if EndianBig
		s--;
#else
		s++;
#endif
	}
	_func_end;
}


extern void ArchiveInt32(int32&num, tSFCStream &s, int32u mode);
extern void ArchiveInt32u(int32u &num, tSFCStream &s, int32u mode);
extern void ArchiveInt16(int16 &num, tSFCStream &s, int32u mode);
extern void ArchiveInt16u(int16u &num, tSFCStream &s, int32u mode);
extern void ArchiveInt8(int8 &num, tSFCStream &s, int32u mode);
extern void ArchiveInt8u(int8u &num, tSFCStream &s, int32u mode);
extern void ArchiveITruth(iTruth &num, tSFCStream &s, int32u mode);
extern void ArchiveFloat(float &num, tSFCStream &s, int32u mode);
extern void ArchiveTPoint3(tPoint3 &num, tSFCStream &s, int32u mode);
extern void ArchiveTMatrix(tMatrix &num, tSFCStream &s, int32u mode);
extern void ArchiveTString(tString &num, tSFCStream &s, int32u mode);
extern void ArchiveTPoint4(tPoint4 &num, tSFCStream &s, int32u mode);
extern void ArchiveTPoint6(tPoint6 &num, tSFCStream &s, int32u mode);
extern void ArchiveTFlt2Int4(tFlt2Int4 &num, tSFCStream &s, int32u mode);
extern void ArchiveTPoint2(tPoint2 &num, tSFCStream &s, int32u mode);
extern void ArchiveTLineSegment(tLineSegment &num, tSFCStream &s, int32u mode);
extern void ArchiveDouble(double &num, tSFCStream &s, int32u mode);
extern void ArchiveInteger(int32 size, void *num, tSFCStream &s, int32u mode);
extern void ArchiveString(char *str, tSFCStream &s, int32u mode);
extern void ArchiveStdString( std::string& str, tSFCStream &s, int32u mode );

extern void ArchiveInt32TM(int32&num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveInt32uTM(int32u &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveInt16TM(int16 &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveInt16uTM(int16u &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveInt8TM(int8 &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveInt8uTM(int8u &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveITruthTM(iTruth &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveFloatTM(float &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTPoint3TM(tPoint3 &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTMatrixTM(tMatrix &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTStringTM(tString &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTPoint4TM(tPoint4 &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTPoint6TM(tPoint6 &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTFlt2Int4TM(tFlt2Int4 &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTPoint2TM(tPoint2 &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTLineSegmentTM(tLineSegment &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveDoubleTM(double &num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveIntegerTM(int32 size, void *num, tSFCStream &s, int32u mode, char *text);
extern void ArchiveStringTM(char *str, tSFCStream &s, int32u mode, char *text);
extern void ArchiveTitleTM(char *title, tSFCStream &s, int32u mode);
extern void ArchiveSeperatorTM(tSFCStream &s, int32u mode);
extern void ArchiveStdStringTM( std::string& str, tSFCStream &s, int32u mode, char *text );

#ifdef _NO_ARCHIVE_TEXT_MODE_
	#define ArchiveInt32TM(a,b,c,d) ArchiveInt32(a,b,c)
	#define ArchiveMinMaxIntTM(a,b,c,d) ArchiveMinMaxInt(a,b,c)
	#define ArchiveInt32uTM(a,b,c,d) ArchiveInt32u(a,b,c)
	#define ArchiveInt16TM(a,b,c,d) ArchiveInt16(a,b,c)
	#define ArchiveInt16uTM(a,b,c,d) ArchiveInt16u(a,b,c)
	#define ArchiveInt8TM(a,b,c,d) ArchiveInt8(a,b,c)
	#define ArchiveInt8uTM(a,b,c,d) ArchiveInt8u(a,b,c)
	#define ArchiveITruthTM(a,b,c,d) ArchiveITruth(a,b,c)
	#define ArchiveFloatTM(a,b,c,d) ArchiveFloat(a,b,c)
	#define ArchiveMinMaxFloatTM(a,b,c,d) ArchiveMinMaxFloat(a,b,c)
	#define ArchiveTPoint3TM(a,b,c,d) ArchiveTPoint3(a,b,c)
	#define ArchiveTMatrixTM(a,b,c,d) ArchiveTMatrix(a,b,c)
	#define ArchiveTStringTM(a,b,c,d) ArchiveTString(a,b,c)
	#define ArchiveTPoint4TM(a,b,c,d) ArchiveTPoint4(a,b,c)
	#define ArchiveTPoint6TM(a,b,c,d) ArchiveTPoint6(a,b,c)
	#define ArchiveTFlt2Int4TM(a,b,c,d) ArchiveTFlt2Int4(a,b,c)
	#define ArchiveTPoint2TM(a,b,c,d) ArchiveTPoint2(a,b,c)
	#define ArchiveTLineSegmentTM(a,b,c,d) ArchiveTLineSegment(a,b,c)
	#define ArchiveDoubleTM(a,b,c,d) ArchiveDouble(a,b,c)
	#define ArchiveIntegerTM(a,b,c,d,e) ArchiveInteger(a,b,c,d)
	#define ArchiveStringTM(a,b,c,d) ArchiveString(a,b,c)
	#define ArchiveTitleTM(a,b,c) 
	#define ArchiveSeperatorTM(a,b)
#endif


// this compresses an int32 -- numbers from -63 to 63 will take only 1 byte...
extern void Int32StreamCompress(int32& num, tSFCStream &s);
extern void Int32StreamUncompress(int32& num, tSFCStream &s);


extern void Int32uStreamCompress(int32u& num, tSFCStream &s);
extern void Int32uStreamUncompress(int32u& num, tSFCStream &s);

extern void Int16StreamCompress(int16& num, tSFCStream &s);
extern void Int16StreamUncompress(int16& num, tSFCStream &s);

extern void Int16uStreamCompress(int16u& num, tSFCStream &s);
extern void Int16uStreamUncompress(int16u& num, tSFCStream &s);

extern void Int8StreamCompress(int8& num, tSFCStream &s);
extern void Int8StreamUncompress(int8& num, tSFCStream &s);

extern void Int8uStreamCompress(int8u& num, tSFCStream &s);
extern void Int8uStreamUncompress(int8u& num, tSFCStream &s);

extern void ITruthStreamCompress(iTruth& num, tSFCStream &s);
extern void ITruthStreamUncompress(iTruth& num, tSFCStream &s);

// you've got some kind of integer and you don't know what size it is 
// Be careful: (we assume it is signed)
extern void IntegerStreamCompress(int32 size, void *num, tSFCStream &s);
extern void IntegerStreamUncompress(int32 size, void *num, tSFCStream &s);

// these compress a stream into 2 bytes keeping up to 10 bits of useful data 
//	(3 nonzero digits) -- if your number can be very large or small this can
//	fail... (small would be around .00000000000009995.f (and will be shortened to 0.f),  
//	and big 999500000000000000.0f -- which will outright fail)...
extern void FloatStreamCompress(float& num, tSFCStream &s);
extern void FloatStreamUncompress(float& num, tSFCStream &s);

extern void TPoint3StreamCompress(tPoint3& num, tSFCStream &s);
extern void TPoint3StreamUncompress(tPoint3& num, tSFCStream &s);

extern void TMatrixStreamCompress(tMatrix& num, tSFCStream &s);
extern void TMatrixStreamUncompress(tMatrix& num, tSFCStream &s);

extern void TStringStreamCompress(tString& num, tSFCStream &s);
extern void TStringStreamUncompress(tString& num, tSFCStream &s);

extern void TStdStringStreamCompress( std::string& str, tSFCStream &s );
extern void TStdStringStreamUncompress( std::string& str, tSFCStream &s );

extern void TPoint4StreamCompress(tPoint4& num, tSFCStream &s);
extern void TPoint4StreamUncompress(tPoint4& num, tSFCStream &s);

extern void TPoint6StreamCompress(tPoint6& num, tSFCStream &s);
extern void TPoint6StreamUncompress(tPoint6& num, tSFCStream &s);

extern void TFlt2Int4StreamCompress(tFlt2Int4& num, tSFCStream &s);
extern void TFlt2Int4StreamUncompress(tFlt2Int4& num, tSFCStream &s);

extern void TPoint2StreamCompress(tPoint2& num, tSFCStream &s);
extern void TPoint2StreamUncompress(tPoint2& num, tSFCStream &s);

extern void TLineSegmentStreamCompress(tLineSegment& num, tSFCStream &s);
extern void TLineSegmentStreamUncompress(tLineSegment& num, tSFCStream &s);

// Beware these convert a double to a float and then do the above -- if that's not appropriate,
//	do not use these
extern void DoubleStreamCompress(double& num, tSFCStream &s);
extern void DoubleStreamUncompress(double& num, tSFCStream &s);
extern void TextStreamCompress(char *text, tSFCStream &s);
extern void TextStreamUncompress(char *text, tSFCStream &s);



// this compresses an int32 -- numbers from -63 to 63 will take only 1 byte...
extern void Int32StreamOutText(char *text, int32& num, tSFCStream &s);
extern void Int32StreamInText(char *text, int32& num, tSFCStream &s);

extern void Int32uStreamOutText(char *text, int32u& num, tSFCStream &s);
extern void Int32uStreamInText(char *text, int32u& num, tSFCStream &s);

extern void Int16StreamOutText(char *text, int16& num, tSFCStream &s);
extern void Int16StreamInText(char *text, int16& num, tSFCStream &s);

extern void Int16uStreamOutText(char *text, int16u& num, tSFCStream &s);
extern void Int16uStreamInText(char *text, int16u& num, tSFCStream &s);

extern void Int8StreamOutText(char *text, int8& num, tSFCStream &s);
extern void Int8StreamInText(char *text, int8& num, tSFCStream &s);

extern void Int8uStreamOutText(char *text, int8u& num, tSFCStream &s);
extern void Int8uStreamInText(char *text, int8u& num, tSFCStream &s);

extern void ITruthStreamOutText(char *text, iTruth& num, tSFCStream &s);
extern void ITruthStreamInText(char *text, iTruth& num, tSFCStream &s);

// you've got some kind of integer and you don't know what size it is
extern void IntegerStreamOutText(char *text, int32 size, void *num, tSFCStream &s);
extern void IntegerStreamInText(char *text, int32 size, void *num, tSFCStream &s);

extern void FloatStreamOutText(char *text, float& num, tSFCStream &s);
extern void FloatStreamInText(char *text, float& num, tSFCStream &s);

extern void TPoint3StreamOutText(char *text, tPoint3& num, tSFCStream &s);
extern void TPoint3StreamInText(char *text, tPoint3& num, tSFCStream &s);

extern void TMatrixStreamOutText(char *text, tMatrix& num, tSFCStream &s);
extern void TMatrixStreamInText(char *text, tMatrix& num, tSFCStream &s);

extern void TStringStreamOutText(tString& text, tString& num, tSFCStream &s);
extern void TStringStreamInText(tString& text, tString& num, tSFCStream &s);

extern void TStdStringStreamOutText( std::string& text, std::string& num, tSFCStream &s);
extern void TStdStringStreamInText( std::string& text, std::string& num, tSFCStream &s);

extern void TPoint4StreamOutText(char *text, tPoint4& num, tSFCStream &s);
extern void TPoint4StreamInText(char *text, tPoint4& num, tSFCStream &s);

extern void TPoint6StreamOutText(char *text, tPoint6& num, tSFCStream &s);
extern void TPoint6StreamInText(char *text, tPoint6& num, tSFCStream &s);

extern void TPoint2StreamOutText(char *text, tPoint2& num, tSFCStream &s);
extern void TPoint2StreamInText(char *text, tPoint2& num, tSFCStream &s);

extern void TFlt2Int4StreamOutText(char *text, tFlt2Int4& num, tSFCStream &s);
extern void TFlt2Int4StreamInText(char *text, tFlt2Int4& num, tSFCStream &s);

extern void TLineSegmentStreamOutText(char *text, tLineSegment& num, tSFCStream &s);
extern void TLineSegmentStreamInText(char *text, tLineSegment& num, tSFCStream &s);


extern void DoubleStreamOutText(char *text, double& num, tSFCStream &s);
extern void DoubleStreamInText(char *text, double& num, tSFCStream &s);
 
extern void TextStreamOutText(char *text, char *num, tSFCStream &s);
extern void TextStreamInText(char *text, char *num, tSFCStream &s);

extern void TextStreamOutSeperator(tSFCStream &s);
extern void TextStreamInSeperator(tSFCStream &s);

extern void TextStreamOutTitle(char *text, tSFCStream &s);
extern void TextStreamInTitle(char *text, tSFCStream &s);

extern void IncrementStreamTab(tSFCStream &s);
extern void DecrementStreamTab(tSFCStream &s);
extern void ResetStreamTab(tSFCStream &s);

#endif
// <History>
// 7/21/98	oschlunk	Created
//	28Jul98		oschlunk added floating point compression
//	3Sept98		oschlunk	added archiving stuff -- greatly expanded everything
//	17Sep99		rliu		Changed some functions to const so const class& can access it.
