//	<Name>	team.h
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	Ships (and some other objects) -- belong to a team -- this manages those teams...
//
#pragma once

#ifndef team_h
#define team_h

#include "tclient.h"
#include "entitymgr.h"
#include "TeamData.h"

typedef xvector(tID) tTeamMemberContainer;

class tEntity;
class tAIAdmiral;
class tBasicTeam;
class tFleetControl;

#define kKilledEntityListSize 20

// do not add any functions of any sort to this
struct tWounded
{
	int32 fEntityID;
	int32 fHitsRemaining;
};
class tBasicTeamMgr
{
public:
	tBasicTeamMgr();
	~tBasicTeamMgr();

	void mBeginGame();
	void mEndGame();

	//A return of kNoTeam means there are no more team slots available
	eTeamID mAddNewTeam(eRaceName AssociatedRace = kFederation);
	tBasicTeam *mGetTeam(eTeamID teamID);
	eTeamID mGetTeamID( tID entityID );

	eTeamRelationship mGetRelationshipBetweenTeams(eTeamID teamID1, eTeamID teamID2);
	eTeamRelationship mGetRelationshipBetweenEntities( tID entity1ID, tID entity2ID );
	void mSetTeamRelationship(eTeamID teamID1, eTeamID teamID2, eTeamRelationship relationship);

	// this function hopes to prevent two weapons on the same team killing the same entity
	iTruth mHasThisEntityAlreadyBeenKilledByOurTeam(int32 entityID, eTeamID teamID);
	void mAddEntityToKilledListOfOurTeam(int32 entityID, eTeamID teamID);
	eTeamRelationship mGetRelationshipOfThisEntityToOurTeam(tEntity *entity, eTeamID teamID);

	// returns ID of team that matches the given relationship for the given team
	// (mGetTeamWithRelations will create one if one does not exist)
	eTeamID mFindTeamWithRelations (eTeamID teamID, eTeamRelationship Relations);
	eTeamID mGetTeamWithRelations (eTeamID teamID, eTeamRelationship Relations);
	void mAddToTotalDamage (eTeamID teamID, int32 Damage, eTeamDamagetypes Type);
	int32 mGetTotalDamage (eTeamID teamID, eTeamRelationship RelationshipToTeam, eTeamDamagetypes Type);


	tWounded *mGetWoundedList(eTeamID teamID);
	int32 mGetSizeOfWoundedList(eTeamID teamID);
	void mAddToWoundedList(eTeamID teamID, int32 ID, int32 hitsRemaining);
	void mRemoveFromWoundedList(eTeamID teamID, int32 ID);
	iTruth mAreEntitiesTeamMates (tID Entity1, tID Entity2);

	tFleetControl *mFindFleet( eTeamID teamID );
	eRace fTeamRaces[kMaxTeams];	//	need to keep races consistent between briefing and tactical
	void mResetTeamRaces( void );
protected:
	void mCleanUpTeams();
	iTruth fGameStarted;
	tBasicTeam*	fTeams[kMaxTeams];
};
class tBasicTeam
{
public:
	tBasicTeam( eTeamID teamID );
	~tBasicTeam();

	void mAddEntityToTeam( int32 shipID,bool dueToCapture = false );
	void mRemoveEntityFromTeam( int32 shipID );
	iTruth mIsEntityOnTeam (int32 shipID);

	void mAddToFleet( int32 shipID );
	void mRemovedFromFleet( int32 shipID );
	tFleetControl *mGetFleet();

	eTeamRelationship mGetRelationshipWithTeam(eTeamID teamID) {_func_start( tBasicTeam::mGetRelationshipWithTeam ); return fRelationships[teamID]; _func_end;};
	void mSetTeamRelationship(eTeamID teamID, eTeamRelationship relationship);

	// this function hopes to prevent two ESGs on the same team killing the same entity
	iTruth mHasThisEntityAlreadyBeenKilledByOurTeam(int32 ID);
	void mAddEntityToKilledList(int32 ID);

	tWounded *mGetWoundedList() {_func_start( tBasicTeam::mGetWoundedList ); mCleanUpWoundedList(); return fWoundedList; _func_end;};
	int32 mGetSizeOfWoundedList() {_func_start( tBasicTeam::mGetSizeOfWoundedList ); return fNumberOnWoundedList; _func_end;};
	void mAddToWoundedList(int32 ID, int32 hitsRemaining);
	void mRemoveFromWoundedList(int32 ID);

	eTeamID mGetTeamID();

	tTeamMemberContainer* mGetListOfTeamMateIDs (void) {_func_start( tBasicTeam::mGetListOfTeamMateIDs );return &fTeamMateIDs;_func_end;};
		
	tTeamMemberContainer fTeamMateIDs;
	// Also sets the default color.
	void mSetRaceAssociation(eRaceName race) {_func_start( tBasicTeam::mSetRaceAssociation );fAssociatedRace = race;fAssociatedColor = (eTeamColor)race; _func_end;};
	eRaceName mGetRaceAssociation(void) {_func_start( tBasicTeam::mGetRaceAssociation ); return fAssociatedRace;_func_end;};
	void mSetColorAssociation(eTeamColor color) {_func_start( tBasicTeam::mSetColorAssociation );fAssociatedColor = color;_func_end;};
	eTeamColor mGetColorAssociation(void) {_func_start( tBasicTeam::mGetColorAssociation ); return fAssociatedColor;_func_end;};

	void mAddToTotalDamage (int32 Damage, eTeamDamagetypes Type);
	int32 mGetTotalDamage (eTeamDamagetypes Type);
protected:
	//Accessors for the AI
	friend class tAIManager;
	tID mGetAdmiralID();
	void mSetAdmiralID( tID admiralID );
	tID mRemoveAdmiral();

	eTeamRelationship fRelationships[kMaxTeams];

	// ESG and things need to keep lists of things they think they've destroyed --this is currently
	//	where we will keep this information...
	int32 fKilledIDList[kKilledEntityListSize];
	int32 fNumberOnKilledIDList;
	int32 fLastKilledIDAdded;
	int32 fTotalDamage[kDamageTypeCombinedInternalAndShields];
	
	tWounded *fWoundedList;
	int32 fNumberOnWoundedList;
	int32 fMaxNumberOnWoundedList;

	bool fPlayersTeam;
	eTeamID fTeamID;
	tID fTeamAdmiralID; //The Ai admiral in charge of this team
	tFleetControl* fFleet;// for attack modes, weapon modes, and formation
	eRaceName fAssociatedRace;
	eTeamColor fAssociatedColor;
	void mCleanUpWoundedList();

};



extern tBasicTeamMgr *gBasicTeamMgr;

#endif

// <History>
// 6/10/98	oschlunk	Created
//	9/1/98	oschlunk	Added wounded list for point defense systems...
// 7Oct98	zdrummond	Rearanged how tBasicTeamMgr works. It now only allocates a new team, if it
//								is requested. When a new team is created it registers it self with the AI.
//								The AI can use new functions to add and remove an admiral from a team.
//								Changed kNoTeam to -1, because 0 is a valid value
//	9Oct98	zdrummond	We now store the admirals ID, instead of using a pointer
//	21Oct98	zdrummond	Added a begin and end game function to the team manager
//	17Nov98	mhertogh	Inplemented the AddEntityToTeam and Remove functions.
//	04Dec98	dboulanger	added extra functions for relationships for scripts - removed unneed functions
//	17Dec98	mhertogh	Added game start flag to make sure we only start(reset) a game once.
//	21Jan99	zdrummond	Added a typedef for the team container. Move the implementaion for mSetTeamRelationship to the source file
//	29Jan99	mhertogh	Added a race association for each team. 
//	22Mar99	zdrummond	Added a mGetTeamID function
//	13Apr99	mhertogh	Added Team colors
//	23Apr99	dboulanger	added function to display game stats

