//	<Name>	
//
//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.
//
//	<Info>	This file is starfleet-command specific. 
//
#pragma once

#ifndef tmissionmap_h
#define tmissionmap_h

#include "scriptsdata.h"
#include "MapObject.h"

	// Forward references
class	tMapInfo;
struct  tPoint3;

struct tMapPoint {
	int16	x;
	int16	y;
	
	void operator=( tMapPoint rhs )
	{
		_func_start( operator = );
		x = rhs.x;
		y = rhs.y;
		_func_end;
	}

	} ;

class tMissionMap
{
public:
/*
enum eMapValue
{
		kOffMapSpace,
		kNearMapEdgeSpace,
		kNormalSpace,
		kDustCloudLight,
		kDustCloudMedium,
		kDustCloudHeavy,
		kAsteroidLight,
		kAsteroidMedium,
		kAsteroidHeavy,
		kIceLight,
		kIceMedium,
		kIceHeavy,
		kNebulaLight,
		kNebulaHeavy,
		kStar,
		kPlanet,
		kBlackHoleSmall,
		kBlackHoleLarge,
		kFissure,
		kWormHole,
		kCorpse
}; */
/*
enum{	// values cannot go beyond 127
	kMapAsteroid	=  1,				// uses 6 types of asteroids
	kMapDust		= 10,				// uses 3 types of dust clouds
	kMapNebula		= 15,				// uses 2 types of nebulas
	kMapStarts		= 20,				// uses 20 start positions
	kMapEnds		= 45,				// uses 20 end positions
	kMapPlanets		= 70,				// uses 7 types of planets
	kMapStars		= 80,				// uses 3 types of suns
	kMapBlackHoles	= 85,				// uses 2 types of black holes
	kMapFissures	= 90,				// uses 1 type of fissure
	kMapWormHoles	= 95,				// uses 1 type of worm hole
	kMapCorpses		=100,				// uses 1 type of corpse
};
*/

	tMissionMap ( void );
	virtual ~tMissionMap();
	

				// Methods to build the map (used by mission loader)
	void mCreateMapObject( const tMapObjectType& mapObject, int32 x, int32 y );
	void mAddAsteroid ( int32 density, tMapPoint location );	
	void mAddDustCloud ( int32 density, tMapPoint location );	
	void mAddEndLoc ( int32 index, tMapPoint location );
	void mAddNebula ( int32 density, tMapPoint location );	
	void mAddPlanet ( int32 index, tMapPoint location );
	void mCreateNonTargetableObject( std::string specName, tMapPoint location );
	void mAddStar ( int32 index, tMapPoint location );
	void mAddStartLoc ( int32 index, tMapPoint location );
	void mAddBlackHole ( int32 size, tMapPoint location );
	void mAddFissure ( int32 size, tMapPoint location );
	void mAddOrganian ( int32 size, tMapPoint location );
	void mAddPulsar ( int32 size, tMapPoint location );
	void mAddWormHole ( int32 size, tMapPoint location );
	void mAddTopRightMapBoundary(tMapPoint location);
	void mAddBotLeftMapBoundary(tMapPoint location);
	void mAddCorpses ( int32 size, tMapPoint location );
	void mAddPrimitiveVertex( int32 ID, tMapPoint location, eScriptPrimitiveFlags options );
	void mAddIonStorm(tMapPoint location);
	
				//	Player location information retrieval
	void	mSetMapSize ( int32 rows, int32 cols );
	void	mGetMapSize ( int32* rows, int32* cols );
	int32	mCountStartLocs ( void );
	int32	mCountStartLocs ( int32 playerno );
	tPoint3	mGetEndLoc ( int32 playerno, iTruth* exists );
	tPoint3	mGetStartLoc ( int32 playerno, iTruth* exists );
	tPoint3	mGetStartLoc ( int32 playerno, int32 num, iTruth* exists );
	const tMapObjectType& mGetMapData( tPoint3 locationIn3D );
	const tMapObjectType& mGetMapData( int32 x, int32 y );
	tMapPoint mFindMapEntry( const tMapObjectType& type );
	tMapPoint mGetValidStartPoint (int32 GridX, int32 GridY);
	tPoint3 mGetPositionOffset( iMapPosition posOffseset );

				// Map to screen location conversion
	tPoint3	mConvertLocation ( tMapPoint );
				// Convert map distance to screen distance
	float	mConvertMapDistance (float Distance);
				// screen to map location conversion
	tMapPoint m3DToMapPoint (tPoint3 LocationIn3D);
	
	iTruth	mIsThisABlackHoleMap(void) {_func_start( tMissionMap::mIsThisABlackHoleMap );return fBlackHoleMap;_func_end;};
				//returns map width and height
	static tMapPoint mGetMapDimension (void) {_func_start( tMissionMap::mGetMapDimension ); return fMapSize;_func_end;};
	
protected:
	iTruth  mAreaClear (int32 GridX, int32 GridY);
				// Map to screen location conversion
	tPoint3	mConvertPlanetLocation	( tMapPoint );

	const tMapObjectType fOffMapEdge;
	tMapInfo*	fInfo;
	iTruth	fBlackHoleMap;
	static tMapPoint fMapSize;
private:
	tMissionMap &operator=(  tMissionMap &Copy );
	tMissionMap ( tMissionMap &Copy );
};

#endif
//	<History>
//	23Apr98		sroney		Started
//	28Apr98		mhertogh	Added ConvertPlanetLocation
//	08Jun98		dmandap		Added ~tMissionMap() destructor so we can cleanup properly
//	20Jul98		mhertogh	removed DisplayTerrain.
//	15Oct98		dboulanger	added extra terrain enums for off map space and black holes
//	21Oct98		mhertogh	Added mGetMapData options.  Can now use tPoint3 or X,Y
//	23Oct98		dboulanger	added mConvertMapDistance function
//	19Nov98		dboulanger	made mGetMapDimension static
//	24Nov98		dboulanger	added CreateMapObject and FindMapEntry calls.
//							changed AddPlanet and AddStar to use given index value.

