// Primary Precompiled Header
#include "Precompiled.h"

#include "Strength.h"

// Team Info
tStrength::tStrength()
{
	_func_start( tStrength::tStrength );
	fShipStrengthList.clear();
	fMinCount = 0;
	fMaxCount = 0;
	_func_end;
}


tStrength& tStrength::operator= ( const tStrength& aStrength )
{
	_func_start( tStrength::operator = );
	if ( this != &aStrength )
	{
		fMinCount = aStrength.fMinCount;
		fMaxCount = aStrength.fMaxCount;
		fShipStrengthList.assign( aStrength.fShipStrengthList.begin(), aStrength.fShipStrengthList.end() );
	}
	return *this;
	_func_end;
}

tStrength::tStrength( std::vector< tShipStrength > shipStrengthList )
{
	_func_start( tStrength::tStrength );
	//
	//	Copy over the ship strength info to the tStrength class member field.
	//
	for ( std::vector< tShipStrength >::iterator i = shipStrengthList.begin(); i != shipStrengthList.end(); i++ )
	{
		fShipStrengthList.push_back( *i );
	}
	if ( shipStrengthList.empty() )
	{
		fMinCount = fMaxCount = 0;
	}
	else
	{
		fMinCount = kMinNonNPCShips;
		if ( shipStrengthList.size() > kMaxNonNPCShips )
		{
			shipStrengthList.resize( kMaxNonNPCShips );
			fMaxCount = kMaxNonNPCShips;
		}
		else
		{
			fMaxCount = shipStrengthList.size();
		}
	}
	_func_end;
}

tStrength::~tStrength( void )
{
	_func_start( tStrength::~tStrength );
	fShipStrengthList.clear();
	_func_end;
}

void tStrength::StreamOut( nDataStore::tOutput& output ) const
{
	_func_start( tStrength::StreamOut );
	output	<<	fMinCount
			<<	fMaxCount;

	output	<<	fShipStrengthList;
	_func_end;
}

void tStrength::StreamIn( nDataStore::tInput& input )
{
	_func_start( tStrength::StreamIn );
	input	>>	fMinCount
			>>	fMaxCount;

	input	>>	fShipStrengthList;
	_func_end;
}

tShipStrength::tShipStrength( void )
:	fMinClass( kClassFreighter ), fMaxClass( kClassStarBase ),
	fMinBPV( kMinBPV ), fMaxBPV( kMaxBPV ),
	fMinRatio( 0.8f ), fMaxRatio( 1.2f ),
	fRelativeToPrimaryTeam( false )
{_func_start( tShipStrength::tShipStrength );_func_end;}

tShipStrength::tShipStrength( eClassTypes minClass, eClassTypes maxClass, int32 minBPV, int32 maxBPV )
:	fMinClass( minClass ), fMaxClass( maxClass ),
	fMinBPV( minBPV ), fMaxBPV( maxBPV ),
	fMinRatio( 0.8f ), fMaxRatio( 1.2f ),
	fRelativeToPrimaryTeam( false )
{_func_start( tShipStrength::tShipStrength );_func_end;}

tShipStrength::tShipStrength( eClassTypes minClass, eClassTypes maxClass, float minRatio, float maxRatio )
:	fMinClass( minClass ), fMaxClass( maxClass ),
	fMinBPV( kMinBPV ), fMaxBPV( kMaxBPV ),
	fMinRatio( minRatio ), fMaxRatio( maxRatio ),
	fRelativeToPrimaryTeam( true )
{_func_start( tShipStrength::tShipStrength );_func_end;}

tShipStrength::tShipStrength( const tShipStrength& rhs )
:	fMinClass( rhs.fMinClass ), fMaxClass( rhs.fMaxClass ),
	fMinBPV( rhs.fMinBPV ), fMaxBPV( rhs.fMaxBPV ),
	fMinRatio( rhs.fMinRatio ), fMaxRatio( rhs.fMaxRatio ),
	fRelativeToPrimaryTeam( rhs.fRelativeToPrimaryTeam )
{_func_start( tShipStrength::tShipStrength );_func_end;}

tShipStrength::operator=( tShipStrength rhs )
{
	_func_start( tShipStrength::operator = );
	fMinClass = rhs.fMinClass;
	fMaxClass = rhs.fMaxClass;
	fMinBPV = rhs.fMinBPV;
	fMaxBPV = rhs.fMaxBPV;
	fMinRatio = rhs.fMinRatio;
	fMaxRatio = rhs.fMaxRatio;
	fRelativeToPrimaryTeam = rhs.fRelativeToPrimaryTeam;
	_func_end;
}

void tShipStrength::StreamOut( nDataStore::tOutput& output ) const
{
	_func_start( tShipStrength::StreamOut );
	output	<<	STREAM_ENUM( fMinClass )
			<<	STREAM_ENUM( fMaxClass );

	output	<<	fMinBPV
			<<	fMaxBPV;

	output	<<	fMinRatio
			<<	fMaxRatio;

	output	<<	fRelativeToPrimaryTeam;

	_func_end;
}

void tShipStrength::StreamIn( nDataStore::tInput& input )
{
	_func_start( tShipStrength::StreamIn );
	input	>>	STREAM_ENUM( fMinClass )
			>>	STREAM_ENUM( fMaxClass );

	input	>>	fMinBPV
			>>	fMaxBPV;

	input	>>	fMinRatio
			>>	fMaxRatio;

	input	>>	fRelativeToPrimaryTeam;
	_func_end;
}

