//	<Name>	missionspec.cpp
//
//
//	<Info>	DESCRIPTION
//

// Primary Precompiled Header
#include "Precompiled.h"


#include "missionspec.h"

tTeamSpec::tTeamSpec(	const std::string&	teamName, 
						int32				teamID, 
						float				chanceTeamIsAvailable, 
						bool				required, 
						eTeamType			teamType, 
						float				chanceTeamCanBePlayedByAI, 
						tStrength			strengthLimits, 
						iMissSpecRace		validRaces, 
						iRaceCriteria		raceCriteria, 
						iAllocatorID		teamAllocTypeID,
						eTeamTag			teamTag,
						const std::string&	teamMissionTitle,
						std::string*		briefings,
						bool				allowJoinInProgress
					 ):
	fTeamName( teamName ),
	fTeamID( teamID ),
	fChanceTeamIsAvailable( chanceTeamIsAvailable ),
	fRequired( required ),
	fTeamType( teamType ),
	fChanceTeamCanBePlayedByAI( chanceTeamCanBePlayedByAI ),
	fStrengthLimits( strengthLimits ),
	fValidRaces( validRaces ),
	fRaceCriteria( raceCriteria ),
	fTeamAllocTypeID( teamAllocTypeID ),
	fTeamTag( teamTag ),
	fTeamMissionTitle( teamMissionTitle ),
	fAllowJoinInProgress( allowJoinInProgress )
{
	_func_start( tTeamSpec::tTeamSpec );
	if ( briefings )
	{
		for ( int32 i = 0; i < kNumActiveRaces; i++ )
			fBriefings[i] = briefings[i];
	}
	_func_end;
}

tTeamSpec::tTeamSpec()
{_func_start( tTeamSpec::tTeamSpec );_func_end;}

tTeamSpec::~tTeamSpec()
{_func_start( tTeamSpec::~tTeamSpec );_func_end;}


tMissionSpec::tMissionSpec() :
	fPlayableTeamCount( 0 ),
	fTeamCount( 0 ),
	fMetaLocationSpec( 0, kPlayerRaceHex, kAnyTerrainHex ),
	fMissionType( kCampaignMission ),
	fPlayerRace( kSpecNoRace ),
	fGenericFlags( kNoGenericFlags )
{
	_func_start( tMissionSpec::tMissionSpec );
	for( int i = 0; i < kMaxTeams; i++ )
	{
		fTeam[i] = NULL;
	}
	fBandwidthPart = static_cast< eBandwidthPart >( kForgotHostInitialization | kForgotClientInitialization );
	_func_end;
}

tMissionSpec::tMissionSpec( const tMissionSpec& spec )
{
	_func_start( tMissionSpec::tMissionSpec );
	copyData( spec );
	_func_end;
}

tMissionSpec& tMissionSpec::operator=( const tMissionSpec& spec )
{
	_func_start( tMissionSpec::operator = );
	if( &spec != this )
	{
		for( int i = 0; i < fTeamCount; i++ )
		{
			if( fTeam[i] )
			{
				delete fTeam[i];
				fTeam[i] = NULL;
			}
		}

		copyData( spec );
	}

	return *this;
	_func_end;
}


tMissionSpec::~tMissionSpec()
{
	_func_start( tMissionSpec::~tMissionSpec );
	for( int i = 0; i < fTeamCount; i++ )
	{
		if( fTeam[i] )
		{
			delete fTeam[i];
		}
	}
	_func_end;
}

void tMissionSpec::copyData( const tMissionSpec& spec )
{
	_func_start( tMissionSpec::copyData );
	fVersion			= spec.fVersion;
	fFileName			= spec.fFileName;
	fMissionTitle		= spec.fMissionTitle;
	fMissionType		= spec.fMissionType;	
	fMissionDescription	= spec.fMissionDescription;	
	fHelpTags			= spec.fHelpTags;
	fPlayerRace			= spec.fPlayerRace;	
	fEnemyRace			= spec.fEnemyRace;	
	fNumShips			= spec.fNumShips;	
	fMetaLocationSpec	= spec.fMetaLocationSpec;
	fMinBPV				= spec.fMinBPV;
	fMaxBPV				= spec.fMaxBPV;
	fMinDate			= spec.fMinDate;
	fMaxDate			= spec.fMaxDate;
	fWait				= spec.fWait;
	fTutorialOrder		= spec.fTutorialOrder;
	fMenuStates			= spec.fMenuStates;
	fBandwidthPart		= spec.fBandwidthPart;
	fTeamCount			= spec.fTeamCount;
	fPlayableTeamCount	= spec.fPlayableTeamCount;
	fScriptMapData		= spec.fScriptMapData;
	fMapOverrides		= spec.fMapOverrides;
	fGenericFlags		= spec.fGenericFlags;

	fMapTerrainList.clear();
	fMapTerrainList.assign( spec.fMapTerrainList.begin(), spec.fMapTerrainList.end() );

	for( int i = 0; i < kMaxTeams; i++ )
	{
		if( i < fTeamCount && spec.fTeam[i] )
		{
			fTeam[i] = new tTeamSpec( *(spec.fTeam[i]) );	
		}
		else
		{
			fTeam[i] = NULL;
		}
	}
	_func_end;
}

void tMissionSpec::mSetupTeam( const tTeamSpec& teamSpec )
{
	_func_start( tMissionSpec::mSetupTeam );
	if( teamSpec.fTeamType >= kPlayableTeam )
	{
		fPlayableTeamCount++;
	} 

	fTeam[fTeamCount++] = new tTeamSpec( teamSpec );
	_func_end;
}

int32 tMissionSpec::mTeamCount()
{
	_func_start( tMissionSpec::mTeamCount );
	return fTeamCount;
	_func_end;
}

int32 tMissionSpec::mPlayableCount()
{
	_func_start( tMissionSpec::mPlayableCount );
	return fPlayableTeamCount;
	_func_end;
}

tTeamSpec* tMissionSpec::mGetTeamSpec( int32 index )
{
	_func_start( tMissionSpec::mGetTeamSpec );
	Assert( index < fTeamCount );
	if( index >= fTeamCount )
	{
		return fTeam[0]; 	
	}
	
	return fTeam[ index ]; 
	_func_end;
}

//	20Mar00	zdrummond	Added Comment Header and Footer's
//	16Jun00	dsuleski	The team race is now chosen in the tTeamSpec constructor.
//	12Jul00	dsuleski	fBandwidthPart added

