//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>

// Project
#include "CRC32.h"

// Statics
const int32 tCRC32::fPolynomial = 0x04c11db7L;

tCRC32::tCRC32()
{
	_func_start( tCRC32::tCRC32 );
	GenerateCRCTable();
	_func_end;
}

tCRC32::~tCRC32()
{_func_start( tCRC32::~tCRC32 );_func_end;}


//----------------------------------------------------------------------------
// Generate the table of CRC remainders for all possible bytes.
//----------------------------------------------------------------------------
void tCRC32::GenerateCRCTable()
{
	_func_start( tCRC32::GenerateCRCTable );
	int32 i, j;
	int32u crc_accum;
	for( i = 0; i < 256;  i++ )
	{
		crc_accum = ((int32u) i << 24);
		for ( j = 0;  j < 8;  j++ )
		{
			if (crc_accum & 0x80000000L)
				crc_accum = (crc_accum << 1) ^ fPolynomial;
			else
				crc_accum = (crc_accum << 1);
		}
		fTable[i] = crc_accum;
	}
	_func_end;
}


//----------------------------------------------------------------------------
// Update the CRC on the data block one byte at a time.
//----------------------------------------------------------------------------
int32u tCRC32::UpdateCRC( int32u crc_accum, const char *data_blk_ptr, int32 data_blk_size )
{
	_func_start( tCRC32::UpdateCRC );
	int32 i, j;
	for( j = 0; j < data_blk_size; j++ )
	{
		i = ((int32) (crc_accum >> 24) ^ *data_blk_ptr++) & 0xff;
		crc_accum = (crc_accum << 8) ^ fTable[i];
	}
	return crc_accum;
	_func_end;
}


//----------------------------------------------------------------------------
// Calculate the CRC on a file.
//----------------------------------------------------------------------------
int32u tCRC32::CalcFileCRC( const std::string& fileName, int32u nSeed /* = kDefaultSeed */ )
{
	_func_start( tCRC32::CalcFileCRC );
	if( fileName.empty() )
	{
		return 0;
	}

	// Create a file handle
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
	//ZDTEST@@@@
	tResourceTracker::Instance()->UsingHandle( tResourceTracker::k_tCRC32_CalcFileCRC );
#endif
	HANDLE fileHandle = ::CreateFile( fileName.c_str(), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL );
	if( fileHandle == INVALID_HANDLE_VALUE )
	{
		return 0;
	}

	char buffer[ kCRCBufferSize ];
	int32u fileSize = ::GetFileSize( fileHandle, NULL );
	int32u CRC = nSeed;
	int32u read = 0;
	int32u totalRead = 0;

	// Read in the file a buffer load at a time and accumulate a CRC for it until the entire file has a single CRC
	while( ::ReadFile( fileHandle, buffer, kCRCBufferSize, &read, NULL ) && read > 0 )
	{
		totalRead += read;
		CRC = UpdateCRC( CRC, buffer, read );
	}
	
	// Close the file
	//ZDTEST@@@@
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
	tResourceTracker::Instance()->ReleaseHandle( tResourceTracker::k_tCRC32_CalcFileCRC );
#endif
	::CloseHandle( fileHandle );

	if( totalRead != fileSize )
	{
		return 0;
	}

	return CRC;
	_func_end;
}

