//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>
#include <FunctionWrapper.h>

#if !defined( _INLINE_DataStoreBuffer )
	// Non-Inline section

#include "DataStoreBuffer.h"

namespace nDataStore
{

std::istream& operator>>( std::istream& inputStream, tBuffer& input )
{
	_func_start( nDataStore::operator >> );
	int32u size;
	inputStream.read( (char*)&size, sizeof( size ) );

	for( int32u i = 0; i < size && !inputStream.eof(); i++ )
	{
		input.Put( inputStream.get() );
	}

	if( i != size )
	{
		//The size was wrong, there was not enough data in the to fill the buffer 
		throw except::tOutOfBounds( "std::istream inputStream", 0, i, size, __FILE__, __LINE__ );
	}

	return inputStream;
	_func_end;
}

std::ostream& operator<<( std::ostream& outputStream, const tBuffer& output )
{
	_func_start( nDataStore::operator << );
	int32u size = output.Size();
	outputStream.write( (char*)&size, sizeof( size ) );
	
	for( int32u i = 0; i < size; i++ )
	{
		outputStream.put( output.Get( i ) );
	}

	return outputStream;
	_func_end;
}

};
	
#else
	// In-Inline section

#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>
#include <Exceptions\OutOfBounds.h>

#include "DataStoreBuffer.h"

namespace nDataStore
{

tBuffer::tBuffer( int32u initSize )
{
	_func_start( nDataStore::tBuffer::tBuffer );
	fBuffer.reserve( initSize );	
	_func_end;
}

/*
tBuffer::tBuffer( const tBuffer& copy )
{
	*this = copy;
}
*/

tBuffer::~tBuffer()
{_func_start( nDataStore::tBuffer::~tBuffer );_func_end;}

tBuffer::operator const byte*() const
{
	_func_start( nDataStore::tBuffer::operator const byte* );
	return fBuffer.begin();
	_func_end;
}

void tBuffer::Clear()
{
	_func_start( nDataStore::tBuffer::Clear );
	fBuffer.clear();
	_func_end;
}

void tBuffer::Put( byte data )
{
	_func_start( nDataStore::tBuffer::Put );
	fBuffer.push_back( data );	
	_func_end;
}

void tBuffer::Write( const byte* data, int32u size )
{
	_func_start( nDataStore::tBuffer::Write );
	fBuffer.insert( fBuffer.end(), data, data + size  );	
	_func_end;
}

byte tBuffer::Get( iPos getPos ) const
{
	_func_start( nDataStore::tBuffer::Get );
	if( getPos >= Size() )
	{
		throw except::tOutOfBounds( "tBuffer", 0, Size(), getPos, __FILE__, __LINE__ );
	}

	return fBuffer[getPos];
	_func_end;
}

void tBuffer::Read( byte* data, int32 size, iPos getPos ) const
{
	_func_start( nDataStore::tBuffer::Read );
	if( getPos + size > Size() )
	{
		throw except::tOutOfBounds( "tBuffer", 0, Size(), getPos + size, __FILE__, __LINE__ );
	}
	
	iBuffer::const_iterator start = fBuffer.begin() + getPos;
	iBuffer::const_iterator end = start + size;
	
	std::copy( start, end, data );
	_func_end;
}

int32u tBuffer::Size() const
{
	_func_start( nDataStore::tBuffer::Size );
	return fBuffer.size();
	_func_end;
} 

};

#endif
