//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( DataStoreInput_h )
#define DataStoreInput_h

#include <BaseTypes.h>

#if !defined( USE_PCH )

#include <string>
#include <map>
#include <vector>
#include <list>
#include <deque>

#endif


#include <Thread\InterlockedScalar.h>

#include "DataStore.h"
#include "DataStoreBuffer.h"

namespace nDataStore
{

// CLASS PURPOSE: To retrive data from a Buffer
class tInput
{
public:
	typedef int32u iInputFlags;
	enum
	{
		kNone			= 0,
		kReadCRC		= 1 << 0,
		kValidateCRC	= 1 << 1, // Can not validate if kReadCRC is not set

		kReadVersion	= 1 << 2,
		kDefaultVersion = 1 << 3, // Make the object the most current version
	};

//Setup
	inline tInput( int32u initSize = 0 );
	inline tInput( const tBuffer* buffer );
	inline tInput( const tInput& copy );
	inline tInput& operator=( const tInput& copy );
	inline ~tInput();

//Accessors
	// If replaceNewFlags is true, newFlags will overwrite the current flags, otherwise
	// newFlags will be ORed into the current flags	
	inline static void	SetDefaultFlags( iInputFlags newDefaultFlags );
	inline void			AddFlags( iInputFlags newFlags, bool replaceNewFlags = false );
	inline void			RemoveFlags( iInputFlags flagsToRemove );
	inline iInputFlags	GetCurFlags() const;
	inline bool			CheckFlags( iInputFlags flags ) const;

	inline const tBuffer* GetBuffer() const;	
	inline tBuffer* Close( bool cleanUpBuffer = false );

//Operations
	// A negative pos, will count from the end
	inline void SetGetPos( int32 pos );
	inline byte Get();
	inline void Read( byte* data, int32 size ); 
	inline void Rewind();

	//STL Input
	template< class tType >
	friend tInput& operator>>( tInput& input, std::basic_string< tType >& string );
	template< class tType >
	friend tInput& operator>>( tInput& input, std::vector< tType >& value );
	template< class tType >
	friend tInput& operator>>( tInput& input, std::list< tType >& value );
	template< class tType >
	friend tInput& operator>>( tInput& input, std::deque< tType >& value );
	template< class tIDType, class tValueType, class tCompareType >
	friend tInput& operator>>( tInput& input, std::map< tIDType, tValueType, tCompareType >& value );
	template< class tType >
	friend tInput& operator>>( tInput& input, tInterlockedScalar< tType >& value );

	inline tInput& operator>>( tInput& (*func)( tInput& input ) );
	inline tInput& operator>>( tBuffer& buffer );
	inline tInput& operator>>( std::string& string );
	inline tInput& operator>>( bool& value );
	inline tInput& operator>>( char& value );	
	inline tInput& operator>>( int8s& value );
	inline tInput& operator>>( int8u& value );
	inline tInput& operator>>( int16u& value );
	inline tInput& operator>>( int16s& value );
	inline tInput& operator>>( int32u& value );
	inline tInput& operator>>( int32s& value );
	inline tInput& operator>>( float& value );
	inline tInput& operator>>( double& value );
	inline tInput& operator>>( long double& value );

	friend std::istream& operator>>( std::istream& inputStream, tInput& input );

protected:
// Fields
	iInputFlags			fInputFlags;
	static iInputFlags	fDefaultInputFlags;

	bool fCleanUpBuffer;	
	iPos fGetPos;
	const tBuffer* fBuffer;
};

};

#define _INLINE_DataStoreInput
#include "DataStoreInput.cpp"
#undef _INLINE_DataStoreInput


#endif

