//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( DataStoreOutput_h )
#define DataStoreOutput_h

#include <BaseTypes.h>

#if !defined( USE_PCH )

#include <string>
#include <map>
#include <vector>
#include <list>
#include <deque>

#endif

#include "DataStoreBuffer.h"

namespace nDataStore
{

// CLASS PURPOSE: To store data in a Buffer
class tOutput
{
public:	
// Types
	typedef int32u iOutputFlags;
	enum 
	{
		kNone			= 0,
		kOutputVersion	= 1 << 0,
		kOutputCRC		= 1 << 1,
	};

// Setup
	inline tOutput( int32u initSize = 0 );
	inline tOutput( tBuffer* buffer );
	inline tOutput( const tOutput& copy );
	inline tOutput& operator=( const tOutput& copy );
	inline ~tOutput();

//Accessors
	// If replaceNewFlags is true, newFlags will overwrite the current flags, otherwise
	// newFlags will be ORed into the current flags	
	inline static void	SetDefaultFlags( iOutputFlags newDefaultFlags );
	inline void			AddFlags( iOutputFlags newFlags, bool replaceNewFlags = false );
	inline void			RemoveFlags( iOutputFlags flagsToRemove );
	inline iOutputFlags	GetCurFlags() const;
	inline bool			CheckFlags( iOutputFlags flags ) const;

	inline const tBuffer* GetBuffer() const;	
	inline tBuffer* Close( bool cleanUpBuffer = false );

//Operations
	inline void Put( byte data );
	inline void Write( const byte* data, int32 size ); 

	//STL Output operators
	template< class tType >
	friend tOutput& operator<<( tOutput& output, const std::basic_string< tType >& string );
	template< class tType >
	friend tOutput& operator<<( tOutput& output, const std::vector< tType >& value );
	template< class tType >
	friend tOutput& operator<<( tOutput& output, const std::list< tType >& value );
	template< class tType >
	friend tOutput& operator<<( tOutput& output, const std::deque< tType >& value );
	template< class tIDType, class tValueType, class tCompareType >
	friend tOutput& operator<<( tOutput& output, const std::map< tIDType, tValueType, tCompareType >& value );
	
	inline tOutput& operator<<( tOutput& (*func)( tOutput& input ) );
	inline tOutput& operator<<( const tBuffer& buffer );
	inline tOutput& operator<<( const std::string& string );
	inline tOutput& operator<<( const char* string );
	inline tOutput& operator<<( bool value );
	inline tOutput& operator<<( char value );
	inline tOutput& operator<<( int8s value );
	inline tOutput& operator<<( int8u value );
	inline tOutput& operator<<( int16u value );
	inline tOutput& operator<<( int16s value );
	inline tOutput& operator<<( int32u value );
	inline tOutput& operator<<( int32s value );
	inline tOutput& operator<<( float value );
	inline tOutput& operator<<( double value );
	inline tOutput& operator<<( long double value );

	friend std::ostream& operator<<( std::ostream& outputStream, const tOutput& output );

protected:
// Fields
	iOutputFlags		fOutputFlags;
	static iOutputFlags fDefaultOutputFlags;
	
	bool fCleanUpBuffer;	
	tBuffer* fBuffer;
};

};

#define _INLINE_DataStoreOutput
#include "DataStoreOutput.cpp"
#undef _INLINE_DataStoreOutput

#endif

