//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( Streamable_h )
#define Streamable_h

#include <BaseTypes.h>
#include <DataStore\DataStore.h>
#include <DataStore\DataStorePre.h>
#include <map>

class type_info;

namespace nDataStore
{

// CLASS PURPOSE: Base class for an object that can stream
class tStreamable
{
public:
// Types
	typedef int32u iObjectID;
	typedef int8u iObjectVersion;

	class tCRCFailed : public except::tBaseException
	{
	public:
		tCRCFailed( const std::string& toObject, const std::string& forObject, const char* file, int32 lineNumber );

		virtual void ReportException( const char* caughtDescription = NULL ) throw();	

	protected:
		std::string fToObject;
		std::string fForObject;
	};

// Setup
	tStreamable( iObjectVersion versionNumber = 0 );
	virtual ~tStreamable();

//Operations
	friend nDataStore::tInput& operator>>( nDataStore::tInput& input, tStreamable& object );
	friend nDataStore::tOutput& operator<<( nDataStore::tOutput& output, const tStreamable& object );

	virtual	void StreamOut( nDataStore::tOutput& output ) const = 0;
	virtual	void StreamIn( nDataStore::tInput& input ) = 0;

// Accesors
	iObjectID		GetObjectID() const;
	iObjectVersion	GetVersion() const;

public:
	class tObjectIDInfo
	{
	public:
		tObjectIDInfo();

		std::string fObjectName;
		int32u		fCRC;
	};
	typedef std::map< const type_info*, tObjectIDInfo > iObjectIDInfoLookup;
	
	static iObjectIDInfoLookup fObjectIDInfoLookup;

	mutable iObjectID	fObjectID;
	iObjectVersion 		fObjectVersion;
};

};



#endif

