//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( DatabaseObject_h )
#define DatabaseObject_h

#include <BaseTypes.h>
#include <DynamicAllocator.h>
#include <DataStore\DataStorePre.h>
#include <DataBase\DatabaseID.h>
#include <DataStore\Streamable.h>
#include <DataBase\Database.h>
#include <Thread\Optex.h>
#include <Thread\ThreadEvent.h>

namespace nDatabase
{

// CLASS PURPOSE: The base class for any object that will be stored
// in the database
class tDatabaseObject : public tAllocatedClass, public nDataStore::tStreamable
{
public:

//Types

// Predicates
	class tFindDatabaseID
	{
	public:
		tFindDatabaseID( const db::tID& id )
			: fID( id )
		{_func_start( nDatabase::tDatabaseObject::tFindDatabaseID::tFindDatabaseID );_func_end;}

		bool operator()( const tDatabaseObject& object )
		{
			_func_start( nDatabase::tDatabaseObject::tFindDatabaseID::operator () );
			if( object.GetID() == fID )
			{
				return true;
			}
			else
			{
				return false;
			}
			_func_end;
		}

	protected:
		db::tID fID;

	};

//Setup
	tDatabaseObject( iObjectVersion versionNumber = 0 );
	tDatabaseObject( const tDatabaseObject& copy );
	tDatabaseObject& operator=( const tDatabaseObject& copy );
	virtual ~tDatabaseObject();

	virtual	void StreamOut( nDataStore::tOutput& output ) const;
	virtual	void StreamIn( nDataStore::tInput& input );	

// STL Helper operators for use with sort() and unique() and find_if()
	virtual bool operator< ( const tDatabaseObject& obj );
	virtual bool operator== ( const tDatabaseObject& obj );

// SQL Operations
	virtual void SQLFillInsertStream( std::stringstream* SQL, bool lastOne = false );
	virtual void SQLFillUpdateStream( std::stringstream* SQL, bool lastOne = false );
	virtual const char* SQLTableName() const;
	virtual const char* SQLFieldNames() const;

//Operations
	virtual void MakeReadOnly();
	virtual void SetID( const tID& id );
	virtual void SetLockID( const tDatabase::iLockID id );

//Accessors
	virtual const tID&			GetID() const;
	virtual tDatabase::iLockID	GetLockID() const; 
	virtual bool				IsLocked() const;

//Persistence
	// These function control when the object is deleted
	void Kill(); // Marks the object as dead, but only deletes it when the ref count is 0
	bool IsDead(); // Returns whether the object is dead or not.
	void AddReference(); // Add a reference to the count, Object will not be deleted untill the ref count is 0
	void RemoveReference(); // Removes a reference

protected:
//Operations
	friend class tDatabase;
	virtual void AttachToDB( const tID& id );
	virtual void RemoveFromDB();
	virtual bool Lock( tDatabase::iLockID lockId );
	virtual void UnLock();
	virtual tThreadEvent* GetLockEvent();
	virtual void CheckForDeath(); // Check if we should be deleted

//Fields
	mutable tOptex fAccess;

	tID fID;
	bool fAlive;
	int32u fReferenceCount;
	
	tThreadEvent*		fLockEvent;
	tDatabase::iLockID	fLockID;
};

};
namespace db = nDatabase;


#endif


