//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( Query_h )
#define Query_h

#include <BaseTypes.h>
#include "Evaluator.h"

namespace nDatabase
{

// CLASS PURPOSE: Query is a unary predicate class then evaluates collection
// of unary evalulators to find out if the object should be in the list
template< class tClass >
class tQuery : public std::unary_function< tClass*, bool > 
{
public:
//Types
	typedef tEval<tClass> iEvaluator;
	typedef std::vector<iEvaluator*> iEvaluators;

	enum eOpperation
	{
		kAnd,
		kOr
	};

//Setup
	tQuery( eOpperation opperation = kAnd ); 
	virtual ~tQuery();

//Operations
	virtual void AddEval( iEvaluator* eval );
	virtual bool operator()( const tClass* value ) const;

protected:
//Fields	
	iEvaluators fEvaluators;
	eOpperation fOpperation;
};




// TEMPLATE FUNCTIONS
template< class tClass >
tQuery<tClass>::tQuery<tClass>( eOpperation opperation ) :
	fOpperation( opperation )
{_func_start( nDatabase::tQuery::tQuery );_func_end;}


template< class tClass >
tQuery<tClass>::~tQuery<tClass>()
{_func_start( nDatabase::tQuery::~tQuery );_func_end;}

template< class tClass >
void tQuery<tClass>::AddEval( iEvaluator* eval )
{
	_func_start( nDatabase::tQuery::AddEval );
	fEvaluators.push_back( eval );	
	_func_end;
}

template< class tClass >
bool tQuery<tClass>::operator()( const tClass* value ) const
{
	_func_start( nDatabase::tQuery::operator () );
	bool result = true;
	bool done = false;
	for( int i = 0; done == false && i < fEvaluators.size(); i++ )
	{
		result = (*fEvaluators[i])( value );
		
		switch( fOpperation )
		{
			case kAnd:
			{
				if( result == false )
				{
					done = true;
					break;
				}
			}
			break;

			case kOr:
			{
				if( result == true )
				{
					done = true;
					break;
				}
			}
			break;

			default:
			{
				done = true;
				Assert( false );
			}
			break;
		}
	}

	return result;
	_func_end;
}

};
namespace db = nDatabase;

#endif


