//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>
#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>
#include <DataStore\DataStore.h>

#include "DateAndTime.h"

tDateAndTime::tDateAndTime()
{
	_func_start( tDateAndTime::tDateAndTime );
	fTime = ::time( NULL );
	_func_end;
}

tDateAndTime::tDateAndTime( time_t time )
{
	_func_start( tDateAndTime::tDateAndTime );
	fTime = time;	
	_func_end;
}

tDateAndTime::tDateAndTime( int32 year, int32 month, int32 day, int32 hour, int32 min, int32 sec, eDaylightSavingMode daylightSaving )
{
	_func_start( tDateAndTime::tDateAndTime );
	tm time;
	time.tm_sec = sec;
	time.tm_min = min;
	time.tm_hour = hour;
	Assert( day >= 1 && day <= 31 );
	time.tm_mday = day;
	Assert( month >= 1 && month <= 12 );
	time.tm_mon = month - 1;        // tm_mon is 0 based
	Assert( year >= 1900 );
	time.tm_year = year - 1900;     // tm_year is 1900 based
	time.tm_isdst = ConvertDaylightSavingModeToTMValue( daylightSaving );

	fTime = ::mktime( &time );
	Assert( fTime != -1 );       // indicates an illegal input time
	_func_end;
}

const tDateAndTime& tDateAndTime::operator=( time_t timeSrc )
{
	_func_start( tDateAndTime::operator = );
	fTime = timeSrc;	

	return *this;
	_func_end;
}

tDateAndTime tDateAndTime::GetCurrentTime()
{
	_func_start( tDateAndTime::GetCurrentTime );
	return tDateAndTime( ::time( NULL ) );
	_func_end;
}

tm* tDateAndTime::GetGmtTm( tm* pTime ) const
{
	_func_start( tDateAndTime::GetGmtTm );
	if( pTime != NULL)
	{
		*pTime = *::gmtime(&fTime);
		return pTime;
	}
	else
	{
		return ::gmtime(&fTime);
	}
	_func_end;
}

tm* tDateAndTime::GetLocalTm( tm* pTime ) const
{
	_func_start( tDateAndTime::GetLocalTm );
	if( pTime != NULL)
	{
		tm* pTempTime = ::localtime(&fTime);
		if( pTempTime == NULL )
		{
			return NULL;    // indicates the fTime was not initialized!
		}

		*pTime = *pTempTime;
		return pTime;
	}
	else
	{
		return ::localtime(&fTime);
	}
	_func_end;
}

void tDateAndTime::SetToCurrentTime()
{
	_func_start( tDateAndTime::SetToCurrentTime );
	fTime = ::time( NULL );
	_func_end;
}

time_t tDateAndTime::GetTime() const
{ 
	_func_start( tDateAndTime::GetTime );
	return fTime; 
	_func_end;
}

int32 tDateAndTime::GetYear() const
{ 
	_func_start( tDateAndTime::GetYear );
	return (GetLocalTm()->tm_year) + 1900; 
	_func_end;
}

int32 tDateAndTime::GetMonth() const
{ 
	_func_start( tDateAndTime::GetMonth );
	return GetLocalTm()->tm_mon + 1;
	_func_end;
}

int32 tDateAndTime::GetDay() const
{ 
	_func_start( tDateAndTime::GetDay );
	return GetLocalTm()->tm_mday; 
	_func_end;
}

int32 tDateAndTime::GetHour() const
{ 
	_func_start( tDateAndTime::GetHour );
	return GetLocalTm()->tm_hour; 
	_func_end;
}

int32 tDateAndTime::GetMinute() const
{ 
	_func_start( tDateAndTime::GetMinute );
	return GetLocalTm()->tm_min; 
	_func_end;
}

int32 tDateAndTime::GetSecond() const
{ 
	_func_start( tDateAndTime::GetSecond );
	return GetLocalTm()->tm_sec; 
	_func_end;
}

int32 tDateAndTime::GetDayOfWeek() const
{ 
	_func_start( tDateAndTime::GetDayOfWeek );
	return GetLocalTm()->tm_wday + 1; 
	_func_end;
}


bool tDateAndTime::operator==( const tDateAndTime& time ) const
{
	_func_start( tDateAndTime::operator == );
	if( fTime == time.fTime )
	{
		return true;
	}
	else
	{
		return false;
	}
	_func_end;
}

bool tDateAndTime::operator<( const tDateAndTime& time ) const
{
	_func_start( tDateAndTime::operator < );
	if( fTime < time.fTime )
	{
		return true;
	}
	else
	{
		return false;
	}
	_func_end;
}

void tDateAndTime::StreamOut( nDataStore::tOutput& output ) const
{
	_func_start( tDateAndTime::StreamOut );
	output << fTime;
	_func_end;
}

void tDateAndTime::StreamIn( nDataStore::tInput& input )
{
	_func_start( tDateAndTime::StreamIn );
	input >> fTime;
	_func_end;
}

int tDateAndTime::ConvertDaylightSavingModeToTMValue( eDaylightSavingMode mode )
{
	_func_start( tDateAndTime::ConvertDaylightSavingModeToTMValue );
	switch( mode )
	{
		case kStandardTime:
			return 0;

		case kDaylightSavingTime:
			return 1;

		case kAutomatic:
			return -1;

		default:
			Assert( false );
			return -1;
	} 
	_func_end;
}

//ZDNF How do I know how to format this?
std::ostream& operator<<( std::ostream& output, const tDateAndTime& dateAndTime)
{
	_func_start( operator << );
	output << dateAndTime.GetMonth() << "/" << dateAndTime.GetDay() << "/" << dateAndTime.GetYear() << " " << dateAndTime.GetHour() << ":" << dateAndTime.GetMinute() << ":" << dateAndTime.GetSecond();
	
	return output;
	_func_end;
}
