//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( DateAndTime_h )
#define DateAndTime_h

#include <BaseTypes.h>
#include <DataStore\Streamable.h>

#if !defined( USE_PCH )

#include <time.h>

#endif

// CLASS PURPOSE: A interface class for date and time
class tDateAndTime : public nDataStore::tStreamable
{
public:
// Types
	enum eDaylightSavingMode
	{
		kStandardTime,
		kDaylightSavingTime,
		kAutomatic
	};

// Setup
	static tDateAndTime GetCurrentTime();

	//Default sets it to current data and time
	tDateAndTime();
	tDateAndTime( time_t time );
	tDateAndTime( int32 year, int32 month, int32 day, int32 hour, int32 min, int32 sec, eDaylightSavingMode daylightSaving = kAutomatic );

	const tDateAndTime& operator=( time_t timeSrc );

// Accesors
	tm* GetGmtTm( tm* pTime = NULL ) const;
	tm* GetLocalTm( tm* pTime = NULL ) const;

	time_t GetTime() const;
	int32 GetYear() const;
	int32 GetMonth() const;       // month of year (1 = Jan)
	int32 GetDay() const;         // day of month
	int32 GetHour() const;
	int32 GetMinute() const;
	int32 GetSecond() const;
	int32 GetDayOfWeek() const;   // 1=Sun, 2=Mon, ..., 7=Sat

// Operations
	void SetToCurrentTime();

	bool operator==( const tDateAndTime& time ) const;
	bool operator<( const tDateAndTime& time ) const;
	void StreamOut( nDataStore::tOutput& output ) const;
	void StreamIn( nDataStore::tInput& input );

	friend std::ostream& operator<<( std::ostream& output, const tDateAndTime& dateAndTime );

protected:
	int ConvertDaylightSavingModeToTMValue( eDaylightSavingMode mode );

	time_t fTime;
};

#endif

