//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>
#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>
#include <DateAndTime.h>
#include <Thread\ThreadState.h>

#if !defined( _WIN32 )
#error This is a Win32 dependent class
#endif

#if !defined( USE_PCH )

#define WIN32_LEAN_AND_MEAN  
#include <Windows.h>
#include <sstream>
#include <fstream>

#endif

#include "Log.h"

namespace nLog
{

tLog* tLog::fLog = NULL;
tCleanupGlobalPointer< tLog > tLog::fCleanupSingletion( &fLog );

tLog::tLog()
	: fLogHandle( NULL )
	, fLogOpen( false )
	, fOutputToDebug( false )
	, fOutputToConsole( false )
	, fOutputToEventLog( false )
	, fOutputToFile( false )
	, fErrorFileOutput( false )
	, fLoggingMutex( tThreadState::GetCurThreadState()->DefaultThreadObjectUse() )
{_func_start( nLog::tLog::tLog );_func_end;}

void tLog::SetupFileOutput( bool turnOn, const std::string& fileName )
{
	_func_start( nLog::tLog::SetupFileOutput );
	tDateAndTime curTime;
	fFileName = fileName;
	std::ofstream logfile( fFileName.c_str(), std::ios_base::out | std::ios_base::app );
	if( logfile.good() )
	{
		logfile << "---------------------------------------- Start Log " << curTime.GetMonth() << "/" << curTime.GetDay() << "/" << curTime.GetYear() << " - " << curTime.GetHour() << ":" << curTime.GetMinute() << ":" << curTime.GetSecond() << " ----------------------------------------" << std::endl << std::endl;
		logfile.flush();
		fOutputToFile = turnOn;
	}
	_func_end;
}

void tLog::SetupDebugOutput( bool turnOn )
{
	_func_start( nLog::tLog::SetupDebugOutput );
	fOutputToDebug = turnOn;
	_func_end;
}

void tLog::SetupConsoleOutput( bool turnOn )
{
	_func_start( nLog::tLog::SetupConsoleOutput );
	fOutputToConsole = turnOn;
	_func_end;
}

void tLog::SetupEventLog( bool turnOn, const std::string& serverName, const std::string source )
{							   
	_func_start( nLog::tLog::SetupEventLog );
	if( fOutputToEventLog == turnOn )
	{
		return;
	}
	
	fOutputToEventLog = turnOn;
	
	if( fOutputToEventLog )
	{
		const char* pServerName = NULL;
		if( serverName.size() )
		{
			pServerName = serverName.c_str();
		}
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )		
		//ZDTEST@@@@
		tResourceTracker::Instance()->UsingHandle( tResourceTracker::k_tLog_SetupEventLog );
#endif
		fLogHandle = ::RegisterEventSource( pServerName, source.c_str() );

		if( fLogHandle )
		{
			fLogOpen = true;
		}
	}
	else
	{
		if( fLogHandle )
		{
			::DeregisterEventSource( fLogHandle );
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
			//ZDTEST@@@@
			tResourceTracker::Instance()->ReleaseHandle( tResourceTracker::k_tLog_SetupEventLog );
#endif
		}
	}
	_func_end;
}

void tLog::InstallEventLog( const std::string& sourceName, const std::string& messageDll, bool uninstall )
{
	_func_start( nLog::tLog::InstallEventLog );
	if( uninstall )
	{
		//ZD_EXCEPT@@@@
		throw;
	}	
    
	HKEY keyHandle; 
 
    // Add your source name as a subkey under the Application 
    // key in the EventLog registry key. 
	std::stringstream sourceKey;
	sourceKey << "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\" << sourceName;
 
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )    
	//ZDTEST@@@@
	tResourceTracker::Instance()->UsingHandle( tResourceTracker::k_tLog_InstallEventLog );
#endif
	if( ::RegCreateKey( HKEY_LOCAL_MACHINE, sourceKey.str().c_str(), &keyHandle ) ) 
	{
        //ZD_EXCEPT@@@@
		throw;
	}

	// Set the supported event types in the TypesSupported subkey. 
    int32u dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE; 
 
    if(	::RegSetValueEx(	keyHandle,			// subkey handle 
							"TypesSupported",	// value name 
							0,					// must be zero 
							REG_DWORD,			// value type 
							(int8u*) &dwData,	// pointer to value data 
							sizeof(dwData)		// length of value data 
						) )
	{
		//ZD_EXCEPT@@@@
		throw;
	}
 
	if( ! messageDll.empty() )
	{
		char curDirectory[_MAX_DIR];
		::GetCurrentDirectory( _MAX_DIR, curDirectory );
		// Set the name of the message file. 
		std::stringstream buildMessageDllKey;
		buildMessageDllKey << curDirectory << "\\" << messageDll;
		std::string messageDllKey = buildMessageDllKey.str();
 
		// Add the name to the EventMessageFile subkey. 
		if( ::RegSetValueEx(	keyHandle,						// subkey handle 
								"EventMessageFile",				// value name 
								0,								//must be zero 
								REG_EXPAND_SZ,					// value type 
								(int8u*) messageDllKey.c_str(),	// pointer to value data 
								messageDllKey.size() + 1 		// length of value data 
							) )
		{
			//ZD_EXCEPT@@@@
			throw;
		}
	}
 
    ::RegCloseKey( keyHandle ); 
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
	//ZDTEST@@@@
	tResourceTracker::Instance()->ReleaseHandle( tResourceTracker::k_tLog_InstallEventLog );
#endif
	_func_end;
}



tLog::~tLog()
{
	_func_start( nLog::tLog::~tLog );
	SetupEventLog( false );	
	SetupDebugOutput( false );
	SetupConsoleOutput( false );
	_func_end;
}

tLog* tLog::Instance()
{
	_func_start( nLog::tLog::Instance );
	if( fLog == NULL )
	{
		fLog = new tLog();
	}
	
	return fLog;
	_func_end;
}

tLog::tLog( const tLog& copy )
{
	_func_start( nLog::tLog::tLog );
	throw except::tInvalidAccess( "tLog::tLog", __FILE__, __LINE__ );
	_func_end;
}

tLog& tLog::operator=( const tLog& copy )
{
	_func_start( nLog::tLog::operator = );
	throw except::tInvalidAccess( "tLog::operator=", __FILE__, __LINE__ );

	return *this;
	_func_end;
}


void tLog::LogReport(	tEventType eventType, 
						const std::string& description, 
						int16u category, 
						int32u eventID, 
						nDataStore::tBuffer* rawData )
{
	_func_start( nLog::tLog::LogReport );
	tOptex::tGuard guard( &fLoggingMutex );

	if( fOutputToFile || (fErrorFileOutput && eventType == kErrorEvent ) )
	{
		LogToFile( eventType, description, category, eventID, rawData );	
	}

	if( fOutputToEventLog )
	{
		LogToEventLog( eventType, description, category, eventID, rawData );
	}

	if( fOutputToConsole )
	{
		LogToConsole( eventType, description, category, eventID, rawData );
	}

	if( fOutputToDebug )
	{
		LogToDebugOut( eventType, description, category, eventID, rawData );
	}

	_func_end;
}

void tLog::LogToFile(	tEventType eventType, 
						const std::string& description, 
						int16u category, 
						int32u eventID, 
						nDataStore::tBuffer* rawData )
{
	_func_start( nLog::tLog::LogToFile );
	std::ofstream logfile( fFileName.c_str(), std::ios_base::out | std::ios_base::app );

	if( logfile.good() )
	{
		logfile << GetEventTypeString( eventType ) << ": " << description << std::endl;
		logfile.flush();
	}
	_func_end;
}

void tLog::LogToEventLog(	tEventType eventType, 
							const std::string& description, 
							int16u category, 
							int32u eventID, 
							nDataStore::tBuffer* rawData )
{
	_func_start( nLog::tLog::LogToEventLog );
	if( !fLogOpen )
	{
		return;
	}
	
	int16u windowsEventType;
	switch( eventType )
	{
		case kErrorEvent:
			windowsEventType = EVENTLOG_ERROR_TYPE;	
			break;

		case kWarningEvent:
			windowsEventType = EVENTLOG_WARNING_TYPE;
			break;

		case kInformationEvent:
			windowsEventType = EVENTLOG_INFORMATION_TYPE;
			break;

		case kDebugEvent:
			windowsEventType = EVENTLOG_INFORMATION_TYPE;
			break;


		case kAuditSuccess:
			windowsEventType = EVENTLOG_AUDIT_SUCCESS;
			break;

		case kAuditFailure:
			windowsEventType = EVENTLOG_AUDIT_FAILURE;	
			break;

		default:
			Assert( false );
	};

	const char* pDescription[1];
	pDescription[0] = description.c_str();

	int32u result = ::ReportEvent( fLogHandle, windowsEventType, category, eventID, NULL, 1, 0, pDescription, NULL );
	_func_end;
}

void tLog::LogToConsole(	tEventType eventType, 
							const std::string& description, 
							int16u category, 
							int32u eventID, 
							nDataStore::tBuffer* rawData )
{
	_func_start( nLog::tLog::LogToConsole );
	
	
	std::cout << GetEventTypeString( eventType ) << ": " << description << std::endl;
	_func_end;
}

void tLog::LogToDebugOut(	tEventType eventType, 
							const std::string& description, 
							int16u category, 
							int32u eventID, 
							nDataStore::tBuffer* rawData )
{
	_func_start( nLog::tLog::LogToDebugOut );
	std::stringstream logMsg;
	logMsg << GetEventTypeString( eventType ) << ": " << description << std::endl;

	::OutputDebugString( logMsg.str().c_str() ); 
	_func_end;
}

std::string tLog::GetEventTypeString( tEventType eventType )
{
	_func_start( nLog::tLog::GetEventTypeString );

	switch( eventType )
	{
		case kErrorEvent:
			return "Error!";

		case kWarningEvent:
			return "Warning";
			break;

		case kInformationEvent:
			return "Info";
			break;

		case kDebugEvent:
			return "Debug";
			break;

		case kAuditSuccess:
			return "Audit Success";
			break;

		case kAuditFailure:
			return "Audit Failed";
			break;

		default:
			Assert( false );
			return "";
	}
	_func_end;
}

};

