//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( EventLog_h )
#define EventLog_h

#include <BaseTypes.h>
#include "CleanUpGlobalPointer.h"
#include <DataStore\DataStoreBuffer.h>

#if !defined( USE_PCH )

#include <string>
#include <iostream>

#endif


#include <Log\MessageDLL\Messages.h>
#include <Thread\Optex.h>

namespace nLog
{

// CLASS PURPOSE: To log import events to a central repository
class tLog
{
public:
//Types
	enum tEventType
	{
		kNoEventType,
		kErrorEvent,
		kWarningEvent,
		kInformationEvent,
		kDebugEvent,
		kAuditSuccess,
		kAuditFailure,
	};

//Setup
	static tLog* Instance();
	void SetupFileOutput( bool turnOn, const std::string& fileName );
	void SetupDebugOutput( bool turnOn );
	void SetupConsoleOutput( bool turnOn );
	void SetupEventLog( bool turnOn, const std::string& serverName = "", const std::string source = "" );

// Install
	void InstallEventLog( const std::string& sourceName, const std::string& messageDll = "", bool uninstall = false );

// Operations
	void LogReport(	tEventType eventType, 
					const std::string& description, 
					int16u category = 0, 
					int32u eventID = kMsgDebug, 
					nDataStore::tBuffer* rawData = NULL );

protected:

// Operations
	void LogToFile( tEventType eventType, 
					const std::string& description, 
					int16u category, 
					int32u eventID, 
					nDataStore::tBuffer* rawData );

	void LogToEventLog( tEventType eventType, 
						const std::string& description, 
						int16u category, 
						int32u eventID, 
						nDataStore::tBuffer* rawData );
	
	void LogToConsole( tEventType eventType, 
						const std::string& description, 
						int16u category, 
						int32u eventID, 
						nDataStore::tBuffer* rawData );

	void LogToDebugOut( tEventType eventType, 
						const std::string& description, 
						int16u category, 
						int32u eventID, 
						nDataStore::tBuffer* rawData );

	std::string GetEventTypeString( tEventType eventType );

// Fields
	mutable tOptex fLoggingMutex;

	//Event log info	
	bool fOutputToEventLog;
	bool fLogOpen;
	void* fLogHandle;
	
	//Console info
	bool fOutputToConsole;

	// File log
	std::string fFileName;
	bool fOutputToFile;
	bool fErrorFileOutput;


	//Debug info
	bool fOutputToDebug;

private:
//Hidden
	friend tCleanupGlobalPointer< tLog >;
	tLog();
	~tLog();
	tLog( const tLog& copy );
	tLog& operator=( const tLog& copy );

	static tLog* fLog;
	static tCleanupGlobalPointer< tLog > fCleanupSingletion;

};

};

namespace logmsg = nLog;

#endif

