//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>
#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>
#include <Thread\ThreadState.h>

#include "LogStream.h"

namespace nLog
{

tLogStream out;
tLogStream warning( tLog::kWarningEvent );
tLogStream error( tLog::kErrorEvent );
tLogStream info( tLog::kInformationEvent );
tLogStream debug( tLog::kDebugEvent );

tLogStream::tStreamBuf::tStreamBuf( tLog* eventLog )
	: fLog( eventLog )
	, fEventType( tLog::kErrorEvent )
	, fLockType( false )
	, fAccess( tThreadState::GetCurThreadState()->DefaultThreadObjectUse() )
{_func_start( nLog::tLogStream::tStreamBuf::tStreamBuf );_func_end;}

tLogStream::tStreamBuf::~tStreamBuf()
{_func_start( nLog::tLogStream::tStreamBuf::~tStreamBuf );_func_end;}

std::streamsize tLogStream::tStreamBuf::xsputn( const char* text, std::streamsize n )
{
	_func_start( nLog::tLogStream::tStreamBuf::xsputn );
	LockAccess();

	if( !n )
	{
		return 0;
	}
	
	
	//If the text starts and ends with a '%' then it is a command
	if( text[0] == '%' && text[n-1] == '%' )
	{
		fCommand.assign( text, n );
		RunCommand();
	}
	else
	{
		fEventBuffer.append( text, n );
	}

	return n;	
	_func_end;
}

int tLogStream::tStreamBuf::overflow(int ch)
{
	_func_start( nLog::tLogStream::tStreamBuf::overflow );
	LockAccess();

	return xsputn( (char*)&ch, 1 );
	_func_end;
}

std::streamsize tLogStream::tStreamBuf::xsgetn( char* text, std::streamsize n )
{
	_func_start( nLog::tLogStream::tStreamBuf::xsgetn );
	return 0;
	_func_end;
}

int tLogStream::tStreamBuf::underflow()
{
	_func_start( nLog::tLogStream::tStreamBuf::underflow );
	return 0;
	_func_end;
}

int tLogStream::tStreamBuf::uflow()
{
	_func_start( nLog::tLogStream::tStreamBuf::uflow );
	return 0;
	_func_end;
}

int tLogStream::tStreamBuf::sync()
{
	_func_start( nLog::tLogStream::tStreamBuf::sync );
	return 0;
	_func_end;
}

void tLogStream::tStreamBuf::Report()
{
	_func_start( nLog::tLogStream::tStreamBuf::Report );
	LockAccess();

	fLog->LogReport( fEventType, fEventBuffer.c_str(), 0 );
	fEventBuffer.erase();

	// The data has been reported, time to release access to the buffer.
	LockAccess( false );
	_func_end;
}

// We want to make sure there is only one thread reporting events at a time.
// So we lock the stream buf while it is recving data. Once it is told
// to report the data, access is released, giving other threads a chance to 
// log data
void tLogStream::tStreamBuf::LockAccess( bool lock )
{
	_func_start( nLog::tLogStream::tStreamBuf::LockAccess );
	if( lock )
	{
		//If we don't own the optex, try to take ownership
		if( fAccess.CurThreadOwnsOptex() == false )
		{
			fAccess.Enter();
		}
	}
	else
	{
		fAccess.Leave();		
	}
	_func_end;
}

void tLogStream::tStreamBuf::RunCommand()
{
	_func_start( nLog::tLogStream::tStreamBuf::RunCommand );
	if( fCommand == "%%" )
	{
		Report();
	}
	else if( !fLockType )
	{
		if( fCommand == "%warning%" )
		{
			fEventType = tLog::kWarningEvent;
		}
		else if( fCommand == "%error%" )
		{
			fEventType = tLog::kErrorEvent;
		}
		else if( fCommand == "%info%" )
		{
			fEventType = tLog::kInformationEvent;
		}
		else if( fCommand == "%audit_success%" )
		{
			fEventType = tLog::kAuditSuccess;
		}
		else if( fCommand == "%audit_fail%" )
		{
			fEventType = tLog::kAuditFailure;
		}
	}
	_func_end;
}

void tLogStream::tStreamBuf::SetEventType( tLog::tEventType eventType )
{
	_func_start( nLog::tLogStream::tStreamBuf::SetEventType );
	fEventType = eventType;
	
	if( fEventType != tLog::kNoEventType )
	{
		fLockType = true;	
	}
	else
	{
		fLockType = false;	
	}
	_func_end;
}

using std::ostream;

tLogStream::tLogStream( tLog::tEventType eventType ) :	
	ostream( 0 ),
	fStreamBuf( tLog::Instance() )
{
	_func_start( nLog::tLogStream::tLogStream );
	fStreamBuf.SetEventType( eventType );
	init( &fStreamBuf );
	_func_end;
}

tLogStream& tLogStream::operator()( const char* category, int8u level )
{
	_func_start( nLog::tLogStream::operator () );
	return *this;
	_func_end;
}

//Event log stream manipulators
std::ostream& report( std::ostream& stream )
{
	_func_start( nLog::report );
	return stream << "%%";
	_func_end;
}

std::ostream& set_warning( std::ostream& stream )
{
	_func_start( nLog::set_warning );
	return stream << "%warning%";
	_func_end;
}

std::ostream& set_error( std::ostream& stream )
{
	_func_start( nLog::set_error );
	return stream << "%error%";
	_func_end;
}

std::ostream& set_info( std::ostream& stream )
{
	_func_start( nLog::set_info );
	return stream << "%info%";
	_func_end;
}

std::ostream& set_audit_success( std::ostream& stream )
{
	_func_start( nLog::set_audit_success );
	return stream << "%audit_success%";
	_func_end;
}

std::ostream& set_audit_fail( std::ostream& stream )
{
	_func_start( nLog::set_audit_fail );
	return stream << "%audit_fail%";
	_func_end;
}

};
