//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( EventLogStream_h )
#define EventLogStream_h

#include <BaseTypes.h>

#if !defined( USE_PCH )

#include <ostream>
#include <string>
#include <streambuf>

#endif

#include "Log.h"
#include <Thread\Optex.h>

namespace nLog
{

// CLASS PURPOSE: Used to log events using a stream
class tLogStream : public std::ostream
{
public:
//Setup
	tLogStream( tLog::tEventType eventType = tLog::kNoEventType );

	tLogStream& operator()( const char* category, int8u level = 10 );

protected:
//Types
	class tStreamBuf : public std::streambuf
	{
	public:
		tStreamBuf( tLog* eventLog );
		~tStreamBuf();

		virtual std::streamsize xsputn(const char* text, std::streamsize n);
		virtual int overflow(int ch);
		virtual std::streamsize xsgetn(char* text, std::streamsize n);
		virtual int underflow();
		virtual int uflow();
		virtual int sync();

		
		virtual void Report();
		virtual void LockAccess( bool lock = true );
		virtual void RunCommand();
		virtual void SetEventType( tLog::tEventType eventType );

	protected:
		tOptex fAccess;
		std::string fEventBuffer;
		std::string fCommand;
		tLog::tEventType fEventType;
		tLog* fLog;
		bool fLockType;
	};

// Fields
	tStreamBuf fStreamBuf;
};


// Event log stream manipulators
// WARNING!! You must stream in report when you are done with your msg!!!!!!!
// If you don't the message stream will be locked forever!!!!!
std::ostream& __cdecl report( std::ostream& stream );
std::ostream& __cdecl set_warning( std::ostream& stream );
std::ostream& __cdecl set_error( std::ostream& stream );
std::ostream& __cdecl set_info( std::ostream& stream );
std::ostream& __cdecl set_audit_success( std::ostream& stream );
std::ostream& __cdecl set_audit_fail( std::ostream& stream );

extern tLogStream out;
extern tLogStream warning;
extern tLogStream error;
extern tLogStream info;
extern tLogStream debug;
};

namespace logmsg = nLog;
#endif

