/************************************************************************************************************
 *
 *	Module:
 *		OSVersion.cpp
 *
 *	Copyright (c) 1996-2000 Kenneth Yeast
 *	Licensed to Taldren, Inc. 9/12/2000
 *
 *	Synopsis:
 *		Simple query of Operating System and version/revision.
 *
 *	Creation:
 *		0.00 NOV-02-96	Kenneth Yeast
 *
 *	Quote:
 *		
 *
 *	Comments:
 *		Modified to fit into Taldren's naming conventions
 *
 *	Public Functions:
 *		
 *
 *	Private Functions:
 *		
 *
 *	Unresolved:
 *		
 *
 *	Modifications:
 *		$Log:  $
 * 
 *
 ************************************************************************************************************/


// System includes

// Outside Package includes
// Project includes
#include <BaseTypes.h>
#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>
#include <OSVersion.h>


//Special OS Specific calls (loads appropriate system DLL at runtime)
namespace OSSPECIFIC
{
	LONG (*OS_InterlockedCompareExchange)( LPLONG Destination, LONG Exchange, LONG Comperand );
}

tOSVersion::iAutoOSVersionPtr tOSVersion::fOSVersion;
OSVERSIONINFO tOSVersion::fInfo;
HINSTANCE tOSVersion::fOSDLL;


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

tOSVersion* tOSVersion::Instance()
{
	_func_start( tOSVersion::Instance );
	if( fOSVersion.get() == NULL )
	{
		fOSVersion = iAutoOSVersionPtr( new tOSVersion );
	} 

	return fOSVersion.get();
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

tOSVersion::tOSVersion()
{
	_func_start( tOSVersion::tOSVersion );
	// Find out our version
	fInfo.dwOSVersionInfoSize = sizeof( OSVERSIONINFO );
	GetVersionEx( &fInfo );

	// Load OS Specific DLL
	// KMY: CRUDE at the moment...
	OSSPECIFIC::OS_InterlockedCompareExchange = (LONG (*)( LPLONG, LONG, LONG )) NULL;
	fOSDLL = LoadLibrary( IsWindows95() ? "OSSpecific95" : "OSSpecific98" );
	if ( fOSDLL )
	{
		// Set all of our "links"
		OSSPECIFIC::OS_InterlockedCompareExchange = (LONG (*)( LPLONG, LONG, LONG )) GetProcAddress( fOSDLL, "OS_InterlockedCompareExchange" );
	}

	if ( OSSPECIFIC::OS_InterlockedCompareExchange == NULL )
	{
		// If we fail, we'll know on first usage
		OSSPECIFIC::OS_InterlockedCompareExchange = (LONG (*)( LPLONG, LONG, LONG )) _OS_InterlockedCompareExchange;
		FreeLibrary( fOSDLL ), fOSDLL = 0;
	}
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

tOSVersion::~tOSVersion()
{
	_func_start( tOSVersion::~tOSVersion );
	// KMY: Because the DLL referenced call herein is used in a *lot* of destructors, I'm hesitant to place
	// FreeLibrary() here.  Perhaps later we can fix this?  Would a smart ptr work as a global?
//	if ( fOSDLL )
//		FreeLibrary( fOSDLL ), fOSDLL = 0;
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::Init()
{
	_func_start( tOSVersion::Init );
	return ( fOSDLL != 0 );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::IsWin32s()
{
	_func_start( tOSVersion::IsWin32s );
	return ( fInfo.dwPlatformId == VER_PLATFORM_WIN32s );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::IsWindows95()
{
	_func_start( tOSVersion::IsWindows95 );
	return ( ( fInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS ) && ( fInfo.dwMinorVersion == 0 ) );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::IsWindows98()
{
	_func_start( tOSVersion::IsWindows98 );
	return ( ( fInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS ) && ( fInfo.dwMinorVersion > 0 ) );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::IsWindowsNT()
{
	_func_start( tOSVersion::IsWindowsNT );
	return ( fInfo.dwPlatformId == VER_PLATFORM_WIN32_NT );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::IsWindowsNT4()
{
	_func_start( tOSVersion::IsWindowsNT4 );
	return ( IsWindowsNT() && ( fInfo.dwMajorVersion == 4 ) );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::IsWindowsNT5()
{
	_func_start( tOSVersion::IsWindowsNT5 );
	return ( IsWindowsNT() && ( fInfo.dwMajorVersion == 5 ) );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

bool tOSVersion::IsWindows2000()
{
	_func_start( tOSVersion::IsWindows2000 );
	return IsWindowsNT5();
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

const DWORD tOSVersion::GetVersion()
{
	_func_start( tOSVersion::GetVersion );
	return fInfo.dwMajorVersion;
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

const DWORD tOSVersion::GetRevision()
{
	_func_start( tOSVersion::GetRevision );
	return fInfo.dwMinorVersion;
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

const DWORD tOSVersion::GetBuild()
{
	_func_start( tOSVersion::GetBuild );
	return ( fInfo.dwBuildNumber & 0xffff );
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

const char* tOSVersion::GetMiscInfo()
{
	_func_start( tOSVersion::GetMiscInfo );
	return fInfo.szCSDVersion;
	_func_end;
}


/************************************************************************************************************
 *
 *	
 *
 *
 *
 ************************************************************************************************************/

LONG tOSVersion::_OS_InterlockedCompareExchange( LPLONG Destination, LONG Exchange, LONG Comperand )
{
	_func_start( tOSVersion::_OS_InterlockedCompareExchange );
/*
	LONG result;

	EnterCriticalSection( tOSVersion::Instance()->GetCriticalSectionPtr() );

	if ( *Destination == Comperand )
	{
		*Destination = Exchange;
	}

	result = *Destination;

	LeaveCriticalSection( tOSVersion::Instance()->GetCriticalSectionPtr() );
*/
	// !!!!!!!!!! 
	// You need to do this call: "tOSVersion::Instance()->Init()" in your app, and have the OSSpecificXX.dll's 
	// available in order to use this.

	// KMY: Hack for now.  To be correct, I guess we must implement a critical section and attempt to
	// actually create a InterlockedCompareExchange function...  *sigh*.
	// Which means that we could change the Win95 version to support this function and rip out the
	// OS Specific references within Optex.cpp.
	throw except::tInvalidAccess( "tOSVersion::_OS_InterlockedCompareExchange", __FILE__, __LINE__ );

	return 0;
	_func_end;
}

