/************************************************************************************************************
 *
 *	Module:
 *		OSVersion.h
 *
 *	Copyright (c) 1996-2000 Kenneth Yeast
 *	Licensed to Taldren, Inc. 9/12/2000
 *
 *	Synopsis:
 *		Simple query of Operating System and version/revision.
 *
 *	Creation:
 *		0.00 NOV-02-96	Kenneth Yeast
 *
 *	Quote:
 *
 *
 *	Comments:
 *		Modified to fit into Taldren's naming conventions
 *
 *	Public Functions:
 *
 *
 *	Private Functions:
 *
 *
 *	Unresolved:
 *
 *
 *	Modifications:
 *		$Log:  $
 * 
 *
 ************************************************************************************************************/

#if	! defined( UTILITY_OSVERSION_H )
#define	UTILITY_OSVERSION_H


#include <BaseTypes.h>

#if !defined( USE_PCH )
	#include <windows.h>
	// STL
	#include <iostream>
	#include <string>
	#include <fstream>
	#include <sstream>
	#include <Utility>
	#include <map>
	#include <list>
	#include <vector>
	#include <limits>
	#include <memory>
	#include <set>
	#include <deque>
	#include <algorithm>  
#endif


//Special OS Specific calls (loads appropriate system DLL at runtime)
namespace OSSPECIFIC
{
	extern LONG (*OS_InterlockedCompareExchange)( LPLONG Destination, LONG Exchange, LONG Comperand );
}


class tOSVersion
{
public:
//Setup
	static tOSVersion* Instance();
	bool Init();

//Queries
	bool IsWin32s();					// Win32s is under Windows 3.1 and is here for entertainment purposes only.
	bool IsWindows95();
	bool IsWindows98();
	bool IsWindowsNT();
	bool IsWindowsNT4();
	bool IsWindowsNT5();
	bool IsWindows2000();
	
//Accessors
	const DWORD GetVersion();
	const DWORD GetRevision();
	const DWORD GetBuild();				// Only in lower word (upper is masked - HAD contained version/revision)
	const char* GetMiscInfo();

protected:
// Fields

private:
//Hidden
	typedef std::auto_ptr<tOSVersion> iAutoOSVersionPtr;
	friend iAutoOSVersionPtr;

	tOSVersion();
	~tOSVersion();
	tOSVersion( const tOSVersion& copy );
	tOSVersion& operator=( const tOSVersion& copy );

	static iAutoOSVersionPtr fOSVersion;

	static OSVERSIONINFO fInfo;

	static HINSTANCE fOSDLL;
	static LONG _OS_InterlockedCompareExchange( LPLONG Destination, LONG Exchange, LONG Comperand );
};


#endif
