//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.


#if !defined( SmartPtr_h )
#define SmartPtr_h


#include <BaseTypes.h>


////////////////////////////////////////////////////////////////////////////////
// tRefCount
// 
////////////////////////////////////////////////////////////////////////////////


class tRefCount
{
public:
// Setup
	tRefCount();

// Operations
	void AddRef() const;
	void Release();

// Accessors	
	int32u GetRefCount();

protected: 
	virtual ~tRefCount();

private:
	mutable int32 fRefCount;
};



////////////////////////////////////////////////////////////////////////////////
// tConstSmartPtr
// 
////////////////////////////////////////////////////////////////////////////////


template< typename tnPtr > 
class tConstSmartPtr
{
public:
// Setup
	tConstSmartPtr();
	tConstSmartPtr( const tnPtr* pObject );
	tConstSmartPtr( const tConstSmartPtr& copy );
	tConstSmartPtr& operator=( const tConstSmartPtr& theCopy );

	~tConstSmartPtr();

// Operations	
	tnPtr* operator=( const tnPtr* pPtr );

// Accesors
	bool IsValid() const;

	// Suply the same accessors that a pointer would
	const tnPtr* operator->() const;
	operator const tnPtr*() const;

	bool operator==( const tnPtr* pPtr) const;
	bool operator!=( const tnPtr* pPtr) const;
	bool operator<( const tnPtr* pPtr) const;
	
	const tnPtr* Get() const;

protected:
	tnPtr* fObject;

};

////////////////////////////////////////////////////////////////////////////////
// tSmartPtr
// 
////////////////////////////////////////////////////////////////////////////////


template< typename tnPtr > 
class tSmartPtr : public tConstSmartPtr< tnPtr >
{
public:
// Setup
	tSmartPtr();
	tSmartPtr( tnPtr* pObject );
	tSmartPtr( const tSmartPtr& copy );
	
// Operations	
	tSmartPtr& operator=( const tSmartPtr& copy );
	tnPtr* operator=( tnPtr* pPtr );

// Accesors
	// Suply the same accessors that a pointer would
	tnPtr* operator->() const;
	operator tnPtr*() const;

	tnPtr* Get() const;
};



/// TEMPLATE FUNCTION DECALRED /////////////////////////////////////////////////
/// Source 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Template functions for SmartPtr
// 
////////////////////////////////////////////////////////////////////////////////

template <class tnPtr > 
tConstSmartPtr<tnPtr>::tConstSmartPtr<tnPtr>() : 
	fObject( NULL ) 
{_func_start( tConstSmartPtr::tConstSmartPtr );_func_end;}

template <class tnPtr > 
tConstSmartPtr<tnPtr>::tConstSmartPtr<tnPtr>( const tnPtr* pObject ) : 
	fObject( const_cast< tnPtr* >( pObject ) ) 
{
	_func_start( tConstSmartPtr::tConstSmartPtr );
	if( fObject ) 
	{
		fObject->AddRef();	
	}
	_func_end;
} 

template <class tnPtr > 
tConstSmartPtr<tnPtr>::tConstSmartPtr<tnPtr>( const tConstSmartPtr<tnPtr>& copy ) : 
	fObject( const_cast< tnPtr* >( copy.Get() ) )
{	
	_func_start( tConstSmartPtr::tConstSmartPtr );
	if( fObject )
	{
		fObject->AddRef();
	}
	_func_end;
}

template <class tnPtr > 
tConstSmartPtr<tnPtr>& tConstSmartPtr<tnPtr>::operator=( const tConstSmartPtr<tnPtr>& copy )
{
	_func_start( tConstSmartPtr::operator = );
	operator=( copy.Get() );

	return *this;
	_func_end;
}

template <class tnPtr > 
tConstSmartPtr<tnPtr>::~tConstSmartPtr<tnPtr>() 
{ 
	_func_start( tConstSmartPtr::~tConstSmartPtr );
	// Release the reference
	if( fObject != NULL ) 
	{
		fObject->Release(); 
	}
	_func_end;
}

template <class tnPtr > 
tnPtr* tConstSmartPtr<tnPtr>::operator=( const tnPtr* pPtrCopy ) 
{ 
	_func_start( tConstSmartPtr::operator = );
	if( fObject != pPtrCopy ) 
	{
		if( fObject != NULL ) 
		{
			fObject->Release(); 
		}

		if( pPtrCopy )
		{
			fObject = const_cast<tnPtr*>( pPtrCopy );
			fObject->AddRef();	
		}
		else
		{
			fObject = NULL;	
		}
	}

	return fObject; 
	_func_end;
}

template <class tnPtr > 
bool tConstSmartPtr<tnPtr>::IsValid() const
{
	_func_start( tConstSmartPtr::IsValid );
	if( fObject )
	{
		return true;
	}
	else
	{
		return false;
	}
	_func_end;
}

template <class tnPtr > 
const tnPtr* tConstSmartPtr<tnPtr>::operator->() const 
{ 
	_func_start( tConstSmartPtr::operator -> );
	return fObject; 
	_func_end;
}

template <class tnPtr > 
tConstSmartPtr<tnPtr>::operator const tnPtr*() const 
{ 
	_func_start( tConstSmartPtr::operator const tnPtr* );
	return fObject; 
	_func_end;
}

template <class tnPtr > 
bool tConstSmartPtr<tnPtr>::operator==( const tnPtr* pPtr ) const 
{ 
	_func_start( tConstSmartPtr::operator == );
	if( fObject == pPtr )
	{
		return true;
	}
	else
	{
		return false;
	}
	_func_end;
}

template <class tnPtr > 
bool tConstSmartPtr<tnPtr>::operator!=( const tnPtr* pPtr ) const 
{ 
	_func_start( tConstSmartPtr::operator != );
	if( fObject != pPtr )
	{
		return true;
	} 
	else
	{
		return false;
	}
	_func_end;
}

template <class tnPtr > 
bool tConstSmartPtr<tnPtr>::operator<( const tnPtr* pPtr ) const 
{ 
	_func_start( tConstSmartPtr::operator < );
	if( fObject < pPtr )
	{
		return true;
	}
	else
	{
		return false;
	}
	_func_end;
} 


template <class tnPtr > 
const tnPtr* tConstSmartPtr<tnPtr>::Get() const
{ 
	_func_start( tConstSmartPtr::Get );
	return fObject;
	_func_end;
}

////////////////////////////////////////////////////////////////////////////////
// Template functions for SmartPtr
// 
////////////////////////////////////////////////////////////////////////////////


template <class tnPtr >
tSmartPtr<tnPtr>::tSmartPtr<tnPtr>()
{_func_start( tSmartPtr::tSmartPtr );_func_end;}

template <class tnPtr >
tSmartPtr<tnPtr>::tSmartPtr<tnPtr>( tnPtr* pObject ) :
	tConstSmartPtr<tnPtr>( pObject )	
{_func_start( tSmartPtr::tSmartPtr );_func_end;}

template <class tnPtr >	
tSmartPtr<tnPtr>::tSmartPtr( const tSmartPtr& copy ) :
	tConstSmartPtr<tnPtr>( copy.Get() )	
{_func_start( tSmartPtr::tSmartPtr );_func_end;}

template <class tnPtr >	
tSmartPtr<tnPtr>& tSmartPtr<tnPtr>::operator=( const tSmartPtr& copy )
{
	_func_start( tSmartPtr::operator = );
	tConstSmartPtr<tnPtr>::operator=( copy.Get() ); 	

	return *this;
	_func_end;
}

template <class tnPtr >	
tnPtr* tSmartPtr<tnPtr>::operator=( tnPtr* pPtr )
{
	_func_start( tSmartPtr::operator = );
	return tConstSmartPtr<tnPtr>::operator=( pPtr ); 

	return pPtr;
	_func_end;
}	

template <class tnPtr >
tnPtr* tSmartPtr<tnPtr>::operator->() const
{
	_func_start( tSmartPtr::operator -> );
	return fObject;
	_func_end;
}

template <class tnPtr >
tSmartPtr<tnPtr>::operator tnPtr*() const
{
	_func_start( tSmartPtr::operator tnPtr* );
	return fObject;
	_func_end;
}

template <class tnPtr >
tnPtr* tSmartPtr<tnPtr>::Get() const
{
	_func_start( tSmartPtr::Get );
	return fObject;
	_func_end;
}


#endif // !defined( SmartPtr_h )


