//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( AccessPointer_h )
#define AccessPointer_h

#include <BaseTypes.h>
#include <superassert.h>
#include <Thread\ThreadState.h>

// CLASS PURPOSE: Used to access a guarded pointer
template< typename tnPtrType, typename tnPtrGuard >
class tAccessPointer
{
public:
// Setup
	tAccessPointer();
	tAccessPointer( tnPtrType* pointer, const tnPtrGuard* pointerGuard );
	tAccessPointer( tAccessPointer<tnPtrType,tnPtrGuard>& copy );
	tAccessPointer& operator=( tAccessPointer<tnPtrType,tnPtrGuard>& copy );
	~tAccessPointer();

// Operations 
	bool IsValid() const;
	void Done();
	
// Accessors
	operator tnPtrType*();
	tnPtrType& operator*();
	tnPtrType* operator->();

protected:
	mutable tOptex fAccess;
	tnPtrType* fPointer;
	const tnPtrGuard* fPointerGuard;
};



//------------------------------------------------------------------------
template< typename tnPtrType, typename tnPtrGuard >
tAccessPointer<tnPtrType,tnPtrGuard>::tAccessPointer<tnPtrType,tnPtrGuard>()
	: fPointer( NULL )
	, fPointerGuard( NULL )
	, fAccess( tThreadState::GetCurThreadState()->DefaultThreadObjectUse() )
{_func_start( tAccessPointer::tAccessPointer );_func_end;}

template< typename tnPtrType, typename tnPtrGuard >
tAccessPointer<tnPtrType,tnPtrGuard>::tAccessPointer<tnPtrType,tnPtrGuard>( tnPtrType* pointer, const tnPtrGuard* pointerGuard  )
	: fPointer( pointer )
	, fPointerGuard( pointerGuard )
	, fAccess( tThreadState::GetCurThreadState()->DefaultThreadObjectUse() )
{_func_start( tAccessPointer::tAccessPointer );_func_end;}

template< typename tnPtrType, typename tnPtrGuard >
tAccessPointer<tnPtrType,tnPtrGuard>::tAccessPointer<tnPtrType,tnPtrGuard>( tAccessPointer<tnPtrType,tnPtrGuard>& copy )
	: fPointer( NULL )
	, fPointerGuard( NULL )
	, fAccess( tThreadState::GetCurThreadState()->DefaultThreadObjectUse() )
{
	_func_start( tAccessPointer::tAccessPointer );
	*this = copy;
	_func_end;
}

template< typename tnPtrType, typename tnPtrGuard >
tAccessPointer<tnPtrType,tnPtrGuard>& tAccessPointer<tnPtrType,tnPtrGuard>::operator=( tAccessPointer<tnPtrType,tnPtrGuard>& copy )
{
	_func_start( tAccessPointer::operator = );
	if( &copy != this )
	{
		tOptex::tGuard guard( &fAccess );
		tOptex::tGuard copyGuard( &copy.fAccess );

		// Only one object can have access to the pointer at a time,
		// So we invaidate the original
		fPointer = copy.fPointer;
		copy.fPointer = NULL;
		fPointerGuard = copy.fPointerGuard;
		copy.fPointerGuard = NULL;
	}

	return *this;
	_func_end;
}

template< typename tnPtrType, typename tnPtrGuard >
tAccessPointer<tnPtrType,tnPtrGuard>::~tAccessPointer<tnPtrType,tnPtrGuard>()
{
	_func_start( tAccessPointer::~tAccessPointer );
	Done();	
	_func_end;
}

template< typename tnPtrType, typename tnPtrGuard >
bool tAccessPointer<tnPtrType,tnPtrGuard>::IsValid() const
{
	_func_start( tAccessPointer::IsValid );
	tOptex::tGuard guard( &fAccess );

	if( fPointer )
	{
		return true;
	} 
	else
	{
		return false;
	}
	_func_end;
}

template< typename tnPtrType, typename tnPtrGuard >
void tAccessPointer<tnPtrType,tnPtrGuard>::Done()
{
	_func_start( tAccessPointer::Done );
	tOptex::tGuard guard( &fAccess );

	if( fPointer )
	{
		fPointerGuard->Done();			
		fPointer = NULL;
	}
	_func_end;
}

template< typename tnPtrType, typename tnPtrGuard >
tAccessPointer<tnPtrType,tnPtrGuard>::operator tnPtrType*()
{
	_func_start( tAccessPointer::operator tnPtrType* );
	tOptex::tGuard guard( &fAccess );

	return fPointer;
	_func_end;
}

template< typename tnPtrType, typename tnPtrGuard >
tnPtrType& tAccessPointer<tnPtrType,tnPtrGuard>::operator*()
{
	_func_start( tAccessPointer::operator * );
	tOptex::tGuard guard( &fAccess );

	Assert( fPointer );
	return *fPointer; 
	_func_end;
}

template< typename tnPtrType, typename tnPtrGuard >
tnPtrType* tAccessPointer<tnPtrType,tnPtrGuard>::operator->()
{
	_func_start( tAccessPointer::operator -> );
	tOptex::tGuard guard( &fAccess );

	Assert( fPointer );
	return fPointer; 
	_func_end;
}

#endif


