//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( InterlockedScalar_h )
#define InterlockedScalar_h

#include <BaseTypes.h>
#include <Thread\InterlockedType.h>

// CLASS PURPOSE: 
// Instances of this class will be accessed by multiple threads. So, 
// all members of this class (except the constructor and destructor) 
// must be thread-safe.
template< typename tType >
class tInterlockedScalar : protected tInterlockedType< tType > 
{
public:
//Setup
	tInterlockedScalar( tType value = tType() );
	tInterlockedScalar( const tInterlockedScalar& copy );
	tInterlockedScalar& operator=( const tInterlockedScalar& copy );
	~tInterlockedScalar();
	
	// C++ does not allow operator cast to be inherited.
	operator tType() const ;
	
	tType operator=( tType value );
	tType operator++( int );
	tType operator--( int );
	tType operator += ( tType op );
	tType operator++();
	tType operator -= ( tType op );
	tType operator--();
	tType operator *= ( tType op );
	tType operator /= ( tType op );
	tType operator %= ( tType op );  
	tType operator ^= ( tType op );  
	tType operator &= ( tType op );  
	tType operator |= ( tType op );  
	tType operator <<=( tType op );  
	tType operator >>=( tType op );  
	
};





// -------------- TEMPLATE FUNCTIONS IMPL ---------------------------
template< typename tType >
tInterlockedScalar<tType>::tInterlockedScalar( tType value ) : 
	tInterlockedType<tType>( value ) 
{_func_start( tInterlockedScalar::tInterlockedScalar );_func_end;}

template< typename tType >
tInterlockedScalar<tType>::~tInterlockedScalar()
{_func_start( tInterlockedScalar::~tInterlockedScalar );_func_end;}

template< typename tType >
tInterlockedScalar<tType>::tInterlockedScalar( const tInterlockedScalar<tType>& copy ) :
	tInterlockedType<tType>( copy ) 
{_func_start( tInterlockedScalar::tInterlockedScalar );_func_end;}

template< typename tType >
tInterlockedScalar<tType>& tInterlockedScalar<tType>::operator=( const tInterlockedScalar<tType>& copy )
{
	_func_start( tInterlockedScalar::operator = );
	tResourceGuard::tGuard guard( fResourceGuard ); 

	SetVal( copy ); 

	return *this;
	_func_end;
}

template< typename tType >
tInterlockedScalar<tType>::operator tType() const 
{ 
	_func_start( tInterlockedScalar::operator tType );
	return(tInterlockedType<tType>::operator tType()); 
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator=(tType value) 
{ 
	_func_start( tInterlockedScalar::operator = );
	tResourceGuard::tGuard guard( fResourceGuard ); 

	return(SetVal(value)); 
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator++( int ) 
{    
	_func_start( tInterlockedScalar::operator ++ );
	tResourceGuard::tGuard guard( fResourceGuard );

	tType prevVal = GetVal();
	SetVal( static_cast<tType>( prevVal + 1 ) );

	return prevVal;      // Return value BEFORE increment
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator--( int ) 
{    
	_func_start( tInterlockedScalar::operator -- );
	tResourceGuard::tGuard guard( fResourceGuard );

	tType prevVal = GetVal();
	SetVal( static_cast<tType>( prevVal - 1 ) );

	return prevVal;      // Return value BEFORE decrement
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator += ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator += );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() + op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator++()            
{ 
	_func_start( tInterlockedScalar::operator ++ );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() + 1 );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator -= ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator -= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() - op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator--()            
{ 
	_func_start( tInterlockedScalar::operator -- );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() - 1 );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator *= ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator *= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() * op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator /= ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator /= );
	tResourceGuard::tGuard guard( fResourceGuard );
	
	return SetVal( GetVal() / op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator %= ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator %= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() % op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator ^= ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator ^= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() ^ op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator &= ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator &= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() & op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator |= ( tType op )   
{ 
	_func_start( tInterlockedScalar::operator |= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() | op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator <<=( tType op )   
{ 
	_func_start( tInterlockedScalar::operator <<= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() << op );
	_func_end;
}

template< typename tType >
tType tInterlockedScalar<tType>::operator >>=( tType op )   
{ 
	_func_start( tInterlockedScalar::operator >>= );
	tResourceGuard::tGuard guard( fResourceGuard );

	return SetVal( GetVal() >> op );
	_func_end;
}


#endif


