//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( InterlockedType_h )
#define InterlockedType_h

#include <BaseTypes.h>
#include <Thread\ResourceGuard.h>
#include <Exceptions\InvalidAccess.h>
#include <superassert.h>

//#if !defined( USE_PCH )  ANF @@@ Find out why this fails if enabled



//#endif 

// CLASS PURPOSE: Instances of this class will be accessed by multiple 
// threads. So, all members of this class (except the constructor and 
// destructor) must be thread-safe.
template< typename tType >
class tInterlockedType 
{
public:     
//Setup
   // Note: Constructors & destructors are always thread safe
   tInterlockedType();
   tInterlockedType( const tType& value );
   virtual ~tInterlockedType();

//Accessors
   // Cast operator to make writing code that uses 
   // thread-safe data type easier
   operator tType() const;

protected:
//Operations
	virtual void OnValChanged( const tType& newValue, const tType& prevValue ) const;
	tType SetVal(const tType& newValue );

//Accessors
	tType& GetVal(); 
	const tType& GetVal() const; 


//Fields
	// Guard for use by derived class functions
	mutable tResourceGuard fResourceGuard;

private:
//Hidden
	tInterlockedType( const tInterlockedType& copy );
	tInterlockedType& operator=( const tInterlockedType& copy );

//Fields
   tType fValue;
};




// -------------- TEMPLATE FUNCTIONS IMPL ---------------------------
template< typename tType >
tInterlockedType<tType>::tInterlockedType()
{_func_start( tInterlockedType::tInterlockedType );_func_end;}

template< typename tType >
tInterlockedType<tType>::tInterlockedType( const tType& value ) 
{ 
	_func_start( tInterlockedType::tInterlockedType );
	fValue = value; 
	_func_end;
}

template< typename tType >
tInterlockedType<tType>::~tInterlockedType()
{_func_start( tInterlockedType::~tInterlockedType );_func_end;}

template< typename tType >
tInterlockedType<tType>::tInterlockedType( const tInterlockedType& copy )
{
	_func_start( tInterlockedType::tInterlockedType );
	throw except::tInvalidAccess( "tInterlockedType<tType>::tInterlockedType", __FILE__, __LINE__ );
	_func_end;
}

template< typename tType >
tInterlockedType<tType>& tInterlockedType<tType>::operator=( const tInterlockedType& copy )
{
	_func_start( tInterlockedType::operator = );
	throw except::tInvalidAccess( "tInterlockedType<tType>::operator=", __FILE__, __LINE__ );

	return *this;
	_func_end;
}

template< typename tType >
tInterlockedType< tType >::operator tType() const 
{ 
	_func_start( tInterlockedType::operator tType );
  tResourceGuard::tGuard guard( fResourceGuard ); 

  return GetVal(); 
	_func_end;
}

template< typename tType >
tType& tInterlockedType< tType >::GetVal() 
{ 
	_func_start( tInterlockedType::GetVal );
  Assert( fResourceGuard.IsGuarded() ); 

  return fValue; 
	_func_end;
}

template< typename tType >
const tType& tInterlockedType< tType >::GetVal() const 
{ 
	_func_start( tInterlockedType::GetVal );
  Assert( fResourceGuard.IsGuarded() ); 

  return fValue; 
	_func_end;
}

template< typename tType >
tType tInterlockedType< tType >::SetVal(const tType& newValue )
{ 
	_func_start( tInterlockedType::SetVal );
  Assert( fResourceGuard.IsGuarded() ); 

  tType& value = GetVal();
  if( value != newValue ) 
  {
     tType prevValue = value;
     value = newValue;
     OnValChanged( newValue, prevValue );
  }

  return value; 
	_func_end;
}

template< typename tType >
void tInterlockedType< tType >::OnValChanged( const tType& newValue, const tType& prevValue ) const
{_func_start( tInterlockedType::OnValChanged );_func_end;}


#endif


