//	<Legal>	Copyright ( c ) by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#include <BaseTypes.h>
#include <FunctionWrapper.h>

#if !defined( Optex_Inline )

////////////////////////////////////////////////////////////////////////////////
// Non Inline code
// 
// The section for the optex code that will not be inlined
// 
////////////////////////////////////////////////////////////////////////////////

#include "Optex.h"
#include "Thread.h"
#include "ThreadExceptions\DeadlockException.h"

tOptex::eProccessorState tOptex::fProccessorState	 = kNotSet;
bool					 tOptex::fIsWindows95		 = true;
bool					 tOptex::fIsWindows98		 = false;
bool					 tOptex::fTreatWindows98As95 = false;

void tOptex::SetTreatWindows98As95( bool treatWindows98As95 )
{
	_func_start( tOptex::SetTreatWindows98As95 );
	fTreatWindows98As95 = treatWindows98As95;
	_func_end;
}

void tOptex::Wait()
{
	_func_start( tOptex::Wait );

	if( gActiveThread )
	{
		gActiveThread->Sleeping( fpSharedInfo->fThreadId, this ); 	
	}

	::WaitForSingleObject( fEvent, INFINITE );

	if( gActiveThread )
	{
		gActiveThread->Awake(); 	
		if( fDeadlocked )
		{
			throw tDeadlockException();
		}
	}

	_func_end;
}

void tOptex::Deadlocked()
{
	_func_start( tOptex::Deadlocked );
	fDeadlocked = true;
	int32u lockCount = fpSharedInfo->fLockCount;
	while( lockCount-- )
	{
		::SetEvent( fEvent );
	}
	_func_end;
}

#else

////////////////////////////////////////////////////////////////////////////////
// Inline code
// 
// The section for the optex code that will be inlined
// 
////////////////////////////////////////////////////////////////////////////////

#include "Optex.h"

#include <Exceptions\Validate.h>
#include <Exceptions\InvalidAccess.h>


#if !defined( NO_WIN95 )
#include <OSVersion.h>
#endif

#if !defined( _WIN32 )
#error This class is only designed for Win32
#endif

#if !defined( USE_PCH )

#include <windows.h>
#include <SuperAssert.h>

#endif


tOptex::tGuard::tGuard( tOptex* optex ) :
	fOptex( optex )
{
	_func_start( tOptex::tGuard::tGuard );
	Enter();	
	_func_end;
}

tOptex::tGuard::~tGuard()
{
	_func_start( tOptex::tGuard::~tGuard );
	if( fNeedToLeave )
	{
		fOptex->Leave();
	}

	_func_end;
}

void tOptex::tGuard::Enter()
{
	_func_start( tOptex::tGuard::Enter );
	fNeedToLeave = true;
	fOptex->Enter();	
	_func_end;
}

void tOptex::tGuard::Leave()
{
	_func_start( tOptex::tGuard::Leave );
	fOptex->Leave();	
	fNeedToLeave = false;
	_func_end;
}

tOptex::tOptex( tThreadState::eUse use, int32u spinCount )
{
	_func_start( tOptex::tOptex );
	CommonConstructor( use, spinCount, false, NULL );
	_func_end;
}

tOptex::tOptex( const char* pName, tThreadState::eUse use, int32u spinCount )
{
	_func_start( tOptex::tOptex );
	CommonConstructor( use, spinCount, false, reinterpret_cast<const void*>( pName ) );
	_func_end;
}

tOptex::tOptex( const wchar_t* pName, tThreadState::eUse use, int32u spinCount )
{
	_func_start( tOptex::tOptex );
	CommonConstructor( use, spinCount, true, reinterpret_cast<const void*>( pName ) );
	_func_end;
}

char* tOptex::ConstructObjectName( char* pResult, const char* pPrefix, bool fUnicode, const void* pName )
{
	_func_start( tOptex::ConstructObjectName );
	pResult[0] = 0;
	if( pName == NULL ) 
	{
		return( NULL );
	}

	wsprintfA( pResult, fUnicode ? "%s%S" : "%s%s", pPrefix, pName );

	return( pResult );
	_func_end;
}

void tOptex::CommonConstructor( tThreadState::eUse use, int32u spinCount, bool fUnicode, const void* pName ) 
{
	_func_start( tOptex::CommonConstructor );
	fUse = use;
	fDeadlocked = false;

	if( fUse == tThreadState::kIgnoreThreadObject )
	{
		return;
	}

	if( fProccessorState == kNotSet ) 
	{
		// This is the 1st object constructed, get the number of CPUs
		SYSTEM_INFO sysInfo;
		::GetSystemInfo( &sysInfo );
		validate( sysInfo.dwNumberOfProcessors > 0 );

		if( sysInfo.dwNumberOfProcessors == 1 )
		{
			fProccessorState = kSingleProccessor;
		}
		else 
		{
			fProccessorState = kMultiProccessor;
		}

		fTreatWindows98As95 = false;

#if	defined( NO_WIN95 )
		fIsWindows95		= false;
		fIsWindows98		= false;
#else
		fIsWindows95		= tOSVersion::Instance()->IsWindows95();
		fIsWindows98		= tOSVersion::Instance()->IsWindows98();
#endif
	}

	fUseCriticalSections = fIsWindows95;
	if ( fTreatWindows98As95 )
	{
		fUseCriticalSections = fIsWindows98;
	}

	fEvent				 = NULL;
	fFileMapping		 = NULL;
	fpSharedInfo		 = NULL;

	if( pName == NULL ) // Creating a single-process optex
	{ 
		if( ! fUseCriticalSections )
		{
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
			//ZDTEST@@@@
			tResourceTracker::Instance()->UsingHandle( tResourceTracker::k_tOptex_CommonConstructor_Event );
#endif
			fEvent = ::CreateEventA( NULL, false, false, NULL );
			Assert( fEvent != NULL );
		}

		fpSharedInfo = new tSharedInfo;
		Assert( fpSharedInfo != NULL );
		::ZeroMemory( fpSharedInfo, sizeof(*fpSharedInfo) );
	} 
	else // Creating a cross-process optex
	{ 

		// Always use ANSI so that this works on Win9x and Windows 2000
		char result[100];
		ConstructObjectName( result, "Optex_Event_", fUnicode, pName );
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
		//ZDTEST@@@@
		tResourceTracker::Instance()->UsingHandle( tResourceTracker::k_tOptex_CommonConstructor_Event );
#endif
		fEvent = ::CreateEventA( NULL, false, false, result );
		Assert( fEvent != NULL );

		ConstructObjectName( result, "Optex_MMF_", fUnicode, pName );
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
		//ZDTEST@@@@
		tResourceTracker::Instance()->UsingHandle( tResourceTracker::k_tOptex_CommonConstructor_MPFileMap );
#endif
		fFileMapping = ::CreateFileMappingA( INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, sizeof( *fpSharedInfo), result );
		Assert( fFileMapping != NULL );

		fpSharedInfo = ( tSharedInfo* ) ::MapViewOfFile( fFileMapping, FILE_MAP_WRITE, 0, 0, 0 );
		Assert( fpSharedInfo != NULL );

		// Note: SHAREDINFO's fLockCount, fThreadId, and fRecurseCount
		// members need to be initialized to 0. Fortunately, a new pagefile 
		// MMF sets all of its data to 0 when created. This saves use from 
		// some thread synchronization work.
	}

	SetSpinCount( spinCount );

	// Windows 9x is slow at CreateEvent, so we use CriticalSections instead.
	if ( fUseCriticalSections )
	{
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
		//ZDTEST@@@@
		tResourceTracker::Instance()->UsingHandle( tResourceTracker::k_tOptex_CommonConstructor_CritSection  );
#endif
		::InitializeCriticalSection( &( fpSharedInfo->fCriticalSection ) );
	}
	_func_end;
}

tOptex::~tOptex() 
{
	_func_start( tOptex::~tOptex );
	if( fUse == tThreadState::kIgnoreThreadObject )
	{
		return;
	}

	// Windows 9x is slow at CreateEvent, so we use CriticalSections instead.
	if ( fUseCriticalSections )
	{
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
		//ZDTEST@@@@
		tResourceTracker::Instance()->ReleaseHandle( tResourceTracker::k_tOptex_CommonConstructor_CritSection );
#endif

		if ( fpSharedInfo )
			::DeleteCriticalSection( &( fpSharedInfo->fCriticalSection ) );
	}

#ifdef _DEBUG
	if( IsSingleProcessOptex() && (fpSharedInfo->fThreadId != 0) )
	{
		// A single-process optex shouldn't be destroyed if any thread owns it
		::DebugBreak();
	}

	if( !IsSingleProcessOptex() && (fpSharedInfo->fThreadId == ::GetCurrentThreadId()) ) 
	{
		// A cross-process optex shouldn't be destroyed if our thread owns it
		::DebugBreak();
	}
#endif

	if( fEvent )
	{
		::CloseHandle( fEvent );
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
		//ZDTEST@@@@
		tResourceTracker::Instance()->ReleaseHandle( tResourceTracker::k_tOptex_CommonConstructor_Event );
#endif
	}

	if( IsSingleProcessOptex() ) 
	{
		delete fpSharedInfo;
	} 
	else 
	{
		::UnmapViewOfFile( fpSharedInfo );
		::CloseHandle( fFileMapping );
#if defined( _DEBUG ) && defined( RESOURCE_TRACKER )
		//ZDTEST@@@@
		tResourceTracker::Instance()->ReleaseHandle( tResourceTracker::k_tOptex_CommonConstructor_MPFileMap );
#endif
	}
	_func_end;
}

bool tOptex::IsSingleProcessOptex() const 
{
	_func_start( tOptex::IsSingleProcessOptex );
   return fFileMapping == NULL;
	_func_end;
}

bool tOptex::CurThreadOwnsOptex() const
{
	_func_start( tOptex::CurThreadOwnsOptex );
	if( fUse == tThreadState::kIgnoreThreadObject )
	{
		return false;
	}

	int32u threadId = ::GetCurrentThreadId();

	if( fpSharedInfo->fThreadId == threadId )
	{
		return true;
	}
	else
	{
		return false;
	}
	_func_end;
}

void tOptex::SetSpinCount( int32u spinCount )
{
	_func_start( tOptex::SetSpinCount );
	if( fUse == tThreadState::kIgnoreThreadObject )
	{
		return;
	}

	// No spinning on single CPU machines
	if( fProccessorState == kMultiProccessor )
	{
		::InterlockedExchange( (int32*) &fpSharedInfo->fSpinCount, spinCount );
	}
	_func_end;
}

void tOptex::Enter() 
{
	_func_start( tOptex::Enter );
	if( fUse == tThreadState::kIgnoreThreadObject )
	{
		return;
	}

	// Windows 9x is slow at CreateEvent, so we use CriticalSections instead.c
	if ( fUseCriticalSections )
	{
		::EnterCriticalSection( &( fpSharedInfo->fCriticalSection ) );
		return;
	}

	// Spin, trying to get the optex
	if( TryEnter() ) 
	{
		return; // We got it, return
	}

	// We couldn't get the optex, wait for it.
	int32u threadId = ::GetCurrentThreadId();

	if( ::InterlockedIncrement( &fpSharedInfo->fLockCount) == 1 ) 
	{
		// Optex is unowned, let this thread own it once
		fpSharedInfo->fThreadId = threadId;
		fpSharedInfo->fRecurseCount = 1;
	} 
	else 
	{
		if( fpSharedInfo->fThreadId == threadId ) 
		{
			// If optex is owned by this thread, own it again
			fpSharedInfo->fRecurseCount++;
		} 
		else 
		{
			// Optex is owned by another thread, wait for it
			Wait();

			// Optex is unowned, let this thread own it once
			fpSharedInfo->fThreadId = threadId;
			fpSharedInfo->fRecurseCount = 1;
		}
	}
	_func_end;
}

bool tOptex::TryEnter()
{
	_func_start( tOptex::TryEnter );
	if( fUse == tThreadState::kIgnoreThreadObject )
	{
		return false ;
	}

	// Windows 9x is slow at CreateEvent, so we use CriticalSections instead.
	if ( fUseCriticalSections )
	{
		// KMY: Should we perform an Enter instead and return true???
		return false;
	}

	int32u threadId = ::GetCurrentThreadId();

	bool thisThreadOwnsTheOptex = false; // Assume a thread owns the optex
	int32u spinCount = fpSharedInfo->fSpinCount; // How many times to spin

	do 
	{
		// If lock count = 0, optex is unowned, we can own it
		// OSVersion loads the appropriate DLL for the OS running (empty if Win95 - see above)
#if defined( NO_WIN95 )
		thisThreadOwnsTheOptex = (0 == ::InterlockedCompareExchange( (void**)&fpSharedInfo->fLockCount, (void*)1, 0) ); 	
#else
		thisThreadOwnsTheOptex = (0 == (*(OSSPECIFIC::OS_InterlockedCompareExchange))( &fpSharedInfo->fLockCount, 1, 0) ); 
#endif

		if( thisThreadOwnsTheOptex ) 
		{
			// Optex is unowned, let this thread own it once
			fpSharedInfo->fThreadId = threadId;
			fpSharedInfo->fRecurseCount = 1;
		} 
		else 
		{
			if( fpSharedInfo->fThreadId == threadId ) 
			{
				// If optex is owned by this thread, own it again
				::InterlockedIncrement( &fpSharedInfo->fLockCount );
				fpSharedInfo->fRecurseCount++;
				thisThreadOwnsTheOptex = TRUE;
			}
		}
	} while ( !thisThreadOwnsTheOptex && (spinCount-- > 0) );

	// Return whether or not this thread owns the optex
	return( thisThreadOwnsTheOptex );
	_func_end;
}


void tOptex::Leave() 
{
	_func_start( tOptex::Leave );
	if( fUse == tThreadState::kIgnoreThreadObject )
	{
		return;
	}

	// Windows 9x is slow at CreateEvent, so we use CriticalSections instead.
	if ( fUseCriticalSections )
	{
		::LeaveCriticalSection( &( fpSharedInfo->fCriticalSection ) );
		return;
	}

#ifdef _DEBUG
	// Only the owning thread can leave the optex
	if( fpSharedInfo->fThreadId != ::GetCurrentThreadId() ) 
	{
		DebugBreak();
	}
#endif

	// Reduce this thread's ownership of the optex
	if( --fpSharedInfo->fRecurseCount > 0 ) 
	{
		// We still own the optex
		InterlockedDecrement( &fpSharedInfo->fLockCount );
	} 
	else 
	{
		// We don't own the optex anymore
		fpSharedInfo->fThreadId = 0;

		if( InterlockedDecrement(&fpSharedInfo->fLockCount) > 0 ) 
		{
			// Other threads are waiting, the auto-reset event wakes one of them
			SetEvent( fEvent );
		}
	}
	_func_end;
}

#endif //#if !defined( Optex_Inline )
