//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( Optex_h )
#define Optex_h


#include <BaseTypes.h> 

#if !defined( USE_PCH )
#include <windows.h>
#endif

#include "ThreadState.h"

// CLASS PURPOSE: Optimized mutex class from Programming Applications 4th Ed
class tOptex
{
public:
	class tGuard
	{
	public:
		inline tGuard( tOptex* optex );
		inline ~tGuard();

		inline void Enter();
		inline void Leave();

	protected:
		bool fNeedToLeave;
		tOptex* fOptex; 
		
	};

//Setup
	inline tOptex( tThreadState::eUse use = tThreadState::kUseThreadObject, int32u spinCount = 4000 );
	inline tOptex( const char* name, tThreadState::eUse use = tThreadState::kUseThreadObject, int32u spinCount = 4000);
	inline tOptex( const wchar_t* pszName, tThreadState::eUse use = tThreadState::kUseThreadObject, int32u spinCount = 4000);	
	inline ~tOptex();

//Operations
	static void SetTreatWindows98As95( bool );
	inline void SetSpinCount( int32u spinCount );
	inline void Enter();
	inline bool TryEnter();
	inline void Leave();
	// This function will cause a throw in all the objects that are waiting for this optex
	void Deadlocked();

//Accessors
	inline bool IsSingleProcessOptex() const;
	inline bool CurThreadOwnsOptex() const;

protected:
//Types
	struct tSharedInfo
	{
	public:
		
		int32u fSpinCount;
		int32  fLockCount;
		int32u fThreadId;
		int32  fRecurseCount;

		// For Windows 95 support ONLY
		CRITICAL_SECTION fCriticalSection;
	};

	enum eProccessorState
	{
		kNotSet, 
		kMultiProccessor,
		kSingleProccessor,
	};

//Setup
	inline void CommonConstructor( tThreadState::eUse use, int32u spinCount, bool unicode, const void* pName );

//Operations
	inline char* ConstructObjectName( char* pResult, const char* pPrefix, bool unicode, const void* pName );
	void Wait();

//Fields
	tThreadState::eUse	fUse;
	bool				fInitComplete;
	bool				fUseCriticalSections;
	void*				fEvent;
	void*				fFileMapping;
	tSharedInfo*		fpSharedInfo;
	bool				fDeadlocked;

	static eProccessorState fProccessorState;

	static bool		fIsWindows95;
	static bool		fIsWindows98;			// KMYNF: Currently means ME as well, change to INSURE.
	static bool		fTreatWindows98As95;

private:
//Hidden
	inline tOptex( const tOptex& copy );
	inline tOptex& operator=( const tOptex& copy );
};

#define Optex_Inline
#include "Optex.cpp"
#undef Optex_Inline

#endif

