//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( SingleAccessWrapper_h )
#define SingleAccessWrapper_h

#include <BaseTypes.h>
#include "Optex.h"
#include "AccessPointer.h"
#include "ThreadState.h"

// CLASS PURPOSE: This class gives you exclusive access to a pointer
template< typename tPtrType >
class tSingleAccessWrapper
{
public:
//Types
	typedef tAccessPointer< tPtrType, tSingleAccessWrapper > iPtr;
	typedef tAccessPointer< const tPtrType, tSingleAccessWrapper > iConstPtr;

// Setup
	tSingleAccessWrapper();
	tSingleAccessWrapper( tPtrType* pointer );
	tSingleAccessWrapper( const tSingleAccessWrapper& copy );
	~tSingleAccessWrapper();
	tSingleAccessWrapper& operator=( const tSingleAccessWrapper& copy );
	tSingleAccessWrapper& operator=( tPtrType* copy );

// Operations
	void SetPointer( tPtrType* copy );
	void Delete();
	iPtr Lock();
	iConstPtr Lock() const;

// Accessor
	bool IsValid() const;

protected:
	friend iPtr;
	friend iConstPtr;

// Hidden

// Operations	
	void Done() const;

// Fields
	tPtrType* fpPointer;
	mutable tOptex fPointerOptex;
};

//------------------------------------------------------------------------------------------
template< typename tPtrType >
tSingleAccessWrapper<tPtrType>::tSingleAccessWrapper<tPtrType>()
	: fpPointer( NULL )
	, fPointerOptex( tThreadState::GetCurThreadState()->DefaultThreadObjectUse() )
{_func_start( tSingleAccessWrapper::tSingleAccessWrapper );_func_end;}

template< typename tPtrType >
tSingleAccessWrapper<tPtrType>::tSingleAccessWrapper<tPtrType>( tPtrType* pointer )
	: fpPointer( pointer )
	, fPointerOptex( tThreadState::GetCurThreadState()->DefaultThreadObjectUse() )
{_func_start( tSingleAccessWrapper::tSingleAccessWrapper );_func_end;}

// this COPIES CONTENTS, not pointers
// <tPtrType> requires a default constructor
template< typename tPtrType >
tSingleAccessWrapper<tPtrType>::tSingleAccessWrapper<tPtrType>( const tSingleAccessWrapper<tPtrType>& copy )
	:fpPointer( NULL )
{
	_func_start( tSingleAccessWrapper::tSingleAccessWrapper );
	if ( copy.IsValid() )
	{
		SetPointer( new tPtrType );
		*Lock() = *copy.Lock();
	}
	_func_end;
}

template< typename tPtrType >
tSingleAccessWrapper<tPtrType>::~tSingleAccessWrapper<tPtrType>()
{
	_func_start( tSingleAccessWrapper::~tSingleAccessWrapper );
	Delete();
	_func_end;
}

// this COPIES CONTENTS, not pointers
// <tPtrType> requires a default constructor
template< typename tPtrType >
tSingleAccessWrapper<tPtrType>& tSingleAccessWrapper<tPtrType>::operator=( const tSingleAccessWrapper<tPtrType>& copy )
{
	_func_start( tSingleAccessWrapper::operator = );
	if ( ( this != &copy ) && copy.IsValid() )
	{
		if ( !IsValid() )
		{
			SetPointer( new tPtrType );
		}
		*Lock() = *copy.Lock();
	}
	return *this;
	_func_end;
}

template< typename tPtrType >
tSingleAccessWrapper<tPtrType>& tSingleAccessWrapper<tPtrType>::operator=( tPtrType* copy )
{
	_func_start( tSingleAccessWrapper::operator = );
	SetPointer( copy );
	return *this;
	_func_end;
}

template< typename tPtrType >
void tSingleAccessWrapper<tPtrType>::SetPointer( tPtrType* copy )
{
	_func_start( tSingleAccessWrapper::SetPointer );
	Delete();

	fpPointer = copy;
	_func_end;
}

template< typename tPtrType >
void tSingleAccessWrapper<tPtrType>::Delete()
{
	_func_start( tSingleAccessWrapper::Delete );
	fPointerOptex.Enter();
	if( fpPointer )
	{
		delete fpPointer;
		fpPointer = NULL;
	}
	fPointerOptex.Leave();
	_func_end;
}

template< typename tPtrType >
tSingleAccessWrapper<tPtrType>::iPtr tSingleAccessWrapper<tPtrType>::Lock()
{
	_func_start( tSingleAccessWrapper::Lock );
	fPointerOptex.Enter();
	
	return iPtr( fpPointer, this ); 	
	_func_end;
}

template< typename tPtrType >
tSingleAccessWrapper<tPtrType>::iConstPtr tSingleAccessWrapper<tPtrType>::Lock() const
{
	_func_start( tSingleAccessWrapper::Lock );
	fPointerOptex.Enter();
	
	return iConstPtr( fpPointer, this ); 	
	_func_end;
}

template< typename tPtrType >
bool tSingleAccessWrapper<tPtrType>::IsValid() const
{
	_func_start( tSingleAccessWrapper::IsValid );
	bool result = false;

	fPointerOptex.Enter();
	if( fpPointer )
	{
		result = true;
	}
	fPointerOptex.Leave();	

	return result;
	_func_end;
}

template< typename tPtrType >
void tSingleAccessWrapper<tPtrType>::Done() const
{
	_func_start( tSingleAccessWrapper::Done );
	fPointerOptex.Leave();	
	_func_end;
}


#endif


