//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( Thread_h )
#define Thread_h

#include <BaseTypes.h>

#include <Debug\CallStack.h>

#include <Callback.h>
#include "InterlockedScalar.h"
#include "Optex.h"
#include "ThreadEvent.h"

// CLASS PURPOSE: A class that encapsulates threading
class tThread
{
public:
// Types
	typedef tCallback< int32 > iThreadCallback;

// Setup
	tThread();
	~tThread();

// Operations	
	bool StartWork( iThreadCallback* workItem, int32 context );
	void StopThread( int32u waitTime = 0 );
	
	// Sleep function
	void Sleeping( int32u threadID, tOptex* waitingFor );
	void Awake();

	enum ThreadPriority
	{
		kThreadPriorityIdle = 0,		// The thread only works when other threads aren't.
		kThreadPriorityLow,
		kThreadPriorityBelowNormal,
		kThreadPriorityNormal,
		kThreadPriorityAboveNormal,
		kThreadPriorityHigh,
		kThreadPriorityRealTime			// The thread preempts all others.  If you're using this, you should ask yourself why.
	};

	void mSetPriority(ThreadPriority priority);

// Accessors
	int32u	GetThreadID() const;
	// The ID of thread, that this is waiting for, to wake up
	// Returns 0, if thread is not sleeping
	int32u	GetWaitingForThreadID() const;
	tOptex*	GetWaitingForOptex();

	bool Working() const;
	bool Alive() const;
	bool operator<( const tThread& compare );

// Callstack Ops
	static void RecordCallstacks( bool startRecordingCallstacks, bool alwaysCompactStack ); 
	static void CatchAllExceptions( bool catchAll, bool displayDialog = false );
	tCallStack& GetCallstack();

protected:
// Types
	enum
	{
		kThreadTimeout = 60000,
	};

// Operations
	void Work();
	static unsigned int __stdcall ThreadProc( void* param );

// Fields

	int32u			fThread;
	tThreadEvent	fWorktimeEvent;
	tThreadEvent	fShutdownCompleteEvent;
	unsigned int	fThreadID;

	// Sleeping data
	int32u		 	fWaitingForThreadID;
	tOptex*			fWaitingForOptex;

	tInterlockedScalar<bool> fWorking;
	tInterlockedScalar<bool> fAlive;
	iThreadCallback* fWorkItem;
	int32 fContext;
	tOptex fStartUpGuard;

	tCallStack fCallStackOfCallingThread;
	static bool fRecordCallstacks;
	static bool fAlwaysCompactStack;
	static bool fCatchAllExceptions;
	static bool fDisplayDialog;

private:
//Hiden
	tThread( const tThread& copy );
	tThread& operator=( const tThread& copy );
};

_declspec( thread ) extern tThread* gActiveThread;

#endif

