//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.


#if !defined( HeartbeatTimer_h )
#define HeartbeatTimer_h


#include <BaseTypes.h>
#include "Timer.h"

#include <memory>
#include <vector>
#include <FunctionCallback.h>
#include <Thread\Thread.h>
#include <SmartPtr.h>

////////////////////////////////////////////////////////////////////////////////
// tHeartbeatTimer
// 
// CLASS PURPOSE: Used to create timer events in a thread-unsafe enviroment
// The game loop must call tHeartbeatTimer::Heartbeat, so that the system will have
// time to activate it's timers. 
// NOTE: This call is thread-unsafe! It is assumed that if you are in a threading 
// enviroment then you will use a system timer class (such as tWinTimer)
// 
////////////////////////////////////////////////////////////////////////////////


class tHeartbeatTimer : public tTimer
{
public:
// Types

// Setup
	tHeartbeatTimer();
	virtual ~tHeartbeatTimer();

// Operations
	virtual void StartTimer( int32u dueTime, int32u period, tTimerHandler* pTimerHandler );
	virtual void StartTimer( int32u dueTime, int32u period, iTimerCallbackFunc* pCallback, int32u context );
	virtual void StartTimer( int32u dueTime, int32u period, iTimerCallback* pCallback, int32u context );
	virtual void ChangeTiming( int32u newDueTime, int32u newPeriod );
	virtual void StopTimer();

	static void AutoRunHeartBeat( int32u frequency = 100 );
	static void OnAutoRunHB( int32 context );
	static void StopAutoRunHeartBeat();

	static void Heartbeat();

//Accessors
	virtual int32u TimePassedSinceStart(); 

// Types
	typedef std::vector<tHeartbeatTimer*> iTimers;
	typedef tFunctionCallback< int32 > iThreadCallback;

	class tHeartbeatTimerData
	{ 
	public:
		static tHeartbeatTimerData* Instance();
		
		tOptex			fAccessGlobalData;
		tClock*			fpClock;          
		iTimers			fTimers;          
		tThread			fThread;          
		iThreadCallback	fThreadCB;        
		int32u			fAutoRunFrequency;

		typedef std::auto_ptr< tHeartbeatTimerData > iPtr;
	private:
		friend iPtr;

		tHeartbeatTimerData();
		~tHeartbeatTimerData();

		static iPtr fHeartbeatTimerData;
	};

public:
// Fields
	int32u						fLastTime;
	int32u						fInitalTime;
	int32						fTimerIndex;
	tHeartbeatTimerData*		fHBData;

	//static tHeartbeatTimerData::iPtr fHeartbeatTimerData;

private:
// Hidden
	tHeartbeatTimer( const tHeartbeatTimer& copy );
	tHeartbeatTimer& operator=( const tHeartbeatTimer& copy );
};

#endif // !defined( HeartbeatTimer_h )