//	<Legal>	Copyright (c)  by Taldren, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Taldren, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Taldren, Inc.

#if !defined( Timer_h )
#define Timer_h

#include <BaseTypes.h>
#include "Callback.h"

class tTimer;
class tClock;

// CLASS PURPOSE: A interface class for a timer callback
class tTimerHandler
{
public:
//Setup
	virtual ~tTimerHandler() {_func_start( tTimerHandler::~tTimerHandler );_func_end;};

//Operations
	virtual void HandlerTimerEvent( tTimer* pTimer ) = 0; 	
};

// CLASS PURPOSE: A interface class for a timer that calls a callback function on a time dependent cycle
class tTimer
{
public:
//Types
	typedef void (iTimerCallbackFunc)( tTimer* pTimer, int32u context );
	typedef tCallback<int32u> iTimerCallback;

//Setup
	tTimer();
	virtual ~tTimer();

//Operations 
	virtual void StartTimer( int32u dueTime, int32u period, tTimerHandler* pTimerHandler ) = 0;
	virtual void StartTimer( int32u dueTime, int32u period, iTimerCallbackFunc* pCallback, int32u context ) = 0;
	virtual void StartTimer( int32u dueTime, int32u period, iTimerCallback* pCallback, int32u context ) = 0;
	virtual void ChangeTiming( int32u newDueTime, int32u newPeriod ) = 0;
	virtual void StopTimer() = 0;

//Accessors
	virtual int32u TimePassedSinceStart() = 0; 

protected:
//Types
	enum eCallbackType
	{
		kNoCallback,
		kTimerHandler,
		kTimerCallbackFuncton,
		kCallbackClass,
	};

// Operations 
	// Called by the child timers to execute a timer callback
	virtual void SignalTimerEvent();

// Fields
	eCallbackType		fCallbackType;
	tTimerHandler*		fpTimerHandler;
	iTimerCallbackFunc* fpCallbackFunc;
	iTimerCallback*		fpCallback;
	int32u				fContext;

	int32u	fInitalTime;
	bool	fTimerHasStarted;
	int32u	fDueTime;
	int32u	fPeriod;
};

#endif


